/*
	Script um die Jahre 1990 bis 2049 in die Periodentabelle zu laden
	11.520 Zeilen für 60 Jahre x 16 Perioden x 12 Geschäftsjahresvarianten

	Toni August für Saxess Software GmbH
	Zuletzt modifiziert: 06/2023 für OCT 5.9

	Testcall
		SELECT * FROM global.tPeriods;

*/

	TRUNCATE TABLE global.tPeriods;

	DECLARE @StartJahr	INT = 1990;
	DECLARE @EndJahr	INT = 2049;

	WITH Jahre AS ( -- Liste of all Years, build over self recursion
		SELECT @StartJahr AS Jahr
		UNION ALL
		SELECT Jahr + 1
		FROM Jahre
		WHERE Jahr < @EndJahr
	)
	-- SELECT * FROM Jahre
	,Perioden AS (	-- List of all Periods, build over self recursion
		SELECT 1 AS Periode
		UNION ALL
		SELECT Periode + 1
		FROM Perioden
		WHERE Periode < 16
	)
	-- SELECT * FROM Perioden
	,GJB AS (	-- List of 1..12, build over self recursion
		SELECT 1 AS Beginn
		UNION ALL
		SELECT Beginn + 1
		FROM GJB
		WHERE Beginn < 12
	)
	-- Select
	INSERT INTO global.tPeriods
		SELECT
			 Jahr																									AS FinancialYear
			,Periode																								AS [Period]
			,Beginn																									AS FinancialYearBegin
			,CASE 
				WHEN Beginn = 1 THEN CAST(Jahr AS NVARCHAR(9)) 
				ELSE CAST(Jahr AS NVARCHAR(4)) + N'/' + CAST(Jahr + 1 AS NVARCHAR(4)) 
			 END																									AS FinancialYearName
			,YEAR(DATEADD (M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) AS CalendarYear
			,MONTH(DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) AS CalendarMonth
			,N'Q' + CAST(DATEPART(QUARTER, DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) AS NVARCHAR(1)) AS CalendarQuarterCODE
			,CASE
				WHEN MONTH(DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) < 7 THEN N'HY1' 
				ELSE N'HY2' 
			 END																									AS CalendarHalfYearCODE
			,YEAR(DATEADD (M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1)))*100
			 +MONTH(DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) AS YYYYMM
			,CONCAT(
				YEAR(DATEADD (M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1)))
				,N'-'
				,FORMAT(MONTH(DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))),'00')
				)																									AS [YYYY-MM]
			,YEAR(DATEADD (M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1)))*10
				+DATEPART(QUARTER, DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1))) AS YYYYQ
			,CONCAT (
					 YEAR(DATEADD (M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1)))
					,N' Q'
					,DATEPART(QUARTER, DATEADD(M, Beginn - 1, DATEFROMPARTS(Jahr, CASE WHEN Periode > 12 THEN 12 ELSE Periode END, 1)))
					)																												AS YYYYQQ

		FROM Jahre
			CROSS APPLY Perioden
			CROSS APPLY GJB
		ORDER BY
			 FinancialYearBegin
			,CalendarYear
			,Period
		OPTION (MAXRECURSION 0);

GO
