
DROP FUNCTION IF EXISTS calc.sfOCTDate2SQLDate;
GO

/*
	Function for converting a OCT Date from PDT (a number like in Excel) to an SQL Date
	The OCT Date is expected to be saved with Scale 1
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function 
		SELECT calc.sfOCTDate2SQLDate (45000)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfOCTDate2SQLDate',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfOCTDate2SQLDate','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfOCTDate2SQLDate
    (
         @OCTDate	BIGINT
    )

RETURNS DATE
AS 
BEGIN
	DECLARE @SQLDate DATE;
	IF @OCTDate > 100000 OR @OCTDate = 0
		BEGIN
			SET @SQLDate = NULL;
		END
	ELSE
		BEGIN
			SET @SQLDate = TRY_CONVERT(DATE,DATEADD(day,(@OCTDate)-2,N'19000101'));
		END

	RETURN @SQLDate;

END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfOCTDate2SQLDate'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to call demo values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@OCTDate';
    SET @value      = N'Date in OCT Format from ProductDataTable (Number) which must be passed in number with scale 1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
