
DROP FUNCTION IF EXISTS calc.sfReplaceNonXMLCharacters;
GO

/*
	Function for replacing all non XML conform characters in a string
	according to https://stackoverflow.com/questions/25147627/xml-path-expression-to-include-special-characters

	It does not remove 0x000A (LineFeed) and 0x000D (CarrigeReturn) which are conform - so any line break stays as it is.
	Table for codes http://redgrittybrick.org/ascii.html
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Function 
		DECLARE @String NVARCHAR(255);
		SET @String = CONCAT ('Test',char(0x000F));
		SELECT CONCAT('Vorher ',LEN(@String),' Zeichen');
		SELECT @String = calc.sfReplaceNonXMLCharacters (@String);
		SELECT CONCAT('Nachher ',LEN(@String),' Zeichen');
		SELECT @String;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfReplaceNonXMLCharacters',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfReplaceNonXMLCharacters','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfReplaceNonXMLCharacters
  (
         @String			NVARCHAR(MAX)
  )
    

RETURNS NVARCHAR(MAX)
AS 
BEGIN
	--DECLARE @Counter INT;
	--SET @Counter = LEN(@String);
	SET @String =	REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( 
						@String
					,char(0x0000),'') ,char(0x0001),'') ,char(0x0002),'') ,char(0x0003),'') ,char(0x0004),'') 
					,char(0x0005),'') ,char(0x0006),'') ,char(0x0007),'') ,char(0x0008),'') ,char(0x000B),'') 
					,char(0x000C),'') ,char(0x000E),'') ,char(0x000F),'') ,char(0x0010),'') ,char(0x0011),'') 
					,char(0x0012),'') ,char(0x0013),'') ,char(0x0014),'') ,char(0x0015),'') ,char(0x0016),'') 
					,char(0x0017),'') ,char(0x0018),'') ,char(0x0019),'') ,char(0x001A),'') ,char(0x001B),'') 
					,char(0x001C),'') ,char(0x001D),'') ,char(0x001E),'') ,char(0x001F),'');
	--SET @Counter = (@Counter - LEN(@String))*-1;
	--Functions can do a PRINT only with very dirty tricks like https://stackoverflow.com/questions/586642/tsql-how-do-you-output-print-in-a-user-defined-function, therefore only contained for debug resons
	--PRINT CONCAT ('Replaced ',@Counter, ' non XML conform characters in the string.');
	RETURN @String;
END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfReplaceNonXMLCharacters'		-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'							-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'								-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;									-- 0 = Demo object - out of shipment process
																				-- STANDARD OBJECTS
																				-- 1 = shiped from saxess standard without modification
																				-- 2 = shiped from saxess standard modified FOR customer from saxess
																				-- 3 = shiped from saxess standard modified FOR customer from partner
																				-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																				-- CUSTOM OBJECTS
																				-- 10 = shiped from saxess as customer specific object
																				-- 11 = shiped from partner	as customer specific object
																				-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to remove all non xml conform characters in a string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'Any string, where non-xml conform characters must be removed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
