
DROP FUNCTION IF EXISTS calc.sfTimeID2SQLDate;
GO

/*
	Function for converting an OCT TimeID to an SQL Date
	Returns NULL for invalid Dates
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function 
		SELECT calc.sfTimeID2SQLDate (20221031)
		SELECT calc.sfTimeID2SQLDate (20221042)
		SELECT calc.sfTimeID2SQLDate (10101010)

		SELECT CONVERT(Date, '20211305')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfTimeID2SQLDate',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfTimeID2SQLDate','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfTimeID2SQLDate
    (
         @TimeID	INT
    )

RETURNS DATE
AS 
BEGIN
		DECLARE @Date DATE;
		IF ISDATE(CAST(@TimeID AS NVARCHAR(50))) = 1 -- must check this, otherwise we can't return NULL instead of an error
			BEGIN
				--SET @Date = DATEFROMPARTS(@TimeID/10000,@TimeID/100%100,@TimeID%100)  -- both are working, the second one is faster (??)
				SET @Date = CONVERT(DATE,CAST(@TimeID AS NVARCHAR(50)))
			END
		ELSE
			BEGIN
				SET @Date = NULL
			END
		RETURN @Date
END

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfTimeID2SQLDate'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function convert an OCT TimeID to an SQL DateFormat.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID, written YYYYMMDD, returns NULL for invalid Dates.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
