
DROP FUNCTION IF EXISTS calc.sfUnifyLinebreaks;
GO

/*
	Function for adjusting Linebreaks of type "LF" to "CRLF"
	Keeps existing "CRLF"
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9
 
	Testcall Function 
		SELECT calc.sfUnifyLinebreaks ('Hallo',)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfUnifyLinebreaks',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfUnifyLinebreaks','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfUnifyLinebreaks
    (
         @String			NVARCHAR(MAX)
    )

RETURNS NVARCHAR(MAX)
AS 
BEGIN
	SET @String = REPLACE(
						REPLACE(
							REPLACE(
									@String,CHAR(13)+CHAR(10),CHAR(27)+CHAR(27) -- existing linebreaks are cached as ESC charactr
									),CHAR(10),CHAR(13)+CHAR(10)				-- effective replace
								),CHAR(27)+CHAR(27),CHAR(13)+CHAR(10)			-- back caching
							);
	RETURN @String;
END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfUnifyLinebreaks'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'DEMO'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 0;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function for adjusting Linebreaks of type "LF" to "CRLF"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'Any string which may contain linebreaks';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
