
DROP FUNCTION IF EXISTS dbo.sx_pf_PivotStringIntoTable;
GO

/*
	Function to split a String into a TableVariable
	The Delimiter String will be replace by a CHAR(28) and the string splitted on this

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT * FROM dbo.sx_pf_PivotStringIntoTable ('xxxNxxx', 'N')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_PivotStringIntoTable',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_PivotStringIntoTable','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_PivotStringIntoTable
	(
		 @String NVARCHAR(MAX)
		,@Delimiter NVARCHAR(5)
	)

RETURNS @tblSplitValues TABLE (txtValues NVARCHAR(MAX))

AS
BEGIN

	DECLARE @spr		NCHAR(1)		= CHAR(28)
	DECLARE @auxString	NVARCHAR (MAX)	= ''
	SET @auxString = REPLACE(@String, @Delimiter,@spr);

	WITH Split(stpos,endpos)
	AS(
		SELECT 0 AS stpos, CHARINDEX(@spr,@auxString) AS endpos
		UNION ALL
		SELECT CAST(endpos AS INT)+1, CHARINDEX(@spr,@auxString,endpos+1)
		FROM Split
		WHERE endpos > 0
	)

	INSERT @tblSplitValues
		SELECT SUBSTRING(@auxString,stpos,COALESCE(NULLIF(endpos,0),LEN(@auxString)+1)-stpos)
		FROM Split OPTION (MAXRECURSION 32766) -- it works ONLY up to 32.766 rows

	RETURN

END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_PivotStringIntoTable TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_PivotStringIntoTable'		-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'						-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1									-- 0 = Demo object - out of shipment process
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to split a String into a TableVariable.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'The pivot string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Delimiter';
    SET @value      = N'The delimiter to split the string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
