﻿
DROP FUNCTION IF EXISTS dbo.sx_pf_pMaskSQL;
GO

/*
	Function to mask strings, which will be part of an SQL Statement @StringValue  - it just doubles all inverted commas.
	returns a masked string
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.8
 
	Testcall Function
		DECLARE @String NVARCHAR(255);
		SET @String = N'ToDo''s';
		PRINT @String;
		SELECT dbo.sx_pf_pMaskSQL(@String);

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pMaskSQL',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pMaskSQL','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pMaskSQL (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(@StringValue,N'''',N'''''')
END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pMaskSQL TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pMaskSQL'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to mask strings, which will be part of an SQL Statement - it just doubles all inverted commas.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@StringValue';
    SET @value      = N'String, which will be returned with doubled inverted commas.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
