﻿
DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectBoolean;
GO
/*
	Function returns a '0' when the passed parameter is not either '0' or '1'
	@BooleanValue ='<#NV>'is ignored and not changed
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT dbo.sx_pf_pProtectBoolean (2)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectBoolean',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectBoolean','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectBoolean (@BooleanValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @BooleanValue <>'<#NV>' 
	
	BEGIN
		IF @BooleanValue <>'0' AND @BooleanValue <>'1'
		SET @BooleanValue= '0'
	END

	RETURN @BooleanValue

END

GO
-- Scalar value functions require EXECUTE permissions
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectBoolean TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectBoolean'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function returns a 0 when the passed parameter is not either 0 or 1. @BooleanValue =<#NV>is ignored and not changed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@BooleanValue';
    SET @value      = N'Boolean parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
