﻿
DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectString;
GO
/*
	Function to eliminate all not permitted special characters (Hochkommata, '|','),(', ';' ) and line breaks at the end from @StringValue
	Function returns a string without the not permitted special characters
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Function
		SELECT dbo.sx_pf_pProtectString ('X),(Y;Z')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectString',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectString','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectString (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN
	-- remove line break only at the end of the string
	IF LEN(@StringValue) > 1
	BEGIN
		SET @StringValue = SUBSTRING(@StringValue, 0, len(@StringValue) - 2) + REPLACE(REPLACE(SUBSTRING(@StringValue, LEN(@StringValue) - 2, 3), CHAR(10), ''), CHAR(13), '')
	END

	-- remove special characters
	RETURN	REPLACE(REPLACE(REPLACE(@StringValue,'|',''), '),(',''), '--','')
END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectString TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectString'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to eliminate all not permitted special characters from @StringValue.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@StringValue';
    SET @value      = N'Parameter string value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
