/*
	Scirpt to create an sequence object
	- must not be deleted an recreated in Update process
	- restarts after 2.000.000.000 LogEntries

	Last modified: 09/2022 for OCT 5.8

	Testcall
		SELECT * FROM sys.sequences WHERE name = 'seqPipelineLog';
		DROP SEQUENCE IF EXISTS system.seqPipelineLog;

		DECLARE @SEQ INT;
		SELECT	@SEQ = NEXT VALUE FOR system.seqPipelineLog; 
		PRINT	@SEQ;
*/

IF OBJECT_ID('system.seqPipelineLog', 'SO') IS NULL  
	BEGIN
		CREATE SEQUENCE system.seqPipelineLog 
		AS			INT
		START		WITH 1 
		INCREMENT	BY 1
		CACHE		10
		CYCLE;

	-- SET documentation variables      ***********************************************************************
	DECLARE
		 @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
		,@level1name        NVARCHAR(255)  = N'seqPipelineLog'				-- enter procedure name
		,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
		,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
		,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																				-- STANDARD OBJECTS
																				-- 1 = shiped from saxess standard without modification
																				-- 2 = shiped from saxess standard modified FOR customer from saxess
																				-- 3 = shiped from saxess standard modified FOR customer from partner
																				-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																				-- CUSTOM OBJECTS
																				-- 10 = shiped from saxess as customer specific object
																				-- 11 = shiped from partner	as customer specific object
																				-- 12 = shiped from customer as own specific object

		,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
	-- KEEP this default constants      *************************************************************************
	DECLARE
		 @name          NVARCHAR(255)   = N'MS_Description'
		,@level0type    NVARCHAR(255)   = N'SCHEMA'
		,@level1type    NVARCHAR(255)   = N'SEQUENCE'
		,@value         NVARCHAR(1000)  = N'';
 
		SET @value = @SX_Owner;
		EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
		SET @value = @SX_Module;
		EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
		SET @value = @SX_ShipmentFlag;
		EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
	-- SET documententation             *************************************************************************
 
		SET @value = N'Sequence Object to create an execution ID for Pipeline runs.';

		EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	
	END 
ELSE 
	PRINT 'Nothing to do, the sequence already exists.';

GO
