
DROP PROCEDURE IF EXISTS control.spSystemFailureReport;
GO


/*
	Procedure to fill the Table control.tSystemFailureReport
	Procedure will be build from many subsnipptes

		Testcall 
			EXEC control.spSystemFailureReport

			SELECT * FROM control.tSystemFailureReport

*/

CREATE PROCEDURE  control.spSystemFailureReport (@FactoryID NVARCHAR(255) = '', @ProductLineID NVARCHAR(255) = '')

AS

DECLARE @Timestamp 	Datetime 	= GETUTCDATE()
DECLARE @TestName	NVARCHAR (255) 	= ''
DECLARE @TestSource	NVARCHAR (255) 	= ''


-- self cleaning older than 14 Days
DELETE FROM control.tSystemFailureReport WHERE Timestamp < DATEADD(day,-14,GETUTCDATE());




SET @TestSource = 'Global_Lists'							-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for necessary System List which must exist.'			-- an friendly description of the test
-- this are the system list, a user could modify (ok) and delete (not ok)
-- there are more system list, but they are created internal and its not possible to modify

	BEGIN TRY

		WITH tmp AS
				(
				SELECT syslsts.ListID
				FROM (
					VALUES 
					  
					  (N'sxResponsiblePersons')
					, (N'sxStatus')
					, (N'sxUnits')
					, (N'sxValueEffectParameters')

				) syslsts (ListID) 
					LEFT JOIN planning.thLists hL 
						ON syslsts.ListID = hL.ListID
				WHERE hL.ListID IS NULL
				)

	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID'	AS FactoryID
				,'AllPLID'	AS ProductLineID
				,'AllPID'	AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE'	AS ResultType

				,'List ' 
					+ tmp.ListID 
					+ ' must exist, but is missing.'
							AS ResultText

				,1			AS ResultNumeric

			FROM tmp
				
					

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Global_Lists'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ListValues which use not exiting Formats for conditional formating.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID' AS FactoryID
				,'AllPLID' AS ProductLineID
				,'AllPID' AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'List ' 
					+ hLV.ListID 
					+ ' uses for Value '
					+	CASE
							WHEN hL.Datentyp = 'String' THEN hLV.ValueText
							ELSE CAST(hLV.ValueInt AS NCHAR(10))
						END
					+ ' the Format ' 
					+ hLV.FormatID + ' which does not exists in Format definitions.'
						
				,1 AS ResultNumeric

			FROM planning.thListValues hLV
				LEFT JOIN planning.thLists hL
					ON hLV.ListID = hL.ListID

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = hLV.FormatID
								)
					AND hLV.FormatID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Format definitions.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductLineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ dVS.ValueSeriesID
					+ ' uses Format ' 
					+ dVS.ValueFormatID + ' which does not exists in Format definitions.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = dVS.ValueFormatID
								)

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found. '
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Lists as ValueSource.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductLineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ ValueSeriesID
					+ ' uses ValueList ' 
					+ ValueListID + ' which does not exists in List Management.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT ListID FROM planning.thLists WHERE ListID = dVS.ValueListID
								)
					AND dVS.ValueListID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'ProductLine'												-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for use of not existing List in Globalattributes.'	-- an friendly description of the test

	BEGIN TRY
				WITH GA AS (

					SELECT N'1' AS Num, FactoryID, ProductLineID, GlobalAttributeSource1 AS GlobalAttributeSource
					FROM planning.tdProductLines WHERE GlobalAttributeSource1 <> N'' UNION ALL
					SELECT N'2', FactoryID, ProductLineID, GlobalAttributeSource2
					FROM planning.tdProductLines WHERE GlobalAttributeSource2 <> N'' UNION ALL
					SELECT N'3', FactoryID, ProductLineID, GlobalAttributeSource3
					FROM planning.tdProductLines WHERE GlobalAttributeSource3 <> N'' UNION ALL
					SELECT N'4', FactoryID, ProductLineID, GlobalAttributeSource4
					FROM planning.tdProductLines WHERE GlobalAttributeSource4 <> N'' UNION ALL
					SELECT N'5', FactoryID, ProductLineID, GlobalAttributeSource5
					FROM planning.tdProductLines WHERE GlobalAttributeSource5 <> N'' UNION ALL
					SELECT N'6', FactoryID, ProductLineID, GlobalAttributeSource6
					FROM planning.tdProductLines WHERE GlobalAttributeSource6 <> N'' UNION ALL
					SELECT N'7', FactoryID, ProductLineID, GlobalAttributeSource7
					FROM planning.tdProductLines WHERE GlobalAttributeSource7 <> N'' UNION ALL
					SELECT N'8', FactoryID, ProductLineID, GlobalAttributeSource8
					FROM planning.tdProductLines WHERE GlobalAttributeSource8 <> N'' UNION ALL
					SELECT N'9', FactoryID, ProductLineID, GlobalAttributeSource9
					FROM planning.tdProductLines WHERE GlobalAttributeSource9 <> N'' UNION ALL
					SELECT N'10', FactoryID, ProductLineID, GlobalAttributeSource10
					FROM planning.tdProductLines WHERE GlobalAttributeSource10 <> N'' UNION ALL
					SELECT N'11', FactoryID, ProductLineID, GlobalAttributeSource11
					FROM planning.tdProductLines WHERE GlobalAttributeSource11 <> N'' UNION ALL
					SELECT N'12', FactoryID, ProductLineID, GlobalAttributeSource12
					FROM planning.tdProductLines WHERE GlobalAttributeSource12 <> N'' UNION ALL
					SELECT N'13', FactoryID, ProductLineID, GlobalAttributeSource13
					FROM planning.tdProductLines WHERE GlobalAttributeSource13 <> N'' UNION ALL
					SELECT N'14', FactoryID, ProductLineID, GlobalAttributeSource14
					FROM planning.tdProductLines WHERE GlobalAttributeSource14 <> N'' UNION ALL
					SELECT N'15', FactoryID, ProductLineID, GlobalAttributeSource15
					FROM planning.tdProductLines WHERE GlobalAttributeSource15 <> N'' UNION ALL
					SELECT N'16', FactoryID, ProductLineID, GlobalAttributeSource16
					FROM planning.tdProductLines WHERE GlobalAttributeSource16 <> N'' UNION ALL
					SELECT N'17', FactoryID, ProductLineID, GlobalAttributeSource17
					FROM planning.tdProductLines WHERE GlobalAttributeSource17 <> N'' UNION ALL
					SELECT N'18', FactoryID, ProductLineID, GlobalAttributeSource18
					FROM planning.tdProductLines WHERE GlobalAttributeSource18 <> N'' UNION ALL
					SELECT N'19', FactoryID, ProductLineID, GlobalAttributeSource19
					FROM planning.tdProductLines WHERE GlobalAttributeSource19 <> N'' UNION ALL
					SELECT N'20', FactoryID, ProductLineID, GlobalAttributeSource20
					FROM planning.tdProductLines WHERE GlobalAttributeSource20 <> N'' UNION ALL
					SELECT N'21', FactoryID, ProductLineID, GlobalAttributeSource21
					FROM planning.tdProductLines WHERE GlobalAttributeSource21 <> N'' UNION ALL
					SELECT N'22', FactoryID, ProductLineID, GlobalAttributeSource22
					FROM planning.tdProductLines WHERE GlobalAttributeSource22 <> N'' UNION ALL
					SELECT N'23', FactoryID, ProductLineID, GlobalAttributeSource23
					FROM planning.tdProductLines WHERE GlobalAttributeSource23 <> N'' UNION ALL
					SELECT N'24', FactoryID, ProductLineID, GlobalAttributeSource24
					FROM planning.tdProductLines WHERE GlobalAttributeSource24 <> N'' UNION ALL
					SELECT N'25', FactoryID, ProductLineID, GlobalAttributeSource25
					FROM planning.tdProductLines WHERE GlobalAttributeSource25 <> N''

					)
	
		INSERT INTO control.tSystemFailureReport
			
			SELECT 
				 @Timestamp,
				 GA.FactoryID
				,GA.ProductLineID
				,'AllPID' AS ProductID
			    ,@TestSource
				,@TestName
				,'FAILURE' AS ResultType -- or WARNING

				,'Not existing List ' + GA.GlobalattributeSource + ' is in use for Globalattribute ' + GA.Num AS ResultText

				,1 AS ResultNumeric -- (0 for Warning, positiv for Failure)

			FROM GA 
				LEFT JOIN planning.thLists hL
					ON GA.GlobalattributeSource = hL.ListID
				WHERE hL.ListID IS NULL
			

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS WITHOUT CHANGE ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH




GO
