
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FlexAction;
GO

/*
	GET Operation for receiving a filterd context sensitiv List with dynamic column count
	The first column is sended back with POST_FlexAction as idenitfier for a database action

	Procedure must be editend to fit user needs and recreated after each API Update.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		--should return Data and 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_FlexAction
			 @Username				= 'SQL'
			,@FactoryID				= 'F1'
			,@ProductLineID			= 'PL1'
			,@ProductID				= 'P1'
			,@ValueSeriesID			= 'Projekt'
			,@TimeID				= 20180115
			,@GlobalattributeNumber = ''
			,@PageType				= ''
			,@SearchString			= 'ei';

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexAction',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexAction','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';	
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,				 N'NULL')					+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL')					+ N''','
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL')					+ N''','''	
			+ ISNULL(@PageType,				 N'NULL')					+ N''',''' 
			+ ISNULL(@SearchString,			 N'NULL')					+ N'''';  
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= N'';
	IF @PageType				IS NULL SET @PageType				= N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
	
		-- STEP 3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Projects AS TABLE
		(
			 ProjectID		NVARCHAR (255) NOT NULL
			,ProjectName	NVARCHAR (255) NOT NULL
			,ProjectManager NVARCHAR (255) NOT NULL
		);

		INSERT INTO @Projects
		VALUES
			(N'P1000',N'Testprojekt in Aachen','Pit Brown'),
			(N'P1001',N'Testprojekt in Augsburg','Pit Brown'),
			(N'P1002',N'Testprojekt in Bergisch Gladbach','Pit Brown'),
			(N'P1003',N'Testprojekt in Berlin','Pit Brown'),
			(N'P1004',N'Testprojekt in Bielefeld','Pit Brown'),
			(N'P1005',N'Testprojekt in Bochum','Pit Brown'),
			(N'P1006',N'Testprojekt in Bonn','Pit Brown'),
			(N'P1007',N'Testprojekt in Bottrop','Pit Brown'),
			(N'P1008',N'Testprojekt in Braunschweig','Pit Brown'),
			(N'P1009',N'Testprojekt in Bremen','Pit Brown'),
			(N'P1010',N'Testprojekt in Bremerhaven','Pit Brown'),
			(N'P1011',N'Testprojekt in Chemnitz','Pit Brown'),
			(N'P1012',N'Testprojekt in Darmstadt','Pit Brown'),
			(N'P1013',N'Testprojekt in Dortmund','Pit Brown'),
			(N'P1014',N'Testprojekt in Dresden','Pit Brown'),
			(N'P1015',N'Testprojekt in Duisburg','Pit Brown'),
			(N'P1016',N'Testprojekt in Düsseldorf','Pit Brown'),
			(N'P1017',N'Testprojekt in Erfurt','Pit Brown'),
			(N'P1018',N'Testprojekt in Erlangen','Pit Brown'),
			(N'P1019',N'Testprojekt in Essen','Pit Brown'),
			(N'P1020',N'Testprojekt in Frankfurt am Main','Pit Brown'),
			(N'P1021',N'Testprojekt in Freiburg im Breisgau','Pit Brown'),
			(N'P1022',N'Testprojekt in Fürth','Pit Brown'),
			(N'P1023',N'Testprojekt in Gelsenkirchen','Pit Brown'),
			(N'P1024',N'Testprojekt in Göttingen','Pit Brown'),
			(N'P1025',N'Testprojekt in Hagen','Pit Brown'),
			(N'P1026',N'Testprojekt in Halle (Saale)','Pit Brown'),
			(N'P1027',N'Testprojekt in Hamburg','Pit Brown'),
			(N'P1028',N'Testprojekt in Hamm','Pit Brown'),
			(N'P1029',N'Testprojekt in Hannover','Pit Brown'),
			(N'P1030',N'Testprojekt in Heidelberg','Cathy Gold'),
			(N'P1032',N'Testprojekt in Heilbronn','Cathy Gold'),
			(N'P1032',N'Testprojekt in Herne','Cathy Gold'),
			(N'P1033',N'Testprojekt in Ingolstadt','Cathy Gold'),
			(N'P1034',N'Testprojekt in Jena','Cathy Gold'),
			(N'P1035',N'Testprojekt in Karlsruhe','Cathy Gold'),
			(N'P1036',N'Testprojekt in Kassel','Cathy Gold'),
			(N'P1037',N'Testprojekt in Kiel','Cathy Gold'),
			(N'P1038',N'Testprojekt in Koblenz','Cathy Gold'),
			(N'P1039',N'Testprojekt in Köln','Cathy Gold'),
			(N'P1040',N'Testprojekt in Krefeld','Cathy Gold'),
			(N'P1041',N'Testprojekt in Leipzig','Cathy Gold'),
			(N'P1042',N'Testprojekt in Leverkusen','Cathy Gold'),
			(N'P1043',N'Testprojekt in Lübeck','Cathy Gold'),
			(N'P1044',N'Testprojekt in Ludwigshafen am Rhein','Cathy Gold'),
			(N'P1045',N'Testprojekt in Magdeburg','Cathy Gold'),
			(N'P1046',N'Testprojekt in Mainz','Cathy Gold'),
			(N'P1047',N'Testprojekt in Mannheim','Cathy Gold'),
			(N'P1048',N'Testprojekt in Moers','Cathy Gold'),
			(N'P1048',N'Testprojekt in Mönchengladbach','Cathy Gold'),
			(N'P1049',N'Testprojekt in Mülheim an der Ruhr','Cathy Gold'),
			(N'P1050',N'Testprojekt in München','Cathy Gold'),
			(N'P1051',N'Testprojekt in Münster','Cathy Gold'),
			(N'P1052',N'Testprojekt in Neuss','Cathy Gold'),
			(N'P1053',N'Testprojekt in Nürnberg','Cathy Gold'),
			(N'P1054',N'Testprojekt in Oberhausen','Cathy Gold'),
			(N'P1055',N'Testprojekt in Offenbach am Main','Cathy Gold'),
			(N'P1056',N'Testprojekt in Oldenburg','Cathy Gold'),
			(N'P1057',N'Testprojekt in Osnabrück','Cathy Gold'),
			(N'P1058',N'Testprojekt in Paderborn','Cathy Gold'),
			(N'P1059',N'Testprojekt in Pforzheim','Cathy Gold'),
			(N'P1060',N'Testprojekt in Potsdam','Cathy Gold'),
			(N'P1061',N'Testprojekt in Recklinghausen','Cathy Gold'),
			(N'P1062',N'Testprojekt in Regensburg','Cathy Gold'),
			(N'P1063',N'Testprojekt in Remscheid','Cathy Gold'),
			(N'P1064',N'Testprojekt in Reutlingen','Cathy Gold'),
			(N'P1065',N'Testprojekt in Rostock','Cathy Gold'),
			(N'P1066',N'Testprojekt in Saarbrücken','Cathy Gold'),
			(N'P1067',N'Testprojekt in Siegen','Cathy Gold'),
			(N'P1068',N'Testprojekt in Solingen','Cathy Gold'),
			(N'P1069',N'Testprojekt in Stuttgart','Cathy Gold'),
			(N'P1070',N'Testprojekt in Trier','Cathy Gold'),
			(N'P1071',N'Testprojekt in Ulm','Cathy Gold'),
			(N'P1072',N'Testprojekt in Wiesbaden','Cathy Gold'),
			(N'P1073',N'Testprojekt in Wolfsburg','Cathy Gold'),
			(N'P1074',N'Testprojekt in Wuppertal','Cathy Gold'),
			(N'P1075',N'Testprojekt in Würzburg','Cathy Gold');
		
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Second Column must have identifier character, its send back with the POST_FlexAction Command
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R= Right, C = Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter

		-- React on content parameters when creating the return list.
		-- You could add a "*" Member to the List and react on it in POST to create a new list member

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			SELECT 'Projekt wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS ProjectID_RS50
				,p.ProjectName AS Projektname_LS300
				,p.ProjectManager AS Projektmanager_CS150
			FROM @Projects p
			WHERE p.ProjectName LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			SELECT 'Mitarbeiter wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS MitarbeiterID_RS50
				,p.ProjectManager AS Projektmanager_LS150
			FROM @Projects p
			WHERE p.ProjectManager LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter')
		BEGIN
			SELECT 'FlexActions sind nur für die Wertreihen "Projekt" und "Mitarbeiter" definiert' AS Hint;
		END

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexAction;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FlexAction'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving a filterd context sensitiv List with dynamic column count
				 The first column is sended back with POST_FlexAction as idenitfier for a database action';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
		