
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FlexDrill;
GO

/*
	GET Operation for receiving a context sensitiv List
	API deliviers List Values fitting to the sended context Parameters
	Procedure must be editend to fit user needs and recreated after each API Update
	When updating this procedure, you must reset the rights for it

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		--should return Data and 200
		EXEC @RC = dbo.sx_pf_GET_FlexDrill 
			@Username				= 'SQL'
			,@FactoryID				= '1'
			,@ProductLineID			= '1'
			,@ProductID				= '1'
			,@ValueSeriesID			= '1'
			,@TimeID				= 20190115
			,@GlobalattributeNumber = ''
			,@PageType				= ''
			,@SearchString			= 'r';

	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexDrill',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexDrill','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FlexDrill
	 @Username				 NVARCHAR(255)
	,@FactoryID				 NVARCHAR(255)
	,@ProductLineID			 NVARCHAR(255)
	,@ProductID				 NVARCHAR(255)
	,@ValueSeriesID			 NVARCHAR(255)
	,@TimeID				 BIGINT
	,@GlobalattributeNumber  NVARCHAR(255)
	,@PageType				 NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			 NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				
	DECLARE @ParameterString	NVARCHAR(MAX)	=		N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			 N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL') + N''',''' 
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL') + N''',''' 
			+ ISNULL(@PageType,				 N'NULL') + N''',''' 
			+ ISNULL(@SearchString,			 N'NULL') + N''''; 
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username			  IS NULL SET @Username				 = N'';
	IF @FactoryID			  IS NULL SET @FactoryID			 = N'';
	IF @ProductLineID		  IS NULL SET @ProductLineID		 = N'';
	IF @ProductID			  IS NULL SET @ProductID			 = N'';
	IF @ValueSeriesID		  IS NULL SET @ValueSeriesID		 = N'';
	IF @TimeID				  IS NULL SET @TimeID				 = 0;
	IF @GlobalattributeNumber IS NULL SET @GlobalattributeNumber = N'';
	IF @PageType			  IS NULL SET @PageType				 = N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexDrill;

		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSeriesID	 = dbo.sx_pf_pProtectID		 (@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
	
		-- STEP 3.1 - Sample for creating a List for a specific ValueSeries in all Products from specific template
		IF @ValueSeriesID = N'that' 
			AND EXISTS(
				SELECT Template FROM planning.tdProducts 
				WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 'ValueSeriesSpecificList' AS Dummy;
			SET @EffectedRows = 1;
		END;

		-- STEP 3.2 - Sample for creating a List for a Globalattribute in one ProductLine
		IF @FactoryID = N'this' AND @ProductLineID = N'that' AND @GlobalattributeNumber = 5
		BEGIN
			SELECT 'ProductLineSpecificList for ValueSeries'; 
			SET @EffectedRows += 1;
		END;
		
		-- STEP 3.3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

		INSERT INTO @Cities VALUES
			(N'Aachen'),
			(N'Augsburg'),
			(N'Bergisch Gladbach'),
			(N'Berlin'),
			(N'Bielefeld'),
			(N'Bochum'),
			(N'Bonn'),
			(N'Bottrop'),
			(N'Braunschweig'),
			(N'Bremen'),
			(N'Bremerhaven'),
			(N'Chemnitz'),
			(N'Darmstadt'),
			(N'Dortmund'),
			(N'Dresden'),
			(N'Duisburg'),
			(N'Düsseldorf'),
			(N'Erfurt'),
			(N'Erlangen'),
			(N'Essen'),
			(N'Frankfurt am Main'),
			(N'Freiburg im Breisgau'),
			(N'Fürth'),
			(N'Gelsenkirchen'),
			(N'Göttingen'),
			(N'Hagen'),
			(N'Halle (Saale)'),
			(N'Hamburg'),
			(N'Hamm'),
			(N'Hannover'),
			(N'Heidelberg'),
			(N'Heilbronn'),
			(N'Herne'),
			(N'Ingolstadt'),
			(N'Jena'),
			(N'Karlsruhe'),
			(N'Kassel'),
			(N'Kiel'),
			(N'Koblenz'),
			(N'Köln'),
			(N'Krefeld'),
			(N'Leipzig'),
			(N'Leverkusen'),
			(N'Lübeck'),
			(N'Ludwigshafen am Rhein'),
			(N'Magdeburg'),
			(N'Mainz'),
			(N'Mannheim'),
			(N'Moers'),
			(N'Mönchengladbach'),
			(N'Mülheim an der Ruhr'),
			(N'München'),
			(N'Münster'),
			(N'Neuss'),
			(N'Nürnberg'),
			(N'Oberhausen'),
			(N'Offenbach am Main'),
			(N'Oldenburg'),
			(N'Osnabrück'),
			(N'Paderborn'),
			(N'Pforzheim'),
			(N'Potsdam'),
			(N'Recklinghausen'),
			(N'Regensburg'),
			(N'Remscheid'),
			(N'Reutlingen'),
			(N'Rostock'),
			(N'Saarbrücken'),
			(N'Siegen'),
			(N'Solingen'),
			(N'Stuttgart'),
			(N'Trier'),
			(N'Ulm'),
			(N'Wiesbaden'),
			(N'Wolfsburg'),
			(N'Wuppertal'),
			(N'Würzburg');

		SELECT 
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R=Right, C=Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter
			'List of Cities to select.'	 AS Hint
			,UPPER(LEFT([Name], 4))		 AS CityCode_LS50
			,[Name]						 AS CityName_CS200
			,'Beschreibung dieser Stadt...' AS Beschreibung_LS300
			,500000						 AS Einwohnerzahl_LM50
		 FROM @Cities
		 WHERE [Name] LIKE @SearchString
		 ORDER BY [Name];

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexDrill;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexDrill;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FlexDrill' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving a context sensitiv List
				API deliviers List Values fitting to the sended context Parameters
				Procedure must be editend to fit user needs and recreated after each API Update
				When updating this procedure, you must reset the rights for it';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
			