﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_FlexAction;
GO

/*
	POST Operation for sending FlexAction Parameter
	Procedure must be editend to fit user needs and recreated after each API Update.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	--should return Data and 200
	DECLARE  @RC					NVARCHAR(255)
			,@Username				NVARCHAR(255) = 'SQL'
			,@FactoryID				NVARCHAR(255) = 'F1'
			,@ProductLineID			NVARCHAR(255) = 'PL1'
			,@ProductID				NVARCHAR(255) = 'P1'
			,@ValueSeriesID			NVARCHAR(255) = 'Mitarbeiter'
			,@TimeID				BIGINT		  = 20180115
			,@GlobalattributeNumber	NVARCHAR(255) = ''
			,@PageType				NVARCHAR(255) = ''
			,@FlexParameter			NVARCHAR(255) = 'P1000'


	EXEC @RC = dbo.sx_pf_POST_FlexAction
		 @Username,
		 @FactoryID,
		 @ProductLineID,
		 @ProductID,
		 @ValueSeriesID,
		 @TimeID,
		 @GlobalattributeNumber,
		 @PageType,
		 @FlexParameter
	 	
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FlexAction',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FlexAction','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@FlexParameter			NVARCHAR(255)  -- The Identifier (Second column of GET_FlexAction, first Display column) from the choosen list element
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';	
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FactoryID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductLineID,			 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ValueSeriesID,			 N'NULL')		+ N''','
			+ CAST(ISNULL(@TimeID, N'NULL') AS NVARCHAR(255))	+ N','''
			+ ISNULL(@GlobalattributeNumber,	 N'NULL')		+ N''',''' 
			+ ISNULL(@PageType,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FlexParameter,			 N'NULL')		+ N'''';  
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	IF @FlexParameter			IS NULL SET @FlexParameter			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @FlexParameter	= dbo.sx_pf_pProtectString	(@FlexParameter);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END

		-- STEP 2 - Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID

		-- STEP 3 - Sample Actions
		-- React on content parameters and FlexParameter for action

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			-- POST_ValueSeries of Type Projekt, ValueSeriesID = FlexParameter
			PRINT 'Did Projekt'
			SET @ResultCode = 200;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			-- POST ValuesSeries of Type Mitarbeiter, ValueSeriesID = FlexParameter
			PRINT 'Did Mitarbeiter'
			SET @ResultCode = 200;
		END
		
		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter' )
		BEGIN
			PRINT 'No fitting action found.'
			SET @ResultCode = 404
		END
			   
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FlexAction;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_FlexAction'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for sending FlexAction Parameter
         Procedure must be editend to fit user needs and recreated after each API Update.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FlexParameter';
    SET @value      = N'FlexParameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
