﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Format;
GO

/*
	POST Operation for Format
	Not existing Format will be created, existing updated.
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE	 @RC					INT
				,@Username				NVARCHAR(255) = 'SQL'					'SQL'
				,@FormatID				NVARCHAR(255) = 'sxTest4'				'sxTest3'
				,@BackgroundColor		NVARCHAR(255) = '100,100,100'			'<#NV>'
				,@FontColor				NVARCHAR(255) = '200,200,200'			'<#NV>'
				,@ValueFormat			NVARCHAR(255) = ''						'<#NV>'
				,@Bold					NVARCHAR(255) = ''						'<#NV>'
				,@Italic				NVARCHAR(255) = ''						'<#NV>'
				,@AlignmentHorizontal	NVARCHAR(255) = ''						'<#NV>'
				,@AlignmentVertical		NVARCHAR(255) = ''						'<#NV>'
				,@TextUnderlined		NVARCHAR(255) = ''						'<#NV>'
				,@BorderToNext			NVARCHAR(255) = ''						'<#NV>'
				,@BorderToPrevious		NVARCHAR(255) = ''  					'<#NV>'  
		
		EXECUTE @RC = dbo.sx_pf_POST_Format 
				@Username,
				@FormatID,
				@BackgroundColor,
				@FontColor,
				@ValueFormat,
				@Bold,				
				@Italic,				
				@AlignmentHorizontal,
				@AlignmentVertical,	
				@TextUnderlined,		
				@BorderToNext,		
				@BorderToPrevious	
		PRINT @RC 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Format
	@Username				NVARCHAR(255),
	@FormatID				NVARCHAR(255),
	@BackgroundColor		NVARCHAR(255),
	@FontColor				NVARCHAR(255),
	@ValueFormat			NVARCHAR(255),
	@Bold					NVARCHAR(255),
	@Italic					NVARCHAR(255),
	@AlignmentHorizontal	NVARCHAR(255),
	@AlignmentVertical		NVARCHAR(255),
	@TextUnderlined			NVARCHAR(255),
	@BorderToNext			NVARCHAR(255),
	@BorderToPrevious		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N''''
			+ ISNULL(@Username				, N'NULL')		+ N''','''
			+ ISNULL(@FormatID				, N'NULL')		+ N''','''
			+ ISNULL(@BackgroundColor		, N'NULL')		+ N''','''
			+ ISNULL(@FontColor				, N'NULL')		+ N''','''
			+ ISNULL(@ValueFormat			, N'NULL')		+ N''',''' 
			+ ISNULL(@Bold					, N'NULL')		+ N''','''
			+ ISNULL(@Italic				, N'NULL')		+ N''',''' 
			+ ISNULL(@AlignmentHorizontal	, N'NULL')		+ N''','''
			+ ISNULL(@AlignmentVertical		, N'NULL')		+ N''','''
			+ ISNULL(@TextUnderlined		, N'NULL')		+ N''','''
			+ ISNULL(@BorderToNext			, N'NULL')		+ N''','''
			+ ISNULL(@BorderToPrevious		, N'NULL')		+ N''','''
			+ N'''';
	DECLARE @EffectedRows	 INT = 0;						
	DECLARE @ResultCode		 INT = 501;						
	DECLARE @TimestampCall	 DATETIME = GETUTCDATE();
	DECLARE @Comment		 NVARCHAR (2000) = N'';
	DECLARE @ErrorText		 NVARCHAR (2000) = N';'			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			 = N'';
	IF @FormatID			IS NULL SET @FormatID			 = N'';
	IF @BackgroundColor		IS NULL SET @BackgroundColor	 = N'';
	IF @FontColor			IS NULL SET @FontColor			 = N'';
	IF @ValueFormat			IS NULL SET @ValueFormat		 = N'';
	IF @Bold				IS NULL SET @Bold				 = N'';
	IF @Italic				IS NULL SET @Italic				 = N'';
	IF @AlignmentHorizontal	IS NULL SET @AlignmentHorizontal = N'';
	IF @AlignmentVertical	IS NULL SET @AlignmentVertical	 = N'';
	IF @TextUnderlined		IS NULL SET @TextUnderlined		 = N'';
	IF @BorderToNext		IS NULL SET @BorderToNext		 = N'';
	IF @BorderToPrevious	IS NULL SET @BorderToPrevious	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Format;

		-- STEP 0.2 - Protect input parameters
		SET @FormatID			 = dbo.sx_pf_pProtectID		(@FormatID);
		SET @BackgroundColor	 = dbo.sx_pf_pProtectString (@BackgroundColor);
		SET @FontColor			 = dbo.sx_pf_pProtectString (@FontColor);
		SET @ValueFormat		 = dbo.sx_pf_pProtectString (@ValueFormat);
		SET @Bold				 = dbo.sx_pf_pProtectInt	(@Bold);
		SET @Italic				 = dbo.sx_pf_pProtectInt	(@Italic);
		SET @AlignmentHorizontal = dbo.sx_pf_pProtectString (@AlignmentHorizontal);
		SET @AlignmentVertical	 = dbo.sx_pf_pProtectString (@AlignmentVertical);
		SET @TextUnderlined		 = dbo.sx_pf_pProtectInt	(@TextUnderlined)
		SET @BorderToNext		 = dbo.sx_pf_pProtectInt	(@BorderToNext)
		SET @BorderToPrevious	 = dbo.sx_pf_pProtectInt	(@BorderToPrevious)

		IF @FormatID = N''
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		IF	@Bold				NOT IN ('<#NV>','-1','0','1') OR
			@Italic				NOT IN ('<#NV>','-1','0','1') OR
			@BorderToNext		NOT IN ('<#NV>','-1','0','1') OR
			@BorderToPrevious	NOT IN ('<#NV>','-1','0','1') 
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter for Bold, Italic, BorderToNext, BorderToPrevios not supported (outside <#NV>,-1,0,1).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentHorizontal NOT IN ('<#NV>','','left','center','right')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentHorizontal + ' for AlignmentHorizontal not supported (outside <#NV>, left, center, rights, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentVertical NOT IN ('<#NV>','','top','center','bottom')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentVertical + ' for AlignmentVertical not supported (outside <#NV>, top, center, bottom, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;


		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @FormatID;
			
		-- STEP 3.1 - If format does not yet exist, it will be created
		IF @FormatKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thFormats (FormatID,BackgroundColor,FontColor,ValueFormat,Bold,Italic,AlignmentHorizontal,AlignmentVertical	
												,TextUnderlined,BorderToNext,BorderToPrevious	
												) 
			OUTPUT INSERTED.FormatKey INTO @Keys([Key]) 
				VALUES (@FormatID,'','','','-1','-1','','','-1','0','0');

			SELECT TOP (1) @FormatKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update format attributes
		UPDATE planning.thFormats 
		SET 
			 BackgroundColor	= IIF(@BackgroundColor		= N'<#NV>', BackgroundColor,		@BackgroundColor)
			,FontColor			= IIF(@FontColor			= N'<#NV>', FontColor,				@FontColor)
			,ValueFormat		= IIF(@ValueFormat			= N'<#NV>', ValueFormat,			@ValueFormat)
			,Bold				= IIF(@Bold					= N'<#NV>', Bold,					@Bold)
			,Italic				= IIF(@Italic				= N'<#NV>', Italic,					@Italic)
			,AlignmentHorizontal= IIF(@AlignmentHorizontal	= N'<#NV>', AlignmentHorizontal,	@AlignmentHorizontal)
			,AlignmentVertical	= IIF(@AlignmentVertical	= N'<#NV>', AlignmentVertical,		@AlignmentVertical)
			,TextUnderlined		= IIF(@TextUnderlined		= N'<#NV>', TextUnderlined,			@TextUnderlined)
			,BorderToNext		= IIF(@BorderToNext			= N'<#NV>', BorderToNext,			@BorderToNext)
			,BorderToPrevious	= IIF(@BorderToPrevious		= N'<#NV>', BorderToPrevious,		@BorderToPrevious)
		FROM planning.thFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Format;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Format'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Format
         Not existing Format will be created, existing updated.
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BackgroundColor';
    SET @value      = N'BackgroundColor';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FontColor';
    SET @value      = N'FontColor';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormat';
    SET @value      = N'ValueFormat';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Bold';
    SET @value      = N'Bold';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Italic';
    SET @value      = N'Italic';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@AlignmentHorizontal';
    SET @value      = N'AlignmentHorizontal';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@AlignmentVertical';
    SET @value      = N'AlignmentVertical';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TextUnderlined';
    SET @value      = N'TextUnderlined';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BorderToNext';
    SET @value      = N'BorderToNext';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BorderToPrevious';
    SET @value      = N'BorderToPrevious';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
