﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Value;
GO

/*
	POST Operation for Value
	The Value will be deleted and writen new

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test calls
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ProductID		NVARCHAR(255) = '1'
				,@ProductLineID	NVARCHAR(255) = 'U'
				,@FactoryID		NVARCHAR(255) = 'ZT'
				,@ValueSeriesID	NVARCHAR(255) = 'E1'
				,@TimeID		BIGINT		  = 20180315
				,@ValueFormula	NVARCHAR(max) = '<#NV>'
				,@ValueInt		NVARCHAR(255) = '555' 
				,@ValueText		NVARCHAR(max) = 'Hase2'
				,@ValueComment	NVARCHAR(max) = 'Hase2'
				,@i				INT = 0

		-- individual values 
		-- Text with Comment
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				'E1',
				20180115,
				'',
				0,
				'Hase',
				'Kommentar für Hase';
		PRINT @RC

		-- Integer with Comment
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				'K1',
				20180215,
				'',
				44,
				'',
				'Kommentar für Hase44';
		PRINT @RC

		-- 1b. Variant with loop
		--WHILE @i < 1000
		--BEGIN
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@ValueSeriesID,
				@TimeID,
				@ValueFormula,
				@i,
				@ValueText,
				@ValueComment
		--SET @i = @i +1
		PRINT @RC
		--END

		-- 2. Variant with Script Creator
		PRINT 'DECLARE @RC as INTEGER = 0'

		WHILE @i < 5000
		BEGIN
			PRINT 'EXECUTE @RC = dbo.sx_pf_POST_Value ''' + @Username + ''','''+ @ProductID+ ''','''+@ProductLineID+ ''','''+ @FactoryID + ''','''+ @ValueSeriesID + ''','+ CAST(@TimeID AS NVARCHAR (8)) + ','''+ @ValueFormula + ''','+ CAST(@i AS NVARCHAR(10)) + ','''+@ValueText+ ''','''+	@ValueComment + ''''
			PRINT 'PRINT @RC' 
			SET @i = @i +1 
		END
		--about 12 sec for 5000 Values Values = 400 / second

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Value',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Value','PARAMETER',NULL)
*/
 
CREATE PROCEDURE dbo.sx_pf_POST_Value
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ValueSeriesID	NVARCHAR(255),		
	@TimeID			INT,
	@ValueFormula	NVARCHAR(MAX),
	@ValueInt		NVARCHAR(255), -- to be able to send <#NV>
	@ValueText		NVARCHAR(MAX),
	@ValueComment	NVARCHAR(MAX)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @ValueKey			BIGINT			= 0;

	DECLARE @OLDValueFormula	NVARCHAR(255)	= N'';
	DECLARE @OLDValueInt		BIGINT			= 0; 
	DECLARE @OLDValueText		NVARCHAR(255)	= N'';
	DECLARE @OLDValueComment	NVARCHAR(MAX)	= N'';

	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
		+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' 
		+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,		N'NULL') + N''',''' 
		+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL') + N',''' 
		+ ISNULL(@ValueFormula,		N'NULL') + N''',''' + ISNULL(@ValueInt,			N'NULL') + N''',''' 
		+ ISNULL(@ValueText,		N'NULL') + N''',''' + ISNULL(@ValueComment,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @ValueFormula	IS NULL SET @ValueFormula	= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueComment	IS NULL SET @ValueComment	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Value;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @TimeID			= dbo.sx_pf_pProtectID		(@TimeID);
		SET @ValueFormula	= dbo.sx_pf_pProtectString	(@ValueFormula);
		SET @ValueInt		= dbo.sx_pf_pProtectString	(@ValueInt);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);

		-- Remove Linebreaks
		SET @ValueFormula	= REPLACE(REPLACE(@ValueFormula	,CHAR(13),''),CHAR(10),'');
		SET @ValueText		= REPLACE(REPLACE(@ValueText	,CHAR(13),''),CHAR(10),'');
		SET @ValueComment	= REPLACE(REPLACE(@ValueComment	,CHAR(13),''),CHAR(10),'');

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N'' OR @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

 	     -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.3 - Try to detect and cancel the ValueSeriesKey if does not exist
		SELECT @ValueSeriesKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey= @ProductKey AND ValueSeriesID = @ValueSeriesID;
		
		IF @ValueSeriesKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2.1 - Cancel if TimeID not exists	
		IF NOT EXISTS(SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('TimeID not exists', 16, 10);
		END;

		-- STEP 3. Determine ValueKey identify and old values
		SELECT @ValueKey = ValueKey, @OLDValueFormula = ValueFormula, @OLDValueInt = ValueInt, @OLDValueText = ValueText, @OLDValueComment = ValueComment
		FROM planning.tfValues 
		WHERE ValueSeriesKey = @ValueSeriesKey AND TimeID = @TimeID;
				
		-- STEP 3.1 - If Value Key exists delete
		IF  @ValueKey > 0 
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ValueKey = @ValueKey;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 204; -- Ok, but deleted

			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', @OLDValueFormula,	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', @OLDValueText,		@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', @OLDValueComment,	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', @OLDValueInt,		@ValueInt);
		END
		ELSE
		BEGIN
			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', N'',	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', N'',	@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', N'',	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', N'0',	@ValueInt);
		END;

		-- STEP 3.2 - Insert, if Value not empty outside *_0 ValueSeries
		IF @ValueFormula = N'' AND @ValueText = N'' AND @ValueInt = N'0' AND RIGHT(@ValueSeriesID,2)<>'_0'
		BEGIN 
			SET @ResultCode = 204; --> discarded !!! 
		END
		ELSE
		BEGIN 
			SET @ValueInt = COALESCE(TRY_CAST(@ValueInt AS BIGINT), 0);

			INSERT INTO planning.tfValues
			VALUES (
				@ValueSeriesKey, 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID, 
				@FactoryID, 
				@ValueSeriesID,  
				@TimeID,  
				@ValueFormula, 
				@ValueInt, 
				@ValueText, 
				@ValueComment);
			
			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 201;
		END;

		COMMIT TRANSACTION sx_pf_POST_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Value'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Value
         The Value will be deleted and writen new';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormula';
    SET @value      = N'ValueFormula';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueComment';
    SET @value      = N'ValueComment';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
