﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ValueSerie;
GO

/*
	POST Operation for one ValueSeries
	Operation will create a ValueSeries if it not exist
	It will change an existing ValueSeries
	Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) ='SQL'
			,@ProductID					NVARCHAR(255) ='1'
			,@ProductLineID				NVARCHAR(255) ='U'
			,@FactoryID					NVARCHAR(255) ='ZT'
			,@ValueSeriesID				NVARCHAR(255) ='38550'
			,@RequestedValueSeriesNo	INT			  = 249
			,@NameShort					NVARCHAR(255) ='<#NV>'
			,@NameLong					NVARCHAR(255) ='<#NV>'
			,@CommentUser				NVARCHAR(255) ='<#NV>'
			,@CommentDev				NVARCHAR(255) ='<#NV>'
			,@ImageName					NVARCHAR(255) ='<#NV>'
			,@IsNumeric					NVARCHAR(255) ='1'
			,@VisibilityLevel			NVARCHAR(255) ='<#NV>'
			,@ValueSource				NVARCHAR(255) ='<#NV>'
			,@ValueListID				NVARCHAR(255) ='<#NV>'
			,@ValueFormatID				NVARCHAR(255) ='<#NV>'
			,@Unit						NVARCHAR(255) ='<#NV>'
			,@Scale						NVARCHAR(255) ='100'
			,@Effect					NVARCHAR(255) ='<#NV>'
			,@EffectParameter			NVARCHAR(255) ='<#NV>'

	EXECUTE @RC = dbo.sx_pf_POST_ValueSerie 
			@Username,	
			@ProductID,	
			@ProductLineID,	
			@FactoryID,	
			@ValueSeriesID,	
			@RequestedValueSeriesNo,
			@NameShort,	
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ImageName,
			@IsNumeric,
			@VisibilityLevel,
			@ValueSource,
			@ValueListID,
			@ValueFormatID,
			@Unit,
			@Scale,
			@Effect,
			@EffectParameter
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ValueSerie',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ValueSerie
	@Username				 NVARCHAR(255),
	@ProductID				 NVARCHAR(255),
	@ProductLineID			 NVARCHAR(255),
	@FactoryID				 NVARCHAR(255),
	@ValueSeriesID			 NVARCHAR(255),
	@RequestedValueSeriesNo	 INT,
	@NameShort				 NVARCHAR(255),
	@NameLong				 NVARCHAR(255),
	@CommentUser			 NVARCHAR(255),
	@CommentDev				 NVARCHAR(255),
	@ImageName				 NVARCHAR(255),
	@IsNumeric				 NVARCHAR(255),
	@VisibilityLevel		 NVARCHAR(255),
	@ValueSource			 NVARCHAR(255),
	@ValueListID			 NVARCHAR(255),
	@ValueFormatID			 NVARCHAR(255),
	@Unit					 NVARCHAR(255),
	@Scale					 NVARCHAR(255),
	@Effect					 NVARCHAR(255),
	@EffectParameter		 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @OldValueSeriesNo	INT				= 0;
	DECLARE @OldIsNumeric		INT				= 0;
	DECLARE @OldScale			INT				= 0;
	DECLARE @OldValueSource		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,										N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,										N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@RequestedValueSeriesNo AS NVARCHAR(255)),	N'NULL') + N',''' 
			+ ISNULL(@NameShort,		N'NULL') + N''',''' + ISNULL(@NameLong,											N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,		N'NULL') + N''',''' + ISNULL(@CommentDev,										N'NULL') + N''',''' 
			+ ISNULL(@ImageName,		N'NULL') + N''',''' + ISNULL(@IsNumeric,										N'NULL') + N''',''' 
			+ ISNULL(@VisibilityLevel,	N'NULL') + N''',''' + ISNULL(@ValueSource,										N'NULL') + N''',''' 
			+ ISNULL(@ValueListID,		N'NULL') + N''',''' + ISNULL(@ValueFormatID,									N'NULL') + N''',''' 
			+ ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@Scale,											N'NULL') + N''',''' 
			+ ISNULL(@Effect,			N'NULL') + N''',''' + ISNULL(@EffectParameter,									N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductID				IS NULL SET @ProductID					= N'NULL';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'NULL';
	IF @FactoryID				IS NULL SET @FactoryID					= N'NULL';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID				= N'NULL';
	IF @RequestedValueSeriesNo	IS NULL SET @RequestedValueSeriesNo		= 0;
	IF @NameShort				IS NULL SET @NameShort					= N'NULL';
	IF @NameLong				IS NULL SET @NameLong					= N'NULL';
	IF @CommentUser				IS NULL SET @CommentUser				= N'NULL';
	IF @CommentDev				IS NULL SET @CommentDev					= N'NULL';
	IF @ImageName				IS NULL SET @ImageName					= N'NULL';
	IF @IsNumeric				IS NULL SET @IsNumeric					= N'NULL';
	IF @VisibilityLevel			IS NULL SET @VisibilityLevel			= N'NULL';
	IF @ValueSource				IS NULL SET @ValueSource				= N'NULL';
	IF @ValueListID				IS NULL SET @ValueListID				= N'NULL';
	IF @ValueFormatID			IS NULL SET @ValueFormatID				= N'NULL';
	IF @Unit					IS NULL SET @Unit						= N'NULL';
	IF @Scale					IS NULL SET @Scale						= N'NULL';
	IF @Effect					IS NULL SET @Effect						= N'NULL';
	IF @EffectParameter			IS NULL SET @EffectParameter			= N'NULL';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ValueSerie;

		-- STEP 0.1 - NULL Protection
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ValueSeriesID		= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @IsNumeric			= dbo.sx_pf_pProtectBoolean (@IsNumeric);
		SET @VisibilityLevel	= dbo.sx_pf_pProtectString	(@VisibilityLevel);
		SET @ValueSource		= dbo.sx_pf_pProtectString	(@ValueSource);
		SET @Unit				= dbo.sx_pf_pProtectString	(@Unit);
		SET @Scale				= dbo.sx_pf_pProtectInt		(@Scale);
		SET @Effect				= dbo.sx_pf_pProtectString	(@Effect);
		SET @EffectParameter	= dbo.sx_pf_pProtectString	(@EffectParameter);
		IF @ValueListID		<> N'<#NV>' SET @ValueListID	= dbo.sx_pf_pProtectID (@ValueListID);
		IF @ValueFormatID	<> N'<#NV>' SET @ValueFormatID	= dbo.sx_pf_pProtectID (@ValueFormatID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys	
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductLineKey = @ProductLinekey AND ProductID= @ProductID;
		SELECT 
				 @ValueSeriesKey = ValueSeriesKey
				,@OldIsNumeric = IsNumeric
				,@OldScale = Scale
				,@OldValueSource = ValueSource 
		FROM planning.tdValueSeries 
		WHERE 
				ProductKey = @ProductKey 
			AND ValueSeriesID = @ValueSeriesID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - Insert the ValueSeries, if Key does not exist
		IF @ValueSeriesKey = 0
		BEGIN
			-- INCREMENT all following Requested ValueSeriesNumbers based on requested No
			UPDATE planning.tdValueSeries
			SET ValueSeriesNo = ValueSeriesNo + 1 
			WHERE 
				ProductKey = @ProductKey 
				AND ValueSeriesNo >= @RequestedValueSeriesNo;

			IF @VisibilityLevel = '<#NV>' SET @VisibilityLevel = 1;
			
			SET @EffectedRows = @@ROWCOUNT;

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			-- Insert with requested number
			INSERT INTO planning.tdValueSeries (
				ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,ValueSeriesID
				,ValueSeriesNo
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,IsNumeric
				,VisibilityLevel
				,ValueSource
				,ValueListID
				,ValueFormatID
				,Unit
				,Scale
				,Effect
				,EffectParameter) 
			OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
			VALUES ( 
				@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter);
			
			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ValueSeriesKey = [Key] FROM @Keys ORDER BY [Key];
		END
		ELSE
		BEGIN 
		-- STEP 3.2 - Update ValueSeries if it already exists
			SELECT @OldValueSeriesNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ValueSeriesKey = @ValueSeriesKey;
									
			-- Use the old one, if no SeriesNo is commited (lower than one means not commited)
			IF @RequestedValueSeriesNo < 1 
				SET @RequestedValueSeriesNo = @OldValueSeriesNo;
			
			-- Adjust Numbers if not equal
			IF @OldValueSeriesNo < @RequestedValueSeriesNo
			BEGIN 
				-- decrement all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo - 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo > @OldValueSeriesNo 
					AND ValueSeriesNo <= @RequestedValueSeriesNo;
														
				SET @EffectedRows += @@ROWCOUNT;
			END
			ELSE IF @OldValueSeriesNo > @RequestedValueSeriesNo
			BEGIN 
				-- increment all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo + 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo < @OldValueSeriesNo 
					AND ValueSeriesNo >= @RequestedValueSeriesNo;

				SET @EffectedRows += @@ROWCOUNT;
			END;

			UPDATE planning.tdValueSeries 
			SET 
				ValueSeriesNo		= @RequestedValueSeriesNo
				, NameShort			= IIF(@NameShort		= N'<#NV>', NameShort,			@NameShort)
				, NameLong			= IIF(@NameLong			= N'<#NV>', NameLong,			@NameLong)
				, CommentUser		= IIF(@CommentUser		= N'<#NV>', CommentUser,		@CommentUser)
				, CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,			@CommentDev)
				, ImageName			= IIF(@ImageName		= N'<#NV>', ImageName,			@ImageName)
				, IsNumeric			= IIF(@IsNumeric		= N'<#NV>', IsNumeric,			@IsNumeric)
				, VisibilityLevel	= IIF(@VisibilityLevel	= N'<#NV>', VisibilityLevel,	@VisibilityLevel)
				, ValueSource		= IIF(@ValueSource		= N'<#NV>', ValueSource,		@ValueSource)
				, ValueListID		= IIF(@ValueListID		= N'<#NV>', ValueListID,		@ValueListID)
				, ValueFormatID		= IIF(@ValueFormatID	= N'<#NV>', ValueFormatID,		@ValueFormatID)
				, Unit				= IIF(@Unit				= N'<#NV>', Unit,				@Unit)
				, Scale				= IIF(@Scale			= N'<#NV>', Scale,				@Scale)
				, Effect			= IIF(@Effect			= N'<#NV>', Effect,				@Effect)
				, EffectParameter	= IIF(@EffectParameter	= N'<#NV>', EffectParameter,	@EffectParameter)
			FROM planning.tdValueSeries
			WHERE ValueSeriesKey = @ValueSeriesKey;
				
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.2a - Delete zero values if ValueSeries is now numeric input series
			IF @IsNumeric = '1' AND @ValueSource = N'Input'
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueInt = 0; 
			END;

			-- Step 3.2b - Delete empty Strings if ValueSeries is now non-numeric input series
			IF @IsNumeric = '0' AND @ValueSource = N'Input' 
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueText = N'';
			END;

				
			-- STEP 3.3 - Update exists Values due to possible rescaling or changing type from Numeric to Non-Numeric
			DECLARE @vals AS TABLE (ValueKey INT NOT NULL, ValueInt INT NOT NULL, ValueText NVARCHAR(MAX) NOT NULL);

			-- Changing type Numeric / Non Numeric
			IF (@IsNumeric <> N'<#NV>' AND @OldIsNumeric <> @IsNumeric)
			BEGIN
				-- Copy ValueText to ValueInt, after Try_Cast to Int to get only valid numbers and multiplied by OldScale, as the int values use scale, while the text value not
				IF @IsNumeric = N'1'
				BEGIN							 
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ISNULL(TRY_CAST(ValueText AS INT), 0) * @OldScale, ''
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					-- If Numeric is made to non numeric the value in the Database must be be divided by scale and converted to text, as text dont uses scale
					IF (@OldScale <> 0)
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt / @OldScale AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey;	
					END
					ELSE
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey; 
					END;
				END;
			END;

			-- changing scaling
			-- If Isnumeric = 0 and not changed Nothing happens with the value, if scale is changed, as text values dont use the scale property.
			IF (@Scale <> N'<#NV>' AND @OldScale <> @Scale) AND (@IsNumeric = N'1' OR (@IsNumeric = N'<#NV>' AND @OldIsNumeric = 1))
			BEGIN
				-- Change Scale of existing values if scale is changed (when scale is changed, e.g. from 100 to 1000 the value in the database must be multiplied by 10)
				DECLARE @k FLOAT = @Scale / CAST(@OldScale AS FLOAT);	
			
				IF NOT EXISTS(SELECT 1 FROM @vals)
				BEGIN
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ValueInt * @k, '' 
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					UPDATE @vals
					SET ValueInt = ValueInt * @k; 
				END;
			END;
			
			IF EXISTS(SELECT 1 FROM @vals)
			BEGIN
				UPDATE v
				SET 
					 v.ValueInt		= newV.ValueInt
					,v.ValueText	= newV.ValueText 
				FROM planning.tfValues v INNER JOIN @vals newV 
					ON v.ValueKey = newV.ValueKey
				WHERE v.ValueSeriesKey = @ValueSeriesKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;

		SET @Resultcode = 201;
		COMMIT TRANSACTION sx_pf_POST_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ValueSerie'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for one ValueSeries
         Operation will create a ValueSeries if it not exist
         It will change an existing ValueSeries
         Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@RequestedValueSeriesNo';
    SET @value      = N'RequestedValueSeriesNo';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsNumeric';
    SET @value      = N'IsNumeric';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@VisibilityLevel';
    SET @value      = N'VisibilityLevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSource';
    SET @value      = N'ValueSource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueListID';
    SET @value      = N'ValueListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormatID';
    SET @value      = N'ValueFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Effect';
    SET @value      = N'Effect';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EffectParameter';
    SET @value      = N'EffectParameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
