﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ClusterPropertiesReadRight;
GO

/*
	GET Operation for read right on Clusterlevel
	Needs already resolved Transaktionsuser, therefore PRIVATE - call only in case of passing in a validated Transactusername !
	Returns in case of read rights for this user 200, else 401
	Read rights have all user which have got read rights in OCT Usermanagement or have the privileged OCT database roles (FactoryService, db_owner)
	This procedure is always called after determination of the @Transactusername over function dbo.sx_pf_Determine_Transactusername,
	therefore it is already ensured that
	 - the delivered username is an active user from OCT Usermangement or a user with privileged role
 
	The Procedure therefore always return 200, except the @Transactusername is '403', which is the return code in case of no access, 
	which was not handle als exception in the calling procedure

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pGET_ClusterPropertiesReadRight 'W10\admin'
		PRINT @RESULT

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterPropertiesReadRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterPropertiesReadRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ClusterPropertiesReadRight 
	@TransactionUsername AS NVARCHAR(255)
AS
	BEGIN

		IF @TransactionUsername = '403'		
			BEGIN
				RETURN 401 
			END
			ELSE
			BEGIN
				RETURN 200
			END
	END
GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ClusterPropertiesReadRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ClusterPropertiesReadRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'	GET Operation for read right on Clusterlevel
					Needs already resolved Transaktionsuser, therefore PRIVATE !
					Returns in case of read rights for this user 200, else 401
					Read rights have all user which have got read rights in OCT Usermanagement or have the privileged OCT database roles (FactoryService, db_owner)
					This procedure is always called after determination of the @Transactusername over function dbo.sx_pf_Determine_Transactusername,
					therefore it is already ensured that
					 - the delivered username is an active user from OCT Usermangement or a user with privileged role
 
					The Procedure therefore always return 200, except the @Transactusername is ''403'', which is the return code in case of no access, 
					which was not handle als exception in the calling procedure';

    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'TransactionUsername - the already als valid and granted name of an existing OCT User or a member of an priviledged role.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
