﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ClusterWriteRight;
GO

/*
	GET Operation for Write Rights on Cluster
	Needs already a validated Transactionsuser !
	Returns for OCT Clusteradmins and privileged roles 200

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	 Testcall
		DECLARE @Returncode INT
		EXEC @Returncode = dbo.sx_pf_pGET_ClusterWriteRight 'W10\admin'
		PRINT @Returncode

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterWriteRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterWriteRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ClusterWriteRight 
	@TransactionUsername AS NVARCHAR(255)
AS
	BEGIN
		DECLARE @Right				NVARCHAR (255)	='none'
		DECLARE @IsPrivilegedUser	INT				= 0

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
			IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200
			END

		-- Check for Write Line on cluster
		SELECT @Right=COALESCE([Right],'no') FROM system.trUserRights WHERE FactoryID = '' AND Username = @TransactionUsername;

		IF @Right = 'Write' 
			BEGIN
				RETURN 200
			END
			ELSE
			BEGIN
				RETURN 401
			END
	END
GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ClusterWriteRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ClusterWriteRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value =	N'GET Operation for Write Rights on Cluster
					Needs already a validated Transactionsuser !
					Returns for OCT Clusteradmins and privileged role member 200';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Name of validated Transactionuser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
