﻿
DROP PROCEDURE IF EXISTS global.spDELETE_Code;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		DECLARE @ReturnCode INT;
		EXEC @ReturnCode	= global.spDELETE_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST';
		
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Code',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Code','PARAMETER',NULL)

*/
 
CREATE PROCEDURE global.spDELETE_Code
	 @ModuleID						NVARCHAR(255)	
	,@LanguageID					NVARCHAR(255)	= 'DE'
	,@CODE							NVARCHAR(255)	= ''
	 
AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';
	IF @CODE			IS NULL SET @CODE			= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''	
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0
		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''',''' 
			+ @LanguageID		+ N''',''' 
			+ @CODE				+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spDELETE_Code;	

		-- Standard PreUser Procedure
		IF OBJECT_ID('global.spDELETE_Code_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spDELETE_Code_PreUser @ModuleID,@LanguageID,@CODE
			END
		SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
		
		-- Always capitalize code																	 		
		SET @CODE = UPPER(@CODE)
																									  		
		-- UPDATE or INSERT in target table															  
		IF EXISTS(
					SELECT TOP(1) 1 
					FROM global.tCodes
					WHERE 
							 ModuleID	= @ModuleID
						AND (LanguageID	= @LanguageID	OR @LanguageID = '')
						AND (Code		= @CODE			OR @CODE = '')
					)
			-- Existing CODE will be deleted
			BEGIN
				DELETE FROM [global].tCodes
				WHERE		ModuleID	= @ModuleID
						AND (LanguageID	= @LanguageID	OR @LanguageID = '')
						AND (Code		= @CODE			OR @CODE = '')

				 SET @EffectedRows = @@ROWCOUNT;
				 SET @Resultcode = 200;
			END
		ELSE
			-- Not existing Code will result in warning
			BEGIN
				SET @EffectedRows = 0;
				SET @Resultcode = 404;
			END

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spDELETE_Code_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spDELETE_Code_PostUser @ModuleID,@LanguageID,@CODE
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));
		
		COMMIT TRANSACTION spDELETE_Code;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spDELETE_Code;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_Code'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to DELETE Codes.
	Can be controled over parameter. ModuleID must be passed
	1. LanguageID = '''' -> deletes all Languages
	2. LanguageID = ''DE'' -> Deletes only German
	3. CODE = '''' -> deletes all Codes
	4. CODE = ''XX'' -> deletes only this code
	Pre / Post Extension possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the code you want to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'Language of the code you want to delete (ISO Format, like DE for Germany.) or empty for all Languages.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CODE';
    SET @value      = N'CODE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
