﻿DROP PROCEDURE IF EXISTS global.spGET_Codes;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcalls
		DECLARE @ReturnCode INT;

		-- this should return all
		EXEC @ReturnCode	= global.spGET_Codes
		PRINT @ReturnCode;
		
		-- this one ModuleID, all Languages
		EXEC @ReturnCode	= global.spGET_Codes
			 @ModuleID		= 'system'
		PRINT @ReturnCode;

		-- this one ModuleID, only DE
		EXEC @ReturnCode	= global.spGET_Codes
			  @ModuleID		= 'system'
			 ,@LanguageID	= 'DE'
		
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes;
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Codes',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Codes','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spGET_Codes
	  @ModuleID						NVARCHAR(255) = ''
	 ,@LanguageID					NVARCHAR(255) = ''

AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0

		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''','''
			+ @LanguageID		+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spGET_Codes;	


			-- Standard PreUser Procedure
			IF OBJECT_ID('global.spGET_Codes_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spGET_Codes_PreUser @ModuleID,@LanguageID
			END
			SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
			
			-- Main Query
			SELECT 
				*
			FROM global.tCodes
			WHERE 	(ModuleID	= @ModuleID		OR @ModuleID = '')
				AND (LanguageID = @LanguageID	OR @LanguageID = '')

			SET @EffectedRows = @@ROWCOUNT;
			SET @Resultcode = 200;

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spGET_Codes_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spGET_Codes_PostUser @ModuleID,@LanguageID
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));

		COMMIT TRANSACTION spGET_Codes;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spGET_Codes;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Codes'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to GET Codes. Parameters can be used to filter output. 
	1. ModuleID = '''' -> all Modules
	2. ModuleID = ''[XX]'' -> specific Module
	3. LanguageID = '''' all Languages
	4. LanguageID = ''[XX]'' -> specific Language
	Pre / Post Extensions possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the codes or empty for all Modules.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'ID of the Language which owns the codes or empty for all Languages.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
