﻿
DROP PROCEDURE IF EXISTS global.spPOST_Code;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		-- creates a Code in German and Englisch
		DECLARE @ReturnCode INT;
		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hallo deutscher Test';
		PRINT @ReturnCode;

		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'EN'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hello englisch Test';
		PRINT @ReturnCode;

		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'finance'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hello englisch Test';
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Code',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Code','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spPOST_Code
	 @ModuleID						NVARCHAR(255)
	,@LanguageID					NVARCHAR(255)	= 'DE'
	,@CODE							NVARCHAR(255) 
	,@Scope							NVARCHAR(255)	= 'default'
	,@Description					NVARCHAR(4000)	= ''

AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';
	IF @CODE			IS NULL SET @CODE			= N'';
	IF @Scope			IS NULL SET @Scope			= N'';
	IF @Description		IS NULL SET @Description	= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''	
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0
		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''',''' 
			+ @LanguageID		+ N''',''' 
			+ @CODE				+ N''',''' 
			+ @Scope			+ N''',''' 
			+ @Description		+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spPOST_Code;	

		-- Standard PreUser Procedure
		IF OBJECT_ID('global.spPOST_Code_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spPOST_Code_PreUser @ModuleID,@LanguageID,@CODE,@Scope,@Description
			END
		SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
		
		-- Always capitalize code																	 		
		SET @CODE = UPPER(@CODE)
																									  		
		-- UPDATE or INSERT in target table															  
		IF EXISTS(
					SELECT TOP(1) 1 
					FROM global.tCodes
					WHERE 
							ModuleID	= @ModuleID
						AND LanguageID	= @LanguageID
						AND Code		= @CODE
					)
			-- Existing CODE will be updated, not passed Values are keept
			BEGIN
				UPDATE [global].tCodes
				SET 
					 Scope = @Scope
					,[Description] = @Description

				WHERE		ModuleID	= @ModuleID
						AND LanguageID	= @LanguageID
						AND Code		= @CODE ;

				 SET @EffectedRows = @@ROWCOUNT;
				 SET @Resultcode = 200;
			END
		ELSE
			-- Not existing Property will be created, not passed Values with defaults
			BEGIN
				INSERT INTO [global].[tCodes]
				(
					  ModuleID
					 ,LanguageID
					 ,CODE
					 ,Scope
					 ,[Description]

				)
				VALUES 
				(
					 @ModuleID
					,@LanguageID
					,@CODE
					,@Scope
					,@Description
				);

				SET @EffectedRows = @@ROWCOUNT;
				SET @Resultcode = 201;
			END

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spPOST_Code_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spPOST_Code_PostUser @ModuleID,@LanguageID,@CODE,@Scope,@Description
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));
		
		COMMIT TRANSACTION spPOST_Code;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spPOST_Code;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Code'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST Codes. Pre / Post Extension possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the code. Use "system" to POST a code which is not module dependend.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'Language of the code you want to create (ISO Format, like DE for Germany.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CODE';
    SET @value      = N'The code you want to POST.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scope';
    SET @value      = N'The scope of the code. Can group codes for specific uses. Use "default" if not needed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Description';
    SET @value      = N'Free Description of the code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
