DROP PROCEDURE IF EXISTS global.spPOST_Company;
GO

/*
	Procedure to POST a Company, it is called if
		- Companies are created by query the from a sources system
		- Company Properties are edited by the Company edit GUI
		- all optional Parameters are keept properties if skip during sended or sended as NULL
		- Companies for Datasource 0 can be created even that this datasource don't exists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = global.spPOST_Company
				 @Username				= 'SQL'
				,@DataSourceID			= '0'
				,@CompanyID				= 'A'
				,@CompanyName			= 'B GmbH'
				,@Description			= 'Das ist die B GmbH'
				,@FinancialYearBegin	= 1
				,@ChartsOfAccountsID	= 'SKR04'
				,@AccountLength			= 4
				,@Currency				= 'EUR'
				,@CustomValuesJSON		= ''
				,@CustomValuesJSONUser	= ''
				,@OwnerCODE				= 'INTEGRATION'

		PRINT    @RC

		SELECT * FROM global.tCompanies;
		SELECT * FROM system.tDataSources;

		DELETE FROM global.tCompanies WHERE CompanyID = 'A';


	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Company',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Company','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spPOST_Company
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50)
		,@CompanyID				NVARCHAR(50)
		,@CompanyName			NVARCHAR(255)	= NULL
		,@Description			NVARCHAR(2000)	= NULL
		,@FinancialYearBegin	INT				= NULL
		,@ChartsOfAccountsID	NVARCHAR(50)	= NULL
		,@AccountLength			INT				= NULL
		,@Currency				NVARCHAR(50)	= NULL
		,@CustomValuesJSON		NVARCHAR(MAX)	= NULL
		,@CustomValuesJSONUser	NVARCHAR(MAX)	= NULL
		,@OwnerCODE				NVARCHAR(50)	= NULL
    )

WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@DataSourceID AS NVARCHAR(255))		,N'NULL')	+ N','''
				+ ISNULL(@CompanyID									,N'NULL')	+ N''','''
				+ ISNULL(@CompanyName								,N'NULL')	+ N''','''
				+ ISNULL(@Description								,N'NULL')	+ N''','
				+ ISNULL(CAST(@FinancialYearBegin AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@ChartsOfAccountsID						,N'NULL')	+ N','
				+ ISNULL(CAST(@AccountLength AS NVARCHAR(255))		,N'NULL')	+ N','''
				+ ISNULL(@Currency									,N'NULL')	+ N''','''
				+ ISNULL(@CustomValuesJSON							,N'NULL')	+ N''','''
				+ ISNULL(@CustomValuesJSONUser						,N'NULL')	+ N''','''
				+ ISNULL(@OwnerCODE									,N'NULL')	+ N''''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@Message			NVARCHAR(2000)	= N''
				,@RowCount			INT				= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID			= N'';
		IF @CompanyID				IS NULL SET @CompanyID				= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Company
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						SET @Message = CONCAT('Transaction user ',@Username,' don`t exists');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END;
				
				-- check existence of DataSource - the Datasource 0 is not checked, it is used to store companies without Datasource
				IF @DataSourceID NOT IN (SELECT DataSourceID FROM system.tDataSources)	AND @DataSourceID <> '0'
					BEGIN
						SET @ResultCode = 403;
						SET @Message = CONCAT('DataSourceID ',@DataSourceID,' don`t exists');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				-- Value Caching
				DROP TABLE IF EXISTS #Company;
				CREATE TABLE #Company
					(
						 DataSourceID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CompanyID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CompanyName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Description]					NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,FinancialYearBegin				INT											NOT NULL
						,ChartsOfAccountsID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,AccountLength					INT											NOT NULL
						,Currency						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CustomValuesJSON				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CustomValuesJSONUser			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OwnerCODE						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					);

				INSERT INTO #Company
					SELECT
						 DataSourceKey
						,CompanyID
						,CompanyName
						,[Description]
						,FinancialYearBegin
						,ChartsOfAccountsID
						,AccountLength
						,Currency
						,CustomValuesJSON
						,CustomValuesJSONUser
						,OwnerCODE
					FROM global.tCompanies
					WHERE 
							DataSourceKey	= CAST(@DataSourceID  AS INT)  -- temporary Compatibiltity 
						AND CompanyID		= @CompanyID;
				

				-- insert or update Pipeline
				SELECT @RowCount = Count (*) FROM #Company; 
				IF @RowCount = 0
					BEGIN
						
						-- INSERT with default values if NULL
						INSERT INTO global.tCompanies
							(
								DataSourceKey
								,CompanyID
								,CompanyName
								,Description
								,FinancialYearBegin
								,ChartsOfAccountsID
								,AccountLength
								,Currency
								,CustomValuesJSON
								,CustomValuesJSONUser
								,OwnerCODE
							)
						VALUES
							(
								 CAST(@DataSourceID	AS INT) -- temporary Compatibiltity 
								,@CompanyID		
								,ISNULL(@CompanyName			,'<set CompanyName>')
								,ISNULL(@Description			,'')
								,ISNULL(@FinancialYearBegin		,1)
								,ISNULL(@ChartsOfAccountsID		,'')
								,ISNULL(@AccountLength			,4)
								,ISNULL(@Currency				,'EUR')
								,ISNULL(@CustomValuesJSON		,'')
								,ISNULL(@CustomValuesJSONUser	,'')
								,ISNULL(@OwnerCODE				,'INTEGRATION')
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						UPDATE C
								SET 
									 CompanyName			= ISNULL(@CompanyName			,CompanyName				)
									,[Description]			= ISNULL(@Description			,[Description]				)
									,FinancialYearBegin		= ISNULL(@FinancialYearBegin	,FinancialYearBegin			)
									,ChartsOfAccountsID		= ISNULL(@ChartsOfAccountsID	,ChartsOfAccountsID			)
									,AccountLength			= ISNULL(@AccountLength			,AccountLength				)
									,Currency				= ISNULL(@Currency				,Currency					)
									,CustomValuesJSON		= ISNULL(@CustomValuesJSON		,CustomValuesJSON			)
									,CustomValuesJSONUser	= ISNULL(@CustomValuesJSONUser	,CustomValuesJSONUser		)
									,OwnerCODE				= ISNULL(@OwnerCODE				,OwnerCODE					)
							FROM global.tCompanies C
							WHERE 
								C.DataSourceKey	= CAST(@DataSourceID AS INT)
							AND C.CompanyID		= @CompanyID;

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

			COMMIT TRANSACTION spPOST_Company
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Company	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- NO GRANT as EXECUTE as 'dbo'
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Company'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure save / edit Companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'Integer number of the DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'ID of the Company, must be unique in this DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyName';
    SET @value      = N'Name of this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Description';
    SET @value      = N'Description of the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FinancialYearBegin';
    SET @value      = N'Month of the start of the financial year - e.g. 1 for January.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ChartsOfAccountsID';
    SET @value      = N'ID of the current Chart of Accounts, this ID must match the ID in Table global.tFIN_ChartsOfAccounts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@AccountLength';
    SET @value      = N'Integer Value for the length of the AccountID - usually between 4 and 8.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Currency';
    SET @value      = N'ISO Code of the currency of the company, e.g. EUR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CustomValuesJSON';
    SET @value      = N'Custom field to store any informations as JSON Array.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CustomValuesJSONUser';
    SET @value      = N'Custom field to store any informations as JSON Array, may have rights difference for reading.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OwnerCode';
    SET @value      = N'CODE Word who the owner of this company is - INTEGRATION or PLANNING.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
