﻿
DROP PROCEDURE IF EXISTS import.spPOST_dProducts;
GO

/*
	Import all values from load.tdProducts with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_dProducts '','','','';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProducts',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProducts','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dProducts
				(		 @Username		AS NVARCHAR(255)
						,@FactoryID		AS NVARCHAR(255)
						,@ProductLineID	AS NVARCHAR(255)
						,@ProductID		AS NVARCHAR(255)			
						,@Source		AS NVARCHAR(255)		
				)		  
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @TimeType					NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(MAX)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ResponsiblePerson			NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @Status						NVARCHAR(255)
	DECLARE @Template					NVARCHAR(255)
	DECLARE @TemplateVersion			NVARCHAR(255)
	DECLARE @GA1 						NVARCHAR(255)
	DECLARE @GA2 						NVARCHAR(255)
	DECLARE @GA3 						NVARCHAR(255)
	DECLARE @GA4 						NVARCHAR(255)
	DECLARE @GA5 						NVARCHAR(255)
	DECLARE @GA6 						NVARCHAR(255)
	DECLARE @GA7 						NVARCHAR(255)
	DECLARE @GA8 						NVARCHAR(255)
	DECLARE @GA9 						NVARCHAR(255)
	DECLARE @GA10						NVARCHAR(255)
	DECLARE @GA11						NVARCHAR(255)
	DECLARE @GA12						NVARCHAR(255)
	DECLARE @GA13						NVARCHAR(255)
	DECLARE @GA14						NVARCHAR(255)
	DECLARE @GA15						NVARCHAR(255)
	DECLARE @GA16						NVARCHAR(255)
	DECLARE @GA17						NVARCHAR(255)
	DECLARE @GA18						NVARCHAR(255)
	DECLARE @GA19						NVARCHAR(255)
	DECLARE @GA20						NVARCHAR(255)
	DECLARE @GA21						NVARCHAR(255)
	DECLARE @GA22						NVARCHAR(255)
	DECLARE @GA23						NVARCHAR(255)
	DECLARE @GA24						NVARCHAR(255)
	DECLARE @GA25						NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TimeType				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ResponsiblePerson		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Status					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Template				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TemplateVersion			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA1 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA2 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA3 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA4 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA5 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA6 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA7 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA8 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA9 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA10					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA11					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA12					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA13					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA14					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA15					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA16					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA17					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA18					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA19					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA20					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA21					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA22					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA23					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA24					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA25					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL		
			 )

	INSERT INTO #tmp
		SELECT   tdP.ProductID
				,tdP.ProductLineID			
				,tdP.FactoryID
				,tdP.TimeType				
				,tdP.NameShort				
				,tdP.NameLong				
				,tdP.CommentUser			
				,tdP.CommentDev			
				,tdP.ResponsiblePerson		
				,tdP.ImageName
				,tdP.Status				
				,tdP.Template
				,tdP.TemplateVersion		
				,tdP.GlobalAttribute1 
				,tdP.GlobalAttribute2 
				,tdP.GlobalAttribute3 
				,tdP.GlobalAttribute4 
				,tdP.GlobalAttribute5 
				,tdP.GlobalAttribute6 
				,tdP.GlobalAttribute7 
				,tdP.GlobalAttribute8 
				,tdP.GlobalAttribute9 
				,tdP.GlobalAttribute10
				,tdP.GlobalAttribute11
				,tdP.GlobalAttribute12
				,tdP.GlobalAttribute13
				,tdP.GlobalAttribute14
				,tdP.GlobalAttribute15
				,tdP.GlobalAttribute16
				,tdP.GlobalAttribute17
				,tdP.GlobalAttribute18
				,tdP.GlobalAttribute19
				,tdP.GlobalAttribute20
				,tdP.GlobalAttribute21
				,tdP.GlobalAttribute22
				,tdP.GlobalAttribute23
				,tdP.GlobalAttribute24
				,tdP.GlobalAttribute25
		FROM	load.tdProducts tdP
		WHERE	(@FactoryID = ''		OR tdP.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdP.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tdP.ProductID = @ProductID)
		 AND    (@Source = ''			OR tdP.Source = @Source)
	 
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Product
				 @Username
				,@ProductID
				,@ProductLineID				
				,@FactoryID			
				,@TimeType			
				,@NameShort			
				,@NameLong			
				,@CommentUser		
				,@CommentDev			
				,@ResponsiblePerson	
				,@ImageName			
				,@Status				
				,@Template			
				,@TemplateVersion	
				,@GA1 				
				,@GA2 				
				,@GA3 				
				,@GA4 				
				,@GA5 				
				,@GA6 				
				,@GA7 				
				,@GA8 				
				,@GA9 				
				,@GA10				
				,@GA11				
				,@GA12				
				,@GA13				
				,@GA14				
				,@GA15				
				,@GA16				
				,@GA17				
				,@GA18				
				,@GA19				
				,@GA20				
				,@GA21				
				,@GA22				
				,@GA23				
				,@GA24				
				,@GA25	
					
		Print @ResultCode
	FETCH MyCursor INTO  @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dProducts'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tdProducts with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
