﻿
DROP PROCEDURE IF EXISTS import.spPOST_fValuesPDTV;
GO
/*
	Import values from load.tfValues with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9

	Testcall
		EXEC import.spPOST_fValuesPDTV 'SQL','','','','',0,'';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValuesPDTV',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValuesPDTV','PARAMETER',NULL)
*/

CREATE PROCEDURE import.spPOST_fValuesPDTV
						 @Username 			AS NVARCHAR(255)
						,@FactoryID 		AS NVARCHAR(255)
						,@ProductLineID 	AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID 	AS NVARCHAR(255)
						,@TimeID			AS INT = 0
						,@Source			AS NVARCHAR(255)		
				
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall						DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName						NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode							INT				= 501;
	DECLARE @EffectedRows						INT				= 0;					
	DECLARE @TransactUsername					NVARCHAR(255)	= N'';
	DECLARE @ParameterString					NVARCHAR(MAX)	= N''''
																+ ISNULL(@Username							, N'NULL')	 + N''',''' 
																+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
																+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
																+ ISNULL(@ProductID							, N'NULL')	 + N''','''
																+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
																+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
																+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment							NVARCHAR(2000) = N'';	
	DECLARE @IsIncrementalValuesFlag			INT;
	DECLARE	@ValuesInBracketsCommaSeparated 	NVARCHAR(MAX);
	DECLARE @ToDoList AS TABLE
		(	 ProductID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID 						NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,FactoryID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,IsIncrementalValuesFlag 			INT				NOT NULL
			,ValuesInBracketsCommaSeparated 	NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
		);
				
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	-- Temporäre Tabelle als Datensammler
	DROP TABLE IF EXISTS #DATA;
	CREATE TABLE #DATA	
			(
				 ProductID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ProductLineID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,TimeID				INT										NOT NULL
				,ValueFormula		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueINT			BIGINT									NOT NULL
				,ValueText			NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueComment		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			);

	INSERT INTO #DATA
			SELECT   tfV.ProductID		
					,tfV.ProductLineID	
					,tfV.FactoryID		
					,tfV.ValueSeriesID	
					,tfV.TimeID			
					,tfV.ValueFormula	
					,tfV.ValueINT		
					,REPLACE(tfV.ValueText,'''','')
					,REPLACE(tfV.ValueComment,'''','')	
			FROM	load.tfValues tfV
			WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
			  AND 	(@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
			  AND	(@ProductID = ''		OR tfV.ProductID = @ProductID)
			  AND	(@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
			  AND   (@TimeID = 0			OR tfV.TimeID = @TimeID)
			  AND   (@Source = ''			OR tfV.Source = @Source);


	-- Tabelle für String
	INSERT INTO @ToDoList
		SELECT	 d.ProductID
				,d.ProductLineId
				,d.FactoryID
				,1
				,(SELECT  N'[(''' 
					+ dxml.ValueSeriesID								+ N''','  
					+ CAST(dxml.TimeID AS NVARCHAR(255))				+ N',''' 
					+ dxml.ValueFormula									+ N''',''' 
					+ CAST(dxml.ValueInt AS NVARCHAR(255))				+ N''',''' 
					+ calc.sfReplaceNonXMLCharacters(dxml.ValueText)	+ N''',''' 
					+ calc.sfReplaceNonXMLCharacters(dxml.ValueComment) + N''')],'
				FROM #DATA dxml
				WHERE d.FactoryID = dxml.FactoryID AND d.ProductLineID = dxml.ProductLineID AND d.ProductID = dxml.ProductID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS ValuesInBracketsCommaSeparated
		FROM #DATA d
		GROUP BY   d.ProductID, d.ProductLineId, d.FactoryID;

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR STATIC FOR

		SELECT tdl.ProductID
			  ,tdl.ProductLineID
			  ,tdl.FactoryID
			  ,tdl.IsIncrementalValuesFlag
			  ,''+LEFT(tdl.ValuesInBracketsCommaSeparated,LEN(tdl.ValuesInBracketsCommaSeparated)-1)+''
		FROM @ToDoList tdl

	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_ProductDataTableValues
			 @UserName
			,@ProductID		
			,@ProductLineID	
			,@FactoryID		
			,@IsIncrementalValuesFlag
			,@ValuesInBracketsCommaSeparated	
											
	Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_fValuesPDTV TO pf_PlanningFactoryUser;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_fValuesPDTV'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Import values from load.tfValues with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
