﻿
DROP PROCEDURE IF EXISTS integration.pGetLastDeltaValues;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'integration', 'PROCEDURE', 'pGetLastDeltaValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'integration', 'PROCEDURE', 'pGetLastDeltaValues','PARAMETER',NULL)
*/
  
CREATE PROCEDURE integration.pGetLastDeltaValues
    @DataSourceKey int
  , @CompanyID     nvarchar(50)
  , @Module        nvarchar(50)
  , @Username      nvarchar(255)
AS
BEGIN
    DECLARE @SQLCommand nvarchar(255);
    DECLARE @ResultCode int = 501;

    BEGIN TRY

        -- execute sync procedure for the selected solution
        IF EXISTS
            (
            SELECT *
            FROM sysobjects
            WHERE
                id = OBJECT_ID(N'integration.pGetDeltaValues' + @Module)
                AND OBJECTPROPERTY(id, N'IsProcedure') = 1
            )
        BEGIN
            SET @SQLCommand = N'integration.pGetDeltaValues' + @Module;
            EXEC @ResultCode = @SQLCommand @DataSourceKey, @CompanyID, @Username;
        END;
        ELSE
            -- if no pGetDeltaValues procedure exists for module, nothing is executed and ResultCode 404 is returned
            SET @ResultCode = 404;

    END TRY
    BEGIN CATCH

        BEGIN
            SET @ResultCode = 500;
        END;

    END CATCH;

    RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'integration'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGetLastDeltaValues'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to get last Delta Load Timestamp per Acounting Area';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@DataSourceKey';
    SET @value      = N'DataSourceKey';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'CompanyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
