
DROP PROCEDURE IF EXISTS planning.spCOPY_Tab;
GO

/*
	Procedure to copy a Tab inside a object or to another target
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spCOPY_Tab
				 @Username					= 'W10\admin'
				,@SourceFactoryID			= 'ZT'
				,@SourceProductLineID		= 'U'
				,@SourceProductID			= ''
				,@SourceTabID				= '4'
				,@TargetFactoryID			= 'ZT'
				,@TargetProductLineID		= ''
				,@TargetProductID			= ''
				,@TargetTabID				= 'T48'
				,@TargetOrderIndex			= 1
				,@TargetTabName				= 'Test2';

		PRINT @RC;
		SELECT * FROM planning.tTabs ORDER BY FactoryID, ProductlineID, OrderIndex;

		TRUNCATE TABLE planning.tTabs;

		SELECT TOP 5 * FROM planning.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Tab','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spCOPY_Tab
         @Username					NVARCHAR(255)
		,@SourceFactoryID			NVARCHAR(255) = ''				
		,@SourceProductLineID		NVARCHAR(255) = ''
		,@SourceProductID			NVARCHAR(255) = ''
		,@SourceTabID				NVARCHAR(255)			-- not optional, needed in all levels
		,@TargetFactoryID			NVARCHAR(255) = ''				
		,@TargetProductLineID		NVARCHAR(255) = ''
		,@TargetProductID			NVARCHAR(255) = ''
		,@TargetTabID				NVARCHAR(255)			-- not optional, needed in all levels
		,@TargetOrderIndex			INT			  = 1
		,@TargetTabName				NVARCHAR(255)
   
AS 
	BEGIN
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username					,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceFactoryID			,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceProductLineID		,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceProductID			,N'NULL')	+ N''','''
				+ ISNULL(@SourceTabID				,N'NULL')	+ N''','''
				+ ISNULL(@TargetFactoryID			,N'NULL')	+ N''',''' 
				+ ISNULL(@TargetProductLineID		,N'NULL')	+ N''',''' 
				+ ISNULL(@TargetProductID			,N'NULL')	+ N''','''
				+ ISNULL(@TargetTabID				,N'NULL')	+ N''','
				+ CAST(	
					ISNULL(@TargetOrderIndex 		,N'NULL')
					AS NVARCHAR(255))							+ N','''
				+ ISNULL(@TargetTabName				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- Procedure specific declarations
		DECLARE  @SourceFactoryKey		INT			= 0
				,@SourceProductLineKey	INT			= 0
				,@SourceProductKey		INT			= 0
				,@TargetFactoryKey		INT			= 0
				,@TargetProductLineKey	INT			= 0
				,@TargetProductKey		INT			= 0
				,@RowKey				BIGINT		= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
		IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
		IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
		IF @SourceTabID				IS NULL SET @SourceTabID			= N'';
		IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
		IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
		IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
		IF @TargetTabID				IS NULL SET @TargetTabID			= N'';
		IF @TargetOrderIndex		IS NULL SET @TargetOrderIndex		= 1;
		IF @TargetTabName			IS NULL SET	@TargetTabName			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID	(@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID	(@SourceProductLineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID	(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID	(@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID	(@TargetProductLineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID	(@TargetProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spCOPY_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @SourceFactoryID		= '' SET @SourceFactoryKey		= -1;
				IF @SourceProductLineID	= '' SET @SourceProductLineKey	= -1;
				IF @SourceProductID		= '' SET @SourceProductKey		= -1;
				IF @TargetFactoryID		= '' SET @TargetFactoryKey		= -1;
				IF @TargetProductLineID	= '' SET @TargetProductLineKey	= -1;
				IF @TargetProductID		= '' SET @TargetProductKey		= -1;

				SELECT @SourceFactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @SourceFactoryID													AND @SourceFactoryKey <> -1;
				SELECT @SourceProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @SourceFactoryKey		AND ProductLineID	= @SourceProductLineID	AND @SourceProductLineKey <> -1;
				SELECT @SourceProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @SourceProductLineKey	AND ProductID		= @SourceProductID		AND @SourceProductKey <> -1;
				SELECT @TargetFactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @TargetFactoryID													AND @TargetFactoryKey <> -1;
				SELECT @TargetProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @TargetFactoryKey		AND ProductLineID	= @TargetProductLineID	AND @TargetProductLineKey <> -1;
				SELECT @TargetProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @TargetProductLineKey	AND ProductID		= @TargetProductID		AND @TargetProductKey <> -1;

				-- error if an ID dont exists
				IF		@SourceFactoryKey		= 0 
					OR	@SourceProductLineKey	= 0
					OR  @SourceProductKey		= 0
					OR	@TargetFactoryKey		= 0 
					OR	@TargetProductLineKey	= 0
					OR  @TargetProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if SourceTabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @SourceFactoryKey
									AND ProductLineKey	= @SourceProductLineKey
									AND ProductKey		= @SourceProductKey
									AND TabID			= @SourceTabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('Source TabID not found.', 16, 10);
							END;

				-- tab with ID 0 cant be moved
				IF @SourceTabID= '0' OR @TargetTabID = '0'
					BEGIN
							SET @ResultCode = 403;
							RAISERROR('Tab with ID 0 can''t be copied', 16, 10);
					END

				-- error if ID have gaps in the line
				IF		@SourceFactoryKey		= -1 AND (@SourceProductLineKey <> -1 OR @SourceProductKey <> -1)
					OR	@SourceProductLineKey	= -1 AND  @SourceProductKey <> -1 
					OR	@TargetFactoryKey		= -1 AND (@TargetProductLineKey <> -1 OR @TargetProductKey <> -1)
					OR	@TargetProductLineKey	= -1 AND  @TargetProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level on Source
				IF @SourceFactoryKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster read.', 16, 10);
								END 
					END

				-- check user rights for Factory level on Source
				IF  @SourceFactoryKey <> -1 AND @SourceProductLineKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername,@SourceFactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory read.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level on Source
				IF @SourceFactoryKey <> -1 AND @SourceProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername,@SourceFactoryID,@SourceProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product read.', 16, 10);
								END 
					END

				-- check user rights for cluster level on Target
				IF @TargetFactoryKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level on Target
				IF @TargetFactoryKey <> -1 AND @TargetProductLineKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername,@TargetFactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level on Target
				IF @TargetFactoryKey <> -1 AND @TargetProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername,@TargetFactoryID,@TargetProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- determine if posted Tab already exists
				SELECT 
					 @Rowkey = RowKey
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @TargetFactoryKey
					AND ProductLineKey	= @TargetProductLineKey
					AND ProductKey		= @TargetProductKey
					AND TabID			= @TargetTabID

				-- data operation starts	***********************************************************************************
				-- delete if exits, to determine Resultcode
				IF @RowKey > 0 
					BEGIN
						BEGIN
							SET @ResultCode = 401
							RAISERROR('Target TabID already exists in Target.', 16, 10);
						END 
					END

				-- Insert new Tab
				INSERT INTO planning.tTabs
					SELECT
						 @TargetFactoryKey
						,@TargetProductLineKey
						,@TargetProductKey
						,@TargetFactoryID
						,@TargetProductLineID
						,@TargetProductID
						,@TargetTabID
						,@TargetOrderIndex   -- TODO: Must be set to right index
						,@TargetTabName
						,TabHint
						,TabText
						,PresentationCODE
						,Datasource
						,IsVisibleBOOL
						,LayoutJSON
						,ParameterJSON

					FROM planning.tTabs

					WHERE 
							FactoryKey		= @SourceFactoryKey
						AND ProductLineKey	= @SourceProductLineKey
						AND ProductKey		= @SourceProductKey
						AND TabID			= @SourceTabID

					-- Increment all Tabs after the inserted Tab and the Tab with the same OrderIndex
					UPDATE planning.tTabs
					SET OrderIndex = Orderindex + 1
					WHERE 
							FactoryKey		= @TargetFactoryKey
						AND ProductLineKey	= @TargetProductLineKey
						AND ProductKey		= @TargetProductKey
						AND Orderindex		>= @TargetOrderIndex
						AND TabID			<> @TargetTabID;

					-- eliminate all Gaps in Tab OrderIndex
					UPDATE planning.tTabs 
					SET OrderIndex = Rownumbers.Number
					FROM planning.tTabs tT INNER JOIN (
																SELECT 
																	 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																	,TabID
																	
																FROM planning.tTabs
																WHERE 
																		FactoryKey		= @TargetFactoryKey
																	AND ProductLineKey	= @TargetProductLineKey
																	AND ProductKey		= @TargetProductKey

														) AS Rownumbers
					ON 
							tT.TabID			= Rownumbers.TabID
						AND tT.FactoryKey		= @TargetFactoryKey
						AND tT.ProductLineKey	= @TargetProductLineKey
						AND tT.ProductKey		= @TargetProductKey;

				SET @ResultCode = 201;

			COMMIT TRANSACTION spCOPY_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spCOPY_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spCOPY_Tab'		-- enter procedure name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Copy a Tab - inside an object or to a different. The copied tab is always puted at the end.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceTabID';
    SET @value      = N'SourceBusiness Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetTabID';
    SET @value      = N'TargetBusiness Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
