
DROP PROCEDURE IF EXISTS planning.spDELETE_Tab;
GO

/*
	Procedure to delete a Tab
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spDELETE_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= 'T7'

		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex

		TRUNCATE TABLE planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_Tab','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spDELETE_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255)	
    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Tab;
				IF	@TabID	= '0' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Tab with ID 0 can''t be deleted.', 16, 10);
					END;

				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;

				--PRINT @FactoryKey 
				--PRINT @ProductLineKey
				--PRINT @ProductKey

				-- error if an ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if TabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey							-- its not possible to assign this rowkey to @Rowkey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @TabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('TabID not found.', 16, 10);
							END;
				
				-- RowKey ermitteln
				SELECT 
						@RowKey=RowKey
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- error if ID have gaps in the line
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <>  -1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- data operation starts	***********************************************************************************

				-- Insert new Tab
				DELETE FROM planning.tTabs
				WHERE
					RowKey		= @RowKey;

				-- eliminate all Gaps
				UPDATE planning.tTabs 
				SET OrderIndex = Rownumbers.Number
				FROM planning.tTabs tT INNER JOIN (
															SELECT 
																 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																,TabID
																	
															FROM planning.tTabs
															WHERE 
																	FactoryKey		= @FactoryKey
																AND ProductLineKey	= @ProductLineKey
																AND ProductKey		= @ProductKey

													) AS Rownumbers
				ON 
						tT.TabID			= Rownumbers.TabID
					AND tT.FactoryKey		= @FactoryKey
					AND tT.ProductLineKey	= @ProductLineKey
					AND tT.ProductKey		= @ProductKey;

				SET @ResultCode = 200;

			COMMIT TRANSACTION spDELETE_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Tab'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name; 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if DELETE is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if DELETE is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if DELETE is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'TabID which is deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
