﻿
DROP PROCEDURE IF EXISTS planning.spGET_FileList;
GO

/*
	Procedure to Return list of existing files on a product
		-Right Check missing
		-Key Determination missing

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	Testcall
		EXEC planning.spGET_FileList
			'SQL'
			,''
			,''
			,''
			,'Rootfolder/Subfolder2';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_FileList',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_FileList','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_FileList
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@Filepath		NVARCHAR(4000)	= N'' -- this may be the full filepath or the start of it

AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username		, N'NULL') + N''', '''
													+ ISNULL(@FactoryID		, N'NULL') + N''', '''
													+ ISNULL(@ProductLineID	, N'NULL') + N''', '''
													+ ISNULL(@ProductID		, N'NULL') + N''', '''
													+ ISNULL(@Filepath		, N'NULL')
													+ N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY

			SET @Username = dbo.sx_pf_pProtectString (@Username);

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
					
			SELECT
				 FactoryID
				,ProductLineID
				,ProductID
				,FileID
				,[FileName] 
				,[Filename] + ISNULL(N'.' + FileExtension, N'') AS 'FileNameWithExtension'
				,FileExtension
				,UploadedBy
				,UploadedAt
				,Filepath
			FROM planning.tFiles
			WHERE 	
					FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID
				AND ProductID		= @ProductID
				AND (
						Filepath LIKE @Filepath + N'%' -- to cover starting with this path
						OR 
						@Filepath = N''
					)

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_FileList TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_FileList'					-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Return list of existing files on a product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Filepath';
    SET @value      = N'This may be the full filepath or the start of it for a like search.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
