﻿
DROP PROCEDURE IF EXISTS planning.spGET_Index_JSON;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Procedure to GET the Structure of Planning Objects
	FactoryID and ProductLineID can be passed in as single Value or as empty string for all

	Output Limit may be F/PL/P/VS

	Testcall
		DECLARE @ParameterJSON NVARCHAR(MAX);
		SET @ParameterJSON = N'{ "connector_type":"read"
								,"version":1
								,"FactoryID":"ZT"		
								,"ProductLineID":"U"	
								,"ProductID":"1"
								,"OutputLimit" : "VS"  
								}
					'
		EXEC planning.spGET_Index_JSON 'SQL',@ParameterJSON

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Index_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Index_JSON','PARAMETER',NULL)

*/
   
CREATE PROCEDURE planning.spGET_Index_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ParameterJSON	NVARCHAR(MAX) = ''

AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables
		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username, N'NULL')		+ N''', '''
														+ ISNULL(@ParameterJSON, N'NULL')	+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';					
			
		-- Procedure Variables
		DECLARE  @StartTime		TIME(4)
				,@ProcessTime	INT
				,@FactoryID		NVARCHAR(255)
				,@ProductLineID NVARCHAR(255)
				,@ProductID		NVARCHAR(255)
				,@OutputLimit	NVARCHAR(255) = ''

		BEGIN TRY

			SET @StartTime = CONVERT (time, SYSUTCDATETIME()) ; 

			-- Test for valid JSON ################################################################################
			IF ISJSON(@ParameterJSON) = 0 
				BEGIN
					PRINT 'Exit due to invalid JSON'
					RETURN 403
				END
	
			-- Extract Parameter FROM JSON ######################################################################
			
			SET @FactoryID		= JSON_Value(@ParameterJSON,'$.FactoryID')		COLLATE DATABASE_DEFAULT;
			SET @ProductLineID	= JSON_Value(@ParameterJSON,'$.ProductLineID')	COLLATE DATABASE_DEFAULT;
			SET @ProductID		= JSON_Value(@ParameterJSON,'$.ProductID')		COLLATE DATABASE_DEFAULT;
			SET @OutputLimit	= JSON_Value(@ParameterJSON,'$.OutputLimit')	COLLATE DATABASE_DEFAULT;

			IF @OutputLimit = N'' OR @OutputLimit IS NULL SET @OutputLimit = N'VS'
		
			-- Check Right and Keys as valid  #################################################################

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Check Rights and flag invalid rows

			-- Invalid IDs get 404 flag

				-- Output als Recordset #################################################################
				IF @OutputLimit = N'VS' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25
							,dVS.ValueSeriesID
							,dVS.[IsNumeric]		
							,dVS.NameShort			AS ValueSeriesName
							,dVS.ValueSource
							,dVS.ValueListID
							,dVS.Scale
							,dVS.Unit
							,dVS.Effect
							,dVS.VisibilityLevel
							,dVS.ValueSeriesNo

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductLineKey = dP.ProductLineKey

							LEFT JOIN planning.tdValueSeries dVS
								ON dP.ProductKey = dVS.ProductKey

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)

							AND 
							(
								@ProductID = N''
								OR
								@ProductID = dP.ProductID
							)

							ORDER BY dF.FactoryID, dPL.ProductLineID, dP.ProductID,dVS.ValueSeriesID

							SET @EffectedRows = @@ROWCOUNT;
					END

				IF @OutputLimit = N'P' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductLineKey = dP.ProductLineKey

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)
							AND 
							(
								@ProductID = N''
								OR
								@ProductID = dP.ProductID
							)

						ORDER BY dF.FactoryID, dPL.ProductLineID, dP.ProductID

						SET @EffectedRows = @@ROWCOUNT;
					END

				IF @OutputLimit = N'PL' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey
						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)

						ORDER BY dF.FactoryID, dPL.ProductLineID
					END

				IF @OutputLimit = N'F' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName

						FROM planning.tdFactories dF

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
						ORDER BY dF.FactoryID

						SET @EffectedRows = @@ROWCOUNT;

					END

				SET @ResultCode = 200;

				-- Runtime, at the moment without output
				SET @ProcessTime = DATEDIFF(ms,@StartTime,CONVERT (time, SYSUTCDATETIME()));

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN 200;
			
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_Index_JSON TO pf_PlanningFactoryUser;
GO
			
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Index_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to GET the Structure of Planning Objects
         FactoryID and ProductLineID can be passed in as single Value or as empty string for all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ParameterJSON';
    SET @value      = N'ParameterJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
