
DROP PROCEDURE IF EXISTS planning.spGET_Tree;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
  
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = planning.spGET_Tree
				 @Username		= 'SQL'
				,@Searchstring	= ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tree','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spGET_Tree
    (
         @Username				NVARCHAR(255)
        ,@Searchstring			NVARCHAR(255)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@Searchstring				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @Searchstring			IS NULL SET @Searchstring		= N'';

		SET @Searchstring = CONCAT('%',@Searchstring,'%');

		-- START TRY			***********************************************************************************
		BEGIN TRY

				PRINT CONCAT(N'Determine Righs of this user',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- get ProductLine rights for this user
				DROP TABLE IF EXISTS #ProductLineRights;
				CREATE TABLE #ProductLineRights
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReadCommentMandatory	INT											NOT NULL
						,WriteCommentMandatory	INT											NOT NULL

						CONSTRAINT PK_ProductLineRights PRIMARY KEY CLUSTERED (RowKey)
					);

				INSERT INTO #ProductLineRights
					SELECT 
						 FactoryID				
						,ProductLineID			
						,[Right]				
						,ReadCommentMandatory
						,WriteCommentMandatory
					FROM system.trUserRights
					WHERE 
							UserName = @TransactUsername
						AND FactoryID <> '';

				-- get Factory rights for this user (separat table for simple join later)
				DROP TABLE IF EXISTS #FactoryRights;
				CREATE TABLE #FactoryRights
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_FactoryRights PRIMARY KEY CLUSTERED (RowKey)
					);

				INSERT INTO #FactoryRights
					SELECT
						 FactoryID							
						,[Right]				
					FROM system.trUserRights
					WHERE 
							UserName		= @TransactUsername
						AND ProductLineID	= '';

				-- Data Transaction
				PRINT CONCAT(N'Ausgabe Tree',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					 dF.FactoryID								AS FactoryID
					,dF.NameShort								AS FactoryNameShort
					,dF.ImageName								AS FactoryImageName
					,RF.[Right]									AS FactoryRight
					,COALESCE(dPL.ProductLineID			,'')	AS ProductLineID
					,COALESCE(dPL.NameShort				,'')	AS ProductLineNameShort
					,COALESCE(dPL.ImageName				,'')	AS ProductLineImageName
					,COALESCE(RPL.[Right]				,'')	AS ProductLineRight
					,COALESCE(dP.ProductID				,'')	AS ProductID
					,COALESCE(dP.NameShort				,'')	AS ProductNameShort
					,COALESCE(dP.ImageName				,'')	AS ProductImageName
					,COALESCE(dP.Template				,'')	AS ProductTemplate
					,COALESCE(RPL.[Right]				,'')	AS ProductRight        --same as PL, but will become use later
					,COALESCE(RPL.ReadCommentMandatory	,0)		AS ReadCommentMandatory
					,COALESCE(RPL.WriteCommentMandatory	,0)		AS WriteCommentMandatory

				FROM planning.tdFactories dF
					LEFT JOIN planning.tdProductLines dPL
						ON dPL.FactoryKey = dF.FactoryKey
					LEFT JOIN planning.tdProducts dP
						ON dP.ProductLineKey = dPL.ProductLineKey

					-- Rights on FactoryLevel
					INNER JOIN #FactoryRights RF
						ON		dF.FactoryID		= RF.FactoryID
		
					-- Rights on ProductLineLevel
					INNER JOIN #ProductLineRights RPL
						ON 
								 dF.FactoryID		= RPL.FactoryID
							AND (dPL.ProductLineID	= RPL.ProductLineID OR dPL.ProductLineID IS NULL) -- maybe there is no ProductLine under this Factory, then the Factory must not be filtered.

					-- Filtering due to search string
				WHERE 
					(
							dF.FactoryID		LIKE @Searchstring 
						OR	dF.NameShort		LIKE @Searchstring
						OR  dPL.ProductLineID	LIKE @Searchstring
						OR  dPL.NameShort		LIKE @Searchstring
						OR  dP.ProductID		LIKE @Searchstring
						OR  dP.NameShort		LIKE @Searchstring

					)

					OR @Searchstring = ''

				ORDER BY -- primary sort numeric, fallback to sort alphabetic
					 ISNULL(TRY_CAST(dF.FactoryID			AS INT),99999999), dF.FactoryID
					,ISNULL(TRY_CAST(dPL.ProductLineID		AS INT),99999999), dPL.ProductLineID
					,ISNULL(TRY_CAST(dP.ProductID			AS INT),99999999), dP.ProductID;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT EXECUTE ON OBJECT :: [planning].[spGET_Tree] TO [Username or rolename]

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Tree'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Returns the Tree of all planning object - Factories/ProductLines/Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Searchstring';
    SET @value      = N'String to filter the output values - will be handled as %Searchstring% search.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
