﻿<#
.SYNOPSIS
    Upload file using [dbo].[test_POST_File] stored procedure
.DESCRIPTION
    If [filekey] parameter is set exists file will be rewrited, otherwise will be created new file.
    File extension extracted from [filepath] parameter.
    After execution output stored procedure result table or error message and return code:
        0 - successful
        -1 - File read error
        -2 - File is empty
        -3 - Sql connection open error or sql command creating error or stored procedure call error
        -4 - Result read error
        -5 - Stored procedure return no result 
.PARAMETER filepath
    Path to file for upload.
.PARAMETER filekey
    Key of exists file.
.EXAMPLE
    UploadFile.ps1 c:\test.txt
.EXAMPLE
    UploadFile.ps1 c:\test.txt 1

    !! Adjust to FID,PLID,PID
#>

param (
    [parameter(Position=0, Mandatory=$true, HelpMessage="Path to file")][ValidateNotNullOrEmpty()][String]$filepath,
    [parameter(Position=1, Mandatory=$false)][Int64]$filekey
)

#############################################################################################
$SqlServer = "SET SQL SERVER";
$SqlCatalog = "SET DB NAME";
$User = "SET SQL DB USER";
$Password = "SET SQL DB USER PASSWORD";
$DBUsername = 'SET PLANNING DB USERNAME'
#############################################################################################

try
{
    $fileBody = [System.IO.File]::ReadAllBytes($filepath)
}
catch
{
    Write-Host $_
    return -1
}

If ($fileBody.Length -lt 1)
{
    Write-Host "File is empty"
    return -2
}

$FileExtension = [System.IO.Path]::GetExtension($filepath).trimstart(".") 

$SqlConnection = New-Object System.Data.SqlClient.SqlConnection
$SqlConnection.ConnectionString = "Data Source=$SqlServer; Authentication=Active Directory Password; Initial Catalog=$SqlCatalog; UID=$User; PWD=$Password"

try 
{
    $SqlConnection.Open()
    $SqlCmd = $SqlConnection.CreateCommand()
    $SqlCmd.CommandType = [System.Data.CommandType]::StoredProcedure
    $SqlCmd.CommandText = "dbo.test_POST_File"

    $p = $SqlCmd.Parameters.AddWithValue("@Username", $DBUsername);
    $p = $SqlCmd.Parameters.AddWithValue("@FileExtension", $FileExtension)
    
    $p = $p = $SqlCmd.Parameters.AddWithValue("@FileBody", $fileBody)
    $p.DbType = [System.Data.DbType]::Binary
    $p.Size = $fileBody.Length

    If ($PSBoundParameters.ContainsKey("filekey"))
    {
        $p = $SqlCmd.Parameters.AddWithValue("@FileKey", $filekey)
    }
        
    $Reader = $SqlCmd.ExecuteReader()

    try
    {   
        If (!$Reader.HasRows)
        {
            Write-Host "Server do not return any result"
            return -5;
        }
            
        while ($Reader.read()) 
        {
            Write-Host "FileKey="$Reader.GetValue(0)", FileName="$Reader.GetValue(1)
        }      
    }
    catch
    {
        Write-Host $_
        return -4
    }
    finally
    {
        $Reader.Close()
    }

    return 0
}
catch
{
    Write-Host $_
    return -3
}
finally
{
    $SqlConnection.Close()
}
   
