﻿
DROP PROCEDURE IF EXISTS planning.spPOST_Products_JSON;
GO

/*
	Procedure to POST Products in JSON Format 
	Preconditions
		- A value can be sended only on full qualified single product -> There is no write on summary level possible
		- multiple values sended for one product -> the last wins

	Functionality:
		- its only possible to change the attributes of an existing products - creating new products is not possible
		- not sended attributes must be sended as NULL
		- to delete an attribute value, an empty string must be sended 
		- user can send only values to products with write rights
		- the Procedure returns all sended products as recordset with the process result

	Workflow
		- put all sended records in a temporary table (SendedRecords)
		- create a temporary classification table with all sended Products (SendedRecordClassification)
			- determine the UserRights in Classification table
			- flag not existing Products
		- return SendedRecords

	Gerd Tautenhahn / Maria Rüdger for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC INT;
		DECLARE @ValuesJSON NVARCHAR(MAX);
			SET @ValuesJSON = N'{

				"connector_type":"write","version":1,"Values":[

				 {"ProductNumber":1	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"NameShort":null		,"CommentUser":""		,"ResponsiblePerson":null	,"ImageName":""		,"Status":""		,"Template":"Test_VM"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":2	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"2"	,"NameShort":""			,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":3	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"3"	,"NameShort":"K3"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":null		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":4	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"4"	,"NameShort":"BETR"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test_VM"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":5	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"4"	,"NameShort":"BEM"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test_V"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":6	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"7"	,"NameShort":"STAT_BUD"	,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":""		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }

			]

			}';

		EXEC @RC = planning.spPOST_Products_JSON 'SQL', @ValuesJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Products_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Products_JSON','PARAMETER',NULL);
*/
   
CREATE PROCEDURE planning.spPOST_Products_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY
			BEGIN TRANSACTION spPOST_Products_JSON
			-- INPUT PROTECTION	  #################################################################################

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (technical)', 100;
			END;
			
			-- Extract Values FROM JSON and put them into #SendedRecords #####################################

			DROP TABLE IF EXISTS #SendedRecords

			;WITH vals AS 
				(
					SELECT
						 ProductNumber							AS 'ProductNumber'
						,dbo.sx_pf_pProtectID (FactoryID)		AS 'FactoryID'
						,dbo.sx_pf_pProtectID (ProductLineID)	AS 'ProductLineID'
						,dbo.sx_pf_pProtectID (ProductID)		AS 'ProductID'
						,NameShort								AS 'NameShort'
						,CommentUser							AS 'CommentUser'		 
						,ResponsiblePerson						AS 'ResponsiblePerson'
						,ImageName								AS 'ImageName'
						,[Status]								AS 'Status'
						,Template								AS 'Template'
						,GlobalAttribute1						AS 'GlobalAttribute1'
						,GlobalAttribute2						AS 'GlobalAttribute2'
						,GlobalAttribute3						AS 'GlobalAttribute3'
						,GlobalAttribute4						AS 'GlobalAttribute4'
						,GlobalAttribute5						AS 'GlobalAttribute5'
						,GlobalAttribute6						AS 'GlobalAttribute6'
						,GlobalAttribute7						AS 'GlobalAttribute7'
						,GlobalAttribute8						AS 'GlobalAttribute8'
						,GlobalAttribute9						AS 'GlobalAttribute9'
						,GlobalAttribute10						AS 'GlobalAttribute10'
						,GlobalAttribute11						AS 'GlobalAttribute11'
						,GlobalAttribute12						AS 'GlobalAttribute12'
						,GlobalAttribute13						AS 'GlobalAttribute13'
						,GlobalAttribute14						AS 'GlobalAttribute14'
						,GlobalAttribute15						AS 'GlobalAttribute15'
						,GlobalAttribute16						AS 'GlobalAttribute16'
						,GlobalAttribute17						AS 'GlobalAttribute17'
						,GlobalAttribute18						AS 'GlobalAttribute18'
						,GlobalAttribute19						AS 'GlobalAttribute19'
						,GlobalAttribute20						AS 'GlobalAttribute20'
						,GlobalAttribute21						AS 'GlobalAttribute21'
						,GlobalAttribute22						AS 'GlobalAttribute22'
						,GlobalAttribute23						AS 'GlobalAttribute23'
						,GlobalAttribute24						AS 'GlobalAttribute24'
						,GlobalAttribute25						AS 'GlobalAttribute25'
					FROM OPENJSON (@ValuesJSON, N'$.Values') 
					WITH (
						 ProductNumber			INT				N'$.ProductNumber'
						,FactoryID				NVARCHAR(255)	N'$.FactoryID'
						,ProductLineID			NVARCHAR(255)	N'$.ProductLineID'
						,ProductID				NVARCHAR(255)	N'$.ProductID'
						,NameShort				NVARCHAR(255)	N'$.NameShort'
						,CommentUser			NVARCHAR(255)	N'$.CommentUser'
						,ResponsiblePerson		NVARCHAR(255)	N'$.ResponsiblePerson'
						,ImageName				NVARCHAR(255)	N'$.ImageName'			
						,Status					NVARCHAR(255)	N'$.Status'			
						,Template				NVARCHAR(255)	N'$.Template'		
						,GlobalAttribute1		NVARCHAR(255)	N'$.GlobalAttribute1'
						,GlobalAttribute2		NVARCHAR(255)	N'$.GlobalAttribute2'
						,GlobalAttribute3		NVARCHAR(255)	N'$.GlobalAttribute3'
						,GlobalAttribute4		NVARCHAR(255)	N'$.GlobalAttribute4'
						,GlobalAttribute5		NVARCHAR(255)	N'$.GlobalAttribute5'
						,GlobalAttribute6		NVARCHAR(255)	N'$.GlobalAttribute6'
						,GlobalAttribute7		NVARCHAR(255)	N'$.GlobalAttribute7'
						,GlobalAttribute8		NVARCHAR(255)	N'$.GlobalAttribute8'
						,GlobalAttribute9		NVARCHAR(255)	N'$.GlobalAttribute9'
						,GlobalAttribute10		NVARCHAR(255)	N'$.GlobalAttribute10'
						,GlobalAttribute11		NVARCHAR(255)	N'$.GlobalAttribute11'
						,GlobalAttribute12		NVARCHAR(255)	N'$.GlobalAttribute12'
						,GlobalAttribute13		NVARCHAR(255)	N'$.GlobalAttribute13'
						,GlobalAttribute14		NVARCHAR(255)	N'$.GlobalAttribute14'
						,GlobalAttribute15		NVARCHAR(255)	N'$.GlobalAttribute15'
						,GlobalAttribute16		NVARCHAR(255)	N'$.GlobalAttribute16'
						,GlobalAttribute17		NVARCHAR(255)	N'$.GlobalAttribute17'
						,GlobalAttribute18		NVARCHAR(255)	N'$.GlobalAttribute18'
						,GlobalAttribute19		NVARCHAR(255)	N'$.GlobalAttribute19'
						,GlobalAttribute20		NVARCHAR(255)	N'$.GlobalAttribute20'
						,GlobalAttribute21		NVARCHAR(255)	N'$.GlobalAttribute21'
						,GlobalAttribute22		NVARCHAR(255)	N'$.GlobalAttribute22'
						,GlobalAttribute23		NVARCHAR(255)	N'$.GlobalAttribute23'
						,GlobalAttribute24		NVARCHAR(255)	N'$.GlobalAttribute24'
						,GlobalAttribute25		NVARCHAR(255)	N'$.GlobalAttribute25'
						)
					)

			SELECT 	
				 ProductNumber 
				,FactoryID				COLLATE DATABASE_DEFAULT  AS FactoryID
				,ProductLineID			COLLATE DATABASE_DEFAULT  AS ProductLineID
				,ProductID				COLLATE DATABASE_DEFAULT  AS ProductID
				,NameShort				COLLATE DATABASE_DEFAULT  AS NameShort			
				,CommentUser			COLLATE DATABASE_DEFAULT  AS CommentUser		
				,ResponsiblePerson		COLLATE DATABASE_DEFAULT  AS ResponsiblePerson	
				,ImageName				COLLATE DATABASE_DEFAULT  AS ImageName			
				,[Status]				COLLATE DATABASE_DEFAULT  AS [Status]			
				,Template				COLLATE DATABASE_DEFAULT  AS Template			
				,GlobalAttribute1		COLLATE DATABASE_DEFAULT  AS GlobalAttribute1	
				,GlobalAttribute2		COLLATE DATABASE_DEFAULT  AS GlobalAttribute2	
				,GlobalAttribute3		COLLATE DATABASE_DEFAULT  AS GlobalAttribute3	
				,GlobalAttribute4		COLLATE DATABASE_DEFAULT  AS GlobalAttribute4	
				,GlobalAttribute5		COLLATE DATABASE_DEFAULT  AS GlobalAttribute5	
				,GlobalAttribute6		COLLATE DATABASE_DEFAULT  AS GlobalAttribute6	
				,GlobalAttribute7		COLLATE DATABASE_DEFAULT  AS GlobalAttribute7	
				,GlobalAttribute8		COLLATE DATABASE_DEFAULT  AS GlobalAttribute8	
				,GlobalAttribute9		COLLATE DATABASE_DEFAULT  AS GlobalAttribute9	
				,GlobalAttribute10		COLLATE DATABASE_DEFAULT  AS GlobalAttribute10	
				,GlobalAttribute11		COLLATE DATABASE_DEFAULT  AS GlobalAttribute11	
				,GlobalAttribute12		COLLATE DATABASE_DEFAULT  AS GlobalAttribute12	
				,GlobalAttribute13		COLLATE DATABASE_DEFAULT  AS GlobalAttribute13	
				,GlobalAttribute14		COLLATE DATABASE_DEFAULT  AS GlobalAttribute14	
				,GlobalAttribute15		COLLATE DATABASE_DEFAULT  AS GlobalAttribute15	
				,GlobalAttribute16		COLLATE DATABASE_DEFAULT  AS GlobalAttribute16	
				,GlobalAttribute17		COLLATE DATABASE_DEFAULT  AS GlobalAttribute17	
				,GlobalAttribute18		COLLATE DATABASE_DEFAULT  AS GlobalAttribute18	
				,GlobalAttribute19		COLLATE DATABASE_DEFAULT  AS GlobalAttribute19	
				,GlobalAttribute20		COLLATE DATABASE_DEFAULT  AS GlobalAttribute20	
				,GlobalAttribute21		COLLATE DATABASE_DEFAULT  AS GlobalAttribute21	
				,GlobalAttribute22		COLLATE DATABASE_DEFAULT  AS GlobalAttribute22	
				,GlobalAttribute23		COLLATE DATABASE_DEFAULT  AS GlobalAttribute23	
				,GlobalAttribute24		COLLATE DATABASE_DEFAULT  AS GlobalAttribute24	
				,GlobalAttribute25		COLLATE DATABASE_DEFAULT  AS GlobalAttribute25	
			INTO #SendedRecords	
			FROM vals;

			-- Debug: 
			-- SELECT * FROM #SendedRecords
	
			-- Check for logical correct values #############################################
			PRINT 'Starting Check for logical correct Values'	
			-- no NULL Values contained in IDs
			IF EXISTS (
				SELECT TOP(1) 1 
				FROM #SendedRecords 
				WHERE 
					   ProductNumber	IS NULL
					OR FactoryID		IS NULL
					OR ProductLineID	IS NULL
					OR ProductID		IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					
					)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid Factory/PL/P IDs.', 100;
			END;

			-- no doubled ProductNumbers contained
			IF EXISTS (
					SELECT TOP (1) 1
					FROM #SendedRecords
					GROUP BY ProductNumber
					HAVING COUNT(ProductNumber) > 1
				)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to duplicate ProductNumbers.', 100;
			END;

			-- User Rights ##################################################################
			PRINT 'Starting User Rights Check'
			-- Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Build classification Table for sended Records  #################################
			PRINT 'Starting classification Process'
			DROP TABLE IF EXISTS #SendedRecordsClassification;

			CREATE TABLE #SendedRecordsClassification
				(
					 ProductNumber				INT											NOT NULL 
					,FactoryID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,NameShort					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,CommentUser				NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,ResponsiblePerson			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,ImageName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,Status						NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,Template					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute1			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute2			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute3			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute4			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute5			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute6			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute7			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute8			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute9			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute10			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute11			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute12			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute13			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute14			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute15			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute16			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute17			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute18			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute19			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute20			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute21			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute22			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute23			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute24			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute25			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,UserRight					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,InvalidTemplateFlag		INT											NOT NULL 
					,TargetIDsNotExistingFlag	INT											NOT NULL 
					,MultipleRecordsFlag	INT											NOT NULL  -- if more than one record per Product is sent, only the last one is posted 
					,ReturnCode					INT											NOT NULL 
					,StatusComment				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductKey					BIGINT										NOT NULL 
					,SaveRecordFlag				INT											NOT NULL 
				);

			INSERT INTO #SendedRecordsClassification 
				SELECT 	
					 ProductNumber												AS ValueNumber
					,FactoryID					COLLATE DATABASE_DEFAULT		AS FactoryID
					,ProductLineID				COLLATE DATABASE_DEFAULT		AS ProductLineID
					,ProductID					COLLATE DATABASE_DEFAULT		AS ProductID
					,NameShort					COLLATE DATABASE_DEFAULT		AS NameShort			
					,CommentUser				COLLATE DATABASE_DEFAULT		AS CommentUser		
					,ResponsiblePerson			COLLATE DATABASE_DEFAULT		AS ResponsiblePerson	
					,ImageName					COLLATE DATABASE_DEFAULT		AS ImageName			
					,Status						COLLATE DATABASE_DEFAULT		AS Status				
					,Template					COLLATE DATABASE_DEFAULT		AS Template			
					,GlobalAttribute1			COLLATE DATABASE_DEFAULT		AS GlobalAttribute1	
					,GlobalAttribute2			COLLATE DATABASE_DEFAULT		AS GlobalAttribute2	
					,GlobalAttribute3			COLLATE DATABASE_DEFAULT		AS GlobalAttribute3	
					,GlobalAttribute4			COLLATE DATABASE_DEFAULT		AS GlobalAttribute4	
					,GlobalAttribute5			COLLATE DATABASE_DEFAULT		AS GlobalAttribute5	
					,GlobalAttribute6			COLLATE DATABASE_DEFAULT		AS GlobalAttribute6	
					,GlobalAttribute7			COLLATE DATABASE_DEFAULT		AS GlobalAttribute7	
					,GlobalAttribute8			COLLATE DATABASE_DEFAULT		AS GlobalAttribute8	
					,GlobalAttribute9			COLLATE DATABASE_DEFAULT		AS GlobalAttribute9	
					,GlobalAttribute10			COLLATE DATABASE_DEFAULT		AS GlobalAttribute10	
					,GlobalAttribute11			COLLATE DATABASE_DEFAULT		AS GlobalAttribute11	
					,GlobalAttribute12			COLLATE DATABASE_DEFAULT		AS GlobalAttribute12	
					,GlobalAttribute13			COLLATE DATABASE_DEFAULT		AS GlobalAttribute13	
					,GlobalAttribute14			COLLATE DATABASE_DEFAULT		AS GlobalAttribute14	
					,GlobalAttribute15			COLLATE DATABASE_DEFAULT		AS GlobalAttribute15	
					,GlobalAttribute16			COLLATE DATABASE_DEFAULT		AS GlobalAttribute16	
					,GlobalAttribute17			COLLATE DATABASE_DEFAULT		AS GlobalAttribute17	
					,GlobalAttribute18			COLLATE DATABASE_DEFAULT		AS GlobalAttribute18	
					,GlobalAttribute19			COLLATE DATABASE_DEFAULT		AS GlobalAttribute19	
					,GlobalAttribute20			COLLATE DATABASE_DEFAULT		AS GlobalAttribute20	
					,GlobalAttribute21			COLLATE DATABASE_DEFAULT		AS GlobalAttribute21	
					,GlobalAttribute22			COLLATE DATABASE_DEFAULT		AS GlobalAttribute22	
					,GlobalAttribute23			COLLATE DATABASE_DEFAULT		AS GlobalAttribute23	
					,GlobalAttribute24			COLLATE DATABASE_DEFAULT		AS GlobalAttribute24	
					,GlobalAttribute25			COLLATE DATABASE_DEFAULT		AS GlobalAttribute25	
				
					,''												AS UserRight
					,0												AS InvalidTemplateFlag
					,0												AS TargetIDsNotExistingFlag
					,0												AS MultipleRecordsFlag 
					,501											AS ReturnCode
					,N'Original'		COLLATE DATABASE_DEFAULT	AS StatusComment
					,0												AS ProductKey
					,0												AS SaveRecordFlag	
				FROM #SendedRecords;
	
			-- add ProductKey and therefore check for the existence of the targeted products
			UPDATE SRC 
			SET
				 SRC.ProductKey		= dp.ProductKey
			FROM #SendedRecordsClassification SRC
				INNER JOIN planning.tdProducts dp  -- only for hits
					ON 
							SRC.FactoryID		= dp.FactoryID
						AND SRC.ProductLineID	= dp.ProductLineID
						AND SRC.ProductID		= dp.ProductID

			-- Flag target products which don't exist
			UPDATE #SendedRecordsClassification 
			SET 
				 TargetIDsNotExistingFlag	= 1
				,ReturnCode					= 404
				,StatusComment				= N'Error: Product does not exist'
				,SaveRecordFlag				= -1
			WHERE ProductKey = 0;

			-- determine UserRights
			UPDATE SRC 
			SET
				SRC.UserRight = tUR.[Right]	
			FROM #SendedRecordsClassification SRC
				LEFT JOIN system.trUserRights tUR
				ON 
						SRC.FactoryID		= tUR.FactoryID
					AND SRC.ProductLineID	= tUR.ProductLineID
					AND tUR.UserName		= @TransactionUsername
			WHERE  tUR.[Right] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
			SET 
				 SaveRecordFlag = -1 
				,ReturnCode = 403
				,StatusComment = 'No write rights.'
			WHERE 
					UserRight <> 'Write'
				AND SaveRecordFlag <> -1;


			-- multiple records per product; only the last one will be posted
			UPDATE #SendedRecordsClassification 
			SET
					MultipleRecordsFlag		= 1
				,ReturnCode					= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,200,204)
				,StatusComment				= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,'Saved record (Multiple records per product)','Multiple records per product, only the last one has been posted')
				,SaveRecordFlag				= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,0,-1)
			FROM #SendedRecordsClassification SRC
				RIGHT JOIN (
					SELECT
							ProductNumber
						,FactoryID
						,ProductLineID
						,ProductID
						,ProductKey
						,COUNT(ProductKey) OVER(PARTITION BY ProductKey)								AS Anzahl
						,ROW_NUMBER() OVER(PARTITION BY ProductKey ORDER BY ProductNumber ASC)			AS Zeile
					FROM #SendedRecordsClassification
					WHERE ProductKey IS NOT NULL
						AND SaveRecordFlag <> -1) mf
					ON SRC.ProductNumber = mf.ProductNumber
			WHERE mf.Anzahl > 1
	
			-- Flag invalid template types (valid are: _VM, _VT, _VW, _VJ, _HM, _HT, _HW, _HJ); old type will be kept (rest will still be posted)
			UPDATE #SendedRecordsClassification 
			SET 
					Template				= NULL
				,InvalidTemplateFlag	= 1
				,ReturnCode				= 200
				,StatusComment			= IIF(MultipleRecordsFlag = 1, 'Partially saved record (Invalid template type, old type is kept; multiple records per product)','Partially saved record (Invalid template type, old type is kept)')
				,SaveRecordFlag			= 1
			FROM #SendedRecordsClassification SRC
			WHERE SRC.Template IS NOT NULL
				AND (LEFT(RIGHT(SRC.Template,3),2) NOT IN ('_V', '_H')
				OR RIGHT(SRC.Template,1) NOT IN ('M','T','W','J'))
				AND SRC.SaveRecordFlag <> -1

			-- Flag all remaining records as to be saved
			UPDATE #SendedRecordsClassification 
			SET	
					SaveRecordFlag		= 1
				,ReturnCode			= 200
				,StatusComment		= IIF(MultipleRecordsFlag = 1, StatusComment,'Saved record')
			WHERE SaveRecordFlag	= 0;

			-- Debug: 
			-- SELECT * FROM #SendedRecordsClassification src

			-- Determination of the final record set and update of product values #######################################################
			DROP TABLE IF EXISTS #SavedRecords;

			CREATE TABLE #SavedRecords
				(
					 ProductNumber				BIGINT										NOT NULL
					,ProductKey					BIGINT										NOT NULL
					,NameShort					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,CommentUser				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ResponsiblePerson			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ImageName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,Status						NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,Template					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute1			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute2			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute3			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute4			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute5			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute6			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute7			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute8			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute9			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute10			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute11			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute12			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute13			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute14			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute15			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute16			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute17			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute18			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute19			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute20			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute21			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute22			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute23			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute24			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute25			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
				)
			
			-- old values in case of NULL
			PRINT 'Determination of old values for sended NULL values'
			INSERT INTO #SavedRecords
				SELECT
					 SRC.ProductNumber
					,SRC.ProductKey		
					,IIF(SRC.NameShort			IS NULL,	dp.NameShort		,	SRC.NameShort			)		
					,IIF(SRC.CommentUser		IS NULL,	dp.CommentUser		,	SRC.CommentUser		)	
					,IIF(SRC.ResponsiblePerson	IS NULL,	dp.ResponsiblePerson,	SRC.ResponsiblePerson	)
					,IIF(SRC.ImageName			IS NULL,	dp.ImageName		,	SRC.ImageName			)		
					,IIF(SRC.Status				IS NULL,	dp.Status			,	SRC.Status				)		
					,IIF(SRC.Template			IS NULL,	dp.Template			,	SRC.Template			)	
					,IIF(SRC.GlobalAttribute1	IS NULL,	dp.GlobalAttribute1	,	SRC.GlobalAttribute1	)
					,IIF(SRC.GlobalAttribute2	IS NULL,	dp.GlobalAttribute2	,	SRC.GlobalAttribute2	)
					,IIF(SRC.GlobalAttribute3	IS NULL,	dp.GlobalAttribute3	,	SRC.GlobalAttribute3	)
					,IIF(SRC.GlobalAttribute4	IS NULL,	dp.GlobalAttribute4	,	SRC.GlobalAttribute4	)
					,IIF(SRC.GlobalAttribute5	IS NULL,	dp.GlobalAttribute5	,	SRC.GlobalAttribute5	)
					,IIF(SRC.GlobalAttribute6	IS NULL,	dp.GlobalAttribute6	,	SRC.GlobalAttribute6	)
					,IIF(SRC.GlobalAttribute7	IS NULL,	dp.GlobalAttribute7	,	SRC.GlobalAttribute7	)
					,IIF(SRC.GlobalAttribute8	IS NULL,	dp.GlobalAttribute8	,	SRC.GlobalAttribute8	)
					,IIF(SRC.GlobalAttribute9	IS NULL,	dp.GlobalAttribute9	,	SRC.GlobalAttribute9	)
					,IIF(SRC.GlobalAttribute10	IS NULL,	dp.GlobalAttribute10,	SRC.GlobalAttribute10	)
					,IIF(SRC.GlobalAttribute11	IS NULL,	dp.GlobalAttribute11,	SRC.GlobalAttribute11	)
					,IIF(SRC.GlobalAttribute12	IS NULL,	dp.GlobalAttribute12,	SRC.GlobalAttribute12	)
					,IIF(SRC.GlobalAttribute13	IS NULL,	dp.GlobalAttribute13,	SRC.GlobalAttribute13	)
					,IIF(SRC.GlobalAttribute14	IS NULL,	dp.GlobalAttribute14,	SRC.GlobalAttribute14	)
					,IIF(SRC.GlobalAttribute15	IS NULL,	dp.GlobalAttribute15,	SRC.GlobalAttribute15	)
					,IIF(SRC.GlobalAttribute16	IS NULL,	dp.GlobalAttribute16,	SRC.GlobalAttribute16	)
					,IIF(SRC.GlobalAttribute17	IS NULL,	dp.GlobalAttribute17,	SRC.GlobalAttribute17	)
					,IIF(SRC.GlobalAttribute18	IS NULL,	dp.GlobalAttribute18,	SRC.GlobalAttribute18	)
					,IIF(SRC.GlobalAttribute19	IS NULL,	dp.GlobalAttribute19,	SRC.GlobalAttribute19	)
					,IIF(SRC.GlobalAttribute20	IS NULL,	dp.GlobalAttribute20,	SRC.GlobalAttribute20	)
					,IIF(SRC.GlobalAttribute21	IS NULL,	dp.GlobalAttribute21,	SRC.GlobalAttribute21	)
					,IIF(SRC.GlobalAttribute22	IS NULL,	dp.GlobalAttribute22,	SRC.GlobalAttribute22	)
					,IIF(SRC.GlobalAttribute23	IS NULL,	dp.GlobalAttribute23,	SRC.GlobalAttribute23	)
					,IIF(SRC.GlobalAttribute24	IS NULL,	dp.GlobalAttribute24,	SRC.GlobalAttribute24	)
					,IIF(SRC.GlobalAttribute25	IS NULL,	dp.GlobalAttribute25,	SRC.GlobalAttribute25	)
				FROM #SendedRecordsClassification SRC
					LEFT JOIN planning.tdProducts dp
						ON SRC.ProductKey = dp.ProductKey
				WHERE SRC.SaveRecordFlag	= 1

			-- Insert new Values
			PRINT 'Update of products'

			UPDATE dp 
				SET 		
					 dp.NameShort			= SR.NameShort			
					,dp.CommentUser			= SR.CommentUser		
					,dp.ResponsiblePerson	= SR.ResponsiblePerson	
					,dp.ImageName			= SR.ImageName			
					,dp.Status				= SR.Status				
					,dp.Template			= SR.Template			
					,dp.GlobalAttribute1	= SR.GlobalAttribute1	
					,dp.GlobalAttribute2	= SR.GlobalAttribute2	
					,dp.GlobalAttribute3	= SR.GlobalAttribute3	
					,dp.GlobalAttribute4	= SR.GlobalAttribute4	
					,dp.GlobalAttribute5	= SR.GlobalAttribute5	
					,dp.GlobalAttribute6	= SR.GlobalAttribute6	
					,dp.GlobalAttribute7	= SR.GlobalAttribute7	
					,dp.GlobalAttribute8	= SR.GlobalAttribute8	
					,dp.GlobalAttribute9	= SR.GlobalAttribute9	
					,dp.GlobalAttribute10	= SR.GlobalAttribute10	
					,dp.GlobalAttribute11	= SR.GlobalAttribute11	
					,dp.GlobalAttribute12	= SR.GlobalAttribute12	
					,dp.GlobalAttribute13	= SR.GlobalAttribute13	
					,dp.GlobalAttribute14	= SR.GlobalAttribute14	
					,dp.GlobalAttribute15	= SR.GlobalAttribute15	
					,dp.GlobalAttribute16	= SR.GlobalAttribute16	
					,dp.GlobalAttribute17	= SR.GlobalAttribute17	
					,dp.GlobalAttribute18	= SR.GlobalAttribute18	
					,dp.GlobalAttribute19	= SR.GlobalAttribute19	
					,dp.GlobalAttribute20	= SR.GlobalAttribute20	
					,dp.GlobalAttribute21	= SR.GlobalAttribute21	
					,dp.GlobalAttribute22	= SR.GlobalAttribute22	
					,dp.GlobalAttribute23	= SR.GlobalAttribute23	
					,dp.GlobalAttribute24	= SR.GlobalAttribute24	
					,dp.GlobalAttribute25	= SR.GlobalAttribute25	
				FROM planning.tdproducts dp
					INNER JOIN	#SavedRecords SR
						ON dp.ProductKey = SR.ProductKey
					
			-- Create result table to be returned #################################################

			DROP TABLE IF EXISTS #ResultSet;

			CREATE TABLE #ResultSet (
				 ProductNumber		INT										 NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,NameShort			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	 NULL 
				,CommentUser		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ResponsiblePerson	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ImageName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,[Status]			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,Template			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute1	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute2	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute3	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute4	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute5	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute6	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute7	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute8	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute9	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute10	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute11	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute12	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute13	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute14	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute15	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute16	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute17	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute18	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute19	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute20	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute21	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute22	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute23	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute24	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute25	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ReturnCode			INT											 NULL
				,ReturnComment		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	 NULL
				);
			
			-- Fill result table with requested values

			PRINT 'Start filling result set.'
			INSERT INTO #ResultSet 
				SELECT 
					 ProductNumber
					,FactoryID
					,ProductLineID
					,ProductID
					,NameShort			
					,CommentUser		
					,ResponsiblePerson
					,ImageName			
					,[Status]			
					,Template			
					,GlobalAttribute1
					,GlobalAttribute2
					,GlobalAttribute3
					,GlobalAttribute4
					,GlobalAttribute5
					,GlobalAttribute6
					,GlobalAttribute7
					,GlobalAttribute8
					,GlobalAttribute9
					,GlobalAttribute10
					,GlobalAttribute11
					,GlobalAttribute12
					,GlobalAttribute13
					,GlobalAttribute14
					,GlobalAttribute15
					,GlobalAttribute16
					,GlobalAttribute17
					,GlobalAttribute18
					,GlobalAttribute19
					,GlobalAttribute20
					,GlobalAttribute21
					,GlobalAttribute22
					,GlobalAttribute23
					,GlobalAttribute24
					,GlobalAttribute25
					,0
					,''
				FROM #SendedRecords;
		
			-- Set Return Codes an comments
			PRINT 'Setting ReturnCodes in Result Set.'
			UPDATE RS	
				SET 
					 RS.ReturnCode		= SRC.ReturnCode
					,RS.ReturnComment	= SRC.StatusComment
			FROM #ResultSet RS
				LEFT JOIN #SendedRecordsClassification SRC
					ON RS.ProductNumber = SRC.ProductNumber;

			DECLARE @ProcessTime INT;
			SET @ProcessTime = DATEDIFF(ms, @TimestampCall, GETUTCDATE());
			SET @ResultCode = 200;

			-- Return result
			SELECT  
					 *
					,@ProcessTime AS ProcessTime
			FROM #ResultSet;
		
			COMMIT TRANSACTION spPOST_Products_JSON;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			
			ROLLBACK TRANSACTION spPOST_Products_JSON;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;
GO
		
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Products_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to POST Values in JSON Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'Valus which are to be saved as JSON object.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
