
DROP PROCEDURE IF EXISTS planning.spPOST_Tab;
GO

/*
	Procedure to POST a Tab for
		- a cluster
		- a factory
		- a productline
		- a product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spPOST_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= 'Td5dd'
				,@OrderIndex		= '3'
				,@TabName			= 'Name'
				,@TabHint			= 'Hint'
				,@TabText			= 'Text'
				,@PresentationCODE	= 'Pivot'
				,@Datasource		= 'fdafds'
				,@IsVisibleBOOL		= '1'
				,@LayoutJSON		= 'dafds'
				,@ParameterJSON		= 'fdafd';
		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex;

		TRUNCATE TABLE planning.tTabs;

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Tab','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255) 
		,@OrderIndex			INT 
		,@TabName				NVARCHAR(255) 
		,@TabHint				NVARCHAR(2000) 
		,@TabText				NVARCHAR(2000) 
		,@PresentationCODE		NVARCHAR(255) 
		,@Datasource			NVARCHAR(2000) 
		,@IsVisibleBOOL			INT
		,@LayoutJSON			NVARCHAR(MAX) 
		,@ParameterJSON			NVARCHAR(MAX) 

    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''','
				+ CAST(ISNULL(@OrderIndex		,N'NULL') AS NVARCHAR(255)) + N','''
				+ ISNULL(@TabName				,N'NULL')					+ N''','''
				+ ISNULL(@TabHint				,N'NULL')					+ N''','''
				+ ISNULL(@TabText				,N'NULL')					+ N''','''
				+ ISNULL(@PresentationCODE		,N'NULL')					+ N''','''
				+ ISNULL(@Datasource			,N'NULL')					+ N''','
				+ CAST(ISNULL(@IsVisibleBOOL	,N'NULL') AS NVARCHAR(255)) + N','''
				+ ISNULL(@LayoutJSON			,N'NULL')					+ N''','''
				+ ISNULL(@ParameterJSON			,N'NULL')					+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0
				,@OldOrderIndex		INT			= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';
		IF @OrderIndex			IS NULL SET @OrderIndex			= 0  ;
		IF @TabName				IS NULL SET @TabName			= N'';
		IF @TabHint				IS NULL SET @TabHint			= N'';
		IF @TabText				IS NULL SET @TabText			= N'';
		IF @PresentationCODE	IS NULL SET @PresentationCODE	= N'';
		IF @Datasource			IS NULL SET @Datasource			= N'';
		IF @IsVisibleBOOL		IS NULL SET @IsVisibleBOOL		= 0  ;
		IF @LayoutJSON			IS NULL SET @LayoutJSON			= N'';
		IF @ParameterJSON		IS NULL SET @ParameterJSON		= N'';

		-- dont use negative index values
		IF @OrderIndex			< 0		SET @OrderIndex			= 1;
		-- protect bool
		IF @IsVisibleBOOL      <> 1		SET @IsVisibleBOOL		= 0;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;
				
				-- error if and ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR(N'Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- error if ID chains has gaps
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <> -1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				SET @ResultCode = 501 -- reset

				-- determine if posted Tab already exists
				SELECT 
					 @Rowkey = RowKey
					,@OldOrderIndex = OrderIndex
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- data operation starts	***********************************************************************************
				-- delete if exits, to determine Resultcode
				IF @RowKey > 0 
					BEGIN
						DELETE FROM planning.tTabs WHERE RowKey = @RowKey
						SET @Resultcode = 200
					END
					
				-- System Tab with TabID 0 can only change visibility
				IF @TabID = '0'
					BEGIN
						INSERT INTO planning.tTabs
							VALUES
							(
								 @FactoryKey
								,@ProductLineKey
								,@ProductKey
								,@FactoryID
								,@ProductLineID
								,@ProductID
								,@TabID
								,0
								,@TabName
								,N''
								,N''
								,N'System'
								,N''
								,@IsVisibleBOOL
								,N''
								,N''
							)
					END
				ELSE
					BEGIN
						-- Insert new Tab
						INSERT INTO planning.tTabs
							VALUES
							(
								 @FactoryKey
								,@ProductLineKey
								,@ProductKey
								,@FactoryID
								,@ProductLineID
								,@ProductID
								,@TabID
								,@OrderIndex
								,@TabName
								,@TabHint
								,@TabText
								,@PresentationCODE
								,@Datasource
								,@IsVisibleBOOL
								,@LayoutJSON
								,@ParameterJSON
							)
					
						-- Update other numbers if Number is changed or new
						IF @OldOrderIndex <> @OrderIndex
							BEGIN			
								-- case if OrderIndex is bigger now and existed before - decrement all in move window
								IF @OldOrderIndex < @OrderIndex AND @RowKey <> 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex - 1 
										WHERE 
												OrderIndex		>= @OldOrderIndex 
											AND OrderIndex		<= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END
								-- case if OrderIndex is smaller now and existed before - increment all in move window
								IF @OldOrderIndex > @OrderIndex AND @RowKey <> 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex + 1 
										WHERE 
												OrderIndex		<= @OldOrderIndex 
											AND OrderIndex		>= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END

								-- case new Tab - increment all following
								IF @RowKey = 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex + 1 
										WHERE 
												OrderIndex		>= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END

								-- eliminate all Gaps
								UPDATE planning.tTabs 
								SET OrderIndex = Rownumbers.Number
								FROM planning.tTabs tT INNER JOIN (
																			SELECT 
																				 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																				,TabID
																	
																			FROM planning.tTabs
																			WHERE 
																					FactoryKey		= @FactoryKey
																				AND ProductLineKey	= @ProductLineKey
																				AND ProductKey		= @ProductKey

																	) AS Rownumbers
								ON 
										tT.TabID			= Rownumbers.TabID
									AND tT.FactoryKey		= @FactoryKey
									AND tT.ProductLineKey	= @ProductLineKey
									AND tT.ProductKey		= @ProductKey

							END		
					END

				IF @ResultCode <> 200 SET @ResultCode = 201;

			COMMIT TRANSACTION spPOST_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Tab'		-- enter procedure name
    ,@SX_Owner          NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module         NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Tab. The Tab with ID 0 is the special system tab, but does not get special handling in databae.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'Business Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderIndex';
    SET @value      = N'Position in the list of Tabs. Any Number > 0 can be passed in, tabs reorder themself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 	SET @level2name = N'@TabName';
    SET @value      = N'Name of the Tab, used as Label.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabHint';
    SET @value      = N'Hint on the Tab, displayed as Tooltip.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabText';
    SET @value      = N'Text for the headerarea inside the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PresentationCODE';
    SET @value      = N'CODE word for the type of presentation, e.g PIVOT, DATAGRID, HTML_URL, HTML_STATIC.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Datasource';
    SET @value      = N'Datasource fitting to PresentationCODE - name of the stored procedure or URL.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsVisibleBool';
    SET @value      = N'1 for visible, 0 for not visible';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutJSON';
    SET @value      = N'Layout definition as JSON String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ParameterJSON';
    SET @value      = N'Parameter definition (List and default value) as JSON String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
