
DROP PROCEDURE IF EXISTS system.spDELETE_DataSource;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Procedure to delete a DataSource an all data which belong to it. 

	A Datasource contains mostly three types of data
		- itselft, its definition of a connnection to an Datasource
		- the data extracted from this datasource (Companies, Dimensions, Facts)
		- process definitions for this datasource (Pipelines, Steps..)
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_DataSource
				 @Username		= 'SQL'
				,@DataSourceID	= '2'
		PRINT    @RC

		SELECT * FROM system.tDataSources

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_DataSource',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_DataSource','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_DataSource
    (
         @Username			NVARCHAR(255)
        ,@DataSourceID		NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID 				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@Tablename			NVARCHAR(255)	= N''
				,@SQL				NVARCHAR(MAX)	= N''
				,@Counter			INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID		= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_DataSource
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR(N'Transaction user don`t exists', 16, 10);
						END;
				-- Check for Cluster Write Rights

				-- Check if Datasource exits
				SELECT @Counter = COUNT(*) 
				FROM system.tDataSources 
				WHERE DatasourceID = @DataSourceID;
				IF @Counter = 0 
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Datasource don`t exists', 16, 10);
					END;



				-- Generic delete is still done over DatasourceKey, as DatasourceID was till 5.5 the DatasourceKey
				PRINT '';
				PRINT 'Starting generic Delete Process for all Tables outside system containing the Column "DatasourceKey"';
				PRINT '#####################################################################################################';
				PRINT '';
				-- DELETE Date in all Non-Core Tables where the column DataSourceKey is used (usually global and integration schema)
				-- Generic delete over dynamic SQL is possible in all OCT schema except "system" in default

				DECLARE TableCursor CURSOR FOR
					SELECT      
						 CONCAT(infs.TABLE_SCHEMA COLLATE DATABASE_DEFAULT,N'.',infs.TABLE_NAME COLLATE DATABASE_DEFAULT)	AS Tablename

					FROM INFORMATION_SCHEMA.COLUMNS infs
						-- Determination Objekttype
						LEFT JOIN 
								(
								SELECT 
									 Name	  				COLLATE DATABASE_DEFAULT    AS Objektname
									,SCHEMA_NAME(Schema_id) COLLATE DATABASE_DEFAULT	AS Schemaname
									,type_desc				COLLATE DATABASE_DEFAULT	AS Objekttyp
								FROM sys.objects
								) syo ON 
											syo.Schemaname = infs.TABLE_SCHEMA
										AND syo.Objektname = infs.TABLE_NAME

					WHERE 
							syo.Objekttyp		= N'USER_TABLE'
						AND infs.COLUMN_NAME	= N'DataSourceKey'
						-- ToDo: Determination not a Core table should be moved to metadata 
						AND infs.TABLE_SCHEMA  <> N'system'
						AND CONCAT(infs.TABLE_SCHEMA,N'.',infs.TABLE_NAME) <> N'global.tCompanies'

				OPEN TableCursor;
				FETCH TableCursor
				INTO @Tablename;
				IF @@CURSOR_ROWS = 0
					BEGIN
						PRINT CONCAT(N'No Tables found for generic deletion over DataSourceKey ',@DataSourceID,N'.')
					END
				WHILE @@FETCH_STATUS = 0
				BEGIN
					-- clear variables
					SET @SQL = N'';
					SET @SQL = CONCAT(N'DELETE FROM ', @Tablename, N' WHERE DataSourceKey =', @DataSourceID,';') -- ToDo: ID is numeric at the moment, change to string when changed					
					
					PRINT CONCAT(N'Delete all Rows with DatasourceID ', @DataSourceID, N' in Table ',@Tablename)
					EXEC (@SQL);

					FETCH TableCursor
					INTO @Tablename;
				END;
				CLOSE TableCursor;
				DEALLOCATE TableCursor;

				-- Generic delete is still done over CompanyKey
				-- GET all Non-Core Tables where the column CompanyKey is used (usually result schema) and delete the Values for this ComanpanyKey
				-- The Company Key is an combinded Key "DatasourceID | CompanyID"
				-- some are deleted the second time, as they have the Datasource Key also !!

				PRINT '';
				PRINT 'Starting generic Delete Process for all Tables outside system containing the Column "CompanyKey" which is a concated Key with the Datasource.';
				PRINT '#####################################################################################################';
				PRINT '';
				DECLARE TableCursor CURSOR FOR
					SELECT      
							CONCAT(infs.TABLE_SCHEMA COLLATE DATABASE_DEFAULT,N'.',infs.TABLE_NAME COLLATE DATABASE_DEFAULT)	AS Tablename

					FROM INFORMATION_SCHEMA.COLUMNS infs
						-- Determination Objekttype
						LEFT JOIN 
								(
								SELECT 
									 Name					COLLATE DATABASE_DEFAULT AS Objektname
									,SCHEMA_NAME(Schema_id) COLLATE DATABASE_DEFAULT AS Schemaname
									,type_desc				COLLATE DATABASE_DEFAULT AS Objekttyp
								FROM sys.objects
								) syo ON 
											syo.Schemaname = infs.TABLE_SCHEMA
										AND syo.Objektname = infs.TABLE_NAME

				WHERE 
							syo.Objekttyp		= N'USER_TABLE'
						AND infs.COLUMN_NAME	= N'CompanyKey'
						-- ToDo: Determination not a Core table should be moved to metadata 
						AND infs.TABLE_SCHEMA  <> N'system'
						AND CONCAT(infs.TABLE_SCHEMA,N'.',infs.TABLE_NAME) <> N'global.tCompanies';
				OPEN TableCursor;
				FETCH TableCursor
				INTO @Tablename;
					IF @@CURSOR_ROWS = 0
						BEGIN
							PRINT CONCAT(N'No Tables found for generic deletion over CompanyKeys starting with ',@DataSourceID,N'| .')
						END
				WHILE @@FETCH_STATUS = 0
					BEGIN
						-- clear variables
						SET @SQL = N'';
						SET @SQL = CONCAT(N'DELETE FROM ', @Tablename, N' WHERE CompanyKey LIKE ''', @DataSourceID,N'|%'';') 

						PRINT CONCAT(N'Delete all Rows with DatasourceID ', @DataSourceID, N' in CompanyKey in Table ',@Tablename)
						EXEC (@SQL);

						FETCH TableCursor
						INTO @Tablename;
					END;
				CLOSE TableCursor;
				DEALLOCATE TableCursor;

				PRINT ''
				PRINT 'End of generic delition'
				PRINT  '######################################################################'
				PRINT ''

				PRINT '';
				PRINT 'Starting Delete for Core Data'
				PRINT '#####################################################################################################';
				PRINT '';
								
				-- PIPELINES - Company selection is deleted in all steps, but not the steps themself using the datasource

				-- system.tPipelineSteps  - TODO - in new field CompaniesJSON
			
				-- global.tCompanies
				DELETE FROM global.tCompanies WHERE DataSourceKey = CAST(@DataSourceID AS INT);

				PRINT 'Delete the datasource itself'
				-- DELETE Datasource (as last due to calculated / dependend columns)
				DELETE FROM system.tDataSources WHERE DataSourceID = @DataSourceID;

				SET @ResultCode = 200;
			COMMIT TRANSACTION spDELETE_DataSource
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_DataSource	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT N'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spDELETE_DataSource TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_DataSource'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Datasouce - the data from its proceeded companies is deleted in all tables with Columns DatasourceKey or CompanyKey too, but the datesource is keept in Process Steps.';    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'ID of the DataSource to be deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO
