
DROP PROCEDURE IF EXISTS system.spEXPORT_Setting;
GO

/*
	Procedure to create an OCT Importscript for an setting which is saved in system.tSettings
	mainly used for 
		- ClusterStartTab
		- DataFlowTab
		- Global parameters

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 08/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Setting
				 @Username		= 'SQL'
				,@SettingID		= 'ClusterStart_Html_Page'  -- ClusterStart_Html_Page, Integration_Html_Page, PipelineGlobalParameters
		PRINT    @RC

	Sourcetable
		SELECT * FROM system.tSettings


	Manual Export / Import for large Settings - long settings (more than 64.000 Charakters = mostly encoded images) are not displayed in SSMS and can't be read over sqlcmd - you must use bcp
		1. write the export to an table
		DROP TABLE IF EXISTS tmpout_Export;
		GO
		CREATE TABLE tmpout_Export
			(
				Command		NVARCHAR(MAX)
			);

		INSERT INTO tmpout_Export
			EXEC system.spEXPORT_Setting 'SQL','ClusterStart_Html_Page';

		SELECT * FROM dbo.tmpout_Export;


		2. EXECUTE this command over CMD or copy it into a bat file (Go to the drive you want to have the output to with D:, and to the fitting folder with cd ...)
		bcp.exe "SELECT Command FROM tmpout_Export" queryout import.sql -d AD00283_Plenovia_Dev -U octadmin -P [password] -S octproviderdev.database.windows.net -e error.log -c -C65001

		3. even you cant read it in SSMS, you can import it over SSMS as long a the file is smaller than 250 MB
		   (Output of SSMS is character limited, Input for SSMS if file size limited)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Setting',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Setting','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Setting
    (
          @Username			NVARCHAR(255)
		 ,@SettingID		NVARCHAR(50)	
    )

AS 
	BEGIN	


		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(@SettingID 				,N'NULL')	+ N''''
			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @SettingID			IS NULL SET @SettingID			= N'';



		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary Table to store the collection of export commands
				DROP TabLE IF EXISTS #tExport;
				CREATE TabLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Setting",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'"'))
						,(0, 7, N'-- }')


				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @SettingID		NVARCHAR(255)		= ''' + @SettingID + N'''')
						,(10, 4, N'--This Tab will be deleted during import, if it exists. You should be sure !')


				-- Create the Setting as new Setting
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.pPOST_Setting '
							,'''SQL'''
							,',@SettingID'
							,','''	,DataTypeCODE					,''''
							,','''	,SettingDescription				,''''	
							,','''	,ValueText						,''''
							,','''	,ValueDateTime					,''''
							,','	,ValueInt						,''
							,','	,ValueIntScale					,''
							,','''	,ValueJSON						,''''
							,','''	,Category						,''''
							,','	,ReadOnlySystemPropertyBool		,''
	
							)	AS Command
					FROM system.tSettings
					WHERE
						SettingID			= @SettingID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');

		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not execuTable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct daTabase access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Setting TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the Table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Setting'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports an Setting in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SettingID';
    SET @value      = N'ID of the Setting to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 


	GO
