
DROP PROCEDURE IF EXISTS system.spPOST_Dashboard;
GO
/*
	Procedure to POST an Dashboard
		- ID will be autogenerated if posted with * as ID
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_Dashboard
				 @Username				= N'SQL'
				,@DashboardID			= N'*'
				,@DashboardName			= N'Test'
				,@DashboardDescription	= N'bla'
				,@LayoutDefinition		= 0x
				,@OutputID				= @ReturnedID OUTPUT;
		PRINT    @RC
		PRINT   'Returned ID: ' + ISNULL(@ReturnedID,'<no ID returned>')

		SELECT * FROM system.tDashboards;
		DELETE FROM system.tDashboards;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Dashboard',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Dashboard','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spPOST_Dashboard
    (
         @Username				NVARCHAR(255)
        ,@DashboardID				NVARCHAR(50)
		,@DashboardName			NVARCHAR(255)	= NULL
		,@DashboardDescription		NVARCHAR(4000)	= NULL
		,@LayoutDefinition		VARBINARY(MAX)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@DashboardID				,N'NULL')	+ N''','''
				+ ISNULL(@DashboardName				,N'NULL')	+ N''','''
				+ ISNULL(@DashboardDescription		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastIDNum			NVARCHAR(50)	= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DashboardID				IS NULL SET @DashboardID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @DashboardID			= dbo.sx_pf_pProtectID	(@DashboardID);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Dashboard
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Transaction user don`t exists', 16, 10);
						END;

				-- Check existence of IDs of depening content

				-- Don't accept DashboardID = ''
				IF @DashboardID  = N'' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('DashboardID cant be empty string', 16, 10);
					END;

				-- cache existing values of this Object
				DROP TABLE IF EXISTS #CurrenctDashboard;
				CREATE TABLE #CurrentDashboard
					(
						 DashboardKey			BIGINT										NOT NULL
						,DashboardID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,DashboardName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,DashboardDescription	NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,LayoutDefinition	VARBINARY(MAX)								NOT NULL
					)
				INSERT INTO #CurrentDashboard
					SELECT
						 DashboardKey
						,DashboardID
						,DashboardName
						,DashboardDescription
						,LayoutDefinition
					FROM system.tDashboards
					WHERE DashboardID = @DashboardID;
			
				-- Data Transaction with * resolving
				PRINT CONCAT(N'Start writing Values...',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT @RowCount = Count (*) FROM #CurrentDashboard; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @DashboardID = '*'
							BEGIN
								SELECT TOP 1 
								@LastIDNum = TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT ) 
								FROM system.tDashboards
								WHERE 
										DashboardID LIKE N'D%'
									AND LEN(DashboardID) > 1
									AND TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) DESC

								IF @LastIDNum IS NULL SET @LastIDNum = 0;
								SET @DashboardID = 'D' + CAST(@LastIDNum +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tDashboards VALUES
							(		 
								 @DashboardID
								,ISNULL(@DashboardName,N'<set name>')
								,ISNULL(@DashboardDescription,'')
								,ISNULL(@LayoutDefinition,0x)
							);
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tR
								SET 
									 DashboardName				= ISNULL(@DashboardName			,cR.DashboardName)
									,DashboardDescription		= ISNULL(@DashboardDescription		,cR.DashboardDescription)
									,LayoutDefinition		= ISNULL(@LayoutDefinition		,cR.LayoutDefinition)
								
							FROM system.tDashboards tR
								LEFT JOIN #CurrentDashboard cR
									ON tR.DashboardKey = cR.DashboardKey
							WHERE tR.DashboardKey = cR.DashboardKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @DashboardID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Dashboard
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Dashboard	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spPOST_Dashboard TO pf_PlanningFactoryService;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Dashboard'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure POST a Dashboard Definition.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@DashboardID';
    SET @value      = N'BusinessID for the Dashboard - optional, autogenerated when posted as *.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DashboardName';
    SET @value      = N'Name for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@DashboardDescription';
    SET @value      = N'Description for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutDefinition';
    SET @value      = N'Layout Definition for the Dashboard as varbinary with images etc..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
