﻿
DROP PROCEDURE IF EXISTS system.spPOST_Icon;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC system.spPOST_Icon
			'SQL'
			,'Hase'
			,'svg'
			,0x3;

		SELECT * FROM system.tIcons

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Icon',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Icon','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Icon
	 @UserName		NVARCHAR(255)	= N''
	,@IconName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', ''' + ISNULL(@IconName, N'NULL') + N''', '''
													+ ISNULL(@FileExtension, N'NULL') + N''', ' + ISNULL(CAST(DATALENGTH(@FileBody) AS NVARCHAR(MAX)) + N' bytes', N'NULL') 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_Icon;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

			-- Delete File
			DELETE FROM system.tIcons WHERE IconName = @IconName
				
				IF LEN(@FileExtension) = 0 
				BEGIN
					SET @FileExtension = NULL;
				END;

				-- STEP 2.2 - Creating new file

				INSERT INTO system.tIcons (IconName,FileExtension, FileBody)
				VALUES (@IconName,@FileExtension, @FileBody);

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_Icon;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_Icon;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryService;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Icon'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'spPOST_Icon';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@UserName';
    SET @value      = N'UserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IconName';
    SET @value      = N'IconName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileExtension';
    SET @value      = N'FileExtension';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileBody';
    SET @value      = N'FileBody';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
