DROP TABLE IF EXISTS global.tCompanies;
GO
/*
	Table to store information about companies.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCompanies

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies','COLUMN',default)
*/

CREATE TABLE global.tCompanies (
     RowKey                   INT        IDENTITY(1,1)                          NOT NULL
    ,CompanyKey               AS (CONCAT(DataSourceKey,N'|',CompanyID))                  
    ,DataSourceKey            INT												NOT NULL -- this is indead an ID which is named Key and INT typed for compatibility
    ,CompanyID                NVARCHAR(50)                                      NOT NULL
    ,CompanyName              NVARCHAR(255)                                     NOT NULL
	,[Description]			  NVARCHAR(2000)									NOT NULL
    ,FinancialYearBegin       INT                                               NOT NULL
    ,ChartsOfAccountsID       NVARCHAR(50)                                      NOT NULL
    ,AccountLength            INT                                               NOT NULL
    ,Currency                 NVARCHAR(50)                                      NOT NULL
    ,CustomValuesJSON         NVARCHAR(MAX)										NOT NULL
    ,CustomValuesJSONUser     NVARCHAR(MAX)                                     NOT NULL
    ,OwnerCODE                NVARCHAR(50)										NOT NULL
    ,CompanyIDResult          NVARCHAR(50)										NOT NULL DEFAULT N''
    ,MappingPriority          INT          										NOT NULL DEFAULT 1

    ,CONSTRAINT PK_global_tCompanies_RowKey PRIMARY KEY CLUSTERED (RowKey)
	,CONSTRAINT AK_global_tCompanies_DataSourceIDCompanyID UNIQUE (DataSourceKey,CompanyID)
);

 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCompanies'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store information about companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyKey';
    SET @value      = N'Technical key for the company consisting of DataSourceID and CompanyID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'DataSourceKey';
    SET @value      = N'Business ID for the Datasource - this column is name Key due to module compatibiltity but is an NVARCHAR ID !.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyID';
    SET @value      = N'Business ID for the company, usually comes from the source system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyName';
    SET @value      = N'The company name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Description';
    SET @value      = N'The description for the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ChartsOfAccountsID';
    SET @value      = N'Business key for the charts of the accounts, referencing global.tFIN_ChartsOfAccounts';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AccountLength';
    SET @value      = N'Number indicating the length of the accounts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Currency';
    SET @value      = N'The currency of the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CustomValuesJSON';
    SET @value      = N'The content of the CostumValues JSON file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CustomValuesJSONUser';
    SET @value      = N'The content of the CostumValues JSON user file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'OwnerCODE';
    SET @value      = N'The Owner CODE, use default INTEGRATION.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CompanyIDResult';
    SET @value      = N'Can be used to change the CompanyID which is used in the result schema for this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'MappingPriority';
    SET @value      = N'In case multiple companies use the same CompanyID in the result this value will be used to determine the priority. The highest priority is 1 followed by 2, 3, ..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
