
DROP TABLE IF EXISTS global.tStructures;
GO

/*
	Table to store structures
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tStructures

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures','COLUMN',default)
*/

CREATE TABLE global.tStructures (
	 RowKey      			BIGINT       IDENTITY(1, 1) 	NOT NULL
	,StructureID 			NVARCHAR(3)   					NOT NULL
	,ParentID    			NVARCHAR(255) 					NOT NULL
	,StructureElementID      NVARCHAR(255) 					NOT NULL
	,Level       			INT          					NOT NULL

	,CONSTRAINT PK_global_tStructures PRIMARY KEY CLUSTERED (RowKey)
);

-- Documentation
DECLARE
	 @name			NVARCHAR(255)	= N'MS_Description'
    ,@level0type	NVARCHAR(255)	= N'Schema'
    ,@level0name	NVARCHAR(255)	= N'global'
    ,@level1type	NVARCHAR(255)	= N'Table'
    ,@level1name	NVARCHAR(255)	= N'tStructures'
	,@level2type	NVARCHAR(255)	= N'Column'
    ,@level2name	NVARCHAR(255)	= N''
	,@value			NVARCHAR(1000)	= N'';

	-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';
   
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tStructures TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tStructures'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store structures.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureID';
    SET @value      = N'Business key for the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ParentID';
    SET @value      = N'Business key for the parent.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureElementID';
    SET @value      = N'Business key for the structure element.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Level';
    SET @value      = N'Number indicating the level of the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
