
DROP TABLE IF EXISTS load.tdValueSeries;
GO
/*
	Table to store valueseries
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdValueSeries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries','COLUMN',default)
*/
 
CREATE TABLE load.tdValueSeries(
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL

	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdValueSeries PRIMARY KEY CLUSTERED (RowKey)

)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdValueSeries'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Business key for the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesNo';
    SET @value      = N'Numer of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsNumeric';
    SET @value      = N'Number to indicate whether valueseries is numeric.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'VisibilityLevel';
    SET @value      = N'Number to indicate whether valueseries is visible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueSource';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueListID';
    SET @value      = N'Valuelist ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormatID';
    SET @value      = N'Valueformat ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'Number indicating the scale of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Effect';
    SET @value      = N'The effect of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectParameter';
    SET @value      = N'The effect parameter of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Source';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
