
DROP TABLE IF EXISTS load.trRights;
GO
/*
	Table to store rights
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.trRights

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights','COLUMN',default)
*/

CREATE TABLE load.trRights (
	
	 RowKey					INT				NOT NULL IDENTITY (1,1) 
	,UserName				NVARCHAR(255)	NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,ProductLineID			NVARCHAR(255)	NOT NULL
	,ProductID				NVARCHAR(255)	NOT NULL
	,[Right]				NVARCHAR(255)	NOT NULL
	,ReadCommentMandatory	INT				NOT NULL
	,WriteCommentMandatory	INT				NOT NULL
	,Source					NVARCHAR(255)	NOT NULL
	,SourceTimeStamp		DATETIME		NOT NULL

	,CONSTRAINT PK_load_trRights PRIMARY KEY CLUSTERED (RowKey)
)
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT load.trRights TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trRights'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store rights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Right';
    SET @value      = N'Right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadCommentMandatory';
    SET @value      = N'Number indicating whether the read comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'WriteCommentMandatory';
    SET @value      = N'Number indicating whether the write comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Source';
    SET @value      = N'The source of the right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
