 
DROP TABLE IF EXISTS planning.tTabs;
GO
/*
	Table for storing Tabs, which can be used for
		- the cluster
		- the factory
		- the productline
		- the product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs','COLUMN',default)
*/
 
CREATE TABLE planning.tTabs
    (
         RowKey					BIGINT IDENTITY(1,1)
		,FactoryKey				INT					NOT NULL 				
		,ProductLineKey			INT 				NOT NULL
		,ProductKey				INT 				NOT NULL
		,FactoryID				NVARCHAR(255)		NOT NULL 				
		,ProductLineID			NVARCHAR(255) 		NOT NULL
		,ProductID				NVARCHAR(255) 		NOT NULL
        ,TabID					NVARCHAR(255)		NOT NULL
		,Orderindex				INT					NOT NULL
		,TabName				NVARCHAR(255)		NOT NULL
		,TabHint				NVARCHAR(2000)		NOT NULL
		,TabText				NVARCHAR(2000)		NOT NULL
		,PresentationCODE		NVARCHAR(255)		NOT NULL
		,Datasource				NVARCHAR(2000)		NOT NULL
		,IsVisibleBOOL			INT					NOT NULL
		,LayoutJSON				NVARCHAR(MAX)		NOT NULL
		,ParameterJSON			NVARCHAR(MAX)		NOT NULL

		,CONSTRAINT PK_tTabs PRIMARY KEY CLUSTERED (RowKey)
    )
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tTabs'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Tabs on all levels (Cluster, Factory, ProductLine, Product). Tabs can contain Pivot Tables, DataGrids, HTML Pages etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'FactoryKey, -1 for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLineKey, -1 for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'ProductKey, -1 for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryID';
    SET @value      = N'FactoryID, empty string for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLineID, empty String for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'ProductID, empty string for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabID';
    SET @value      = N'Business ID for the Tab, used as business identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Orderindex';
    SET @value      = N'Order number for the Tab in the list of tabs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabName';
    SET @value      = N'Name of the Tab, this Name is used as label for the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabHint';
    SET @value      = N'Text, which is show as Tooltip on the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TabText';
    SET @value      = N'Text, displayed in the header area inside the tab as explination';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PresentationCODE';
    SET @value      = N'Codeword for Presentationtype of the content, e.g. PIVOT, DATAGRID,..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Datasource';
    SET @value      = N'Datasource for the Tab, depending on Presentation CODE - StoredProcedure Name, URL, ...';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutJSON';
    SET @value      = N'Layout definition saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ParameterJSON';
    SET @value      = N'Parameter definition, saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
