DROP TABLE IF EXISTS planning.tdProducts;
GO

/*
	Table for Dimension Product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts','COLUMN',default)
*/

CREATE TABLE planning.tdProducts

	(
		 ProductKey			INT				NOT NULL IDENTITY (1,1)
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductID			NVARCHAR(255)	NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,TimeType			NVARCHAR(255)	NOT NULL
		,NameShort			NVARCHAR(255)	NOT NULL
		,NameLong			NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,ResponsiblePerson	NVARCHAR(255)	NOT NULL
		,ImageName			NVARCHAR(255)	NOT NULL
		,[Status]			NVARCHAR(255)	NOT NULL
		,Template			NVARCHAR(255)	NOT NULL
		,TemplateVersion	NVARCHAR(255)	NOT NULL
		,GlobalAttribute1	NVARCHAR(255)	NOT NULL
		,GlobalAttribute2	NVARCHAR(255)	NOT NULL
		,GlobalAttribute3	NVARCHAR(255)	NOT NULL
		,GlobalAttribute4	NVARCHAR(255)	NOT NULL
		,GlobalAttribute5	NVARCHAR(255)	NOT NULL
		,GlobalAttribute6	NVARCHAR(255)	NOT NULL
		,GlobalAttribute7	NVARCHAR(255)	NOT NULL
		,GlobalAttribute8	NVARCHAR(255)	NOT NULL
		,GlobalAttribute9	NVARCHAR(255)	NOT NULL
		,GlobalAttribute10	NVARCHAR(255)	NOT NULL
		,GlobalAttribute11	NVARCHAR(255)	NOT NULL
		,GlobalAttribute12	NVARCHAR(255)	NOT NULL
		,GlobalAttribute13	NVARCHAR(255)	NOT NULL
		,GlobalAttribute14	NVARCHAR(255)	NOT NULL
		,GlobalAttribute15	NVARCHAR(255)	NOT NULL
		,GlobalAttribute16	NVARCHAR(255)	NOT NULL
		,GlobalAttribute17	NVARCHAR(255)	NOT NULL
		,GlobalAttribute18	NVARCHAR(255)	NOT NULL
		,GlobalAttribute19	NVARCHAR(255)	NOT NULL
		,GlobalAttribute20	NVARCHAR(255)	NOT NULL
		,GlobalAttribute21	NVARCHAR(255)	NOT NULL
		,GlobalAttribute22	NVARCHAR(255)	NOT NULL
		,GlobalAttribute23	NVARCHAR(255)	NOT NULL
		,GlobalAttribute24	NVARCHAR(255)	NOT NULL
		,GlobalAttribute25	NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_dProducts PRIMARY KEY CLUSTERED (ProductKey)
	);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProducts'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ProductKey';
    SET @value      = N'Primary Key for product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeType';
    SET @value      = N'Time type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value      = N'The status of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Template';
    SET @value      = N'The template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TemplateVersion';
    SET @value      = N'The version of the template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute1';
    SET @value      = N'The 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute2';
    SET @value      = N'The 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute3';
    SET @value      = N'The 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute4';
    SET @value      = N'The 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute5';
    SET @value      = N'The 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute6';
    SET @value      = N'The 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute7';
    SET @value      = N'The 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute8';
    SET @value      = N'The 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute9';
    SET @value      = N'The 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute10';
    SET @value      = N'The 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute11';
    SET @value      = N'The 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute12';
    SET @value      = N'The 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute13';
    SET @value      = N'The 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute14';
    SET @value      = N'The 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute15';
    SET @value      = N'The 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute16';
    SET @value      = N'The 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute17';
    SET @value      = N'The 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute18';
    SET @value      = N'The 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute19';
    SET @value      = N'The 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute20';
    SET @value      = N'The 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttribute21';
    SET @value      = N'The 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute22';
    SET @value      = N'The 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute23';
    SET @value      = N'The 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute24';
    SET @value      = N'The 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute25';
    SET @value      = N'The 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
