
DROP TABLE IF EXISTS planning.tfValues;
GO
/*
	Table to store all OCT Values
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tfValues

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues','COLUMN',default)
*/

CREATE TABLE planning.tfValues
	(
		 ValueKey		BIGINT			NOT NULL IDENTITY (1,1)
		,ValueSeriesKey INT				NOT NULL
		,ProductKey		INT				NOT NULL
		,ProductLineKey INT				NOT NULL
		,FactoryKey		INT				NOT NULL
		,ProductID		NVARCHAR(255)	NOT NULL
		,ProductLineID	NVARCHAR(255)	NOT NULL
		,FactoryID		NVARCHAR(255)	NOT NULL
		,ValueSeriesID	NVARCHAR(255)	NOT NULL
		,TimeID			INT				NOT NULL
		,ValueFormula	NVARCHAR(MAX)	NOT NULL	
		,ValueInt		BIGINT			NOT NULL
		,ValueText		NVARCHAR(MAX)	NOT NULL
		,ValueComment	NVARCHAR(MAX)	NOT NULL

		,CONSTRAINT PK_fValues_ValueKey PRIMARY KEY CLUSTERED (ValueKey)
	);

-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE NONCLUSTERED INDEX IX_fValues_ProductKey ON planning.tfValues (ProductKey)
INCLUDE (ValueSeriesKey,ProductID,ProductLineID,FactoryID,ValueSeriesID,TimeID,ValueFormula,ValueInt,ValueText,ValueComment)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfValues'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store all OCT Values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ValueKey';
    SET @value      = N'Primary Key for value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ValueSeriesKey';
    SET @value      = N'Valueseries key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Valueseries ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimeID';
    SET @value      = N'Time ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormula';
    SET @value      = N'The value of the formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
