-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tConnectorQueries;

DROP TABLE IF EXISTS system.tConnectors;
/*
	Table to store connectors.

	This DELETE will delete also
			- the connector queries
	due to DELETE CASCADE
 
	Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectors

		INSERT INTO system.tConnectors VALUES
			 ('NAVISION','MSSQL','FIN','Connector for Navision MSSQL for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/ms-dynamics-nav/', 1)
			,('PROALPHA','ODBC','FIN','Connector for proAlpha ODBC for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/proalpha/',1)
			,('WODIS','ORACLE','FIN','Connector for Wodis Sigma ORACLE for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/wodis-sigma/',1)

		DELETE FROM system.tConnectors WHERE ConnectorKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors','COLUMN',default)
*/

CREATE TABLE system.tConnectors
    (
       ConnectorKey				INT				IDENTITY(1, 1) NOT NULL
      ,SourceSystemID			NVARCHAR(50)	NOT NULL
      ,SourceSystemType			NVARCHAR(50)	NOT NULL
      ,ModuleID					NVARCHAR(50)	NOT NULL
      ,ConnectorDescription		NVARCHAR(4000)	NOT NULL
      ,ConnectorDescriptionURL	NVARCHAR(255)	NOT NULL
      ,ConnectorVersion			INT				NOT NULL

      ,CONSTRAINT PK_tConnectors_ConnectorKey PRIMARY KEY CLUSTERED (ConnectorKey)
      ,CONSTRAINT AK_tConnectors_SourceSystemID_SourceSystemType_ModuleID
            UNIQUE (
                   SourceSystemID
                 , SourceSystemType
                 , ModuleID
                   )
    );

GO

ALTER TABLE system.tConnectors WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectors'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemID';
    SET @value      = N'Source System ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemType';
    SET @value      = N'Source System type for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ModuleID';
    SET @value      = N'Module ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescription';
    SET @value      = N'Description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescriptionURL';
    SET @value      = N'Link to Saxess website with detailed description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'ConnectorVersion';
    SET @value      = N'Version of the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
