
DROP TABLE IF EXISTS system.tEmails;
GO
/*
	Table to store Emails.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tEmails

		INSERT INTO system.tEmails VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe',3,'ALWAYS')

	Open Questions:
		- DELETE CASCADE to Files
		- DELETE CASCADE to Pipeline ?

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails','COLUMN',default)
*/

CREATE TABLE system.tEmails 
	(
     EmailKey					INT			IDENTITY(1,1)		NOT NULL
	,EmailID					NVARCHAR(50)					NOT NULL
	,EmailSubject				NVARCHAR(2000)					NOT NULL
    ,EmailRecipients			NVARCHAR(4000)					NOT NULL --List, demlited by ;
	,EmailText					NVARCHAR(MAX)					NOT NULL
	,EmailAttachments			NVARCHAR(2000)					NOT NULL --List of FileIDs, delimited by ;							
	,EmailStatusCODE			NVARCHAR(50)					NOT NULL --PENDING, ERROR, SENT
	,SourceTypeCODE				NVARCHAR(50)					NOT NULL --PIPELINE, EVENT, MANUALLY
	,SourceID					NVARCHAR(50)					NOT NULL --PipelineID or FID-PLID-PID
	,CreationDate				DATETIME						NOT NULL
	,PostDate					DATETIME						NULL
	
    ,CONSTRAINT PK_tEmails_EmailKey PRIMARY KEY CLUSTERED (EmailKey)
	);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tEmails'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Emails as a queue.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'EmailKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailID';
    SET @value      = N'BusinessID for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailSubject';
    SET @value      = N'Subject for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EmailRecipients';
    SET @value      = N'List of Email addresses, separated by semikolon.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'EmailText';
    SET @value      = N'Body for the Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailAttachments';
    SET @value      = N'List of FileIDs, which are stored separate in planning.tFiles';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
		
	SET @level2name = N'EmailStatusCODE';
    SET @value      = N'CODE for the sendig status of an email, PENDING, ERROR, SENT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceTypeCODE';
    SET @value      = N'CODE for the Source, which created this Email PIPELINE, EVENT, MANUALLY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceID';
    SET @value      = N'The ID chain of the object which created this email, eg. DataSource / Pipeline - the DSID-PID or an FID-PLID-PID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CreationDate';
    SET @value      = N'Datetime,when the Email was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PostDate';
    SET @value      = N'Datetime,when the Email was posted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
