﻿
DROP TABLE IF EXISTS system.tPipelineSchedules;
GO
/*
	Table to store defined schedules for pipelines.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineSchedules

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules','COLUMN',default)
*/

CREATE TABLE system.tPipelineSchedules
	(
	 ScheduleKey			INT IDENTITY(1,1)	NOT NULL
	,PipelineKey			INT					NOT NULL
	,ScheduleID				NVARCHAR(50)		NOT NULL
	,ScheduleName			NVARCHAR(255)		NOT NULL
	,ScheduleDescription	NVARCHAR(4000)		NOT NULL 
	,ScheduleDetailsJSON	NVARCHAR(MAX)		NOT NULL
	,Active					BIT					NOT NULL
	,CONSTRAINT PK_tPipelineSchedules PRIMARY KEY CLUSTERED (ScheduleKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSchedules_PipelineKeyScheduleID UNIQUE (PipelineKey,ScheduleID)
	,CONSTRAINT FK_tPipelineSchedules_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSchedules' -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store defined schedules for pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ScheduleKey';
    SET @value      = N'Stores the generated primary key of the table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Stores the PipelineKey that is executed with this scheduler job.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleID';
    SET @value      = N'ID for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleName';
    SET @value      = N'Name for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleDescription';
    SET @value      = N'Description of the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ScheduleDetailsJSON';
    SET @value      = N'Stores the rhythms which determines when and how often a pipeline is executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Stores if the scheduled job is currently active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
