
-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineSteps;
DROP TABLE IF EXISTS system.tPipelineSchedules;

DROP TABLE IF EXISTS system.tPipelines;
/*
	Table to store pipelines.
	This DELETE will delete also
			- the steps
			- the schedules
	due to DELETE CASCADE
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines

		INSERT INTO system.tPipelines VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe','#808080',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe','#808080',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe','#808080',3,'ALWAYS')

		DELETE FROM system.tPipelines WHERE PipelineID = 'P2';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines','COLUMN',default)
*/

CREATE TABLE system.tPipelines 
	(
     PipelineKey					INT			IDENTITY(1,1)		NOT NULL
	,PipelineID						NVARCHAR(50)					NOT NULL
    ,PipelineName					NVARCHAR(255)					NOT NULL   
	,PipelineDescription			NVARCHAR(4000)					NOT NULL
	,PipelineDescriptionIconColor	NVARCHAR(255)					NOT NULL 
	,OrderOfDisplay					INT								NOT NULL
	,EmailNotificationCODE			NVARCHAR(50)					NOT NULL --NEVER, ALWAYS, ONERROR
	
    ,CONSTRAINT PK_tPipelines_PipelineKey PRIMARY KEY CLUSTERED (PipelineKey)
	,CONSTRAINT AK_tPipelines_PipelineID  UNIQUE (PipelineID)
	);

GO

ALTER TABLE system.tPipelines WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelines'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineID';
    SET @value      = N'Business ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineName';
    SET @value      = N'Name of the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'PipelineDescription';
    SET @value      = N'Description for the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineDescriptionIconColor';
    SET @value      = N'Color of the Pipeline Icon, written as Hex Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfDisplay';
    SET @value      = N'Stores at which position the process pipeline should be displayed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailNotificationCODE';
    SET @value      = N'CODE word to aktivate Email notifications - NEVER, ALWAYS, ONERROR.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
