
DROP TABLE IF EXISTS system.tSettings;
GO
/*
	Table to store global database properties.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tSettings

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings','COLUMN',default)
*/

CREATE TABLE system.tSettings (
     RowKey						INT  IDENTITY(1,1)  NOT NULL
    ,SettingID					NVARCHAR(50)		NOT NULL 
	,SettingDescription			NVARCHAR(4000)		NOT NULL
	,DataTypeCODE				NVARCHAR(255)		NOT NULL
	,ValueText					NVARCHAR(4000)		NOT NULL CONSTRAINT Default_ValueText		DEFAULT ''
	,ValueDateTime				DATETIME			NOT NULL CONSTRAINT Default_ValueDateTime	DEFAULT '1900-01-01 00:00:00.000'
	,ValueInt					BIGINT				NOT NULL CONSTRAINT Default_ValueInt		DEFAULT 0
	,ValueIntScale				INT					NOT NULL CONSTRAINT Default_ValueIntScale	DEFAULT 1
	,ValueJSON					NVARCHAR(MAX)		NOT NULL CONSTRAINT Default_ValueJSON		DEFAULT ''
	,Category					NVARCHAR(255)		NOT NULL
	,ReadOnlySystemPropertyBool  INT				NOT NULL

   ,CONSTRAINT PK_tSettings PRIMARY KEY CLUSTERED (RowKey)
);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tSettings'      -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store global database properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SettingID';
    SET @value      = N'SettingID as business identifier for the entry.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SettingDescription';
    SET @value      = N'Description or comment of this setting for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DataTypeCODE';
    SET @value      = N'DataTypeCODE of setting that shows which value column is to used and how it should be used. Can be TEXT, DATETIME, BOOLEAN, NUMBER, JSON. See documentation for DataTypeCodes.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'Text value if value TEXT is set for column DataType, optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueDateTime';
    SET @value      = N'DateTime value if value DATETIME is set for column DataType, optional to submit default Value 1900-01-01 00:00:00.000 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'Integer value if value BOOLEAN or NUMBER is set for Column DataType. In Case of Boolean: 0,1,-1 for false,true and unknown. NUMBER - an Integer Number which must be scaled with column ValueIntScale. Optional to submit default Value 0 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueIntScale';
    SET @value      = N'Integer value of 1 for BOOLEAN value. Value of 1, 10, 100, 10000 for scaling type NUMBER to decimal value -> divide column ValueInt by column ValueIntScale. Optional to submit default Value 1 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueJSON';
    SET @value      = N'JSON value if DateTypeCODE is set to JSON. Optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Category';
    SET @value      = N'Category to group items.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadOnlySystemPropertyBool';
    SET @value      = N'Flag to signalize if property is changeable by user or is a Ready Only System Value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
