
DROP TABLE IF EXISTS system.trUser;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trUser

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser','COLUMN',default)
*/

CREATE TABLE system.trUser
	(
		 UserKey					INT				NOT NULL IDENTITY (1,1) 
		,UserName					NVARCHAR (255)	NOT NULL
		,PersonSurname				NVARCHAR (255)	NOT NULL
		,PersonFirstName			NVARCHAR (255)	NOT NULL
		,Email						NVARCHAR (255)	NOT NULL
		,LDAPIP						NVARCHAR (255)	NOT NULL
		,Status						NVARCHAR (255)	NOT NULL
		,DataFlowRightsCODE			NVARCHAR (255)	NOT NULL -- DESIGNER, EXECUTOR, NONE
		,VisualizationRightsCODE	NVARCHAR (255)	NOT NULL -- DESIGNER, VIEWER, NONE
		,IsAdministratorFlag		BIT				NOT NULL

		,CONSTRAINT PK_rUser_PK PRIMARY KEY CLUSTERED (UserKey)
	);

GO

-- GRANT
-- temporary GRANTs for procedures / functions in dbo schema (e.g. dbo.sx_pf_GET_ClusterAccessRight)
GRANT SELECT ON system.trUser TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUser'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store user of OCT.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'UserKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Username';
    SET @value		= N'Username for login - usually an User with Domain e.g. SX\GTA or an email Address.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonSurname';
    SET @value		= N'Surname of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonFirstName';
    SET @value		= N'FirstName of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Email';
    SET @value		= N'Email Address of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'LDAPIP';
    SET @value		= N'LDAPIP.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value		= N'Status of the user account. Capitalized word - may be "Active" or "Inactive".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DataFlowRightsCODE';
    SET @value		= N'CODE Word for the DataFlow Rights of the user (DESIGNER, EXECUTOR, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'VisualizationRightsCODE';
    SET @value		= N'CODE Word for the Visualization Rights of the user (DESIGNER, VIEWER, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsAdministratorFlag';
    SET @value		= N'Value 0/1 to flag if this user is an administorator of this OCT Database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
