
DROP TRIGGER IF EXISTS system.mqt_trUserRights_from_trUser;
GO
/*
	Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trUser
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		None - is triggerd

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser','TRIGGER','mqt_trUserRights_from_trUser')
*/

CREATE TRIGGER system.mqt_trUserRights_from_trUser

ON system.trUser

AFTER INSERT, UPDATE, DELETE 

AS 
BEGIN
	EXEC system.spMaterialize_trUserRights;
END

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    
    ,@level1name        NVARCHAR(255)   = N'trUser'			
	,@level2name		NVARCHAR(255)	= N'mqt_trUserRights_from_trUser'
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the trigger from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'TRIGGER'

    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Trigger documentation
    SET @value = N'Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trUser.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

GO
