
-- Script for creating a OCT datamodel in an existing database
-- Gerd Tautenhahn for saxess-software gmbh 09/2023
-- Version OCT 5.9.17
-- Coding UTF-8 without BOM 

SET QUOTED_IDENTIFIER ON
GO

-- Check for Version SQL 2016 or newer
-- Azure uses different Version number and is not checked

DECLARE @Versioninfo NVARCHAR(2000) = '';
SELECT @Versioninfo = 

	CASE 
		WHEN CONVERT(NVARCHAR(255),SERVERPROPERTY ('Edition')) LIKE '%Azure%'			THEN  'Azure Database - please keep / set compatibility level of database to highest version.'
		WHEN LEFT(CONVERT(NVARCHAR(255),SERVERPROPERTY ('productversion')),2) < 13		THEN  'SQL SERVER VERSION ALT !!!!! - Versionen vor SQL 2016 werden nicht unterstützt '
		WHEN LEFT(CONVERT(NVARCHAR(255),SERVERPROPERTY ('productversion')),2) > = 13	THEN  'SQL SERVER VERSION in Ordnung - Version 2016+'
		ELSE 'Fehler bei Versionbestimmung SQL Server'
	END;
PRINT @Versioninfo;

-- Check for correct compatibilty Level
SELECT @Versioninfo = 

	CASE 
		WHEN compatibility_level > = 130	THEN  'Datenbank Kompatibilität in Ordnung, Version ' + CAST(compatibility_level AS NVARCHAR(255))
		WHEN compatibility_level < 130		THEN  'KOMPATIBILITÄTSLEVEL ZU KLEIN !!!!! - Muss auf 130 oder höher für SQL 2016 Funktionsset geändert werden.'
		ELSE 'Fehler bei Kompatibilitätsbestimmung für  SQL Server'
	END

FROM sys.databases WHERE name = DB_NAME();

PRINT @Versioninfo;

/*
	Saxess Software GmbH
    Last modified 09/2022 for OCT 5.8
*/

IF schema_id('agent') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA agent AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used by agent procedures, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'agent'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used by agent procedures, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'agent'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('calc') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA calc AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used for calculation objects, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'calc'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used for calculation objects, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'calc'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('control') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA control AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used for control flows, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'control'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used for control flows, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'control'
GO
/*
    Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('global') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA global AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store global informations, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'global'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store global informations, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'global'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('import') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA import AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store planning import objects, to transport values from load tables to planning tables. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'import'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store planning import objects, to transport values from load tables to planning tables. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'import'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('integration') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA integration AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store integration data, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'integration'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store integration data, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'integration'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('load') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA load AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store generic import data for planning, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'load'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store generic import data for planning, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'load'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('param') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA param AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store parameter tables, created by product materialization. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'param'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store parameter tables, created by product materialization. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'param'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('planning') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA planning AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store objects for planning, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'planning'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store objects for planning, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'planning'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('result') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA result AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store objects which deliver data to enduser, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'result'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store objects which deliver data to enduser, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'result'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('staging') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA staging AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Depreciated Schema for planning datastaging - use integration now. Only delivered for compatibility reasons. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'staging'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Depreciated Schema for planning datastaging - use integration now. Only delivered for compatibility reasons. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'staging'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('system') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA system AUTHORIZATION dbo';
    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store system data and Metadata. Schema owned by dbo, NO table access for OCTService or other user.',
        @level0type = N'Schema',
        @level0name = N'system'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store system data and Metadata. Schema owned by dbo, NO table access for OCTService or other user.',
    @level0type = N'Schema',
    @level0name = N'system'
GO

/*
	Creates Database Roles for OCT
	Last modified 09/2022 for OCT 5.8
*/

IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryUser')		IS NULL CREATE ROLE pf_PlanningFactoryUser;
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryService')	IS NULL CREATE ROLE pf_PlanningFactoryService;
GO

-- this give the OCT Service the Right to create objects in all schema he owns. This are all schemas, except the schema system.
-- Owning a schema gives only ALTER Permissions to all objects, not CREATE Permissions
GRANT CREATE TABLE		TO pf_PlanningFactoryService;
GRANT CREATE PROCEDURE	TO pf_PlanningFactoryService;
GRANT CREATE VIEW		TO pf_PlanningFactoryService;
GRANT CREATE FUNCTION	TO pf_PlanningFactoryService;
GO
 DROP TABLE IF EXISTS control.tCustomFailureReport; 
 GO
/*
	Table for failure reporting
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM control.tCustomFailureReport

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport','COLUMN',default)
*/

CREATE TABLE control.tCustomFailureReport
(
	 RowKey				BIGINT IDENTITY (1,1) 
	,Timestamp			DATETIME		NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,TestSource			NVARCHAR(255)	NOT NULL
	,TestName			NVARCHAR(255)	NOT NULL
	,ResultType			NVARCHAR(255)	NOT NULL -- OK, ERROR, WARNING
	,ResultText			NVARCHAR(MAX)	NOT NULL
	,ResultNumeric		MONEY			NOT NULL

	,CONSTRAINT PK_tCustomFailureReport PRIMARY KEY NONCLUSTERED (RowKey)
)

GO

-- Table is clustered by Timestamp for faster query the last failure report
CREATE CLUSTERED INDEX IX_tCustomFailureReport_TimeFID ON control.tCustomFailureReport (Timestamp DESC, FactoryID ASC);

GO 

-- Hint Entry on Startup - will be deleted due to self cleaning in control procedure
INSERT INTO control.tCustomFailureReport 
	SELECT
		 GETUTCDATE() - 14	AS Timestamp
		,'AllFID'			AS FactoryID
		,'AllPLID'			AS ProductLineID
		,'AllPID'			AS ProductID
		,'Startup'			AS TestSoure
		,'Startup Hint'		AS TestName
		,'HINT'				AS ResultType
		,'Execute (everyday by agent) Stored Procedure control.spCustomFailureReport to see test results here.'	AS ResultText
		,0					AS ResultNumeric

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT control.tCustomFailureReport TO [Username or rolename]
 
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'control'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCustomFailureReport'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'				-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for failure reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the failure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TestSource';
    SET @value      = N'The test source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'TestName';
    SET @value      = N'The test name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultType';
    SET @value      = N'The result type of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ResultText';
    SET @value      = N'The content of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultNumeric';
    SET @value      = N'The numeric result.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS control.tSystemFailureReport; 
GO
/*
	Table for failure reporting
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM control.tSystemFailureReport

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tSystemFailureReport',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tSystemFailureReport','COLUMN',default)
*/

CREATE TABLE control.tSystemFailureReport
(
	 RowKey				BIGINT IDENTITY (1,1) 
	,Timestamp			DATETIME		NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,TestSource			NVARCHAR(255)	NOT NULL
	,TestName			NVARCHAR(255)	NOT NULL
	,ResultType			NVARCHAR(255)	NOT NULL -- OK, ERROR, WARNING
	,ResultText			NVARCHAR(MAX)	NOT NULL
	,ResultNumeric		MONEY			NOT NULL

	,CONSTRAINT PK_tSystemFailureReport PRIMARY KEY NONCLUSTERED (RowKey)
)

GO

-- Table is clustered by Timestamp for faster query the last failure report
CREATE CLUSTERED INDEX IX_tSystemFailureReport_TimeFID ON control.tSystemFailureReport (Timestamp DESC, FactoryID ASC);

GO 

-- Hint Entry on Startup - will be deleted due to self cleaning in control procedure
INSERT INTO control.tSystemFailureReport 
	SELECT
		 GETUTCDATE() - 14	AS Timestamp
		,'AllFID'			AS FactoryID
		,'AllPLID'			AS ProductLineID
		,'AllPID'			AS ProductID
		,'Startup'			AS TestSoure
		,'Startup Hint'		AS TestName
		,'HINT'				AS ResultType
		,'Execute (everyday by agent) Stored Procedure control.spSystemFailureReport to see test results here.'	AS ResultText
		,0					AS ResultNumeric

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT control.tSystemFailureReport TO [Username or rolename]
 
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'control'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tSystemFailureReport'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'				-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for failure reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the failure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TestSource';
    SET @value      = N'The test source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'TestName';
    SET @value      = N'The test name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultType';
    SET @value      = N'The result type of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ResultText';
    SET @value      = N'The content of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultNumeric';
    SET @value      = N'The numeric result.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	
	
	




	
DROP TABLE IF EXISTS global.tCodes;
GO

/*
	Table to store informations about CODEs
	these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCodes

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes','COLUMN',default)
*/

CREATE TABLE global.tCodes (
    RowKey              BIGINT        IDENTITY(1,1)     NOT NULL     
	,ModuleID			NVARCHAR(255)					NOT NULL
	,LanguageID			NVARCHAR(10)                    NOT NULL 
    ,CODE               NVARCHAR(50)                    NOT NULL
	,Scope				NVARCHAR(255)					NOT NULL
    ,[Description]		NVARCHAR(255)                   NOT NULL
	
    ,CONSTRAINT PK_global_tCodes PRIMARY KEY CLUSTERED (RowKey)
);

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tCodes TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCodes'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';

    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
  
 	SET @value = N'Table to store informations about CODEs - these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';

 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value		= N'ID for the Modul which created this Code. Part of combined Business Key ModuleID, LanguageID, CODE. For System Codes use ''system'' as ModuleID.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LanguageID';
    SET @value		= N'Language identifier for the code description. Part of combined Business Key ModuleID, LanguageID, CODE. Identifiers are stored in ISO 639-1 standard (e.g. DE, EN, FR, ...).';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CODE';
    SET @value		= N'Code written in uppercase letters, the CODE must be unique for the modul. Part of combined Business Key ModuleID, LanguageID, CODE.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scope';
    SET @value		= N'Scope for the CODE - can be used to allocate Codes to groups or objects. Use ''default'' if no scope is needed.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	       
	SET @level2name = N'Description';
    SET @value		= N'Description of CODE in specific language.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS global.tCompanies;
GO
/*
	Table to store information about companies.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCompanies

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies','COLUMN',default)
*/

CREATE TABLE global.tCompanies (
     RowKey                   INT        IDENTITY(1,1)                          NOT NULL
    ,CompanyKey               AS (CONCAT(DataSourceKey,N'|',CompanyID))                  
    ,DataSourceKey            INT												NOT NULL -- this is indead an ID which is named Key and INT typed for compatibility
    ,CompanyID                NVARCHAR(50)                                      NOT NULL
    ,CompanyName              NVARCHAR(255)                                     NOT NULL
	,[Description]			  NVARCHAR(2000)									NOT NULL
    ,FinancialYearBegin       INT                                               NOT NULL
    ,ChartsOfAccountsID       NVARCHAR(50)                                      NOT NULL
    ,AccountLength            INT                                               NOT NULL
    ,Currency                 NVARCHAR(50)                                      NOT NULL
    ,CustomValuesJSON         NVARCHAR(MAX)										NOT NULL
    ,CustomValuesJSONUser     NVARCHAR(MAX)                                     NOT NULL
    ,OwnerCODE                NVARCHAR(50)										NOT NULL
    ,CompanyIDResult          NVARCHAR(50)										NOT NULL DEFAULT N''
    ,MappingPriority          INT          										NOT NULL DEFAULT 1

    ,CONSTRAINT PK_global_tCompanies_RowKey PRIMARY KEY CLUSTERED (RowKey)
	,CONSTRAINT AK_global_tCompanies_DataSourceIDCompanyID UNIQUE (DataSourceKey,CompanyID)
);

 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCompanies'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store information about companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyKey';
    SET @value      = N'Technical key for the company consisting of DataSourceID and CompanyID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'DataSourceKey';
    SET @value      = N'Business ID for the Datasource - this column is name Key due to module compatibiltity but is an NVARCHAR ID !.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyID';
    SET @value      = N'Business ID for the company, usually comes from the source system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyName';
    SET @value      = N'The company name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Description';
    SET @value      = N'The description for the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ChartsOfAccountsID';
    SET @value      = N'Business key for the charts of the accounts, referencing global.tFIN_ChartsOfAccounts';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AccountLength';
    SET @value      = N'Number indicating the length of the accounts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Currency';
    SET @value      = N'The currency of the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CustomValuesJSON';
    SET @value      = N'The content of the CostumValues JSON file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CustomValuesJSONUser';
    SET @value      = N'The content of the CostumValues JSON user file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'OwnerCODE';
    SET @value      = N'The Owner CODE, use default INTEGRATION.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CompanyIDResult';
    SET @value      = N'Can be used to change the CompanyID which is used in the result schema for this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'MappingPriority';
    SET @value      = N'In case multiple companies use the same CompanyID in the result this value will be used to determine the priority. The highest priority is 1 followed by 2, 3, ..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
DROP TABLE IF EXISTS global.tPeriods;
PRINT N'Creating table global.tPeriods'
GO
/*
	Table to store informations about time periods.
	Each Year contains 16 Accounting periods, the periods 12...16 are mapped to month 12
	Each Year can be a Financal Year starting with calender month 1 (Financal Year = Calender Year),
	or an Financial Year starting in calender month <> 1.
	 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tPeriods;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods','COLUMN',default)
*/

CREATE TABLE global.tPeriods (
     RowKey                   INT        IDENTITY(1,1)     NOT NULL
	,PeriodKey  AS 
		  (FinancialYear * 10000) 
		+ (FinancialYearBegin * 100) 
		+ [Period]
    ,FinancialYear            INT							NOT NULL -- 2023
    ,[Period]                 INT							NOT NULL -- 1..16
    ,FinancialYearBegin       INT							NOT NULL -- 1..12
    ,FinancialYearName        NVARCHAR(50)					NOT NULL -- 2023 or 2023/2024	 
    ,CalendarYear             INT							NOT NULL -- 2023
    ,CalendarMonth            INT							NOT NULL -- 1..12
    ,CalendarQuarterCODE      NVARCHAR(50)					NOT NULL -- Q1...Q4
    ,CalendarHalfYearCODE     NVARCHAR(50)					NOT NULL -- HY1, HY2
	,YYYYMM					  INT							NOT NULL -- 202301
	,[YYYY-MM]				  NVARCHAR(50)					NOT NULL -- 2023-01						
	,YYYYQ					  INT							NOT NULL -- 20231...20234
	,[YYYY QQ]				  NVARCHAR(50)					NOT NULL --	2023 Q1..2023 Q4
    
	,CONSTRAINT PK_global_tPeriods PRIMARY KEY CLUSTERED (RowKey)
);

-- Default Content Flag
EXEC sp_addextendedproperty
    @name		= N'IsDefaultContent',
    @value		= N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tPeriods'
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tPeriods TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPeriods'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store informations about accounting time periods. Each Finanancial Year contains 16 periods.
				Financial years are defined for each possible start monat. FinancialYearBegin = 1 describes the Financial Year which is equal the calender year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PeriodKey';
    SET @value      = N'Key to identify a period, its automaticly calculated as (FinancialYear * 10000) + (FinancialYearBegin * 100) + [Period].';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYear';
    SET @value      = N'The financial Year YYYY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Period';
    SET @value      = N'The periods 1..16.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year. 1.. 12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearName';
    SET @value      = N'The name of the financial year. 2023 or 2023/2024';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarYear';
    SET @value      = N'The calendar year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarMonth';
    SET @value      = N'The calendar month.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarQuarterCODE';
    SET @value      = N'The CODE for the calendar quarter. Q1..Q4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarHalfYearCODE';
    SET @value      = N'The CODE for the calendar half year. HY1..HY2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYMM';
    SET @value      = N'Year and Month formated as YYYYMM';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY-MM';
    SET @value      = N'Year and Month formated as YYYY-MM, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYQ';
    SET @value      = N'Year and Quarter formated as YYYYQ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY QQ';
    SET @value      = N'Year and Quarter formated as YYYY QQ, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS global.tStructureDescriptions;
GO
/*
	Table to store structures.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tStructureDescriptions

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructureDescriptions',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructureDescriptions','COLUMN',default)
*/

CREATE TABLE global.tStructureDescriptions (
	 RowKey      			BIGINT IDENTITY(1, 1) 			NOT NULL
	,StructureElementID     NVARCHAR(255) 					NOT NULL
	,LanguageID         	NVARCHAR(255) 					NOT NULL
	,Description        	NVARCHAR(255) 					NOT NULL

	,CONSTRAINT PK_global_tStructureDescriptions PRIMARY KEY CLUSTERED (RowKey)
);

--Documentation
DECLARE
     @name          NVARCHAR(255)    = N'MS_Description'
    ,@level0type    NVARCHAR(255)    = N'Schema'
    ,@level0name    NVARCHAR(255)    = N'global'
    ,@level1type    NVARCHAR(255)    = N'Table'
    ,@level1name    NVARCHAR(255)    = N'tStructureDescriptions'
    ,@level2type    NVARCHAR(255)    = N'Column'
    ,@level2name    NVARCHAR(255)    = N''
    ,@value         NVARCHAR(1000)   = N'';

-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';
   
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tStructureDescriptions TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tStructureDescriptions'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'					-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'						-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1								-- 0 = Demo object - out of shipment process
																		-- STANDARD OBJECTS
																		-- 1 = shiped from saxess standard without modification
																		-- 2 = shiped from saxess standard modified FOR customer from saxess
																		-- 3 = shiped from saxess standard modified FOR customer from partner
																		-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		-- CUSTOM OBJECTS
																		-- 10 = shiped from saxess as customer specific object
																		-- 11 = shiped from partner	as customer specific object
																		-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store structures.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureElementID';
    SET @value      = N'Business key for the structure element.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'LanguageID';
    SET @value      = N'Business key for the language.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Description';
    SET @value      = N'The discription of the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP TABLE IF EXISTS global.tStructures;
GO

/*
	Table to store structures
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tStructures

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures','COLUMN',default)
*/

CREATE TABLE global.tStructures (
	 RowKey      			BIGINT       IDENTITY(1, 1) 	NOT NULL
	,StructureID 			NVARCHAR(3)   					NOT NULL
	,ParentID    			NVARCHAR(255) 					NOT NULL
	,StructureElementID      NVARCHAR(255) 					NOT NULL
	,Level       			INT          					NOT NULL

	,CONSTRAINT PK_global_tStructures PRIMARY KEY CLUSTERED (RowKey)
);

-- Documentation
DECLARE
	 @name			NVARCHAR(255)	= N'MS_Description'
    ,@level0type	NVARCHAR(255)	= N'Schema'
    ,@level0name	NVARCHAR(255)	= N'global'
    ,@level1type	NVARCHAR(255)	= N'Table'
    ,@level1name	NVARCHAR(255)	= N'tStructures'
	,@level2type	NVARCHAR(255)	= N'Column'
    ,@level2name	NVARCHAR(255)	= N''
	,@value			NVARCHAR(1000)	= N'';

	-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';
   
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tStructures TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tStructures'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store structures.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureID';
    SET @value      = N'Business key for the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ParentID';
    SET @value      = N'Business key for the parent.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureElementID';
    SET @value      = N'Business key for the structure element.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Level';
    SET @value      = N'Number indicating the level of the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdFactories;
GO
/*
	Table to store factories
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdFactories','COLUMN',default)
*/

CREATE TABLE load.tdFactories(
	
	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,FactoryID			NVARCHAR(255)	NOT NULL
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdFactories PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdFactories'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store factories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The person responsible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The name of the image.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'Source';
    SET @value      = N'The source of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdProductLines;
GO
/*
	Table to store ProductLines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProductLines','COLUMN',default)
*/

CREATE TABLE load.tdProductLines(
	 RowKey						INT				NOT NULL IDENTITY (1,1)
	,ProductLineID				NVARCHAR(255)	NOT NULL
	,FactoryID					NVARCHAR(255)	NOT NULL
	,NameShort					NVARCHAR(255)	NOT NULL
	,NameLong					NVARCHAR(255)	NOT NULL
	,CommentUser				NVARCHAR(MAX)	NOT NULL
	,CommentDev					NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson			NVARCHAR(255)	NOT NULL
	,ImageName					NVARCHAR(255)	NOT NULL
	,DefaultTemplate			NVARCHAR(255)	NOT NULL

	,GlobalAttributeSource1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource10	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias10		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource11	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias11		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource12	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias12		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource13	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias13		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource14	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias14		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource15	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias15		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource16	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias16		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource17	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias17		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource18	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias18		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource19	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias19		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource20	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias20		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource21	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias21		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource22	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias22		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource23	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias23		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource24	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias24		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource25	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias25		NVARCHAR(255)	NOT NULL
	,Source						NVARCHAR(255)	NOT NULL
	,SourceTimeStamp			DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdProductLines PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProductLines' -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store ProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DefaultTemplate';
    SET @value      = N'The default template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource1';
    SET @value      = N'The source of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias1';
    SET @value      = N'The alias of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource2';
    SET @value      = N'The source of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias2';
    SET @value      = N'The alias of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource3';
    SET @value      = N'The source of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias3';
    SET @value      = N'The alias of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource4';
    SET @value      = N'The source of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias4';
    SET @value      = N'The alias of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource5';
    SET @value      = N'The source of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias5';
    SET @value      = N'The alias of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource6';
    SET @value      = N'The source of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias6';
    SET @value      = N'The alias of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource7';
    SET @value      = N'The source of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias7';
    SET @value      = N'The alias of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource8';
    SET @value      = N'The source of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias8';
    SET @value      = N'The alias of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource9';
    SET @value      = N'The source of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias9';
    SET @value      = N'The alias of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource10';
    SET @value      = N'The source of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias10';
    SET @value      = N'The alias of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource11';
    SET @value      = N'The source of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias11';
    SET @value      = N'The alias of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource12';
    SET @value      = N'The source of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias12';
    SET @value      = N'The alias of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource13';
    SET @value      = N'The source of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias13';
    SET @value      = N'The alias of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource14';
    SET @value      = N'The source of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias14';
    SET @value      = N'The alias of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource15';
    SET @value      = N'The source of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias15';
    SET @value      = N'The alias of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource16';
    SET @value      = N'The source of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias16';
    SET @value      = N'The alias of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource17';
    SET @value      = N'The source of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias17';
    SET @value      = N'The alias of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource18';
    SET @value      = N'The source of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias18';
    SET @value      = N'The alias of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource19';
    SET @value      = N'The source of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias19';
    SET @value      = N'The alias of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource20';
    SET @value      = N'The source of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias20';
    SET @value      = N'The alias of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	    SET @level2name = N'GlobalAttributeSource21';
    SET @value      = N'The source of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias21';
    SET @value      = N'The alias of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource22';
    SET @value      = N'The source of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias22';
    SET @value      = N'The alias of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource23';
    SET @value      = N'The source of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias23';
    SET @value      = N'The alias of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource24';
    SET @value      = N'The source of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias24';
    SET @value      = N'The alias of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource25';
    SET @value      = N'The source of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias25';
    SET @value      = N'The alias of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdProducts;
GO
/*
	Table to store products
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProducts','COLUMN',default)
*/

CREATE TABLE load.tdProducts(

	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeType			NVARCHAR(255)	NOT NULL

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL

	,[Status]			NVARCHAR(255)	NOT NULL
	,Template			NVARCHAR(255)	NOT NULL
	,TemplateVersion	NVARCHAR(255)	NOT NULL

	,GlobalAttribute1	NVARCHAR(255)	NOT NULL
	,GlobalAttribute2	NVARCHAR(255)	NOT NULL
	,GlobalAttribute3	NVARCHAR(255)	NOT NULL
	,GlobalAttribute4	NVARCHAR(255)	NOT NULL
	,GlobalAttribute5	NVARCHAR(255)	NOT NULL
	,GlobalAttribute6	NVARCHAR(255)	NOT NULL
	,GlobalAttribute7	NVARCHAR(255)	NOT NULL
	,GlobalAttribute8	NVARCHAR(255)	NOT NULL
	,GlobalAttribute9	NVARCHAR(255)	NOT NULL
	,GlobalAttribute10	NVARCHAR(255)	NOT NULL
	,GlobalAttribute11	NVARCHAR(255)	NOT NULL
	,GlobalAttribute12	NVARCHAR(255)	NOT NULL
	,GlobalAttribute13	NVARCHAR(255)	NOT NULL
	,GlobalAttribute14	NVARCHAR(255)	NOT NULL
	,GlobalAttribute15	NVARCHAR(255)	NOT NULL
	,GlobalAttribute16	NVARCHAR(255)	NOT NULL
	,GlobalAttribute17	NVARCHAR(255)	NOT NULL
	,GlobalAttribute18	NVARCHAR(255)	NOT NULL
	,GlobalAttribute19	NVARCHAR(255)	NOT NULL
	,GlobalAttribute20	NVARCHAR(255)	NOT NULL
	,GlobalAttribute21	NVARCHAR(255)	NOT NULL
	,GlobalAttribute22	NVARCHAR(255)	NOT NULL
	,GlobalAttribute23	NVARCHAR(255)	NOT NULL
	,GlobalAttribute24	NVARCHAR(255)	NOT NULL
	,GlobalAttribute25	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdProducts PRIMARY KEY CLUSTERED (RowKey)

	)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProducts'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store products.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeType';
    SET @value      = N'Time type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value      = N'The status of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Template';
    SET @value      = N'The template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TemplateVersion';
    SET @value      = N'The version of the template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute1';
    SET @value      = N'The 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute2';
    SET @value      = N'The 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute3';
    SET @value      = N'The 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute4';
    SET @value      = N'The 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute5';
    SET @value      = N'The 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute6';
    SET @value      = N'The 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute7';
    SET @value      = N'The 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute8';
    SET @value      = N'The 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute9';
    SET @value      = N'The 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute10';
    SET @value      = N'The 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute11';
    SET @value      = N'The 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute12';
    SET @value      = N'The 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute13';
    SET @value      = N'The 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute14';
    SET @value      = N'The 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute15';
    SET @value      = N'The 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute16';
    SET @value      = N'The 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute17';
    SET @value      = N'The 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute18';
    SET @value      = N'The 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute19';
    SET @value      = N'The 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute20';
    SET @value      = N'The 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttribute21';
    SET @value      = N'The 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute22';
    SET @value      = N'The 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute23';
    SET @value      = N'The 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute24';
    SET @value      = N'The 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute25';
    SET @value      = N'The 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Source';
    SET @value      = N'The source of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdTime;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdTime

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdTime',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdTime','COLUMN',default)
*/

CREATE TABLE load.tdTime (
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeID				BIGINT			NOT NULL
	,FormatID			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdTime PRIMARY KEY (RowKey)

	)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdTime'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the timeline definition per Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimeID';
    SET @value      = N'Business key for the time.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FormatID';
    SET @value      = N'Business key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	    
	SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdValueSeries;
GO
/*
	Table to store valueseries
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdValueSeries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries','COLUMN',default)
*/
 
CREATE TABLE load.tdValueSeries(
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL

	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdValueSeries PRIMARY KEY CLUSTERED (RowKey)

)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdValueSeries'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Business key for the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesNo';
    SET @value      = N'Numer of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsNumeric';
    SET @value      = N'Number to indicate whether valueseries is numeric.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'VisibilityLevel';
    SET @value      = N'Number to indicate whether valueseries is visible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueSource';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueListID';
    SET @value      = N'Valuelist ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormatID';
    SET @value      = N'Valueformat ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'Number indicating the scale of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Effect';
    SET @value      = N'The effect of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectParameter';
    SET @value      = N'The effect parameter of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Source';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tfValues;
GO
/*
	Table to store values.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tfValues

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tfValues',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tfValues','COLUMN',default)
*/

CREATE TABLE load.tfValues(
 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL

	,TimeID				INT				NOT NULL

	,ValueFormula		NVARCHAR(MAX)	NOT NULL	
	,ValueInt			BIGINT			NOT NULL
	,ValueText			NVARCHAR(MAX)	NOT NULL
	,ValueComment		NVARCHAR(MAX)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tfValues PRIMARY KEY CLUSTERED (RowKey)
)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfValues'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Business key for the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimeID';
    SET @value      = N'Business key for the time.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormula';
    SET @value      = N'The value of the formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Source';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.trRights;
GO
/*
	Table to store rights
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.trRights

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights','COLUMN',default)
*/

CREATE TABLE load.trRights (
	
	 RowKey					INT				NOT NULL IDENTITY (1,1) 
	,UserName				NVARCHAR(255)	NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,ProductLineID			NVARCHAR(255)	NOT NULL
	,ProductID				NVARCHAR(255)	NOT NULL
	,[Right]				NVARCHAR(255)	NOT NULL
	,ReadCommentMandatory	INT				NOT NULL
	,WriteCommentMandatory	INT				NOT NULL
	,Source					NVARCHAR(255)	NOT NULL
	,SourceTimeStamp		DATETIME		NOT NULL

	,CONSTRAINT PK_load_trRights PRIMARY KEY CLUSTERED (RowKey)
)
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT load.trRights TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trRights'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store rights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Right';
    SET @value      = N'Right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadCommentMandatory';
    SET @value      = N'Number indicating whether the read comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'WriteCommentMandatory';
    SET @value      = N'Number indicating whether the write comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Source';
    SET @value      = N'The source of the right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.trUser;
GO
/*
	Table to store user of OCT.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.trUser

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trUser',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trUser','COLUMN',default)
*/

CREATE TABLE load.trUser(
	
	 RowKey						INT				NOT NULL IDENTITY (1,1) 
	,UserName					NVARCHAR(255)	NOT NULL
	,PersonSurname				NVARCHAR(255)	NOT NULL
	,PersonFirstName			NVARCHAR(255)	NOT NULL
	,Email						NVARCHAR(255)	NOT NULL
	,LDAPIP						NVARCHAR(255)	NOT NULL
	,Status						NVARCHAR(255)	NOT NULL
	,DataFlowRightsCODE			NVARCHAR(255)	NOT NULL
	,VisualizationRightsCODE	NVARCHAR(255)	NOT NULL
	,IsAdministratorFlag		BIT	NOT NULL
	,Source						NVARCHAR(255)	NOT NULL
	,SourceTimeStamp			DATETIME		NOT NULL

	,CONSTRAINT PK_load_trUser PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT load.trUser TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUser'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store user of OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'PersonSurname';
    SET @value      = N'The surname of the person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PersonFirstName';
    SET @value      = N'The first name of the person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Email';
    SET @value      = N'Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'LDAPIP';
    SET @value      = N'LDAPIP.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Status';
    SET @value      = N'The status of the user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source of the user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdFactories;
GO
/*
	Table for Dimension Factories
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdFactories','COLUMN',default)
*/
 
CREATE TABLE planning.tdFactories(

	 FactoryKey			INT				NOT NULL IDENTITY (1,1)
	,FactoryID			NVARCHAR(255)	NOT NULL
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dFactories_FactoryKey PRIMARY KEY CLUSTERED (FactoryKey)
);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdFactories'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension Factories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Primary Key for factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The person responsible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The name of the image.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdProductLines;
GO

/*
	Table for Dimension ProductLines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProductLines','COLUMN',default)
*/

CREATE TABLE planning.tdProductLines(
	 ProductLineKey				INT				NOT NULL IDENTITY (1,1)
	,FactoryKey					INT				NOT NULL 
	,ProductLineID				NVARCHAR(255)	NOT NULL
	,FactoryID					NVARCHAR(255)	NOT NULL
	,NameShort					NVARCHAR(255)	NOT NULL
	,NameLong					NVARCHAR(255)	NOT NULL
	,CommentUser				NVARCHAR(MAX)	NOT NULL
	,CommentDev					NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson			NVARCHAR(255)	NOT NULL
	,ImageName					NVARCHAR(255)	NOT NULL
	,DefaultTemplate			NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource10	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias10		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource11	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias11		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource12	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias12		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource13	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias13		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource14	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias14		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource15	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias15		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource16	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias16		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource17	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias17		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource18	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias18		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource19	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias19		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource20	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias20		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource21	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias21		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource22	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias22		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource23	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias23		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource24	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias24		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource25	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias25		NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dProductLines PRIMARY KEY CLUSTERED (ProductLineKey)
	);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProductLines'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension ProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ProductLineKey';
    SET @value      = N'Primary Key for productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DefaultTemplate';
    SET @value      = N'The default template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource1';
    SET @value      = N'The source of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias1';
    SET @value      = N'The alias of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource2';
    SET @value      = N'The source of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias2';
    SET @value      = N'The alias of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource3';
    SET @value      = N'The source of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias3';
    SET @value      = N'The alias of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource4';
    SET @value      = N'The source of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias4';
    SET @value      = N'The alias of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource5';
    SET @value      = N'The source of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias5';
    SET @value      = N'The alias of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource6';
    SET @value      = N'The source of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias6';
    SET @value      = N'The alias of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource7';
    SET @value      = N'The source of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias7';
    SET @value      = N'The alias of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource8';
    SET @value      = N'The source of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias8';
    SET @value      = N'The alias of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource9';
    SET @value      = N'The source of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias9';
    SET @value      = N'The alias of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource10';
    SET @value      = N'The source of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias10';
    SET @value      = N'The alias of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource11';
    SET @value      = N'The source of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias11';
    SET @value      = N'The alias of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource12';
    SET @value      = N'The source of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias12';
    SET @value      = N'The alias of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource13';
    SET @value      = N'The source of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias13';
    SET @value      = N'The alias of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource14';
    SET @value      = N'The source of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias14';
    SET @value      = N'The alias of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource15';
    SET @value      = N'The source of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias15';
    SET @value      = N'The alias of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource16';
    SET @value      = N'The source of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias16';
    SET @value      = N'The alias of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource17';
    SET @value      = N'The source of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias17';
    SET @value      = N'The alias of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource18';
    SET @value      = N'The source of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias18';
    SET @value      = N'The alias of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource19';
    SET @value      = N'The source of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias19';
    SET @value      = N'The alias of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource20';
    SET @value      = N'The source of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias20';
    SET @value      = N'The alias of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	    SET @level2name = N'GlobalAttributeSource21';
    SET @value      = N'The source of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias21';
    SET @value      = N'The alias of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource22';
    SET @value      = N'The source of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias22';
    SET @value      = N'The alias of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource23';
    SET @value      = N'The source of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias23';
    SET @value      = N'The alias of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource24';
    SET @value      = N'The source of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias24';
    SET @value      = N'The alias of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource25';
    SET @value      = N'The source of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias25';
    SET @value      = N'The alias of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS planning.tdProducts;
GO

/*
	Table for Dimension Product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts','COLUMN',default)
*/

CREATE TABLE planning.tdProducts

	(
		 ProductKey			INT				NOT NULL IDENTITY (1,1)
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductID			NVARCHAR(255)	NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,TimeType			NVARCHAR(255)	NOT NULL
		,NameShort			NVARCHAR(255)	NOT NULL
		,NameLong			NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,ResponsiblePerson	NVARCHAR(255)	NOT NULL
		,ImageName			NVARCHAR(255)	NOT NULL
		,[Status]			NVARCHAR(255)	NOT NULL
		,Template			NVARCHAR(255)	NOT NULL
		,TemplateVersion	NVARCHAR(255)	NOT NULL
		,GlobalAttribute1	NVARCHAR(255)	NOT NULL
		,GlobalAttribute2	NVARCHAR(255)	NOT NULL
		,GlobalAttribute3	NVARCHAR(255)	NOT NULL
		,GlobalAttribute4	NVARCHAR(255)	NOT NULL
		,GlobalAttribute5	NVARCHAR(255)	NOT NULL
		,GlobalAttribute6	NVARCHAR(255)	NOT NULL
		,GlobalAttribute7	NVARCHAR(255)	NOT NULL
		,GlobalAttribute8	NVARCHAR(255)	NOT NULL
		,GlobalAttribute9	NVARCHAR(255)	NOT NULL
		,GlobalAttribute10	NVARCHAR(255)	NOT NULL
		,GlobalAttribute11	NVARCHAR(255)	NOT NULL
		,GlobalAttribute12	NVARCHAR(255)	NOT NULL
		,GlobalAttribute13	NVARCHAR(255)	NOT NULL
		,GlobalAttribute14	NVARCHAR(255)	NOT NULL
		,GlobalAttribute15	NVARCHAR(255)	NOT NULL
		,GlobalAttribute16	NVARCHAR(255)	NOT NULL
		,GlobalAttribute17	NVARCHAR(255)	NOT NULL
		,GlobalAttribute18	NVARCHAR(255)	NOT NULL
		,GlobalAttribute19	NVARCHAR(255)	NOT NULL
		,GlobalAttribute20	NVARCHAR(255)	NOT NULL
		,GlobalAttribute21	NVARCHAR(255)	NOT NULL
		,GlobalAttribute22	NVARCHAR(255)	NOT NULL
		,GlobalAttribute23	NVARCHAR(255)	NOT NULL
		,GlobalAttribute24	NVARCHAR(255)	NOT NULL
		,GlobalAttribute25	NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_dProducts PRIMARY KEY CLUSTERED (ProductKey)
	);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProducts'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ProductKey';
    SET @value      = N'Primary Key for product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeType';
    SET @value      = N'Time type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value      = N'The status of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Template';
    SET @value      = N'The template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TemplateVersion';
    SET @value      = N'The version of the template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute1';
    SET @value      = N'The 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute2';
    SET @value      = N'The 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute3';
    SET @value      = N'The 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute4';
    SET @value      = N'The 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute5';
    SET @value      = N'The 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute6';
    SET @value      = N'The 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute7';
    SET @value      = N'The 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute8';
    SET @value      = N'The 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute9';
    SET @value      = N'The 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute10';
    SET @value      = N'The 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute11';
    SET @value      = N'The 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute12';
    SET @value      = N'The 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute13';
    SET @value      = N'The 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute14';
    SET @value      = N'The 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute15';
    SET @value      = N'The 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute16';
    SET @value      = N'The 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute17';
    SET @value      = N'The 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute18';
    SET @value      = N'The 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute19';
    SET @value      = N'The 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute20';
    SET @value      = N'The 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttribute21';
    SET @value      = N'The 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute22';
    SET @value      = N'The 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute23';
    SET @value      = N'The 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute24';
    SET @value      = N'The 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute25';
    SET @value      = N'The 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdTime;
GO

/*
	Table for Dimension Time in OCT
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdTime

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdTime',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdTime','COLUMN',default)
*/

CREATE TABLE planning.tdTime
	(
	 TimeKey		INT				NOT NULL IDENTITY (1,1)
	,ProductKey		INT				NOT NULL
	,ProductLineKey INT				NOT NULL
	,FactoryKey		INT				NOT NULL
	,ProductID		NVARCHAR(255)	NOT NULL
	,ProductLineID	NVARCHAR(255)	NOT NULL
	,FactoryID		NVARCHAR(255)	NOT NULL
	,TimeID			BIGINT			NOT NULL
	,FormatID		NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dTime_TimeKey PRIMARY KEY CLUSTERED (TimeKey)
	);

-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE INDEX IX_dTime_PKey ON planning.tdTime (ProductKey ASC);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdTime'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the timeline definition per Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'TimeKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductKey';
    SET @value      = N'Foreign technical Key to table planning.tdProducts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'Foreign technical Key to table planning.tdProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Foreign technical Key to table planning.tdFactories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Composite business Key (togehter with ProductLineID, FactoryID) to table planning.tdProducts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Composite business Key (togehter with FactoryID) to table planning.tdProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Foreign business Key to table planning.tdFactories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimeID';
    SET @value      = N'Primary Business Key, used for time allocation of the value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FormatID';
    SET @value      = N'Foreign Key to table planning.thFormats, to enable format defintion for a TimeID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdValueSeries;
GO
/*
	Table for Dimension ValueSeries
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdValueSeries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdValueSeries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdValueSeries','COLUMN',default)
*/

CREATE TABLE planning.tdValueSeries
	(
	 ValueSeriesKey		INT				NOT NULL IDENTITY (1,1)
	,ProductKey			INT				NOT NULL
	,ProductLineKey		INT				NOT NULL
	,FactoryKey			INT				NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL
	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dValueSeries PRIMARY KEY CLUSTERED (ValueSeriesKey)
	);

	-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE INDEX IX_dValueSeries_ProductKey ON planning.tdValueSeries (ProductKey ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdValueSeries'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ValueSeriesKey';
    SET @value      = N'Primary Key for valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Valueseries ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesNo';
    SET @value      = N'Valueseries Number.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsNumeric';
    SET @value      = N'Number to indicate whether valueseries is numeric.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'VisibilityLevel';
    SET @value      = N'Number to indicate whether valueseries is visible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueSource';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueListID';
    SET @value      = N'Valuelist ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormatID';
    SET @value      = N'Valueformat ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'The scale the valueseries is set to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Effect';
    SET @value      = N'The effect of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectParameter';
    SET @value      = N'The effect parameter of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS planning.tFiles;
GO
/*
	Table to store files
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tFiles

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tFiles',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tFiles','COLUMN',default)
*/

CREATE TABLE planning.tFiles (
	 FileKey			BIGINT IDENTITY(1,1)	NOT NULL
	,FactoryKey			INT						NOT NULL
	,ProductLineKey		INT						NOT NULL
	,ProductKey			INT						NOT NULL
	,FactoryID			NVARCHAR(50)			NOT NULL
	,ProductLineID		NVARCHAR(50)			NOT NULL
	,ProductID			NVARCHAR(50)			NOT NULL
	,FileID				NVARCHAR(50)			NOT NULL
	,[FileName]			NVARCHAR(255)			NOT NULL
	,FileExtension		NVARCHAR(50)			NULL
	,FileBody			VARBINARY(MAX)			NULL
	,UploadedBy			NVARCHAR(255)			NOT NULL
	,UploadedAt			DATETIME				NOT NULL
	,Filepath			NVARCHAR(4000)			NOT NULL

	,CONSTRAINT pk_tFiles PRIMARY KEY CLUSTERED (FileKey)
);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tFiles'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store files.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FileKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryKey';
    SET @value      = N'Technical Key for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ProductLineKey';
    SET @value      = N'Technical Key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductKey';
    SET @value      = N'Technical Key for the Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryID';
    SET @value      = N'Business ID for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business ID for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductID';
    SET @value      = N'Business ID for the Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileID';
    SET @value      = N'Business ID for the File.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileName';
    SET @value      = N'Name of the File without extension.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'FileExtension';
    SET @value      = N'Separated File extension for better handling.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileBody';
    SET @value      = N'File Body as binary data.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UploadedBy';
    SET @value      = N'Name of the user which uploaded this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UploadedAt';
    SET @value      = N'Datetime of upload of this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Filepath';
    SET @value      = N'Virtual path of this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tfStatements;
GO
/*
	Table to store Statements
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tfStatements

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfStatements',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfStatements','COLUMN',default)
*/

CREATE TABLE planning.tfStatements
	(
		 StatementKey	INT				NOT NULL IDENTITY (1,1)
		,ProductKey		INT				NOT NULL
		,ProductLineKey INT				NOT NULL
		,FactoryKey		INT				NOT NULL
		,ProductID		NVARCHAR(255)	NOT NULL
		,ProductLineID	NVARCHAR(255)	NOT NULL
		,FactoryID		NVARCHAR(255)	NOT NULL
		,ActionType		NVARCHAR(255)	NOT NULL
		,[Statement]	NVARCHAR(MAX)	NOT NULL
		,UserName		NVARCHAR(255)	NOT NULL
		,PCName			NVARCHAR(255)	NOT NULL
		,ProcessorCode	NVARCHAR(255)	NOT NULL
		,IPAddresses	NVARCHAR(255)	NOT NULL
		,Timestamp		DATETIME		NOT NULL
		,IsDeleted		INT				NOT NULL
		,IsResolved		INT				NOT NULL

		,CONSTRAINT PK_fStatements PRIMARY KEY CLUSTERED (StatementKey)
	);

-- Table is indexed by ProductKey for fasten Query on ProductDataTable 
CREATE INDEX IX_fStatements_ProductKey ON planning.tfStatements (ProductKey ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfStatements'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store statements.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'StatementKey';
    SET @value      = N'Primary Key for the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ActionType';
    SET @value      = N'Action type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Statement';
    SET @value      = N'The content of the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'PCName';
    SET @value      = N'Processor Code name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProcessorCode';
    SET @value      = N'Processor Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IPAddresses';
    SET @value      = N'IP adresses.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsDeleted';
    SET @value      = N'Number to indicate whether the statement is deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IsResolved';
    SET @value      = N'Number to indicate whether the statement is resolved.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tfValues;
GO
/*
	Table to store all OCT Values
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tfValues

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues','COLUMN',default)
*/

CREATE TABLE planning.tfValues
	(
		 ValueKey		BIGINT			NOT NULL IDENTITY (1,1)
		,ValueSeriesKey INT				NOT NULL
		,ProductKey		INT				NOT NULL
		,ProductLineKey INT				NOT NULL
		,FactoryKey		INT				NOT NULL
		,ProductID		NVARCHAR(255)	NOT NULL
		,ProductLineID	NVARCHAR(255)	NOT NULL
		,FactoryID		NVARCHAR(255)	NOT NULL
		,ValueSeriesID	NVARCHAR(255)	NOT NULL
		,TimeID			INT				NOT NULL
		,ValueFormula	NVARCHAR(MAX)	NOT NULL	
		,ValueInt		BIGINT			NOT NULL
		,ValueText		NVARCHAR(MAX)	NOT NULL
		,ValueComment	NVARCHAR(MAX)	NOT NULL

		,CONSTRAINT PK_fValues_ValueKey PRIMARY KEY CLUSTERED (ValueKey)
	);

-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE NONCLUSTERED INDEX IX_fValues_ProductKey ON planning.tfValues (ProductKey)
INCLUDE (ValueSeriesKey,ProductID,ProductLineID,FactoryID,ValueSeriesID,TimeID,ValueFormula,ValueInt,ValueText,ValueComment)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfValues'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store all OCT Values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ValueKey';
    SET @value      = N'Primary Key for value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ValueSeriesKey';
    SET @value      = N'Valueseries key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Valueseries ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimeID';
    SET @value      = N'Time ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormula';
    SET @value      = N'The value of the formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgCluster;
GO

/*
	Table to store global Cluster Informations
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgCluster

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgCluster',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgCluster','COLUMN',default)
*/

CREATE TABLE planning.tgCluster

	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL
	
		,CONSTRAINT PK_gCluster_PK PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gCluster_PropID ON planning.tgCluster (PropertyID ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgCluster'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store global Cluster Informations.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentUser';
    SET @value      = N'User comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CommentDev';
    SET @value      = N'Development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgFactories;
GO
/*
	Table to store Factory Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgFactories','COLUMN',default)
*/

CREATE TABLE planning.tgFactories
  (
	 PropertyKey			INT				NOT NULL IDENTITY (1,1)
    ,FactoryKey				INT				NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,PropertyID				NVARCHAR(255)	NOT NULL
	,PropertyName			NVARCHAR(255)	NOT NULL
	,CommentUser			NVARCHAR(MAX)	NOT NULL
	,CommentDev				NVARCHAR(MAX)	NOT NULL
	,Unit					NVARCHAR(255)	NOT NULL
	,ValueText				NVARCHAR(MAX)	NOT NULL
	,ValueInt				INT				NOT NULL
	,Scale					INT				NOT NULL
	,IsROSystemProperty		INT				NOT NULL
	,FormatID				NVARCHAR(255)	NOT NULL

    ,CONSTRAINT PK_gFactories PRIMARY KEY CLUSTERED (PropertyKey)
  );

  -- Clustered on Factory Key, as Queries call Properties per Factory
  CREATE NONCLUSTERED INDEX IX_gFactories_FKey ON planning.tgFactories (FactoryKey ASC);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgFactories'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Factory Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation

	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgProductLines;
GO
/*
	Table to Store ProductLine Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProductLines','COLUMN',default)
*/

CREATE TABLE planning.tgProductLines
	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gProductLines PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gProductLines_PLKey ON planning.tgProductLines (ProductLineKey ASC);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgProductLines'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to Store ProductLine Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgProducts;
GO
/*
	Table to store Product Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProducts','COLUMN',default)
*/

CREATE TABLE planning.tgProducts
	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,ProductKey			INT				NOT NULL
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductID			NVARCHAR(255)	NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gProducts PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gProducts_PKey ON planning.tgProducts (ProductKey ASC);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgProducts'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Product Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO

DROP TABLE IF EXISTS planning.tgValueEffects;
GO
/*
	Table to store ValueEffects
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgValueEffects

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgValueEffects',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgValueEffects','COLUMN',default)
*/

CREATE TABLE planning.tgValueEffects
	(
		 EffectKey					INT				NOT NULL IDENTITY (1,1)
		,EffectID					NVARCHAR (255)  NOT NULL
		,ProfitLossEffect			INT				NOT NULL
		,BalanceEffect				INT				NOT NULL
		,CashEffect					INT				NOT NULL
		,QuantityEffect				INT				NOT NULL
		,AggregationTypTime			NVARCHAR(255)	NOT NULL
		,AggregationTypStructure	NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gValueEffects_PK PRIMARY KEY CLUSTERED (EffectKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgValueEffects'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store ValueEffects.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'EffectKey';
    SET @value      = N'Primary Key for the value effect.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectID';
    SET @value      = N'ID of the value effect.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProfitLossEffect';
    SET @value      = N'Effect of the profit loss.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'BalanceEffect';
    SET @value      = N'Effect of the balance.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CashEffect';
    SET @value      = N'Effect of the cash.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'QuantityEffect';
    SET @value      = N'Effect of the quantity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AggregationTypTime';
    SET @value      = N'The time of the aggregation typ.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'AggregationTypStructure';
    SET @value      = N'The structure of the aggregation typ.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP TABLE IF EXISTS planning.thFormats;
GO
/*
	Table for format definitions. They are applied on:
		- ValueSeries
		- TimeIDs (only borders at the moment)
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thFormats

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats','COLUMN',default)
*/

CREATE TABLE planning.thFormats 

	(
	 FormatKey				INT				NOT NULL IDENTITY (1,1)
	,FormatID				NVARCHAR(255)	NOT NULL
	,BackgroundColor		NVARCHAR(255)	NOT NULL
	,FontColor				NVARCHAR(255)	NOT NULL
	,ValueFormat			NVARCHAR(255)	NOT NULL
	,Bold					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,Italic					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,AlignmentHorizontal	NVARCHAR(255)	NOT NULL -- left, center, right
	,AlignmentVertical		NVARCHAR(255)	NOT NULL -- top, center, bottom
	,TextUnderlined			INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,BorderToNext			INT				NOT NULL -- Boolean
	,BorderToPrevious		INT				NOT NULL -- Boolean

	,CONSTRAINT PK_hFormats_PK PRIMARY KEY NONCLUSTERED (FormatKey)
	);

CREATE CLUSTERED INDEX IX_hFormats_FormatID ON planning.thFormats (FormatID ASC);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thFormats'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Format Definitions, which are applied on ValueSeries, GlobalAttributes and TimeIDs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FormatKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'Business Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BackgroundColor';
    SET @value      = N'RGB Color Definition, which is applied for Background eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FontColor';
    SET @value      = N'RGB Color Definition, which is applied for Fonts eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueFormat';
    SET @value      = N'Format Definition in englisch XLS Style, which is applied to format values, e.g. #,###0.00 for Thousand Separator and two decimal places.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Bold';
    SET @value      = N'Font is bold Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Italic';
    SET @value      = N'Font is italic Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'AlignmentHorizontal';
    SET @value      = N'Definition of horizontal font alignment, allowed values are left, center, right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AlignmentVertical';
    SET @value      = N'Definition of vertical font alignment, allowed values are top, center, bottom.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TextUnderlined';
    SET @value      = N'Font is underlined Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BorderToNext';
    SET @value      = N'Definition Flag if a border to the next (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'BorderToPrevious';
    SET @value      = N'Definition Flag if a border to the previous (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.thLists;
GO
/*
	Table to store Listnames
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thLists 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thLists ',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thLists ','COLUMN',default)
*/

CREATE TABLE planning.thLists 
	(
		 ListKey		INT NOT NULL IDENTITY (1,1)
		,ListID			NVARCHAR (255)	NOT NULL
		,NameShort		NVARCHAR (255)	NOT NULL
		,NameLong		NVARCHAR (255)	NOT NULL
		,CommentDev		NVARCHAR (255)	NOT NULL
		,CommentUser	NVARCHAR (255)	NOT NULL
		,Datentyp		NVARCHAR (255)	NOT NULL
		,[Source]		NVARCHAR (255)	NOT NULL
		,SourceFormula	NVARCHAR (1000) NOT NULL
		,FormatID		NVARCHAR (255)	NOT NULL

		,CONSTRAINT PK_hLists_PK PRIMARY KEY CLUSTERED (ListKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thLists '       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Listnames.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ListKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ListID';
    SET @value      = N'Bussines Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Datentyp';
    SET @value      = N'The data type of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'SourceFormula';
    SET @value      = N'The source formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'Bussines Key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.thListValues;
GO
/*
	Table to store List Values
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thListValues 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thListValues ',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thListValues ','COLUMN',default)
*/

CREATE TABLE planning.thListValues 
	(
		 ListValueKey	INT				NOT NULL IDENTITY (1,1)
		,ListID			NVARCHAR (255)	NOT NULL
		,ValueInt		BIGINT			NOT NULL
		,Scale			BIGINT			NOT NULL
		,ValueText		NVARCHAR (255)	NOT NULL
		,ValueComment	NVARCHAR (255)	NOT NULL
		,FormatID		NVARCHAR (255)	NOT NULL

		,CONSTRAINT PK_hListValues_PK PRIMARY KEY CLUSTERED (ListValueKey)
	);

	CREATE NONCLUSTERED INDEX IX_hListValues_ListID ON planning.thListValues (ListID ASC);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thListValues '          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store List Values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ListValueKey';
    SET @value      = N'Technical Primary Key for list value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ListID';
    SET @value      = N'Business key for List.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'The scale of the list value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FormatID';
    SET @value      = N'Business key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.trPreferences;
GO
/*
	Table to store Preferences - in contrast to properties, preferences can be userdependend
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.trPreferences

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'trPreferences',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'trPreferences','COLUMN',default)
*/

CREATE TABLE planning.trPreferences
	(
		 PreferenceKey	INT				NOT NULL IDENTITY (1,1) 
		,UserKey		INT				NOT NULL
		,UserName		NVARCHAR (255)	NOT NULL
		,FactoryID		NVARCHAR (255)	NOT NULL
		,ProductLineID	NVARCHAR (255)	NOT NULL
		,SettingName	NVARCHAR (255)	NOT NULL
		,ValueInt		INT				NOT NULL
		,ValueText		NVARCHAR (MAX)	NOT NULL

		,CONSTRAINT PK_rPreferences_PK PRIMARY KEY CLUSTERED (PreferenceKey)
	);

	-- clusterd by FactoryID may not be the best choise - the useful index depends strong on Cluster Configuration
	CREATE NONCLUSTERED INDEX IX_rPreferences_FID ON planning.trPreferences (FactoryID ASC);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trPreferences'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Preferences - in contrast to properties, preferences can be userdependend.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PreferenceKey';
    SET @value      = N'Technical Primary Key for Preference.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserKey';
    SET @value      = N'Technical Key for User.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'UserName';
    SET @value      = N'User Name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SettingName';
    SET @value      = N'The setting for the name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP TABLE IF EXISTS planning.tTabs;
GO
/*
	Table for storing Tabs, which can be used for
		- the cluster
		- the factory
		- the productline
		- the product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs','COLUMN',default)
*/
 
CREATE TABLE planning.tTabs
    (
         RowKey					BIGINT IDENTITY(1,1)
		,FactoryKey				INT					NOT NULL 				
		,ProductLineKey			INT 				NOT NULL
		,ProductKey				INT 				NOT NULL
		,FactoryID				NVARCHAR(255)		NOT NULL 				
		,ProductLineID			NVARCHAR(255) 		NOT NULL
		,ProductID				NVARCHAR(255) 		NOT NULL
        ,TabID					NVARCHAR(255)		NOT NULL
		,Orderindex				INT					NOT NULL
		,TabName				NVARCHAR(255)		NOT NULL
		,TabHint				NVARCHAR(2000)		NOT NULL
		,TabText				NVARCHAR(2000)		NOT NULL
		,PresentationCODE		NVARCHAR(255)		NOT NULL
		,Datasource				NVARCHAR(2000)		NOT NULL
		,IsVisibleBOOL			INT					NOT NULL
		,LayoutJSON				NVARCHAR(MAX)		NOT NULL
		,ParameterJSON			NVARCHAR(MAX)		NOT NULL

		,CONSTRAINT PK_tTabs PRIMARY KEY CLUSTERED (RowKey)
    )
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tTabs'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Tabs on all levels (Cluster, Factory, ProductLine, Product). Tabs can contain Pivot Tables, DataGrids, HTML Pages etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'FactoryKey, -1 for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLineKey, -1 for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'ProductKey, -1 for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryID';
    SET @value      = N'FactoryID, empty string for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLineID, empty String for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'ProductID, empty string for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabID';
    SET @value      = N'Business ID for the Tab, used as business identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Orderindex';
    SET @value      = N'Order number for the Tab in the list of tabs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabName';
    SET @value      = N'Name of the Tab, this Name is used as label for the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabHint';
    SET @value      = N'Text, which is show as Tooltip on the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TabText';
    SET @value      = N'Text, displayed in the header area inside the tab as explination';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PresentationCODE';
    SET @value      = N'Codeword for Presentationtype of the content, e.g. PIVOT, DATAGRID,..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Datasource';
    SET @value      = N'Datasource for the Tab, depending on Presentation CODE - StoredProcedure Name, URL, ...';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutJSON';
    SET @value      = N'Layout definition saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ParameterJSON';
    SET @value      = N'Parameter definition, saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tConnectorQueries;

DROP TABLE IF EXISTS system.tConnectors;
/*
	Table to store connectors.

	This DELETE will delete also
			- the connector queries
	due to DELETE CASCADE
 
	Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectors

		INSERT INTO system.tConnectors VALUES
			 ('NAVISION','MSSQL','FIN','Connector for Navision MSSQL for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/ms-dynamics-nav/', 1)
			,('PROALPHA','ODBC','FIN','Connector for proAlpha ODBC for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/proalpha/',1)
			,('WODIS','ORACLE','FIN','Connector for Wodis Sigma ORACLE for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/wodis-sigma/',1)

		DELETE FROM system.tConnectors WHERE ConnectorKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors','COLUMN',default)
*/

CREATE TABLE system.tConnectors
    (
       ConnectorKey				INT				IDENTITY(1, 1) NOT NULL
      ,SourceSystemID			NVARCHAR(50)	NOT NULL
      ,SourceSystemType			NVARCHAR(50)	NOT NULL
      ,ModuleID					NVARCHAR(50)	NOT NULL
      ,ConnectorDescription		NVARCHAR(4000)	NOT NULL
      ,ConnectorDescriptionURL	NVARCHAR(255)	NOT NULL
      ,ConnectorVersion			INT				NOT NULL

      ,CONSTRAINT PK_tConnectors_ConnectorKey PRIMARY KEY CLUSTERED (ConnectorKey)
      ,CONSTRAINT AK_tConnectors_SourceSystemID_SourceSystemType_ModuleID
            UNIQUE (
                   SourceSystemID
                 , SourceSystemType
                 , ModuleID
                   )
    );

GO

ALTER TABLE system.tConnectors WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectors'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemID';
    SET @value      = N'Source System ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemType';
    SET @value      = N'Source System type for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ModuleID';
    SET @value      = N'Module ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescription';
    SET @value      = N'Description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescriptionURL';
    SET @value      = N'Link to Saxess website with detailed description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'ConnectorVersion';
    SET @value      = N'Version of the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tAPI_Log;
GO

/*
	Table for logging API Procedure Calls
	Primary Clusterd Key is ordered descending for fast opening the recent log
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tAPI_Log

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tAPI_Log',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tAPI_Log','COLUMN',default)
*/

CREATE TABLE system.tAPI_Log

(
	 LogKey				BIGINT			NOT NULL IDENTITY (1,1)
	,SQLSystemUser		NVARCHAR (255)  NOT NULL
	,UserName			NVARCHAR (255)  NOT NULL
	,TransactUsername	NVARCHAR (255)  NOT NULL
	,ProcedureName		NVARCHAR (255)  NOT NULL
	,ParameterString	NVARCHAR (MAX)  NOT NULL
	,EffectedRows		INT				NOT NULL
	,ReturnCode			INT				NOT NULL
	,TimestampCall		DATETIME		NOT NULL
	,TimestampReturn	DATETIME		NOT NULL
	,Comment			NVARCHAR (4000) NOT NULL
	,ProcessCode		NVARCHAR (255)  NOT NULL

	,CONSTRAINT PK_API_LogKey PRIMARY KEY CLUSTERED (LogKey DESC) 
);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tAPI_Log'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store user of OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SQLSystemUser';
    SET @value      = N'Name of the SQL system user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TransactUsername';
    SET @value      = N'Name of the transact user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProcedureName';
    SET @value      = N'Name of the procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ParameterString';
    SET @value      = N'String that holds the parameters.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EffectedRows';
    SET @value      = N'Number of rows affected.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReturnCode';
    SET @value      = N'The return code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimestampCall';
    SET @value      = N'The timestamp of the call.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampReturn';
    SET @value      = N'The timestamp of the return.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Comment';
    SET @value      = N'The comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcessCode';
    SET @value      = N'The process code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
DROP TABLE IF EXISTS system.tConnectorQueries;

/*
	Table to store connector queries.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectorQueries

		INSERT INTO system.tConnectorQueries VALUES
			 (1,'GetCompanies','SELECT 1 AS CompanyID, ''Company A'' AS CompanyName, 1 AS FinancialYearBegin',1,'',GETUTCDATE(),ORIGINAL_LOGIN()),
			 (1,'Accounts','SELECT 1 AS CompanyID, ''SKTO'' AS AccountGroupCODE, ''1234'' AS AccountID, ''BILANZ'' AS AccoutTypeCode, ''Account 1234'' AS AccountName, ''{}'' AS CustomValuesJSON',1,'',GETUTCDATE(),ORIGINAL_LOGIN())

		DELETE FROM system.tConnectorQueries WHERE ConnectorQueryKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries','COLUMN',default)
*/

CREATE TABLE system.tConnectorQueries
    (
       ConnectorQueryKey   INT            IDENTITY(1, 1) NOT NULL
      ,ConnectorKey        INT            NOT NULL
      ,QueryName           NVARCHAR(255)  NOT NULL
      ,QuerySQL            NVARCHAR(MAX)  NOT NULL
      ,QueryVersion        INT            NOT NULL
      ,QueryVersionComment NVARCHAR(4000) NOT NULL
      ,Timestamp           DATETIME       NOT NULL
      ,CreatedBy           NVARCHAR(255)  NOT NULL
       
	  ,CONSTRAINT PK_tConnectorQueries_ConnectorQueryKey PRIMARY KEY CLUSTERED (ConnectorQueryKey)
      ,CONSTRAINT FK_tConnectorQueries_ConnectorKey
            FOREIGN KEY (ConnectorKey)
            REFERENCES system.tConnectors (ConnectorKey)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectorQueries'	-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorQueryKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ConnectorKey';
    SET @value      = N'Foreign Key to system.tConnectors.ConnectorKey.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'QueryName';
    SET @value      = N'Name of the query which also determines the target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'QuerySQL';
    SET @value      = N'SQL query which is executed by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'QueryVersion';
    SET @value      = N'Version of the query. Default = 1. If the query is edited the version goes up in a separate entry. Only MAX(Version) is used by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'QueryVersionComment';
    SET @value      = N'If query is edited it should include a comment about what was changed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Timestamp';
    SET @value      = N'Timestamp when the query was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'CreatedBy';
    SET @value      = N'User who created or edited the query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.tDashboards;
GO

/*
	Table for storing Dashboard definitions
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tDashboards

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDashboards',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDashboards','COLUMN',default)
*/
 
CREATE TABLE system.tDashboards
    (
         DashboardKey			BIGINT				IDENTITY(1,1)
		,DashboardID			NVARCHAR(50)		NOT NULL
		,DashboardName			NVARCHAR(255)		NOT NULL
		,DashboardDescription	NVARCHAR(4000)		NOT NULL
        ,LayoutDefinition		VARBINARY(MAX)      NOT NULL

		,CONSTRAINT PK_tDashboards_DashboardKey PRIMARY KEY CLUSTERED (DashboardKey)
		,CONSTRAINT AK_tDashboards_DashboardID UNIQUE (DashboardID)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tDashboards'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;               -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Report definitions.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'DashboardKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DashboardID';
    SET @value      = N'DashboardID as Business Identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DashboardName';
    SET @value      = N'Name for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DashboardDescription';
    SET @value      = N'Description for the Dashboard';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutDefinition';
    SET @value      = N'Definition for the Dashboard (including images etc.) as varbinary content.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.tDataSources;
GO

/*
	Table to store connections to source systems.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tDataSources

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDataSources',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDataSources','COLUMN',default)
*/

CREATE TABLE system.tDataSources (
     DataSourceKey			BIGINT IDENTITY(1,1)		 NOT NULL
	,DataSourceID			NVARCHAR(50)				 NOT NULL
    ,DataSourceName			NVARCHAR(255)                NOT NULL
    ,SourceType				NVARCHAR(50)                 NOT NULL
    ,ConnectionValuesJSON	NVARCHAR(max)                NOT NULL
    ,CONSTRAINT PK_system_tDataSources_DataSourceKey PRIMARY KEY CLUSTERED (DataSourceKey)
	,CONSTRAINT AK_system_tDataSources_DataSourceID UNIQUE (DataSourceID)

);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tDataSources'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connections to source systems.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'DataSourceKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DataSourceID';
    SET @value      = N'Business ID of the Datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'DataSourceName';
    SET @value      = N'Name of the Datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SourceType';
    SET @value      = N'Sourcetype of the datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectionValuesJSON';
    SET @value      = N'The content of the ConnectionValues JSON file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tEmails;
GO
/*
	Table to store Emails.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tEmails

		INSERT INTO system.tEmails VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe',3,'ALWAYS')

	Open Questions:
		- DELETE CASCADE to Files
		- DELETE CASCADE to Pipeline ?

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails','COLUMN',default)
*/

CREATE TABLE system.tEmails 
	(
     EmailKey					INT			IDENTITY(1,1)		NOT NULL
	,EmailID					NVARCHAR(50)					NOT NULL
	,EmailSubject				NVARCHAR(2000)					NOT NULL
    ,EmailRecipients			NVARCHAR(4000)					NOT NULL --List, demlited by ;
	,EmailText					NVARCHAR(MAX)					NOT NULL
	,EmailAttachments			NVARCHAR(2000)					NOT NULL --List of FileIDs, delimited by ;							
	,EmailStatusCODE			NVARCHAR(50)					NOT NULL --PENDING, ERROR, SENT
	,SourceTypeCODE				NVARCHAR(50)					NOT NULL --PIPELINE, EVENT, MANUALLY
	,SourceID					NVARCHAR(50)					NOT NULL --PipelineID or FID-PLID-PID
	,CreationDate				DATETIME						NOT NULL
	,PostDate					DATETIME						NULL
	
    ,CONSTRAINT PK_tEmails_EmailKey PRIMARY KEY CLUSTERED (EmailKey)
	);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tEmails'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Emails as a queue.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'EmailKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailID';
    SET @value      = N'BusinessID for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailSubject';
    SET @value      = N'Subject for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EmailRecipients';
    SET @value      = N'List of Email addresses, separated by semikolon.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'EmailText';
    SET @value      = N'Body for the Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailAttachments';
    SET @value      = N'List of FileIDs, which are stored separate in planning.tFiles';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
		
	SET @level2name = N'EmailStatusCODE';
    SET @value      = N'CODE for the sendig status of an email, PENDING, ERROR, SENT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceTypeCODE';
    SET @value      = N'CODE for the Source, which created this Email PIPELINE, EVENT, MANUALLY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceID';
    SET @value      = N'The ID chain of the object which created this email, eg. DataSource / Pipeline - the DSID-PID or an FID-PLID-PID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CreationDate';
    SET @value      = N'Datetime,when the Email was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PostDate';
    SET @value      = N'Datetime,when the Email was posted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tIcons;
GO
/*
	Table to store Icons for GUI
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tIcons

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tIcons',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tIcons','COLUMN',default)
*/

CREATE TABLE system.tIcons (
	 IconKey		BIGINT IDENTITY (1,1)	NOT NULL
	,IconName		NVARCHAR (255)			NOT NULL
	,FileExtension	NVARCHAR (50)			NULL
	,FileBody		VARBINARY (MAX)			NOT NULL

	,CONSTRAINT PK_tIcons PRIMARY KEY CLUSTERED (IconKey)
);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tIcons'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store icons for use in application GUI.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'IconKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IconName';
    SET @value      = N'Name for the Icon as full name including Extension.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FileExtension';
    SET @value      = N'Extension of the image file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FileBody';
    SET @value      = N'Filebody as binary storage of the icon data.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

	GO

DROP TABLE IF EXISTS system.tLogicIndex;
GO
/*
	Table to store informations about activated Logicmodules.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tLogicIndex

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tLogicIndex',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tLogicIndex','COLUMN',default)
*/

CREATE TABLE system.tLogicIndex
	(
		 RowKey				INT				NOT NULL IDENTITY (1,1) 
		,ModuleID			NVARCHAR(50)	NOT NULL
		,ModuleVersion		NVARCHAR(50)	NOT NULL
		,ActivationDate		DATE			NOT NULL

		,CONSTRAINT PK_LogicIndex_PK PRIMARY KEY NONCLUSTERED (RowKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tLogicIndex'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store informations about activated Logicmodules. All activated modules are included. On deactivation modules will be deleted in this table.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'RowKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value		= N'ID of the module as string like FIN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleVersion';
    SET @value		= N'Version of the module as String like 2.0';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ActivationDate';
    SET @value		= N'Date when this module were activated. Only the last activation date is to see.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineStepLog;
 
DROP TABLE IF EXISTS system.tPipelineLog;
GO
/*
	Table for storing pipeline executions. Table materialises all details, to show full informations also in case of deleted / modified pipelines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineLog;

		DELETE FROM system.tPipelineLog WHERE LogKey = 2; --testing delete cascade

	TestData
		INSERT INTO system.tPipelineLog VALUES
			('1','Pipe','E1','Server','RUNNING','Hase','20221230 08:08:08.888','20221230 08:08:08.888')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineLog
    (
         LogKey					BIGINT					IDENTITY(1,1)
		,PipelineID				NVARCHAR(50)			NOT NULL
		,PipelineName			NVARCHAR(255)			NOT NULL
		,ExecutionID			NVARCHAR(50)			NOT NULL
		,ExecutionServerName	NVARCHAR(255)			NOT NULL
        ,StatusCODE				NVARCHAR(50)			NOT NULL
		,StartedBy				NVARCHAR(255)			NOT NULL
		,StartTime				DATETIME				NULL
		,EndTime				DATETIME				NULL

		,CONSTRAINT PK_tPipelineLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC)
		,CONSTRAINT AK_tPipelines_PID_EID  UNIQUE (PipelineID,ExecutionID)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineLog'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;               -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to Logs per Pipeline Execution. Step Logs are saved separatly in system.StepLog';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'This column just holds a sample text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineName';
    SET @value      = N'Name of the pipeline - materialized to be historic correct.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Generated Key on first call to Kep Pipeline and Steps together.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionServerName';
    SET @value      = N'URL of the Server, which executed this pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartedBy';
    SET @value      = N'Name of the user who started this pipeline, or the word scheduler if it was scheduler started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime showing start of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime showing end of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineSteps;
DROP TABLE IF EXISTS system.tPipelineSchedules;

DROP TABLE IF EXISTS system.tPipelines;
/*
	Table to store pipelines.
	This DELETE will delete also
			- the steps
			- the schedules
	due to DELETE CASCADE
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines

		INSERT INTO system.tPipelines VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe','#808080',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe','#808080',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe','#808080',3,'ALWAYS')

		DELETE FROM system.tPipelines WHERE PipelineID = 'P2';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines','COLUMN',default)
*/

CREATE TABLE system.tPipelines 
	(
     PipelineKey					INT			IDENTITY(1,1)		NOT NULL
	,PipelineID						NVARCHAR(50)					NOT NULL
    ,PipelineName					NVARCHAR(255)					NOT NULL   
	,PipelineDescription			NVARCHAR(4000)					NOT NULL
	,PipelineDescriptionIconColor	NVARCHAR(255)					NOT NULL 
	,OrderOfDisplay					INT								NOT NULL
	,EmailNotificationCODE			NVARCHAR(50)					NOT NULL --NEVER, ALWAYS, ONERROR
	
    ,CONSTRAINT PK_tPipelines_PipelineKey PRIMARY KEY CLUSTERED (PipelineKey)
	,CONSTRAINT AK_tPipelines_PipelineID  UNIQUE (PipelineID)
	);

GO

ALTER TABLE system.tPipelines WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelines'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineID';
    SET @value      = N'Business ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineName';
    SET @value      = N'Name of the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'PipelineDescription';
    SET @value      = N'Description for the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineDescriptionIconColor';
    SET @value      = N'Color of the Pipeline Icon, written as Hex Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfDisplay';
    SET @value      = N'Stores at which position the process pipeline should be displayed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailNotificationCODE';
    SET @value      = N'CODE word to aktivate Email notifications - NEVER, ALWAYS, ONERROR.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tPipelineSchedules;
GO
/*
	Table to store defined schedules for pipelines.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineSchedules

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules','COLUMN',default)
*/

CREATE TABLE system.tPipelineSchedules
	(
	 ScheduleKey			INT IDENTITY(1,1)	NOT NULL
	,PipelineKey			INT					NOT NULL
	,ScheduleID				NVARCHAR(50)		NOT NULL
	,ScheduleName			NVARCHAR(255)		NOT NULL
	,ScheduleDescription	NVARCHAR(4000)		NOT NULL 
	,ScheduleDetailsJSON	NVARCHAR(MAX)		NOT NULL
	,Active					BIT					NOT NULL
	,CONSTRAINT PK_tPipelineSchedules PRIMARY KEY CLUSTERED (ScheduleKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSchedules_PipelineKeyScheduleID UNIQUE (PipelineKey,ScheduleID)
	,CONSTRAINT FK_tPipelineSchedules_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSchedules' -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store defined schedules for pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ScheduleKey';
    SET @value      = N'Stores the generated primary key of the table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Stores the PipelineKey that is executed with this scheduler job.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleID';
    SET @value      = N'ID for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleName';
    SET @value      = N'Name for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleDescription';
    SET @value      = N'Description of the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ScheduleDetailsJSON';
    SET @value      = N'Stores the rhythms which determines when and how often a pipeline is executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Stores if the scheduled job is currently active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tPipelineStepLog;
GO
/*
	Table for storing Log events per Step
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineStepLog

	Dependend Tables
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps
		SELECT * FROM system.tPipelineLog

	TestData
		INSERT INTO system.tPipelineStepLog VALUES
			 ('1','1','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')
			,('1','2','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineStepLog
    (
         LogKey			BIGINT					IDENTITY(1,1)
		,PipelineID		NVARCHAR(50)			NOT NULL
		,StepID			NVARCHAR(50)			NOT NULL
		,ExecutionID	NVARCHAR(50)			NOT NULL
		,StepName		NVARCHAR(255)			NOT NULL
		,StartTime		DATETIME				NULL
		,EndTime		DATETIME				NULL
		,LogMessage		NVARCHAR(MAX)			NOT NULL
		,StatusCODE		NVARCHAR(50)			NOT NULL
		,HandleID		NVARCHAR(50)			NOT NULL

		,CONSTRAINT PK_tPipelineStepLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC) 
		,CONSTRAINT AK_tPipelineStepLog_StepIDExecutionID UNIQUE (StepID,ExecutionID)
		-- Foreign Key relation to PipelineLog to delete Step Logs, when deleting the PipelineLog ExecutionID
		,CONSTRAINT FK_tPipelineLog_LogKey FOREIGN KEY (PipelineID,ExecutionID)
			REFERENCES system.tPipelineLog (PipelineID,ExecutionID)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineStepLog'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;                -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store logs per exectuted step'
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'ID of the pipeline this step belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepID';
    SET @value      = N'ID of the step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Relation to the pipeline log.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepName';
    SET @value      = N'Name of the Step to keep history.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime the step was started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime the step was finished.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LogMessage';
    SET @value      = N'Messages from logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, WAITING (Pipeline was started and step queued), STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'HandleID';
    SET @value      = N'ID for Penatho Job handling.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
	
DROP TABLE IF EXISTS system.tPipelineSteps;
GO
/*
	Table to store pipeline steps

	They have a foreign key constraint to the the system.tPipelinesTable and are deleted automaticly, if the Pipeline is deleted
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps

		INSERT INTO system.tPipelineSteps VALUES
			 (1,'1','Step 1','Hallo Step',1,'1','{}','{}',30,1,'FIN',0)
			,(1,'2','Step 2','Hallo Step',2,'1','{}','{}',30,1,'FIN',0)
			,(1,'3','Step 3','Hallo Step',3,'1','{}','{}',30,1,'FIN',0)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps','COLUMN',default)
*/

CREATE TABLE system.tPipelineSteps (
     StepKey					INT        IDENTITY(1,1)    NOT NULL		
	,PipelineKey				INT                         NOT NULL
	,StepID						NVARCHAR(50)				NOT NULL
    ,StepName					NVARCHAR(255)               NOT NULL		
	,StepDescription			NVARCHAR(4000)				NOT NULL
	,OrderOfExecution			INT                         NOT NULL
	,StepDataSourceID			NVARCHAR(50)				NOT NULL
	,StepCompanyIDsJSON			NVARCHAR(MAX)  				NOT NULL
	,StepDetailsJSON			NVARCHAR(MAX)  				NOT NULL		
	,TimeExecutionLimit			INT							NOT NULL		
	,Active						BIT							NOT NULL
	,ModuleID					NVARCHAR(255)				NOT NULL
	,StartNextStepOnError		BIT							NOT NULL
	-- Primary Key
    ,CONSTRAINT PK_tPipelineSteps_StepKey PRIMARY KEY CLUSTERED (StepKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSteps_PipelineKeyStepID  UNIQUE (PipelineKey,StepID)
	,CONSTRAINT FK_tPipelines_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSteps'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process steps.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'StepKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Foreign key to tPipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepName';
    SET @value      = N'Name of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepDescription';
    SET @value      = N'Description of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfExecution';
    SET @value      = N'Stores at which position of the process pipeline the step should be executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepDataSourceID';
    SET @value      = N'ID of the Datasource which is used from this Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepCompanyIDsJSON';
    SET @value      = N'JSON Array with the IDs of the companies activated for this step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepDetailsJSON';
    SET @value      = N'The details of this step, content depending from the step type CODE.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeExecutionLimit';
    SET @value      = N'The time limit for the execution in minutes, step is skiped after this limit, use 0 for no limit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Flag if the step is active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value      = N'ID of the Module, if the Step has the property module, else empty string';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tProcessLogs;
GO
/*
	Table to store log entries of different application processes.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tProcessLogs;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tProcessLogs',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tProcessLogs','COLUMN',default)
*/

CREATE TABLE system.tProcessLogs
	(
		 RowKey				INT IDENTITY(1,1)	NOT NULL
		,ModuleCODE			NVARCHAR(50)		NOT NULL
		,ProcessName		NVARCHAR(255)		NOT NULL
		,ProcessID			NVARCHAR(50)		NULL
		,SubProcessName		NVARCHAR(255)		NULL
		,SubProcessID		NVARCHAR(50)		NULL
		,SubSubProcessName	NVARCHAR(255)		NULL
		,EventCODE			NVARCHAR(20)		NOT NULL
		,Timestamp			DATETIME			NOT NULL
		,ReturnCODE			INT					NULL
		,EventMessage		NVARCHAR(MAX)		NULL

		,CONSTRAINT PK_tProcessLogs PRIMARY KEY CLUSTERED  (RowKey DESC)
 );

 -- OCTService has direkt SELECT rights due to sending this log table to monitoring service.
 GRANT SELECT ON system.tProcessLogs TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tProcessLogs'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store log entries of all application processes which use monitoring services.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ModuleCODE';
    SET @value      = N'Specification of the module that created this log entry, e.g. Server, Integration, Planning, Visualization.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProcessName';
    SET @value      = N'Name of the process that is logged here.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcessID';
    SET @value      = N'ID of the process that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SubProcessName';
    SET @value      = N'Name of the subprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SubProcessID';
    SET @value      = N'ID of the subprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
		 
    SET @level2name = N'SubSubProcessName';
    SET @value      = N'Name of the subsubprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'EventCODE';
    SET @value      = N'Description of the event that occurred in the process, can be: Start, End, Info.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'Date and time when this event occurred.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReturnCODE';
    SET @value      = N'Return code of the process that has been executed, use HTTP return codes (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EventMessage';
    SET @value      = N'Details of this event, e.g. a log message or an exception that occurred (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tReports;
GO
/*
	Table for storing Reportdefinitions
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tReports

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tReports',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tReports','COLUMN',default)
*/
 
CREATE TABLE system.tReports
    (
         ReportKey			BIGINT				IDENTITY(1,1)
		,ReportID			NVARCHAR(50)		NOT NULL
		,ReportName			NVARCHAR(255)		NOT NULL
		,ReportDescription	NVARCHAR(4000)		NOT NULL
        ,LayoutDefinition	VARBINARY(MAX)      NOT NULL

		,CONSTRAINT PK_tReports_ReportKey PRIMARY KEY CLUSTERED (ReportKey)
		,CONSTRAINT AK_tReports_ReportID UNIQUE (ReportID)
	);
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tReports'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;                -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Report definitions.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ReportKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReportID';
    SET @value      = N'ReportID as Business Identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReportName';
    SET @value      = N'Name for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReportDescription';
    SET @value      = N'Description for the Report';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutDefinition';
    SET @value      = N'Definition for the report (including images etc.) as varbinary content.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.trRights;
GO

/*
	Table to store defined Rights per User.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trRights;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trRights','COLUMN',default)
*/
 
CREATE TABLE system.trRights
	(
		 RightKey				INT				NOT NULL IDENTITY (1,1) 
		,UserKey				INT				NOT NULL
		,UserName				NVARCHAR (255)	NOT NULL
		,FactoryID				NVARCHAR (255)	NOT NULL
		,ProductLineID			NVARCHAR (255)	NOT NULL
		,ProductID				NVARCHAR (255)	NOT NULL
		,[Right]				NVARCHAR (255)	NOT NULL
		,ReadCommentMandatory	INTEGER			NOT NULL
		,WriteCommentMandatory	INTEGER			NOT NULL

		,CONSTRAINT PK_rRights_PK PRIMARY KEY CLUSTERED (RightKey)
	);

	CREATE NONCLUSTERED INDEX IX_rRights_UserKey ON system.trRights (UserKey ASC);

GO
-- GRANT
-- NO GRANT - table in system schema can be accessed from FactoryService only over Stored procedures
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trRights'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store defined Rights per User - only positiv given rights are stored on highest level.
					Resolving Rights to effectiv Rights for all levels is done to table system.trUserRights';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'RightKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserKey';
    SET @value		= N'Technical Key for join with system.trUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value		= N'Business Key for join with system.trUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value		= N'FactoryID to which the right belongs. Empty FactoryID stands for clusterrights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value		= N'ProduclineID to which the right belongs. Empty ProductLineID stands for FactoryRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value		= N'not in use in this version';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Right';
    SET @value		= N'Capitalized Word to define the right. Only "Read" or "Write".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReadCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every read action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'WriteCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every write action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.trUser;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trUser

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser','COLUMN',default)
*/

CREATE TABLE system.trUser
	(
		 UserKey					INT				NOT NULL IDENTITY (1,1) 
		,UserName					NVARCHAR (255)	NOT NULL
		,PersonSurname				NVARCHAR (255)	NOT NULL
		,PersonFirstName			NVARCHAR (255)	NOT NULL
		,Email						NVARCHAR (255)	NOT NULL
		,LDAPIP						NVARCHAR (255)	NOT NULL
		,Status						NVARCHAR (255)	NOT NULL
		,DataFlowRightsCODE			NVARCHAR (255)	NOT NULL -- DESIGNER, EXECUTOR, NONE
		,VisualizationRightsCODE	NVARCHAR (255)	NOT NULL -- DESIGNER, VIEWER, NONE
		,IsAdministratorFlag		BIT				NOT NULL

		,CONSTRAINT PK_rUser_PK PRIMARY KEY CLUSTERED (UserKey)
	);

GO

-- GRANT
-- temporary GRANTs for procedures / functions in dbo schema (e.g. dbo.sx_pf_GET_ClusterAccessRight)
GRANT SELECT ON system.trUser TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUser'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store user of OCT.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'UserKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Username';
    SET @value		= N'Username for login - usually an User with Domain e.g. SX\GTA or an email Address.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonSurname';
    SET @value		= N'Surname of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonFirstName';
    SET @value		= N'FirstName of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Email';
    SET @value		= N'Email Address of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'LDAPIP';
    SET @value		= N'LDAPIP.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value		= N'Status of the user account. Capitalized word - may be "Active" or "Inactive".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DataFlowRightsCODE';
    SET @value		= N'CODE Word for the DataFlow Rights of the user (DESIGNER, EXECUTOR, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'VisualizationRightsCODE';
    SET @value		= N'CODE Word for the Visualization Rights of the user (DESIGNER, VIEWER, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsAdministratorFlag';
    SET @value		= N'Value 0/1 to flag if this user is an administorator of this OCT Database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.trUserRights;
GO
/*
	Table to store effective positiv Rights per User for all level.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trUserRights

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUserRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUserRights','COLUMN',default)
*/

CREATE TABLE system.trUserRights

	(
		 [Row]					BIGINT			NOT NULL IDENTITY (1,1) 
		,UserKey				INT				NOT NULL
		,UserName				NVARCHAR (255)	NOT NULL
		,FactoryID				NVARCHAR (255)	NOT NULL
		,ProductLineID			NVARCHAR (255)	NOT NULL
		,[Right]				NVARCHAR (255)	NOT NULL
		,ReadCommentMandatory	INT				NOT NULL
		,WriteCommentMandatory	INT				NOT NULL

		,CONSTRAINT PK_rUserRights_PK PRIMARY KEY CLUSTERED ([Row])
	);

GO

-- explicit GRANT for FactoryService, as Table must be joined in many Procedures
GRANT SELECT ON system.trUserRights TO pf_PlanningFactoryService;
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUserRights'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store effective positiv Rights per User for all level. Table will be automaticly managed by calculation proceduer. Use in most outputs, which filter on Userrights.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'Row';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserKey';
    SET @value		= N'Technical Key for User.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value		= N'Business Key for User';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value		= N'FactoryID to which the right belongs. Empty FactoryID stands for clusterrights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value		= N'ProduclineID to which the right belongs. Empty ProductLineID stands for FactoryRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Right';
    SET @value		= N'Capitalized Word to define the right. Only "Read" or "Write".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReadCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every read action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'WriteCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every write action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tSettings;
GO
/*
	Table to store global database properties.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tSettings

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings','COLUMN',default)
*/

CREATE TABLE system.tSettings (
     RowKey						INT  IDENTITY(1,1)  NOT NULL
    ,SettingID					NVARCHAR(50)		NOT NULL 
	,SettingDescription			NVARCHAR(4000)		NOT NULL
	,DataTypeCODE				NVARCHAR(255)		NOT NULL
	,ValueText					NVARCHAR(4000)		NOT NULL CONSTRAINT Default_ValueText		DEFAULT ''
	,ValueDateTime				DATETIME			NOT NULL CONSTRAINT Default_ValueDateTime	DEFAULT '1900-01-01 00:00:00.000'
	,ValueInt					BIGINT				NOT NULL CONSTRAINT Default_ValueInt		DEFAULT 0
	,ValueIntScale				INT					NOT NULL CONSTRAINT Default_ValueIntScale	DEFAULT 1
	,ValueJSON					NVARCHAR(MAX)		NOT NULL CONSTRAINT Default_ValueJSON		DEFAULT ''
	,Category					NVARCHAR(255)		NOT NULL
	,ReadOnlySystemPropertyBool  INT				NOT NULL

   ,CONSTRAINT PK_tSettings PRIMARY KEY CLUSTERED (RowKey)
);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tSettings'      -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store global database properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SettingID';
    SET @value      = N'SettingID as business identifier for the entry.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SettingDescription';
    SET @value      = N'Description or comment of this setting for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DataTypeCODE';
    SET @value      = N'DataTypeCODE of setting that shows which value column is to used and how it should be used. Can be TEXT, DATETIME, BOOLEAN, NUMBER, JSON. See documentation for DataTypeCodes.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'Text value if value TEXT is set for column DataType, optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueDateTime';
    SET @value      = N'DateTime value if value DATETIME is set for column DataType, optional to submit default Value 1900-01-01 00:00:00.000 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'Integer value if value BOOLEAN or NUMBER is set for Column DataType. In Case of Boolean: 0,1,-1 for false,true and unknown. NUMBER - an Integer Number which must be scaled with column ValueIntScale. Optional to submit default Value 0 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueIntScale';
    SET @value      = N'Integer value of 1 for BOOLEAN value. Value of 1, 10, 100, 10000 for scaling type NUMBER to decimal value -> divide column ValueInt by column ValueIntScale. Optional to submit default Value 1 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueJSON';
    SET @value      = N'JSON value if DateTypeCODE is set to JSON. Optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Category';
    SET @value      = N'Category to group items.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadOnlySystemPropertyBool';
    SET @value      = N'Flag to signalize if property is changeable by user or is a Ready Only System Value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
