
-- Script for creating a planning factory datamodel in an existing database
-- Gerd Tautenhahn for saxess-software gmbh 04/2019
-- Version PlanningFactory 4.0
-- Coding UTF-8 without BOM 

SET QUOTED_IDENTIFIER ON
GO
/*
    Product: OCT 5.0
    Object: agent
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('agent') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA agent AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'agent'
END


/*
    Product: OCT 5.0
    Object: calc
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('calc') IS NOT NULL
BEGIN
      -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'calc'
END



/*
    Product: OCT 5.0
    Object: control
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('control') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'control'
END



/*
    Product: OCT 5.0
    Object: global
    Type: Schema
    Description: Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('global') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA global AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.',
        @level0type = N'Schema',
        @level0name = N'global'
END


/*
    Product: OCT 5.0
    Object: import
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('import') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'import'
END


/*
    Product: OCT 5.0
    Object: integration
    Type: Schema
    Description: Schema to store integration data. Temporary and no enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('integration') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA integration AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store integration data. Temporary and no enduser access.',
        @level0type = N'Schema',
        @level0name = N'integration'
END



/*
    Product: OCT 5.0
    Object: load
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('load') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'load'
END



/*
    Product: OCT 5.0
    Object: param
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('param') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'param'
END



/*
    Product: OCT 5.0
    Object: result
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('result') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'result'
END


/*
    Product: OCT 5.0
    Object: staging
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('staging') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'staging'
END


/*
    Product: OCT 5.0
    Object: sxdf
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('sxdf') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA sxdf AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'sxdf'
END


IF schema_id('sxdf') IS NOT NULL
BEGIN

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'sxdf'
END



/*
    Product: OCT 5.0
    Object: system
    Type: Schema
    Description: Schema to store system data and Metadata. No enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('system') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA system AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store system data and Metadata. No enduser access.',
        @level0type = N'Schema',
        @level0name = N'system'
END



/*
    Product: OCT 5.0
    Object: global.tCodes
    Type: Table
    Description: Table to store informations about CODEs used for the dimensions. The translated description is stored is stored in rows
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS global.tCodes;

CREATE TABLE global.tCodes (
    RowKey              BIGINT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    CODE                NVARCHAR(50)                    NOT NULL,     -- Code written in uppercase letters
    Language            NVARCHAR(10)                    NOT NULL,     -- Language identifier for the code description. Identifiers are stored in ISO 639-1 standard (e.g. de, en, fr, ...).
    CODEDescription     NVARCHAR(255)                   NOT NULL,     -- Description of CODE in specific language
    CONSTRAINT PK_global_tCodes PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store informations about CODEs used for the dimensions. The translated description is stored is stored in rows',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Code written in uppercase letters',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes',
    @level2type = N'Column',
    @level2name = N'CODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Language identifier for the code description. Identifiers are stored in ISO 639-1 standard (e.g. de, en, fr, ...).',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes',
    @level2type = N'Column',
    @level2name = N'Language'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Description of CODE in specific language',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes',
    @level2type = N'Column',
    @level2name = N'CODEDescription'
GO

-- Default Content Flag
EXEC sp_addextendedproperty
    @name = N'IsDefaultContent',
    @value = N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCodes'
GO

/*
    Product: OCT 5.0
    Object: system.tCompanies
    Type: Table
    Description: Table to store information about companies.
    Author: saxess-software GmbH
    Script Date: 23.07.2019 14:56
*/

DROP TABLE IF EXISTS global.tCompanies;

CREATE TABLE global.tCompanies (
    RowKey                 INT        IDENTITY(1,1)                          NOT NULL,
    CompanyKey             AS (concat([DataSourceKey],N'|',[CompanyID]))             ,
    DataSourceKey          INT                                               NOT NULL,
    CompanyID              NVARCHAR(50)                                      NOT NULL,
    ActiveFlag             INT                                               NOT NULL,
    CompanyName            NVARCHAR(255)                                     NOT NULL,
    OwnerCODE              NVARCHAR(50)                                      NOT NULL,
    FinancialYearBegin     INT                                               NOT NULL,
    ChartsOfAccountsID     NVARCHAR(50)                                      NOT NULL,
    AccountLength          INT                                               NOT NULL,
    Currency               NVARCHAR(50)                                      NOT NULL,
    MappingKey             INT                                               NULL,
    UpdatedFlag            INT                                               NOT NULL,
    CustomValuesJSON       NVARCHAR(4000)                                    NULL,
    CONSTRAINT PK_global_tCompanies PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store information about companies.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tCompanies'
GO

/*
    Product: OCT 5.0
    Object: global.tDataSources
    Type: Table
    Description: Table to store connections to source systems.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS global.tDataSources;

CREATE TABLE global.tDataSources (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,
    SourceType               NVARCHAR(50)                 NOT NULL,
    ETLPackage               NVARCHAR(50)                 NOT NULL,
    ConnectionValuesJSON     NVARCHAR(4000)               NOT NULL,
    CONSTRAINT PK_global_tDataSources PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store connections to source systems.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tDataSources'
GO

/*
    Product: OCT 5.0
    Object: system.tFinAccounts
    Type: Table
    Description: Table to store informations about acccounts. Table contains merged data from integration and planning.
    Author: saxess-software GmbH
    Script Date: 23.07.2019 14:56
*/

DROP TABLE IF EXISTS global.tFinAccounts;

CREATE TABLE global.tFinAccounts (
    RowKey               BIGINT        IDENTITY(1,1)                                                                NOT NULL,     -- Technical primary Key for row identity.
    AccountKey           AS (concat([DataSourceKey],N'|',[CompanyID],N'|',[AccountGroupCODE],N'|',[AccountID]))             ,     -- Primary Key based on CompanyKey (DataSourceKey + CompanyID) , AccountID, AccountGroupCODE for join operations.
    DataSourceKey        INT                                                                                        NOT NULL,
    CompanyID            NVARCHAR(50)                                                                               NOT NULL,     -- OCT identifier for the company. Part of AccountKey.
    AccountGroupCODE     NVARCHAR(50)                                                                               NOT NULL,     -- Classification for a group of accounts, e.g. SKTO=Sachkonto/GLAccount, KOART=Kostenart/CostType, PERSKTO = Personenkonto/PersonalAccount. Part of AccountKey - changes must be done in fact tables too !
    AccountID            NVARCHAR(50)                                                                               NOT NULL,     -- Identifier for the account within one type. Part of AccountKey.
    AccountTypeCODE      NVARCHAR(50)                                                                               NOT NULL,     -- Classification for an account within a group of Accounts, e.g. GUV, BILANZ, DEBITOR, KREDITOR, STAT
    AccountOwnerCODE     NVARCHAR(50)                                                                               NOT NULL,     -- Owner of the Account. INTEGRATION=Integration, PLANNING=Planning or ORPHAN=Orphan - Integration will dominate Planning. Orphans arise if integration accounts are deleted.
    AccountName          NVARCHAR(255)                                                                              NOT NULL,     -- Name of the Account.
    MappingKey           INT                                                                                        NULL,
    UpdatedFlag          INT                                                                                        NOT NULL,
    CustomValuesJSON     NVARCHAR(4000)                                                                             NOT NULL,     -- Custom field to store any customer individual informations.
    CONSTRAINT PK_global_tFinAccounts PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store informations about acccounts. Table contains merged data from integration and planning.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Primary Key based on CompanyKey (DataSourceKey + CompanyID) , AccountID, AccountGroupCODE for join operations.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'OCT identifier for the company. Part of AccountKey.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'CompanyID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of accounts, e.g. SKTO=Sachkonto/GLAccount, KOART=Kostenart/CostType, PERSKTO = Personenkonto/PersonalAccount. Part of AccountKey - changes must be done in fact tables too !',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the account within one type. Part of AccountKey.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for an account within a group of Accounts, e.g. GUV, BILANZ, DEBITOR, KREDITOR, STAT',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountTypeCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Owner of the Account. INTEGRATION=Integration, PLANNING=Planning or ORPHAN=Orphan - Integration will dominate Planning. Orphans arise if integration accounts are deleted.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountOwnerCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the Account.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountName'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Custom field to store any customer individual informations.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'CustomValuesJSON'
GO

/*
    Product: OCT 5.0
    Object: global.tFinChartsOfAccounts
    Type: Table
    Description: Table to store account systems for automatically allocation of accounts for agent.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS global.tFinChartsOfAccounts;

CREATE TABLE global.tFinChartsOfAccounts (
    RowKey               INT        IDENTITY(1,1)     NOT NULL,
    ChartsOfAccounts     NVARCHAR(50)                 NOT NULL,
    AccountLength        INT                          NOT NULL,
    AccountFrom          NVARCHAR(50)                 NOT NULL,
    AccountTo            NVARCHAR(50)                 NOT NULL,
    AccountTypeCODE      NVARCHAR(50)                 NOT NULL,
    CONSTRAINT PK_global_tFinChartsOfAccounts PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store account systems for automatically allocation of accounts for agent.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinChartsOfAccounts'
GO

-- Default Content Flag
EXEC sp_addextendedproperty
    @name = N'IsDefaultContent',
    @value = N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinChartsOfAccounts'
GO

/*
    Product: OCT 5.0
    Object: system.tFinCostObjects
    Type: Table
    Description: Table to store informations about cost objects (cost centers, cost units, ...). Table contains merged data from integration and planning.
    Author: saxess-software GmbH
    Script Date: 23.07.2019 14:56
*/

DROP TABLE IF EXISTS global.tFinCostObjects;

CREATE TABLE global.tFinCostObjects (
    RowKey                  BIGINT        IDENTITY(1,1)                                                                      NOT NULL,     -- Technical primary Key for row identity.
    CostObjectKey           AS (concat([DataSourceKey],N'|',[CompanyID],N'|',[CostObjectID],N'|',[CostObjectGroupCODE]))             ,     -- Primary Key based on CompanyID, CostObjectID, CostObjectGroupCODE for join operations.
    DataSourceKey           INT                                                                                              NOT NULL,
    CompanyID               NVARCHAR(50)                                                                                     NOT NULL,     -- OCT identifier for the company. Part of CostObjectKey.
    CostObjectGroupCODE     NVARCHAR(50)                                                                                     NOT NULL,     -- Classification for a group of CostObjects, e.g. KST=Kostenstelle/cost center, KTR=Kostentraeger/cost unit, PC = ProfitCenter, OR=Auftrag/order. Part of CostObjectKey - changes must be done in fact tables too !
    CostObjectID            NVARCHAR(50)                                                                                     NOT NULL,     -- Identifier for the CostObject within one type. Part of CostObjectKey.
    CostObjectOwnerCODE     NVARCHAR(50)                                                                                     NOT NULL,     -- Owner of the CostObject. INTEGRATION=Integration, PLANNING=Planning or ORPHAN=Orphan - Integration will dominate Planning. Orphans arise if integration CostObjects are deleted.
    CostObjectName          NVARCHAR(255)                                                                                    NOT NULL,     -- Name of the CostObject.
    MappingKey              INT                                                                                              NULL,
    UpdatedFlag             INT                                                                                              NOT NULL,
    CustomValuesJSON        NVARCHAR(4000)                                                                                   NOT NULL,     -- Custom field to store any customer individual informations.
    CONSTRAINT PK_global_tFinCostObjects PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store informations about cost objects (cost centers, cost units, ...). Table contains merged data from integration and planning.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Primary Key based on CompanyID, CostObjectID, CostObjectGroupCODE for join operations.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'OCT identifier for the company. Part of CostObjectKey.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CompanyID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of CostObjects, e.g. KST=Kostenstelle/cost center, KTR=Kostentraeger/cost unit, PC = ProfitCenter, OR=Auftrag/order. Part of CostObjectKey - changes must be done in fact tables too !',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the CostObject within one type. Part of CostObjectKey.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Owner of the CostObject. INTEGRATION=Integration, PLANNING=Planning or ORPHAN=Orphan - Integration will dominate Planning. Orphans arise if integration CostObjects are deleted.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectOwnerCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the CostObject.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectName'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Custom field to store any customer individual informations.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CustomValuesJSON'
GO

/*
    Product: OCT 5.0
    Object: global.tPeriods
    Type: Table
    Description: Table to store information about time periods.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS global.tPeriods;

CREATE TABLE global.tPeriods (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,
    FinancialYear            INT                          NOT NULL,
    Period                   INT                          NOT NULL,
    FinancialYearBegin       INT                          NOT NULL,
    FinancialYearName        NVARCHAR(50)                 NOT NULL,
    CalendarYear             INT                          NOT NULL,
    CalendarMonth            INT                          NOT NULL,
    CalendarQuarterCODE      NVARCHAR(50)                 NOT NULL,
    CalendarHalfYearCODE     NVARCHAR(50)                 NOT NULL,
    CONSTRAINT PK_global_tPeriods PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store information about time periods.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tPeriods'
GO

-- Default Content Flag
EXEC sp_addextendedproperty
    @name = N'IsDefaultContent',
    @value = N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tPeriods'
GO

/*
    Product: OCT 5.0
    Object: global.tFinAccountingJournal
    Type: Table
    Description: Table to store facts of solution Finance.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS integration.tFinAccountingJournal;

CREATE TABLE integration.tFinAccountingJournal (
    RowKey                     BIGINT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    DataSourceKey              INT                             NOT NULL,     -- DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.
    CompanyID                  NVARCHAR(50)                    NOT NULL,     -- Company ID which is used in the source database.
    AccountingAreaCODE         NVARCHAR(50)                    NOT NULL,     -- Classification for a group of postings. CODEs are defined in table global.tAttributesCODE. (e.g. FIBU, KORE, PCA, ...)
    AccountID                  NVARCHAR(255)                   NOT NULL,     -- Identifier for the account within one group.
    AccountGroupCODE           NVARCHAR(50)                    NOT NULL,     -- Classification for a group of accounts. CODEs are defined in table global.tAttributesCODE
    OffsetAccountID            NVARCHAR(255)                   NULL,         -- Identifier for the offset account within one group.
    OffsetAccountGroupCODE     NVARCHAR(50)                    NULL,         -- Classification for a group of offset accounts. CODEs are defined in table global.tAttributesCODE
    CostObject1ID              NVARCHAR(255)                   NULL,         -- Identifier for the fist cost object within one group. This will be usually cost centers
    CostObject1GroupCODE       NVARCHAR(50)                    NULL,         -- Classification for a group of the first cost object. CODEs are defined in table global.tAttributesCODE
    CostObject2ID              NVARCHAR(255)                   NULL,         -- Identifier for the second cost object within one group. This will be usually cost units
    CostObject2GroupCODE       NVARCHAR(50)                    NULL,         -- Classification for a group of the second cost object. CODEs are defined in table global.tAttributesCODE
    FinancialYear              SMALLINT                        NOT NULL,     -- Fiscal year in which posting is made.
    FinancialPeriod            TINYINT                         NOT NULL,     -- Financial period in which posting is made.
    OpeningBalance             BIGINT                          NOT NULL,     -- Opening balance value. Decimal places are eliminated through multiplication with 100.
    Debit                      BIGINT                          NOT NULL,     -- Debit value. Decimal places are eliminated through multiplication with 100.
    Credit                     BIGINT                          NOT NULL,     -- Credit value. Decimal places are eliminated through multiplication with 100.
    PostingDate                DATE                            NULL,         -- Posting date of the posting.
    PostingText                NVARCHAR(255)                   NULL,         -- Posting text of the posting.
    DocumentDate               DATE                            NULL,         -- Document date of the posting.
    DocumentNumber             NVARCHAR(255)                   NULL,         -- Document number of the posting.
    DocumentLine               INT                             NULL,         -- Line number of document.
    ICPartnerID                NVARCHAR(50)                    NULL,         -- Identifier for the intercompany partner. Should be local other LocalCompanyID from source system.
    SourceID                   BIGINT                          NULL,         -- Identifier of dataset in source system (e.g. row number)
    DeltaValue                 BIGINT                          NULL,         -- Delta value to enable delta load. Should be the timestamp when dataset was inserted in source system or sequential number from source system.
    CustomValuesJSON           NVARCHAR(4000)                  NULL,         -- Custom field to store any customer individual informations.
);

-- Indexes
CREATE CLUSTERED COLUMNSTORE INDEX IX_integration_tFinAccountingJournal_1 ON integration.tFinAccountingJournal;

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store facts of solution Finance.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'DataSourceKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Company ID which is used in the source database.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CompanyID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of postings. CODEs are defined in table global.tAttributesCODE. (e.g. FIBU, KORE, PCA, ...)',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'AccountingAreaCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the account within one group.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'AccountID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of accounts. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'AccountGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the offset account within one group.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'OffsetAccountID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of offset accounts. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'OffsetAccountGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the fist cost object within one group. This will be usually cost centers',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CostObject1ID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of the first cost object. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CostObject1GroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the second cost object within one group. This will be usually cost units',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CostObject2ID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of the second cost object. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CostObject2GroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Fiscal year in which posting is made.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'FinancialYear'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Financial period in which posting is made.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'FinancialPeriod'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Opening balance value. Decimal places are eliminated through multiplication with 100.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'OpeningBalance'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Debit value. Decimal places are eliminated through multiplication with 100.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'Debit'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Credit value. Decimal places are eliminated through multiplication with 100.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'Credit'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Posting date of the posting.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'PostingDate'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Posting text of the posting.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'PostingText'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Document date of the posting.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'DocumentDate'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Document number of the posting.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'DocumentNumber'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Line number of document.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'DocumentLine'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the intercompany partner. Should be local other LocalCompanyID from source system.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'ICPartnerID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier of dataset in source system (e.g. row number)',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'SourceID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Delta value to enable delta load. Should be the timestamp when dataset was inserted in source system or sequential number from source system.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'DeltaValue'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Custom field to store any customer individual informations.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccountingJournal',
    @level2type = N'Column',
    @level2name = N'CustomValuesJSON'
GO

/*
    Product: OCT 5.0
    Object: global.tFinAccounts
    Type: Table
    Description: Table to store informations about acccounts. Table contains data from source system.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS integration.tFinAccounts;

CREATE TABLE integration.tFinAccounts (
    RowKey               BIGINT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    DataSourceKey        INT                             NOT NULL,     -- DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.
    CompanyID            NVARCHAR(50)                    NOT NULL,     -- Company ID which is used in the source database.
    AccountGroupCODE     NVARCHAR(50)                    NOT NULL,     -- Classification for a group of accounts. CODEs are defined in table global.tAttributesCODE
    AccountID            NVARCHAR(50)                    NOT NULL,     -- Identifier for the account within one type.
    AccountTypeCODE      NVARCHAR(50)                    NOT NULL,     -- Classification for an account within a group of Accounts, e.g. GUV, BILANZ, DEBITOR, KREDITOR. When no classification is given from source system, the classification will be done through agent.
    AccountName          NVARCHAR(255)                   NOT NULL,     -- Name of the Account.
    CustomValuesJSON     NVARCHAR(4000)                  NULL,         -- Custom field to store any customer individual informations.
    CONSTRAINT PK_integration_tFinAccounts PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store informations about acccounts. Table contains data from source system.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'DataSourceKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Company ID which is used in the source database.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'CompanyID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of accounts. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the account within one type.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for an account within a group of Accounts, e.g. GUV, BILANZ, DEBITOR, KREDITOR. When no classification is given from source system, the classification will be done through agent.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountTypeCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the Account.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'AccountName'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Custom field to store any customer individual informations.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinAccounts',
    @level2type = N'Column',
    @level2name = N'CustomValuesJSON'
GO

/*
    Product: OCT 5.0
    Object: global.tFinCostObjects
    Type: Table
    Description: Table to store informations about cost objects (cost centers, cost units, ...). Table contains merged data from integration and planning.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP TABLE IF EXISTS integration.tFinCostObjects;

CREATE TABLE integration.tFinCostObjects (
    RowKey                  BIGINT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    DataSourceKey           INT                             NOT NULL,     -- DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.
    CompanyID               NVARCHAR(50)                    NOT NULL,     -- Company ID which is used in the source database.
    CostObjectGroupCODE     NVARCHAR(50)                    NOT NULL,     -- Classification for a group of CostObjects. CODEs are defined in table global.tAttributesCODE
    CostObjectID            NVARCHAR(255)                   NOT NULL,     -- Identifier for the CostObject within one type.
    CostObjectName          NVARCHAR(255)                   NOT NULL,     -- Name of the CostObject.
    CustomValuesJSON        NVARCHAR(4000)                  NULL,         -- Custom field to store any customer individual informations.
    CONSTRAINT PK_integration_tFinCostObjects PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store informations about cost objects (cost centers, cost units, ...). Table contains merged data from integration and planning.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'RowKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'DataSourceKey is used to create a unique identifier when paired with CompanyID. DataSourceKey is provided with the ETL parameters.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'DataSourceKey'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Company ID which is used in the source database.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CompanyID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Classification for a group of CostObjects. CODEs are defined in table global.tAttributesCODE',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectGroupCODE'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Identifier for the CostObject within one type.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectID'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the CostObject.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CostObjectName'
GO

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Custom field to store any customer individual informations.',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Table',
    @level1name = N'tFinCostObjects',
    @level2type = N'Column',
    @level2name = N'CustomValuesJSON'
GO


IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tdFactories') AND type in (N'U'))

CREATE TABLE load.tdFactories(
	
	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,FactoryID			NVARCHAR(255)	NOT NULL
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_tdFactories PRIMARY KEY CLUSTERED (RowKey)
)
GO

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tdProductLines') AND type in (N'U'))

CREATE TABLE load.tdProductLines(
	 RowKey						INT				NOT NULL IDENTITY (1,1)
	,ProductLineID				NVARCHAR(255)	NOT NULL
	,FactoryID					NVARCHAR(255)	NOT NULL
	,NameShort					NVARCHAR(255)	NOT NULL
	,NameLong					NVARCHAR(255)	NOT NULL
	,CommentUser				NVARCHAR(MAX)	NOT NULL
	,CommentDev					NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson			NVARCHAR(255)	NOT NULL
	,ImageName					NVARCHAR(255)	NOT NULL
	,DefaultTemplate			NVARCHAR(255)	NOT NULL

	,GlobalAttributeSource1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource10	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias10		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource11	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias11		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource12	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias12		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource13	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias13		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource14	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias14		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource15	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias15		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource16	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias16		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource17	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias17		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource18	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias18		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource19	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias19		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource20	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias20		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource21	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias21		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource22	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias22		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource23	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias23		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource24	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias24		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource25	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias25		NVARCHAR(255)	NOT NULL
	,Source						NVARCHAR(255)	NOT NULL
	,SourceTimeStamp			DATETIME		NOT NULL

	CONSTRAINT PK_load_tdProductLines PRIMARY KEY CLUSTERED (RowKey)
	)

GO

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tdProducts') AND type in (N'U'))


CREATE TABLE load.tdProducts(

	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeType			NVARCHAR(255)	NOT NULL

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL

	,[Status]			NVARCHAR(255)	NOT NULL
	,Template			NVARCHAR(255)	NOT NULL
	,TemplateVersion	NVARCHAR(255)	NOT NULL

	,GlobalAttribute1	NVARCHAR(255)	NOT NULL
	,GlobalAttribute2	NVARCHAR(255)	NOT NULL
	,GlobalAttribute3	NVARCHAR(255)	NOT NULL
	,GlobalAttribute4	NVARCHAR(255)	NOT NULL
	,GlobalAttribute5	NVARCHAR(255)	NOT NULL
	,GlobalAttribute6	NVARCHAR(255)	NOT NULL
	,GlobalAttribute7	NVARCHAR(255)	NOT NULL
	,GlobalAttribute8	NVARCHAR(255)	NOT NULL
	,GlobalAttribute9	NVARCHAR(255)	NOT NULL
	,GlobalAttribute10	NVARCHAR(255)	NOT NULL
	,GlobalAttribute11	NVARCHAR(255)	NOT NULL
	,GlobalAttribute12	NVARCHAR(255)	NOT NULL
	,GlobalAttribute13	NVARCHAR(255)	NOT NULL
	,GlobalAttribute14	NVARCHAR(255)	NOT NULL
	,GlobalAttribute15	NVARCHAR(255)	NOT NULL
	,GlobalAttribute16	NVARCHAR(255)	NOT NULL
	,GlobalAttribute17	NVARCHAR(255)	NOT NULL
	,GlobalAttribute18	NVARCHAR(255)	NOT NULL
	,GlobalAttribute19	NVARCHAR(255)	NOT NULL
	,GlobalAttribute20	NVARCHAR(255)	NOT NULL
	,GlobalAttribute21	NVARCHAR(255)	NOT NULL
	,GlobalAttribute22	NVARCHAR(255)	NOT NULL
	,GlobalAttribute23	NVARCHAR(255)	NOT NULL
	,GlobalAttribute24	NVARCHAR(255)	NOT NULL
	,GlobalAttribute25	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_tdProducts PRIMARY KEY CLUSTERED (RowKey)

)
GO

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tdTime') AND type in (N'U'))


CREATE TABLE load.tdTime (
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeID				BIGINT			NOT NULL
	,FormatID			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_tdTime PRIMARY KEY (RowKey)

)
GO


IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tdValueSeries') AND type in (N'U'))


CREATE TABLE load.tdValueSeries(
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL

	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_tdValueSeries PRIMARY KEY CLUSTERED (RowKey)

)
GO
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.tfValues') AND type in (N'U'))


CREATE TABLE load.tfValues(
 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL

	,TimeID				INT				NOT NULL

	,ValueFormula		NVARCHAR(MAX)	NOT NULL	
	,ValueInt			BIGINT			NOT NULL
	,ValueText			NVARCHAR(MAX)	NOT NULL
	,ValueComment		NVARCHAR(MAX)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_tfValues PRIMARY KEY CLUSTERED (RowKey)
)
GO

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.trRights') AND type in (N'U'))

CREATE TABLE load.trRights (
	
	 RowKey					INT				NOT NULL IDENTITY (1,1) 
	,UserName				NVARCHAR(255)	NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,ProductLineID			NVARCHAR(255)	NOT NULL
	,ProductID				NVARCHAR(255)	NOT NULL
	,[Right]				NVARCHAR(255)	NOT NULL
	,ReadCommentMandatory	INT				NOT NULL
	,WriteCommentMandatory	INT				NOT NULL
	,Source					NVARCHAR(255)	NOT NULL
	,SourceTimeStamp		DATETIME		NOT NULL

	CONSTRAINT PK_load_trRights PRIMARY KEY CLUSTERED (RowKey)
)
GO

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'load.trUser') AND type in (N'U'))


CREATE TABLE load.trUser(
	
	 RowKey				INT				NOT NULL IDENTITY (1,1) 
	,UserName			NVARCHAR(255)	NOT NULL
	,PersonSurname		NVARCHAR(255)	NOT NULL
	,PersonFirstName	NVARCHAR(255)	NOT NULL
	,Email				NVARCHAR(255)	NOT NULL
	,LDAPIP				NVARCHAR(255)	NOT NULL
	,Status				NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	CONSTRAINT PK_load_trUser PRIMARY KEY CLUSTERED (RowKey)
)

GO
/*
    Product: OCT 5.0
    Object: global.tSettings
    Type: Table
    Author: saxess-software GmbH
    Script Date: 18.07.2019 14:35
*/

DROP TABLE IF EXISTS system.tSettings;

CREATE TABLE system.tSettings (
    SettingKey     NVARCHAR(255)   NOT NULL,
    Value          NVARCHAR(255)   NOT NULL,
    CONSTRAINT PK_system_tSettings PRIMARY KEY NONCLUSTERED (SettingKey)
);

-- Default Content Flag
EXEC sp_addextendedproperty
    @name = N'IsDefaultContent',
    @value = N'1',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Table',
    @level1name = N'tSettings'
GO


IF NOT EXISTS(SELECT 1 FROM system.tSettings)
BEGIN

    INSERT INTO system.tSettings (SettingKey, Value) VALUES
    ('DBVersion', '500')
END;
GO

-- Script for creating a planning factory datamodel in an existing database
-- Gerd Tautenhahn for saxess-software gmbh 04/2019
-- Version PlanningFactory 4.0
-- Coding UTF-8 without BOM 

SET QUOTED_IDENTIFIER ON
GO
/*
    Product: OCT 5.0
    Object: agent
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('agent') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA agent AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'agent'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent'
GO

/*
    Product: OCT 5.0
    Object: calc
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('calc') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA calc AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'calc'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'calc'
GO

/*
    Product: OCT 5.0
    Object: control
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('control') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA control AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'control'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'control'
GO

/*
    Product: OCT 5.0
    Object: global
    Type: Schema
    Description: Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('global') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA global AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.',
        @level0type = N'Schema',
        @level0name = N'global'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.',
    @level0type = N'Schema',
    @level0name = N'global'
GO

/*
    Product: OCT 5.0
    Object: import
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('import') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA import AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'import'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'import'
GO

/*
    Product: OCT 5.0
    Object: integration
    Type: Schema
    Description: Schema to store integration data. Temporary and no enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('integration') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA integration AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store integration data. Temporary and no enduser access.',
        @level0type = N'Schema',
        @level0name = N'integration'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store integration data. Temporary and no enduser access.',
    @level0type = N'Schema',
    @level0name = N'integration'
GO

/*
    Product: OCT 5.0
    Object: load
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('load') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA load AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'load'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'load'
GO

/*
    Product: OCT 5.0
    Object: param
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('param') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA param AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'param'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'param'
GO

/*
    Product: OCT 5.0
    Object: result
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('result') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA result AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'result'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'result'
GO

/*
    Product: OCT 5.0
    Object: staging
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('staging') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA staging AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'staging'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'staging'
GO

/*
    Product: OCT 5.0
    Object: sxdf
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('sxdf') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA sxdf AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'sxdf'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'sxdf'
GO

/*
    Product: OCT 5.0
    Object: system
    Type: Schema
    Description: Schema to store system data and Metadata. No enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('system') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA system AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store system data and Metadata. No enduser access.',
        @level0type = N'Schema',
        @level0name = N'system'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store system data and Metadata. No enduser access.',
    @level0type = N'Schema',
    @level0name = N'system'
GO


/*
Creates Database Roles for PlanningFactory
PlanningFactory 4.0
07/2016
*/

IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryUser') IS NULL CREATE ROLE pf_PlanningFactoryUser
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryService') IS NULL CREATE ROLE pf_PlanningFactoryService
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryAnalyst') IS NULL CREATE ROLE pf_PlanningFactoryAnalyst
GO

-- Function to split a String into a TableVariable
-- The Delimiter String will be replace by a CHAR(28) and the string splitted on this


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_PivotStringIntoTable]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[sx_pf_PivotStringIntoTable]
GO

CREATE FUNCTION dbo.sx_pf_PivotStringIntoTable
(@String NVARCHAR(MAX),
@Delimiter NVARCHAR(5))

RETURNS @tblSplitValues TABLE (txtValues NVARCHAR(MAX))

AS
BEGIN

	DECLARE @spr		NCHAR(1)		= CHAR(28)
	DECLARE @auxString	NVARCHAR (MAX)	= ''
	SET @auxString = REPLACE(@String, @Delimiter,@spr);

	WITH Split(stpos,endpos)
	AS(
		SELECT 0 AS stpos, CHARINDEX(@spr,@auxString) AS endpos
		UNION ALL
		SELECT CAST(endpos AS INT)+1, CHARINDEX(@spr,@auxString,endpos+1)
		FROM Split
		WHERE endpos > 0
	)

	INSERT @tblSplitValues
		SELECT SUBSTRING(@auxString,stpos,COALESCE(NULLIF(endpos,0),LEN(@auxString)+1)-stpos)
		FROM Split OPTION (MAXRECURSION 32766) -- it works ONLY up to 32.766 rows

	RETURN

END

GO

/* 
function to mask strings, which will be part of an SQL Statement @StringValue 
returns a masked string
Gerd Tautenhahn for saxess-software gmbh
12/2016 for DataFactory 4.0

Testcall
DECLARE @String NVARCHAR(255)
SET @String = 'ToDo''s'
Print @String
SELECT dbo.sx_pf_pMaskSQL(@String)
*/



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pMaskSQL]') AND type in (N'FN'))
DROP FUNCTION sx_pf_pMaskSQL
GO


CREATE FUNCTION sx_pf_pMaskSQL (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(@StringValue,'''','''''')
END

GO
-- Funktion gibt zurück einen '0', wenn der übergebene @BooleanValue ist nicht etweder '0' oder '1'
-- @BooleanValue ='<#NV>' wird ignoriert und nicht geändert

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pProtectBoolean]') AND type in (N'FN'))
DROP FUNCTION sx_pf_pProtectBoolean
GO


CREATE FUNCTION sx_pf_pProtectBoolean (@BooleanValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @BooleanValue <>'<#NV>' 
	
	BEGIN
		IF @BooleanValue <>'0' AND @BooleanValue <>'1'
		SET @BooleanValue= '0'
	END

	RETURN @BooleanValue

END

GO

-- Function to replace all special signs from ID
-- An ID may contain Letters, Numbers,'_' and '-', but nothing else
-- Function returns a clean string

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pProtectID]') AND type in (N'FN'))
DROP FUNCTION sx_pf_pProtectID
GO


CREATE FUNCTION sx_pf_pProtectID (@ElementID AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	SET @ElementID = RTRIM(LTRIM(@ElementID))
	RETURN	REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(@ElementID,'|',''),
							   ';',''),'''',''),'/',''),'\',''),'"',''),',',''),
							   '>',''),'<',''),'[',''),']',''),'(',''),')',''),'{',''),'}',''),'!',''),'$',''),'&',''),'^',''),'@',''),
							   '%',''),':',''),'`',''),'´',''),'?',''),'§',''),'#',''),'~',''),'+',''),'=',''),'.',''),'€',''),' ','')
END

GO

/*
Function cast all expression to int or return 0 for non-int values, except the <#NV> String this is passed anytime

Testcalls
SELECT dbo.sx_pf_pProtectInt('-1')
SELECT dbo.sx_pf_pProtectInt('1')
SELECT dbo.sx_pf_pProtectInt('Hase')
SELECT dbo.sx_pf_pProtectInt('<#NV>')
SELECT dbo.sx_pf_pProtectInt('')
SELECT dbo.sx_pf_pProtectInt('0')
SELECT dbo.sx_pf_pProtectInt('9999999999999999999999999999999999999999999999999999999999')

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pProtectInt]') AND type in (N'FN'))
DROP FUNCTION sx_pf_pProtectInt
GO


CREATE FUNCTION sx_pf_pProtectInt (@IntValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @IntValue <>'<#NV>' 
		BEGIN
			SET @IntValue = COALESCE(CAST(TRY_CAST(@IntValue AS INT) AS NVARCHAR(255)),0)
		END

RETURN @IntValue

END

GO

-- Funktion entfern alle nicht erlaubte Sonderzeichen ( Hochommata,'|','),(', ';' ) aus @StringValue 
-- Die Funktion gibt zurück einen string, ohne nicht erlaubte Sonderzeichen

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pProtectString]') AND type in (N'FN'))
DROP FUNCTION sx_pf_pProtectString
GO


CREATE FUNCTION sx_pf_pProtectString (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(REPLACE(REPLACE(@StringValue,'|',''), '),(',''), '--','')
END

GO

/* 
function to remove Linebreaks from a string
Gerd Tautenhahn for saxess-software gmbh
11/2018 for DataFactory 4.0

Testcall
DECLARE @String NVARCHAR(255)
SET @String = 'ToDo'+CHAR(13)+CHAR(10)+' for you'
Print @String
SELECT @String = dbo.sx_pf_pRemoveLinebreaks(@String)
Print @String
*/



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_pRemoveLinebreaks') AND type in (N'FN'))
DROP FUNCTION sx_pf_pRemoveLinebreaks
GO


CREATE FUNCTION sx_pf_pRemoveLinebreaks (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(REPLACE(@StringValue,CHAR(10),''),CHAR(13),'')

END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[Day_of_Week_DE]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [Day_of_Week_DE]
GO

CREATE FUNCTION dbo.Day_of_Week_DE (@d DATETIME)
RETURNS INT
  BEGIN
      DECLARE @t INT

      SET @t = Datepart(weekday, @d) + @@DATEFIRST - 1

      IF @t > 7
        SET @t = @t - 7

      RETURN @t
  END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[Week_of_Year_DE]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [Week_of_Year_DE]
GO

CREATE FUNCTION dbo.Week_of_Year_DE (@d DATETIME)
returns INT
  BEGIN
      DECLARE @k INT
      DECLARE @x DATETIME

      SET @x = Str(Year(@d)) + '-01-01'
      SET @k = Datepart(week, @d)

      IF dbo.Day_of_Week_DE(@x) > 4
        BEGIN
            SET @k = @k - 1

            IF @k = 0
              SET @k = 53
        END

      RETURN @k
  END 
GO
/*
    Product: OCT 5.0
    Object: system.GetSetting
    Type: Function
    Author: saxess-software GmbH
    Script Date: 18.07.2019 14:17
*/

DROP FUNCTION IF EXISTS system.GetSetting;
GO

CREATE FUNCTION system.GetSetting
(
	@SettingKey NVARCHAR(255)
)
RETURNS NVARCHAR(255)
AS
BEGIN
	DECLARE @Value NVARCHAR(255);

	SELECT @Value = Value
	FROM system.tSettings
	WHERE SettingKey = @SettingKey;

	RETURN @Value;
END
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Function',
    @level1name = N'GetSetting'
GO


/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODEL_dFactories]'))
DROP VIEW [dbo].[sx_pf_DATAMODEL_dFactories]
GO

CREATE VIEW sx_pf_DATAMODEL_dFactories

AS

	SELECT 
			 FactoryKey
			,dF.FactoryID
			,NameShort		AS FactoryNameShort
			,dF.FactoryID + ' ' + NameShort AS FactoryIDName
			,NameLong		AS FactoryNameLong
			,CommentUser	AS FactoryCommentUser
			,CommentDev	AS FactoryCommentDev
			,ResponsiblePerson AS FactoryResponsiblePerson
			,ImageName			AS FactoryImageName

	FROM   sx_pf_dFactories dF

			INNER JOIN 
				(SELECT FactoryID FROM sx_pf_vUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID) vUR
				ON dF.FactoryID = vUR.FactoryID
				

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODEL_dProductLines]'))
DROP VIEW [dbo].[sx_pf_DATAMODEL_dProductLines]
GO

CREATE VIEW sx_pf_DATAMODEL_dProductLines 

AS

	SELECT 
			 ProductLineKey
			,dPL.ProductLineID
			,dPL.FactoryID
			,NameShort	AS ProductLineNameShort
			,dPL.ProductLineID + ' ' + NameShort AS ProductLineIDName
			,NameLong	AS ProductLineNameLong
			,CommentUser AS ProductLineCommentUser
			,CommentDev			AS ProductLineCommentDev
			,ResponsiblePerson	AS ProductLineResponsiblePerson
			,ImageName			AS ProductLineImageName
			,DefaultTemplate
			,GlobalAttributeAlias1
			,GlobalAttributeAlias2
			,GlobalAttributeAlias3
			,GlobalAttributeAlias4
			,GlobalAttributeAlias5
			,GlobalAttributeAlias6
			,GlobalAttributeAlias7
			,GlobalAttributeAlias8
			,GlobalAttributeAlias9
			,GlobalAttributeAlias10
			,GlobalAttributeAlias11
			,GlobalAttributeAlias12
			,GlobalAttributeAlias13
			,GlobalAttributeAlias14
			,GlobalAttributeAlias15
			,GlobalAttributeAlias16
			,GlobalAttributeAlias17
			,GlobalAttributeAlias18
			,GlobalAttributeAlias19
			,GlobalAttributeAlias20
			,GlobalAttributeAlias21
			,GlobalAttributeAlias22
			,GlobalAttributeAlias23
			,GlobalAttributeAlias24
			,GlobalAttributeAlias25
	FROM   sx_pf_dProductLines dPL

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID

GO
				

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODEL_dProducts]'))
DROP VIEW [dbo].[sx_pf_DATAMODEL_dProducts]
GO

CREATE VIEW sx_pf_DATAMODEL_dProducts

AS

	SELECT 
			 ProductKey
			,ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort	AS ProductNameShort
			,ProductID + ' ' + NameShort AS ProductIDName
			,NameLong	AS ProductNameLong
			,CommentUser	AS ProductCommentUser
			,CommentDev	AS ProductCommentDev
			,ResponsiblePerson	AS ProductResponsiblePerson
			,ImageName			AS ProductImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
	FROM   sx_pf_dProducts dP

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dP.FactoryID = vUR.FactoryID AND
					dP.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODEL_dValueSeries]'))
DROP VIEW [dbo].[sx_pf_DATAMODEL_dValueSeries]
GO

CREATE VIEW sx_pf_DATAMODEL_dValueSeries 

AS

		SELECT 
			 ValueSeriesKey
			,ProductID
			,dVS.ProductLineID
			,dVS.FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort AS ValueSeriesNameShort
			,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
			,NameLong AS ValueSeriesNameLong
			,CommentUser AS ValueSeriesCommentUser
			,CommentDev AS ValueSeriesCommentDev
			,ImageName AS ValueSeriesImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
		FROM   sx_pf_dValueSeries dVS 

				INNER JOIN 
					(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
					ON
						dVS.FactoryID = vUR.FactoryID AND
						dVS.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODEL_fValues]'))
DROP VIEW [dbo].[sx_pf_DATAMODEL_fValues]
GO

CREATE VIEW sx_pf_DATAMODEL_fValues

AS

	SELECT 
			 fV.ProductKey
			,fV.ProductLineKey
			,fV.FactoryKey
			,fV.ValueSeriesKey
			,fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,CAST(LEFT(TimeID,4) AS int) AS [Year]
			,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Month]
			,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
			,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
			,ValueFormula
			,CAST(ValueInt AS Money) / Coalesce(dVS.Scale,1) AS Value
			,ValueText
			,ValueComment
					
	FROM   sx_pf_fValues fV 

			LEFT JOIN sx_pf_dValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					fV.FactoryID = vUR.FactoryID AND
					fV.ProductLineID = vUR.ProductLineID


			WHERE ValueInt != 0 OR ( ValueText != ''  AND ValueText != '-')
GO
			

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODELCLOUD_dFactories]'))
DROP VIEW [dbo].[sx_pf_DATAMODELCLOUD_dFactories]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dFactories

AS

	SELECT 
			 FactoryKey
			,dF.FactoryID
			,NameShort		AS FactoryNameShort
			,dF.FactoryID + ' ' + NameShort AS FactoryIDName
			,NameLong		AS FactoryNameLong
			,CommentUser	AS FactoryCommentUser
			,CommentDev	AS FactoryCommentDev
			,ResponsiblePerson AS FactoryResponsiblePerson
			,ImageName			AS FactoryImageName

	FROM   sx_pf_dFactories dF

			INNER JOIN 
				(SELECT FactoryID FROM sx_pf_vUserRights WHERE Username = HOST_NAME() AND [Right] IN ('Write','Read') GROUP BY FactoryID) vUR
				ON dF.FactoryID = vUR.FactoryID
				

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODELCLOUD_dProductLines]'))
DROP VIEW [dbo].[sx_pf_DATAMODELCLOUD_dProductLines]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dProductLines 

AS

	SELECT 
			 ProductLineKey
			,dPL.ProductLineID
			,dPL.FactoryID
			,NameShort	AS ProductLineNameShort
			,dPL.ProductLineID + ' ' + NameShort AS ProductLineIDName
			,NameLong	AS ProductLineNameLong
			,CommentUser AS ProductLineCommentUser
			,CommentDev			AS ProductLineCommentDev
			,ResponsiblePerson	AS ProductLineResponsiblePerson
			,ImageName			AS ProductLineImageName
			,DefaultTemplate
			,GlobalAttributeAlias1
			,GlobalAttributeAlias2
			,GlobalAttributeAlias3
			,GlobalAttributeAlias4
			,GlobalAttributeAlias5
			,GlobalAttributeAlias6
			,GlobalAttributeAlias7
			,GlobalAttributeAlias8
			,GlobalAttributeAlias9
			,GlobalAttributeAlias10
			,GlobalAttributeAlias11
			,GlobalAttributeAlias12
			,GlobalAttributeAlias13
			,GlobalAttributeAlias14
			,GlobalAttributeAlias15
			,GlobalAttributeAlias16
			,GlobalAttributeAlias17
			,GlobalAttributeAlias18
			,GlobalAttributeAlias19
			,GlobalAttributeAlias20
			,GlobalAttributeAlias21
			,GlobalAttributeAlias22
			,GlobalAttributeAlias23
			,GlobalAttributeAlias24
			,GlobalAttributeAlias25
	FROM   sx_pf_dProductLines dPL

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID

GO
				

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODELCLOUD_dProducts]'))
DROP VIEW [dbo].[sx_pf_DATAMODELCLOUD_dProducts]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dProducts

AS

	SELECT 
			 ProductKey
			,ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort	AS ProductNameShort
			,ProductID + ' ' + NameShort AS ProductIDName
			,NameLong	AS ProductNameLong
			,CommentUser	AS ProductCommentUser
			,CommentDev	AS ProductCommentDev
			,ResponsiblePerson	AS ProductResponsiblePerson
			,ImageName			AS ProductImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
	FROM   sx_pf_dProducts dP

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dP.FactoryID = vUR.FactoryID AND
					dP.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODELCLOUD_dValueSeries]'))
DROP VIEW [dbo].[sx_pf_DATAMODELCLOUD_dValueSeries]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dValueSeries 

AS

		SELECT 
			 ValueSeriesKey
			,ProductID
			,dVS.ProductLineID
			,dVS.FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort AS ValueSeriesNameShort
			,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
			,NameLong AS ValueSeriesNameLong
			,CommentUser AS ValueSeriesCommentUser
			,CommentDev AS ValueSeriesCommentDev
			,ImageName AS ValueSeriesImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
		FROM   sx_pf_dValueSeries dVS 

				INNER JOIN 
					(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
					ON
						dVS.FactoryID = vUR.FactoryID AND
						dVS.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAMODELCLOUD_fValues]'))
DROP VIEW [dbo].[sx_pf_DATAMODELCLOUD_fValues]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_fValues

AS

	SELECT 
			 fV.ProductKey
			,fV.ProductLineKey
			,fV.FactoryKey
			,fV.ValueSeriesKey
			,fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,CAST(LEFT(TimeID,4) AS int) AS [Year]
			,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Month]
			,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
			,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
			,ValueFormula
			,CAST(ValueInt AS Money) / Coalesce(dVS.Scale,1) AS Value
			,ValueText
			,ValueComment
					
	FROM   sx_pf_fValues fV 

			LEFT JOIN sx_pf_dValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					fV.FactoryID = vUR.FactoryID AND
					fV.ProductLineID = vUR.ProductLineID

			WHERE ValueInt != 0 OR ( ValueText != ''  AND ValueText != '-')
GO
			

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_BALANCE
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_RESULT_Balance]'))
DROP VIEW [dbo].[sx_pf_RESULT_Balance]
GO


CREATE VIEW sx_pf_RESULT_Balance 

AS


	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT
					 *
				 FROM sx_pf_vUserRights 
				 WHERE 
					Username = ORIGINAL_LOGIN() AND
					[Right] IN ('Write','Read') AND
					FactoryID <> '' AND
					ProductLineID <> ''
				) vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.FactoryID != 'ZT' AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0

GO

GRANT SELECT ON OBJECT ::[dbo].[sx_pf_RESULT_Balance] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
04/2018

Testcall
SELECT * FROM sx_pf_RESULT_Cash

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'dbo.sx_pf_RESULT_Cash'))
DROP VIEW dbo.sx_pf_RESULT_Cash
GO


CREATE VIEW sx_pf_RESULT_Cash

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson
		 ,dP.Template								AS Template

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM sx_pf_vUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID <> 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[dbo].[sx_pf_RESULT_Cash] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_Numeric

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_RESULT_Numeric]'))
DROP VIEW [dbo].[sx_pf_RESULT_Numeric]
GO


CREATE VIEW sx_pf_RESULT_Numeric

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM sx_pf_vUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[dbo].[sx_pf_RESULT_Numeric] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_Profit

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_RESULT_Profit]'))
DROP VIEW [dbo].[sx_pf_RESULT_Profit]
GO


CREATE VIEW sx_pf_RESULT_Profit

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson
		 ,dP.Template								AS Template

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM sx_pf_vUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[dbo].[sx_pf_RESULT_Profit] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
01/2018

Testcall
SELECT * FROM sx_pf_RESULT_Text

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_RESULT_Text]'))
DROP VIEW [dbo].[sx_pf_RESULT_Text]
GO


CREATE VIEW sx_pf_RESULT_Text

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000														AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100													AS Month
		 ,	  RIGHT('0'+ CAST((TRY_CAST(fV.TimeID AS Integer) % 10000) /100 AS NVARCHAR(255)),2)
			+ '/' 
			+ CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS NVARCHAR(255))								AS MonthYear
			
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,   TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) 
			+'-'
			+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00')	AS YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   dbo.sx_pf_fValues fV
         LEFT JOIN dbo.sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN dbo.sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN dbo.sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN dbo.sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN dbo.sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM dbo.sx_pf_vUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND
		dVS.[IsNumeric] = 0 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::dbo.sx_pf_RESULT_Text TO pf_PlanningFactoryUser;

GO


/*
View for analysing the LogEntrys of PlanningFactory API

11/2015 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
*/


IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_vLogEntrys]'))
DROP VIEW sx_pf_vLogEntrys
GO

CREATE VIEW sx_pf_vLogEntrys AS 

SELECT 
	 @@Servername AS Servername
	,DB_NAME()  AS ClusterName
	,SQLSystemUser
	,tL.Username AS ServiceUsername
	,IIF(tL.Username = 'SQL','Active',rU2.Status) AS ServiceUserStatus
	,TransactUsername
	,rU1.Status AS TransactUserStatus
	,ProcedureName
	,ParameterString
	,EffectedRows
	,ReturnCode
	,TimestampCall
	,TimestampReturn
	,CASE
		WHEN DATEDIFF(SECOND,TimestampCall,TimestampReturn) < 10 THEN
			DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn)
		ELSE
			10000
	END AS DurationInMilliseconds
	
	--,DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn) AS DurationInMilliseconds
	,DATEDIFF(SECOND,TimestampCall,TimestampReturn) AS DurationInSeconds
	,Comment
	,ProcessCode

FROM 
	sx_pf_API_Log tL 
		LEFT JOIN sx_pf_rUser rU1 ON tL.TransactUsername = rU1.Username
		LEFT JOIN sx_pf_rUser rU2 ON tL.UserName = rU2.UserName

GO

/*
View to pivot ProcessLog to table view
Gerd Tautenhahn for Saxess Software GmbH
07/2018 DataFactory 4.0

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'dbo.sx_pf_vProcess_Start_End'))
DROP VIEW dbo.sx_pf_vProcess_Start_End
GO

CREATE VIEW dbo.sx_pf_vProcess_Start_End

AS

SELECT 
		 PivotTable.MainProcess
		,PivotTable.SubProcess
		,PivotTable.Step
		,MIN(PivotTable.Start) AS Start
		,MAX(PivotTable.[End]) AS [End]
		,CAST(MAX(PivotTable.[End]) - MIN(PivotTable.Start) AS TIME) AS Duration

FROM dbo.sx_pf_Process_Log PL

PIVOT
	(	
		-- List of the needed ValueSeries in SquareBrackets
		MAX(Timestamp) FOR Start_End_Info IN ([Start],[End])
	) AS PivotTable

GROUP BY 
	 PivotTable.MainProcess
	,PivotTable.SubProcess
	,PivotTable.Step

GO

-- No GRANT as only used by db_owners etc.
/*
    Product: OCT 5.0
    Object: agent.pDoMaintenance
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pDoMaintenance;
GO


CREATE PROCEDURE [agent].[pDoMaintenance]
AS
BEGIN
	-- Shrink Log File
	DBCC SHRINKFILE(2,1) WITH NO_INFOMSGS;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDoMaintenance'
GO

/*
    Product: OCT 5.0
    Object: agent.pDoMaintenanceFin
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pDoMaintenanceFin;
GO

CREATE PROCEDURE [agent].[pDoMaintenanceFin]
AS
	-- Set Delta Values

	-- Update History Table

	-- Reorganize Columnstore Indizes in Result schema
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDoMaintenanceFin'
GO

/*
    Product: OCT 5.0
    Object: agent.pSync
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pSync;
GO

CREATE PROCEDURE [agent].[pSync]
	@Solution NVARCHAR(50),
	@ResultOnly INT = 0,
	@CompanyKeys NVARCHAR(MAX) -- delimited by ;
AS
BEGIN
	DECLARE @SQLCommand NVARCHAR(255)
	SET @SQLCommand = 'EXEC agent.pSync' + @Solution + ' @ResultOnly = ' + CAST(@ResultOnly AS NVARCHAR(1)) + ', @CompanyKeys = ''' + @CompanyKeys + '''';

	EXEC (@SQLCommand);

	EXEC agent.pDoMaintenance;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSync'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncFin
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pSyncFin;
GO

CREATE PROCEDURE [agent].[pSyncFin]
	@ResultOnly INT,
	@CompanyKeys NVARCHAR(MAX) -- delimited by ;
AS
BEGIN
	IF @ResultOnly = 1
	BEGIN
		--EXEC agent.pSyncFinAccountsResult @CompanyKeys = @CompanyKeys;
		--EXEC agent.pSyncFinCostObjectsResult @CompanyKeys = @CompanyKeys;
		Select 1
	END
	ELSE
	BEGIN
		--EXEC agent.DoQualityCheckFin @CompanyKeys = @CompanyKeys;		
		EXEC agent.pSyncFinAccountsGlobal @CompanyKeys = @CompanyKeys;
		--EXEC agent.pSyncFinAccountsResult @CompanyKeys = @CompanyKeys;
		EXEC agent.pSyncFinCostObjectsGlobal @CompanyKeys = @CompanyKeys;
		--EXEC agent.pSyncFinCostObjectsResult @CompanyKeys = @CompanyKeys;
		--EXEC pSyncFinAccountingJournal @CompanyKeys = @CompanyKeys;
	END

	EXEC agent.pDoMaintenanceFin;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncFin'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncFinAccountsGlobal
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pSyncFinAccountsGlobal;
GO

CREATE PROCEDURE [agent].[pSyncFinAccountsGlobal]
	@CompanyKeys NVARCHAR(MAX)
AS
BEGIN
	-- Update/Insert 
	MERGE global.tFinAccounts WITH (HOLDLOCK) AS Target
	USING (
		SELECT TOP 100 PERCENT *
		FROM integration.tFinAccounts
		WHERE CAST(DataSourceKey AS NVARCHAR(50)) + N'|' + CompanyID IN (
			SELECT value
			FROM STRING_SPLIT(@CompanyKeys,';')
		)
	) AS Source
	ON (
		Target.DataSourceKey = Source.DataSourceKey
		AND Target.CompanyID = Source.CompanyID
		AND Target.AccountGroupCODE = Source.AccountGroupCODE
		AND Target.AccountID = Source.AccountID
	)

	WHEN
		MATCHED
		AND (
			Target.AccountTypeCODE <> Source.AccountTypeCODE
			OR Target.AccountName <> Source.AccountName
			OR Target.AccountOwnerCODE <> 'OWNER_INT'
		)
	THEN
		UPDATE SET
			Target.AccountName = Source.AccountName,
			Target.AccountTypeCODE = Source.AccountTypeCODE,
			Target.AccountOwnerCODE = 'OWNER_INT',
			Target.UpdatedFlag = 1

	WHEN NOT MATCHED BY TARGET THEN
		INSERT (
			DataSourceKey,
			CompanyID,
			AccountGroupCODE,
			AccountID, 
			AccountTypeCODE, 
			AccountOwnerCODE, 
			AccountName, 
			UpdatedFlag, 
			CustomValuesJSON
		)
		VALUES (
			Source.DataSourceKey,
			Source.CompanyID,
			Source.AccountGroupCODE,
			Source.AccountID, 
			Source.AccountTypeCODE, 
			'INT', 
			Source.AccountName, 
			1, 
			COALESCE(Source.CustomValuesJSON, '{}')
		)

	WHEN NOT MATCHED BY SOURCE AND Target.AccountOwnerCode <> 'OWNER_UNBEKANNT' THEN
		UPDATE SET
			Target.AccountOwnerCODE = 'OWNER_UNBEKANNT'
	;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncFinAccountsGlobal'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncFinCostObjectsGlobal
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

DROP PROCEDURE IF EXISTS agent.pSyncFinCostObjectsGlobal;
GO

CREATE PROCEDURE [agent].[pSyncFinCostObjectsGlobal]
	@CompanyKeys NVARCHAR(MAX)
AS
BEGIN
	-- Update/Insert 
	MERGE global.tFinCostObjects WITH (HOLDLOCK) AS Target
	USING (
		SELECT *
		FROM integration.tFinCostObjects
		WHERE CAST(DataSourceKey AS NVARCHAR(50)) + N'|' + CompanyID IN (
			SELECT value
			FROM STRING_SPLIT(@CompanyKeys,';')
		)
	) AS Source
	ON (
		Target.DataSourceKey = Source.DataSourceKey
		AND Target.CompanyID = Source.CompanyID
		AND Target.CostObjectGroupCODE = Source.CostObjectGroupCODE
		AND Target.CostObjectID = Source.CostObjectID
	)

	WHEN
		MATCHED
		AND (
			Target.CostObjectName <> Source.CostObjectName
			OR Target.CostObjectOwnerCODE <> 'OWNER_INT'
		)
	THEN
		UPDATE SET
			Target.CostObjectName = Source.CostObjectName,
			Target.CostObjectOwnerCODE = 'OWNER_INT',
			Target.UpdatedFlag = 1

	WHEN NOT MATCHED BY TARGET THEN
		INSERT (
			DataSourceKey,
			CompanyID,
			CostObjectGroupCODE,
			CostObjectID,
			CostObjectOwnerCODE, 
			CostObjectName,
			UpdatedFlag, 
			CustomValuesJSON
		)
		VALUES (
			Source.DataSourceKey,
			Source.CompanyID,
			Source.CostObjectGroupCODE,
			Source.CostObjectID,
			'INT',
			Source.CostObjectName,
			1,
			COALESCE(Source.CustomValuesJSON, '{}')
		)

	WHEN NOT MATCHED BY SOURCE AND Target.CostObjectOwnerCode <> 'OWNER_UNBEKANNT' THEN
		UPDATE SET
			Target.CostObjectOwnerCode = 'OWNER_UNBEKANNT'
	;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncFinCostObjectsGlobal'
GO

/*
COPY Operation for a Factory
Not yet existing factory is copied when Source Factory exists

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty  source or target FactoryID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists Source Factory => 404 Not Found
5. Target Factory key already exists => 403 Forbidden
6. Factory created > 201 Created

DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) 
DECLARE @SourceFactoryID		NVARCHAR(255)
DECLARE @TargetFactoryID		NVARCHAR(255)
DECLARE @TargetFactoryNameShort NVARCHAR(255)

-- CASE 1
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= ''
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = dbo.sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

-- CASE 4
SET  @Username				 = 'SQL'
SET  @SourceFactoryID		 = ''
SET  @TargetFactoryID		 = 'ZTKopie'
SET  @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = dbo.sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

-- CASE 5 
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= 'ZT'
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = dbo.sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

--CASE 6
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= 'ZTKopie'
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = dbo.sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_Factory;
GO

CREATE PROCEDURE dbo.sx_pf_COPY_Factory
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetFactoryNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey		INT				= 0;
	DECLARE @SourceFactoryKey		INT				= 0;
	DECLARE @SourceFactoryNameShort NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetFactoryNameShort	IS NULL SET @TargetFactoryNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Factory;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetFactoryNameShort = dbo.sx_pf_pProtectString	(@TargetFactoryNameShort);
		
		IF	@SourceFactoryID	= N'' 
			OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
					
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM dbo.sx_pf_dFactories WHERE FactoryID = @SourceFactoryID;
		
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @TargetFactoryID;
		
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
		-- dFactory
		INSERT INTO dbo.sx_pf_dFactories
		OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
		SELECT @TargetFactoryID			AS FactoryID
			, @TargetFactoryNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName 
		FROM dbo.sx_pf_dFactories 
		WHERE FactoryID = @SourceFactoryID; 

		SET @EffectedRows = @@ROWCOUNT;	

		SELECT TOP 1 @TargetFactoryKey = [Key]
		FROM @Keys;
		
		--dProductLine 
		INSERT INTO dbo.sx_pf_dProductLines
		SELECT @TargetFactoryKey AS FactoryKey
			, ProductlineID
			, @TargetFactoryID	 AS FactoryID
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3 
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6 
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9 	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12 
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18 
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21 	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM dbo.sx_pf_dProductLines 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;	

		--dProduct 
		INSERT INTO dbo.sx_pf_dProducts
		SELECT PL.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, ProductID
			, dP.ProductlineID
			, @TargetFactoryID		AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM dbo.sx_pf_dProducts dP
				LEFT JOIN (
					SELECT ProductLineKey, ProductLineID 
					FROM dbo.sx_pf_dProductLines 
					WHERE FactoryKey = @TargetFactoryKey) PL 
				ON PL.ProductLineID = dP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO dbo.sx_pf_dValueSeries  
		SELECT P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dVS.ProductID
			, dVS.ProductLineID 
			, @TargetFactoryID	AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM dbo.sx_pf_dValueSeries dVS
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM dbo.sx_pf_dProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dVS.ProductID AND P.ProductLineID = dVS.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO dbo.sx_pf_dTime  
		SELECT 
			 P.ProductKey AS ProductKey
			, P.ProductLineKey AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dT.ProductID
			, dT.ProductLineID 
			, @TargetFactoryID AS FactoryID
			, TimeID
			, FormatID
		FROM dbo.sx_pf_dTime dT
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM dbo.sx_pf_dProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dT.ProductID AND P.ProductLineID = dT.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO dbo.sx_pf_fValues 
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, VS.ProductLineKey		AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, fV.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM dbo.sx_pf_fValues fV
				LEFT JOIN (
					SELECT ProductKey, ValueSeriesKey, ProductLineKey, ProductID, ValueSeriesID, ProductLineID 
					FROM dbo.sx_pf_dValueSeries 
					WHERE FactoryKey = @TargetFactoryKey) VS 
				ON fV.ValueSeriesID = VS.ValueSeriesID AND fV.ProductID = VS.ProductID AND fV.ProductLineID = VS.ProductLineID
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gFactories
		INSERT INTO dbo.sx_pf_gFactories 
		SELECT @TargetFactoryKey	AS FactoryKey
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gFactories
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gProductLines
		INSERT INTO dbo.sx_pf_gProductLines
		SELECT PL.ProductLineKey	AS ProductLineKey 
			, @TargetFactoryKey		AS FactoryKey
			, gPL.ProductlineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gProductLines gPL
			LEFT JOIN (
				SELECT ProductLineKey, ProductLineID 
				FROM sx_pf_dProductLines 
				WHERE FactoryKey = @TargetFactoryKey) 
			PL ON PL.ProductLineID = gPL.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO dbo.sx_pf_gProducts
		SELECT P.ProductKey		AS ProductKey
			, 0					AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, gP.ProductID
			, gP.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
				FROM dbo.sx_pf_dProducts 
				WHERE FactoryKey = @TargetFactoryKey) P 
			ON P.ProductID = gP.ProductID AND P.ProductLineID = gP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rPreferences
		INSERT INTO dbo.sx_pf_rPreferences
		SELECT @TransactUserKey AS UserKey 
			, @TransactUsername AS UserName 
			, @TargetFactoryID	AS FactoryID
			, ProductLineID
			, SettingName 
			, ValueInt 
			, ValueText
		FROM dbo.sx_pf_rPreferences 
		WHERE FactoryID = @SourceFactoryID;

		SET @EffectedRows += @@ROWCOUNT;

		--fStatements CopyInformation
		INSERT INTO dbo.sx_pf_fStatements
		SELECT ProductKey
			, dp.ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, ProductID
			, dp.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, N'COPY'			AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'" / ProductLine <' + dp.ProductlineID + N'>"' + dpl.Nameshort
				+ N'" / Product <' + ProductID + N'>"' + dp.Nameshort + '"' AS Statement
			, @TransactUsername AS UserName 
			, N''				AS PCName
			, N''				AS ProcessorCode
			, N''				AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp 
			, 0					AS IsDeleted 
			, 0					AS IsResolved
		FROM dbo.sx_pf_dProducts dp LEFT JOIN dbo.sx_pf_dProductLines dpl 
			ON dp.ProductLinekey= dpl.ProductLineKey
		WHERE dp.FactoryKey = @TargetFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Right materialization
		EXEC dbo.sx_pf_materialize_vUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a Format
Not yet existing Format is copied if source Format is available

07/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard


DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @SourceFormatID NVARCHAR(255) = 'EingabeGanzzahl'
DECLARE @TargetFormatID NVARCHAR(255) = '88C26'

EXECUTE @RC = dbo.sx_pf_COPY_Format 
	 @Username
	,@SourceFormatID
	,@TargetFormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_Format
GO

CREATE PROCEDURE dbo.sx_pf_COPY_Format
	@Username		NVARCHAR(255),
	@SourceFormatID NVARCHAR(255),
	@TargetFormatID NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFormatKey	INT = 0;
	DECLARE @TargetFormatKey	INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@SourceFormatID,	N'NULL') + N''',''' 
			+ ISNULL(@TargetFormatID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceFormatID	IS NULL SET @SourceFormatID = N'';
	IF @TargetFormatID	IS NULL SET @TargetFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Format;

		-- STEP 0.2 - Protect input parameters			
		SET @Username		= dbo.sx_pf_pProtectString  (@Username);
		SET @SourceFormatID	= dbo.sx_pf_pProtectID		(@SourceFormatID);
		SET @TargetFormatID = dbo.sx_pf_pProtectID		(@TargetFormatID);

		IF  @SourceFormatID		= N'' 
			OR @TargetFormatID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFormatKey = FormatKey FROM dbo.sx_pf_hFormats WHERE FormatID = @SourceFormatID;
		
		IF @SourceFormatKey = 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFormatKey = FormatKey FROM dbo.sx_pf_hFormats WHERE FormatID = @TargetFormatID;
		
		IF @TargetFormatKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hFormats
		INSERT INTO dbo.sx_pf_hFormats
			SELECT @TargetFormatID AS FormatID
				, BackgroundColor
				, FontColor
				, ValueFormat
				, Bold
				, Italic
				, AlignmentHorizontal
				, AlignmentVertical
				, TextUnderlined
				, BorderToNext
				, BorderToPrevious
			FROM dbo.sx_pf_hFormats
			WHERE FormatKey = @SourceFormatKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Format TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a List
Not yet existing list is copied if Sources List is available

07/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL';
DECLARE @SourceListID	NVARCHAR(255) = 'suEvents';
DECLARE @TargetListID	NVARCHAR(255) = '88C26';

EXECUTE @RC = dbo.sx_pf_COPY_List 
	@Username
	,@SourceListID
	,@TargetListID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_List;
GO

CREATE PROCEDURE dbo.sx_pf_COPY_List
	@Username		NVARCHAR(255),
	@SourceListID	NVARCHAR(255),
	@TargetListID	NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceListKey INT = 0;
	DECLARE @TargetListKey INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@SourceListID, N'NULL')	+ N''',''' 
			+ ISNULL(@TargetListID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceListID	IS NULL SET @SourceListID	= N'';
	IF @TargetListID	IS NULL SET @TargetListID	= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_List;

		-- STEP 0.2 - Protect input parameters			
		SET @Username	  =	dbo.sx_pf_pProtectString (@Username);
		SET @SourceListID = dbo.sx_pf_pProtectID	 (@SourceListID);
		SET @TargetListID = dbo.sx_pf_pProtectID	 (@TargetListID);

		IF	@SourceListID	 = N'' 
			OR @TargetListID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceListKey = ListKey 
		FROM dbo.sx_pf_hLists 
		WHERE ListID = @SourceListID;
		
		IF @SourceListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetListKey = ListKey 
		FROM dbo.sx_pf_hLists 
		WHERE ListID = @TargetListID;

		IF @TargetListKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target List key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hLists
		INSERT INTO dbo.sx_pf_hLists
			SELECT @TargetListID AS ListsID
				, NameShort
				, NameLong
				, CommentDev
				, CommentUser
				, Datentyp
				, Source
				, SourceFormula
				, FormatID
			FROM dbo.sx_pf_hLists
			WHERE ListKey = @SourceListKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--hListValues
		INSERT INTO dbo.sx_pf_hListValues 
			SELECT @TargetListID AS ListsID
				, ValueInt
				, Scale
				, ValueText
				, ValueComment
				, FormatID
			FROM dbo.sx_pf_hListValues
			WHERE ListID = @SourceListID;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_List TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a Product
 - creates a new Product from an Template
 - * as Target ProductID is supported, the next free numeric ID is used then
		
05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product => 404 Not Found
5. Not exists target Factory / Productline => 404 Not Found
6. Target Product key already exists => 403 Forbidden
7. Product copied => 201 Created

DECLARE  @RC						INT
DECLARE  @Username					NVARCHAR(255) = 'SQL'
DECLARE  @SourceFactoryID			NVARCHAR(255) = 'WSB'
DECLARE  @SourceProductLineID		NVARCHAR(255) = 'KRED1'
DECLARE  @SourceProductID			NVARCHAR(255) = '0'
DECLARE  @TargetFactoryID			NVARCHAR(255) = 'WSB'
DECLARE  @TargetProductLineID		NVARCHAR(255) = 'KRED1'
DECLARE  @TargetProductID			NVARCHAR(255) = '*'
DECLARE  @TargetProductNameShort	NVARCHAR(255) = 'Test'

EXECUTE @RC = dbo.sx_pf_COPY_Product 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID
		  ,@TargetProductNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_Product;
GO

CREATE PROCEDURE dbo.sx_pf_COPY_Product
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@SourceProductLineID	NVARCHAR(255),
	@SourceProductID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetProductLineID	NVARCHAR(255),
	@TargetProductID		NVARCHAR(255),
	@TargetProductNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
	IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
	IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
	IF @TargetProductNameShort	IS NULL SET @TargetProductNameShort = N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Product;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID	= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID	= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetProductNameShort = dbo.sx_pf_pProtectString	(@TargetProductNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM dbo.sx_pf_dFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM dbo.sx_pf_dProducts WHERE FactoryKey = @SourceFactoryKey AND ProductlineKey = @SourceProductlineKey AND ProductID = @SourceProductID;
		
		IF	@SourceFactoryKey			= 0 
			OR @SourceProductLineKey	= 0 
			OR @SourceProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryKey = @TargetFactoryKey AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		
		IF	@TargetFactoryKey		 = 0 
			OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
		
		-- STEP 1.4 - If ProductID * is submitted, search the next free numeric ProductID
		IF @TargetProductID = '*' 
		BEGIN
			SELECT @TargetProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)), 0) + 1 
			FROM dbo.sx_pf_dProducts  
			WHERE ProductLineKey = @TargetProductLineKey 
				AND TRY_CAST(ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @TargetProductID = '*' 
				SELECT @TargetProductID = 1;
		END

		-- STEP 3 - Copy data		
		--dProduct
		DECLARE @TargetProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

		INSERT INTO dbo.sx_pf_dProducts
		OUTPUT INSERTED.ProductKey INTO @TargetProductKeyTable
		SELECT @TargetProductLineKey	AS ProductLineKey
			, @TargetFactoryKey			AS FactoryKey
			, @TargetProductID			AS ProductID
			, @TargetProductlineID		AS ProductlineID
			, @TargetFactoryID			AS FactoryID
			, TimeType
			, @TargetProductNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1 , GlobalAttribute2 , GlobalAttribute3 , GlobalAttribute4 , GlobalAttribute5 , GlobalAttribute6 , GlobalAttribute7
			, GlobalAttribute8 , GlobalAttribute9 , GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM dbo.sx_pf_dProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductKey = KeyNumber 
		FROM @TargetProductKeyTable;

		--gProducts Inhalt
		INSERT INTO dbo.sx_pf_gProducts
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO dbo.sx_pf_dValueSeries
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM dbo.sx_pf_dValueSeries
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO dbo.sx_pf_dTime
		SELECT
			 @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM dbo.sx_pf_dTime
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO dbo.sx_pf_fValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM dbo.sx_pf_fValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM dbo.sx_pf_dValueSeries 
				WHERE ProductKey = @TargetProductKey) VS 
			ON VS.ValueSeriesID = fV.ValueSeriesID 
		WHERE fV.ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fStatements CopyInformation speichern
		INSERT INTO dbo.sx_pf_fStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
				+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;
		
		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;
			
		COMMIT TRANSACTION sx_pf_COPY_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a ProductLine
Not yet existing ProductLine is copied when Source ProductLine is available

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline => 404 Not Found
5. Not exists target Factory => 404 Not Found
6. Target ProductLine key already exists => 403 Forbidden
7. ProductLine copied => 201 Created

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @SourceFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @SourceProductLineID		NVARCHAR(255) = 'ZT'
DECLARE @TargetFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TargetProductLineID		NVARCHAR(255) = 'ZT32'
DECLARE @TargetProductLineNameShort NVARCHAR(255) = 'Neue ProductLine32'

EXECUTE @RC = dbo.sx_pf_COPY_ProductLine 
		@Username
		,@SourceFactoryID
		,@SourceProductLineID
		,@TargetFactoryID
		,@TargetProductLineID
		,@TargetProductLineNameShort

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_ProductLine;
GO

CREATE PROCEDURE dbo.sx_pf_COPY_ProductLine
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductLineNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey			INT				= 0;
	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					  N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			  N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			  N'NULL') + N''','''
			+ ISNULL(@TargetProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineNameShort, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductLineNameShort	IS NULL SET @TargetProductLineNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID		= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID		= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductLineNameShort = dbo.sx_pf_pProtectString	(@TargetProductLineNameShort);

		IF	@SourceFactoryID				= N'' 
			OR @SourceProductLineID			= N'' 
			OR @TargetFactoryID				= N''
			OR @TargetProductlineID			= N'' 
			OR @TargetProductLineNameShort	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey,@SourceFactoryNameShort = NameShort FROM dbo.sx_pf_dFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey,@SourceProductLineNameShort = Nameshort FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		
		IF	@SourceFactoryKey		 = 0 
			OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Copy data		
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
		
		--dProductLine
		INSERT INTO dbo.sx_pf_dProductLines
		OUTPUT INSERTED.ProductLineKey INTO @Keys([Key]) 
		SELECT @TargetFactoryKey			AS FactoryKey
			, @TargetProductlineID			AS ProductlineID
			, @TargetFactoryID				AS FactoryID
			, @TargetProductLineNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate 
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3  
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6  
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9  	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12  
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15 
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18  
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21  	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24 
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM dbo.sx_pf_dProductLines
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductLineKey = [Key] 
		FROM @Keys;
		
		--dProduct
		INSERT INTO dbo.sx_pf_dProducts 
		SELECT @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		 AS FactoryKey
			, ProductID
			, @TargetProductlineID	 AS ProductlineID
			, @TargetFactoryID		 AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion 
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7  
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14 
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21  
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM dbo.sx_pf_dProducts 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO dbo.sx_pf_dValueSeries 
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dVS.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID 
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM dbo.sx_pf_dValueSeries dVS
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM dbo.sx_pf_dProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dVS.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO dbo.sx_pf_dTime 
		SELECT 
			P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dT.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM dbo.sx_pf_dTime dT
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM dbo.sx_pf_dProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dT.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO dbo.sx_pf_fValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM dbo.sx_pf_fValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID, ProductID 
				FROM dbo.sx_pf_dValueSeries
				WHERE ProductLineKey = @TargetProductLineKey) VS 
			ON fV.ValueSeriesID=VS.ValueSeriesID AND fV.ProductID = VS.ProductID
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		--gProductLines 
		INSERT INTO dbo.sx_pf_gProductLines
		SELECT @TargetProductLineKey AS ProductLineKey 
			, @TargetFactoryKey		 AS FactoryKey
			, @TargetProductlineID	 AS ProductlineID
			, @TargetFactoryID		 AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gProductLines
		WHERE ProductLineKey = @SourceProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO dbo.sx_pf_gProducts  
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, gP.ProductID
			, @TargetProductlineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM dbo.sx_pf_gProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM dbo.sx_pf_dProducts  
				WHERE ProductLineKey = @TargetProductLineKey) P
			ON P.ProductID = gP.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rRights - no definition, write Rights on Factory will lead to implicit Write Rights for creator

		--rPreferences
		INSERT INTO dbo.sx_pf_rPreferences
		SELECT @TransactUserKey		AS UserKey 
			, @TransactUsername		AS UserName
			, @TargetFactoryID		AS FactoryID
			, @TargetProductlineID	AS ProductLineID 
			, SettingName
			, ValueInt
			, ValueText
		FROM dbo.sx_pf_rPreferences
		WHERE FactoryID = @SourceFactoryID 
			AND ProductlineID = @SourceProductlineID;
		
		SET @EffectedRows += @@ROWCOUNT;

		--fStatements CopyInformation
		INSERT INTO dbo.sx_pf_fStatements
		SELECT ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey 
			, ProductID
			, @TargetProductlineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort
				+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted
			, 0						AS IsResolved
		FROM dbo.sx_pf_dProducts 
		WHERE ProductLineKey = @TargetProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Rights materialization 
		EXEC dbo.sx_pf_materialize_vUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryService;
GO
/*
COPY Operation for one ValueSerie
The ValueSerie will be copied and inserted at last Position

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3 .User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product / ValueSerie => 404 Not Found
5. Not exists target Factory / Productline / Product => 404 Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ValueSerie copied => 201 Created

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @SourceFactoryID			NVARCHAR(255) = '230'
DECLARE @SourceProductLineID		NVARCHAR(255) = 'I-ST-IT'
DECLARE @SourceProductID			NVARCHAR(255) = '888'
DECLARE @SourceValueSerieID			NVARCHAR(255) = 'C2'
DECLARE @TargetFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TargetProductLineID		NVARCHAR(255) = 'ZT32'
DECLARE @TargetProductID			NVARCHAR(255) = '1'
DECLARE @TargetValueSerieID			NVARCHAR(255) = '88C2'
DECLARE @TargetValueSerieNameShort  NVARCHAR(255) = 'Kosten Test'

EXECUTE @RC = dbo.sx_pf_COPY_ValueSerie 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@SourceValueSerieID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID
		  ,@TargetValueSerieID
		  ,@TargetValueSerieNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_COPY_ValueSerie;
GO

CREATE PROCEDURE dbo.sx_pf_COPY_ValueSerie
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@SourceProductID			NVARCHAR(255),
	@SourceValueSerieID			NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductID			NVARCHAR(255),
	@TargetValueSerieID			NVARCHAR(255),
	@TargetValueSerieNameShort	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceValueSerieKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceValueSerieNameShort	NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @TargetValueSerieKey		INT				= 0;
	DECLARE @TargetValueSerieNo			INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+ ISNULL(@Username,						N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,				N'NULL') + N''',''' +
			+ ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @SourceProductID				IS NULL SET @SourceProductID			= N'';
	IF @SourceValueSerieID			IS NULL SET @SourceValueSerieID			= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductID				IS NULL SET @TargetProductID			= N'';
	IF @TargetValueSerieID			IS NULL SET @TargetValueSerieID			= N'';
	IF @TargetValueSerieNameShort	IS NULL SET @TargetValueSerieNameShort	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID		= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @SourceProductID			= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @SourceValueSerieID			= dbo.sx_pf_pProtectID		(@SourceValueSerieID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID		= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductID			= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetValueSerieID			= dbo.sx_pf_pProtectID		(@TargetValueSerieID);
		SET @TargetValueSerieNameShort	= dbo.sx_pf_pProtectString	(@TargetValueSerieNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N'' 
			OR @SourceValueSerieID	= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N'' 
			OR @TargetValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername,@SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM dbo.sx_pf_dFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = NameShort FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM dbo.sx_pf_dProducts WHERE ProductlineKey = @SourceProductlineKey AND ProductID= @SourceProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey, @SourceValueSerieNameShort = NameShort FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @SourceProductKey  AND ValueSeriesID= @SourceValueSerieID;	
		
		IF	@SourceFactoryKey = 0 
			OR @SourceProductLineKey = 0 
			OR @SourceProductKey = 0 
			OR @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;
		
		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE  FactoryKey = @TargetFactoryKey  AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @TargetProductKey  AND ValueSeriesID= @TargetValueSerieID;
				
		IF	@TargetFactoryKey = 0 
			OR @TargetProductLineKey = 0 
			OR @TargetProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data 
		SELECT @TargetValueSerieNo = MAX(ValueSeriesNo) + 1 FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @TargetProductKey;
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		INSERT INTO dbo.sx_pf_dValueSeries 
		OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
		SELECT @TargetProductKey			AS ProductKey
			, @TargetProductLineKey			AS ProductLineKey
			, @TargetFactoryKey				AS FactoryKey
			, @TargetProductID				AS ProductID
			, @TargetProductlineID			AS ProductLineID
			, @TargetFactoryID				AS FactoryID
			, @TargetValueSerieID			AS ValueSeriesID
			, @TargetValueSerieNo			AS ValueSeriesNo
			, @TargetValueSerieNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM dbo.sx_pf_dValueSeries 
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetValueSerieKey = [Key] 
		FROM @Keys;
		
		--fValues
		INSERT INTO dbo.sx_pf_fValues  
		SELECT @TargetValueSerieKey AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, @TargetValueSerieID	AS ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM dbo.sx_pf_fValues
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fStatements 
		INSERT INTO dbo.sx_pf_fStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'New ValueSerieID <' + @TargetValueSerieID + N'>"' + @TargetValueSerieNameShort
				+ N'" for product produced as a copy of Factory <' + @SourceFactoryID + N'>"' +  @SourceFactoryNameShort
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' +  @SourceProductLineNameShort
				+ N'"/ Product <' + @SourceProductID + N'>"' +  @SourceProductNameShort
				+ N'"/ ValueSerie <' + @SourceValueSerieID + N'>"' +  @SourceValueSerieNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, ''					AS PCName 
			, ''					AS ProcessorCode 
			, ''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ValueSerie TO pf_PlanningFactoryService;
GO

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_Balance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_Balance]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_Balance (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_Balance] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_Balance] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_Balance] 
   @Username
  ,@FactoryID
  ,@ProductLineID
Print @RC
GO


*/

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_CashValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_CashValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_CashValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_CashValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_CashValues] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_CashValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC
GO


*/

/*
Procedure to deliver dimensional data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_dFactories]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_dFactories]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_dFactories (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
BEGIN
	 BEGIN TRY
		BEGIN TRANSACTION ONE

		DECLARE @TransactUsername AS NVARCHAR (255)
		DECLARE @FactoryKey AS BIGINT
		DECLARE @ProductLineKey AS BIGINT
		DECLARE @RightFlag AS INT

		--<fix log block>
		DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID)
		DECLARE @ParameterString AS NVARCHAR (MAX) = ''				-- SET HERE (all Parameters)
		DECLARE @EffectedRows AS INTEGER = 0						-- SET during Execution
		DECLARE @ResultCode AS INTEGER = 501						-- SET during Execution
		DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP
		DECLARE @Comment AS NVARCHAR (2000) = ''					-- SET during Execution

		SET @ParameterString = '''' + @Username + ''',''' +  @FactoryID +''','''+ @ProductLineID +''
		--<fix log block/>

		-- clean IDs
		SET @Username = dbo.sx_pf_pProtectString(@Username)
		SET @FactoryID = dbo.sx_pf_pProtectID(@FactoryID)
		SET @ProductLineID = dbo.sx_pf_pProtectID(@ProductLineID)

		-- determine Transaction User
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
		IF @TransactUsername  = '403'	BEGIN
											SET @ResultCode = 403
											ROLLBACK TRANSACTION ONE
											EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
											RETURN @ResultCode
										END
		



		-- CASE 1 - Called for the whole cluster - deliver whatever the user gets with his rights
		IF @FactoryID = '' AND @ProductLineID = '' 
			BEGIN
				SELECT 
					  FactoryKey
					 ,dF.FactoryID
					 ,NameShort		AS FactoryNameShort
					 ,dF.FactoryID + ' ' + NameShort AS FactoryIDName
					 ,NameLong		AS FactoryNameLong
					 ,CommentUser	AS FactoryCommentUser
					 ,CommentDev	AS FactoryCommentDev
					 ,ResponsiblePerson AS FactoryResponsiblePerson
					 ,ImageName			AS FactoryImageName

				FROM   sx_pf_dFactories dF

					 INNER JOIN 
							(SELECT FactoryID FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') GROUP BY FactoryID) vUR
							ON dF.FactoryID = vUR.FactoryID
				SET @ResultCode = 200
			END
								
		-- CASE 2 - Called for one Factory - delivier only this if Rights fit
		IF (@FactoryID <> '' AND @ProductLineID = '') OR (@FactoryID <> '' AND @ProductLineID <> '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				IF @FactoryKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(FactoryID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						-- SELECT Values
						SELECT 
							  FactoryKey
							 ,dF.FactoryID
							 ,NameShort		AS FactoryNameShort
							 ,dF.FactoryID + ' ' + NameShort AS FactoryIDName
							 ,NameLong		AS FactoryNameLong
							 ,CommentUser	AS FactoryCommentUser
							 ,CommentDev	AS FactoryCommentDev
							 ,ResponsiblePerson AS FactoryResponsiblePerson
							 ,ImageName			AS FactoryImageName

						FROM   sx_pf_dFactories dF
						WHERE FactoryKey = @FactoryKey
						SET @ResultCode = 200
					END
				ELSE
					SET @ResultCode = 401
			END
		ELSE

		-- CASE 3 - Called for one ProductLine
			-- doesnt matter, always Case 2 as no Productline in dFactories

		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		COMMIT TRANSACTION ONE
		RETURN @ResultCode

	END TRY

	BEGIN CATCH
		PRINT 'Rollback due to not executable command.'	
		ROLLBACK TRANSACTION ONE
		SET @EffectedRows = 0
		SET @ResultCode = 500
		SET @Comment = ERROR_MESSAGE()
		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		RETURN @ResultCode
	END CATCH
END

GO
   
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dFactories] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dFactories] TO pf_PlanningFactoryService;
GO


/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = ''
DECLARE @ProductLineID nvarchar(255) = ''

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_dFactories] 
   @Username
  ,@FactoryID
  ,@ProductLineID

PRINT @RC

TestCases:
	- With empty Factory/PL ID
	- With FactoryID
	- With ProductlineID
	- With unknown User
	- With User without Rights

*/

/*
Procedure to deliver dimensional data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_dProductLines]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_dProductLines]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_dProductLines (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
BEGIN
	 BEGIN TRY
		BEGIN TRANSACTION ONE

		DECLARE @TransactUsername AS NVARCHAR (255)
		DECLARE @FactoryKey AS BIGINT
		DECLARE @ProductLineKey AS BIGINT
		DECLARE @RightFlag AS INT

		--<fix log block>
		DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID)
		DECLARE @ParameterString AS NVARCHAR (MAX) = ''				-- SET HERE (all Parameters)
		DECLARE @EffectedRows AS INTEGER = 0						-- SET during Execution
		DECLARE @ResultCode AS INTEGER = 501						-- SET during Execution
		DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP
		DECLARE @Comment AS NVARCHAR (2000) = ''					-- SET during Execution

		SET @ParameterString = '''' + @Username + ''',''' +  @FactoryID +''','''+ @ProductLineID +''
		--<fix log block/>

		-- clean IDs
		SET @Username = dbo.sx_pf_pProtectString(@Username)
		SET @FactoryID = dbo.sx_pf_pProtectID(@FactoryID)
		SET @ProductLineID = dbo.sx_pf_pProtectID(@ProductLineID)

		-- determine Transaction User
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
		IF @TransactUsername  = '403'	BEGIN
											SET @ResultCode = 403
											ROLLBACK TRANSACTION ONE
											EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
											RETURN @ResultCode
										END
		
		-- CASE 1 - Called for the whole cluster - deliver whatever the user gets with his rights
		IF @FactoryID = '' AND @ProductLineID = '' 
			BEGIN
				SELECT 
					    ProductLineKey
					   ,dPL.ProductLineID
					   ,dPL.FactoryID
					   ,NameShort	AS ProductLineNameShort
					   ,dPL.ProductLineID + ' ' + NameShort AS ProductLineIDName
					   ,NameLong	AS ProductLineNameLong
					   ,CommentUser AS ProductLineCommentUser
					   ,CommentDev			AS ProductLineCommentDev
					   ,ResponsiblePerson	AS ProductLineResponsiblePerson
					   ,ImageName			AS ProductLineImageName
					   ,DefaultTemplate
					   ,GlobalAttributeAlias1
					   ,GlobalAttributeAlias2
					   ,GlobalAttributeAlias3
					   ,GlobalAttributeAlias4
					   ,GlobalAttributeAlias5
					   ,GlobalAttributeAlias6
					   ,GlobalAttributeAlias7
					   ,GlobalAttributeAlias8
					   ,GlobalAttributeAlias9
					   ,GlobalAttributeAlias10
					   ,GlobalAttributeAlias11
					   ,GlobalAttributeAlias12
					   ,GlobalAttributeAlias13
					   ,GlobalAttributeAlias14
					   ,GlobalAttributeAlias15
					   ,GlobalAttributeAlias16
					   ,GlobalAttributeAlias17
					   ,GlobalAttributeAlias18
					   ,GlobalAttributeAlias19
					   ,GlobalAttributeAlias20
					   ,GlobalAttributeAlias21
					   ,GlobalAttributeAlias22
					   ,GlobalAttributeAlias23
					   ,GlobalAttributeAlias24
					   ,GlobalAttributeAlias25
				FROM   sx_pf_dProductLines dPL

						INNER JOIN 
							(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
							ON
								dPL.FactoryID = vUR.FactoryID AND
								dPL.ProductLineID = vUR.ProductLineID

				SET @ResultCode = 200
			END
								
		-- CASE 2 - Called for one Factory - delivier only this if Rights fit
		IF (@FactoryID <> '' AND @ProductLineID = '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				IF @FactoryKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(FactoryID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						
						SELECT 
					    ProductLineKey
					   ,ProductLineID
					   ,FactoryID
					   ,NameShort	AS ProductLineNameShort
					   ,ProductLineID + ' ' + NameShort AS ProductLineIDName
					   ,NameLong	AS ProductLineNameLong
					   ,CommentUser AS ProductLineCommentUser
					   ,CommentDev			AS ProductLineCommentDev
					   ,ResponsiblePerson	AS ProductLineResponsiblePerson
					   ,ImageName			AS ProductLineImageName
					   ,DefaultTemplate
					   ,GlobalAttributeAlias1
					   ,GlobalAttributeAlias2
					   ,GlobalAttributeAlias3
					   ,GlobalAttributeAlias4
					   ,GlobalAttributeAlias5
					   ,GlobalAttributeAlias6
					   ,GlobalAttributeAlias7
					   ,GlobalAttributeAlias8
					   ,GlobalAttributeAlias9
					   ,GlobalAttributeAlias10
					   ,GlobalAttributeAlias11
					   ,GlobalAttributeAlias12
					   ,GlobalAttributeAlias13
					   ,GlobalAttributeAlias14
					   ,GlobalAttributeAlias15
					   ,GlobalAttributeAlias16
					   ,GlobalAttributeAlias17
					   ,GlobalAttributeAlias18
					   ,GlobalAttributeAlias19
					   ,GlobalAttributeAlias20
					   ,GlobalAttributeAlias21
					   ,GlobalAttributeAlias22
					   ,GlobalAttributeAlias23
					   ,GlobalAttributeAlias24
					   ,GlobalAttributeAlias25
						FROM   sx_pf_dProductLines

						WHERE FactoryKey = @FactoryKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		-- CASE 3 - Called for one ProductLine
		IF (@FactoryID <> '' AND @ProductLineID <> '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				SELECT @ProductLineKey = ProductLineKey FROM sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID
				IF @FactoryKey IS NULL OR @ProductLineKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(ProductLineID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						SELECT 
					    ProductLineKey
					   ,ProductLineID
					   ,FactoryID
					   ,NameShort	AS ProductLineNameShort
					   ,ProductLineID + ' ' + NameShort AS ProductLineIDName
					   ,NameLong	AS ProductLineNameLong
					   ,CommentUser AS ProductLineCommentUser
					   ,CommentDev			AS ProductLineCommentDev
					   ,ResponsiblePerson	AS ProductLineResponsiblePerson
					   ,ImageName			AS ProductLineImageName
					   ,DefaultTemplate
					   ,GlobalAttributeAlias1
					   ,GlobalAttributeAlias2
					   ,GlobalAttributeAlias3
					   ,GlobalAttributeAlias4
					   ,GlobalAttributeAlias5
					   ,GlobalAttributeAlias6
					   ,GlobalAttributeAlias7
					   ,GlobalAttributeAlias8
					   ,GlobalAttributeAlias9
					   ,GlobalAttributeAlias10
					   ,GlobalAttributeAlias11
					   ,GlobalAttributeAlias12
					   ,GlobalAttributeAlias13
					   ,GlobalAttributeAlias14
					   ,GlobalAttributeAlias15
					   ,GlobalAttributeAlias16
					   ,GlobalAttributeAlias17
					   ,GlobalAttributeAlias18
					   ,GlobalAttributeAlias19
					   ,GlobalAttributeAlias20
					   ,GlobalAttributeAlias21
					   ,GlobalAttributeAlias22
					   ,GlobalAttributeAlias23
					   ,GlobalAttributeAlias24
					   ,GlobalAttributeAlias25
						FROM   sx_pf_dProductLines

						WHERE ProductLineKey = @ProductLineKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		COMMIT TRANSACTION ONE
		RETURN @ResultCode

	END TRY

	BEGIN CATCH
		PRINT 'Rollback due to not executable command.'	
		ROLLBACK TRANSACTION ONE
		SET @EffectedRows = 0
		SET @ResultCode = 500
		SET @Comment = ERROR_MESSAGE()
		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		RETURN @ResultCode
	END CATCH
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dProductLines] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dProductLines] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = ''
DECLARE @ProductLineID nvarchar(255) = ''

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_dProductLines] 
   @Username
  ,@FactoryID
  ,@ProductLineID

PRINT @RC

TestCases:
	- With empty Factory/PL ID
	- With FactoryID
	- With ProductlineID
	- With unknown User
	- With User without Rights

*/

/*
Procedure to deliver dimensional data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_dProducts]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_dProducts]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_dProducts (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
BEGIN
	 BEGIN TRY
		BEGIN TRANSACTION ONE

		DECLARE @TransactUsername AS NVARCHAR (255)
		DECLARE @FactoryKey AS BIGINT
		DECLARE @ProductLineKey AS BIGINT
		DECLARE @RightFlag AS INT

		--<fix log block>
		DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID)
		DECLARE @ParameterString AS NVARCHAR (MAX) = ''				-- SET HERE (all Parameters)
		DECLARE @EffectedRows AS INTEGER = 0						-- SET during Execution
		DECLARE @ResultCode AS INTEGER = 501						-- SET during Execution
		DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP
		DECLARE @Comment AS NVARCHAR (2000) = ''					-- SET during Execution

		SET @ParameterString = '''' + @Username + ''',''' +  @FactoryID +''','''+ @ProductLineID +''
		--<fix log block/>

		-- clean IDs
		SET @Username = dbo.sx_pf_pProtectString(@Username)
		SET @FactoryID = dbo.sx_pf_pProtectID(@FactoryID)
		SET @ProductLineID = dbo.sx_pf_pProtectID(@ProductLineID)

		-- determine Transaction User
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
		IF @TransactUsername  = '403'	BEGIN
											SET @ResultCode = 403
											ROLLBACK TRANSACTION ONE
											EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
											RETURN @ResultCode
										END
		
		-- CASE 1 - Called for the whole cluster - deliver whatever the user gets with his rights
		IF @FactoryID = '' AND @ProductLineID = '' 
			BEGIN
				SELECT 
					    ProductKey
					   ,ProductID
					   ,dP.ProductLineID
					   ,dP.FactoryID
					   ,TimeType
					   ,NameShort	AS ProductNameShort
					   ,ProductID + ' ' + NameShort AS ProductIDName
					   ,NameLong	AS ProductNameLong
					   ,CommentUser	AS ProductCommentUser
					   ,CommentDev	AS ProductCommentDev
					   ,ResponsiblePerson	AS ProductResponsiblePerson
					   ,ImageName			AS ProductImageName
					   ,[Status]
					   ,Template
					   ,TemplateVersion
					   ,GlobalAttribute1
					   ,GlobalAttribute2
					   ,GlobalAttribute3
					   ,GlobalAttribute4
					   ,GlobalAttribute5
					   ,GlobalAttribute6
					   ,GlobalAttribute7
					   ,GlobalAttribute8
					   ,GlobalAttribute9
					   ,GlobalAttribute10
					   ,GlobalAttribute11
					   ,GlobalAttribute12
					   ,GlobalAttribute13
					   ,GlobalAttribute14
					   ,GlobalAttribute15
					   ,GlobalAttribute16
					   ,GlobalAttribute17
					   ,GlobalAttribute18
					   ,GlobalAttribute19
					   ,GlobalAttribute20
					   ,GlobalAttribute21
					   ,GlobalAttribute22
					   ,GlobalAttribute23
					   ,GlobalAttribute24
					   ,GlobalAttribute25
				FROM   sx_pf_dProducts dP

						INNER JOIN 
							(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
							ON
								dP.FactoryID = vUR.FactoryID AND
								dP.ProductLineID = vUR.ProductLineID

				SET @ResultCode = 200
			END
								
		-- CASE 2 - Called for one Factory - delivier only this if Rights fit
		IF (@FactoryID <> '' AND @ProductLineID = '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				IF @FactoryKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(FactoryID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						
						SELECT 
							ProductKey
						   ,ProductID
						   ,ProductLineID
						   ,FactoryID
						   ,TimeType
						   ,NameShort	AS ProductNameShort
						   ,ProductID + ' ' + NameShort AS ProductIDName
						   ,NameLong	AS ProductNameLong
						   ,CommentUser	AS ProductCommentUser
						   ,CommentDev	AS ProductCommentDev
						   ,ResponsiblePerson	AS ProductResponsiblePerson
						   ,ImageName			AS ProductImageName
						   ,[Status]
						   ,Template
						   ,TemplateVersion
						   ,GlobalAttribute1
						   ,GlobalAttribute2
						   ,GlobalAttribute3
						   ,GlobalAttribute4
						   ,GlobalAttribute5
						   ,GlobalAttribute6
						   ,GlobalAttribute7
						   ,GlobalAttribute8
						   ,GlobalAttribute9
						   ,GlobalAttribute10
						   ,GlobalAttribute11
						   ,GlobalAttribute12
						   ,GlobalAttribute13
						   ,GlobalAttribute14
						   ,GlobalAttribute15
						   ,GlobalAttribute16
						   ,GlobalAttribute17
						   ,GlobalAttribute18
						   ,GlobalAttribute19
						   ,GlobalAttribute20
						   ,GlobalAttribute21
						   ,GlobalAttribute22
						   ,GlobalAttribute23
						   ,GlobalAttribute24
						   ,GlobalAttribute25
						FROM   sx_pf_dProducts

						WHERE FactoryKey = @FactoryKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		-- CASE 3 - Called for one ProductLine
		IF (@FactoryID <> '' AND @ProductLineID <> '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				SELECT @ProductLineKey = ProductLineKey FROM sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID
				IF @FactoryKey IS NULL OR @ProductLineKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(ProductLineID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						SELECT 
						    ProductKey
						   ,ProductID
						   ,ProductLineID
						   ,FactoryID
						   ,TimeType
						   ,NameShort	AS ProductNameShort
						   ,ProductID + ' ' + NameShort AS ProductIDName
						   ,NameLong	AS ProductNameLong
						   ,CommentUser	AS ProductCommentUser
						   ,CommentDev	AS ProductCommentDev
						   ,ResponsiblePerson	AS ProductResponsiblePerson
						   ,ImageName			AS ProductImageName
						   ,[Status]
						   ,Template
						   ,TemplateVersion
						   ,GlobalAttribute1
						   ,GlobalAttribute2
						   ,GlobalAttribute3
						   ,GlobalAttribute4
						   ,GlobalAttribute5
						   ,GlobalAttribute6
						   ,GlobalAttribute7
						   ,GlobalAttribute8
						   ,GlobalAttribute9
						   ,GlobalAttribute10
						   ,GlobalAttribute11
						   ,GlobalAttribute12
						   ,GlobalAttribute13
						   ,GlobalAttribute14
						   ,GlobalAttribute15
						   ,GlobalAttribute16
						   ,GlobalAttribute17
						   ,GlobalAttribute18
						   ,GlobalAttribute19
						   ,GlobalAttribute20
						   ,GlobalAttribute21
						   ,GlobalAttribute22
						   ,GlobalAttribute23
						   ,GlobalAttribute24
						   ,GlobalAttribute25
						FROM   sx_pf_dProducts

						WHERE ProductLineKey = @ProductLineKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		COMMIT TRANSACTION ONE
		RETURN @ResultCode

	END TRY

	BEGIN CATCH
		PRINT 'Rollback due to not executable command.'	
		ROLLBACK TRANSACTION ONE
		SET @EffectedRows = 0
		SET @ResultCode = 500
		SET @Comment = ERROR_MESSAGE()
		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		RETURN @ResultCode
	END CATCH
END

GO
   
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dProducts] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dProducts] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_dProducts] 
   @Username
  ,@FactoryID
  ,@ProductLineID

PRINT @RC

TestCases:
	- With empty Factory/PL ID
	- With FactoryID
	- With ProductlineID
	- With unknown User
	- With User without Rights

*/

/*
Procedure to deliver dimensional data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_dValueSeries]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_dValueSeries]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_dValueSeries (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
BEGIN
	 BEGIN TRY
		BEGIN TRANSACTION ONE

		DECLARE @TransactUsername AS NVARCHAR (255)
		DECLARE @FactoryKey AS BIGINT
		DECLARE @ProductLineKey AS BIGINT
		DECLARE @RightFlag AS INT

		--<fix log block>
		DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID)
		DECLARE @ParameterString AS NVARCHAR (MAX) = ''				-- SET HERE (all Parameters)
		DECLARE @EffectedRows AS INTEGER = 0						-- SET during Execution
		DECLARE @ResultCode AS INTEGER = 501						-- SET during Execution
		DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP
		DECLARE @Comment AS NVARCHAR (2000) = ''					-- SET during Execution

		SET @ParameterString = '''' + @Username + ''',''' +  @FactoryID +''','''+ @ProductLineID +''
		--<fix log block/>

		-- clean IDs
		SET @Username = dbo.sx_pf_pProtectString(@Username)
		SET @FactoryID = dbo.sx_pf_pProtectID(@FactoryID)
		SET @ProductLineID = dbo.sx_pf_pProtectID(@ProductLineID)

		-- determine Transaction User
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
		IF @TransactUsername  = '403'	BEGIN
											SET @ResultCode = 403
											ROLLBACK TRANSACTION ONE
											EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
											RETURN @ResultCode
										END
		
		-- CASE 1 - Called for the whole cluster - deliver whatever the user gets with his rights
		IF @FactoryID = '' AND @ProductLineID = '' 
			BEGIN
				SELECT 
					 ValueSeriesKey
					,ProductID
					,dVS.ProductLineID
					,dVS.FactoryID
					,ValueSeriesID
					,ValueSeriesNo
					,NameShort AS ValueSeriesNameShort
					,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
					,NameLong AS ValueSeriesNameLong
					,CommentUser AS ValueSeriesCommentUser
					,CommentDev AS ValueSeriesCommentDev
					,ImageName AS ValueSeriesImageName
					,[IsNumeric]
					,VisibilityLevel
					,ValueSource
					,ValueListID
					,ValueFormatID
					,Unit
					,Scale
					,Effect
					,EffectParameter
				FROM   sx_pf_dValueSeries dVS 

						INNER JOIN 
							(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
							ON
								dVS.FactoryID = vUR.FactoryID AND
								dVS.ProductLineID = vUR.ProductLineID

				SET @ResultCode = 200
			END
								
		-- CASE 2 - Called for one Factory - delivier only this if Rights fit
		IF (@FactoryID <> '' AND @ProductLineID = '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				IF @FactoryKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(FactoryID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						
						SELECT
							 ValueSeriesKey
							,ProductID
							,ProductLineID
							,FactoryID
							,ValueSeriesID
							,ValueSeriesNo
							,NameShort AS ValueSeriesNameShort
							,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
							,NameLong AS ValueSeriesNameLong
							,CommentUser AS ValueSeriesCommentUser
							,CommentDev AS ValueSeriesCommentDev
							,ImageName AS ValueSeriesImageName
							,[IsNumeric]
							,VisibilityLevel
							,ValueSource
							,ValueListID
							,ValueFormatID
							,Unit
							,Scale
							,Effect
							,EffectParameter
						FROM   sx_pf_dValueSeries

						WHERE FactoryKey = @FactoryKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		-- CASE 3 - Called for one ProductLine
		IF (@FactoryID <> '' AND @ProductLineID <> '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				SELECT @ProductLineKey = ProductLineKey FROM sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID
				IF @FactoryKey IS NULL OR @ProductLineKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(ProductLineID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						SELECT 
							 ValueSeriesKey
							,ProductID
							,ProductLineID
							,FactoryID
							,ValueSeriesID
							,ValueSeriesNo
							,NameShort AS ValueSeriesNameShort
							,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
							,NameLong AS ValueSeriesNameLong
							,CommentUser AS ValueSeriesCommentUser
							,CommentDev AS ValueSeriesCommentDev
							,ImageName AS ValueSeriesImageName
							,[IsNumeric]
							,VisibilityLevel
							,ValueSource
							,ValueListID
							,ValueFormatID
							,Unit
							,Scale
							,Effect
							,EffectParameter
						FROM   sx_pf_dValueSeries

						WHERE ProductLineKey = @ProductLineKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		COMMIT TRANSACTION ONE
		RETURN @ResultCode

	END TRY

	BEGIN CATCH
		PRINT 'Rollback due to not executable command.'	
		ROLLBACK TRANSACTION ONE
		SET @EffectedRows = 0
		SET @ResultCode = 500
		SET @Comment = ERROR_MESSAGE()
		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		RETURN @ResultCode
	END CATCH
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dValueSeries] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_dValueSeries] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_dValueSeries] 
   @Username
  ,@FactoryID
  ,@ProductLineID

PRINT @RC

TestCases:
	- With empty Factory/PL ID
	- With FactoryID
	- With ProductlineID
	- With unknown User
	- With User without Rights

*/

/*
Procedure to deliver analytic data as factory summary
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_FactorySummary]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_FactorySummary]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_FactorySummary (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END
	SELECT 

		  TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month

         ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect) AS Cash
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect) AS Profit
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect) AS Balance
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.QuantityEffect) AS Quantity
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1)) AS Value

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName


  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		dF.FactoryID = @FactoryID

  GROUP BY

  		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100																

		 ,fV.ProductLineID + ' ' + dPL.NameShort	
         ,dPL.NameShort												

		 ,fV.ProductID + ' ' + dP.NameShort			
         ,dP.NameShort												
	

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_FactorySummary] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_FactorySummary] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = 'PE'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_FactorySummary] 
   @Username
  ,@FactoryID

Print @RC
GO


*/

/*
Procedure to deliver dimensional data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_fValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_fValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_fValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
BEGIN
	 BEGIN TRY
		BEGIN TRANSACTION ONE

		DECLARE @TransactUsername AS NVARCHAR (255)
		DECLARE @FactoryKey AS BIGINT
		DECLARE @ProductLineKey AS BIGINT
		DECLARE @RightFlag AS INT

		--<fix log block>
		DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID)
		DECLARE @ParameterString AS NVARCHAR (MAX) = ''				-- SET HERE (all Parameters)
		DECLARE @EffectedRows AS INTEGER = 0						-- SET during Execution
		DECLARE @ResultCode AS INTEGER = 501						-- SET during Execution
		DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP
		DECLARE @Comment AS NVARCHAR (2000) = ''					-- SET during Execution

		SET @ParameterString = '''' + @Username + ''',''' +  @FactoryID +''','''+ @ProductLineID +''
		--<fix log block/>

		-- clean IDs
		SET @Username = dbo.sx_pf_pProtectString(@Username)
		SET @FactoryID = dbo.sx_pf_pProtectID(@FactoryID)
		SET @ProductLineID = dbo.sx_pf_pProtectID(@ProductLineID)

		-- determine Transaction User
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
		IF @TransactUsername  = '403'	BEGIN
											SET @ResultCode = 403
											ROLLBACK TRANSACTION ONE
											EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
											RETURN @ResultCode
										END
		
		-- CASE 1 - Called for the whole cluster - deliver whatever the user gets with his rights
		IF @FactoryID = '' AND @ProductLineID = '' 
			BEGIN
				SELECT
					  fV.ProductKey
					 ,fV.ProductLineKey
					 ,fV.FactoryKey
					 ,fV.ValueSeriesKey
					 ,fV.ProductID
					 ,fV.ProductLineID
					 ,fV.FactoryID
					 ,fV.ValueSeriesID
					 ,TimeID
					 ,CAST(LEFT(TimeID,4) AS int) AS [Year]
					 ,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Monath]
					 ,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
					 ,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
					 ,ValueFormula
					 ,CAST(ValueInt AS Money) / COALESCE(dVS.Scale,1) AS Value
					 ,ValueText
					 ,ValueComment
					
				FROM   sx_pf_fValues fV

						LEFT JOIN
							sx_pf_dValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey

						INNER JOIN 
							(SELECT FactoryID,ProductLineID FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
							ON
								fV.FactoryID = vUR.FactoryID AND
								fV.ProductLineID = vUR.ProductLineID

				SET @ResultCode = 200
			END
								
		-- CASE 2 - Called for one Factory - delivier only this if Rights fit
		IF (@FactoryID <> '' AND @ProductLineID = '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				IF @FactoryKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(FactoryID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						
						SELECT 
						  fV.ProductKey
						 ,fV.ProductLineKey
						 ,fV.FactoryKey
						 ,fV.ValueSeriesKey
						 ,fV.ProductID
						 ,fV.ProductLineID
						 ,fV.FactoryID
						 ,fV.ValueSeriesID
						 ,TimeID
						 ,CAST(LEFT(TimeID,4) AS int) AS [Year]
						 ,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Monath]
						 ,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
						 ,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
						 ,ValueFormula
						 ,CAST(ValueInt AS Money) / COALESCE(dVS.Scale,1) AS Value
						 ,ValueText
						 ,ValueComment
					
						FROM   sx_pf_fValues fV 

						LEFT JOIN
							sx_pf_dValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey

						WHERE fV.FactoryKey = @FactoryKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		-- CASE 3 - Called for one ProductLine
		IF (@FactoryID <> '' AND @ProductLineID <> '')
			BEGIN
				-- 3. If Factory not exists break
				SELECT @FactoryKey = FactoryKey FROM sx_pf_dFactories WHERE FactoryID = @FactoryID
				SELECT @ProductLineKey = ProductLineKey FROM sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID
				IF @FactoryKey IS NULL OR @ProductLineKey IS NULL
									BEGIN
										SET @ResultCode = 404
										ROLLBACK TRANSACTION ONE
										EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
										RETURN @ResultCode
									END
				-- Check Rights
				SELECT @RightFlag = Count(ProductLineID) FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND [Right] IN ('Write','Read')
				IF @RightFlag > 0
					BEGIN
						SELECT
							  fV.ProductKey
							 ,fV.ProductLineKey
							 ,fV.FactoryKey
							 ,fV.ValueSeriesKey
							 ,fV.ProductID
							 ,fV.ProductLineID
							 ,fV.FactoryID
							 ,fV.ValueSeriesID
							 ,TimeID
							 ,CAST(LEFT(TimeID,4) AS int) AS [Year]
							 ,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Monath]
							 ,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
							 ,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
							 ,ValueFormula
							 ,CAST(ValueInt AS Money) / COALESCE(dVS.Scale,1) AS Value
							 ,ValueText
							 ,ValueComment
					
						FROM   sx_pf_fValues fV
						 
							LEFT JOIN
								sx_pf_dValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey


						WHERE fV.ProductLineKey = @ProductLineKey

						SET @ResultCode = 200
					END
				ELSE 
					SET @ResultCode = 401 -- Rechte waren nicht da
			END

		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		COMMIT TRANSACTION ONE
		RETURN @ResultCode

	END TRY

	BEGIN CATCH
		PRINT 'Rollback due to not executable command.'	
		ROLLBACK TRANSACTION ONE
		SET @EffectedRows = 0
		SET @ResultCode = 500
		SET @Comment = ERROR_MESSAGE()
		EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
		RETURN @ResultCode
	END CATCH
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_fValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_fValues] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '2'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_fValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID

PRINT @RC

TestCases:
	- With empty Factory/PL ID
	- With FactoryID
	- With ProductlineID
	- With unknown User
	- With User without Rights

*/

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_NumericValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_NumericValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_NumericValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	 
	 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 

IF @FactoryID <> '' AND @ProductLineID = ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID


SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_NumericValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_NumericValues] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = ''

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_NumericValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID
GO


*/

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_Profit]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_Profit]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_Profit (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
			
	SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN sx_pf_gValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_Profit] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_Profit] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_Profit] 
   @Username
  ,@FactoryID
  ,@ProductLineID
GO


*/

/*
Procedure to deliver analytic data to the enduser
PlanningFactory 4.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sx_pf_DATAOUTPUT_TextValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sx_pf_DATAOUTPUT_TextValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_TextValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0


IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   sx_pf_fValues fV
         LEFT JOIN sx_pf_dValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN sx_pf_dProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN sx_pf_dProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN sx_pf_dFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM sx_pf_vUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID 

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
   
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_TextValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_DATAOUTPUT_TextValues] TO pf_PlanningFactoryService;
GO


/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [dbo].[sx_pf_DATAOUTPUT_TextValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC
GO


*/
/*
DELETE Operation for a ClusterProperty
Deletes a Cluster Property.

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Property deleted => 200 OK
5. PropertyID not found => 204 No Content

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @PropertyID			NVARCHAR(255) = 'rre'

EXECUTE @RC = dbo.sx_pf_DELETE_ClusterProperty 
		@Username,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ClusterProperty;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ClusterProperty;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);
		
		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete
		DELETE FROM dbo.sx_pf_gCluster 
		WHERE PropertyID = @PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Factory
Deletes an entire Factory (include ProductLines, Products, Values)

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. FactoryID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Factory deleted => 200 OK
5. FactoryID not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FactoryID	NVARCHAR(255) = 'Test'

EXECUTE @RC = dbo.sx_pf_DELETE_Factory 
		@Username,
		@FactoryID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Factory;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Factory
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Factory;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID; 

		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 204;
		END
		ELSE
		BEGIN
			-- STEP 3 - Delete Factory items from all tables
			DELETE FROM dbo.sx_pf_dProductLines 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_dProducts 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_dValueSeries 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_dTime 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_fStatements 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_fValues 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_gFactories
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_gProductLines 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_gProducts 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_dFactories 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Last, because of Right materialization
			DELETE FROM dbo.sx_pf_rRights 
			WHERE FactoryID = @FactoryID;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION sx_pf_DELETE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Factory TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a FactoryProperty
Deletes an entire Factory Property

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID or FactoryID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the Factory => 401 Unauthorized
4. Property deleted => 200 OK
5. PropertyID not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FactoryID	NVARCHAR(255) = 'ZT'
DECLARE @PropertyID NVARCHAR(255) = 'rre'

EXECUTE @RC = dbo.sx_pf_DELETE_FactoryProperty 
		@Username,
		@FactoryID,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_FactoryProperty;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @PropertyKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID = N'' 
			OR @PropertyID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID; 
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete		
		DELETE FROM dbo.sx_pf_gFactories 
		WHERE PropertyKey = @PropertyKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Format

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. FormatID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Format deleted => 200 OK
5. Format not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FormatID	NVARCHAR(255) = '510'
		
EXECUTE @RC = dbo.sx_pf_DELETE_Format 
		@Username,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Format;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username, N'NULL') + N''',''' 
			+ ISNULL(@FormatID, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM dbo.sx_pf_hFormats 
		WHERE FormatID = @FormatID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Format TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a List
Delete a List 

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ListID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. List deleted => 200 OK
5. List not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @ListID		NVARCHAR(255) = '510'
		
EXECUTE @RC = dbo.sx_pf_DELETE_List 
		@Username,
		@ListID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_List;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM dbo.sx_pf_hListValues 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_hLists 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;
			
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_List TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for one ListValue
Operates over Key, as the ListValue has no ID

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ListID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. ListValue deleted => 200 OK
5. ListValue not found => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @ListValueKey	INT				= 18

		
EXECUTE @RC = dbo.sx_pf_DELETE_ListValue 
		@Username,
		@ListValueKey

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ListValue') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ListValue;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ListValue
	@Username		NVARCHAR(255),
	@ListValueKey	INT
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	 IS NULL SET @Username		= N'';
	IF @ListValueKey IS NULL SET @ListValueKey	= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ListValue;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_hListValues WHERE ListValueKey = @ListValueKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ListValue TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ListValue TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Preference

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID or empty ProductLine => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Non exists Factory / Productline => 404 Not Found
5. Product not found => 204 No Content
6. Product deleted => 200 OK

DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) = 'SQL'
DECLARE @PreferenceUserName		NVARCHAR(255) = 'W8\admin'
DECLARE @ProductLineID			NVARCHAR(255) = 'ZT'
DECLARE @FactoryID				NVARCHAR(255) = 'U'
DECLARE @SettingName			NVARCHAR(255) = 'Demopref'

EXECUTE @RC = dbo.sx_pf_DELETE_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Preference;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@PreferenceUsername,	N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SettingName,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Preference;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername		= dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName			= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @SettingName		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		IF @SettingName = N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid value of Setting Name', 16, 10);
		END;

		IF NOT EXISTS(SELECT Username FROM dbo.sx_pf_rUser WHERE Username = @PreferenceUsername)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Preferences Users don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_rPreferences
		WHERE FactoryID			= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Preference TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Product
Deletes a product and all Values and Attributes

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ProductID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Product not found => 204 No Content
5. Product deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @ProductID		NVARCHAR(255) = '10'

EXECUTE @RC = dbo.sx_pf_DELETE_Product 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Product;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF @ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_dValueSeries 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dTime 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_fStatements 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_fValues 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_gProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Product TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for all Values of the ProductDateTabele

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

DECLARE @RC							INT
		DECLARE @Username			NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID		NVARCHAR(255) = 'U'
		DECLARE @ProductID			NVARCHAR(255) = '1'
		DECLARE @DeleteFormulaFlag	INT = 0

EXECUTE @RC = dbo.sx_pf_DELETE_ProductDataTableValues 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@DeleteFormulaFlag

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_DELETE_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ProductDataTableValues;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductDataTableValues
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@DeleteFormulaFlag	INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username, N'NULL')								+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')								+ N''',''' 
								+ CAST(ISNULL(@DeleteFormulaFlag, N'NULL') AS NVARCHAR(10))	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION DELETE_PDTValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine ProductKey
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID	= @ProductID;

		IF @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - DELETE
		-- If also formulas, we can delete everything
		IF @DeleteFormulaFlag = 1
		BEGIN
			DELETE FROM dbo.sx_pf_fValues 
			WHERE ProductKey = @ProductKey;

			SET @EffectedRows = @@ROWCOUNT;
		END;
		-- If not formulas, only Input Values and update Formula Values to 0/''
		IF @DeleteFormulaFlag = 0
		BEGIN
			-- alle Input Werte komplett
			DELETE fV
			FROM dbo.sx_pf_fValues fV
				LEFT JOIN dbo.sx_pf_dValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey
				AND dVS.ValueSource IN ('Input','List-Strict','Link');

			-- bei Formelbasierten Werten nur den Wert, nicht die Formel
			UPDATE fV
				SET fV.ValueInt = 0
				,fV.ValueText = ''
			FROM dbo.sx_pf_fValues fV
				LEFT JOIN dbo.sx_pf_dValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey	= @ProductKey
				AND dVS.ValueSource IN ('XLS', 'XLS-Strict');

			SET @EffectedRows = @@ROWCOUNT;
		END;

		SET @ResultCode	= IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION DELETE_PDTValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT	= ERROR_STATE();
		SET @Comment				= ERROR_MESSAGE();

		ROLLBACK TRANSACTION DELETE_PDTValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductLine
Deletes an entire Product Line (Products, Values, etc.)

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ProductLineID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. ProductLine not found => 204 No Content
5. ProductLine deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'V'

EXECUTE @RC = dbo.sx_pf_DELETE_ProductLine 
		@Username,
		@FactoryID,
		@ProductLineID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ProductLine;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductLine
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLine;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF	@ProductLineID	= N'' 
			OR @FactoryID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- 4. Factory und ProductLineKey ermitteln
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0
 		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_dProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dValueSeries 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dTime 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_fStatements 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_fValues 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_gProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_gProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- last because of matrialization
		DELETE FROM dbo.sx_pf_rRights 
		WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductLine TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductLineProperty
Delete a ProductLineProperty

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Property not found => 204 No Content
5. Property deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @PropertyID		NVARCHAR(255) = 'test'

EXECUTE @RC = dbo.sx_pf_DELETE_ProductLineProperty 
		@Username,
		@FactoryID,
		@ProductLineID,
		@PropertyID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ProductLineProperty;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + '''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLineProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gProductLines WHERE ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_gProductLines 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductProperty
Delete a ProductProperty

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Property not found => 204 No Content
5. Property deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @ProductID		NVARCHAR(255) = '10'
DECLARE @PropertyID		NVARCHAR(255) = 'test'

EXECUTE @RC = sx_pf_DELETE_ProductProperty 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ProductProperty;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gProducts WHERE ProductKey = @ProductKey AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_gProducts 
		WHERE PropertyKey = @PropertyKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Right
Deletes a Right of a user, user can therefore also delete its own cluster write and lock yourself out for yourself to! 
Must frontend intercept with security inquiry!

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. DeleteUserName is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Right deleted => 200 OK
5. Right not found => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @DeleteUserName NVARCHAR(255) = 'test2'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		
EXECUTE @RC = dbo.sx_pf_DELETE_Right 
		@Username,
		@DeleteUserName,
		@FactoryID,
        @ProductLineID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Right') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Right;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Right
	@Username			NVARCHAR(255),
	@DeleteUserName		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
    @ProductLineID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @DeleteUserKey	INT = 0;
	DECLARE @RightKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Right;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString	(@DeleteUserName);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF  @DeleteUserName		= N'' 
			OR @FactoryID		= N'' 
			OR @ProductLineID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- STEP 1.2 - Determine keys
		SELECT @DeleteUserKey = UserKey FROM dbo.sx_pf_rUser WHERE UserName = @DeleteUserName;
		SELECT @RightKey = RightKey FROM dbo.sx_pf_rRights WHERE UserKey = @DeleteUserKey AND ProductLineID = @ProductLineID AND FactoryID = @FactoryID;
		
		IF @DeleteUserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values		
		DELETE FROM dbo.sx_pf_rRights
		WHERE RightKey = @RightKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Right TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Right TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a TimeID
Deletes a TimeID

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID / ProductLineID / ProductID / TimeID = 0 => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Value of TimeID deleted => 200 OK
5. There is none ProductID => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '10'
DECLARE @TimeID			INT				= '20180515'

EXECUTE @RC = dbo.sx_pf_DELETE_TimeID 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@TimeID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_TimeID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_TimeID;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_TimeID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_TimeID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3.1 - Delete values
		DELETE FROM dbo.sx_pf_fValues 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		-- STEP 3.2 - Delete Timeline

		DELETE FROM dbo.sx_pf_dTime 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_TimeID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_TimeID TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a User
Deletes a User with all his rights. A cluster administrator can't delete himself

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. DeleteUserName is empty => 404 Not Found
2. Delete command comes from non clusteradmin => 403 Forbidden
3. User wants to delete himself => 403 Forbidden
4. DeleteUser don`t exists => 204 No Content
5. deleted user successful => 200 OK

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @DeleteUserName		NVARCHAR(255) = 'W10\admin'
		
EXECUTE @RC = dbo.sx_pf_DELETE_User 
		@Username,
		@DeleteUserName

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_User;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_User
	@Username		NVARCHAR(255),
	@DeleteUserName NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_User;

		-- Protect Strings
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString (@DeleteUserName);

		IF @DeleteUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
			
		-- STEP 1.2 - Stop, if Transactuser wants to delete himself
		IF @TransactUsername = @DeleteUsername
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction can`t delete himself', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - delete User		
		DELETE FROM dbo.sx_pf_rRights 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_rUser 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@@ROWCOUNT = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_User TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Value
Delete a Value of ValueSeries with specified TimeID

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. All IDs are empty or TimeID is 0 => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Value not found => 204 No Content
5. Value deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '10'
DECLARE @ValueSerieID	NVARCHAR(255)	= 'K1'
DECLARE @TimeID			INT				= 20180615

EXECUTE @RC = dbo.sx_pf_DELETE_Value 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID,
		@TimeID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Value;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	DECLARE @ValueKey			INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Value;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey; --  AND FactoryKey = @FactoryKey;
		SELECT @ValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ValueSeriesID = @ValueSerieID AND ProductKey = @ProductKey; -- AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey;
		SELECT @ValueKey = ValueKey FROM dbo.sx_pf_fValues WHERE ValueSeriesKey = @ValueSerieKey AND TimeID = @TimeID; -- AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND ValueSeriesKey = @ValueSeriesKey;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_fValues 
		WHERE ValueKey = @ValueKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Value TO pf_PlanningFactoryService;
GO

/*
DELETE Operation for Values
Delete all Values of Value Series of Products

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. All IDs are empty  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Values not found => 204 No Content
5. Values deleted => 200 OK

DECLARE @RC						INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @ProductID		NVARCHAR(255) = '10'
		DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

EXECUTE @RC = dbo.sx_pf_DELETE_Values 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Values') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_Values;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Values;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @ValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ValueSeriesID = @ValueSerieID AND ProductKey = @ProductKey;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM dbo.sx_pf_fValues 
		WHERE ValueSeriesKey = @ValueSerieKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF (@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Values TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Values TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for one ValueSerie
Deletes a ValueSerie and changes the Numbers of the other ValueSeries. If Series No 1 is delete, Procedure creates Placeholder in a new ValueSeries 1.

05/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ValueSerieID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. ValueSerie not found => 204 No Content
5. ValueSerie deleted => 200 OK

DECLARE @RC						INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @ProductID		NVARCHAR(255) = '10'
		DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

EXECUTE @RC = dbo.sx_pf_DELETE_ValueSerie 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_DELETE_ValueSerie;
GO

CREATE PROCEDURE dbo.sx_pf_DELETE_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey			INT				= 0;
	DECLARE @ProductKey				INT				= 0;
	DECLARE @DelValueSerieKey		INT				= 0;
	DECLARE @DelValueSerieNo		INT				= 0;
	DECLARE @NewFirstValueSerieKey	INT				= 0;
	DECLARE @NewFirstValueSerieNo	INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
	
		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				 
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @DelValueSerieKey = ValueSeriesKey, @DelValueSerieNo = ValueSeriesNo FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		IF @DelValueSerieKey = 0
		BEGIN
			SET @ResultCode = 204;
		END 
		ELSE
		BEGIN				
			-- STEP 3.2 - Delete ValueSerie 				
			DELETE FROM dbo.sx_pf_fValues 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM dbo.sx_pf_dValueSeries 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.3 - Decrement all ValueSeriesNo which are bigger than the deleted on
			UPDATE dbo.sx_pf_dValueSeries
			SET ValueSeriesNo = ValueSeriesNo - 1
			FROM dbo.sx_pf_dValueSeries
			WHERE 
				ProductKey = @ProductKey
				AND ValueSeriesNo > @DelValueSerieNo;

			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.4 - if the new first ValueSeries is not NO 1 (1,3..... Numbers), set it to 1
			IF @NewFirstValueSerieNo - 1 > 1 
			BEGIN
				UPDATE dbo.sx_pf_dValueSeries 
				SET	ValueSeriesNo = 1
				FROM dbo.sx_pf_dValueSeries
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesKey = @NewFirstValueSerieKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;
		
		COMMIT TRANSACTION sx_pf_DELETE_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryService;
GO
/* 
EXPORT operation of a single Factory, including
- its properties in gtables
- its global attributes
- its Productline
- its products
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_pMaskSQL
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_FactoryReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

Test Call:
Results must be stored in table, as column width is to big for SSMS or SQLCMD

0a. Activate Stepnumber for output in Line 624 around and execute prodecedure
0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

1. Execute this to store the results in the table
	IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[tmpout_Export]') AND type in (N'U'))
	DROP TABLE [dbo].[tmpout_Export]
	GO
	CREATE TABLE tmpout_Export (
		StepNumber INT,
		Command NVARCHAR(MAX)
		)
	INSERT INTO tmpout_Export
		EXEC sx_pf_EXPORT_Factory 'SQL','AT',1

2. Copy this table Content in a file over a .bat file with this command
	bcp.exe "SELECT Command FROM PlanningFactoryCars.dbo.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Factory]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_EXPORT_Factory];
GO

CREATE PROCEDURE [dbo].[sx_pf_EXPORT_Factory]
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = N'''' 
						+ ISNULL(@Username, N'NULL')									+ N''',''' 
						+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
						+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER = 0;						
	DECLARE @ResultCode			INTEGER = 501;					
	DECLARE @TimestampCall		DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @FactoryKey BIGINT = 0;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID(@FactoryID);

		IF @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	
		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_FactoryReadRight] @TransactUsername, @FactoryID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM [dbo].[sx_pf_dValueSeries] WHERE FactoryKey = @FactoryKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource2		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource3		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource4		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource5		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource6		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource7		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource8		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource9		FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource10	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource11	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource12	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource13	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource14	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource15	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource16	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource17	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource18	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource19	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource20	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource21	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource22	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource23	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource24	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource25	FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################

		IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
				 (0,1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0,1, N'DECLARE @Username NVARCHAR(255)		= ''SQL''')
				,(0,1, N'DECLARE @FactoryID NVARCHAR(255)		= ''' + @FactoryID + N'''')
				,(0,1, N'--This FactoryID will be deleted during import, if it exists. You should be sure !');

			-- DELETE FactoryID if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'EXEC dbo.sx_pf_DELETE_Factory @Username, @FactoryID;');
			
			-- POST Factory
			INSERT INTO @ExportTable
				SELECT 1001,1, N'EXEC dbo.sx_pf_POST_Factory @Username, @FactoryID,'''
					+ [dbo].[sx_pf_pMaskSQL](NameShort)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](NameLong)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''',''' 
					+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''',''' 
					+ [dbo].[sx_pf_pMaskSQL](ResponsiblePerson) + N''',''' 
					+ [dbo].[sx_pf_pMaskSQL](ImageName)			+ N''';'
				 FROM dbo.sx_pf_dFactories dF WHERE dF.FactoryKey = @FactoryKey

			-- POST Productlines
			INSERT INTO @ExportTable 
			SELECT 1002,1, N'EXEC dbo.sx_pf_POST_Productline @Username,''' + ProductLineID + N''', @FactoryID,'''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)					+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)					+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ResponsiblePerson)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ImageName)					+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](DefaultTemplate)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource1)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias1)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource2)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias2)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource3)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias3)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource4)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias4)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource5)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias5)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource6)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias6)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource7)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias7)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource8)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias8)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource9)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias9)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource10)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias10)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource11)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias11)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource12)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias12)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource13)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias13)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource14)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias14)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource15)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias15)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource16)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias16)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource17)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias17)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource18)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias18)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource19)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias19)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource20)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias20)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource21)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias21)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource22)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias22)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource23)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias23)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource24)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias24)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource25)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias25)	+ N''';'
			FROM [dbo].[sx_pf_dProductlines] WHERE FactoryKey = @FactoryKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''',''' + ProductLineID + N''', @FactoryID,''' 
				+ TimeType									+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](NameShort)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ResponsiblePerson) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ImageName)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL]([Status])			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](Template)			+ N''',''' 
				+ TemplateVersion							+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute1)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute2)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute3)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute4)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute5)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute6)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute7)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute8)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute9)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute10) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute11) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute12) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute13) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute14) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute15) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute16) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute17) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute18) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute19) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute20) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute21) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute22) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute23) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute24) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute25) + N''';'
			FROM [dbo].[sx_pf_dProducts] WHERE FactoryKey = @FactoryKey;
					
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				,ROW_NUMBER()OVER (ORDER BY FactoryID,ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				,'EXEC dbo.sx_pf_POST_ValueSerie @Username,''' + ProductID + ''', ''' + ProductLineID +''', @FactoryID,''' +
				+ ValueSeriesID								+ N''','''
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](ImageName)			+ N''','''
				+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''','''
				+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''','''
				+ ValueSource								+ N''','''
				+ ValueListID								+ N''','''
				+ ValueFormatID								+ N''','''
				+ Unit										+ N''','''
				+ CAST(Scale AS NVARCHAR (255))				+ N''','''
				+ Effect									+ N''','''
				+ EffectParameter							+ N''';'
			FROM [dbo].[sx_pf_dValueSeries]

			WHERE FactoryKey = @FactoryKey
			
			ORDER BY 
				 FactoryID
				,ProductLineID
				,ProductID
				,ValueSeriesNo;

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],' -- with DeleteFlag, AddValue,FormatID 
					FROM [dbo].[sx_pf_dTime] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, ''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,1,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(''''' 
							+ ValueSeriesID							+ N''''',' 
							+ CAST(TimeID AS NVARCHAR(255))			+ N',''''' 
							+ ValueFormula 							+ N''''','''''
							+ CAST(ValueInt AS NVARCHAR(255)) 		+ N''''',''''' 
							+ [dbo].[sx_pf_pMaskSQL](ValueText)		+ N''''',''''' 
							+ [dbo].[sx_pf_pMaskSQL](ValueComment)	+ ''''')],' 
					FROM [dbo].[sx_pf_fValues] t
					WHERE t.ProductKey = dP.ProductKey
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,0,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Factory Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'EXEC dbo.sx_pf_POST_FactoryProperty @Username, @FactoryID,''' 
					+ PropertyID									+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](PropertyName)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](CommentUser)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](CommentDev)			+ N''','''
					+ Unit											+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](ValueText)				+ N''','''
					+ CAST(ValueInt				AS NVARCHAR(255))	+ N''','''
					+ CAST(Scale				AS NVARCHAR(255))	+ N''','''
					+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''','''
					+ FormatID										+ N''';'
			FROM [dbo].[sx_pf_gFactories]
			WHERE FactoryKey = @FactoryKey; 

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductlineProperty @Username,''' + ProductLineID +''', @FactoryID,''' 
					+ PropertyID									+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](PropertyName)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](CommentUser)			+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](CommentDev)			+ N''','''
					+ Unit											+ N''','''
					+ [dbo].[sx_pf_pMaskSQL](ValueText)				+ N''','''
					+ CAST(ValueInt				AS NVARCHAR(255))	+ N''','''
					+ CAST(Scale				AS NVARCHAR(255))	+ N''','''
					+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''','''
					+ FormatID										+ N''';'
			FROM [dbo].[sx_pf_gProductLines]
			WHERE FactoryKey = @FactoryKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6002,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,''' 
				+ PropertyID									+ N''', '''
				+ [dbo].[sx_pf_pMaskSQL](PropertyName)			+ N''', '''
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)			+ N''', '''
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)			+ N''', '''
				+ Unit											+ N''', '''
				+ [dbo].[sx_pf_pMaskSQL](ValueText)				+ N''', '''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''', '''
				+ CAST(Scale				AS NVARCHAR(255))	+ N''', '''
				+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''', '''
				+ FormatID										+ N''';'
			FROM [dbo].[sx_pf_gProducts]
			WHERE  FactoryKey = @FactoryKey; 
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ [dbo].[sx_pf_pMaskSQL](ValueText)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ValueComment)	+ N''',''' 
				+ FormatID								+ N''';'
			FROM [dbo].[sx_pf_hListValues]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM [dbo].[sx_pf_dValueSeries] 
				WHERE FactoryKey = @FactoryKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM [dbo].[sx_pf_gProducts] 
				WHERE 
						FactoryKey = @FactoryKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID										+ N''','''
				+ BackgroundColor								+ N''',''' 
				+ FontColor										+ N''',''' 
				+ ValueFormat									+ N''','''
				+ CAST(Bold				AS NVARCHAR(255))		+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))		+ N''','''
				+ AlignmentHorizontal							+ N''','''
				+ AlignmentVertical								+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))		+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))		+ N''','''
				+ CAST(BorderToPrevious AS NVARCHAR(255))		+ N''';'
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (8000,1, N'GO');

		END -- of Output for SQL Server
		
	-- STEP 3.3 - OUTPUT Type for PF Client Execution ##################################################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0,1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0,1, N'--DECLARE #Username NVARCHAR(255)	= ''SQL''')
				,(0,1, N'--DECLARE #FactoryID NVARCHAR(255)	= ''' + @FactoryID + N'''')
				,(0,1, N'--This FactoryID will be deleted during import, if it exists. You should be sure !');

		
			-- DELETE Factory if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'sx_pf_DELETE_Factory' + @spr + N'@Username' + @spr + N'#Username' + @spr + N'@FactoryID' + @spr + N'#FactoryID');

			-- POST Factory
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'sx_pf_POST_Factory' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'
					+ @spr + N'@NameShort'			+ @spr + NameShort 
					+ @spr + N'@NameLong'			+ @spr + NameLong  
					+ @spr + N'@CommentUser'		+ @spr + CommentUser  
					+ @spr + N'@CommentDev'			+ @spr + CommentDev  
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson  
					+ @spr + N'@ImageName'			+ @spr + ImageName  
			FROM dbo.sx_pf_dFactories dF WHERE dF.FactoryKey = @FactoryKey
						
			-- POST Productlines
			INSERT INTO @ExportTable 
			SELECT 1002,1, N'sx_pf_POST_Productline' 
				+ @spr + N'@Username'					+ @spr + N'#Username' 
				+ @spr + N'@ProductlineID'				+ @spr + ProductlineID 
				+ @spr + N'@FactoryID'					+ @spr + N'#FactoryID'
				+ @spr + N'@NameShort'					+ @spr + NameShort 
				+ @spr + N'@NameLong'					+ @spr + NameLong  
				+ @spr + N'@CommentUser'				+ @spr + CommentUser  
				+ @spr + N'@CommentDev'					+ @spr + CommentDev  
				+ @spr + N'@ResponsiblePerson'			+ @spr + ResponsiblePerson  
				+ @spr + N'@ImageName'					+ @spr + ImageName  
				+ @spr + N'@DefaultTemplate'			+ @spr + DefaultTemplate  
				+ @spr + N'@GlobalAttributeSource1'		+ @spr + GlobalAttributeSource1  
				+ @spr + N'@GlobalAttributeAlias1'		+ @spr + GlobalAttributeAlias1  
				+ @spr + N'@GlobalAttributeSource2'		+ @spr + GlobalAttributeSource2  
				+ @spr + N'@GlobalAttributeAlias2'		+ @spr + GlobalAttributeAlias2  
				+ @spr + N'@GlobalAttributeSource3'		+ @spr + GlobalAttributeSource3  
				+ @spr + N'@GlobalAttributeAlias3'		+ @spr + GlobalAttributeAlias3 
				+ @spr + N'@GlobalAttributeSource4'		+ @spr + GlobalAttributeSource4  
				+ @spr + N'@GlobalAttributeAlias4'		+ @spr + GlobalAttributeAlias4 
				+ @spr + N'@GlobalAttributeSource5'		+ @spr + GlobalAttributeSource5  
				+ @spr + N'@GlobalAttributeAlias5'		+ @spr + GlobalAttributeAlias5 
				+ @spr + N'@GlobalAttributeSource6'		+ @spr + GlobalAttributeSource6  
				+ @spr + N'@GlobalAttributeAlias6'		+ @spr + GlobalAttributeAlias6 
				+ @spr + N'@GlobalAttributeSource7'		+ @spr + GlobalAttributeSource7  
				+ @spr + N'@GlobalAttributeAlias7'		+ @spr + GlobalAttributeAlias7 
				+ @spr + N'@GlobalAttributeSource8'		+ @spr + GlobalAttributeSource8  
				+ @spr + N'@GlobalAttributeAlias8'		+ @spr + GlobalAttributeAlias8 
				+ @spr + N'@GlobalAttributeSource9'		+ @spr + GlobalAttributeSource9  
				+ @spr + N'@GlobalAttributeAlias9'		+ @spr + GlobalAttributeAlias9 
				+ @spr + N'@GlobalAttributeSource10'	+ @spr + GlobalAttributeSource10  
				+ @spr + N'@GlobalAttributeAlias10'		+ @spr + GlobalAttributeAlias10 
				+ @spr + N'@GlobalAttributeSource11'	+ @spr + GlobalAttributeSource11  
				+ @spr + N'@GlobalAttributeAlias11'		+ @spr + GlobalAttributeAlias11  
				+ @spr + N'@GlobalAttributeSource12'	+ @spr + GlobalAttributeSource12  
				+ @spr + N'@GlobalAttributeAlias12'		+ @spr + GlobalAttributeAlias12  
				+ @spr + N'@GlobalAttributeSource13'	+ @spr + GlobalAttributeSource13  
				+ @spr + N'@GlobalAttributeAlias13'		+ @spr + GlobalAttributeAlias13 
				+ @spr + N'@GlobalAttributeSource14'	+ @spr + GlobalAttributeSource14  
				+ @spr + N'@GlobalAttributeAlias14'		+ @spr + GlobalAttributeAlias14 
				+ @spr + N'@GlobalAttributeSource15'	+ @spr + GlobalAttributeSource15  
				+ @spr + N'@GlobalAttributeAlias15'		+ @spr + GlobalAttributeAlias15 
				+ @spr + N'@GlobalAttributeSource16'	+ @spr + GlobalAttributeSource16  
				+ @spr + N'@GlobalAttributeAlias16'		+ @spr + GlobalAttributeAlias16 
				+ @spr + N'@GlobalAttributeSource17'	+ @spr + GlobalAttributeSource17  
				+ @spr + N'@GlobalAttributeAlias17'		+ @spr + GlobalAttributeAlias17 
				+ @spr + N'@GlobalAttributeSource18'	+ @spr + GlobalAttributeSource18  
				+ @spr + N'@GlobalAttributeAlias18'		+ @spr + GlobalAttributeAlias18 
				+ @spr + N'@GlobalAttributeSource19'	+ @spr + GlobalAttributeSource19  
				+ @spr + N'@GlobalAttributeAlias19'		+ @spr + GlobalAttributeAlias19 
				+ @spr + N'@GlobalAttributeSource20'	+ @spr + GlobalAttributeSource20  
				+ @spr + N'@GlobalAttributeAlias20'		+ @spr + GlobalAttributeAlias20 
				+ @spr + N'@GlobalAttributeSource21'	+ @spr + GlobalAttributeSource21  
				+ @spr + N'@GlobalAttributeAlias21'		+ @spr + GlobalAttributeAlias21  
				+ @spr + N'@GlobalAttributeSource22'	+ @spr + GlobalAttributeSource22  
				+ @spr + N'@GlobalAttributeAlias22'		+ @spr + GlobalAttributeAlias22  
				+ @spr + N'@GlobalAttributeSource23'	+ @spr + GlobalAttributeSource23  
				+ @spr + N'@GlobalAttributeAlias23'		+ @spr + GlobalAttributeAlias23 
				+ @spr + N'@GlobalAttributeSource24'	+ @spr + GlobalAttributeSource24  
				+ @spr + N'@GlobalAttributeAlias24'		+ @spr + GlobalAttributeAlias24 
				+ @spr + N'@GlobalAttributeSource25'	+ @spr + GlobalAttributeSource25  
				+ @spr + N'@GlobalAttributeAlias25'		+ @spr + GlobalAttributeAlias25 + N''
			FROM [dbo].[sx_pf_dProductlines] WHERE FactoryKey = @FactoryKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'sx_pf_POST_ProductEmpty' 
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + ProductID 
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@TimeType'			+ @spr + TimeType
				+ @spr + N'@NameShort'			+ @spr + NameShort
				+ @spr + N'@NameLong'			+ @spr + NameLong
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson
				+ @spr + N'@ImageName'			+ @spr + ImageName
				+ @spr + N'@Status'				+ @spr + [Status] 
				+ @spr + N'@Template'			+ @spr + Template 
				+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion
				+ @spr + N'@GA1'				+ @spr + GlobalAttribute1
				+ @spr + N'@GA2'				+ @spr + GlobalAttribute2
				+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
				+ @spr + N'@GA4'				+ @spr + GlobalAttribute4
				+ @spr + N'@GA5'				+ @spr + GlobalAttribute5
				+ @spr + N'@GA6'				+ @spr + GlobalAttribute6
				+ @spr + N'@GA7'				+ @spr + GlobalAttribute7
				+ @spr + N'@GA8'				+ @spr + GlobalAttribute8
				+ @spr + N'@GA9'				+ @spr + GlobalAttribute9
				+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
				+ @spr + N'@GA11'				+ @spr + GlobalAttribute11
				+ @spr + N'@GA12'				+ @spr + GlobalAttribute12
				+ @spr + N'@GA13'				+ @spr + GlobalAttribute13
				+ @spr + N'@GA14'				+ @spr + GlobalAttribute14
				+ @spr + N'@GA15'				+ @spr + GlobalAttribute15
				+ @spr + N'@GA16'				+ @spr + GlobalAttribute16
				+ @spr + N'@GA17'				+ @spr + GlobalAttribute17
				+ @spr + N'@GA18'				+ @spr + GlobalAttribute18
				+ @spr + N'@GA19'				+ @spr + GlobalAttribute19
				+ @spr + N'@GA20'				+ @spr + GlobalAttribute20
				+ @spr + N'@GA21'				+ @spr + GlobalAttribute21
				+ @spr + N'@GA22'				+ @spr + GlobalAttribute22
				+ @spr + N'@GA23'				+ @spr + GlobalAttribute23
				+ @spr + N'@GA24'				+ @spr + GlobalAttribute24
				+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM [dbo].[sx_pf_dProducts] WHERE FactoryKey = @FactoryKey; 
									
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				,ROW_NUMBER() OVER (ORDER BY FactoryID,ProductlineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'sx_pf_POST_ValueSerie'
				+ @spr + N'@Username'				+ @spr + N'#Username' 
				+ @spr + N'@ProductID'				+ @spr + ProductID
				+ @spr + N'@ProductLineID'			+ @spr + ProductLineID
				+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
				+ @spr + N'@ValueSeriesID'			+ @spr + ValueSeriesID 
				+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))
				+ @spr + N'@NameShort'				+ @spr + NameShort
				+ @spr + N'@NameLong'				+ @spr + NameLong
				+ @spr + N'@CommentUser'			+ @spr + CommentUser
				+ @spr + N'@CommentDev'				+ @spr + CommentDev
				+ @spr + N'@ImageName'				+ @spr + ImageName
				+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255))
				+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255))
				+ @spr + N'@ValueSource'			+ @spr + ValueSource
				+ @spr + N'@ValueListID'			+ @spr + ValueListID
				+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID
				+ @spr + N'@Unit'					+ @spr + Unit
				+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255))
				+ @spr + N'@Effect'					+ @spr + Effect
				+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM dbo.sx_pf_dValueSeries
			WHERE FactoryKey = @FactoryKey
			ORDER BY 
				 FactoryID
				,ProductlineID
				,ProductID
				,ValueSeriesNo; 

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10))+',0,0,'+ FormatID + ')],' -- with DeleteFlag, AddValue,FormatID 
					FROM [dbo].[sx_pf_dTime] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'sx_pf_POST_Timeline'
					+ @spr + N'@Username'							+ @spr + N'#Username' 
					+ @spr + N'@ProductID'							+ @spr + ProductID
					+ @spr + N'@ProductLineID'						+ @spr + ProductLineID
					+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
					+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' 
					+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;		
			
			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID, dP.ProductLineID
					,(SELECT N'[(''' 
							+ ValueSeriesID									+ N''','
							+ CAST(TimeID	AS NVARCHAR(255))				+ N',''' 
							+ dbo.sx_pf_pRemoveLinebreaks(ValueFormula)		+ N''',''' 
							+ CAST(ValueInt AS NVARCHAR(255)) 				+ N''',''' 
							+ ValueText 									+ N''',''' 
							+ ValueComment									+ ''')],' 
					FROM [dbo].[sx_pf_fValues] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.ValueSeriesID, t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'sx_pf_POST_ProductDataTableValues'
				+ @spr + N'@Username'						+ @spr + N'#Username' 
				+ @spr + N'@ProductID'						+ @spr + ProductID
				+ @spr + N'@ProductLineID'					+ @spr + ProductLineID 
				+ @spr + N'@FactoryID'						+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'		+ @spr + N'0' 
				+ @spr + N'@ValuesInBracketsCommaSeparated' + @spr + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;
			
			-- POST Factory Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'sx_pf_POST_FactoryProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gFactories] 
			WHERE FactoryKey = @FactoryKey; 

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'sx_pf_POST_ProductlineProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gProductLines]
			WHERE FactoryKey = @FactoryKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6003,1, N'sx_pf_POST_ProductProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + ProductID
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@PropertyID'			+ @spr + PropertyID
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255)) 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gProducts]
			WHERE  FactoryKey = @FactoryKey;
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'sx_pf_POST_List'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort
				+ @spr + N'@NameLong'		+ @spr + NameLong
				+ @spr + N'@CommentDev'		+ @spr + CommentDev
				+ @spr + N'@CommentUser'	+ @spr + CommentUser
				+ @spr + N'@Datentyp'		+ @spr + Datentyp
				+ @spr + N'@Source'			+ @spr + [Source]
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'sx_pf_POST_ListValue'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListValuekey'	+ @spr + N'0'
				+ @spr + N'@ListID'			+ @spr + ListID
				+ @spr + N'@ValueInt'		+ @spr + CAST(ValueInt	AS NVARCHAR(255))
				+ @spr + N'@Scale'			+ @spr + CAST(Scale		AS NVARCHAR(255))
				+ @spr + N'@ValueText'		+ @spr + ValueText
				+ @spr + N'@ValueComment'	+ @spr + ValueComment
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hListValues]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL SELECT ValueFormatID FROM [dbo].[sx_pf_dValueSeries] WHERE FactoryKey = @FactoryKey
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'sx_pf_POST_Format'
					 + @spr + N'@Username'				+ @spr + N'#Username'
					 + @spr + N'@FormatID'				+ @spr + FormatID
					 + @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					 + @spr + N'@FontColor'				+ @spr + FontColor
					 + @spr + N'@ValueFormat'			+ @spr + ValueFormat 
					 + @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255))
					 + @spr + N'@Italic'				+ @spr + CAST(Italic			AS NVARCHAR(255))
					 + @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal
					 + @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical
					 + @spr + N'@TextUnderlined'		+ @spr + CAST(TextUnderlined	AS NVARCHAR(255))
					 + @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255))
					 + @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client	
	
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY 
			 StepNumber
			,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_Factory] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_Factory] TO pf_PlanningFactoryService;
GO



	


/* Procedure to export one single product, including
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_pMaskSQL
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

TestCall
DECLARE @RC int
DECLARE @Username			nvarchar(255) = 'SQL'
DECLARE @FactoryID			nvarchar(255) = 'ZT'
DECLARE @ProductLineID		nvarchar(255) = 'U'
DECLARE @ProductID			nvarchar(255) = '1'
DECLARE @OutputForSQLServer INT = 1

EXECUTE @RC = [dbo].[sx_pf_EXPORT_Product] 
   @Username
  ,@FactoryID
  ,@ProductLineID
  ,@ProductID
  ,@OutputForSQLServer

 PRINT @RC
 */

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Product]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_EXPORT_Product]
GO

CREATE  PROCEDURE [sx_pf_EXPORT_Product]
		@Username			NVARCHAR(255),
		@FactoryID			NVARCHAR(255),
		@ProductLineID		NVARCHAR(255),
		@ProductID			NVARCHAR(255),
		@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username, N'NULL')									+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')									+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER = 0;						
	DECLARE @ResultCode 		INTEGER = 501;						
	DECLARE @TimestampCall		DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';				
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username = N'';
	IF @FactoryID			IS NULL SET @FactoryID = N'';
	IF @ProductLineID		IS NULL SET @ProductLineID = N'';
	IF @ProductID			IS NULL SET @ProductID = N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @ProductKey BIGINT = 0;
		DECLARE @ProductLineKey BIGINT = 0;
		DECLARE @Template NVARCHAR (255) = '';

		-- STEP 0.2 - Protect input parameters
		SET @Username		= [dbo].[sx_pf_pProtectString] (@Username);
		SET @FactoryID		= [dbo].[sx_pf_pProtectID] (@FactoryID);
		SET @ProductLineID	= [dbo].[sx_pf_pProtectID] (@ProductLineID);
		SET @ProductID		= [dbo].[sx_pf_pProtectID] (@ProductID);

		IF @FactoryID = N'' OR @ProductLineID = N''  OR @ProductID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductlineKey = ProductlineKey FROM [dbo].[sx_pf_dProductLines] dPL WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM [dbo].[sx_pf_dProducts] WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	 
		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ProductLineReadRight] @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine Template
		SELECT @Template = Template FROM sx_pf_dProducts WHERE ProductKey = @ProductKey;

		-- Deterime all Productlines using this Template
		DECLARE @DependendProductLinesIDs AS TABLE (ProductlineKey BIGINT);
		INSERT INTO @DependendProductLinesIDs (ProductlineKey)
			SELECT DISTINCT ProductlineKey FROM sx_pf_dProducts WHERE Template = @Template;

		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM [dbo].[sx_pf_dValueSeries] WHERE ProductKey = @ProductKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT DISTINCT GlobalAttributeSource1	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource2	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource3	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource4	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource5	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource6	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource7	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource8	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource9	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource10	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource11	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource12	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource13	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource14	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource15	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource16	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource17	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource18	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource19	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource20	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource21	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource22	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource23	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource24	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource25	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
		)									
		INSERT INTO @DependendListIDs (ListID)
			SELECT DISTINCT ValueListID
			FROM lists
			WHERE ValueListID != '';
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
											 StepNumber INT NOT NULL
											,RowNumber	INT NOT NULL
											,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################
	   	IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
			  (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			, (0, 1, N'DECLARE @Username		NVARCHAR(255)	= ''SQL''')
			, (0, 1, N'DECLARE @FactoryID		NVARCHAR(255)	= ''' + @FactoryID		+ N'''')
			, (0, 1, N'DECLARE @ProductLineID	NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
			, (0, 1, N'DECLARE @ProductID		NVARCHAR(255)	= ''' + @ProductID		+ N'''')
			, (0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Product if exists
			INSERT INTO @ExportTable 
			VALUES (1000, 1, N'EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;');
			
			-- POST Product
			INSERT INTO @ExportTable 
			SELECT 2000, 1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,''' 
				+ TimeType + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (NameShort)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (NameLong)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentUser)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentDev)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (ResponsiblePerson)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (ImageName)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] ([Status])				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (Template)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (TemplateVersion)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute1)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute2)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute3)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute4)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute5)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute6)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute7)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute8)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute9)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute10)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute11)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute12)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute13)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute14)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute15)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute16)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute17)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute18)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute19)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute20)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute21)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute22)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute23)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute24)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (GlobalAttribute25)	+ N''';'
			FROM [dbo].[sx_pf_dProducts] WHERE ProductKey = @ProductKey;

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 3000, 1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username, @ProductID, @ProductLineID, @FactoryID,''' +
				+ PropertyID								+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (PropertyName)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (CommentUser)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (CommentDev)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (Unit)				+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (ValueText)		+ N''','''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''','''
				+ CAST(Scale AS NVARCHAR(255))				+ N''','''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''','''
				+ FormatID									+ N''';'
			FROM [dbo].[sx_pf_gProducts]
			WHERE  ProductKey = @ProductKey; 

			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable
			SELECT 4000 
				, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
				, N'EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,''' +
				+ ValueSeriesID									+ N''',''' 
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (NameShort)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (NameLong)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentUser)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentDev)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (ImageName)			+ N''',''' 
				+ CAST([IsNumeric]		AS NVARCHAR (255))		+ N''',''' 
				+ CAST(VisibilityLevel	AS NVARCHAR (255))		+ N''',''' 
				+ ValueSource									+ N''',''' 
				+ ValueListID									+ N''',''' 
				+ ValueFormatID									+ N''',''' 
				+ Unit											+ N''',''' 
				+ CAST(Scale AS NVARCHAR (255))					+ N''',''' 
				+ Effect										+ N''',''' 
				+ EffectParameter								+ N''';'
			FROM [dbo].[sx_pf_dValueSeries]
			WHERE ProductKey = @ProductKey
			ORDER BY
				 ProductID
				,ValueSeriesNo; 
		   
			;WITH tms AS (
				SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],'
				FROM [dbo].[sx_pf_dTime]
				WHERE  ProductKey = @ProductKey
				ORDER BY TimeID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 5000, 1, N'EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,''' + LEFT(val, LEN(val) - 1) + N''';'
			FROM tms
			WHERE NOT val IS NULL;
				  
			-- POST ProductDataTableValues
			;WITH vals AS (
				SELECT (SELECT  N'[(''''' 
					+ fV.ValueSeriesID							+ N''''','  
					+ CAST(fV.TimeID	AS NVARCHAR(255))		+ N',''''' 
					+ fV.ValueFormula							+ N''''',''''' 
					+ CAST(fV.ValueInt	AS NVARCHAR(255))		+ N''''',''''' 
					+ [dbo].[sx_pf_pMaskSQL] (fV.ValueText)		+ N''''',''''' 
					+ [dbo].[sx_pf_pMaskSQL] (fV.ValueComment)	+ N''''')],'
				FROM [dbo].[sx_pf_fValues] fV LEFT JOIN [dbo].[sx_pf_dValueSeries] dVS 
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
				WHERE fV.ProductKey = @ProductKey 
				ORDER BY dVS.ValueSeriesNo
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 6000, 1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,'''
					+ LEFT(val, LEN(val) - 1) + N''';' 
			FROM vals
			WHERE NOT val IS NULL;		

			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001, 1, N'EXECUTE dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (NameShort)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL] (NameLong)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentUser)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002, 1, N'EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ [dbo].[sx_pf_pMaskSQL] (ValueText)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL] (ValueComment) + N''',''' 
				+ FormatID								+ N''';'
		   	FROM [dbo].[sx_pf_hListValues]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
			
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT ValueFormatID 
				FROM [dbo].[sx_pf_dValueSeries] 
				WHERE ProductKey = @ProductKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM [dbo].[sx_pf_gProducts] 
				WHERE 
						ProductKey = @ProductKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 8001, 1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID									+ N''','''
				+ BackgroundColor							+ N''',''' 
				+ FontColor									+ N''',''' 
				+ ValueFormat								+ N''',''' 
				+ CAST(Bold				AS NVARCHAR(255))	+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))	+ N''','''
				+ AlignmentHorizontal						+ N''','''
				+ AlignmentVertical							+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToPrevious	AS NVARCHAR(255))	+ N''';'
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (9000, 1, N'GO');

		END -- of Output for SQL Server

		 -- STEP 3.3 - OUTPUT Type for PF Client Execution, NO dbo. allowed !! due to hard parameter coding in XLS Client###############################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'--DECLARE #Username		NVARCHAR(255)	= ''SQL''')
				,(0, 1, N'--DECLARE #FactoryID		NVARCHAR(255)	= ''' + @FactoryID		+ N'''')
				,(0, 1, N'--DECLARE #ProductLineID NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
				,(0, 1, N'--DECLARE #ProductID		NVARCHAR(255)	= ''' + @ProductID		+ N'''')
				,(0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Product if exists
			INSERT INTO @ExportTable 
			VALUES (1000, 1, N'sx_pf_DELETE_Product' 
				+ @spr + '@Username'		+ @spr + '#Username' 
				+ @spr + '@FactoryID'		+ @spr + '#FactoryID' 
				+ @spr + '@ProductlineID'	+ @spr + '#ProductLineID' 
				+ @spr + '@ProductID'		+ @spr + '#ProductID');

			-- POST Product
			INSERT INTO @ExportTable 
			SELECT 2000, 1, N'sx_pf_POST_ProductEmpty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'#ProductID' 
					+ @spr + N'@ProductlineID'		+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@TimeType'			+ @spr + TimeType 
					+ @spr + N'@NameShort'			+ @spr + NameShort 
					+ @spr + N'@NameLong'			+ @spr + NameLong 
					+ @spr + N'@CommentUser'		+ @spr + CommentUser 
					+ @spr + N'@CommentDev'			+ @spr + CommentDev 
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson 
					+ @spr + N'@ImageName'			+ @spr + ImageName 
					+ @spr + N'@Status'				+ @spr + [Status] 
					+ @spr + N'@Template'			+ @spr + Template 
					+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion 
					+ @spr + N'@GA1'				+ @spr + GlobalAttribute1 
					+ @spr + N'@GA2'				+ @spr + GlobalAttribute2 
					+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
					+ @spr + N'@GA4'				+ @spr + GlobalAttribute4 
					+ @spr + N'@GA5'				+ @spr + GlobalAttribute5 
					+ @spr + N'@GA6'				+ @spr + GlobalAttribute6 
					+ @spr + N'@GA7'				+ @spr + GlobalAttribute7 
					+ @spr + N'@GA8'				+ @spr + GlobalAttribute8 
					+ @spr + N'@GA9'				+ @spr + GlobalAttribute9 
					+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
					+ @spr + N'@GA11'				+ @spr + GlobalAttribute11 
					+ @spr + N'@GA12'				+ @spr + GlobalAttribute12 
					+ @spr + N'@GA13'				+ @spr + GlobalAttribute13 
					+ @spr + N'@GA14'				+ @spr + GlobalAttribute14 
					+ @spr + N'@GA15'				+ @spr + GlobalAttribute15 
					+ @spr + N'@GA16'				+ @spr + GlobalAttribute16 
					+ @spr + N'@GA17'				+ @spr + GlobalAttribute17 
					+ @spr + N'@GA18'				+ @spr + GlobalAttribute18 
					+ @spr + N'@GA19'				+ @spr + GlobalAttribute19 
					+ @spr + N'@GA20'				+ @spr + GlobalAttribute20 
					+ @spr + N'@GA21'				+ @spr + GlobalAttribute21 
					+ @spr + N'@GA22'				+ @spr + GlobalAttribute22 
					+ @spr + N'@GA23'				+ @spr + GlobalAttribute23 
					+ @spr + N'@GA24'				+ @spr + GlobalAttribute24 
					+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM [dbo].[sx_pf_dProducts] WHERE ProductKey = @ProductKey;

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 3000, 1, N'sx_pf_POST_ProductProperty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'#ProductID' 
					+ @spr + N'@ProductLineID'		+ @spr + N'#ProductlineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@PropertyID'			+ @spr + PropertyID 
					+ @spr + N'@PropertyName'		+ @spr + PropertyName 
					+ @spr + N'@CommentUser'		+ @spr + CommentUser 
					+ @spr + N'@CommentDev'			+ @spr + CommentDev 
					+ @spr + N'@Unit'				+ @spr + Unit 
					+ @spr + N'@ValueText'			+ @spr + ValueText 
					+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt AS NVARCHAR(255)) 
					+ @spr + N'@Scale'				+ @spr + CAST(Scale AS NVARCHAR(255)) 
					+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty AS NVARCHAR(255)) 
					+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gProducts]
			WHERE ProductKey = @ProductKey; 
			
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 4000 AS Step
					, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
					, N'sx_pf_POST_ValueSerie' 
					+ @spr + N'@Username'				+ @spr + N'#Username' 
					+ @spr + N'@ProductID'				+ @spr + N'#ProductID' 
					+ @spr + N'@ProductLineID'			+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
					+ @spr + N'@ValueSeriesID'			+ @spr + N'' + ValueSeriesID  
					+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))  
					+ @spr + N'@NameShort'				+ @spr + NameShort 
					+ @spr + N'@NameLong'				+ @spr + NameLong 
					+ @spr + N'@CommentUser'			+ @spr + CommentUser 
					+ @spr + N'@CommentDev'				+ @spr + CommentDev 
					+ @spr + N'@ImageName'				+ @spr + ImageName 
					+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255)) 
					+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255)) 
					+ @spr + N'@ValueSource'			+ @spr + ValueSource 
					+ @spr + N'@ValueListID'			+ @spr + ValueListID 
					+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID 
					+ @spr + N'@Unit'					+ @spr + Unit 
					+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255)) 
					+ @spr + N'@Effect'					+ @spr + Effect 
					+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM [dbo].[sx_pf_dValueSeries]
			WHERE ProductKey = @ProductKey 
			ORDER BY
				 ProductID
				,ValueSeriesNo;

			-- POST Timeline
			;WITH tms AS (
				SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],'
				FROM [dbo].[sx_pf_dTime]
				WHERE  ProductKey = @ProductKey
				ORDER BY TimeID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 5000, 1, N'sx_pf_POST_Timeline' 
				+ @spr + N'@Username'							+ @spr + N'#Username' 
				+ @spr + N'@ProductID'							+ @spr + N'#ProductID' 
				+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID'
				+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'0' + N'' 
				+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + N'' + LEFT(val, LEN(val) - 1) + N''
			FROM tms
			WHERE NOT val IS NULL;
			
			-- POST Product DataTable
			;WITH vals AS (
				SELECT (SELECT  N'[(''' 
					+ fV.ValueSeriesID													+ N''',' 
					+ CAST(fV.TimeID AS NVARCHAR(255))									+ N',''' 
					+ dbo.sx_pf_pRemoveLinebreaks(fV.ValueFormula)						+ N''',''' 
					+ CAST(fV.ValueInt AS NVARCHAR(255))								+ N''',''' 
					+ fV.ValueText														+ N''',''' 
					+ fV.ValueComment													+ N''')],'
				FROM [dbo].[sx_pf_fValues] fV LEFT JOIN [dbo].[sx_pf_dValueSeries] dVS 
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
				WHERE fV.ProductKey = @ProductKey 
				ORDER BY dVS.ValueSeriesNo
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable 
			SELECT 6000, 1, N'sx_pf_POST_ProductDataTableValues' 
				+ @spr + N'@Username'							+ @spr + N'#Username' 
				+ @spr + N'@ProductID'							+ @spr + N'#ProductID' 
				+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' 
				+ @spr + N'@ValuesInBracketsCommaSeparated'		+ @spr + N''
				+ LEFT(val, LEN(val) - 1) + N''
			FROM vals
			WHERE NOT val IS NULL;

			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001, 1, N'sx_pf_POST_List' 
				+ @spr + N'@Username'		+ @spr + N'#Username' 
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort 
				+ @spr + N'@NameLong'		+ @spr + NameLong 
				+ @spr + N'@CommentUser'	+ @spr + CommentUser 
				+ @spr + N'@CommentDev'		+ @spr + CommentDev 
				+ @spr + N'@Datentyp'		+ @spr + Datentyp 
				+ @spr + N'@Source'			+ @spr + Source 
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula 
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN	(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			--Create the List Values
			INSERT INTO @ExportTable 
			SELECT 8001, 1, N'sx_pf_POST_ListValue' 
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ListValueKey'		+ @spr + N'0' 
				+ @spr + N'@ListID'				+ @spr + ListID 
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt	AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale		AS NVARCHAR(255)) 
				+ @spr + N'@ValueText'			+ @spr + ValueText 
				+ @spr + N'@ValueComment'		+ @spr + ValueComment 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hListValues] 
			WHERE ListID IN	(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT ValueFormatID 
				FROM [dbo].[sx_pf_dValueSeries] 
				WHERE ProductKey = @ProductKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM [dbo].[sx_pf_gProducts] 
				WHERE 
						ProductKey = @ProductKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable
			SELECT 8001, 1, N'sx_pf_POST_Format' 
					+ @spr + N'@Username'				+ @spr + N'#Username' 
					+ @spr + N'@FormatID'				+ @spr + FormatID
					+ @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					+ @spr + N'@FontColor'				+ @spr + FontColor
					+ @spr + N'@ValueFormat'			+ @spr + ValueFormat
					+ @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255))
					+ @spr + N'@Italic'					+ @spr + CAST(Italic			AS NVARCHAR(255))
					+ @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal
					+ @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical
					+ @spr + N'@TextUnderlined'			+ @spr + CAST(TextUnderlined	AS NVARCHAR(255))
					+ @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255))
					+ @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client
		
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT Command 
		FROM @ExportTable 
		ORDER BY StepNumber, RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_Product] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_Product] TO pf_PlanningFactoryService;
GO

/* 
EXPORT operation of a single productline, including
- its properties in gtables
- its global attributes
- its products
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_pMaskSQL
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

Test Call:
Results must be stored in table, as column width is to big for SSMS or SQLCMD

0a. Activate Stepnumber for output in Line 753 around and execute prodecedure
0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

1. Execute this to store the results in the table
	IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[tmpout_Export]') AND type in (N'U'))
	DROP TABLE [dbo].[tmpout_Export]
	GO
	CREATE TABLE tmpout_Export (
		StepNumber INT,
		Command NVARCHAR(MAX)
		)
	INSERT INTO tmpout_Export
		EXEC sx_pf_EXPORT_Productline 'SQL','ZT','U',1

2. Copy this table Content in a file over a .bat file with this command
	bcp.exe "SELECT Command FROM PlanningFactoryCars.dbo.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Productline]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_EXPORT_Productline];
GO

CREATE PROCEDURE [dbo].[sx_pf_EXPORT_Productline]
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	AS NVARCHAR(255);
	DECLARE @ProcedureName		AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	AS NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username, N'NULL')									+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		AS INTEGER = 0;						
	DECLARE @ResultCode			AS INTEGER = 501;					
	DECLARE @TimestampCall		AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			AS NVARCHAR (2000) = N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username = N'';
	IF @FactoryID			IS NULL SET @FactoryID = N'';
	IF @ProductLineID		IS NULL SET @ProductLineID = N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @ProductLineKey BIGINT = 0;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID(@ProductLineID);

		IF @FactoryID = N'' OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductLineKey = ProductlineKey FROM [dbo].[sx_pf_dProductLines] WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

		IF @ProductLineKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	
		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ProductLineReadRight] @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM [dbo].[sx_pf_dValueSeries] WHERE ProductlineKey = @ProductLineKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource2		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource3		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource4		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource5		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource6		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource7		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource8		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource9		FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource10	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource11	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource12	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource13	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource14	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource15	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource16	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource17	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource18	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource19	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource20	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource21	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource22	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource23	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource24	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource25	FROM [dbo].[sx_pf_dProductLines] WHERE ProductLineKey = @ProductLineKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (	 
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################

		IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
				,(0, 1, N'DECLARE @FactoryID NVARCHAR(255)			= ''' + @FactoryID + N'''')
				,(0, 1, N'DECLARE @ProductLineID  NVARCHAR(255)	= ''' + @ProductLineID + N'''')
				,(0, 1, N'--This ProductlineID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Productline if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'EXEC dbo.sx_pf_DELETE_Productline @Username, @FactoryID, @ProductLineID;');
				
			-- POST Productline
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'EXEC dbo.sx_pf_POST_Productline @Username, @ProductLineID, @FactoryID,'''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)					+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)					+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)				+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ResponsiblePerson)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ImageName)					+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](DefaultTemplate)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource1)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias1)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource2)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias2)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource3)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias3)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource4)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias4)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource5)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias5)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource6)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias6)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource7)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias7)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource8)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias8)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource9)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias9)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource10)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias10)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource11)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias11)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource12)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias12)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource13)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias13)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource14)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias14)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource15)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias15)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource16)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias16)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource17)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias17)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource18)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias18)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource19)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias19)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource20)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias20)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource21)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias21)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource22)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias22)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource23)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias23)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource24)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias24)	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeSource25)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttributeAlias25)	+ N''';'
			FROM [dbo].[sx_pf_dProductlines] WHERE ProductlineKey = @ProductLineKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,''' 
				+ TimeType									+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](NameShort)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ResponsiblePerson) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ImageName)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL]([Status])			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](Template)			+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](TemplateVersion)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute1)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute2)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute3)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute4)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute5)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute6)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute7)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute8)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute9)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute10) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute11) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute12) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute13) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute14) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute15) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute16) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute17) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute18) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute19) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute20) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute21) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute22) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute23) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute24) + N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](GlobalAttribute25) + N''';'
			FROM [dbo].[sx_pf_dProducts] WHERE ProductlineKey = @ProductLineKey;
							
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000 AS Step
				, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'EXEC dbo.sx_pf_POST_ValueSerie @Username,''' + ProductID + ''', @ProductLineID, @FactoryID,''' +
				+ ValueSeriesID								+ N''','''
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)			+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](ImageName)			+ N''','''
				+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''','''
				+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''','''
				+ ValueSource								+ N''','''
				+ ValueListID								+ N''','''
				+ ValueFormatID								+ N''','''
				+ Unit										+ N''','''
				+ CAST(Scale			AS NVARCHAR (255))	+ N''','''
				+ Effect									+ N''','''
				+ EffectParameter							+ N''';'
			FROM [dbo].[sx_pf_dValueSeries]
			WHERE  ProductlineKey = @ProductLineKey
			ORDER BY 
				 ProductLineID
				,ProductID
				,ValueSeriesNo;
				
			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + ',0,0,'''''+ FormatID + ''''')],' -- with DeleteFlag, AddValue,FormatID 
					FROM [dbo].[sx_pf_dTime] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, ''' + ProductID + N''', @ProductLineID, @FactoryID,1,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[('''''		+ ValueSeriesID 
							+ N''''','		+ CAST(TimeID AS NVARCHAR(255)) 
							+ N','''''		+ ValueFormula 
							+ N''''','''''	+ CAST(ValueInt AS NVARCHAR(255)) 
							+ N''''','''''	+ [dbo].[sx_pf_pMaskSQL](ValueText) 
							+ N''''','''''	+ [dbo].[sx_pf_pMaskSQL](ValueComment) + ''''')],' 
					FROM [dbo].[sx_pf_fValues] t
					WHERE t.ProductKey = dP.ProductKey
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,0,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'EXEC dbo.sx_pf_POST_ProductlineProperty @Username, @ProductLineID, @FactoryID,''' 
					+ PropertyID								+ N''', '''
					+ [dbo].[sx_pf_pMaskSQL](PropertyName)		+ N''', '''
					+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''', '''
					+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''', '''
					+ Unit										+ N''', '''
					+ [dbo].[sx_pf_pMaskSQL](ValueText)			+ N''', '''
					+ CAST(ValueInt AS NVARCHAR(255))			+ N''', '''
					+ CAST(Scale AS NVARCHAR(255))				+ N''', '''
					+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''', '''
					+ FormatID									+ N''';'
			FROM [dbo].[sx_pf_gProductLines]
			WHERE ProductlineKey = @ProductLineKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,''' 
				+ PropertyID								+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](PropertyName)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)		+ N''','''
				+ Unit										+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](ValueText)			+ N''','''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''','''
				+ CAST(Scale AS NVARCHAR(255))				+ N''','''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''','''
				+ FormatID									+ N''';'
			FROM [dbo].[sx_pf_gProducts]
			WHERE  ProductlineKey = @ProductLineKey; 
						
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ ''','''
				+ [dbo].[sx_pf_pMaskSQL](NameShort)		+ N''','''
				+ [dbo].[sx_pf_pMaskSQL](NameLong)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentUser)	+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ [dbo].[sx_pf_pMaskSQL](ValueText)		+ N''',''' 
				+ [dbo].[sx_pf_pMaskSQL](ValueComment)	+ N''',''' 
				+ FormatID								+ N''';'
			FROM [dbo].[sx_pf_hListValues]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM [dbo].[sx_pf_dValueSeries] 
				WHERE ProductlineKey = @ProductLineKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM [dbo].[sx_pf_gProducts] 
				WHERE 
						ProductlineKey = @ProductLineKey
					AND PropertyID LIKE 'PDTRange_%'
			) 

			INSERT INTO @ExportTable 
			SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID									+ N''','''
				+ BackgroundColor							+ N''',''' 
				+ FontColor									+ N''',''' 
				+ ValueFormat								+ N''',''' 
				+ CAST(Bold				AS NVARCHAR(255))	+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))	+ N''','''
				+ AlignmentHorizontal						+ N''','''
				+ AlignmentVertical							+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToPrevious AS NVARCHAR(255))	+ N''';'
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (8000,1, N'GO');

		END -- of Output for SQL Server
		
	-- STEP 3.3 - OUTPUT Type for PF Client Execution ##################################################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'--DECLARE #Username NVARCHAR(255)		= ''SQL''')
				,(0, 1, N'--DECLARE #FactoryID NVARCHAR(255)		= ''' + @FactoryID		+ N'''')
				,(0, 1, N'--DECLARE #ProductLineID  NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
				,(0, 1, N'--This ProductlineID will be deleted during import, if it exists. You should be sure !');

		
			-- DELETE Productline if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'sx_pf_DELETE_Productline' + @spr + N'@Username' + @spr + N'#Username' + @spr + N'@FactoryID' + @spr + N'#FactoryID' + @spr + N'@ProductLineID' + @spr + N'#ProductLineID');
						
			-- POST Productline
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'sx_pf_POST_Productline' 
						+ @spr + N'@Username'					+ @spr + N'#Username' 
						+ @spr + N'@ProductlineID'				+ @spr + N'#ProductlineID' 
						+ @spr + N'@FactoryID'					+ @spr + N'#FactoryID'
						+ @spr + N'@NameShort'					+ @spr + NameShort 
						+ @spr + N'@NameLong'					+ @spr + NameLong  
						+ @spr + N'@CommentUser'				+ @spr + CommentUser  
						+ @spr + N'@CommentDev'					+ @spr + CommentDev  
						+ @spr + N'@ResponsiblePerson'			+ @spr + ResponsiblePerson  
						+ @spr + N'@ImageName'					+ @spr + ImageName  
						+ @spr + N'@DefaultTemplate'			+ @spr + DefaultTemplate  
						+ @spr + N'@GlobalAttributeSource1'		+ @spr + GlobalAttributeSource1  
						+ @spr + N'@GlobalAttributeAlias1'		+ @spr + GlobalAttributeAlias1  
						+ @spr + N'@GlobalAttributeSource2'		+ @spr + GlobalAttributeSource2  
						+ @spr + N'@GlobalAttributeAlias2'		+ @spr + GlobalAttributeAlias2  
						+ @spr + N'@GlobalAttributeSource3'		+ @spr + GlobalAttributeSource3  
						+ @spr + N'@GlobalAttributeAlias3'		+ @spr + GlobalAttributeAlias3 
						+ @spr + N'@GlobalAttributeSource4'		+ @spr + GlobalAttributeSource4  
						+ @spr + N'@GlobalAttributeAlias4'		+ @spr + GlobalAttributeAlias4 
						+ @spr + N'@GlobalAttributeSource5'		+ @spr + GlobalAttributeSource5  
						+ @spr + N'@GlobalAttributeAlias5'		+ @spr + GlobalAttributeAlias5 
						+ @spr + N'@GlobalAttributeSource6'		+ @spr + GlobalAttributeSource6  
						+ @spr + N'@GlobalAttributeAlias6'		+ @spr + GlobalAttributeAlias6 
						+ @spr + N'@GlobalAttributeSource7'		+ @spr + GlobalAttributeSource7  
						+ @spr + N'@GlobalAttributeAlias7'		+ @spr + GlobalAttributeAlias7 
						+ @spr + N'@GlobalAttributeSource8'		+ @spr + GlobalAttributeSource8  
						+ @spr + N'@GlobalAttributeAlias8'		+ @spr + GlobalAttributeAlias8 
						+ @spr + N'@GlobalAttributeSource9'		+ @spr + GlobalAttributeSource9  
						+ @spr + N'@GlobalAttributeAlias9'		+ @spr + GlobalAttributeAlias9 
						+ @spr + N'@GlobalAttributeSource10'	+ @spr + GlobalAttributeSource10  
						+ @spr + N'@GlobalAttributeAlias10'		+ @spr + GlobalAttributeAlias10 
						+ @spr + N'@GlobalAttributeSource11'	+ @spr + GlobalAttributeSource11  
						+ @spr + N'@GlobalAttributeAlias11'		+ @spr + GlobalAttributeAlias11  
						+ @spr + N'@GlobalAttributeSource12'	+ @spr + GlobalAttributeSource12  
						+ @spr + N'@GlobalAttributeAlias12'		+ @spr + GlobalAttributeAlias12  
						+ @spr + N'@GlobalAttributeSource13'	+ @spr + GlobalAttributeSource13  
						+ @spr + N'@GlobalAttributeAlias13'		+ @spr + GlobalAttributeAlias13 
						+ @spr + N'@GlobalAttributeSource14'	+ @spr + GlobalAttributeSource14  
						+ @spr + N'@GlobalAttributeAlias14'		+ @spr + GlobalAttributeAlias14 
						+ @spr + N'@GlobalAttributeSource15'	+ @spr + GlobalAttributeSource15  
						+ @spr + N'@GlobalAttributeAlias15'		+ @spr + GlobalAttributeAlias15 
						+ @spr + N'@GlobalAttributeSource16'	+ @spr + GlobalAttributeSource16  
						+ @spr + N'@GlobalAttributeAlias16'		+ @spr + GlobalAttributeAlias16 
						+ @spr + N'@GlobalAttributeSource17'	+ @spr + GlobalAttributeSource17  
						+ @spr + N'@GlobalAttributeAlias17'		+ @spr + GlobalAttributeAlias17 
						+ @spr + N'@GlobalAttributeSource18'	+ @spr + GlobalAttributeSource18  
						+ @spr + N'@GlobalAttributeAlias18'		+ @spr + GlobalAttributeAlias18 
						+ @spr + N'@GlobalAttributeSource19'	+ @spr + GlobalAttributeSource19  
						+ @spr + N'@GlobalAttributeAlias19'		+ @spr + GlobalAttributeAlias19 
						+ @spr + N'@GlobalAttributeSource20'	+ @spr + GlobalAttributeSource20  
						+ @spr + N'@GlobalAttributeAlias20'		+ @spr + GlobalAttributeAlias20 
						+ @spr + N'@GlobalAttributeSource21'	+ @spr + GlobalAttributeSource21  
						+ @spr + N'@GlobalAttributeAlias21'		+ @spr + GlobalAttributeAlias21  
						+ @spr + N'@GlobalAttributeSource22'	+ @spr + GlobalAttributeSource22  
						+ @spr + N'@GlobalAttributeAlias22'		+ @spr + GlobalAttributeAlias22  
						+ @spr + N'@GlobalAttributeSource23'	+ @spr + GlobalAttributeSource23  
						+ @spr + N'@GlobalAttributeAlias23'		+ @spr + GlobalAttributeAlias23 
						+ @spr + N'@GlobalAttributeSource24'	+ @spr + GlobalAttributeSource24  
						+ @spr + N'@GlobalAttributeAlias24'		+ @spr + GlobalAttributeAlias24 
						+ @spr + N'@GlobalAttributeSource25'	+ @spr + GlobalAttributeSource25  
						+ @spr + N'@GlobalAttributeAlias25'		+ @spr + GlobalAttributeAlias25 + N''
			FROM [dbo].[sx_pf_dProductlines] WHERE ProductlineKey = @ProductLineKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'sx_pf_POST_ProductEmpty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'' + ProductID + N'' 
					+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@TimeType'			+ @spr + TimeType
					+ @spr + N'@NameShort'			+ @spr + NameShort
					+ @spr + N'@NameLong'			+ @spr + NameLong
					+ @spr + N'@CommentUser'		+ @spr + CommentUser
					+ @spr + N'@CommentDev'			+ @spr + CommentDev
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson
					+ @spr + N'@ImageName'			+ @spr + ImageName
					+ @spr + N'@Status'				+ @spr + [Status] 
					+ @spr + N'@Template'			+ @spr + Template 
					+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion
					+ @spr + N'@GA1'				+ @spr + GlobalAttribute1
					+ @spr + N'@GA2'				+ @spr + GlobalAttribute2
					+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
					+ @spr + N'@GA4'				+ @spr + GlobalAttribute4
					+ @spr + N'@GA5'				+ @spr + GlobalAttribute5
					+ @spr + N'@GA6'				+ @spr + GlobalAttribute6
					+ @spr + N'@GA7'				+ @spr + GlobalAttribute7
					+ @spr + N'@GA8'				+ @spr + GlobalAttribute8
					+ @spr + N'@GA9'				+ @spr + GlobalAttribute9
					+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
					+ @spr + N'@GA11'				+ @spr + GlobalAttribute11
					+ @spr + N'@GA12'				+ @spr + GlobalAttribute12
					+ @spr + N'@GA13'				+ @spr + GlobalAttribute13
					+ @spr + N'@GA14'				+ @spr + GlobalAttribute14
					+ @spr + N'@GA15'				+ @spr + GlobalAttribute15
					+ @spr + N'@GA16'				+ @spr + GlobalAttribute16
					+ @spr + N'@GA17'				+ @spr + GlobalAttribute17
					+ @spr + N'@GA18'				+ @spr + GlobalAttribute18
					+ @spr + N'@GA19'				+ @spr + GlobalAttribute19
					+ @spr + N'@GA20'				+ @spr + GlobalAttribute20
					+ @spr + N'@GA21'				+ @spr + GlobalAttribute21
					+ @spr + N'@GA22'				+ @spr + GlobalAttribute22
					+ @spr + N'@GA23'				+ @spr + GlobalAttribute23
					+ @spr + N'@GA24'				+ @spr + GlobalAttribute24
					+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM [dbo].[sx_pf_dProducts] WHERE ProductlineKey = @ProductLineKey; 
									
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'sx_pf_POST_ValueSerie'
				+ @spr + N'@Username'				+ @spr + N'#Username' 
				+ @spr + N'@ProductID'				+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'			+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
				+ @spr + N'@ValueSeriesID'			+ @spr + ValueSeriesID 
				+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))
				+ @spr + N'@NameShort'				+ @spr + NameShort
				+ @spr + N'@NameLong'				+ @spr + NameLong
				+ @spr + N'@CommentUser'			+ @spr + CommentUser
				+ @spr + N'@CommentDev'				+ @spr + CommentDev
				+ @spr + N'@ImageName'				+ @spr + ImageName
				+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255))
				+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255))
				+ @spr + N'@ValueSource'			+ @spr + ValueSource
				+ @spr + N'@ValueListID'			+ @spr + ValueListID
				+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID
				+ @spr + N'@Unit'					+ @spr + Unit
				+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255))
				+ @spr + N'@Effect'					+ @spr + Effect
				+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM [dbo].[sx_pf_dValueSeries]
			WHERE  ProductlineKey = @ProductLineKey
			ORDER BY 
				 ProductLineID
				,ProductID
				,ValueSeriesNo;

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10))+',0,0,'+ FormatID +')],' -- with DeleteFlag, AddValue,FormatID 
					FROM [dbo].[sx_pf_dTime] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'sx_pf_POST_Timeline'
					+ @spr + N'@Username'							+ @spr + N'#Username' 
					+ @spr + N'@ProductID'							+ @spr + N'' + ProductID + N'' 
					+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
					+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' + N''
					+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + N'' + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;		
			
			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[('''	+ ValueSeriesID 
							+ N''','	+ CAST(TimeID	AS NVARCHAR(255)) 
							+ N','''	+ dbo.sx_pf_pRemoveLinebreaks(ValueFormula) 
							+ N''','''	+ CAST(ValueInt AS NVARCHAR(255)) 
							+ N''','''	+ ValueText 
							+ N''','''	+ ValueComment + ''')],' 
					FROM [dbo].[sx_pf_fValues] t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.ValueSeriesID, t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM [dbo].[sx_pf_dProducts] dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'sx_pf_POST_ProductDataTableValues'
				+ @spr + N'@Username'						+ @spr + N'#Username' 
				+ @spr + N'@ProductID'						+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'					+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'						+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'		+ @spr + N'0' + N''
				+ @spr + N'@ValuesInBracketsCommaSeparated' + @spr + N'' + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'sx_pf_POST_ProductlineProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gProductLines]
			WHERE ProductlineKey = @ProductLineKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 2001,1, N'sx_pf_POST_ProductProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@PropertyID'			+ @spr + PropertyID
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255)) 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_gProducts]
			WHERE  ProductlineKey = @ProductLineKey;
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'sx_pf_POST_List'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort
				+ @spr + N'@NameLong'		+ @spr + NameLong
				+ @spr + N'@CommentDev'		+ @spr + CommentDev
				+ @spr + N'@CommentUser'	+ @spr + CommentUser
				+ @spr + N'@Datentyp'		+ @spr + Datentyp
				+ @spr + N'@Source'			+ @spr + [Source]
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hLists]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'sx_pf_POST_ListValue'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListValuekey'	+ @spr + N'' + N'0'
				+ @spr + N'@ListID'			+ @spr + ListID
				+ @spr + N'@ValueInt'		+ @spr + CAST(ValueInt	AS NVARCHAR(255))
				+ @spr + N'@Scale'			+ @spr + CAST(Scale		AS NVARCHAR(255))
				+ @spr + N'@ValueText'		+ @spr + ValueText
				+ @spr + N'@ValueComment'	+ @spr + ValueComment
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM [dbo].[sx_pf_hListValues]
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM [dbo].[sx_pf_hListValues] 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM [dbo].[sx_pf_dValueSeries] 
				WHERE ProductlineKey = @ProductLineKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM [dbo].[sx_pf_gProducts] 
				WHERE 
						ProductlineKey = @ProductLineKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'sx_pf_POST_Format'
					 + @spr + N'@Username'				+ @spr + N'#Username'
					 + @spr + N'@FormatID'				+ @spr + FormatID
					 + @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					 + @spr + N'@FontColor'				+ @spr + FontColor
					 + @spr + N'@ValueFormat'			+ @spr + ValueFormat
					 + @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255)) 
					 + @spr + N'@Italic'				+ @spr + CAST(Italic			AS NVARCHAR(255))
					 + @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal 
					 + @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical 
					 + @spr + N'@TextUnderlined'		+ @spr + CAST(TextUnderlined	AS NVARCHAR(255)) 
					 + @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255)) 
					 + @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM [dbo].[sx_pf_hFormats]
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client	
	
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY StepNumber,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_ProductLine] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_EXPORT_ProductLine] TO pf_PlanningFactoryService;
GO



	
/*
Procedure to fill the Table control.tSystemFailureReport
Procedure will be build from many subsnipptes

Testcall 
EXEC control.spSystemFailureReport

SELECT * FROM control.tSystemFailureReport

*/


IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.spSystemFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE control.spSystemFailureReport
GO


CREATE PROCEDURE  control.spSystemFailureReport (@FactoryID NVARCHAR(255) = '', @ProductlineID NVARCHAR(255) = '')

AS

DECLARE @Timestamp 	Datetime 	= GETDATE()
DECLARE @TestName	NVARCHAR (255) 	= ''
DECLARE @TestSource	NVARCHAR (255) 	= ''


-- self cleaning older than 14 Days
DELETE FROM control.tSystemFailureReport WHERE Timestamp < DATEADD(day,-14,GETDATE());




SET @TestSource = 'Global_Lists'							-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for necessary System List which must exist.'			-- an friendly description of the test
-- this are the system list, a user could modify (ok) and delete (not ok)
-- there are more system list, but they are created internal and its not possible to modify

	BEGIN TRY

		WITH tmp AS
				(
				SELECT syslsts.ListID
				FROM (
					VALUES 
					  
					  (N'sxResponsiblePersons')
					, (N'sxStatus')
					, (N'sxUnits')
					, (N'sxValueEffectParameters')

				) syslsts (ListID) 
					LEFT JOIN dbo.sx_pf_hLists hL 
						ON syslsts.ListID = hL.ListID
				WHERE hL.ListID IS NULL
				)

	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID'	AS FactoryID
				,'AllPLID'	AS ProductlineID
				,'AllPID'	AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE'	AS ResultType

				,'List ' 
					+ tmp.ListID 
					+ ' must exist, but is missing.'
							AS ResultText

				,1			AS ResultNumeric

			FROM tmp
				
					

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Global_Lists'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ListValues which use not exiting Formats for conditional formating.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID' AS FactoryID
				,'AllPLID' AS ProductlineID
				,'AllPID' AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'List ' 
					+ hLV.ListID 
					+ ' uses for Value '
					+	CASE
							WHEN hL.Datentyp = 'String' THEN hLV.ValueText
							ELSE CAST(hLV.ValueInt AS NCHAR(10))
						END
					+ ' the Format ' 
					+ hLV.FormatID + ' which does not exists in Format definitions.'
						
				,1 AS ResultNumeric

			FROM dbo.sx_pf_hListValues hLV
				LEFT JOIN dbo.sx_pf_hLists hL
					ON hLV.ListID = hL.ListID

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM dbo.sx_pf_hFormats WHERE FormatID = hLV.FormatID
								)
					AND hLV.FormatID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Format definitions.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductlineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ dVS.ValueSeriesID
					+ ' uses Format ' 
					+ dVS.ValueFormatID + ' which does not exists in Format definitions.'

				,1 AS ResultNumeric

			FROM dbo.sx_pf_dValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM dbo.sx_pf_hFormats WHERE FormatID = dVS.ValueFormatID
								)

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found. '
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Lists as ValueSource.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductlineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ ValueSeriesID
					+ ' uses ValueList ' 
					+ ValueListID + ' which does not exists in List Management.'

				,1 AS ResultNumeric

			FROM dbo.sx_pf_dValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT ListID FROM dbo.sx_pf_hLists WHERE ListID = dVS.ValueListID
								)
					AND dVS.ValueListID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Productline'												-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for use of not existing List in Globalattributes.'	-- an friendly description of the test

	BEGIN TRY
				WITH GA AS (

					SELECT N'1' AS Num, FactoryID, ProductlineID, GlobalAttributeSource1 AS GlobalAttributeSource
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource1 <> N'' UNION ALL
					SELECT N'2', FactoryID, ProductlineID, GlobalAttributeSource2
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource2 <> N'' UNION ALL
					SELECT N'3', FactoryID, ProductlineID, GlobalAttributeSource3
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource3 <> N'' UNION ALL
					SELECT N'4', FactoryID, ProductlineID, GlobalAttributeSource4
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource4 <> N'' UNION ALL
					SELECT N'5', FactoryID, ProductlineID, GlobalAttributeSource5
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource5 <> N'' UNION ALL
					SELECT N'6', FactoryID, ProductlineID, GlobalAttributeSource6
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource6 <> N'' UNION ALL
					SELECT N'7', FactoryID, ProductlineID, GlobalAttributeSource7
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource7 <> N'' UNION ALL
					SELECT N'8', FactoryID, ProductlineID, GlobalAttributeSource8
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource8 <> N'' UNION ALL
					SELECT N'9', FactoryID, ProductlineID, GlobalAttributeSource9
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource9 <> N'' UNION ALL
					SELECT N'10', FactoryID, ProductlineID, GlobalAttributeSource10
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource10 <> N'' UNION ALL
					SELECT N'11', FactoryID, ProductlineID, GlobalAttributeSource11
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource11 <> N'' UNION ALL
					SELECT N'12', FactoryID, ProductlineID, GlobalAttributeSource12
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource12 <> N'' UNION ALL
					SELECT N'13', FactoryID, ProductlineID, GlobalAttributeSource13
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource13 <> N'' UNION ALL
					SELECT N'14', FactoryID, ProductlineID, GlobalAttributeSource14
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource14 <> N'' UNION ALL
					SELECT N'15', FactoryID, ProductlineID, GlobalAttributeSource15
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource15 <> N'' UNION ALL
					SELECT N'16', FactoryID, ProductlineID, GlobalAttributeSource16
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource16 <> N'' UNION ALL
					SELECT N'17', FactoryID, ProductlineID, GlobalAttributeSource17
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource17 <> N'' UNION ALL
					SELECT N'18', FactoryID, ProductlineID, GlobalAttributeSource18
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource18 <> N'' UNION ALL
					SELECT N'19', FactoryID, ProductlineID, GlobalAttributeSource19
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource19 <> N'' UNION ALL
					SELECT N'20', FactoryID, ProductlineID, GlobalAttributeSource20
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource20 <> N'' UNION ALL
					SELECT N'21', FactoryID, ProductlineID, GlobalAttributeSource21
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource21 <> N'' UNION ALL
					SELECT N'22', FactoryID, ProductlineID, GlobalAttributeSource22
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource22 <> N'' UNION ALL
					SELECT N'23', FactoryID, ProductlineID, GlobalAttributeSource23
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource23 <> N'' UNION ALL
					SELECT N'24', FactoryID, ProductlineID, GlobalAttributeSource24
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource24 <> N'' UNION ALL
					SELECT N'25', FactoryID, ProductlineID, GlobalAttributeSource25
					FROM dbo.sx_pf_dProductlines WHERE GlobalAttributeSource25 <> N''

					)
	
		INSERT INTO control.tSystemFailureReport
			
			SELECT 
				 @Timestamp,
				 GA.FactoryID
				,GA.ProductlineID
				,'AllPID' AS ProductID
			    ,@TestSource
				,@TestName
				,'FAILURE' AS ResultType -- or WARNING

				,'Not existing List ' + GA.GlobalattributeSource + ' is in use for Globalattribute ' + GA.Num AS ResultText

				,1 AS ResultNumeric -- (0 for Warning, positiv for Failure)

			FROM GA 
				LEFT JOIN dbo.sx_pf_hLists hL
					ON GA.GlobalattributeSource = hL.ListID
				WHERE hL.ListID IS NULL
			

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS WITHOUT CHANGE ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH




GO
/*
GET Operation for all ListValues a product needs for data entry in ProductDataTable
- contains the GUI defined lists of the products (defined by user c* or solution su*)
- contains the dynamic system lists (sx* - which are editable by the user)
- adds interal system lists (sx* - which are not visible on GUI)

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call - Must return Table and Code 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_AllProductListValues 
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';

PRINT @RC

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_AllProductListValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_AllProductListValues;
GO

CREATE PROCEDURE dbo.sx_pf_GET_AllProductListValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_AllProductListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
			
		-- Helper Table with all list values
		;WITH list AS (
			SELECT 
				hL.ListID
				,hL.Datentyp 
				,CAST(ISNULL(hLV.ValueInt	, 0) AS INT) AS ValueInt -- must return int !
                ,CAST(ISNULL(hLV.Scale		, 0) AS INT) AS Scale -- must return int !
				,ISNULL(hLV.ValueText		, N'')		 AS ValueText
				,ISNULL(hLV.ValueComment	, N'')		 AS ValueComment
				,ISNULL(hLV.FormatID		, N'')		 AS FormatID
				,ISNULL(hF.BackgroundColor	, N'')		 AS BackgroundColor
				,ISNULL(hF.FontColor		, N'')		 AS FontColor
				,ISNULL(hF.ValueFormat		, N'')		 AS ValueFormat
			FROM dbo.sx_pf_hLists hL 
				LEFT JOIN dbo.sx_pf_hListValues hLV	ON hL.ListID = hLV.ListID
				-- for the value
				LEFT JOIN dbo.sx_pf_hFormats hF	ON hLV.FormatID = hF.FormatID
				-- filter the lists, which where till 4.0.57 GUI defined and are now internal system lists
			WHERE hL.ListID NOT IN (
				'sxIsNumeric'
				,'sxVisibilityLevel'
				,'sxValueLists'
				,'sxValueSources'
				,'sxValueFormats'
				,'sxScales'
				,'sxValueEffects')
		)
		-- STEP 3.1 Dynamic System Lists
		SELECT *
		FROM list
		WHERE ListID IN ('sxUnits','sxValueEffectParameters')	
		
		UNION ALL
		-- STEP 3.2 Product specific ListSet based on defined ValueLists
		SELECT *
		FROM list
		WHERE ListID IN (
			SELECT DISTINCT ValueListID
			FROM dbo.sx_pf_dValueSeries
			WHERE ProductKey = @ProductKey)

		-- add internal system lists ----------------------------------------------------------
		UNION ALL 
		--sxIsNumeric - static  TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES 
				 ('sxIsNumeric','String',0,1,'0','','','','','')
				,('sxIsNumeric','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL	
		--sxVisibilityLevel - static TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES	
				 ('sxVisibilityLevel','String',0,1,'0','','','','','')
				,('sxVisibilityLevel','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		UNION ALL			
		--sxValueLists - dynamic List of GUI Lists
		SELECT 
			'sxValueLists'
			,'String'
			,0		AS ValueInt 
            ,1		AS Scale 
			,ListID AS ValueText
			,''		AS ValueComment
			,''		AS FormatID
			,''		AS BackgroundColor
			,''		AS FontColor
			,''		AS ValueFormat
		FROM dbo.sx_pf_hLists
			-- Filter the GUI Lists, which are not useful as ValueList
		WHERE ListID NOT IN ('sxValueEffectParameter')

		UNION ALL
		--sxValueSources - static (english strings for all languages
		SELECT * 
		FROM (
			VALUES	
				 ('sxValueSources','String',0,1,'Input'		 ,'','','','','')
				,('sxValueSources','String',0,1,'List-Strict','','','','','')
				,('sxValueSources','String',0,1,'XLS'		 ,'','','','','')
				,('sxValueSources','String',0,1,'XLS-Strict' ,'','','','','')
				,('sxValueSources','String',0,1,'Link'		 ,'','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueFormats - dynamic, list of defined formats
		SELECT 
			 'sxValueFormats'
			,'String'
			,0					AS ValueInt 
            ,1					AS Scale 
			,FormatID			AS ValueText
			,''					AS ValueComment
			,FormatID			AS FormatID
			,BackgroundColor	AS BackgroundColor
			,FontColor			AS FontColor
			,ValueFormat		AS ValueFormat
		FROM dbo.sx_pf_hFormats 
		
		UNION ALL
		--sxScales - static - this List is also static defined in GET_ListValues
		SELECT * 
		FROM (
			VALUES	
				 ('sxScales','Integer',1	,1,'','','','','','')
				,('sxScales','Integer',10	,1,'','','','','','')
				,('sxScales','Integer',100	,1,'','','','','','')
				,('sxScales','Integer',1000	,1,'','','','','','')
				,('sxScales','Integer',10000,1,'','','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueEffects -dynamic, list of defined ValueEffects in gValueEffects
		SELECT 
			 'sxValueEffects'
			,'String'
			,0			AS ValueInt 
            ,1			AS Scale 
			,EffectID	AS ValueText
			,''			AS ValueComment
			,''			AS FormatID
			,''			AS BackgroundColor
			,''			AS FontColor
			,''			AS ValueFormat
		FROM dbo.sx_pf_gValueEffects

		ORDER BY ListID, ValueText, ValueInt;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_AllProductListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_AllProductListValues;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryService;
GO

/*
GET operation for User status check on cluster
If User has an registered Username and his status is 'Active' then Cluster Access Rights 
are granted. (which still can mean he sees nothing than an empty cluster)


04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call

-- Should Return 200
DECLARE @RC AS NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterAccessRight 'SQL'
PRINT @RC

-- Should Return 403
DECLARE @RC AS NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterAccessRight 'SQL4'
PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterAccessRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ClusterAccessRight;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ClusterAccessRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution
	DECLARE @LogMessage			NVARCHAR(MAX)	= N'';

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterAccessRight;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			SET @LogMessage = 'Transaction user don`t exists, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END;
		
		-- STEP 3 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM dbo.sx_pf_rUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
		BEGIN
			SET @LogMessage = 'User is not active, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterAccessRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterAccessRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for Cluster Properties

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

--should return 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterProperties 'SQL'
PRINT @RC

--should return 403
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterProperties 'SQL4'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ClusterProperties
GO

CREATE PROCEDURE dbo.sx_pf_GET_ClusterProperties
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	 NVARCHAR(255)		= N'';
	DECLARE @Right				 NVARCHAR(255)		= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)		= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)		= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME			= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000)		= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperties;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = 'Write';
			END 
			ELSE 
			BEGIN 
				SET @Right = 'Read';
				SET @ResultCode = 200;
			END;
		END;
	
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;	
		
		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gCluster
		ORDER BY COALESCE(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperties;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a cluster Property

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterProperty 
	@Username	= 'SQL',
	@PropertyID = 'C01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ClusterProperty;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gCluster WHERE PropertyID = @PropertyID;

		IF @PropertyKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM dbo.sx_pf_gCluster
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
Operation to check Cluster
Running tests:
    Lists, which are never used
    ListMembers, which are never used
    Lists which are used but dont exists
    List members which are Used but dont exist
    Formats which are used but dont exist
    Formats which are defined but not used
    count ValueSeries per Product
    count ValuesPer Product
User must be active to run tests

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call					   
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterValidation 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterValidation') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ClusterValidation;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ClusterValidation
	@Username NVARCHAR(255) 
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(255)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterValidation;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
			  
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM dbo.sx_pf_rUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User is not active', 16, 10);
		
		-- STEP 3 - Run tests
		DECLARE @Tests AS TABLE (
			N INT IDENTITY(1,1)		NOT NULL, 
			[Name] NVARCHAR(200)	NOT NULL, 
			[Status] NVARCHAR(10)	NOT NULL,
			Result NVARCHAR(MAX)	NOT NULL,
			RowsCount BIGINT		NOT NULL);

-- Lists, which are never used			
		;WITH lst AS (
			SELECT l.ListID
				, l.ListKey
				, l.NameShort AS ListName
			FROM dbo.sx_pf_hLists l 
				LEFT JOIN (SELECT * FROM dbo.sx_pf_dValueSeries WHERE ValueListID <> N'') vs ON l.ListID = vs.ValueListID
			WHERE vs.ValueSeriesKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Lists', N'Info', N'List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + ListName + N'''', 1
		FROM lst;

		SET @EffectedRows += @@ROWCOUNT;

-- Lists which are used but dont exists	
		;WITH lst AS (
			SELECT vs.ValueListID	AS ListID
				, vs.NameShort AS VSName
				, vs.ValueSeriesKey AS VSKey
				, vs.ValueSeriesID AS VSID
			FROM (SELECT * FROM dbo.sx_pf_dValueSeries WHERE ValueListID <> N'') vs
				LEFT JOIN dbo.sx_pf_hLists  l ON l.ListID = vs.ValueListID
			WHERE l.ListID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists Lists', N'Error', N'ListID ' + ListID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), VSKey) + N', ' + VSID + N' ''' + VSName + N''')', 1
		FROM lst;
		
		SET @EffectedRows += @@ROWCOUNT;			  

-- ListMembers, which are never used		
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), lv.ListValueKey) + N' ''' 
				+ CASE WHEN l.Datentyp = N'INT' THEN CONVERT(NVARCHAR(MAX), lv.ValueInt) ELSE lv.ValueText END 
				+ N''' (List #' + CONVERT(NVARCHAR(MAX), l.ListKey) + N', ' + lv.ListID + N' ''' + l.NameShort + N''')' AS Result
			FROM dbo.sx_pf_hListValues lv 
				LEFT JOIN dbo.sx_pf_hLists l ON lv.ListID = l.ListID							
				LEFT JOIN dbo.sx_pf_dValueSeries vs ON l.ListID = vs.ValueListID AND ((vs.[IsNumeric] = 1 AND l.Datentyp = N'INT') OR (vs.[IsNumeric] = 0 AND l.Datentyp = N'String'))
				LEFT JOIN dbo.sx_pf_fValues v ON vs.ValueSeriesKey = v.ValueSeriesKey
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE v.ValueKey IS NULL
		 )
		 INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		 SELECT N'Unused List Values', N'Info', Result, 1
		 FROM mmbr;

		 SET @EffectedRows += @@ROWCOUNT;

--  List members which are Used but dont exist
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), v.ValueKey) + N' ''' + CASE WHEN vs.IsNumeric = 1 THEN CONVERT(NVARCHAR(MAX), v.ValueInt) ELSE v.ValueText END
				+ N''' ListID ' + vs.ValueListID 
				+ N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), v.ValueSeriesKey) + N', ' + vs.ValueSeriesID
				+ N' ''' + vs.NameShort + N''')' AS Result
			FROM dbo.sx_pf_fValues v
				INNER JOIN dbo.sx_pf_dValueSeries vs ON vs.ValueListID <> N'' AND vs.ValueSeriesKey = v.ValueSeriesKey
				LEFT JOIN dbo.sx_pf_hLists l ON vs.ValueListID = l.ListID AND ((vs.IsNumeric = 1 AND l.Datentyp = N'INT') OR (vs.IsNumeric = 0 AND l.Datentyp = N'String'))
				LEFT JOIN dbo.sx_pf_hListValues lv ON lv.ListID = l.ListID
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE lv.ListValueKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists List Values', N'Error', Result, 1
		FROM mmbr;

		SET @EffectedRows += @@ROWCOUNT;

		;WITH frm AS (									 
			SELECT ValueFormatID AS FormatID , N'FormatID ' + ValueFormatID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), ValueSeriesKey) + N', ' + ValueSeriesID + N' ''' + NameShort + N''')' AS Result			 
			FROM dbo.sx_pf_dValueSeries
			WHERE ValueFormatID <> N''
			UNION ALL												   
			SELECT FormatID , N'FormatID ' + FormatID + N' (Cluster Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM dbo.sx_pf_gCluster
			WHERE FormatID <> N'' 								  
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Factory Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result								   
			FROM dbo.sx_pf_gFactories
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ProductLine Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result							  		
			FROM dbo.sx_pf_gProductLines
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Product Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM dbo.sx_pf_gProducts
			WHERE FormatID <> N'' 
			UNION ALL										 
			SELECT FormatID , N'FormatID ' + FormatID + N' (List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + NameShort + N''')' AS Result
			FROM dbo.sx_pf_hLists
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ListValue #' + CONVERT(NVARCHAR(MAX), ListValueKey) + N', ListID ' + ListID + N')' AS Result
			FROM dbo.sx_pf_hListValues
			WHERE FormatID <> N'' 
		)
	-- Formats which are used but don`t exist
		, unusedFrmts AS (
			SELECT N'Format #' + CONVERT(NVARCHAR(MAX), FormatKey) + N', ' + FormatID AS Result												
			FROM dbo.sx_pf_hFormats
			WHERE FormatID NOT IN (SELECT DISTINCT FormatID FROM frm)
		)
	-- Formats which are defined but not used	
		, notExistsFrmts AS (
			SELECT Result 
			FROM frm LEFT JOIN dbo.sx_pf_hFormats f ON frm.FormatID	= f.FormatID
			WHERE f.FormatID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Formats', N'Info', Result, 1
		FROM unusedFrmts
		UNION ALL
		SELECT N'Don`t exists Formats', N'Error', Result, 1
		FROM notExistsFrmts;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValueSeries per Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'ValueSeries per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM dbo.sx_pf_dValueSeries	vs INNER JOIN dbo.sx_pf_dProducts p ON vs.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValuesPer Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Values per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM dbo.sx_pf_fValues v INNER JOIN dbo.sx_pf_dProducts p ON v.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;								   	   
		
		SET @EffectedRows += @@ROWCOUNT;

		SELECT * 
		FROM @Tests
		ORDER BY N;

		COMMIT TRANSACTION sx_pf_GET_ClusterValidation;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterValidation;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryService;
GO
/*
GET Operation for Write Rights on a Cluster
Returns 200 if User has ClusterWriteRights

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ClusterWriteRight 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterWriteRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ClusterWriteRight;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ClusterWriteRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterWriteRight;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
				
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 3 - Test that user has write access to the cluster
		SELECT @ResultCode = IIF([Right] = N'Write', 200, 401) FROM dbo.sx_pf_vUserRights WHERE FactoryID = N'' AND Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User have no write access to the cluster', 16, 10);

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterWriteRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterWriteRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;

END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for all eligible Factories
Returns the list of all the factories, for which the user has rights

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call - should return Table and 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Factories 'SQL'
PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Factories') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Factories;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Factories
	@Username NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factories;

		-- STEP 0.1 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 3. Show public values	
		;WITH rights AS (
			SELECT 
				FactoryID,  
				[Right],
				ISNULL(TRY_CAST(FactoryID AS INT), 999999999) AS orderID 
			FROM dbo.sx_pf_vUserRights
			WHERE ProductLineID = N'' 
				AND Username = @TransactUsername
				AND [Right] IN (N'Read', N'Write')
		)
		SELECT   
			dF.FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,vUR.[Right]
		FROM dbo.sx_pf_dFactories dF 
			INNER JOIN rights vUR ON dF.FactoryID = vUR.FactoryID	
		ORDER BY vUR.orderID, dF.FactoryID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Factories;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factories;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Factories TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Factories TO pf_PlanningFactoryService;
GO
/*
GET Operation for one Factory
Delivers all informations about one factory

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

-- test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Factory
	@Username	= 'SQL',
	@FactoryID	= 'ZT'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Factory;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Factory
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,@Right AS 'Right'
		FROM dbo.sx_pf_dFactories
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Factory TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a Factory Properties

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_FactoryProperties 
	@Username	= 'SQL',
	@FactoryID	= 'D'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FactoryProperties;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FactoryProperties
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N''		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gFactories
		WHERE FactoryKey = @FactoryKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a Factory Property

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_FactoryProperty 
	@Username	= 'SQL',
	@FactoryID	= 'D',
	@PropertyID = 'F01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FactoryProperty;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				 = 0;
	DECLARE @PropertyKey		INT				 = 0;
	DECLARE @Right				NVARCHAR(10)	 = N'';
	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	 = N'''' 
			+ ISNULL(@Username,		 N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	 + N''',''' 
			+ ISNULL(@PropertyID,	 N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;				-- SET during Execution
	DECLARE @ResultCode			INT				 = 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username		= N'';
	IF @FactoryID	IS NULL SET @FactoryID		= N'';
	IF @PropertyID	IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);	

		IF	@FactoryID = N'' 
			OR @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF @FactoryKey = 0 OR @PropertyKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gFactories
		WHERE PropertyKey = @PropertyKey; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining the Presentation Content of a Factory

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Supports three types of content:
	Pivot:				Data from an StoredProcedure is displayed as Pivot Table
	Pivot_Chart:		Data from an StoredProcedure is displayed as Pivot Chart
	HTML_URL:			HTML Content based on a given URL is displayed
	HTML_Static:		HTML Content based on direct submitted HTML Code from an StoredProcedure is displayed
	ProductlineList:	List of Productlines inside the Factory

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_FactoryTabs 
	@Username = 'SQL',
	@FactoryID = 'ZT'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryTabs') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FactoryTabs;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FactoryTabs
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryTabs;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID (@FactoryID);

		IF @FactoryID = N''		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- Show Properties of Type TAB*
		SELECT 
			 PropertyID		AS TabID				-- PropertyID	- ID of the Tab, also ORDER BY Criteria (TAB01, TAB02...)
			,PropertyName	AS Label				-- PropertyName - Label for the tab
			,CommentUser	AS CommentUser			-- CommentUser	- displayed as HeaderComment for Pivottables
			,Unit			AS PresentationType		-- Unit			- Pivot, Pivot_Chart HTML_URL, HTML_Static,ProductlineList
			,ValueText		AS Source				-- ValueText	- URL oder SQL 
			,CommentDev		AS Layout				-- CommentDev	- String for saved Layout 
			
			--TODO !! - Layout must be handled in Backend as Userdependend or not ! - maybe we use this Layout here as Standard and give the possibility to save another one for user
		FROM dbo.sx_pf_gFactories
		WHERE 
				FactoryID = @FactoryID 
			AND PropertyID LIKE 'TAB%'

		ORDER BY PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryTabs;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryTabs;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryTabs TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryTabs TO pf_PlanningFactoryService;
/*
GET Operation for receiving a filterd context sensitiv List with dynamic column count
The first column is sended back with POST_FlexAction as idenitfier for a database action

Procedure must be editend to fit user needs and recreated after each API Update.

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call
--should return Data and 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_FlexAction
	 @Username				= 'SQL'
	,@FactoryID				= 'F1'
	,@ProductLineID			= 'PL1'
	,@ProductID				= 'P1'
	,@ValueSeriesID			= 'Projekt'
	,@TimeID				= 20180115
	,@GlobalattributeNumber = ''
	,@PageType				= ''
	,@SearchString			= 'ei';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_GET_FlexAction') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FlexAction;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				

	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,				 N'NULL')					+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL')					+ N''','
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL')					+ N''','''	
			+ ISNULL(@PageType,				 N'NULL')					+ N''',''' 
			+ ISNULL(@SearchString,			 N'NULL')					+ N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= N'';
	IF @PageType				IS NULL SET @PageType				= N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
	
		-- STEP 3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Projects AS TABLE
		(
			ProjectID		NVARCHAR (255) NOT NULL
			,ProjectName	NVARCHAR (255) NOT NULL
			,ProjectManager NVARCHAR (255) NOT NULL
		);

		INSERT INTO @Projects
		VALUES
			(N'P1000',N'Testprojekt in Aachen','Pit Brown'),
			(N'P1001',N'Testprojekt in Augsburg','Pit Brown'),
			(N'P1002',N'Testprojekt in Bergisch Gladbach','Pit Brown'),
			(N'P1003',N'Testprojekt in Berlin','Pit Brown'),
			(N'P1004',N'Testprojekt in Bielefeld','Pit Brown'),
			(N'P1005',N'Testprojekt in Bochum','Pit Brown'),
			(N'P1006',N'Testprojekt in Bonn','Pit Brown'),
			(N'P1007',N'Testprojekt in Bottrop','Pit Brown'),
			(N'P1008',N'Testprojekt in Braunschweig','Pit Brown'),
			(N'P1009',N'Testprojekt in Bremen','Pit Brown'),
			(N'P1010',N'Testprojekt in Bremerhaven','Pit Brown'),
			(N'P1011',N'Testprojekt in Chemnitz','Pit Brown'),
			(N'P1012',N'Testprojekt in Darmstadt','Pit Brown'),
			(N'P1013',N'Testprojekt in Dortmund','Pit Brown'),
			(N'P1014',N'Testprojekt in Dresden','Pit Brown'),
			(N'P1015',N'Testprojekt in Duisburg','Pit Brown'),
			(N'P1016',N'Testprojekt in D�sseldorf','Pit Brown'),
			(N'P1017',N'Testprojekt in Erfurt','Pit Brown'),
			(N'P1018',N'Testprojekt in Erlangen','Pit Brown'),
			(N'P1019',N'Testprojekt in Essen','Pit Brown'),
			(N'P1020',N'Testprojekt in Frankfurt am Main','Pit Brown'),
			(N'P1021',N'Testprojekt in Freiburg im Breisgau','Pit Brown'),
			(N'P1022',N'Testprojekt in F�rth','Pit Brown'),
			(N'P1023',N'Testprojekt in Gelsenkirchen','Pit Brown'),
			(N'P1024',N'Testprojekt in G�ttingen','Pit Brown'),
			(N'P1025',N'Testprojekt in Hagen','Pit Brown'),
			(N'P1026',N'Testprojekt in Halle (Saale)','Pit Brown'),
			(N'P1027',N'Testprojekt in Hamburg','Pit Brown'),
			(N'P1028',N'Testprojekt in Hamm','Pit Brown'),
			(N'P1029',N'Testprojekt in Hannover','Pit Brown'),
			(N'P1030',N'Testprojekt in Heidelberg','Cathy Gold'),
			(N'P1032',N'Testprojekt in Heilbronn','Cathy Gold'),
			(N'P1032',N'Testprojekt in Herne','Cathy Gold'),
			(N'P1033',N'Testprojekt in Ingolstadt','Cathy Gold'),
			(N'P1034',N'Testprojekt in Jena','Cathy Gold'),
			(N'P1035',N'Testprojekt in Karlsruhe','Cathy Gold'),
			(N'P1036',N'Testprojekt in Kassel','Cathy Gold'),
			(N'P1037',N'Testprojekt in Kiel','Cathy Gold'),
			(N'P1038',N'Testprojekt in Koblenz','Cathy Gold'),
			(N'P1039',N'Testprojekt in K�ln','Cathy Gold'),
			(N'P1040',N'Testprojekt in Krefeld','Cathy Gold'),
			(N'P1041',N'Testprojekt in Leipzig','Cathy Gold'),
			(N'P1042',N'Testprojekt in Leverkusen','Cathy Gold'),
			(N'P1043',N'Testprojekt in L�beck','Cathy Gold'),
			(N'P1044',N'Testprojekt in Ludwigshafen am Rhein','Cathy Gold'),
			(N'P1045',N'Testprojekt in Magdeburg','Cathy Gold'),
			(N'P1046',N'Testprojekt in Mainz','Cathy Gold'),
			(N'P1047',N'Testprojekt in Mannheim','Cathy Gold'),
			(N'P1048',N'Testprojekt in Moers','Cathy Gold'),
			(N'P1048',N'Testprojekt in M�nchengladbach','Cathy Gold'),
			(N'P1049',N'Testprojekt in M�lheim an der Ruhr','Cathy Gold'),
			(N'P1050',N'Testprojekt in M�nchen','Cathy Gold'),
			(N'P1051',N'Testprojekt in M�nster','Cathy Gold'),
			(N'P1052',N'Testprojekt in Neuss','Cathy Gold'),
			(N'P1053',N'Testprojekt in N�rnberg','Cathy Gold'),
			(N'P1054',N'Testprojekt in Oberhausen','Cathy Gold'),
			(N'P1055',N'Testprojekt in Offenbach am Main','Cathy Gold'),
			(N'P1056',N'Testprojekt in Oldenburg','Cathy Gold'),
			(N'P1057',N'Testprojekt in Osnabr�ck','Cathy Gold'),
			(N'P1058',N'Testprojekt in Paderborn','Cathy Gold'),
			(N'P1059',N'Testprojekt in Pforzheim','Cathy Gold'),
			(N'P1060',N'Testprojekt in Potsdam','Cathy Gold'),
			(N'P1061',N'Testprojekt in Recklinghausen','Cathy Gold'),
			(N'P1062',N'Testprojekt in Regensburg','Cathy Gold'),
			(N'P1063',N'Testprojekt in Remscheid','Cathy Gold'),
			(N'P1064',N'Testprojekt in Reutlingen','Cathy Gold'),
			(N'P1065',N'Testprojekt in Rostock','Cathy Gold'),
			(N'P1066',N'Testprojekt in Saarbr�cken','Cathy Gold'),
			(N'P1067',N'Testprojekt in Siegen','Cathy Gold'),
			(N'P1068',N'Testprojekt in Solingen','Cathy Gold'),
			(N'P1069',N'Testprojekt in Stuttgart','Cathy Gold'),
			(N'P1070',N'Testprojekt in Trier','Cathy Gold'),
			(N'P1071',N'Testprojekt in Ulm','Cathy Gold'),
			(N'P1072',N'Testprojekt in Wiesbaden','Cathy Gold'),
			(N'P1073',N'Testprojekt in Wolfsburg','Cathy Gold'),
			(N'P1074',N'Testprojekt in Wuppertal','Cathy Gold'),
			(N'P1075',N'Testprojekt in W�rzburg','Cathy Gold');
		
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Second Column must have identifier character, its send back with the POST_FlexAction Command
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R= Right, C = Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter

		-- React on content parameters when creating the return list.
		-- You could add a "*" Member to the List and react on it in POST to create a new list member

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			SELECT 'Projekt wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS ProjectID_RS50
				,p.ProjectName AS Projektname_LS300
				,p.ProjectManager AS Projektmanager_CS150
			FROM @Projects p
			WHERE p.ProjectName LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			SELECT 'Mitarbeiter wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS MitarbeiterID_RS50
				,p.ProjectManager AS Projektmanager_LS150
			FROM @Projects p
			WHERE p.ProjectManager LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter')
		BEGIN
			SELECT 'FlexActions sind nur f�r die Wertreihen "Projekt" und "Mitarbeiter" definiert' AS Hint;
		END

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexAction;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryService
GO
/*
GET Operation for receiving a context sensitiv List
API deliviers List Values fitting to the sended context Parameters
Procedure must be editend to fit user needs and recreated after each API Update
When updating this procedure, you must reset the rights for it

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
--should return Data and 200
EXEC @RC = dbo.sx_pf_GET_FlexDrill 
	@Username				= 'SQL'
	,@FactoryID				= '1'
	,@ProductLineID			= '1'
	,@ProductID				= '1'
	,@ValueSeriesID			= '1'
	,@GlobalattributeNumber = ''
	,@PageType				= ''
	,@SearchString			= 'r';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FlexDrill') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FlexDrill;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FlexDrill
	 @Username				 NVARCHAR(255)
	,@FactoryID				 NVARCHAR(255)
	,@ProductLineID			 NVARCHAR(255)
	,@ProductID				 NVARCHAR(255)
	,@ValueSeriesID			 NVARCHAR(255)
	,@GlobalattributeNumber  NVARCHAR(255)
	,@PageType				 NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			 NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	DECLARE @ParameterString	NVARCHAR(MAX)	=		N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			 N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL') + N''',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL') + N''',''' 
			+ ISNULL(@PageType,				 N'NULL') + N''',''' 
			+ ISNULL(@SearchString,			 N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username			  IS NULL SET @Username				 = N'';
	IF @FactoryID			  IS NULL SET @FactoryID			 = N'';
	IF @ProductLineID		  IS NULL SET @ProductlineID		 = N'';
	IF @ProductID			  IS NULL SET @ProductID			 = N'';
	IF @ValueSeriesID		  IS NULL SET @ValueSeriesID		 = N'';
	IF @GlobalattributeNumber IS NULL SET @GlobalattributeNumber = N'';
	IF @PageType			  IS NULL SET @PageType				 = N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexDrill;

		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSeriesID	 = dbo.sx_pf_pProtectID		 (@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
	
		-- STEP 3.1 - Sample for creating a List for a specific ValueSeries in all Products from specific template
		IF @ValueSeriesID = N'that' 
			AND EXISTS(
				SELECT Template FROM dbo.sx_pf_dProducts 
				WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 'ValueSeriesSpecificList' AS Dummy;
			SET @EffectedRows = 1;
		END;

		-- STEP 3.2 - Sample for creating a List for a Globalattribute in one Productline
		IF @FactoryID = N'this' AND @ProductlineID = N'that' AND @GlobalattributeNumber = 5
		BEGIN
			SELECT 'ProductlineSpecificList for ValueSeries'; 
			SET @EffectedRows += 1;
		END;
		
		-- STEP 3.3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

		INSERT INTO @Cities VALUES
			(N'Aachen'),
			(N'Augsburg'),
			(N'Bergisch Gladbach'),
			(N'Berlin'),
			(N'Bielefeld'),
			(N'Bochum'),
			(N'Bonn'),
			(N'Bottrop'),
			(N'Braunschweig'),
			(N'Bremen'),
			(N'Bremerhaven'),
			(N'Chemnitz'),
			(N'Darmstadt'),
			(N'Dortmund'),
			(N'Dresden'),
			(N'Duisburg'),
			(N'Düsseldorf'),
			(N'Erfurt'),
			(N'Erlangen'),
			(N'Essen'),
			(N'Frankfurt am Main'),
			(N'Freiburg im Breisgau'),
			(N'Fürth'),
			(N'Gelsenkirchen'),
			(N'Göttingen'),
			(N'Hagen'),
			(N'Halle (Saale)'),
			(N'Hamburg'),
			(N'Hamm'),
			(N'Hannover'),
			(N'Heidelberg'),
			(N'Heilbronn'),
			(N'Herne'),
			(N'Ingolstadt'),
			(N'Jena'),
			(N'Karlsruhe'),
			(N'Kassel'),
			(N'Kiel'),
			(N'Koblenz'),
			(N'Köln'),
			(N'Krefeld'),
			(N'Leipzig'),
			(N'Leverkusen'),
			(N'Lübeck'),
			(N'Ludwigshafen am Rhein'),
			(N'Magdeburg'),
			(N'Mainz'),
			(N'Mannheim'),
			(N'Moers'),
			(N'Mönchengladbach'),
			(N'Mülheim an der Ruhr'),
			(N'München'),
			(N'Münster'),
			(N'Neuss'),
			(N'Nürnberg'),
			(N'Oberhausen'),
			(N'Offenbach am Main'),
			(N'Oldenburg'),
			(N'Osnabrück'),
			(N'Paderborn'),
			(N'Pforzheim'),
			(N'Potsdam'),
			(N'Recklinghausen'),
			(N'Regensburg'),
			(N'Remscheid'),
			(N'Reutlingen'),
			(N'Rostock'),
			(N'Saarbrücken'),
			(N'Siegen'),
			(N'Solingen'),
			(N'Stuttgart'),
			(N'Trier'),
			(N'Ulm'),
			(N'Wiesbaden'),
			(N'Wolfsburg'),
			(N'Wuppertal'),
			(N'Würzburg');

		SELECT 
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R=Right, C=Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter
			'List of Cities to select.'	 AS Hint
			,UPPER(LEFT([Name], 4))		 AS CityCode_LS50
			,[Name]						 AS CityName_CS200
			,'Beschreibung dieser Stadt...' AS Beschreibung_LS300
			,500000						 AS Einwohnerzahl_LM50
		 FROM @Cities
		 WHERE [Name] LIKE @SearchString
		 ORDER BY [Name];

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexDrill;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexDrill;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryService
GO
/*
GET Operation for receiving a context sensitiv List
API deliviers List Values fitting to the sended context Parameters
Procedure must be editend to fit user needs and recreated after each API Update
When updating this procedure, you must reset the rights for it

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC AS NVARCHAR(255)
--should return Data and 200
EXEC @RC = dbo.sx_pf_GET_FlexList 'SQL','1','1','1','1','','','',''
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FlexList') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_FlexList;
GO

CREATE PROCEDURE dbo.sx_pf_GET_FlexList
	@Username				NVARCHAR(255)
	,@FactoryID 			NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @GlobalattributeAlias	NVARCHAR(255)	= N'';
	DECLARE @SearchDoneFlag			INT				= 0;
	DECLARE @ProductKey				BIGINT			= 0;
	DECLARE @DummyTable TABLE (Dummy NVARCHAR(255) NOT NULL)  -- only for the samples to compile
	DECLARE @DummyInt				INT
	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductID,				N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,			N'NULL') + N''','
			+ CAST(ISNULL(@TimeID,				N'NULL') AS NVARCHAR(255)) + N','''
			+ ISNULL(@GlobalattributeNumber,	N'NULL') + N''',''' 
			+ ISNULL(@PageType,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexList;

		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Preparing of useful context informations 
		-- ****************************************************************************************************************************************************
		-- Try to determine ProductKey
		IF @ProductID			<> N'' 
			AND @ProductLineID	<> N'' 
			AND @FactoryID		<> N''
		BEGIN
			SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
		END

		-- Determine AliasName of Globalattribute, to define actions for Globalattributes by name
		IF @GlobalattributeNumber <> ''
		BEGIN
			IF OBJECT_ID('tempdb..#GANames') IS NOT NULL
				DROP TABLE #GANames;

			CREATE TABLE #GANames
			(
				RowKey					BIGINT IDENTITY (1,1)	NOT NULL
				,FactoryID				NVARCHAR (255)			NOT NULL
				,ProductLineID			NVARCHAR (255)			NOT NULL
				,Globalattribute		NVARCHAR (255) 			NOT NULL
				,GlobalattributeNumber	INT						NOT NULL
				,Aliasname				NVARCHAR (255)			NOT NULL
			)
			
			INSERT INTO #GANames
			SELECT
				FactoryID
				,ProductLineID
				,Globalattribute
				,COALESCE(TRY_CAST(RIGHT(Globalattribute,2) AS INT), TRY_CAST(RIGHT(Globalattribute,1) AS INT)) AS GlobalattributeNumber
				,Aliasname
			FROM dbo.sx_pf_dProductLines
			UNPIVOT
			(
				Aliasname FOR Globalattribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalattributeAlias3, GlobalattributeAlias4, GlobalattributeAlias5
					,GlobalattributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10
					,GlobalattributeAlias11, GlobalattributeAlias12, GlobalattributeAlias13, GlobalattributeAlias14, GlobalattributeAlias15
					,GlobalattributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20
					,GlobalattributeAlias21, GlobalattributeAlias22, GlobalattributeAlias23, GlobalattributeAlias24, GlobalattributeAlias25
				)
			) unpiv
			WHERE 
					FactoryID		= @FactoryID 
				AND	ProductlineID	= @ProductLineID;

			SELECT @GlobalattributeAlias = Aliasname FROM #GANames WHERE GlobalattributeNumber = @GlobalattributeNumber;
		END

		/*
		DEFINITION OF THE FLEX LISTS
		*****************************************************************************************************************************************************************
		First Column must be "Hint" an commits the display hint to the frontend
		Second Column must have identifier character, this value is sended back to the selected cell
		Column Names must have an Alias (AS Displayname) with ONLY ONE "_", after which 
			Orientation		(L=Left, R= Right, C = Center)
			DataType		(M=Money, S=String, I=Int) 
			ColumnWidth		(Integer Number)
		is coded as Frontend Parameter, e.g. Name_LS50  means the column ist labeled Name, has left orientation,
		is formated as sting an 50px width
		*/

		-- Sample for creating a List for all ValueSeries with a specific ID
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable; 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;
		
		-- Sample for creating a List for all ValueSeries with a specific ID in a specific Template
		IF @ValueSeriesID = N'SKTO' AND EXISTS
		(
			SELECT Template 
			FROM dbo.sx_pf_dProducts 
			WHERE 
				ProductKey = @ProductKey 
				AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;

		-- Sample for Seach over parameter Product
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT DISTINCT
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,fV.ValueText	AS KontenID_LS40
				,fV2.ValueText	AS KontenName_LS180
			FROM dbo.sx_pf_fValues fV 
				-- Kontoname aus anderer Spalte
				LEFT JOIN
					(
						SELECT
							TimeID
							,ValueText 
						FROM dbo.sx_pf_fValues 
						WHERE 
								FactoryID = 'ZP'
							AND ProductLineID = '1'
							AND ProductID = '1'
							AND ValueSeriesID = 'KTO_Name'
						
					) fV2 ON fV.TimeID = fV2.TimeID
			WHERE 
				fV.FactoryID = 'ZP'
				AND fV.ProductLineID = '1'
				AND fV.ProductID = '1'
				AND fV.ValueSeriesID = 'KTO_ID'
				AND (fV.ValueText LIKE @SearchString OR fV2.ValueText LIKE @SearchString)
			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END

		-- Samples for other context
		IF	@GlobalattributeAlias = N'SKTO'		-- for this Globalattribute
			OR LEFT(@ValueSeriesID, 3) = N'KST' -- for all ValueSeries starting with this letters
		BEGIN	
			SET @DummyInt = 1 
		END	
		
		-- Sample Fallback -- delete in productive use to change fallback to Productive Fallback
		-- **************************************************************************************************************************************
		IF @SearchDoneFlag  = 0
		BEGIN
			DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

			INSERT INTO @Cities 
			VALUES
				(N'Aachen'),
				(N'Augsburg'),
				(N'Bergisch Gladbach'),
				(N'Berlin'),
				(N'Bielefeld'),
				(N'Bochum'),
				(N'Bonn'),
				(N'Bottrop'),
				(N'Braunschweig'),
				(N'Bremen'),
				(N'Bremerhaven'),
				(N'Chemnitz'),
				(N'Darmstadt'),
				(N'Dortmund'),
				(N'Dresden'),
				(N'Duisburg'),
				(N'Düsseldorf'),
				(N'Erfurt'),
				(N'Erlangen'),
				(N'Essen'),
				(N'Frankfurt am Main'),
				(N'Freiburg im Breisgau'),
				(N'Fürth'),
				(N'Gelsenkirchen'),
				(N'Göttingen'),
				(N'Hagen'),
				(N'Halle (Saale)'),
				(N'Hamburg'),
				(N'Hamm'),
				(N'Hannover'),
				(N'Heidelberg'),
				(N'Heilbronn'),
				(N'Herne'),
				(N'Ingolstadt'),
				(N'Jena'),
				(N'Karlsruhe'),
				(N'Kassel'),
				(N'Kiel'),
				(N'Koblenz'),
				(N'Köln'),
				(N'Krefeld'),
				(N'Leipzig'),
				(N'Leverkusen'),
				(N'Lübeck'),
				(N'Ludwigshafen am Rhein'),
				(N'Magdeburg'),
				(N'Mainz'),
				(N'Mannheim'),
				(N'Moers'),
				(N'Mönchengladbach'),
				(N'Mülheim an der Ruhr'),
				(N'München'),
				(N'Münster'),
				(N'Neuss'),
				(N'Nürnberg'),
				(N'Oberhausen'),
				(N'Offenbach am Main'),
				(N'Oldenburg'),
				(N'Osnabrück'),
				(N'Paderborn'),
				(N'Pforzheim'),
				(N'Potsdam'),
				(N'Recklinghausen'),
				(N'Regensburg'),
				(N'Remscheid'),
				(N'Reutlingen'),
				(N'Rostock'),
				(N'Saarbrücken'),
				(N'Siegen'),
				(N'Solingen'),
				(N'Stuttgart'),
				(N'Trier'),
				(N'Ulm'),
				(N'Wiesbaden'),
				(N'Wolfsburg'),
				(N'Wuppertal'),
				(N'Würzburg');
		
			SELECT 
				'List of Cities, you can search with substring.' AS Hint
				,UPPER(LEFT([Name], 4)) AS CityCode_LS50
				,[Name] AS CityName_CS200
			FROM @Cities
		 	WHERE [Name] LIKE @SearchString
		 	ORDER BY [Name];
			
			SET @SearchDoneFlag = 1
		END

		-- Productive Fallback - if no flex search matched the query
		-- **************************************************************************************
		IF @SearchDoneFlag = 0 
		BEGIN
			SELECT 
				'For this Seach Context, we sadly must tell to have no fitting content to offer. ' AS Hint
				,'Nothing' AS Content_LS50
				,'is all I have to offer for you.' AS Sorry_LS200
		END

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexList;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexList;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexList TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexList TO pf_PlanningFactoryService
GO
/*
GET Operation for a Format

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Format 
	@Username = 'SQL', 
	@FormatID = 'HintergrundRot'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Format;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			 INT			= 0;
	DECLARE @Right				 NVARCHAR(10)	= '';
	DECLARE @TransactUsername	 NVARCHAR(255)	= '';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@FormatID, N'NULL')		+ N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT  @FormatKey = FormatKey FROM dbo.sx_pf_hFormats WHERE FormatID = @FormatID;
		
		IF @FormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM dbo.sx_pf_hFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Format TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Formats in Cluster

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Formats 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Formats') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Formats;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Formats
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				 NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;						
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Formats;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
	
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM dbo.sx_pf_hFormats
		ORDER BY ISNULL(TRY_CAST(FormatID AS INT), 999999999), FormatID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Formats;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Formats;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Formats TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Formats TO pf_PlanningFactoryService;
GO
/*
GET Operation for receiving full text search results for a given textstring
Search will be done in all DataFactory Elements

10/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC AS NVARCHAR(255)
--should return Data and 200
EXEC @RC = dbo.sx_pf_GET_GlobalSearch 'SQL','Unikum'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_GlobalSearch') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_GlobalSearch;
GO

CREATE PROCEDURE dbo.sx_pf_GET_GlobalSearch
	 @Username				NVARCHAR(255)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SearchDoneFlag			INT				= 0;

	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username				= N'';
	
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_GlobalSearch;

		-- Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Table for Structure Cache
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#Structure') IS NOT NULL
				DROP TABLE #Structure;

			CREATE TABLE #Structure
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryKey					BIGINT											 NOT NULL
				,ProductlineKey				BIGINT											 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,ValueSeriesKey				BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductlineNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #Structure
			-- must also return Factories / Productlines without Products
				SELECT
					  dF.FactoryKey
					 ,ISNULL(dPL.ProductlineKey	,-1)					AS ProductlineKey
					 ,ISNULL(dP.ProductKey		,-1)					AS ProductKey
					 ,ISNULL(dVS.ValueSeriesKey	,-1)					AS ValueSeriesKey
					 ,dF.FactoryID		
					 ,ISNULL(dPL.ProductLineID	,'-1')					AS ProductlineID
					 ,ISNULL(dP.ProductID		,'-1')					AS ProductID
					 ,ISNULL(dVS.ValueSeriesID	,'-1')					AS ValueSeriesID
					 ,dF.NameShort										AS FactoryNameShort
					 ,ISNULL(dPL.NameShort	,'<without Productline>')	AS ProductlineNameShort
					 ,ISNULL(dP.NameShort	,'<without Product>')		AS ProductNameShort
					 ,ISNULL(dVS.NameShort	,'<without ValueSeries>')	AS ValueSeriesNameShort

			  FROM  dbo.sx_pf_dFactories dF

					LEFT JOIN dbo.sx_pf_dProductLines dPL
						ON dF.FactoryKey = dPL.FactoryKey

					LEFT JOIN dbo.sx_pf_dProducts dP
						ON dPL.ProductlineKey = dP.ProductLineKey

					LEFT JOIN dbo.sx_pf_dValueSeries dVS
						ON dP.ProductKey = dVS.ProductKey
				
			
		-- Helper Table for Globalattributes
		-- ****************************************************************************************************************************************************
			IF OBJECT_ID('tempdb..#GA') IS NOT NULL
				DROP TABLE #GA;

			CREATE TABLE #GA
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Globalattribute			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Content					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #GA
				SELECT 
						 ProductKey
						,FactoryID
						,ProductLineID
						,ProductID
						,UnpivotTable.Globalattribute
						,UnpivotTable.Content

				FROM
					dbo.sx_pf_dProducts
		
				UNPIVOT
					(	

						Content FOR Globalattribute IN (
													 Globalattribute1,Globalattribute2,Globalattribute3,Globalattribute4,Globalattribute5
													,Globalattribute6,Globalattribute7,Globalattribute8,Globalattribute9,Globalattribute10
													,Globalattribute11,Globalattribute12,Globalattribute13,Globalattribute14,Globalattribute15
													,Globalattribute16,Globalattribute17,Globalattribute18,Globalattribute19,Globalattribute20
													,Globalattribute21,Globalattribute22,Globalattribute23,Globalattribute24,Globalattribute25
												)
					) AS UnpivotTable

				WHERE Content <>''


		-- Table for Search Result Collection
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#SearchResult') IS NOT NULL
				DROP TABLE #SearchResult;

			CREATE TABLE #SearchResult
			(
				 RowKey					BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resultgroup			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resulttype				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,DataFactoryPath		NVARCHAR (1000)			COLLATE DATABASE_DEFAULT NOT NULL
				,Result					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)
		-- Getting Content
		-- ****************************************************************************************************************************************************
		
		-- Factories BEGIN #####

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'FactoryID'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.FactoryID
			FROM dbo.sx_pf_dFactories dF
			WHERE dF.FactoryID LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'NameShort'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.NameShort
			FROM dbo.sx_pf_dFactories dF
			WHERE dF.NameShort LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'ResponsiblePerson'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.ResponsiblePerson
			FROM dbo.sx_pf_dFactories dF
			WHERE dF.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Factories END #####

		-- Productlines BEGIN #####
		
		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'ProductlineID'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ProductLineID
			FROM dbo.sx_pf_dProductLines dPL
					LEFT JOIN (
								SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
							   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ProductLineID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.NameShort
			FROM dbo.sx_pf_dProductLines dPL
					LEFT JOIN (
									SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
								   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ResponsiblePerson
			FROM dbo.sx_pf_dProductLines dPL
					LEFT JOIN (
						SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
						) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Productlines END #####

		-- Products BEGIN ####
		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ProductID'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ProductID
			FROM dbo.sx_pf_dProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.ProductID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.NameShort
			FROM dbo.sx_pf_dProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ResponsiblePerson
			FROM dbo.sx_pf_dProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.ResponsiblePerson LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'CommmentUser'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,LEFT(dP.CommentUser,250)+' ...'
			FROM dbo.sx_pf_dProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.CommentUser LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,GA.Globalattribute
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,GA.Content
			FROM dbo.sx_pf_dProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
					LEFT JOIN #GA GA
						ON dP.ProductKey = GA.ProductKey
			WHERE GA.Content LIKE @SearchString

		-- Products END ####

		-- ValueSeries Start ####
		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'ValueSeriesID'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.ValueSeriesID
			FROM dbo.sx_pf_dValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.ValueSeriesID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameShort
			FROM dbo.sx_pf_dValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameLong'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameLong
			FROM dbo.sx_pf_dValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameLong LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'CommentUser'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,LEFT(dVS.CommentUser,250)+' ...'
			FROM dbo.sx_pf_dValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.CommentUser LIKE @SearchString

		-- ValueSeries END ####

		-- Value Start ###
		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueText'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueText,250)+' ...'
			FROM dbo.sx_pf_fValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductlineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueText LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueComment'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueComment,250)+' ...'
			FROM dbo.sx_pf_fValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductlineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueComment LIKE @SearchString

			
		-- Output

		SELECT TOP (100)
			   sr.FactoryID
		      ,sr.ProductLineID
			  ,sr.ProductID
			  ,sr.ValueSeriesID
			  ,sr.Resultgroup
			  ,sr.Resulttype
			  ,sr.DataFactoryPath
			  ,sr.Result
		FROM #SearchResult sr

			INNER JOIN 
				(	
					SELECT *
					FROM dbo.sx_pf_vUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> ''
				) vUR

				ON 
						sr.FactoryID = vUR.FactoryID 
					AND (sr.ProductLineID = vUR.ProductLineID OR sr.ProductLineID = '-1') -- as Output for not existing Productline possible

		ORDER BY CASE sr.Resultgroup 
						WHEN 'Factory'		THEN 1
						WHEN 'Productline'	THEN 2
						WHEN 'Product'		THEN 3
						WHEN 'ValueSeries'	THEN 4
						WHEN 'Values'		THEN 5
						ELSE 99
				 END
					
				,sr.Resulttype,sr.FactoryID,sr.ProductLineID,sr.ProductID,sr.ValueSeriesID

		SET @SearchDoneFlag = 1

		SET @ResultCode = 200

		COMMIT TRANSACTION sx_pf_GET_GlobalSearch;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_GlobalSearch;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryService
GO
/*
GET Operation for HTML Pages
10/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_HTMLPage 
	@Username		= 'SQL',
	@FactoryID		= '', 
	@ProductLineID	= '', 
	@ProductID		= '',
	@PageID			= 'Page1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_HTMLPage') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_HTMLPage;
GO

CREATE PROCEDURE dbo.sx_pf_GET_HTMLPage
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PageID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @PageKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PageID,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PageID			IS NULL SET @PageID			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPage;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PageID			= dbo.sx_pf_pProtectString	(@PageID);

		IF @PageID		= N'' 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM dbo.sx_pf_dFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM dbo.sx_pf_dProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM dbo.sx_pf_dProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
		
		SELECT @PageKey = PageKey FROM dbo.sx_pf_gHTMLPages
		WHERE
				FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND PageID			= @PageID

		IF @PageKey = 0 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;
			
		-- Check rights on different levels 
			-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
			,PageContent
		
		FROM dbo.sx_pf_gHTMLPages 
		WHERE PageKey = @PageKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_HTMLPage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_HTMLPage TO pf_PlanningFactoryService;
GO
/*
GET Operation for a List of all HTML Pages
10/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_HTMLPages 
	@Username		= 'SQL',
	@FactoryID		= '', 
	@ProductLineID	= '', 
	@ProductID		= ''

PRINT @RC

SELECT * FROM dbo.sx_pf_gHTMLPages
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_HTMLPages') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_HTMLPages;
GO

CREATE PROCEDURE dbo.sx_pf_GET_HTMLPages 
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPages;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM dbo.sx_pf_dFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM dbo.sx_pf_dProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM dbo.sx_pf_dProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
			
		-- Check rights on different levels 
		-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
		
		FROM dbo.sx_pf_gHTMLPages;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPages;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPages;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_HTMLPages TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_HTMLPages TO pf_PlanningFactoryService;
GO
/*
GET Operation for cross query of ALL attributes usage

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_AllGlobalAttributeUsage 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_AllGlobalAttributeUsage') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_AllGlobalAttributeUsage;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_AllGlobalAttributeUsage
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		;WITH dt AS (
			SELECT GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
				GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
				GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
				GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
				GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25
			FROM dbo.sx_pf_dProductLines dPL LEFT JOIN dbo.sx_pf_vUserRights vUR 
				ON	dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID AND
					vUR.[Right] IN (N'Read', N'Write') AND
					vUR.UserName = @TransactUsername
		)
		, attrs AS (
			SELECT DISTINCT Attribute, Value 
			FROM 
				(SELECT * FROM dt) pvt
					UNPIVOT ([Value] FOR Attribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
					GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
					GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
					GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
					GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25)
				) AS unpvt
		)
		SELECT CAST(SUBSTRING(Attribute, 21, 2) AS INT) AS N, Value AS GlobalAttribute
		FROM attrs
		ORDER BY N, GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_AllGlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryService;
GO

/*
GET Operation for CustomFailureReport

04/2019 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights
- Returns always at least one row


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_CustomFailureReport 'SQL','ZT'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_CustomFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_CustomFailureReport;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_CustomFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_CustomFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             CFR.Timestamp								AS LastCheckDate
            ,CFR.FactoryID
			,COALESCE(
						 CFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,CFR.ProductLineID
			,COALESCE(	
						 CFR.ProductLineID + ' ' + dPL.NameShort
						,'All Productlines'
					 )
														AS ProductlineIDName
            ,CFR.ProductID
			,COALESCE(
						 CFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,CFR.FactoryID + ' / ' + CFR.ProductLineID +' / ' + CFR.ProductID AS DataFactoryPath
            ,CFR.TestSource
            ,CFR.TestName
			,CFR.ResultType
            ,CFR.ResultText
            ,CFR.ResultNumeric
		FROM control.tCustomFailureReport CFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM dbo.sx_pf_vUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM dbo.sx_pf_vUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM dbo.sx_pf_vUserRights GROUP BY UserName
						) vUR 

				ON	CFR.FactoryID		= vUR.FactoryID
				AND CFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN dbo.sx_pf_dFactories dF 
				ON	CFR.FactoryID		= dF.FactoryID

			-- for Productline Name
			LEFT JOIN dbo.sx_pf_dProductLines dPL
				ON	CFR.FactoryID		= dPL.FactoryID
				AND CFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN dbo.sx_pf_dProducts dP
				ON	CFR.FactoryID		= dP.FactoryID
				AND CFR.ProductLineID	= dP.ProductLineID
				AND CFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			CFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tCustomFailureReport
							)
			-- called for one or all Factories
			AND (
					CFR.FactoryID = @FactoryID OR @FactoryID = '' OR CFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN
					SELECT 
					  GETDATE()						AS LastCheckDate
					,'All'							AS FactoryID
					,'All Factories'				AS FactoryIDName
					,'All'							AS ProductLineID
					,'All Productlines'				AS ProductlineIDName
					,'All'							AS ProductID
					,'All Products'					AS ProductIDName
					,'All / All / All'				AS DataFactoryPath
					,'Hint'							AS TestSource
					,'Default Entry'				AS Testname
					,'OK'							AS ResultType
					,'Keine Eintr�ge vorhanden.'	AS ResultText
					,0								AS ResultNumeric
			END
			

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_CustomFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_CustomFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryService;
GO


/*
GET Operation for cross query of attribute usage

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC  NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_GlobalAttributeUsage 'SQL','1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_GlobalAttributeUsage') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_GlobalAttributeUsage;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_GlobalAttributeUsage
	@Username				NVARCHAR(255),
	@GlobalattributeNumber	INT
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+		ISNULL(@Username,				N'NULL') + N''',' 
			+ CAST(	ISNULL(@GlobalattributeNumber,	N'NULL') AS NVARCHAR(255));
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_GlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		SELECT DISTINCT
		CASE @GlobalattributeNumber
			WHEN 1 THEN GlobalattributeAlias1
			WHEN 2 THEN GlobalattributeAlias2
			WHEN 3 THEN GlobalAttributeAlias3
			WHEN 4 THEN GlobalAttributeAlias4
			WHEN 5 THEN GlobalattributeAlias5
			WHEN 6 THEN GlobalAttributeAlias6
			WHEN 7 THEN GlobalattributeAlias7
			WHEN 8 THEN GlobalattributeAlias8
			WHEN 9 THEN GlobalattributeAlias9
			WHEN 10 THEN GlobalattributeAlias10
			WHEN 11 THEN GlobalattributeAlias11
			WHEN 12 THEN GlobalattributeAlias12
			WHEN 13 THEN GlobalAttributeAlias13
			WHEN 14 THEN GlobalAttributeAlias14
			WHEN 15 THEN GlobalattributeAlias15
			WHEN 16 THEN GlobalAttributeAlias16
			WHEN 17 THEN GlobalattributeAlias17
			WHEN 18 THEN GlobalattributeAlias18
			WHEN 19 THEN GlobalattributeAlias19
			WHEN 20 THEN GlobalattributeAlias20		
			WHEN 21 THEN GlobalattributeAlias21
			WHEN 22 THEN GlobalattributeAlias22
			WHEN 23 THEN GlobalAttributeAlias23
			WHEN 24 THEN GlobalAttributeAlias24
			WHEN 25 THEN GlobalattributeAlias25
			ELSE N'not existing' END AS GlobalAttribute	
		FROM dbo.sx_pf_dProductLines dPL LEFT JOIN dbo.sx_pf_vUserRights vUR 
			ON	dPL.FactoryID = vUR.FactoryID AND
				dPL.ProductLineID = vUR.ProductLineID AND
				vUR.[Right] IN (N'Read',N'Write') AND
				vUR.UserName = @TransactUsername
		ORDER BY GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_GlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_GlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryService;
GO

/*
GET Operation for monitoring Properties

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_Monitoring 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_Monitoring') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_Monitoring;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_Monitoring
	@Username NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_Monitoring;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3 - Select Values

		IF OBJECT_ID('tempdb..#temp') IS NOT NULL
			DROP TABLE #temp;

		CREATE TABLE #temp (
			Property NVARCHAR (255) NOT NULL
			,[Value] NVARCHAR (255) NOT NULL
		);

		INSERT INTO #temp
			-- Language
			SELECT 'Cluster_C03' AS Property, ValueText FROM dbo.sx_pf_gCluster WHERE PropertyID = 'C03'
			UNION ALL
			-- Clustername
			SELECT 'Cluster_C01' AS Property, ValueText FROM dbo.sx_pf_gCluster WHERE PropertyID = 'C01'	
			UNION ALL
			-- API Version
			SELECT 'Cluster_C04' AS Property, ValueText FROM dbo.sx_pf_gCluster WHERE PropertyID = 'C04'	
			UNION ALL
			-- FactoryCount
			SELECT 'FactoryCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM dbo.sx_pf_dFactories
			UNION ALL
			-- ProductLineCount
			SELECT 'ProductlineCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM dbo.sx_pf_dProductLines
			UNION ALL
			-- ProductCount
			SELECT 'ProductCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM dbo.sx_pf_dProducts
			UNION ALL
			-- API Error 500 Count
			SELECT 'API_Error500' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM dbo.sx_pf_API_Log WHERE ReturnCode = 500 AND ProcessCode = ''
			UNION ALL
			-- API Error 404 Count
			SELECT 'API_Error404' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM dbo.sx_pf_API_Log WHERE ReturnCode = 404 AND ProcessCode = '';

		SELECT * FROM #temp;
		
		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_Monitoring;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_Monitoring;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_Monitoring TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_Monitoring TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free FactoryID
Returns the next numeric ID or the number 1 if other FactoryIDs are non-numeric

03/2019 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_NextFreeFactoryID 
	@Username		= 'SQL'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeFactoryID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_NextFreeFactoryID;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeFactoryID
	@Username		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FreeFactoryID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(FactoryID AS BIGINT) id 
			FROM dbo.sx_pf_dFactories
		)
		SELECT @FreeFactoryID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeFactoryID IS NOT NULL
		BEGIN
			SET @FreeFactoryID = @FreeFactoryID + 1;
		END
		ELSE
		BEGIN
			SET @FreeFactoryID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT FactoryID FROM dbo.sx_pf_dFactories WHERE FactoryID = @FreeFactoryID)
		BEGIN
			SET @FreeFactoryID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeFactoryID AS NextFreeFactoryID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free ProductID
Returns for numeric ID the next number for Productlines with only string IDs the number 1 

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_NextFreeProductID 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'V'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeProductID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductID;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FreeProductID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeProductID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		
		IF @ProductLineKey = 0 OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductID AS BIGINT) id 
			FROM dbo.sx_pf_dProducts 
			WHERE ProductLineKey = @ProductLineKey 
		)
		SELECT @FreeProductID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductID IS NOT NULL
		BEGIN
			SET @FreeProductID = @FreeProductID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductID FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @FreeProductID)
		BEGIN
			SET @FreeProductID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductID AS NextFreeProductID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeProductID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeProductID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free ProductlineID
Returns the next numeric ID or the number 1 if other ProductlineIDs are non-numeric

03/2019 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_NextFreeProductLineID 
	@Username		= 'SQL',
	@FactoryID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeProductLineID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductLineID;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductLineID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @FreeProductLineID	NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreePLID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);

		IF	@FactoryID			= N'' 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16 ,10);
			END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		
		IF @FactoryKey = 0
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductLineID AS BIGINT) id 
			FROM dbo.sx_pf_dProductLines 
			WHERE FactoryKey = @FactoryKey 
		)
		SELECT @FreeProductLineID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductLineID IS NOT NULL
		BEGIN
			SET @FreeProductLineID = @FreeProductLineID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductLineID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductLineID FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @FreeProductLineID)
		BEGIN
			SET @FreeProductLineID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductLineID AS NextFreeProductLineID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreePLID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreePLID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryService;
GO

/*
GET Operation for SystemFailureReport

09/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_SystemFailureReport 'SQL','ZT'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_SystemFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_SystemFailureReport;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_SystemFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_SystemFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             SFR.Timestamp								AS LastSystemCheckDate
            ,SFR.FactoryID
			,COALESCE(
						 SFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,SFR.ProductLineID
			,COALESCE(	
						 SFR.ProductLineID + ' ' + dPL.NameShort
						,'All Productlines'
					 )
														AS ProductlineIDName
            ,SFR.ProductID
			,COALESCE(
						 SFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,SFR.FactoryID + ' / ' + SFR.ProductLineID +' / ' + SFR.ProductID AS DataFactoryPath
            ,SFR.TestSource
            ,SFR.TestName
			,SFR.ResultType
            ,SFR.ResultText
            ,SFR.ResultNumeric
		FROM control.tSystemFailureReport SFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM dbo.sx_pf_vUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM dbo.sx_pf_vUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM dbo.sx_pf_vUserRights GROUP BY UserName
						) vUR 

				ON	SFR.FactoryID		= vUR.FactoryID
				AND SFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN dbo.sx_pf_dFactories dF 
				ON	SFR.FactoryID		= dF.FactoryID

			-- for Productline Name
			LEFT JOIN dbo.sx_pf_dProductLines dPL
				ON	SFR.FactoryID		= dPL.FactoryID
				AND SFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN dbo.sx_pf_dProducts dP
				ON	SFR.FactoryID		= dP.FactoryID
				AND SFR.ProductLineID	= dP.ProductLineID
				AND SFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			SFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tSystemFailureReport
							)
			-- called for one or all Factories
			AND (
					SFR.FactoryID = @FactoryID OR @FactoryID = '' OR SFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_SystemFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_SystemFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryService;
GO



/*
GET Operation for UserRight Overview

09/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
@FactoryID, @ProductlineID only Dummy Parameter to call over csv Route
Works only for Clusteradmins


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Info_UserRights 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_UserRights') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Info_UserRights;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Info_UserRights
	@Username NVARCHAR(255),@FactoryID NVARCHAR(255)='',@ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_UserRights;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			BEGIN
				SELECT 
					 'None'												AS UserName
					,'None'												AS Status
					,'None'												AS FactoryID
					,'None'												AS FactoryName
					,'None'												AS ProductLineID
					,'None'												AS ProductlineName
					,'You need Cluster Admin privileges to see this.'	AS [Right]
					,-1													AS RightLevel;
			END;
		ELSE
			BEGIN
				SELECT 
					 rU.UserName
					,rU.Status	
					,dPL.FactoryID
					,dF.NameShort				AS FactoryName
					,dPL.ProductLineID
					,dPL.NameShort				AS ProductlineName
					,ISNULL(vUR.[Right],'Deny') AS UserRight
					,CASE vUR.[Right]
						WHEN 'Read'		THEN 0
						WHEN 'Write'	THEN 1
						ELSE -1
					 END							AS RightLevel
				FROM dbo.sx_pf_rUser rU
					-- for full Productline Structure
					LEFT JOIN dbo.sx_pf_dProductLines dPL ON
						 1=1
					-- for FactoryName
					LEFT JOIN dbo.sx_pf_dFactories dF ON
						dPL.FactoryKey = dF.FactoryKey
					-- for positiv Rights
					LEFT JOIN dbo.sx_pf_vUserRights vUR	ON
							rU.UserKey			= vUR.UserKey
						AND vUR.FactoryID		= dPL.FactoryID
						AND vUR.ProductLineID	= dPL.ProductLineID;
			END;

			SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_UserRights;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_UserRights;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_UserRights TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_UserRights TO pf_PlanningFactoryService;
GO


/*
GET Operation for a List 
Shows the properties of one List
This procedure don't shows internal system lists

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_List 
	@Username	= 'SQL', 
	@ListID		= 'sxStatus';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_List;
GO

CREATE PROCEDURE dbo.sx_pf_GET_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL	SET @Username	= N'';
	IF @ListID		IS NULL	SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT @ListKey = ListKey FROM dbo.sx_pf_hLists WHERE ListID = @ListID;

		IF @ListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_hLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_List TO pf_PlanningFactoryService;
GO
/*
GET Operation for all GUI Lists in Cluster
Internal system lists are not shown

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call 

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Lists 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Lists') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Lists
GO

CREATE PROCEDURE dbo.sx_pf_GET_Lists
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(10)   = N'';
	DECLARE @TransactUsername	NVARCHAR(255)  = N'';
	DECLARE @ProcedureName		NVARCHAR(255)  = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)  = N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT			   = 0;						
	DECLARE @ResultCode			INT			   = 501;					
	DECLARE @TimestampCall		DATETIME	   = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Lists;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT 
			ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_hLists
		ORDER BY ISNULL(TRY_CAST(ListID AS INT), 999999999), ListID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Lists;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Lists;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Lists TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Lists TO pf_PlanningFactoryService;
GO
/*
GET Operation for all List Values
- shows values of all GUI Lists
- shows values of the internal list "sxScales", as this list is need for the other lists

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ListValues 
	@Username	= 'SQL', 
	@ListID		= 'sxEvents'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ListValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ListValues
GO

CREATE PROCEDURE dbo.sx_pf_GET_ListValues
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		IF @ListID != 'sxScales'
		BEGIN
			-- STEP 1.2 - Determine Keys
			SELECT @ListKey = ListKey FROM dbo.sx_pf_hLists WHERE ListID = @ListID;

			IF @ListKey = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

			-- STEP 3. Show public values, always one dummy line, if list exists	
			SELECT 
				ISNULL(hLV.ListValueKey, 0)		AS ListValueKey  --returns the Key as the Listvalue has no ID
				,hL.ListID
				,hL.Datentyp
				,ISNULL(hLV.ValueInt, 0)		AS ValueInt
				,ISNULL(hLV.Scale, 0)			AS Scale
				,ISNULL(hLV.ValueText, N'')		AS ValueText
				,ISNULL(hLV.ValueComment, N'')	AS ValueComment
				,ISNULL(hLV.FormatID, N'')		AS FormatID
				,@Right AS 'Right'
			FROM dbo.sx_pf_hLists hL
				LEFT JOIN dbo.sx_pf_hListValues hLV	ON hL.ListID = hLV.ListID
			WHERE hL.ListID = @ListID 
			ORDER BY ISNULL(TRY_CAST(hLV.ValueText AS INT), 999999999), hLV.ValueText, hLV.ValueInt;

			SET @EffectedRows = @@ROWCOUNT;
		END;

		-- static definition of sxScales to be loaded for list defintion
		IF @ListID = 'sxScales'
		BEGIN
			SELECT * 
			FROM (
				VALUES	
					(1,'sxScales' ,'INT',1    ,1,'','','','Read')
					,(2,'sxScales','INT',10   ,1,'','','','Read')
					,(3,'sxScales','INT',100  ,1,'','','','Read')
					,(4,'sxScales','INT',1000 ,1,'','','','Read')
					,(5,'sxScales','INT',10000,1,'','','','Read')
			) tmp (ListValueKey, ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, [Right])

			SET @EffectedRows = @@ROWCOUNT;
		END;

		COMMIT TRANSACTION sx_pf_GET_ListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ListValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ListValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ListValues TO pf_PlanningFactoryService;
GO
/*
GET Operation for Prefences

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

- Preferences may exist for the Cluster, for Factories or for Productlines - all in the same table
- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"

--Test call

1. Empty FactoryID or empty ProductLineID => 404 Not Found
2. Username non Cluster Admin => 403 Forbidden
3. User does not have rights to read ProductLine => 401 Unauthorized
5. No exists Factory / Productline > 404 Not Found

DECLARE  @RC					INT
		,@Username				NVARCHAR(255) = 'SQL'
		,@PreferenceUserName	NVARCHAR(255) = 'W10\admin'
		,@ProductLineID			NVARCHAR(255) = 'U'
		,@FactoryID				NVARCHAR(255) = 'ZT'
		,@SettingName			NVARCHAR(255) = 'Demopref'


EXECUTE @RC = sx_pf_GET_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Preference;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Preference
	 @Username				NVARCHAR(255)
	,@PreferenceUsername	NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@SettingName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUsername					NVARCHAR(255)= N''
			,@FactoryKey						INT = 0
			,@ProductLineKey					INT = 0
			,@UserExistsFlag					INT = 0
			,@PreferenceIsUserDependendFlag		INT = 0
			,@ProcedureName						NVARCHAR(255) = OBJECT_NAME(@@PROCID)

			,@ParameterString					NVARCHAR(MAX) = N''''
							+ ISNULL(@Username, N'NULL')			+ N''',''' 
							+ ISNULL(@PreferenceUsername, N'NULL')	+ N''',''' 
							+ ISNULL(@ProductLineID, N'NULL')		+ N''',''' 
							+ ISNULL(@FactoryID, N'NULL')			+ N''','''
							+ ISNULL(@SettingName, N'NULL')			+ N''''
			
			,@EffectedRows						INT = 0					
			,@ResultCode 						INT = 501						
			,@TimestampCall						DATETIME = CURRENT_TIMESTAMP
			,@Comment							NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_Preference;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername = dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName		= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @SettingName		= N'' 
			OR @SettingName		= N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.3 - Determine, if Preferences are saved per user

		-- for Clusterpreferences
		IF @FactoryID = N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gCluster WHERE PropertyID = N'C06';
			END 
		-- for Factorypreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gFactories WHERE PropertyID = N'F06';
			END
		-- for ProductlinePreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gProductLines WHERE PropertyID = N'P06';
			END;
		
		-- STEP 1.1 - Determine Keys
		SELECT @FactoryKey		= FactoryKey		FROM sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		IF (@FactoryKey = 0 AND @FactoryID <> N'')
			OR (@ProductlineKey = 0 AND @ProductLineID <> N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Change PreferenceUser if Preferences User-independend
		IF @PreferenceIsUserDependendFlag <> 1
		BEGIN 
			SET @PreferenceUsername = N'All';
		END;

		--- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights - TODO: depdendend from preference type
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT 
			 UserName
			,FactoryID
			,ProductlineID
			,SettingName
			,ValueInt
			,ValueText
		FROM dbo.sx_pf_rPreferences 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryService;
GO
/*
GET Operation for Product

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Product 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Product
GO

CREATE PROCEDURE dbo.sx_pf_GET_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID = dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID = dbo.sx_pf_pProtectID (@ProductLineID);	
		SET @ProductID = dbo.sx_pf_pProtectID (@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
			,@Right AS 'Right'
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
		FROM dbo.sx_pf_dProducts dP LEFT JOIN dbo.sx_pf_vUserRights vUR 
			ON  dP.FactoryID = vUR.FactoryID AND
				dP.ProductLineID = vUR.ProductLineID AND
				vUR.UserName = @TransactUsername
		WHERE dP.ProductKey = @ProductKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Product TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Values of ProductDataTable

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according HTTP Standard

Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductDataTableValues
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';
PRINT @RC

--should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductDataTableValues 
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT2',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductDataTableValues;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductDataTableValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID  NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductDataTableValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right	AS 'Right'
		FROM dbo.sx_pf_fValues fV 
			LEFT JOIN dbo.sx_pf_dValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey
		WHERE fV.ProductKey = @ProductKey
		ORDER BY ValueSeriesNo, TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductDataTableValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductDataTableValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
GET Operation for ProductLine
Provides complete information about a Product Line

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductLine 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U' 
	
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductLine;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductLine
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			ProductLineID 
			,FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,DefaultTemplate
			,GlobalAttributeSource1
			,GlobalAttributeAlias1
			,GlobalAttributeSource2
			,GlobalAttributeAlias2
			,GlobalAttributeSource3
			,GlobalAttributeAlias3
			,GlobalAttributeSource4
			,GlobalAttributeAlias4
			,GlobalAttributeSource5
			,GlobalAttributeAlias5
			,GlobalAttributeSource6
			,GlobalAttributeAlias6
			,GlobalAttributeSource7
			,GlobalAttributeAlias7
			,GlobalAttributeSource8
			,GlobalAttributeAlias8
			,GlobalAttributeSource9
			,GlobalAttributeAlias9
			,GlobalAttributeSource10
			,GlobalAttributeAlias10

			,GlobalAttributeSource11
			,GlobalAttributeAlias11
			,GlobalAttributeSource12
			,GlobalAttributeAlias12
			,GlobalAttributeSource13
			,GlobalAttributeAlias13
			,GlobalAttributeSource14
			,GlobalAttributeAlias14
			,GlobalAttributeSource15
			,GlobalAttributeAlias15
			,GlobalAttributeSource16
			,GlobalAttributeAlias16
			,GlobalAttributeSource17
			,GlobalAttributeAlias17
			,GlobalAttributeSource18
			,GlobalAttributeAlias18
			,GlobalAttributeSource19
			,GlobalAttributeAlias19
			,GlobalAttributeSource20
			,GlobalAttributeAlias20

			,GlobalAttributeSource21
			,GlobalAttributeAlias21
			,GlobalAttributeSource22
			,GlobalAttributeAlias22
			,GlobalAttributeSource23
			,GlobalAttributeAlias23
			,GlobalAttributeSource24
			,GlobalAttributeAlias24
			,GlobalAttributeSource25
			,GlobalAttributeAlias25

			,@Right AS 'Right'
		FROM dbo.sx_pf_dProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLine TO pf_PlanningFactoryService;
GO
/*
GET Operation for determination of all properties of ProductLine

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductLineProperties 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U' 

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLineProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductLineProperties;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductLineProperties
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gProductLines
		WHERE 
			FactoryKey = @FactoryKey 
			AND ProductLineKey = @ProductLineKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a ProductLine Property

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductLineProperty 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@PropertyID		= 'PL01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductLineProperty;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @PropertyKey = PropertyKey FROM sx_pf_gProductLines WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID

		IF	@PropertyKey		= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
GET Operation for ProductLines 
	A: returns the Productlines for a single Factory
	B: returns all Productlines when Factory '' is requested

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductLines 
	@Username	= 'SQL',
	@FactoryID	= 'ZT'

PRINT @RC

EXEC @RC = dbo.sx_pf_GET_ProductLines 
	@Username = 'SQL'

PRINT @RC


*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLines') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductLines;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductLines
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'' ;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLines;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys if single Factory is requested
		IF @FactoryID <> ''
			BEGIN
				SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

				IF @FactoryKey = 0	
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Keys don`t exists', 16, 10);
				END;
			END

		-- STEP 3. Show public values	
		;WITH vUR AS (
			SELECT 
				FactoryID, 
				ProductLineID, 
				[Right]
			FROM dbo.sx_pf_vUserRights 
			WHERE ProductLineID <> N'' 
				AND Username = @TransactUsername 
				AND	[Right] IN (N'Read', N'Write')
		)
		SELECT  
			 dPL.ProductLineID 
			,dPL.FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,DefaultTemplate
			,GlobalAttributeSource1
			,GlobalAttributeAlias1
			,GlobalAttributeSource2
			,GlobalAttributeAlias2
			,GlobalAttributeSource3
			,GlobalAttributeAlias3
			,GlobalAttributeSource4
			,GlobalAttributeAlias4
			,GlobalAttributeSource5
			,GlobalAttributeAlias5
			,GlobalAttributeSource6
			,GlobalAttributeAlias6
			,GlobalAttributeSource7
			,GlobalAttributeAlias7
			,GlobalAttributeSource8
			,GlobalAttributeAlias8
			,GlobalAttributeSource9
			,GlobalAttributeAlias9
			,GlobalAttributeSource10
			,GlobalAttributeAlias10

			,GlobalAttributeSource11
			,GlobalAttributeAlias11
			,GlobalAttributeSource12
			,GlobalAttributeAlias12
			,GlobalAttributeSource13
			,GlobalAttributeAlias13
			,GlobalAttributeSource14
			,GlobalAttributeAlias14
			,GlobalAttributeSource15
			,GlobalAttributeAlias15
			,GlobalAttributeSource16
			,GlobalAttributeAlias16
			,GlobalAttributeSource17
			,GlobalAttributeAlias17
			,GlobalAttributeSource18
			,GlobalAttributeAlias18
			,GlobalAttributeSource19
			,GlobalAttributeAlias19
			,GlobalAttributeSource20
			,GlobalAttributeAlias20

			,GlobalAttributeSource21
			,GlobalAttributeAlias21
			,GlobalAttributeSource22
			,GlobalAttributeAlias22
			,GlobalAttributeSource23
			,GlobalAttributeAlias23
			,GlobalAttributeSource24
			,GlobalAttributeAlias24
			,GlobalAttributeSource25
			,GlobalAttributeAlias25

			,vUR.[Right]
		FROM dbo.sx_pf_dProductLines dPL INNER JOIN vUR
			ON dPL.FactoryID = vUR.FactoryID AND
			   dPL.ProductLineID = vUR.ProductLineID
		WHERE FactoryKey = @FactoryKey 
			OR @FactoryID = ''
		ORDER BY ISNULL(TRY_CAST(dPL.ProductLineID AS BIGINT), 999999999), dPL.ProductLineID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_ProductLines;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLines;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryService;
GO

/*
GET Operation for identifying all Properties of Products

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductProperties
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductProperties;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductProperties
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gProducts
		WHERE ProductKey = @ProductKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a ProductProperty

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductProperty 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1',
	@PropertyID		= 'P1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductProperty;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductProperty
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''  
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gProducts WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @PropertyKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 2.1 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM dbo.sx_pf_gProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperty TO pf_PlanningFactoryService;
GO
/*
Procedure to GET all ProductRanges for an Product

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Samplescript to POST an ProductRange

EXEC dbo.sx_pf_POST_ProductProperty 
	 @Username				= 'SQL'
	,@ProductID				= '1'
	,@ProductLineID			= 'U'
	,@FactoryID				= 'ZT'
	,@PropertyID			= 'PDTRange_1'				-- fixed Name PDTRange_1...N
	,@PropertyName			= 'PDTRangeProperty'		-- fixed Name PDTRangeProperty
	,@CommentUser			= ''
	,@CommentDev			= ''
	,@Unit					= ''
	,@ValueText				= 'K1:20170115:K2:20171215'	-- Range in DF Coordinates, separated with :
	,@ValueInt				= '1'						-- Protected 1 = TRUE
	,@Scale					= '1'
	,@IsROSystemproperty	= '0'
	,@FormatID				= 'rngRed'					-- Name of FormatID or '<#NV>' to set only Protection 
														-- the Format Property Border* will be ignored on Cell Level

EXEC dbo.sx_pf_POST_ProductProperty 
	 @Username				= 'SQL'
	,@ProductID				= '1'
	,@ProductLineID			= 'U'
	,@FactoryID				= 'ZT'
	,@PropertyID			= 'PDTRange_2'				
	,@PropertyName			= 'PDTRangeProperty'		
	,@CommentUser			= ''
	,@CommentDev			= ''
	,@Unit					= ''
	,@ValueText				= 'K3:20180615:K3:20190115'	
	,@ValueInt				= '1'						
	,@Scale					= '1'
	,@IsROSystemproperty	= '0'
	,@FormatID				= 'rngRed'					
														
--Testcall

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ProductRanges 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductRanges') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ProductRanges;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ProductRanges
	 @Username		NVARCHAR(255)
	,@FactoryID		NVARCHAR(255)
	,@ProductLineID	NVARCHAR(255)
	,@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		
			+ ISNULL(@Username,			N'NULL') + N''','''	
			+ ISNULL(@FactoryID,		N'NULL') + N''','''	
			+ ISNULL(@ProductLineID,	N'NULL') + N''','''	
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username		= N'';
	IF @FactoryID				IS NULL SET @FactoryID		= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID	= N'';
	IF @ProductID				IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_ProductRanges;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);	

		IF	@FactoryID			= N'' 
			OR  @ProductlineID	= N'' 
			OR  @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine Keys
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryID = @FactoryID AND ProductlineID = @ProductlineID AND ProductID = @ProductID;
		
		IF @ProductKey = 0 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 PropertyID
			,ValueText	AS DFRange
			,ValueInt	AS ProtectedFlag
			,FormatID
			,@Right		AS 'Right'
		FROM dbo.sx_pf_gProducts
		WHERE 
			    ProductKey = @ProductKey
			AND PropertyID LIKE 'PDTRange_%'; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_ProductRanges;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_ProductRanges;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductRanges TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductRanges TO pf_PlanningFactoryService;
GO

/*
GET Operation for all Product of ProductLine

11/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

-- should return 200 and the Products of one Productline
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U'

PRINT @RC

-- should return 200 and the Products of one Factory
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= ''

PRINT @RC

-- should return 200 and the Products of all Factories
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= '',
	@ProductLineID	= ''

PRINT @RC

-- should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Products') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Products;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Products
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Products;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys - maybe 0 if ProductlineID or FactoryID is empty
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		-- STEP 3. Show public values	
		SELECT 
			 dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,dP.TimeType
			,dP.NameShort
			,dP.NameLong
			,dP.CommentUser
			,dP.CommentDev
			,dP.ResponsiblePerson
			,dP.ImageName
			,dP.[Status]
			,dP.Template
			,dP.TemplateVersion
			,dP.GlobalAttribute1
			,dP.GlobalAttribute2
			,dP.GlobalAttribute3
			,dP.GlobalAttribute4
			,dP.GlobalAttribute5
			,dP.GlobalAttribute6
			,dP.GlobalAttribute7
			,dP.GlobalAttribute8
			,dP.GlobalAttribute9
			,dP.GlobalAttribute10
			,dP.GlobalAttribute11
			,dP.GlobalAttribute12
			,dP.GlobalAttribute13
			,dP.GlobalAttribute14
			,dP.GlobalAttribute15
			,dP.GlobalAttribute16
			,dP.GlobalAttribute17
			,dP.GlobalAttribute18
			,dP.GlobalAttribute19
			,dP.GlobalAttribute20
			,dP.GlobalAttribute21
			,dP.GlobalAttribute22
			,dP.GlobalAttribute23
			,dP.GlobalAttribute24
			,dP.GlobalAttribute25
			,vUR.[Right]				AS [Right]
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
		FROM dbo.sx_pf_dProducts dP 
			LEFT JOIN dbo.sx_pf_vUserRights vUR 
				ON		dP.FactoryID = vUR.FactoryID 
					AND	dP.ProductLineID = vUR.ProductLineID 
					AND	vUR.UserName = @TransactUsername
		WHERE 
				(
					 dP.ProductLineKey = @ProductLineKey 
				OR  (dP.FactoryID = @FactoryID AND @ProductLineID = '')
				OR  (@FactoryID = '')
				)

				AND vUR.[Right] IS NOT NULL

		ORDER BY 
			 dP.FactoryID
			,dP.ProductLineID	
			,ISNULL(TRY_CAST(dP.ProductID AS INT), 999999999), dP.ProductID;

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Products;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Products;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Products TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Products TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Statements of Products

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Statements 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Statements') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Statements;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Statements
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Statements;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ActionType
			,[Statement]
			,UserName
			,PCName
			,ProcessorCode
			,IPAddresses
			,[Timestamp]
			,IsDeleted
			,IsResolved
			,@Right AS 'Right'
		FROM dbo.sx_pf_fStatements
		WHERE ProductKey = @ProductKey 
		ORDER BY [Timestamp] DESC;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Statements;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Statements;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Statements TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Statements TO pf_PlanningFactoryService;
GO
/*
GET Operation for all TimeIDs of ValueSeries

04/2018 for DataFactory 4.5
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_TimeIDs 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_TimeIDs') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_TimeIDs;
GO

CREATE PROCEDURE dbo.sx_pf_GET_TimeIDs
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_TimeIDs;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;		
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		FROM dbo.sx_pf_dTime
		WHERE ProductKey = @ProductKey 
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_TimeIDs;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_TimeIDs;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_TimeIDs TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_TimeIDs TO pf_PlanningFactoryService;
GO
/*
GET Operation for a User

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_User 
	@Username			= 'SQL', 
	@RequestedUserName	= 'W10\admin'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_User;
GO

CREATE PROCEDURE dbo.sx_pf_GET_User
	@Username			NVARCHAR(255),
	@RequestedUserName	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_User;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		IF @RequestedUserName = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - don't deliver the member of PlannigFactoryService
		IF ISNULL(IS_ROLEMEMBER('pf_PlanningFactoryService', @RequestedUsername), 2) = 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Member of PlannigFactoryService', 16, 10);
		END

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @UserKey = UserKey FROM dbo.sx_pf_rUser WHERE UserName = @RequestedUserName;
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right = N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			UserKey
			,UserName
			,PersonSurname
			,PersonFirstName
			,Email
			,LDAPIP
			,[Status]
			,@Right AS 'Right'
		FROM dbo.sx_pf_rUser
		WHERE UserKey = @UserKey; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_User TO pf_PlanningFactoryService;
GO
/*
GET Operation for User rights

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_UserRight 
	@Username = 'SQL', 
	@RequestedUserName = 'W10\admin'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_UserRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_UserRight;
GO

CREATE PROCEDURE dbo.sx_pf_GET_UserRight
	@Username			NVARCHAR(255),
	@RequstedUserName	NVARCHAR(255) = NULL, -- at the Moment written wrong, due to API compatibility !!!
	@RequestedUserName  NVARCHAR(255) = NULL 
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@RequstedUserName,		N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)  = N'';				

	SET @RequestedUserName = ISNULL(@RequestedUserName, @RequstedUserName);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY	
		BEGIN TRANSACTION GET_UserRight;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		-- Check on empty name or SystemUser
		IF @RequestedUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameter <RequestedUserName>', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @UserKey = UserKey FROM dbo.sx_pf_rUser WHERE UserName = @RequestedUserName
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right		= N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- Helper Table with Full Stucture - to Output Deny Rights too   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
			DROP TABLE #FullStructure;

		CREATE TABLE #FullStructure
		(
			RowKey			BIGINT IDENTITY (1,1)								NOT NULL
			,[Level]		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey		BIGINT												NOT NULL
			,ProductLineKey BIGINT												NOT NULL
			,FactoryID		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductlineID	NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
		);
		-- C Level
		INSERT INTO #FullStructure
		SELECT 
			'Cluster'	AS 'Level'
			,0			AS FactoryKey
			,0			AS ProductLineKey
			,''			AS FactoryID
			,''			AS ProductLineID;
			 
		-- F Level
		INSERT INTO #FullStructure
		SELECT
			'Factory'		AS 'Level'
			,dF.FactoryKey
			,0				AS ProductLineKey
			,dF.FactoryID
			,''				AS ProductLineID
		FROM dbo.sx_pf_dFactories dF;
				
		-- PL Level
		INSERT INTO #FullStructure
		SELECT
			'Productline'	AS 'Level'
			,dF.FactoryKey
			,dPL.ProductLineKey
			,dF.FactoryID
			,dPL.ProductLineID
		FROM dbo.sx_pf_dProductLines dPL 
			LEFT JOIN dbo.sx_pf_dFactories dF
				ON dPL.FactoryKey = dF.FactoryKey;
		
		-- STEP 3. Show public values	
		IF @Right = N'Read' --User is allowed to see of himself only positive rights
		BEGIN

			SELECT 
				vUR.Username
				,vUR.FactoryID
				,COALESCE(dF.NameShort,'')	AS FactoryName
				,vUR.ProductLineID
				,COALESCE(dPL.NameShort,'')	AS ProductlineName
				,vUR.[Right]
				,vUR.ReadCommentMandatory
				,vUR.WriteCommentMandatory
			FROM dbo.sx_pf_vUserRights vUR
				-- for FactoryName
				LEFT JOIN dbo.sx_pf_dFactories dF
					ON vUR.FactoryID = dF.FactoryID
				-- for ProductlineName
				LEFT JOIN dbo.sx_pf_dProductLines dPL
					ON		vUR.FactoryID = dPL.FactoryID 
						AND	vUR.ProductLineID = dPL.ProductLineID
			WHERE vUR.UserKey = @UserKey 
			ORDER BY 
				ISNULL(TRY_CAST(vUR.FactoryID AS INT), 999999999)
				,vUR.FactoryID
				,ISNULL(TRY_CAST(vUR.ProductLineID AS INT), 999999999)
				,vUR.ProductLineID;
		END
		ELSE
		IF @Right = N'Write' -- Cluster admin can see also negative rights (really don`t exist)
		BEGIN
				
			SELECT
				@RequestedUsername						AS Username
				,fs.FactoryID
				,COALESCE(dF.NameShort,'')				AS FactoryName
				,fs.ProductlineID
				,COALESCE(dPL.NameShort,'')				AS ProductlineName
				,COALESCE(vUR.[Right],'Deny')			AS 'Right'
                ,COALESCE(vUR.ReadCommentMandatory,0)	AS ReadCommentMandatory
				,COALESCE(vUR.WriteCommentMandatory,0)	AS WriteCommentMandatory
			FROM #FullStructure fs
				-- for FactoryName
				LEFT JOIN dbo.sx_pf_dFactories dF
					ON fs.FactoryID = dF.FactoryID
				-- for ProductlineName
				LEFT JOIN dbo.sx_pf_dProductLines dPL
					ON		fs.FactoryID = dPL.FactoryID 
						AND	fs.ProductLineID = dPL.ProductLineID
				-- positive Rechte
				LEFT JOIN
					(
						SELECT 
							vUR.FactoryID
							,vUR.ProductLineID
							,vUR.[Right]
							,vUR.ReadCommentMandatory
							,vUR.WriteCommentMandatory
						FROM dbo.sx_pf_vUserRights vUR
						WHERE vUR.UserKey = @UserKey
					) vUR
					ON		fs.FactoryID = vUR.FactoryID
						AND fs.ProductlineID = vUR.ProductlineID
			ORDER BY  
				ISNULL(TRY_CAST(fs.FactoryID AS INT), 999999999)
				,fs.FactoryID
				,ISNULL(TRY_CAST(fs.ProductLineID AS INT), 999999999)
				,fs.ProductLineID;
		END;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_UserRight;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_UserRight;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_UserRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_UserRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Users

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Users 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Users') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Users;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Users
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Users;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			UserKey
			,UserName
			,PersonSurname
			,PersonFirstName
			,Email
			,LDAPIP
			,[Status]
			,N'Write' AS 'Right'
		FROM dbo.sx_pf_rUser
		WHERE Username NOT IN (
			SELECT m.[name] COLLATE DATABASE_DEFAULT
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m ON rm.member_principal_id = m.principal_id
			WHERE r.[name] = N'pf_PlanningFactoryService')
		ORDER BY ISNULL(TRY_CAST(UserName AS INT), 999999999), UserName
		
		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Users;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Users;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Users TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Users TO pf_PlanningFactoryService;
GO
/*
GET Operation for a Value

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Value 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1',
	@TimeID			=  20180115

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Value;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @ValueKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')+ N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL')+ N''',''' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(10)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @FactoryID		IS NULL SET @FactoryID		 = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	 = N'';
	IF @ProductID		IS NULL SET @ProductID		 = N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Value;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSerieID	 = dbo.sx_pf_pProtectID		 (@ValueSerieID);
		
		IF	@FactoryID				= N'' 
			OR @ProductLineID		= N'' 
			OR @ProductID			= N'' 
			OR @ValueSerieID		= N'' 
			OR ISNULL(@TimeID, 0)	<= 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		SELECT @ValueKey = ValueKey FROM dbo.sx_pf_fValues WHERE ValueSeriesKey = @ValueSerieKey AND TimeID = @TimeID;

		IF @ProductKey = 0 OR @ProductLineKey = 0 OR @FactoryKey = 0 OR @ValueSerieKey = 0 OR @ValueKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2.1 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM dbo.sx_pf_fValues
		WHERE ValueKey = @ValueKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Value TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Values of ValueSeries

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Values
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1'

PRINT @RC

--should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_Values 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1test'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Values') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_Values;
GO

CREATE PROCEDURE dbo.sx_pf_GET_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Values;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 
			OR @ValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM dbo.sx_pf_fValues
		WHERE ValueSeriesKey = @ValueSerieKey
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Values TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Values TO pf_PlanningFactoryService;
GO
/*
GET Operation for all ValueSerie

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ValueSerie 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ValueSerie;
GO

CREATE PROCEDURE dbo.sx_pf_GET_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSerie;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N'' 
			OR @ValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM dbo.sx_pf_dValueSeries
		WHERE ValueSeriesKey = @ValueSerieKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSerie TO pf_PlanningFactoryService;
GO
/*
GET Operation for all ValueSeries of Product

04/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = dbo.sx_pf_GET_ValueSeries 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ValueSeries') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_GET_ValueSeries;
GO

CREATE  PROCEDURE dbo.sx_pf_GET_ValueSeries
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10);
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSeries;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM dbo.sx_pf_dValueSeries
		WHERE ProductKey = @ProductKey 
		ORDER BY ValueSeriesNo;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSeries;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSeries;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSeries TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSeries TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/08
Import all values from load.tdFactories with specific load-procedure as source
	EXEC import.spPOST_dFactories 'SQL','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dFactories') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dFactories
GO

CREATE PROCEDURE import.spPOST_dFactories
				(		 @Username AS NVARCHAR(255)
						,@FactoryID AS NVARCHAR(255)
						,@Source AS NVARCHAR(255)				
				)
AS

BEGIN
	SET NOCOUNT ON;

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode				INT				= 501;
	DECLARE @EffectedRows			INT				= 0;					
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username	, N'NULL')	+ N''',''' 
													+ ISNULL(@FactoryID	, N'NULL')	+ N''','''
													+ ISNULL(@Source	, N'NULL')	+ N'''';
	DECLARE @Comment				NVARCHAR(2000)	 = N'';					

	DECLARE @NameShort				NVARCHAR(255)
	DECLARE @NameLong				NVARCHAR(255)
	DECLARE @CommentUser			NVARCHAR(MAX)
	DECLARE @CommentDev				NVARCHAR(255)
	DECLARE @ResponsiblePerson		NVARCHAR(255)
	DECLARE @ImageName				NVARCHAR(255)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPOARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(		 FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL 
						,NameShort			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,NameLong			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentUser		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentDev			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ResponsiblePerson	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ImageName			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL	
				)
	INSERT INTO #tmp
		SELECT tdF.FactoryID,
			   tdF.NameShort,
			   tdF.NameLong,
			   tdF.CommentUser,
			   tdF.CommentDev,
			   tdF.ResponsiblePerson,
			   tdF.ImageName 
		FROM   load.tdFactories tdF
		WHERE	(@FactoryID = '' OR tdF.FactoryID = @FactoryID)
		AND		(@Source = '' OR tdF.Source = @Source) 

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT 
			 FactoryID			
			,NameShort			
			,NameLong			
			,CommentUser		
			,CommentDev			
			,ResponsiblePerson	
			,ImageName			
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = [dbo].[sx_pf_POST_Factory]
				 @Username			--@Username
				,@FactoryID			--@FactoryID
				,@NameShort			--@NameShort
				,@NameLong			--@NameLong
				,@CommentUser		--@CommentUser
				,@CommentDev		--@CommentDev
				,@ResponsiblePerson	--@ResponsiblePerson
				,@ImageName			--@ImageName	

		Print @ResultCode
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor
	
	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;
END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dFactories] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dFactories] TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import all values from load.tdProductlines with specific load-procedure as source

Testcall
EXEC import.spPOST_dProductlines 'SQL','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dProductlines') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dProductlines
GO

CREATE PROCEDURE import.spPOST_dProductlines
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductLineID	AS NVARCHAR(255),
						@Source			AS NVARCHAR(255)		  
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode						INT				= 501;
	DECLARE @EffectedRows					INT				= 0;					
	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @ParameterString				NVARCHAR(MAX)	= N''''
															+ ISNULL(@Username		, N'NULL')	 + N''',''' 
															+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
															+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
															+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment						NVARCHAR(2000)	= N'';						
	DECLARE @NameShort						NVARCHAR(255)
	DECLARE @NameLong						NVARCHAR(255)
	DECLARE @CommentUser					NVARCHAR(MAX)
	DECLARE @CommentDev						NVARCHAR(MAX)
	DECLARE @ResponsiblePerson				NVARCHAR(255)
	DECLARE @ImageName						NVARCHAR(255)
	DECLARE @DefaultTemplate				NVARCHAR(255)
	DECLARE @GlobalAttributeSource1			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias1 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource2			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias2 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource3			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias3 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource4			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias4 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource5			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias5 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource6			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias6 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource7			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias7 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource8			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias8 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource9			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias9 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource10		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias10			NVARCHAR(255)
	DECLARE @GlobalAttributeSource11		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias11			NVARCHAR(255)
	DECLARE @GlobalAttributeSource12		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias12			NVARCHAR(255)
	DECLARE @GlobalAttributeSource13		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias13			NVARCHAR(255)
	DECLARE @GlobalAttributeSource14		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias14			NVARCHAR(255)
	DECLARE @GlobalAttributeSource15		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias15			NVARCHAR(255)
	DECLARE @GlobalAttributeSource16		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias16			NVARCHAR(255)
	DECLARE @GlobalAttributeSource17		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias17			NVARCHAR(255)
	DECLARE @GlobalAttributeSource18		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias18			NVARCHAR(255)
	DECLARE @GlobalAttributeSource19		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias19			NVARCHAR(255)
	DECLARE @GlobalAttributeSource20		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias20			NVARCHAR(255)
	DECLARE @GlobalAttributeSource21		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias21			NVARCHAR(255)
	DECLARE @GlobalAttributeSource22		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias22			NVARCHAR(255)
	DECLARE @GlobalAttributeSource23		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias23			NVARCHAR(255)
	DECLARE @GlobalAttributeSource24		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias24			NVARCHAR(255)
	DECLARE @GlobalAttributeSource25		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias25			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(	ProductLineID			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,FactoryID				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameShort				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameLong				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentUser				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentDev				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ResponsiblePerson		NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ImageName				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,DefaultTemplate			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource1  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias1 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource2	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias2 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource3	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias3 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource4	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias4 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource5	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias5 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource6	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias6 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource7	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias7 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource8	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias8 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource9	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias9 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource10 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias10  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource11 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias11  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource12 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias12  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource13 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias13  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource14 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias14  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource15 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias15  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource16 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias16  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource17 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias17  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource18 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias18  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource19 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias19  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource20 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias20  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource21 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias21  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource22 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias22  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource23 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias23  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource24 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias24  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource25 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias25  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   )

	INSERT INTO #tmp
		SELECT   tdPL.ProductLineID			
				,tdPL.FactoryID				
				,tdPL.NameShort				
				,tdPL.NameLong				
				,tdPL.CommentUser			
				,tdPL.CommentDev			
				,tdPL.ResponsiblePerson		
				,tdPL.ImageName				
				,tdPL.DefaultTemplate		
				,tdPL.GlobalAttributeSource1
				,tdPL.GlobalAttributeAlias1 
				,tdPL.GlobalAttributeSource2
				,tdPL.GlobalAttributeAlias2 
				,tdPL.GlobalAttributeSource3
				,tdPL.GlobalAttributeAlias3 
				,tdPL.GlobalAttributeSource4
				,tdPL.GlobalAttributeAlias4 
				,tdPL.GlobalAttributeSource5
				,tdPL.GlobalAttributeAlias5 
				,tdPL.GlobalAttributeSource6
				,tdPL.GlobalAttributeAlias6 
				,tdPL.GlobalAttributeSource7
				,tdPL.GlobalAttributeAlias7 
				,tdPL.GlobalAttributeSource8
				,tdPL.GlobalAttributeAlias8 
				,tdPL.GlobalAttributeSource9
				,tdPL.GlobalAttributeAlias9 
				,tdPL.GlobalAttributeSource10
				,tdPL.GlobalAttributeAlias10 
				,tdPL.GlobalAttributeSource11
				,tdPL.GlobalAttributeAlias11 
				,tdPL.GlobalAttributeSource12
				,tdPL.GlobalAttributeAlias12 
				,tdPL.GlobalAttributeSource13
				,tdPL.GlobalAttributeAlias13 
				,tdPL.GlobalAttributeSource14
				,tdPL.GlobalAttributeAlias14 
				,tdPL.GlobalAttributeSource15
				,tdPL.GlobalAttributeAlias15 
				,tdPL.GlobalAttributeSource16
				,tdPL.GlobalAttributeAlias16 
				,tdPL.GlobalAttributeSource17
				,tdPL.GlobalAttributeAlias17 
				,tdPL.GlobalAttributeSource18
				,tdPL.GlobalAttributeAlias18 
				,tdPL.GlobalAttributeSource19
				,tdPL.GlobalAttributeAlias19 
				,tdPL.GlobalAttributeSource20
				,tdPL.GlobalAttributeAlias20 
				,tdPL.GlobalAttributeSource21
				,tdPL.GlobalAttributeAlias21 
				,tdPL.GlobalAttributeSource22
				,tdPL.GlobalAttributeAlias22 
				,tdPL.GlobalAttributeSource23
				,tdPL.GlobalAttributeAlias23 
				,tdPL.GlobalAttributeSource24
				,tdPL.GlobalAttributeAlias24 
				,tdPL.GlobalAttributeSource25
				,tdPL.GlobalAttributeAlias25  
		FROM	load.tdProductLines tdPL
		WHERE	(@FactoryID = ''		OR tdPL.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdPL.ProductLineID = @ProductLineID)
		 AND    (@Source = ''			OR tdPL.Source = @Source)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ProductLine
				 @Username
				,@ProductLineID				
				,@FactoryID			
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ResponsiblePerson
				,@ImageName
				,@DefaultTemplate
				,@GlobalAttributeSource1  
				,@GlobalAttributeAlias1 
				,@GlobalAttributeSource2
				,@GlobalAttributeAlias2 
				,@GlobalAttributeSource3
				,@GlobalAttributeAlias3 
				,@GlobalAttributeSource4
				,@GlobalAttributeAlias4 
				,@GlobalAttributeSource5
				,@GlobalAttributeAlias5 
				,@GlobalAttributeSource6
				,@GlobalAttributeAlias6 
				,@GlobalAttributeSource7
				,@GlobalAttributeAlias7 
				,@GlobalAttributeSource8
				,@GlobalAttributeAlias8 
				,@GlobalAttributeSource9
				,@GlobalAttributeAlias9 
				,@GlobalAttributeSource10 
				,@GlobalAttributeAlias10  
				,@GlobalAttributeSource11 
				,@GlobalAttributeAlias11  
				,@GlobalAttributeSource12 
				,@GlobalAttributeAlias12  
				,@GlobalAttributeSource13 
				,@GlobalAttributeAlias13  
				,@GlobalAttributeSource14 
				,@GlobalAttributeAlias14  
				,@GlobalAttributeSource15 
				,@GlobalAttributeAlias15  
				,@GlobalAttributeSource16 
				,@GlobalAttributeAlias16  
				,@GlobalAttributeSource17 
				,@GlobalAttributeAlias17  
				,@GlobalAttributeSource18 
				,@GlobalAttributeAlias18  
				,@GlobalAttributeSource19 
				,@GlobalAttributeAlias19  
				,@GlobalAttributeSource20 
				,@GlobalAttributeAlias20  
				,@GlobalAttributeSource21 
				,@GlobalAttributeAlias21  
				,@GlobalAttributeSource22 
				,@GlobalAttributeAlias22  
				,@GlobalAttributeSource23 
				,@GlobalAttributeAlias23  
				,@GlobalAttributeSource24 
				,@GlobalAttributeAlias24  
				,@GlobalAttributeSource25 
				,@GlobalAttributeAlias25 

		PRINT @ResultCode
	FETCH MyCursor INTO  @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProductlines TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_dProductlines TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/08

Import all values from load.tdProducts with specific load-procedure as source
EXEC import.spPOST_dProducts '','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dProducts') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dProducts
GO

CREATE PROCEDURE import.spPOST_dProducts
				(		 @Username		AS NVARCHAR(255)
						,@FactoryID		AS NVARCHAR(255)
						,@ProductLineID	AS NVARCHAR(255)
						,@ProductID		AS NVARCHAR(255)			
						,@Source		AS NVARCHAR(255)		
				)		  
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @TimeType					NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(MAX)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ResponsiblePerson			NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @Status						NVARCHAR(255)
	DECLARE @Template					NVARCHAR(255)
	DECLARE @TemplateVersion			NVARCHAR(255)
	DECLARE @GA1 						NVARCHAR(255)
	DECLARE @GA2 						NVARCHAR(255)
	DECLARE @GA3 						NVARCHAR(255)
	DECLARE @GA4 						NVARCHAR(255)
	DECLARE @GA5 						NVARCHAR(255)
	DECLARE @GA6 						NVARCHAR(255)
	DECLARE @GA7 						NVARCHAR(255)
	DECLARE @GA8 						NVARCHAR(255)
	DECLARE @GA9 						NVARCHAR(255)
	DECLARE @GA10						NVARCHAR(255)
	DECLARE @GA11						NVARCHAR(255)
	DECLARE @GA12						NVARCHAR(255)
	DECLARE @GA13						NVARCHAR(255)
	DECLARE @GA14						NVARCHAR(255)
	DECLARE @GA15						NVARCHAR(255)
	DECLARE @GA16						NVARCHAR(255)
	DECLARE @GA17						NVARCHAR(255)
	DECLARE @GA18						NVARCHAR(255)
	DECLARE @GA19						NVARCHAR(255)
	DECLARE @GA20						NVARCHAR(255)
	DECLARE @GA21						NVARCHAR(255)
	DECLARE @GA22						NVARCHAR(255)
	DECLARE @GA23						NVARCHAR(255)
	DECLARE @GA24						NVARCHAR(255)
	DECLARE @GA25						NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TimeType				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ResponsiblePerson		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Status					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Template				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TemplateVersion			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA1 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA2 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA3 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA4 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA5 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA6 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA7 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA8 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA9 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA10					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA11					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA12					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA13					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA14					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA15					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA16					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA17					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA18					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA19					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA20					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA21					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA22					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA23					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA24					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA25					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL		
			 )

	INSERT INTO #tmp
		SELECT   tdP.ProductID
				,tdP.ProductLineID			
				,tdP.FactoryID
				,tdP.TimeType				
				,tdP.NameShort				
				,tdP.NameLong				
				,tdP.CommentUser			
				,tdP.CommentDev			
				,tdP.ResponsiblePerson		
				,tdP.ImageName
				,tdP.Status				
				,tdP.Template
				,tdP.TemplateVersion		
				,tdP.GlobalAttribute1 
				,tdP.GlobalAttribute2 
				,tdP.GlobalAttribute3 
				,tdP.GlobalAttribute4 
				,tdP.GlobalAttribute5 
				,tdP.GlobalAttribute6 
				,tdP.GlobalAttribute7 
				,tdP.GlobalAttribute8 
				,tdP.GlobalAttribute9 
				,tdP.GlobalAttribute10
				,tdP.GlobalAttribute11
				,tdP.GlobalAttribute12
				,tdP.GlobalAttribute13
				,tdP.GlobalAttribute14
				,tdP.GlobalAttribute15
				,tdP.GlobalAttribute16
				,tdP.GlobalAttribute17
				,tdP.GlobalAttribute18
				,tdP.GlobalAttribute19
				,tdP.GlobalAttribute20
				,tdP.GlobalAttribute21
				,tdP.GlobalAttribute22
				,tdP.GlobalAttribute23
				,tdP.GlobalAttribute24
				,tdP.GlobalAttribute25
		FROM	load.tdProducts tdP
		WHERE	(@FactoryID = ''		OR tdP.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdP.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tdP.ProductID = @ProductID)
		 AND    (@Source = ''			OR tdP.Source = @Source)
	 
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Product
				 @Username
				,@ProductID
				,@ProductLineID				
				,@FactoryID			
				,@TimeType			
				,@NameShort			
				,@NameLong			
				,@CommentUser		
				,@CommentDev			
				,@ResponsiblePerson	
				,@ImageName			
				,@Status				
				,@Template			
				,@TemplateVersion	
				,@GA1 				
				,@GA2 				
				,@GA3 				
				,@GA4 				
				,@GA5 				
				,@GA6 				
				,@GA7 				
				,@GA8 				
				,@GA9 				
				,@GA10				
				,@GA11				
				,@GA12				
				,@GA13				
				,@GA14				
				,@GA15				
				,@GA16				
				,@GA17				
				,@GA18				
				,@GA19				
				,@GA20				
				,@GA21				
				,@GA22				
				,@GA23				
				,@GA24				
				,@GA25	
					
		Print @ResultCode
	FETCH MyCursor INTO  @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import ValueSeries from load.tdValueSeries with specific load-procedure as source
	EXEC import.spPOST_dValueSeries 'SQL','','','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dValueSeries') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dValueSeries
GO

CREATE PROCEDURE import.spPOST_dValueSeries
				(	 	 @Username			AS NVARCHAR(255)
						,@FactoryID			AS NVARCHAR(255)
						,@ProductlineID		AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID		AS NVARCHAR(255)
						,@Source			AS NVARCHAR(255)		
				)		   
AS

BEGIN	
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID	, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @RequestedValueSeriesNo		NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(255)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @IsNumeric					NVARCHAR(255)
	DECLARE @VisibilityLevel			NVARCHAR(255)
	DECLARE @ValueSource				NVARCHAR(255)
	DECLARE @ValueListID				NVARCHAR(255)
	DECLARE @ValueFormatID				NVARCHAR(255)
	DECLARE @Unit						NVARCHAR(255)
	DECLARE @Scale						NVARCHAR(255)
	DECLARE @Effect						NVARCHAR(255)
	DECLARE @EffectParameter			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(		ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSeriesID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,RequestedValueSeriesNo	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,IsNumeric				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,VisibilityLevel			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSource				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueListID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueFormatID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Unit					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Scale					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Effect					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,EffectParameter			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			 )

	INSERT INTO #tmp
		SELECT   tdVS.ProductID				
				,tdVS.ProductLineID			
				,tdVS.FactoryID				
				,tdVS.ValueSeriesID			
				,tdVS.ValueSeriesNo	
				,tdVS.NameShort				
				,tdVS.NameLong				
				,tdVS.CommentUser				
				,tdVS.CommentDev				
				,tdVS.ImageName				
				,tdVS.IsNumeric				
				,tdVS.VisibilityLevel			
				,tdVS.ValueSource				
				,tdVS.ValueListID				
				,tdVS.ValueFormatID			
				,tdVS.Unit					
				,tdVS.Scale					
				,tdVS.Effect					
				,tdVS.EffectParameter	
		FROM	load.tdValueSeries tdVS
		WHERE	(@FactoryID = '' OR tdVS.FactoryID = @FactoryID)
		 AND    (@ProductLineID = '' OR tdVS.ProductLineID = @ProductLineID)
		 AND    (@ProductID = '' OR tdVS.ProductID = @ProductID)
		 AND    (@ValueSeriesID = '' OR tdVS.ValueSeriesID = @ValueSeriesID)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter

	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ValueSerie
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter				
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;
-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dValueSeries] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dValueSeries] TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/07
Import all values from load.tfValues with specific load-procedure as source
	 EXEC import.spPOST_fValues 'SQL','','','','',0,''
*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_fValues') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_fValues
GO

CREATE PROCEDURE import.spPOST_fValues
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductlineID	AS NVARCHAR(255),
						@ProductID		AS NVARCHAR(255),
						@ValueSeriesID	AS NVARCHAR(255),
						@TimeID			AS INT,
						@Source			AS NVARCHAR(255)		
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username							, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
														+ ISNULL(@ProductID							, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
														+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
														+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';	
	DECLARE @ValueFormula				NVARCHAR(255)
	DECLARE @ValueINT					INT
	DECLARE @ValueText					NVARCHAR(MAX)
	DECLARE @ValueComment				NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(	 ProductID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,TimeID			INT										NOT NULL
			,ValueFormula	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueINT		INT										NOT NULL
			,ValueText		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueComment	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL		
		)

	INSERT INTO #tmp
		SELECT   tfV.ProductID		
				,tfV.ProductLineID	
				,tfV.FactoryID		
				,tfV.ValueSeriesID	
				,tfV.TimeID			
				,tfV.ValueFormula	
				,tfV.ValueINT		
				,tfV.ValueText		
				,tfV.ValueComment			
		FROM	load.tfValues tfV
		WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tfV.ProductID = @ProductID)
		 AND    (@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
		 AND    (@TimeID = 0			OR tfV.TimeID = @TimeID)
		 AND    (@Source = ''			OR tfV.Source = @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Value
				 @Username
				,@ProductID		
				,@ProductLineID	
				,@FactoryID		
				,@ValueSeriesID	
				,@TimeID			
				,@ValueFormula	
				,@ValueINT		
				,@ValueText		
				,@ValueComment	
										
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValues] TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import values from load.tfValues with specific load-procedure as source
	EXEC [import].[spPOST_fValuesPDTV] 'SQL','','','','',0,''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[import].[spPOST_fValuesPDTV]') AND type in (N'P', N'PC'))
DROP PROCEDURE [import].[spPOST_fValuesPDTV]
GO

CREATE PROCEDURE [import].[spPOST_fValuesPDTV]
				(		 @Username 			AS NVARCHAR(255)
						,@FactoryID 		AS NVARCHAR(255)
						,@ProductlineID 	AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID 	AS NVARCHAR(255)
						,@TimeID			AS INT = 0
						,@Source			AS NVARCHAR(255)		
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall						DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName						NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode							INT			= 501;
	DECLARE @EffectedRows						INT			= 0;					
	DECLARE @TransactUsername					NVARCHAR(255)		= N'';
	DECLARE @ParameterString					NVARCHAR(MAX)	= N''''
																+ ISNULL(@Username							, N'NULL')	 + N''',''' 
																+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
																+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
																+ ISNULL(@ProductID							, N'NULL')	 + N''','''
																+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
																+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
																+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment							NVARCHAR(2000) = N'';	
	DECLARE @IsIncrementalValuesFlag			INT
	DECLARE	@ValuesInBracketsCommaSeparated 	NVARCHAR(MAX) 
	DECLARE @ToDoList AS TABLE
		(	 ProductID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID 						NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,FactoryID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,IsIncrementalValuesFlag 			INT				NOT NULL
			,ValuesInBracketsCommaSeparated 	NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
		)
				
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	-- Temporäre Tabelle als Datensammler
	IF OBJECT_ID('tempdb..#DATA') IS NOT NULL DROP TABLE #DATA
	CREATE TABLE #DATA	
			(
				 ProductID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ProductLineID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,TimeID				INT										NOT NULL
				,ValueFormula		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueINT			BIGINT									NOT NULL
				,ValueText			NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueComment		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			)

	INSERT INTO #DATA
			SELECT   tfV.ProductID		
					,tfV.ProductLineID	
					,tfV.FactoryID		
					,tfV.ValueSeriesID	
					,tfV.TimeID			
					,tfV.ValueFormula	
					,tfV.ValueINT		
					,tfV.ValueText		
					,tfV.ValueComment			
			FROM	load.tfValues tfV
			WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
			  AND 	(@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
			  AND	(@ProductID = ''		OR tfV.ProductID = @ProductID)
			  AND	(@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
			  AND   (@TimeID = 0			OR tfV.TimeID = @TimeID)
			  AND   (@Source = ''			OR tfV.Source = @Source)


	-- Tabelle für String
	INSERT INTO @ToDoList
		SELECT	 d.ProductID
				,d.ProductLineId
				,d.FactoryID
				,1
				,(SELECT  N'[(''' 
					+ dxml.ValueSeriesID + N''','  
					+ CAST(dxml.TimeID AS NVARCHAR(255))+ N',''' 
					+ dxml.ValueFormula + N''',''' 
					+ CAST(dxml.ValueInt AS NVARCHAR(255)) + N''',''' 
					+ dxml.ValueText + N''',''' 
					+ dxml.ValueComment + N''')],'
				FROM #DATA dxml
				WHERE d.FactoryID = dxml.FactoryID AND d.ProductLineID = dxml.ProductLineID AND d.ProductID = dxml.ProductID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS ValuesInBracketsCommaSeparated
		FROM #DATA d
		GROUP BY   d.ProductID, d.ProductLineId, d.FactoryID

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR STATIC FOR

		SELECT tdl.ProductID
			  ,tdl.ProductLineID
			  ,tdl.FactoryID
			  ,tdl.IsIncrementalValuesFlag
			  ,''+LEFT(tdl.ValuesInBracketsCommaSeparated,LEN(tdl.ValuesInBracketsCommaSeparated)-1)+''
		FROM @ToDoList tdl

	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_ProductDataTableValues
			 @UserName
			,@ProductID		
			,@ProductLineID	
			,@FactoryID		
			,@IsIncrementalValuesFlag
			,@ValuesInBracketsCommaSeparated	
											
	Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValuesPDTV] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValuesPDTV] TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/07
Import rights from loadtable with specific load-procedure as source

Testcall
EXEC import.spPOST_rRights 'SQL', '','',''
*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_rRights') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_rRights
GO

CREATE PROCEDURE import.spPOST_rRights
				(		 @Username		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Right			AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)			
AS

BEGIN
	SET NOCOUNT ON
		
	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
														+ ISNULL(@Right			, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';
	DECLARE @FactoryID					NVARCHAR(255)	= ''
	DECLARE @ProductLineID				NVARCHAR(255)	= ''
	DECLARE @ProductID					NVARCHAR(255)	= ''
	DECLARE	@ReadCommentMandatory		INT 			= ''
	DECLARE @WriteCommentMandatory		INT				= ''

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
		CREATE TABLE #tmp 
						(	 UserName		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,FactoryID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductLineID   NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,[Right]		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ReadCommentMandatory	INT								NOT NULL
							,WriteCommentMandatory	INT								NOT NULL
						)
		INSERT INTO #tmp
			SELECT   trR.UserName		 
					,trR.FactoryID		 
					,trR.ProductLineID  
					,trR.ProductID		 
					,trR.[Right]		 
					,trR.ReadCommentMandatory
					,trR.WriteCommentMandatory
			FROM	load.trRights trR
			WHERE	(@PostUserName = '' OR trR.UserName = @PostUserName)
			AND 	(@Right = ''		OR trR.[Right] = @Right)
			AND 	(@Source = ''		OR trR.Source = @Source)   
			AND		trR.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT * 
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_Right
			 @Username
			,@PostUserName				
			,@FactoryID				
			,@ProductLineID						
			,@Right					
			,@ReadCommentMandatory 
			,@WriteCommentMandatory

	Print @ResultCode
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rRights TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_rRights TO pf_PlanningFactoryService;
GO

/*
Stefan Lindenlaub
2018/07
Import User from load.trUser

Testcall
EXEC import.spPOST_rUser 'SQL','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_rUser') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_rUser
GO

CREATE PROCEDURE import.spPOST_rUser
			     (		 @UserName		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)

AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode				INT				= 501;
	DECLARE @EffectedRows			INT				= 0;					
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username		, N'NULL')	 + N''',''' 
													+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
													+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment				NVARCHAR(2000) = N'';	
	DECLARE @PersonSurname			NVARCHAR(255)
	DECLARE @PersonFirstName		NVARCHAR(255)
	DECLARE @EMail					NVARCHAR(255)
	DECLARE @LDAPIP					NVARCHAR(255)
	DECLARE @Status					NVARCHAR(255)
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	 UserName		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonSurname	 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonFirstName NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,EMail			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,LDAPIP			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,Status			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			)

	INSERT INTO #tmp
		SELECT   trU.UserName
				,trU.PersonSurname
				,trU.PersonFirstName
				,trU.EMail		
				,trU.LDAPIP		
				,trU.Status		
		FROM	load.trUser trU
		WHERE	(@PostUserName = '' OR trU.UserName = @PostUserName)
		  AND   (@Source = ''		OR trU.Source = @Source)
		  AND	trU.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_User
				 @UserName				
				,@PostUserName
				,@PersonSurname	
				,@PersonFirstName
				,@EMail			
				,@LDAPIP			
				,@Status			

		Print @ResultCode
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rUser TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_rUser TO pf_PlanningFactoryService;
GO
/*
MOVE Operation for a Factory
Factory moved by the update IDs without key change

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_pProtectString
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory => 404 Not Found
5. Exists target Factory => 403 Forbidden
6. Factory moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT'

EXECUTE @RC = [sx_pf_MOVE_Factory] 
		   @Username
		  ,@SourceFactoryID
		  ,@TargetFactoryID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Factory]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_Factory];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_Factory]
	@Username AS NVARCHAR(255),
	@SourceFactoryID AS NVARCHAR(255),
	@TargetFactoryID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey AS INT = 0;
	DECLARE @SourceFactoryKey AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@SourceFactoryID, N'NULL') + N''',''' + ISNULL(@TargetFactoryID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @SourceFactoryID = [dbo].[sx_pf_pProtectID] (@SourceFactoryID);
		SET @TargetFactoryID = [dbo].[sx_pf_pProtectID] (@TargetFactoryID);

		IF @SourceFactoryID = N'' OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @SourceFactoryID;
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @TargetFactoryID;
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Factory key already exists', 16, 10);
		END;
		
		-- STERP 3 - Move data
		--dFactory 
		UPDATE [dbo].[sx_pf_dFactories]
		SET FactoryID = @TargetFactoryID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--dProductLine  
			UPDATE [dbo].[sx_pf_dProductLines]
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dProduct 
			UPDATE [dbo].[sx_pf_dProducts]
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			UPDATE [dbo].[sx_pf_dValueSeries]
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE [dbo].[sx_pf_dValueSeries]
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE [dbo].[sx_pf_fValues]
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gFactories
			UPDATE [dbo].[sx_pf_gFactories] 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gProductLines Inhalt
			UPDATE [dbo].[sx_pf_gProductLines] 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--gProducts 
			UPDATE [dbo].[sx_pf_gProducts] 
			SET FactoryID  = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;
		
			SET @EffectedRows += @@ROWCOUNT;

			--rRights 
			UPDATE [dbo].[sx_pf_rRights] 
			SET FactoryID = @TargetFactoryID
			WHERE FactoryID = @SourceFactoryID;

			SET @EffectedRows += @@ROWCOUNT;

			--rPreferences
			UPDATE [dbo].[sx_pf_rPreferences] 
			SET FactoryID = @TargetFactoryID,
				UserKey = @TransactUserKey,
				UserName = @TransactUsername
			WHERE FactoryID = @SourceFactoryID;

			SET @EffectedRows += @@ROWCOUNT;

			SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

			--fStatements
			UPDATE [dbo].[sx_pf_fStatements] 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			INSERT INTO [dbo].[sx_pf_fStatements]   
			SELECT ProductKey
				, dp.ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, ProductID
				, dp.ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'Factory is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
					+ N'" / ProductLine <' + dp.ProductlineID + N'>"' + ISNULL(dpl.Nameshort, N'NULL') 
					+ N'" / Product <' + ProductID  + N'>"' + dp.Nameshort +  N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved
			FROM [dbo].[sx_pf_dProducts] dp LEFT JOIN [dbo].[sx_pf_dProductLines] dpl
				ON dp.ProductLinekey = dpl.ProductLineKey
			WHERE dp.FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Rights materialization
			EXEC [dbo].[sx_pf_materialize_vUserRights];

		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Factory] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Factory] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a Format - all objects, which an FormatID are switched to use a new FormatID.
Replace the OldFormatID with the new FormatID in sx_pf_dValueSeries
Replace the OldFormatID with the new FormatID in sx_pf_hListValues
Delete the OldFormatID in sx_pf_hFormats

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

12/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FormatID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Format => 404 Not Found
5. Not exists target Format => 404 Not Found
6. Format moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldFormatID AS NVARCHAR(255)=''
		DECLARE  @NewFormatID AS NVARCHAR(255)=''

EXECUTE @RC = [sx_pf_MOVE_Format]
		   @Username
		  ,@OldFormatID
		  ,@NewFormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Format]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_Format];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_Format]
	@Username AS NVARCHAR(255),
	@OldFormatID AS NVARCHAR(255),
	@NewFormatID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldFormatKey AS INT = 0;
	DECLARE @NewFormatKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldFormatID, N'NULL') + N''',''' + ISNULL(@NewFormatID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldFormatID IS NULL SET @OldFormatID = N'';
	IF @NewFormatID IS NULL SET @NewFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @OldFormatID = [dbo].[sx_pf_pProtectID] (@OldFormatID);
		SET @NewFormatID = [dbo].[sx_pf_pProtectID] (@NewFormatID);

		IF (@OldFormatID = N'' OR @NewFormatID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldFormatKey = FormatKey FROM [dbo].[sx_pf_hFormats] WHERE [FormatID] = @OldFormatID;
		
		IF @OldFormatKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewFormatKey = FormatKey FROM [dbo].[sx_pf_hFormats] WHERE [FormatID] = @NewFormatID;
		
		IF @NewFormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		UPDATE [dbo].[sx_pf_dValueSeries]
		   SET [ValueFormatID] = @NewFormatID 
		WHERE [ValueFormatID] = @OldFormatID;
																							
		SET @EffectedRows += @@ROWCOUNT;
																						   
		UPDATE [dbo].[sx_pf_hListValues]
		   SET [FormatID] = @NewFormatID 
		WHERE [FormatID] = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		UPDATE [dbo].[sx_pf_dTime]
		   SET [FormatID] = @NewFormatID 
		WHERE [FormatID] = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM [dbo].[sx_pf_hFormats] WHERE [FormatID] = @OldFormatID;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Format] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Format] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a List
The Procedure will change the OldListID into the NewListID in tables:
[sx_pf_dValueSeries], column ValueListID
[sx_pf_dProductlines], columns GlobalAttributeSource 1..25
create members from OldListID in NewListID if they don't exist there
(for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
delete the OldListID in [sx_pf_hLists]
delete the Old List Values in [sx_pf_hListValues]

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

12/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target ListID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source List => 404 Not Found
5. Not exists target List => 404 Not Found
6. List moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldListID AS NVARCHAR(255)='suEvents'
		DECLARE  @NewListID AS NVARCHAR(255)='suYesNo'

EXECUTE @RC = [sx_pf_MOVE_List]
		   @Username
		  ,@OldListID
		  ,@NewListID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_List]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_List];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_List]
	@Username AS NVARCHAR(255),
	@OldListID AS NVARCHAR(255),
	@NewListID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldListKey AS INT = 0;
	DECLARE @NewListKey AS INT = 0;
	DECLARE @NewListType AS NVARCHAR(255)= N'';

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldListID, N'NULL') + N''',''' + ISNULL(@NewListID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldListID IS NULL SET @OldListID = N'';
	IF @NewListID IS NULL SET @NewListID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @OldListID = [dbo].[sx_pf_pProtectID] (@OldListID);
		SET @NewListID = [dbo].[sx_pf_pProtectID] (@NewListID);

		IF (@OldListID = N'' OR @NewListID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @OldListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
			OR @NewListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel')
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
 		END;;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldListKey = ListKey FROM [dbo].[sx_pf_hLists] WHERE [ListID] = @OldListID;
		
		IF @OldListKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewListKey = [ListKey], @NewListType = [Datentyp] FROM [dbo].[sx_pf_hLists] WHERE [ListID] = @NewListID;
		
		IF @NewListKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
							 
		-- STEP 3 - Move data
		-- STEP 3.1 - Move not exists UNIQUE list members

		IF @NewListType = 'Integer'
		BEGIN

			;WITH vals AS (
				SELECT v1.[ListValueKey] 
					, v1.[ValueInt]
					, v1.[Scale]
					, v1.[ValueText]
					, v1.[ValueComment]
					, v1.[FormatID]
				FROM [dbo].[sx_pf_hListValues] v1 LEFT JOIN  [dbo].[sx_pf_hListValues] v2 
					ON v2.[ListID] = @NewListID AND v1.ValueInt = v2.ValueInt
				WHERE v1.[ListID] = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES [ListValueKey]
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY [ValueInt] ORDER BY [ListValueKey])
			)																				   
			INSERT INTO [dbo].[sx_pf_hListValues] ([ListID], [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID])
			SELECT @NewListID, [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID]
			FROM vals INNER JOIN uniq ON vals.[ListValueKey] = uniq.[ListValueKey];

			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE IF @NewListType = 'String'
		BEGIN

			;WITH vals AS (
				SELECT v1.[ListValueKey] 
					, v1.[ValueInt]
					, v1.[Scale]
					, v1.[ValueText]
					, v1.[ValueComment]
					, v1.[FormatID]
				FROM [dbo].[sx_pf_hListValues] v1 LEFT JOIN  [dbo].[sx_pf_hListValues] v2 
					ON v2.[ListID] = @NewListID AND v1.[ValueText] = v2.[ValueText]
				WHERE v1.[ListID] = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES [ListValueKey]
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY [ValueText] ORDER BY [ListValueKey])
			)																				   
			INSERT INTO [dbo].[sx_pf_hListValues] ([ListID], [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID])
			SELECT @NewListID, [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID]
			FROM vals INNER JOIN uniq ON vals.[ListValueKey] = uniq.[ListValueKey];

			SET @EffectedRows += @@ROWCOUNT;
		END;

		-- STEP 3.2 - Force all objects, which use a old list, to use a new list now
		UPDATE [dbo].[sx_pf_dValueSeries]
		   SET [ValueListID] = @NewListID 
		WHERE [ValueListID] = @OldListID;
																							
		SET @EffectedRows += @@ROWCOUNT;
		
		;WITH lines AS (
			SELECT [ProductLineKey] 
			FROM [dbo].[sx_pf_dProductLines]
			WHERE [GlobalAttributeSource1] = @OldListID OR [GlobalAttributeSource2] = @OldListID OR [GlobalAttributeSource3] = @OldListID OR
				[GlobalAttributeSource4] = @OldListID OR [GlobalAttributeSource5] = @OldListID OR [GlobalAttributeSource6] = @OldListID OR
				[GlobalAttributeSource7] = @OldListID OR [GlobalAttributeSource8] = @OldListID OR [GlobalAttributeSource9] = @OldListID OR
				[GlobalAttributeSource10] = @OldListID OR [GlobalAttributeSource11] = @OldListID OR [GlobalAttributeSource12] = @OldListID OR
				[GlobalAttributeSource13] = @OldListID OR [GlobalAttributeSource14] = @OldListID OR [GlobalAttributeSource15] = @OldListID OR
				[GlobalAttributeSource16] = @OldListID OR [GlobalAttributeSource17] = @OldListID OR [GlobalAttributeSource18] = @OldListID OR
				[GlobalAttributeSource19] = @OldListID OR [GlobalAttributeSource20] = @OldListID OR [GlobalAttributeSource21] = @OldListID OR
				[GlobalAttributeSource22] = @OldListID OR [GlobalAttributeSource23] = @OldListID OR [GlobalAttributeSource24] = @OldListID OR
				[GlobalAttributeSource25] = @OldListID 
		)
		UPDATE pl
		SET [GlobalAttributeSource1] = IIF([GlobalAttributeSource1] = @OldListID, @NewListID, [GlobalAttributeSource1])
			, [GlobalAttributeSource2] = IIF([GlobalAttributeSource2] = @OldListID, @NewListID, [GlobalAttributeSource2])
			, [GlobalAttributeSource3] = IIF([GlobalAttributeSource3] = @OldListID, @NewListID, [GlobalAttributeSource3])
			, [GlobalAttributeSource4] = IIF([GlobalAttributeSource4] = @OldListID, @NewListID, [GlobalAttributeSource4])
			, [GlobalAttributeSource5] = IIF([GlobalAttributeSource5] = @OldListID, @NewListID, [GlobalAttributeSource5])
			, [GlobalAttributeSource6] = IIF([GlobalAttributeSource6] = @OldListID, @NewListID, [GlobalAttributeSource6])
			, [GlobalAttributeSource7] = IIF([GlobalAttributeSource7] = @OldListID, @NewListID, [GlobalAttributeSource7])
			, [GlobalAttributeSource8] = IIF([GlobalAttributeSource8] = @OldListID, @NewListID, [GlobalAttributeSource8])
			, [GlobalAttributeSource9] = IIF([GlobalAttributeSource9] = @OldListID, @NewListID, [GlobalAttributeSource9])
			, [GlobalAttributeSource10] = IIF([GlobalAttributeSource10] = @OldListID, @NewListID, [GlobalAttributeSource10])
			, [GlobalAttributeSource11] = IIF([GlobalAttributeSource11] = @OldListID, @NewListID, [GlobalAttributeSource11])
			, [GlobalAttributeSource12] = IIF([GlobalAttributeSource12] = @OldListID, @NewListID, [GlobalAttributeSource12])
			, [GlobalAttributeSource13] = IIF([GlobalAttributeSource13] = @OldListID, @NewListID, [GlobalAttributeSource13])
			, [GlobalAttributeSource14] = IIF([GlobalAttributeSource14] = @OldListID, @NewListID, [GlobalAttributeSource14])
			, [GlobalAttributeSource15] = IIF([GlobalAttributeSource15] = @OldListID, @NewListID, [GlobalAttributeSource15])
			, [GlobalAttributeSource16] = IIF([GlobalAttributeSource16] = @OldListID, @NewListID, [GlobalAttributeSource16])
			, [GlobalAttributeSource17] = IIF([GlobalAttributeSource17] = @OldListID, @NewListID, [GlobalAttributeSource17])
			, [GlobalAttributeSource18] = IIF([GlobalAttributeSource18] = @OldListID, @NewListID, [GlobalAttributeSource18])
			, [GlobalAttributeSource19] = IIF([GlobalAttributeSource19] = @OldListID, @NewListID, [GlobalAttributeSource19])
			, [GlobalAttributeSource20] = IIF([GlobalAttributeSource20] = @OldListID, @NewListID, [GlobalAttributeSource20])
			, [GlobalAttributeSource21] = IIF([GlobalAttributeSource21] = @OldListID, @NewListID, [GlobalAttributeSource21])
			, [GlobalAttributeSource22] = IIF([GlobalAttributeSource22] = @OldListID, @NewListID, [GlobalAttributeSource22])
			, [GlobalAttributeSource23] = IIF([GlobalAttributeSource23] = @OldListID, @NewListID, [GlobalAttributeSource23])
			, [GlobalAttributeSource24] = IIF([GlobalAttributeSource24] = @OldListID, @NewListID, [GlobalAttributeSource24])
			, [GlobalAttributeSource25] = IIF([GlobalAttributeSource25] = @OldListID, @NewListID, [GlobalAttributeSource25])
		FROM [dbo].[sx_pf_dProductLines] pl INNER JOIN lines l ON pl.ProductLineKey = l.ProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Remove old list
				
		DELETE FROM [dbo].[sx_pf_hListValues] WHERE [ListID] = @OldListID;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM [dbo].[sx_pf_hLists] WHERE [ListKey] = @OldListKey;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_List] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_List] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a Product
Product moved by ID update

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_pProtectString
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineReadRight
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product => 404 Not Found
5. Not exists target Factory / Productline => 404
6. Exists TargetProduct => 403 Forbidden 
7. Product moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='230'
		DECLARE  @SourceProductLineID AS NVARCHAR(255)='I-ST-IT'
		DECLARE  @SourceProductID AS NVARCHAR(255)='888'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductLineID AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductID AS NVARCHAR(255)='RZ888'

EXECUTE @RC = [sx_pf_MOVE_Product] 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Product]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_Product];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_Product]
	@Username AS NVARCHAR(255),
	@SourceFactoryID AS NVARCHAR(255),
	@SourceProductLineID AS NVARCHAR(255),
	@SourceProductID AS NVARCHAR(255),
	@TargetFactoryID AS NVARCHAR(255),
	@TargetProductLineID AS NVARCHAR(255),
	@TargetProductID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey AS INT = 0;
	DECLARE @SourceProductKey AS INT = 0;
	DECLARE @SourceProductLineKey AS INT = 0;
	DECLARE @TargetProductKey AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductLineNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey AS INT = 0;
	DECLARE @TargetProductLineKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@SourceFactoryID, N'NULL') + N''',''' + ISNULL(@SourceProductLineID, N'NULL') + N''',''' + ISNULL(@SourceProductID, N'NULL') 
			+ N''',''' + ISNULL(@TargetFactoryID, N'NULL') + N''',''' + ISNULL(@TargetProductLineID, N'NULL') + N''',''' + ISNULL(@TargetProductID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID = N'';
	IF @SourceProductID IS NULL SET @SourceProductID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID = N'';
	IF @TargetProductID IS NULL SET @TargetProductID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @SourceFactoryID = [dbo].[sx_pf_pProtectID] (@SourceFactoryID);
		SET @SourceProductlineID = [dbo].[sx_pf_pProtectID] (@SourceProductlineID);
		SET @SourceProductID = [dbo].[sx_pf_pProtectID] (@SourceProductID);
		SET @TargetFactoryID = [dbo].[sx_pf_pProtectID] (@TargetFactoryID);
		SET @TargetProductlineID = [dbo].[sx_pf_pProtectID] (@TargetProductlineID);
		SET @TargetProductID = [dbo].[sx_pf_pProtectID] (@TargetProductID);

		IF (@SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @SourceProductID = N''
			OR @TargetFactoryID = N'' OR @TargetProductLineID = N'' OR @TargetProductID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ProductLineWriteRight] @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ProductLineReadRight] @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM [dbo].[sx_pf_dProducts] WHERE FactoryKey = @SourceFactoryKey AND ProductlineKey = @SourceProductlineKey AND ProductID = @SourceProductID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0 OR @SourceProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM [dbo].[sx_pf_dProducts] WHERE FactoryKey = @TargetFactoryKey AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		
		IF @TargetFactoryKey = 0 OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		--dProduct 
		UPDATE [dbo].[sx_pf_dProducts] 
		SET FactoryKey = @TargetFactoryKey
			, ProductLineKey = @TargetProductLineKey
			, FactoryID = @TargetFactoryID
			, ProductLineID = @TargetProductLineID
			, ProductID = @TargetProductID 
		WHERE ProductKey = @SourceProductKey;
					 
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--gProducts 
			UPDATE [dbo].[sx_pf_gProducts] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--dValueSeries
			UPDATE [dbo].[sx_pf_dValueSeries] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE [dbo].[sx_pf_dTime] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE [dbo].[sx_pf_fValues] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements 
			UPDATE [dbo].[sx_pf_fStatements] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			INSERT INTO [dbo].[sx_pf_fStatements] 
			SELECT @SourceProductKey AS ProductKey
				, @TargetProductLineKey AS ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, @TargetProductID  AS ProductID
				, @TargetProductlineID AS ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'Product is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
					+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
					+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Product] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_Product] TO pf_PlanningFactoryService;
GO
/*
MOVE Operation for a ProductLine
Productline moved by the update IDs without key change

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_pProtectString
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_FactoryReadRight
		- sx_pf_pGET_FactoryWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline => 404 Not Found
5. Not exists target  Factory und existierende Target Productline => 404  Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ProductLine moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @SourceProductLineID AS NVARCHAR(255)='ZT'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductLineID AS NVARCHAR(255)='ZT32'

EXECUTE @RC = [sx_pf_MOVE_ProductLine] 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@TargetFactoryID
		  ,@TargetProductLineID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_ProductLine]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_ProductLine];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_ProductLine]
	@Username AS NVARCHAR(255),
	@SourceFactoryID AS NVARCHAR(255),
	@SourceProductLineID AS NVARCHAR(255),
	@TargetFactoryID AS NVARCHAR(255),
	@TargetProductLineID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey AS INT = 0;
	DECLARE @SourceFactoryKey AS INT = 0;
	DECLARE @SourceProductLineKey AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductLineNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey AS INT = 0;
	DECLARE @TargetProductLineKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@SourceFactoryID, N'NULL') + N''',''' + ISNULL(@SourceProductLineID, N'NULL') + N''',''' + ISNULL(@TargetFactoryID, N'NULL') 
		 + N''',''' + ISNULL(@TargetProductLineID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;

		-- STEP 0.2 - Protect input parameters
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @SourceFactoryID = [dbo].[sx_pf_pProtectID] (@SourceFactoryID);
		SET @SourceProductlineID = [dbo].[sx_pf_pProtectID] (@SourceProductlineID);
		SET @TargetFactoryID = [dbo].[sx_pf_pProtectID] (@TargetFactoryID);
		SET @TargetProductlineID = [dbo].[sx_pf_pProtectID] (@TargetProductlineID);

		IF @SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @TargetFactoryID = N''
			OR @TargetProductlineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_FactoryWriteRight] @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = [dbo].[sx_pf_pGET_FactoryReadRight] @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);
		
		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Move data		
		--dProductLine 
		UPDATE [dbo].[sx_pf_dProductLines] 
		SET FactoryKey = @TargetFactoryKey
			, ProductLineID = @TargetProductLineID
			, FactoryID = @TargetFactoryID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--dProduct
			UPDATE [dbo].[sx_pf_dProducts] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			UPDATE [dbo].[sx_pf_dValueSeries] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE [dbo].[sx_pf_dTime] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE [dbo].[sx_pf_fValues] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gProductLines
			UPDATE [dbo].[sx_pf_gProductLines] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--gProducts 
			UPDATE [dbo].[sx_pf_gProducts] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--rRights
			UPDATE [dbo].[sx_pf_rRights] 
			SET FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
			WHERE FactoryID = @SourceFactoryID 
				AND ProductLineID = @SourceProductLineID;

			SET @EffectedRows += @@ROWCOUNT;

			--rPreferences
			UPDATE [dbo].[sx_pf_rPreferences] 
			SET ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID
				, UserKey = @TransactUserKey
				, UserName = @TransactUsername
			WHERE FactoryID = @SourceFactoryID 
				AND ProductlineID = @SourceProductlineID;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements
			UPDATE [dbo].[sx_pf_fStatements] 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			INSERT INTO [dbo].[sx_pf_fStatements]
			SELECT ProductKey
				, @SourceProductLineKey AS ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, ProductID
				, @TargetProductlineID AS ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'ProductLine is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
					+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
					+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved
			FROM [dbo].[sx_pf_dProducts] 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Rights materialization
			EXEC [dbo].[sx_pf_materialize_vUserRights];

		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_ProductLine] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_ProductLine] TO pf_PlanningFactoryService;
GO
/*
Move Operation for one ValueSeries
The ValueSeries will be moved to a new ID, only if this ID is free
Works only inside one Product

Dependencies:
	- Functions: 
		- sx_pf_pProtectID
		- sx_pf_pProtectString
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty  Source or Target  FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists Source Factory / Productline / Product / ValueSerie => 404 Not Found
5. Not exists Target  Factory / Productline / Product => 404 Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ValueSerie moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @FactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @ProductLineID AS NVARCHAR(255)='U'
		DECLARE  @ProductID AS NVARCHAR(255)='2'
		DECLARE  @SourceValueSerieID  AS NVARCHAR(255)='K1'
		DECLARE  @TargetValueSerieID AS NVARCHAR(255)='K1'

EXECUTE @RC = [sx_pf_MOVE_ValueSerie] 
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		  ,@ProductID
		  ,@SourceValueSerieID
		  ,@TargetValueSerieID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_ValueSerie]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_MOVE_ValueSerie];
GO

CREATE PROCEDURE [dbo].[sx_pf_MOVE_ValueSerie]
	@Username AS NVARCHAR(255),
	@FactoryID AS NVARCHAR(255),
	@ProductLineID AS NVARCHAR(255),
	@ProductID AS NVARCHAR(255),
	@SourceValueSerieID AS NVARCHAR(255),
	@TargetValueSerieID AS NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey AS INT = 0;
	DECLARE @ProductLineKey AS INT = 0;
	DECLARE @ProductKey AS INT = 0;
	DECLARE @SourceValueSerieKey AS INT = 0;
	DECLARE @TargetValueSerieKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@FactoryID, N'NULL') + N''',''' + ISNULL(@ProductLineID, N'NULL') + N''',''' + ISNULL(@ProductID, N'NULL') + N''',''' +
		 N''',''' + ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL') + N''',''' + ISNULL(@TargetValueSerieID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FactoryID IS NULL SET @FactoryID = N'';
	IF @ProductLineID IS NULL SET @ProductLineID = N'';
	IF @ProductID IS NULL SET @ProductID = N'';
	IF @SourceValueSerieID IS NULL SET @SourceValueSerieID = N'';
	IF @TargetValueSerieID IS NULL SET @TargetValueSerieID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;

		-- STEP 0.2 - Protect input parameters
		SET @Username = [dbo].[sx_pf_pProtectString] (@Username);
		SET @FactoryID = [dbo].[sx_pf_pProtectID] (@FactoryID);
		SET @ProductLineID = [dbo].[sx_pf_pProtectID] (@ProductLineID);
		SET @ProductID = [dbo].[sx_pf_pProtectID] (@ProductID);
		SET @SourceValueSerieID = [dbo].[sx_pf_pProtectID] (@SourceValueSerieID);
		SET @TargetValueSerieID = [dbo].[sx_pf_pProtectID] (@TargetValueSerieID);

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR  @SourceValueSerieID = N'' OR  @TargetValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = [dbo].[sx_pf_Determine_TransactionUsername] (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [dbo].[sx_pf_pGET_ProductLineWriteRight] @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @FactoryKey = FactoryKey FROM [dbo].[sx_pf_dFactories] WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM [dbo].[sx_pf_dProductLines] WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey FROM [dbo].[sx_pf_dProducts] WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey FROM [dbo].[sx_pf_dValueSeries] WHERE ProductKey = @ProductKey  AND ValueSeriesID = @SourceValueSerieID;	
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 OR  @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM [dbo].[sx_pf_dValueSeries] WHERE ProductKey = @ProductKey AND ValueSeriesID = @TargetValueSerieID;

		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Move data
		--dValueSeries 
		UPDATE [dbo].[sx_pf_dValueSeries]
		SET ValueSeriesID = @TargetValueSerieID
		WHERE ValueSeriesKey = @SourceValueSerieKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN			  
			--fValues
			UPDATE [dbo].[sx_pf_fValues] 
			SET ValueSeriesID = @TargetValueSerieID
			WHERE ValueSeriesKey = @SourceValueSerieKey;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);
				
			INSERT INTO [dbo].[sx_pf_fStatements]  
			SELECT @ProductKey AS ProductKey
				, @ProductLineKey AS ProductLineKey
				, @FactoryKey AS FactoryKey
				, @ProductID  AS ProductID
				, @ProductlineID  AS ProductLineID
				, @FactoryID  AS FactoryID
				, N'MOVE' AS ActionType
				, N'ValueSeries ' + @SourceValueSerieID + N' was moved to new ID ' + @TargetValueSerieID AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [dbo].[sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_ValueSerie] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[dbo].[sx_pf_MOVE_ValueSerie] TO pf_PlanningFactoryService;
GO
/*
Procedure to auto create param tables from all products in Productline PARAM in Factory ZT

Testcall
EXEC control.spParamTables 'ANLAG'
09/2018 for DataFactory 4.0
Mandy Hauck / Gerd Tautenhahn

*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.spParamTables') AND type in (N'P', N'PC'))
DROP PROCEDURE control.spParamTables
GO

CREATE PROCEDURE control.spParamTables (@ProductID NVARCHAR(255))
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TableDelete				NVARCHAR(MAX) = ''
	DECLARE @SQLDrop					NVARCHAR(MAX) = ''
	DECLARE @ProductKey					BIGINT	= 0
	DECLARE @ResultCode					INT		= 0

	-------------------------------------------------------------------------------------------------------------------
	-- Delete param.tPivot table
	SET @TableDelete = 'param.[tPivot' + @ProductID+ ']'
	SET @SQLDrop = 'IF OBJECT_ID(''' + 'param.[tPivot' + @ProductID+ ']' + ''', ''U'') IS NOT NULL DROP TABLE ' + @TableDelete
	EXECUTE sp_executesql @SQLDrop
	-------------------------------------------------------------------------------------------------------------------
	-- ##### CREATE ###########
	
	SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE FactoryID = 'ZT' AND ProductLineID = 'PARAM' AND ProductID = @ProductID

	IF @ProductKey > 0
		BEGIN
				-- CREATE TABLE -------------------------------------------------------------------------------------------------------------------
				DECLARE	@TableSchema			NVARCHAR(255)	= 'param.'
				DECLARE	@TablePreName			NVARCHAR(255)	= 'tPivot'																						
				DECLARE @StringBasic			NVARCHAR(MAX)	= 'RowKey BIGINT IDENTITY(1,1),FactoryID NVARCHAR(255),ProductLineID NVARCHAR(255),ProductID NVARCHAR(255),TimeID INT'
				DECLARE @StringColumns			NVARCHAR(MAX)	= ''	
				DECLARE @SQLCreate				NVARCHAR(MAX)	= ''
				DECLARE	@TableName				NVARCHAR(MAX)	= ''

				SELECT @TableName = @TableSchema + '[' + @TablePreName + @ProductID + ']'

				-- Gets all ValueSeriesIDs from a Product in ProductlLine 'PARAM' in Factory 'ZT', ValueSeriesIDs will become column headers
				-- All Text Values will become NVARCHAR(4000), all Numeric Columns Type Money

				-- Collect definition of all columns
				SELECT @StringColumns = COALESCE(@StringColumns + ',', '') +  
						IIF(
							dVS.[IsNumeric]=0
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] NVARCHAR(4000)'
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] MONEY'
							)
				FROM dbo.sx_pf_dValueseries dVS 
				WHERE dVS.ProductKey = @ProductKey

				-- create the table
				SET @SQLCreate = 'CREATE TABLE ' 
							+ @TableName 
							+ '(' 
							+ @StringBasic 
							+ @StringColumns
							+ ' CONSTRAINT PK_' + @TablePreName + @ProductID +' PRIMARY KEY NONCLUSTERED (RowKey))'
				EXECUTE sp_executesql @SQLCreate

				-- Fill Table -------------------------------------------------------------------------------------------------------
			
				DECLARE @STRING_PivotList					NVARCHAR(MAX)  
				DECLARE @STRING_SelectColums				NVARCHAR(MAX)  
				DECLARE @STRING_Insert						NVARCHAR(MAX)   

				-- Get string for dynamic SQL used in PIVOT command (...ValueSeriesID IN ([...]))
				SELECT @STRING_PivotList = COALESCE(@STRING_PivotList + ',', '') +  '[' + CONVERT(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	dbo.sx_pf_dValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Get string for dynamic SQL - ColumnNames
				SELECT @STRING_SelectColums = COALESCE(@STRING_SelectColums + ',', '') 
					+ 'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''') AS [' + convert(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	dbo.sx_pf_dValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Get string for dynamic SQL used in INSERT command
				SELECT @STRING_Insert = COALESCE(@STRING_Insert + ',', '')
							+IIF(
								 dVS.[IsNumeric]=0
								,'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''')'
								,'COALESCE(CAST([' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] AS MONEY) ' + ',0) '
							)
				FROM	dbo.sx_pf_dValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Select Product Values to #table to not use dynamic SQL on real table
				IF OBJECT_ID('tempdb..#ParamValues') IS NOT NULL
					DROP TABLE #ParamValues
				
				CREATE TABLE #ParamValues
					(
						 FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductlineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,TimeID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,Value				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					)

				INSERT INTO #ParamValues
					SELECT 
						 fV.FactoryID
						,fV.ProductLineID
						,fV.ProductID
						,fV.ValueSeriesID
						,fV.TimeID
						,IIF(dVS.[IsNumeric] =0,fV.ValueText,CAST(CAST(fV.ValueInt AS Money)/dVS.Scale AS NVARCHAR(255))) AS Value
					FROM dbo.sx_pf_fValues fV
							LEFT JOIN dbo.sx_pf_dValueSeries dVS
								ON fV.ValueSeriesKey = dVS.ValueSeriesKey
							LEFT JOIN dbo.sx_pf_dProducts dP 
								ON fV.ProductKey = dP.ProductKey
					WHERE fV.ProductKey = @ProductKey

				DECLARE @SQLText	NVARCHAR(MAX) = 'INSERT INTO ' + @TableName + '
					SELECT	 PivotT.FactoryID,PivotT.ProductLineID,PivotT.ProductID,PivotT.TimeID
							,' + @STRING_Insert + '													
					FROM #ParamValues
					PIVOT
						(MAX(Value) FOR ValueSeriesID IN (' + @STRING_PivotList + ')) AS PivotT'

				EXECUTE sp_executesql @SQLText

				-- job well done
				SET @ResultCode = 200
		END
	ELSE
		BEGIN
			SET @ResultCode = 404
		END
	RETURN @ResultCode

END
GO

-- usually no grant necessary, grant only for the case that materialization is called after import of values
GRANT EXECUTE ON OBJECT :: control.spParamTables TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT :: control.spParamTables TO pf_PlanningFactoryService;
/*
Cleanup Operation, which will clean the values in sx_pf_fValues for a given scope.
sx_pf_POST_Cleanup 'SQL', 'F1', 'PL1',	'P1',	'VS1',	201703012	-- will delete all values in dbo.sx_pf_fValues for this combination (this means one row)
sx_pf_POST_Cleanup 'SQL', '',	'PL1',	'P1',	'VS1',	20170312	-- would delete this value in all Factories
sx_pf_POST_Cleanup 'SQL', '',	'',		'',		'',		20170312	-- would delete all values in for this TimeID in sx_pf_fValues.

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE	 @RC				INT
DECLARE  @Username			NVARCHAR(255)	= 'SQL'
DECLARE  @FactoryID			NVARCHAR(255)	= 'ZT'
DECLARE  @ProductLineID		NVARCHAR(255)	= 'U'
DECLARE  @ProductID			NVARCHAR(255)	= ''
DECLARE  @ValueSeriesID		NVARCHAR(255)	= 'K3'
DECLARE  @TimeID			INT				= 20180515	

EXECUTE	@RC = dbo.sx_pf_POST_Cleanup 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSeriesID,
		@TimeID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Cleanup') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Cleanup;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Cleanup
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),	
	@ValueSeriesID	NVARCHAR(255),
	@TimeID			INT = 0
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username						, N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID					, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSeriesID					, N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255))	, N'NULL');
	DECLARE @EffectedRows		INTEGER			= 0;					-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @ProductID		IS NULL	SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Cleanup;

		-- empty TimeID is not allowed
		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID (@ValueSeriesID);

		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
		
		-- STEP 2 - Check rights
		DECLARE @Rights AS TABLE(FactoryID NVARCHAR(255) NOT NULL, ProductLineID NVARCHAR(255) NOT NULL);

		INSERT INTO @Rights (FactoryID, ProductLineID) 
			SELECT
				  FactoryID
				 ,ProductLineID
			FROM dbo.sx_pf_vUserRights 							 
			WHERE 
				Username = @TransactUsername 
				AND [Right]	= N'Write'
				AND (@FactoryID	= N'' OR FactoryID = @FactoryID)
				AND (@ProductLineID	= N'' OR ProductLineID = @ProductLineID);

		IF NOT EXISTS(SELECT 1 FROM @Rights)
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3 - Delete values where user has rights
		DELETE FROM fV
		FROM dbo.sx_pf_fValues fV 
			INNER JOIN @Rights r ON fV.FactoryID = r.FactoryID COLLATE DATABASE_DEFAULT AND fV.ProductLineID = r.ProductLineID COLLATE DATABASE_DEFAULT
		WHERE fV.TimeID = @TimeID
			AND (@ProductID		 = N''	OR fV.ProductID		 = @ProductID)
			AND (@ValueSeriesID	 = N''	OR fV.ValueSeriesID	 = @ValueSeriesID); 

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;
		
		COMMIT TRANSACTION sx_pf_POST_Cleanup;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Cleanup;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryService;
GO
/*
POST Operation for a ClusterProperty
A not existing Property will be created, an existing Property will be updated
If the value <#NV> is passed as parameter, it means keep the existing value 

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to the cluster at Create => 401 Unauthorized
4. Property created => 201 Created
5. Property updated => 200 OK

DECLARE @RC					 INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @PropertyID			 NVARCHAR(255) = 'rre'
DECLARE @PropertyName		 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @Unit				 NVARCHAR(255) = ''
DECLARE @ValueText			 NVARCHAR(255) = ''
DECLARE @ValueInt			 NVARCHAR(255) = '678'
DECLARE @Scale				 NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	 NVARCHAR(255) = '0'
DECLARE @FormatID			 NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ClusterProperty 
		@Username,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ClusterProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ClusterProperty
	@Username			 NVARCHAR(255),
	@PropertyID			 NVARCHAR(255),
	@PropertyName		 NVARCHAR(255),
	@CommentUser		 NVARCHAR(255),
	@CommentDev			 NVARCHAR(255),
	@Unit				 NVARCHAR(255),
	@ValueText			 NVARCHAR(255),
	@ValueInt			 NVARCHAR(255),
	@Scale				 NVARCHAR(255),
	@IsROSystemProperty	 NVARCHAR(255),
	@FormatID			 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		 INT			= 0;
	DECLARE @SytemROBlocker		 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username				, N'NULL') + N''',''' 
			+ ISNULL(@PropertyID			, N'NULL') + N''','''
			+ ISNULL(@PropertyName			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser			, N'NULL') + N''','''
			+ ISNULL(@CommentDev			, N'NULL') + N''',''' 
			+ ISNULL(@Unit					, N'NULL') + N''','''
			+ ISNULL(@ValueText				, N'NULL') + N''','''
			+ ISNULL(@ValueInt				, N'NULL') + N''','''
			+ ISNULL(@Scale					, N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty	, N'NULL') + N''',''' 
			+ ISNULL(@FormatID				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ClusterProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @PropertyName	 = dbo.sx_pf_pProtectString	 (@PropertyName);
		SET @CommentUser	 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev		 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @Unit			 = dbo.sx_pf_pProtectString	 (@Unit);
		SET @ValueText		 = dbo.sx_pf_pProtectString	 (@ValueText);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);
		SET @FormatID		 = dbo.sx_pf_pProtectString	 (@FormatID);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gCluster WHERE PropertyID = @PropertyID; 
				
		-- STEP 1.3 - Don`t allow changing of system RO Properties
		SELECT @SytemROBlocker = IsROSystemProperty FROM dbo.sx_pf_gCluster WHERE PropertyID = @PropertyID;
		
		IF @SytemROBlocker = 1 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_gCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemProperty,FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key])
			VALUES (@PropertyID,'','','','','',0,1,0,'');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;		

		-- STEP 3.2 - Update Cluster attributes
		UPDATE dbo.sx_pf_gCluster
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev				= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty		= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM dbo.sx_pf_gCluster 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for a Factory
Not existing Factories are created, existing updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC					 INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			 NVARCHAR(255) = 'ZT'
DECLARE @NameShort			 NVARCHAR(255) = ''
DECLARE @NameLong			 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @ResponsiblePerson	 NVARCHAR(255) = ''
DECLARE @ImageName			 NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_Factory 
		@Username,
		@FactoryID,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName

PRINT @RC
*/

IF  EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Factory;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Factory
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson	NVARCHAR(255),
	@ImageName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')			 + N''','''
			+ ISNULL(@NameShort			, N'NULL')			 + N''','''
			+ ISNULL(@NameLong			, N'NULL')			 + N''','''
			+ ISNULL(@CommentUser		, N'NULL')			 + N''','''
			+ ISNULL(@CommentDev		, N'NULL')			 + N''','''
			+ ISNULL(@ResponsiblePerson	, N'NULL')			 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')			 + N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID			 = N'';
	IF @NameShort			 IS NULL SET @NameShort			 = N'';
	IF @NameLong			 IS NULL SET @NameLong			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser		 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev		 = N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson	 = N'';
	IF @ImageName			 IS NULL SET @ImageName			 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @NameShort			 = dbo.sx_pf_pProtectString	 (@NameShort);
		SET @NameLong			 = dbo.sx_pf_pProtectString	 (@NameLong);
		SET @CommentUser		 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev			 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @ResponsiblePerson	 = dbo.sx_pf_pProtectString	 (@ResponsiblePerson);
		SET @ImageName			 = dbo.sx_pf_pProtectString	 (@ImageName);
		
		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

		-- STEP 2 - Check rights
		IF @FactoryKey = 0
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		ELSE
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Factory does not yet exist, it will be created in minimal configuration
		IF @FactoryKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
			INSERT INTO dbo.sx_pf_dFactories (FactoryID, NameShort,NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName) 
				OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
				VALUES (@FactoryID, @NameShort, '', '', '', '', '');

			SELECT TOP(1) @FactoryKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;		
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update Factory attributes
		UPDATE dbo.sx_pf_dFactories 
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			@NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			@NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	@ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			@ImageName)
		FROM dbo.sx_pf_dFactories 
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (no insert needed because only ClusterAdmin can create Factory)
		EXEC dbo.sx_pf_materialize_vUserRights;	

		COMMIT TRANSACTION sx_pf_POST_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Factory;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryService;
GO
/*
POST Operation for a FactoryProperty
Not existing Property will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					  INT
DECLARE @Username			  NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			  NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			  NVARCHAR(255) = 'rr3e'
DECLARE @PropertyName		  NVARCHAR(255) = ''
DECLARE @CommentUser		  NVARCHAR(255) = ''
DECLARE @CommentDev			  NVARCHAR(255) = ''
DECLARE @Unit				  NVARCHAR(255) = ''
DECLARE @ValueText			  NVARCHAR(255) = ''
DECLARE @ValueInt			  NVARCHAR(255) = '678'
DECLARE @Scale				  NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	  NVARCHAR(255) = '2'
DECLARE @FormatID			  NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_FactoryProperty 
		@Username,
		@FactoryID,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FactoryProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FactoryProperty
	@Username			  NVARCHAR(255),
	@FactoryID			  NVARCHAR(255),
	@PropertyID			  NVARCHAR(255),
	@PropertyName		  NVARCHAR(255),
	@CommentUser		  NVARCHAR(255),
	@CommentDev			  NVARCHAR(255),
	@Unit				  NVARCHAR(255),
	@ValueText			  NVARCHAR(255),
	@ValueInt			  NVARCHAR(255),
	@Scale				  NVARCHAR(255),
	@IsROSystemProperty	  NVARCHAR(255),
	@FormatID			  NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			  INT = 0;
	DECLARE @PropertyKey		  INT = 0;
	DECLARE @TransactUsername	  NVARCHAR(255) = N'';
	DECLARE @ProcedureName		  NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	  NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyName,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev,			 N'NULL')	+ N''',''' 
			+ ISNULL(@Unit,					 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueText,			 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueInt,				 N'NULL')	+ N''',''' 
			+ ISNULL(@Scale,				 N'NULL')	+ N''',''' 
			+ ISNULL(@IsROSystemProperty,	 N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID,				 N'NULL')	+ N'''';		
	DECLARE @EffectedRows		  INT			 = 0;				
	DECLARE @ResultCode			  INT			 = 501;				
	DECLARE @TimestampCall		  DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			  NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @PropertyName	 = dbo.sx_pf_pProtectString	 (@PropertyName);
		SET @CommentUser	 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev		 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @Unit			 = dbo.sx_pf_pProtectString	 (@Unit);
		SET @ValueText		 = dbo.sx_pf_pProtectString	 (@ValueText);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);
		SET @FormatID		 = dbo.sx_pf_pProtectString	 (@FormatID); --has String Protection, as also #<NV> can be used as parameter
				
		IF	@PropertyID = N''
			OR @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - disabled from Version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		-- STEP 3.1 - If property not exist, it will created minimalistic	
		IF @PropertyKey = 0
		BEGIN
			INSERT INTO dbo.sx_pf_gFactories (FactoryKey, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys
			VALUES (@FactoryKey, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
		   
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update Factory property
		UPDATE dbo.sx_pf_gFactories
		SET 
			 PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM dbo.sx_pf_gFactories
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for a FactoryTab
Not existing Property will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

07/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard


TestCalls

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TabID				NVARCHAR(255) = 'Tab01'
DECLARE @Label				NVARCHAR(255) = 'Productlines'
DECLARE @CommentUser		NVARCHAR(MAX) = 'List of Productlines'
DECLARE @PresentationType	NVARCHAR(255) = 'ProductlineList'	
DECLARE @Source				NVARCHAR(MAX) = ''		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @TabID				NVARCHAR(255) = 'Tab02'
DECLARE @Label				NVARCHAR(255) = 'Profit'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can pivot your P&L here.'
DECLARE @PresentationType	NVARCHAR(255) = 'Pivot'	
DECLARE @Source				NVARCHAR(MAX) = 'sx_pf_DATAOUTPUT_Profit'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

SET @TabID				= 'Tab03'
SET @Label				= 'Balance'
SET @CommentUser		= 'You can pivot your Balance here.'
SET @PresentationType	= 'Pivot'	
SET @Source				= 'sx_pf_DATAOUTPUT_Balance'		
SET @Layout				= ''

EXECUTE @RC = sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID, 
		@TabID, 
		@Label, 
		@CommentUser, 
		@PresentationType, 
		@Source, 
		@Layout

PRINT @RC

SET @TabID				= 'Tab04'
SET @Label				= 'Manual'
SET @CommentUser		= ''
SET @PresentationType	= 'HTML_URL'	
SET @Source				= 'http://www.saxess-software.de'		
SET @Layout				= ''

EXECUTE @RC = sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

SET @TabID				= 'Tab05'
SET @Label				= 'Report'
SET @CommentUser		= ''
SET @PresentationType	= 'HTML_Static'	
SET @Source				= 'sx_pf_DATAOUTPUT_PLReport' -- this sp don't exists, asume it retruns an HTML block as nvarchar(max)
SET @Layout				= ''

EXECUTE @RC = sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @TabID				NVARCHAR(255) = 'Tab06'
DECLARE @Label				NVARCHAR(255) = 'ProfitChart'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can pivot your P&L here.'
DECLARE @PresentationType	NVARCHAR(255) = 'Pivot_Chart'	
DECLARE @Source				NVARCHAR(MAX) = 'sx_pf_DATAOUTPUT_Profit'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = '2'
DECLARE @TabID				NVARCHAR(255) = 'Tab07'
DECLARE @Label				NVARCHAR(255) = 'Eventauswertung'
DECLARE @CommentUser		NVARCHAR(MAX) = 'This is your ToDo List.'
DECLARE @PresentationType	NVARCHAR(255) = 'DataGrid'	
DECLARE @Source				NVARCHAR(MAX) = 'result.spEventsPlan'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = 'BK'
DECLARE @TabID				NVARCHAR(255) = 'Tab08'
DECLARE @Label				NVARCHAR(255) = 'Anleitung'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can create own pages here.'
DECLARE @PresentationType	NVARCHAR(255) = 'HTML_Editor'	
DECLARE @Source				NVARCHAR(MAX) = ''		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC



*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_FactoryTab') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FactoryTab;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FactoryTab
	 @Username			NVARCHAR(255)
	,@FactoryID			NVARCHAR(255)
	,@TabID				NVARCHAR(255)
	,@Label				NVARCHAR(255)
	,@CommentUser		NVARCHAR(MAX)
	,@PresentationType	NVARCHAR(255)	
	,@Source			NVARCHAR(MAX)		
	,@Layout			NVARCHAR(MAX)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @PropertyOwner		INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		 N'NULL')	+ N''',''' 
			+ ISNULL(@TabID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@Label,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser,		 N'NULL')	+ N''',''' 
			+ ISNULL(@PresentationType,	 N'NULL')	+ N''',''' 
			+ ISNULL(@Source,			 N'NULL')	+ N''','''
			+ ISNULL(@Layout,			 N'NULL')	+ N'''';		
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @TabID				IS NULL SET @TabID				= N'';
	IF @Label				IS NULL SET @Label				= N'';
	IF @CommentUser			IS NULL SET @CommentUser		= N'';
	IF @PresentationType	IS NULL SET @PresentationType	= N'';
	IF @Source				IS NULL SET @Source				= N'';
	IF @Layout				IS NULL SET @Layout				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FactoryTab;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TabID				= dbo.sx_pf_pProtectID		(@TabID);
		SET @Label				= dbo.sx_pf_pProtectString	(@Label);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @PresentationType	= dbo.sx_pf_pProtectString	(@PresentationType);
		SET @Source				= dbo.sx_pf_pProtectString	(@Source);
		SET @Layout				= dbo.sx_pf_pProtectString	(@Layout);

				
		IF  @TabID = N'' 
			OR @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @TabID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties
		SELECT @PropertyOwner = IsROSystemProperty FROM dbo.sx_pf_gFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @TabID;

		IF @PropertyOwner = 1
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);			
		
		-- STEP 3.1 - If property not exist, it will created minimalistic	
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_gFactories (FactoryKey, FactoryID,PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemProperty,FormatID) 
				OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
				VALUES (@FactoryKey, @FactoryID,@TabID,'','','','','',0,1,0,'');	

			SELECT TOP(1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update Factory property
		UPDATE dbo.sx_pf_gFactories
		SET		
			 PropertyName	= IIF(@Label			= N'<#NV>', PropertyName	,@Label)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser		,@CommentUser)
			,Unit			= IIF(@PresentationType = N'<#NV>', Unit			,@PresentationType)
			,ValueText		= IIF(@Source			= N'<#NV>', ValueText		,@Source)
			,CommentDev		= IIF(@Layout			= N'<#NV>', CommentDev		,@Layout)
		FROM dbo.sx_pf_gFactories 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FactoryTab;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FactoryTab;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryTab TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryTab TO pf_PlanningFactoryService;
GO

/*
POST Operation for sending FlexAction Parameter
Procedure must be editend to fit user needs and recreated after each API Update.

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
--should return Data and 200
DECLARE @RC						NVARCHAR(255)
DECLARE @Username				NVARCHAR(255) = 'SQL'
DECLARE @FactoryID				NVARCHAR(255) = 'F1'
DECLARE @ProductLineID			NVARCHAR(255) = 'PL1'
DECLARE @ProductID				NVARCHAR(255) = 'P1'
DECLARE @ValueSeriesID			NVARCHAR(255) = 'Mitarbeiter'
DECLARE @TimeID					BIGINT		  = 20180115
DECLARE @GlobalattributeNumber	NVARCHAR(255) = ''
DECLARE @PageType				NVARCHAR(255) = ''
DECLARE @FlexParameter			NVARCHAR(255) = 'P1000'


EXEC @RC = dbo.sx_pf_POST_FlexAction
	 @Username,
	 @FactoryID,
	 @ProductLineID,
	 @ProductID,
	 @ValueSeriesID,
	 @TimeID,
	 @GlobalattributeNumber,
	 @PageType,
	 @FlexParameter
	 	
PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_FlexAction') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FlexAction;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@FlexParameter			NVARCHAR(255)  -- The Identifier (Second column of GET_FlexAction, first Display column) from the choosen list element
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FactoryID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductLineID,			 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ValueSeriesID,			 N'NULL')		+ N''','
			+ CAST(ISNULL(@TimeID, N'NULL') AS NVARCHAR(255))	+ N','''
			+ ISNULL(@GlobalattributeNumber,	 N'NULL')		+ N''',''' 
			+ ISNULL(@PageType,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FlexParameter,			 N'NULL')		+ N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	IF @FlexParameter			IS NULL SET @FlexParameter			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @FlexParameter	= dbo.sx_pf_pProtectString	(@FlexParameter);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END

		-- Sample Actions
		-- React on content parameters and FlexParameter for action

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			-- POST_ValueSeries of Type Projekt, ValueSeriesID = FlexParameter
			PRINT 'Did Projekt'
			SET @ResultCode = 200;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			-- POST ValuesSeries of Type Mitarbeiter, ValueSeriesID = FlexParameter
			PRINT 'Did Mitarbeiter'
			SET @ResultCode = 200;
		END
		
		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter' )
		BEGIN
			PRINT 'No fitting action found.'
			SET @ResultCode = 404
		END
			   
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FlexAction;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryService
GO
/*
POST Operation for Format
Not existing Format will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call - Blockstyle !
DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) = 'SQL'					'SQL'
DECLARE @FormatID				NVARCHAR(255) = 'sxTest4'				'sxTest3'
DECLARE @BackgroundColor		NVARCHAR(255) = '100,100,100'			'<#NV>'
DECLARE @FontColor				NVARCHAR(255) = '200,200,200'			'<#NV>'
DECLARE @ValueFormat			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@Bold					NVARCHAR(255) = ''						'<#NV>'
DECLARE	@Italic					NVARCHAR(255) = ''						'<#NV>'
DECLARE	@AlignmentHorizontal	NVARCHAR(255) = ''						'<#NV>'
DECLARE	@AlignmentVertical		NVARCHAR(255) = ''						'<#NV>'
DECLARE	@TextUnderlined			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@BorderToNext			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@BorderToPrevious		NVARCHAR(255) = ''  					'<#NV>'  
		

EXECUTE @RC = sx_pf_POST_Format 
		@Username,
		@FormatID,
		@BackgroundColor,
		@FontColor,
		@ValueFormat,
		@Bold,				
		@Italic,				
		@AlignmentHorizontal,
		@AlignmentVertical,	
		@TextUnderlined,		
		@BorderToNext,		
		@BorderToPrevious	

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Format;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Format
	@Username				NVARCHAR(255),
	@FormatID				NVARCHAR(255),
	@BackgroundColor		NVARCHAR(255),
	@FontColor				NVARCHAR(255),
	@ValueFormat			NVARCHAR(255),
	@Bold					NVARCHAR(255),
	@Italic					NVARCHAR(255),
	@AlignmentHorizontal	NVARCHAR(255),
	@AlignmentVertical		NVARCHAR(255),
	@TextUnderlined			NVARCHAR(255),
	@BorderToNext			NVARCHAR(255),
	@BorderToPrevious		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N''''
			+ ISNULL(@Username				, N'NULL')		+ N''','''
			+ ISNULL(@FormatID				, N'NULL')		+ N''','''
			+ ISNULL(@BackgroundColor		, N'NULL')		+ N''','''
			+ ISNULL(@FontColor				, N'NULL')		+ N''','''
			+ ISNULL(@ValueFormat			, N'NULL')		+ N''',''' 
			+ ISNULL(@Bold					, N'NULL')		+ N''','''
			+ ISNULL(@Italic				, N'NULL')		+ N''',''' 
			+ ISNULL(@AlignmentHorizontal	, N'NULL')		+ N''','''
			+ ISNULL(@AlignmentVertical		, N'NULL')		+ N''','''
			+ ISNULL(@TextUnderlined		, N'NULL')		+ N''','''
			+ ISNULL(@BorderToNext			, N'NULL')		+ N''','''
			+ ISNULL(@BorderToPrevious		, N'NULL')		+ N''','''
			+ N'''';
	DECLARE @EffectedRows	 INT = 0;						
	DECLARE @ResultCode		 INT = 501;						
	DECLARE @TimestampCall	 DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment		 NVARCHAR (2000) = N'';
	DECLARE @ErrorText		 NVARCHAR (2000) = N';'			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			 = N'';
	IF @FormatID			IS NULL SET @FormatID			 = N'';
	IF @BackgroundColor		IS NULL SET @BackgroundColor	 = N'';
	IF @FontColor			IS NULL SET @FontColor			 = N'';
	IF @ValueFormat			IS NULL SET @ValueFormat		 = N'';
	IF @Bold				IS NULL SET @Bold				 = N'';
	IF @Italic				IS NULL SET @Italic				 = N'';
	IF @AlignmentHorizontal	IS NULL SET @AlignmentHorizontal = N'';
	IF @AlignmentVertical	IS NULL SET @AlignmentVertical	 = N'';
	IF @TextUnderlined		IS NULL SET @TextUnderlined		 = N'';
	IF @BorderToNext		IS NULL SET @BorderToNext		 = N'';
	IF @BorderToPrevious	IS NULL SET @BorderToPrevious	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Format;

		-- STEP 0.2 - Protect input parameters
		SET @FormatID			 = dbo.sx_pf_pProtectID		(@FormatID);
		SET @BackgroundColor	 = dbo.sx_pf_pProtectString (@BackgroundColor);
		SET @FontColor			 = dbo.sx_pf_pProtectString (@FontColor);
		SET @ValueFormat		 = dbo.sx_pf_pProtectString (@ValueFormat);
		SET @Bold				 = dbo.sx_pf_pProtectInt	(@Bold);
		SET @Italic				 = dbo.sx_pf_pProtectInt	(@Italic);
		SET @AlignmentHorizontal = dbo.sx_pf_pProtectString (@AlignmentHorizontal);
		SET @AlignmentVertical	 = dbo.sx_pf_pProtectString (@AlignmentVertical);
		SET @TextUnderlined		 = dbo.sx_pf_pProtectInt	(@TextUnderlined)
		SET @BorderToNext		 = dbo.sx_pf_pProtectInt	(@BorderToNext)
		SET @BorderToPrevious	 = dbo.sx_pf_pProtectInt	(@BorderToPrevious)

		IF @FormatID = N''
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		IF	@Bold				NOT IN ('<#NV>','-1','0','1') OR
			@Italic				NOT IN ('<#NV>','-1','0','1') OR
			@BorderToNext		NOT IN ('<#NV>','-1','0','1') OR
			@BorderToPrevious	NOT IN ('<#NV>','-1','0','1') 
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter for Bold, Italic, BorderToNext, BorderToPrevios not supported (outside <#NV>,-1,0,1).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentHorizontal NOT IN ('<#NV>','','left','center','right')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentHorizontal + ' for AlignmentHorizontal not supported (outside <#NV>, left, center, rights, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentVertical NOT IN ('<#NV>','','top','center','bottom')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentVertical + ' for AlignmentVertical not supported (outside <#NV>, top, center, bottom, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;


		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FormatKey = FormatKey FROM dbo.sx_pf_hFormats WHERE FormatID = @FormatID;
			
		-- STEP 3.1 - If format does not yet exist, it will be created
		IF @FormatKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_hFormats (FormatID,BackgroundColor,FontColor,ValueFormat,Bold,Italic,AlignmentHorizontal,AlignmentVertical	
												,TextUnderlined,BorderToNext,BorderToPrevious	
												) 
			OUTPUT INSERTED.FormatKey INTO @Keys([Key]) 
				VALUES (@FormatID,'','','','-1','-1','','','-1','0','0');

			SELECT TOP (1) @FormatKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update format attributes
		UPDATE dbo.sx_pf_hFormats 
		SET 
			 BackgroundColor	= IIF(@BackgroundColor		= N'<#NV>', BackgroundColor,		@BackgroundColor)
			,FontColor			= IIF(@FontColor			= N'<#NV>', FontColor,				@FontColor)
			,ValueFormat		= IIF(@ValueFormat			= N'<#NV>', ValueFormat,			@ValueFormat)
			,Bold				= IIF(@Bold					= N'<#NV>', Bold,					@Bold)
			,Italic				= IIF(@Italic				= N'<#NV>', Italic,					@Italic)
			,AlignmentHorizontal= IIF(@AlignmentHorizontal	= N'<#NV>', AlignmentHorizontal,	@AlignmentHorizontal)
			,AlignmentVertical	= IIF(@AlignmentVertical	= N'<#NV>', AlignmentVertical,		@AlignmentVertical)
			,TextUnderlined		= IIF(@TextUnderlined		= N'<#NV>', TextUnderlined,			@TextUnderlined)
			,BorderToNext		= IIF(@BorderToNext			= N'<#NV>', BorderToNext,			@BorderToNext)
			,BorderToPrevious	= IIF(@BorderToPrevious		= N'<#NV>', BorderToPrevious,		@BorderToPrevious)
		FROM dbo.sx_pf_hFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Format;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryService;
GO
/*
POST Operation for an HTML Page

10/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC			INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = ''
DECLARE @ProductLineID	NVARCHAR(255) = ''
DECLARE @FactoryID		NVARCHAR(255) = ''
DECLARE @PageID			NVARCHAR(255) = 'Page1'
DECLARE @PageName		NVARCHAR(255) = 'A great page'
DECLARE @PageContent	NVARCHAR(255) = 'Hallo Gerd'

EXECUTE @RC = dbo.sx_pf_POST_HTMLPage
		 @Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@PageID
		,@PageName
		,@PageContent

PRINT @RC

SELECT * FROM dbo.sx_pf_gHTMLPages

*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_HTMLPage') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_HTMLPage;
GO

CREATE PROCEDURE dbo.sx_pf_POST_HTMLPage
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PageID				NVARCHAR(255),
	@PageName			NVARCHAR(255),
	@PageContent		NVARCHAR(MAX)		

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@PageID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@PageName,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PageID			IS NULL SET @PageID			= 0;
	IF @PageName		IS NULL SET @PageName		= '';
	IF @PageContent		IS NULL SET @PageContent	= '';


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_HTMLPage;

		-- Page must have an ID, all other parameter may be empty
		IF @PageID = ''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @PageID			= dbo.sx_pf_pProtectString (@PageID);
		SET @PageName		= dbo.sx_pf_pProtectString (@PageName);


		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;	

		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM dbo.sx_pf_dFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM dbo.sx_pf_dProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM dbo.sx_pf_dProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;

		-- Check rights on different levels
		IF @FactoryKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Delete old PageID
		DELETE FROM dbo.sx_pf_gHTMLPages 
		WHERE 
				ProductKey		= @ProductKey 
			AND ProductLineKey	= @ProductLineKey
			AND FactoryKey		= @FactoryKey
			AND PageID			= @PageID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- Create new Page
		INSERT INTO dbo.sx_pf_gHTMLPages	(
			 ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,PageID
			,PageName
			,PageContent
			)
			VALUES (
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@PageID,
				@PageName,
				@PageContent
			);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryService;
GO
/*
POST Operation for a List
Not existing Format will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2017 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ListID			NVARCHAR(255) = 'sxScales'
DECLARE @NameShort		NVARCHAR(255) = '<#NV>'
DECLARE @NameLong		NVARCHAR(255) = '<#NV>'
DECLARE @CommentDev		NVARCHAR(255) = '<#NV>'
DECLARE @CommentUser	NVARCHAR(255) = '<#NV>'
DECLARE @Datentyp		NVARCHAR(255) = '<#NV>'
DECLARE @Source			NVARCHAR(255) = '<#NV>'
DECLARE @SourceFormula	NVARCHAR(255) = 'Testen'
DECLARE @FormatID		NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_List 
		@Username,
		@ListID,
		@NameShort,
		@NameLong,
		@CommentDev,
		@CommentUser,
		@Datentyp,
		@Source,
		@SourceFormula,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_List;
GO

CREATE PROCEDURE dbo.sx_pf_POST_List
	 @Username		NVARCHAR(255)
	,@ListID		NVARCHAR(255)
	,@NameShort		NVARCHAR(255)
	,@NameLong		NVARCHAR(255)
	,@CommentDev	NVARCHAR(255)
	,@CommentUser	NVARCHAR(255)
	,@Datentyp		NVARCHAR(255)
	,@Source		NVARCHAR(255)
	,@SourceFormula NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username		,N'NULL')	+ N''',''' 
			+ ISNULL(@ListID		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameShort		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameLong		,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev	,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser	,N'NULL')	+ N''',''' 
			+ ISNULL(@Datentyp		,N'NULL')	+ N''',''' 
			+ ISNULL(@Source		,N'NULL')	+ N''',''' 
			+ ISNULL(@SourceFormula	,N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID		,N'NULL')	+ N'''';	
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ListID			IS NULL SET @ListID			= N'';
	IF @NameShort		IS NULL SET @NameShort		= N'';
	IF @NameLong		IS NULL SET @NameLong		= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Datentyp		IS NULL SET @Datentyp		= N'';
	IF @Source			IS NULL SET @Source			= N'';
	IF @SourceFormula	IS NULL SET @SourceFormula	= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_List;

		-- STEP 0.2 - Protect input parameters
		SET @ListID			= dbo.sx_pf_pProtectID (@ListID);
		SET @NameShort		= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong		= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser	= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString (@CommentDev);
		SET @Datentyp		= dbo.sx_pf_pProtectString (@Datentyp);
		SET @Source			= dbo.sx_pf_pProtectString (@Source);
		SET @SourceFormula	= dbo.sx_pf_pProtectString (@SourceFormula);
		SET @FormatID		= dbo.sx_pf_pProtectString (@FormatID); -- as it can be #<NV>

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @ListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @ListKey = ListKey FROM dbo.sx_pf_hLists WHERE ListID = @ListID;

		-- STEP 3.1 - If List is does not exist, it will be created
		IF @ListKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_hLists (ListID, NameShort, NameLong, CommentDev, CommentUser, Datentyp, Source, SourceFormula, FormatID) 
			OUTPUT INSERTED.ListKey INTO @Keys([Key])
			VALUES (@ListID, '', '', '', '', '', '', '', '');

			SELECT TOP(1) @ListKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update List attributes
		UPDATE dbo.sx_pf_hLists 
		SET	 
			 NameShort		= IIF(@NameShort		= N'<#NV>', NameShort,		@NameShort)
			,NameLong		= IIF(@NameLong			= N'<#NV>', NameLong,		@NameLong)
			,CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,		@CommentDev)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser,	@CommentUser)
			,Datentyp		= IIF(@Datentyp			= N'<#NV>', Datentyp,		@Datentyp)
			,Source			= IIF(@Source			= N'<#NV>', Source,			@Source)
			,SourceFormula  = IIF(@SourceFormula	= N'<#NV>', SourceFormula,	@SourceFormula)
			,FormatID		= IIF(@FormatID			= N'<#NV>', FormatID,		@FormatID)
		FROM dbo.sx_pf_hLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryService;
GO
/*
POST Operation for ListValue
Not existing ListValue will be created, existing updated.
This procedure allows to pass a Key value, as there is no ID for a ListValue


03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Status

Test call

DECLARE @RC				 INT
DECLARE @Username		 NVARCHAR(255)	= 'SQL'
DECLARE @ListValueKey	 INT			= 0
DECLARE @ListID			 NVARCHAR(255)	= 'sxStatus'
DECLARE @ValueInt		 INT			= 5
DECLARE @Scale			 INT			= 1
DECLARE @ValueText		 NVARCHAR(255)	= 'Testo'
DECLARE @ValueComment	 NVARCHAR(255)	= ''
DECLARE @FormatID		 NVARCHAR(255)	= ''
		

EXECUTE @RC = dbo.sx_pf_POST_ListValue 
		@Username,
		@ListValueKey,
		@ListID,
		@ValueInt,
		@Scale,
		@ValueText,
		@ValueComment,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ListValue') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ListValue
GO

CREATE PROCEDURE dbo.sx_pf_POST_ListValue
	@Username		NVARCHAR(255),
	@ListValueKey	INT,
	@ListID			NVARCHAR(255),
	@ValueInt		INT,
	@Scale			INT,
	@ValueText		NVARCHAR(255),
	@ValueComment	NVARCHAR(255),
	@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)  = N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username,								 N'NULL') + N''',' 
								+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)),	 N'NULL') + N',''' 
								+ ISNULL(@ListID,								 N'NULL') + N''',' 
								+ ISNULL(CAST(@ValueInt		AS NVARCHAR(255)),	 N'NULL') + N',' 
								+ ISNULL(CAST(@Scale		AS NVARCHAR(255)),	 N'NULL') + N',''' 
								+ ISNULL(@ValueText,							 N'NULL') + N''',''' 
								+ ISNULL(@ValueComment,							 N'NULL') + N''',''' 
								+ ISNULL(@FormatID,								 N'NULL') + N'''';	
	DECLARE @EffectedRows		 INT			= 0;				
	DECLARE @ResultCode			 INT			= 501;				
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @ListValueKey	IS NULL SET @ListValueKey	 = 0;
	IF @ListID			IS NULL SET @ListID			 = N'';
	IF @ValueInt		IS NULL SET @ValueInt		 = 0;
	IF @Scale			IS NULL SET @Scale			 = 0;
	IF @ValueText		IS NULL SET @ValueText		 = N'';
	IF @ValueComment	IS NULL SET @ValueComment	 = N'';
	IF @FormatID		IS NULL SET @FormatID		 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ListValue;

		-- STEP 0.2 - Protect input parameters
		SET @ListValueKey	= dbo.sx_pf_pProtectInt		(@ListValueKey);
		SET @ListID			= dbo.sx_pf_pProtectID		(@ListID);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);
		SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys and break if not existing
		SELECT @ListKey = ListKey FROM dbo.sx_pf_hLists WHERE ListID = @ListID;
		
		IF @ListKey = 0	BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check if Listvalue exists, when ListValueKey 0 is submitted - 0 means new entry, but API must check if entry is really new
		IF @ListValueKey = 0
		BEGIN
			DECLARE @tmpKey			BIGINT		  = 0;
			DECLARE @tmpListType	NVARCHAR(255) = '';
			
			SELECT @tmpListType = hL.Datentyp FROM dbo.sx_pf_hLists hL WHERE hL.ListID = @ListID;

			IF @tmpListType = 'String'
			BEGIN
				SELECT @tmpKey = ListValueKey FROM dbo.sx_pf_hListValues hLV WHERE ListID = @ListID AND hLV.ValueText = @ValueText;
			END
			ELSE
			BEGIN
				SELECT @tmpKey = ListValueKey FROM dbo.sx_pf_hListValues hLV WHERE ListID = @ListID AND hLV.ValueInt = @ValueInt;
			END

			IF (@tmpKey IS NOT NULL AND @tmpKey != 0) SET @ListValueKey = @tmpKey
		END

		-- STEP 3.1 - If ListValueKey  = 0 is submitted, create new value
		IF @ListValueKey = 0
		BEGIN
			INSERT INTO dbo.sx_pf_hListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			VALUES (@ListID, @ValueInt, @Scale, @ValueText, @ValueComment, @FormatID);

			SET @Resultcode = 201;
			SET @EffectedRows = 1;
		END
		ELSE
		BEGIN
		-- STEP 3.2 - If existing key is committed, update the Key

			UPDATE dbo.sx_pf_hListValues 
			SET 
				ValueInt		 = @ValueInt
				, Scale			 = @Scale
				, ValueText		 = @ValueText
				, ValueComment	 = @ValueComment
				, FormatID		 = @FormatID
			WHERE ListValueKey	 = @ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END;	

		COMMIT TRANSACTION sx_pf_POST_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryService;
GO
/*
POST Operation for Preference
Not existing Preference will be created, existing updated.

02/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
-- make Property UserDependend
EXEC sx_pf_POST_ProductlineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','1','1','2',''

-- make Property NOT UserDependend
EXEC sx_pf_POST_ProductlineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','0','1','2',''


DECLARE	 @RC					INT
DECLARE	 @Username 				NVARCHAR(255) = 'SQL'
DECLARE	 @PreferenceUserName	NVARCHAR(255) = 'AD-ONE\gtautenh'
DECLARE	 @ProductLineID			NVARCHAR(255) = 'U'
DECLARE	 @FactoryID				NVARCHAR(255) = 'ZT'
DECLARE	 @SettingName			NVARCHAR(255) = 'Demopref'
DECLARE	 @ValueInt				NVARCHAR(255) = '8'
DECLARE	 @ValueText				NVARCHAR(255) = 'uztr'

EXECUTE @RC = dbo.sx_pf_POST_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName,
		@ValueInt,
		@ValueText

PRINT @RC 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Preference;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255),
	@ValueInt			INT,
	@ValueText			NVARCHAR(MAX)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
	DECLARE @FactoryKey						INT				= 0
	DECLARE @ProductLineKey					INT				= 0
	DECLARE @PreferenceIsUserDependendFlag	INT				= 0
	DECLARE @ProcedureName					NVARCHAR (255)	= OBJECT_NAME(@@PROCID)
	DECLARE @ParameterString				NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,							N'NULL')	+ N''',''' 
			+ ISNULL(@PreferenceUsername,				N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,					N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,						N'NULL')	+ N''',''' 
			+ ISNULL(@SettingName,						N'NULL') 	+ N''',' 
			+ ISNULL(CAST(@ValueInt AS NVARCHAR(255)),	N'NULL')	+ N',''' 
			+ ISNULL(@ValueText,						N'NULL')	+ N''''
	DECLARE @EffectedRows					INT				= 0
	DECLARE @ResultCode						INTEGER			= 501					
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP
	DECLARE @Comment						NVARCHAR(2000)	= N'';	
	DECLARE @UserKey						INT				= 0;				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';
	IF @ValueInt			IS NULL SET @ValueInt			= 0;
	IF @ValueText			IS NULL SET @ValueText			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Preference

		-- STEP 0.2 - Protect input parameters
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @SettingName	= dbo.sx_pf_pProtectString	(@SettingName);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);

		IF	@SettingName = N'' 
			OR @SettingName = '<#NV>'
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine, if Preferences are saved per User
		-- for global Preferences
		IF	@FactoryID = N'' 
			AND @ProductLineID = N''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gCluster WHERE PropertyID = 'C06';
		END
		-- for Preferences per Factory
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID = ''
		BEGIN 
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gFactories WHERE PropertyID = 'F06';
		END
		-- for Preferences per Productline
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID <> ''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM dbo.sx_pf_gProductlines WHERE PropertyID = 'P06';
		END;

		-- STEP 1.2 - if Preferences are not Userdependend, only a clusteradmin can save them, else everybody with
		-- Factory / Productline Write Rights
		ELSE IF @PreferenceIsUserDependendFlag <> 1 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
			
			IF @ResultCode <> 200
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- STEP 1.3 - Determine Key, they may be 0 if the preference is for Cluster / Factory
		SELECT @FactoryKey	= ISNULL(FactoryKey, 0)	FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ISNULL(ProductLineKey, 0) FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		-- Cancel if Factory or Product Line does not exist
		IF	(@FactoryKey = 0 AND @FactoryID <> N'')
			OR	(@ProductlineKey = 0 AND @ProductLineID <> N'')
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Determine UserKey
		SELECT @UserKey = UserKey FROM dbo.sx_pf_rUser WHERE Username = @PreferenceUsername

		-- STEP 1.4 - Set Preference User to "All" if Preferences are not Userdependend
		IF @PreferenceIsUserDependendFlag <> 1 
			BEGIN
				SET @PreferenceUsername = N'All';
				SET @UserKey			= 0 ;
			END
		ELSE
		-- STEP 1.5 - Test existence of Preferences Users if Preference is Userdepend
		IF	@UserKey = 0 AND @PreferenceIsUserDependendFlag <> 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User don`t exists', 16, 10);
		END;	
		
		-- STEP 2. Rights check
		IF @ProductLineID <> N''
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		END
		ELSE IF @FactoryID <> N''
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		END
		ELSE IF @FactoryID = N''
				AND @ProductLineID = N''
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		END

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3. Clear Preference and edit
		DELETE FROM dbo.sx_pf_rPreferences
		WHERE 
			FactoryID			= @FactoryID 
			AND ProductLineID	= @ProductLineID
			AND Username		= @PreferenceUsername
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		-- TODO handle #NV - at the moment not handled
		SET @ValueText = IIF(@ValueText = N'<#NV>', N'', @ValueText);

		-- Save Preference
		INSERT INTO dbo.sx_pf_rPreferences (UserKey,UserName, FactoryID, ProductlineID, SettingName, ValueInt, ValueText)
		VALUES (@UserKey,@PreferenceUsername, @FactoryID, @ProductLineID, @SettingName, @ValueInt, @ValueText);

		SET @EffectedRows += @@ROWCOUNT;
		SET @Resultcode = 201;

		COMMIT TRANSACTION sx_pf_POST_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryService;
GO
/*
POST Operation for one Product
Not existing Product will be created if it exists in ZT Factory. Existing Product updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight 
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) = '*'
DECLARE @ProductLineID		NVARCHAR(255) = 'U'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TimeType			NVARCHAR(255) = '<#NV>'			--'VM'
DECLARE @NameShort			NVARCHAR(255) = 'Testproduct'
DECLARE @NameLong			NVARCHAR(255) = '3 test'
DECLARE @CommentUser		NVARCHAR(MAX) = 'komment'
DECLARE @CommentDev			NVARCHAR(MAX) = 'DEVKOmm'
DECLARE @ResponsiblePerson	NVARCHAR(255) = 'RR'
DECLARE @ImageName			NVARCHAR(255) = 'ImgName test'
DECLARE @Status				NVARCHAR(255) = 'Aktiv'
DECLARE @Template			NVARCHAR(255) = 'Unikum_VM'
DECLARE @TemplateVersion	NVARCHAR(255) = 'VT'
DECLARE @GA1				NVARCHAR(255) = 'Wert für GA1'	--'<#NV>'
DECLARE @GA2				NVARCHAR(255) = '<#NV>'
DECLARE @GA3				NVARCHAR(255) = '<#NV>'
DECLARE @GA4				NVARCHAR(255) = 'rer'
DECLARE @GA5				NVARCHAR(255) = '<#NV>'
DECLARE @GA6				NVARCHAR(255) = '<#NV>'
DECLARE @GA7				NVARCHAR(255) = '<#NV>'
DECLARE @GA8				NVARCHAR(255) = '<#NV>'
DECLARE @GA9				NVARCHAR(255) = '<#NV>'
DECLARE @GA10				NVARCHAR(255) = '<#NV>'
DECLARE @GA11				NVARCHAR(255) = '<#NV>'
DECLARE @GA12				NVARCHAR(255) = '<#NV>'
DECLARE @GA13				NVARCHAR(255) = '<#NV>'
DECLARE @GA14				NVARCHAR(255) = '<#NV>'
DECLARE @GA15				NVARCHAR(255) = '<#NV>'
DECLARE @GA16				NVARCHAR(255) = '<#NV>'
DECLARE @GA17				NVARCHAR(255) = '<#NV>'
DECLARE @GA18				NVARCHAR(255) = '<#NV>'
DECLARE @GA19				NVARCHAR(255) = '<#NV>'
DECLARE @GA20				NVARCHAR(255) = '<#NV>'
DECLARE @GA21				NVARCHAR(255) = '<#NV>'
DECLARE @GA22				NVARCHAR(255) = '<#NV>'
DECLARE @GA23				NVARCHAR(255) = '<#NV>'
DECLARE @GA24				NVARCHAR(255) = '<#NV>'
DECLARE @GA25				NVARCHAR(255) = '<#NV>'


EXECUTE @RC = dbo.sx_pf_POST_Product 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@TimeType,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName,
		@Status,
		@Template,
		@TemplateVersion,
		@GA1,
		@GA2,
		@GA3,
		@GA4,
		@GA5,
		@GA6,
		@GA7,
		@GA8,
		@GA9,
		@GA10,
		@GA11,
		@GA12,
		@GA13,
		@GA14,
		@GA15,
		@GA16,
		@GA17,
		@GA18,
		@GA19,
		@GA20,
		@GA21,
		@GA22,
		@GA23,
		@GA24,
		@GA25;

PRINT @RC;
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Product
GO

CREATE PROCEDURE dbo.sx_pf_POST_Product
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5				NVARCHAR(255),
	@GA6				NVARCHAR(255),
	@GA7				NVARCHAR(255),
	@GA8				NVARCHAR(255),
	@GA9				NVARCHAR(255),
	@GA10				NVARCHAR(255),
	@GA11				NVARCHAR(255),
	@GA12				NVARCHAR(255),
	@GA13				NVARCHAR(255),
	@GA14				NVARCHAR(255),
	@GA15				NVARCHAR(255),
	@GA16				NVARCHAR(255),
	@GA17				NVARCHAR(255),
	@GA18				NVARCHAR(255),
	@GA19				NVARCHAR(255),
	@GA20				NVARCHAR(255),
	@GA21				NVARCHAR(255),
	@GA22				NVARCHAR(255),
	@GA23				NVARCHAR(255),
	@GA24				NVARCHAR(255),
	@GA25				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductID			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')	 + N''',''' 
			+ ISNULL(@TimeType			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameShort			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameLong			, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL')	 + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL')	 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')	 + N''',''' 
			+ ISNULL(@Status			, N'NULL')	 + N''',''' 
			+ ISNULL(@Template			, N'NULL')	 + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL')	 + N''',''' 
			+ ISNULL(@GA1				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA2				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA3				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA4				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA5				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA6				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA7				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA8				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA9				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA10				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA11				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA12				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA13				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA14				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA15				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA16				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA17				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA18				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA19				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA20				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA21				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA22				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA23				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA24				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA25				, N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;						
	DECLARE @ResultCode			INT				 = 501;					
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';		

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			= N'';
	IF @ProductID			 IS NULL SET @ProductID			= N'';
	IF @ProductLineID		 IS NULL SET @ProductLineID		= N'';
	IF @FactoryID			 IS NULL SET @FactoryID			= N'';
	IF @TimeType			 IS NULL SET @TimeType			= N'';
	IF @NameShort			 IS NULL SET @NameShort			= N'';
	IF @NameLong			 IS NULL SET @NameLong			= N'';
	IF @CommentUser			 IS NULL SET @CommentUser		= N'';
	IF @CommentDev			 IS NULL SET @CommentDev		= N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson = N'';
	IF @ImageName			 IS NULL SET @ImageName			= N'';
	IF @Status				 IS NULL SET @Status			= N'';
	IF @Template			 IS NULL SET @Template			= N'';
	IF @TemplateVersion		 IS NULL SET @TemplateVersion	= N'';
	IF @GA1					 IS NULL SET @GA1				= N'';
	IF @GA2					 IS NULL SET @GA2				= N'';
	IF @GA3					 IS NULL SET @GA3 				= N'';
	IF @GA4					 IS NULL SET @GA4 				= N'';
	IF @GA5					 IS NULL SET @GA5 				= N'';
	IF @GA6					 IS NULL SET @GA6 				= N'';
	IF @GA7					 IS NULL SET @GA7 				= N'';
	IF @GA8					 IS NULL SET @GA8 				= N'';
	IF @GA9					 IS NULL SET @GA9 				= N'';
	IF @GA10				 IS NULL SET @GA10 				= N'';
	IF @GA11				 IS NULL SET @GA11 				= N'';
	IF @GA12				 IS NULL SET @GA12 				= N'';
	IF @GA13				 IS NULL SET @GA13 				= N'';
	IF @GA14				 IS NULL SET @GA14 				= N'';
	IF @GA15				 IS NULL SET @GA15 				= N'';
	IF @GA16				 IS NULL SET @GA16 				= N'';
	IF @GA17				 IS NULL SET @GA17 				= N'';
	IF @GA18				 IS NULL SET @GA18 				= N'';
	IF @GA19				 IS NULL SET @GA19 				= N'';
	IF @GA20				 IS NULL SET @GA20 				= N'';
	IF @GA21				 IS NULL SET @GA21 				= N'';
	IF @GA22				 IS NULL SET @GA22 				= N'';
	IF @GA23				 IS NULL SET @GA23 				= N'';
	IF @GA24				 IS NULL SET @GA24 				= N'';
	IF @GA25				 IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Product;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status				= dbo.sx_pf_pProtectString	(@Status);
		SET @Template			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2 				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3 				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4 				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5 				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6 				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7 				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8 				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9 				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10 				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11 				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12 				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13 				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14 				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15 				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16 				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17 				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18 				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19 				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20 				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21 				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22 				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23 				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24 				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25 				= dbo.sx_pf_pProtectString	(@GA25);

		IF	   @FactoryID		= N''
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey FROM dbo.sx_pf_dProducts WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;

		IF	@FactoryKey			 = 0 
			OR @ProductLineKey	 = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - If ProductID * is submitted, search the next free numeric ProductID
		IF @ProductID = '*' 
		BEGIN
			SELECT @ProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)),0)+1 FROM dbo.sx_pf_dProducts dP WHERE dP.ProductLineKey = @ProductLineKey AND TRY_CAST(dP.ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @ProductID = '*' 
			SELECT @ProductID = 1;
		END

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product dont exists, create it from the Template			
		IF @ProductKey = 0
		BEGIN
			DECLARE @TemplateFactoryID				 NVARCHAR(255)	= N'ZT';
			DECLARE @TemplateProductLineID			 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductID				 NVARCHAR(255)	= N'';
			DECLARE @TemplateFactoryKey				 INT			= 0;
			DECLARE @TemplateProductlineKey			 INT			= 0;
			DECLARE @TemplateProductKey				 INT			= 0;
			DECLARE @TemplateFactoryNameShort		 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductlineNameShort	 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductNameShort		 NVARCHAR(255)	= N'';

			-- Determine Templateproduct - it will be searched only in Factory ZT
			SELECT TOP (1) 
				@TemplateProductID		= ProductID, 
				@TemplateProductLineID	= ProductLineID 
			FROM dbo.sx_pf_dProducts
			WHERE 
					FactoryID = @TemplateFactoryID 
				AND Template = @Template
			ORDER BY ProductID;

			IF @TemplateProductID = N''	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template ProductID not found', 16, 10);
			END;

			-- Determine Template Keys
			SELECT 
					 @TemplateFactoryKey = FactoryKey
					,@TemplateFactoryNameShort = NameShort 
			FROM dbo.sx_pf_dFactories 
			WHERE FactoryID = @TemplateFactoryID;

			SELECT TOP (1)
					 @TemplateProductLineKey = ProductLineKey
					,@TemplateProductlineNameShort = NameShort 
			FROM dbo.sx_pf_dProductLines 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductlineID = @TemplateProductlineID
			ORDER BY ProductLineID;

			SELECT TOP (1) 
				 @TemplateProductKey = ProductKey
				,@TemplateProductNameShort = NameShort 
			FROM dbo.sx_pf_dProducts 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductlineKey = @TemplateProductlineKey 
				AND ProductID = @TemplateProductID
			ORDER BY ProductID;
			
			IF	   @TemplateFactoryKey		 = 0 
				OR @TemplateProductLineKey	 = 0
				OR @TemplateProductKey		 = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template keys not found', 16, 10);
			END;

			-- Copy Template Product		
			-- dProducts

			DECLARE @ProductKeyTable TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO dbo.sx_pf_dProducts 
			OUTPUT INSERTED.ProductKey INTO @ProductKeyTable
			SELECT @ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductlineID, 
				@FactoryID,
				TimeType, 
				@NameShort, 
				NameLong, 
				CommentUser,
				CommentDev, 
				ResponsiblePerson, 
				ImageName, 
				[Status], 
				Template, 
				TemplateVersion,
				GlobalAttribute1,
				GlobalAttribute2, 
				GlobalAttribute3, 
				GlobalAttribute4, 
				GlobalAttribute5, 
				GlobalAttribute6, 
				GlobalAttribute7,  
				GlobalAttribute8, 
				GlobalAttribute9, 
				GlobalAttribute10, 
				GlobalAttribute11, 
				GlobalAttribute12, 
				GlobalAttribute13, 
				GlobalAttribute14,  
				GlobalAttribute15, 
				GlobalAttribute16, 
				GlobalAttribute17, 
				GlobalAttribute18, 
				GlobalAttribute19, 
				GlobalAttribute20, 
				GlobalAttribute21,  
				GlobalAttribute22, 
				GlobalAttribute23, 
				GlobalAttribute24, 
				GlobalAttribute25 
			FROM dbo.sx_pf_dProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ProductKey = KeyNumber FROM @ProductKeyTable ORDER BY KeyNumber;

			--gProducts
			INSERT INTO dbo.sx_pf_gProducts 
			SELECT @ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductlineID,
				@FactoryID, 
				PropertyID, 
				PropertyName,
				CommentUser, 
				CommentDev, 
				Unit, 
				ValueText, 
				ValueInt, 
				Scale, 
				IsROSystemProperty, 
				FormatID
			FROM dbo.sx_pf_gProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			INSERT INTO dbo.sx_pf_dValueSeries 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID
				,@FactoryID 
				,ValueSeriesID 
				,ValueSeriesNo 
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,[IsNumeric] 
				,VisibilityLevel 
				,ValueSource 
				,ValueListID 
				,ValueFormatID 
				,Unit 
				,Scale 
				,Effect 
				,EffectParameter
			FROM dbo.sx_pf_dValueSeries 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			INSERT INTO dbo.sx_pf_dTime 
			SELECT
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID
				,@FactoryID 
				,TimeID 
				,FormatID
			FROM dbo.sx_pf_dTime 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			;WITH VS AS (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM dbo.sx_pf_dValueSeries
				WHERE ProductKey = @ProductKey
			)
			INSERT INTO dbo.sx_pf_fValues 
			SELECT 
				 VS.ValueSeriesKey 
				,@ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID 
				,@FactoryID 
				,fV.ValueSeriesID 
				,fV.TimeID 
				,fV.ValueFormula 
				,fV.ValueInt 
				,fV.ValueText 
				,fV.ValueComment
			FROM dbo.sx_pf_fValues fV 
				LEFT JOIN VS ON VS.ValueSeriesID= fV.ValueSeriesID 
			WHERE fV.ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements - create copy informations
			INSERT INTO dbo.sx_pf_fStatements 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID 
				,@FactoryID 
				,N'COPY'			AS ActionType
				,N'Product produced as a copy of Factory <' 
					+ @TemplateFactoryID 
					+ N'>"' 
					+ @TemplateFactoryNameShort 
					+ N'"/ ProductLine <'
					+ @TemplateProductlineID 
					+ N'>"' 
					+ @TemplateProductLineNameShort 
					+ N'"/ Product <' 
					+ @TemplateProductID 
					+ N'>"' 
					+ @TemplateProductNameShort 
					+ N'"'			AS Statement
				,@TransactUsername	AS UserName
				,N''					AS PCName
				,N''					AS ProcessorCode
				,N''					AS IPAddresses
				,CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 
					+ CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5),':',N'') AS BIGINT) 
									AS Timestamp
				,0					AS IsDeleted
				,0					AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE dbo.sx_pf_dProducts 
		SET 
			 TimeType			= IIF(@Template				= N'<#NV>', TimeType,			 RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			 @NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			 @NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		 @CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			 @CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	 @ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			 @ImageName)
			,[Status]			= IIF(@Status				= N'<#NV>', [Status],			 @Status)
			,Template			= IIF(@Template				= N'<#NV>', Template,			 @Template)
			,TemplateVersion	= IIF(@TemplateVersion		= N'<#NV>', TemplateVersion,	 @TemplateVersion)
			,GlobalAttribute1	= IIF(@GA1					= N'<#NV>', GlobalAttribute1,	 @GA1)
			,GlobalAttribute2	= IIF(@GA2					= N'<#NV>', GlobalAttribute2,	 @GA2)	
			,GlobalAttribute3	= IIF(@GA3					= N'<#NV>', GlobalAttribute3,	 @GA3)
			,GlobalAttribute4	= IIF(@GA4					= N'<#NV>', GlobalAttribute4,	 @GA4)	
			,GlobalAttribute5	= IIF(@GA5					= N'<#NV>', GlobalAttribute5,	 @GA5)
			,GlobalAttribute6	= IIF(@GA6					= N'<#NV>', GlobalAttribute6,	 @GA6)	
			,GlobalAttribute7	= IIF(@GA7					= N'<#NV>', GlobalAttribute7,	 @GA7)
			,GlobalAttribute8	= IIF(@GA8					= N'<#NV>', GlobalAttribute8,	 @GA8)	
			,GlobalAttribute9	= IIF(@GA9					= N'<#NV>', GlobalAttribute9,	 @GA9)
			,GlobalAttribute10	= IIF(@GA10					= N'<#NV>', GlobalAttribute10,	 @GA10)
			,GlobalAttribute11	= IIF(@GA11					= N'<#NV>', GlobalAttribute11,	 @GA11)
			,GlobalAttribute12	= IIF(@GA12					= N'<#NV>', GlobalAttribute12,	 @GA12)
			,GlobalAttribute13	= IIF(@GA13					= N'<#NV>', GlobalAttribute13,	 @GA13)
			,GlobalAttribute14	= IIF(@GA14					= N'<#NV>', GlobalAttribute14,	 @GA14)
			,GlobalAttribute15	= IIF(@GA15					= N'<#NV>', GlobalAttribute15,	 @GA15)
			,GlobalAttribute16	= IIF(@GA16					= N'<#NV>', GlobalAttribute16,	 @GA16)
			,GlobalAttribute17	= IIF(@GA17					= N'<#NV>', GlobalAttribute17,	 @GA17)
			,GlobalAttribute18	= IIF(@GA18					= N'<#NV>', GlobalAttribute18,	 @GA18)
			,GlobalAttribute19	= IIF(@GA19					= N'<#NV>', GlobalAttribute19,	 @GA19)
			,GlobalAttribute20	= IIF(@GA20					= N'<#NV>', GlobalAttribute20,	 @GA20)
			,GlobalAttribute21	= IIF(@GA21					= N'<#NV>', GlobalAttribute21,	 @GA21)
			,GlobalAttribute22	= IIF(@GA22					= N'<#NV>', GlobalAttribute22,	 @GA22)
			,GlobalAttribute23	= IIF(@GA23					= N'<#NV>', GlobalAttribute23,	 @GA23)
			,GlobalAttribute24	= IIF(@GA24					= N'<#NV>', GlobalAttribute24,	 @GA24)
			,GlobalAttribute25	= IIF(@GA25					= N'<#NV>', GlobalAttribute25,	 @GA25)	
		FROM dbo.sx_pf_dProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryService;
GO
/*
POST Operation for all values of the ProductDataTable
The method can be switched between incremental POST and full POST

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Concept:
- all Values are deliverd as Textarray - this requires a big split operation
- the Text array must also deliver deleted values if it does a incrementel transmisson (deleted Values are delivered with empty ValueInt, ValueText, ValueFormula)
- Values may be sended with '<#NV>' to keep existing values

Test call
1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
4. Non exists Factory / Productline / Product keys => 404 Not Found
5. If ValueSeriesID not exists in [sx_pf_dValueSeries] for this ProductKey => 403 Forbidden
6. Product updated => 200 OK

Test case 1 - Incremental Sending ***************************************************************
1. One Value for an existing Value must replace them
2. One Value for an not exiting Value must create it
3. If all Parameter are posted empty, the value is deleted
4. If one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
5. An ZeroEmpty Value in old or sended values is deleted

DECLARE @RC							INT
DECLARE @Username					NVARCHAR (255) = 'SQL'
DECLARE @ProductID					NVARCHAR (255) = '1'
DECLARE @ProductLineID				NVARCHAR (255) = 'U'
DECLARE @FactoryID					NVARCHAR (255) = 'ZT'
DECLARE @IsIncrementalValuesFlag	INT			   = 1

-- 3 Values with comments  - NEVER place a linebreak in the sample array !
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) =  '[(''K1'',20170115,''=IF(H25=4,3,2.1)'',''6'',''HaseTextValue'',''HaseComment'')],[(''K2'',20170215,''=IF(H25=4,3,2)'',''555'',''HaseTextValue'',''HaseComment2'')],[(''E1'',20170515,'''','''',''Text'',''Comment'')]'

-- Delete the three values by sending them empty
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''','''','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''','''','''')]'

-- Send an <#NV> Value for an existing numeric and an Text Value and delete one
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''',''<#NV>'','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''',''<#NV>'','''')]'

EXECUTE @RC = [sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;
 
Test case 2 - fill all Values of an Product ************************************************
DECLARE @RC								INT
DECLARE @Username						NVARCHAR(255) = 'SQL'
DECLARE @ProductID						NVARCHAR(255) = '1'
DECLARE @ProductLineID					NVARCHAR(255) = 'U'
DECLARE @FactoryID						NVARCHAR(255) = 'ZT'
DECLARE @IsIncrementalValuesFlag		INT = 0
DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = ''
DECLARE @start_time						DATETIME

-- Generate fictive values for all TimeIDs of Products
SELECT @ValuesInBracketsCommaSeparated = @ValuesInBracketsCommaSeparated + '[(''' + dVS.ValueSeriesID +''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',''1000'',''Hase | Maus'',''Komment|ar'')],' 
FROM 
	dbo.sx_pf_dValueSeries dVS LEFT JOIN dbo.sx_pf_dTime dT
		ON dVS.ProductKey = dT.ProductKey
WHERE 
	dVS.FactoryID			= @FactoryID 
	AND dVS.ProductLineID	= @ProductLineID 
	AND dVS.ProductID		= @ProductID 

SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated) -1)
PRINT @ValuesInBracketsCommaSeparated

SET @start_time = GETDATE()
EXECUTE @RC = [sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;

SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETDATE()) AS CHAR(10))) AS 'TimeTaken'

-- Testcase 3 - delete all values
DECLARE @RC								INT
DECLARE @Username						NVARCHAR (255)	= 'SQL'
DECLARE @ProductID						NVARCHAR (255)	= '1'
DECLARE @ProductLineID					NVARCHAR (255)	= 'U'
DECLARE @FactoryID						NVARCHAR (255)	= 'ZT'
DECLARE @IsIncrementalValuesFlag		INT				= 0
DECLARE @ValuesInBracketsCommaSeparated NVARCHAR (255)	= ''

EXECUTE @RC = [sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductDataTableValues
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductDataTableValues
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@ValuesInBracketsCommaSeparated		NVARCHAR(MAX) 
/*
@ValuesInBracketsCommaSeparated format is: (ValueSeriesID,TimeID,Formula,Value,ValueText,ValueComment),(ValueSeriesIDTimeID,Formula,Value,ValueText,ValueComment)
The whole string is single quoted and every parameter inside except the TimeID is single quoted
if you check this in SQL Server Profiler or Table sx_pf_APILog, every String must have two inverted commas, the whole Parameter must have Single inverted comma
this must show profiler: '[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')],[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')]'
In further releases the )],[( delimiter will be replaced with CHAR(29)
*/
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey 		INT				= 0;
	DECLARE @ProductKey 			INT				= 0;
	DECLARE @errorvalue				NVARCHAR(255);
	DECLARE @errormessage			NVARCHAR(2000);

	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString		NVARCHAR(MAX)	= N'''' 
									+ ISNULL(@Username													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductID													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductLineID												, N'NULL')	+ N''',''' 
									+ ISNULL(@FactoryID													, N'NULL')	+ N''','
									+ ISNULL(CAST(@IsIncrementalValuesFlag AS NVARCHAR(255))			, N'NULL')	+ N','''
									+ ISNULL(REPLACE(@ValuesInBracketsCommaSeparated, N'''', N'''''')	, N'NULL')	+ N'''';  -- WHY the replace - as there are double inverted comma's ?

	DECLARE @EffectedRows			INT				= 0;						
	DECLARE @ResultCode				INT				= 501;						
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)	= N'';			
	-- SET Debug Flag for debugging and disable Transcation 
	DECLARE @DebugFlag				INT				= 0
	/*
		--Use for debugging
		SELECT * FROM ##Debug_ValuesInBracketsReceived 
		SELECT * FROM ##Debug_ValueMergeTableFilled ORDER BY ValueSeriesID,TimeID
		SELECT * FROM ##Debug_ValueMergeTableBeforeTransmissions ORDER BY ValueSeriesID,TimeID
	*/

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag		= 0;
	IF @ValuesInBracketsCommaSeparated	IS NULL SET @ValuesInBracketsCommaSeparated = N'';

	-- To delete Values in NOT Incremental Transmission, an empty @ValuesInBracketsCommaSeparated Parameter is sended
	IF @ValuesInBracketsCommaSeparated = '' AND @IsIncrementalValuesFlag = 0 
	BEGIN
		SET @ValuesInBracketsCommaSeparated = '[(''DeleteFlagID'',99999999,'''',''0'','''','''')]'  --special ID with prevents from rejection from errorhandler
	END

	BEGIN TRY
		BEGIN TRANSACTION POST_PDT;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters, one or more IDs are empty', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM dbo.sx_pf_dProducts		WHERE ProductlineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF	@FactoryKey			= 0 
			OR @ProductLineKey	= 0 
			OR @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('One or more IDs dont exist (No Keys found)', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			BEGIN
				SET @errormessage = 'Invalid rights - no write Rights for ' + @TransactUsername + '.';
				RAISERROR(@errormessage, 16, 10);
			END

		-- STEP 3.0 - Store Timeline, only TimeID, as Timeline is not changed during Posting
		IF OBJECT_ID('tempdb..#Timeline') IS NOT NULL
		DROP TABLE #Timeline;	

		CREATE TABLE #Timeline (																																							
			TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT NOT NULL UNIQUE CLUSTERED
		);
				
		INSERT INTO #Timeline
		SELECT TimeID
		FROM dbo.sx_pf_dTime
		WHERE ProductKey = @ProductKey;

		-- STEP 3.1 - ValueMergeTable Merge the current values of the product into the ValueMergeTable
		IF OBJECT_ID('tempdb..#ValueMergeTable') IS NOT NULL
		DROP TABLE #ValueMergeTable;

		CREATE TABLE #ValueMergeTable ( 
			 ValueSeriesKey			BIGINT										NOT NULL
			,ValueSeriesID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,TimeID					NVARCHAR(10)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueFormula			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueInt				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueText				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueComment			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,[IsNumeric]			INT											NOT NULL 
			,IsFormula				INT											NOT NULL 
			,ProcessCode			NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,DocuCode				NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueSeriesID_TimeID 	NVARCHAR(265)	COLLATE DATABASE_DEFAULT	NOT NULL 
			);

		-- First Type "Old Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0												AS ValueSeriesKey  -- No Values from dValueSeries, this Informations are later added when old and sended Values are together
			,ValueSeriesID  
			,v.TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0												AS IsNumeric
			,0												AS IsFormula
			,N'OLD VALUES'									AS ProcessCode
			,N'NONE'										AS DocuCode
			,ValueSeriesID + CAST(v.TimeID AS NVARCHAR(10))	AS ValueSeries_TimeID
		FROM dbo.sx_pf_fValues v INNER JOIN #Timeline tl ON v.TimeID = tl.TimeID
		WHERE ProductKey = @ProductKey;		
			
		-- Store Header, which is dValueSeries Information which will added to old and sended Values
		IF OBJECT_ID('tempdb..#Header') IS NOT NULL
		DROP TABLE #Header;																			

		CREATE TABLE #Header (
			ValueSeriesKey		BIGINT									NOT NULL
			,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[IsNumeric]		INT										NOT NULL
			,IsFormula			INT										NOT NULL
			,UNIQUE CLUSTERED (ValueSeriesID, ValueSeriesKey, [IsNumeric], IsFormula)
			);
			
		INSERT INTO #Header
		SELECT 
			ValueSeriesKey
			,ValueSeriesID
			,[IsNumeric]
			,IIF(ValueSource IN (N'XLS',N'XLS-Strict'), 1, 0) AS IsFormula 
		FROM dbo.sx_pf_dValueSeries 
		WHERE ProductKey = @ProductKey;

		-- STEP 3.3 - Check for count of delimiter in Array
		DECLARE @OpenDelimetersCount	INT = 0;
		DECLARE @DelimetersIndex		INT = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated);

		WHILE @DelimetersIndex > 0 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount > 32765 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueArray to big (More then 32750 Values need special API Update.)', 16, 10);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.4 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated,  LEN(@ValuesInBracketsCommaSeparated)-2);
		SET @ValuesInBracketsCommaSeparated = RIGHT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated)-2);

		CREATE TABLE #Values1  (txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values2  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL)
		CREATE TABLE #Values3  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values4  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values5  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, ValueInt NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL)
		CREATE TABLE #Values6  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, ValueInt NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, ValueText NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL, ValueComment NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)

		DECLARE @spr NCHAR (1) = CHAR(29)

		-- Remove existing CHAR(29) from string
		SET @ValuesInBracketsCommaSeparated = REPLACE(@ValuesInBracketsCommaSeparated,@spr,N'');
		--PRINT @ValuesInBracketsCommaSeparated

		-- Split String into Rows - all Values except the TimeID are single quoted after that an separated by @spr																													
		INSERT INTO #Values1 
		SELECT REPLACE(REPLACE(txtValues,N',''', @spr + ''''), N''',', '''' + @spr) 
		FROM dbo.sx_pf_PivotStringIntoTable(@ValuesInBracketsCommaSeparated, N')],[(');

		-- Split string into two Columns (separate first column ValueSeriesID)
		INSERT INTO #Values2
		SELECT  
			LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values1;

		-- Split last columns once more (separate TimeID)
		INSERT INTO #Values3
		SELECT  
			ValueSeriesID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values2;

		-- Delete Implicit sended periods (dont exit in current Product)
		DELETE FROM #Values3 
		WHERE TimeID NOT IN (SELECT CAST(TimeID AS NVARCHAR(10)) FROM #Timeline);

		-- Split last columns once more (separate ValueFormula)
		INSERT INTO #Values4
		SELECT  
			ValueSeriesID
			,TimeID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values3;

		-- Split last columns once more (separate ValueInt)
		INSERT INTO #Values5
		SELECT
			ValueSeriesID  
			,TimeID
			,ValueFormula
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values4;

		-- Split last columns once more (Separate ValueText and ValueComment)
		INSERT INTO #Values6
		SELECT 
			ValueSeriesID 
			,TimeID
			,ValueFormula
			,ValueInt
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values5;

		-- Replace Quotes from all strings (all except TimeID)
		UPDATE #Values6
		SET	 
			ValueSeriesID	= RIGHT(LEFT(ValueSeriesID,	(LEN(ValueSeriesID)-1)),	LEN(ValueSeriesID)-2)
			,ValueFormula	= RIGHT(LEFT(ValueFormula,	(LEN(ValueFormula)-1)),		LEN(ValueFormula)-2)
			,ValueInt		= RIGHT(LEFT(ValueInt,		(LEN(ValueInt)-1)),			LEN(ValueInt)-2)
			,ValueText		= RIGHT(LEFT(ValueText,		(LEN(ValueText)-1)),		LEN(ValueText)-2)
			,ValueComment	= RIGHT(LEFT(ValueComment,	(LEN(ValueComment)-1)),		LEN(ValueComment)-2);

		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_Values6') IS NOT NULL DROP TABLE ##Debug_Values6;
				SELECT * INTO ##Debug_Values6 FROM #Values6;
		END;


		-- STEP 3.5 - UNION both temporary tables #######################################################################################################################

		-- Second Typ "Sended Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0 AS ValueSeriesKey
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0 AS [IsNumeric]
			,0 AS IsFormula
			,N'SENDED VALUES'
			,N'NONE'
			,ValueSeriesID+TimeID
		FROM #Values6;

		-- Add the header informations to the values
		UPDATE VMT 
		SET	
			 VMT.ValueSeriesKey = H.ValueSeriesKey
			,VMT.[IsNumeric]	= H.[IsNumeric]
			,VMT.IsFormula		= H.IsFormula 
		FROM #ValueMergeTable VMT INNER JOIN #Header H 
			ON VMT.ValueSeriesID = H.ValueSeriesID;

		-- check for not existing ValueSeriesIDs which may have been sended
		IF EXISTS(SELECT TOP (1) ValueSeriesID FROM #ValueMergeTable WHERE ValueSeriesKey = 0)
		BEGIN
			SELECT TOP(1) @errorvalue = ValueSeriesID 
			FROM #ValueMergeTable 
			WHERE ValueSeriesKey = 0 
			ORDER BY TimeID;

			IF @errorvalue = 'DeleteFlagID'
			BEGIN
				DELETE FROM #ValueMergeTable
			END
			ELSE
			BEGIN
				SET @errormessage = 'One or more sended ValueSeriesIDs were not found, e.g. '+ @errorvalue +'.'
				SET @ResultCode = 403;
				RAISERROR(@errormessage, 16, 10);
			END
		END;

		-- check if more than one value is sended for a ValueSeries/TimeID combination
		IF EXISTS(
			SELECT COUNT(TimeID) 
			FROM #ValueMergeTable 
			WHERE ProcessCode = 'SENDED VALUES'
			GROUP BY ValueSeriesID_TimeID
			HAVING COUNT(TimeID) > 1
		)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Douplicate Values found in ValueArray', 16, 10);
		END;

		-- STEP 3.6 - Prepare the ValueMergeTable #######################################################################################################################	
		-- FLAG OLD Values the user want to delete in Case of Numeric Series without formula
		-- Value 0 maybe sended as '000'
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula	= 0 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND	(TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula = 1  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode	 = N'SENDED VALUES' 
						AND ValueFormula = N''
						AND (TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series without formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 0  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND ValueText	= N''
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 1 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode		= N'SENDED VALUES' 
						AND	ValueText		= N'' 
						AND ValueFormula	= N''
				);

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableFilled') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableFilled;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableFilled 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.7 - Handle VMT #######################################################################################################################

		-- Documentation of changes in log table

			/* 

			-- NEW is a Value which is not contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'NEW VALUE'
			WHERE 
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND ValueSeriesID + N'_' + TimeID NOT IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			-- CHANGED is a Value which is contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'CHANGED VALUE'
			WHERE	
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND	ValueSeriesID + N'_' + TimeID IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			
			INSERT INTO sx_pf_AuditLog VALUES
				SELECT * FROM #ValueMergeTable WHERE 
					ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) IN 
					(
					SELECT
						ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) FROM #ValueMergeTable WHERE ProcessCode = 'SENDED VALUES' AND
						DocuCode != 'NONE'
					)
			*/

		-- Handle <#NV> Parameters - Replace them with the old Values (if exist !)

		UPDATE	VMT 
		SET		VMT.ValueInt = OV.ValueInt 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueInt, ValueSeriesID, TimeID  FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueInt LIKE N'<#NV>' 
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueText = OV.ValueText
		FROM	#ValueMergeTable VMT
			, (SELECT ValueText, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode		= N'SENDED VALUES' 
			AND VMT.ValueText = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueFormula = OV.ValueFormula 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueFormula, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueFormula = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueComment = OV.ValueComment 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueComment, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueComment = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		-- Handle <#NV> Parameter, where no old Values existed
		UPDATE #ValueMergeTable SET ValueInt		= N'0'	WHERE ValueInt		= N'<#NV>' OR [IsNumeric] = 0;
		UPDATE #ValueMergeTable SET ValueText		= N''	WHERE ValueText		= N'<#NV>' OR [IsNumeric] = 1;
		UPDATE #ValueMergeTable SET ValueFormula	= N''	WHERE ValueFormula	= N'<#NV>' OR IsFormula = 0;
		UPDATE #ValueMergeTable SET ValueComment	= N''	WHERE ValueComment	= N'<#NV>';

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableBeforeTransmissions') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableBeforeTransmissions;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableBeforeTransmissions 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.8 - Save data
		IF @IsIncrementalValuesFlag <> 1 
		BEGIN
			-- CASE FULL transmission #########################################
			-- Delete all Values
			DELETE FROM dbo.sx_pf_fValues WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;
			
			-- DELETE Useless NullEmpty Values in ALL Values

			-- einfache Nullwerte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		= 1 
				AND IsFormula	= 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0;

			-- Numerische Formelzellen ohne Formel mit Wert 0
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]			= 1 
				AND IsFormula		= 1 
				AND ValueFormula	= '' 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0;

			-- einfach Leertexte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]	  = 0 
				AND IsFormula = 0 
				AND ValueText = N'';

			-- Nichtnumerische Formelzellen ohne Formel mit Leertext
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		 = 0 
				AND IsFormula	 = 1 
				AND ValueFormula = '' 
				AND ValueText	 = N'';

			-- Insert all Values
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,ValueInt
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES');
				
			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE
		BEGIN
			-- CASE INCREMENTAL Transmission - it also leads to a full delete and insert !! #########################################
			-- DELETE Values, which are sended to delete in OLD VALUES
			DELETE FROM #ValueMergeTable WHERE ProcessCode = N'OLD VALUES TO DELETE'; 

			-- DELETE sended non-formula values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0'						-- send Null Values for ValueSeries *_0;

			--empty values must delete 0 in _0 Series
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND ValueInt = ''
				AND RIGHT(ValueSeriesID,2) = '_0';
			
			-- delete deleted formulas 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 1 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0 
				AND ValueFormula = N''
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0;

			-- Delete 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES'
				AND [IsNumeric] = 0
				AND IsFormula = 0 
				AND ValueText = N''
				

			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode		 = N'SENDED VALUES' 
				AND [IsNumeric]	 = 0 
				AND IsFormula	 = 1 
				AND ValueText	 = N'' 
				AND ValueFormula = N'';

			-- DELETE Useless NullEmpty Values in OLD Values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES'
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0

			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES' 
				AND [IsNumeric] = 0 
				AND IsFormula = 0 
				AND ValueText = N'';
					 
			-- DELETE all Values
			DELETE FROM dbo.sx_pf_fValues
			WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- Delete OLD VALUES which are new sended
			DELETE	FROM #ValueMergeTable 
			WHERE 
				ProcessCode = N'OLD VALUES'  
				AND ValueSeriesID_TimeID IN 
					(
						SELECT ValueSeriesID_TimeID
						FROM  #ValueMergeTable 
						WHERE ProcessCode = N'SENDED VALUES'
					);

			-- Insert all Values
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,CAST(ValueInt AS BIGINT)
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES', N'OLD VALUES');
			
			SET @EffectedRows += @@ROWCOUNT;


			--- materialize Product, if it is in ZT-PARAM
			IF	@FactoryID = 'ZT' AND @ProductLineID LIKE 'PARAM%'
					BEGIN
						EXEC control.spParamTables @ProductID
					END
		END;
			
		COMMIT TRANSACTION POST_PDT;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION POST_PDT;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
POST Operation for one empty Product
Not existing Product will be created, existing gets an Update
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
If product will have no TimeIDs, as it has no values
If product will have no ValueSeries, when created new 

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE	@RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) =	'53'
DECLARE @ProductLineID		NVARCHAR(255) =	'U'
DECLARE @FactoryID			NVARCHAR(255) =	'ZT'
DECLARE @TimeType			NVARCHAR(255) =	'<#NV>'--'VM'
DECLARE @NameShort			NVARCHAR(255) =	'Testproduct'
DECLARE @NameLong			NVARCHAR(255) =	'3 test'
DECLARE @CommentUser		NVARCHAR(255) =	'komment'
DECLARE @CommentDev			NVARCHAR(255) =	'DEVKOmm'
DECLARE @ResponsiblePerson	NVARCHAR(255) =	'RR'
DECLARE @ImageName			NVARCHAR(255) =	'ImgName test'
DECLARE @Status				NVARCHAR(255) =	'Aktiv'
DECLARE @Template			NVARCHAR(255) =	'Test_VM'
DECLARE @TemplateVersion	NVARCHAR(255) =	'4.0'
DECLARE @GA1				NVARCHAR(255) =	'Wert für GA1'--'<#NV>'
DECLARE @GA2				NVARCHAR(255) =	'<#NV>'
DECLARE @GA3				NVARCHAR(255) =	'<#NV>'
DECLARE @GA4				NVARCHAR(255) =	'rer'
DECLARE @GA5				NVARCHAR(255) =	'<#NV>'
DECLARE @GA6				NVARCHAR(255) =	'<#NV>'
DECLARE @GA7				NVARCHAR(255) =	'<#NV>'
DECLARE @GA8				NVARCHAR(255) =	'<#NV>'
DECLARE @GA9				NVARCHAR(255) =	'<#NV>'
DECLARE @GA10				NVARCHAR(255) =	'<#NV>'
DECLARE @GA11				NVARCHAR(255) =	'<#NV>'
DECLARE @GA12				NVARCHAR(255) =	'<#NV>'
DECLARE @GA13				NVARCHAR(255) =	'<#NV>'
DECLARE @GA14				NVARCHAR(255) =	'<#NV>'
DECLARE @GA15				NVARCHAR(255) =	'<#NV>'
DECLARE @GA16				NVARCHAR(255) =	'<#NV>'
DECLARE @GA17				NVARCHAR(255) =	'<#NV>'
DECLARE @GA18				NVARCHAR(255) =	'<#NV>'
DECLARE @GA19				NVARCHAR(255) =	'<#NV>'
DECLARE @GA20				NVARCHAR(255) =	'<#NV>'
DECLARE @GA21				NVARCHAR(255) =	'<#NV>'
DECLARE @GA22				NVARCHAR(255) =	'<#NV>'
DECLARE @GA23				NVARCHAR(255) =	'<#NV>'
DECLARE @GA24				NVARCHAR(255) =	'<#NV>'
DECLARE @GA25				NVARCHAR(255) =	'<#NV>'


EXECUTE @RC = dbo.sx_pf_POST_ProductEmpty 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@TimeType,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName,
		@Status,
		@Template,
		@TemplateVersion,
		@GA1,
		@GA2,
		@GA3,
		@GA4,
		@GA5,
		@GA6,
		@GA7,
		@GA8,
		@GA9,
		@GA10,
		@GA11,
		@GA12,
		@GA13,
		@GA14,
		@GA15,
		@GA16,
		@GA17,
		@GA18,
		@GA19,
		@GA20,
		@GA21,
		@GA22,
		@GA23,
		@GA24,
		@GA25

PRINT @RC
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ProductEmpty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductEmpty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductEmpty
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5  				NVARCHAR(255),
	@GA6  				NVARCHAR(255),
	@GA7  				NVARCHAR(255),
	@GA8  				NVARCHAR(255),
	@GA9  				NVARCHAR(255),
	@GA10  				NVARCHAR(255),
	@GA11  				NVARCHAR(255),
	@GA12  				NVARCHAR(255),
	@GA13  				NVARCHAR(255),
	@GA14  				NVARCHAR(255),
	@GA15  				NVARCHAR(255),
	@GA16  				NVARCHAR(255),
	@GA17  				NVARCHAR(255),
	@GA18  				NVARCHAR(255),
	@GA19  				NVARCHAR(255),
	@GA20  				NVARCHAR(255),
	@GA21  				NVARCHAR(255),
	@GA22  				NVARCHAR(255),
	@GA23  				NVARCHAR(255),
	@GA24  				NVARCHAR(255),
	@GA25 				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @ProductLineKey		 INT			= 0;
	DECLARE @ProductKey			 INT			= 0;
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N'''' 
			+ ISNULL(@Username			, N'NULL') + N''',''' 
			+ ISNULL(@ProductID			, N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL') + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL') + N''',''' 
			+ ISNULL(@TimeType			, N'NULL') + N''',''' 
			+ ISNULL(@NameShort			, N'NULL') + N''',''' 
			+ ISNULL(@NameLong			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL') + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL') + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL') + N''',''' 
			+ ISNULL(@ImageName			, N'NULL') + N''',''' 
			+ ISNULL(@Status			, N'NULL') + N''',''' 
			+ ISNULL(@Template			, N'NULL') + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL') + N''',''' 
			+ ISNULL(@GA1				, N'NULL') + N''',''' 
			+ ISNULL(@GA2				, N'NULL') + N''',''' 
			+ ISNULL(@GA3				, N'NULL') + N''',''' 
			+ ISNULL(@GA4				, N'NULL') + N''',''' 
			+ ISNULL(@GA5				, N'NULL') + N''',''' 
			+ ISNULL(@GA6				, N'NULL') + N''',''' 
			+ ISNULL(@GA7				, N'NULL') + N''',''' 
			+ ISNULL(@GA8				, N'NULL') + N''',''' 
			+ ISNULL(@GA9				, N'NULL') + N''',''' 
			+ ISNULL(@GA10				, N'NULL') + N''',''' 
			+ ISNULL(@GA11				, N'NULL') + N''',''' 
			+ ISNULL(@GA12				, N'NULL') + N''',''' 
			+ ISNULL(@GA13				, N'NULL') + N''',''' 
			+ ISNULL(@GA14				, N'NULL') + N''',''' 
			+ ISNULL(@GA15				, N'NULL') + N''',''' 
			+ ISNULL(@GA16				, N'NULL') + N''',''' 
			+ ISNULL(@GA17				, N'NULL') + N''',''' 
			+ ISNULL(@GA18				, N'NULL') + N''',''' 
			+ ISNULL(@GA19				, N'NULL') + N''',''' 
			+ ISNULL(@GA20				, N'NULL') + N''',''' 
			+ ISNULL(@GA21				, N'NULL') + N''',''' 
			+ ISNULL(@GA22				, N'NULL') + N''',''' 
			+ ISNULL(@GA23				, N'NULL') + N''',''' 
			+ ISNULL(@GA24				, N'NULL') + N''',''' 
			+ ISNULL(@GA25				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;				-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;				-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username 			IS NULL SET @Username			= N'';
	IF @ProductID 			IS NULL SET @ProductID			= N'';
	IF @ProductLineID 		IS NULL SET @ProductLineID		= N'';
	IF @FactoryID 			IS NULL SET @FactoryID			= N'';
	IF @TimeType 			IS NULL SET @TimeType			= N'';
	IF @NameShort 			IS NULL SET @NameShort			= N'';
	IF @NameLong 			IS NULL SET @NameLong			= N'';
	IF @CommentUser			IS NULL SET @CommentUser		= N'';
	IF @CommentDev			IS NULL SET @CommentDev 		= N'';
	IF @ResponsiblePerson	IS NULL SET @ResponsiblePerson  = N'';
	IF @ImageName 			IS NULL SET @ImageName			= N'';
	IF @Status 				IS NULL SET @Status				= N'';
	IF @Template 			IS NULL SET @Template			= N'';
	IF @TemplateVersion 	IS NULL SET @TemplateVersion	= N'';
	IF @GA1 				IS NULL SET @GA1				= N'';
	IF @GA2  				IS NULL SET @GA2				= N'';
	IF @GA3  				IS NULL SET @GA3				= N'';
	IF @GA4  				IS NULL SET @GA4				= N'';
	IF @GA5  				IS NULL SET @GA5				= N'';
	IF @GA6  				IS NULL SET @GA6 				= N'';
	IF @GA7  				IS NULL SET @GA7 				= N'';
	IF @GA8  				IS NULL SET @GA8 				= N'';
	IF @GA9  				IS NULL SET @GA9 				= N'';
	IF @GA10  				IS NULL SET @GA10 				= N'';
	IF @GA11  				IS NULL SET @GA11 				= N'';
	IF @GA12  				IS NULL SET @GA12 				= N'';
	IF @GA13  				IS NULL SET @GA13 				= N'';
	IF @GA14  				IS NULL SET @GA14 				= N'';
	IF @GA15  				IS NULL SET @GA15 				= N'';
	IF @GA16  				IS NULL SET @GA16 				= N'';
	IF @GA17  				IS NULL SET @GA17 				= N'';
	IF @GA18  				IS NULL SET @GA18 				= N'';
	IF @GA19  				IS NULL SET @GA19 				= N'';
	IF @GA20  				IS NULL SET @GA20 				= N'';
	IF @GA21  				IS NULL SET @GA21 				= N'';
	IF @GA22  				IS NULL SET @GA22 				= N'';
	IF @GA23  				IS NULL SET @GA23 				= N'';
	IF @GA24  				IS NULL SET @GA24 				= N'';
	IF @GA25  				IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductEmpty;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID 			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType 			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort 			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong 			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser 		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName 			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status  			= dbo.sx_pf_pProtectString	(@Status);
		SET @Template  			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1  				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2  				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3  				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4  				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5  				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6  				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7  				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8  				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9  				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10  				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11  				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12  				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13  				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14  				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15  				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16  				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17  				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18  				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19  				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20  				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21  				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22  				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23  				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24  				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25  				= dbo.sx_pf_pProtectString	(@GA25);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey		= @FactoryKey		AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM dbo.sx_pf_dProducts		WHERE ProductlineKey	= @ProductlineKey	AND ProductID = @ProductID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
			
		-- STEP 3.1 - If Product dont exists, create a new row in dProducts	
		IF @ProductKey = 0
		BEGIN
			--Handle wrong time types, cut them to two characters
			IF	LEN(@TimeType) > 2 
				SET @TimeType = LEFT(@TimeType,2);

			--Create dProducts
			DECLARE @ProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO dbo.sx_pf_dProducts  
			OUTPUT Inserted.ProductKey INTO @ProductKeyTable 
			VALUES (@ProductLineKey
					,@FactoryKey
					,@ProductID
					,@ProductlineID
					,@FactoryID
					,@TimeType
					,@NameShort
					,@NameLong
					,@CommentUser
					,@CommentDev
					,@ResponsiblePerson
					,@ImageName
					,@Status
					,@Template
					,@TemplateVersion
					,IIF(@GA1  = N'<#NV>', N'', @GA1)
					,IIF(@GA2  = N'<#NV>', N'', @GA2)
					,IIF(@GA3  = N'<#NV>', N'', @GA3)
					,IIF(@GA4  = N'<#NV>', N'', @GA4)
					,IIF(@GA5  = N'<#NV>', N'', @GA5)
					,IIF(@GA6  = N'<#NV>', N'', @GA6)
					,IIF(@GA7  = N'<#NV>', N'', @GA7)
					,IIF(@GA8  = N'<#NV>', N'', @GA8)
					,IIF(@GA9  = N'<#NV>', N'', @GA9)
					,IIF(@GA10 = N'<#NV>', N'', @GA10)
					,IIF(@GA11 = N'<#NV>', N'', @GA11)
					,IIF(@GA12 = N'<#NV>', N'', @GA12)
					,IIF(@GA13 = N'<#NV>', N'', @GA13)
					,IIF(@GA14 = N'<#NV>', N'', @GA14)
					,IIF(@GA15 = N'<#NV>', N'', @GA15)
					,IIF(@GA16 = N'<#NV>', N'', @GA16)
					,IIF(@GA17 = N'<#NV>', N'', @GA17)
					,IIF(@GA18 = N'<#NV>', N'', @GA18)
					,IIF(@GA19 = N'<#NV>', N'', @GA19)
					,IIF(@GA20 = N'<#NV>', N'', @GA20)
					,IIF(@GA21 = N'<#NV>', N'', @GA21)
					,IIF(@GA22 = N'<#NV>', N'', @GA22)
					,IIF(@GA23 = N'<#NV>', N'', @GA23)
					,IIF(@GA24 = N'<#NV>', N'', @GA24)
					,IIF(@GA25 = N'<#NV>', N'', @GA25));

			SET @EffectedRows += @@ROWCOUNT;

			SELECT @ProductKey = KeyNumber 
			FROM @ProductKeyTable;

			--fStatements - create copy  informations
			INSERT INTO dbo.sx_pf_fStatements
			SELECT 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID,  
				@ProductlineID, 
				@FactoryID, 
				N'COPY' AS ActionType,
				N'Product imported by Single Product Import ' AS Statement,
				@TransactUsername AS UserName,
				N'' AS PCName, 
				N'' AS ProcessorCode, 
				N'' AS IPAddresses,
				CAST(CONVERT(NVARCHAR(10),GETDATE(),112) as BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5),N':',N'') AS BIGINT) AS Timestamp,
				0 AS IsDeleted, 
				0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201;
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE dbo.sx_pf_dProducts 
		SET  
			 TimeType			 =  IIF(@Template			 = N'<#NV>', TimeType,			RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			 =  IIF(@NameShort			 = N'<#NV>', NameShort,			@NameShort)
			,NameLong			 =  IIF(@NameLong			 = N'<#NV>', NameLong,			@NameLong)
			,CommentUser		 =  IIF(@CommentUser		 = N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			 =  IIF(@CommentDev			 = N'<#NV>', CommentDev,		@CommentDev)
			,ResponsiblePerson	 =  IIF(@ResponsiblePerson	 = N'<#NV>', ResponsiblePerson, @ResponsiblePerson)
			,ImageName			 =  IIF(@ImageName			 = N'<#NV>', ImageName,			@ImageName)
			,Status				 =  IIF(@Status				 = N'<#NV>', Status,			@Status)
			,Template			 =  IIF(@Template			 = N'<#NV>', Template,			@Template)
			,TemplateVersion	 =  IIF(@TemplateVersion	 = N'<#NV>', TemplateVersion,	@TemplateVersion)
			,GlobalAttribute1	 =  IIF(@GA1				 = N'<#NV>', GlobalAttribute1,	@GA1)
			,GlobalAttribute2	 =  IIF(@GA2 				 = N'<#NV>', GlobalAttribute2,	@GA2)	
			,GlobalAttribute3	 =  IIF(@GA3 				 = N'<#NV>', GlobalAttribute3,	@GA3)
			,GlobalAttribute4	 =  IIF(@GA4 				 = N'<#NV>', GlobalAttribute4,	@GA4)	
			,GlobalAttribute5	 =  IIF(@GA5 				 = N'<#NV>', GlobalAttribute5,	@GA5)
			,GlobalAttribute6	 =  IIF(@GA6 				 = N'<#NV>', GlobalAttribute6,	@GA6)	
			,GlobalAttribute7	 =  IIF(@GA7 				 = N'<#NV>', GlobalAttribute7,	@GA7)
			,GlobalAttribute8	 =  IIF(@GA8 				 = N'<#NV>', GlobalAttribute8,	@GA8)	
			,GlobalAttribute9	 =  IIF(@GA9 				 = N'<#NV>', GlobalAttribute9,	@GA9)
			,GlobalAttribute10	 =  IIF(@GA10 				 = N'<#NV>', GlobalAttribute10, @GA10)
			,GlobalAttribute11	 =  IIF(@GA11 				 = N'<#NV>', GlobalAttribute11, @GA11)
			,GlobalAttribute12	 =  IIF(@GA12 				 = N'<#NV>', GlobalAttribute12, @GA12)
			,GlobalAttribute13	 =  IIF(@GA13 				 = N'<#NV>', GlobalAttribute13, @GA13)
			,GlobalAttribute14	 =  IIF(@GA14 				 = N'<#NV>', GlobalAttribute14, @GA14)
			,GlobalAttribute15	 =  IIF(@GA15 				 = N'<#NV>', GlobalAttribute15, @GA15)
			,GlobalAttribute16	 =  IIF(@GA16 				 = N'<#NV>', GlobalAttribute16, @GA16)
			,GlobalAttribute17	 =  IIF(@GA17 				 = N'<#NV>', GlobalAttribute17, @GA17)
			,GlobalAttribute18	 =  IIF(@GA18 				 = N'<#NV>', GlobalAttribute18, @GA18)
			,GlobalAttribute19	 =  IIF(@GA19 				 = N'<#NV>', GlobalAttribute19, @GA19)
			,GlobalAttribute20	 =  IIF(@GA20 				 = N'<#NV>', GlobalAttribute20, @GA20)
			,GlobalAttribute21	 =  IIF(@GA21 				 = N'<#NV>', GlobalAttribute21, @GA21)
			,GlobalAttribute22	 =  IIF(@GA22 				 = N'<#NV>', GlobalAttribute22, @GA22)
			,GlobalAttribute23	 =  IIF(@GA23 				 = N'<#NV>', GlobalAttribute23, @GA23)
			,GlobalAttribute24	 =  IIF(@GA24 				 = N'<#NV>', GlobalAttribute24, @GA24)
			,GlobalAttribute25	 =  IIF(@GA25				 = N'<#NV>', GlobalAttribute25, @GA25)	
		FROM dbo.sx_pf_dProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;	

		COMMIT TRANSACTION sx_pf_POST_ProductEmpty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductEmpty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductLine
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pGET_FactoryWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
3. Username does not have write access to Factory on Create => 401 Unauthorized
5. Non exists Factory / Productline keys => 404 Not Found
6. ProductLine created => 201 Created
7. ProductLine updated => 200 OK

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @ProductLineID				NVARCHAR(255) = 'ee'
DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
DECLARE @NameShort					NVARCHAR(255) = ''
DECLARE @NameLong					NVARCHAR(255) = ''
DECLARE @CommentUser				NVARCHAR(255) = ''
DECLARE @CommentDev					NVARCHAR(255) = ''
DECLARE @ResponsiblePerson			NVARCHAR(255) = ''
DECLARE @ImageName					NVARCHAR(255) = ''
DECLARE @DefaultTemplate			NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource1		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias1		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource2		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias2		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource3		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias3		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource4		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias4		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource5		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias5		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource6		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias6		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource7		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias7		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource8		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias8		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource9		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias9		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource10	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias10		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource11	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias11		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource12	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias12		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource13	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias13		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource14	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias14		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource15	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias15		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource16	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias16		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource17	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias17		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource18	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias18		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource19	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias19		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource20	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias20		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource21	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias21		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource22	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias22		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource23	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias23		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource24	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias24		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource25	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias25		NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_ProductLine @Username, @ProductLineID, @FactoryID, @NameShort, @NameLong, @CommentUser, @CommentDev, @ResponsiblePerson, @ImageName, @DefaultTemplate, 
		@GlobalAttributeSource1, @GlobalAttributeAlias1, @GlobalAttributeSource2, @GlobalAttributeAlias2, @GlobalAttributeSource3, @GlobalAttributeAlias3, @GlobalAttributeSource4, 
		@GlobalAttributeAlias4, @GlobalAttributeSource5, @GlobalAttributeAlias5, @GlobalAttributeSource6, @GlobalAttributeAlias6, @GlobalAttributeSource7, @GlobalAttributeAlias7, 
		@GlobalAttributeSource8, @GlobalAttributeAlias8, @GlobalAttributeSource9, @GlobalAttributeAlias9, @GlobalAttributeSource10, @GlobalAttributeAlias10, @GlobalAttributeSource11, 
		@GlobalAttributeAlias11, @GlobalAttributeSource12, @GlobalAttributeAlias12, @GlobalAttributeSource13, @GlobalAttributeAlias13, @GlobalAttributeSource14, @GlobalAttributeAlias14, 
		@GlobalAttributeSource15, @GlobalAttributeAlias15, @GlobalAttributeSource16, @GlobalAttributeAlias16, @GlobalAttributeSource17, @GlobalAttributeAlias17, @GlobalAttributeSource18, 
		@GlobalAttributeAlias18, @GlobalAttributeSource19, @GlobalAttributeAlias19, @GlobalAttributeSource20, @GlobalAttributeAlias20, @GlobalAttributeSource21, @GlobalAttributeAlias21, 
		@GlobalAttributeSource22, @GlobalAttributeAlias22, @GlobalAttributeSource23, @GlobalAttributeAlias23, @GlobalAttributeSource24, @GlobalAttributeAlias24, @GlobalAttributeSource25, 
		@GlobalAttributeAlias25

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductLine;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductLine
	@Username					NVARCHAR(255),
	@ProductLineID				NVARCHAR(255),
	@FactoryID					NVARCHAR(255),
	@NameShort					NVARCHAR(255),
	@NameLong					NVARCHAR(255),
	@CommentUser				NVARCHAR(MAX),
	@CommentDev					NVARCHAR(MAX),
	@ResponsiblePerson			NVARCHAR(255),
	@ImageName					NVARCHAR(255),
	@DefaultTemplate			NVARCHAR(255),
	@GlobalAttributeSource1		NVARCHAR(255),
	@GlobalAttributeAlias1		NVARCHAR(255),
	@GlobalAttributeSource2		NVARCHAR(255),
	@GlobalAttributeAlias2		NVARCHAR(255),
	@GlobalAttributeSource3		NVARCHAR(255),
	@GlobalAttributeAlias3		NVARCHAR(255),
	@GlobalAttributeSource4		NVARCHAR(255),
	@GlobalAttributeAlias4		NVARCHAR(255),
	@GlobalAttributeSource5		NVARCHAR(255),
	@GlobalAttributeAlias5		NVARCHAR(255),
	@GlobalAttributeSource6		NVARCHAR(255),
	@GlobalAttributeAlias6		NVARCHAR(255),
	@GlobalAttributeSource7		NVARCHAR(255),
	@GlobalAttributeAlias7		NVARCHAR(255),
	@GlobalAttributeSource8		NVARCHAR(255),
	@GlobalAttributeAlias8		NVARCHAR(255),
	@GlobalAttributeSource9		NVARCHAR(255),
	@GlobalAttributeAlias9		NVARCHAR(255),
	@GlobalAttributeSource10	NVARCHAR(255),
	@GlobalAttributeAlias10		NVARCHAR(255),
	@GlobalAttributeSource11	NVARCHAR(255),
	@GlobalAttributeAlias11		NVARCHAR(255),
	@GlobalAttributeSource12	NVARCHAR(255),
	@GlobalAttributeAlias12		NVARCHAR(255),
	@GlobalAttributeSource13	NVARCHAR(255),
	@GlobalAttributeAlias13		NVARCHAR(255),
	@GlobalAttributeSource14	NVARCHAR(255),
	@GlobalAttributeAlias14		NVARCHAR(255),
	@GlobalAttributeSource15	NVARCHAR(255),
	@GlobalAttributeAlias15		NVARCHAR(255),
	@GlobalAttributeSource16	NVARCHAR(255),
	@GlobalAttributeAlias16		NVARCHAR(255),
	@GlobalAttributeSource17	NVARCHAR(255),
	@GlobalAttributeAlias17		NVARCHAR(255),
	@GlobalAttributeSource18	NVARCHAR(255),
	@GlobalAttributeAlias18		NVARCHAR(255),
	@GlobalAttributeSource19	NVARCHAR(255),
	@GlobalAttributeAlias19		NVARCHAR(255),
	@GlobalAttributeSource20	NVARCHAR(255),
	@GlobalAttributeAlias20		NVARCHAR(255),
	@GlobalAttributeSource21	NVARCHAR(255),
	@GlobalAttributeAlias21		NVARCHAR(255),
	@GlobalAttributeSource22	NVARCHAR(255),
	@GlobalAttributeAlias22		NVARCHAR(255),
	@GlobalAttributeSource23	NVARCHAR(255),
	@GlobalAttributeAlias23		NVARCHAR(255),
	@GlobalAttributeSource24	NVARCHAR(255),
	@GlobalAttributeAlias24		NVARCHAR(255),
	@GlobalAttributeSource25	NVARCHAR(255),
	@GlobalAttributeAlias25		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)= N'';
	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProcedureName		NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = 
			N''''	   + ISNULL(@Username,					N'NULL')
			+ N''',''' + ISNULL(@ProductLineID,				N'NULL') + N''',''' + ISNULL(@FactoryID,				N'NULL') + N''',''' + ISNULL(@NameShort,				N'NULL')
			+ N''',''' + ISNULL(@NameLong,					N'NULL') + N''',''' + ISNULL(@CommentUser,				N'NULL') + N''',''' + ISNULL(@CommentDev,				N'NULL') 
			+ N''',''' + ISNULL(@ResponsiblePerson,			N'NULL') + N''',''' + ISNULL(@ImageName,				N'NULL') + N''',''' + ISNULL(@DefaultTemplate,			N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource2,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias2,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource3,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias3,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource5,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias5,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource6,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias6,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource8,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias8,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource9,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias9,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource11,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias11,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource12,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias12,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource14,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias14,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource15,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias15,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource17,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias17,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource18,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias18,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource20,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias20,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource21,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias21,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource23,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias23,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource24,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias24,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource25,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias25,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'';
	IF @FactoryID				IS NULL SET @FactoryID					= N'';
	IF @NameShort				IS NULL SET @NameShort					= N'';
	IF @NameLong				IS NULL SET @NameLong					= N'';
	IF @CommentUser				IS NULL SET @CommentUser				= N'';
	IF @CommentDev				IS NULL SET @CommentDev					= N'';
	IF @ResponsiblePerson		IS NULL SET @ResponsiblePerson			= N'';
	IF @ImageName				IS NULL SET @ImageName					= N'';
	IF @DefaultTemplate			IS NULL SET @DefaultTemplate			= N'';
	IF @GlobalAttributeSource1	IS NULL SET @GlobalAttributeSource1		= N'';
	IF @GlobalAttributeAlias1	IS NULL SET @GlobalAttributeAlias1		= N'';
	IF @GlobalAttributeSource2	IS NULL SET @GlobalAttributeSource2		= N'';
	IF @GlobalAttributeAlias2	IS NULL SET @GlobalAttributeAlias2		= N'';
	IF @GlobalAttributeSource3	IS NULL SET @GlobalAttributeSource3		= N'';
	IF @GlobalAttributeAlias3	IS NULL SET @GlobalAttributeAlias3		= N'';
	IF @GlobalAttributeSource4	IS NULL SET @GlobalAttributeSource4		= N'';
	IF @GlobalAttributeAlias4	IS NULL SET @GlobalAttributeAlias4		= N'';
	IF @GlobalAttributeSource5	IS NULL SET @GlobalAttributeSource5		= N'';
	IF @GlobalAttributeAlias5	IS NULL SET @GlobalAttributeAlias5		= N'';
	IF @GlobalAttributeSource6	IS NULL SET @GlobalAttributeSource6		= N'';
	IF @GlobalAttributeAlias6	IS NULL SET @GlobalAttributeAlias6		= N'';
	IF @GlobalAttributeSource7	IS NULL SET @GlobalAttributeSource7		= N'';
	IF @GlobalAttributeAlias7	IS NULL SET @GlobalAttributeAlias7		= N'';
	IF @GlobalAttributeSource8	IS NULL SET @GlobalAttributeSource8		= N'';
	IF @GlobalAttributeAlias8	IS NULL SET @GlobalAttributeAlias8		= N'';
	IF @GlobalAttributeSource9	IS NULL SET @GlobalAttributeSource9		= N'';
	IF @GlobalAttributeAlias9	IS NULL SET @GlobalAttributeAlias9		= N'';
	IF @GlobalAttributeSource10 IS NULL SET @GlobalAttributeSource10	= N'';
	IF @GlobalAttributeAlias10	IS NULL SET @GlobalAttributeAlias10		= N'';
	IF @GlobalAttributeSource11 IS NULL SET @GlobalAttributeSource11	= N'';
	IF @GlobalAttributeAlias11	IS NULL SET @GlobalAttributeAlias11		= N'';
	IF @GlobalAttributeSource12 IS NULL SET @GlobalAttributeSource12	= N'';
	IF @GlobalAttributeAlias12	IS NULL SET @GlobalAttributeAlias12		= N'';
	IF @GlobalAttributeSource13 IS NULL SET @GlobalAttributeSource13	= N'';
	IF @GlobalAttributeAlias13	IS NULL SET @GlobalAttributeAlias13		= N'';
	IF @GlobalAttributeSource14 IS NULL SET @GlobalAttributeSource14	= N'';
	IF @GlobalAttributeAlias14	IS NULL SET @GlobalAttributeAlias14		= N'';
	IF @GlobalAttributeSource15 IS NULL SET @GlobalAttributeSource15	= N'';
	IF @GlobalAttributeAlias15	IS NULL SET @GlobalAttributeAlias15		= N'';
	IF @GlobalAttributeSource16 IS NULL SET @GlobalAttributeSource16	= N'';
	IF @GlobalAttributeAlias16	IS NULL SET @GlobalAttributeAlias16		= N'';
	IF @GlobalAttributeSource17 IS NULL SET @GlobalAttributeSource17	= N'';
	IF @GlobalAttributeAlias17	IS NULL SET @GlobalAttributeAlias17		= N'';
	IF @GlobalAttributeSource18 IS NULL SET @GlobalAttributeSource18	= N'';
	IF @GlobalAttributeAlias18	IS NULL SET @GlobalAttributeAlias18		= N'';
	IF @GlobalAttributeSource19 IS NULL SET @GlobalAttributeSource19	= N'';
	IF @GlobalAttributeAlias19	IS NULL SET @GlobalAttributeAlias19		= N'';
	IF @GlobalAttributeSource20 IS NULL SET @GlobalAttributeSource20	= N'';
	IF @GlobalAttributeAlias20	IS NULL SET @GlobalAttributeAlias20		= N'';
	IF @GlobalAttributeSource21 IS NULL SET @GlobalAttributeSource21	= N'';
	IF @GlobalAttributeAlias21	IS NULL SET @GlobalAttributeAlias21		= N'';
	IF @GlobalAttributeSource22 IS NULL SET @GlobalAttributeSource22	= N'';
	IF @GlobalAttributeAlias22	IS NULL SET @GlobalAttributeAlias22		= N'';
	IF @GlobalAttributeSource23 IS NULL SET @GlobalAttributeSource23	= N'';
	IF @GlobalAttributeAlias23	IS NULL SET @GlobalAttributeAlias23		= N'';
	IF @GlobalAttributeSource24 IS NULL SET @GlobalAttributeSource24	= N'';
	IF @GlobalAttributeAlias24	IS NULL SET @GlobalAttributeAlias24		= N'';
	IF @GlobalAttributeSource25 IS NULL SET @GlobalAttributeSource25	= N'';
	IF @GlobalAttributeAlias25	IS NULL SET @GlobalAttributeAlias25		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLine;
		
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID			= dbo.sx_pf_pProtectID     (@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID	   (@FactoryID);
		SET @NameShort				= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong				= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser			= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev				= dbo.sx_pf_pProtectString (@CommentDev);
		SET @ResponsiblePerson		= dbo.sx_pf_pProtectString (@ResponsiblePerson);
		SET @ImageName				= dbo.sx_pf_pProtectString (@ImageName);
		SET @DefaultTemplate		= dbo.sx_pf_pProtectString (@DefaultTemplate);
		SET @GlobalAttributeAlias1	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias1);
		SET @GlobalAttributeAlias2	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias2);
		SET @GlobalAttributeAlias3	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias3);
		SET @GlobalAttributeAlias4	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias4);
		SET @GlobalAttributeAlias5	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias5);
		SET @GlobalAttributeAlias6	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias6);
		SET @GlobalAttributeAlias7	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias7);
		SET @GlobalAttributeAlias8	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias8);
		SET @GlobalAttributeAlias9	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias9);
		SET @GlobalAttributeAlias10 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias10);
		SET @GlobalAttributeAlias11 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias11);
		SET @GlobalAttributeAlias12 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias12);
		SET @GlobalAttributeAlias13 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias13);
		SET @GlobalAttributeAlias14 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias14);
		SET @GlobalAttributeAlias15 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias15);
		SET @GlobalAttributeAlias16 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias16);
		SET @GlobalAttributeAlias17 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias17);
		SET @GlobalAttributeAlias18 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias18);
		SET @GlobalAttributeAlias19 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias19);
		SET @GlobalAttributeAlias20 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias20);
		SET @GlobalAttributeAlias21 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias21);
		SET @GlobalAttributeAlias22 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias22);
		SET @GlobalAttributeAlias23 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias23);
		SET @GlobalAttributeAlias24 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias24);
		SET @GlobalAttributeAlias25 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias25);

		IF @GlobalAttributeSource1	!='<#NV>' SET @GlobalAttributeSource1  = dbo.sx_pf_pProtectID (@GlobalAttributeSource1)
		IF @GlobalAttributeSource2	!='<#NV>' SET @GlobalAttributeSource2  = dbo.sx_pf_pProtectID (@GlobalAttributeSource2)
		IF @GlobalAttributeSource3	!='<#NV>' SET @GlobalAttributeSource3  = dbo.sx_pf_pProtectID (@GlobalAttributeSource3)
		IF @GlobalAttributeSource4	!='<#NV>' SET @GlobalAttributeSource4  = dbo.sx_pf_pProtectID (@GlobalAttributeSource4)
		IF @GlobalAttributeSource5	!='<#NV>' SET @GlobalAttributeSource5  = dbo.sx_pf_pProtectID (@GlobalAttributeSource5)
		IF @GlobalAttributeSource6	!='<#NV>' SET @GlobalAttributeSource6  = dbo.sx_pf_pProtectID (@GlobalAttributeSource6)
		IF @GlobalAttributeSource7	!='<#NV>' SET @GlobalAttributeSource7  = dbo.sx_pf_pProtectID (@GlobalAttributeSource7)
		IF @GlobalAttributeSource8	!='<#NV>' SET @GlobalAttributeSource8  = dbo.sx_pf_pProtectID (@GlobalAttributeSource8)
		IF @GlobalAttributeSource9	!='<#NV>' SET @GlobalAttributeSource9  = dbo.sx_pf_pProtectID (@GlobalAttributeSource9)
		IF @GlobalAttributeSource10 !='<#NV>' SET @GlobalAttributeSource10 = dbo.sx_pf_pProtectID (@GlobalAttributeSource10)
		IF @GlobalAttributeSource11 !='<#NV>' SET @GlobalAttributeSource11 = dbo.sx_pf_pProtectID (@GlobalAttributeSource11)
		IF @GlobalAttributeSource12 !='<#NV>' SET @GlobalAttributeSource12 = dbo.sx_pf_pProtectID (@GlobalAttributeSource12)
		IF @GlobalAttributeSource13 !='<#NV>' SET @GlobalAttributeSource13 = dbo.sx_pf_pProtectID (@GlobalAttributeSource13)
		IF @GlobalAttributeSource14 !='<#NV>' SET @GlobalAttributeSource14 = dbo.sx_pf_pProtectID (@GlobalAttributeSource14)
		IF @GlobalAttributeSource15 !='<#NV>' SET @GlobalAttributeSource15 = dbo.sx_pf_pProtectID (@GlobalAttributeSource15)
		IF @GlobalAttributeSource16 !='<#NV>' SET @GlobalAttributeSource16 = dbo.sx_pf_pProtectID (@GlobalAttributeSource16)
		IF @GlobalAttributeSource17 !='<#NV>' SET @GlobalAttributeSource17 = dbo.sx_pf_pProtectID (@GlobalAttributeSource17)
		IF @GlobalAttributeSource18 !='<#NV>' SET @GlobalAttributeSource18 = dbo.sx_pf_pProtectID (@GlobalAttributeSource18)
		IF @GlobalAttributeSource19 !='<#NV>' SET @GlobalAttributeSource19 = dbo.sx_pf_pProtectID (@GlobalAttributeSource19)
		IF @GlobalAttributeSource20 !='<#NV>' SET @GlobalAttributeSource20 = dbo.sx_pf_pProtectID (@GlobalAttributeSource20)
		IF @GlobalAttributeSource21 !='<#NV>' SET @GlobalAttributeSource21 = dbo.sx_pf_pProtectID (@GlobalAttributeSource21)
		IF @GlobalAttributeSource22 !='<#NV>' SET @GlobalAttributeSource22 = dbo.sx_pf_pProtectID (@GlobalAttributeSource22)
		IF @GlobalAttributeSource23 !='<#NV>' SET @GlobalAttributeSource23 = dbo.sx_pf_pProtectID (@GlobalAttributeSource23)
		IF @GlobalAttributeSource24 !='<#NV>' SET @GlobalAttributeSource24 = dbo.sx_pf_pProtectID (@GlobalAttributeSource24)
		IF @GlobalAttributeSource25 !='<#NV>' SET @GlobalAttributeSource25 = dbo.sx_pf_pProtectID (@GlobalAttributeSource25)

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		IF @ProductLineKey = 0
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		ELSE
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product Line does not yet exist, it will be created in minimal configuration		
		IF @ProductLineKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_dProductlines (FactoryKey, ProductlineID, FactoryID, NameShort, NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName, DefaultTemplate
				, GlobalattributeSource1,  GlobalattributeAlias1,  GlobalattributeSource2,  GlobalattributeAlias2,  GlobalattributeSource3,  GlobalattributeAlias3,  GlobalattributeSource4,  GlobalattributeAlias4,  GlobalattributeSource5,  GlobalattributeAlias5
				, GlobalattributeSource6,  GlobalattributeAlias6,  GlobalattributeSource7,  GlobalattributeAlias7,  GlobalattributeSource8,  GlobalattributeAlias8,  GlobalattributeSource9,  GlobalattributeAlias9,  GlobalattributeSource10, GlobalattributeAlias10
				, GlobalattributeSource11, GlobalattributeAlias11, GlobalattributeSource12, GlobalattributeAlias12, GlobalattributeSource13, GlobalattributeAlias13, GlobalattributeSource14, GlobalattributeAlias14, GlobalattributeSource15, GlobalattributeAlias15
				, GlobalattributeSource16, GlobalattributeAlias16, GlobalattributeSource17, GlobalattributeAlias17, GlobalattributeSource18, GlobalattributeAlias18, GlobalattributeSource19, GlobalattributeAlias19, GlobalattributeSource20, GlobalattributeAlias20
				, GlobalattributeSource21, GlobalattributeAlias21, GlobalattributeSource22, GlobalattributeAlias22, GlobalattributeSource23, GlobalattributeAlias23, GlobalattributeSource24, GlobalattributeAlias24, GlobalattributeSource25, GlobalattributeAlias25
			) 
			OUTPUT INSERTED.ProductLineKey INTO @Keys([Key])  
			VALUES (@FactoryKey, @ProductLineID, @FactoryID, @NameShort, '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
			);

			SELECT TOP (1) @ProductLineKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode	  = 201;
		END;

		-- STEP 3.2 - Update values
		UPDATE dbo.sx_pf_dProductLines
		SET 
			NameShort					= IIF(@NameShort				= N'<#NV>', NameShort,				 @NameShort)
			, NameLong					= IIF(@NameLong					= N'<#NV>', NameLong,				 @NameLong)
			, CommentUser				= IIF(@CommentUser				= N'<#NV>', CommentUser,			 @CommentUser)
			, CommentDev				= IIF(@CommentDev				= N'<#NV>', CommentDev,				 @CommentDev)
			, ResponsiblePerson			= IIF(@ResponsiblePerson		= N'<#NV>', ResponsiblePerson,		 @ResponsiblePerson)
			, ImageName					= IIF(@ImageName				= N'<#NV>', ImageName,				 @ImageName)
			, DefaultTemplate			= IIF(@DefaultTemplate			= N'<#NV>', DefaultTemplate,		 @DefaultTemplate)
			, GlobalAttributeSource1	= IIF(@GlobalAttributeSource1	= N'<#NV>', GlobalAttributeSource1,	 @GlobalAttributeSource1)
			, GlobalAttributeAlias1		= IIF(@GlobalAttributeAlias1	= N'<#NV>', GlobalAttributeAlias1,	 @GlobalAttributeAlias1)			
			, GlobalAttributeSource2	= IIF(@GlobalAttributeSource2	= N'<#NV>', GlobalAttributeSource2,	 @GlobalAttributeSource2)
			, GlobalAttributeAlias2		= IIF(@GlobalAttributeAlias2	= N'<#NV>', GlobalAttributeAlias2,	 @GlobalAttributeAlias2)
			, GlobalAttributeSource3	= IIF(@GlobalAttributeSource3	= N'<#NV>', GlobalAttributeSource3,	 @GlobalAttributeSource3)
			, GlobalAttributeAlias3		= IIF(@GlobalAttributeAlias3	= N'<#NV>', GlobalAttributeAlias3,	 @GlobalAttributeAlias3)
			, GlobalAttributeSource4	= IIF(@GlobalAttributeSource4	= N'<#NV>', GlobalAttributeSource4,	 @GlobalAttributeSource4)
			, GlobalAttributeAlias4		= IIF(@GlobalAttributeAlias4	= N'<#NV>', GlobalAttributeAlias4,	 @GlobalAttributeAlias4)
			, GlobalAttributeSource5	= IIF(@GlobalAttributeSource5	= N'<#NV>', GlobalAttributeSource5,	 @GlobalAttributeSource5)
			, GlobalAttributeAlias5		= IIF(@GlobalAttributeAlias5	= N'<#NV>', GlobalAttributeAlias5,	 @GlobalAttributeAlias5)
			, GlobalAttributeSource6	= IIF(@GlobalAttributeSource6	= N'<#NV>', GlobalAttributeSource6,	 @GlobalAttributeSource6)
			, GlobalAttributeAlias6		= IIF(@GlobalAttributeAlias6	= N'<#NV>', GlobalAttributeAlias6,	 @GlobalAttributeAlias6)
			, GlobalAttributeSource7	= IIF(@GlobalAttributeSource7	= N'<#NV>', GlobalAttributeSource7,	 @GlobalAttributeSource7)
			, GlobalAttributeAlias7		= IIF(@GlobalAttributeAlias7	= N'<#NV>', GlobalAttributeAlias7,	 @GlobalAttributeAlias7)
			, GlobalAttributeSource8	= IIF(@GlobalAttributeSource8	= N'<#NV>', GlobalAttributeSource8,	 @GlobalAttributeSource8)
			, GlobalAttributeAlias8		= IIF(@GlobalAttributeAlias8	= N'<#NV>', GlobalAttributeAlias8,	 @GlobalAttributeAlias8)
			, GlobalAttributeSource9	= IIF(@GlobalAttributeSource9	= N'<#NV>', GlobalAttributeSource9,	 @GlobalAttributeSource9)
			, GlobalAttributeAlias9		= IIF(@GlobalAttributeAlias9	= N'<#NV>', GlobalAttributeAlias9,	 @GlobalAttributeAlias9)
			, GlobalAttributeSource10	= IIF(@GlobalAttributeSource10	= N'<#NV>', GlobalAttributeSource10, @GlobalAttributeSource10)
			, GlobalAttributeAlias10	= IIF(@GlobalAttributeAlias10	= N'<#NV>', GlobalAttributeAlias10,	 @GlobalAttributeAlias10)
			, GlobalAttributeSource11	= IIF(@GlobalAttributeSource11	= N'<#NV>', GlobalAttributeSource11, @GlobalAttributeSource11)
			, GlobalAttributeAlias11	= IIF(@GlobalAttributeAlias11	= N'<#NV>', GlobalAttributeAlias11,	 @GlobalAttributeAlias11)			
			, GlobalAttributeSource12	= IIF(@GlobalAttributeSource12	= N'<#NV>', GlobalAttributeSource12, @GlobalAttributeSource12)
			, GlobalAttributeAlias12	= IIF(@GlobalAttributeAlias12	= N'<#NV>', GlobalAttributeAlias12,	 @GlobalAttributeAlias12)
			, GlobalAttributeSource13	= IIF(@GlobalAttributeSource13	= N'<#NV>', GlobalAttributeSource13, @GlobalAttributeSource13)
			, GlobalAttributeAlias13	= IIF(@GlobalAttributeAlias13	= N'<#NV>', GlobalAttributeAlias13,	 @GlobalAttributeAlias13)
			, GlobalAttributeSource14	= IIF(@GlobalAttributeSource14	= N'<#NV>', GlobalAttributeSource14, @GlobalAttributeSource14)
			, GlobalAttributeAlias14	= IIF(@GlobalAttributeAlias14	= N'<#NV>', GlobalAttributeAlias14,	 @GlobalAttributeAlias14)
			, GlobalAttributeSource15	= IIF(@GlobalAttributeSource15	= N'<#NV>', GlobalAttributeSource15, @GlobalAttributeSource15)
			, GlobalAttributeAlias15	= IIF(@GlobalAttributeAlias15	= N'<#NV>', GlobalAttributeAlias15,	 @GlobalAttributeAlias15)
			, GlobalAttributeSource16	= IIF(@GlobalAttributeSource16	= N'<#NV>', GlobalAttributeSource16, @GlobalAttributeSource16)
			, GlobalAttributeAlias16	= IIF(@GlobalAttributeAlias16	= N'<#NV>', GlobalAttributeAlias16,	 @GlobalAttributeAlias16)
			, GlobalAttributeSource17	= IIF(@GlobalAttributeSource17	= N'<#NV>', GlobalAttributeSource17, @GlobalAttributeSource17)
			, GlobalAttributeAlias17	= IIF(@GlobalAttributeAlias17	= N'<#NV>', GlobalAttributeAlias17,	 @GlobalAttributeAlias17)
			, GlobalAttributeSource18	= IIF(@GlobalAttributeSource18	= N'<#NV>', GlobalAttributeSource18, @GlobalAttributeSource18)
			, GlobalAttributeAlias18	= IIF(@GlobalAttributeAlias18	= N'<#NV>', GlobalAttributeAlias18,	 @GlobalAttributeAlias18)
			, GlobalAttributeSource19	= IIF(@GlobalAttributeSource19	= N'<#NV>', GlobalAttributeSource19, @GlobalAttributeSource19)
			, GlobalAttributeAlias19	= IIF(@GlobalAttributeAlias19	= N'<#NV>', GlobalAttributeAlias19,	 @GlobalAttributeAlias19)
			, GlobalAttributeSource20	= IIF(@GlobalAttributeSource20	= N'<#NV>', GlobalAttributeSource20, @GlobalAttributeSource20)
			, GlobalAttributeAlias20	= IIF(@GlobalAttributeAlias20	= N'<#NV>', GlobalAttributeAlias20,	 @GlobalAttributeAlias20)
			, GlobalAttributeSource21	= IIF(@GlobalAttributeSource21	= N'<#NV>', GlobalAttributeSource21, @GlobalAttributeSource21)
			, GlobalAttributeAlias21	= IIF(@GlobalAttributeAlias21	= N'<#NV>', GlobalAttributeAlias21,	 @GlobalAttributeAlias21)			
			, GlobalAttributeSource22	= IIF(@GlobalAttributeSource22	= N'<#NV>', GlobalAttributeSource22, @GlobalAttributeSource22)
			, GlobalAttributeAlias22	= IIF(@GlobalAttributeAlias22	= N'<#NV>', GlobalAttributeAlias22,	 @GlobalAttributeAlias22)
			, GlobalAttributeSource23	= IIF(@GlobalAttributeSource23	= N'<#NV>', GlobalAttributeSource23, @GlobalAttributeSource23)
			, GlobalAttributeAlias23	= IIF(@GlobalAttributeAlias23	= N'<#NV>', GlobalAttributeAlias23,	 @GlobalAttributeAlias23)
			, GlobalAttributeSource24	= IIF(@GlobalAttributeSource24	= N'<#NV>', GlobalAttributeSource24, @GlobalAttributeSource24)
			, GlobalAttributeAlias24	= IIF(@GlobalAttributeAlias24	= N'<#NV>', GlobalAttributeAlias24,	 @GlobalAttributeAlias24)
			, GlobalAttributeSource25	= IIF(@GlobalAttributeSource25	= N'<#NV>', GlobalAttributeSource25, @GlobalAttributeSource25)
			, GlobalAttributeAlias25	= IIF(@GlobalAttributeAlias25	= N'<#NV>', GlobalAttributeAlias25,	 @GlobalAttributeAlias25)
		FROM dbo.sx_pf_dProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (only possible for users with Factory WriteRight or Cluster Admin)
		EXEC dbo.sx_pf_materialize_vUserRights;

		COMMIT TRANSACTION sx_pf_POST_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductLineProperty
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_pProtectInt
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
5. Property created => 201 Created
6. Property updated => 200 OK

DECLARE @RC					INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @ProductLineID		 NVARCHAR(255) = 'U'
DECLARE @FactoryID			 NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			 NVARCHAR(255) = 'rreo'
DECLARE @PropertyName		 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @Unit				 NVARCHAR(255) = ''
DECLARE @ValueText			 NVARCHAR(255) = ''
DECLARE @ValueInt			 NVARCHAR(255) = '678'
DECLARE @Scale				 NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty  NVARCHAR(255) = '1'
DECLARE @FormatID			 NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ProductLineProperty 
		@Username,
		@ProductLineID,
		@FactoryID,
		@PropertyID,	
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,	
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductLineProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductLineProperty
	@Username			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PropertyID			NVARCHAR(255),
	@PropertyName		NVARCHAR(255),
	@CommentUser		NVARCHAR(255),
	@CommentDev			NVARCHAR(255),
	@Unit 				NVARCHAR(255),
	@ValueText 			NVARCHAR(255),
	@ValueInt 			NVARCHAR(255),
	@Scale				NVARCHAR(255),
	@IsROSystemProperty NVARCHAR(255),
	@FormatID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,	N'NULL') + N''',''' + ISNULL(@PropertyName,		N'NULL') + N''',''' + ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,	N'NULL') + N''',''' + ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,		N'NULL') + N''',''' + ISNULL(@Scale,			N'NULL') + N''',''' + ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLineProperty;
				
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @FormatID		= dbo.sx_pf_pProtectString	(@FormatID); --as it can be #<NV>

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @PropertyKey = PropertyKey FROM dbo.sx_pf_gProductLines WHERE ProductLineKey =  @ProductLineKey AND PropertyID = @PropertyID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM dbo.sx_pf_gProductLines WHERE ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_gProductLines (ProductlineKey, FactoryKey, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductLineKey, @FactoryKey, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE dbo.sx_pf_gProductLines 
		SET 
			PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			, CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			, CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			, Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			, ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			, ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			, Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			, IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			, FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM dbo.sx_pf_gProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductProperty
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- sx_pf_pProtectString
		- sx_pf_pProtectID
		- sx_pf_pProtectInt
		- sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
5. Property created => 201 Created
6. Property updated => 200 OK

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) = '1'
DECLARE @ProductLineID		NVARCHAR(255) = 'U'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			NVARCHAR(255) = 'rreo'
DECLARE @PropertyName		NVARCHAR(255) = ''
DECLARE @CommentUser		NVARCHAR(255) = ''
DECLARE @CommentDev			NVARCHAR(255) = ''
DECLARE @Unit				NVARCHAR(255) = ''
DECLARE @ValueText			NVARCHAR(255) = ''
DECLARE @ValueInt			NVARCHAR(255) = '678'
DECLARE @Scale				NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	NVARCHAR(255) = '1'
DECLARE @FormatID			NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ProductProperty 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductProperty
	@Username				NVARCHAR(255),
	@ProductID				NVARCHAR(255),
	@ProductLineID			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
	@PropertyID				NVARCHAR(255),
	@PropertyName			NVARCHAR(255),
	@CommentUser			NVARCHAR(255),
	@CommentDev				NVARCHAR(255),
	@Unit					NVARCHAR(255),
	@ValueText				NVARCHAR(255),
	@ValueInt				NVARCHAR(255),
	@Scale					NVARCHAR(255),
	@IsROSystemProperty		NVARCHAR(255),
	@FormatID				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyName,			N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,			N'NULL') + N''',''' 
			+ ISNULL(@Unit,					N'NULL') + N''',''' 
			+ ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,				N'NULL') + N''',''' 
			+ ISNULL(@Scale,				N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,				N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductProperty;
			
		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		IF @FormatID <> '<#NV>'
			SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		--SET @PropertyOwner = dbo.sx_pf_pProtectBoolean (@IsROSystemProperty); -- NO Protection at the moment, as value can be 0,1,2

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM dbo.sx_pf_dProducts		WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey		= PropertyKey		FROM dbo.sx_pf_gProducts		WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;
		
		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0 
			OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM dbo.sx_pf_gProducts WHERE ProductKey =  @ProductKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration				
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO dbo.sx_pf_gProducts (ProductKey, ProductLineKey, FactoryKey, ProductID, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID)
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductKey, @ProductLineKey, @FactoryKey, @ProductID, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');
			
			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @Resultcode = 201;
			SET @EffectedRows = 1;
		END

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE dbo.sx_pf_gProducts
		SET  PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,		@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,				@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,			@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,			@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,				@Scale)
			,IsROSystemProperty = IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty, @IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,			@FormatID)
		FROM dbo.sx_pf_gProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for UserRight
UserRight will be created always, not fitting other rights will be deleted to ensure enheritance

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC						INT
DECLARE @UserName				NVARCHAR(255) = 'SQL'
DECLARE @PostUserName			NVARCHAR(255) = 'Hase'
DECLARE @FactoryID				NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID			NVARCHAR(255) = 'U'
DECLARE @Right					NVARCHAR(255) = 'Write'
DECLARE @ReadCommentMandatory	NVARCHAR(255) = ''
DECLARE @WriteCommentMandatory	NVARCHAR(255) = ''
		

EXECUTE @RC = dbo.sx_pf_POST_Right 
		@UserName,
		@PostUserName,
		@FactoryID,
        @ProductLineID,
        @Right,
        @ReadCommentMandatory,
        @WriteCommentMandatory

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Right') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Right;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Right
	@UserName				NVARCHAR(255),
	@PostUserName			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
    @ProductLineID			NVARCHAR(255),
    @Right					NVARCHAR(255),
    @ReadCommentMandatory	NVARCHAR(255),
    @WriteCommentMandatory  NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @PostUserKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ClusterRight		NVARCHAR(255)	= N'';
	DECLARE @FactoryRight		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'ISNULL') + N''',''' + ISNULL(@PostUserName,			N'ISNULL') + N''',''' 
			+ ISNULL(@FactoryID,				N'ISNULL') + N''',''' + ISNULL(@ProductLineID,			N'ISNULL') + N''',''' 
			+ ISNULL(@Right,					N'ISNULL') + N''',''' + ISNULL(@ReadCommentMandatory,	N'ISNULL') + N''',''' 
			+ ISNULL(@WriteCommentMandatory,	N'ISNULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @PostUserName			IS NULL SET @PostUserName			= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @Right					IS NULL SET @Right					= N'';
	IF @ReadCommentMandatory	IS NULL SET @ReadCommentMandatory	= N'';
	IF @WriteCommentMandatory	IS NULL SET @WriteCommentMandatory	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Right;	

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName			= dbo.sx_pf_pProtectString	(@PostUserName);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @Right					= dbo.sx_pf_pProtectString	(@Right);
		SET @ReadCommentMandatory	= dbo.sx_pf_pProtectBoolean (@ReadCommentMandatory);
		SET @WriteCommentMandatory	= dbo.sx_pf_pProtectBoolean (@WriteCommentMandatory);
		
		-- STEP 0.3 - Clean IDs
		SET @Right			= LTRIM(RTRIM(@Right));
		SET @PostUserName	= LTRIM(RTRIM(@PostUserName));
		SET @FactoryID		= LTRIM(RTRIM(@FactoryID));
		SET @ProductLineID	= LTRIM(RTRIM(@ProductLineID));

		IF	@Right = N'' 
			OR @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
				
		-- STEP 1.1 - It`s forbidden to post rights for system users with name 'SQL'
		IF	@PostUserName = N'SQL' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden to post rights for system users reserved name.', 16, 10, @PostUserName);
		END;	

		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to post rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to post rights', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 2.1 - It`s forbidden for a ClusterAdmin to change his own rights
		IF @PostUserName = @TransactUsername	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden for a ClusterAdmin to change his own rights', 16, 10);
		END;

		-- STEP 2.2 - Empty factory ID must always be connected to empty ProductLineID
		IF	@FactoryID = N'' 
			AND @ProductLineID <> N''
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Empty factory ID must always be connected to empty ProductLineID', 16, 10);
		END;
		
		-- STEP 2.3 - Check keys
		IF @FactoryID <> N''
		BEGIN
			SELECT @FactoryKey = FactoryKey FROM dbo.sx_pf_dFactories WHERE FactoryID = @FactoryID;

			IF @FactoryKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		IF @ProductLineID <> N'' 
		BEGIN
			SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
			
			IF @ProductLineKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		SELECT @PostUserKey = UserKey FROM dbo.sx_pf_rUser WHERE Username = @PostUserName;

		IF @PostUserKey  = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys not exists', 16, 10);
		END;

		-- STEP 3 - Set Rule
		IF @Right = N'Deny'
		BEGIN
			-- 'Deny' will delete a right, but never create an entry

			IF @FactoryID = N''
				DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey;
			ELSE IF @ProductLineID = N''
				DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
			ELSE
				DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 204;
		END
		ELSE
		BEGIN
			-- Saves only explicit rights on objects and deletes rights which get invalid through right definition on upper levels
			-- Check always Clusteright 
			
			SELECT TOP (1) @ClusterRight = COALESCE([Right], N'None') 
			FROM dbo.sx_pf_rRights 
			WHERE 
				UserKey = @PostUserKey 
				AND FactoryID = N''
			ORDER BY [Right];

			SET @ResultCode = 204;

			-- CLUSTER RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			-- Write / Read -> delete all other Rights and insert
			IF	@FactoryID = N'' AND @ProductLineID = N''
			BEGIN
				DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey;

				SET @EffectedRows += @@ROWCOUNT;

				INSERT INTO dbo.sx_pf_rRights
						(UserKey
						,UserName
						,FactoryID
						,ProductLineID
						,ProductID
						,[Right]
						,ReadCommentMandatory
						,WriteCommentMandatory)
				VALUES  (@PostUserKey			-- UserKey					int
						,@PostUserName			-- UserName					nvarchar(255)
						,N''					-- FactoryID				nvarchar(255)
						,N''					-- ProductLineID			nvarchar(255)
						,N''					-- ProductID				nvarchar(255)
						,@Right					-- Right					nvarchar(255)
						,@ReadCommentMandatory  -- ReadCommentMandatory		int
						,@WriteCommentMandatory -- WriteCommentMandatory	int
						);
				
				SET @EffectedRows += @@ROWCOUNT;
				SET @ResultCode = 200;
			END;

			-- FACTORY RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN
				-- Write -> delete all other Right in this Factory, dont accept if user has write on Cluster
				IF @Right = N'Write' AND @ClusterRight <> N'Write'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO dbo.sx_pf_rRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  (@PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if no cluster rights set -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO dbo.sx_pf_rRights
							(UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if Clusterwrite rights -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight = N'Read'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END
			END;

			-- PRODUCTLINE RIGHT is posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN
				-- Look for Rights on Factory Level
				SELECT TOP (1)  @FactoryRight = COALESCE([Right], N'None') 
				FROM dbo.sx_pf_rRights 
				WHERE 
						UserKey = @PostUserKey 
					AND FactoryID = @FactoryID 
					AND ProductLineID = N''
				ORDER BY [Right];

				-- Write -> dont accept if user has already write on Cluster or Factory
				IF @Right = N'Write' AND @ClusterRight <> N'Write' AND @FactoryRight <> N'Write'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO dbo.sx_pf_rRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,@ProductLineID			-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read -> don`t accept if user has read or write on Cluster or Factory
				IF	@Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read' 
					AND @FactoryRight <> N'Write' AND @FactoryRight <> N'Read'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO dbo.sx_pf_rRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey				-- UserKey					int
							,@PostUserName				-- UserName					nvarchar(255)
							,@FactoryID					-- FactoryID				nvarchar(255)
							,@ProductLineID				-- ProductLineID			nvarchar(255)
							,N''						-- ProductID				nvarchar(255)
							,@Right						-- Right					nvarchar(255)
							,@ReadCommentMandatory		-- ReadCommentMandatory		int
							,@WriteCommentMandatory		-- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;
						
				-- Read -> if read should only kill an existing write in Factories with global Read Rights, just delete Write Right
				IF @Right = N'Read' AND @FactoryRight = N'Read'
				BEGIN
					DELETE FROM dbo.sx_pf_rRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;
				END;
			END;
		END;

		COMMIT TRANSACTION sx_pf_POST_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Right TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Right TO pf_PlanningFactoryService;
GO
/*
POST Operation for a new Statement
A Statement can only created, there is no Update or Delete

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = '1'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ActionType		NVARCHAR(255) = 'SAVE'
DECLARE @Statement		NVARCHAR(255) = 'ALLES GUT'
DECLARE @PCName			NVARCHAR(255) = ''
DECLARE @ProcessorCode	NVARCHAR(255) = ''
DECLARE @IPAddresses	NVARCHAR(255) = ''
DECLARE @IsDeleted		NVARCHAR(255) = ''
DECLARE @IsResolved		NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_Statement 
		@Username,	
		@ProductID,	
		@ProductLineID,	
		@FactoryID,	
		@ActionType,
		@Statement,	
		@PCName,
		@ProcessorCode,	
		@IPAddresses,
		@IsDeleted,
		@IsResolved

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Statement') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Statement;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Statement
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ActionType		NVARCHAR(255),
	@Statement		NVARCHAR(255),
	@PCName			NVARCHAR(255),
	@ProcessorCode	NVARCHAR(255),
	@IPAddresses	NVARCHAR(255),
	@IsDeleted		NVARCHAR(255),
	@IsResolved		NVARCHAR(255)	
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,	N'NULL') + N''',''' + ISNULL(@ActionType,		N'NULL') + N''',''' + ISNULL(@Statement,		N'NULL') + N''',''' 
			+ ISNULL(@PCName,		N'NULL') + N''',''' + ISNULL(@ProcessorCode,	N'NULL') + N''',''' + ISNULL(@IPAddresses,		N'NULL') + N''',''' 
			+ ISNULL(@IsDeleted,	N'NULL') + N''',''' + ISNULL(@IsResolved,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ActionType		IS NULL SET @ActionType		= N'';
	IF @Statement		IS NULL SET @Statement		= N'';
	IF @PCName			IS NULL SET @PCName			= N'';
	IF @ProcessorCode	IS NULL SET @ProcessorCode	= N'';
	IF @IPAddresses		IS NULL SET @IPAddresses	= N'';
	IF @IsDeleted		IS NULL SET @IsDeleted		= N'';
	IF @IsResolved		IS NULL SET @IsResolved		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Statement

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ActionType		= dbo.sx_pf_pProtectString	(@ActionType);
		SET @Statement		= dbo.sx_pf_pProtectString	(@Statement);
		SET @PCName			= dbo.sx_pf_pProtectString	(@PCName);
		SET @ProcessorCode	= dbo.sx_pf_pProtectString	(@ProcessorCode);
		SET @IPAddresses	= dbo.sx_pf_pProtectString	(@IPAddresses);
		SET @IsDeleted		= dbo.sx_pf_pProtectBoolean (@IsDeleted);
		SET @IsResolved		= dbo.sx_pf_pProtectBoolean (@IsResolved);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
				
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM dbo.sx_pf_dProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3 - Insert new Statement
		INSERT INTO dbo.sx_pf_fStatements (
			ProductKey,
			ProductLineKey,
			FactoryKey,
			ProductID,
			ProductLineID,
			FactoryID,
			ActionType,
			Statement,
			UserName,
			PCName,
			ProcessorCode,
			IPAddresses,
			Timestamp,
			IsDeleted,
			IsResolved) 
		VALUES
			(@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@ActionType,
			@Statement,
			@TransactUsername,
			@PCName,
			@ProcessorCode,
			@IPAddresses,
			CAST(YEAR(CURRENT_TIMESTAMP) AS BIGINT)*10000000000 + CAST(MONTH(CURRENT_TIMESTAMP) as BIGINT)*100000000 + DAY(CURRENT_TIMESTAMP)*1000000 + DATEPART(hh,CURRENT_TIMESTAMP)*10000 + DATEPART(MINUTE,CURRENT_TIMESTAMP) *100 + DATEPART(ss, CURRENT_TIMESTAMP),
			@IsDeleted,
			@IsResolved);

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_POST_Statement;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Statement;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryService;
GO
/*
POST Operation to apply a template (again) on a Product
Existing Values in the Product will be saved, new/other logic will be added
It means the Template inherits its properties again to the object

The logical process is

1. replace the Target Product by a new copy of the Template
2. transfer all global attributes whitout changing
2. Import all Values from ValueSeries which are of type "Input" in the Template to the Template


03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC								INT 
DECLARE @Username						NVARCHAR(255) = 'SQL'
DECLARE @ProductID						NVARCHAR(255) = '2'
DECLARE @ProductLineID					NVARCHAR(255) = 'U'
DECLARE @FactoryID						NVARCHAR(255) = 'ZT'

DECLARE @TemplateProductID				NVARCHAR(255) = '1'
DECLARE @TemplateProductLineID			NVARCHAR(255) = 'U'
DECLARE @TemplateFactoryID				NVARCHAR(255) = 'ZT'
DECLARE @CompareByNameInsteadOfIDFlag	INT = 0

EXECUTE @RC = dbo.sx_pf_POST_Template 

   @Username
  ,@ProductID
  ,@ProductLineID
  ,@FactoryID
  ,@TemplateProductID
  ,@TemplateProductLineID
  ,@TemplateFactoryID
  ,@CompareByNameInsteadOfIDFlag

PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Template') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Template
GO

CREATE PROCEDURE dbo.sx_pf_POST_Template
	@Username						NVARCHAR(255),
	@ProductID						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN
	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @FactoryKey						INT				= 0;
	DECLARE @ProductLineKey					INT				= 0;
	DECLARE @ProductKey						INT				= 0;
	DECLARE @TemplateFactoryKey				INT				= 0;
	DECLARE @TemplateProductLineKey			INT				= 0;
	DECLARE @TemplateProductKey				INT				= 0;
	DECLARE @TemplateFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateProductLineNameShort	NVARCHAR(255)	= N'';
	DECLARE @TemplateProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateName					NVARCHAR(255)	= N'';
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username,					N'NULL') + N''',''' 
														+ ISNULL(@ProductID,				N'NULL') + N''',''' 
														+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
														+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
														+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
														+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows					INT				= 0;				
	DECLARE @ResultCode						INT				= 501;				
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment						NVARCHAR(2000)	= N'';				


	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductlineID			= dbo.sx_pf_pProtectID	(@TemplateProductlineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);
	
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 0.3. - Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey	= @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID		= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey	= @TemplateFactoryKey AND ProductlineID = @TemplateProductlineID;

		-- Stop on invalid Keys
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @TemplateFactoryKey = 0 OR @TemplateProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		 -- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN

			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode <> 200
				RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 2.1 - Try to identify the product keys
		SELECT @ProductKey			= ProductKey FROM dbo.sx_pf_dProducts WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;
		SELECT @TemplateProductKey	= ProductKey FROM dbo.sx_pf_dProducts WHERE ProductlineKey = @TemplateProductlineKey AND ProductID = @TemplateProductID;
		
		-- Stop if Keys don't exist
		IF @ProductKey = 0 OR @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- Stop if Template is posted on itself
		IF @ProductKey = @TemplateProductKey
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Stop, as Template cant be posted on itself.', 16, 10);
		END;

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in Template
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND	EXISTS (SELECT COUNT(NameShort) FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @TemplateProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Template', 16, 10);
		END

		-- STEP 2.3 - Stop, if compare by name and there are doubled ValueSerieNames in Product
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND EXISTS (SELECT COUNT(NameShort) FROM dbo.sx_pf_dValueSeries WHERE ProductKey = @ProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Target Product', 16, 10);
		END

		-- STEP 3.1 - Determine Template values
		SELECT @TemplateFactoryNameShort		= NameShort FROM dbo.sx_pf_dFactories		WHERE FactoryKey = @FactoryKey;
		SELECT @TemplateProductLineNameShort	= NameShort FROM dbo.sx_pf_dProductLines	WHERE ProductLineKey = @ProductLineKey;
		SELECT @TemplateProductNameShort		= NameShort FROM dbo.sx_pf_dProducts		WHERE ProductKey = @ProductKey;
		SELECT @TemplateName					= Template	FROM dbo.sx_pf_dProducts		WHERE ProductKey = @TemplateProductKey;

		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			  ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			, Scale				INT											NOT NULL
			, TimeID			INT											NOT NULL
			, ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueInt			BIGINT										NOT NULL
			, ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueComment		NVARCHAR (MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Store Data of Product in Storage (but not formulas, they come from template)
		INSERT INTO #ProductDataStorage    
		SELECT 
			fV.ValueSeriesID
			, dVS.NameShort AS ValueSeriesName
			, dVS.Scale
			, fV.TimeID
			, COALESCE(fVT.ValueFormula,'') AS ValueFormula -- the new formulas will be catched from the template 
			, fV.ValueInt
			, fV.ValueText
			, fV.ValueComment 
		FROM 
			dbo.sx_pf_fValues fV LEFT JOIN dbo.sx_pf_dValueSeries dVS 
				ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
			-- the new Template Formulas
			LEFT JOIN (SELECT ValueSeriesID, TimeID, fV.ValueFormula FROM dbo.sx_pf_fValues fV WHERE fV.ProductKey = @TemplateProductKey) fVT
				ON fV.ValueSeriesID = fVT.ValueSeriesID AND	fV.TimeID = fVT.TimeID
		WHERE fV.ProductKey =  @ProductKey;
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Product (but not the dProduct, Statements...)
		DELETE FROM dbo.sx_pf_fValues		WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dValueSeries  WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dTime			WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_gProducts		WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,Scale				INT											NOT NULL
		);

		-- STEP 3.3 Copy ValueSeries of Template to Product 
		INSERT INTO dbo.sx_pf_dValueSeries 
		OUTPUT inserted.ValueSeriesKey, inserted.ValueSeriesID, Inserted.NameShort, Inserted.Scale INTO #VSKeys
		SELECT 
			@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,dVS.ValueSeriesID
			,dVS.ValueSeriesNo
			,dVS.NameShort
			,dVS.NameLong
			,dVS.CommentUser
			,dVS.CommentDev
			,dVS.ImageName
			,dVS.IsNumeric
			,dVS.VisibilityLevel
			,dVS.ValueSource
			,dVS.ValueListID
			,dVS.ValueFormatID
			,dVS.Unit
			,dVS.Scale
			,dVS.Effect
			,dVS.EffectParameter
		FROM dbo.sx_pf_dValueSeries dVS 
		WHERE dVS.ProductKey = @TemplateProductKey

		-- STEP 3.4 Copy Timeline of Template to Product 
		INSERT INTO dbo.sx_pf_dTime
		SELECT 
			@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,TimeID
			,FormatID
		FROM dbo.sx_pf_dTime dT 
		WHERE dT.ProductKey = @TemplateProductKey

		-- STEP 3.4 Copy Values of Template to Product 
		INSERT INTO dbo.sx_pf_fValues
		SELECT
			vk.ValueSeriesKey
			,@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,fV.ValueSeriesID
			,fV.TimeID
			,fV.ValueFormula
			,fV.ValueInt
			,fV.ValueText
			,fV.ValueComment
		FROM dbo.sx_pf_fValues fV LEFT JOIN #VSKeys vk 
			ON fV.ValueSeriesID = vk.ValueSeriesID
		WHERE fV.ProductKey = @TemplateProductKey

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO dbo.sx_pf_gProducts
			SELECT
				 @ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID
			FROM dbo.sx_pf_gProducts gP 
			WHERE gP.ProductKey = @TemplateProductKey


		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled


		-- STEP 3.5 - Update new Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
		BEGIN
			-- Delete Values, which belong to no longer existing IDs
			DELETE FROM #ProductDataStorage 
			WHERE ValueSeriesID NOT IN	(
				SELECT dVS.ValueSeriesID 
				FROM dbo.sx_pf_dValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);
				
			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage 
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM dbo.sx_pf_dTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE FROM dbo.sx_pf_fValues 
			WHERE ProductKey = @ProductKey 
				AND	ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
					(	
						SELECT ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) 
						FROM #ProductDataStorage pds
					);

			-- Insert all other Values - this are Values for input cells and values formula Cell
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				vk.ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,pds.ValueSeriesID
				,pds.TimeID
				,ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
				ON pds.ValueSeriesID = vk.ValueSeriesID
		END
		ELSE
		BEGIN -- compare by Name
			-- Delete Values, which belong to no longer existing Names
			DELETE FROM #ProductDataStorage 
			WHERE ValueSeriesName NOT IN (
				SELECT dVS.NameShort
				FROM dbo.sx_pf_dValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);

			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage 
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM dbo.sx_pf_dTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE fV 
			FROM dbo.sx_pf_fValues fV LEFT JOIN sx_pf_dValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey 
				AND	dVS.NameShort + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
				(	
					SELECT pds.ValueSeriesName + '_' + CAST(TimeID AS NVARCHAR(255)) 
					FROM #ProductDataStorage pds
				);

			-- Insert all other Values, if they fit to timeline
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				vk.ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,dVS.ValueSeriesID
				,pds.TimeID
				,ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk  -- to determine ValueSeries Key
					ON pds.ValueSeriesName = vk.ValueSeriesName
					-- to determine new ValueSeriesID
				LEFT JOIN (SELECT dVS.ValueSeriesID, dVS.NameShort FROM dbo.sx_pf_dValueSeries dVS WHERE dVS.ProductKey = @TemplateProductKey) dVS 
					ON dVS.NameShort = pds.ValueSeriesName
			-- filter only fitting TimeIDs
			WHERE pds.TimeID IN (
				SELECT TimeID 
				FROM dbo.sx_pf_dTime dT 
				WHERE dT.ProductKey = @TemplateProductKey
			);

		END;

		-- Create a System Statement
		INSERT INTO dbo.sx_pf_fStatements  
		SELECT  
			  @ProductKey
			, @ProductLineKey
			, @FactoryKey
			, @ProductID
			, @ProductlineID
			, @FactoryID
			, N'POST_Template' AS ActionType
			, N'Product Template updated with Template Factory <' 
				+ @TemplateFactoryID			+ N'>"' 
				+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
				+ @TemplateProductlineID		+ N'>"' 
				+ @TemplateProductLineNameShort + N'"/ Product <'	
				+ @TemplateProductID			+ N'>"' 
				+ @TemplateProductNameShort		+ N'"' AS Statement
			, @TransactUsername AS UserName
			, N'' AS PCName
			, N'' AS ProcessorCode
			, N'' AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0 AS IsDeleted
			, 0 AS IsResolved;

		-- STEP 3.4 - Update the TemplateName of the Product
		UPDATE dbo.sx_pf_dProducts
		SET   
			Template = @TemplateName
			, TimeType = RIGHT(Template, 2) 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_Template;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryService;
GO
/*
POST Operation to apply a template (again) to many Products
same process logic as POST_Template, but with extension to apply to many products.

Search Conditions:
- empty Factory ID		-> all Factories exept ZT are effected
- empty ProductlineID	-> all Productlines effected
- empty Search Template -> all Products effected

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Testcall - makes a whole Productline to Unikum Month
-- Copy Productline U in ZT to create Productline U2 in TT (can't use Factory ZT !)
DECLARE @RC								INT
DECLARE @Username						NVARCHAR(255) = 'SQL'

DECLARE @ProductLineID					NVARCHAR(255) = 'U2'
DECLARE @FactoryID						NVARCHAR(255) = 'TT'
DECLARE @SearchTemplateName				NVARCHAR(255) = ''

DECLARE @TemplateProductID				NVARCHAR(255) = '1'
DECLARE @TemplateProductLineID			NVARCHAR(255) = 'U' 
DECLARE @TemplateFactoryID				NVARCHAR(255) = 'ZT'

DECLARE @CompareByNameInsteadOfIDFlag	INT			  = 0

EXEC @RC = dbo.sx_pf_POST_Template_All 
	 @Username,
	 @ProductLineID,
	 @FactoryID,
	 @SearchTemplateName,
	 @TemplateProductID,
	 @TemplateProductLineID,
	 @TemplateFactoryID,
	 @CompareByNameInsteadOfIDFlag

PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Template_All') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Template_All
GO

CREATE PROCEDURE dbo.sx_pf_POST_Template_All
	@Username						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@SearchTemplateName				NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN

	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @FactoryKey						INT				= 0;
	DECLARE @ProductLineKey					INT				= 0;
	DECLARE @TemplateFactoryKey				INT				= 0;
	DECLARE @TemplateProductLineKey			INT				= 0;
	DECLARE @TemplateProductKey				INT				= 0;
	DECLARE @TemplateFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateProductLineNameShort	NVARCHAR(255)	= N'';
	DECLARE @TemplateProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateName					NVARCHAR(255)	= N'';
	DECLARE @ProductCount					BIGINT;
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR(MAX)	= N'''' 
															+ ISNULL(@Username,					N'NULL') + N''',''' 
															+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
															+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
															+ ISNULL(@SearchTemplateName,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
															+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
															+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows					INT				= 0;			-- SET during Execution
	DECLARE @ResultCode						INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment						NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @SearchTemplateName				IS NULL SET @SearchTemplateName				= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template_All;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @SearchTemplateName				= dbo.sx_pf_pProtectID	(@SearchTemplateName);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductlineID			= dbo.sx_pf_pProtectID	(@TemplateProductlineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);

		
		-- STEP 0.3. - Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

-- STEP 1.2 - CREATE TODO Liste

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey		= @FactoryKey				AND ProductlineID	= @ProductlineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID			= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey		= @TemplateFactoryKey		AND ProductlineID	= @TemplateProductlineID;
		SELECT @TemplateProductKey		= ProductKey		FROM dbo.sx_pf_dProducts		WHERE ProductlineKey	= @TemplateProductlineKey	AND ProductID		= @TemplateProductID;
		SELECT @TemplateName			= Template			FROM dbo.sx_pf_dProducts dP		WHERE dP.ProductKey		= @TemplateProductKey

		-- STEP 1.3 - Stop if Template is invalid
		IF @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		IF OBJECT_ID('tempdb..#ProductToDoList') IS NOT NULL
		DROP TABLE #ProductToDoList

		CREATE TABLE #ProductToDoList
		(
			 FactoryKey		BIGINT										NOT NULL
			,ProductlineKey BIGINT										NOT NULL
			,ProductKey		BIGINT										NOT NULL
			,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Set all Products of scope on the ToDo List
		INSERT INTO #ProductToDoList
		SELECT 
			dP.FactoryKey
			,dP.ProductLineKey
			,dP.ProductKey
			,dP.FactoryID
			,dP.ProductLineID 
			,dP.ProductID
		FROM dbo.sx_pf_dProducts dP 
		WHERE 
			dP.FactoryID != 'ZT'
			AND (FactoryKey = @FactoryKey OR @FactoryID = '') 
			AND (dP.ProductLineKey = @ProductLineKey OR @ProductLineID = '') 
			AND (dP.Template = @SearchTemplateName OR @SearchTemplateName = '');
			
		
		-- Clean ToDo List from Products where user don't has rights
		DELETE FROM #ProductToDoList 
		WHERE ProductKey IN	
		(
			SELECT ProductKey 
			FROM #ProductToDoList ToDo LEFT JOIN dbo.sx_pf_vUserRights	vUR
				ON ToDo.FactoryID = vUR.FactoryID AND ToDo.ProductLineID = vUR.ProductLineID
			WHERE 
				vUR.[Right] != 'Write' 
				AND vUR.UserName = @TransactUsername
		);

		SELECT @ProductCount = COUNT(ProductKey) FROM #ProductToDoList;

		IF @ProductCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No fitting products found', 16, 10);
		END

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in any product
		IF	@CompareByNameInsteadOfIDFlag = 1 
		AND	EXISTS (SELECT 
						 dVS.ProductKey
						,COUNT(NameShort)
					FROM dbo.sx_pf_dValueSeries dVS 
						-- INNER JOIN to filter on Product on ToDo List
						INNER JOIN #ProductToDoList ToDO
							ON ToDo.ProductKey = dVS.ProductKey
					 GROUP BY dVS.ProductKey, NameShort 
					 HAVING COUNT(NameShort)>1
					)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique', 16, 10);
		END;


		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			 ProductKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,TimeID				INT											NOT NULL
			,ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueInt			BIGINT										NOT NULL
			,ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueComment		NVARCHAR (MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			);

		-- Store Data of Products in Storage (but not formulas, they come from template)
		INSERT INTO #ProductDataStorage    
			SELECT
				 fV.ProductKey 
				,fV.ValueSeriesID
				,dVS.NameShort AS ValueSeriesName
				,dVS.Scale
				,fV.TimeID
				,COALESCE(fVT.ValueFormula, '') AS ValueFormula
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment 
			FROM 
				dbo.sx_pf_fValues fV 
				-- for ValueSeriesNames
				LEFT JOIN dbo.sx_pf_dValueSeries dVS 
					ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
				-- the new Template Formulas
				LEFT JOIN (SELECT ValueSeriesID, TimeID, fV.ValueFormula FROM dbo.sx_pf_fValues fV WHERE fV.ProductKey = @TemplateProductKey) fVT
					ON fV.ValueSeriesID = fVT.ValueSeriesID AND fV.TimeID = fVT.TimeID
				-- filter Products on ToDo List
				INNER JOIN	#ProductToDoList ToDo 
					ON fV.ProductKey = ToDo.ProductKey;
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Products (but not the dProduct, Statements...)
		DELETE FROM dbo.sx_pf_fValues		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dValueSeries	WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_dTime			WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_gProducts		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ProductKey			BIGINT										NOT NULL
			,ProductlineKey		BIGINT										NOT NULL
			,FactoryKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
		);

		-- STEP 3.3 Copy ValueSeries of Template to all Products
		INSERT INTO dbo.sx_pf_dValueSeries 
		OUTPUT INSERTED.ValueSeriesKey, INSERTED.ProductKey, INSERTED.ProductlineKey, INSERTED.FactoryKey,
			INSERTED.ValueSeriesID, INSERTED.ProductID, INSERTED.ProductLineID, INSERTED.FactoryID,
			INSERTED.NameShort, INSERTED.Scale INTO #VSKeys
		SELECT 
			ToDo.ProductKey
			,ToDo.ProductLineKey
			,ToDo.FactoryKey
			,ToDo.ProductID
			,ToDo.ProductLineID
			,ToDo.FactoryID
			,dVS.ValueSeriesID
			,dVS.ValueSeriesNo
			,dVS.NameShort
			,dVS.NameLong
			,dVS.CommentUser
			,dVS.CommentDev
			,dVS.ImageName
			,dVS.IsNumeric
			,dVS.VisibilityLevel
			,dVS.ValueSource
			,dVS.ValueListID
			,dVS.ValueFormatID
			,dVS.Unit
			,dVS.Scale
			,dVS.Effect
			,dVS.EffectParameter
		FROM dbo.sx_pf_dValueSeries dVS
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
		WHERE dVS.ProductKey = @TemplateProductKey;

		-- STEP 3.4 Copy Timeline of Template to Product 
		INSERT INTO dbo.sx_pf_dTime
		SELECT 
			 ToDo.ProductKey
			,ToDo.ProductLineKey
			,ToDo.FactoryKey
			,ToDo.ProductID
			,ToDo.ProductLineID
			,ToDo.FactoryID
			,dT.TimeID
			,dT.FormatID
		FROM dbo.sx_pf_dTime dT-- spread to all products
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
		WHERE dT.ProductKey = @TemplateProductKey;

		-- STEP 3.4 Copy Values of Template to Product 
		INSERT INTO dbo.sx_pf_fValues
		SELECT
			 vk.ValueSeriesKey
			,vk.ProductKey
			,vk.ProductLineKey
			,vk.FactoryKey
			,vk.ProductID
			,vk.ProductLineID
			,vk.FactoryID
			,fV.ValueSeriesID
			,fV.TimeID
			,fV.ValueFormula
			,fV.ValueInt
			,fV.ValueText
			,fV.ValueComment
		FROM dbo.sx_pf_fValues fV LEFT JOIN #VSKeys vk 
			ON	fV.ValueSeriesID = vk.ValueSeriesID
		WHERE fV.ProductKey = @TemplateProductKey;

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO dbo.sx_pf_gProducts
			SELECT
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID

			FROM dbo.sx_pf_gProducts gP 
			-- spread to all products
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
                    
			WHERE gP.ProductKey = @TemplateProductKey

		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled

		-- STEP 3.5 - Update new Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
		BEGIN
			-- Delete Values, which belong to no longer existing IDs
			DELETE FROM #ProductDataStorage 
			WHERE ValueSeriesID NOT IN	(
				SELECT dVS.ValueSeriesID 
				FROM dbo.sx_pf_dValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);	
			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM dbo.sx_pf_dTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE FROM dbo.sx_pf_fValues 
			WHERE ProductKey IN 
				(
					SELECT ProductKey 
					FROM #ProductToDoList
				) 
			AND ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
				(	
					SELECT ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) 
					FROM #ProductDataStorage pds
				);

			-- Insert all other Values
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				 vk.ValueSeriesKey
				,vk.ProductKey
				,vk.ProductLineKey
				,vk.FactoryKey
				,vk.ProductID
				,vk.ProductLineID
				,vk.FactoryID
				,pds.ValueSeriesID
				,pds.TimeID
				,pds.ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
				ON pds.ProductKey = vk.ProductKey AND pds.ValueSeriesID = vk.ValueSeriesID
		END
		ELSE
		BEGIN -- compare by Name

			-- Delete Values, which belong to no longer existing Names
			DELETE FROM #ProductDataStorage
			WHERE ValueSeriesName NOT IN (
				SELECT dVS.NameShort
				FROM dbo.sx_pf_dValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);

			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage 
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM dbo.sx_pf_dTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE fV 
			FROM dbo.sx_pf_fValues fV LEFT JOIN sx_pf_dValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey IN (SELECT ProductKey FROM #ProductToDoList) 
				AND dVS.NameShort + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
					(	
						SELECT pds.ValueSeriesName + '_' + CAST(TimeID AS NVARCHAR(255)) 
						FROM #ProductDataStorage pds
					);

			-- Insert all other Values, if they fit to timeline
			INSERT INTO dbo.sx_pf_fValues
			SELECT 
				 vk.ValueSeriesKey
				,vk.ProductKey
				,vk.ProductLineKey
				,vk.FactoryKey
				,vk.ProductID
				,vk.ProductLineID
				,vk.FactoryID
				,dVS.ValueSeriesID
				,pds.TimeID
				,pds.ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds 
				-- to determine ValueSeries Key
				LEFT JOIN #VSKeys vk 
				ON pds.ProductKey = vk.ProductKey AND pds.ValueSeriesName = vk.ValueSeriesName
				-- to determine new ValueSeriesID
				LEFT JOIN (SELECT dVS.ValueSeriesID,dVS.NameShort FROM dbo.sx_pf_dValueSeries dVS WHERE dVS.ProductKey = @TemplateProductKey) dVS 
				ON dVS.NameShort = pds.ValueSeriesName
			-- filter only fitting TimeIDs
			WHERE 
				pds.TimeID IN (
					SELECT TimeID 
					FROM dbo.sx_pf_dTime dT 
					WHERE dT.ProductKey = @TemplateProductKey
				);
		END;

		-- Create a System Statement
		INSERT INTO dbo.sx_pf_fStatements  
		SELECT  
			 ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductlineID
			,FactoryID
			,N'POST_Template_All' AS ActionType
			,N'Product Template updated with Template Factory <'  
				+ @TemplateFactoryID			+ N'>"' 
				+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
				+ @TemplateProductlineID		+ N'>"' 
				+ @TemplateProductLineNameShort + N'"/ Product <' 
				+ @TemplateProductID			+ N'>"' 
				+ @TemplateProductNameShort		+ '"' AS Statement
			,@TransactUsername AS UserName
			,N'' AS PCName
			,N'' AS ProcessorCode
			,N'' AS IPAddresses
			,CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
			,0 AS IsDeleted
			,0 AS IsResolved
			FROM #ProductToDoList 
			
		-- STEP 3.4 - Update the TemplateName of the Product
		UPDATE dbo.sx_pf_dProducts
		SET  
			 Template = @TemplateName
			,TimeType = RIGHT(@TemplateName, 2) 
		WHERE 
			ProductKey IN (SELECT ProductKey FROM #ProductToDoList);

		SET @EffectedRows += @@ROWCOUNT;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_POST_Template_All;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template_All;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryService;
GO
/*
POST Operation for TimeID
Create new TimeID

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE  @RC			INT
DECLARE  @Username		NVARCHAR(255) = 'SQL'
DECLARE  @ProductID		NVARCHAR(255) = '1'
DECLARE  @ProductLineID NVARCHAR(255) = 'U'
DECLARE  @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE  @TimeID		INT			  = 20150107
DECLARE  @FormatID		NVARCHAR(255) = 'TimeBorderNext'

EXECUTE @RC = dbo.sx_pf_POST_TimeID 
		@Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@TimeID
		,@FormatID

PRINT @RC
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_TimeID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_TimeID;
GO

CREATE PROCEDURE dbo.sx_pf_POST_TimeID
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeID				INT,
	@FormatID			NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@FormatID,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @FormatID		IS NULL SET @FormatID		= '';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_TimeID;

		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM dbo.sx_pf_dProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 2.1 - Delete TimeID
		DELETE FROM dbo.sx_pf_dTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- STEP 3 - Create new TimeID

		INSERT INTO dbo.sx_pf_dTime	(
			ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		)
		VALUES (
			@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@TimeID,
			@FormatID
		);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryService;
GO
/*
POST Operation for the Timeline
The method can be switched between incremental POST and full POST
Values with DeleteFlag will be deleted

03/2018 for DataFactory 4.0

Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
Test call

Test case 1 - Incremental Sended
1. One Value for an existing Value must replace them
2. One Value for an not exiting Value must create it
3. If all Parameter are posted empty, the value is deleted
4. if one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
DECLARE @RC int
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @ProductID					NVARCHAR(255) = '1'
DECLARE @ProductLineID				NVARCHAR(255) = 'U'
DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
DECLARE @IsIncrementalValuesFlag	INT = 0

--3 TimeIDs
DECLARE @TimeIDsInBracketsCommaSeparated nvarchar(max) = '[(20160115,0,0,''FormatID1'')],[(20160215,0,0,''FormatID2'')],[(20160415,0,0,''FormatID3'')]'
EXECUTE @RC = dbo.sx_pf_POST_Timeline
	 @Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@TimeIDsInBracketsCommaSeparated
PRINT @RC

Test case 2 - all values in a Value Series of Products:
Additional 1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
DECLARE @RC int
DECLARE @Username							NVARCHAR(255) = 'SQL'
DECLARE @ProductID							NVARCHAR(255) = '21'
DECLARE @ProductLineID						NVARCHAR(255) = '1'
DECLARE @FactoryID							NVARCHAR(255) = 'U'
DECLARE @IsIncrementalValuesFlag			INT = 0
DECLARE @TimeIDsInBracketsCommaSeparated	NVARCHAR(max) = ''
DECLARE @start_time DATETIME

-- Generate fictitious values for all TimeIDs of Products
SELECT @TimeIDsInBracketsCommaSeparated =  @TimeIDsInBracketsCommaSeparated + '[(''' + dVS2.ValueSeriesID + ''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',1000,''Hase'',''Kommentar'')],' 
FROM sx_pf_fValues fV LEFT JOIN 
	sx_pf_dValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey LEFT JOIN
	(SELECT ValueSeriesID FROM sx_pf_dValueSeries WHERE FactoryID =  @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID GROUP BY ValueSeriesID) dVS2
		ON 1=1 
WHERE fV.FactoryID =  @FactoryID AND fV.ProductLineID = @ProductLineID AND fV.ProductID = @ProductID AND dVS.ValueSeriesNo = 1
SET @TimeIDsInBracketsCommaSeparated = Left(@TimeIDsInBracketsCommaSeparated,Len(@TimeIDsInBracketsCommaSeparated) -1)
PRINT @TimeIDsInBracketsCommaSeparated
SET @start_time = GETDATE()
EXECUTE @RC = sx_pf_POST_ProductDataTableValues
	 @Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@TimeIDsInBracketsCommaSeparated
PRINT @RC
SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETDATE()) AS CHAR(10))) AS 'TimeTaken'
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Timeline') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Timeline;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Timeline
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@TimeIDsInBracketsCommaSeparated	NVARCHAR(MAX)

/*
@TimeIDsInBracketsCommaSeparated format: (TimeID,DeleteFlag,AddValue,FormatID)
if you check this in SQL Server Profiler, every String must have two inverted commas, the whole Parameter must have Single inverted comma
this must show profiler: '[(''TimeID,DeleteFlag,AddValue,''FormatID'')],[(TimeID,DeleteFlag,AddValue,FormatID)]'
if you check this in table sx_pf_API_Log, every String must have one inverted comma, whole String has no inverted comma
this must show Log: [(TimeID,DeleteFlag,AddValue,FormatID)],[(TimeID,DeleteFlag,AddValue,'FormatID')]
*/

AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;

	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductID							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductLineID						, N'NULL')	+ N''',''' 
								+ ISNULL(@FactoryID							, N'NULL')	+ N''',''' 
								+ ISNULL(@TimeIDsInBracketsCommaSeparated	, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username							= N'';
	IF @ProductID						IS NULL SET @ProductID							= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID						= N'';
	IF @FactoryID						IS NULL SET @FactoryID							= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag			= 0;
	IF @TimeIDsInBracketsCommaSeparated IS NULL SET @TimeIDsInBracketsCommaSeparated	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Timeline;
		
		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID (@FactoryID)
		SET @ProductID			= dbo.sx_pf_pProtectID (@ProductID)
		SET @ProductLineID		= dbo.sx_pf_pProtectID (@ProductLineID)

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM dbo.sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM dbo.sx_pf_dProducts		WHERE ProductlineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
				
		-- STEP 3.2 - Check for count of delimiter in Array

		DECLARE @OpenDelimetersCount INT = 0;
		DECLARE @DelimetersIndex INT = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.3 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @TimeIDsInBracketsCommaSeparated = LEFT(@TimeIDsInBracketsCommaSeparated,  LEN(@TimeIDsInBracketsCommaSeparated) - 2);
		SET @TimeIDsInBracketsCommaSeparated = RIGHT(@TimeIDsInBracketsCommaSeparated, LEN(@TimeIDsInBracketsCommaSeparated) - 2);

		DECLARE @tblValues1 TABLE (txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues2 TABLE (TimeID		 NVARCHAR(MAX) NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues3 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues4 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, AddValue		 BIGINT		   NOT NULL, FormatID NVARCHAR(255) NOT NULL);

		-- Split String into Rows
		INSERT INTO @tblValues1 
			SELECT txtValues FROM [dbo].sx_pf_PivotStringIntoTable(@TimeIDsInBracketsCommaSeparated, N')],[(');

		-- Split one column into two Columns (separate TimeID)
		INSERT INTO @tblValues2
			SELECT  
				 LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe)-1 ) AS TimeID
				,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)) AS Rest
			FROM @tblValues1;

		IF EXISTS(SELECT TimeID FROM @tblValues2 GROUP BY TimeID HAVING(COUNT(TimeID) > 1))
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Doubled TimeID in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- Split last columns once more (separate DeleteFlag)
		INSERT INTO @tblValues3
			SELECT  
				 TimeID
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS DeleteFlag
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)-1) AS Rest
			FROM @tblValues2;

		-- Split last columns once more (separate AddValue)
		INSERT INTO @tblValues4
			SELECT  
				 TimeID
				,DeleteFlag
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS AddValue
			,REPLACE(Right(txtValuesPipe, LEN(txtValuesPipe) - 1), N'''','') AS FormatID
			FROM @tblValues3;

		-- STEP 3.4 - Actions #######################################################################################################################
	
		-- Delete TimeIDs which are sended as deleted in fValues and in dTime
		DELETE FROM dbo.sx_pf_dTime 
		WHERE 
			@ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);
	
		SET @EffectedRows = @@ROWCOUNT;

		DELETE FROM dbo.sx_pf_fValues 
		WHERE 
			ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);

		SET @EffectedRows += @@ROWCOUNT;
		-- STEP 3.5 - CASE FULL transmission or Incremental does not matter anymore #########################################
		IF @IsIncrementalValuesFlag != 1 OR  @IsIncrementalValuesFlag = 1 
		BEGIN
			-- Insert all New TimeIDs
			INSERT INTO dbo.sx_pf_dTime
			SELECT  
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				TimeID,
				FormatID
			FROM @tblValues4 
			WHERE TimeID NOT IN (SELECT TimeID FROM sx_pf_dTime WHERE ProductKey = @ProductKey);

			SET @EffectedRows += @@ROWCOUNT;
		END;
			
		COMMIT TRANSACTION sx_pf_POST_Timeline;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Timeline;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryService;
GO
/*
POST Operation for a User
Not existing User will be created, existing one updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @PostUserName		NVARCHAR(255) = 'sx'
DECLARE @PersonSurname		NVARCHAR(255) = ''
DECLARE @PersonFirstName	NVARCHAR(255) = ''
DECLARE @Email				NVARCHAR(255) = ''
DECLARE @LDAPIP				NVARCHAR(255) = ''
DECLARE @Status				NVARCHAR(255) = ''
DECLARE @ImageName			NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_User 
		@Username,
		@PostUserName,
		@PersonSurname,
		@PersonFirstName,
		@Email,
		@LDAPIP,
		@Status

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_User;
GO

CREATE PROCEDURE dbo.sx_pf_POST_User
	@Username			NVARCHAR(255),
	@PostUserName		NVARCHAR(255),
    @PersonSurname		NVARCHAR(255),
    @PersonFirstName	NVARCHAR(255),
    @Email				NVARCHAR(255),
    @LDAPIP				NVARCHAR(255),
    @Status				NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @PostUserKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,			N'NULL') + N''',''' 
								+ ISNULL(@PostUserName,		N'NULL') + N''',''' 
								+ ISNULL(@PersonSurname,	N'NULL') + N''',''' 
								+ ISNULL(@PersonFirstName,	N'NULL') + N''',''' 
								+ ISNULL(@Email,			N'NULL') + N''',''' 
								+ ISNULL(@LDAPIP,			N'NULL') + N''',''' 
								+ ISNULL(@Status,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @PostUserName		IS NULL SET @PostUserName		= N'';
	IF @PersonSurname		IS NULL SET @PersonSurname		= N'';
	IF @PersonFirstName		IS NULL SET @PersonFirstName	= N'';
	IF @Email				IS NULL SET @Email				= N'';
	IF @LDAPIP				IS NULL SET @LDAPIP				= N'';
	IF @Status				IS NULL SET @Status				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_User;

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName		= dbo.sx_pf_pProtectString (@PostUserName);
		SET @PersonSurname		= dbo.sx_pf_pProtectString (@PersonSurname);
		SET @PersonFirstName	= dbo.sx_pf_pProtectString (@PersonFirstName);
		SET @Email				= dbo.sx_pf_pProtectString (@Email);
		SET @LDAPIP				= dbo.sx_pf_pProtectString (@LDAPIP);
		SET @Status				= dbo.sx_pf_pProtectString (@Status);

		IF @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Don`t allow creation or changing of systemuser SQL
		-- the Username "FactoryService" is not allowed, as it would lead to confusion with the similar named db role
		IF @PostUserName = N'SQL' OR @PostUserName = N'FactoryService'	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden creation or changing of systemuser', 16, 10);
		END;
			
		-- STEP 1.2 - Don't allow changes on the members of the DB role pf_PlanningFactoryService (which are not visible in the frontend) - not over the function IS_ROLEMEMBER, its buggy
		IF EXISTS(SELECT r.name 
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r	ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m	ON rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = @PostUserName)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changes on the members of the DB role pf_PlanningFactoryService', 16, 10);
		END;

		-- STEP 1.2 - Don`t allow the user 'public' to change his Rights
		IF LOWER(@UserName) = 'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow the user ''public'' to change his Rights', 16, 10);
		END;

		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to change rights', 16, 10);
		END;
					
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- The Clusteradmin can`t set himself to Inactive
		IF @TransactUsername = @PostUsername AND @Status <> 'Active' 
			SET @Status = 'Active';
		
		-- STEP 2 - Get keys
		SELECT @PostUserKey = UserKey FROM dbo.sx_pf_rUser WHERE Username = @PostUserName;
				
		-- STEP 3.1 - If user does not yet exist, it will be created
		IF @PostUserKey = 0
		BEGIN
			INSERT INTO dbo.sx_pf_rUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status) 
			OUTPUT INSERTED.UserKey INTO @tmpUserTable 
			VALUES (@PostUserName, '', '', '', '', '');

			SET @EffectedRows = 1;
				
			-- the determination on the table variable is necessary because of TRIGGERS on rUser
			SELECT TOP (1) @PostUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
			SET @Resultcode = 201;
		END

		-- STEP 3.2 - Update the User with the commited attributes
		UPDATE dbo.sx_pf_rUser
		SET
			 PersonSurname		= IIF(@PersonSurname	= N'<#NV>', PersonSurname,		@PersonSurname)
			,PersonFirstName	= IIF(@PersonFirstName	= N'<#NV>', PersonFirstName,	@PersonFirstName)
			,Email				= IIF(@Email			= N'<#NV>', Email,				@Email)
			,LDAPIP				= IIF(@LDAPIP			= N'<#NV>', LDAPIP,				@LDAPIP)
			,Status				= IIF(@Status			= N'<#NV>', Status,				@Status)
		WHERE UserKey = @PostUserKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_User TO pf_PlanningFactoryService;
GO
/*
POST Operation for Value
The Value will be deleted and writen new

03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = '1'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ValueSeriesID	NVARCHAR(255) = 'E1'
DECLARE @TimeID			BIGINT		  = 20180315
DECLARE @ValueFormula	NVARCHAR(max) = '<#NV>'
DECLARE @ValueInt		NVARCHAR(255) = '555' 
DECLARE @ValueText		NVARCHAR(max) = 'Hase2'
DECLARE @ValueComment	NVARCHAR(max) = 'Hase2'
DECLARE @i				INT = 0

-- individual values 
-- Text with Comment
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		'E1',
		20180115,
		'',
		0,
		'Hase',
		'Kommentar für Hase';

PRINT @RC

-- Integer with Comment
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		'K1',
		20180215,
		'',
		44,
		'',
		'Kommentar für Hase44';

PRINT @RC

-- 1b. Variant with loop
--WHILE @i < 1000
--BEGIN
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@ValueSeriesID,
		@TimeID,
		@ValueFormula,
		@i,
		@ValueText,
		@ValueComment
--SET @i = @i +1
PRINT @RC
--END

-- 2. Variant with Script Creator
PRINT 'DECLARE @RC as INTEGER = 0'

WHILE @i < 5000
BEGIN
	PRINT 'EXECUTE @RC = dbo.sx_pf_POST_Value ''' + @Username + ''','''+ @ProductID+ ''','''+@ProductLineID+ ''','''+ @FactoryID + ''','''+ @ValueSeriesID + ''','+ CAST(@TimeID AS NVARCHAR (8)) + ','''+ @ValueFormula + ''','+ CAST(@i AS NVARCHAR(10)) + ','''+@ValueText+ ''','''+	@ValueComment + ''''
	PRINT 'PRINT @RC' 
	SET @i = @i +1 
END
--about 12 sec for 5000 Values Values = 400 / second
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Value;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Value
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ValueSeriesID	NVARCHAR(255),		
	@TimeID			INT,
	@ValueFormula	NVARCHAR(MAX),
	@ValueInt		NVARCHAR(255), -- to be able to send <#NV>
	@ValueText		NVARCHAR(MAX),
	@ValueComment	NVARCHAR(MAX)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @ValueKey			INT				= 0;
	DECLARE @ValueSeriesNo		INT				= 0;

	DECLARE @OLDValueFormula	NVARCHAR(255)	= N'';
	DECLARE @OLDValueInt		BIGINT			= 0; 
	DECLARE @OLDValueText		NVARCHAR(255)	= N'';
	DECLARE @OLDValueComment	NVARCHAR(MAX)	= N'';

	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
		+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' 
		+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,		N'NULL') + N''',''' 
		+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL') + N',''' 
		+ ISNULL(@ValueFormula,		N'NULL') + N''',''' + ISNULL(@ValueInt,			N'NULL') + N''',''' 
		+ ISNULL(@ValueText,		N'NULL') + N''',''' + ISNULL(@ValueComment,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @ValueFormula	IS NULL SET @ValueFormula	= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueComment	IS NULL SET @ValueComment	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Value;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @TimeID			= dbo.sx_pf_pProtectID		(@TimeID);
		SET @ValueFormula	= dbo.sx_pf_pProtectString	(@ValueFormula);
		SET @ValueInt		= dbo.sx_pf_pProtectString	(@ValueInt);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N'' OR @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey			FROM dbo.sx_pf_dProducts		WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

 	     -- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.3 - Try to detect and cancel the ValueSeriesKey if does not exist
		SELECT @ValueSeriesKey = ValueSeriesKey, @ValueSeriesNo = ValueSeriesNo FROM dbo.sx_pf_dValueSeries WHERE ProductKey= @ProductKey AND ValueSeriesID = @ValueSeriesID;
		
		IF @ValueSeriesKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2.1 - Cancel if TimeID not exists	
		IF NOT EXISTS(SELECT TimeID FROM dbo.sx_pf_dTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('TimeID not exists', 16, 10);
		END;

		-- STEP 3. Determine ValueKey identify and old values
		SELECT @ValueKey = ValueKey, @OLDValueFormula = ValueFormula, @OLDValueInt = ValueInt, @OLDValueText = ValueText, @OLDValueComment = ValueComment
		FROM dbo.sx_pf_fValues 
		WHERE ValueSeriesKey = @ValueSeriesKey AND TimeID = @TimeID;
				
		-- STEP 3.1 - If Value Key exists delete
		IF  @ValueKey > 0 
		BEGIN
			DELETE FROM dbo.sx_pf_fValues 
			WHERE ValueKey = @ValueKey;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 204; -- Ok, but deleted

			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', @OLDValueFormula,	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', @OLDValueText,		@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', @OLDValueComment,	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', @OLDValueInt,		@ValueInt);
		END
		ELSE
		BEGIN
			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', N'',	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', N'',	@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', N'',	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', N'0',	@ValueInt);
		END;

		-- STEP 3.2 - Insert, while no empty values outside of column 1
		IF @ValueFormula = N'' AND @ValueText = N'' AND @ValueInt = N'0'
		BEGIN 
			SET @ResultCode = 204; --> discarded !!! 
		END
		ELSE
		BEGIN 
			SET @ValueInt = COALESCE(TRY_CAST(@ValueInt AS BIGINT), 0);

			INSERT INTO dbo.sx_pf_fValues
			VALUES (
				@ValueSeriesKey, 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID, 
				@FactoryID, 
				@ValueSeriesID,  
				@TimeID,  
				@ValueFormula, 
				@ValueInt, 
				@ValueText, 
				@ValueComment);
			
			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 201;
		END;

		COMMIT TRANSACTION sx_pf_POST_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryService;
GO
/*
POST Operation for one ValueSeries
Operation will create a ValueSeries if it not exist
It will change an existing ValueSeries
Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries


03/2018 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) ='SQL'
DECLARE @ProductID					NVARCHAR(255) ='1'
DECLARE @ProductLineID				NVARCHAR(255) ='U'
DECLARE @FactoryID					NVARCHAR(255) ='ZT'
DECLARE @ValueSeriesID				NVARCHAR(255) ='38550'
DECLARE @RequestedValueSeriesNo		INT			  = 249
DECLARE @NameShort					NVARCHAR(255) ='<#NV>'
DECLARE @NameLong					NVARCHAR(255) ='<#NV>'
DECLARE @CommentUser				NVARCHAR(255) ='<#NV>'
DECLARE @CommentDev					NVARCHAR(255) ='<#NV>'
DECLARE @ImageName					NVARCHAR(255) ='<#NV>'
DECLARE @IsNumeric					NVARCHAR(255) ='1'
DECLARE @VisibilityLevel			NVARCHAR(255) ='<#NV>'
DECLARE @ValueSource				NVARCHAR(255) ='<#NV>'
DECLARE @ValueListID				NVARCHAR(255) ='<#NV>'
DECLARE @ValueFormatID				NVARCHAR(255) ='<#NV>'
DECLARE @Unit						NVARCHAR(255) ='<#NV>'
DECLARE @Scale						NVARCHAR(255) ='100'
DECLARE @Effect						NVARCHAR(255) ='<#NV>'
DECLARE @EffectParameter			NVARCHAR(255) ='<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ValueSerie 
		@Username,	
		@ProductID,	
		@ProductLineID,	
		@FactoryID,	
		@ValueSeriesID,	
		@RequestedValueSeriesNo,
		@NameShort,	
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ImageName,
		@IsNumeric,
		@VisibilityLevel,
		@ValueSource,
		@ValueListID,
		@ValueFormatID,
		@Unit,
		@Scale,
		@Effect,
		@EffectParameter

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ValueSerie;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ValueSerie
	@Username				 NVARCHAR(255),
	@ProductID				 NVARCHAR(255),
	@ProductLineID			 NVARCHAR(255),
	@FactoryID				 NVARCHAR(255),
	@ValueSeriesID			 NVARCHAR(255),
	@RequestedValueSeriesNo	 INT,
	@NameShort				 NVARCHAR(255),
	@NameLong				 NVARCHAR(255),
	@CommentUser			 NVARCHAR(255),
	@CommentDev				 NVARCHAR(255),
	@ImageName				 NVARCHAR(255),
	@IsNumeric				 NVARCHAR(255),
	@VisibilityLevel		 NVARCHAR(255),
	@ValueSource			 NVARCHAR(255),
	@ValueListID			 NVARCHAR(255),
	@ValueFormatID			 NVARCHAR(255),
	@Unit					 NVARCHAR(255),
	@Scale					 NVARCHAR(255),
	@Effect					 NVARCHAR(255),
	@EffectParameter		 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @OldValueSeriesNo	INT				= 0;
	DECLARE @OldIsNumeric		INT				= 0;
	DECLARE @OldScale			INT				= 0;
	DECLARE @OldValueSource		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,										N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,										N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@RequestedValueSeriesNo AS NVARCHAR(255)),	N'NULL') + N',''' 
			+ ISNULL(@NameShort,		N'NULL') + N''',''' + ISNULL(@NameLong,											N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,		N'NULL') + N''',''' + ISNULL(@CommentDev,										N'NULL') + N''',''' 
			+ ISNULL(@ImageName,		N'NULL') + N''',''' + ISNULL(@IsNumeric,										N'NULL') + N''',''' 
			+ ISNULL(@VisibilityLevel,	N'NULL') + N''',''' + ISNULL(@ValueSource,										N'NULL') + N''',''' 
			+ ISNULL(@ValueListID,		N'NULL') + N''',''' + ISNULL(@ValueFormatID,									N'NULL') + N''',''' 
			+ ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@Scale,											N'NULL') + N''',''' 
			+ ISNULL(@Effect,			N'NULL') + N''',''' + ISNULL(@EffectParameter,									N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductID				IS NULL SET @ProductID					= N'NULL';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'NULL';
	IF @FactoryID				IS NULL SET @FactoryID					= N'NULL';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID				= N'NULL';
	IF @RequestedValueSeriesNo	IS NULL SET @RequestedValueSeriesNo		= 0;
	IF @NameShort				IS NULL SET @NameShort					= N'NULL';
	IF @NameLong				IS NULL SET @NameLong					= N'NULL';
	IF @CommentUser				IS NULL SET @CommentUser				= N'NULL';
	IF @CommentDev				IS NULL SET @CommentDev					= N'NULL';
	IF @ImageName				IS NULL SET @ImageName					= N'NULL';
	IF @IsNumeric				IS NULL SET @IsNumeric					= N'NULL';
	IF @VisibilityLevel			IS NULL SET @VisibilityLevel			= N'NULL';
	IF @ValueSource				IS NULL SET @ValueSource				= N'NULL';
	IF @ValueListID				IS NULL SET @ValueListID				= N'NULL';
	IF @ValueFormatID			IS NULL SET @ValueFormatID				= N'NULL';
	IF @Unit					IS NULL SET @Unit						= N'NULL';
	IF @Scale					IS NULL SET @Scale						= N'NULL';
	IF @Effect					IS NULL SET @Effect						= N'NULL';
	IF @EffectParameter			IS NULL SET @EffectParameter			= N'NULL';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ValueSerie;

		-- STEP 0.1 - NULL Protection
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ValueSeriesID		= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @IsNumeric			= dbo.sx_pf_pProtectBoolean (@IsNumeric);
		SET @VisibilityLevel	= dbo.sx_pf_pProtectString	(@VisibilityLevel);
		SET @ValueSource		= dbo.sx_pf_pProtectString	(@ValueSource);
		SET @Unit				= dbo.sx_pf_pProtectString	(@Unit);
		SET @Scale				= dbo.sx_pf_pProtectInt		(@Scale);
		SET @Effect				= dbo.sx_pf_pProtectString	(@Effect);
		SET @EffectParameter	= dbo.sx_pf_pProtectString	(@EffectParameter);
		IF @ValueListID		<> N'<#NV>' SET @ValueListID	= dbo.sx_pf_pProtectID (@ValueListID);
		IF @ValueFormatID	<> N'<#NV>' SET @ValueFormatID	= dbo.sx_pf_pProtectID (@ValueFormatID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys	
		SELECT @FactoryKey = FactoryKey			FROM dbo.sx_pf_dFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM dbo.sx_pf_dProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey			FROM dbo.sx_pf_dProducts		WHERE ProductLineKey = @ProductLinekey AND ProductID= @ProductID;
		SELECT 
				 @ValueSeriesKey = ValueSeriesKey
				,@OldIsNumeric = IsNumeric
				,@OldScale = Scale
				,@OldValueSource = ValueSource 
		FROM dbo.sx_pf_dValueSeries 
		WHERE 
				ProductKey = @ProductKey 
			AND ValueSeriesID = @ValueSeriesID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - Insert the ValueSeries, if Key does not exist
		IF @ValueSeriesKey = 0
		BEGIN
			-- INCREMENT all following Requested ValueSeriesNumbers based on requested No
			UPDATE dbo.sx_pf_dValueSeries
			SET ValueSeriesNo = ValueSeriesNo + 1 
			WHERE 
				ProductKey = @ProductKey 
				AND ValueSeriesNo >= @RequestedValueSeriesNo;

			IF @VisibilityLevel = '<#NV>' SET @VisibilityLevel = 1;
			
			SET @EffectedRows = @@ROWCOUNT;

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			-- Insert with requested number
			INSERT INTO dbo.sx_pf_dValueSeries (
				ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,ValueSeriesID
				,ValueSeriesNo
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,IsNumeric
				,VisibilityLevel
				,ValueSource
				,ValueListID
				,ValueFormatID
				,Unit
				,Scale
				,Effect
				,EffectParameter) 
			OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
			VALUES ( 
				@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter);
			
			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ValueSeriesKey = [Key] FROM @Keys ORDER BY [Key];
		END
		ELSE
		BEGIN 
		-- STEP 3.2 - Update ValueSeries if it already exists
			SELECT @OldValueSeriesNo = ValueSeriesNo FROM dbo.sx_pf_dValueSeries WHERE ValueSeriesKey = @ValueSeriesKey;
									
			-- Use the old one, if no SeriesNo is commited (lower than one means not commited)
			IF @RequestedValueSeriesNo < 1 
				SET @RequestedValueSeriesNo = @OldValueSeriesNo;
			
			-- Adjust Numbers if not equal
			IF @OldValueSeriesNo < @RequestedValueSeriesNo
			BEGIN 
				-- decrement all between actual and requsted Position
				UPDATE dbo.sx_pf_dValueSeries 
				SET ValueSeriesNo = ValueSeriesNo - 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo > @OldValueSeriesNo 
					AND ValueSeriesNo <= @RequestedValueSeriesNo;
														
				SET @EffectedRows += @@ROWCOUNT;
			END
			ELSE IF @OldValueSeriesNo > @RequestedValueSeriesNo
			BEGIN 
				-- increment all between actual and requsted Position
				UPDATE dbo.sx_pf_dValueSeries 
				SET ValueSeriesNo = ValueSeriesNo + 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo < @OldValueSeriesNo 
					AND ValueSeriesNo >= @RequestedValueSeriesNo;

				SET @EffectedRows += @@ROWCOUNT;
			END;

			UPDATE dbo.sx_pf_dValueSeries 
			SET 
				ValueSeriesNo		= @RequestedValueSeriesNo
				, NameShort			= IIF(@NameShort		= N'<#NV>', NameShort,			@NameShort)
				, NameLong			= IIF(@NameLong			= N'<#NV>', NameLong,			@NameLong)
				, CommentUser		= IIF(@CommentUser		= N'<#NV>', CommentUser,		@CommentUser)
				, CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,			@CommentDev)
				, ImageName			= IIF(@ImageName		= N'<#NV>', ImageName,			@ImageName)
				, IsNumeric			= IIF(@IsNumeric		= N'<#NV>', IsNumeric,			@IsNumeric)
				, VisibilityLevel	= IIF(@VisibilityLevel	= N'<#NV>', VisibilityLevel,	@VisibilityLevel)
				, ValueSource		= IIF(@ValueSource		= N'<#NV>', ValueSource,		@ValueSource)
				, ValueListID		= IIF(@ValueListID		= N'<#NV>', ValueListID,		@ValueListID)
				, ValueFormatID		= IIF(@ValueFormatID	= N'<#NV>', ValueFormatID,		@ValueFormatID)
				, Unit				= IIF(@Unit				= N'<#NV>', Unit,				@Unit)
				, Scale				= IIF(@Scale			= N'<#NV>', Scale,				@Scale)
				, Effect			= IIF(@Effect			= N'<#NV>', Effect,				@Effect)
				, EffectParameter	= IIF(@EffectParameter	= N'<#NV>', EffectParameter,	@EffectParameter)
			FROM dbo.sx_pf_dValueSeries
			WHERE ValueSeriesKey = @ValueSeriesKey;
				
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.2a - Delete zero values if ValueSeries is now numeric input series
			IF @IsNumeric = '1' AND @ValueSource = N'Input'
			BEGIN
				DELETE FROM dbo.sx_pf_fValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueInt = 0; 
			END;

			-- Step 3.2b - Delete empty Strings if ValueSeries is now non-numeric input series
			IF @IsNumeric = '0' AND @ValueSource = N'Input' 
			BEGIN
				DELETE FROM dbo.sx_pf_fValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueText = N'';
			END;

				
			-- STEP 3.3 - Update exists Values due to possible rescaling or changing type from Numeric to Non-Numeric
			DECLARE @vals AS TABLE (ValueKey INT NOT NULL, ValueInt INT NOT NULL, ValueText NVARCHAR(MAX) NOT NULL);

			-- Changing type Numeric / Non Numeric
			IF (@IsNumeric <> N'<#NV>' AND @OldIsNumeric <> @IsNumeric)
			BEGIN
				-- Copy ValueText to ValueInt, after Try_Cast to Int to get only valid numbers and multiplied by OldScale, as the int values use scale, while the text value not
				IF @IsNumeric = N'1'
				BEGIN							 
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ISNULL(TRY_CAST(ValueText AS INT), 0) * @OldScale, ''
					FROM dbo.sx_pf_fValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					-- If Numeric is made to non numeric the value in the Database must be be divided by scale and converted to text, as text dont uses scale
					IF (@OldScale <> 0)
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt / @OldScale AS NVARCHAR(MAX))
						FROM dbo.sx_pf_fValues
						WHERE ValueSeriesKey = @ValueSeriesKey;	
					END
					ELSE
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt AS NVARCHAR(MAX))
						FROM dbo.sx_pf_fValues
						WHERE ValueSeriesKey = @ValueSeriesKey; 
					END;
				END;
			END;

			-- changing scaling
			-- If Isnumeric = 0 and not changed Nothing happens with the value, if scale is changed, as text values dont use the scale property.
			IF (@Scale <> N'<#NV>' AND @OldScale <> @Scale) AND (@IsNumeric = N'1' OR (@IsNumeric = N'<#NV>' AND @OldIsNumeric = 1))
			BEGIN
				-- Change Scale of existing values if scale is changed (when scale is changed, e.g. from 100 to 1000 the value in the database must be multiplied by 10)
				DECLARE @k FLOAT = @Scale / CAST(@OldScale AS FLOAT);	
			
				IF NOT EXISTS(SELECT 1 FROM @vals)
				BEGIN
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ValueInt * @k, '' 
					FROM dbo.sx_pf_fValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					UPDATE @vals
					SET ValueInt = ValueInt * @k; 
				END;
			END;
			
			IF EXISTS(SELECT 1 FROM @vals)
			BEGIN
				UPDATE v
				SET 
					 v.ValueInt		= newV.ValueInt
					,v.ValueText	= newV.ValueText 
				FROM dbo.sx_pf_fValues v INNER JOIN @vals newV 
					ON v.ValueKey = newV.ValueKey
				WHERE v.ValueSeriesKey = @ValueSeriesKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;

		SET @Resultcode = 201;
		COMMIT TRANSACTION sx_pf_POST_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryService;
GO
/*
MOVE rows older than 7 days from [dbo].[sx_pf_API_Log] to [dbo].[sx_pf_API_Log_History] table

Dependencies: No															

12/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return No Value

Test call
EXEC  [dbo].[sx_pf_pPOST_LogArchive]
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pPOST_Archive_Historic_LogEntrys]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [dbo].[sx_pf_pPOST_Archive_Historic_LogEntrys];
GO

CREATE PROCEDURE [dbo].[sx_pf_pPOST_Archive_Historic_LogEntrys]
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @logKeys AS TABLE ([LogKey] [bigint] NOT NULL);
	DECLARE @Date DATETIME = GETDATE();

	INSERT TOP (500) INTO [dbo].[sx_pf_API_Log_History] (
		[LogKey]
        , [SQLSystemUser]
        , [UserName]
        , [TransactUsername]
        , [ProcedureName]
        , [ParameterString]
        , [EffectedRows]
        , [ReturnCode]
        , [TimestampCall]
        , [TimestampReturn]
        , [Comment]
        , [ProcessCode] 
	)
	OUTPUT inserted.[LogKey] INTO @logKeys([LogKey])
   	SELECT [LogKey]
        , [SQLSystemUser]
        , [UserName]
        , [TransactUsername]
        , [ProcedureName]
        , [ParameterString]
        , [EffectedRows]
        , [ReturnCode]
        , [TimestampCall]
        , [TimestampReturn]
        , [Comment]
        , [ProcessCode]
	FROM [dbo].[sx_pf_API_Log]
	WHERE DATEDIFF(DAY, [TimestampCall], @Date) > 7
	ORDER BY [LogKey]; 
											 
    WHILE @@ROWCOUNT > 0
	BEGIN
		DELETE FROM [dbo].[sx_pf_API_Log] WHERE [LogKey] IN (SELECT [LogKey] FROM @logKeys);
		DELETE FROM @logKeys;

		INSERT TOP (500) INTO [dbo].[sx_pf_API_Log_History] (
			[LogKey]
			, [SQLSystemUser]
			, [UserName]
			, [TransactUsername]
			, [ProcedureName]
			, [ParameterString]
			, [EffectedRows]
			, [ReturnCode]
			, [TimestampCall]
			, [TimestampReturn]
			, [Comment]
			, [ProcessCode] 
		)
		OUTPUT inserted.[LogKey] INTO @logKeys([LogKey])
   		SELECT [LogKey]
			, [SQLSystemUser]
			, [UserName]
			, [TransactUsername]
			, [ProcedureName]
			, [ParameterString]
			, [EffectedRows]
			, [ReturnCode]
			, [TimestampCall]
			, [TimestampReturn]
			, [Comment]
			, [ProcessCode]
		FROM [dbo].[sx_pf_API_Log]
		WHERE DATEDIFF(DAY, [TimestampCall], @Date) > 7
		ORDER BY [LogKey];

	END;
END
GO

/*
Internal POST Operation for one API LogEntry, always caused, when a public procedure is called
Dependencies:
	- none
10/2015 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
ReturnCode according to http standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pPOST_API_LogEntry]') AND type in (N'P', N'PC'))
DROP PROCEDURE sx_pf_pPOST_API_LogEntry
GO

CREATE PROCEDURE sx_pf_pPOST_API_LogEntry 
	 @Username AS NVARCHAR(255)
	,@TransactUsername AS NVARCHAR(255)
	,@ProcedureName AS NVARCHAR(255)
	,@ParameterString AS NVARCHAR (MAX)
	,@EffectedRows AS INT
	,@ReturnCode AS INT
	,@TimestampCall AS Datetime
	,@Comment AS NVARCHAR (2000)

AS

		DECLARE @TimestampReturn AS Datetime = CURRENT_TIMESTAMP
		DECLARE @ProcessCode AS NVARCHAR(255) = ''
		DECLARE @DatabaseIsReadOnly AS INTEGER
	
		SELECT @DatabaseIsReadOnly = is_read_only FROM sys.databases WHERE Name = DB_NAME()
		
		IF NOT @DatabaseIsReadOnly = 1
			BEGIN
				INSERT INTO sx_pf_API_Log VALUES (SYSTEM_USER,@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
			END
GO

/*Testaufruf
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pPOST_API_LogEntry 'SQL','W8\admin','TestProcedure','ThisAreTheParameter',1,200,'20150731','Comment'
PRINT @RESULT
*/
/*
Procedure to POST an Process Log Entry
Gerd Tautenhahn 
07/2018

DataFactory 4.0

Sample

-- Start Log Entry
EXEC dbo.sx_pf_pPOST_Process_LogEntry 'Load', 'BusinessUnits', 'Create Factories', 'Start',200

-- anything happens now....


-- End Log Entry
EXEC dbo.sx_pf_pPOST_Process_LogEntry 'Load', 'BusinessUnits', 'Create Factories', 'End',200

SELECT *  FROM dbo.sx_pf_Process_Log
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pPOST_Process_LogEntry') AND type in (N'P', N'PC'))
DROP PROCEDURE dbo.sx_pf_pPOST_Process_LogEntry
GO

CREATE PROCEDURE dbo.sx_pf_pPOST_Process_LogEntry 
						 @MainProcess		NVARCHAR (255)
						,@SubProcess		NVARCHAR (255)
						,@Step				NVARCHAR (255)
						,@Start_End_Info	NVARCHAR (255)
						,@ResultCode		INT				= 0
						,@ValueText			NVARCHAR (MAX)	= ''
						,@ValueInt			BIGINT			= 0
AS 
	
	BEGIN

		INSERT INTO dbo.sx_pf_Process_Log VALUES
				 (@MainProcess, @SubProcess, @Step, @Start_End_Info, CURRENT_TIMESTAMP, @ResultCode,@ValueText,@ValueInt)
	END

GO

-- No GRANT as only used inside other stored procedures
/*
Procedure to calculate the Rights Credits of an Cluster

- Every User with Cluserwrite Rights gets 2 C Points
- Every User with Clusterread Rights gets 1 C Points
- Every User with an Write Right on Factory Level gets 2 F Points per Factory
- Every User with an Read Right on Factory Level gets 1 Point per Factory
- Every User with an Write Right on an Productline gets 2 PL Points
- Every User with an Read Rights on an Productline gets 1 PL Point 

Its used to monitor Point count and see unusual values due to failures in Right import

Testcall
EXEC control.pGET_RightPointSummary

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.pGET_RightPointSummary') AND type in (N'P', N'PC'))
DROP PROCEDURE control.pGET_RightPointSummary
GO

CREATE PROCEDURE control.pGET_RightPointSummary

AS

SELECT
	 
	 Points.UserName
	,SUM(Points.C_Points) AS C_Points
	,SUM(Points.F_Points) AS F_Points
	,SUM(Points.PL_Points) AS PL_Points

	,SUM(Points.C_Points
		+Points.F_Points
		+Points.PL_Points) AS Full_Points

	,SUM( SUM(Points.C_Points )
		+ SUM(Points.F_Points )
		+ SUM(Points.PL_Points)) OVER () AS TotalPoints
FROM 
	(
		SELECT 
			 vUR.UserName
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS C_Points
			,0 AS F_Points
			,0 AS PL_Points
	
		FROM dbo.sx_pf_vUserRights vUR

		WHERE vUR.FactoryID = '' AND vUR.ProductLineID = ''

	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS F_Points
			,0 AS PL_Points
	
		FROM dbo.sx_pf_vUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID = ''


	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,0 AS F_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS PL_Points
	
		FROM dbo.sx_pf_vUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID <> ''

	) Points

GROUP BY Points.UserName


GO
/*
Determines the User for the Transaction

This function returns:
	A: An active username from the internal DataFactory Usermanagement
	B: The string '403' if no authorized user was found

The function does it by
	1. checking the SQL connected User
	2. checking for FactoryService Action
	3. checking for action over public user 

Don't use the function IS_ROLEMEMBER, its buggy

02/2017 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
NO HTTP Standard - no public call

Testcall
DECLARE @Result NVARCHAR(255)
EXEC @Result = sx_pf_Determine_TransactionUsername 'SQL'
PRINT @Result

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_Determine_TransactionUsername') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION dbo.sx_pf_Determine_TransactionUsername;
GO

CREATE FUNCTION dbo.sx_pf_Determine_TransactionUsername (@Username AS NVARCHAR(255)='')
RETURNS NVARCHAR (255) 
AS
BEGIN 
	DECLARE @TransactionUsername AS NVARCHAR (255) = '403'; --Default Value

	-- clean and declare
		SET @Username = LTRIM(RTRIM(@Username));
		DECLARE @IsFactoryService INT = 0;
		DECLARE @UserBlocked INT = 0;

	-- 1. Determine if Factory Service is acting (this means a DatabaseUser with the role "pf_PlanningFactoryService")
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;

	-- 2. If FactoryService is acting, every Username is accepted, else only the Original_Login() - which must be clearly requested by Keyword 'SQL'
		IF  @IsFactoryService = 1 
			SET @TransactionUsername = @Username;
		ELSE
			BEGIN
				IF @Username = 'SQL' SET @TransactionUsername = ORIGINAL_LOGIN()
					ELSE SET @TransactionUsername = '403'
			END

	-- 3. Block the user if this user don't exists a an active User
		IF NOT (SELECT Count(Userkey) FROM sx_pf_rUser WHERE Username = @TransactionUsername AND Status = 'Active') = 1
			BEGIN
				SET @TransactionUsername = '403'
				SET	@UserBlocked = 1
			END
	
	-- 4. If the user was blocked, check if a public user exists, which can be used for the transaction
		IF @UserBlocked = 1
			BEGIN
				IF (SELECT Count(Userkey) FROM sx_pf_rUser WHERE Username = 'public' AND Status = 'Active') = 1
					SET @TransactionUsername = 'public'
			END

	RETURN @TransactionUsername
END

GO

/*Testing
Assumtion:
- Database has three SQL User ('W10\admin', 'FactoryService' and 'sx')
- DFU has user 'W10\admin', 'sx' and user 'public'
- User public is Active\Inactive depending of test
- User 'W10\admin' has role db_owner
- User sx has role pf_PlanningFactoryUser
- User FactoryService has role pf_PlanningFactoryService



A: Test as User with role pf_PlanningFactoryUser (SQL User sx)
	A1: User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'sx'

	A2: User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'sx'

	A3: User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return '403'

	A4: User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'public'

B: Test as User with role db_owner (SQL User W10\admin)
	B1: User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'W10\admin'
																					
	B2: User exists in DFU, a public user exists									
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')		-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'W10\admin'

	B3: User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return '403'

	B4: User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'public'


C: Test as User with role pf_PlanningFactoryService (SQL User 'FactoryService') - the User FactoryService don't exits in DFU
	C1: Requested User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'

	C2: Requested User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'

	C3: Requested User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return '403'

	C4: Requested User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'public'

D: Test as User with role pf_PlanningFactoryService (SQL User 'FactoryService') - the User FactoryService exits in DFU
	D1: Requested User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'FactoryService'

	D2: Requested User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'FactoryService'

	D3: Requested User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'

	D4: Requested User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'


*/
/*
Procedure for materialization of UserRights
Gerd Tautenhahn for saxess software gmbh
PlanningFactory 4.0 
02/2018

Testcall
EXEC dbo.sx_pf_materialize_vUserRights
SELECT * FROM dbo.sx_pf_vUserRights order by Username, FactoryID, ProductlineID
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_materialize_vUserRights]') AND type in (N'P', N'PC'))
DROP PROCEDURE sx_pf_materialize_vUserRights

GO

CREATE PROCEDURE sx_pf_materialize_vUserRights 
AS 

BEGIN
	BEGIN TRY
		BEGIN TRANSACTION materialize_vUserRights


		-- Table to store the calculated Rights
		IF OBJECT_ID('tempdb..#RightsTemp') IS NOT NULL
		DROP TABLE #RightsTemp;
		
		CREATE TABLE #RightsTemp 
		(
			 Source					NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL							
			,UserKey				BIGINT									NOT NULL
			,Username				NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey				BIGINT									NOT NULL
			,ProductlineKey			BIGINT									NOT NULL
			,FactoryID				NVARCHAR (255) COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID			NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightExplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightImplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ReadCommentMandatory	INT 									NOT NULL
			,WriteCommentMandatory	INT										NOT NULL
		);

		-- Concept for saving only positive Rights, no Denys

		-- ClusterLevel			********************************************************************************************************************
		-- this are Rights for FactoryID = '' AND ProductLineID = ''

		-- Clusteradmins		*****************************
		-- Has Write Rights on Clusterlevel, no limitation possible
		-- Insert Write Rights on ClusterLevel
		-- Insert Write Rights on FactoryLevel
		-- Insert Write Rights on Productline Level

		-- ClusterReader		******************************
		-- Has Read Rights on Clusterlevel, may have additional Write Rights on Factory / Productline Level
		-- Insert Read Rights on ClusterLevel
		-- Insert Read Rights on FactoryLevel
		-- Insert Read Rights on Productline Level

		-- FactoryLevel			********************************************************************************************************************
		-- Rights defined on FactoryLevel with FactoryID <> 0 AND ProductlineID = ''
		-- Write Rights on Factories 
		-- Has Write Rights on a Factory - no limitation under this Factory possible
		-- Read Rights on Factories
		-- Has Read Rights on Factory Level, may have additional Write Rights on ProductlineLevel

		-- ProductlineLevel		*************************************************************************
		-- Rights defined on ProductlineLevel with FactoryID <> 0 AND ProductlineID <> 0
		-- Write Rights on Productlines
		-- Read Rights on Productlines

		-- Preparation			**************************************************************************
		-- Create Table with all rows, this are:
			-- for User with Rights on ClusterLevel		Linecount = 1 + FactoryCount				+ PLCount 
			-- for User with Rights on FactoryLevel		Linecount = 1 + FactoryWithRightCount		+ PLCount of FactoriesWith Rights
			-- for User with Rights on ProductlineLevel Linecount = 1 + FactoryCountPLWithRights	+ PLCount with Rights

		-- Helper Table with Stucture   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
		DROP TABLE #FullStructure;
				CREATE TABLE #FullStructure
						(
							 RowKey			BIGINT	IDENTITY (1,1)								NOT NULL
							,[Level]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryKey		BIGINT												NOT NULL
							,ProductLineKey BIGINT												NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductlineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
						);

				-- C Level
				INSERT INTO #FullStructure
					SELECT 
						 'Cluster'	AS Level
						,0			AS FactoryKey
						,0			AS ProductLineKey
						,''			AS FactoryID
						,''			ASProductLineID;
			 
				-- F Level
				INSERT INTO #FullStructure
					SELECT
						 'Factory'		AS Level
						,dF.FactoryKey
						,0				AS ProductLineKey
						,dF.FactoryID
						,''				AS ProductLineID
					FROM dbo.sx_pf_dFactories dF;
				
				-- PL Level
				INSERT INTO #FullStructure
					SELECT
						 'Productline'	AS Level
						,dF.FactoryKey
						,dPL.ProductLineKey
						,dF.FactoryID
						,dPL.ProductLineID
					FROM dbo.sx_pf_dProductLines dPL 
						LEFT JOIN dbo.sx_pf_dFactories dF
							ON dPL.FactoryKey = dF.FactoryKey;

		-- SELECT * FROM #FullStructure

		-- Fill UserRight Table with fitting RowCount and set all to read ************************************************************************

		-- 1a. The Full Structure Rows for User with Clusterrights  -> no additional Rows for Rights on Factory / Productline Level need
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ClusterRight' AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1				AS ReadCommentMandatory
					,1				AS WriteCommentMandatory
			FROM #FullStructure fs 
				LEFT JOIN dbo.sx_pf_rUser rU
					ON 1=1
			WHERE 
					rU.Status = 'Active'
				-- Clusterrights
				AND rU.UserKey IN (SELECT UserKey FROM dbo.sx_pf_rRights WHERE FactoryID = '' AND ProductLineID = '');

		-- 1b. Cluster Read right Row for Transfer for alle user with any right except the already done users with clusterrights
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-TransferCluser'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'				AS [RightExplicit]
					,'Read'				AS [RightImplizit]
					,1					AS ReadCommentMandatory
					,1					AS WriteCommentMandatory
			FROM #FullStructure fs
				
				LEFT JOIN dbo.sx_pf_rUser rU
					ON 1=1

			WHERE 
					rU.Status = 'Active'
				AND fs.Level = 'Cluster'
				-- anybody who has any set right 
				AND rU.UserKey IN (SELECT DISTINCT UserKey FROM dbo.sx_pf_rRights)
				-- and is not already ClusterRight Owner
				AND rU.Userkey NOT IN (SELECT DISTINCT UserKey FROM dbo.sx_pf_rRights WHERE FactoryID = '');
			
		-- 1c. Rows for User with direct Factory Rights 
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-FactoryRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID FROM dbo.sx_pf_rRights WHERE FactoryID <> '' AND ProductLineID = ''
					) rFactoryDirect
					
					ON FS.FactoryID = rFactoryDirect.FactoryID COLLATE DATABASE_DEFAULT
				
				INNER JOIN dbo.sx_pf_rUser rU
					ON rFactoryDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					AND rFactoryDirect.FactoryID IS NOT NULL
					-- and is not already ClusterRight Owner
					AND rU.UserKey NOT IN (SELECT DISTINCT UserKey FROM dbo.sx_pf_rRights WHERE FactoryID = '');

		--1d. Rows for Factory Transfer Rights for User with direct ProduclineRights Rights (but not already direct Factory / Clusterrights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-Transfer Factory'	AS [Source]
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM dbo.sx_pf_rRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID  COLLATE DATABASE_DEFAULT-- Join only over Factory
				
				INNER JOIN dbo.sx_pf_rUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status	= 'Active'
					AND fs.Level	= 'Factory'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM dbo.sx_pf_rRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM dbo.sx_pf_rRights WHERE FactoryID = '' )

			GROUP BY 
                 rU.UserKey
				,rU.UserName
				,fs.FactoryKey
				,fs.ProductLineKey
				,fs.FactoryID
				,fs.ProductlineID;

		--1e. Rows Rights for User with direct ProduclineRights Rights (which not already have this due to FactoryRights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ProductlineRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM dbo.sx_pf_rRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID COLLATE DATABASE_DEFAULT
						AND FS.ProductlineID = rPLDirect.ProductLineID COLLATE DATABASE_DEFAULT
				
				INNER JOIN dbo.sx_pf_rUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM dbo.sx_pf_rRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM dbo.sx_pf_rRights WHERE FactoryID = '' );

			--SELECT * FROM #RightsTemp rt ORDER BY Username,FactoryID,rt.ProductLineID;

	

			--2. Evaluate Rights to give write Rights and set Commentproperties#########################################################################

			-- 2a. SET Clusterrights
			UPDATE #RightsTemp 
		
			SET	 [Source]				= 'ClusterRight'		
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM dbo.sx_pf_rRights 
										WHERE	
												FactoryID = '' 
											AND ProductLineID = ''
									) rR

						ON rt.UserKey = rR.UserKey
					;

			-- 2a. Set Factory Rights with downwards inheritance
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'FactoryRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM dbo.sx_pf_rRights 
										WHERE	
												FactoryID <> '' 
											AND ProductLineID = ''
									) rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT;

			-- 2a. Set Productline Rights 
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'ProductlineRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM dbo.sx_pf_rRights 
										WHERE	
												FactoryID <> '' 
											AND ProductLineID <> ''
										) rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
							AND rt.ProductLineID = rR.ProductLineID COLLATE DATABASE_DEFAULT;
					
		
			--SELECT * FROM	#RightsTemp ORDER BY Username,FactoryID,ProductLineID


			-- OUTPUT
			-- TRUNCATE TABLE sx_pf_vUserRights -- deactivated, because a 
			/*	The minimum permission required is ALTER on table_name. TRUNCATE TABLE permissions default to the table owner,
				members of the sysadmin fixed server role, and the db_owner and db_ddladmin fixed database roles, 
				and are not transferable. However, you can incorporate the TRUNCATE TABLE statement within a module,
				such as a stored procedure, and grant appropriate permissions to the module using the EXECUTE AS clause.
			
				The EXECUTE AS Clause I did not master till now in foreign environments. 
			 */
			DELETE FROM sx_pf_vUserRights; 

			INSERT INTO sx_pf_vUserRights 
			SELECT 
				 UserKey
				,Username
				,FactoryID
				,ProductLineID
				,IIF(RightExplicit='-',RightImplicit,RightExplicit) AS [Right]
				,ReadCommentMandatory
				,WriteCommentMandatory

			FROM #RightsTemp

			COMMIT TRANSACTION materialize_vUserRights
		END TRY
		BEGIN CATCH
			Print 'Da ging was schief'
			ROLLBACK TRANSACTION materialize_vUserRights
		END CATCH
	END
GO

/*
GET Operation für Read Rechte auf die Clusterproperties
Benötigt schon Transaktionsuser, daher PRIVATE !
Gibt bei positivem Test auf Read Rechte für den angefragten Transaktionsuser Status 200 zurück
Abhängigkeiten:
	- keine
05/2015 für PlanningFactory 4.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ClusterPropertiesReadRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ClusterPropertiesReadRight]
GO

CREATE PROCEDURE sx_pf_pGET_ClusterPropertiesReadRight @TransactionUsername AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 2. testen ob der User auf Active gesetzt ist, jeder User darf die Properties lesen
		SELECT @RightCode=IIF ([Status] = 'Active',0, 401)FROM sx_pf_rUser WHERE Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200
GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_ClusterPropertiesReadRight 'W8\admin'
		PRINT @RESULT
		*/

/*
GET Operation for Write Rights on Cluster
Needs already a Transactionsuser, thats why PRIVATE
Returns for Clusteradmins 200
Dependencies:
	- none
08/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ClusterWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ClusterWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_ClusterWriteRight @TransactionUsername AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --not implemented, must be never returned
		DECLARE @Right AS NVARCHAR (255) ='none'

		-- Check for Write Line on cluster
		SELECT @Right=Coalesce([Right],'no') FROM sx_pf_vUserRights WHERE FactoryID = '' AND Username = @TransactionUsername
			-- return 200 only for Clusteradmin
			IF @Right <>'Write' SET @RightCode = 401 
			IF @Right = 'Write' SET @RightCode = 200
			RETURN @RightCode
GO

/*Testcall

-- Clusteradmin must return 200
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'W8\admin'
PRINT @RESULT

-- User which exists, but is no Cluster Admin must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'GUEBAU\wenzel'
PRINT @RESULT

-- User which not exists, must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'Dummy\Dummy'
PRINT @RESULT
*/




/*
GET Operation für Read Rechte auf einer Factory
Benötigt schon Transaktionsuser, daher PRIVATE !
Gibt bei positivem Test auf Read Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
07/2015 für PlanningFactory 4.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_FactoryReadRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_FactoryReadRight]
GO

CREATE PROCEDURE sx_pf_pGET_FactoryReadRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Factory besitzt
		SELECT @RightCode=IIF ([Right] = 'Write' OR [Right] = 'Read',0, 401)FROM sx_pf_vUserRights WHERE FactoryID = @FactoryID AND ProductlineID = '' AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200
GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_FactoryReadRight 'W8\admin','ZT'
		PRINT @RESULT
		*/

/*
GET Operation für Write Rechte auf einer Factory
Benötigt schon Transaktionsuser
Gibt bei positivem Test auf Write Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
04/2015 für PlanningFactory 4.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_FactoryWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_FactoryWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_FactoryWriteRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Factory besitzt
		SELECT @RightCode=IIF ([Right] = 'Write',0, 401)FROM sx_pf_vUserRights WHERE FactoryID = @FactoryID AND ProductlineID = '' AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200

GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_FactoryWriteRight 'W8\admin','ZT'
		PRINT @RESULT
		*/

/*
GET Operation für Read Rechte auf einer Productline
Benötigt schon Transaktionsuser
Gibt bei positivem Test auf Read Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
04/2015 für PlanningFactory 4.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ProductLineReadRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ProductLineReadRight]
GO

CREATE PROCEDURE sx_pf_pGET_ProductLineReadRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255), @ProductlineID AS NVARCHAR(255)

AS

		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Productline besitzt
		SELECT @RightCode=IIF ([Right] = 'Write' OR [Right] = 'Read',0, 401)FROM sx_pf_vUserRights WHERE FactoryID = @FactoryID AND ProductlineID = @ProductLineID AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200

GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_ProductLineReadRight 'W8\adminr','ZT','ZT'
		PRINT @RESULT
		*/

/*
GET Operation for Write Rights on Productline
Needs already a Transactionsuser, thats why PRIVATE
Returns for User with Write Rights 200
Dependencies:
	- none
08/2016 for PlanningFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ProductLineWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ProductLineWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_ProductLineWriteRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255), @ProductlineID AS NVARCHAR(255)

AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Productline besitzt
		SELECT @Right=Coalesce([Right],'none')FROM sx_pf_vUserRights WHERE FactoryID = @FactoryID AND ProductlineID = @ProductLineID AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @Right = 'Write' SET @RightCode = 200
			IF @Right <>'Write' SET @RightCode = 401 
			RETURN @RightCode

GO

/*Testcall
-- Clusteradmin must return 200
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'W8\admin','ZT','U'
PRINT @RESULT


-- User which exists, but is no Cluster Admin must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'GUEBAU\wenzel','ZT','U'
PRINT @RESULT


-- User which not exists, must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'Dummy\Dummy','ZT','U'
PRINT @RESULT
*/
/*
    Product: OCT 5.0
    Object: system.SetSetting
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 18.07.2019 14:14
*/

DROP PROCEDURE IF EXISTS system.SetSetting;
GO

CREATE PROCEDURE system.SetSetting
	@SettingKey NVARCHAR(255),
	@Value NVARCHAR(255)
AS
BEGIN
	IF EXISTS(SELECT 1 FROM system.tSettings WHERE SettingKey = @SettingKey)
		UPDATE system.tSettings
		SET Value = @Value
		WHERE SettingKey = @SettingKey
	ELSE
		INSERT INTO system.tSettings (SettingKey, Value)
		VALUES (@SettingKey, @Value)
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Procedure',
    @level1name = N'SetSetting'
GO


/*
Trigger um Userrechte zu materialisieren
Werden ausgelöst, sobald in sx_pf_rRights oder sx_pf_rUser Update, Insert oder Delete Operationen erfolgen
Gerd Tautenhahn für saxess-software gmbh
PlanningFactory Version 3.0.7
06/2015
*/

-- für sx_pf_rRights Aktionen
IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[mqt_vUserRights_from_rRights]'))
DROP TRIGGER [mqt_vUserRights_from_rRights]
GO

CREATE TRIGGER mqt_vUserRights_from_rRights

ON sx_pf_rRights 

AFTER INSERT, UPDATE, DELETE 

AS 

BEGIN
	EXEC sx_pf_materialize_vUserRights
END
GO



-- für sx_pf_rUser Aktionen
IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[mqt_vUserRights_from_rUser]'))
DROP TRIGGER [mqt_vUserRights_from_rUser]
GO

CREATE TRIGGER mqt_vUserRights_from_rUser

ON sx_pf_rUser

AFTER INSERT, UPDATE, DELETE 

AS 
BEGIN
	EXEC sx_pf_materialize_vUserRights
END

GO
