
-- Script for creating a OCT datamodel in an existing database
-- Gerd Tautenhahn for saxess-software gmbh 09/2020
-- Version OCT 5.2.15
-- Coding UTF-8 without BOM 

SET QUOTED_IDENTIFIER ON
GO



-- Check for Version SQL 2016 or newer
-- Azure uses different Version number and is not checked

DECLARE @Versioninfo NVARCHAR(255) = ''
SELECT @Versioninfo = 

	CASE 
		WHEN CONVERT(NVARCHAR(255),SERVERPROPERTY ('Edition')) LIKE '%Azure%' THEN  'Azure Database - please keep / set compatibility level of database to highest version.'
		WHEN LEFT(CONVERT(NVARCHAR(255),SERVERPROPERTY ('productversion')),2) < 13 THEN  'SQL SERVER VERSION ALT !!!!! - Versionen vor SQL 2016 werden nicht unterstützt '
		WHEN LEFT(CONVERT(NVARCHAR(255),SERVERPROPERTY ('productversion')),2) > = 13 THEN  'SQL SERVER VERSION in Ordnung - Version 2016+'
		ELSE 'Fehler bei Versionbestimmung SQL Server'
	END
PRINT @Versioninfo


-- Check for correct compatibilty Level
SELECT @Versioninfo = 

	CASE 
		WHEN compatibility_level > = 130 THEN  'Datenbank Kompatibilität in Ordnung, Version ' + CAST(compatibility_level AS NVARCHAR(255))
		WHEN compatibility_level < 130 THEN  'KOMPATIBILITÄTSLEVEL ZU KLEIN !!!!! - Muss auf 130 oder höher für SQL 2016 Funktionsset geändert werden.'
		ELSE 'Fehler bei Kompatibilitätsbestimmung für  SQL Server'
	END

FROM sys.databases WHERE name = DB_NAME();

PRINT @Versioninfo

/*
    Product: OCT 5.0
    Object: agent
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('agent') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA agent AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'agent'
END

GRANT EXECUTE ON SCHEMA ::agent TO pf_PlanningFactoryService;
GRANT SELECT ON SCHEMA::agent TO pf_PlanningFactoryService;
GRANT ALTER ON SCHEMA::agent TO pf_PlanningFactoryService;

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent'
GO

/*
    Product: OCT 5.0
    Object: calc
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('calc') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA calc AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'calc'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'calc'
GO

/*
    Product: OCT 5.0
    Object: control
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('control') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA control AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'control'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'control'
GO

/*
    Product: OCT 5.0
    Object: global
    Type: Schema
    Description: Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('global') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA global AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.',
        @level0type = N'Schema',
        @level0name = N'global'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store global informations. Merged from integration and planning, the treasure vault for OCT.',
    @level0type = N'Schema',
    @level0name = N'global'
GO


-- don't grant ALTER schema, user with ALTER Rights can access over owner chaning also other schema
GRANT SELECT, UPDATE, INSERT, DELETE ON SCHEMA :: global TO pf_PlanningFactoryService;

/*
    Product: OCT 5.0
    Object: import
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('import') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA import AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'import'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'import'
GO

/*
    Product: OCT 5.0
    Object: integration
    Type: Schema
    Description: Schema to store integration data. Temporary and no enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('integration') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA integration AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store integration data. Temporary and no enduser access.',
        @level0type = N'Schema',
        @level0name = N'integration'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store integration data. Temporary and no enduser access.',
    @level0type = N'Schema',
    @level0name = N'integration'
GO

-- don't grant ALTER schema, user with ALTER Rights can access over owner chaning also other schema
GRANT SELECT, UPDATE, INSERT, DELETE ON SCHEMA :: integration TO pf_PlanningFactoryService;
/*
    Product: OCT 5.0
    Object: load
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('load') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA load AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'load'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'load'
GO

/*
    Product: OCT 5.0
    Object: param
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('param') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA param AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'param'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'param'
GO

-- GRANT TABLE on PARAM Schema to autocreate Param Tables on saving
GRANT CREATE TABLE TO pf_PlanningFactoryUser;
GRANT CREATE TABLE TO pf_PlanningFactoryService;

--For Param Tables
GRANT ALTER ON SCHEMA :: param TO pf_PlanningFactoryService;
GRANT INSERT ON SCHEMA :: param TO pf_PlanningFactoryService;
GRANT SELECT ON SCHEMA :: param TO pf_PlanningFactoryService;

-- This temporary for XLS Client
GRANT ALTER ON SCHEMA :: param TO pf_PlanningFactoryUser; 
GRANT INSERT ON SCHEMA :: param TO pf_PlanningFactoryUser;
GRANT SELECT ON SCHEMA :: param TO pf_PlanningFactoryUser;

/*
    Product: OCT 5.0
    Object: planning
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('planning') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA planning AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'planning'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'planning'
GO

/*
    Product: OCT 5.0
    Object: result
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('result') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA result AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'result'
END

GRANT ALTER ON SCHEMA::result TO pf_PlanningFactoryService;

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'result'
GO

/*
    Product: OCT 5.0
    Object: staging
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('staging') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA staging AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'staging'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'staging'
GO

/*
    Product: OCT 5.0
    Object: sxdf
    Type: Schema
    Author: saxess-software GmbH
    Script Date: 15.07.2019 15:12
*/

IF schema_id('sxdf') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA sxdf AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'',
        @level0type = N'Schema',
        @level0name = N'sxdf'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'sxdf'
GO

/*
    Product: OCT 5.0
    Object: system
    Type: Schema
    Description: Schema to store system data and Metadata. No enduser access.
    Author: saxess-software GmbH
    Script Date: 05.07.2019 16:28
*/

IF schema_id('system') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA system AUTHORIZATION dbo'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store system data and Metadata. No enduser access.',
        @level0type = N'Schema',
        @level0name = N'system'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store system data and Metadata. No enduser access.',
    @level0type = N'Schema',
    @level0name = N'system'
GO

-- don't grant ALTER schema, user with ALTER Rights can access over owner chaning also other schema
GRANT SELECT, UPDATE, INSERT, DELETE ON SCHEMA :: system TO pf_PlanningFactoryService;

/*
Creates Database Roles for PlanningFactory
DataFactory 5.0
07/2016
*/

IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryUser') IS NULL CREATE ROLE pf_PlanningFactoryUser
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryService') IS NULL CREATE ROLE pf_PlanningFactoryService
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryAnalyst') IS NULL CREATE ROLE pf_PlanningFactoryAnalyst
GO

GRANT CREATE VIEW TO pf_PlanningFactoryService;
GRANT CREATE TABLE TO pf_PlanningFactoryService;
GRANT VIEW DEFINITION TO pf_PlanningFactoryService;
	-- create no longer existing Function system.GetSetting for all Versions where it don't exist anymore
	-- thats from 5.2.5 the case
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'system.GetSetting'))

		BEGIN EXEC ('
		CREATE FUNCTION system.GetSetting (
				@PropertyID NVARCHAR(50)
				) 
				RETURNS INT
			AS
			BEGIN
				DECLARE @Version INT = 50205
				RETURN @Version
			END')
		END
	

DECLARE @ThisDBVersion		int = 0;
DECLARE @CurrentDBVersion	int;

-- Update DB from 5.0.0 to 5.0.1
BEGIN
	SET @ThisDBVersion = '501'

	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		-- new default content for global.tCodes
		INSERT INTO global.tCodes (CODE, Language, CODEDescription) VALUES
		('SKTO', 'de', 'Sachkonto')
		,('SKTO', 'en', 'GLAccount')
		,('KOART', 'de', 'Kostenart')
		,('KOART', 'en', 'CostType')
		,('PERSKTO', 'de', 'Personenkonto')
		,('PERSKTO', 'en', 'Person holding account')
		,('BILANZ', 'de', 'Bilanz')
		,('BILANZ', 'en', 'Balance')
		,('GUV', 'de', 'GuV')
		,('GUV', 'en', 'PnL')
		,('DEBITOR', 'de', 'Debitor')
		,('DEBITOR', 'en', 'Customer')
		,('KREDITOR', 'de', 'Kreditor')
		,('KREDITOR', 'en', 'Supplier')
		,('STAT', 'de', 'Statistik')
		,('STAT', 'en', 'Statistic')
		,('KST', 'de', 'Kostenstelle')
		,('KST', 'en', 'Cost center')
		,('KTR', 'de', 'Kostentraeger')
		,('KTR', 'en', 'Cost unit')
		,('PC', 'de', 'Profit center')
		,('PC', 'en', 'Profit center')
		,('OR', 'de', 'Auftrag')
		,('OR', 'en', 'Order')
		,('FIBU', 'de', 'Finanzbuchhaltung')
		,('FIBU', 'en', 'Financial accounting')
		,('FIBU_ABSCHL_HGB', 'de', 'Finanzbuchhaltung nur Abschlussbuchungen HGB')
		,('FIBU_ABSCHL_HGB', 'en', 'Financial accounting only closing entries HGB')
		,('FIBU_ABSCHL_IFRS', 'de', 'Finanzbuchhaltung nur Abschlussbuchungen IFRS')
		,('FIBU_ABSCHL_IFRS', 'en', 'Financial accounting only closing entries IFRS')
		,('FIBU_ABSCHL_STEUER', 'de', 'Finanzbuchhaltung nur Abschlussbuchungen Steuerbilanz')
		,('FIBU_ABSCHL_STEUER', 'en', 'Financial accounting only closing entries tax balance')
		,('KORE', 'de', 'Kostenrechnung')
		,('KORE', 'en', 'Cost accounting')
		,('PCA', 'de', 'Profitcenterrechnung')
		,('PCA', 'en', 'Profit center accounting')
		,('PCAILV', 'de', 'Profitcenterrechnung - interne Leistungsverrechnung (ILV)')
		,('PCAILV', 'en', 'Profit center accounting - internal cost allocation');
		-- Create planning schema
		IF schema_id('planning') IS NULL
			BEGIN
				EXEC sp_executesql N'CREATE SCHEMA planning AUTHORIZATION dbo'

				-- Documentation
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'',
					@level0type = N'Schema',
					@level0name = N'planning'
			END

		-- Rename planning tables with schema transfer
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_API_Log;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_API_Log_History;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_dFactories;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_dProductLines;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_dProducts;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_dTime;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_dValueSeries;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_fStatements;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_fValues;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gCluster;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gFactories;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gHTMLPages;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gProductLines;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gProducts;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_gValueEffects;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_hFormats;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_hLists;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_hListValues;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_Process_Log;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_rPreferences;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_rRights;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_rUser;
		ALTER SCHEMA planning TRANSFER dbo.sx_pf_vUserRights;

		EXEC sp_rename 'planning.sx_pf_API_Log'			,'API_Log'			;
		EXEC sp_rename 'planning.sx_pf_API_Log_History'	,'API_Log_History'	;
		EXEC sp_rename 'planning.sx_pf_dFactories'		,'dFactories'		;
		EXEC sp_rename 'planning.sx_pf_dProductLines'	,'dProductLines'	;
		EXEC sp_rename 'planning.sx_pf_dProducts'		,'dProducts'		;
		EXEC sp_rename 'planning.sx_pf_dTime'			,'dTime'			;
		EXEC sp_rename 'planning.sx_pf_dValueSeries'	,'dValueSeries'		;
		EXEC sp_rename 'planning.sx_pf_fStatements'		,'fStatements'		;
		EXEC sp_rename 'planning.sx_pf_fValues'			,'fValues'			;
		EXEC sp_rename 'planning.sx_pf_gCluster'		,'gCluster'			;
		EXEC sp_rename 'planning.sx_pf_gFactories'		,'gFactories'		;
		EXEC sp_rename 'planning.sx_pf_gHTMLPages'		,'gHTMLPages'		;
		EXEC sp_rename 'planning.sx_pf_gProductLines'	,'gProductLines'	;
		EXEC sp_rename 'planning.sx_pf_gProducts'		,'gProducts'		;
		EXEC sp_rename 'planning.sx_pf_gValueEffects'	,'gValueEffects'	;
		EXEC sp_rename 'planning.sx_pf_hFormats'		,'hFormats'			;
		EXEC sp_rename 'planning.sx_pf_hLists'			,'hLists'			;
		EXEC sp_rename 'planning.sx_pf_hListValues'		,'hListValues'		;
		EXEC sp_rename 'planning.sx_pf_Process_Log'		,'Process_Log'		;
		EXEC sp_rename 'planning.sx_pf_rPreferences'	,'rPreferences'		;
		EXEC sp_rename 'planning.sx_pf_rRights'			,'rRights'			;
		EXEC sp_rename 'planning.sx_pf_rUser'			,'rUser'			;
		EXEC sp_rename 'planning.sx_pf_vUserRights'		,'rUserRights'		;

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.1 to 5.0.2
BEGIN
	SET @ThisDBVersion = 502
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		
		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.2 to 5.0.3
BEGIN
	SET @ThisDBVersion = 503
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		/*
    Product: OCT 5.0
    Object: global.tProcessCompanies
    Type: Table
    Description: Table to store selected companies for process configuration.
    Author: saxess-software GmbH
    Script Date: 29.08.2019 11:23
*/

DROP TABLE IF EXISTS global.tProcessCompanies;

CREATE TABLE global.tProcessCompanies (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
	ProcessConfigurationKey  INT                          NOT NULL,     -- Foreign key of tProcessConfigurations
	CompanyKey               INT                          NOT NULL,     -- Foreign key of tCompanies.
    CONSTRAINT PK_global_tProcessCompanies PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store selected companies for process configuration.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessCompanies'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessCompanies',
    @level2type = N'Column',
    @level2name = N'RowKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Foreign key of tCompanies.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessCompanies',
    @level2type = N'Column',
    @level2name = N'CompanyKey'
/*
    Product: OCT 5.0
    Object: global.tProcessConfigurations
    Type: Table
    Description: Table to store process configurations.
    Author: saxess-software GmbH
    Script Date: 29.08.2019 11:23
*/

DROP TABLE IF EXISTS global.tProcessConfigurations;

CREATE TABLE global.tProcessConfigurations (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
	ProcessPipelineKey       INT                          NOT NULL,     -- Foreign key of tProcessPipelines
    ProcessConfigurationName NVARCHAR(255)                NOT NULL,     -- Name of the ProcessConfiguration.
	DataSourceKey            INT                          NOT NULL,     -- Foreign key of tDataSources.
	OnlyMasterData           BIT                          NOT NULL,     -- Should only master data be exported from data source?
	PeriodStart              NVARCHAR(7)                  NOT NULL,     -- Financial period to start with the export.
	PeriodEnd                NVARCHAR(7)                  NOT NULL,     -- Financial period to end with the export.
	OrderOfExecution         INT                          NOT NULL,     -- Order of execution in process pipeline.
    CONSTRAINT PK_global_tProcessConfigurations PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store process configurations.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'RowKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Foreign key of tProcessPipelines.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'ProcessPipelineKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the ProcessConfiguration.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'ProcessConfigurationName'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Foreign key of tDataSources.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'DataSourceKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Should only master data be exported from data source?',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'OnlyMasterData'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Financial period to start with the export.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'PeriodStart'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Financial period to end with the export.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessConfigurations',
    @level2type = N'Column',
    @level2name = N'PeriodEnd'
/*
    Product: OCT 5.0
    Object: global.tProcessPipelines
    Type: Table
    Description: Table to store process pipelines.
    Author: saxess-software GmbH
    Script Date: 29.08.2019 11:13
*/

DROP TABLE IF EXISTS global.tProcessPipelines;

CREATE TABLE global.tProcessPipelines (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    ProcessPipelineName      NVARCHAR(255)                NOT NULL,     -- Name of the ProcessPipeline.
    CONSTRAINT PK_global_tProcessPipelines PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store process pipelines.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessPipelines'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessPipelines',
    @level2type = N'Column',
    @level2name = N'RowKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Name of the ProcessPipeline.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tProcessPipelines',
    @level2type = N'Column',
    @level2name = N'ProcessPipelineName'
/*
    Product: OCT 5.0
    Object: global.tSolutionsDataSource
    Type: Table
    Description: Table to store relation between datasources and solutions.
    Author: saxess-software GmbH
    Script Date: 16.08.2019 15:28
*/

DROP TABLE IF EXISTS global.tSolutions;

CREATE TABLE global.tSolutions (
    RowKey                   INT        IDENTITY(1,1)     NOT NULL,
    DataSourceKey            INT                          NOT NULL,
    SolutionCODE             NVARCHAR(50)                 NOT NULL,
    CONSTRAINT PK_global_tSolutions PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Table to store relation between datasources and solutions.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tSolutions'

-- GRANT Rigts on schema global
GRANT SELECT, UPDATE, INSERT, DELETE ON SCHEMA :: global TO pf_PlanningFactoryService;


		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.3 to 5.0.4
BEGIN
	SET @ThisDBVersion = 504
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

/*
    Product: OCT 5.0
    Object: .tJobStatus
    Type: Table
    Author: saxess-software GmbH
    Script Date: 23.09.2019 14:28
*/

DROP TABLE IF EXISTS global.tJobStatus;

CREATE TABLE global.tJobStatus (
    RowKey                     INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
    KettleJobId                NVARCHAR(50)                 NOT NULL,     -- Stores the kettle job id that is returned after job registration.
    JobStatusCODE              NVARCHAR(50)                 NOT NULL,     -- Stores the Job status returned by Kettle, e.g. Running, Finished.
    JobLog                     NVARCHAR(MAX)                NOT NULL,     -- Stores the current job log returned by kettle.
    StartTime                  DATETIME                     NOT NULL,     -- Stores the date and time when the job started.
    LastUpdated                DATETIME                     NOT NULL,     -- Stores the date and time when this entry was last updated.
    ExecutingUser              NVARCHAR(50)                 NOT NULL,     -- Stores the user name of the user that started the job.
    ProcessPipelineKey         INT                          NOT NULL,     -- Stores the primary key of the process pipeline that this job executes.
    ExecutionModeCODE          NVARCHAR(20)                 NOT NULL,     -- Stores the execution mode, either manual or automatic.
    JsonExecutionParameter     NVARCHAR(MAX)                NOT NULL,     -- Stores the JSON parameter that was sent to Kettle to execute this job.
    CONSTRAINT PK_global_tJobStatus PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Technical primary Key for row identity.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'RowKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the kettle job id that is returned after job registration.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'KettleJobId'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the Job status returned by Kettle, e.g. Running, Finished.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'JobStatusCODE'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the current job log returned by kettle.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'JobLog'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the date and time when the job started.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'StartTime'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the date and time when this entry was last updated.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'LastUpdated'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the user name of the user that started the job.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'ExecutingUser'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the primary key of the process pipeline that this job executes.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'ProcessPipelineKey'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the execution mode, either manual or automatic.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'ExecutionModeCODE'

EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Stores the JSON parameter that was sent to Kettle to execute this job.',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tJobStatus',
    @level2type = N'Column',
    @level2name = N'JsonExecutionParameter'

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.5 to 5.0.5
BEGIN
	SET @ThisDBVersion = 505
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		-- Rename Properties for Cluster Logo and Cluster HTML Page
			UPDATE planning.gHTMLPages SET PageID = 'Cluster_HTMLEditorPage', PageName = 'Cluster_HTMLEditorPage' WHERE PageID LIKE '%Page' AND FactoryKey = -1
			UPDATE planning.gHTMLPages SET PageID = 'Cluster_Logo', PageName = 'Cluster_Logo' WHERE PageID LIKE '%Logo' AND FactoryKey = -1

			--  nur eines Property bei Doubletten erhalten
			DELETE FROM planning.gHTMLPages WHERE PageID = 'Cluster_HTMLEditorPage' AND PageKey NOT IN
				(SELECT TOP 1 PageKey FROM planning.gHTMLPages WHERE PageID = 'Cluster_HTMLEditorPage' ORDER BY PageKey DESC)

			DELETE FROM planning.gHTMLPages WHERE PageID = 'Cluster_Logo' AND PageKey NOT IN 
				(SELECT TOP 1 PageKey FROM planning.gHTMLPages WHERE PageID = 'Cluster_Logo' ORDER BY PageKey DESC)

		-- Tabellen in anderes Schema schieben
		ALTER SCHEMA system TRANSFER global.tProcessCompanies;
		ALTER SCHEMA system TRANSFER global.tProcessConfigurations;
		ALTER SCHEMA system TRANSFER global.tProcessPipelines;
		
		-- alte Tabelle global.tJobStatus l�schen

		DROP TABLE IF EXISTS global.tJobStatus;
		
		-- neue Tabelle system.tPipelineStatus
		
		DROP TABLE IF EXISTS system.tPipelineStatus;

		CREATE TABLE system.tPipelineStatus (
			RowKey                     INT        IDENTITY(1,1)		NOT NULL,     -- Technical primary Key for row identity.
			KettleJobId                NVARCHAR(50)                 NOT NULL,     -- Stores the kettle job id that is returned after job registration.
			KettleStatusCODE           NVARCHAR(50)                 NULL,         -- Stores the Job status returned by Kettle, e.g. Running, Finished.
			DeleteStatusCODE           NVARCHAR(50)                 NULL,         -- Stores the status of the deletion process of the integration data.
			BulkLoadStatusCODE         NVARCHAR(50)                 NULL,         -- Stores the status of the bulk loading of the integration data.
			KettleJobLog               NVARCHAR(MAX)                NOT NULL,     -- Stores the current job log returned by kettle.
			StartTime                  DATETIME                     NOT NULL,     -- Stores the date and time when the job started.
			LastUpdated                DATETIME                     NOT NULL,     -- Stores the date and time when this entry was last updated.
			ExecutingUser              NVARCHAR(50)                 NOT NULL,     -- Stores the user name of the user that started the job.
			ProcessPipelineKey         INT                          NOT NULL,     -- Stores the primary key of the process pipeline that this job executes.
			ExecutionModeCODE          NVARCHAR(20)					NOT NULL,     -- Stores the execution mode, either manual or automatic.
			JsonExecutionParameter     NVARCHAR(MAX)                NOT NULL,     -- Stores the JSON parameter that was sent to Kettle to execute this job.
			CONSTRAINT PK_system_tPipelineStatus PRIMARY KEY NONCLUSTERED (RowKey)
		);

		-- Documentation
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'RowKey'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the kettle job id that is returned after job registration.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'KettleJobId'
	

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the Job status returned by Kettle, e.g. Running, Finished.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'KettleStatusCODE'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the status of the deletion process of the integration data.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'DeleteStatusCODE'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the status of the bulk loading of the integration data.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'BulkLoadStatusCODE'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the current job log returned by kettle.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'KettleJobLog'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the date and time when the job started.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'StartTime'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the date and time when this entry was last updated.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'LastUpdated'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the user name of the user that started the job.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'ExecutingUser'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the primary key of the process pipeline that this job executes.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'ProcessPipelineKey'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the execution mode, either manual or automatic.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'ExecutionModeCODE'
		

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the JSON parameter that was sent to Kettle to execute this job.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'JsonExecutionParameter'


		-- Neue Tabelle tProcessSolutions

		DROP TABLE IF EXISTS system.tProcessSolutions;

		CREATE TABLE system.tProcessSolutions (
			RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
			ProcessConfigurationKey  INT                          NOT NULL,     -- Foreign key of tProcessConfigurations
			SolutionKey              INT                          NOT NULL,     -- Foreign key of tSolutions.
			CONSTRAINT PK_system_tProcessSolutions PRIMARY KEY NONCLUSTERED (RowKey)
		);

		-- Documentation
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store selected solutions for process configuration.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSolutions'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSolutions',
			@level2type = N'Column',
			@level2name = N'RowKey'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Foreign key of tSolutions.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSolutions',
			@level2type = N'Column',
			@level2name = N'SolutionKey';
		
		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.5 to 5.0.6
BEGIN
	SET @ThisDBVersion = 506
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		ALTER TABLE system.tPipelineStatus ADD AgentStatusCODE VARCHAR(50) NULL;
		
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Stores the status of the agent execution for the integration data.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tPipelineStatus',
			@level2type = N'Column',
			@level2name = N'AgentStatusCODE';
		
		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.6 to 5.0.7
BEGIN
	SET @ThisDBVersion = 507
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

		-- new table for jobs
		DROP TABLE IF EXISTS system.tJobs;

		CREATE TABLE system.tJobs
			(
			RowKey		INT IDENTITY(1,1)	NOT NULL,
			PipelineID	INT					NOT NULL,
			Schedule	NVARCHAR(1000)		NOT NULL,
			Active		BIT					NOT NULL,
			CONSTRAINT PK_tJobs PRIMARY KEY NONCLUSTERED (RowKey)
			);

		ALTER TABLE system.tJobs ADD  CONSTRAINT DF_tScheduler_Active  DEFAULT ((1)) FOR Active

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores the generated primary key of the table.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tJobs'
			,@level2type=N'COLUMN'
			,@level2name=N'RowKey'

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores the pipeline that should be executed with this scheduler job.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tJobs'
			,@level2type=N'COLUMN'
			,@level2name=N'PipelineID'

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores the rhythm which determines when and how often a pipeline should be executed.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tJobs'
			,@level2type=N'COLUMN'
			,@level2name=N'Schedule'

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores if the scheduled job is currently active.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tJobs'
			,@level2type=N'COLUMN'
			,@level2name=N'Active'

		-- New Column in tPipelineStatus to save Deltaload Flag
		ALTER TABLE system.tProcessConfigurations ADD DeltaLoadEnabled BIT NOT NULL;

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores if delta load should be applied on this data source.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessConfigurations'
			,@level2type=N'COLUMN'
			,@level2name=N'DeltaLoadEnabled'
		
		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.7 to 5.0.8
BEGIN
	SET @ThisDBVersion = 508
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

		-- adds in all planning tables the "t" according to naming convention
		EXEC sp_rename 'planning.API_Log'			,'tAPI_Log'			;
		EXEC sp_rename 'planning.API_Log_History'	,'tAPI_Log_History'	;
		EXEC sp_rename 'planning.dFactories'		,'tdFactories'		;
		EXEC sp_rename 'planning.dProductLines'		,'tdProductLines'	;
		EXEC sp_rename 'planning.dProducts'			,'tdProducts'		;
		EXEC sp_rename 'planning.dTime'				,'tdTime'			;
		EXEC sp_rename 'planning.dValueSeries'		,'tdValueSeries'	;
		EXEC sp_rename 'planning.fStatements'		,'tfStatements'		;
		EXEC sp_rename 'planning.fValues'			,'tfValues'			;
		EXEC sp_rename 'planning.gCluster'			,'tgCluster'		;
		EXEC sp_rename 'planning.gFactories'		,'tgFactories'		;
		EXEC sp_rename 'planning.gHTMLPages'		,'tgHTMLPages'		;
		EXEC sp_rename 'planning.gProductLines'		,'tgProductLines'	;
		EXEC sp_rename 'planning.gProducts'			,'tgProducts'		;
		EXEC sp_rename 'planning.gValueEffects'		,'tgValueEffects'	;
		EXEC sp_rename 'planning.hFormats'			,'thFormats'		;
		EXEC sp_rename 'planning.hLists'			,'thLists'			;
		EXEC sp_rename 'planning.hListValues'		,'thListValues'		;
		EXEC sp_rename 'planning.Process_Log'		,'tProcess_Log'		;
		EXEC sp_rename 'planning.rPreferences'		,'trPreferences'	;
		EXEC sp_rename 'planning.rRights'			,'trRights'			;
		EXEC sp_rename 'planning.rUser'				,'trUser'			;
		EXEC sp_rename 'planning.rUserRights'		,'trUserRights'		;

		-- Remove old table version
		DROP TABLE IF EXISTS system.tPipelineStatus;

		-- New table Version
		DROP TABLE IF EXISTS [system].tProcessExecutionLog;

		CREATE TABLE [system].tProcessExecutionLog
			(
				RowKey					INT IDENTITY(1,1)	NOT NULL,
				ExecutionID				INT					NULL,
				StartTime				DATETIME			NOT NULL,
				EndTime					DATETIME			NULL,
				ExecutingUser			NVARCHAR(50)		NULL,
				StatusCODE				NVARCHAR(50)		NOT NULL,
				ExecutionModeCODE		NVARCHAR(20)		NOT NULL,
				Log						NVARCHAR(max)		NULL,
				ProcessConfigurationKey	INT					NULL,
				HandleID				NVARCHAR(50)		NULL,
				DetailsJSON				NVARCHAR(3000)		NULL,
		 CONSTRAINT PK_tProcessExecutionLog PRIMARY KEY NONCLUSTERED (RowKey)
			);


		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Technical primary key for row identity.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'RowKey';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'ID of the pipeline execution instance that this row belongs to.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'ExecutionID';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Time at which this execution step started.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'StartTime';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Time at which this execution step ended.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'EndTime';
		

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description'
			,@value=N'User that has executed the process, if it has been started manually.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'ExecutingUser';
		

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description'
			,@value=N'Status of this execution step.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE' 
			,@level1name=N'tProcessExecutionLog' 
			,@level2type=N'COLUMN'
			,@level2name=N'StatusCODE';
		

		EXEC sys.sp_addextendedproperty @name=N'MS_Description'
			,@value=N'The execution mode of the pipeline, can be manual and automatic.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'ExecutionModeCODE';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'A log that belongs to this execution.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'Log';
			

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'The process configuration that has been executed in this step.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'ProcessConfigurationKey';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'A process handle that is needed for this execution.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'HandleID';
		

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Some additional information that belongs to this execution.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'DetailsJSON';



		-- Drop and recreate tProcessConfigurations (switch to JSON storage)
		DROP TABLE IF EXISTS [system].tProcessConfigurations;

		CREATE TABLE [system].tProcessConfigurations (
			RowKey						INT        IDENTITY(1,1)    NOT NULL,		-- Technical primary Key for row identity.
			ProcessPipelineKey			INT                         NOT NULL,		-- Foreign key of tProcessPipelines
			ProcessConfigurationName	NVARCHAR(255)               NOT NULL,		-- Name of the ProcessConfiguration.
			OrderOfExecution			INT                         NOT NULL,		-- Order of execution in process pipeline.
			ConfigurationDetailsJSON	NVARCHAR(1000)  			NOT NULL,		-- Flag if loaded with delta load
			CONSTRAINT PK_system_tProcessConfigurations PRIMARY KEY NONCLUSTERED (RowKey)
		);

		-- Documentation
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store process configurations.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessConfigurations';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessConfigurations',
			@level2type = N'Column',
			@level2name = N'RowKey';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Foreign key of tProcessPipelines.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessConfigurations',
			@level2type = N'Column',
			@level2name = N'ProcessPipelineKey';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Name of the ProcessConfiguration.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessConfigurations',
			@level2type = N'Column',
			@level2name = N'ProcessConfigurationName';

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores at which position of the process pipeline the configuration should be executed.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessConfigurations'
			,@level2type=N'COLUMN'
			,@level2name=N'OrderOfExecution';

		EXEC sys.sp_addextendedproperty 
			 @name=N'MS_Description'
			,@value=N'The details of this configuration, vary with the configuration type.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessConfigurations'
			,@level2type=N'COLUMN'
			,@level2name=N'ConfigurationDetailsJSON';
		

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.8 to 5.0.9
BEGIN
	SET @ThisDBVersion = 509
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

		

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.9 to 5.0.10
BEGIN
	SET @ThisDBVersion = 050010
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF CAST(@ThisDBVersion AS INT) > CAST(@CurrentDBVersion AS INT) --because of changing numbering System
	BEGIN

		

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.0.10 to 5.1.0
BEGIN
	SET @ThisDBVersion = 050100
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		-- Rename ProcessConfiguration to ProcesStep
		EXEC sp_rename 'system.tProcessCompanies.ProcessConfigurationKey', 'ProcessStepKey', 'COLUMN';
		EXEC sp_updateextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store selected companies for process steps.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessCompanies';
		
		EXEC sp_rename 'system.tProcessExecutionLog.ProcessConfigurationKey', 'ProcessStepKey', 'COLUMN';
		EXEC sys.sp_updateextendedproperty 
			 @name=N'MS_Description'
			,@value=N'The process step that has been executed.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessExecutionLog'
			,@level2type=N'COLUMN'
			,@level2name=N'ProcessStepKey';
		
		EXEC sp_rename 'system.tProcessSolutions.ProcessConfigurationKey', 'ProcessStepKey', 'COLUMN';
		
		EXEC sp_rename 'system.tProcessConfigurations', 'tProcessSteps';
		EXEC sp_rename 'system.tProcessSteps.ProcessConfigurationName', 'Name', 'COLUMN';
		EXEC sp_rename 'system.tProcessSteps.ConfigurationDetailsJSON', 'StepDetailsJSON', 'COLUMN';
		EXEC sp_rename 'system.PK_system_tProcessConfigurations', 'PK_system_tProcessSteps', 'OBJECT';
		
		EXEC sp_updateextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store process steps.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSteps';

		EXEC sp_updateextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSteps',
			@level2type = N'Column',
			@level2name = N'RowKey';

		EXEC sp_updateextendedproperty
			@name = N'MS_Description',
			@value = N'Foreign key of tProcessPipelines.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSteps',
			@level2type = N'Column',
			@level2name = N'ProcessPipelineKey';

		EXEC sp_updateextendedproperty
			@name = N'MS_Description',
			@value = N'Name of the ProcessStep.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessSteps',
			@level2type = N'Column',
			@level2name = N'Name';

		EXEC sp_updateextendedproperty 
			 @name=N'MS_Description'
			,@value=N'Stores at which position of the process pipeline the step should be executed.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessSteps'
			,@level2type=N'COLUMN'
			,@level2name=N'OrderOfExecution';

		EXEC sp_updateextendedproperty 
			 @name=N'MS_Description'
			,@value=N'The details of this step, varies with the step type.' 
			,@level0type=N'SCHEMA'
			,@level0name=N'system'
			,@level1type=N'TABLE'
			,@level1name=N'tProcessSteps'
			,@level2type=N'COLUMN'
			,@level2name=N'StepDetailsJSON';
	
		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.1.0 to 5.1.1
BEGIN
	SET @ThisDBVersion = 050101
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

		-- Drop and recreate tProcessCompanies (add additional options json)
		DROP TABLE IF EXISTS system.tProcessCompanies;

		CREATE TABLE system.tProcessCompanies (
			RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
			ProcessStepKey           INT                          NOT NULL,     -- Foreign key of tProcessSteps
			CompanyKey               INT                          NOT NULL,     -- Foreign key of tCompanies.
			AdditionalOptionsJSON    NVARCHAR(4000)               NULL,         -- Json string for flex parameter content
			CONSTRAINT PK_system_tProcessCompanies PRIMARY KEY NONCLUSTERED (RowKey)
		);

		-- Documentation
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store selected companies for process steps.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessCompanies'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessCompanies',
			@level2type = N'Column',
			@level2name = N'RowKey'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Foreign key of tCompanies.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessCompanies',
			@level2type = N'Column',
			@level2name = N'CompanyKey'

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Json string for flex parameter content',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tProcessCompanies',
			@level2type = N'Column',
			@level2name = N'AdditionalOptionsJSON'


		-- Table result.tFinAccountingJournal was already for 5.0.4 created, but forgotten in Update script

		DROP TABLE IF EXISTS result.tFinAccountingJournal;

		CREATE TABLE result.tFinAccountingJournal (
			CompanyKey_Original           NVARCHAR(63)    NOT NULL,
			AccountKey_Original           NVARCHAR(370)   NULL,
			OffsetAccountKey_Original     NVARCHAR(370)   NULL,
			CostObject1Key_Original       NVARCHAR(370)   NULL,
			CostObject2Key_Original       NVARCHAR(370)   NULL,
			CompanyKey                    NVARCHAR(63)    NULL,
			AccountKey                    NVARCHAR(370)   NULL,
			OffsetAccountKey              NVARCHAR(370)   NULL,
			CostObject1Key                NVARCHAR(370)   NULL,
			CostObject2Key                NVARCHAR(370)   NULL,
			AccountingAreaCODE            NVARCHAR(50)    NOT NULL,
			FinancialYear                 SMALLINT        NOT NULL,
			FinancialPeriod               TINYINT         NOT NULL,
			PeriodKey                     NVARCHAR(8)      NULL,
			OpeningBalance                BIGINT          NOT NULL,
			Debit                         BIGINT          NOT NULL,
			Credit                        BIGINT          NOT NULL,
			PostingDate                   DATE            NULL,
			PostingText                   NVARCHAR(255)   NULL,
			DocumentDate                  DATE            NULL,
			DocumentNumber                NVARCHAR(255)   NULL,
			DocumentLine                  INT             NULL,
			ICPartnerID                   NVARCHAR(50)    NULL,
			SourceID                      BIGINT          NULL,
			RowKey                        BIGINT          NULL
		);
		
		-- Table result.tCompanies was already for 5.0.4 created, but forgotten in Update script

		DROP TABLE IF EXISTS result.tCompanies;

		CREATE TABLE result.tCompanies (
			CompanyKey             NVARCHAR(63)    NOT NULL,
			DataSourceKey          INT             NOT NULL,
			CompanyID              NVARCHAR(50)    NOT NULL,
			ActiveFlag             INT             NOT NULL,
			CompanyName            NVARCHAR(255)   NOT NULL,
			FinancialYearBegin     INT             NOT NULL,
			ChartsOfAccountsID     NVARCHAR(50)    NOT NULL,
			AccountLength          INT             NOT NULL,
			Currency               NVARCHAR(50)    NOT NULL,
			OwnerCODE              NVARCHAR(50)    NOT NULL
		);


		-- Table result.tFinAccounts was already for 5.0.4 created, but forgotten in Update script

		DROP TABLE IF EXISTS result.tFinAccounts;

		CREATE TABLE result.tFinAccounts (
			AccountKey           NVARCHAR(165)   NOT NULL,
			DataSourceKey        INT             NOT NULL,
			CompanyID            NVARCHAR(50)    NOT NULL,
			AccountGroupCODE     NVARCHAR(50)    NOT NULL,
			AccountID            NVARCHAR(50)    NOT NULL,
			AccountTypeCODE      NVARCHAR(50)    NOT NULL,
			AccountName          NVARCHAR(255)   NOT NULL,
			OwnerCODE            NVARCHAR(50)    NOT NULL
		);

		-- Table result.tFinCostObjects was already for 5.0.4 created, but forgotten in Update script
		
		DROP TABLE IF EXISTS result.tFinCostObjects;

		CREATE TABLE result.tFinCostObjects (
			CostObjectKey           NVARCHAR(165)   NOT NULL,
			DataSourceKey           INT             NOT NULL,
			CompanyID               NVARCHAR(50)    NOT NULL,
			CostObjectGroupCODE     NVARCHAR(50)    NOT NULL,
			CostObjectID            NVARCHAR(50)    NOT NULL,
			CostObjectName          NVARCHAR(255)   NOT NULL,
			OwnerCODE               NVARCHAR(50)    NOT NULL
		);

		
		-- GRANT TABLE on PARAM Schema to autocreate Param Tables on saving
		GRANT CREATE TABLE TO pf_PlanningFactoryUser;
		GRANT CREATE TABLE TO pf_PlanningFactoryService;

		--For Param Tables
		GRANT ALTER ON SCHEMA :: param TO pf_PlanningFactoryService;
		GRANT INSERT ON SCHEMA :: param TO pf_PlanningFactoryService;
		GRANT SELECT ON SCHEMA :: param TO pf_PlanningFactoryService;

		-- This temporary for XLS Client
		GRANT ALTER ON SCHEMA :: param TO pf_PlanningFactoryUser; 
		GRANT INSERT ON SCHEMA :: param TO pf_PlanningFactoryUser;
		GRANT SELECT ON SCHEMA :: param TO pf_PlanningFactoryUser;

		-- DB Version aktualisieren
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END

-- Script to update 5.1.1 to 5.2.0
BEGIN
	SET @ThisDBVersion = 050200
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN
		-- we were lazy, nothing happened here :)

		-- Update database version
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END
-- Script to update 5.2.0 to 5.2.1
BEGIN
	SET @ThisDBVersion = 050201
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN

		-- added table system.tProcessLogs
		DROP TABLE IF EXISTS system.tProcessLogs;

		CREATE TABLE [system].[tProcessLogs](
			[RowKey] [int] IDENTITY(1,1) NOT NULL,
			[ModuleCODE] [nvarchar](50) NOT NULL,
			[ProcessName] [nvarchar](250) NOT NULL,
			[ProcessID] [nvarchar](50) NULL,
			[SubProcessName] [nvarchar](250) NULL,
			[SubProcessID] [nvarchar](50) NULL,
			[SubSubProcessName] [nvarchar](250) NULL,
			[EventCODE] [nchar](10) NOT NULL,
			[Timestamp] [datetime] NOT NULL,
			[ReturnCODE] [int] NULL,
			[EventMessage] [nvarchar](max) NULL,
		 CONSTRAINT [PK_tProcessLogs] PRIMARY KEY CLUSTERED 
		(
			[RowKey] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Specification of the module that created this log entry, e.g. Server, Integration, Planning, Visualization' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'ModuleCODE'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Name of the process that is logged here.' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'ProcessName'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'ID of the process that is logged here (optional)' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'ProcessID'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Name of the subprocess that is logged here (optional)' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'SubProcessName'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'ID of the subprocess that is logged here (optional)' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'SubProcessID'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Description of the event that occurred in the process, can be: Start, End, Info' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'EventCODE'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Date and time when this event occurred' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'Timestamp'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Return code of the process that has been executed, use HTTP return codes (optional)' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'ReturnCODE'

		EXEC sys.sp_addextendedproperty 
			@name=N'MS_Description', 
			@value=N'Details of this event, e.g. a log message or an exception that occurred (optional)' , 
			@level0type=N'SCHEMA',
			@level0name=N'system', 
			@level1type=N'TABLE',
			@level1name=N'tProcessLogs', 
			@level2type=N'COLUMN',
			@level2name=N'EventMessage'


		-- new columns, which were forgoten update script 5.0.1
		IF COL_LENGTH ('global.tCompanies','CustomValuesJSONUser') IS NULL
			BEGIN
			  ALTER TABLE global.tCompanies
				ADD CustomValuesJSONUser NVARCHAR(MAX) NULL
			END;

		IF COL_LENGTH ('global.tFinAccounts','CustomValuesJSONUser') IS NULL
			BEGIN
			  ALTER TABLE global.tFinAccounts
				ADD CustomValuesJSONUser NVARCHAR(MAX) NULL
			END;

		IF COL_LENGTH ('global.tFinCostObjects','CustomValuesJSONUser') IS NULL
			BEGIN
			  ALTER TABLE global.tFinCostObjects
				ADD CustomValuesJSONUser NVARCHAR(MAX) NULL
			END;


		-- set type of json columns to nvarchar(max)
		ALTER TABLE system.tProcessExecutionLog		ALTER COLUMN DetailsJSON			NVARCHAR(max);
		ALTER TABLE global.tCompanies				ALTER COLUMN CustomValuesJSON		NVARCHAR(max);
		ALTER TABLE global.tCompanies				ALTER COLUMN CustomValuesJSONUser	NVARCHAR(max);
		ALTER TABLE global.tDataSources				ALTER COLUMN ConnectionValuesJSON	NVARCHAR(max);
		ALTER TABLE global.tFinAccounts				ALTER COLUMN CustomValuesJSON		NVARCHAR(max);
		ALTER TABLE global.tFinAccounts				ALTER COLUMN CustomValuesJSONUser	NVARCHAR(max);
		ALTER TABLE global.tFinCostObjects			ALTER COLUMN CustomValuesJSON		NVARCHAR(max);
		ALTER TABLE global.tFinCostObjects			ALTER COLUMN CustomValuesJSONUser	NVARCHAR(max);
		ALTER TABLE integration.tFinAccounts		ALTER COLUMN CustomValuesJSON		NVARCHAR(max);
		ALTER TABLE integration.tFinCostObjects		ALTER COLUMN CustomValuesJSON		NVARCHAR(max);
		ALTER TABLE system.tProcessCompanies		ALTER COLUMN AdditionalOptionsJSON	NVARCHAR(max);
		ALTER TABLE system.tProcessSteps			ALTER COLUMN StepDetailsJSON		NVARCHAR(max);

		-- set column ExecutionID in table system.tProcessExecution to NOT NULL
		UPDATE system.tProcessExecutionLog SET ExecutionID=0 WHERE ExecutionID IS NULL
		ALTER TABLE system.tProcessExecutionLog ALTER COLUMN ExecutionID int NOT NULL


		-- Update database version
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END


-- Script to update 5.2.1 to 5.2.2
BEGIN
	SET @ThisDBVersion = 050202
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN


		-- Update database version
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END


-- Script to update 5.2.2 to 5.2.3
BEGIN
	SET @ThisDBVersion = 050203
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN


		-- Update database version
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END


-- Script to update 5.2.3 to 5.2.4
BEGIN
	SET @ThisDBVersion = 050204
	
	SELECT @CurrentDBVersion = system.GetSetting('DBVersion');
	
	IF @ThisDBVersion > @CurrentDBVersion
	BEGIN


		-- Update database version
		EXEC system.SetSetting 'DBVersion', @ThisDBVersion;
	END
END


-- Script to update 5.2.5 to 5.2.5
-- SCRIPT CHANGES TO NEW BATCH SEPARATION

GO -- end the old batches

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	UPDATE #Version SET ThisVersion = 050205;  -- will be converted to int without leading zero
	UPDATE #Version SET CurrentVersion = system.GetSetting('DBVersion')
	
	-- Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
	BEGIN

		-- new Settings table

		DROP TABLE IF EXISTS system.tSettings;

		CREATE TABLE system.tSettings (
			RowKey						INT  IDENTITY(1,1)  NOT NULL,
			SettingID					NVARCHAR(50)		NOT NULL, 
			SettingDescription			NVARCHAR(4000)		NOT NULL,
			DataTypeCODE				NVARCHAR(255)		NOT NULL,
			ValueText					NVARCHAR(4000)		NOT NULL CONSTRAINT Default_ValueText		DEFAULT '', 
			ValueDateTime				DATETIME			NOT NULL CONSTRAINT Default_ValueDateTime	DEFAULT '1900-01-01 00:00:00.000',
			ValueInt					BIGINT				NOT NULL CONSTRAINT Default_ValueInt		DEFAULT 0,
			ValueIntScale				INT					NOT NULL CONSTRAINT Default_ValueIntScale	DEFAULT 1,
			ValueJSON					NVARCHAR(MAX)		NOT NULL CONSTRAINT Default_ValueJSON		DEFAULT '',
			Category					NVARCHAR(255)		NOT NULL CONSTRAINT Default_Category		DEFAULT '',
			ReadOnlySystemPropertyBool  INT					NOT NULL
			CONSTRAINT PK_tSettings PRIMARY KEY NONCLUSTERED (RowKey)
		);

		-- Documentation
		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Table to store global database properties.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Technical primary Key for row identity.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'RowKey';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'SettingID as business identifier for the entry.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'SettingID';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Description or comment of this setting for.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'SettingDescription';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'DataTypeCODE of setting that shows which value column is to used and how it should be used. Can be TEXT, DATETIME, BOOLEAN, NUMBER, JSON. See documentation for DataTypeCodes',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'DataTypeCODE';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Text value if value TEXT is set for column DataType, optional to submit default Value '' is set.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ValueText';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'DateTime value if value DATETIME is set for column DataType, optional to submit default Value 1900-01-01 00:00:00.000 is set.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ValueDateTime';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Integer value if value BOOLEAN or NUMBER is set for Column DataType. In Case of Boolean: 0,1,-1 for false,true and unknown. NUMBER - an Integer Number which must be scaled with column ValueIntScale. Optional to submit default Value 0 is set.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ValueInt';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Integer value of 1 for BOOLEAN value. Value of 1, 10, 100, 10000 for scaling type NUMBER to decimal value -> divide column ValueInt by column ValueIntScale. Optional to submit default Value 1 is set.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ValueIntScale';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'JSON value if DateTypeCODE is set to JSON. Optional to submit default Value '' is set.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ValueJSON';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Category to group items.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'Category';

		EXEC sp_addextendedproperty
			@name = N'MS_Description',
			@value = N'Flag to signalize if property is changeable by user or is a Ready Only System Value.',
			@level0type = N'Schema',
			@level0name = N'system',
			@level1type = N'Table',
			@level1name = N'tSettings',
			@level2type = N'Column',
			@level2name = N'ReadOnlySystemPropertyBool';

		-- adjust global.Codes (add column)
		ALTER TABLE global.tCodes ADD Scope NVARCHAR(255) NOT NULL
		CONSTRAINT Default_Scope DEFAULT '';
	END
	GO

	-- Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
	BEGIN

		-- set Scope dimension for existing entrys
		UPDATE global.tCodes SET Scope = 'Dimensions';

		-- add entries for Datatypes
		INSERT INTO global.tCodes (CODE, Scope, Language, CODEDescription) VALUES
			 ('TEXT'				,'DataType'		, 'en', 'DataType in multitype tables for Text')
			,('DATE'				,'DataType'		, 'en', 'DataType in multitype tables for DataValues')
			,('DATETIME'			,'DataType'		, 'en', 'DataType in multitype tables for Datetime Values')
			,('BOOLEAN'				,'DataType'		, 'en', 'DataType in multitype tables for Boolean Values')
			,('NUMBER'				,'DataType'		, 'en', 'DataType in multitype tables for Numbers')
			,('JSON'				,'DataType'		, 'en', 'DataType in multitype tables for JSON');

		-- Update database version - INSERT, as new POST is not avaiable in this step
		
		INSERT INTO system.tSettings (SettingID, SettingDescription,DataTypeCODE,ValueInt,ReadOnlySystemPropertyBool) VALUES
		('DBVersion','Version of Database','NUMBER',050205,1);

		-- EXEC system.pPOST_Setting @Username = 'SQL', @SettingID = 'DBVersion', @DataTypeCODE = 'NUMBER', @ValueInt = @ThisDBVersion, @ReadOnlySystemPropertyBool = 1
	END
	GO



-- Script to update 5.2.5 to 5.2.6

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050206;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			-- new tables or alter tables
		
			-- No longer existing Function which was needed from update und temporary created in Updatescript
			EXEC ('DROP FUNCTION IF EXISTS system.GetSetting');


		END
	GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version


			-- add entries to tCodes for cost objects - projects
			INSERT INTO global.tCodes (CODE, Scope, Language, CODEDescription) VALUES
				 ('PROJ'				,'Dimensions'	, 'de', 'Projekt')
				,('PROJ'				,'Dimensions'	, 'en', 'Project')

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.6 to 5.2.7

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050207;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
	BEGIN

		-- new tables or alter tables

		CREATE TABLE planning.tFiles (
			 FileKey		BIGINT IDENTITY(1,1)	NOT NULL
			,FactoryID		NVARCHAR(50)			NOT NULL
			,ProductLineID	NVARCHAR(50)			NOT NULL
			,ProductID		NVARCHAR(50)			NOT NULL
			,FileID			NVARCHAR(50)			NOT NULL
			,[FileName]		NVARCHAR(255)			NOT NULL
			,FileExtension	NVARCHAR(50)			NULL
			,FileBody		VARBINARY(MAX)			NOT NULL
		) 
	


	END
	GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version
			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.7 to 5.2.8

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050208;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			-- new tables or alter tables
			PRINT 'No new or alterd tables in 5.2.8'
		END
	GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version
	
			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.8 to 5.2.9

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050209;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			-- new tables or alter tables
			PRINT 'New table for icon storage'

				CREATE TABLE system.tIcons (
					 IconKey		BIGINT IDENTITY (1,1)	NOT NULL
					,IconName		NVARCHAR (255)			NOT NULL
					,FileExtension	NVARCHAR (50)			NULL
					,FileBody		VARBINARY (MAX)			NOT NULL
				);

				-- Documentation
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Table to store icons for use in application GUI',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tIcons';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical primary Key for row identity.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tIcons',
					@level2type = N'Column',
					@level2name = N'IconKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Name for the Icon as full name including Extension.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tIcons',
					@level2type = N'Column',
					@level2name = N'IconName';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Extension of the image file.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tIcons',
					@level2type = N'Column',
					@level2name = N'FileExtension';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Filebody as binary storage of the icon data.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tIcons',
					@level2type = N'Column',
					@level2name = N'FileBody';

			PRINT 'Update for planning.tFiles'
				-- Rename old table
				EXEC sp_rename 'planning.tFiles','tFiles_bak','OBJECT'

				-- create new table

				CREATE TABLE planning.tFiles (
						FileKey		BIGINT IDENTITY(1,1)	NOT NULL
					,FactoryKey		INT						NOT NULL
					,ProductLineKey	INT						NOT NULL
					,ProductKey		INT						NOT NULL
					,FactoryID		NVARCHAR(50)			NOT NULL
					,ProductLineID	NVARCHAR(50)			NOT NULL
					,ProductID		NVARCHAR(50)			NOT NULL
					,FileID			NVARCHAR(50)			NOT NULL
					,[FileName]		NVARCHAR(255)			NOT NULL
					,FileExtension	NVARCHAR(50)			NULL
					,FileBody		VARBINARY(MAX)			NOT NULL
				) 

				-- Documentation
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Table to store files attached to products',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical primary Key for row identity.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FileKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical Key for the Factory.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FactoryKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical Key for the ProductLine.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'ProductLineKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical Key for the Product.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'ProductKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Business ID for the Factory.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FactoryID';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Business ID for the ProductLine.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'ProductlineID';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Business ID for the Product.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'ProductID';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Business ID for the File.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FileID';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Name of the File without extension.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FileName';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Separated File extension for better handling.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FileExtension';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'File Body as binary data.',
					@level0type = N'Schema',
					@level0name = N'planning',
					@level1type = N'Table',
					@level1name = N'tFiles',
					@level2type = N'Column',
					@level2name = N'FileBody';

				-- transfer Values
				INSERT INTO planning.tFiles
					SELECT 
							dF.FactoryKey
						,dPL.ProductLineKey
						,dP.ProductKey
						,tFb.FactoryID
						,tFb.ProductlineID
						,tFb.ProductID
						,tFb.FileID
						,tFb.FileName
						,tFb.FileExtension
						,tFb.FileBody

					FROM planning.tFiles_bak tFb
						LEFT JOIN planning.tdFactories dF
							ON		tFb.FactoryID = dF.FactoryID
						LEFT JOIN planning.tdProductLines dPL
							ON 
									tFb.FactoryID		= dPL.FactoryID
								AND tFb.ProductLineID	= dPL.ProductLineID
						LEFT JOIN planning.tdProducts dP
							ON 
									tFb.FactoryID		= dPL.FactoryID
								AND tFb.ProductLineID	= dPL.ProductLineID
								AND tFb.ProductID		= dP.ProductID

				-- delete old table
				DROP TABLE planning.tFiles_bak


			END
			GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'New Codes for global.tCodes - cost objects - projects'

			INSERT INTO global.tCodes (CODE, Scope, Language, CODEDescription) VALUES
				 ('FIBU_SUM'			,'Dimensions'	, 'de', 'Finanzbuchhaltung Summen')
    			,('FIBU_SUM'			,'Dimensions'	, 'en', 'Financial accounting totals')
				,('KORE_SUM'			,'Dimensions'	, 'de', 'Kostenrechnung Summen')
				,('KORE_SUM'			,'Dimensions'	, 'en', 'Cost accounting totals')
				,('PCA_SUM'				,'Dimensions'	, 'de', 'Profitcenterrechnung Summen')
				,('PCA_SUM'				,'Dimensions'	, 'en', 'Profit center accounting totals')

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.9 to 5.2.10

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050210;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			PRINT 'No changed table configurations in 5.2.10'
		END
	GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'No new table values to deliver in 5.2.10'

			

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.10 to 5.2.11

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050211;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			PRINT 'INFO: No changed table configurations in 5.2.11'
		END
	GO

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'INFO: No new table values to deliver in 5.2.11'

			

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.11 to 5.2.12

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050212;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			PRINT 'INFO: New Table for DebCredAccounts'
			CREATE TABLE integration.tDebCredAccounts (
				RowKey               BIGINT        IDENTITY(1,1)     NOT NULL,
				DataSourceKey        INT                             NOT NULL,
				CompanyID            NVARCHAR(50)                    NOT NULL,
				AccountID            NVARCHAR(50)                    NOT NULL,
				AccountTypeCODE      NVARCHAR(50)                    NOT NULL,
				AccountName          NVARCHAR(255)                   NOT NULL,
				Street               NVARCHAR(255)                   NULL,
				PostalCode           NVARCHAR(255)                   NULL,
				City                 NVARCHAR(255)                   NULL,
				CountryIsoCODE       NVARCHAR(50)                    NULL,
				ICPartnerID          NVARCHAR(50)                    NULL,
				CustomValuesJSON     NVARCHAR(max)                   NULL        
				CONSTRAINT PK_integration_tDebCredAccounts PRIMARY KEY NONCLUSTERED (RowKey)
			);

			PRINT 'INFO: New Table for DebCredJournal'
			CREATE TABLE integration.tDebCredJournal (
				RowKey                     BIGINT        IDENTITY(1,1)     NOT NULL,
				DataSourceKey              INT                             NOT NULL,
				CompanyID                  NVARCHAR(50)                    NOT NULL,    -- Mandanten_ID
				AccountID                  NVARCHAR(255)                   NOT NULL,    -- ID des Personenkontos
				FinancialYear              SMALLINT                        NOT NULL,
				FinancialPeriod            TINYINT                         NOT NULL,    
				ItemStatusCODE             NVARCHAR(255)                   NOT NULL,    -- Status des Posten (offen o. ausgeglichen)
				OpeningBalance             BIGINT                          NOT NULL,    -- Opening balance value. Decimal places are eliminated through multiplication with 100.
				Debit                      BIGINT                          NOT NULL,    -- Debit value. Decimal places are eliminated through multiplication with 100.
				Credit                     BIGINT                          NOT NULL,    -- Credit value. Decimal places are eliminated through multiplication with 100.
				PostingDate                DATE                            NULL,        -- Buchungsdatum
				PostingText                NVARCHAR(255)                   NULL,        -- Buchungstext
				DocumentNumber             NVARCHAR(255)                   NOT NULL,    -- Belegnummer (intern vergeben)
				DocumentDate               DATE                            NOT NULL,    -- Belegdatum
				DocumentLine               INT                             NULL,        -- Position im Beleg
				DocumentTypeCODE           NVARCHAR(255)                   NULL,        -- Typ des Belegs (Rechnung, Zahlung, Gutschrift, ...)
				ExternalInvoiceNumber      NVARCHAR(255)                   NULL,        -- Rechnungsnummer
				SourceID                   BIGINT                          NULL,        -- Interne ID des Datensatzes im Vorsystem (wenn vorhanden)
				DeltaValue                 BIGINT                          NULL,
				CustomValuesJSON           NVARCHAR(4000)                  NULL,
			);

			-- Indexes
			CREATE CLUSTERED COLUMNSTORE INDEX IX_integration_tDebCredJournal_1 ON integration.tDebCredJournal;

		END
			

	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'INFO: No new table values to deliver in 5.2.12'

			

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.12 to 5.2.13

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050213;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			PRINT 'INFO: Changing table system.tProcessPipelines'

			-- Rename old table to bak table and drop constaints
				EXEC sp_rename 'system.tProcessPipelines','tProcessPipelines_bak','OBJECT';
				ALTER TABLE system.tProcessPipelines_bak
				DROP CONSTRAINT IF EXISTS PK_system_tProcessPipelines;
			
			-- Create new table
				CREATE TABLE system.tProcessPipelines (
					RowKey                   INT        IDENTITY(1,1)     NOT NULL,     -- Technical primary Key for row identity.
					ProcessPipelineName      NVARCHAR(255)                NOT NULL,     -- Name of the ProcessPipeline.
					OrderOfDisplay			 INT                          NOT NULL,		-- Order of display in process pipeline overview.
					CONSTRAINT PK_system_tProcessPipelines PRIMARY KEY NONCLUSTERED (RowKey)
				);

				-- Documentation
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Table to store process pipelines.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessPipelines';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical primary Key for row identity.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessPipelines',
					@level2type = N'Column',
					@level2name = N'RowKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Name of the ProcessPipeline.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessPipelines',
					@level2type = N'Column',
					@level2name = N'ProcessPipelineName';

				EXEC sys.sp_addextendedproperty 
					 @name=N'MS_Description'
					,@value=N'Stores at which position the process pipeline should be displayed.' 
					,@level0type=N'SCHEMA'
					,@level0name=N'system'
					,@level1type=N'TABLE'
					,@level1name=N'tProcessPipelines'
					,@level2type=N'COLUMN'
					,@level2name=N'OrderOfDisplay';
			
			-- Transfer Values from bak table to table
				SET IDENTITY_INSERT system.tProcessPipelines ON;
				EXEC('INSERT INTO system.tProcessPipelines (RowKey,ProcessPipelineName,OrderOfDisplay)
						SELECT RowKey,ProcessPipelineName,0 AS OrderOfDisplay FROM system.tProcessPipelines_bak');
				SET IDENTITY_INSERT system.tProcessPipelines OFF;

			-- delete bak table
				DROP TABLE system.tProcessPipelines_bak;


			PRINT 'INFO: Changing table system.tProcessSteps'

			-- Rename old table to bak table and drop constaints
				EXEC sp_rename 'system.tProcessSteps','tProcessSteps_bak','OBJECT';
				ALTER TABLE system.tProcessSteps_bak
				DROP CONSTRAINT IF EXISTS PK_system_tProcessSteps;

			-- Create new table
				CREATE TABLE [system].tProcessSteps (
					RowKey						INT        IDENTITY(1,1)    NOT NULL,		-- Technical primary Key for row identity.
					ProcessPipelineKey			INT                         NOT NULL,		-- Foreign key of tProcessPipelines
					Name						NVARCHAR(255)               NOT NULL,		-- Name of the ProcessStep.
					OrderOfExecution			INT                         NOT NULL,		-- Order of execution in process pipeline.
					StepDetailsJSON				NVARCHAR(max)  				NOT NULL,		-- The content of the step, varying with the step type
					Active						BIT							NOT NULL,		-- Flag if the step is active
					CONSTRAINT PK_system_tProcessSteps PRIMARY KEY NONCLUSTERED (RowKey)
				);

				-- Documentation
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Table to store process steps.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessSteps';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Technical primary Key for row identity.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessSteps',
					@level2type = N'Column',
					@level2name = N'RowKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Foreign key of tProcessPipelines.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessSteps',
					@level2type = N'Column',
					@level2name = N'ProcessPipelineKey';

				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Name of the ProcessStep.',
					@level0type = N'Schema',
					@level0name = N'system',
					@level1type = N'Table',
					@level1name = N'tProcessSteps',
					@level2type = N'Column',
					@level2name = N'Name';

				EXEC sys.sp_addextendedproperty 
						@name=N'MS_Description'
					,@value=N'Stores at which position of the process pipeline the step should be executed.' 
					,@level0type=N'SCHEMA'
					,@level0name=N'system'
					,@level1type=N'TABLE'
					,@level1name=N'tProcessSteps'
					,@level2type=N'COLUMN'
					,@level2name=N'OrderOfExecution';

				EXEC sys.sp_addextendedproperty 
						@name=N'MS_Description'
					,@value=N'The details of this step, varies with the step type.' 
					,@level0type=N'SCHEMA'
					,@level0name=N'system'
					,@level1type=N'TABLE'
					,@level1name=N'tProcessSteps'
					,@level2type=N'COLUMN'
					,@level2name=N'StepDetailsJSON';

				EXEC sys.sp_addextendedproperty 
					 @name=N'MS_Description'
					,@value=N'Flag if the step is active' 
					,@level0type=N'SCHEMA'
					,@level0name=N'system'
					,@level1type=N'TABLE'
					,@level1name=N'tProcessSteps'
					,@level2type=N'COLUMN'
					,@level2name=N'Active';

			-- Transfer Values from bak table to table
				SET IDENTITY_INSERT system.tProcessSteps ON;
				EXEC('INSERT INTO system.tProcessSteps (RowKey,ProcessPipelineKey,Name,OrderOfExecution,StepDetailsJSON,Active)
						SELECT RowKey,ProcessPipelineKey,Name,OrderOfExecution,StepDetailsJSON, 1 AS Active FROM system.tProcessSteps_bak');
				SET IDENTITY_INSERT system.tProcessSteps OFF;

			-- delete bak table
				DROP TABLE system.tProcessSteps_bak;

			PRINT 'INFO: Changing table global.tCompanies to have nullable DatasourceKey'

			-- Rename old table to bak table and drop constaints
				EXEC sp_rename 'global.tCompanies','tCompanies_bak','OBJECT';
				ALTER TABLE global.tCompanies_bak
				DROP CONSTRAINT IF EXISTS PK_global_tCompanies;

			-- Create new table
			CREATE TABLE global.tCompanies (
				RowKey                   INT        IDENTITY(1,1)                          NOT NULL,
				CompanyKey               AS (concat([DataSourceKey],N'|',[CompanyID]))             ,
				DataSourceKey            INT                                               NULL,
				CompanyID                NVARCHAR(50)                                      NOT NULL,
				ActiveFlag               INT                                               NOT NULL,
				CompanyName              NVARCHAR(255)                                     NOT NULL,
				FinancialYearBegin       INT                                               NOT NULL,
				ChartsOfAccountsID       NVARCHAR(50)                                      NOT NULL,
				AccountLength            INT                                               NOT NULL,
				Currency                 NVARCHAR(50)                                      NOT NULL,
				CustomValuesJSON         NVARCHAR(max)	                                   NULL,
				CustomValuesJSONUser     NVARCHAR(max)                                     NULL,
				OwnerCODE                NVARCHAR(50)	                                   NOT NULL,
				MappingKey               INT                                               NULL,
				UpdatedFlag              INT                                               NOT NULL,
				CONSTRAINT PK_global_tCompanies PRIMARY KEY NONCLUSTERED (RowKey)
			);

			-- Transfer Values from bak table to table
			SET IDENTITY_INSERT global.tCompanies ON;
			EXEC('INSERT INTO global.tCompanies (RowKey,DataSourceKey,CompanyID,ActiveFlag,CompanyName,FinancialYearBegin,ChartsOfAccountsID,AccountLength,Currency
					, CustomValuesJSON,CustomValuesJSONUser,OwnerCode, MappingKey,UpdatedFlag)
						SELECT RowKey,DataSourceKey,CompanyID,ActiveFlag,CompanyName,FinancialYearBegin,ChartsOfAccountsID,AccountLength,Currency
					, CustomValuesJSON,CustomValuesJSONUser,OwnerCode, MappingKey,UpdatedFlag FROM global.tCompanies_bak');
			SET IDENTITY_INSERT global.tCompanies OFF;

			-- delete bak table
			DROP TABLE global.tCompanies_bak;

		END

	GO
			
	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'INFO: No new table values to deliver in 5.2.13'

			-- Update Version - not over system.pPOST_Settings as it exists no in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Script to update 5.2.13 to 5.2.14

	DROP TABLE IF EXISTS #Version;
	GO

	CREATE TABLE #Version 
		(
		 RowKey			INT NOT NULL IDENTITY (1,1)
		,ThisVersion	INT NOT NULL 
		,CurrentVersion INT NOT NULL
		);

	INSERT INTO #Version VALUES (0,0);
	DECLARE @Version INT
	UPDATE #Version SET ThisVersion = 050214;  -- will be converted to int without leading zero
	-- Version determination over table instead of system.pGET_Setting, as difficult access on a stored procedure and not nesting possible
	SELECT @Version = ValueInt FROM system.tSettings
	UPDATE #Version SET CurrentVersion = @Version
		
	-- 1. Table Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN

			PRINT 'INFO: Changing table integration.tDebCredJournal'
			PRINT 'INFO: New column DocumentClosedDate'
			IF COL_LENGTH ('integration.tDebCredJournal','DocumentClosedDate') IS NULL
			BEGIN
			  ALTER TABLE integration.tDebCredJournal ADD DocumentClosedDate DATE NULL;

				-- Column: DocumentClosedDate
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Date when document was closed through payment or refund.',
					@level0type = N'Schema',
					@level0name = N'integration',
					@level1type = N'Table',
					@level1name = N'tDebCredJournal',
					@level2type = N'Column',
					@level2name = N'DocumentClosedDate';
			END;

			PRINT 'INFO: New column DocumentDueDate'
			IF COL_LENGTH ('integration.tDebCredJournal','DocumentDueDate') IS NULL
			BEGIN
			  ALTER TABLE integration.tDebCredJournal ADD DocumentDueDate DATE NULL;

				-- Column: DocumentDueDate
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Due date of the document',
					@level0type = N'Schema',
					@level0name = N'integration',
					@level1type = N'Table',
					@level1name = N'tDebCredJournal',
					@level2type = N'Column',
					@level2name = N'DocumentDueDate';
			END;

			PRINT 'INFO: New column AccountTypeCODE'
			IF COL_LENGTH ('integration.tDebCredJournal','AccountTypeCODE') IS NULL
			BEGIN
			  ALTER TABLE integration.tDebCredJournal ADD AccountTypeCODE NVARCHAR(50) NOT NULL;

				-- Column: AccountTypeCODE
				EXEC sp_addextendedproperty
					@name = N'MS_Description',
					@value = N'Classification for an account, e.g. DEBITOR or KREDITOR. When no classification is given from source system, the classification will be done later through agent.',
					@level0type = N'Schema',
					@level0name = N'integration',
					@level1type = N'Table',
					@level1name = N'tDebCredJournal',
					@level2type = N'Column',
					@level2name = N'AccountTypeCODE';
			END;

			PRINT 'INFO: New Table global.tDebCredAccounts'
			CREATE TABLE global.tDebCredAccounts (
				RowKey                   BIGINT        IDENTITY(1,1)                                        NOT NULL,
				AccountKey               AS (concat([DataSourceKey],N'|',[CompanyID],N'|',[AccountID]))             ,     -- Primary Key based on CompanyKey (DataSourceKey + CompanyID) , AccountID, AccountGroupCODE for join operations.
				DataSourceKey            INT                                                                NOT NULL,
				CompanyID                NVARCHAR(50)                                                       NOT NULL,
				AccountID                NVARCHAR(50)                                                       NOT NULL,
				AccountTypeCODE          NVARCHAR(50)                                                       NOT NULL,
				AccountName              NVARCHAR(255)                                                      NOT NULL,
				Street                   NVARCHAR(255)                                                      NULL,
				PostalCode               NVARCHAR(255)                                                      NULL,
				City                     NVARCHAR(255)                                                      NULL,
				CountryIsoCODE           NVARCHAR(50)                                                       NULL,
				ICPartnerID              NVARCHAR(50)                                                       NULL,
				CustomValuesJSON         NVARCHAR(max)                                                      NULL,
				CustomValuesJSONUser     NVARCHAR(max)                                                      NULL,
				OwnerCODE                NVARCHAR(50)                                                       NOT NULL,     -- Owner of the Account. INTEGRATION=Integration, PLANNING=Planning or ORPHAN=Orphan - Integration will dominate Planning. Orphans arise if integration accounts are deleted.
				MappingKey               INT                                                                NULL,
				UpdatedFlag              INT                                                                NOT NULL,
				CONSTRAINT PK_global_tDebCredAccounts PRIMARY KEY NONCLUSTERED (RowKey)
			);

			-- Documentation
			-- Table
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'	Dimension table to store informations about debtor and creditor acccounts.
							Table contains all accounts per company with current (latest) name.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts';

			-- Columns
			-- Column: RowKey
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Technical primary Key for row identity.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'RowKey';

			-- Column: AccountKey
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Primary Key based on CompanyKey (DataSourceKey + CompanyID) , AccountID, AccountGroupCODE for join operations.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'AccountKey';

			-- Column: DataSourceKey
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Number of the Datasoure. DataSourceKey is used to create a unique identifier when paired with CompanyID.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'DataSourceKey';

			-- Column: CompanyID
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Company ID which is used in the source database.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'CompanyID';

			-- Column: AccountID
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Identifier for the customer or vendor account.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'AccountID';

			-- Column: AccountTypeCODE
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Classification for an account, e.g. DEBITOR or KREDITOR. When no classification is given from source system, the classification will be done later through agent.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'AccountTypeCODE';

			-- Column: AccountName
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Name of the Account. If name changes in time the last name.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'AccountName';

			-- Column: Street
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Street and house number of customer or vendor.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'Street';

			-- Column: PostalCode
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Postal code of customer or vendor.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'PostalCode';

			-- Column: City
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'City of customer or vendor.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'City';

			-- Column: CountryIsoCODE
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'ISO 3166-1 alpha-2 – two-letter country codes.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'CountryIsoCODE';

			-- Column: ICPartnerID
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Identifier for the intercompany partner.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'ICPartnerID';

			-- Column: CustomValuesJSON
			EXEC sp_addextendedproperty
				@name = N'MS_Description',
				@value = N'Custom field to store any customer individual informations.',
				@level0type = N'Schema',
				@level0name = N'global',
				@level1type = N'Table',
				@level1name = N'tDebCredAccounts',
				@level2type = N'Column',
				@level2name = N'CustomValuesJSON';
			
			PRINT 'INFO: New Table result.tDebCredAccounts'
			CREATE TABLE result.tDebCredAccounts (
				AccountKey           NVARCHAR(165)                   NOT NULL,
				DataSourceKey        INT                             NOT NULL,
				CompanyID            NVARCHAR(50)                    NOT NULL,
				AccountID            NVARCHAR(50)                    NOT NULL,
				AccountTypeCODE      NVARCHAR(50)                    NOT NULL,
				AccountName          NVARCHAR(255)                   NOT NULL,
				Street               NVARCHAR(255)                   NULL,
				PostalCode           NVARCHAR(255)                   NULL,
				City                 NVARCHAR(255)                   NULL,
				CountryIsoCODE       NVARCHAR(50)                    NULL,
				ICPartnerID          NVARCHAR(50)                    NULL,
				OwnerCODE            NVARCHAR(50)                    NOT NULL
			);

			PRINT 'INFO: New Table result.tDebCredJournal'
			CREATE TABLE result.tDebCredJournal (
				RowKey                     BIGINT                          NULL,
				CompanyKey_Original        NVARCHAR(63)                    NOT NULL,
				AccountKey_Original        NVARCHAR(370)                   NULL, 
				CompanyKey                 NVARCHAR(63)                    NULL,
				AccountKey                 NVARCHAR(370)                   NULL,
				FinancialYear              SMALLINT                        NOT NULL,
				FinancialPeriod            TINYINT                         NOT NULL,    
				ItemStatusCODE             NVARCHAR(255)                   NOT NULL,
				OpeningBalance             BIGINT                          NOT NULL,
				Debit                      BIGINT                          NOT NULL,
				Credit                     BIGINT                          NOT NULL,
				PostingDate                DATE                            NULL,
				PostingText                NVARCHAR(255)                   NULL,
				DocumentNumber             NVARCHAR(255)                   NOT NULL,
				DocumentDate               DATE                            NOT NULL,
				DocumentClosedDate         DATE                            NULL,
				DocumentDueDate            DATE                            NULL,
				DocumentLine               INT                             NULL,
				DocumentTypeCODE           NVARCHAR(255)                   NULL,
				ExternalInvoiceNumber      NVARCHAR(255)                   NULL,
				SourceID                   BIGINT                          NULL
			);
		END

	GO
			
	-- 2. Value Changing Batch
	IF EXISTS (SELECT TOP 1 * FROM #Version WHERE ThisVersion > CurrentVersion)
		BEGIN
			DECLARE @ThisVersion INT
			SELECT @ThisVersion = ThisVersion FROM #Version

			PRINT 'INFO: No new table values to deliver in 5.2.14'

			-- Update Version - not over system.pPOST_Settings as it exists not in older versions
			UPDATE system.tSettings SET ValueInt=@ThisVersion WHERE SettingID = 'DBVersion'
		END
	GO



-- Just to end the whole Update Process with a GO
GO

-- Function to split a String into a TableVariable
-- The Delimiter String will be replace by a CHAR(28) and the string splitted on this


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_PivotStringIntoTable') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION dbo.sx_pf_PivotStringIntoTable
GO

CREATE FUNCTION dbo.sx_pf_PivotStringIntoTable
(@String NVARCHAR(MAX),
@Delimiter NVARCHAR(5))

RETURNS @tblSplitValues TABLE (txtValues NVARCHAR(MAX))

AS
BEGIN

	DECLARE @spr		NCHAR(1)		= CHAR(28)
	DECLARE @auxString	NVARCHAR (MAX)	= ''
	SET @auxString = REPLACE(@String, @Delimiter,@spr);

	WITH Split(stpos,endpos)
	AS(
		SELECT 0 AS stpos, CHARINDEX(@spr,@auxString) AS endpos
		UNION ALL
		SELECT CAST(endpos AS INT)+1, CHARINDEX(@spr,@auxString,endpos+1)
		FROM Split
		WHERE endpos > 0
	)

	INSERT @tblSplitValues
		SELECT SUBSTRING(@auxString,stpos,COALESCE(NULLIF(endpos,0),LEN(@auxString)+1)-stpos)
		FROM Split OPTION (MAXRECURSION 32766) -- it works ONLY up to 32.766 rows

	RETURN

END

GO

/* 
function to mask strings, which will be part of an SQL Statement @StringValue 
returns a masked string
Gerd Tautenhahn for saxess-software gmbh
12/2016 for DataFactory 5.0

Testcall
DECLARE @String NVARCHAR(255)
SET @String = 'ToDo''s'
Print @String
SELECT dbo.sx_pf_pMaskSQL(@String)
*/



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pMaskSQL') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pMaskSQL
GO


CREATE FUNCTION dbo.sx_pf_pMaskSQL (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(@StringValue,'''','''''')
END

GO
-- Funktion gibt zurück einen '0', wenn der übergebene @BooleanValue ist nicht etweder '0' oder '1'
-- @BooleanValue ='<#NV>' wird ignoriert und nicht geändert

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pProtectBoolean') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pProtectBoolean
GO


CREATE FUNCTION dbo.sx_pf_pProtectBoolean (@BooleanValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @BooleanValue <>'<#NV>' 
	
	BEGIN
		IF @BooleanValue <>'0' AND @BooleanValue <>'1'
		SET @BooleanValue= '0'
	END

	RETURN @BooleanValue

END

GO

-- Function to replace all special signs from ID
-- An ID may contain Letters, Numbers,'_' and '-', but nothing else
-- Function returns a clean string

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pProtectID') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pProtectID
GO


CREATE FUNCTION dbo.sx_pf_pProtectID (@ElementID AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	SET @ElementID = RTRIM(LTRIM(@ElementID))
	RETURN	REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(@ElementID,'|',''),
							   ';',''),'''',''),'/',''),'\',''),'"',''),',',''),
							   '>',''),'<',''),'[',''),']',''),'(',''),')',''),'{',''),'}',''),'!',''),'$',''),'&',''),'^',''),'@',''),
							   '%',''),':',''),'`',''),'´',''),'?',''),'§',''),'#',''),'~',''),'+',''),'=',''),'.',''),'€',''),' ','')
END

GO

/*
Function cast all expression to int or return 0 for non-int values, except the <#NV> String this is passed anytime

Testcalls
SELECT dbo.sx_pf_pProtectInt('-1')
SELECT dbo.sx_pf_pProtectInt('1')
SELECT dbo.sx_pf_pProtectInt('Hase')
SELECT dbo.sx_pf_pProtectInt('<#NV>')
SELECT dbo.sx_pf_pProtectInt('')
SELECT dbo.sx_pf_pProtectInt('0')
SELECT dbo.sx_pf_pProtectInt('9999999999999999999999999999999999999999999999999999999999')

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pProtectInt') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pProtectInt
GO


CREATE FUNCTION dbo.sx_pf_pProtectInt (@IntValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @IntValue <>'<#NV>' 
		BEGIN
			SET @IntValue = COALESCE(CAST(TRY_CAST(@IntValue AS INT) AS NVARCHAR(255)),0)
		END

RETURN @IntValue

END

GO

-- Funktion entfern alle nicht erlaubte Sonderzeichen ( Hochommata,'|','),(', ';' ) aus @StringValue 
-- Die Funktion gibt zurück einen string, ohne nicht erlaubte Sonderzeichen

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pProtectString') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pProtectString
GO


CREATE FUNCTION dbo.sx_pf_pProtectString (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(REPLACE(REPLACE(@StringValue,'|',''), '),(',''), '--','')
END

GO

/* 
function to remove Linebreaks from a string
Gerd Tautenhahn for saxess-software gmbh
11/2018 for DataFactory 5.0

Testcall
DECLARE @String NVARCHAR(255)
SET @String = 'ToDo'+CHAR(13)+CHAR(10)+' for you'
Print @String
SELECT @String = dbo.sx_pf_pRemoveLinebreaks(@String)
Print @String
*/



IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pRemoveLinebreaks') AND type in (N'FN'))
DROP FUNCTION dbo.sx_pf_pRemoveLinebreaks
GO


CREATE FUNCTION dbo.sx_pf_pRemoveLinebreaks (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(REPLACE(@StringValue,CHAR(10),''),CHAR(13),'')

END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.Day_of_Week_DE') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION dbo.Day_of_Week_DE
GO

CREATE FUNCTION dbo.Day_of_Week_DE (@d DATETIME)
RETURNS INT
  BEGIN
      DECLARE @t INT

      SET @t = Datepart(weekday, @d) + @@DATEFIRST - 1

      IF @t > 7
        SET @t = @t - 7

      RETURN @t
  END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.Week_of_Year_DE') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION dbo.Week_of_Year_DE
GO

CREATE FUNCTION dbo.Week_of_Year_DE (@d DATETIME)
returns INT
  BEGIN
      DECLARE @k INT
      DECLARE @x DATETIME

      SET @x = Str(Year(@d)) + '-01-01'
      SET @k = Datepart(week, @d)

      IF dbo.Day_of_Week_DE(@x) > 4
        BEGIN
            SET @k = @k - 1

            IF @k = 0
              SET @k = 53
        END

      RETURN @k
  END 
GO
/*
    Product: OCT 5.0
    Object: system.fGetColumns
    Type: Function
    Author: saxess-software GmbH
    Script Date: 02.08.2019 16:32
*/

DROP FUNCTION IF EXISTS system.fGetColumns;
GO

CREATE FUNCTION system.fGetColumns (
	@Schema NVARCHAR(50),
	@Table NVARCHAR(50)
)
RETURNS TABLE 
AS
RETURN 
(
	SELECT
		c.Name AS ColName,
		c.is_computed AS IsCalculatedKey,
		CASE WHEN EXISTS(
			SELECT 1
			FROM sys.computed_columns cc
			WHERE cc.object_id = t.object_id
			AND cc.definition LIKE '%![' + c.Name + '!]%' ESCAPE '!'
		) THEN 1 ELSE 0 END AS IsPartOfKey
	FROM sys.columns c
	JOIN sys.tables t ON t.object_id = c.object_id
	JOIN sys.schemas s ON s.schema_id = t.schema_id
	LEFT JOIN sys.computed_columns cc ON cc.column_id = c.column_id AND cc.object_id = t.object_id
	WHERE s.name = @Schema
	AND t.name = @Table
	
	UNION ALL

	SELECT
		c.name as ColName,
		c.is_computed AS IsCalculatedKey,
		CASE WHEN EXISTS(
			SELECT 1
			FROM sys.computed_columns cc
			WHERE cc.object_id = v.object_id
			AND cc.definition LIKE '%![' + c.Name + '!]%' ESCAPE '!'
		) THEN 1 ELSE 0 END AS IsPartOfKey
	FROM sys.columns c
	JOIN sys.views v ON v.object_id = c.object_id
	JOIN sys.schemas s ON s.schema_id = v.schema_id
	LEFT JOIN sys.computed_columns cc ON cc.column_id = c.column_id AND cc.object_id = v.object_id
	WHERE v.name = @Table
	AND s.name = @Schema
)
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Function',
    @level1name = N'fGetColumns'
GO

/*
    Product: OCT 5.0
    Object: agent.vResult_FiBuBuchungsjournal
    Type: View
    Author: saxess-software GmbH
    Script Date: 13.12.2019 17:11
*/

DROP VIEW IF EXISTS agent.vResult_FiBuBuchungsjournal;
GO

CREATE VIEW [agent].[vResult_FiBuBuchungsjournal] AS
SELECT
	c.CompanyID AS Mandanten_ID,
	c.CompanyName AS MandantenName,
	c.CompanyID + ' - '+ c.CompanyName AS Mandant,
	f.FinancialYear AS PeriodenFinanzjahr,
	f.FinancialPeriod AS Periode,
	(f.FinancialYear * 10000) + (c.FinancialYearBegin * 100) + f.FinancialPeriod AS Perioden_Key,
	f.AccountingAreaCODE AS Buchungskreis,
	COALESCE(aac.CodeDescription ,f.AccountingAreaCODE) as BuchungskreisName,
	a.AccountID AS Konten_ID,
	a.AccountName AS KontenName,
	COALESCE(at.CodeDescription, a.AccountTypeCODE) AS KontenTyp,
	a.AccountID + ' - ' + a.AccountName AS Konto,
	oa.AccountID AS Gegenkonten_ID,
	oa.AccountName AS GegenkontenName,
	COALESCE(oat.CodeDescription, oa.AccountTypeCODE) AS GegenkontenTyp,
	oa.AccountID + ' - ' + oa.AccountName AS Gegenkonto,
	f.ICPartnerID AS ICPartner_ID,
	f.PostingDate AS Buchungsdatum,
	f.DocumentDate AS Belegdatum,
	f.DocumentNumber AS Belegnummer,
	f.DocumentLine as Belegzeile,
	f.PostingText AS Buchungstext,
	CAST(f.OpeningBalance as money) / 100 AS Saldovortrag,
	CAST(f.Debit as money) / 100 AS Soll,
	CAST(f.Credit as money) / 100 AS Haben,
	CAST((f.OpeningBalance + f.Debit - f.Credit) as money) / 100 AS Saldo_Buchung
FROM result.tFinAccountingJournal f
JOIN result.tCompanies c ON c.CompanyKey = f.CompanyKey
JOIN result.tFinAccounts a ON a.AccountKey = f.AccountKey
LEFT JOIN result.tFinAccounts oa ON oa.AccountKey = f.OffsetAccountKey
LEFT JOIN global.tCodes aac ON aac.CODE = f.AccountingAreaCODE AND aac.Language = 'de'
LEFT JOIN global.tCodes at ON at.CODE = a.AccountTypeCODE AND at.Language = 'de'
LEFT JOIN global.tCodes oat ON oat.CODE = oa.AccountTypeCODE AND oat.Language = 'de'
WHERE f.AccountingAreaCODE LIKE 'FIBU%'
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'View',
    @level1name = N'vResult_FiBuBuchungsjournal'
GO

/*
    Product: OCT 5.0
    Object: agent.vResult_FiBuSaldenliste
    Type: View
    Author: saxess-software GmbH
    Script Date: 13.12.2019 17:11
*/

DROP VIEW IF EXISTS agent.vResult_FiBuSaldenliste;
GO

CREATE VIEW agent.vResult_FiBuSaldenliste AS

	-- Ermittlung ersten und letztes Jahr pro Mandant im Datenbestand
	WITH Mandant_MinMax AS (
		SELECT
			 MIN(FinancialYear) AS MinJahr
			,MAX(FinancialYear) AS MaxJahr
			,CompanyKey
		FROM result.tFinAccountingJournal
		GROUP BY CompanyKey
	),

	-- Alle Perioden aller Finanzjahre
	Zeit AS (
		SELECT DISTINCT
			 [Period] AS FinancialPeriod
			,FinancialYear
		FROM global.tPeriods
	),

	-- alle verwendeten FiBu Codes zur Auflösung in deutsche Namen
	Dimensionen AS (
		SELECT DISTINCT
			 CompanyKey
			,AccountKey
			,AccountingAreaCODE
			,ICPartnerID
		FROM result.tFinAccountingJournal
		WHERE AccountingAreaCODE LIKE 'FIBU%'
	),

	Daten AS (
		SELECT
			 z.FinancialYear
			,z.FinancialPeriod
			,d.CompanyKey
			,d.AccountKey
			,d.AccountingAreaCODE
			,d.ICPartnerID
			,SUM(COALESCE(OpeningBalance, 0))	AS OpeningBalance
			,SUM(COALESCE(Debit, 0))			AS Debit
			,SUM(COALESCE(Credit, 0))			AS Credit
		FROM Zeit z
		CROSS APPLY Dimensionen d
		LEFT JOIN result.tFinAccountingJournal b 
			ON		b.AccountKey			= d.AccountKey
				AND b.FinancialYear			= z.FinancialYear
				AND b.FinancialPeriod		= z.FinancialPeriod
				AND b.AccountingAreaCODE	= d.AccountingAreaCODE
				AND b.ICPartnerID = d.ICPartnerID
		JOIN Mandant_MinMax m
			ON m.CompanyKey = d.CompanyKey

		WHERE z.FinancialYear BETWEEN m.MinJahr AND m.MaxJahr
			AND d.AccountingAreaCODE LIKE 'FIBU%'
		GROUP BY
			z.FinancialYear,
			z.FinancialPeriod,
			d.CompanyKey,
			d.AccountKey,
			d.AccountingAreaCODE,
			d.ICPartnerID
	)

	-- Auslieferung
	SELECT
		 c.CompanyID											AS Mandanten_ID
		,c.CompanyName											AS MandantenName
		,c.CompanyID + ' - '+ c.CompanyName						AS Mandant
		,d.FinancialYear										AS Finanzjahr
		,d.FinancialPeriod										AS Periode
		,d.FinancialYear * 10000 								
			+ c.FinancialYearBegin * 100 						
			+ d.FinancialPeriod									AS Perioden_Key
		,d.AccountingAreaCODE									AS Buchungskreis
		,COALESCE(aac.CodeDescription ,d.AccountingAreaCODE)	AS BuchungskreisName
		,a.AccountID											AS Konten_ID
		,a.AccountName											AS Kontoname
		,COALESCE(atc.CodeDescription, a.AccountTypeCODE)		AS KontenTyp
		,a.AccountID + ' - ' + a.AccountName					AS Konto
		,d.ICPartnerID											AS ICPartner_ID
		,CAST(d.OpeningBalance AS MONEY) / 100					AS Monat_Saldovortrag
		,CAST(d.Debit AS MONEY) / 100							AS Monat_Soll
		,CAST(d.Credit AS MONEY) / 100							AS Monat_Haben
		,CAST(d.Debit - d.Credit AS MONEY) / 100				AS Monat_Saldo
		,CAST(SUM(d.OpeningBalance) 
			OVER(
				PARTITION BY d.FinancialYear, d.AccountingAreaCODE, d.AccountKey, d.ICPartnerID 
				ORDER BY d.FinancialYear, d.FinancialPeriod 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW
				) AS MONEY) / 100								AS Jahr_Saldovortrag
		,CAST(SUM(d.Debit) 
			OVER(
				PARTITION BY d.FinancialYear, d.AccountingAreaCODE, d.AccountKey, d.ICPartnerID 
				ORDER BY d.FinancialYear, d.FinancialPeriod 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW
				) AS MONEY) / 100								AS Jahr_Soll
		,CAST(SUM(d.Credit)
			OVER(
				PARTITION BY d.FinancialYear, d.AccountingAreaCODE, d.AccountKey, d.ICPartnerID 
				ORDER BY d.FinancialYear, d.FinancialPeriod 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW
				) AS MONEY) / 100								AS Jahr_Haben
		,CAST(SUM(d.OpeningBalance + d.Debit - d.Credit)
			OVER(
				PARTITION BY d.FinancialYear, d.AccountingAreaCODE, d.AccountKey, d.ICPartnerID
				ORDER BY d.FinancialYear, d.FinancialPeriod 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW
				) AS MONEY) / 100								AS Jahr_Saldo
	FROM Daten d
		LEFT JOIN result.tfinAccounts a 
			ON a.AccountKey = d.AccountKey
		LEFT JOIN result.tCompanies c
			ON c.CompanyKey = d.CompanyKey
		LEFT JOIN global.tCodes aac
			ON		aac.CODE = d.AccountingAreaCODE
				AND aac.Language = 'de'
		LEFT JOIN global.tCodes atc 
			ON		atc.CODE = a.AccountTypeCODE	
				AND atc.Language = 'de'
	GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'View',
    @level1name = N'vResult_FiBuSaldenliste'
GO

/*
    Product: OCT 5.0
    Object: agent.vResult_KoReBuchungsjournal
    Type: View
    Author: saxess-software GmbH
    Script Date: 13.12.2019 17:11
*/

DROP VIEW IF EXISTS agent.vResult_KoReBuchungsjournal;
GO


CREATE VIEW [agent].[vResult_KoReBuchungsjournal] AS
SELECT
	c.CompanyID AS Mandanten_ID,
	c.CompanyName AS MandantenName,
	c.CompanyID + ' - ' + c.CompanyName AS Mandant,
	f.FinancialYear AS PeriodenFinanzjahr,
	f.FinancialPeriod AS Periode,
	(f.FinancialYear * 10000) + (c.FinancialYearBegin * 100) + f.FinancialPeriod AS Perioden_Key,
	f.AccountingAreaCODE AS Buchungskreis,
	COALESCE(aac.CodeDescription ,f.AccountingAreaCODE) as BuchungskreisName,
	a.AccountID AS Konten_ID,
	a.AccountName AS KontenName,
	a.AccountID + ' - ' + a.AccountName AS Konto,
	COALESCE(atc.CodeDescription, a.AccountTypeCODE) AS KontenTyp,
	oa.AccountID AS Gegenkonten_ID,
	oa.AccountName AS GegenkontenName,
	COALESCE(oatc.CodeDescription, oa.AccountTypeCODE) AS GegenkontenTyp,
	oa.AccountID + ' - ' + oa.AccountName AS Gegenkonto,
	co1.CostObjectID AS Kostenstellen_ID,
	co1.CostObjectName AS KostenstellenName,
	COALESCE(co1c.CodeDescription, co1.CostObjectGroupCode) AS KostenstellenTyp,
	co1.CostObjectID + ' - ' + co1.CostObjectName AS Kostenstelle,
	co2.CostObjectID AS Kostentraeger_ID,
	co2.CostObjectName AS KostentraegerName,
	COALESCE(co2c.CodeDescription, co2.CostObjectGroupCode) AS KostentraegerTyp,
	co2.CostObjectID + ' - ' + co2.CostObjectName AS Kostentraeger,
	f.ICPartnerID AS ICPartner_ID,
	f.PostingDate AS Buchungsdatum,
	f.DocumentDate AS Belegdatum,
	f.DocumentNumber AS Belegnummer,
	f.DocumentLine as Belegzeile,
	f.PostingText AS Buchungstext,
	CAST(f.OpeningBalance as money) / 100 AS Saldovortrag,
	CAST(f.Debit as money) / 100 AS Soll,
	CAST(f.Credit as money) / 100 AS Haben,
	CAST((f.OpeningBalance + f.Debit - f.Credit) as money) / 100 AS Saldo_Buchung
FROM result.tFinAccountingJournal f
JOIN result.tCompanies c ON c.CompanyKey = f.CompanyKey
JOIN result.tFinAccounts a ON a.AccountKey = f.AccountKey
LEFT JOIN result.tFinAccounts oa ON oa.AccountKey = f.OffsetAccountKey
LEFT JOIN result.tFinCostObjects co1 ON co1.CostObjectKey = f.CostObject1Key
LEFT JOIN result.tFinCostObjects co2 ON co2.CostObjectKey = f.CostObject2Key
LEFT JOIN global.tCodes aac ON aac.CODE = f.AccountingAreaCODE AND aac.Language = 'de'
LEFT JOIN global.tCodes atc ON atc.CODE = a.AccountTypeCODE AND atc.Language = 'de'
LEFT JOIN global.tCodes oatc ON oatc.CODE = oa.AccountTypeCODE AND oatc.Language = 'de'
LEFT JOIN global.tCodes co1c ON co1c.CODE = co1.CostObjectGroupCODE AND co1c.Language = 'de'
LEFT JOIN global.tCodes co2c ON co2c.CODE = co2.CostObjectGroupCODE AND co2c.Language = 'de'
WHERE f.AccountingAreaCODE LIKE 'KORE%'
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'View',
    @level1name = N'vResult_KoReBuchungsjournal'
GO

/*
    Product: OCT 5.0
    Object: agent.vResult_KoReSaldenliste
    Type: View
    Author: saxess-software GmbH
    Script Date: 13.12.2019 17:11
*/

DROP VIEW IF EXISTS agent.vResult_KoReSaldenliste;
GO

CREATE VIEW [agent].[vResult_KoReSaldenliste] AS
SELECT
	c.CompanyID AS Mandanten_ID,
	c.CompanyName AS MandantenName,
	c.CompanyID + ' - '+ c.CompanyName AS Mandant,
	f.FinancialYear AS PeriodenFinanzjahr,
	f.FinancialPeriod AS Periode,
	(f.FinancialYear * 10000) + (c.FinancialYearBegin * 100) + f.FinancialPeriod AS Perioden_Key,
	f.AccountingAreaCODE AS Buchungskreis,
	COALESCE(aac.CodeDescription ,f.AccountingAreaCODE) as BuchungskreisName,
	a.AccountID AS Konten_ID,
	a.AccountName AS KontenName,
	COALESCE(atc.CodeDescription, a.AccountTypeCODE) AS KontenTyp,
	a.AccountID + ' - ' + a.AccountName AS Konto,
	co1.CostObjectID AS Kostenstellen_ID,
	co1.CostObjectName AS KostenstellenName,
	COALESCE(co1c.CodeDescription, co1.CostObjectGroupCode) AS KostenstellenTyp,
	co1.CostObjectID + ' - ' + co1.CostObjectName AS Kostenstelle,
	co2.CostObjectID AS Kostentraeger_ID,
	co2.CostObjectName AS KostentraegerName,
	COALESCE(co2c.CodeDescription, co2.CostObjectGroupCode) AS KostentraegerTyp,
	co2.CostObjectID + ' - ' + co2.CostObjectName AS Kostentraeger,
	f.ICPartnerID AS ICPartner_ID,
	SUM(CAST(f.OpeningBalance as money) / 100) AS Saldovortrag,
	SUM(CAST(f.Debit as money) / 100) AS MonatSoll,
	SUM(CAST(f.Credit as money) / 100) AS MonatHaben,
	SUM(CAST((f.OpeningBalance + f.Debit - f.Credit) as money) / 100) AS Saldo
FROM result.tFinAccountingJournal f
JOIN result.tCompanies c ON c.CompanyKey = f.CompanyKey
JOIN result.tFinAccounts a ON a.AccountKey = f.AccountKey
LEFT JOIN result.tFinCostObjects co1 ON co1.CostObjectKey = f.CostObject1Key
LEFT JOIN result.tFinCostObjects co2 ON co2.CostObjectKey = f.CostObject2Key
LEFT JOIN global.tCodes aac ON aac.CODE = f.AccountingAreaCODE AND aac.Language = 'de'
LEFT JOIN global.tCodes atc ON atc.CODE = a.AccountTypeCODE AND atc.Language = 'de'
LEFT JOIN global.tCodes co1c ON co1c.CODE = co1.CostObjectGroupCODE AND co1c.Language = 'de'
LEFT JOIN global.tCodes co2c ON co2c.CODE = co2.CostObjectGroupCODE AND co2c.Language = 'de'
WHERE f.AccountingAreaCODE LIKE 'KORE%'
GROUP BY
  	c.CompanyID,
	c.CompanyName,
	c.FinancialYearBegin,
	f.FinancialYear,
	f.FinancialPeriod,
	f.AccountingAreaCODE,
	COALESCE(aac.CodeDescription ,f.AccountingAreaCODE),
	a.AccountID,
	a.AccountName,
	COALESCE(atc.CodeDescription, a.AccountTypeCODE),
	co1.CostObjectID,
	co1.CostObjectName,
	COALESCE(co1c.CodeDescription, co1.CostObjectGroupCode),
	co2.CostObjectID,
	co2.CostObjectName,
	COALESCE(co2c.CodeDescription, co2.CostObjectGroupCode),
	f.ICPartnerID
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'View',
    @level1name = N'vResult_KoReSaldenliste'
GO


/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODEL_dFactories]'))
DROP VIEW [sx_pf_DATAMODEL_dFactories]
GO

CREATE VIEW sx_pf_DATAMODEL_dFactories

AS

	SELECT 
			 FactoryKey
			,dF.FactoryID
			,NameShort		AS FactoryNameShort
			,dF.FactoryID + ' ' + NameShort AS FactoryIDName
			,NameLong		AS FactoryNameLong
			,CommentUser	AS FactoryCommentUser
			,CommentDev	AS FactoryCommentDev
			,ResponsiblePerson AS FactoryResponsiblePerson
			,ImageName			AS FactoryImageName

	FROM   planning.tdFactories dF

			INNER JOIN 
				(SELECT FactoryID FROM planning.trUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID) vUR
				ON dF.FactoryID = vUR.FactoryID
				

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODEL_dProductLines]'))
DROP VIEW [sx_pf_DATAMODEL_dProductLines]
GO

CREATE VIEW sx_pf_DATAMODEL_dProductLines 

AS

	SELECT 
			 ProductLineKey
			,dPL.ProductLineID
			,dPL.FactoryID
			,NameShort	AS ProductLineNameShort
			,dPL.ProductLineID + ' ' + NameShort AS ProductLineIDName
			,NameLong	AS ProductLineNameLong
			,CommentUser AS ProductLineCommentUser
			,CommentDev			AS ProductLineCommentDev
			,ResponsiblePerson	AS ProductLineResponsiblePerson
			,ImageName			AS ProductLineImageName
			,DefaultTemplate
			,GlobalAttributeAlias1
			,GlobalAttributeAlias2
			,GlobalAttributeAlias3
			,GlobalAttributeAlias4
			,GlobalAttributeAlias5
			,GlobalAttributeAlias6
			,GlobalAttributeAlias7
			,GlobalAttributeAlias8
			,GlobalAttributeAlias9
			,GlobalAttributeAlias10
			,GlobalAttributeAlias11
			,GlobalAttributeAlias12
			,GlobalAttributeAlias13
			,GlobalAttributeAlias14
			,GlobalAttributeAlias15
			,GlobalAttributeAlias16
			,GlobalAttributeAlias17
			,GlobalAttributeAlias18
			,GlobalAttributeAlias19
			,GlobalAttributeAlias20
			,GlobalAttributeAlias21
			,GlobalAttributeAlias22
			,GlobalAttributeAlias23
			,GlobalAttributeAlias24
			,GlobalAttributeAlias25
	FROM   planning.tdProductLines dPL

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID

GO
				

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODEL_dProducts]'))
DROP VIEW [sx_pf_DATAMODEL_dProducts]
GO

CREATE VIEW sx_pf_DATAMODEL_dProducts

AS

	SELECT 
			 ProductKey
			,ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort	AS ProductNameShort
			,ProductID + ' ' + NameShort AS ProductIDName
			,NameLong	AS ProductNameLong
			,CommentUser	AS ProductCommentUser
			,CommentDev	AS ProductCommentDev
			,ResponsiblePerson	AS ProductResponsiblePerson
			,ImageName			AS ProductImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
	FROM   planning.tdProducts dP

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dP.FactoryID = vUR.FactoryID AND
					dP.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODEL_dValueSeries]'))
DROP VIEW [sx_pf_DATAMODEL_dValueSeries]
GO

CREATE VIEW sx_pf_DATAMODEL_dValueSeries 

AS

		SELECT 
			 ValueSeriesKey
			,ProductID
			,dVS.ProductLineID
			,dVS.FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort AS ValueSeriesNameShort
			,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
			,NameLong AS ValueSeriesNameLong
			,CommentUser AS ValueSeriesCommentUser
			,CommentDev AS ValueSeriesCommentDev
			,ImageName AS ValueSeriesImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
		FROM   planning.tdValueSeries dVS 

				INNER JOIN 
					(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
					ON
						dVS.FactoryID = vUR.FactoryID AND
						dVS.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODEL_fValues]'))
DROP VIEW [sx_pf_DATAMODEL_fValues]
GO

CREATE VIEW sx_pf_DATAMODEL_fValues

AS

	SELECT 
			 fV.ProductKey
			,fV.ProductLineKey
			,fV.FactoryKey
			,fV.ValueSeriesKey
			,fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,CAST(LEFT(TimeID,4) AS int) AS [Year]
			,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Month]
			,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
			,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
			,ValueFormula
			,CAST(ValueInt AS Money) / Coalesce(dVS.Scale,1) AS Value
			,ValueText
			,ValueComment
					
	FROM   planning.tfValues fV 

			LEFT JOIN planning.tdValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = SYSTEM_USER AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					fV.FactoryID = vUR.FactoryID AND
					fV.ProductLineID = vUR.ProductLineID


			WHERE ValueInt != 0 OR ( ValueText != ''  AND ValueText != '-')
GO
			

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODELCLOUD_dFactories]'))
DROP VIEW [sx_pf_DATAMODELCLOUD_dFactories]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dFactories

AS

	SELECT 
			 FactoryKey
			,dF.FactoryID
			,NameShort		AS FactoryNameShort
			,dF.FactoryID + ' ' + NameShort AS FactoryIDName
			,NameLong		AS FactoryNameLong
			,CommentUser	AS FactoryCommentUser
			,CommentDev	AS FactoryCommentDev
			,ResponsiblePerson AS FactoryResponsiblePerson
			,ImageName			AS FactoryImageName

	FROM   planning.tdFactories dF

			INNER JOIN 
				(SELECT FactoryID FROM planning.trUserRights WHERE Username = HOST_NAME() AND [Right] IN ('Write','Read') GROUP BY FactoryID) vUR
				ON dF.FactoryID = vUR.FactoryID
				

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODELCLOUD_dProductLines]'))
DROP VIEW [sx_pf_DATAMODELCLOUD_dProductLines]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dProductLines 

AS

	SELECT 
			 ProductLineKey
			,dPL.ProductLineID
			,dPL.FactoryID
			,NameShort	AS ProductLineNameShort
			,dPL.ProductLineID + ' ' + NameShort AS ProductLineIDName
			,NameLong	AS ProductLineNameLong
			,CommentUser AS ProductLineCommentUser
			,CommentDev			AS ProductLineCommentDev
			,ResponsiblePerson	AS ProductLineResponsiblePerson
			,ImageName			AS ProductLineImageName
			,DefaultTemplate
			,GlobalAttributeAlias1
			,GlobalAttributeAlias2
			,GlobalAttributeAlias3
			,GlobalAttributeAlias4
			,GlobalAttributeAlias5
			,GlobalAttributeAlias6
			,GlobalAttributeAlias7
			,GlobalAttributeAlias8
			,GlobalAttributeAlias9
			,GlobalAttributeAlias10
			,GlobalAttributeAlias11
			,GlobalAttributeAlias12
			,GlobalAttributeAlias13
			,GlobalAttributeAlias14
			,GlobalAttributeAlias15
			,GlobalAttributeAlias16
			,GlobalAttributeAlias17
			,GlobalAttributeAlias18
			,GlobalAttributeAlias19
			,GlobalAttributeAlias20
			,GlobalAttributeAlias21
			,GlobalAttributeAlias22
			,GlobalAttributeAlias23
			,GlobalAttributeAlias24
			,GlobalAttributeAlias25
	FROM   planning.tdProductLines dPL

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID

GO
				

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODELCLOUD_dProducts]'))
DROP VIEW [sx_pf_DATAMODELCLOUD_dProducts]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dProducts

AS

	SELECT 
			 ProductKey
			,ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort	AS ProductNameShort
			,ProductID + ' ' + NameShort AS ProductIDName
			,NameLong	AS ProductNameLong
			,CommentUser	AS ProductCommentUser
			,CommentDev	AS ProductCommentDev
			,ResponsiblePerson	AS ProductResponsiblePerson
			,ImageName			AS ProductImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
	FROM   planning.tdProducts dP

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					dP.FactoryID = vUR.FactoryID AND
					dP.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODELCLOUD_dValueSeries]'))
DROP VIEW [sx_pf_DATAMODELCLOUD_dValueSeries]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_dValueSeries 

AS

		SELECT 
			 ValueSeriesKey
			,ProductID
			,dVS.ProductLineID
			,dVS.FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort AS ValueSeriesNameShort
			,ValueSeriesID + ' ' + NameShort AS ValueSeriesIDName
			,NameLong AS ValueSeriesNameLong
			,CommentUser AS ValueSeriesCommentUser
			,CommentDev AS ValueSeriesCommentDev
			,ImageName AS ValueSeriesImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
		FROM   planning.tdValueSeries dVS 

				INNER JOIN 
					(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
					ON
						dVS.FactoryID = vUR.FactoryID AND
						dVS.ProductLineID = vUR.ProductLineID

GO

/*
View to deliver dimensional data to the enduser with direct database access
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2016
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_DATAMODELCLOUD_fValues]'))
DROP VIEW [sx_pf_DATAMODELCLOUD_fValues]
GO

CREATE VIEW sx_pf_DATAMODELCLOUD_fValues

AS

	SELECT 
			 fV.ProductKey
			,fV.ProductLineKey
			,fV.FactoryKey
			,fV.ValueSeriesKey
			,fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,CAST(LEFT(TimeID,4) AS int) AS [Year]
			,CAST(Right(LEFT(TimeID,6),2) AS int) AS [Month]
			,TRY_CONVERT(Date,CAST(TimeID AS nvarchar),102) AS [Date]
			,TRY_CONVERT(Datetime,CAST(TimeID AS nvarchar),102) AS [Datetime]
			,ValueFormula
			,CAST(ValueInt AS Money) / Coalesce(dVS.Scale,1) AS Value
			,ValueText
			,ValueComment
					
	FROM   planning.tfValues fV 

			LEFT JOIN planning.tdValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey

			INNER JOIN 
				(SELECT FactoryID,ProductLineID FROM planning.trUserRights WHERE Username = HOST_NAME () AND [Right] IN ('Write','Read') GROUP BY FactoryID,ProductLineID) vUR
				ON
					fV.FactoryID = vUR.FactoryID AND
					fV.ProductLineID = vUR.ProductLineID

			WHERE ValueInt != 0 OR ( ValueText != ''  AND ValueText != '-')
GO
			

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_BALANCE
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_RESULT_Balance]'))
DROP VIEW [sx_pf_RESULT_Balance]
GO


CREATE VIEW sx_pf_RESULT_Balance 

AS


	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT
					 *
				 FROM planning.trUserRights 
				 WHERE 
					Username = ORIGINAL_LOGIN() AND
					[Right] IN ('Write','Read') AND
					FactoryID <> '' AND
					ProductLineID <> ''
				) vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.FactoryID != 'ZT' AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0

GO

GRANT SELECT ON OBJECT ::[sx_pf_RESULT_Balance] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
04/2018

Testcall
SELECT * FROM sx_pf_RESULT_Cash

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'sx_pf_RESULT_Cash'))
DROP VIEW sx_pf_RESULT_Cash
GO


CREATE VIEW sx_pf_RESULT_Cash

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson
		 ,dP.Template								AS Template

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID <> 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[sx_pf_RESULT_Cash] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_Numeric

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_RESULT_Numeric]'))
DROP VIEW [sx_pf_RESULT_Numeric]
GO


CREATE VIEW sx_pf_RESULT_Numeric

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[sx_pf_RESULT_Numeric] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
07/2017

Testcall
SELECT * FROM sx_pf_RESULT_Profit

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_RESULT_Profit]'))
DROP VIEW [sx_pf_RESULT_Profit]
GO


CREATE VIEW sx_pf_RESULT_Profit

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect<0,gVE.ProfitLossEffect,0)) AS Value_CostAbsolut

		 ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * ABS(IIF(gVE.ProfitLossEffect>0,gVE.ProfitLossEffect,0)) AS Value_EarningAbsolut

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson
		 ,dP.Template								AS Template

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::[sx_pf_RESULT_Profit] TO pf_PlanningFactoryUser;

GO


/*
View to deliver analytic result data to the enduser
Only for direct use with SQL Server Connection
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
01/2018

Testcall
SELECT * FROM sx_pf_RESULT_Text

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_RESULT_Text]'))
DROP VIEW [sx_pf_RESULT_Text]
GO


CREATE VIEW sx_pf_RESULT_Text

AS

	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000														AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100													AS Month
		 ,	  RIGHT('0'+ CAST((TRY_CAST(fV.TimeID AS Integer) % 10000) /100 AS NVARCHAR(255)),2)
			+ '/' 
			+ CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS NVARCHAR(255))								AS MonthYear
			
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,   TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) 
			+'-'
			+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00')	AS YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Effect

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = ORIGINAL_LOGIN() AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND
		dVS.[IsNumeric] = 0 AND
		fV.FactoryID != 'ZT'

		
GO
  
GRANT SELECT ON OBJECT ::sx_pf_RESULT_Text TO pf_PlanningFactoryUser;

GO


/*
View for analysing the LogEntrys of PlanningFactory API

11/2015 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
*/


IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[sx_pf_vLogEntrys]'))
DROP VIEW sx_pf_vLogEntrys
GO

CREATE VIEW sx_pf_vLogEntrys AS 

SELECT 
	 @@Servername AS Servername
	,DB_NAME()  AS ClusterName
	,SQLSystemUser
	,tL.Username AS ServiceUsername
	,IIF(tL.Username = 'SQL','Active',rU2.Status) AS ServiceUserStatus
	,TransactUsername
	,rU1.Status AS TransactUserStatus
	,ProcedureName
	,ParameterString
	,EffectedRows
	,ReturnCode
	,TimestampCall
	,TimestampReturn
	,CASE
		WHEN DATEDIFF(SECOND,TimestampCall,TimestampReturn) < 10 THEN
			DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn)
		ELSE
			10000
	END AS DurationInMilliseconds
	
	--,DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn) AS DurationInMilliseconds
	,DATEDIFF(SECOND,TimestampCall,TimestampReturn) AS DurationInSeconds
	,Comment
	,ProcessCode

FROM 
	planning.tAPI_Log tL 
		LEFT JOIN planning.trUser rU1 ON tL.TransactUsername = rU1.Username
		LEFT JOIN planning.trUser rU2 ON tL.UserName = rU2.UserName

GO

/*
View to pivot ProcessLog to table view
Gerd Tautenhahn for Saxess Software GmbH
07/2018 DataFactory 5.0

*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'sx_pf_vProcess_Start_End'))
DROP VIEW sx_pf_vProcess_Start_End
GO

CREATE VIEW sx_pf_vProcess_Start_End

AS

SELECT 
		 PivotTable.MainProcess
		,PivotTable.SubProcess
		,PivotTable.Step
		,MIN(PivotTable.Start) AS Start
		,MAX(PivotTable.[End]) AS [End]
		,CAST(MAX(PivotTable.[End]) - MIN(PivotTable.Start) AS TIME) AS Duration

FROM planning.tProcess_Log PL

PIVOT
	(	
		-- List of the needed ValueSeries in SquareBrackets
		MAX(Timestamp) FOR Start_End_Info IN ([Start],[End])
	) AS PivotTable

GROUP BY 
	 PivotTable.MainProcess
	,PivotTable.SubProcess
	,PivotTable.Step

GO

-- No GRANT as only used by db_owners etc.
/*
    Product: OCT 5.0
    Object: agent.pCreateJSONViewINT
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pCreateJSONViewINT;
GO

CREATE PROCEDURE [agent].[pCreateJSONViewINT] (
	@Username		NVARCHAR(255),		-- used for logging
	@Schema			AS NVARCHAR(50),
	@Table			AS NVARCHAR(50),
	@UseRowKey		INT = 0,
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
)
AS
BEGIN
	DECLARE @KeyCol NVARCHAR(50);
	DECLARE @ColName NVARCHAR(50);
	DECLARE @ColType int;
	DECLARE @SQLDrop NVARCHAR(MAX);
	DECLARE @SQLHeader NVARCHAR(MAX);
	DECLARE @SQL NVARCHAR(MAX);
	DECLARE @SQLFooter NVARCHAR(MAX);
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		-- Temp Tabelle mit JSON Spalten erstellen
		DROP TABLE IF EXISTS ##JSONCols;
		CREATE TABLE ##JSONCols (ColName NVARCHAR(50), ColType int);
		EXEC ('INSERT INTO ##JSONCols SELECT DISTINCT b.[Key], b.Type FROM ' + @Schema + '.' + @Table + ' a CROSS APPLY OPENJSON(a.CustomValuesJSON) b');

		-- Schlüsselspalte auslesen
		SELECT @KeyCol = ColName
		FROM system.fGetColumns(@Schema, @Table)
		WHERE (@UseRowKey = 0 AND IsCalculatedKey = 1) OR (@UseRowKey = 1 AND ColName = 'RowKey');

		-- Skript definieren
		SET @SQLDrop = 'DROP VIEW IF EXISTS agent.vJSON_' + @Table;
		SET @SQLHeader = 'CREATE VIEW agent.vJSON_' + @Table + ' AS
			SELECT
			a.' + @KeyCol;
		SET @SQLFooter = '
		FROM ' + @Schema + '.' + @Table + ' a
		';
		SET @SQL = '';

		IF EXISTS(SELECT 1 FROM ##JSONCols)
		BEGIN
			SET @SQLFooter = @SQLFooter + ' CROSS APPLY OPENJSON (a.CustomValuesJSON)
			WITH (';

			-- JSON Spalten durchgehen und Skript dynamisch ergänzen
			DECLARE MyCursor CURSOR FOR
				SELECT ColName, ColType
				FROM ##JSONCols
				WHERE ColType IN (1,2,3)
			OPEN MyCursor
			FETCH MyCursor INTO @ColName, @ColType

			WHILE @@FETCH_STATUS = 0
			BEGIN
				SET @SQL = @SQL + ',
				' + @Colname + ' AS ' + @ColName + '_INT';

				SET @SQLFooter = @SQLFooter + '
				' + @ColName + ' ' +
				CASE @ColType
					WHEN 1 THEN 'nvarchar(255)'
					WHEN 2 THEN 'float'
					WHEN 3 THEN 'bit'
				END + ','

				FETCH MyCursor INTO @ColName, @ColType
			END

			CLOSE MyCursor
			DEALLOCATE MyCursor
		
			SET @SQL = @SQLHeader + @SQL + LEFT(@SQLFooter, LEN(@SQLFooter) - 1) + ')';
		END
		ELSE
		BEGIN
			-- keine Custom Spalten
			SET @SQL = @SQLHeader + @SQL + @SQLFooter;
		END;

		--PRINT @SQL;
	
		EXEC (@SQLDrop);
		EXEC (@SQL);

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pCreateJSONViewINT', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pCreateJSONViewINT'
GO

/*
    Product: OCT 5.0
    Object: agent.pCreateJSONViewUSER
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pCreateJSONViewUSER;
GO

CREATE PROCEDURE [agent].[pCreateJSONViewUSER] (
	@Username		NVARCHAR(255),		-- used for logging
	@Schema			AS NVARCHAR(50),
	@Table			AS NVARCHAR(50),
	@UseRowKey		INT = 0,
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
)
AS
BEGIN
	DECLARE @KeyCol NVARCHAR(50);
	DECLARE @ColName NVARCHAR(50);
	DECLARE @ColType int;
	DECLARE @SQLDrop NVARCHAR(MAX);
	DECLARE @SQLHeader NVARCHAR(MAX);
	DECLARE @SQL NVARCHAR(MAX);
	DECLARE @SQLFooter NVARCHAR(MAX);
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		-- Temp Tabelle mit JSON Spalten erstellen
		DROP TABLE IF EXISTS ##JSONCols;
		CREATE TABLE ##JSONCols (ColName NVARCHAR(50), ColType int);

		EXEC ('INSERT INTO ##JSONCols SELECT DISTINCT b.[Key], b.Type FROM ' + @Schema + '.' + @Table + ' a CROSS APPLY OPENJSON(a.CustomValuesJSON) b');

		-- Schlüsselspalte auslesen
		SELECT @KeyCol = ColName
		FROM system.fGetColumns(@Schema, @Table)
		WHERE IsCalculatedKey = 1;

		-- Skript definieren
		SET @SQLDrop = 'DROP VIEW IF EXISTS agent.vJSONUser_' + @Table;
		SET @SQLHeader = 'CREATE VIEW agent.vJSONUser_' + @Table + ' AS
		SELECT
			a.' + @KeyCol;
		SET @SQLFooter = '
		FROM ' + @Schema + '.' + @Table + ' a
		';
		SET @SQL = '';

		IF EXISTS(SELECT 1 FROM ##JSONCols)
		BEGIN
			SET @SQLFooter = @SQLFooter + ' CROSS APPLY OPENJSON (a.CustomValuesJSON)
			WITH (';

			-- JSON Spalten durchgehen und Skript dynamisch ergänzen
			DECLARE MyCursor CURSOR FOR
				SELECT ColName, ColType
				FROM ##JSONCols
				WHERE ColType IN (1,2,3)
			OPEN MyCursor
			FETCH MyCursor INTO @ColName, @ColType

			WHILE @@FETCH_STATUS = 0
			BEGIN
				SET @SQL = @SQL + ',
				' + @Colname + ' AS ' + @ColName + '_USER';

				SET @SQLFooter = @SQLFooter + '
				' + @ColName + ' ' +
				CASE @ColType
					WHEN 1 THEN 'nvarchar(255)'
					WHEN 2 THEN 'float'
					WHEN 3 THEN 'bit'
					ELSE 'bla'
				END + ','

				FETCH MyCursor INTO @ColName, @ColType
			END

			CLOSE MyCursor
			DEALLOCATE MyCursor
		
			SET @SQL = @SQLHeader + @SQL + LEFT(@SQLFooter, LEN(@SQLFooter) - 1) + ')';
		END
		ELSE
		BEGIN
			-- keine Custom Spalten
			SET @SQL = @SQLHeader + @SQL + @SQLFooter;
		END;

		--PRINT @SQL;

		EXEC (@SQLDrop);
		EXEC (@SQL);

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pCreateJSONViewUSER', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pCreateJSONViewUSER'
GO

/*
    Product: OCT 5.0
    Object: agent.pDeleteIntegrationValues
    Type: Stored Procedure
    Author: saxess-software GmbH

	Testcall

	EXEC agent.pDeleteIntegrationValues
		 @Solution			= Fin
		,@CompanyKeys		= '1'
		,@PeriodFrom		= 202001
		,@PeriodTo			= 202012
		,@MasterDataOnly	= 0
		,@Username			= 'SQL'
		,@ProcessID			= '0'
		,@SubProcessID		= '0'
		,@IsDeltaloadBool	= 0
		
*/

DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValues;
GO

CREATE PROCEDURE agent.pDeleteIntegrationValues
     @Solution			NVARCHAR(50)
    ,@CompanyKeys		NVARCHAR(MAX)			-- delimited by ;
    ,@PeriodFrom		INT
    ,@PeriodTo			INT
    ,@MasterDataOnly	BIT				= 0
    ,@Username			NVARCHAR(255)			-- used for logging
	,@ProcessID			NVARCHAR(50)	= '0'	-- used for logging
	,@SubProcessID		NVARCHAR(50)	= '0'	-- used for logging
	,@IsDeltaloadBool	BIT				= 0
AS
BEGIN
    DECLARE @SQLCommand			NVARCHAR(MAX)
    DECLARE @CompanyKey			NVARCHAR(100);
    DECLARE @DataSourceKey		int;
    DECLARE @CompanyID			NVARCHAR(50);
    DECLARE @ResultCode			INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY

		BEGIN TRANSACTION pDeleteIntegrationValues;

		DECLARE CompanyKeyCursor CURSOR FOR
		SELECT value AS CompanyKey
		FROM STRING_SPLIT(@CompanyKeys, ';')

		OPEN CompanyKeyCursor
		FETCH CompanyKeyCursor INTO @CompanyKey
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SELECT
				@DataSourceKey = Reverse(ParseName(Replace(Reverse(@CompanyKey), '|', '.'), 1)),
				@CompanyID = Reverse(ParseName(Replace(Reverse(@CompanyKey), '|', '.'), 2));

			-- dynamically execute delete procedure for the selected solution, Company and Period
			SET @SQLCommand = 'EXEC agent.pDeleteIntegrationValues' + @Solution +
				' @MasterDataOnly = ' + CAST(@MasterDataOnly AS NCHAR(1)) +
				', @DataSourceKey = ' + CAST(@DataSourceKey AS NCHAR(10)) + 
				', @CompanyID = ''' + @CompanyID + '''' + 
				', @PeriodFrom = ' + CAST(@PeriodFrom AS NCHAR(6)) + 
				', @PeriodTo = ' + CAST(@PeriodTo AS NCHAR(6)) +
				', @Username = ''' + @Username + '''' +
				', @ProcessID = ''' + @ProcessID + '''' +
				', @SubProcessID = ''' + @SubProcessID + '''' +
				', @IsDeltaloadBool = ' + CAST(@IsDeltaloadBool AS NCHAR(1)) +
				';';
			EXEC (@SQLCommand);

			FETCH CompanyKeyCursor INTO @CompanyKey
		END
		CLOSE CompanyKeyCursor
		DEALLOCATE CompanyKeyCursor

		SET @ResultCode = 200;

		COMMIT TRANSACTION pDeleteIntegrationValues;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		ROLLBACK TRANSACTION pDeleteIntegrationValues;

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Data Deletion',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pDeleteIntegrationValues', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDeleteIntegrationValues', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'This stored procedure is called before new data is imported into the integration tables.',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDeleteIntegrationValues'
GO

/*
    Product: OCT 5.0
    Object: agent.pDeleteIntegrationValuesDebCred
    Type: Stored Procedure
    Author: saxess-software GmbH

	Testcall

	EXEC agent.pDeleteIntegrationValuesDebCred
		 @DataSourceKey	   = 1
		,@CompanyID		   = '1'
		,@PeriodFrom	   = 202001
		,@PeriodTo		   = 202012
		,@MasterDataOnly   = 0
		,@Username		   = 'SQL'
		,@ProcessID		   = '0'
		,@SubProcessID	   = '0'
		,@IsDeltaloadBool  = 0

*/

DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValuesDebCred;
GO

CREATE PROCEDURE agent.pDeleteIntegrationValuesDebCred
     @DataSourceKey		INT
    ,@CompanyID			NVARCHAR(50)
    ,@PeriodFrom		INT
    ,@PeriodTo			INT
    ,@MasterDataOnly	BIT
    ,@Username			NVARCHAR(255)			-- used for logging
	,@ProcessID			NVARCHAR(50)	= N'0'	-- used for logging
	,@SubProcessID		NVARCHAR(50)	= N'0'	-- used for logging
	,@IsDeltaloadBool	BIT				= 0
AS
BEGIN
	DECLARE @ResultCode			INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY

		DELETE FROM integration.tDebCredAccounts
		WHERE 1=1
			AND DataSourceKey = @DataSourceKey
			AND CompanyID = @CompanyID;

		IF @MasterDataOnly = 0 AND @IsDeltaloadBool = 0
		BEGIN
			DELETE FROM integration.tDebCredJournal
			WHERE 1=1
				AND DataSourceKey = @DataSourceKey
				AND CompanyID = @CompanyID
				AND ( 
						FinancialYear * 100 + FinancialPeriod BETWEEN @PeriodFrom AND @PeriodTo
						OR ItemStatusCODE = 'OPEN'
					);
		END

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Data Deletion',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pDeleteIntegrationValues', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDeleteIntegrationValuesDebCred', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'This stored procedure is used to delete data from all tables of the solution "DebCred".',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDeleteIntegrationValuesDebCred'
GO

/*
    Product: OCT 5.0
    Object: agent.pDeleteIntegrationValuesFin
    Type: Stored Procedure
    Author: saxess-software GmbH

	Testcall

	EXEC agent.pDeleteIntegrationValuesFin
		 @DataSourceKey	   = 1
		,@CompanyID		   = '1'
		,@PeriodFrom	   = 202001
		,@PeriodTo		   = 202012
		,@MasterDataOnly   = 0
		,@Username		   = 'SQL'
		,@ProcessID		   = '0'
		,@SubProcessID	   = '0'
		,@IsDeltaloadBool  = 0

*/

DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValuesFin;
GO

CREATE PROCEDURE agent.pDeleteIntegrationValuesFin
     @DataSourceKey		INT
    ,@CompanyID			NVARCHAR(50)
    ,@PeriodFrom		INT
    ,@PeriodTo			INT
    ,@MasterDataOnly	BIT
    ,@Username			NVARCHAR(255)			-- used for logging
	,@ProcessID			NVARCHAR(50)	= N'0'	-- used for logging
	,@SubProcessID		NVARCHAR(50)	= N'0'	-- used for logging
	,@IsDeltaloadBool	BIT				= 0
AS
BEGIN
	DECLARE @ResultCode			INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY

		DELETE FROM integration.tFinAccounts
		WHERE 1=1
			AND DataSourceKey = @DataSourceKey
			AND CompanyID = @CompanyID;

		DELETE FROM integration.tFinCostObjects
		WHERE 1=1
			AND DataSourceKey = @DataSourceKey
			AND CompanyID = @CompanyID;

		IF @MasterDataOnly = 0 AND @IsDeltaloadBool = 0
		BEGIN
			DELETE FROM integration.tFinAccountingJournal
			WHERE 1=1
				AND DataSourceKey = @DataSourceKey
				AND CompanyID = @CompanyID
				AND FinancialYear * 100 + FinancialPeriod BETWEEN @PeriodFrom AND @PeriodTo;
		END

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Data Deletion',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pDeleteIntegrationValues', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDeleteIntegrationValuesFin', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'This stored procedure is used to delete data from all tables of the solution "Fin".',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDeleteIntegrationValuesFin'
GO

/*
    Product: OCT 5.0
    Object: agent.pDoMaintenance
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 02.08.2019 16:29
*/

DROP PROCEDURE IF EXISTS agent.pDoMaintenance;
GO


CREATE PROCEDURE [agent].[pDoMaintenance]
	@Username		NVARCHAR(255),		-- used for logging
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		BEGIN TRANSACTION pDoMaintenance;
	-- Shrink Log File (needs db_owner rights, which are not possible here)
	-- DBCC SHRINKFILE(2,1) WITH NO_INFOMSGS;

		SET @ResultCode = 200;

		COMMIT TRANSACTION pDoMaintenance;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		ROLLBACK TRANSACTION pDoMaintenance;

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDoMaintenance', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDoMaintenance'
GO

/*
    Product: OCT 5.0
    Object: agent.pDoMaintenanceDebCred
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 19.08.2020
*/

DROP PROCEDURE IF EXISTS agent.pDoMaintenanceDebCred;
GO

CREATE PROCEDURE [agent].[pDoMaintenanceDebCred]
	@Username		NVARCHAR(255),		-- used for logging
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		-- Set Delta Values

		-- Update History Table

		-- Update persisted tables in result
		EXECUTE result.pDebCredAccountBalancesPeriods  @Username = 'SQL'

		CREATE CLUSTERED COLUMNSTORE INDEX icc_result_tDebCredAccountBalancesPeriods ON result.tDebCredAccountBalancesPeriods;

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDoMaintenanceDebCred', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDoMaintenanceDebCred'
GO

/*
    Product: OCT 5.0
    Object: agent.pDoMaintenanceFin
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 02.08.2019 16:29
*/

DROP PROCEDURE IF EXISTS agent.pDoMaintenanceFin;
GO

CREATE PROCEDURE [agent].[pDoMaintenanceFin]
	@Username		NVARCHAR(255),		-- used for logging
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		-- Set Delta Values

		-- Update History Table

		-- Update persisted views in result
		DROP TABLE IF EXISTS result.tFinFiBuBuchungsjournal;
		DROP TABLE IF EXISTS result.tFinKoReBuchungsjournal;
		DROP TABLE IF EXISTS result.tFinFiBuSaldenliste;
		DROP TABLE IF EXISTS result.tFinKoReSaldenliste;
	
		SELECT *
		INTO result.tFinFiBuBuchungsjournal
		FROM agent.vResult_FiBuBuchungsjournal;

		SELECT *
		INTO result.tFinKoReBuchungsjournal
		FROM agent.vResult_KoReBuchungsjournal;

		SELECT *
		INTO result.tFinFiBuSaldenliste
		FROM agent.vResult_FiBuSaldenliste;

		SELECT *
		INTO result.tFinKoReSaldenliste
		FROM agent.vResult_KoReSaldenliste;

		CREATE CLUSTERED COLUMNSTORE INDEX icc_result_tFinFiBuBuchungsjournal ON result.tFinFiBuBuchungsjournal;
		CREATE CLUSTERED COLUMNSTORE INDEX icc_result_tFinKoReBuchungsjournal ON result.tFinKoReBuchungsjournal;
		CREATE CLUSTERED COLUMNSTORE INDEX icc_result_tFinFiBuSaldenliste ON result.tFinFiBuSaldenliste;
		CREATE CLUSTERED COLUMNSTORE INDEX icc_result_tFinKoReSaldenliste ON result.tFinKoReSaldenliste;

		SET @ResultCode = 200;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pDoMaintenanceFin', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pDoMaintenanceFin'
GO

/*
    Product: OCT 5.0
    Object: agent.pSync
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 02.08.2019 16:29

	Testcall
	DECLARE @RC INT
	DECLARE @Username		NVARCHAR(255) ='SQL'
	DECLARE @Solution		NVARCHAR(50) = 'FIN'
	DECLARE @ResultOnly		INT = 0
	DECLARE @CompanyKeys	NVARCHAR(MAX) = '1|1;1|2' -- List must be ; separated

	EXECUTE @RC = [agent].[pSync] 
	   @Username
	  ,@Solution
	  ,@ResultOnly
	  ,@CompanyKeys
	GO
*/

DROP PROCEDURE IF EXISTS agent.pSync;
GO

CREATE PROCEDURE [agent].[pSync]
	@Username		NVARCHAR(255),
	@Solution		NVARCHAR(50),
	@ResultOnly		INT = 0,
	@CompanyKeys	NVARCHAR(MAX), -- delimited by ;
	@ProcessID		NVARCHAR(50) = '0', -- used for logging
	@SubProcessID	NVARCHAR(50) = '0'  -- used for logging
AS
BEGIN
	DECLARE @SQLCommand			NVARCHAR(255)
	DECLARE @ResultCode			INT = 501;
	DECLARE @MaxRowKey			INT = 0;
	DECLARE @CurrentTimestamp	DATETIME;

	-- write log entry for begin of agent execution
	IF @ProcessID = '0'
		BEGIN
			SELECT @MaxRowKey = MAX(RowKey)
			FROM system.tProcessLogs
			SET @ProcessID = @MaxRowKey + 1;
		END
		
	SET @CurrentTimestamp = CURRENT_TIMESTAMP;
	
	-- protect input parameters
	DECLARE @Module NVARCHAR(50) = @Solution -- Set Variable Module, will be used in the future. Changes have to be made to OCT to set parameter @Module instead of @Solution
	SET @Username = dbo.sx_pf_pProtectString(@Username);
	SET @Module = dbo.sx_pf_pProtectString(@Module);
	SET @ResultOnly = dbo.sx_pf_pProtectInt(@ResultOnly);

	-- execute sync procedure for the selected solution
	IF EXISTS ( SELECT *
				FROM   sysobjects 
				WHERE  id = object_id(N'[agent].[pSync'+@Module+']') 
					   and OBJECTPROPERTY(id, N'IsProcedure') = 1 )
	BEGIN
		SET @SQLCommand = N'agent.pSync'+@Module
		EXEC @ResultCode = @SQLCommand @Username, @ResultOnly, @CompanyKeys, @ProcessID, @SubProcessID;
	END

	IF @ResultCode < 400
		-- do maintenance jobs at the end of each sync
		EXEC @ResultCode = agent.pDoMaintenance @Username, @ProcessID, @SubProcessID;

	-- write log entry for end of agent execution
	SET @CurrentTimestamp = CURRENT_TIMESTAMP;
	
	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Procedure to sync the integration data of any solution to the global and result schema. List of CompanyKeys must be semicolon(;) separated. Result only = 1
			stands for transfer only from global to result, Result only <> 1 means integration to global and global to result.',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSync'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncDebCred
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pSyncDebCred;
GO

CREATE PROCEDURE [agent].[pSyncDebCred]
	@Username		NVARCHAR(255),
	@ResultOnly		INT,
	@CompanyKeys	NVARCHAR(MAX), -- delimited by ;
	@ProcessID		NVARCHAR(50) = '0',
	@SubProcessID	NVARCHAR(50) = '0'
AS
BEGIN
	DECLARE @ResultCode INT = 501;

	IF @ResultOnly = 1
	BEGIN
		EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tCompanies', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tDebCredAccounts',@ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
	END
	ELSE
	BEGIN			
		EXEC @ResultCode = agent.pSyncDimensionToGlobal @Username, @CompanyKeys = @CompanyKeys, @TableName = 'tDebCredAccounts', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;


		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tCompanies', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tDebCredAccounts', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDebCredJournal @Username, @CompanyKeys = @CompanyKeys, @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
	END

	IF @ResultCode < 400
		EXEC agent.pDoMaintenanceDebCred @Username, @ProcessID, @SubProcessID;

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncDebCred'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncDebCredJournal
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 19.08.2020
*/

DROP PROCEDURE IF EXISTS agent.pSyncDebCredJournal;
GO

CREATE PROCEDURE agent.pSyncDebCredJournal
	@Username		NVARCHAR(255),		-- used for logging
	@CompanyKeys NVARCHAR(MAX),
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		EXEC agent.pCreateJSONViewINT
			@UserName,
			@Schema = 'integration',
			@Table = 'tDebCredJournal',
			@UseRowKey = 1,
			@ProcessID = @ProcessID,
			@SubProcessID = @SubProcessID;

		DROP TABLE IF EXISTS result.tDebCredJournal;
		--TRUNCATE TABLE result.tDebCredJournal;
		WITH j AS (
			SELECT
				j.RowKey,
				CONCAT(j.DataSourceKey, N'|', j.CompanyID) CompanyKey,
				CASE WHEN j.AccountID IS NOT NULL THEN CONCAT(j.DataSourceKey, N'|', j.CompanyID,N'|',j.AccountTypeCODE, N'|', j.AccountID) END AS AccountKey,
				FinancialYear,
				FinancialPeriod,
				CAST(FinancialYear AS varchar(4)) + RIGHT('0' + CAST(c.FinancialYearBegin AS varchar(2)), 2) + RIGHT('0' + CAST(FinancialPeriod AS varchar(2)), 2) AS PeriodKey,
				ItemStatusCODE,
				OpeningBalance,
				Debit,
				Credit,
				PostingDate,
				PostingText,
				DocumentNumber,
				DocumentDate,
				DocumentClosedDate,
				DocumentDueDate,
				DocumentLine,
				DocumentTypeCODE,
				ExternalInvoiceNumber,
				SourceID
			FROM integration.tDebCredJournal j
			JOIN global.tCompanies c ON c.CompanyKey = CONCAT(j.DataSourceKey, N'|', j.CompanyID)
		)
		--INSERT INTO result.tDebCredJournal
		SELECT
			j.CompanyKey AS CompanyKey_Original,
			j.AccountKey AS AccountKey_Original,
			COALESCE(cm.CompanyKey, j.CompanyKey) AS CompanyKey,
			COALESCE(am.AccountKey, j.AccountKey) AS AccountKey,
			j.FinancialYear,
			j.FinancialPeriod,
			j.PeriodKey,
			j.ItemStatusCODE,
			j.OpeningBalance,
			j.Debit,
			j.Credit,
			j.PostingDate,
			j.PostingText,
			j.DocumentNumber,
			j.DocumentDate,
			j.DocumentClosedDate,
			j.DocumentDueDate,
			j.DocumentLine,
			j.DocumentTypeCODE,
			j.ExternalInvoiceNumber,
			j.SourceID,
			jjson.*
		INTO result.tDebCredJournal
		FROM j
		LEFT JOIN agent.vJSON_tDebCredJournal jjson ON jjson.RowKey = j.RowKey
		JOIN global.tCompanies c ON c.CompanyKey = j.CompanyKey
		LEFT JOIN global.tCompanies cm ON cm.RowKey = c.MappingKey
		LEFT JOIN global.tDebCredAccounts a ON a.AccountKey = j.AccountKey
		LEFT JOIN global.tDebCredAccounts am ON am.RowKey = a.MappingKey

		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH 
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncDebCredJournal', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;

GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncDebCredJournal'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncDimensionToGlobal
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pSyncDimensionToGlobal;
GO

CREATE PROCEDURE [agent].[pSyncDimensionToGlobal]
	@Username		NVARCHAR(255),		-- used for logging
	@CompanyKeys	NVARCHAR(MAX),
	@TableName		NVARCHAR(100),
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX);
	DECLARE @SQLMerge NVARCHAR(MAX) = '';
	DECLARE @SQLMatched NVARCHAR(MAX) = '';
	DECLARE @SQLUpdate NVARCHAR(MAX) = '';
	DECLARE @SQLInsert NVARCHAR(MAX) = '';
	DECLARE @SQLValues NVARCHAR(MAX) = '';
	DECLARE @ResultCode INT = 501;
	DECLARE @ColName NVARCHAR(50);
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY

		BEGIN TRANSACTION pSyncDimensionToGlobal;

		-- Cursor for key columns
		DECLARE KeyCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('global', @TableName)
			WHERE IsPartOfKey = 1

		-- Cursor for regular columns
		DECLARE ColCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('integration', @TableName)
			WHERE IsPartOfKey = 0
			AND IsCalculatedKey = 0
			AND ColName <> 'RowKey'
			AND ColName NOT IN (
				SELECT ColName
				FROM system.fGetColumns('global', @TableName)
				WHERE IsPartOfKey = 1
			)

		-- generate lists of columns needed inside the script
		OPEN KeyCursor
		FETCH KeyCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLMerge = @SQLMerge + '
			AND Target.' + @ColName + ' = Source.' + @ColName;

			SET @SQLInsert = @SQLInsert + '
			' + @ColName + ',';

			SET @SQLValues = @SQLValues + '
			Source.' + @ColName + ',';

			FETCH KeyCursor INTO @ColName
		END
		CLOSE KeyCursor
		DEALLOCATE KeyCursor

		OPEN ColCursor
		FETCH ColCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLMatched = @SQLMatched + '
			OR Target.' + @ColName + ' <> Source.' + @ColName;
		
			SET @SQLUpdate = @SQLUpdate + '
			,Target.' + @ColName + ' = Source.' + @ColName;

			SET @SQLInsert = @SQLInsert + '
			' + @ColName + ',';

			SET @SQLValues = @SQLValues + '
			Source.' + @ColName + ',';

			FETCH ColCursor INTO @ColName
		END
		CLOSE ColCursor
		DEALLOCATE ColCursor
	
		-- concatenate the whole MERGE script
		SET @SQL = '
		DECLARE @SummaryOfChanges TABLE(Change VARCHAR(20));

		-- Update/Insert 
		MERGE global.' + @TableName + ' WITH (HOLDLOCK) AS Target
		USING (
			SELECT TOP 100 PERCENT *
			FROM integration.' + @TableName + '
			WHERE CAST(DataSourceKey AS NVARCHAR(50)) + N''|'' + CompanyID IN (
				SELECT value
				FROM STRING_SPLIT(''' + @CompanyKeys + ''','';'')
			)
		) AS Source
		ON (
			Target.DataSourceKey = Source.DataSourceKey
			AND Target.CompanyID = Source.CompanyID'
			+ @SQLMerge + '
		)

		WHEN
			MATCHED
			AND (
				Target.OwnerCODE <> ''INTEGRATION'''
				+ @SQLMatched + '
		)
		THEN
			UPDATE SET
				Target.OwnerCODE = ''INTEGRATION'',
				Target.UpdatedFlag = 1'
				+ @SQLUpdate + '
		WHEN NOT MATCHED BY TARGET THEN
			INSERT ('
				+ @SQLInsert + '
				OwnerCODE,
				UpdatedFlag
			)
			VALUES ('
				+ @SQLValues + '
				''INTEGRATION'', 
				1
			)

		WHEN NOT MATCHED BY SOURCE AND Target.OwnerCode <> ''UNBEKANNT'' THEN
			UPDATE SET
				Target.OwnerCODE = ''UNBEKANNT''
		OUTPUT $action INTO @SummaryOfChanges;

		SELECT ''' + @TableName + ''' AS TableName, Change AS Action, COUNT(*) AS Rows
		FROM @SummaryOfChanges
		GROUP BY Change;';

		--PRINT @SQL;

		EXEC (@SQL);
		SET @ResultCode = 200;

		COMMIT TRANSACTION pSyncDimensionToGlobal;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		ROLLBACK TRANSACTION pSyncDimensionToGlobal;

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncDimensionToGlobal', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncDimensionToGlobal'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncDimensionToResult
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pSyncDimensionToResult;
GO

CREATE PROCEDURE [agent].[pSyncDimensionToResult]
	@Username		NVARCHAR(255),		-- used for logging
	@TableName		NVARCHAR(100),
	@ProcessID		NVARCHAR(50) = '0', -- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ColList			NVARCHAR(1000);
	DECLARE @ColListJSON		NVARCHAR(1000);
	DECLARE @SQL				NVARCHAR(MAX);
	DECLARE @KeyCol				NVARCHAR(50);
	DECLARE @ResultCode			INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		
		-- drop old result table
		EXEC ('DROP TABLE IF EXISTS result.' + @TableName);

		-- extract JSON values to views
		EXEC @ResultCode = agent.pCreateJSONViewINT @Username, @Schema = 'global', @Table = @TableName, @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF	@ResultCode <> 200	
			THROW 51000, 'Error in procedure agent.pCreateJSONViewINT.', 1;

		EXEC @ResultCode = agent.pCreateJSONViewUSER @Username, @Schema = 'global', @Table = @TableName, @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF	@ResultCode <> 200	
			THROW 51000, 'Error in procedure agent.pCreateJSONViewUSER.', 1;

		-- get name of calculated key column
		SELECT @KeyCol = ColName
		FROM system.fGetColumns('global', @TableName)
		WHERE IsCalculatedKey = 1;
	
		-- get list of all other non default columns
		SELECT @ColList = COALESCE(@ColList + ',
		g.' + ColName, ',
		g.' + Colname, '')
		FROM system.fGetColumns('global', @TableName)
		WHERE ColName NOT IN ('RowKey', 'UpdatedFlag', 'MappingKey', 'CustomValuesJSON', 'CustomValuesJSONUser');

		-- get list of all JSON columns	
		SELECT @ColListJSON = COALESCE(@ColListJSON + ',
		' + ColName, ',
		' + Colname, '')
		FROM (
			SELECT ColName
			FROM system.fGetColumns('agent', 'vJSON_' + @TableName)
		
			UNION ALL

			SELECT ColName
			FROM system.fGetColumns('agent', 'vJSONUser_' + @TableName)
		) a
		WHERE ColName NOT IN (
			SELECT ColName
			FROM system.fGetColumns('global', @TableName)
		)
		ORDER BY ColName;
	
		-- combine all column lists and create the result table dynamically
		SET @SQL = 'SELECT' + RIGHT(COALESCE(@ColList, ''), LEN(@ColList) - 1) + COALESCE(@ColListJSON, '') +
			' INTO result.' + @TableName + '
			FROM global.' + @TableName + ' g
			LEFT JOIN agent.vJSON_' + @TableName + ' a ON a.' + @KeyCol + ' = g.' + @KeyCol + '
			LEFT JOIN agent.vJSONUser_' + @TableName + ' u ON u.' + @KeyCol + ' = g.' + @KeyCol + '
			WHERE g.MappingKey IS NULL'
	
		--PRINT @SQL;

		EXEC (@SQL);

		-- set the UpdatedFlag to 0
		SET @SQL = 'UPDATE global.' + @TableName + ' SET UpdatedFlag = 0';
		EXEC (@SQL);
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncDimensionToResult', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncDimensionToResult'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncFin
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:12
*/

DROP PROCEDURE IF EXISTS agent.pSyncFin;
GO

CREATE PROCEDURE [agent].[pSyncFin]
	@Username		NVARCHAR(255),
	@ResultOnly		INT,
	@CompanyKeys	NVARCHAR(MAX), -- delimited by ;
	@ProcessID		NVARCHAR(50) = '0',
	@SubProcessID	NVARCHAR(50) = '0'
AS
BEGIN
	DECLARE @ResultCode INT = 501;

	IF @ResultOnly = 1
	BEGIN
		EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tCompanies', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tFinAccounts',@ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tFinCostObjects', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
	END
	ELSE
	BEGIN			
		EXEC @ResultCode = agent.pSyncDimensionToGlobal @Username, @CompanyKeys = @CompanyKeys, @TableName = 'tFinAccounts', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToGlobal @Username, @CompanyKeys = @CompanyKeys, @TableName = 'tFinCostObjects', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tCompanies', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tFinAccounts', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncDimensionToResult @Username, @TableName = 'tFinCostObjects', @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;

		IF @ResultCode < 400
			EXEC @ResultCode = agent.pSyncFinAccountingJournal @Username, @CompanyKeys = @CompanyKeys, @ProcessID = @ProcessID, @SubProcessID = @SubProcessID;
	END

	IF @ResultCode < 400
		EXEC agent.pDoMaintenanceFin @Username, @ProcessID, @SubProcessID;

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncFin'
GO

/*
    Product: OCT 5.0
    Object: agent.pSyncFinAccountingJournal
    Type: Stored Procedure
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:13
*/

DROP PROCEDURE IF EXISTS agent.pSyncFinAccountingJournal;
GO

CREATE PROCEDURE agent.pSyncFinAccountingJournal
	@Username		NVARCHAR(255),		-- used for logging
	@CompanyKeys NVARCHAR(MAX),
	@ProcessID		NVARCHAR(50) = '0',	-- used for logging
	@SubProcessID	NVARCHAR(50) = '0'	-- used for logging
AS
BEGIN
	DECLARE @ResultCode INT = 501;
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);

	BEGIN TRY
		EXEC agent.pCreateJSONViewINT
			@UserName,
			@Schema = 'integration',
			@Table = 'tFinAccountingJournal',
			@UseRowKey = 1,
			@ProcessID = @ProcessID,
			@SubProcessID = @SubProcessID;

		DROP TABLE IF EXISTS result.tFinAccountingJournal;
		--TRUNCATE TABLE result.tFinAccountingJournal;
		WITH j AS (
			SELECT
				j.RowKey,
				CONCAT(j.DataSourceKey, N'|', j.CompanyID) CompanyKey,
				CASE WHEN j.AccountID IS NOT NULL THEN CONCAT(j.DataSourceKey, N'|', j.CompanyID, N'|', j.AccountGroupCODE, N'|', j.AccountID) END AS AccountKey,
				CASE WHEN j.OffsetAccountID IS NOT NULL THEN CONCAT(j.DataSourceKey, N'|', j.CompanyID, N'|', j.OffsetAccountGroupCODE, N'|', j.OffsetAccountID) END AS OffsetAccountKey,
				CASE WHEN j.CostObject1ID IS NOT NULL THEN CONCAT(j.DataSourceKey, N'|', j.CompanyID, N'|', j.CostObject1GroupCODE, N'|', j.CostObject1ID) END AS CostObject1Key,
				CASE WHEN j.CostObject2ID IS NOT NULL THEN CONCAT(j.DataSourceKey, N'|', j.CompanyID, N'|', j.CostObject2GroupCODE, N'|', j.CostObject2ID) END AS CostObject2Key,
				AccountingAreaCODE,
				FinancialYear,
				FinancialPeriod,
				CAST(FinancialYear AS varchar(4)) + RIGHT('0' + CAST(c.FinancialYearBegin AS varchar(2)), 2) + RIGHT('0' + CAST(FinancialPeriod AS varchar(2)), 2) AS PeriodKey,
				OpeningBalance,
				Debit,
				Credit,
				PostingDate,
				PostingText,
				DocumentDate,
				DocumentNumber,
				DocumentLine,
				ICPartnerID,
				SourceID
			FROM integration.tFinAccountingJournal j
			JOIN global.tCompanies c ON c.CompanyKey = CONCAT(j.DataSourceKey, N'|', j.CompanyID)
		)
		--INSERT INTO result.tFinAccountingJournal
		SELECT
			j.CompanyKey AS CompanyKey_Original,
			j.AccountKey AS AccountKey_Original,
			j.OffsetAccountKey AS OffsetAccountKey_Original,
			j.CostObject1Key AS CostObject1Key_Original,
			j.CostObject2Key AS CostObject2Key_Original,
			COALESCE(cm.CompanyKey, j.CompanyKey) AS CompanyKey,
			COALESCE(am.AccountKey, j.AccountKey) AS AccountKey,
			COALESCE(oam.AccountKey, j.OffsetAccountKey) AS OffsetAccountKey,
			COALESCE(co1m.CostObjectKey, j.CostObject1Key) AS CostObject1Key,
			COALESCE(co2m.CostObjectKey, j.CostObject2Key) AS CostObject2Key,
			j.AccountingAreaCODE,
			j.FinancialYear,
			j.FinancialPeriod,
			j.PeriodKey,
			j.OpeningBalance,
			j.Debit,
			j.Credit,
			j.PostingDate,
			j.PostingText,
			j.DocumentDate,
			j.DocumentNumber,
			j.DocumentLine,
			j.ICPartnerID,
			j.SourceID,
			jjson.*
		INTO result.tFinAccountingJournal
		FROM j
		LEFT JOIN agent.vJSON_tFinAccountingJournal jjson ON jjson.RowKey = j.RowKey
		JOIN global.tCompanies c ON c.CompanyKey = j.CompanyKey
		LEFT JOIN global.tCompanies cm ON cm.RowKey = c.MappingKey
		LEFT JOIN global.tFinAccounts a ON a.AccountKey = j.AccountKey
		LEFT JOIN global.tFinAccounts am ON am.RowKey = a.MappingKey
		LEFT JOIN global.tFinAccounts oa ON oa.AccountKey = j.OffsetAccountKey
		LEFT JOIN global.tFinAccounts oam ON oam.RowKey = oa.MappingKey
		LEFT JOIN global.tFinCostObjects co1 ON co1.CostObjectKey = j.CostObject1Key
		LEFT JOIN global.tFinCostObjects co1m ON co1m.RowKey = co1.MappingKey
		LEFT JOIN global.tFinCostObjects co2 ON co2.CostObjectKey = j.CostObject2Key
		LEFT JOIN global.tFinCostObjects co2m ON co2m.RowKey = co2.MappingKey

		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH 
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();

		SET @CurrentTimestamp = CURRENT_TIMESTAMP;
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncFinAccountingJournal', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;
	END CATCH

	RETURN @ResultCode;
END;
GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'agent',
    @level1type = N'Procedure',
    @level1name = N'pSyncFinAccountingJournal'
GO

/*
COPY Operation for a Factory
Not yet existing factory is copied when Source Factory exists

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty  source or target FactoryID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists Source Factory => 404 Not Found
5. Target Factory key already exists => 403 Forbidden
6. Factory created > 201 Created

DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) 
DECLARE @SourceFactoryID		NVARCHAR(255)
DECLARE @TargetFactoryID		NVARCHAR(255)
DECLARE @TargetFactoryNameShort NVARCHAR(255)

-- CASE 1
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= ''
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

-- CASE 4
SET  @Username				 = 'SQL'
SET  @SourceFactoryID		 = ''
SET  @TargetFactoryID		 = 'ZTKopie'
SET  @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

-- CASE 5 
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= 'ZT'
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC

--CASE 6
SET @Username				= 'SQL'
SET @SourceFactoryID		= 'ZT'
SET @TargetFactoryID		= 'ZTKopie'
SET @TargetFactoryNameShort = 'Neue Factory'

EXECUTE @RC = sx_pf_COPY_Factory 
	@Username
	,@SourceFactoryID
	,@TargetFactoryID
	,@TargetFactoryNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_Factory;
GO

CREATE PROCEDURE sx_pf_COPY_Factory
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetFactoryNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey		INT				= 0;
	DECLARE @SourceFactoryKey		INT				= 0;
	DECLARE @SourceFactoryNameShort NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetFactoryNameShort	IS NULL SET @TargetFactoryNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Factory;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetFactoryNameShort = dbo.sx_pf_pProtectString	(@TargetFactoryNameShort);
		
		IF	@SourceFactoryID	= N'' 
			OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
					
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
		-- dFactory
		INSERT INTO planning.tdFactories
		OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
		SELECT @TargetFactoryID			AS FactoryID
			, @TargetFactoryNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName 
		FROM planning.tdFactories 
		WHERE FactoryID = @SourceFactoryID; 

		SET @EffectedRows = @@ROWCOUNT;	

		SELECT TOP 1 @TargetFactoryKey = [Key]
		FROM @Keys;
		
		--dProductLine 
		INSERT INTO planning.tdProductLines
		SELECT @TargetFactoryKey AS FactoryKey
			, ProductlineID
			, @TargetFactoryID	 AS FactoryID
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3 
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6 
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9 	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12 
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18 
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21 	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;	

		--dProduct 
		INSERT INTO planning.tdProducts
		SELECT PL.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, ProductID
			, dP.ProductlineID
			, @TargetFactoryID		AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts dP
				LEFT JOIN (
					SELECT ProductLineKey, ProductLineID 
					FROM planning.tdProductLines 
					WHERE FactoryKey = @TargetFactoryKey) PL 
				ON PL.ProductLineID = dP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries  
		SELECT P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dVS.ProductID
			, dVS.ProductLineID 
			, @TargetFactoryID	AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dVS.ProductID AND P.ProductLineID = dVS.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime  
		SELECT 
			 P.ProductKey AS ProductKey
			, P.ProductLineKey AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dT.ProductID
			, dT.ProductLineID 
			, @TargetFactoryID AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dT.ProductID AND P.ProductLineID = dT.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues 
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, VS.ProductLineKey		AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, fV.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
				LEFT JOIN (
					SELECT ProductKey, ValueSeriesKey, ProductLineKey, ProductID, ValueSeriesID, ProductLineID 
					FROM planning.tdValueSeries 
					WHERE FactoryKey = @TargetFactoryKey) VS 
				ON fV.ValueSeriesID = VS.ValueSeriesID AND fV.ProductID = VS.ProductID AND fV.ProductLineID = VS.ProductLineID
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gFactories
		INSERT INTO planning.tgFactories 
		SELECT @TargetFactoryKey	AS FactoryKey
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgFactories
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gProductLines
		INSERT INTO planning.tgProductLines
		SELECT PL.ProductLineKey	AS ProductLineKey 
			, @TargetFactoryKey		AS FactoryKey
			, gPL.ProductlineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProductLines gPL
			LEFT JOIN (
				SELECT ProductLineKey, ProductLineID 
				FROM planning.tdProductLines 
				WHERE FactoryKey = @TargetFactoryKey) 
			PL ON PL.ProductLineID = gPL.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts
		SELECT 
			  P.ProductKey		AS ProductKey
			, P.ProductlineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, gP.ProductID
			, gP.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
				FROM planning.tdProducts 
				WHERE FactoryKey = @TargetFactoryKey) P 
			ON P.ProductID = gP.ProductID AND P.ProductLineID = gP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rPreferences
		INSERT INTO planning.trPreferences
		SELECT @TransactUserKey AS UserKey 
			, @TransactUsername AS UserName 
			, @TargetFactoryID	AS FactoryID
			, ProductLineID
			, SettingName 
			, ValueInt 
			, ValueText
		FROM planning.trPreferences 
		WHERE FactoryID = @SourceFactoryID;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,P.ProductLineKey
				,P.ProductKey
				,@TargetFactoryID	AS FactoryID
				,tF.ProductLineID		AS ProductLineID
				,tF.ProductID			AS ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = tF.ProductID AND P.ProductLineID = tF.ProductLineID
			WHERE FactoryID = @SourceFactoryID;

		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
		SELECT ProductKey
			, dp.ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, ProductID
			, dp.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, N'COPY'			AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'" / ProductLine <' + dp.ProductlineID + N'>"' + dpl.Nameshort
				+ N'" / Product <' + ProductID + N'>"' + dp.Nameshort + '"' AS Statement
			, @TransactUsername AS UserName 
			, N''				AS PCName
			, N''				AS ProcessorCode
			, N''				AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp 
			, 0					AS IsDeleted 
			, 0					AS IsResolved
		FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl 
			ON dp.ProductLinekey= dpl.ProductLineKey
		WHERE dp.FactoryKey = @TargetFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Right materialization
		EXEC dbo.sx_pf_materialize_vUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Factory TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a Format
Not yet existing Format is copied if source Format is available

07/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard


DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @SourceFormatID NVARCHAR(255) = 'EingabeGanzzahl'
DECLARE @TargetFormatID NVARCHAR(255) = '88C26'

EXECUTE @RC = sx_pf_COPY_Format 
	 @Username
	,@SourceFormatID
	,@TargetFormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_Format
GO

CREATE PROCEDURE sx_pf_COPY_Format
	@Username		NVARCHAR(255),
	@SourceFormatID NVARCHAR(255),
	@TargetFormatID NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFormatKey	INT = 0;
	DECLARE @TargetFormatKey	INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@SourceFormatID,	N'NULL') + N''',''' 
			+ ISNULL(@TargetFormatID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceFormatID	IS NULL SET @SourceFormatID = N'';
	IF @TargetFormatID	IS NULL SET @TargetFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Format;

		-- STEP 0.2 - Protect input parameters			
		SET @Username		= dbo.sx_pf_pProtectString  (@Username);
		SET @SourceFormatID	= dbo.sx_pf_pProtectID		(@SourceFormatID);
		SET @TargetFormatID = dbo.sx_pf_pProtectID		(@TargetFormatID);

		IF  @SourceFormatID		= N'' 
			OR @TargetFormatID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @SourceFormatID;
		
		IF @SourceFormatKey = 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @TargetFormatID;
		
		IF @TargetFormatKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hFormats
		INSERT INTO planning.thFormats
			SELECT @TargetFormatID AS FormatID
				, BackgroundColor
				, FontColor
				, ValueFormat
				, Bold
				, Italic
				, AlignmentHorizontal
				, AlignmentVertical
				, TextUnderlined
				, BorderToNext
				, BorderToPrevious
			FROM planning.thFormats
			WHERE FormatKey = @SourceFormatKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Format TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a List
Not yet existing list is copied if Sources List is available

07/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL';
DECLARE @SourceListID	NVARCHAR(255) = 'suEvents';
DECLARE @TargetListID	NVARCHAR(255) = '88C26';

EXECUTE @RC = sx_pf_COPY_List 
	@Username
	,@SourceListID
	,@TargetListID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_List;
GO

CREATE PROCEDURE sx_pf_COPY_List
	@Username		NVARCHAR(255),
	@SourceListID	NVARCHAR(255),
	@TargetListID	NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceListKey INT = 0;
	DECLARE @TargetListKey INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@SourceListID, N'NULL')	+ N''',''' 
			+ ISNULL(@TargetListID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceListID	IS NULL SET @SourceListID	= N'';
	IF @TargetListID	IS NULL SET @TargetListID	= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_List;

		-- STEP 0.2 - Protect input parameters			
		SET @Username	  =	dbo.sx_pf_pProtectString (@Username);
		SET @SourceListID = dbo.sx_pf_pProtectID	 (@SourceListID);
		SET @TargetListID = dbo.sx_pf_pProtectID	 (@TargetListID);

		IF	@SourceListID	 = N'' 
			OR @TargetListID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceListKey = ListKey 
		FROM planning.thLists 
		WHERE ListID = @SourceListID;
		
		IF @SourceListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetListKey = ListKey 
		FROM planning.thLists 
		WHERE ListID = @TargetListID;

		IF @TargetListKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target List key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hLists
		INSERT INTO planning.thLists
			SELECT @TargetListID AS ListsID
				, NameShort
				, NameLong
				, CommentDev
				, CommentUser
				, Datentyp
				, Source
				, SourceFormula
				, FormatID
			FROM planning.thLists
			WHERE ListKey = @SourceListKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--hListValues
		INSERT INTO planning.thListValues 
			SELECT @TargetListID AS ListsID
				, ValueInt
				, Scale
				, ValueText
				, ValueComment
				, FormatID
			FROM planning.thListValues
			WHERE ListID = @SourceListID;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_List TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a Product
 - creates a new Product from an Template
 - * as Target ProductID is supported, the next free numeric ID is used then
		
05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product => 404 Not Found
5. Not exists target Factory / Productline => 404 Not Found
6. Target Product key already exists => 403 Forbidden
7. Product copied => 201 Created

DECLARE  @RC						INT
DECLARE  @Username					NVARCHAR(255) = 'SQL'
DECLARE  @SourceFactoryID			NVARCHAR(255) = 'WSB'
DECLARE  @SourceProductLineID		NVARCHAR(255) = 'KRED1'
DECLARE  @SourceProductID			NVARCHAR(255) = '0'
DECLARE  @TargetFactoryID			NVARCHAR(255) = 'WSB'
DECLARE  @TargetProductLineID		NVARCHAR(255) = 'KRED1'
DECLARE  @TargetProductID			NVARCHAR(255) = '*'
DECLARE  @TargetProductNameShort	NVARCHAR(255) = 'Test'

EXECUTE @RC = sx_pf_COPY_Product 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID
		  ,@TargetProductNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_Product;
GO

CREATE PROCEDURE sx_pf_COPY_Product
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@SourceProductLineID	NVARCHAR(255),
	@SourceProductID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetProductLineID	NVARCHAR(255),
	@TargetProductID		NVARCHAR(255),
	@TargetProductNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
	IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
	IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
	IF @TargetProductNameShort	IS NULL SET @TargetProductNameShort = N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Product;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID	= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID	= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetProductNameShort = dbo.sx_pf_pProtectString	(@TargetProductNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE FactoryKey = @SourceFactoryKey AND ProductlineKey = @SourceProductlineKey AND ProductID = @SourceProductID;
		
		IF	@SourceFactoryKey			= 0 
			OR @SourceProductLineKey	= 0 
			OR @SourceProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @TargetFactoryKey AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		
		IF	@TargetFactoryKey		 = 0 
			OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
		
		-- STEP 1.4 - If ProductID * is submitted, search the next free numeric ProductID
		IF @TargetProductID = '*' 
		BEGIN
			SELECT @TargetProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)), 0) + 1 
			FROM planning.tdProducts  
			WHERE ProductLineKey = @TargetProductLineKey 
				AND TRY_CAST(ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @TargetProductID = '*' 
				SELECT @TargetProductID = 1;
		END

		-- STEP 3 - Copy data		
		--dProduct
		DECLARE @TargetProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

		INSERT INTO planning.tdProducts
		OUTPUT INSERTED.ProductKey INTO @TargetProductKeyTable
		SELECT @TargetProductLineKey	AS ProductLineKey
			, @TargetFactoryKey			AS FactoryKey
			, @TargetProductID			AS ProductID
			, @TargetProductlineID		AS ProductlineID
			, @TargetFactoryID			AS FactoryID
			, TimeType
			, @TargetProductNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1 , GlobalAttribute2 , GlobalAttribute3 , GlobalAttribute4 , GlobalAttribute5 , GlobalAttribute6 , GlobalAttribute7
			, GlobalAttribute8 , GlobalAttribute9 , GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductKey = KeyNumber 
		FROM @TargetProductKeyTable;

		--gProducts Inhalt
		INSERT INTO planning.tgProducts
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries
		SELECT
		      @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime
		SELECT
			 @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM planning.tdValueSeries 
				WHERE ProductKey = @TargetProductKey) VS 
			ON VS.ValueSeriesID = fV.ValueSeriesID 
		WHERE fV.ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT
				 @TargetFactoryKey
				,@TargetProductLineKey
				,@TargetProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,@TargetProductID		AS ProductID
				,FileID
				,FileName
				,FileExtension
				,FileBody
			FROM planning.tFiles
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID
				AND ProductID		= @SourceProductID;

		--fStatements CopyInformation speichern
		INSERT INTO planning.tfStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
				+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;
		
		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;
			
		COMMIT TRANSACTION sx_pf_COPY_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_Product TO pf_PlanningFactoryService;
GO
/*
COPY Operation for a ProductLine
Not yet existing ProductLine is copied when Source ProductLine is available

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline => 404 Not Found
5. Not exists target Factory => 404 Not Found
6. Target ProductLine key already exists => 403 Forbidden
7. ProductLine copied => 201 Created

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @SourceFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @SourceProductLineID		NVARCHAR(255) = 'ZT'
DECLARE @TargetFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TargetProductLineID		NVARCHAR(255) = 'ZT32'
DECLARE @TargetProductLineNameShort NVARCHAR(255) = 'Neue ProductLine32'

EXECUTE @RC = sx_pf_COPY_ProductLine 
		@Username
		,@SourceFactoryID
		,@SourceProductLineID
		,@TargetFactoryID
		,@TargetProductLineID
		,@TargetProductLineNameShort

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_ProductLine;
GO

CREATE PROCEDURE sx_pf_COPY_ProductLine
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductLineNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey			INT				= 0;
	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					  N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			  N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			  N'NULL') + N''','''
			+ ISNULL(@TargetProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineNameShort, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductLineNameShort	IS NULL SET @TargetProductLineNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID		= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID		= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductLineNameShort = dbo.sx_pf_pProtectString	(@TargetProductLineNameShort);

		IF	@SourceFactoryID				= N'' 
			OR @SourceProductLineID			= N'' 
			OR @TargetFactoryID				= N''
			OR @TargetProductlineID			= N'' 
			OR @TargetProductLineNameShort	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey,@SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey,@SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		
		IF	@SourceFactoryKey		 = 0 
			OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Copy data		
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
		
		--dProductLine
		INSERT INTO planning.tdProductLines
		OUTPUT INSERTED.ProductLineKey INTO @Keys([Key]) 
		SELECT @TargetFactoryKey			AS FactoryKey
			, @TargetProductlineID			AS ProductlineID
			, @TargetFactoryID				AS FactoryID
			, @TargetProductLineNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate 
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3  
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6  
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9  	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12  
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15 
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18  
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21  	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24 
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductLineKey = [Key] 
		FROM @Keys;
		
		--dProduct
		INSERT INTO planning.tdProducts 
		SELECT @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		 AS FactoryKey
			, ProductID
			, @TargetProductlineID	 AS ProductlineID
			, @TargetFactoryID		 AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion 
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7  
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14 
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21  
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries 
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dVS.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID 
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dVS.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime 
		SELECT 
			P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dT.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dT.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID, ProductID 
				FROM planning.tdValueSeries
				WHERE ProductLineKey = @TargetProductLineKey) VS 
			ON fV.ValueSeriesID=VS.ValueSeriesID AND fV.ProductID = VS.ProductID
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		--gProductLines 
		INSERT INTO planning.tgProductLines
		SELECT @TargetProductLineKey AS ProductLineKey 
			, @TargetFactoryKey		 AS FactoryKey
			, @TargetProductlineID	 AS ProductlineID
			, @TargetFactoryID		 AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProductLines
		WHERE ProductLineKey = @SourceProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts  
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, gP.ProductID
			, @TargetProductlineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts  
				WHERE ProductLineKey = @TargetProductLineKey) P
			ON P.ProductID = gP.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rRights - no definition, write Rights on Factory will lead to implicit Write Rights for creator

		--rPreferences
		INSERT INTO planning.trPreferences
			SELECT 
				 UserKey				AS UserKey 
				,Username				AS UserName
				,@TargetFactoryID		AS FactoryID
				,@TargetProductlineID	AS ProductLineID 
				,SettingName
				,ValueInt
				,ValueText
			FROM planning.trPreferences
			WHERE 
					FactoryID		= @SourceFactoryID 
				AND ProductlineID	= @SourceProductlineID;
		
		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,@TargetProductLineKey
				,P.ProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,tF.ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductID 
					FROM planning.tdProducts  
					WHERE ProductLineKey = @TargetProductLineKey) P
				ON P.ProductID = tF.ProductID 
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID;

		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
		SELECT ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey 
			, ProductID
			, @TargetProductlineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort
				+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted
			, 0						AS IsResolved
		FROM planning.tdProducts 
		WHERE ProductLineKey = @TargetProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Rights materialization 
		EXEC dbo.sx_pf_materialize_vUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_ProductLine TO pf_PlanningFactoryService;
GO
/*
COPY Operation for one ValueSerie
The ValueSerie will be copied and inserted at last Position

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3 .User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product / ValueSerie => 404 Not Found
5. Not exists target Factory / Productline / Product => 404 Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ValueSerie copied => 201 Created

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @SourceFactoryID			NVARCHAR(255) = '230'
DECLARE @SourceProductLineID		NVARCHAR(255) = 'I-ST-IT'
DECLARE @SourceProductID			NVARCHAR(255) = '888'
DECLARE @SourceValueSerieID			NVARCHAR(255) = 'C2'
DECLARE @TargetFactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TargetProductLineID		NVARCHAR(255) = 'ZT32'
DECLARE @TargetProductID			NVARCHAR(255) = '1'
DECLARE @TargetValueSerieID			NVARCHAR(255) = '88C2'
DECLARE @TargetValueSerieNameShort  NVARCHAR(255) = 'Kosten Test'

EXECUTE @RC = sx_pf_COPY_ValueSerie 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@SourceValueSerieID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID
		  ,@TargetValueSerieID
		  ,@TargetValueSerieNameShort

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_COPY_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_COPY_ValueSerie;
GO

CREATE PROCEDURE sx_pf_COPY_ValueSerie
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@SourceProductID			NVARCHAR(255),
	@SourceValueSerieID			NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductID			NVARCHAR(255),
	@TargetValueSerieID			NVARCHAR(255),
	@TargetValueSerieNameShort	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceValueSerieKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceValueSerieNameShort	NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @TargetValueSerieKey		INT				= 0;
	DECLARE @TargetValueSerieNo			INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+ ISNULL(@Username,						N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,				N'NULL') + N''',''' +
			+ ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @SourceProductID				IS NULL SET @SourceProductID			= N'';
	IF @SourceValueSerieID			IS NULL SET @SourceValueSerieID			= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductID				IS NULL SET @TargetProductID			= N'';
	IF @TargetValueSerieID			IS NULL SET @TargetValueSerieID			= N'';
	IF @TargetValueSerieNameShort	IS NULL SET @TargetValueSerieNameShort	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductlineID		= dbo.sx_pf_pProtectID		(@SourceProductlineID);
		SET @SourceProductID			= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @SourceValueSerieID			= dbo.sx_pf_pProtectID		(@SourceValueSerieID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductlineID		= dbo.sx_pf_pProtectID		(@TargetProductlineID);
		SET @TargetProductID			= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetValueSerieID			= dbo.sx_pf_pProtectID		(@TargetValueSerieID);
		SET @TargetValueSerieNameShort	= dbo.sx_pf_pProtectString	(@TargetValueSerieNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N'' 
			OR @SourceValueSerieID	= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N'' 
			OR @TargetValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername,@SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = NameShort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE ProductlineKey = @SourceProductlineKey AND ProductID= @SourceProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey, @SourceValueSerieNameShort = NameShort FROM planning.tdValueSeries WHERE ProductKey = @SourceProductKey  AND ValueSeriesID= @SourceValueSerieID;	
		
		IF	@SourceFactoryKey = 0 
			OR @SourceProductLineKey = 0 
			OR @SourceProductKey = 0 
			OR @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;
		
		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE  FactoryKey = @TargetFactoryKey  AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @TargetProductKey  AND ValueSeriesID= @TargetValueSerieID;
				
		IF	@TargetFactoryKey = 0 
			OR @TargetProductLineKey = 0 
			OR @TargetProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data 
		SELECT @TargetValueSerieNo = MAX(ValueSeriesNo) + 1 FROM planning.tdValueSeries WHERE ProductKey = @TargetProductKey;
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		INSERT INTO planning.tdValueSeries 
		OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
		SELECT @TargetProductKey			AS ProductKey
			, @TargetProductLineKey			AS ProductLineKey
			, @TargetFactoryKey				AS FactoryKey
			, @TargetProductID				AS ProductID
			, @TargetProductlineID			AS ProductLineID
			, @TargetFactoryID				AS FactoryID
			, @TargetValueSerieID			AS ValueSeriesID
			, @TargetValueSerieNo			AS ValueSeriesNo
			, @TargetValueSerieNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries 
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetValueSerieKey = [Key] 
		FROM @Keys;
		
		--fValues
		INSERT INTO planning.tfValues  
		SELECT @TargetValueSerieKey AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, @TargetValueSerieID	AS ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fStatements 
		INSERT INTO planning.tfStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductlineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'New ValueSerieID <' + @TargetValueSerieID + N'>"' + @TargetValueSerieNameShort
				+ N'" for product produced as a copy of Factory <' + @SourceFactoryID + N'>"' +  @SourceFactoryNameShort
				+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' +  @SourceProductLineNameShort
				+ N'"/ Product <' + @SourceProductID + N'>"' +  @SourceProductNameShort
				+ N'"/ ValueSerie <' + @SourceValueSerieID + N'>"' +  @SourceValueSerieNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, ''					AS PCName 
			, ''					AS ProcessorCode 
			, ''					AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_COPY_ValueSerie TO pf_PlanningFactoryService;
GO

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_Balance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_Balance]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_Balance (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_Balance] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_Balance] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [sx_pf_DATAOUTPUT_Balance] 
   @Username
  ,@FactoryID
  ,@ProductLineID
Print @RC
GO


*/

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_CashValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_CashValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_CashValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_CashValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_CashValues] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [sx_pf_DATAOUTPUT_CashValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC
GO


*/

/*
Procedure to deliver analytic data as factory summary
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
03/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_FactorySummary]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_FactorySummary]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_FactorySummary (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END
	SELECT 

		  TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month

         ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect) AS Cash
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect) AS Profit
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect) AS Balance
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.QuantityEffect) AS Quantity
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1)) AS Value

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName


  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		dF.FactoryID = @FactoryID

  GROUP BY

  		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100																

		 ,fV.ProductLineID + ' ' + dPL.NameShort	
         ,dPL.NameShort												

		 ,fV.ProductID + ' ' + dP.NameShort			
         ,dP.NameShort												
	

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_FactorySummary] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_FactorySummary] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = 'PE'

EXECUTE @RC = [sx_pf_DATAOUTPUT_FactorySummary] 
   @Username
  ,@FactoryID

Print @RC
GO


*/

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_NumericValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_NumericValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_NumericValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	 
	 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 

IF @FactoryID <> '' AND @ProductLineID = ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		 SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]


         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID


SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_NumericValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_NumericValues] TO pf_PlanningFactoryService;
GO



/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = ''

EXECUTE @RC = [sx_pf_DATAOUTPUT_NumericValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID
GO


*/

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_Profit]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_Profit]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_Profit (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
			
	SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT  
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.ProfitLossEffect <> 0 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_Profit] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_Profit] TO pf_PlanningFactoryService;
GO

/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [sx_pf_DATAOUTPUT_Profit] 
   @Username
  ,@FactoryID
  ,@ProductLineID
GO


*/

/*
Procedure to deliver analytic data to the enduser
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
12/2015
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_DATAOUTPUT_TextValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_DATAOUTPUT_TextValues]
GO

CREATE PROCEDURE sx_pf_DATAOUTPUT_TextValues (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0


IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM planning.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID 

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
   
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_TextValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_DATAOUTPUT_TextValues] TO pf_PlanningFactoryService;
GO


/*Testcall

DECLARE @RC int
DECLARE @Username nvarchar(255) = 'SQL'
DECLARE @FactoryID nvarchar(255) = '1'
DECLARE @ProductLineID nvarchar(255) = '1'

EXECUTE @RC = [sx_pf_DATAOUTPUT_TextValues] 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC
GO


*/
/*
DELETE Operation for a ClusterProperty
Deletes a Cluster Property.

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Property deleted => 200 OK
5. PropertyID not found => 204 No Content

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @PropertyID			NVARCHAR(255) = 'rre'

EXECUTE @RC = sx_pf_DELETE_ClusterProperty 
		@Username,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ClusterProperty;
GO

CREATE PROCEDURE sx_pf_DELETE_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ClusterProperty;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);
		
		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete
		DELETE FROM planning.tgCluster 
		WHERE PropertyID = @PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Factory
Deletes an entire Factory (include ProductLines, Products, Values)

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. FactoryID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Factory deleted => 200 OK
5. FactoryID not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FactoryID	NVARCHAR(255) = 'Test'

EXECUTE @RC = sx_pf_DELETE_Factory 
		@Username,
		@FactoryID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Factory;
GO

CREATE PROCEDURE sx_pf_DELETE_Factory
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Factory;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID; 

		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 204;
		END
		ELSE
		BEGIN
			-- STEP 3 - Delete Factory items from all tables
			DELETE FROM planning.tdProductLines 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdProducts 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdValueSeries 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdTime 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tfStatements 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tfValues 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tgFactories
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tgProductLines 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tgProducts 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdFactories 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tFiles 
			WHERE FactoryKey = @FactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Last, because of Right materialization
			DELETE FROM planning.trRights 
			WHERE FactoryID = @FactoryID;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION sx_pf_DELETE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Factory TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a FactoryProperty
Deletes an entire Factory Property

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID or FactoryID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the Factory => 401 Unauthorized
4. Property deleted => 200 OK
5. PropertyID not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FactoryID	NVARCHAR(255) = 'ZT'
DECLARE @PropertyID NVARCHAR(255) = 'rre'

EXECUTE @RC = sx_pf_DELETE_FactoryProperty 
		@Username,
		@FactoryID,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_FactoryProperty;
GO

CREATE PROCEDURE sx_pf_DELETE_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @PropertyKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID = N'' 
			OR @PropertyID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID; 
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete		
		DELETE FROM planning.tgFactories 
		WHERE PropertyKey = @PropertyKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Format

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. FormatID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Format deleted => 200 OK
5. Format not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @FormatID	NVARCHAR(255) = '510'
		
EXECUTE @RC = sx_pf_DELETE_Format 
		@Username,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Format;
GO

CREATE PROCEDURE sx_pf_DELETE_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username, N'NULL') + N''',''' 
			+ ISNULL(@FormatID, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM planning.thFormats 
		WHERE FormatID = @FormatID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Format TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a List
Delete a List 

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ListID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. List deleted => 200 OK
5. List not found => 204 No Content

DECLARE @RC			INT
DECLARE @Username	NVARCHAR(255) = 'SQL'
DECLARE @ListID		NVARCHAR(255) = '510'
		
EXECUTE @RC = sx_pf_DELETE_List 
		@Username,
		@ListID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_List;
GO

CREATE PROCEDURE sx_pf_DELETE_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM planning.thListValues 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thLists 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;
			
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_List TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for one ListValue
Operates over Key, as the ListValue has no ID

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ListID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. ListValue deleted => 200 OK
5. ListValue not found => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @ListValueKey	INT				= 18

		
EXECUTE @RC = sx_pf_DELETE_ListValue 
		@Username,
		@ListValueKey

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ListValue') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ListValue;
GO

CREATE PROCEDURE sx_pf_DELETE_ListValue
	@Username		NVARCHAR(255),
	@ListValueKey	INT
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	 IS NULL SET @Username		= N'';
	IF @ListValueKey IS NULL SET @ListValueKey	= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ListValue;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values
		DELETE FROM planning.thListValues WHERE ListValueKey = @ListValueKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ListValue TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ListValue TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Preference

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID or empty ProductLine => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Non exists Factory / Productline => 404 Not Found
5. Product not found => 204 No Content
6. Product deleted => 200 OK

DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) = 'SQL'
DECLARE @PreferenceUserName		NVARCHAR(255) = 'W8\admin'
DECLARE @ProductLineID			NVARCHAR(255) = 'ZT'
DECLARE @FactoryID				NVARCHAR(255) = 'U'
DECLARE @SettingName			NVARCHAR(255) = 'Demopref'

EXECUTE @RC = sx_pf_DELETE_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Preference;
GO

CREATE PROCEDURE sx_pf_DELETE_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@PreferenceUsername,	N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SettingName,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Preference;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername		= dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName			= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @SettingName		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		IF @SettingName = N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid value of Setting Name', 16, 10);
		END;

		IF NOT EXISTS(SELECT Username FROM planning.trUser WHERE Username = @PreferenceUsername)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Preferences Users don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM planning.trPreferences
		WHERE FactoryID			= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Preference TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Product
Deletes a product and all Values and Attributes

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ProductID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Product not found => 204 No Content
5. Product deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @ProductID		NVARCHAR(255) = '10'

EXECUTE @RC = sx_pf_DELETE_Product 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Product;
GO

CREATE PROCEDURE sx_pf_DELETE_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF @ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM planning.tdValueSeries 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfStatements 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tFiles
		WHERE		ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;


		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Product TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for all Values of the ProductDateTabele

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

DECLARE @RC							INT
		DECLARE @Username			NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID		NVARCHAR(255) = 'U'
		DECLARE @ProductID			NVARCHAR(255) = '1'
		DECLARE @DeleteFormulaFlag	INT = 0

EXECUTE @RC = sx_pf_DELETE_ProductDataTableValues 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@DeleteFormulaFlag

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ProductDataTableValues;
GO

CREATE PROCEDURE sx_pf_DELETE_ProductDataTableValues
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@DeleteFormulaFlag	INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username, N'NULL')								+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')								+ N''',''' 
								+ CAST(ISNULL(@DeleteFormulaFlag, N'NULL') AS NVARCHAR(10))	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION DELETE_PDTValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine ProductKey
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID	= @ProductID;

		IF @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - DELETE
		-- If also formulas, we can delete everything
		IF @DeleteFormulaFlag = 1
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ProductKey = @ProductKey;

			SET @EffectedRows = @@ROWCOUNT;
		END;
		-- If not formulas, only Input Values and update Formula Values to 0/''
		IF @DeleteFormulaFlag = 0
		BEGIN
			-- alle Input Werte komplett
			DELETE fV
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey
				AND dVS.ValueSource IN ('Input','List-Strict','Link');

			-- bei Formelbasierten Werten nur den Wert, nicht die Formel
			UPDATE fV
				SET fV.ValueInt = 0
				,fV.ValueText = ''
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey	= @ProductKey
				AND dVS.ValueSource IN ('XLS', 'XLS-Strict');

			SET @EffectedRows = @@ROWCOUNT;
		END;

		SET @ResultCode	= IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION DELETE_PDTValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT	= ERROR_STATE();
		SET @Comment				= ERROR_MESSAGE();

		ROLLBACK TRANSACTION DELETE_PDTValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductLine
Deletes an entire Product Line (Products, Values, etc.)

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ProductLineID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. ProductLine not found => 204 No Content
5. ProductLine deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'V'

EXECUTE @RC = sx_pf_DELETE_ProductLine 
		@Username,
		@FactoryID,
		@ProductLineID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ProductLine;
GO

CREATE PROCEDURE sx_pf_DELETE_ProductLine
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLine;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF	@ProductLineID	= N'' 
			OR @FactoryID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- 4. Factory und ProductLineKey ermitteln
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0
 		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3 - Delete values
		DELETE FROM planning.tdProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfStatements 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		DELETE FROM planning.trPreferences 
		WHERE		FactoryID		= @FactoryID
				AND ProductlineID	= @ProductLineID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tFiles
		WHERE		ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;


		-- last because of matrialization
		DELETE FROM planning.trRights 
		WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductLine TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductLineProperty
Delete a ProductLineProperty

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Property not found => 204 No Content
5. Property deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @PropertyID		NVARCHAR(255) = 'test'

EXECUTE @RC = sx_pf_DELETE_ProductLineProperty 
		@Username,
		@FactoryID,
		@ProductLineID,
		@PropertyID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ProductLineProperty;
GO

CREATE PROCEDURE sx_pf_DELETE_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + '''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLineProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Delete values
		DELETE FROM planning.tgProductLines 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a ProductProperty
Delete a ProductProperty

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Property not found => 204 No Content
5. Property deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @ProductID		NVARCHAR(255) = '10'
DECLARE @PropertyID		NVARCHAR(255) = 'test'

EXECUTE @RC = sx_pf_DELETE_ProductProperty 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@PropertyID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ProductProperty;
GO

CREATE PROCEDURE sx_pf_DELETE_ProductProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProducts WHERE ProductKey = @ProductKey AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM planning.tgProducts 
		WHERE PropertyKey = @PropertyKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Right
Deletes a Right of a user, user can therefore also delete its own cluster write and lock yourself out for yourself to! 
Must frontend intercept with security inquiry!

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. DeleteUserName is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to the cluster => 401 Unauthorized
4. Right deleted => 200 OK
5. Right not found => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @DeleteUserName NVARCHAR(255) = 'test2'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		
EXECUTE @RC = sx_pf_DELETE_Right 
		@Username,
		@DeleteUserName,
		@FactoryID,
        @ProductLineID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Right') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Right;
GO

CREATE PROCEDURE sx_pf_DELETE_Right
	@Username			NVARCHAR(255),
	@DeleteUserName		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
    @ProductLineID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @DeleteUserKey	INT = 0;
	DECLARE @RightKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Right;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString	(@DeleteUserName);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF  @DeleteUserName		= N'' 
			OR @FactoryID		= N'' 
			OR @ProductLineID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- STEP 1.2 - Determine keys
		SELECT @DeleteUserKey = UserKey FROM planning.trUser WHERE UserName = @DeleteUserName;
		SELECT @RightKey = RightKey FROM planning.trRights WHERE UserKey = @DeleteUserKey AND ProductLineID = @ProductLineID AND FactoryID = @FactoryID;
		
		IF @DeleteUserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values		
		DELETE FROM planning.trRights
		WHERE RightKey = @RightKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Right TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Right TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a TimeID
Deletes a TimeID

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID / ProductLineID / ProductID / TimeID = 0 => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Value of TimeID deleted => 200 OK
5. There is none ProductID => 204 No Content

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '10'
DECLARE @TimeID			INT				= '20180515'

EXECUTE @RC = sx_pf_DELETE_TimeID 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@TimeID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_TimeID') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_TimeID;
GO

CREATE PROCEDURE sx_pf_DELETE_TimeID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_TimeID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3.1 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		-- STEP 3.2 - Delete Timeline

		DELETE FROM planning.tdTime 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_TimeID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_TimeID TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a User
Deletes a User with all his rights. A cluster administrator can't delete himself

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. DeleteUserName is empty => 404 Not Found
2. Delete command comes from non clusteradmin => 403 Forbidden
3. User wants to delete himself => 403 Forbidden
4. DeleteUser don`t exists => 204 No Content
5. deleted user successful => 200 OK

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @DeleteUserName		NVARCHAR(255) = 'W10\admin'
		
EXECUTE @RC = sx_pf_DELETE_User 
		@Username,
		@DeleteUserName

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_User;
GO

CREATE PROCEDURE sx_pf_DELETE_User
	@Username		NVARCHAR(255),
	@DeleteUserName NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_User;

		-- Protect Strings
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString (@DeleteUserName);

		IF @DeleteUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
			
		-- STEP 1.2 - Stop, if Transactuser wants to delete himself
		IF @TransactUsername = @DeleteUsername
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction can`t delete himself', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - delete User		
		DELETE FROM planning.trRights 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.trUser 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@@ROWCOUNT = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_User TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for a Value
Delete a Value of ValueSeries with specified TimeID

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. All IDs are empty or TimeID is 0 => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Value not found => 204 No Content
5. Value deleted => 200 OK

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '10'
DECLARE @ValueSerieID	NVARCHAR(255)	= 'K1'
DECLARE @TimeID			INT				= 20180615

EXECUTE @RC = sx_pf_DELETE_Value 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID,
		@TimeID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Value;
GO

CREATE PROCEDURE sx_pf_DELETE_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	DECLARE @ValueKey			INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Value;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey; --  AND FactoryKey = @FactoryKey;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ValueSeriesID = @ValueSerieID AND ProductKey = @ProductKey; -- AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey;
		SELECT @ValueKey = ValueKey FROM planning.tfValues WHERE ValueSeriesKey = @ValueSerieKey AND TimeID = @TimeID; -- AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND ValueSeriesKey = @ValueSeriesKey;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ValueKey = @ValueKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Value TO pf_PlanningFactoryService;
GO

/*
DELETE Operation for Values
Delete all Values of Value Series of Products

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. All IDs are empty  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. Values not found => 204 No Content
5. Values deleted => 200 OK

DECLARE @RC						INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @ProductID		NVARCHAR(255) = '10'
		DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

EXECUTE @RC = sx_pf_DELETE_Values 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_Values') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_Values;
GO

CREATE PROCEDURE sx_pf_DELETE_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Values;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ValueSeriesID = @ValueSerieID AND ProductKey = @ProductKey;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ValueSeriesKey = @ValueSerieKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF (@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Values TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_Values TO pf_PlanningFactoryService;
GO
/*
DELETE Operation for one ValueSerie
Deletes a ValueSerie and changes the Numbers of the other ValueSeries. If Series No 1 is delete, Procedure creates Placeholder in a new ValueSeries 1.

05/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. ValueSerieID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User does not have write access to ProductLine => 401 Unauthorized
4. ValueSerie not found => 204 No Content
5. ValueSerie deleted => 200 OK

DECLARE @RC						INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @ProductID		NVARCHAR(255) = '10'
		DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

EXECUTE @RC = sx_pf_DELETE_ValueSerie 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSerieID

PRINT @RC
*/

IF EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_DELETE_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_DELETE_ValueSerie;
GO

CREATE PROCEDURE sx_pf_DELETE_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey			INT				= 0;
	DECLARE @ProductKey				INT				= 0;
	DECLARE @DelValueSerieKey		INT				= 0;
	DECLARE @DelValueSerieNo		INT				= 0;
	DECLARE @NewFirstValueSerieKey	INT				= 0;
	DECLARE @NewFirstValueSerieNo	INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
	
		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				 
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @DelValueSerieKey = ValueSeriesKey, @DelValueSerieNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		IF @DelValueSerieKey = 0
		BEGIN
			SET @ResultCode = 204;
		END 
		ELSE
		BEGIN				
			-- STEP 3.2 - Delete ValueSerie 				
			DELETE FROM planning.tfValues 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdValueSeries 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.3 - Decrement all ValueSeriesNo which are bigger than the deleted on
			UPDATE planning.tdValueSeries
			SET ValueSeriesNo = ValueSeriesNo - 1
			FROM planning.tdValueSeries
			WHERE 
				ProductKey = @ProductKey
				AND ValueSeriesNo > @DelValueSerieNo;

			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.4 - if the new first ValueSeries is not NO 1 (1,3..... Numbers), set it to 1
			IF @NewFirstValueSerieNo - 1 > 1 
			BEGIN
				UPDATE planning.tdValueSeries 
				SET	ValueSeriesNo = 1
				FROM planning.tdValueSeries
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesKey = @NewFirstValueSerieKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;
		
		COMMIT TRANSACTION sx_pf_DELETE_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryService;
GO
/* 
EXPORT operation of a single Factory, including
- its properties in gtables
- its global attributes
- its Productline
- its products
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pMaskSQL
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- dbo.sx_pf_pGET_FactoryReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

Test Call:
Results must be stored in table, as column width is to big for SSMS or SQLCMD

0a. Activate Stepnumber for output in Line 624 around and execute prodecedure
0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

1. Execute this to store the results in the table
	IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[tmpout_Export]') AND type in (N'U'))
	DROP TABLE [tmpout_Export]
	GO
	CREATE TABLE tmpout_Export (
		StepNumber INT,
		Command NVARCHAR(MAX)
		)
	INSERT INTO tmpout_Export
		EXEC sx_pf_EXPORT_Factory 'SQL','AT',1

2. Copy this table Content in a file over a .bat file with this command
	bcp.exe "SELECT Command FROM PlanningFactoryCars.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Factory]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_EXPORT_Factory];
GO

CREATE PROCEDURE [sx_pf_EXPORT_Factory]
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = N'''' 
						+ ISNULL(@Username, N'NULL')									+ N''',''' 
						+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
						+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER = 0;						
	DECLARE @ResultCode			INTEGER = 501;					
	DECLARE @TimestampCall		DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @FactoryKey BIGINT = 0;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID(@FactoryID);

		IF @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE FactoryKey = @FactoryKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource2		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource3		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource4		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource5		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource6		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource7		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource8		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource9		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource10	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource11	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource12	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource13	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource14	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource15	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource16	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource17	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource18	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource19	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource20	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource21	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource22	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource23	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource24	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource25	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################

		IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
				 (0,1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0,1, N'DECLARE @Username NVARCHAR(255)		= ''SQL''')
				,(0,1, N'DECLARE @FactoryID NVARCHAR(255)		= ''' + @FactoryID + N'''')
				,(0,1, N'--This FactoryID will be deleted during import, if it exists. You should be sure !');

			-- DELETE FactoryID if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'EXEC sx_pf_DELETE_Factory @Username, @FactoryID;');
			
			-- POST Factory
			INSERT INTO @ExportTable
				SELECT 1001,1, N'EXEC dbo.sx_pf_POST_Factory @Username, @FactoryID,'''
					+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',''' 
					+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',''' 
					+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',''' 
					+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''';'
				 FROM planning.tdFactories dF WHERE dF.FactoryKey = @FactoryKey

			-- POST Productlines
			INSERT INTO @ExportTable 
			SELECT 1002,1, N'EXEC dbo.sx_pf_POST_Productline @Username,''' + ProductLineID + N''', @FactoryID,'''
				+ dbo.sx_pf_pMaskSQL(NameShort)					+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)					+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ResponsiblePerson)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ImageName)					+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(DefaultTemplate)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource1)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias1)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource2)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias2)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource3)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias3)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource4)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias4)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource5)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias5)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource6)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias6)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource7)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias7)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource8)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias8)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource9)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias9)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource10)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias10)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource11)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias11)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource12)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias12)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource13)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias13)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource14)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias14)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource15)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias15)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource16)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias16)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource17)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias17)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource18)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias18)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource19)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias19)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource20)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias20)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource21)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias21)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource22)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias22)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource23)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias23)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource24)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias24)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource25)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias25)	+ N''';'
			FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''',''' + ProductLineID + N''', @FactoryID,''' 
				+ TimeType									+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL([Status])			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(Template)			+ N''',''' 
				+ TemplateVersion							+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute1)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute2)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute3)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute4)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute5)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute6)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute7)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute8)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute9)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute10) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute11) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute12) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute13) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute14) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute15) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute16) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute17) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute18) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute19) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute20) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute21) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute22) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute23) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute24) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute25) + N''';'
			FROM planning.tdProducts WHERE FactoryKey = @FactoryKey;
					
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				,ROW_NUMBER()OVER (ORDER BY FactoryID,ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				,'EXEC dbo.sx_pf_POST_ValueSerie @Username,''' + ProductID + ''', ''' + ProductLineID +''', @FactoryID,''' +
				+ ValueSeriesID								+ N''','''
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''','''
				+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''','''
				+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''','''
				+ ValueSource								+ N''','''
				+ ValueListID								+ N''','''
				+ ValueFormatID								+ N''','''
				+ Unit										+ N''','''
				+ CAST(Scale AS NVARCHAR (255))				+ N''','''
				+ Effect									+ N''','''
				+ EffectParameter							+ N''';'
			FROM planning.tdValueSeries

			WHERE FactoryKey = @FactoryKey
			
			ORDER BY 
				 FactoryID
				,ProductLineID
				,ProductID
				,ValueSeriesNo;

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],' -- with DeleteFlag, AddValue,FormatID 
					FROM planning.tdTime t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, ''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,1,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(''''' 
							+ ValueSeriesID							+ N''''',' 
							+ CAST(TimeID AS NVARCHAR(255))			+ N',''''' 
							+ ValueFormula 							+ N''''','''''
							+ CAST(ValueInt AS NVARCHAR(255)) 		+ N''''',''''' 
							+ dbo.sx_pf_pMaskSQL(ValueText)		+ N''''',''''' 
							+ dbo.sx_pf_pMaskSQL(ValueComment)	+ ''''')],' 
					FROM planning.tfValues t
					WHERE t.ProductKey = dP.ProductKey
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,0,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Factory Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'EXEC dbo.sx_pf_POST_FactoryProperty @Username, @FactoryID,''' 
					+ PropertyID									+ N''','''
					+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''','''
					+ Unit											+ N''','''
					+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''','''
					+ CAST(ValueInt				AS NVARCHAR(255))	+ N''','''
					+ CAST(Scale				AS NVARCHAR(255))	+ N''','''
					+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''','''
					+ FormatID										+ N''';'
			FROM planning.tgFactories
			WHERE FactoryKey = @FactoryKey; 

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductlineProperty @Username,''' + ProductLineID +''', @FactoryID,''' 
					+ PropertyID									+ N''','''
					+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''','''
					+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''','''
					+ Unit											+ N''','''
					+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''','''
					+ CAST(ValueInt				AS NVARCHAR(255))	+ N''','''
					+ CAST(Scale				AS NVARCHAR(255))	+ N''','''
					+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''','''
					+ FormatID										+ N''';'
			FROM planning.tgProductLines
			WHERE FactoryKey = @FactoryKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6002,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,''' + ProductID + N''',''' + ProductLineID +''', @FactoryID,''' 
				+ PropertyID									+ N''', '''
				+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''', '''
				+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''', '''
				+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''', '''
				+ Unit											+ N''', '''
				+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''', '''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''', '''
				+ CAST(Scale				AS NVARCHAR(255))	+ N''', '''
				+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''', '''
				+ FormatID										+ N''';'
			FROM planning.tgProducts
			WHERE  FactoryKey = @FactoryKey; 
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameShort)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM planning.thLists
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ dbo.sx_pf_pMaskSQL(ValueText)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ValueComment)	+ N''',''' 
				+ FormatID								+ N''';'
			FROM planning.thListValues
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM planning.tdValueSeries 
				WHERE FactoryKey = @FactoryKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM planning.tgProducts 
				WHERE 
						FactoryKey = @FactoryKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID										+ N''','''
				+ BackgroundColor								+ N''',''' 
				+ FontColor										+ N''',''' 
				+ ValueFormat									+ N''','''
				+ CAST(Bold				AS NVARCHAR(255))		+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))		+ N''','''
				+ AlignmentHorizontal							+ N''','''
				+ AlignmentVertical								+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))		+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))		+ N''','''
				+ CAST(BorderToPrevious AS NVARCHAR(255))		+ N''';'
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (8000,1, N'GO');

		END -- of Output for SQL Server
		
	-- STEP 3.3 - OUTPUT Type for PF Client Execution ##################################################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0,1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0,1, N'--DECLARE #Username NVARCHAR(255)	= ''SQL''')
				,(0,1, N'--DECLARE #FactoryID NVARCHAR(255)	= ''' + @FactoryID + N'''')
				,(0,1, N'--This FactoryID will be deleted during import, if it exists. You should be sure !');

		
			-- DELETE Factory if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'sx_pf_DELETE_Factory' + @spr + N'@Username' + @spr + N'#Username' + @spr + N'@FactoryID' + @spr + N'#FactoryID');

			-- POST Factory
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'dbo.sx_pf_POST_Factory' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'
					+ @spr + N'@NameShort'			+ @spr + NameShort 
					+ @spr + N'@NameLong'			+ @spr + NameLong  
					+ @spr + N'@CommentUser'		+ @spr + CommentUser  
					+ @spr + N'@CommentDev'			+ @spr + CommentDev  
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson  
					+ @spr + N'@ImageName'			+ @spr + ImageName  
			FROM planning.tdFactories dF WHERE dF.FactoryKey = @FactoryKey
						
			-- POST Productlines
			INSERT INTO @ExportTable 
			SELECT 1002,1, N'dbo.sx_pf_POST_Productline' 
				+ @spr + N'@Username'					+ @spr + N'#Username' 
				+ @spr + N'@ProductlineID'				+ @spr + ProductlineID 
				+ @spr + N'@FactoryID'					+ @spr + N'#FactoryID'
				+ @spr + N'@NameShort'					+ @spr + NameShort 
				+ @spr + N'@NameLong'					+ @spr + NameLong  
				+ @spr + N'@CommentUser'				+ @spr + CommentUser  
				+ @spr + N'@CommentDev'					+ @spr + CommentDev  
				+ @spr + N'@ResponsiblePerson'			+ @spr + ResponsiblePerson  
				+ @spr + N'@ImageName'					+ @spr + ImageName  
				+ @spr + N'@DefaultTemplate'			+ @spr + DefaultTemplate  
				+ @spr + N'@GlobalAttributeSource1'		+ @spr + GlobalAttributeSource1  
				+ @spr + N'@GlobalAttributeAlias1'		+ @spr + GlobalAttributeAlias1  
				+ @spr + N'@GlobalAttributeSource2'		+ @spr + GlobalAttributeSource2  
				+ @spr + N'@GlobalAttributeAlias2'		+ @spr + GlobalAttributeAlias2  
				+ @spr + N'@GlobalAttributeSource3'		+ @spr + GlobalAttributeSource3  
				+ @spr + N'@GlobalAttributeAlias3'		+ @spr + GlobalAttributeAlias3 
				+ @spr + N'@GlobalAttributeSource4'		+ @spr + GlobalAttributeSource4  
				+ @spr + N'@GlobalAttributeAlias4'		+ @spr + GlobalAttributeAlias4 
				+ @spr + N'@GlobalAttributeSource5'		+ @spr + GlobalAttributeSource5  
				+ @spr + N'@GlobalAttributeAlias5'		+ @spr + GlobalAttributeAlias5 
				+ @spr + N'@GlobalAttributeSource6'		+ @spr + GlobalAttributeSource6  
				+ @spr + N'@GlobalAttributeAlias6'		+ @spr + GlobalAttributeAlias6 
				+ @spr + N'@GlobalAttributeSource7'		+ @spr + GlobalAttributeSource7  
				+ @spr + N'@GlobalAttributeAlias7'		+ @spr + GlobalAttributeAlias7 
				+ @spr + N'@GlobalAttributeSource8'		+ @spr + GlobalAttributeSource8  
				+ @spr + N'@GlobalAttributeAlias8'		+ @spr + GlobalAttributeAlias8 
				+ @spr + N'@GlobalAttributeSource9'		+ @spr + GlobalAttributeSource9  
				+ @spr + N'@GlobalAttributeAlias9'		+ @spr + GlobalAttributeAlias9 
				+ @spr + N'@GlobalAttributeSource10'	+ @spr + GlobalAttributeSource10  
				+ @spr + N'@GlobalAttributeAlias10'		+ @spr + GlobalAttributeAlias10 
				+ @spr + N'@GlobalAttributeSource11'	+ @spr + GlobalAttributeSource11  
				+ @spr + N'@GlobalAttributeAlias11'		+ @spr + GlobalAttributeAlias11  
				+ @spr + N'@GlobalAttributeSource12'	+ @spr + GlobalAttributeSource12  
				+ @spr + N'@GlobalAttributeAlias12'		+ @spr + GlobalAttributeAlias12  
				+ @spr + N'@GlobalAttributeSource13'	+ @spr + GlobalAttributeSource13  
				+ @spr + N'@GlobalAttributeAlias13'		+ @spr + GlobalAttributeAlias13 
				+ @spr + N'@GlobalAttributeSource14'	+ @spr + GlobalAttributeSource14  
				+ @spr + N'@GlobalAttributeAlias14'		+ @spr + GlobalAttributeAlias14 
				+ @spr + N'@GlobalAttributeSource15'	+ @spr + GlobalAttributeSource15  
				+ @spr + N'@GlobalAttributeAlias15'		+ @spr + GlobalAttributeAlias15 
				+ @spr + N'@GlobalAttributeSource16'	+ @spr + GlobalAttributeSource16  
				+ @spr + N'@GlobalAttributeAlias16'		+ @spr + GlobalAttributeAlias16 
				+ @spr + N'@GlobalAttributeSource17'	+ @spr + GlobalAttributeSource17  
				+ @spr + N'@GlobalAttributeAlias17'		+ @spr + GlobalAttributeAlias17 
				+ @spr + N'@GlobalAttributeSource18'	+ @spr + GlobalAttributeSource18  
				+ @spr + N'@GlobalAttributeAlias18'		+ @spr + GlobalAttributeAlias18 
				+ @spr + N'@GlobalAttributeSource19'	+ @spr + GlobalAttributeSource19  
				+ @spr + N'@GlobalAttributeAlias19'		+ @spr + GlobalAttributeAlias19 
				+ @spr + N'@GlobalAttributeSource20'	+ @spr + GlobalAttributeSource20  
				+ @spr + N'@GlobalAttributeAlias20'		+ @spr + GlobalAttributeAlias20 
				+ @spr + N'@GlobalAttributeSource21'	+ @spr + GlobalAttributeSource21  
				+ @spr + N'@GlobalAttributeAlias21'		+ @spr + GlobalAttributeAlias21  
				+ @spr + N'@GlobalAttributeSource22'	+ @spr + GlobalAttributeSource22  
				+ @spr + N'@GlobalAttributeAlias22'		+ @spr + GlobalAttributeAlias22  
				+ @spr + N'@GlobalAttributeSource23'	+ @spr + GlobalAttributeSource23  
				+ @spr + N'@GlobalAttributeAlias23'		+ @spr + GlobalAttributeAlias23 
				+ @spr + N'@GlobalAttributeSource24'	+ @spr + GlobalAttributeSource24  
				+ @spr + N'@GlobalAttributeAlias24'		+ @spr + GlobalAttributeAlias24 
				+ @spr + N'@GlobalAttributeSource25'	+ @spr + GlobalAttributeSource25  
				+ @spr + N'@GlobalAttributeAlias25'		+ @spr + GlobalAttributeAlias25 + N''
			FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'dbo.sx_pf_POST_ProductEmpty' 
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + ProductID 
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@TimeType'			+ @spr + TimeType
				+ @spr + N'@NameShort'			+ @spr + NameShort
				+ @spr + N'@NameLong'			+ @spr + NameLong
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson
				+ @spr + N'@ImageName'			+ @spr + ImageName
				+ @spr + N'@Status'				+ @spr + [Status] 
				+ @spr + N'@Template'			+ @spr + Template 
				+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion
				+ @spr + N'@GA1'				+ @spr + GlobalAttribute1
				+ @spr + N'@GA2'				+ @spr + GlobalAttribute2
				+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
				+ @spr + N'@GA4'				+ @spr + GlobalAttribute4
				+ @spr + N'@GA5'				+ @spr + GlobalAttribute5
				+ @spr + N'@GA6'				+ @spr + GlobalAttribute6
				+ @spr + N'@GA7'				+ @spr + GlobalAttribute7
				+ @spr + N'@GA8'				+ @spr + GlobalAttribute8
				+ @spr + N'@GA9'				+ @spr + GlobalAttribute9
				+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
				+ @spr + N'@GA11'				+ @spr + GlobalAttribute11
				+ @spr + N'@GA12'				+ @spr + GlobalAttribute12
				+ @spr + N'@GA13'				+ @spr + GlobalAttribute13
				+ @spr + N'@GA14'				+ @spr + GlobalAttribute14
				+ @spr + N'@GA15'				+ @spr + GlobalAttribute15
				+ @spr + N'@GA16'				+ @spr + GlobalAttribute16
				+ @spr + N'@GA17'				+ @spr + GlobalAttribute17
				+ @spr + N'@GA18'				+ @spr + GlobalAttribute18
				+ @spr + N'@GA19'				+ @spr + GlobalAttribute19
				+ @spr + N'@GA20'				+ @spr + GlobalAttribute20
				+ @spr + N'@GA21'				+ @spr + GlobalAttribute21
				+ @spr + N'@GA22'				+ @spr + GlobalAttribute22
				+ @spr + N'@GA23'				+ @spr + GlobalAttribute23
				+ @spr + N'@GA24'				+ @spr + GlobalAttribute24
				+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM planning.tdProducts WHERE FactoryKey = @FactoryKey; 
									
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				,ROW_NUMBER() OVER (ORDER BY FactoryID,ProductlineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'dbo.sx_pf_POST_ValueSerie'
				+ @spr + N'@Username'				+ @spr + N'#Username' 
				+ @spr + N'@ProductID'				+ @spr + ProductID
				+ @spr + N'@ProductLineID'			+ @spr + ProductLineID
				+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
				+ @spr + N'@ValueSeriesID'			+ @spr + ValueSeriesID 
				+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))
				+ @spr + N'@NameShort'				+ @spr + NameShort
				+ @spr + N'@NameLong'				+ @spr + NameLong
				+ @spr + N'@CommentUser'			+ @spr + CommentUser
				+ @spr + N'@CommentDev'				+ @spr + CommentDev
				+ @spr + N'@ImageName'				+ @spr + ImageName
				+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255))
				+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255))
				+ @spr + N'@ValueSource'			+ @spr + ValueSource
				+ @spr + N'@ValueListID'			+ @spr + ValueListID
				+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID
				+ @spr + N'@Unit'					+ @spr + Unit
				+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255))
				+ @spr + N'@Effect'					+ @spr + Effect
				+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM planning.tdValueSeries
			WHERE FactoryKey = @FactoryKey
			ORDER BY 
				 FactoryID
				,ProductlineID
				,ProductID
				,ValueSeriesNo; 

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID,dP.ProductLineID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10))+',0,0,'+ FormatID + ')],' -- with DeleteFlag, AddValue,FormatID 
					FROM planning.tdTime t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'dbo.sx_pf_POST_Timeline'
					+ @spr + N'@Username'							+ @spr + N'#Username' 
					+ @spr + N'@ProductID'							+ @spr + ProductID
					+ @spr + N'@ProductLineID'						+ @spr + ProductLineID
					+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
					+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' 
					+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;		
			
			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID, dP.ProductLineID
					,(SELECT N'[(''' 
							+ ValueSeriesID									+ N''','
							+ CAST(TimeID	AS NVARCHAR(255))				+ N',''' 
							+ dbo.sx_pf_pRemoveLinebreaks(ValueFormula)		+ N''',''' 
							+ CAST(ValueInt AS NVARCHAR(255)) 				+ N''',''' 
							+ ValueText 									+ N''',''' 
							+ ValueComment									+ ''')],' 
					FROM planning.tfValues t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.ValueSeriesID, t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.FactoryKey = @FactoryKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'dbo.sx_pf_POST_ProductDataTableValues'
				+ @spr + N'@Username'						+ @spr + N'#Username' 
				+ @spr + N'@ProductID'						+ @spr + ProductID
				+ @spr + N'@ProductLineID'					+ @spr + ProductLineID 
				+ @spr + N'@FactoryID'						+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'		+ @spr + N'0' 
				+ @spr + N'@ValuesInBracketsCommaSeparated' + @spr + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;
			
			-- POST Factory Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'dbo.sx_pf_POST_FactoryProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgFactories 
			WHERE FactoryKey = @FactoryKey; 

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'dbo.sx_pf_POST_ProductlineProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgProductLines
			WHERE FactoryKey = @FactoryKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6003,1, N'dbo.sx_pf_POST_ProductProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + ProductID
				+ @spr + N'@ProductLineID'		+ @spr + ProductLineID
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@PropertyID'			+ @spr + PropertyID
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255)) 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgProducts
			WHERE  FactoryKey = @FactoryKey;
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'dbo.sx_pf_POST_List'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort
				+ @spr + N'@NameLong'		+ @spr + NameLong
				+ @spr + N'@CommentDev'		+ @spr + CommentDev
				+ @spr + N'@CommentUser'	+ @spr + CommentUser
				+ @spr + N'@Datentyp'		+ @spr + Datentyp
				+ @spr + N'@Source'			+ @spr + [Source]
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM planning.thLists
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'dbo.sx_pf_POST_ListValue'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListValuekey'	+ @spr + N'0'
				+ @spr + N'@ListID'			+ @spr + ListID
				+ @spr + N'@ValueInt'		+ @spr + CAST(ValueInt	AS NVARCHAR(255))
				+ @spr + N'@Scale'			+ @spr + CAST(Scale		AS NVARCHAR(255))
				+ @spr + N'@ValueText'		+ @spr + ValueText
				+ @spr + N'@ValueComment'	+ @spr + ValueComment
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM planning.thListValues
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL SELECT ValueFormatID FROM planning.tdValueSeries WHERE FactoryKey = @FactoryKey
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'dbo.sx_pf_POST_Format'
					 + @spr + N'@Username'				+ @spr + N'#Username'
					 + @spr + N'@FormatID'				+ @spr + FormatID
					 + @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					 + @spr + N'@FontColor'				+ @spr + FontColor
					 + @spr + N'@ValueFormat'			+ @spr + ValueFormat 
					 + @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255))
					 + @spr + N'@Italic'				+ @spr + CAST(Italic			AS NVARCHAR(255))
					 + @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal
					 + @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical
					 + @spr + N'@TextUnderlined'		+ @spr + CAST(TextUnderlined	AS NVARCHAR(255))
					 + @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255))
					 + @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client	
	
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY 
			 StepNumber
			,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

  
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_Factory] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_Factory] TO pf_PlanningFactoryService;
GO



	


/* Procedure to export one single product, including
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pMaskSQL
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- dbo.sx_pf_pGET_ProductlineReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

TestCall
DECLARE @RC int
DECLARE @Username			nvarchar(255) = 'SQL'
DECLARE @FactoryID			nvarchar(255) = 'ZT'
DECLARE @ProductLineID		nvarchar(255) = 'U'
DECLARE @ProductID			nvarchar(255) = '1'
DECLARE @OutputForSQLServer INT = 1

EXECUTE @RC = [sx_pf_EXPORT_Product] 
   @Username
  ,@FactoryID
  ,@ProductLineID
  ,@ProductID
  ,@OutputForSQLServer

 PRINT @RC
 */

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Product]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_EXPORT_Product]
GO

CREATE  PROCEDURE [sx_pf_EXPORT_Product]
		@Username			NVARCHAR(255),
		@FactoryID			NVARCHAR(255),
		@ProductLineID		NVARCHAR(255),
		@ProductID			NVARCHAR(255),
		@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username, N'NULL')									+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')									+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER = 0;						
	DECLARE @ResultCode 		INTEGER = 501;						
	DECLARE @TimestampCall		DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';				
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username = N'';
	IF @FactoryID			IS NULL SET @FactoryID = N'';
	IF @ProductLineID		IS NULL SET @ProductLineID = N'';
	IF @ProductID			IS NULL SET @ProductID = N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @ProductKey BIGINT = 0;
		DECLARE @ProductLineKey BIGINT = 0;
		DECLARE @Template NVARCHAR (255) = '';

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);

		IF @FactoryID = N'' OR @ProductLineID = N''  OR @ProductID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductlineKey = ProductlineKey FROM planning.tdProductLines dPL WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	 
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine Template
		SELECT @Template = Template FROM planning.tdProducts WHERE ProductKey = @ProductKey;

		-- Deterime all Productlines using this Template
		DECLARE @DependendProductLinesIDs AS TABLE (ProductlineKey BIGINT);
		INSERT INTO @DependendProductLinesIDs (ProductlineKey)
			SELECT DISTINCT ProductlineKey FROM planning.tdProducts WHERE Template = @Template;

		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductKey = @ProductKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT DISTINCT GlobalAttributeSource1	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource2	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource3	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource4	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource5	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource6	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource7	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource8	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource9	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
		)									
		INSERT INTO @DependendListIDs (ListID)
			SELECT DISTINCT ValueListID
			FROM lists
			WHERE ValueListID != '';
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
											 StepNumber INT NOT NULL
											,RowNumber	INT NOT NULL
											,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################
	   	IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
			  (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			, (0, 1, N'DECLARE @Username		NVARCHAR(255)	= ''SQL''')
			, (0, 1, N'DECLARE @FactoryID		NVARCHAR(255)	= ''' + @FactoryID		+ N'''')
			, (0, 1, N'DECLARE @ProductLineID	NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
			, (0, 1, N'DECLARE @ProductID		NVARCHAR(255)	= ''' + @ProductID		+ N'''')
			, (0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Product if exists
			INSERT INTO @ExportTable 
			VALUES (1000, 1, N'EXEC sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;');
			
			-- POST Product
			INSERT INTO @ExportTable 
			SELECT 2000, 1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,''' 
				+ TimeType + N''',''' 
				+ dbo.sx_pf_pMaskSQL (NameShort)			+ N''','''
				+ dbo.sx_pf_pMaskSQL (NameLong)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentUser)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (ResponsiblePerson)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (ImageName)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL ([Status])				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (Template)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (TemplateVersion)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute1)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute2)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute3)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute4)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute5)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute6)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute7)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute8)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute9)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute10)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute11)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute12)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute13)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute14)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute15)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute16)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute17)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute18)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute19)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute20)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute21)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute22)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute23)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute24)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (GlobalAttribute25)	+ N''';'
			FROM planning.tdProducts WHERE ProductKey = @ProductKey;

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 3000, 1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username, @ProductID, @ProductLineID, @FactoryID,''' +
				+ PropertyID								+ N''','''
				+ dbo.sx_pf_pMaskSQL (PropertyName)		+ N''','''
				+ dbo.sx_pf_pMaskSQL (CommentUser)		+ N''','''
				+ dbo.sx_pf_pMaskSQL (CommentDev)		+ N''','''
				+ dbo.sx_pf_pMaskSQL (Unit)				+ N''','''
				+ dbo.sx_pf_pMaskSQL (ValueText)		+ N''','''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''','''
				+ CAST(Scale AS NVARCHAR(255))				+ N''','''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''','''
				+ FormatID									+ N''';'
			FROM planning.tgProducts
			WHERE  ProductKey = @ProductKey; 

			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable
			SELECT 4000 
				, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
				, N'EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,''' +
				+ ValueSeriesID									+ N''',''' 
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (NameShort)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (NameLong)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentUser)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (ImageName)			+ N''',''' 
				+ CAST([IsNumeric]		AS NVARCHAR (255))		+ N''',''' 
				+ CAST(VisibilityLevel	AS NVARCHAR (255))		+ N''',''' 
				+ ValueSource									+ N''',''' 
				+ ValueListID									+ N''',''' 
				+ ValueFormatID									+ N''',''' 
				+ Unit											+ N''',''' 
				+ CAST(Scale AS NVARCHAR (255))					+ N''',''' 
				+ Effect										+ N''',''' 
				+ EffectParameter								+ N''';'
			FROM planning.tdValueSeries
			WHERE ProductKey = @ProductKey
			ORDER BY
				 ProductID
				,ValueSeriesNo; 
		   
			;WITH tms AS (
				SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],'
				FROM planning.tdTime
				WHERE  ProductKey = @ProductKey
				ORDER BY TimeID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 5000, 1, N'EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,''' + LEFT(val, LEN(val) - 1) + N''';'
			FROM tms
			WHERE NOT val IS NULL;
				  
			-- POST ProductDataTableValues
			;WITH vals AS (
				SELECT (SELECT  N'[(''''' 
					+ fV.ValueSeriesID							+ N''''','  
					+ CAST(fV.TimeID	AS NVARCHAR(255))		+ N',''''' 
					+ fV.ValueFormula							+ N''''',''''' 
					+ CAST(fV.ValueInt	AS NVARCHAR(255))		+ N''''',''''' 
					+ dbo.sx_pf_pMaskSQL (fV.ValueText)		+ N''''',''''' 
					+ dbo.sx_pf_pMaskSQL (fV.ValueComment)	+ N''''')],'
				FROM planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
				WHERE fV.ProductKey = @ProductKey 
				ORDER BY dVS.ValueSeriesNo
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 6000, 1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,'''
					+ LEFT(val, LEN(val) - 1) + N''';' 
			FROM vals
			WHERE NOT val IS NULL;		

			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001, 1, N'EXECUTE dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ N''','''
				+ dbo.sx_pf_pMaskSQL (NameShort)	+ N''','''
				+ dbo.sx_pf_pMaskSQL (NameLong)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentUser)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM planning.thLists
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002, 1, N'EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ dbo.sx_pf_pMaskSQL (ValueText)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL (ValueComment) + N''',''' 
				+ FormatID								+ N''';'
		   	FROM planning.thListValues
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
			
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT ValueFormatID 
				FROM planning.tdValueSeries 
				WHERE ProductKey = @ProductKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM planning.tgProducts 
				WHERE 
						ProductKey = @ProductKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 8001, 1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID									+ N''','''
				+ BackgroundColor							+ N''',''' 
				+ FontColor									+ N''',''' 
				+ ValueFormat								+ N''',''' 
				+ CAST(Bold				AS NVARCHAR(255))	+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))	+ N''','''
				+ AlignmentHorizontal						+ N''','''
				+ AlignmentVertical							+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToPrevious	AS NVARCHAR(255))	+ N''';'
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (9000, 1, N'GO');

		END -- of Output for SQL Server

		 -- STEP 3.3 - OUTPUT Type for PF Client Execution, NO  allowed !! due to hard parameter coding in XLS Client###############################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'--DECLARE #Username		NVARCHAR(255)	= ''SQL''')
				,(0, 1, N'--DECLARE #FactoryID		NVARCHAR(255)	= ''' + @FactoryID		+ N'''')
				,(0, 1, N'--DECLARE #ProductLineID NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
				,(0, 1, N'--DECLARE #ProductID		NVARCHAR(255)	= ''' + @ProductID		+ N'''')
				,(0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Product if exists
			INSERT INTO @ExportTable 
			VALUES (1000, 1, N'sx_pf_DELETE_Product' 
				+ @spr + '@Username'		+ @spr + '#Username' 
				+ @spr + '@FactoryID'		+ @spr + '#FactoryID' 
				+ @spr + '@ProductlineID'	+ @spr + '#ProductLineID' 
				+ @spr + '@ProductID'		+ @spr + '#ProductID');

			-- POST Product
			INSERT INTO @ExportTable 
			SELECT 2000, 1, N'dbo.sx_pf_POST_ProductEmpty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'#ProductID' 
					+ @spr + N'@ProductlineID'		+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@TimeType'			+ @spr + TimeType 
					+ @spr + N'@NameShort'			+ @spr + NameShort 
					+ @spr + N'@NameLong'			+ @spr + NameLong 
					+ @spr + N'@CommentUser'		+ @spr + CommentUser 
					+ @spr + N'@CommentDev'			+ @spr + CommentDev 
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson 
					+ @spr + N'@ImageName'			+ @spr + ImageName 
					+ @spr + N'@Status'				+ @spr + [Status] 
					+ @spr + N'@Template'			+ @spr + Template 
					+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion 
					+ @spr + N'@GA1'				+ @spr + GlobalAttribute1 
					+ @spr + N'@GA2'				+ @spr + GlobalAttribute2 
					+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
					+ @spr + N'@GA4'				+ @spr + GlobalAttribute4 
					+ @spr + N'@GA5'				+ @spr + GlobalAttribute5 
					+ @spr + N'@GA6'				+ @spr + GlobalAttribute6 
					+ @spr + N'@GA7'				+ @spr + GlobalAttribute7 
					+ @spr + N'@GA8'				+ @spr + GlobalAttribute8 
					+ @spr + N'@GA9'				+ @spr + GlobalAttribute9 
					+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
					+ @spr + N'@GA11'				+ @spr + GlobalAttribute11 
					+ @spr + N'@GA12'				+ @spr + GlobalAttribute12 
					+ @spr + N'@GA13'				+ @spr + GlobalAttribute13 
					+ @spr + N'@GA14'				+ @spr + GlobalAttribute14 
					+ @spr + N'@GA15'				+ @spr + GlobalAttribute15 
					+ @spr + N'@GA16'				+ @spr + GlobalAttribute16 
					+ @spr + N'@GA17'				+ @spr + GlobalAttribute17 
					+ @spr + N'@GA18'				+ @spr + GlobalAttribute18 
					+ @spr + N'@GA19'				+ @spr + GlobalAttribute19 
					+ @spr + N'@GA20'				+ @spr + GlobalAttribute20 
					+ @spr + N'@GA21'				+ @spr + GlobalAttribute21 
					+ @spr + N'@GA22'				+ @spr + GlobalAttribute22 
					+ @spr + N'@GA23'				+ @spr + GlobalAttribute23 
					+ @spr + N'@GA24'				+ @spr + GlobalAttribute24 
					+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM planning.tdProducts WHERE ProductKey = @ProductKey;

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 3000, 1, N'dbo.sx_pf_POST_ProductProperty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'#ProductID' 
					+ @spr + N'@ProductLineID'		+ @spr + N'#ProductlineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@PropertyID'			+ @spr + PropertyID 
					+ @spr + N'@PropertyName'		+ @spr + PropertyName 
					+ @spr + N'@CommentUser'		+ @spr + CommentUser 
					+ @spr + N'@CommentDev'			+ @spr + CommentDev 
					+ @spr + N'@Unit'				+ @spr + Unit 
					+ @spr + N'@ValueText'			+ @spr + ValueText 
					+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt AS NVARCHAR(255)) 
					+ @spr + N'@Scale'				+ @spr + CAST(Scale AS NVARCHAR(255)) 
					+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty AS NVARCHAR(255)) 
					+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgProducts
			WHERE ProductKey = @ProductKey; 
			
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 4000 AS Step
					, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
					, N'dbo.sx_pf_POST_ValueSerie' 
					+ @spr + N'@Username'				+ @spr + N'#Username' 
					+ @spr + N'@ProductID'				+ @spr + N'#ProductID' 
					+ @spr + N'@ProductLineID'			+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
					+ @spr + N'@ValueSeriesID'			+ @spr + N'' + ValueSeriesID  
					+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))  
					+ @spr + N'@NameShort'				+ @spr + NameShort 
					+ @spr + N'@NameLong'				+ @spr + NameLong 
					+ @spr + N'@CommentUser'			+ @spr + CommentUser 
					+ @spr + N'@CommentDev'				+ @spr + CommentDev 
					+ @spr + N'@ImageName'				+ @spr + ImageName 
					+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255)) 
					+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255)) 
					+ @spr + N'@ValueSource'			+ @spr + ValueSource 
					+ @spr + N'@ValueListID'			+ @spr + ValueListID 
					+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID 
					+ @spr + N'@Unit'					+ @spr + Unit 
					+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255)) 
					+ @spr + N'@Effect'					+ @spr + Effect 
					+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM planning.tdValueSeries
			WHERE ProductKey = @ProductKey 
			ORDER BY
				 ProductID
				,ValueSeriesNo;

			-- POST Timeline
			;WITH tms AS (
				SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + ',0,0,''''' + FormatID + ''''')],'
				FROM planning.tdTime
				WHERE  ProductKey = @ProductKey
				ORDER BY TimeID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable
			SELECT 5000, 1, N'dbo.sx_pf_POST_Timeline' 
				+ @spr + N'@Username'							+ @spr + N'#Username' 
				+ @spr + N'@ProductID'							+ @spr + N'#ProductID' 
				+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID'
				+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'0' + N'' 
				+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + N'' + LEFT(val, LEN(val) - 1) + N''
			FROM tms
			WHERE NOT val IS NULL;
			
			-- POST Product DataTable
			;WITH vals AS (
				SELECT (SELECT  N'[(''' 
					+ fV.ValueSeriesID													+ N''',' 
					+ CAST(fV.TimeID AS NVARCHAR(255))									+ N',''' 
					+ dbo.sx_pf_pRemoveLinebreaks(fV.ValueFormula)						+ N''',''' 
					+ CAST(fV.ValueInt AS NVARCHAR(255))								+ N''',''' 
					+ fV.ValueText														+ N''',''' 
					+ fV.ValueComment													+ N''')],'
				FROM planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
				WHERE fV.ProductKey = @ProductKey 
				ORDER BY dVS.ValueSeriesNo
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS val 
			)
			INSERT INTO @ExportTable 
			SELECT 6000, 1, N'dbo.sx_pf_POST_ProductDataTableValues' 
				+ @spr + N'@Username'							+ @spr + N'#Username' 
				+ @spr + N'@ProductID'							+ @spr + N'#ProductID' 
				+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' 
				+ @spr + N'@ValuesInBracketsCommaSeparated'		+ @spr + N''
				+ LEFT(val, LEN(val) - 1) + N''
			FROM vals
			WHERE NOT val IS NULL;

			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001, 1, N'dbo.sx_pf_POST_List' 
				+ @spr + N'@Username'		+ @spr + N'#Username' 
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort 
				+ @spr + N'@NameLong'		+ @spr + NameLong 
				+ @spr + N'@CommentUser'	+ @spr + CommentUser 
				+ @spr + N'@CommentDev'		+ @spr + CommentDev 
				+ @spr + N'@Datentyp'		+ @spr + Datentyp 
				+ @spr + N'@Source'			+ @spr + Source 
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula 
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM planning.thLists
			WHERE ListID IN	(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			--Create the List Values
			INSERT INTO @ExportTable 
			SELECT 8001, 1, N'dbo.sx_pf_POST_ListValue' 
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ListValueKey'		+ @spr + N'0' 
				+ @spr + N'@ListID'				+ @spr + ListID 
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt	AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale		AS NVARCHAR(255)) 
				+ @spr + N'@ValueText'			+ @spr + ValueText 
				+ @spr + N'@ValueComment'		+ @spr + ValueComment 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.thListValues 
			WHERE ListID IN	(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT ValueFormatID 
				FROM planning.tdValueSeries 
				WHERE ProductKey = @ProductKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM planning.tgProducts 
				WHERE 
						ProductKey = @ProductKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable
			SELECT 8001, 1, N'dbo.sx_pf_POST_Format' 
					+ @spr + N'@Username'				+ @spr + N'#Username' 
					+ @spr + N'@FormatID'				+ @spr + FormatID
					+ @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					+ @spr + N'@FontColor'				+ @spr + FontColor
					+ @spr + N'@ValueFormat'			+ @spr + ValueFormat
					+ @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255))
					+ @spr + N'@Italic'					+ @spr + CAST(Italic			AS NVARCHAR(255))
					+ @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal
					+ @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical
					+ @spr + N'@TextUnderlined'			+ @spr + CAST(TextUnderlined	AS NVARCHAR(255))
					+ @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255))
					+ @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client
		
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT Command 
		FROM @ExportTable 
		ORDER BY StepNumber, RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_Product] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_Product] TO pf_PlanningFactoryService;
GO

/* 
EXPORT operation of a single productline, including
- its properties in gtables
- its global attributes
- its products
- it values
- its dependend lists, which have to be merged in the existing lists during import
- in two styles, one for import as sql script, one for Import over the GUI Import

Dependencies:
	 - Functions:
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pMaskSQL
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- dbo.sx_pf_pGET_ProductlineReadRight
		- sx_pf_pPOST_API_LogEntry

01/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

Test Call:
Results must be stored in table, as column width is to big for SSMS or SQLCMD

0a. Activate Stepnumber for output in Line 753 around and execute prodecedure
0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

1. Execute this to store the results in the table
	IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[tmpout_Export]') AND type in (N'U'))
	DROP TABLE [tmpout_Export]
	GO
	CREATE TABLE tmpout_Export (
		StepNumber INT,
		Command NVARCHAR(MAX)
		)
	INSERT INTO tmpout_Export
		EXEC sx_pf_EXPORT_Productline 'SQL','ZT','U',1

2. Copy this table Content in a file over a .bat file with this command
	bcp.exe "SELECT Command FROM PlanningFactoryCars.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_EXPORT_Productline]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_EXPORT_Productline];
GO

CREATE PROCEDURE [sx_pf_EXPORT_Productline]
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@OutputForSQLServer INT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	AS NVARCHAR(255);
	DECLARE @ProcedureName		AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	AS NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username, N'NULL')									+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N'''';
	DECLARE @EffectedRows		AS INTEGER = 0;						
	DECLARE @ResultCode			AS INTEGER = 501;					
	DECLARE @TimestampCall		AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment			AS NVARCHAR (2000) = N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username = N'';
	IF @FactoryID			IS NULL SET @FactoryID = N'';
	IF @ProductLineID		IS NULL SET @ProductLineID = N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 0;

	BEGIN TRY

		DECLARE @ProductLineKey BIGINT = 0;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID(@ProductLineID);

		IF @FactoryID = N'' OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductLineKey = ProductlineKey FROM planning.tdProductLines WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

		IF @ProductLineKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;	
	
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductlineKey = @ProductLineKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource2		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource3		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource4		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource5		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource6		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource7		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource8		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource9		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (	 
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX));
		
		-- STEP 3.2 - OUTPUT Type for SQL Server ##################################################################################################

		IF @OutputForSQLServer = 1
		BEGIN
			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
				,(0, 1, N'DECLARE @FactoryID NVARCHAR(255)			= ''' + @FactoryID + N'''')
				,(0, 1, N'DECLARE @ProductLineID  NVARCHAR(255)	= ''' + @ProductLineID + N'''')
				,(0, 1, N'--This ProductlineID will be deleted during import, if it exists. You should be sure !');

			-- DELETE Productline if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'EXEC sx_pf_DELETE_Productline @Username, @FactoryID, @ProductLineID;');
				
			-- POST Productline
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'EXEC dbo.sx_pf_POST_Productline @Username, @ProductLineID, @FactoryID,'''
				+ dbo.sx_pf_pMaskSQL(NameShort)					+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)					+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ResponsiblePerson)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ImageName)					+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(DefaultTemplate)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource1)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias1)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource2)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias2)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource3)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias3)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource4)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias4)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource5)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias5)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource6)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias6)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource7)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias7)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource8)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias8)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource9)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias9)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource10)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias10)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource11)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias11)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource12)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias12)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource13)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias13)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource14)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias14)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource15)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias15)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource16)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias16)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource17)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias17)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource18)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias18)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource19)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias19)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource20)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias20)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource21)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias21)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource22)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias22)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource23)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias23)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource24)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias24)	+ N''','''
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource25)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias25)	+ N''';'
			FROM planning.tdProductLines WHERE ProductlineKey = @ProductLineKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,''' 
				+ TimeType									+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL([Status])			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(Template)			+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(TemplateVersion)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute1)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute2)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute3)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute4)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute5)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute6)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute7)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute8)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute9)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute10) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute11) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute12) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute13) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute14) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute15) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute16) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute17) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute18) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute19) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute20) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute21) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute22) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute23) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute24) + N''',''' 
				+ dbo.sx_pf_pMaskSQL(GlobalAttribute25) + N''';'
			FROM planning.tdProducts WHERE ProductlineKey = @ProductLineKey;
							
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000 AS Step
				, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'EXEC dbo.sx_pf_POST_ValueSerie @Username,''' + ProductID + ''', @ProductLineID, @FactoryID,''' +
				+ ValueSeriesID								+ N''','''
				+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''','''
				+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''','''
				+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''','''
				+ ValueSource								+ N''','''
				+ ValueListID								+ N''','''
				+ ValueFormatID								+ N''','''
				+ Unit										+ N''','''
				+ CAST(Scale			AS NVARCHAR (255))	+ N''','''
				+ Effect									+ N''','''
				+ EffectParameter							+ N''';'
			FROM planning.tdValueSeries
			WHERE  ProductlineKey = @ProductLineKey
			ORDER BY 
				 ProductLineID
				,ProductID
				,ValueSeriesNo;
				
			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + ',0,0,'''''+ FormatID + ''''')],' -- with DeleteFlag, AddValue,FormatID 
					FROM planning.tdTime t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, ''' + ProductID + N''', @ProductLineID, @FactoryID,1,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[('''''		+ ValueSeriesID 
							+ N''''','		+ CAST(TimeID AS NVARCHAR(255)) 
							+ N','''''		+ ValueFormula 
							+ N''''','''''	+ CAST(ValueInt AS NVARCHAR(255)) 
							+ N''''','''''	+ dbo.sx_pf_pMaskSQL(ValueText) 
							+ N''''','''''	+ dbo.sx_pf_pMaskSQL(ValueComment) + ''''')],' 
					FROM planning.tfValues t
					WHERE t.ProductKey = dP.ProductKey
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,0,''' + LEFT(vals, LEN(vals) - 1) + N''';'
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'EXEC dbo.sx_pf_POST_ProductlineProperty @Username, @ProductLineID, @FactoryID,''' 
					+ PropertyID								+ N''', '''
					+ dbo.sx_pf_pMaskSQL(PropertyName)		+ N''', '''
					+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''', '''
					+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''', '''
					+ Unit										+ N''', '''
					+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''', '''
					+ CAST(ValueInt AS NVARCHAR(255))			+ N''', '''
					+ CAST(Scale AS NVARCHAR(255))				+ N''', '''
					+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''', '''
					+ FormatID									+ N''';'
			FROM planning.tgProductLines
			WHERE ProductlineKey = @ProductLineKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,''' 
				+ PropertyID								+ N''','''
				+ dbo.sx_pf_pMaskSQL(PropertyName)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''','''
				+ Unit										+ N''','''
				+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''','''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''','''
				+ CAST(Scale AS NVARCHAR(255))				+ N''','''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''','''
				+ FormatID									+ N''';'
			FROM planning.tgProducts
			WHERE  ProductlineKey = @ProductLineKey; 
						
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,'''
				+ ListID								+ ''','''
				+ dbo.sx_pf_pMaskSQL(NameShort)		+ N''','''
				+ dbo.sx_pf_pMaskSQL(NameLong)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentUser)	+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)	+ N''',''' 
				+ Datentyp								+ N''',''' 
				+ Source								+ N''',''' 
				+ SourceFormula							+ N''',''' 
				+ FormatID								+ N''';'
			FROM planning.thLists
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,'''
				+ ListID								+ N''',' 
				+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
				+ CAST(Scale AS NVARCHAR(255))			+ N',''' 
				+ dbo.sx_pf_pMaskSQL(ValueText)		+ N''',''' 
				+ dbo.sx_pf_pMaskSQL(ValueComment)	+ N''',''' 
				+ FormatID								+ N''';'
			FROM planning.thListValues
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM planning.tdValueSeries 
				WHERE ProductlineKey = @ProductLineKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM planning.tgProducts 
				WHERE 
						ProductlineKey = @ProductLineKey
					AND PropertyID LIKE 'PDTRange_%'
			) 

			INSERT INTO @ExportTable 
			SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,'''
				+ FormatID									+ N''','''
				+ BackgroundColor							+ N''',''' 
				+ FontColor									+ N''',''' 
				+ ValueFormat								+ N''',''' 
				+ CAST(Bold				AS NVARCHAR(255))	+ N''','''
				+ CAST(Italic			AS NVARCHAR(255))	+ N''','''
				+ AlignmentHorizontal						+ N''','''
				+ AlignmentVertical							+ N''','''
				+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''','''
				+ CAST(BorderToPrevious AS NVARCHAR(255))	+ N''';'
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
			INSERT INTO @ExportTable
			VALUES (8000,1, N'GO');

		END -- of Output for SQL Server
		
	-- STEP 3.3 - OUTPUT Type for PF Client Execution ##################################################################################################
		ELSE IF @OutputForSQLServer = 0
		BEGIN
			DECLARE @spr CHAR = CHAR(31); 

			INSERT INTO @ExportTable 
			VALUES 
				 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
				,(0, 1, N'--DECLARE #Username NVARCHAR(255)		= ''SQL''')
				,(0, 1, N'--DECLARE #FactoryID NVARCHAR(255)		= ''' + @FactoryID		+ N'''')
				,(0, 1, N'--DECLARE #ProductLineID  NVARCHAR(255)	= ''' + @ProductLineID	+ N'''')
				,(0, 1, N'--This ProductlineID will be deleted during import, if it exists. You should be sure !');

		
			-- DELETE Productline if exists
			INSERT INTO @ExportTable 
			VALUES (1000,1, N'sx_pf_DELETE_Productline' + @spr + N'@Username' + @spr + N'#Username' + @spr + N'@FactoryID' + @spr + N'#FactoryID' + @spr + N'@ProductLineID' + @spr + N'#ProductLineID');
						
			-- POST Productline
			INSERT INTO @ExportTable 
			SELECT 1001,1, N'dbo.sx_pf_POST_Productline' 
						+ @spr + N'@Username'					+ @spr + N'#Username' 
						+ @spr + N'@ProductlineID'				+ @spr + N'#ProductlineID' 
						+ @spr + N'@FactoryID'					+ @spr + N'#FactoryID'
						+ @spr + N'@NameShort'					+ @spr + NameShort 
						+ @spr + N'@NameLong'					+ @spr + NameLong  
						+ @spr + N'@CommentUser'				+ @spr + CommentUser  
						+ @spr + N'@CommentDev'					+ @spr + CommentDev  
						+ @spr + N'@ResponsiblePerson'			+ @spr + ResponsiblePerson  
						+ @spr + N'@ImageName'					+ @spr + ImageName  
						+ @spr + N'@DefaultTemplate'			+ @spr + DefaultTemplate  
						+ @spr + N'@GlobalAttributeSource1'		+ @spr + GlobalAttributeSource1  
						+ @spr + N'@GlobalAttributeAlias1'		+ @spr + GlobalAttributeAlias1  
						+ @spr + N'@GlobalAttributeSource2'		+ @spr + GlobalAttributeSource2  
						+ @spr + N'@GlobalAttributeAlias2'		+ @spr + GlobalAttributeAlias2  
						+ @spr + N'@GlobalAttributeSource3'		+ @spr + GlobalAttributeSource3  
						+ @spr + N'@GlobalAttributeAlias3'		+ @spr + GlobalAttributeAlias3 
						+ @spr + N'@GlobalAttributeSource4'		+ @spr + GlobalAttributeSource4  
						+ @spr + N'@GlobalAttributeAlias4'		+ @spr + GlobalAttributeAlias4 
						+ @spr + N'@GlobalAttributeSource5'		+ @spr + GlobalAttributeSource5  
						+ @spr + N'@GlobalAttributeAlias5'		+ @spr + GlobalAttributeAlias5 
						+ @spr + N'@GlobalAttributeSource6'		+ @spr + GlobalAttributeSource6  
						+ @spr + N'@GlobalAttributeAlias6'		+ @spr + GlobalAttributeAlias6 
						+ @spr + N'@GlobalAttributeSource7'		+ @spr + GlobalAttributeSource7  
						+ @spr + N'@GlobalAttributeAlias7'		+ @spr + GlobalAttributeAlias7 
						+ @spr + N'@GlobalAttributeSource8'		+ @spr + GlobalAttributeSource8  
						+ @spr + N'@GlobalAttributeAlias8'		+ @spr + GlobalAttributeAlias8 
						+ @spr + N'@GlobalAttributeSource9'		+ @spr + GlobalAttributeSource9  
						+ @spr + N'@GlobalAttributeAlias9'		+ @spr + GlobalAttributeAlias9 
						+ @spr + N'@GlobalAttributeSource10'	+ @spr + GlobalAttributeSource10  
						+ @spr + N'@GlobalAttributeAlias10'		+ @spr + GlobalAttributeAlias10 
						+ @spr + N'@GlobalAttributeSource11'	+ @spr + GlobalAttributeSource11  
						+ @spr + N'@GlobalAttributeAlias11'		+ @spr + GlobalAttributeAlias11  
						+ @spr + N'@GlobalAttributeSource12'	+ @spr + GlobalAttributeSource12  
						+ @spr + N'@GlobalAttributeAlias12'		+ @spr + GlobalAttributeAlias12  
						+ @spr + N'@GlobalAttributeSource13'	+ @spr + GlobalAttributeSource13  
						+ @spr + N'@GlobalAttributeAlias13'		+ @spr + GlobalAttributeAlias13 
						+ @spr + N'@GlobalAttributeSource14'	+ @spr + GlobalAttributeSource14  
						+ @spr + N'@GlobalAttributeAlias14'		+ @spr + GlobalAttributeAlias14 
						+ @spr + N'@GlobalAttributeSource15'	+ @spr + GlobalAttributeSource15  
						+ @spr + N'@GlobalAttributeAlias15'		+ @spr + GlobalAttributeAlias15 
						+ @spr + N'@GlobalAttributeSource16'	+ @spr + GlobalAttributeSource16  
						+ @spr + N'@GlobalAttributeAlias16'		+ @spr + GlobalAttributeAlias16 
						+ @spr + N'@GlobalAttributeSource17'	+ @spr + GlobalAttributeSource17  
						+ @spr + N'@GlobalAttributeAlias17'		+ @spr + GlobalAttributeAlias17 
						+ @spr + N'@GlobalAttributeSource18'	+ @spr + GlobalAttributeSource18  
						+ @spr + N'@GlobalAttributeAlias18'		+ @spr + GlobalAttributeAlias18 
						+ @spr + N'@GlobalAttributeSource19'	+ @spr + GlobalAttributeSource19  
						+ @spr + N'@GlobalAttributeAlias19'		+ @spr + GlobalAttributeAlias19 
						+ @spr + N'@GlobalAttributeSource20'	+ @spr + GlobalAttributeSource20  
						+ @spr + N'@GlobalAttributeAlias20'		+ @spr + GlobalAttributeAlias20 
						+ @spr + N'@GlobalAttributeSource21'	+ @spr + GlobalAttributeSource21  
						+ @spr + N'@GlobalAttributeAlias21'		+ @spr + GlobalAttributeAlias21  
						+ @spr + N'@GlobalAttributeSource22'	+ @spr + GlobalAttributeSource22  
						+ @spr + N'@GlobalAttributeAlias22'		+ @spr + GlobalAttributeAlias22  
						+ @spr + N'@GlobalAttributeSource23'	+ @spr + GlobalAttributeSource23  
						+ @spr + N'@GlobalAttributeAlias23'		+ @spr + GlobalAttributeAlias23 
						+ @spr + N'@GlobalAttributeSource24'	+ @spr + GlobalAttributeSource24  
						+ @spr + N'@GlobalAttributeAlias24'		+ @spr + GlobalAttributeAlias24 
						+ @spr + N'@GlobalAttributeSource25'	+ @spr + GlobalAttributeSource25  
						+ @spr + N'@GlobalAttributeAlias25'		+ @spr + GlobalAttributeAlias25 + N''
			FROM planning.tdProductLines WHERE ProductlineKey = @ProductLineKey;
					
			-- POST Products
			INSERT INTO @ExportTable
			SELECT 2000,1, N'dbo.sx_pf_POST_ProductEmpty' 
					+ @spr + N'@Username'			+ @spr + N'#Username' 
					+ @spr + N'@ProductID'			+ @spr + N'' + ProductID + N'' 
					+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
					+ @spr + N'@TimeType'			+ @spr + TimeType
					+ @spr + N'@NameShort'			+ @spr + NameShort
					+ @spr + N'@NameLong'			+ @spr + NameLong
					+ @spr + N'@CommentUser'		+ @spr + CommentUser
					+ @spr + N'@CommentDev'			+ @spr + CommentDev
					+ @spr + N'@ResponsiblePerson'	+ @spr + ResponsiblePerson
					+ @spr + N'@ImageName'			+ @spr + ImageName
					+ @spr + N'@Status'				+ @spr + [Status] 
					+ @spr + N'@Template'			+ @spr + Template 
					+ @spr + N'@TemplateVersion'	+ @spr + TemplateVersion
					+ @spr + N'@GA1'				+ @spr + GlobalAttribute1
					+ @spr + N'@GA2'				+ @spr + GlobalAttribute2
					+ @spr + N'@GA3'				+ @spr + GlobalAttribute3
					+ @spr + N'@GA4'				+ @spr + GlobalAttribute4
					+ @spr + N'@GA5'				+ @spr + GlobalAttribute5
					+ @spr + N'@GA6'				+ @spr + GlobalAttribute6
					+ @spr + N'@GA7'				+ @spr + GlobalAttribute7
					+ @spr + N'@GA8'				+ @spr + GlobalAttribute8
					+ @spr + N'@GA9'				+ @spr + GlobalAttribute9
					+ @spr + N'@GA10'				+ @spr + GlobalAttribute10 
					+ @spr + N'@GA11'				+ @spr + GlobalAttribute11
					+ @spr + N'@GA12'				+ @spr + GlobalAttribute12
					+ @spr + N'@GA13'				+ @spr + GlobalAttribute13
					+ @spr + N'@GA14'				+ @spr + GlobalAttribute14
					+ @spr + N'@GA15'				+ @spr + GlobalAttribute15
					+ @spr + N'@GA16'				+ @spr + GlobalAttribute16
					+ @spr + N'@GA17'				+ @spr + GlobalAttribute17
					+ @spr + N'@GA18'				+ @spr + GlobalAttribute18
					+ @spr + N'@GA19'				+ @spr + GlobalAttribute19
					+ @spr + N'@GA20'				+ @spr + GlobalAttribute20
					+ @spr + N'@GA21'				+ @spr + GlobalAttribute21
					+ @spr + N'@GA22'				+ @spr + GlobalAttribute22
					+ @spr + N'@GA23'				+ @spr + GlobalAttribute23
					+ @spr + N'@GA24'				+ @spr + GlobalAttribute24
					+ @spr + N'@GA25'				+ @spr + GlobalAttribute25 + N''
			FROM planning.tdProducts WHERE ProductlineKey = @ProductLineKey; 
									
			-- POST Statements -- statements are not exported

			-- POST all ValueSeries as NewValueSeries
			INSERT INTO @ExportTable 
			SELECT 3000
				, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
				, 'dbo.sx_pf_POST_ValueSerie'
				+ @spr + N'@Username'				+ @spr + N'#Username' 
				+ @spr + N'@ProductID'				+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'			+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'				+ @spr + N'#FactoryID' 
				+ @spr + N'@ValueSeriesID'			+ @spr + ValueSeriesID 
				+ @spr + N'@RequestedValueSeriesNo' + @spr + CAST(ValueSeriesNo		AS NVARCHAR (255))
				+ @spr + N'@NameShort'				+ @spr + NameShort
				+ @spr + N'@NameLong'				+ @spr + NameLong
				+ @spr + N'@CommentUser'			+ @spr + CommentUser
				+ @spr + N'@CommentDev'				+ @spr + CommentDev
				+ @spr + N'@ImageName'				+ @spr + ImageName
				+ @spr + N'@IsNumeric'				+ @spr + CAST([IsNumeric]		AS NVARCHAR (255))
				+ @spr + N'@VisibilityLevel'		+ @spr + CAST(VisibilityLevel	AS NVARCHAR (255))
				+ @spr + N'@ValueSource'			+ @spr + ValueSource
				+ @spr + N'@ValueListID'			+ @spr + ValueListID
				+ @spr + N'@ValueFormatID'			+ @spr + ValueFormatID
				+ @spr + N'@Unit'					+ @spr + Unit
				+ @spr + N'@Scale'					+ @spr + CAST(Scale				AS NVARCHAR (255))
				+ @spr + N'@Effect'					+ @spr + Effect
				+ @spr + N'@EffectParameter'		+ @spr + EffectParameter + N''
			FROM planning.tdValueSeries
			WHERE  ProductlineKey = @ProductLineKey
			ORDER BY 
				 ProductLineID
				,ProductID
				,ValueSeriesNo;

			-- POST dTime 
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10))+',0,0,'+ FormatID +')],' -- with DeleteFlag, AddValue,FormatID 
					FROM planning.tdTime t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 4000,1, N'dbo.sx_pf_POST_Timeline'
					+ @spr + N'@Username'							+ @spr + N'#Username' 
					+ @spr + N'@ProductID'							+ @spr + N'' + ProductID + N'' 
					+ @spr + N'@ProductLineID'						+ @spr + N'#ProductLineID' 
					+ @spr + N'@FactoryID'							+ @spr + N'#FactoryID' 
					+ @spr + N'@IsIncrementalValuesFlag'			+ @spr + N'1' + N''
					+ @spr + N'@TimeIDsInBracketsCommaSeparated'	+ @spr + N'' + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;		
			
			-- POST ProductDataTableValues
			;WITH tms AS (
				SELECT dP.ProductID
					,(SELECT N'[('''	+ ValueSeriesID 
							+ N''','	+ CAST(TimeID	AS NVARCHAR(255)) 
							+ N','''	+ dbo.sx_pf_pRemoveLinebreaks(ValueFormula) 
							+ N''','''	+ CAST(ValueInt AS NVARCHAR(255)) 
							+ N''','''	+ ValueText 
							+ N''','''	+ ValueComment + ''')],' 
					FROM planning.tfValues t
					WHERE t.ProductKey = dP.ProductKey
					ORDER BY t.ValueSeriesID, t.TimeID
					FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS vals
				FROM planning.tdProducts dP 
				WHERE dP.ProductLineKey = @ProductLineKey 
			)
			INSERT INTO @ExportTable
			SELECT 5000,1, N'dbo.sx_pf_POST_ProductDataTableValues'
				+ @spr + N'@Username'						+ @spr + N'#Username' 
				+ @spr + N'@ProductID'						+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'					+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'						+ @spr + N'#FactoryID' 
				+ @spr + N'@IsIncrementalValuesFlag'		+ @spr + N'0' + N''
				+ @spr + N'@ValuesInBracketsCommaSeparated' + @spr + N'' + LEFT(vals, LEN(vals) - 1) + N''
			FROM tms
			WHERE NOT vals IS NULL;

			-- POST Productline Properties
			INSERT INTO @ExportTable 
			SELECT 6000,1, N'dbo.sx_pf_POST_ProductlineProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID'  
				+ @spr + N'@PropertyID'			+ @spr + PropertyID 
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255))
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255))
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgProductLines
			WHERE ProductlineKey = @ProductLineKey; 

			-- POST Product Properties
			INSERT INTO @ExportTable 
			SELECT 2001,1, N'dbo.sx_pf_POST_ProductProperty'
				+ @spr + N'@Username'			+ @spr + N'#Username' 
				+ @spr + N'@ProductID'			+ @spr + N'' + ProductID + N'' 
				+ @spr + N'@ProductLineID'		+ @spr + N'#ProductLineID' 
				+ @spr + N'@FactoryID'			+ @spr + N'#FactoryID' 
				+ @spr + N'@PropertyID'			+ @spr + PropertyID
				+ @spr + N'@PropertyName'		+ @spr + PropertyName
				+ @spr + N'@CommentUser'		+ @spr + CommentUser
				+ @spr + N'@CommentDev'			+ @spr + CommentDev
				+ @spr + N'@Unit'				+ @spr + Unit
				+ @spr + N'@ValueText'			+ @spr + ValueText
				+ @spr + N'@ValueInt'			+ @spr + CAST(ValueInt				AS NVARCHAR(255)) 
				+ @spr + N'@Scale'				+ @spr + CAST(Scale					AS NVARCHAR(255))
				+ @spr + N'@IsROSystemProperty' + @spr + CAST(IsROSystemProperty	AS NVARCHAR(255)) 
				+ @spr + N'@FormatID'			+ @spr + FormatID + N''
			FROM planning.tgProducts
			WHERE  ProductlineKey = @ProductLineKey;
		
			-- Create the lists
			INSERT INTO @ExportTable 
			SELECT 7001,1, N'dbo.sx_pf_POST_List'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListID'			+ @spr + ListID 
				+ @spr + N'@NameShort'		+ @spr + NameShort
				+ @spr + N'@NameLong'		+ @spr + NameLong
				+ @spr + N'@CommentDev'		+ @spr + CommentDev
				+ @spr + N'@CommentUser'	+ @spr + CommentUser
				+ @spr + N'@Datentyp'		+ @spr + Datentyp
				+ @spr + N'@Source'			+ @spr + [Source]
				+ @spr + N'@SourceFormula'	+ @spr + SourceFormula
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM planning.thLists
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
			-- Create List Values
			INSERT INTO @ExportTable 
			SELECT 7002,1, N'dbo.sx_pf_POST_ListValue'
				+ @spr + N'@Username'		+ @spr + N'#Username'
				+ @spr + N'@ListValuekey'	+ @spr + N'' + N'0'
				+ @spr + N'@ListID'			+ @spr + ListID
				+ @spr + N'@ValueInt'		+ @spr + CAST(ValueInt	AS NVARCHAR(255))
				+ @spr + N'@Scale'			+ @spr + CAST(Scale		AS NVARCHAR(255))
				+ @spr + N'@ValueText'		+ @spr + ValueText
				+ @spr + N'@ValueComment'	+ @spr + ValueComment
				+ @spr + N'@FormatID'		+ @spr + FormatID + N''
			FROM planning.thListValues
			WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
			-- Create Formats
			;WITH DependendFormats AS (
				-- used in depended lists from global attributes
				SELECT FormatID
				FROM planning.thListValues 
				WHERE ListID IN
					(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
				UNION ALL
				-- used to format ValueSeries 
				SELECT DISTINCT ValueFormatID 
				FROM planning.tdValueSeries 
				WHERE ProductlineKey = @ProductLineKey
				UNION ALL
				-- used to format Ranges
				SELECT DISTINCT FormatID
				FROM planning.tgProducts 
				WHERE 
						ProductlineKey = @ProductLineKey
					AND PropertyID LIKE 'PDTRange_%'
			) 
			INSERT INTO @ExportTable 
			SELECT 7003,1, N'dbo.sx_pf_POST_Format'
					 + @spr + N'@Username'				+ @spr + N'#Username'
					 + @spr + N'@FormatID'				+ @spr + FormatID
					 + @spr + N'@BackgroundColor'		+ @spr + BackgroundColor
					 + @spr + N'@FontColor'				+ @spr + FontColor
					 + @spr + N'@ValueFormat'			+ @spr + ValueFormat
					 + @spr + N'@Bold'					+ @spr + CAST(Bold				AS NVARCHAR(255)) 
					 + @spr + N'@Italic'				+ @spr + CAST(Italic			AS NVARCHAR(255))
					 + @spr + N'@AlignmentHorizontal'	+ @spr + AlignmentHorizontal 
					 + @spr + N'@AlignmentVertical'		+ @spr + AlignmentVertical 
					 + @spr + N'@TextUnderlined'		+ @spr + CAST(TextUnderlined	AS NVARCHAR(255)) 
					 + @spr + N'@BorderToNext'			+ @spr + CAST(BorderToNext		AS NVARCHAR(255)) 
					 + @spr + N'@BorderToPrevious'		+ @spr + CAST(BorderToPrevious	AS NVARCHAR(255)) + N''
			FROM planning.thFormats
			WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);

		END; -- Output for PF Client	
	
		-- STEP 3.4 - Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY StepNumber,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

  
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_ProductLine] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_EXPORT_ProductLine] TO pf_PlanningFactoryService;
GO



	
/*
Procedure to fill the Table control.tSystemFailureReport
Procedure will be build from many subsnipptes

Testcall 
EXEC control.spSystemFailureReport

SELECT * FROM control.tSystemFailureReport

*/


IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.spSystemFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE control.spSystemFailureReport
GO


CREATE PROCEDURE  control.spSystemFailureReport (@FactoryID NVARCHAR(255) = '', @ProductlineID NVARCHAR(255) = '')

AS

DECLARE @Timestamp 	Datetime 	= GETDATE()
DECLARE @TestName	NVARCHAR (255) 	= ''
DECLARE @TestSource	NVARCHAR (255) 	= ''


-- self cleaning older than 14 Days
DELETE FROM control.tSystemFailureReport WHERE Timestamp < DATEADD(day,-14,GETDATE());




SET @TestSource = 'Global_Lists'							-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for necessary System List which must exist.'			-- an friendly description of the test
-- this are the system list, a user could modify (ok) and delete (not ok)
-- there are more system list, but they are created internal and its not possible to modify

	BEGIN TRY

		WITH tmp AS
				(
				SELECT syslsts.ListID
				FROM (
					VALUES 
					  
					  (N'sxResponsiblePersons')
					, (N'sxStatus')
					, (N'sxUnits')
					, (N'sxValueEffectParameters')

				) syslsts (ListID) 
					LEFT JOIN planning.thLists hL 
						ON syslsts.ListID = hL.ListID
				WHERE hL.ListID IS NULL
				)

	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID'	AS FactoryID
				,'AllPLID'	AS ProductlineID
				,'AllPID'	AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE'	AS ResultType

				,'List ' 
					+ tmp.ListID 
					+ ' must exist, but is missing.'
							AS ResultText

				,1			AS ResultNumeric

			FROM tmp
				
					

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Global_Lists'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ListValues which use not exiting Formats for conditional formating.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID' AS FactoryID
				,'AllPLID' AS ProductlineID
				,'AllPID' AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'List ' 
					+ hLV.ListID 
					+ ' uses for Value '
					+	CASE
							WHEN hL.Datentyp = 'String' THEN hLV.ValueText
							ELSE CAST(hLV.ValueInt AS NCHAR(10))
						END
					+ ' the Format ' 
					+ hLV.FormatID + ' which does not exists in Format definitions.'
						
				,1 AS ResultNumeric

			FROM planning.thListValues hLV
				LEFT JOIN planning.thLists hL
					ON hLV.ListID = hL.ListID

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = hLV.FormatID
								)
					AND hLV.FormatID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Format definitions.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductlineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ dVS.ValueSeriesID
					+ ' uses Format ' 
					+ dVS.ValueFormatID + ' which does not exists in Format definitions.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = dVS.ValueFormatID
								)

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found. '
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Lists as ValueSource.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductlineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ ValueSeriesID
					+ ' uses ValueList ' 
					+ ValueListID + ' which does not exists in List Management.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT ListID FROM planning.thLists WHERE ListID = dVS.ValueListID
								)
					AND dVS.ValueListID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Productline'												-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for use of not existing List in Globalattributes.'	-- an friendly description of the test

	BEGIN TRY
				WITH GA AS (

					SELECT N'1' AS Num, FactoryID, ProductlineID, GlobalAttributeSource1 AS GlobalAttributeSource
					FROM planning.tdProductLines WHERE GlobalAttributeSource1 <> N'' UNION ALL
					SELECT N'2', FactoryID, ProductlineID, GlobalAttributeSource2
					FROM planning.tdProductLines WHERE GlobalAttributeSource2 <> N'' UNION ALL
					SELECT N'3', FactoryID, ProductlineID, GlobalAttributeSource3
					FROM planning.tdProductLines WHERE GlobalAttributeSource3 <> N'' UNION ALL
					SELECT N'4', FactoryID, ProductlineID, GlobalAttributeSource4
					FROM planning.tdProductLines WHERE GlobalAttributeSource4 <> N'' UNION ALL
					SELECT N'5', FactoryID, ProductlineID, GlobalAttributeSource5
					FROM planning.tdProductLines WHERE GlobalAttributeSource5 <> N'' UNION ALL
					SELECT N'6', FactoryID, ProductlineID, GlobalAttributeSource6
					FROM planning.tdProductLines WHERE GlobalAttributeSource6 <> N'' UNION ALL
					SELECT N'7', FactoryID, ProductlineID, GlobalAttributeSource7
					FROM planning.tdProductLines WHERE GlobalAttributeSource7 <> N'' UNION ALL
					SELECT N'8', FactoryID, ProductlineID, GlobalAttributeSource8
					FROM planning.tdProductLines WHERE GlobalAttributeSource8 <> N'' UNION ALL
					SELECT N'9', FactoryID, ProductlineID, GlobalAttributeSource9
					FROM planning.tdProductLines WHERE GlobalAttributeSource9 <> N'' UNION ALL
					SELECT N'10', FactoryID, ProductlineID, GlobalAttributeSource10
					FROM planning.tdProductLines WHERE GlobalAttributeSource10 <> N'' UNION ALL
					SELECT N'11', FactoryID, ProductlineID, GlobalAttributeSource11
					FROM planning.tdProductLines WHERE GlobalAttributeSource11 <> N'' UNION ALL
					SELECT N'12', FactoryID, ProductlineID, GlobalAttributeSource12
					FROM planning.tdProductLines WHERE GlobalAttributeSource12 <> N'' UNION ALL
					SELECT N'13', FactoryID, ProductlineID, GlobalAttributeSource13
					FROM planning.tdProductLines WHERE GlobalAttributeSource13 <> N'' UNION ALL
					SELECT N'14', FactoryID, ProductlineID, GlobalAttributeSource14
					FROM planning.tdProductLines WHERE GlobalAttributeSource14 <> N'' UNION ALL
					SELECT N'15', FactoryID, ProductlineID, GlobalAttributeSource15
					FROM planning.tdProductLines WHERE GlobalAttributeSource15 <> N'' UNION ALL
					SELECT N'16', FactoryID, ProductlineID, GlobalAttributeSource16
					FROM planning.tdProductLines WHERE GlobalAttributeSource16 <> N'' UNION ALL
					SELECT N'17', FactoryID, ProductlineID, GlobalAttributeSource17
					FROM planning.tdProductLines WHERE GlobalAttributeSource17 <> N'' UNION ALL
					SELECT N'18', FactoryID, ProductlineID, GlobalAttributeSource18
					FROM planning.tdProductLines WHERE GlobalAttributeSource18 <> N'' UNION ALL
					SELECT N'19', FactoryID, ProductlineID, GlobalAttributeSource19
					FROM planning.tdProductLines WHERE GlobalAttributeSource19 <> N'' UNION ALL
					SELECT N'20', FactoryID, ProductlineID, GlobalAttributeSource20
					FROM planning.tdProductLines WHERE GlobalAttributeSource20 <> N'' UNION ALL
					SELECT N'21', FactoryID, ProductlineID, GlobalAttributeSource21
					FROM planning.tdProductLines WHERE GlobalAttributeSource21 <> N'' UNION ALL
					SELECT N'22', FactoryID, ProductlineID, GlobalAttributeSource22
					FROM planning.tdProductLines WHERE GlobalAttributeSource22 <> N'' UNION ALL
					SELECT N'23', FactoryID, ProductlineID, GlobalAttributeSource23
					FROM planning.tdProductLines WHERE GlobalAttributeSource23 <> N'' UNION ALL
					SELECT N'24', FactoryID, ProductlineID, GlobalAttributeSource24
					FROM planning.tdProductLines WHERE GlobalAttributeSource24 <> N'' UNION ALL
					SELECT N'25', FactoryID, ProductlineID, GlobalAttributeSource25
					FROM planning.tdProductLines WHERE GlobalAttributeSource25 <> N''

					)
	
		INSERT INTO control.tSystemFailureReport
			
			SELECT 
				 @Timestamp,
				 GA.FactoryID
				,GA.ProductlineID
				,'AllPID' AS ProductID
			    ,@TestSource
				,@TestName
				,'FAILURE' AS ResultType -- or WARNING

				,'Not existing List ' + GA.GlobalattributeSource + ' is in use for Globalattribute ' + GA.Num AS ResultText

				,1 AS ResultNumeric -- (0 for Warning, positiv for Failure)

			FROM GA 
				LEFT JOIN planning.thLists hL
					ON GA.GlobalattributeSource = hL.ListID
				WHERE hL.ListID IS NULL
			

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS WITHOUT CHANGE ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH




GO
/*
GET Operation for all ListValues a product needs for data entry in ProductDataTable
- contains the GUI defined lists of the products (defined by user c* or solution su*)
- contains the dynamic system lists (sx* - which are editable by the user)
- adds interal system lists (sx* - which are not visible on GUI)

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call - Must return Table and Code 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_AllProductListValues 
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';

PRINT @RC

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_AllProductListValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_AllProductListValues;
GO

CREATE PROCEDURE sx_pf_GET_AllProductListValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_AllProductListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
			
		-- Helper Table with all list values
		;WITH list AS (
			SELECT 
				hL.ListID
				,hL.Datentyp 
				,CAST(ISNULL(hLV.ValueInt	, 0) AS INT) AS ValueInt -- must return int !
                ,CAST(ISNULL(hLV.Scale		, 0) AS INT) AS Scale -- must return int !
				,ISNULL(hLV.ValueText		, N'')		 AS ValueText
				,ISNULL(hLV.ValueComment	, N'')		 AS ValueComment
				,ISNULL(hLV.FormatID		, N'')		 AS FormatID
				,ISNULL(hF.BackgroundColor	, N'')		 AS BackgroundColor
				,ISNULL(hF.FontColor		, N'')		 AS FontColor
				,ISNULL(hF.ValueFormat		, N'')		 AS ValueFormat
			FROM planning.thLists hL 
				LEFT JOIN planning.thListValues hLV	ON hL.ListID = hLV.ListID
				-- for the value
				LEFT JOIN planning.thFormats hF	ON hLV.FormatID = hF.FormatID
				-- filter the lists, which where till 4.0.57 GUI defined and are now internal system lists
			WHERE hL.ListID NOT IN (
				'sxIsNumeric'
				,'sxVisibilityLevel'
				,'sxValueLists'
				,'sxValueSources'
				,'sxValueFormats'
				,'sxScales'
				,'sxValueEffects')
		)
		-- STEP 3.1 Dynamic System Lists
		SELECT *
		FROM list
		WHERE ListID IN ('sxUnits','sxValueEffectParameters')	
		
		UNION ALL
		-- STEP 3.2 Product specific ListSet based on defined ValueLists
		SELECT *
		FROM list
		WHERE ListID IN (
			SELECT DISTINCT ValueListID
			FROM planning.tdValueSeries
			WHERE ProductKey = @ProductKey)

		-- add internal system lists ----------------------------------------------------------
		UNION ALL 
		--sxIsNumeric - static  TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES 
				 ('sxIsNumeric','String',0,1,'0','','','','','')
				,('sxIsNumeric','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL	
		--sxVisibilityLevel - static TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES	
				 ('sxVisibilityLevel','String',0,1,'0','','','','','')
				,('sxVisibilityLevel','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		UNION ALL			
		--sxValueLists - dynamic List of GUI Lists
		SELECT 
			'sxValueLists'
			,'String'
			,0		AS ValueInt 
            ,1		AS Scale 
			,ListID AS ValueText
			,''		AS ValueComment
			,''		AS FormatID
			,''		AS BackgroundColor
			,''		AS FontColor
			,''		AS ValueFormat
		FROM planning.thLists
			-- Filter the GUI Lists, which are not useful as ValueList
		WHERE ListID NOT IN ('sxValueEffectParameter')

		UNION ALL
		--sxValueSources - static (english strings for all languages
		SELECT * 
		FROM (
			VALUES	
				 ('sxValueSources','String',0,1,'Input'		 ,'','','','','')
				,('sxValueSources','String',0,1,'List-Strict','','','','','')
				,('sxValueSources','String',0,1,'XLS'		 ,'','','','','')
				,('sxValueSources','String',0,1,'XLS-Strict' ,'','','','','')
				,('sxValueSources','String',0,1,'Link'		 ,'','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueFormats - dynamic, list of defined formats
		SELECT 
			 'sxValueFormats'
			,'String'
			,0					AS ValueInt 
            ,1					AS Scale 
			,FormatID			AS ValueText
			,''					AS ValueComment
			,FormatID			AS FormatID
			,BackgroundColor	AS BackgroundColor
			,FontColor			AS FontColor
			,ValueFormat		AS ValueFormat
		FROM planning.thFormats 
		
		UNION ALL
		--sxScales - static - this List is also static defined in GET_ListValues
		SELECT * 
		FROM (
			VALUES	
				 ('sxScales','Integer',1	,1,'','','','','','')
				,('sxScales','Integer',10	,1,'','','','','','')
				,('sxScales','Integer',100	,1,'','','','','','')
				,('sxScales','Integer',1000	,1,'','','','','','')
				,('sxScales','Integer',10000,1,'','','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueEffects -dynamic, list of defined ValueEffects in gValueEffects
		SELECT 
			 'sxValueEffects'
			,'String'
			,0			AS ValueInt 
            ,1			AS Scale 
			,EffectID	AS ValueText
			,''			AS ValueComment
			,''			AS FormatID
			,''			AS BackgroundColor
			,''			AS FontColor
			,''			AS ValueFormat
		FROM planning.tgValueEffects

		ORDER BY ListID, ValueText, ValueInt;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_AllProductListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_AllProductListValues;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_AllProductListValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_AllProductListValues TO pf_PlanningFactoryService;
GO

/*
GET operation for User status check on cluster
If User has an registered Username and his status is 'Active' then Cluster Access Rights 
are granted. (which still can mean he sees nothing than an empty cluster)


04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call

-- Should Return 200
DECLARE @RC AS NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterAccessRight 'SQL'
PRINT @RC

-- Should Return 403
DECLARE @RC AS NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterAccessRight 'SQL4'
PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterAccessRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ClusterAccessRight;
GO

CREATE PROCEDURE sx_pf_GET_ClusterAccessRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution
	DECLARE @LogMessage			NVARCHAR(MAX)	= N'';

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterAccessRight;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			SET @LogMessage = 'Transaction user don`t exists, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END;
		
		-- STEP 3 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM planning.trUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
		BEGIN
			SET @LogMessage = 'User is not active, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterAccessRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterAccessRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for Cluster Properties

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

--should return 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterProperties 'SQL'
PRINT @RC

--should return 403
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterProperties 'SQL4'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ClusterProperties
GO

CREATE PROCEDURE sx_pf_GET_ClusterProperties
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	 NVARCHAR(255)		= N'';
	DECLARE @Right				 NVARCHAR(255)		= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)		= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)		= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME			= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000)		= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperties;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = 'Write';
			END 
			ELSE 
			BEGIN 
				SET @Right = 'Read';
				SET @ResultCode = 200;
			END;
		END;
	
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;	
		
		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgCluster
		ORDER BY COALESCE(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperties;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a cluster Property

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterProperty 
	@Username	= 'SQL',
	@PropertyID = 'LS01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ClusterProperty;
GO

CREATE PROCEDURE sx_pf_GET_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @TransactUsername = 'FactoryService';
		END;

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM planning.tgCluster WHERE PropertyID = @PropertyID;

		IF @PropertyKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
		IF @TransactUsername = 'FactoryService' SET @ResultCode = 200


		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgCluster
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
Operation to check Cluster
Running tests:
    Lists, which are never used
    ListMembers, which are never used
    Lists which are used but dont exists
    List members which are Used but dont exist
    Formats which are used but dont exist
    Formats which are defined but not used
    count ValueSeries per Product
    count ValuesPer Product
User must be active to run tests

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return values according to HTTP standard

--Test call					   
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterValidation 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterValidation') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ClusterValidation;
GO

CREATE PROCEDURE sx_pf_GET_ClusterValidation
	@Username NVARCHAR(255) 
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(255)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterValidation;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
			  
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM planning.trUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User is not active', 16, 10);
		
		-- STEP 3 - Run tests
		DECLARE @Tests AS TABLE (
			N INT IDENTITY(1,1)		NOT NULL, 
			[Name] NVARCHAR(200)	NOT NULL, 
			[Status] NVARCHAR(10)	NOT NULL,
			Result NVARCHAR(MAX)	NOT NULL,
			RowsCount BIGINT		NOT NULL);

-- Lists, which are never used			
		;WITH lst AS (
			SELECT l.ListID
				, l.ListKey
				, l.NameShort AS ListName
			FROM planning.thLists l 
				LEFT JOIN (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs ON l.ListID = vs.ValueListID
			WHERE vs.ValueSeriesKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Lists', N'Info', N'List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + ListName + N'''', 1
		FROM lst;

		SET @EffectedRows += @@ROWCOUNT;

-- Lists which are used but dont exists	
		;WITH lst AS (
			SELECT vs.ValueListID	AS ListID
				, vs.NameShort AS VSName
				, vs.ValueSeriesKey AS VSKey
				, vs.ValueSeriesID AS VSID
			FROM (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs
				LEFT JOIN planning.thLists  l ON l.ListID = vs.ValueListID
			WHERE l.ListID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists Lists', N'Error', N'ListID ' + ListID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), VSKey) + N', ' + VSID + N' ''' + VSName + N''')', 1
		FROM lst;
		
		SET @EffectedRows += @@ROWCOUNT;			  

-- ListMembers, which are never used		
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), lv.ListValueKey) + N' ''' 
				+ CASE WHEN l.Datentyp = N'INT' THEN CONVERT(NVARCHAR(MAX), lv.ValueInt) ELSE lv.ValueText END 
				+ N''' (List #' + CONVERT(NVARCHAR(MAX), l.ListKey) + N', ' + lv.ListID + N' ''' + l.NameShort + N''')' AS Result
			FROM planning.thListValues lv 
				LEFT JOIN planning.thLists l ON lv.ListID = l.ListID							
				LEFT JOIN planning.tdValueSeries vs ON l.ListID = vs.ValueListID AND ((vs.[IsNumeric] = 1 AND l.Datentyp = N'INT') OR (vs.[IsNumeric] = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.tfValues v ON vs.ValueSeriesKey = v.ValueSeriesKey
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE v.ValueKey IS NULL
		 )
		 INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		 SELECT N'Unused List Values', N'Info', Result, 1
		 FROM mmbr;

		 SET @EffectedRows += @@ROWCOUNT;

--  List members which are Used but dont exist
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), v.ValueKey) + N' ''' + CASE WHEN vs.IsNumeric = 1 THEN CONVERT(NVARCHAR(MAX), v.ValueInt) ELSE v.ValueText END
				+ N''' ListID ' + vs.ValueListID 
				+ N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), v.ValueSeriesKey) + N', ' + vs.ValueSeriesID
				+ N' ''' + vs.NameShort + N''')' AS Result
			FROM planning.tfValues v
				INNER JOIN planning.tdValueSeries vs ON vs.ValueListID <> N'' AND vs.ValueSeriesKey = v.ValueSeriesKey
				LEFT JOIN planning.thLists l ON vs.ValueListID = l.ListID AND ((vs.IsNumeric = 1 AND l.Datentyp = N'INT') OR (vs.IsNumeric = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.thListValues lv ON lv.ListID = l.ListID
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE lv.ListValueKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists List Values', N'Error', Result, 1
		FROM mmbr;

		SET @EffectedRows += @@ROWCOUNT;

		;WITH frm AS (									 
			SELECT ValueFormatID AS FormatID , N'FormatID ' + ValueFormatID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), ValueSeriesKey) + N', ' + ValueSeriesID + N' ''' + NameShort + N''')' AS Result			 
			FROM planning.tdValueSeries
			WHERE ValueFormatID <> N''
			UNION ALL												   
			SELECT FormatID , N'FormatID ' + FormatID + N' (Cluster Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgCluster
			WHERE FormatID <> N'' 								  
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Factory Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result								   
			FROM planning.tgFactories
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ProductLine Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result							  		
			FROM planning.tgProductLines
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Product Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgProducts
			WHERE FormatID <> N'' 
			UNION ALL										 
			SELECT FormatID , N'FormatID ' + FormatID + N' (List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + NameShort + N''')' AS Result
			FROM planning.thLists
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ListValue #' + CONVERT(NVARCHAR(MAX), ListValueKey) + N', ListID ' + ListID + N')' AS Result
			FROM planning.thListValues
			WHERE FormatID <> N'' 
		)
	-- Formats which are used but don`t exist
		, unusedFrmts AS (
			SELECT N'Format #' + CONVERT(NVARCHAR(MAX), FormatKey) + N', ' + FormatID AS Result												
			FROM planning.thFormats
			WHERE FormatID NOT IN (SELECT DISTINCT FormatID FROM frm)
		)
	-- Formats which are defined but not used	
		, notExistsFrmts AS (
			SELECT Result 
			FROM frm LEFT JOIN planning.thFormats f ON frm.FormatID	= f.FormatID
			WHERE f.FormatID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Formats', N'Info', Result, 1
		FROM unusedFrmts
		UNION ALL
		SELECT N'Don`t exists Formats', N'Error', Result, 1
		FROM notExistsFrmts;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValueSeries per Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'ValueSeries per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tdValueSeries	vs INNER JOIN planning.tdProducts p ON vs.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValuesPer Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Values per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tfValues v INNER JOIN planning.tdProducts p ON v.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;								   	   
		
		SET @EffectedRows += @@ROWCOUNT;

		SELECT * 
		FROM @Tests
		ORDER BY N;

		COMMIT TRANSACTION sx_pf_GET_ClusterValidation;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterValidation;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterValidation TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterValidation TO pf_PlanningFactoryService;
GO
/*
GET Operation for Write Rights on a Cluster
Returns 200 if User has ClusterWriteRights

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ClusterWriteRight 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ClusterWriteRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ClusterWriteRight;
GO

CREATE PROCEDURE sx_pf_GET_ClusterWriteRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterWriteRight;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
				
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 3 - Test that user has write access to the cluster
		SELECT @ResultCode = IIF([Right] = N'Write', 200, 401) FROM planning.trUserRights WHERE FactoryID = N'' AND Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User have no write access to the cluster', 16, 10);

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterWriteRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterWriteRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;

END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for all eligible Factories
Returns the list of all the factories, for which the user has rights

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call - should return Table and 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Factories 'SQL'
PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Factories') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Factories;
GO

CREATE PROCEDURE sx_pf_GET_Factories
	@Username NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factories;

		-- STEP 0.1 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 3. Show public values	
		;WITH rights AS (
			SELECT 
				FactoryID,  
				[Right],
				ISNULL(TRY_CAST(FactoryID AS INT), 999999999) AS orderID 
			FROM planning.trUserRights
			WHERE ProductLineID = N'' 
				AND Username = @TransactUsername
				AND [Right] IN (N'Read', N'Write')
		)
		SELECT   
			dF.FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,vUR.[Right]
		FROM planning.tdFactories dF 
			INNER JOIN rights vUR ON dF.FactoryID = vUR.FactoryID	
		ORDER BY vUR.orderID, dF.FactoryID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Factories;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factories;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Factories TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Factories TO pf_PlanningFactoryService;
GO
/*
GET Operation for one Factory
Delivers all informations about one factory

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

-- test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Factory
	@Username	= 'SQL',
	@FactoryID	= 'ZT'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Factory;
GO

CREATE PROCEDURE sx_pf_GET_Factory
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,@Right AS 'Right'
		FROM planning.tdFactories
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Factory TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a Factory Properties

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_FactoryProperties 
	@Username	= 'SQL',
	@FactoryID	= 'D'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FactoryProperties;
GO

CREATE PROCEDURE sx_pf_GET_FactoryProperties
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N''		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgFactories
		WHERE FactoryKey = @FactoryKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a Factory Property

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_FactoryProperty 
	@Username	= 'SQL',
	@FactoryID	= 'D',
	@PropertyID = 'F01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FactoryProperty;
GO

CREATE PROCEDURE sx_pf_GET_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				 = 0;
	DECLARE @PropertyKey		INT				 = 0;
	DECLARE @Right				NVARCHAR(10)	 = N'';
	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	 = N'''' 
			+ ISNULL(@Username,		 N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	 + N''',''' 
			+ ISNULL(@PropertyID,	 N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;				-- SET during Execution
	DECLARE @ResultCode			INT				 = 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username		= N'';
	IF @FactoryID	IS NULL SET @FactoryID		= N'';
	IF @PropertyID	IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);	

		IF	@FactoryID = N'' 
			OR @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF @FactoryKey = 0 OR @PropertyKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgFactories
		WHERE PropertyKey = @PropertyKey; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining the Presentation Content of a Factory

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Supports three types of content:
	Pivot:				Data from an StoredProcedure is displayed as Pivot Table
	Pivot_Chart:		Data from an StoredProcedure is displayed as Pivot Chart
	HTML_URL:			HTML Content based on a given URL is displayed
	HTML_Static:		HTML Content based on direct submitted HTML Code from an StoredProcedure is displayed
	ProductlineList:	List of Productlines inside the Factory

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_FactoryTabs 
	@Username = 'SQL',
	@FactoryID = 'ZT'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FactoryTabs') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FactoryTabs;
GO

CREATE PROCEDURE sx_pf_GET_FactoryTabs
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryTabs;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID (@FactoryID);

		IF @FactoryID = N''		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- Show Properties of Type TAB*
		SELECT 
			 PropertyID		AS TabID				-- PropertyID	- ID of the Tab, also ORDER BY Criteria (TAB01, TAB02...)
			,PropertyName	AS Label				-- PropertyName - Label for the tab
			,CommentUser	AS CommentUser			-- CommentUser	- displayed as HeaderComment for Pivottables
			,Unit			AS PresentationType		-- Unit			- Pivot, Pivot_Chart HTML_URL, HTML_Static,ProductlineList
			,ValueText		AS Source				-- ValueText	- URL oder SQL 
			,CommentDev		AS Layout				-- CommentDev	- String for saved Layout 
			
			--TODO !! - Layout must be handled in Backend as Userdependend or not ! - maybe we use this Layout here as Standard and give the possibility to save another one for user
		FROM planning.tgFactories
		WHERE 
				FactoryID = @FactoryID 
			AND PropertyID LIKE 'TAB%'

		ORDER BY PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryTabs;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryTabs;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryTabs TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FactoryTabs TO pf_PlanningFactoryService;
/*
GET Operation for receiving a filterd context sensitiv List with dynamic column count
The first column is sended back with POST_FlexAction as idenitfier for a database action

Procedure must be editend to fit user needs and recreated after each API Update.

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call
--should return Data and 200
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_FlexAction
	 @Username				= 'SQL'
	,@FactoryID				= 'F1'
	,@ProductLineID			= 'PL1'
	,@ProductID				= 'P1'
	,@ValueSeriesID			= 'Projekt'
	,@TimeID				= 20180115
	,@GlobalattributeNumber = ''
	,@PageType				= ''
	,@SearchString			= 'ei';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FlexAction') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FlexAction;
GO

CREATE PROCEDURE sx_pf_GET_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				

	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,				 N'NULL')					+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL')					+ N''','
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL')					+ N''','''	
			+ ISNULL(@PageType,				 N'NULL')					+ N''',''' 
			+ ISNULL(@SearchString,			 N'NULL')					+ N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= N'';
	IF @PageType				IS NULL SET @PageType				= N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
	
		-- STEP 3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Projects AS TABLE
		(
			ProjectID		NVARCHAR (255) NOT NULL
			,ProjectName	NVARCHAR (255) NOT NULL
			,ProjectManager NVARCHAR (255) NOT NULL
		);

		INSERT INTO @Projects
		VALUES
			(N'P1000',N'Testprojekt in Aachen','Pit Brown'),
			(N'P1001',N'Testprojekt in Augsburg','Pit Brown'),
			(N'P1002',N'Testprojekt in Bergisch Gladbach','Pit Brown'),
			(N'P1003',N'Testprojekt in Berlin','Pit Brown'),
			(N'P1004',N'Testprojekt in Bielefeld','Pit Brown'),
			(N'P1005',N'Testprojekt in Bochum','Pit Brown'),
			(N'P1006',N'Testprojekt in Bonn','Pit Brown'),
			(N'P1007',N'Testprojekt in Bottrop','Pit Brown'),
			(N'P1008',N'Testprojekt in Braunschweig','Pit Brown'),
			(N'P1009',N'Testprojekt in Bremen','Pit Brown'),
			(N'P1010',N'Testprojekt in Bremerhaven','Pit Brown'),
			(N'P1011',N'Testprojekt in Chemnitz','Pit Brown'),
			(N'P1012',N'Testprojekt in Darmstadt','Pit Brown'),
			(N'P1013',N'Testprojekt in Dortmund','Pit Brown'),
			(N'P1014',N'Testprojekt in Dresden','Pit Brown'),
			(N'P1015',N'Testprojekt in Duisburg','Pit Brown'),
			(N'P1016',N'Testprojekt in D�sseldorf','Pit Brown'),
			(N'P1017',N'Testprojekt in Erfurt','Pit Brown'),
			(N'P1018',N'Testprojekt in Erlangen','Pit Brown'),
			(N'P1019',N'Testprojekt in Essen','Pit Brown'),
			(N'P1020',N'Testprojekt in Frankfurt am Main','Pit Brown'),
			(N'P1021',N'Testprojekt in Freiburg im Breisgau','Pit Brown'),
			(N'P1022',N'Testprojekt in F�rth','Pit Brown'),
			(N'P1023',N'Testprojekt in Gelsenkirchen','Pit Brown'),
			(N'P1024',N'Testprojekt in G�ttingen','Pit Brown'),
			(N'P1025',N'Testprojekt in Hagen','Pit Brown'),
			(N'P1026',N'Testprojekt in Halle (Saale)','Pit Brown'),
			(N'P1027',N'Testprojekt in Hamburg','Pit Brown'),
			(N'P1028',N'Testprojekt in Hamm','Pit Brown'),
			(N'P1029',N'Testprojekt in Hannover','Pit Brown'),
			(N'P1030',N'Testprojekt in Heidelberg','Cathy Gold'),
			(N'P1032',N'Testprojekt in Heilbronn','Cathy Gold'),
			(N'P1032',N'Testprojekt in Herne','Cathy Gold'),
			(N'P1033',N'Testprojekt in Ingolstadt','Cathy Gold'),
			(N'P1034',N'Testprojekt in Jena','Cathy Gold'),
			(N'P1035',N'Testprojekt in Karlsruhe','Cathy Gold'),
			(N'P1036',N'Testprojekt in Kassel','Cathy Gold'),
			(N'P1037',N'Testprojekt in Kiel','Cathy Gold'),
			(N'P1038',N'Testprojekt in Koblenz','Cathy Gold'),
			(N'P1039',N'Testprojekt in K�ln','Cathy Gold'),
			(N'P1040',N'Testprojekt in Krefeld','Cathy Gold'),
			(N'P1041',N'Testprojekt in Leipzig','Cathy Gold'),
			(N'P1042',N'Testprojekt in Leverkusen','Cathy Gold'),
			(N'P1043',N'Testprojekt in L�beck','Cathy Gold'),
			(N'P1044',N'Testprojekt in Ludwigshafen am Rhein','Cathy Gold'),
			(N'P1045',N'Testprojekt in Magdeburg','Cathy Gold'),
			(N'P1046',N'Testprojekt in Mainz','Cathy Gold'),
			(N'P1047',N'Testprojekt in Mannheim','Cathy Gold'),
			(N'P1048',N'Testprojekt in Moers','Cathy Gold'),
			(N'P1048',N'Testprojekt in M�nchengladbach','Cathy Gold'),
			(N'P1049',N'Testprojekt in M�lheim an der Ruhr','Cathy Gold'),
			(N'P1050',N'Testprojekt in M�nchen','Cathy Gold'),
			(N'P1051',N'Testprojekt in M�nster','Cathy Gold'),
			(N'P1052',N'Testprojekt in Neuss','Cathy Gold'),
			(N'P1053',N'Testprojekt in N�rnberg','Cathy Gold'),
			(N'P1054',N'Testprojekt in Oberhausen','Cathy Gold'),
			(N'P1055',N'Testprojekt in Offenbach am Main','Cathy Gold'),
			(N'P1056',N'Testprojekt in Oldenburg','Cathy Gold'),
			(N'P1057',N'Testprojekt in Osnabr�ck','Cathy Gold'),
			(N'P1058',N'Testprojekt in Paderborn','Cathy Gold'),
			(N'P1059',N'Testprojekt in Pforzheim','Cathy Gold'),
			(N'P1060',N'Testprojekt in Potsdam','Cathy Gold'),
			(N'P1061',N'Testprojekt in Recklinghausen','Cathy Gold'),
			(N'P1062',N'Testprojekt in Regensburg','Cathy Gold'),
			(N'P1063',N'Testprojekt in Remscheid','Cathy Gold'),
			(N'P1064',N'Testprojekt in Reutlingen','Cathy Gold'),
			(N'P1065',N'Testprojekt in Rostock','Cathy Gold'),
			(N'P1066',N'Testprojekt in Saarbr�cken','Cathy Gold'),
			(N'P1067',N'Testprojekt in Siegen','Cathy Gold'),
			(N'P1068',N'Testprojekt in Solingen','Cathy Gold'),
			(N'P1069',N'Testprojekt in Stuttgart','Cathy Gold'),
			(N'P1070',N'Testprojekt in Trier','Cathy Gold'),
			(N'P1071',N'Testprojekt in Ulm','Cathy Gold'),
			(N'P1072',N'Testprojekt in Wiesbaden','Cathy Gold'),
			(N'P1073',N'Testprojekt in Wolfsburg','Cathy Gold'),
			(N'P1074',N'Testprojekt in Wuppertal','Cathy Gold'),
			(N'P1075',N'Testprojekt in W�rzburg','Cathy Gold');
		
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Second Column must have identifier character, its send back with the POST_FlexAction Command
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R= Right, C = Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter

		-- React on content parameters when creating the return list.
		-- You could add a "*" Member to the List and react on it in POST to create a new list member

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			SELECT 'Projekt wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS ProjectID_RS50
				,p.ProjectName AS Projektname_LS300
				,p.ProjectManager AS Projektmanager_CS150
			FROM @Projects p
			WHERE p.ProjectName LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			SELECT 'Mitarbeiter wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS MitarbeiterID_RS50
				,p.ProjectManager AS Projektmanager_LS150
			FROM @Projects p
			WHERE p.ProjectManager LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter')
		BEGIN
			SELECT 'FlexActions sind nur f�r die Wertreihen "Projekt" und "Mitarbeiter" definiert' AS Hint;
		END

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexAction;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexAction TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexAction TO pf_PlanningFactoryService
GO
/*
GET Operation for receiving a context sensitiv List
API deliviers List Values fitting to the sended context Parameters
Procedure must be editend to fit user needs and recreated after each API Update
When updating this procedure, you must reset the rights for it

08/2019 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
--should return Data and 200
EXEC @RC = sx_pf_GET_FlexDrill 
	@Username				= 'SQL'
	,@FactoryID				= '1'
	,@ProductLineID			= '1'
	,@ProductID				= '1'
	,@ValueSeriesID			= '1'
	,@TimeID				= 20190115
	,@GlobalattributeNumber = ''
	,@PageType				= ''
	,@SearchString			= 'r';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FlexDrill') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FlexDrill;
GO

CREATE PROCEDURE sx_pf_GET_FlexDrill
	 @Username				 NVARCHAR(255)
	,@FactoryID				 NVARCHAR(255)
	,@ProductLineID			 NVARCHAR(255)
	,@ProductID				 NVARCHAR(255)
	,@ValueSeriesID			 NVARCHAR(255)
	,@TimeID				 BIGINT
	,@GlobalattributeNumber  NVARCHAR(255)
	,@PageType				 NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			 NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	DECLARE @ParameterString	NVARCHAR(MAX)	=		N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			 N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL') + N''',''' 
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL') + N''',''' 
			+ ISNULL(@PageType,				 N'NULL') + N''',''' 
			+ ISNULL(@SearchString,			 N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username			  IS NULL SET @Username				 = N'';
	IF @FactoryID			  IS NULL SET @FactoryID			 = N'';
	IF @ProductLineID		  IS NULL SET @ProductlineID		 = N'';
	IF @ProductID			  IS NULL SET @ProductID			 = N'';
	IF @ValueSeriesID		  IS NULL SET @ValueSeriesID		 = N'';
	IF @TimeID				  IS NULL SET @TimeID				 = 0;
	IF @GlobalattributeNumber IS NULL SET @GlobalattributeNumber = N'';
	IF @PageType			  IS NULL SET @PageType				 = N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexDrill;

		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSeriesID	 = dbo.sx_pf_pProtectID		 (@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
	
		-- STEP 3.1 - Sample for creating a List for a specific ValueSeries in all Products from specific template
		IF @ValueSeriesID = N'that' 
			AND EXISTS(
				SELECT Template FROM planning.tdProducts 
				WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 'ValueSeriesSpecificList' AS Dummy;
			SET @EffectedRows = 1;
		END;

		-- STEP 3.2 - Sample for creating a List for a Globalattribute in one Productline
		IF @FactoryID = N'this' AND @ProductlineID = N'that' AND @GlobalattributeNumber = 5
		BEGIN
			SELECT 'ProductlineSpecificList for ValueSeries'; 
			SET @EffectedRows += 1;
		END;
		
		-- STEP 3.3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

		INSERT INTO @Cities VALUES
			(N'Aachen'),
			(N'Augsburg'),
			(N'Bergisch Gladbach'),
			(N'Berlin'),
			(N'Bielefeld'),
			(N'Bochum'),
			(N'Bonn'),
			(N'Bottrop'),
			(N'Braunschweig'),
			(N'Bremen'),
			(N'Bremerhaven'),
			(N'Chemnitz'),
			(N'Darmstadt'),
			(N'Dortmund'),
			(N'Dresden'),
			(N'Duisburg'),
			(N'Düsseldorf'),
			(N'Erfurt'),
			(N'Erlangen'),
			(N'Essen'),
			(N'Frankfurt am Main'),
			(N'Freiburg im Breisgau'),
			(N'Fürth'),
			(N'Gelsenkirchen'),
			(N'Göttingen'),
			(N'Hagen'),
			(N'Halle (Saale)'),
			(N'Hamburg'),
			(N'Hamm'),
			(N'Hannover'),
			(N'Heidelberg'),
			(N'Heilbronn'),
			(N'Herne'),
			(N'Ingolstadt'),
			(N'Jena'),
			(N'Karlsruhe'),
			(N'Kassel'),
			(N'Kiel'),
			(N'Koblenz'),
			(N'Köln'),
			(N'Krefeld'),
			(N'Leipzig'),
			(N'Leverkusen'),
			(N'Lübeck'),
			(N'Ludwigshafen am Rhein'),
			(N'Magdeburg'),
			(N'Mainz'),
			(N'Mannheim'),
			(N'Moers'),
			(N'Mönchengladbach'),
			(N'Mülheim an der Ruhr'),
			(N'München'),
			(N'Münster'),
			(N'Neuss'),
			(N'Nürnberg'),
			(N'Oberhausen'),
			(N'Offenbach am Main'),
			(N'Oldenburg'),
			(N'Osnabrück'),
			(N'Paderborn'),
			(N'Pforzheim'),
			(N'Potsdam'),
			(N'Recklinghausen'),
			(N'Regensburg'),
			(N'Remscheid'),
			(N'Reutlingen'),
			(N'Rostock'),
			(N'Saarbrücken'),
			(N'Siegen'),
			(N'Solingen'),
			(N'Stuttgart'),
			(N'Trier'),
			(N'Ulm'),
			(N'Wiesbaden'),
			(N'Wolfsburg'),
			(N'Wuppertal'),
			(N'Würzburg');

		SELECT 
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R=Right, C=Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter
			'List of Cities to select.'	 AS Hint
			,UPPER(LEFT([Name], 4))		 AS CityCode_LS50
			,[Name]						 AS CityName_CS200
			,'Beschreibung dieser Stadt...' AS Beschreibung_LS300
			,500000						 AS Einwohnerzahl_LM50
		 FROM @Cities
		 WHERE [Name] LIKE @SearchString
		 ORDER BY [Name];

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexDrill;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexDrill;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexDrill TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexDrill TO pf_PlanningFactoryService
GO
/*
GET Operation for receiving a context sensitiv List
API deliviers List Values fitting to the sended context Parameters
Procedure must be editend to fit user needs and recreated after each API Update
When updating this procedure, you must reset the rights for it

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC AS NVARCHAR(255)
--should return Data and 200
EXEC @RC = sx_pf_GET_FlexList 'SQL','1','1','1','1','','','',''
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_FlexList') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_FlexList;
GO

CREATE PROCEDURE sx_pf_GET_FlexList
	@Username				NVARCHAR(255)
	,@FactoryID 			NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @GlobalattributeAlias	NVARCHAR(255)	= N'';
	DECLARE @SearchDoneFlag			INT				= 0;
	DECLARE @ProductKey				BIGINT			= 0;
	DECLARE @DummyTable TABLE (Dummy NVARCHAR(255) NOT NULL)  -- only for the samples to compile
	DECLARE @DummyInt				INT
	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductID,				N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,			N'NULL') + N''','
			+ CAST(ISNULL(@TimeID,				N'NULL') AS NVARCHAR(255)) + N','''
			+ ISNULL(@GlobalattributeNumber,	N'NULL') + N''',''' 
			+ ISNULL(@PageType,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexList;

		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Preparing of useful context informations 
		-- ****************************************************************************************************************************************************
		-- Try to determine ProductKey
		IF @ProductID			<> N'' 
			AND @ProductLineID	<> N'' 
			AND @FactoryID		<> N''
		BEGIN
			SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
		END

		-- Determine AliasName of Globalattribute, to define actions for Globalattributes by name
		IF @GlobalattributeNumber <> ''
		BEGIN
			IF OBJECT_ID('tempdb..#GANames') IS NOT NULL
				DROP TABLE #GANames;

			CREATE TABLE #GANames
			(
				RowKey					BIGINT IDENTITY (1,1)	NOT NULL
				,FactoryID				NVARCHAR (255)			NOT NULL
				,ProductLineID			NVARCHAR (255)			NOT NULL
				,Globalattribute		NVARCHAR (255) 			NOT NULL
				,GlobalattributeNumber	INT						NOT NULL
				,Aliasname				NVARCHAR (255)			NOT NULL
			)
			
			INSERT INTO #GANames
			SELECT
				FactoryID
				,ProductLineID
				,Globalattribute
				,COALESCE(TRY_CAST(RIGHT(Globalattribute,2) AS INT), TRY_CAST(RIGHT(Globalattribute,1) AS INT)) AS GlobalattributeNumber
				,Aliasname
			FROM planning.tdProductLines
			UNPIVOT
			(
				Aliasname FOR Globalattribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalattributeAlias3, GlobalattributeAlias4, GlobalattributeAlias5
					,GlobalattributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10
					,GlobalattributeAlias11, GlobalattributeAlias12, GlobalattributeAlias13, GlobalattributeAlias14, GlobalattributeAlias15
					,GlobalattributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20
					,GlobalattributeAlias21, GlobalattributeAlias22, GlobalattributeAlias23, GlobalattributeAlias24, GlobalattributeAlias25
				)
			) unpiv
			WHERE 
					FactoryID		= @FactoryID 
				AND	ProductlineID	= @ProductLineID;

			SELECT @GlobalattributeAlias = Aliasname FROM #GANames WHERE GlobalattributeNumber = @GlobalattributeNumber;
		END

		/*
		DEFINITION OF THE FLEX LISTS
		*****************************************************************************************************************************************************************
		First Column must be "Hint" an commits the display hint to the frontend
		Second Column must have identifier character, this value is sended back to the selected cell
		Column Names must have an Alias (AS Displayname) with ONLY ONE "_", after which 
			Orientation		(L=Left, R= Right, C = Center)
			DataType		(M=Money, S=String, I=Int) 
			ColumnWidth		(Integer Number)
		is coded as Frontend Parameter, e.g. Name_LS50  means the column ist labeled Name, has left orientation,
		is formated as sting an 50px width
		*/

		-- Sample for creating a List for all ValueSeries with a specific ID
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable; 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;
		
		-- Sample for creating a List for all ValueSeries with a specific ID in a specific Template
		IF @ValueSeriesID = N'SKTO' AND EXISTS
		(
			SELECT Template 
			FROM planning.tdProducts 
			WHERE 
				ProductKey = @ProductKey 
				AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;

		-- Sample for Seach over parameter Product
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT DISTINCT
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,fV.ValueText	AS KontenID_LS40
				,fV2.ValueText	AS KontenName_LS180
			FROM planning.tfValues fV 
				-- Kontoname aus anderer Spalte
				LEFT JOIN
					(
						SELECT
							TimeID
							,ValueText 
						FROM planning.tfValues 
						WHERE 
								FactoryID = 'ZP'
							AND ProductLineID = '1'
							AND ProductID = '1'
							AND ValueSeriesID = 'KTO_Name'
						
					) fV2 ON fV.TimeID = fV2.TimeID
			WHERE 
				fV.FactoryID = 'ZP'
				AND fV.ProductLineID = '1'
				AND fV.ProductID = '1'
				AND fV.ValueSeriesID = 'KTO_ID'
				AND (fV.ValueText LIKE @SearchString OR fV2.ValueText LIKE @SearchString)
			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END

		-- Samples for other context
		IF	@GlobalattributeAlias = N'SKTO'		-- for this Globalattribute
			OR LEFT(@ValueSeriesID, 3) = N'KST' -- for all ValueSeries starting with this letters
		BEGIN	
			SET @DummyInt = 1 
		END	
		
		-- Sample Fallback -- delete in productive use to change fallback to Productive Fallback
		-- **************************************************************************************************************************************
		IF @SearchDoneFlag  = 0
		BEGIN
			DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

			INSERT INTO @Cities 
			VALUES
				(N'Aachen'),
				(N'Augsburg'),
				(N'Bergisch Gladbach'),
				(N'Berlin'),
				(N'Bielefeld'),
				(N'Bochum'),
				(N'Bonn'),
				(N'Bottrop'),
				(N'Braunschweig'),
				(N'Bremen'),
				(N'Bremerhaven'),
				(N'Chemnitz'),
				(N'Darmstadt'),
				(N'Dortmund'),
				(N'Dresden'),
				(N'Duisburg'),
				(N'Düsseldorf'),
				(N'Erfurt'),
				(N'Erlangen'),
				(N'Essen'),
				(N'Frankfurt am Main'),
				(N'Freiburg im Breisgau'),
				(N'Fürth'),
				(N'Gelsenkirchen'),
				(N'Göttingen'),
				(N'Hagen'),
				(N'Halle (Saale)'),
				(N'Hamburg'),
				(N'Hamm'),
				(N'Hannover'),
				(N'Heidelberg'),
				(N'Heilbronn'),
				(N'Herne'),
				(N'Ingolstadt'),
				(N'Jena'),
				(N'Karlsruhe'),
				(N'Kassel'),
				(N'Kiel'),
				(N'Koblenz'),
				(N'Köln'),
				(N'Krefeld'),
				(N'Leipzig'),
				(N'Leverkusen'),
				(N'Lübeck'),
				(N'Ludwigshafen am Rhein'),
				(N'Magdeburg'),
				(N'Mainz'),
				(N'Mannheim'),
				(N'Moers'),
				(N'Mönchengladbach'),
				(N'Mülheim an der Ruhr'),
				(N'München'),
				(N'Münster'),
				(N'Neuss'),
				(N'Nürnberg'),
				(N'Oberhausen'),
				(N'Offenbach am Main'),
				(N'Oldenburg'),
				(N'Osnabrück'),
				(N'Paderborn'),
				(N'Pforzheim'),
				(N'Potsdam'),
				(N'Recklinghausen'),
				(N'Regensburg'),
				(N'Remscheid'),
				(N'Reutlingen'),
				(N'Rostock'),
				(N'Saarbrücken'),
				(N'Siegen'),
				(N'Solingen'),
				(N'Stuttgart'),
				(N'Trier'),
				(N'Ulm'),
				(N'Wiesbaden'),
				(N'Wolfsburg'),
				(N'Wuppertal'),
				(N'Würzburg');
		
			SELECT 
				'List of Cities, you can search with substring.' AS Hint
				,UPPER(LEFT([Name], 4)) AS CityCode_LS50
				,[Name] AS CityName_CS200
			FROM @Cities
		 	WHERE [Name] LIKE @SearchString
		 	ORDER BY [Name];
			
			SET @SearchDoneFlag = 1
		END

		-- Productive Fallback - if no flex search matched the query
		-- **************************************************************************************
		IF @SearchDoneFlag = 0 
		BEGIN
			SELECT 
				'For this Seach Context, we sadly must tell to have no fitting content to offer. ' AS Hint
				,'Nothing' AS Content_LS50
				,'is all I have to offer for you.' AS Sorry_LS200
		END

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexList;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexList;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexList TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::sx_pf_GET_FlexList TO pf_PlanningFactoryService
GO
/*
GET Operation for a Format

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Format 
	@Username = 'SQL', 
	@FormatID = 'HintergrundRot'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Format;
GO

CREATE PROCEDURE sx_pf_GET_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			 INT			= 0;
	DECLARE @Right				 NVARCHAR(10)	= '';
	DECLARE @TransactUsername	 NVARCHAR(255)	= '';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@FormatID, N'NULL')		+ N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT  @FormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @FormatID;
		
		IF @FormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM planning.thFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Format TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Formats in Cluster

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Formats 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Formats') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Formats;
GO

CREATE PROCEDURE sx_pf_GET_Formats
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				 NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;						
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Formats;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
	
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM planning.thFormats
		ORDER BY ISNULL(TRY_CAST(FormatID AS INT), 999999999), FormatID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Formats;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Formats;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Formats TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Formats TO pf_PlanningFactoryService;
GO
/*
GET Operation for receiving full text search results for a given textstring
Search will be done in all DataFactory Elements

10/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC AS NVARCHAR(255)
--should return Data and 200
EXEC @RC = sx_pf_GET_GlobalSearch 'SQL','whixc'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_GlobalSearch') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_GlobalSearch;
GO

CREATE PROCEDURE sx_pf_GET_GlobalSearch
	 @Username				NVARCHAR(255)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SearchDoneFlag			INT				= 0;

	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username				= N'';
	
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_GlobalSearch;

		-- Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Table for Structure Cache
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#Structure') IS NOT NULL
				DROP TABLE #Structure;

			CREATE TABLE #Structure
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryKey					BIGINT											 NOT NULL
				,ProductlineKey				BIGINT											 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,ValueSeriesKey				BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductlineNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #Structure
			-- must also return Factories / Productlines without Products
				SELECT
					  dF.FactoryKey
					 ,ISNULL(dPL.ProductlineKey	,-1)					AS ProductlineKey
					 ,ISNULL(dP.ProductKey		,-1)					AS ProductKey
					 ,ISNULL(dVS.ValueSeriesKey	,-1)					AS ValueSeriesKey
					 ,dF.FactoryID		
					 ,ISNULL(dPL.ProductLineID	,'-1')					AS ProductlineID
					 ,ISNULL(dP.ProductID		,'-1')					AS ProductID
					 ,ISNULL(dVS.ValueSeriesID	,'-1')					AS ValueSeriesID
					 ,dF.NameShort										AS FactoryNameShort
					 ,ISNULL(dPL.NameShort	,'<without Productline>')	AS ProductlineNameShort
					 ,ISNULL(dP.NameShort	,'<without Product>')		AS ProductNameShort
					 ,ISNULL(dVS.NameShort	,'<without ValueSeries>')	AS ValueSeriesNameShort

			  FROM  planning.tdFactories dF

					LEFT JOIN planning.tdProductLines dPL
						ON dF.FactoryKey = dPL.FactoryKey

					LEFT JOIN planning.tdProducts dP
						ON dPL.ProductlineKey = dP.ProductLineKey

					LEFT JOIN planning.tdValueSeries dVS
						ON dP.ProductKey = dVS.ProductKey
				
			
		-- Helper Table for Globalattributes
		-- ****************************************************************************************************************************************************
			IF OBJECT_ID('tempdb..#GA') IS NOT NULL
				DROP TABLE #GA;

			CREATE TABLE #GA
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Globalattribute			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Content					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #GA
				SELECT 
						 ProductKey
						,FactoryID
						,ProductLineID
						,ProductID
						,UnpivotTable.Globalattribute
						,UnpivotTable.Content

				FROM
					planning.tdProducts
		
				UNPIVOT
					(	

						Content FOR Globalattribute IN (
													 Globalattribute1,Globalattribute2,Globalattribute3,Globalattribute4,Globalattribute5
													,Globalattribute6,Globalattribute7,Globalattribute8,Globalattribute9,Globalattribute10
													,Globalattribute11,Globalattribute12,Globalattribute13,Globalattribute14,Globalattribute15
													,Globalattribute16,Globalattribute17,Globalattribute18,Globalattribute19,Globalattribute20
													,Globalattribute21,Globalattribute22,Globalattribute23,Globalattribute24,Globalattribute25
												)
					) AS UnpivotTable

				WHERE Content <>''


		-- Table for Search Result Collection
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#SearchResult') IS NOT NULL
				DROP TABLE #SearchResult;

			CREATE TABLE #SearchResult
			(
				 RowKey					BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resultgroup			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resulttype				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,DataFactoryPath		NVARCHAR (1000)			COLLATE DATABASE_DEFAULT NOT NULL
				,Result					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)
		-- Getting Content
		-- ****************************************************************************************************************************************************
		
		-- Factories BEGIN #####

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'FactoryID'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.FactoryID
			FROM planning.tdFactories dF
			WHERE dF.FactoryID LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'NameShort'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.NameShort
			FROM planning.tdFactories dF
			WHERE dF.NameShort LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'ResponsiblePerson'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.ResponsiblePerson
			FROM planning.tdFactories dF
			WHERE dF.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Factories END #####

		-- Productlines BEGIN #####
		
		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'ProductlineID'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ProductLineID
			FROM planning.tdProductLines dPL
					LEFT JOIN (
								SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
							   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ProductLineID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.NameShort
			FROM planning.tdProductLines dPL
					LEFT JOIN (
									SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
								   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'Productline'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ResponsiblePerson
			FROM planning.tdProductLines dPL
					LEFT JOIN (
						SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
						) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Productlines END #####

		-- Products BEGIN ####
		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ProductID'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ProductID
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.ProductID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.NameShort
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ResponsiblePerson
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.ResponsiblePerson LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'CommmentUser'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,LEFT(dP.CommentUser,250)+' ...'
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
			WHERE dP.CommentUser LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,GA.Globalattribute
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,GA.Content
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductlineKey
					LEFT JOIN #GA GA
						ON dP.ProductKey = GA.ProductKey
			WHERE GA.Content LIKE @SearchString

		-- Documents on Products
		INSERT INTO #SearchResult
			SELECT tF.FactoryID
			      ,tF.ProductLineID
				  ,tF.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'Documents'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + ' (' + tF.FactoryID + '>'+tF.ProductLineID + '>' + tF.ProductID + ')' AS Path
				  ,tF.[FileName]
			FROM planning.tFiles tF
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON tF.ProductLineKey = S.ProductlineKey
			WHERE tf.[FileName] LIKE @SearchString


		-- Products END ####

		-- ValueSeries Start ####
		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'ValueSeriesID'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.ValueSeriesID
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.ValueSeriesID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameShort
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameLong'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameLong
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameLong LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'CommentUser'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,LEFT(dVS.CommentUser,250)+' ...'
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductlineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.CommentUser LIKE @SearchString

		-- ValueSeries END ####

		-- Value Start ###
		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueText'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueText,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductlineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueText LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueComment'
				  ,S.FactoryNameShort + '>' + S.ProductlineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueComment,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductlineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueComment LIKE @SearchString

			
		-- Output

		SELECT TOP (100)
			   sr.FactoryID
		      ,sr.ProductLineID
			  ,sr.ProductID
			  ,sr.ValueSeriesID
			  ,sr.Resultgroup
			  ,sr.Resulttype
			  ,sr.DataFactoryPath
			  ,sr.Result
		FROM #SearchResult sr

			INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						(
							FactoryID <> '' AND ProductLineID <> ''
							OR
							FactoryID <> '' AND ProductLineID = ''
						)
				) vUR

				ON 
						sr.FactoryID = vUR.FactoryID 
					AND (sr.ProductLineID = vUR.ProductLineID OR sr.ProductLineID = '-1') -- as Output for not existing Productline possible

		ORDER BY CASE sr.Resultgroup 
						WHEN 'Factory'		THEN 1
						WHEN 'Productline'	THEN 2
						WHEN 'Product'		THEN 3
						WHEN 'ValueSeries'	THEN 4
						WHEN 'Values'		THEN 5
						ELSE 99
				 END
					
				,sr.Resulttype,sr.FactoryID,sr.ProductLineID,sr.ProductID,sr.ValueSeriesID

		SET @SearchDoneFlag = 1

		SET @ResultCode = 200

		COMMIT TRANSACTION sx_pf_GET_GlobalSearch;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_GlobalSearch;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_GlobalSearch TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::sx_pf_GET_GlobalSearch TO pf_PlanningFactoryService
GO
/*
GET Operation for HTML Pages
10/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_HTMLPage 
	@Username		= 'SQL',
	@FactoryID		= '', 
	@ProductLineID	= '', 
	@ProductID		= '',
	@PageID			= 'Page1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_HTMLPage') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_HTMLPage;
GO

CREATE PROCEDURE sx_pf_GET_HTMLPage
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PageID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @PageKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PageID,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PageID			IS NULL SET @PageID			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPage;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PageID			= dbo.sx_pf_pProtectString	(@PageID);

		IF @PageID		= N'' 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
		
		SELECT @PageKey = PageKey FROM planning.tgHTMLPages
		WHERE
				FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND PageID			= @PageID

		IF @PageKey = 0 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;
			
		-- Check rights on different levels 
			-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
			,PageContent
		
		FROM planning.tgHTMLPages 
		WHERE PageKey = @PageKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryService;
GO
/*
GET Operation for a List of all HTML Pages
10/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_HTMLPages 
	@Username		= 'SQL',
	@FactoryID		= '', 
	@ProductLineID	= '', 
	@ProductID		= ''

PRINT @RC

SELECT * FROM planning.tgHTMLPages
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_HTMLPages') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_HTMLPages;
GO

CREATE PROCEDURE sx_pf_GET_HTMLPages 
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPages;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
			
		-- Check rights on different levels 
		-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
		
		FROM planning.tgHTMLPages;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPages;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPages;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPages TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPages TO pf_PlanningFactoryService;
GO
/*
GET Operation for cross query of ALL attributes usage

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_AllGlobalAttributeUsage 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_AllGlobalAttributeUsage') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_AllGlobalAttributeUsage;
GO

CREATE PROCEDURE sx_pf_GET_Info_AllGlobalAttributeUsage
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		;WITH dt AS (
			SELECT GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
				GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
				GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
				GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
				GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25
			FROM planning.tdProductLines dPL LEFT JOIN planning.trUserRights vUR 
				ON	dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID AND
					vUR.[Right] IN (N'Read', N'Write') AND
					vUR.UserName = @TransactUsername
		)
		, attrs AS (
			SELECT DISTINCT Attribute, Value 
			FROM 
				(SELECT * FROM dt) pvt
					UNPIVOT ([Value] FOR Attribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
					GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
					GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
					GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
					GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25)
				) AS unpvt
		)
		SELECT CAST(SUBSTRING(Attribute, 21, 2) AS INT) AS N, Value AS GlobalAttribute
		FROM attrs
		ORDER BY N, GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_AllGlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryService;
GO

/*
GET Operation for CustomFailureReport

04/2019 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights
- Returns always at least one row


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_CustomFailureReport 'SQL','ZT'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_CustomFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_CustomFailureReport;
GO

CREATE PROCEDURE sx_pf_GET_Info_CustomFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_CustomFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             CFR.Timestamp								AS LastCheckDate
            ,CFR.FactoryID
			,COALESCE(
						 CFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,CFR.ProductLineID
			,COALESCE(	
						 CFR.ProductLineID + ' ' + dPL.NameShort
						,'All Productlines'
					 )
														AS ProductlineIDName
            ,CFR.ProductID
			,COALESCE(
						 CFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,CFR.FactoryID + ' / ' + CFR.ProductLineID +' / ' + CFR.ProductID AS DataFactoryPath
            ,CFR.TestSource
            ,CFR.TestName
			,CFR.ResultType
            ,CFR.ResultText
            ,CFR.ResultNumeric
		FROM control.tCustomFailureReport CFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM planning.trUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM planning.trUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM planning.trUserRights GROUP BY UserName
						) vUR 

				ON	CFR.FactoryID		= vUR.FactoryID
				AND CFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN planning.tdFactories dF 
				ON	CFR.FactoryID		= dF.FactoryID

			-- for Productline Name
			LEFT JOIN planning.tdProductLines dPL
				ON	CFR.FactoryID		= dPL.FactoryID
				AND CFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN planning.tdProducts dP
				ON	CFR.FactoryID		= dP.FactoryID
				AND CFR.ProductLineID	= dP.ProductLineID
				AND CFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			CFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tCustomFailureReport
							)
			-- called for one or all Factories
			AND (
					CFR.FactoryID = @FactoryID OR @FactoryID = '' OR CFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN
					SELECT 
					  GETDATE()						AS LastCheckDate
					,'All'							AS FactoryID
					,'All Factories'				AS FactoryIDName
					,'All'							AS ProductLineID
					,'All Productlines'				AS ProductlineIDName
					,'All'							AS ProductID
					,'All Products'					AS ProductIDName
					,'All / All / All'				AS DataFactoryPath
					,'Hint'							AS TestSource
					,'Default Entry'				AS Testname
					,'OK'							AS ResultType
					,'Keine Eintr�ge vorhanden.'	AS ResultText
					,0								AS ResultNumeric
			END
			

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_CustomFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_CustomFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryService;
GO


/*
GET Operation for cross query of attribute usage

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC  NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_GlobalAttributeUsage 'SQL','1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_GlobalAttributeUsage') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_GlobalAttributeUsage;
GO

CREATE PROCEDURE sx_pf_GET_Info_GlobalAttributeUsage
	@Username				NVARCHAR(255),
	@GlobalattributeNumber	INT
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+		ISNULL(@Username,				N'NULL') + N''',' 
			+ CAST(	ISNULL(@GlobalattributeNumber,	N'NULL') AS NVARCHAR(255));
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_GlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		SELECT DISTINCT
		CASE @GlobalattributeNumber
			WHEN 1 THEN GlobalattributeAlias1
			WHEN 2 THEN GlobalattributeAlias2
			WHEN 3 THEN GlobalAttributeAlias3
			WHEN 4 THEN GlobalAttributeAlias4
			WHEN 5 THEN GlobalattributeAlias5
			WHEN 6 THEN GlobalAttributeAlias6
			WHEN 7 THEN GlobalattributeAlias7
			WHEN 8 THEN GlobalattributeAlias8
			WHEN 9 THEN GlobalattributeAlias9
			WHEN 10 THEN GlobalattributeAlias10
			WHEN 11 THEN GlobalattributeAlias11
			WHEN 12 THEN GlobalattributeAlias12
			WHEN 13 THEN GlobalAttributeAlias13
			WHEN 14 THEN GlobalAttributeAlias14
			WHEN 15 THEN GlobalattributeAlias15
			WHEN 16 THEN GlobalAttributeAlias16
			WHEN 17 THEN GlobalattributeAlias17
			WHEN 18 THEN GlobalattributeAlias18
			WHEN 19 THEN GlobalattributeAlias19
			WHEN 20 THEN GlobalattributeAlias20		
			WHEN 21 THEN GlobalattributeAlias21
			WHEN 22 THEN GlobalattributeAlias22
			WHEN 23 THEN GlobalAttributeAlias23
			WHEN 24 THEN GlobalAttributeAlias24
			WHEN 25 THEN GlobalattributeAlias25
			ELSE N'not existing' END AS GlobalAttribute	
		FROM planning.tdProductLines dPL LEFT JOIN planning.trUserRights vUR 
			ON	dPL.FactoryID = vUR.FactoryID AND
				dPL.ProductLineID = vUR.ProductLineID AND
				vUR.[Right] IN (N'Read',N'Write') AND
				vUR.UserName = @TransactUsername
		ORDER BY GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_GlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_GlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryService;
GO

/*
GET Operation for monitoring Properties

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
ReturnValues according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_Monitoring 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_Monitoring') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_Monitoring;
GO

CREATE PROCEDURE sx_pf_GET_Info_Monitoring
	@Username NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_Monitoring;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3 - Select Values

		IF OBJECT_ID('tempdb..#temp') IS NOT NULL
			DROP TABLE #temp;

		CREATE TABLE #temp (
			Property NVARCHAR (255) NOT NULL
			,[Value] NVARCHAR (255) NOT NULL
		);

		INSERT INTO #temp
			-- Language
			SELECT 'Cluster_C03' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C03'
			UNION ALL
			-- Clustername
			SELECT 'Cluster_C01' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C01'	
			UNION ALL
			-- API Version
			SELECT 'Cluster_C04' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C04'	
			UNION ALL
			-- FactoryCount
			SELECT 'FactoryCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdFactories
			UNION ALL
			-- ProductLineCount
			SELECT 'ProductlineCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdProductLines
			UNION ALL
			-- ProductCount
			SELECT 'ProductCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdProducts
			UNION ALL
			-- API Error 500 Count
			SELECT 'API_Error500' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tAPI_Log WHERE ReturnCode = 500 AND ProcessCode = ''
			UNION ALL
			-- API Error 404 Count
			SELECT 'API_Error404' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tAPI_Log WHERE ReturnCode = 404 AND ProcessCode = '';

		SELECT * FROM #temp;
		
		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_Monitoring;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_Monitoring;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_Monitoring TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_Monitoring TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free FactoryID
Returns the next numeric ID or the number 1 if other FactoryIDs are non-numeric

03/2019 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_NextFreeFactoryID 
	@Username		= 'SQL'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeFactoryID') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_NextFreeFactoryID;
GO

CREATE PROCEDURE sx_pf_GET_Info_NextFreeFactoryID
	@Username		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FreeFactoryID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(FactoryID AS BIGINT) id 
			FROM planning.tdFactories
		)
		SELECT @FreeFactoryID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeFactoryID IS NOT NULL
		BEGIN
			SET @FreeFactoryID = @FreeFactoryID + 1;
		END
		ELSE
		BEGIN
			SET @FreeFactoryID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT FactoryID FROM planning.tdFactories WHERE FactoryID = @FreeFactoryID)
		BEGIN
			SET @FreeFactoryID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeFactoryID AS NextFreeFactoryID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free ProductID
Returns for numeric ID the next number for Productlines with only string IDs the number 1 

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_NextFreeProductID 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'V'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeProductID') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_NextFreeProductID;
GO

CREATE PROCEDURE sx_pf_GET_Info_NextFreeProductID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FreeProductID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeProductID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		
		IF @ProductLineKey = 0 OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductID AS BIGINT) id 
			FROM planning.tdProducts 
			WHERE ProductLineKey = @ProductLineKey 
		)
		SELECT @FreeProductID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductID IS NOT NULL
		BEGIN
			SET @FreeProductID = @FreeProductID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductID FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @FreeProductID)
		BEGIN
			SET @FreeProductID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductID AS NextFreeProductID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeProductID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeProductID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryService;
GO
/*
GET Operation for the next free ProductlineID
Returns the next numeric ID or the number 1 if other ProductlineIDs are non-numeric

03/2019 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_NextFreeProductLineID 
	@Username		= 'SQL',
	@FactoryID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_NextFreeProductLineID') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_NextFreeProductLineID;
GO

CREATE PROCEDURE sx_pf_GET_Info_NextFreeProductLineID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @FreeProductLineID	NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreePLID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);

		IF	@FactoryID			= N'' 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16 ,10);
			END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		
		IF @FactoryKey = 0
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductLineID AS BIGINT) id 
			FROM planning.tdProductLines 
			WHERE FactoryKey = @FactoryKey 
		)
		SELECT @FreeProductLineID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductLineID IS NOT NULL
		BEGIN
			SET @FreeProductLineID = @FreeProductLineID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductLineID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductLineID FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @FreeProductLineID)
		BEGIN
			SET @FreeProductLineID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductLineID AS NextFreeProductLineID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreePLID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreePLID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryService;
GO

/*
GET Operation for SystemFailureReport

09/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_SystemFailureReport 'SQL','ZT'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_SystemFailureReport') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_SystemFailureReport;
GO

CREATE PROCEDURE sx_pf_GET_Info_SystemFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_SystemFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             SFR.Timestamp								AS LastSystemCheckDate
            ,SFR.FactoryID
			,COALESCE(
						 SFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,SFR.ProductLineID
			,COALESCE(	
						 SFR.ProductLineID + ' ' + dPL.NameShort
						,'All Productlines'
					 )
														AS ProductlineIDName
            ,SFR.ProductID
			,COALESCE(
						 SFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,SFR.FactoryID + ' / ' + SFR.ProductLineID +' / ' + SFR.ProductID AS DataFactoryPath
            ,SFR.TestSource
            ,SFR.TestName
			,SFR.ResultType
            ,SFR.ResultText
            ,SFR.ResultNumeric
		FROM control.tSystemFailureReport SFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM planning.trUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM planning.trUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM planning.trUserRights GROUP BY UserName
						) vUR 

				ON	SFR.FactoryID		= vUR.FactoryID
				AND SFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN planning.tdFactories dF 
				ON	SFR.FactoryID		= dF.FactoryID

			-- for Productline Name
			LEFT JOIN planning.tdProductLines dPL
				ON	SFR.FactoryID		= dPL.FactoryID
				AND SFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN planning.tdProducts dP
				ON	SFR.FactoryID		= dP.FactoryID
				AND SFR.ProductLineID	= dP.ProductLineID
				AND SFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			SFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tSystemFailureReport
							)
			-- called for one or all Factories
			AND (
					SFR.FactoryID = @FactoryID OR @FactoryID = '' OR SFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_SystemFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_SystemFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryService;
GO



/*
GET Operation for UserRight Overview

09/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
@FactoryID, @ProductlineID only Dummy Parameter to call over csv Route
Works only for Clusteradmins


--Test call
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Info_UserRights 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Info_UserRights') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Info_UserRights;
GO

CREATE PROCEDURE sx_pf_GET_Info_UserRights
	@Username NVARCHAR(255),@FactoryID NVARCHAR(255)='',@ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_UserRights;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			BEGIN
				SELECT 
					 'None'												AS UserName
					,'None'												AS Status
					,'None'												AS FactoryID
					,'None'												AS FactoryName
					,'None'												AS ProductLineID
					,'None'												AS ProductlineName
					,'You need Cluster Admin privileges to see this.'	AS [Right]
					,-1													AS RightLevel;
			END;
		ELSE
			BEGIN
				SELECT 
					 rU.UserName
					,rU.Status	
					,dPL.FactoryID
					,dF.NameShort				AS FactoryName
					,dPL.ProductLineID
					,dPL.NameShort				AS ProductlineName
					,ISNULL(vUR.[Right],'Deny') AS UserRight
					,CASE vUR.[Right]
						WHEN 'Read'		THEN 0
						WHEN 'Write'	THEN 1
						ELSE -1
					 END							AS RightLevel
				FROM planning.trUser rU
					-- for full Productline Structure
					LEFT JOIN planning.tdProductLines dPL ON
						 1=1
					-- for FactoryName
					LEFT JOIN planning.tdFactories dF ON
						dPL.FactoryKey = dF.FactoryKey
					-- for positiv Rights
					LEFT JOIN planning.trUserRights vUR	ON
							rU.UserKey			= vUR.UserKey
						AND vUR.FactoryID		= dPL.FactoryID
						AND vUR.ProductLineID	= dPL.ProductLineID;
			END;

			SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_UserRights;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_UserRights;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_UserRights TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Info_UserRights TO pf_PlanningFactoryService;
GO


/*
GET Operation for a List 
Shows the properties of one List
This procedure don't shows internal system lists

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_List 
	@Username	= 'SQL', 
	@ListID		= 'sxStatus';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_List;
GO

CREATE PROCEDURE sx_pf_GET_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL	SET @Username	= N'';
	IF @ListID		IS NULL	SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

		IF @ListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM planning.thLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_List TO pf_PlanningFactoryService;
GO
/*
GET Operation for all GUI Lists in Cluster
Internal system lists are not shown

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call 

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Lists 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Lists') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Lists
GO

CREATE PROCEDURE sx_pf_GET_Lists
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(10)   = N'';
	DECLARE @TransactUsername	NVARCHAR(255)  = N'';
	DECLARE @ProcedureName		NVARCHAR(255)  = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)  = N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT			   = 0;						
	DECLARE @ResultCode			INT			   = 501;					
	DECLARE @TimestampCall		DATETIME	   = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Lists;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT 
			ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM planning.thLists
		ORDER BY ISNULL(TRY_CAST(ListID AS INT), 999999999), ListID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Lists;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Lists;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Lists TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Lists TO pf_PlanningFactoryService;
GO
/*
GET Operation for all List Values
- shows values of all GUI Lists
- shows values of the internal list "sxScales", as this list is need for the other lists

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ListValues 
	@Username	= 'SQL', 
	@ListID		= 'sxEvents'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ListValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ListValues
GO

CREATE PROCEDURE sx_pf_GET_ListValues
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		IF @ListID != 'sxScales'
		BEGIN
			-- STEP 1.2 - Determine Keys
			SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

			IF @ListKey = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

			-- STEP 3. Show public values, always one dummy line, if list exists	
			SELECT 
				ISNULL(hLV.ListValueKey, 0)		AS ListValueKey  --returns the Key as the Listvalue has no ID
				,hL.ListID
				,hL.Datentyp
				,ISNULL(hLV.ValueInt, 0)		AS ValueInt
				,ISNULL(hLV.Scale, 0)			AS Scale
				,ISNULL(hLV.ValueText, N'')		AS ValueText
				,ISNULL(hLV.ValueComment, N'')	AS ValueComment
				,ISNULL(hLV.FormatID, N'')		AS FormatID
				,@Right AS 'Right'
			FROM planning.thLists hL
				LEFT JOIN planning.thListValues hLV	ON hL.ListID = hLV.ListID
			WHERE hL.ListID = @ListID 
			ORDER BY ISNULL(TRY_CAST(hLV.ValueText AS INT), 999999999), hLV.ValueText, hLV.ValueInt;

			SET @EffectedRows = @@ROWCOUNT;
		END;

		-- static definition of sxScales to be loaded for list defintion
		IF @ListID = 'sxScales'
		BEGIN
			SELECT * 
			FROM (
				VALUES	
					(1,'sxScales' ,'INT',1    ,1,'','','','Read')
					,(2,'sxScales','INT',10   ,1,'','','','Read')
					,(3,'sxScales','INT',100  ,1,'','','','Read')
					,(4,'sxScales','INT',1000 ,1,'','','','Read')
					,(5,'sxScales','INT',10000,1,'','','','Read')
			) tmp (ListValueKey, ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, [Right])

			SET @EffectedRows = @@ROWCOUNT;
		END;

		COMMIT TRANSACTION sx_pf_GET_ListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ListValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ListValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ListValues TO pf_PlanningFactoryService;
GO
/*
GET Operation for Prefences

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

- Preferences may exist for the Cluster, for Factories or for Productlines - all in the same table
- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"

--Test call

1. Empty FactoryID or empty ProductLineID => 404 Not Found
2. Username non Cluster Admin => 403 Forbidden
3. User does not have rights to read ProductLine => 401 Unauthorized
5. No exists Factory / Productline > 404 Not Found

DECLARE  @RC					INT
		,@Username				NVARCHAR(255) = 'SQL'
		,@PreferenceUserName	NVARCHAR(255) = 'W10\admin'
		,@ProductLineID			NVARCHAR(255) = 'U'
		,@FactoryID				NVARCHAR(255) = 'ZT'
		,@SettingName			NVARCHAR(255) = 'Demopref'


EXECUTE @RC = sx_pf_GET_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Preference;
GO

CREATE PROCEDURE sx_pf_GET_Preference
	 @Username				NVARCHAR(255)
	,@PreferenceUsername	NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@SettingName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUsername					NVARCHAR(255)= N''
			,@FactoryKey						INT = 0
			,@ProductLineKey					INT = 0
			,@UserExistsFlag					INT = 0
			,@PreferenceIsUserDependendFlag		INT = 0
			,@ProcedureName						NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			,@ParameterString					NVARCHAR(MAX) = N''''
							+ ISNULL(@Username, N'NULL')			+ N''',''' 
							+ ISNULL(@PreferenceUsername, N'NULL')	+ N''',''' 
							+ ISNULL(@ProductLineID, N'NULL')		+ N''',''' 
							+ ISNULL(@FactoryID, N'NULL')			+ N''','''
							+ ISNULL(@SettingName, N'NULL')			+ N''''
			
			,@EffectedRows						INT = 0					
			,@ResultCode 						INT = 501						
			,@TimestampCall						DATETIME = CURRENT_TIMESTAMP
			,@Comment							NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_Preference;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername = dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName		= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @SettingName		= N'' 
			OR @SettingName		= N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.3 - Determine, if Preferences are saved per user

		-- for Clusterpreferences
		IF @FactoryID = N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = N'C06';
			END 
		-- for Factorypreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = N'F06';
			END
		-- for ProductlinePreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = N'P06';
			END;
		
		-- STEP 1.1 - Determine Keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		IF (@FactoryKey = 0 AND @FactoryID <> N'')
			OR (@ProductlineKey = 0 AND @ProductLineID <> N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Change PreferenceUser if Preferences User-independend
		IF @PreferenceIsUserDependendFlag <> 1
		BEGIN 
			SET @PreferenceUsername = N'All';
		END;

		--- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights - TODO: depdendend from preference type
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT 
			 UserName
			,FactoryID
			,ProductlineID
			,SettingName
			,ValueInt
			,ValueText
		FROM planning.trPreferences 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Preference TO pf_PlanningFactoryService;
GO
/*
GET Operation for Product

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Product 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Product
GO

CREATE PROCEDURE sx_pf_GET_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID = dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID = dbo.sx_pf_pProtectID (@ProductLineID);	
		SET @ProductID = dbo.sx_pf_pProtectID (@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
			,@Right AS 'Right'
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
		FROM planning.tdProducts dP LEFT JOIN planning.trUserRights vUR 
			ON  dP.FactoryID = vUR.FactoryID AND
				dP.ProductLineID = vUR.ProductLineID AND
				vUR.UserName = @TransactUsername
		WHERE dP.ProductKey = @ProductKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Product TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Values of ProductDataTable

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Values according HTTP Standard

Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductDataTableValues
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';
PRINT @RC

--should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductDataTableValues 
	@Username		 = 'SQL',
	@FactoryID		 = 'ZT2',
	@ProductLineID	 = 'U',
	@ProductID		 = '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductDataTableValues;
GO

CREATE PROCEDURE sx_pf_GET_ProductDataTableValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID  NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductDataTableValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right	AS 'Right'
		FROM planning.tfValues fV 
			LEFT JOIN planning.tdValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey
		WHERE fV.ProductKey = @ProductKey
		ORDER BY ValueSeriesNo, TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductDataTableValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductDataTableValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
GET Operation for ProductLine
Provides complete information about a Product Line

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductLine 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U' 
	
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductLine;
GO

CREATE PROCEDURE sx_pf_GET_ProductLine
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			ProductLineID 
			,FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,DefaultTemplate
			,GlobalAttributeSource1
			,GlobalAttributeAlias1
			,GlobalAttributeSource2
			,GlobalAttributeAlias2
			,GlobalAttributeSource3
			,GlobalAttributeAlias3
			,GlobalAttributeSource4
			,GlobalAttributeAlias4
			,GlobalAttributeSource5
			,GlobalAttributeAlias5
			,GlobalAttributeSource6
			,GlobalAttributeAlias6
			,GlobalAttributeSource7
			,GlobalAttributeAlias7
			,GlobalAttributeSource8
			,GlobalAttributeAlias8
			,GlobalAttributeSource9
			,GlobalAttributeAlias9
			,GlobalAttributeSource10
			,GlobalAttributeAlias10

			,GlobalAttributeSource11
			,GlobalAttributeAlias11
			,GlobalAttributeSource12
			,GlobalAttributeAlias12
			,GlobalAttributeSource13
			,GlobalAttributeAlias13
			,GlobalAttributeSource14
			,GlobalAttributeAlias14
			,GlobalAttributeSource15
			,GlobalAttributeAlias15
			,GlobalAttributeSource16
			,GlobalAttributeAlias16
			,GlobalAttributeSource17
			,GlobalAttributeAlias17
			,GlobalAttributeSource18
			,GlobalAttributeAlias18
			,GlobalAttributeSource19
			,GlobalAttributeAlias19
			,GlobalAttributeSource20
			,GlobalAttributeAlias20

			,GlobalAttributeSource21
			,GlobalAttributeAlias21
			,GlobalAttributeSource22
			,GlobalAttributeAlias22
			,GlobalAttributeSource23
			,GlobalAttributeAlias23
			,GlobalAttributeSource24
			,GlobalAttributeAlias24
			,GlobalAttributeSource25
			,GlobalAttributeAlias25

			,@Right AS 'Right'
		FROM planning.tdProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLine TO pf_PlanningFactoryService;
GO
/*
GET Operation for determination of all properties of ProductLine

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductLineProperties 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U' 

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLineProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductLineProperties;
GO

CREATE PROCEDURE sx_pf_GET_ProductLineProperties
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProductLines
		WHERE 
			FactoryKey = @FactoryKey 
			AND ProductLineKey = @ProductLineKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a ProductLine Property

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductLineProperty 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@PropertyID		= 'PL01'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductLineProperty;
GO

CREATE PROCEDURE sx_pf_GET_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID

		IF	@PropertyKey		= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
GET Operation for ProductLines 
	A: returns the Productlines for a single Factory
	B: returns all Productlines when Factory '' is requested

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductLines 
	@Username	= 'SQL',
	@FactoryID	= 'ZT'

PRINT @RC

EXEC @RC = sx_pf_GET_ProductLines 
	@Username = 'SQL'

PRINT @RC


*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductLines') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductLines;
GO

CREATE PROCEDURE sx_pf_GET_ProductLines
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'' ;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLines;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys if single Factory is requested
		IF @FactoryID <> ''
			BEGIN
				SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

				IF @FactoryKey = 0	
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Keys don`t exists', 16, 10);
				END;
			END

		-- STEP 3. Show public values	
		;WITH vUR AS (
			SELECT 
				FactoryID, 
				ProductLineID, 
				[Right]
			FROM planning.trUserRights 
			WHERE ProductLineID <> N'' 
				AND Username = @TransactUsername 
				AND	[Right] IN (N'Read', N'Write')
		)
		SELECT  
			 dPL.ProductLineID 
			,dPL.FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,DefaultTemplate
			,GlobalAttributeSource1
			,GlobalAttributeAlias1
			,GlobalAttributeSource2
			,GlobalAttributeAlias2
			,GlobalAttributeSource3
			,GlobalAttributeAlias3
			,GlobalAttributeSource4
			,GlobalAttributeAlias4
			,GlobalAttributeSource5
			,GlobalAttributeAlias5
			,GlobalAttributeSource6
			,GlobalAttributeAlias6
			,GlobalAttributeSource7
			,GlobalAttributeAlias7
			,GlobalAttributeSource8
			,GlobalAttributeAlias8
			,GlobalAttributeSource9
			,GlobalAttributeAlias9
			,GlobalAttributeSource10
			,GlobalAttributeAlias10

			,GlobalAttributeSource11
			,GlobalAttributeAlias11
			,GlobalAttributeSource12
			,GlobalAttributeAlias12
			,GlobalAttributeSource13
			,GlobalAttributeAlias13
			,GlobalAttributeSource14
			,GlobalAttributeAlias14
			,GlobalAttributeSource15
			,GlobalAttributeAlias15
			,GlobalAttributeSource16
			,GlobalAttributeAlias16
			,GlobalAttributeSource17
			,GlobalAttributeAlias17
			,GlobalAttributeSource18
			,GlobalAttributeAlias18
			,GlobalAttributeSource19
			,GlobalAttributeAlias19
			,GlobalAttributeSource20
			,GlobalAttributeAlias20

			,GlobalAttributeSource21
			,GlobalAttributeAlias21
			,GlobalAttributeSource22
			,GlobalAttributeAlias22
			,GlobalAttributeSource23
			,GlobalAttributeAlias23
			,GlobalAttributeSource24
			,GlobalAttributeAlias24
			,GlobalAttributeSource25
			,GlobalAttributeAlias25

			,vUR.[Right]
		FROM planning.tdProductLines dPL INNER JOIN vUR
			ON dPL.FactoryID = vUR.FactoryID AND
			   dPL.ProductLineID = vUR.ProductLineID
		WHERE FactoryKey = @FactoryKey 
			OR @FactoryID = ''
		ORDER BY ISNULL(TRY_CAST(dPL.ProductLineID AS BIGINT), 999999999), dPL.ProductLineID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_ProductLines;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLines;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLines TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductLines TO pf_PlanningFactoryService;
GO

/*
GET Operation for identifying all Properties of Products

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductProperties
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductProperties') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductProperties;
GO

CREATE PROCEDURE sx_pf_GET_ProductProperties
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProducts
		WHERE ProductKey = @ProductKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductProperties TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductProperties TO pf_PlanningFactoryService;
GO
/*
GET Operation for determining a ProductProperty

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductProperty 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1',
	@PropertyID		= 'P1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductProperty;
GO

CREATE PROCEDURE sx_pf_GET_ProductProperty
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''  
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProducts WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @PropertyKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 2.1 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductProperty TO pf_PlanningFactoryService;
GO
/*
Procedure to GET all ProductRanges for an Product

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Samplescript to POST an ProductRange

EXEC dbo.sx_pf_POST_ProductProperty 
	 @Username				= 'SQL'
	,@ProductID				= '1'
	,@ProductLineID			= 'U'
	,@FactoryID				= 'ZT'
	,@PropertyID			= 'PDTRange_1'				-- fixed Name PDTRange_1...N
	,@PropertyName			= 'PDTRangeProperty'		-- fixed Name PDTRangeProperty
	,@CommentUser			= ''
	,@CommentDev			= ''
	,@Unit					= ''
	,@ValueText				= 'K1:20170115:K2:20171215'	-- Range in DF Coordinates, separated with :
	,@ValueInt				= '1'						-- Protected 1 = TRUE
	,@Scale					= '1'
	,@IsROSystemproperty	= '0'
	,@FormatID				= 'rngRed'					-- Name of FormatID or '<#NV>' to set only Protection 
														-- the Format Property Border* will be ignored on Cell Level

EXEC dbo.sx_pf_POST_ProductProperty 
	 @Username				= 'SQL'
	,@ProductID				= '1'
	,@ProductLineID			= 'U'
	,@FactoryID				= 'ZT'
	,@PropertyID			= 'PDTRange_2'				
	,@PropertyName			= 'PDTRangeProperty'		
	,@CommentUser			= ''
	,@CommentDev			= ''
	,@Unit					= ''
	,@ValueText				= 'K3:20180615:K3:20190115'	
	,@ValueInt				= '1'						
	,@Scale					= '1'
	,@IsROSystemproperty	= '0'
	,@FormatID				= 'rngRed'					
														
--Testcall

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ProductRanges 
	@Username		= 'SQL',
	@FactoryID		= 'ZT', 
	@ProductLineID	= 'U', 
	@ProductID		= '1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ProductRanges') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ProductRanges;
GO

CREATE PROCEDURE sx_pf_GET_ProductRanges
	 @Username		NVARCHAR(255)
	,@FactoryID		NVARCHAR(255)
	,@ProductLineID	NVARCHAR(255)
	,@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		
			+ ISNULL(@Username,			N'NULL') + N''','''	
			+ ISNULL(@FactoryID,		N'NULL') + N''','''	
			+ ISNULL(@ProductLineID,	N'NULL') + N''','''	
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username		= N'';
	IF @FactoryID				IS NULL SET @FactoryID		= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID	= N'';
	IF @ProductID				IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_ProductRanges;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);	

		IF	@FactoryID			= N'' 
			OR  @ProductlineID	= N'' 
			OR  @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine Keys
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductlineID = @ProductlineID AND ProductID = @ProductID;
		
		IF @ProductKey = 0 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductlineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 PropertyID
			,ValueText	AS DFRange
			,ValueInt	AS ProtectedFlag
			,FormatID
			,@Right		AS 'Right'
		FROM planning.tgProducts
		WHERE 
			    ProductKey = @ProductKey
			AND PropertyID LIKE 'PDTRange_%'; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_ProductRanges;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_ProductRanges;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductRanges TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ProductRanges TO pf_PlanningFactoryService;
GO

/*
GET Operation for all Product of ProductLine

11/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

-- should return 200 and the Products of one Productline
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U'

PRINT @RC

-- should return 200 and the Products of one Factory
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= ''

PRINT @RC

-- should return 200 and the Products of all Factories
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= '',
	@ProductLineID	= ''

PRINT @RC

-- should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Products 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U1'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Products') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Products;
GO

CREATE PROCEDURE sx_pf_GET_Products
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Products;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys - maybe 0 if ProductlineID or FactoryID is empty
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		-- STEP 3. Show public values	
		SELECT 
			 dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,dP.TimeType
			,dP.NameShort
			,dP.NameLong
			,dP.CommentUser
			,dP.CommentDev
			,dP.ResponsiblePerson
			,dP.ImageName
			,dP.[Status]
			,dP.Template
			,dP.TemplateVersion
			,dP.GlobalAttribute1
			,dP.GlobalAttribute2
			,dP.GlobalAttribute3
			,dP.GlobalAttribute4
			,dP.GlobalAttribute5
			,dP.GlobalAttribute6
			,dP.GlobalAttribute7
			,dP.GlobalAttribute8
			,dP.GlobalAttribute9
			,dP.GlobalAttribute10
			,dP.GlobalAttribute11
			,dP.GlobalAttribute12
			,dP.GlobalAttribute13
			,dP.GlobalAttribute14
			,dP.GlobalAttribute15
			,dP.GlobalAttribute16
			,dP.GlobalAttribute17
			,dP.GlobalAttribute18
			,dP.GlobalAttribute19
			,dP.GlobalAttribute20
			,dP.GlobalAttribute21
			,dP.GlobalAttribute22
			,dP.GlobalAttribute23
			,dP.GlobalAttribute24
			,dP.GlobalAttribute25
			,vUR.[Right]				AS [Right]
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
			,(SELECT COUNT(FileID) FROM planning.tFiles tF WHERE dP.FactoryID = tF.FactoryID AND dP.ProductlineID = tF.ProductlineID AND dP.ProductID = tF.ProductID) AS FileCount
		FROM planning.tdProducts dP 
			LEFT JOIN planning.trUserRights vUR 
				ON		dP.FactoryID = vUR.FactoryID 
					AND	dP.ProductLineID = vUR.ProductLineID 
					AND	vUR.UserName = @TransactUsername
		WHERE 
				(
					 dP.ProductLineKey = @ProductLineKey 
				OR  (dP.FactoryID = @FactoryID AND @ProductLineID = '')
				OR  (@FactoryID = '')
				)

				AND vUR.[Right] IS NOT NULL

		ORDER BY 
			 dP.FactoryID
			,dP.ProductLineID	
			,ISNULL(TRY_CAST(dP.ProductID AS INT), 999999999), dP.ProductID;

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Products;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Products;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Products TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Products TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Statements of Products

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Statements 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Statements') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Statements;
GO

CREATE PROCEDURE sx_pf_GET_Statements
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Statements;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ActionType
			,[Statement]
			,UserName
			,PCName
			,ProcessorCode
			,IPAddresses
			,[Timestamp]
			,IsDeleted
			,IsResolved
			,@Right AS 'Right'
		FROM planning.tfStatements
		WHERE ProductKey = @ProductKey 
		ORDER BY [Timestamp] DESC;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Statements;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Statements;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Statements TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Statements TO pf_PlanningFactoryService;
GO
/*
GET Operation for all TimeIDs of ValueSeries

04/2018 for DataFactory 4.5
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_TimeIDs 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1';

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_TimeIDs') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_TimeIDs;
GO

CREATE PROCEDURE sx_pf_GET_TimeIDs
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_TimeIDs;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;		
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		FROM planning.tdTime
		WHERE ProductKey = @ProductKey 
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_TimeIDs;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_TimeIDs;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_TimeIDs TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_TimeIDs TO pf_PlanningFactoryService;
GO
/*
GET Operation for a User

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_User 
	@Username			= 'SQL', 
	@RequestedUserName	= 'W10\admin'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_User;
GO

CREATE PROCEDURE sx_pf_GET_User
	@Username			NVARCHAR(255),
	@RequestedUserName	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_User;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		IF @RequestedUserName = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - don't deliver the member of PlannigFactoryService
		IF ISNULL(IS_ROLEMEMBER('pf_PlanningFactoryService', @RequestedUsername), 2) = 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Member of PlannigFactoryService', 16, 10);
		END

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @UserKey = UserKey FROM planning.trUser WHERE UserName = @RequestedUserName;
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right = N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 UserKey
			,UserName
			,PersonSurname
			,PersonFirstName
			,Email
			,LDAPIP
			,[Status]
			,@Right AS 'Right'
		FROM planning.trUser
		WHERE UserKey = @UserKey; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_User TO pf_PlanningFactoryService;
GO
/*
GET Operation for User rights

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_UserRight 
	@Username = 'SQL', 
	@RequestedUserName = 'W10\admin'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_UserRight') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_UserRight;
GO

CREATE PROCEDURE sx_pf_GET_UserRight
	@Username			NVARCHAR(255),
	@RequstedUserName	NVARCHAR(255) = NULL, -- at the Moment written wrong, due to API compatibility !!!
	@RequestedUserName  NVARCHAR(255) = NULL 
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@RequstedUserName,		N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)  = N'';				

	SET @RequestedUserName = ISNULL(@RequestedUserName, @RequstedUserName);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY	
		BEGIN TRANSACTION GET_UserRight;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		-- Check on empty name or SystemUser
		IF @RequestedUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameter <RequestedUserName>', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @UserKey = UserKey FROM planning.trUser WHERE UserName = @RequestedUserName
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right		= N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- Helper Table with Full Stucture - to Output Deny Rights too   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
			DROP TABLE #FullStructure;

		CREATE TABLE #FullStructure
		(
			RowKey			BIGINT IDENTITY (1,1)								NOT NULL
			,[Level]		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey		BIGINT												NOT NULL
			,ProductLineKey BIGINT												NOT NULL
			,FactoryID		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductlineID	NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
		);
		-- C Level
		INSERT INTO #FullStructure
		SELECT 
			'Cluster'	AS 'Level'
			,0			AS FactoryKey
			,0			AS ProductLineKey
			,''			AS FactoryID
			,''			AS ProductLineID;
			 
		-- F Level
		INSERT INTO #FullStructure
		SELECT
			'Factory'		AS 'Level'
			,dF.FactoryKey
			,0				AS ProductLineKey
			,dF.FactoryID
			,''				AS ProductLineID
		FROM planning.tdFactories dF;
				
		-- PL Level
		INSERT INTO #FullStructure
		SELECT
			'Productline'	AS 'Level'
			,dF.FactoryKey
			,dPL.ProductLineKey
			,dF.FactoryID
			,dPL.ProductLineID
		FROM planning.tdProductLines dPL 
			LEFT JOIN planning.tdFactories dF
				ON dPL.FactoryKey = dF.FactoryKey;
		
		-- STEP 3. Show public values	
		IF @Right = N'Read' --User is allowed to see of himself only positive rights
		BEGIN

			SELECT 
				 vUR.Username				AS UserName
				,vUR.FactoryID				AS FactoryID
				,COALESCE(dF.NameShort,'')	AS FactoryName
				,vUR.ProductLineID			AS ProductLineID
				,COALESCE(dPL.NameShort,'')	AS ProductLineName
				,vUR.[Right]				AS [Right]
				,vUR.ReadCommentMandatory	AS ReadCommentMandatory
				,vUR.WriteCommentMandatory	AS WriteCommentMandatory
			FROM planning.trUserRights vUR
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON vUR.FactoryID = dF.FactoryID
				-- for ProductlineName
				LEFT JOIN planning.tdProductLines dPL
					ON		vUR.FactoryID = dPL.FactoryID 
						AND	vUR.ProductLineID = dPL.ProductLineID
			WHERE vUR.UserKey = @UserKey 
			ORDER BY 
				ISNULL(TRY_CAST(vUR.FactoryID AS INT), 999999999)
				,vUR.FactoryID
				,ISNULL(TRY_CAST(vUR.ProductLineID AS INT), 999999999)
				,vUR.ProductLineID;
		END
		ELSE
		IF @Right = N'Write' -- Cluster admin can see also negative rights (really don`t exist)
		BEGIN
				
			SELECT
				 @RequestedUsername						AS UserName
				,fs.FactoryID							AS FactoryID
				,COALESCE(dF.NameShort,'')				AS FactoryName
				,fs.ProductlineID						AS ProductLineID
				,COALESCE(dPL.NameShort,'')				AS ProductLineName
				,COALESCE(vUR.[Right],'Deny')			AS 'Right'
                ,COALESCE(vUR.ReadCommentMandatory,0)	AS ReadCommentMandatory
				,COALESCE(vUR.WriteCommentMandatory,0)	AS WriteCommentMandatory
			FROM #FullStructure fs
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON fs.FactoryID = dF.FactoryID
				-- for ProductlineName
				LEFT JOIN planning.tdProductLines dPL
					ON		fs.FactoryID = dPL.FactoryID 
						AND	fs.ProductLineID = dPL.ProductLineID
				-- positive Rechte
				LEFT JOIN
					(
						SELECT 
							vUR.FactoryID
							,vUR.ProductLineID
							,vUR.[Right]
							,vUR.ReadCommentMandatory
							,vUR.WriteCommentMandatory
						FROM planning.trUserRights vUR
						WHERE vUR.UserKey = @UserKey
					) vUR
					ON		fs.FactoryID = vUR.FactoryID
						AND fs.ProductlineID = vUR.ProductlineID
			ORDER BY  
				ISNULL(TRY_CAST(fs.FactoryID AS INT), 999999999)
				,fs.FactoryID
				,ISNULL(TRY_CAST(fs.ProductLineID AS INT), 999999999)
				,fs.ProductLineID;
		END;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_UserRight;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_UserRight;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_UserRight TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_UserRight TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Users

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Users 'SQL'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Users') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Users;
GO

CREATE PROCEDURE sx_pf_GET_Users
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Users;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 UserKey
			,UserName
			,PersonSurname
			,PersonFirstName
			,Email
			,LDAPIP
			,[Status]
			,N'Write' AS 'Right'
		FROM planning.trUser
		WHERE Username NOT IN (
			SELECT m.[name] COLLATE DATABASE_DEFAULT
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m ON rm.member_principal_id = m.principal_id
			WHERE r.[name] = N'pf_PlanningFactoryService')
		ORDER BY ISNULL(TRY_CAST(UserName AS INT), 999999999), UserName
		
		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Users;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Users;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Users TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Users TO pf_PlanningFactoryService;
GO
/*
GET Operation for a Value

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Value 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1',
	@TimeID			=  20180115

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Value;
GO

CREATE PROCEDURE sx_pf_GET_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @ValueKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')+ N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL')+ N''',''' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(10)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @FactoryID		IS NULL SET @FactoryID		 = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	 = N'';
	IF @ProductID		IS NULL SET @ProductID		 = N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Value;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSerieID	 = dbo.sx_pf_pProtectID		 (@ValueSerieID);
		
		IF	@FactoryID				= N'' 
			OR @ProductLineID		= N'' 
			OR @ProductID			= N'' 
			OR @ValueSerieID		= N'' 
			OR ISNULL(@TimeID, 0)	<= 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		SELECT @ValueKey = ValueKey FROM planning.tfValues WHERE ValueSeriesKey = @ValueSerieKey AND TimeID = @TimeID;

		IF @ProductKey = 0 OR @ProductLineKey = 0 OR @FactoryKey = 0 OR @ValueSerieKey = 0 OR @ValueKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2.1 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM planning.tfValues
		WHERE ValueKey = @ValueKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Value TO pf_PlanningFactoryService;
GO
/*
GET Operation for all Values of ValueSeries

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Values
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1'

PRINT @RC

--should return 404
DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_Values 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1test'

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_Values') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_Values;
GO

CREATE PROCEDURE sx_pf_GET_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Values;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 
			OR @ValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM planning.tfValues
		WHERE ValueSeriesKey = @ValueSerieKey
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Values TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Values TO pf_PlanningFactoryService;
GO
/*
GET Operation for all ValueSerie

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ValueSerie 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1',
	@ValueSerieID	= 'K1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ValueSerie;
GO

CREATE PROCEDURE sx_pf_GET_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSerie;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N'' 
			OR @ValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM planning.tdValueSeries
		WHERE ValueSeriesKey = @ValueSerieKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ValueSerie TO pf_PlanningFactoryService;
GO
/*
GET Operation for all ValueSeries of Product

04/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

--Test call

DECLARE @RC NVARCHAR(255)
EXEC @RC = sx_pf_GET_ValueSeries 
	@Username		= 'SQL',
	@FactoryID		= 'ZT',
	@ProductLineID	= 'U',
	@ProductID		= '1'
PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_GET_ValueSeries') AND type in (N'P', N'PC'))
	DROP PROCEDURE sx_pf_GET_ValueSeries;
GO

CREATE  PROCEDURE sx_pf_GET_ValueSeries
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10);
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSeries;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM planning.tdValueSeries
		WHERE ProductKey = @ProductKey 
		ORDER BY ValueSeriesNo;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSeries;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSeries;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::sx_pf_GET_ValueSeries TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_ValueSeries TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/08
Import all values from load.tdFactories with specific load-procedure as source
	EXEC import.spPOST_dFactories 'SQL','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dFactories') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dFactories
GO

CREATE PROCEDURE import.spPOST_dFactories
				(		 @Username AS NVARCHAR(255)
						,@FactoryID AS NVARCHAR(255)
						,@Source AS NVARCHAR(255)				
				)
AS

BEGIN
	SET NOCOUNT ON;

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode				INT				= 501;
	DECLARE @EffectedRows			INT				= 0;					
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username	, N'NULL')	+ N''',''' 
													+ ISNULL(@FactoryID	, N'NULL')	+ N''','''
													+ ISNULL(@Source	, N'NULL')	+ N'''';
	DECLARE @Comment				NVARCHAR(2000)	 = N'';					

	DECLARE @NameShort				NVARCHAR(255)
	DECLARE @NameLong				NVARCHAR(255)
	DECLARE @CommentUser			NVARCHAR(MAX)
	DECLARE @CommentDev				NVARCHAR(255)
	DECLARE @ResponsiblePerson		NVARCHAR(255)
	DECLARE @ImageName				NVARCHAR(255)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPOARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(		 FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL 
						,NameShort			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,NameLong			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentUser		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentDev			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ResponsiblePerson	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ImageName			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL	
				)
	INSERT INTO #tmp
		SELECT tdF.FactoryID,
			   tdF.NameShort,
			   tdF.NameLong,
			   tdF.CommentUser,
			   tdF.CommentDev,
			   tdF.ResponsiblePerson,
			   tdF.ImageName 
		FROM   load.tdFactories tdF
		WHERE	(@FactoryID = '' OR tdF.FactoryID = @FactoryID)
		AND		(@Source = '' OR tdF.Source = @Source) 

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT 
			 FactoryID			
			,NameShort			
			,NameLong			
			,CommentUser		
			,CommentDev			
			,ResponsiblePerson	
			,ImageName			
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Factory
				 @Username			--@Username
				,@FactoryID			--@FactoryID
				,@NameShort			--@NameShort
				,@NameLong			--@NameLong
				,@CommentUser		--@CommentUser
				,@CommentDev		--@CommentDev
				,@ResponsiblePerson	--@ResponsiblePerson
				,@ImageName			--@ImageName	

		Print @ResultCode
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor
	
	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;
END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dFactories] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dFactories] TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import all values from load.tdProductlines with specific load-procedure as source

Testcall
EXEC import.spPOST_dProductlines 'SQL','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dProductlines') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dProductlines
GO

CREATE PROCEDURE import.spPOST_dProductlines
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductLineID	AS NVARCHAR(255),
						@Source			AS NVARCHAR(255)		  
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode						INT				= 501;
	DECLARE @EffectedRows					INT				= 0;					
	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @ParameterString				NVARCHAR(MAX)	= N''''
															+ ISNULL(@Username		, N'NULL')	 + N''',''' 
															+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
															+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
															+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment						NVARCHAR(2000)	= N'';						
	DECLARE @NameShort						NVARCHAR(255)
	DECLARE @NameLong						NVARCHAR(255)
	DECLARE @CommentUser					NVARCHAR(MAX)
	DECLARE @CommentDev						NVARCHAR(MAX)
	DECLARE @ResponsiblePerson				NVARCHAR(255)
	DECLARE @ImageName						NVARCHAR(255)
	DECLARE @DefaultTemplate				NVARCHAR(255)
	DECLARE @GlobalAttributeSource1			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias1 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource2			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias2 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource3			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias3 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource4			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias4 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource5			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias5 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource6			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias6 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource7			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias7 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource8			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias8 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource9			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias9 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource10		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias10			NVARCHAR(255)
	DECLARE @GlobalAttributeSource11		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias11			NVARCHAR(255)
	DECLARE @GlobalAttributeSource12		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias12			NVARCHAR(255)
	DECLARE @GlobalAttributeSource13		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias13			NVARCHAR(255)
	DECLARE @GlobalAttributeSource14		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias14			NVARCHAR(255)
	DECLARE @GlobalAttributeSource15		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias15			NVARCHAR(255)
	DECLARE @GlobalAttributeSource16		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias16			NVARCHAR(255)
	DECLARE @GlobalAttributeSource17		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias17			NVARCHAR(255)
	DECLARE @GlobalAttributeSource18		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias18			NVARCHAR(255)
	DECLARE @GlobalAttributeSource19		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias19			NVARCHAR(255)
	DECLARE @GlobalAttributeSource20		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias20			NVARCHAR(255)
	DECLARE @GlobalAttributeSource21		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias21			NVARCHAR(255)
	DECLARE @GlobalAttributeSource22		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias22			NVARCHAR(255)
	DECLARE @GlobalAttributeSource23		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias23			NVARCHAR(255)
	DECLARE @GlobalAttributeSource24		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias24			NVARCHAR(255)
	DECLARE @GlobalAttributeSource25		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias25			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(	ProductLineID			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,FactoryID				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameShort				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameLong				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentUser				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentDev				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ResponsiblePerson		NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ImageName				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,DefaultTemplate			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource1  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias1 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource2	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias2 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource3	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias3 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource4	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias4 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource5	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias5 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource6	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias6 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource7	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias7 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource8	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias8 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource9	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias9 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource10 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias10  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource11 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias11  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource12 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias12  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource13 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias13  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource14 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias14  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource15 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias15  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource16 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias16  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource17 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias17  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource18 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias18  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource19 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias19  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource20 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias20  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource21 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias21  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource22 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias22  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource23 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias23  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource24 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias24  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource25 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias25  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   )

	INSERT INTO #tmp
		SELECT   tdPL.ProductLineID			
				,tdPL.FactoryID				
				,tdPL.NameShort				
				,tdPL.NameLong				
				,tdPL.CommentUser			
				,tdPL.CommentDev			
				,tdPL.ResponsiblePerson		
				,tdPL.ImageName				
				,tdPL.DefaultTemplate		
				,tdPL.GlobalAttributeSource1
				,tdPL.GlobalAttributeAlias1 
				,tdPL.GlobalAttributeSource2
				,tdPL.GlobalAttributeAlias2 
				,tdPL.GlobalAttributeSource3
				,tdPL.GlobalAttributeAlias3 
				,tdPL.GlobalAttributeSource4
				,tdPL.GlobalAttributeAlias4 
				,tdPL.GlobalAttributeSource5
				,tdPL.GlobalAttributeAlias5 
				,tdPL.GlobalAttributeSource6
				,tdPL.GlobalAttributeAlias6 
				,tdPL.GlobalAttributeSource7
				,tdPL.GlobalAttributeAlias7 
				,tdPL.GlobalAttributeSource8
				,tdPL.GlobalAttributeAlias8 
				,tdPL.GlobalAttributeSource9
				,tdPL.GlobalAttributeAlias9 
				,tdPL.GlobalAttributeSource10
				,tdPL.GlobalAttributeAlias10 
				,tdPL.GlobalAttributeSource11
				,tdPL.GlobalAttributeAlias11 
				,tdPL.GlobalAttributeSource12
				,tdPL.GlobalAttributeAlias12 
				,tdPL.GlobalAttributeSource13
				,tdPL.GlobalAttributeAlias13 
				,tdPL.GlobalAttributeSource14
				,tdPL.GlobalAttributeAlias14 
				,tdPL.GlobalAttributeSource15
				,tdPL.GlobalAttributeAlias15 
				,tdPL.GlobalAttributeSource16
				,tdPL.GlobalAttributeAlias16 
				,tdPL.GlobalAttributeSource17
				,tdPL.GlobalAttributeAlias17 
				,tdPL.GlobalAttributeSource18
				,tdPL.GlobalAttributeAlias18 
				,tdPL.GlobalAttributeSource19
				,tdPL.GlobalAttributeAlias19 
				,tdPL.GlobalAttributeSource20
				,tdPL.GlobalAttributeAlias20 
				,tdPL.GlobalAttributeSource21
				,tdPL.GlobalAttributeAlias21 
				,tdPL.GlobalAttributeSource22
				,tdPL.GlobalAttributeAlias22 
				,tdPL.GlobalAttributeSource23
				,tdPL.GlobalAttributeAlias23 
				,tdPL.GlobalAttributeSource24
				,tdPL.GlobalAttributeAlias24 
				,tdPL.GlobalAttributeSource25
				,tdPL.GlobalAttributeAlias25  
		FROM	load.tdProductLines tdPL
		WHERE	(@FactoryID = ''		OR tdPL.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdPL.ProductLineID = @ProductLineID)
		 AND    (@Source = ''			OR tdPL.Source = @Source)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ProductLine
				 @Username
				,@ProductLineID				
				,@FactoryID			
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ResponsiblePerson
				,@ImageName
				,@DefaultTemplate
				,@GlobalAttributeSource1  
				,@GlobalAttributeAlias1 
				,@GlobalAttributeSource2
				,@GlobalAttributeAlias2 
				,@GlobalAttributeSource3
				,@GlobalAttributeAlias3 
				,@GlobalAttributeSource4
				,@GlobalAttributeAlias4 
				,@GlobalAttributeSource5
				,@GlobalAttributeAlias5 
				,@GlobalAttributeSource6
				,@GlobalAttributeAlias6 
				,@GlobalAttributeSource7
				,@GlobalAttributeAlias7 
				,@GlobalAttributeSource8
				,@GlobalAttributeAlias8 
				,@GlobalAttributeSource9
				,@GlobalAttributeAlias9 
				,@GlobalAttributeSource10 
				,@GlobalAttributeAlias10  
				,@GlobalAttributeSource11 
				,@GlobalAttributeAlias11  
				,@GlobalAttributeSource12 
				,@GlobalAttributeAlias12  
				,@GlobalAttributeSource13 
				,@GlobalAttributeAlias13  
				,@GlobalAttributeSource14 
				,@GlobalAttributeAlias14  
				,@GlobalAttributeSource15 
				,@GlobalAttributeAlias15  
				,@GlobalAttributeSource16 
				,@GlobalAttributeAlias16  
				,@GlobalAttributeSource17 
				,@GlobalAttributeAlias17  
				,@GlobalAttributeSource18 
				,@GlobalAttributeAlias18  
				,@GlobalAttributeSource19 
				,@GlobalAttributeAlias19  
				,@GlobalAttributeSource20 
				,@GlobalAttributeAlias20  
				,@GlobalAttributeSource21 
				,@GlobalAttributeAlias21  
				,@GlobalAttributeSource22 
				,@GlobalAttributeAlias22  
				,@GlobalAttributeSource23 
				,@GlobalAttributeAlias23  
				,@GlobalAttributeSource24 
				,@GlobalAttributeAlias24  
				,@GlobalAttributeSource25 
				,@GlobalAttributeAlias25 

		PRINT @ResultCode
	FETCH MyCursor INTO  @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProductlines TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_dProductlines TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/08

Import all values from load.tdProducts with specific load-procedure as source
EXEC import.spPOST_dProducts '','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dProducts') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dProducts
GO

CREATE PROCEDURE import.spPOST_dProducts
				(		 @Username		AS NVARCHAR(255)
						,@FactoryID		AS NVARCHAR(255)
						,@ProductLineID	AS NVARCHAR(255)
						,@ProductID		AS NVARCHAR(255)			
						,@Source		AS NVARCHAR(255)		
				)		  
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @TimeType					NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(MAX)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ResponsiblePerson			NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @Status						NVARCHAR(255)
	DECLARE @Template					NVARCHAR(255)
	DECLARE @TemplateVersion			NVARCHAR(255)
	DECLARE @GA1 						NVARCHAR(255)
	DECLARE @GA2 						NVARCHAR(255)
	DECLARE @GA3 						NVARCHAR(255)
	DECLARE @GA4 						NVARCHAR(255)
	DECLARE @GA5 						NVARCHAR(255)
	DECLARE @GA6 						NVARCHAR(255)
	DECLARE @GA7 						NVARCHAR(255)
	DECLARE @GA8 						NVARCHAR(255)
	DECLARE @GA9 						NVARCHAR(255)
	DECLARE @GA10						NVARCHAR(255)
	DECLARE @GA11						NVARCHAR(255)
	DECLARE @GA12						NVARCHAR(255)
	DECLARE @GA13						NVARCHAR(255)
	DECLARE @GA14						NVARCHAR(255)
	DECLARE @GA15						NVARCHAR(255)
	DECLARE @GA16						NVARCHAR(255)
	DECLARE @GA17						NVARCHAR(255)
	DECLARE @GA18						NVARCHAR(255)
	DECLARE @GA19						NVARCHAR(255)
	DECLARE @GA20						NVARCHAR(255)
	DECLARE @GA21						NVARCHAR(255)
	DECLARE @GA22						NVARCHAR(255)
	DECLARE @GA23						NVARCHAR(255)
	DECLARE @GA24						NVARCHAR(255)
	DECLARE @GA25						NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TimeType				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ResponsiblePerson		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Status					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Template				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TemplateVersion			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA1 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA2 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA3 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA4 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA5 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA6 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA7 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA8 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA9 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA10					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA11					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA12					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA13					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA14					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA15					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA16					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA17					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA18					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA19					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA20					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA21					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA22					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA23					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA24					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA25					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL		
			 )

	INSERT INTO #tmp
		SELECT   tdP.ProductID
				,tdP.ProductLineID			
				,tdP.FactoryID
				,tdP.TimeType				
				,tdP.NameShort				
				,tdP.NameLong				
				,tdP.CommentUser			
				,tdP.CommentDev			
				,tdP.ResponsiblePerson		
				,tdP.ImageName
				,tdP.Status				
				,tdP.Template
				,tdP.TemplateVersion		
				,tdP.GlobalAttribute1 
				,tdP.GlobalAttribute2 
				,tdP.GlobalAttribute3 
				,tdP.GlobalAttribute4 
				,tdP.GlobalAttribute5 
				,tdP.GlobalAttribute6 
				,tdP.GlobalAttribute7 
				,tdP.GlobalAttribute8 
				,tdP.GlobalAttribute9 
				,tdP.GlobalAttribute10
				,tdP.GlobalAttribute11
				,tdP.GlobalAttribute12
				,tdP.GlobalAttribute13
				,tdP.GlobalAttribute14
				,tdP.GlobalAttribute15
				,tdP.GlobalAttribute16
				,tdP.GlobalAttribute17
				,tdP.GlobalAttribute18
				,tdP.GlobalAttribute19
				,tdP.GlobalAttribute20
				,tdP.GlobalAttribute21
				,tdP.GlobalAttribute22
				,tdP.GlobalAttribute23
				,tdP.GlobalAttribute24
				,tdP.GlobalAttribute25
		FROM	load.tdProducts tdP
		WHERE	(@FactoryID = ''		OR tdP.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdP.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tdP.ProductID = @ProductID)
		 AND    (@Source = ''			OR tdP.Source = @Source)
	 
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Product
				 @Username
				,@ProductID
				,@ProductLineID				
				,@FactoryID			
				,@TimeType			
				,@NameShort			
				,@NameLong			
				,@CommentUser		
				,@CommentDev			
				,@ResponsiblePerson	
				,@ImageName			
				,@Status				
				,@Template			
				,@TemplateVersion	
				,@GA1 				
				,@GA2 				
				,@GA3 				
				,@GA4 				
				,@GA5 				
				,@GA6 				
				,@GA7 				
				,@GA8 				
				,@GA9 				
				,@GA10				
				,@GA11				
				,@GA12				
				,@GA13				
				,@GA14				
				,@GA15				
				,@GA16				
				,@GA17				
				,@GA18				
				,@GA19				
				,@GA20				
				,@GA21				
				,@GA22				
				,@GA23				
				,@GA24				
				,@GA25	
					
		Print @ResultCode
	FETCH MyCursor INTO  @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import all values from load.tfValues with specific load-procedure as source
	 EXEC [import].[spPOST_dTime] 'SQL','','','',0,''
*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[import].[spPOST_dTime]') AND type in (N'P', N'PC'))
DROP PROCEDURE [import].[spPOST_dTime]
GO

CREATE PROCEDURE [import].[spPOST_dTime]
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductlineID	AS NVARCHAR(255),
						@ProductID		AS NVARCHAR(255),
						@TimeID			AS INT,
						@Source			AS NVARCHAR(255)		)
		   
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username							, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
														+ ISNULL(@ProductID							, N'NULL')	 + N''','''
														+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
														+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';	
	DECLARE @ValueFormula				NVARCHAR(255)
	DECLARE @ValueINT					INT
	DECLARE @ValueText					NVARCHAR(MAX)
	DECLARE @ValueComment				NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(	 ProductID		NVARCHAR(255)
			,ProductLineID	NVARCHAR(255)
			,FactoryID		NVARCHAR(255)
			,TimeID			INT
     	)

	INSERT INTO #tmp
		SELECT   tt.ProductID		
				,tt.ProductLineID	
				,tt.FactoryID		
				,tt.TimeID					
		FROM	load.tdTime tt
		WHERE	(@FactoryID = '' OR tt.FactoryID = @FactoryID)
		 AND    (@ProductLineID = '' OR tt.ProductLineID = @ProductLineID)
		 AND    (@ProductID = '' OR tt.ProductID = @ProductID)
		 AND    (@TimeID = 0 OR tt.TimeID = @TimeID)
		 AND    (@Source = '' OR tt.Source = @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeID
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = [dbo].[sx_pf_POST_TimeID]
		                                               
		
				 @Username
				,@ProductID		
				,@ProductLineID	
				,@FactoryID		
				,@TimeID	
				,''

										
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeID
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dTime] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dTime] TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import ValueSeries from load.tdValueSeries with specific load-procedure as source
	EXEC import.spPOST_dValueSeries 'SQL','','','','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_dValueSeries') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_dValueSeries
GO

CREATE PROCEDURE import.spPOST_dValueSeries
				(	 	 @Username			AS NVARCHAR(255)
						,@FactoryID			AS NVARCHAR(255)
						,@ProductlineID		AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID		AS NVARCHAR(255)
						,@Source			AS NVARCHAR(255)		
				)		   
AS

BEGIN	
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID	, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @RequestedValueSeriesNo		NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(255)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @IsNumeric					NVARCHAR(255)
	DECLARE @VisibilityLevel			NVARCHAR(255)
	DECLARE @ValueSource				NVARCHAR(255)
	DECLARE @ValueListID				NVARCHAR(255)
	DECLARE @ValueFormatID				NVARCHAR(255)
	DECLARE @Unit						NVARCHAR(255)
	DECLARE @Scale						NVARCHAR(255)
	DECLARE @Effect						NVARCHAR(255)
	DECLARE @EffectParameter			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(		ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSeriesID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,RequestedValueSeriesNo	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,IsNumeric				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,VisibilityLevel			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSource				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueListID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueFormatID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Unit					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Scale					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Effect					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,EffectParameter			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			 )

	INSERT INTO #tmp
		SELECT   tdVS.ProductID				
				,tdVS.ProductLineID			
				,tdVS.FactoryID				
				,tdVS.ValueSeriesID			
				,tdVS.ValueSeriesNo	
				,tdVS.NameShort				
				,tdVS.NameLong				
				,tdVS.CommentUser				
				,tdVS.CommentDev				
				,tdVS.ImageName				
				,tdVS.IsNumeric				
				,tdVS.VisibilityLevel			
				,tdVS.ValueSource				
				,tdVS.ValueListID				
				,tdVS.ValueFormatID			
				,tdVS.Unit					
				,tdVS.Scale					
				,tdVS.Effect					
				,tdVS.EffectParameter	
		FROM	load.tdValueSeries tdVS
		WHERE	(@FactoryID		= '' OR tdVS.FactoryID		= @FactoryID)
		 AND    (@ProductLineID = '' OR tdVS.ProductLineID	= @ProductLineID)
		 AND    (@ProductID		= '' OR tdVS.ProductID		= @ProductID)
		 AND    (@ValueSeriesID = '' OR tdVS.ValueSeriesID	= @ValueSeriesID)
		 AND	(@Source		= '' OR tdVS.Source			= @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter

	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ValueSerie
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter				
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;
-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dValueSeries] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_dValueSeries] TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/07
Import all values from load.tfValues with specific load-procedure as source
	 EXEC import.spPOST_fValues 'SQL','','','','',0,''
*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_fValues') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_fValues
GO

CREATE PROCEDURE import.spPOST_fValues
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductlineID	AS NVARCHAR(255),
						@ProductID		AS NVARCHAR(255),
						@ValueSeriesID	AS NVARCHAR(255),
						@TimeID			AS INT,
						@Source			AS NVARCHAR(255)		
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username							, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
														+ ISNULL(@ProductID							, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
														+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
														+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';	
	DECLARE @ValueFormula				NVARCHAR(255)
	DECLARE @ValueINT					INT
	DECLARE @ValueText					NVARCHAR(MAX)
	DECLARE @ValueComment				NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(	 ProductID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,TimeID			INT										NOT NULL
			,ValueFormula	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueINT		INT										NOT NULL
			,ValueText		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueComment	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL		
		)

	INSERT INTO #tmp
		SELECT   tfV.ProductID		
				,tfV.ProductLineID	
				,tfV.FactoryID		
				,tfV.ValueSeriesID	
				,tfV.TimeID			
				,tfV.ValueFormula	
				,tfV.ValueINT		
				,tfV.ValueText		
				,tfV.ValueComment			
		FROM	load.tfValues tfV
		WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tfV.ProductID = @ProductID)
		 AND    (@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
		 AND    (@TimeID = 0			OR tfV.TimeID = @TimeID)
		 AND    (@Source = ''			OR tfV.Source = @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Value
				 @Username
				,@ProductID		
				,@ProductLineID	
				,@FactoryID		
				,@ValueSeriesID	
				,@TimeID			
				,@ValueFormula	
				,@ValueINT		
				,@ValueText		
				,@ValueComment	
										
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValues] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValues] TO pf_PlanningFactoryService;
GO

/*
Mandy Hauck, Stefan Lindenlaub
2018/06
Import values from load.tfValues with specific load-procedure as source
	EXEC [import].[spPOST_fValuesPDTV] 'SQL','','','','',0,''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[import].[spPOST_fValuesPDTV]') AND type in (N'P', N'PC'))
DROP PROCEDURE [import].[spPOST_fValuesPDTV]
GO

CREATE PROCEDURE [import].[spPOST_fValuesPDTV]
				(		 @Username 			AS NVARCHAR(255)
						,@FactoryID 		AS NVARCHAR(255)
						,@ProductlineID 	AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID 	AS NVARCHAR(255)
						,@TimeID			AS INT = 0
						,@Source			AS NVARCHAR(255)		
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall						DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName						NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode							INT			= 501;
	DECLARE @EffectedRows						INT			= 0;					
	DECLARE @TransactUsername					NVARCHAR(255)		= N'';
	DECLARE @ParameterString					NVARCHAR(MAX)	= N''''
																+ ISNULL(@Username							, N'NULL')	 + N''',''' 
																+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
																+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
																+ ISNULL(@ProductID							, N'NULL')	 + N''','''
																+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
																+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
																+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment							NVARCHAR(2000) = N'';	
	DECLARE @IsIncrementalValuesFlag			INT
	DECLARE	@ValuesInBracketsCommaSeparated 	NVARCHAR(MAX) 
	DECLARE @ToDoList AS TABLE
		(	 ProductID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID 						NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,FactoryID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,IsIncrementalValuesFlag 			INT				NOT NULL
			,ValuesInBracketsCommaSeparated 	NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
		)
				
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	-- Temporäre Tabelle als Datensammler
	IF OBJECT_ID('tempdb..#DATA') IS NOT NULL DROP TABLE #DATA
	CREATE TABLE #DATA	
			(
				 ProductID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ProductLineID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,TimeID				INT										NOT NULL
				,ValueFormula		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueINT			BIGINT									NOT NULL
				,ValueText			NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueComment		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			)

	INSERT INTO #DATA
			SELECT   tfV.ProductID		
					,tfV.ProductLineID	
					,tfV.FactoryID		
					,tfV.ValueSeriesID	
					,tfV.TimeID			
					,tfV.ValueFormula	
					,tfV.ValueINT		
					,REPLACE(tfV.ValueText,'''','')
					,REPLACE(tfV.ValueComment,'''','')	
			FROM	load.tfValues tfV
			WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
			  AND 	(@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
			  AND	(@ProductID = ''		OR tfV.ProductID = @ProductID)
			  AND	(@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
			  AND   (@TimeID = 0			OR tfV.TimeID = @TimeID)
			  AND   (@Source = ''			OR tfV.Source = @Source)


	-- Tabelle für String
	INSERT INTO @ToDoList
		SELECT	 d.ProductID
				,d.ProductLineId
				,d.FactoryID
				,1
				,(SELECT  N'[(''' 
					+ dxml.ValueSeriesID + N''','  
					+ CAST(dxml.TimeID AS NVARCHAR(255))+ N',''' 
					+ dxml.ValueFormula + N''',''' 
					+ CAST(dxml.ValueInt AS NVARCHAR(255)) + N''',''' 
					+ dxml.ValueText + N''',''' 
					+ dxml.ValueComment + N''')],'
				FROM #DATA dxml
				WHERE d.FactoryID = dxml.FactoryID AND d.ProductLineID = dxml.ProductLineID AND d.ProductID = dxml.ProductID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS ValuesInBracketsCommaSeparated
		FROM #DATA d
		GROUP BY   d.ProductID, d.ProductLineId, d.FactoryID

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR STATIC FOR

		SELECT tdl.ProductID
			  ,tdl.ProductLineID
			  ,tdl.FactoryID
			  ,tdl.IsIncrementalValuesFlag
			  ,''+LEFT(tdl.ValuesInBracketsCommaSeparated,LEN(tdl.ValuesInBracketsCommaSeparated)-1)+''
		FROM @ToDoList tdl

	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_ProductDataTableValues
			 @UserName
			,@ProductID		
			,@ProductLineID	
			,@FactoryID		
			,@IsIncrementalValuesFlag
			,@ValuesInBracketsCommaSeparated	
											
	Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValuesPDTV] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[import].[spPOST_fValuesPDTV] TO pf_PlanningFactoryService;
GO
/*
Mandy Hauck, Stefan Lindenlaub
2018/07
Import rights from loadtable with specific load-procedure as source

Testcall
EXEC import.spPOST_rRights 'SQL', '','',''
*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_rRights') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_rRights
GO

CREATE PROCEDURE import.spPOST_rRights
				(		 @Username		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Right			AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)			
AS

BEGIN
	SET NOCOUNT ON
		
	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
														+ ISNULL(@Right			, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';
	DECLARE @FactoryID					NVARCHAR(255)	= ''
	DECLARE @ProductLineID				NVARCHAR(255)	= ''
	DECLARE @ProductID					NVARCHAR(255)	= ''
	DECLARE	@ReadCommentMandatory		INT 			= ''
	DECLARE @WriteCommentMandatory		INT				= ''

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
		CREATE TABLE #tmp 
						(	 UserName		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,FactoryID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductLineID   NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,[Right]		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ReadCommentMandatory	INT								NOT NULL
							,WriteCommentMandatory	INT								NOT NULL
						)
		INSERT INTO #tmp
			SELECT   trR.UserName		 
					,trR.FactoryID		 
					,trR.ProductLineID  
					,trR.ProductID		 
					,trR.[Right]		 
					,trR.ReadCommentMandatory
					,trR.WriteCommentMandatory
			FROM	load.trRights trR
			WHERE	(@PostUserName = '' OR trR.UserName = @PostUserName)
			AND 	(@Right = ''		OR trR.[Right] = @Right)
			AND 	(@Source = ''		OR trR.Source = @Source)   
			AND		trR.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT * 
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_Right
			 @Username
			,@PostUserName				
			,@FactoryID				
			,@ProductLineID						
			,@Right					
			,@ReadCommentMandatory 
			,@WriteCommentMandatory

	Print @ResultCode
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rRights TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_rRights TO pf_PlanningFactoryService;
GO

/*
Stefan Lindenlaub
2018/07
Import User from load.trUser

Testcall
EXEC import.spPOST_rUser 'SQL','',''
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'import.spPOST_rUser') AND type in (N'P', N'PC'))
DROP PROCEDURE import.spPOST_rUser
GO

CREATE PROCEDURE import.spPOST_rUser
			     (		 @UserName		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)

AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode				INT				= 501;
	DECLARE @EffectedRows			INT				= 0;					
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username		, N'NULL')	 + N''',''' 
													+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
													+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment				NVARCHAR(2000) = N'';	
	DECLARE @PersonSurname			NVARCHAR(255)
	DECLARE @PersonFirstName		NVARCHAR(255)
	DECLARE @EMail					NVARCHAR(255)
	DECLARE @LDAPIP					NVARCHAR(255)
	DECLARE @Status					NVARCHAR(255)
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	 UserName		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonSurname	 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonFirstName NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,EMail			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,LDAPIP			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,Status			 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			)

	INSERT INTO #tmp
		SELECT   trU.UserName
				,trU.PersonSurname
				,trU.PersonFirstName
				,trU.EMail		
				,trU.LDAPIP		
				,trU.Status		
		FROM	load.trUser trU
		WHERE	(@PostUserName = '' OR trU.UserName = @PostUserName)
		  AND   (@Source = ''		OR trU.Source = @Source)
		  AND	trU.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_User
				 @UserName				
				,@PostUserName
				,@PersonSurname	
				,@PersonFirstName
				,@EMail			
				,@LDAPIP			
				,@Status			

		Print @ResultCode
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rUser TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::import.spPOST_rUser TO pf_PlanningFactoryService;
GO
/*
MOVE Operation for a Factory
Factory moved by the update IDs without key change

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory => 404 Not Found
5. Exists target Factory => 403 Forbidden
6. Factory moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT1'

EXECUTE @RC = [sx_pf_MOVE_Factory] 
		   @Username
		  ,@SourceFactoryID
		  ,@TargetFactoryID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Factory]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_Factory];
GO

CREATE PROCEDURE [sx_pf_MOVE_Factory]
	@Username			AS NVARCHAR(255),
	@SourceFactoryID	AS NVARCHAR(255),
	@TargetFactoryID	AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey		AS INT = 0;
	DECLARE @SourceFactoryKey		AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey		AS INT = 0;

	DECLARE @TransactUsername		AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName			AS NVARCHAR (255) =	OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString		AS NVARCHAR (MAX) = N'''' 
										+ ISNULL(@Username, N'NULL')		+ N''',''' 
										+ ISNULL(@SourceFactoryID, N'NULL') + N''',''' 
										+ ISNULL(@TargetFactoryID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						
	DECLARE @ResultCode AS INT = 501;						
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID = dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @TargetFactoryID = dbo.sx_pf_pProtectID (@TargetFactoryID);

		IF @SourceFactoryID = N'' OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Factory key already exists', 16, 10);
		END;
		
		-- STERP 3 - Move data
		--dFactory 
		UPDATE planning.tdFactories
		SET FactoryID = @TargetFactoryID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--dProductLine  
			UPDATE planning.tdProductLines
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dProduct 
			UPDATE planning.tdProducts
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			UPDATE planning.tdValueSeries
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE planning.tdTime
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE planning.tfValues
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gFactories
			UPDATE planning.tgFactories 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gProductLines Inhalt
			UPDATE planning.tgProductLines 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--gProducts 
			UPDATE planning.tgProducts 
			SET FactoryID  = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;
		
			SET @EffectedRows += @@ROWCOUNT;

			--rRights 
			UPDATE planning.trRights 
			SET FactoryID = @TargetFactoryID
			WHERE FactoryID = @SourceFactoryID;

			SET @EffectedRows += @@ROWCOUNT;

			--rPreferences
			UPDATE planning.trPreferences 
			SET FactoryID = @TargetFactoryID,
				UserKey = @TransactUserKey,
				UserName = @TransactUsername
			WHERE FactoryID = @SourceFactoryID;

			SET @EffectedRows += @@ROWCOUNT;

			SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

			--fStatements
			UPDATE planning.tfStatements 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- tFiles
			UPDATE planning.tFiles 
			SET FactoryID = @TargetFactoryID 
			WHERE FactoryKey = @SourceFactoryKey;

			-- write action statement
			INSERT INTO planning.tfStatements   
			SELECT ProductKey
				, dp.ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, ProductID
				, dp.ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'Factory is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
					+ N'" / ProductLine <' + dp.ProductlineID + N'>"' + ISNULL(dpl.Nameshort, N'NULL') 
					+ N'" / Product <' + ProductID  + N'>"' + dp.Nameshort +  N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved
			FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl
				ON dp.ProductLinekey = dpl.ProductLineKey
			WHERE dp.FactoryKey = @SourceFactoryKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Rights materialization
			EXEC dbo.sx_pf_materialize_vUserRights;

		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Factory] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Factory] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a Format - all objects, which an FormatID are switched to use a new FormatID.
Replace the OldFormatID with the new FormatID in planning.tdValueSeries
Replace the OldFormatID with the new FormatID in planning.thListValues
Delete the OldFormatID in planning.thFormats

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

12/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FormatID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Format => 404 Not Found
5. Not exists target Format => 404 Not Found
6. Format moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldFormatID AS NVARCHAR(255)=''
		DECLARE  @NewFormatID AS NVARCHAR(255)=''

EXECUTE @RC = [sx_pf_MOVE_Format]
		   @Username
		  ,@OldFormatID
		  ,@NewFormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Format]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_Format];
GO

CREATE PROCEDURE [sx_pf_MOVE_Format]
	@Username AS NVARCHAR(255),
	@OldFormatID AS NVARCHAR(255),
	@NewFormatID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldFormatKey AS INT = 0;
	DECLARE @NewFormatKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldFormatID, N'NULL') + N''',''' + ISNULL(@NewFormatID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldFormatID IS NULL SET @OldFormatID = N'';
	IF @NewFormatID IS NULL SET @NewFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @OldFormatID = dbo.sx_pf_pProtectID (@OldFormatID);
		SET @NewFormatID = dbo.sx_pf_pProtectID (@NewFormatID);

		IF (@OldFormatID = N'' OR @NewFormatID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldFormatKey = FormatKey FROM planning.thFormats WHERE [FormatID] = @OldFormatID;
		
		IF @OldFormatKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewFormatKey = FormatKey FROM planning.thFormats WHERE [FormatID] = @NewFormatID;
		
		IF @NewFormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		UPDATE planning.tdValueSeries
		   SET [ValueFormatID] = @NewFormatID 
		WHERE [ValueFormatID] = @OldFormatID;
																							
		SET @EffectedRows += @@ROWCOUNT;
																						   
		UPDATE planning.thListValues
		   SET [FormatID] = @NewFormatID 
		WHERE [FormatID] = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		UPDATE planning.tdTime
		   SET [FormatID] = @NewFormatID 
		WHERE [FormatID] = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thFormats WHERE [FormatID] = @OldFormatID;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Format] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Format] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a List
The Procedure will change the OldListID into the NewListID in tables:
[planning.tdValueSeries], column ValueListID
[planning.tdProductLines], columns GlobalAttributeSource 1..25
create members from OldListID in NewListID if they don't exist there
(for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
delete the OldListID in planning.thLists
delete the Old List Values in planning.thListValues

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ClusterWriteRight
		- sx_pf_pPOST_API_LogEntry

12/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target ListID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source List => 404 Not Found
5. Not exists target List => 404 Not Found
6. List moved => 200 OK
7. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldListID AS NVARCHAR(255)='suEvents'
		DECLARE  @NewListID AS NVARCHAR(255)='suYesNo'

EXECUTE @RC = [sx_pf_MOVE_List]
		   @Username
		  ,@OldListID
		  ,@NewListID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_List]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_List];
GO

CREATE PROCEDURE [sx_pf_MOVE_List]
	@Username AS NVARCHAR(255),
	@OldListID AS NVARCHAR(255),
	@NewListID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldListKey AS INT = 0;
	DECLARE @NewListKey AS INT = 0;
	DECLARE @NewListType AS NVARCHAR(255)= N'';

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldListID, N'NULL') + N''',''' + ISNULL(@NewListID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldListID IS NULL SET @OldListID = N'';
	IF @NewListID IS NULL SET @NewListID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @OldListID = dbo.sx_pf_pProtectID (@OldListID);
		SET @NewListID = dbo.sx_pf_pProtectID (@NewListID);

		IF (@OldListID = N'' OR @NewListID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @OldListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
			OR @NewListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel')
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
 		END;;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_ClusterWriteRight] @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldListKey = ListKey FROM planning.thLists WHERE [ListID] = @OldListID;
		
		IF @OldListKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewListKey = [ListKey], @NewListType = [Datentyp] FROM planning.thLists WHERE [ListID] = @NewListID;
		
		IF @NewListKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
							 
		-- STEP 3 - Move data
		-- STEP 3.1 - Move not exists UNIQUE list members

		IF @NewListType = 'Integer'
		BEGIN

			;WITH vals AS (
				SELECT v1.[ListValueKey] 
					, v1.[ValueInt]
					, v1.[Scale]
					, v1.[ValueText]
					, v1.[ValueComment]
					, v1.[FormatID]
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.[ListID] = @NewListID AND v1.ValueInt = v2.ValueInt
				WHERE v1.[ListID] = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES [ListValueKey]
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY [ValueInt] ORDER BY [ListValueKey])
			)																				   
			INSERT INTO planning.thListValues ([ListID], [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID])
			SELECT @NewListID, [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID]
			FROM vals INNER JOIN uniq ON vals.[ListValueKey] = uniq.[ListValueKey];

			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE IF @NewListType = 'String'
		BEGIN

			;WITH vals AS (
				SELECT v1.[ListValueKey] 
					, v1.[ValueInt]
					, v1.[Scale]
					, v1.[ValueText]
					, v1.[ValueComment]
					, v1.[FormatID]
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.[ListID] = @NewListID AND v1.[ValueText] = v2.[ValueText]
				WHERE v1.[ListID] = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES [ListValueKey]
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY [ValueText] ORDER BY [ListValueKey])
			)																				   
			INSERT INTO planning.thListValues ([ListID], [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID])
			SELECT @NewListID, [ValueInt], [Scale], [ValueText], [ValueComment], [FormatID]
			FROM vals INNER JOIN uniq ON vals.[ListValueKey] = uniq.[ListValueKey];

			SET @EffectedRows += @@ROWCOUNT;
		END;

		-- STEP 3.2 - Force all objects, which use a old list, to use a new list now
		UPDATE planning.tdValueSeries
		   SET [ValueListID] = @NewListID 
		WHERE [ValueListID] = @OldListID;
																							
		SET @EffectedRows += @@ROWCOUNT;
		
		;WITH lines AS (
			SELECT [ProductLineKey] 
			FROM planning.tdProductLines
			WHERE [GlobalAttributeSource1] = @OldListID OR [GlobalAttributeSource2] = @OldListID OR [GlobalAttributeSource3] = @OldListID OR
				[GlobalAttributeSource4] = @OldListID OR [GlobalAttributeSource5] = @OldListID OR [GlobalAttributeSource6] = @OldListID OR
				[GlobalAttributeSource7] = @OldListID OR [GlobalAttributeSource8] = @OldListID OR [GlobalAttributeSource9] = @OldListID OR
				[GlobalAttributeSource10] = @OldListID OR [GlobalAttributeSource11] = @OldListID OR [GlobalAttributeSource12] = @OldListID OR
				[GlobalAttributeSource13] = @OldListID OR [GlobalAttributeSource14] = @OldListID OR [GlobalAttributeSource15] = @OldListID OR
				[GlobalAttributeSource16] = @OldListID OR [GlobalAttributeSource17] = @OldListID OR [GlobalAttributeSource18] = @OldListID OR
				[GlobalAttributeSource19] = @OldListID OR [GlobalAttributeSource20] = @OldListID OR [GlobalAttributeSource21] = @OldListID OR
				[GlobalAttributeSource22] = @OldListID OR [GlobalAttributeSource23] = @OldListID OR [GlobalAttributeSource24] = @OldListID OR
				[GlobalAttributeSource25] = @OldListID 
		)
		UPDATE pl
		SET [GlobalAttributeSource1] = IIF([GlobalAttributeSource1] = @OldListID, @NewListID, [GlobalAttributeSource1])
			, [GlobalAttributeSource2] = IIF([GlobalAttributeSource2] = @OldListID, @NewListID, [GlobalAttributeSource2])
			, [GlobalAttributeSource3] = IIF([GlobalAttributeSource3] = @OldListID, @NewListID, [GlobalAttributeSource3])
			, [GlobalAttributeSource4] = IIF([GlobalAttributeSource4] = @OldListID, @NewListID, [GlobalAttributeSource4])
			, [GlobalAttributeSource5] = IIF([GlobalAttributeSource5] = @OldListID, @NewListID, [GlobalAttributeSource5])
			, [GlobalAttributeSource6] = IIF([GlobalAttributeSource6] = @OldListID, @NewListID, [GlobalAttributeSource6])
			, [GlobalAttributeSource7] = IIF([GlobalAttributeSource7] = @OldListID, @NewListID, [GlobalAttributeSource7])
			, [GlobalAttributeSource8] = IIF([GlobalAttributeSource8] = @OldListID, @NewListID, [GlobalAttributeSource8])
			, [GlobalAttributeSource9] = IIF([GlobalAttributeSource9] = @OldListID, @NewListID, [GlobalAttributeSource9])
			, [GlobalAttributeSource10] = IIF([GlobalAttributeSource10] = @OldListID, @NewListID, [GlobalAttributeSource10])
			, [GlobalAttributeSource11] = IIF([GlobalAttributeSource11] = @OldListID, @NewListID, [GlobalAttributeSource11])
			, [GlobalAttributeSource12] = IIF([GlobalAttributeSource12] = @OldListID, @NewListID, [GlobalAttributeSource12])
			, [GlobalAttributeSource13] = IIF([GlobalAttributeSource13] = @OldListID, @NewListID, [GlobalAttributeSource13])
			, [GlobalAttributeSource14] = IIF([GlobalAttributeSource14] = @OldListID, @NewListID, [GlobalAttributeSource14])
			, [GlobalAttributeSource15] = IIF([GlobalAttributeSource15] = @OldListID, @NewListID, [GlobalAttributeSource15])
			, [GlobalAttributeSource16] = IIF([GlobalAttributeSource16] = @OldListID, @NewListID, [GlobalAttributeSource16])
			, [GlobalAttributeSource17] = IIF([GlobalAttributeSource17] = @OldListID, @NewListID, [GlobalAttributeSource17])
			, [GlobalAttributeSource18] = IIF([GlobalAttributeSource18] = @OldListID, @NewListID, [GlobalAttributeSource18])
			, [GlobalAttributeSource19] = IIF([GlobalAttributeSource19] = @OldListID, @NewListID, [GlobalAttributeSource19])
			, [GlobalAttributeSource20] = IIF([GlobalAttributeSource20] = @OldListID, @NewListID, [GlobalAttributeSource20])
			, [GlobalAttributeSource21] = IIF([GlobalAttributeSource21] = @OldListID, @NewListID, [GlobalAttributeSource21])
			, [GlobalAttributeSource22] = IIF([GlobalAttributeSource22] = @OldListID, @NewListID, [GlobalAttributeSource22])
			, [GlobalAttributeSource23] = IIF([GlobalAttributeSource23] = @OldListID, @NewListID, [GlobalAttributeSource23])
			, [GlobalAttributeSource24] = IIF([GlobalAttributeSource24] = @OldListID, @NewListID, [GlobalAttributeSource24])
			, [GlobalAttributeSource25] = IIF([GlobalAttributeSource25] = @OldListID, @NewListID, [GlobalAttributeSource25])
		FROM planning.tdProductLines pl INNER JOIN lines l ON pl.ProductLineKey = l.ProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Remove old list
				
		DELETE FROM planning.thListValues WHERE [ListID] = @OldListID;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thLists WHERE [ListKey] = @OldListKey;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_List] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_List] TO pf_PlanningFactoryService;
GO
/*
Move Operation for a Product
Product moved by ID update

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- dbo.sx_pf_pGET_ProductlineReadRight
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline / Product => 404 Not Found
5. Not exists target Factory / Productline => 404
6. Exists TargetProduct => 403 Forbidden 
7. Product moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='230'
		DECLARE  @SourceProductLineID AS NVARCHAR(255)='I-ST-IT'
		DECLARE  @SourceProductID AS NVARCHAR(255)='888'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductLineID AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductID AS NVARCHAR(255)='RZ888'

EXECUTE @RC = [sx_pf_MOVE_Product] 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@SourceProductID
		  ,@TargetFactoryID
		  ,@TargetProductLineID
		  ,@TargetProductID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_Product]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_Product];
GO

CREATE PROCEDURE [sx_pf_MOVE_Product]
	@Username AS NVARCHAR(255),
	@SourceFactoryID AS NVARCHAR(255),
	@SourceProductLineID AS NVARCHAR(255),
	@SourceProductID AS NVARCHAR(255),
	@TargetFactoryID AS NVARCHAR(255),
	@TargetProductLineID AS NVARCHAR(255),
	@TargetProductID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey AS INT = 0;
	DECLARE @SourceProductKey AS INT = 0;
	DECLARE @SourceProductLineKey AS INT = 0;
	DECLARE @TargetProductKey AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductLineNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey AS INT = 0;
	DECLARE @TargetProductLineKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);;
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@SourceFactoryID, N'NULL') + N''',''' + ISNULL(@SourceProductLineID, N'NULL') + N''',''' + ISNULL(@SourceProductID, N'NULL') 
			+ N''',''' + ISNULL(@TargetFactoryID, N'NULL') + N''',''' + ISNULL(@TargetProductLineID, N'NULL') + N''',''' + ISNULL(@TargetProductID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID = N'';
	IF @SourceProductID IS NULL SET @SourceProductID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID = N'';
	IF @TargetProductID IS NULL SET @TargetProductID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID = dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @SourceProductlineID = dbo.sx_pf_pProtectID (@SourceProductlineID);
		SET @SourceProductID = dbo.sx_pf_pProtectID (@SourceProductID);
		SET @TargetFactoryID = dbo.sx_pf_pProtectID (@TargetFactoryID);
		SET @TargetProductlineID = dbo.sx_pf_pProtectID (@TargetProductlineID);
		SET @TargetProductID = dbo.sx_pf_pProtectID (@TargetProductID);

		IF (@SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @SourceProductID = N''
			OR @TargetFactoryID = N'' OR @TargetProductLineID = N'' OR @TargetProductID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_ProductLineWriteRight] @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE FactoryKey = @SourceFactoryKey AND ProductlineKey = @SourceProductlineKey AND ProductID = @SourceProductID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0 OR @SourceProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @TargetFactoryKey AND ProductlineKey = @TargetProductlineKey AND ProductID = @TargetProductID;
		
		IF @TargetFactoryKey = 0 OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		--dProduct 
		UPDATE planning.tdProducts 
		SET FactoryKey = @TargetFactoryKey
			, ProductLineKey = @TargetProductLineKey
			, FactoryID = @TargetFactoryID
			, ProductLineID = @TargetProductLineID
			, ProductID = @TargetProductID 
		WHERE ProductKey = @SourceProductKey;
					 
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--gProducts 
			UPDATE planning.tgProducts 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--dValueSeries
			UPDATE planning.tdValueSeries 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE planning.tdTime 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE planning.tfValues 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements 
			UPDATE planning.tfStatements 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineKey = @TargetProductLineKey
				, FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
				, ProductID = @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- tFiles
			UPDATE planning.tFiles 
			SET  FactoryID		= @TargetFactoryID
				,ProductLineID	= @TargetProductlineID
				,ProductID		= @TargetProductID

			WHERE 
				ProductKey = @SourceProductKey;

			-- write action statement
			INSERT INTO planning.tfStatements 
			SELECT @SourceProductKey AS ProductKey
				, @TargetProductLineKey AS ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, @TargetProductID  AS ProductID
				, @TargetProductlineID AS ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'Product is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
					+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
					+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Product] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_Product] TO pf_PlanningFactoryService;
GO
/*
MOVE Operation for a ProductLine
Productline moved by the update IDs without key change

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- dbo.sx_pf_pGET_FactoryReadRight
		- sx_pf_pGET_FactoryWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists source Factory / Productline => 404 Not Found
5. Not exists target  Factory und existierende Target Productline => 404  Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ProductLine moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @SourceFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @SourceProductLineID AS NVARCHAR(255)='ZT'
		DECLARE  @TargetFactoryID  AS NVARCHAR(255)='ZT'
		DECLARE  @TargetProductLineID AS NVARCHAR(255)='ZT32'

EXECUTE @RC = [sx_pf_MOVE_ProductLine] 
		   @Username
		  ,@SourceFactoryID
		  ,@SourceProductLineID
		  ,@TargetFactoryID
		  ,@TargetProductLineID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_ProductLine]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_ProductLine];
GO

CREATE PROCEDURE [sx_pf_MOVE_ProductLine]
	@Username AS NVARCHAR(255),
	@SourceFactoryID AS NVARCHAR(255),
	@SourceProductLineID AS NVARCHAR(255),
	@TargetFactoryID AS NVARCHAR(255),
	@TargetProductLineID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey AS INT = 0;
	DECLARE @SourceFactoryKey AS INT = 0;
	DECLARE @SourceProductLineKey AS INT = 0;
	DECLARE @SourceFactoryNameShort AS NVARCHAR(255) = N'';
	DECLARE @SourceProductLineNameShort AS NVARCHAR(255) = N'';
	DECLARE @TargetFactoryKey AS INT = 0;
	DECLARE @TargetProductLineKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@SourceFactoryID, N'NULL') + N''',''' + ISNULL(@SourceProductLineID, N'NULL') + N''',''' + ISNULL(@TargetFactoryID, N'NULL') 
		 + N''',''' + ISNULL(@TargetProductLineID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID = N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID = N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID = N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID = dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @SourceProductlineID = dbo.sx_pf_pProtectID (@SourceProductlineID);
		SET @TargetFactoryID = dbo.sx_pf_pProtectID (@TargetFactoryID);
		SET @TargetProductlineID = dbo.sx_pf_pProtectID (@TargetProductlineID);

		IF @SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @TargetFactoryID = N''
			OR @TargetProductlineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_FactoryWriteRight] @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);
		
		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductlineID = @SourceProductlineID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductlineID = @TargetProductlineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Move data		
		--dProductLine 
		UPDATE planning.tdProductLines 
		SET FactoryKey = @TargetFactoryKey
			, ProductLineID = @TargetProductLineID
			, FactoryID = @TargetFactoryID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--dProduct
			UPDATE planning.tdProducts 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			UPDATE planning.tdValueSeries 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE planning.tdTime 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE planning.tfValues 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--gProductLines
			UPDATE planning.tgProductLines 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--gProducts 
			UPDATE planning.tgProducts 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			--rRights
			UPDATE planning.trRights 
			SET FactoryID = @TargetFactoryID
				, ProductLineID = @TargetProductLineID
			WHERE FactoryID = @SourceFactoryID 
				AND ProductLineID = @SourceProductLineID;

			SET @EffectedRows += @@ROWCOUNT;

			--rPreferences
			UPDATE planning.trPreferences 
			SET ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID
				, UserKey = @TransactUserKey
				, UserName = @TransactUsername
			WHERE FactoryID = @SourceFactoryID 
				AND ProductlineID = @SourceProductlineID;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements
			UPDATE planning.tfStatements 
			SET FactoryKey = @TargetFactoryKey
				, ProductLineID = @TargetProductLineID
				, FactoryID = @TargetFactoryID 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- tFiles
			UPDATE planning.tFiles 
			SET FactoryID		= @TargetFactoryID, 
				ProductLineID	= @TargetProductlineID
			WHERE 
				ProductLineKey = @SourceProductLineKey;

			INSERT INTO planning.tfStatements
			SELECT ProductKey
				, @SourceProductLineKey AS ProductLineKey
				, @TargetFactoryKey AS FactoryKey
				, ProductID
				, @TargetProductlineID AS ProductLineID
				, @TargetFactoryID AS FactoryID
				, N'MOVE' AS ActionType
				, N'ProductLine is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
					+ N'"/ ProductLine <' + @SourceProductlineID + N'>"' + @SourceProductLineNameShort 
					+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved
			FROM planning.tdProducts 
			WHERE ProductLineKey = @SourceProductLineKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- Rights materialization
			EXEC dbo.sx_pf_materialize_vUserRights;

		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_ProductLine] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_ProductLine] TO pf_PlanningFactoryService;
GO
/*
Move Operation for one ValueSeries
The ValueSeries will be moved to a new ID, only if this ID is free
Works only inside one Product

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

09/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty  Source or Target  FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. User has no Rights => 401 Unauthorized
4. Not exists Source Factory / Productline / Product / ValueSerie => 404 Not Found
5. Not exists Target  Factory / Productline / Product => 404 Not Found
6. Target ValueSerie key already exists => 403 Forbidden
7. ValueSerie moved => 200 OK
8. Don`t moved anything => 204 No content

DECLARE @RC int
		DECLARE  @Username				AS NVARCHAR(255) = 'SQL'
		DECLARE  @FactoryID				AS NVARCHAR(255) ='ZT'
		DECLARE  @ProductLineID			AS NVARCHAR(255) ='U'
		DECLARE  @ProductID				AS NVARCHAR(255) ='2'
		DECLARE  @SourceValueSerieID	AS NVARCHAR(255) ='K1'
		DECLARE  @TargetValueSerieID	AS NVARCHAR(255) ='K7'

EXECUTE @RC = [sx_pf_MOVE_ValueSerie] 
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		  ,@ProductID
		  ,@SourceValueSerieID
		  ,@TargetValueSerieID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_MOVE_ValueSerie]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_MOVE_ValueSerie];
GO

CREATE PROCEDURE [sx_pf_MOVE_ValueSerie]
	@Username			AS NVARCHAR(255),
	@FactoryID			AS NVARCHAR(255),
	@ProductLineID		AS NVARCHAR(255),
	@ProductID			AS NVARCHAR(255),
	@SourceValueSerieID AS NVARCHAR(255),
	@TargetValueSerieID AS NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey				AS INT = 0;
	DECLARE @ProductLineKey			AS INT = 0;
	DECLARE @ProductKey				AS INT = 0;
	DECLARE @SourceValueSerieKey	AS INT = 0;
	DECLARE @TargetValueSerieKey	AS INT = 0;

	DECLARE @TransactUsername		AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName			AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);;
	DECLARE @ParameterString		AS NVARCHAR (MAX) = N'''' 
												+ ISNULL(@Username, N'NULL')									+ N''',''' 
												+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
												+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
												+ ISNULL(@ProductID, N'NULL')									+ N''',''' 
												+ ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL')	+ N''',''' 
												+ ISNULL(@TargetValueSerieID, N'NULL')							+ N'''';
	DECLARE @EffectedRows AS INT = 0;						
	DECLARE @ResultCode AS INT = 501;						
	DECLARE @TimestampCall AS DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment AS NVARCHAR (2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FactoryID IS NULL SET @FactoryID = N'';
	IF @ProductLineID IS NULL SET @ProductLineID = N'';
	IF @ProductID IS NULL SET @ProductID = N'';
	IF @SourceValueSerieID IS NULL SET @SourceValueSerieID = N'';
	IF @TargetValueSerieID IS NULL SET @TargetValueSerieID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID = dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID = dbo.sx_pf_pProtectID (@ProductLineID);
		SET @ProductID = dbo.sx_pf_pProtectID (@ProductID);
		SET @SourceValueSerieID = dbo.sx_pf_pProtectID (@SourceValueSerieID);
		SET @TargetValueSerieID = dbo.sx_pf_pProtectID (@TargetValueSerieID);

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR  @SourceValueSerieID = N'' OR  @TargetValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = [sx_pf_pGET_ProductLineWriteRight] @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey  AND ValueSeriesID = @SourceValueSerieID;	
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 OR  @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @TargetValueSerieID;

		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Move data
		--dValueSeries 
		UPDATE planning.tdValueSeries
		SET ValueSeriesID = @TargetValueSerieID
		WHERE ValueSeriesKey = @SourceValueSerieKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN			  
			--fValues
			UPDATE planning.tfValues 
			SET ValueSeriesID = @TargetValueSerieID
			WHERE ValueSeriesKey = @SourceValueSerieKey;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);
				
			INSERT INTO planning.tfStatements  
			SELECT @ProductKey AS ProductKey
				, @ProductLineKey AS ProductLineKey
				, @FactoryKey AS FactoryKey
				, @ProductID  AS ProductID
				, @ProductlineID  AS ProductLineID
				, @FactoryID  AS FactoryID
				, N'MOVE' AS ActionType
				, N'ValueSeries ' + @SourceValueSerieID + N' was moved to new ID ' + @TargetValueSerieID AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5), N':', N'') AS BIGINT) AS [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC [sx_pf_pPOST_API_LogEntry] @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_ValueSerie] TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::[sx_pf_MOVE_ValueSerie] TO pf_PlanningFactoryService;
GO
/*
Procedure to auto create param tables from all products in Productline PARAM in Factory ZT

Testcall
EXEC control.spParamTables 'ANLAG'
09/2018 for DataFactory 5.0
Mandy Hauck / Gerd Tautenhahn

*/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.spParamTables') AND type in (N'P', N'PC'))
DROP PROCEDURE control.spParamTables
GO

CREATE PROCEDURE control.spParamTables (@ProductID NVARCHAR(255))
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TableDelete				NVARCHAR(MAX) = ''
	DECLARE @SQLDrop					NVARCHAR(MAX) = ''
	DECLARE @ProductKey					BIGINT	= 0
	DECLARE @ResultCode					INT		= 0

	-------------------------------------------------------------------------------------------------------------------
	-- Delete param.tPivot table
	SET @TableDelete = 'param.[tPivot' + @ProductID+ ']'
	SET @SQLDrop = 'IF OBJECT_ID(''' + 'param.[tPivot' + @ProductID+ ']' + ''', ''U'') IS NOT NULL DROP TABLE ' + @TableDelete
	EXECUTE sp_executesql @SQLDrop
	-------------------------------------------------------------------------------------------------------------------
	-- ##### CREATE ###########
	
	SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = 'ZT' AND ProductLineID = 'PARAM' AND ProductID = @ProductID

	IF @ProductKey > 0
		BEGIN
				-- CREATE TABLE -------------------------------------------------------------------------------------------------------------------
				DECLARE	@TableSchema			NVARCHAR(255)	= 'param.'
				DECLARE	@TablePreName			NVARCHAR(255)	= 'tPivot'																						
				DECLARE @StringBasic			NVARCHAR(MAX)	= 'RowKey BIGINT IDENTITY(1,1),FactoryID NVARCHAR(255),ProductLineID NVARCHAR(255),ProductID NVARCHAR(255),TimeID INT'
				DECLARE @StringColumns			NVARCHAR(MAX)	= ''	
				DECLARE @SQLCreate				NVARCHAR(MAX)	= ''
				DECLARE	@TableName				NVARCHAR(MAX)	= ''

				SELECT @TableName = @TableSchema + '[' + @TablePreName + @ProductID + ']'

				-- Gets all ValueSeriesIDs from a Product in ProductlLine 'PARAM' in Factory 'ZT', ValueSeriesIDs will become column headers
				-- All Text Values will become NVARCHAR(4000), all Numeric Columns Type Money

				-- Collect definition of all columns
				SELECT @StringColumns = COALESCE(@StringColumns + ',', '') +  
						IIF(
							dVS.[IsNumeric]=0
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] NVARCHAR(4000)'
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] MONEY'
							)
				FROM planning.tdValueSeries dVS 
				WHERE dVS.ProductKey = @ProductKey

				-- create the table
				SET @SQLCreate = 'CREATE TABLE ' 
							+ @TableName 
							+ '(' 
							+ @StringBasic 
							+ @StringColumns
							+ ' CONSTRAINT PK_' + @TablePreName + @ProductID +' PRIMARY KEY NONCLUSTERED (RowKey))'
				EXECUTE sp_executesql @SQLCreate

				-- Fill Table -------------------------------------------------------------------------------------------------------
			
				DECLARE @STRING_PivotList					NVARCHAR(MAX)  
				DECLARE @STRING_SelectColums				NVARCHAR(MAX)  
				DECLARE @STRING_Insert						NVARCHAR(MAX)   

				-- Get string for dynamic SQL used in PIVOT command (...ValueSeriesID IN ([...]))
				SELECT @STRING_PivotList = COALESCE(@STRING_PivotList + ',', '') +  '[' + CONVERT(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Get string for dynamic SQL - ColumnNames
				SELECT @STRING_SelectColums = COALESCE(@STRING_SelectColums + ',', '') 
					+ 'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''') AS [' + convert(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Get string for dynamic SQL used in INSERT command
				SELECT @STRING_Insert = COALESCE(@STRING_Insert + ',', '')
							+IIF(
								 dVS.[IsNumeric]=0
								,'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''')'
								,'COALESCE(CAST([' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] AS MONEY) ' + ',0) '
							)
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey

				-- Select Product Values to #table to not use dynamic SQL on real table
				IF OBJECT_ID('tempdb..#ParamValues') IS NOT NULL
					DROP TABLE #ParamValues
				
				CREATE TABLE #ParamValues
					(
						 FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductlineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,TimeID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,Value				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NOT NULL
					)

				INSERT INTO #ParamValues
					SELECT 
						 fV.FactoryID
						,fV.ProductLineID
						,fV.ProductID
						,fV.ValueSeriesID
						,fV.TimeID
						,CASE
							WHEN dVS.[IsNumeric] = 0 THEN fV.ValueText
							WHEN dVS.[IsNumeric] = 1 THEN CONVERT(NVARCHAR(255),CAST(fV.ValueInt AS MONEY)/dVS.Scale,2) --warum Parameter 2 im Convert ?
						 END AS Value
					FROM planning.tfValues fV
							LEFT JOIN planning.tdValueSeries dVS
								ON fV.ValueSeriesKey = dVS.ValueSeriesKey
							LEFT JOIN planning.tdProducts dP 
								ON fV.ProductKey = dP.ProductKey
					WHERE fV.ProductKey = @ProductKey

				DECLARE @SQLText	NVARCHAR(MAX) = 'INSERT INTO ' + @TableName + '
					SELECT	 PivotT.FactoryID,PivotT.ProductLineID,PivotT.ProductID,PivotT.TimeID
							,' + @STRING_Insert + '													
					FROM #ParamValues
					PIVOT
						(MAX(Value) FOR ValueSeriesID IN (' + @STRING_PivotList + ')) AS PivotT'

				EXECUTE sp_executesql @SQLText

				-- job well done
				SET @ResultCode = 200
		END
	ELSE
		BEGIN
			SET @ResultCode = 404
		END
	RETURN @ResultCode

END
GO

-- usually no grant necessary, grant only for the case that materialization is called after import of values
GRANT EXECUTE ON OBJECT :: control.spParamTables TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT :: control.spParamTables TO pf_PlanningFactoryService;
/*
Cleanup Operation, which will clean the values in planning.tfValues for a given scope.
dbo.sx_pf_POST_Cleanup 'SQL', 'F1', 'PL1',	'P1',	'VS1',	201703012	-- will delete all values in planning.tfValues for this combination (this means one row)
dbo.sx_pf_POST_Cleanup 'SQL', '',	'PL1',	'P1',	'VS1',	20170312	-- would delete this value in all Factories
dbo.sx_pf_POST_Cleanup 'SQL', '',	'',		'',		'',		20170312	-- would delete all values in for this TimeID in planning.tfValues.

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE	 @RC				INT
DECLARE  @Username			NVARCHAR(255)	= 'SQL'
DECLARE  @FactoryID			NVARCHAR(255)	= 'ZT'
DECLARE  @ProductLineID		NVARCHAR(255)	= 'U'
DECLARE  @ProductID			NVARCHAR(255)	= ''
DECLARE  @ValueSeriesID		NVARCHAR(255)	= 'K3'
DECLARE  @TimeID			INT				= 20180515	

EXECUTE	@RC = dbo.sx_pf_POST_Cleanup 
		@Username,
		@FactoryID,
		@ProductLineID,
		@ProductID,
		@ValueSeriesID,
		@TimeID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Cleanup') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Cleanup;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Cleanup
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),	
	@ValueSeriesID	NVARCHAR(255),
	@TimeID			INT = 0
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username						, N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID					, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSeriesID					, N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255))	, N'NULL');
	DECLARE @EffectedRows		INTEGER			= 0;					-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @ProductID		IS NULL	SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Cleanup;

		-- empty TimeID is not allowed
		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID (@ValueSeriesID);

		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
		
		-- STEP 2 - Check rights
		DECLARE @Rights AS TABLE(FactoryID NVARCHAR(255) NOT NULL, ProductLineID NVARCHAR(255) NOT NULL);

		INSERT INTO @Rights (FactoryID, ProductLineID) 
			SELECT
				  FactoryID
				 ,ProductLineID
			FROM planning.trUserRights 							 
			WHERE 
				Username = @TransactUsername 
				AND [Right]	= N'Write'
				AND (@FactoryID	= N'' OR FactoryID = @FactoryID)
				AND (@ProductLineID	= N'' OR ProductLineID = @ProductLineID);

		IF NOT EXISTS(SELECT 1 FROM @Rights)
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3 - Delete values where user has rights
		DELETE FROM fV
		FROM planning.tfValues fV 
			INNER JOIN @Rights r ON fV.FactoryID = r.FactoryID COLLATE DATABASE_DEFAULT AND fV.ProductLineID = r.ProductLineID COLLATE DATABASE_DEFAULT
		WHERE fV.TimeID = @TimeID
			AND (@ProductID		 = N''	OR fV.ProductID		 = @ProductID)
			AND (@ValueSeriesID	 = N''	OR fV.ValueSeriesID	 = @ValueSeriesID); 

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;
		
		COMMIT TRANSACTION sx_pf_POST_Cleanup;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Cleanup;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryService;
GO
/*
POST Operation for a ClusterProperty
A not existing Property will be created, an existing Property will be updated
If the value <#NV> is passed as parameter, it means keep the existing value 

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to the cluster at Create => 401 Unauthorized
4. Property created => 201 Created
5. Property updated => 200 OK

DECLARE @RC					 INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @PropertyID			 NVARCHAR(255) = 'rre'
DECLARE @PropertyName		 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @Unit				 NVARCHAR(255) = ''
DECLARE @ValueText			 NVARCHAR(255) = ''
DECLARE @ValueInt			 NVARCHAR(255) = '678'
DECLARE @Scale				 NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	 NVARCHAR(255) = '0'
DECLARE @FormatID			 NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ClusterProperty 
		@Username,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ClusterProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ClusterProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ClusterProperty
	@Username			 NVARCHAR(255),
	@PropertyID			 NVARCHAR(255),
	@PropertyName		 NVARCHAR(255),
	@CommentUser		 NVARCHAR(255),
	@CommentDev			 NVARCHAR(255),
	@Unit				 NVARCHAR(255),
	@ValueText			 NVARCHAR(255),
	@ValueInt			 NVARCHAR(255),
	@Scale				 NVARCHAR(255),
	@IsROSystemProperty	 NVARCHAR(255),
	@FormatID			 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		 INT			= 0;
	DECLARE @SytemROBlocker		 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username				, N'NULL') + N''',''' 
			+ ISNULL(@PropertyID			, N'NULL') + N''','''
			+ ISNULL(@PropertyName			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser			, N'NULL') + N''','''
			+ ISNULL(@CommentDev			, N'NULL') + N''',''' 
			+ ISNULL(@Unit					, N'NULL') + N''','''
			+ ISNULL(@ValueText				, N'NULL') + N''','''
			+ ISNULL(@ValueInt				, N'NULL') + N''','''
			+ ISNULL(@Scale					, N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty	, N'NULL') + N''',''' 
			+ ISNULL(@FormatID				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ClusterProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @PropertyName	 = dbo.sx_pf_pProtectString	 (@PropertyName);
		SET @CommentUser	 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev		 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @Unit			 = dbo.sx_pf_pProtectString	 (@Unit);
		SET @ValueText		 = dbo.sx_pf_pProtectString	 (@ValueText);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);
		SET @FormatID		 = dbo.sx_pf_pProtectString	 (@FormatID);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @TransactUsername = 'FactoryService';
		END;

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM planning.tgCluster WHERE PropertyID = @PropertyID; 
				
		-- STEP 1.3 - Don`t allow changing of system RO Properties
		SELECT @SytemROBlocker = IsROSystemProperty FROM planning.tgCluster WHERE PropertyID = @PropertyID;
		
		IF @SytemROBlocker = 1 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200 AND NOT @TransactUsername = 'FactoryService' 
			BEGIN
				RAISERROR('Invalid rights', 16, 10);
			END

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemProperty,FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key])
			VALUES (@PropertyID,'','','','','',0,1,0,'');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;		

		-- STEP 3.2 - Update Cluster attributes
		UPDATE planning.tgCluster
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev				= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty		= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgCluster 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF NOT @ResultCode = 201 SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_POST_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for a Factory
Not existing Factories are created, existing updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC					 INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			 NVARCHAR(255) = 'ZT'
DECLARE @NameShort			 NVARCHAR(255) = ''
DECLARE @NameLong			 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @ResponsiblePerson	 NVARCHAR(255) = ''
DECLARE @ImageName			 NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_Factory 
		@Username,
		@FactoryID,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName

PRINT @RC
*/

IF  EXISTS(SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Factory') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Factory;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Factory
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson	NVARCHAR(255),
	@ImageName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')			 + N''','''
			+ ISNULL(@NameShort			, N'NULL')			 + N''','''
			+ ISNULL(@NameLong			, N'NULL')			 + N''','''
			+ ISNULL(@CommentUser		, N'NULL')			 + N''','''
			+ ISNULL(@CommentDev		, N'NULL')			 + N''','''
			+ ISNULL(@ResponsiblePerson	, N'NULL')			 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')			 + N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID			 = N'';
	IF @NameShort			 IS NULL SET @NameShort			 = N'';
	IF @NameLong			 IS NULL SET @NameLong			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser		 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev		 = N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson	 = N'';
	IF @ImageName			 IS NULL SET @ImageName			 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @NameShort			 = dbo.sx_pf_pProtectString	 (@NameShort);
		SET @NameLong			 = dbo.sx_pf_pProtectString	 (@NameLong);
		SET @CommentUser		 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev			 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @ResponsiblePerson	 = dbo.sx_pf_pProtectString	 (@ResponsiblePerson);
		SET @ImageName			 = dbo.sx_pf_pProtectString	 (@ImageName);
		
		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		-- STEP 2 - Check rights
		IF @FactoryKey = 0
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		ELSE
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Factory does not yet exist, it will be created in minimal configuration
		IF @FactoryKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
			INSERT INTO planning.tdFactories (FactoryID, NameShort,NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName) 
				OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
				VALUES (@FactoryID, @NameShort, '', '', '', '', '');

			SELECT TOP(1) @FactoryKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;		
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update Factory attributes
		UPDATE planning.tdFactories 
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			@NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			@NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	@ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			@ImageName)
		FROM planning.tdFactories 
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (no insert needed because only ClusterAdmin can create Factory)
		EXEC dbo.sx_pf_materialize_vUserRights;	

		COMMIT TRANSACTION sx_pf_POST_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Factory;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryService;
GO
/*
POST Operation for a FactoryProperty
Not existing Property will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					  INT
DECLARE @Username			  NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			  NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			  NVARCHAR(255) = 'rr3e'
DECLARE @PropertyName		  NVARCHAR(255) = ''
DECLARE @CommentUser		  NVARCHAR(255) = ''
DECLARE @CommentDev			  NVARCHAR(255) = ''
DECLARE @Unit				  NVARCHAR(255) = ''
DECLARE @ValueText			  NVARCHAR(255) = ''
DECLARE @ValueInt			  NVARCHAR(255) = '678'
DECLARE @Scale				  NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	  NVARCHAR(255) = '2'
DECLARE @FormatID			  NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_FactoryProperty 
		@Username,
		@FactoryID,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_FactoryProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FactoryProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FactoryProperty
	@Username			  NVARCHAR(255),
	@FactoryID			  NVARCHAR(255),
	@PropertyID			  NVARCHAR(255),
	@PropertyName		  NVARCHAR(255),
	@CommentUser		  NVARCHAR(255),
	@CommentDev			  NVARCHAR(255),
	@Unit				  NVARCHAR(255),
	@ValueText			  NVARCHAR(255),
	@ValueInt			  NVARCHAR(255),
	@Scale				  NVARCHAR(255),
	@IsROSystemProperty	  NVARCHAR(255),
	@FormatID			  NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			  INT = 0;
	DECLARE @PropertyKey		  INT = 0;
	DECLARE @TransactUsername	  NVARCHAR(255) = N'';
	DECLARE @ProcedureName		  NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	  NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyName,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev,			 N'NULL')	+ N''',''' 
			+ ISNULL(@Unit,					 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueText,			 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueInt,				 N'NULL')	+ N''',''' 
			+ ISNULL(@Scale,				 N'NULL')	+ N''',''' 
			+ ISNULL(@IsROSystemProperty,	 N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID,				 N'NULL')	+ N'''';		
	DECLARE @EffectedRows		  INT			 = 0;				
	DECLARE @ResultCode			  INT			 = 501;				
	DECLARE @TimestampCall		  DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			  NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @PropertyName	 = dbo.sx_pf_pProtectString	 (@PropertyName);
		SET @CommentUser	 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev		 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @Unit			 = dbo.sx_pf_pProtectString	 (@Unit);
		SET @ValueText		 = dbo.sx_pf_pProtectString	 (@ValueText);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);
		SET @FormatID		 = dbo.sx_pf_pProtectString	 (@FormatID); --has String Protection, as also #<NV> can be used as parameter
				
		IF	@PropertyID = N''
			OR @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - disabled from Version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		-- STEP 3.1 - If property not exist, it will created minimalistic	
		IF @PropertyKey = 0
		BEGIN
			INSERT INTO planning.tgFactories (FactoryKey, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys
			VALUES (@FactoryKey, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
		   
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update Factory property
		UPDATE planning.tgFactories
		SET 
			 PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgFactories
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for a FactoryTab
Not existing Property will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

07/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard


TestCalls

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TabID				NVARCHAR(255) = 'Tab01'
DECLARE @Label				NVARCHAR(255) = 'Productlines'
DECLARE @CommentUser		NVARCHAR(MAX) = 'List of Productlines'
DECLARE @PresentationType	NVARCHAR(255) = 'ProductlineList'	
DECLARE @Source				NVARCHAR(MAX) = ''		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @TabID				NVARCHAR(255) = 'Tab02'
DECLARE @Label				NVARCHAR(255) = 'Profit'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can pivot your P&L here.'
DECLARE @PresentationType	NVARCHAR(255) = 'Pivot'	
DECLARE @Source				NVARCHAR(MAX) = 'sx_pf_DATAOUTPUT_Profit'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

SET @TabID				= 'Tab03'
SET @Label				= 'Balance'
SET @CommentUser		= 'You can pivot your Balance here.'
SET @PresentationType	= 'Pivot'	
SET @Source				= 'sx_pf_DATAOUTPUT_Balance'		
SET @Layout				= ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID, 
		@TabID, 
		@Label, 
		@CommentUser, 
		@PresentationType, 
		@Source, 
		@Layout

PRINT @RC

SET @TabID				= 'Tab04'
SET @Label				= 'Manual'
SET @CommentUser		= ''
SET @PresentationType	= 'HTML_URL'	
SET @Source				= 'http://www.saxess-software.de'		
SET @Layout				= ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

SET @TabID				= 'Tab05'
SET @Label				= 'Report'
SET @CommentUser		= ''
SET @PresentationType	= 'HTML_Static'	
SET @Source				= 'sx_pf_DATAOUTPUT_PLReport' -- this sp don't exists, asume it retruns an HTML block as nvarchar(max)
SET @Layout				= ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @TabID				NVARCHAR(255) = 'Tab06'
DECLARE @Label				NVARCHAR(255) = 'ProfitChart'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can pivot your P&L here.'
DECLARE @PresentationType	NVARCHAR(255) = 'Pivot_Chart'	
DECLARE @Source				NVARCHAR(MAX) = 'sx_pf_DATAOUTPUT_Profit'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = '2'
DECLARE @TabID				NVARCHAR(255) = 'Tab07'
DECLARE @Label				NVARCHAR(255) = 'Eventauswertung'
DECLARE @CommentUser		NVARCHAR(MAX) = 'This is your ToDo List.'
DECLARE @PresentationType	NVARCHAR(255) = 'DataGrid'	
DECLARE @Source				NVARCHAR(MAX) = 'result.spEventsPlan'		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC

DECLARE @RC					INT
DECLARE @Username 			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = 'BK'
DECLARE @TabID				NVARCHAR(255) = 'Tab08'
DECLARE @Label				NVARCHAR(255) = 'Anleitung'
DECLARE @CommentUser		NVARCHAR(MAX) = 'You can create own pages here.'
DECLARE @PresentationType	NVARCHAR(255) = 'HTML_Editor'	
DECLARE @Source				NVARCHAR(MAX) = ''		
DECLARE @Layout				NVARCHAR(MAX) = ''

EXECUTE @RC = dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout

PRINT @RC



*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_FactoryTab') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FactoryTab;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FactoryTab
	 @Username			NVARCHAR(255)
	,@FactoryID			NVARCHAR(255)
	,@TabID				NVARCHAR(255)
	,@Label				NVARCHAR(255)
	,@CommentUser		NVARCHAR(MAX)
	,@PresentationType	NVARCHAR(255)	
	,@Source			NVARCHAR(MAX)		
	,@Layout			NVARCHAR(MAX)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @PropertyOwner		INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		 N'NULL')	+ N''',''' 
			+ ISNULL(@TabID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@Label,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser,		 N'NULL')	+ N''',''' 
			+ ISNULL(@PresentationType,	 N'NULL')	+ N''',''' 
			+ ISNULL(@Source,			 N'NULL')	+ N''','''
			+ ISNULL(@Layout,			 N'NULL')	+ N'''';		
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @TabID				IS NULL SET @TabID				= N'';
	IF @Label				IS NULL SET @Label				= N'';
	IF @CommentUser			IS NULL SET @CommentUser		= N'';
	IF @PresentationType	IS NULL SET @PresentationType	= N'';
	IF @Source				IS NULL SET @Source				= N'';
	IF @Layout				IS NULL SET @Layout				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FactoryTab;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TabID				= dbo.sx_pf_pProtectID		(@TabID);
		SET @Label				= dbo.sx_pf_pProtectString	(@Label);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @PresentationType	= dbo.sx_pf_pProtectString	(@PresentationType);
		SET @Source				= dbo.sx_pf_pProtectString	(@Source);
				
		IF  @TabID = N'' 
			OR @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @TabID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties
		SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @TabID;

		IF @PropertyOwner = 1
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);			
		
		-- STEP 3.1 - If property not exist, it will created minimalistic	
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgFactories (FactoryKey, FactoryID,PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemProperty,FormatID) 
				OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
				VALUES (@FactoryKey, @FactoryID,@TabID,'','','','','',0,1,0,'');	

			SELECT TOP(1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update Factory property
		UPDATE planning.tgFactories
		SET		
			 PropertyName	= IIF(@Label			= N'<#NV>', PropertyName	,@Label)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser		,@CommentUser)
			,Unit			= IIF(@PresentationType = N'<#NV>', Unit			,@PresentationType)
			,ValueText		= IIF(@Source			= N'<#NV>', ValueText		,@Source)
			,CommentDev		= IIF(@Layout			= N'<#NV>', CommentDev		,@Layout)
		FROM planning.tgFactories 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FactoryTab;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FactoryTab;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryTab TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryTab TO pf_PlanningFactoryService;
GO

/*
POST Operation for sending FlexAction Parameter
Procedure must be editend to fit user needs and recreated after each API Update.

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
--should return Data and 200
DECLARE @RC						NVARCHAR(255)
DECLARE @Username				NVARCHAR(255) = 'SQL'
DECLARE @FactoryID				NVARCHAR(255) = 'F1'
DECLARE @ProductLineID			NVARCHAR(255) = 'PL1'
DECLARE @ProductID				NVARCHAR(255) = 'P1'
DECLARE @ValueSeriesID			NVARCHAR(255) = 'Mitarbeiter'
DECLARE @TimeID					BIGINT		  = 20180115
DECLARE @GlobalattributeNumber	NVARCHAR(255) = ''
DECLARE @PageType				NVARCHAR(255) = ''
DECLARE @FlexParameter			NVARCHAR(255) = 'P1000'


EXEC @RC = dbo.sx_pf_POST_FlexAction
	 @Username,
	 @FactoryID,
	 @ProductLineID,
	 @ProductID,
	 @ValueSeriesID,
	 @TimeID,
	 @GlobalattributeNumber,
	 @PageType,
	 @FlexParameter
	 	
PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_FlexAction') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_FlexAction;
GO

CREATE PROCEDURE dbo.sx_pf_POST_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@FlexParameter			NVARCHAR(255)  -- The Identifier (Second column of GET_FlexAction, first Display column) from the choosen list element
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FactoryID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductLineID,			 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ValueSeriesID,			 N'NULL')		+ N''','
			+ CAST(ISNULL(@TimeID, N'NULL') AS NVARCHAR(255))	+ N','''
			+ ISNULL(@GlobalattributeNumber,	 N'NULL')		+ N''',''' 
			+ ISNULL(@PageType,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FlexParameter,			 N'NULL')		+ N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductlineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	IF @FlexParameter			IS NULL SET @FlexParameter			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @FlexParameter	= dbo.sx_pf_pProtectString	(@FlexParameter);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END

		-- Sample Actions
		-- React on content parameters and FlexParameter for action

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			-- POST_ValueSeries of Type Projekt, ValueSeriesID = FlexParameter
			PRINT 'Did Projekt'
			SET @ResultCode = 200;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			-- POST ValuesSeries of Type Mitarbeiter, ValueSeriesID = FlexParameter
			PRINT 'Did Mitarbeiter'
			SET @ResultCode = 200;
		END
		
		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter' )
		BEGIN
			PRINT 'No fitting action found.'
			SET @ResultCode = 404
		END
			   
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FlexAction;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryUser
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryService
GO
/*
POST Operation for Format
Not existing Format will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call - Blockstyle !
DECLARE @RC						INT
DECLARE @Username				NVARCHAR(255) = 'SQL'					'SQL'
DECLARE @FormatID				NVARCHAR(255) = 'sxTest4'				'sxTest3'
DECLARE @BackgroundColor		NVARCHAR(255) = '100,100,100'			'<#NV>'
DECLARE @FontColor				NVARCHAR(255) = '200,200,200'			'<#NV>'
DECLARE @ValueFormat			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@Bold					NVARCHAR(255) = ''						'<#NV>'
DECLARE	@Italic					NVARCHAR(255) = ''						'<#NV>'
DECLARE	@AlignmentHorizontal	NVARCHAR(255) = ''						'<#NV>'
DECLARE	@AlignmentVertical		NVARCHAR(255) = ''						'<#NV>'
DECLARE	@TextUnderlined			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@BorderToNext			NVARCHAR(255) = ''						'<#NV>'
DECLARE	@BorderToPrevious		NVARCHAR(255) = ''  					'<#NV>'  
		

EXECUTE @RC = dbo.sx_pf_POST_Format 
		@Username,
		@FormatID,
		@BackgroundColor,
		@FontColor,
		@ValueFormat,
		@Bold,				
		@Italic,				
		@AlignmentHorizontal,
		@AlignmentVertical,	
		@TextUnderlined,		
		@BorderToNext,		
		@BorderToPrevious	

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Format') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Format;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Format
	@Username				NVARCHAR(255),
	@FormatID				NVARCHAR(255),
	@BackgroundColor		NVARCHAR(255),
	@FontColor				NVARCHAR(255),
	@ValueFormat			NVARCHAR(255),
	@Bold					NVARCHAR(255),
	@Italic					NVARCHAR(255),
	@AlignmentHorizontal	NVARCHAR(255),
	@AlignmentVertical		NVARCHAR(255),
	@TextUnderlined			NVARCHAR(255),
	@BorderToNext			NVARCHAR(255),
	@BorderToPrevious		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N''''
			+ ISNULL(@Username				, N'NULL')		+ N''','''
			+ ISNULL(@FormatID				, N'NULL')		+ N''','''
			+ ISNULL(@BackgroundColor		, N'NULL')		+ N''','''
			+ ISNULL(@FontColor				, N'NULL')		+ N''','''
			+ ISNULL(@ValueFormat			, N'NULL')		+ N''',''' 
			+ ISNULL(@Bold					, N'NULL')		+ N''','''
			+ ISNULL(@Italic				, N'NULL')		+ N''',''' 
			+ ISNULL(@AlignmentHorizontal	, N'NULL')		+ N''','''
			+ ISNULL(@AlignmentVertical		, N'NULL')		+ N''','''
			+ ISNULL(@TextUnderlined		, N'NULL')		+ N''','''
			+ ISNULL(@BorderToNext			, N'NULL')		+ N''','''
			+ ISNULL(@BorderToPrevious		, N'NULL')		+ N''','''
			+ N'''';
	DECLARE @EffectedRows	 INT = 0;						
	DECLARE @ResultCode		 INT = 501;						
	DECLARE @TimestampCall	 DATETIME = CURRENT_TIMESTAMP;
	DECLARE @Comment		 NVARCHAR (2000) = N'';
	DECLARE @ErrorText		 NVARCHAR (2000) = N';'			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			 = N'';
	IF @FormatID			IS NULL SET @FormatID			 = N'';
	IF @BackgroundColor		IS NULL SET @BackgroundColor	 = N'';
	IF @FontColor			IS NULL SET @FontColor			 = N'';
	IF @ValueFormat			IS NULL SET @ValueFormat		 = N'';
	IF @Bold				IS NULL SET @Bold				 = N'';
	IF @Italic				IS NULL SET @Italic				 = N'';
	IF @AlignmentHorizontal	IS NULL SET @AlignmentHorizontal = N'';
	IF @AlignmentVertical	IS NULL SET @AlignmentVertical	 = N'';
	IF @TextUnderlined		IS NULL SET @TextUnderlined		 = N'';
	IF @BorderToNext		IS NULL SET @BorderToNext		 = N'';
	IF @BorderToPrevious	IS NULL SET @BorderToPrevious	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Format;

		-- STEP 0.2 - Protect input parameters
		SET @FormatID			 = dbo.sx_pf_pProtectID		(@FormatID);
		SET @BackgroundColor	 = dbo.sx_pf_pProtectString (@BackgroundColor);
		SET @FontColor			 = dbo.sx_pf_pProtectString (@FontColor);
		SET @ValueFormat		 = dbo.sx_pf_pProtectString (@ValueFormat);
		SET @Bold				 = dbo.sx_pf_pProtectInt	(@Bold);
		SET @Italic				 = dbo.sx_pf_pProtectInt	(@Italic);
		SET @AlignmentHorizontal = dbo.sx_pf_pProtectString (@AlignmentHorizontal);
		SET @AlignmentVertical	 = dbo.sx_pf_pProtectString (@AlignmentVertical);
		SET @TextUnderlined		 = dbo.sx_pf_pProtectInt	(@TextUnderlined)
		SET @BorderToNext		 = dbo.sx_pf_pProtectInt	(@BorderToNext)
		SET @BorderToPrevious	 = dbo.sx_pf_pProtectInt	(@BorderToPrevious)

		IF @FormatID = N''
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		IF	@Bold				NOT IN ('<#NV>','-1','0','1') OR
			@Italic				NOT IN ('<#NV>','-1','0','1') OR
			@BorderToNext		NOT IN ('<#NV>','-1','0','1') OR
			@BorderToPrevious	NOT IN ('<#NV>','-1','0','1') 
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter for Bold, Italic, BorderToNext, BorderToPrevios not supported (outside <#NV>,-1,0,1).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentHorizontal NOT IN ('<#NV>','','left','center','right')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentHorizontal + ' for AlignmentHorizontal not supported (outside <#NV>, left, center, rights, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentVertical NOT IN ('<#NV>','','top','center','bottom')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentVertical + ' for AlignmentVertical not supported (outside <#NV>, top, center, bottom, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;


		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @FormatID;
			
		-- STEP 3.1 - If format does not yet exist, it will be created
		IF @FormatKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thFormats (FormatID,BackgroundColor,FontColor,ValueFormat,Bold,Italic,AlignmentHorizontal,AlignmentVertical	
												,TextUnderlined,BorderToNext,BorderToPrevious	
												) 
			OUTPUT INSERTED.FormatKey INTO @Keys([Key]) 
				VALUES (@FormatID,'','','','-1','-1','','','-1','0','0');

			SELECT TOP (1) @FormatKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update format attributes
		UPDATE planning.thFormats 
		SET 
			 BackgroundColor	= IIF(@BackgroundColor		= N'<#NV>', BackgroundColor,		@BackgroundColor)
			,FontColor			= IIF(@FontColor			= N'<#NV>', FontColor,				@FontColor)
			,ValueFormat		= IIF(@ValueFormat			= N'<#NV>', ValueFormat,			@ValueFormat)
			,Bold				= IIF(@Bold					= N'<#NV>', Bold,					@Bold)
			,Italic				= IIF(@Italic				= N'<#NV>', Italic,					@Italic)
			,AlignmentHorizontal= IIF(@AlignmentHorizontal	= N'<#NV>', AlignmentHorizontal,	@AlignmentHorizontal)
			,AlignmentVertical	= IIF(@AlignmentVertical	= N'<#NV>', AlignmentVertical,		@AlignmentVertical)
			,TextUnderlined		= IIF(@TextUnderlined		= N'<#NV>', TextUnderlined,			@TextUnderlined)
			,BorderToNext		= IIF(@BorderToNext			= N'<#NV>', BorderToNext,			@BorderToNext)
			,BorderToPrevious	= IIF(@BorderToPrevious		= N'<#NV>', BorderToPrevious,		@BorderToPrevious)
		FROM planning.thFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Format;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryService;
GO
/*
POST Operation for an HTML Page

10/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC			INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = ''
DECLARE @ProductLineID	NVARCHAR(255) = ''
DECLARE @FactoryID		NVARCHAR(255) = ''
DECLARE @PageID			NVARCHAR(255) = 'Page1'
DECLARE @PageName		NVARCHAR(255) = 'A great page'
DECLARE @PageContent	NVARCHAR(255) = 'Hallo Gerd'

EXECUTE @RC = dbo.sx_pf_POST_HTMLPage
		 @Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@PageID
		,@PageName
		,@PageContent

PRINT @RC

SELECT * FROM planning.tgHTMLPages

*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_HTMLPage') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_HTMLPage;
GO

CREATE PROCEDURE dbo.sx_pf_POST_HTMLPage
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PageID				NVARCHAR(255),
	@PageName			NVARCHAR(255),
	@PageContent		NVARCHAR(MAX)		

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@PageID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@PageName,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PageID			IS NULL SET @PageID			= 0;
	IF @PageName		IS NULL SET @PageName		= '';
	IF @PageContent		IS NULL SET @PageContent	= '';


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_HTMLPage;

		-- Page must have an ID, all other parameter may be empty
		IF @PageID = ''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @PageID			= dbo.sx_pf_pProtectString (@PageID);
		SET @PageName		= dbo.sx_pf_pProtectString (@PageName);


		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;	

		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;

		-- Check rights on different levels
		IF @FactoryKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Delete old PageID
		DELETE FROM planning.tgHTMLPages 
		WHERE 
				ProductKey		= @ProductKey 
			AND ProductLineKey	= @ProductLineKey
			AND FactoryKey		= @FactoryKey
			AND PageID			= @PageID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- Create new Page
		INSERT INTO planning.tgHTMLPages	(
			 ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,PageID
			,PageName
			,PageContent
			)
			VALUES (
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@PageID,
				@PageName,
				@PageContent
			);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryService;
GO
/*
POST Operation for a List
Not existing Format will be created, existing updated.
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2017 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ListID			NVARCHAR(255) = 'sxScales'
DECLARE @NameShort		NVARCHAR(255) = '<#NV>'
DECLARE @NameLong		NVARCHAR(255) = '<#NV>'
DECLARE @CommentDev		NVARCHAR(255) = '<#NV>'
DECLARE @CommentUser	NVARCHAR(255) = '<#NV>'
DECLARE @Datentyp		NVARCHAR(255) = '<#NV>'
DECLARE @Source			NVARCHAR(255) = '<#NV>'
DECLARE @SourceFormula	NVARCHAR(255) = 'Testen'
DECLARE @FormatID		NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_List 
		@Username,
		@ListID,
		@NameShort,
		@NameLong,
		@CommentDev,
		@CommentUser,
		@Datentyp,
		@Source,
		@SourceFormula,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_List') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_List;
GO

CREATE PROCEDURE dbo.sx_pf_POST_List
	 @Username		NVARCHAR(255)
	,@ListID		NVARCHAR(255)
	,@NameShort		NVARCHAR(255)
	,@NameLong		NVARCHAR(255)
	,@CommentDev	NVARCHAR(255)
	,@CommentUser	NVARCHAR(255)
	,@Datentyp		NVARCHAR(255)
	,@Source		NVARCHAR(255)
	,@SourceFormula NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username		,N'NULL')	+ N''',''' 
			+ ISNULL(@ListID		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameShort		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameLong		,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev	,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser	,N'NULL')	+ N''',''' 
			+ ISNULL(@Datentyp		,N'NULL')	+ N''',''' 
			+ ISNULL(@Source		,N'NULL')	+ N''',''' 
			+ ISNULL(@SourceFormula	,N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID		,N'NULL')	+ N'''';	
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ListID			IS NULL SET @ListID			= N'';
	IF @NameShort		IS NULL SET @NameShort		= N'';
	IF @NameLong		IS NULL SET @NameLong		= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Datentyp		IS NULL SET @Datentyp		= N'';
	IF @Source			IS NULL SET @Source			= N'';
	IF @SourceFormula	IS NULL SET @SourceFormula	= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_List;

		-- STEP 0.2 - Protect input parameters
		SET @ListID			= dbo.sx_pf_pProtectID (@ListID);
		SET @NameShort		= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong		= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser	= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString (@CommentDev);
		SET @Datentyp		= dbo.sx_pf_pProtectString (@Datentyp);
		SET @Source			= dbo.sx_pf_pProtectString (@Source);
		SET @SourceFormula	= dbo.sx_pf_pProtectString (@SourceFormula);
		SET @FormatID		= dbo.sx_pf_pProtectString (@FormatID); -- as it can be #<NV>

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @ListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

		-- STEP 3.1 - If List is does not exist, it will be created
		IF @ListKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thLists (ListID, NameShort, NameLong, CommentDev, CommentUser, Datentyp, Source, SourceFormula, FormatID) 
			OUTPUT INSERTED.ListKey INTO @Keys([Key])
			VALUES (@ListID, '', '', '', '', '', '', '', '');

			SELECT TOP(1) @ListKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update List attributes
		UPDATE planning.thLists 
		SET	 
			 NameShort		= IIF(@NameShort		= N'<#NV>', NameShort,		@NameShort)
			,NameLong		= IIF(@NameLong			= N'<#NV>', NameLong,		@NameLong)
			,CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,		@CommentDev)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser,	@CommentUser)
			,Datentyp		= IIF(@Datentyp			= N'<#NV>', Datentyp,		@Datentyp)
			,Source			= IIF(@Source			= N'<#NV>', Source,			@Source)
			,SourceFormula  = IIF(@SourceFormula	= N'<#NV>', SourceFormula,	@SourceFormula)
			,FormatID		= IIF(@FormatID			= N'<#NV>', FormatID,		@FormatID)
		FROM planning.thLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryService;
GO
/*
POST Operation for ListValue
Not existing ListValue will be created, existing updated.
This procedure allows to pass a Key value, as there is no ID for a ListValue


03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Status

Test call

DECLARE @RC				 INT
DECLARE @Username		 NVARCHAR(255)	= 'SQL'
DECLARE @ListValueKey	 INT			= 0
DECLARE @ListID			 NVARCHAR(255)	= 'sxStatus'
DECLARE @ValueInt		 INT			= 5
DECLARE @Scale			 INT			= 1
DECLARE @ValueText		 NVARCHAR(255)	= 'Testo'
DECLARE @ValueComment	 NVARCHAR(255)	= ''
DECLARE @FormatID		 NVARCHAR(255)	= ''
		

EXECUTE @RC = dbo.sx_pf_POST_ListValue 
		@Username,
		@ListValueKey,
		@ListID,
		@ValueInt,
		@Scale,
		@ValueText,
		@ValueComment,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ListValue') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ListValue
GO

CREATE PROCEDURE dbo.sx_pf_POST_ListValue
	@Username		NVARCHAR(255),
	@ListValueKey	INT,
	@ListID			NVARCHAR(255),
	@ValueInt		INT,
	@Scale			INT,
	@ValueText		NVARCHAR(255),
	@ValueComment	NVARCHAR(255),
	@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)  = N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username,								 N'NULL') + N''',' 
								+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)),	 N'NULL') + N',''' 
								+ ISNULL(@ListID,								 N'NULL') + N''',' 
								+ ISNULL(CAST(@ValueInt		AS NVARCHAR(255)),	 N'NULL') + N',' 
								+ ISNULL(CAST(@Scale		AS NVARCHAR(255)),	 N'NULL') + N',''' 
								+ ISNULL(@ValueText,							 N'NULL') + N''',''' 
								+ ISNULL(@ValueComment,							 N'NULL') + N''',''' 
								+ ISNULL(@FormatID,								 N'NULL') + N'''';	
	DECLARE @EffectedRows		 INT			= 0;				
	DECLARE @ResultCode			 INT			= 501;				
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @ListValueKey	IS NULL SET @ListValueKey	 = 0;
	IF @ListID			IS NULL SET @ListID			 = N'';
	IF @ValueInt		IS NULL SET @ValueInt		 = 0;
	IF @Scale			IS NULL SET @Scale			 = 0;
	IF @ValueText		IS NULL SET @ValueText		 = N'';
	IF @ValueComment	IS NULL SET @ValueComment	 = N'';
	IF @FormatID		IS NULL SET @FormatID		 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ListValue;

		-- STEP 0.2 - Protect input parameters
		SET @ListValueKey	= dbo.sx_pf_pProtectInt		(@ListValueKey);
		SET @ListID			= dbo.sx_pf_pProtectID		(@ListID);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);
		SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys and break if not existing
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;
		
		IF @ListKey = 0	BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check if Listvalue exists, when ListValueKey 0 is submitted - 0 means new entry, but API must check if entry is really new
		IF @ListValueKey = 0
		BEGIN
			DECLARE @tmpKey			BIGINT		  = 0;
			DECLARE @tmpListType	NVARCHAR(255) = '';
			
			SELECT @tmpListType = hL.Datentyp FROM planning.thLists hL WHERE hL.ListID = @ListID;

			IF @tmpListType = 'String'
			BEGIN
				SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueText = @ValueText;
			END
			ELSE
			BEGIN
				SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueInt = @ValueInt;
			END

			IF (@tmpKey IS NOT NULL AND @tmpKey != 0) SET @ListValueKey = @tmpKey
		END

		-- STEP 3.1 - If ListValueKey  = 0 is submitted, create new value
		IF @ListValueKey = 0
		BEGIN
			INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			VALUES (@ListID, @ValueInt, @Scale, @ValueText, @ValueComment, @FormatID);

			SET @Resultcode = 201;
			SET @EffectedRows = 1;
		END
		ELSE
		BEGIN
		-- STEP 3.2 - If existing key is committed, update the Key

			UPDATE planning.thListValues 
			SET 
				ValueInt		 = @ValueInt
				, Scale			 = @Scale
				, ValueText		 = @ValueText
				, ValueComment	 = @ValueComment
				, FormatID		 = @FormatID
			WHERE ListValueKey	 = @ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END;	

		COMMIT TRANSACTION sx_pf_POST_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryService;
GO
/*
POST Operation for Preference
Not existing Preference will be created, existing updated.

02/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
-- make Property UserDependend
EXEC dbo.sx_pf_POST_ProductlineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','1','1','2',''

-- make Property NOT UserDependend
EXEC dbo.sx_pf_POST_ProductlineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','0','1','2',''


DECLARE	 @RC					INT
DECLARE	 @Username 				NVARCHAR(255) = 'SQL'
DECLARE	 @PreferenceUserName	NVARCHAR(255) = 'AD-ONE\gtautenh'
DECLARE	 @ProductLineID			NVARCHAR(255) = 'U'
DECLARE	 @FactoryID				NVARCHAR(255) = 'ZT'
DECLARE	 @SettingName			NVARCHAR(255) = 'Demopref'
DECLARE	 @ValueInt				NVARCHAR(255) = '8'
DECLARE	 @ValueText				NVARCHAR(255) = 'uztr'

EXECUTE @RC = dbo.sx_pf_POST_Preference 
		@Username,
		@PreferenceUsername,
		@ProductLineID,
		@FactoryID,
		@SettingName,
		@ValueInt,
		@ValueText

PRINT @RC 
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Preference') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Preference;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255),
	@ValueInt			INT,
	@ValueText			NVARCHAR(MAX)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
	DECLARE @FactoryKey						INT				= 0
	DECLARE @ProductLineKey					INT				= 0
	DECLARE @PreferenceIsUserDependendFlag	INT				= 0
	DECLARE @ProcedureName					NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,							N'NULL')	+ N''',''' 
			+ ISNULL(@PreferenceUsername,				N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,					N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,						N'NULL')	+ N''',''' 
			+ ISNULL(@SettingName,						N'NULL') 	+ N''',' 
			+ ISNULL(CAST(@ValueInt AS NVARCHAR(255)),	N'NULL')	+ N',''' 
			+ ISNULL(@ValueText,						N'NULL')	+ N''''
	DECLARE @EffectedRows					INT				= 0
	DECLARE @ResultCode						INTEGER			= 501					
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP
	DECLARE @Comment						NVARCHAR(2000)	= N'';	
	DECLARE @UserKey						INT				= 0;				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';
	IF @ValueInt			IS NULL SET @ValueInt			= 0;
	IF @ValueText			IS NULL SET @ValueText			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Preference

		-- STEP 0.2 - Protect input parameters
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @SettingName	= dbo.sx_pf_pProtectString	(@SettingName);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);

		IF	@SettingName = N'' 
			OR @SettingName = '<#NV>'
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine, if Preferences are saved per User
		-- for global Preferences
		IF	@FactoryID = N'' 
			AND @ProductLineID = N''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = 'C06';
		END
		-- for Preferences per Factory
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID = ''
		BEGIN 
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = 'F06';
		END
		-- for Preferences per Productline
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID <> ''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = 'P06';
		END;

		-- STEP 1.2 - if Preferences are not Userdependend, only a clusteradmin can save them, else everybody with
		-- Factory / Productline Write Rights
		ELSE IF @PreferenceIsUserDependendFlag <> 1 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
			
			IF @ResultCode <> 200
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- STEP 1.3 - Determine Key, they may be 0 if the preference is for Cluster / Factory
		SELECT @FactoryKey	= ISNULL(FactoryKey, 0)	FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ISNULL(ProductLineKey, 0) FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		-- Cancel if Factory or Product Line does not exist
		IF	(@FactoryKey = 0 AND @FactoryID <> N'')
			OR	(@ProductlineKey = 0 AND @ProductLineID <> N'')
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Determine UserKey
		SELECT @UserKey = UserKey FROM planning.trUser WHERE Username = @PreferenceUsername

		-- STEP 1.4 - Set Preference User to "All" if Preferences are not Userdependend
		IF @PreferenceIsUserDependendFlag <> 1 
			BEGIN
				SET @PreferenceUsername = N'All';
				SET @UserKey			= 0 ;
			END
		ELSE
		-- STEP 1.5 - Test existence of Preferences Users if Preference is Userdepend
		IF	@UserKey = 0 AND @PreferenceIsUserDependendFlag <> 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User don`t exists', 16, 10);
		END;	
		
		-- STEP 2. Rights check
		IF @ProductLineID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		END
		ELSE IF @FactoryID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		END
		ELSE IF @FactoryID = N''
				AND @ProductLineID = N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		END

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3. Clear Preference and edit
		DELETE FROM planning.trPreferences
		WHERE 
			FactoryID			= @FactoryID 
			AND ProductLineID	= @ProductLineID
			AND Username		= @PreferenceUsername
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		-- TODO handle #NV - at the moment not handled
		SET @ValueText = IIF(@ValueText = N'<#NV>', N'', @ValueText);

		-- Save Preference
		INSERT INTO planning.trPreferences (UserKey,UserName, FactoryID, ProductlineID, SettingName, ValueInt, ValueText)
		VALUES (@UserKey,@PreferenceUsername, @FactoryID, @ProductLineID, @SettingName, @ValueInt, @ValueText);

		SET @EffectedRows += @@ROWCOUNT;
		SET @Resultcode = 201;

		COMMIT TRANSACTION sx_pf_POST_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryService;
GO
/*
POST Operation for one Product
Not existing Product will be created if it exists in ZT Factory. Existing Product updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight 
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) = '*'
DECLARE @ProductLineID		NVARCHAR(255) = 'U'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @TimeType			NVARCHAR(255) = '<#NV>'			--'VM'
DECLARE @NameShort			NVARCHAR(255) = 'Testproduct'
DECLARE @NameLong			NVARCHAR(255) = '3 test'
DECLARE @CommentUser		NVARCHAR(MAX) = 'komment'
DECLARE @CommentDev			NVARCHAR(MAX) = 'DEVKOmm'
DECLARE @ResponsiblePerson	NVARCHAR(255) = 'RR'
DECLARE @ImageName			NVARCHAR(255) = 'ImgName test'
DECLARE @Status				NVARCHAR(255) = 'Aktiv'
DECLARE @Template			NVARCHAR(255) = 'Unikum_VM'
DECLARE @TemplateVersion	NVARCHAR(255) = 'VT'
DECLARE @GA1				NVARCHAR(255) = 'Wert für GA1'	--'<#NV>'
DECLARE @GA2				NVARCHAR(255) = '<#NV>'
DECLARE @GA3				NVARCHAR(255) = '<#NV>'
DECLARE @GA4				NVARCHAR(255) = 'rer'
DECLARE @GA5				NVARCHAR(255) = '<#NV>'
DECLARE @GA6				NVARCHAR(255) = '<#NV>'
DECLARE @GA7				NVARCHAR(255) = '<#NV>'
DECLARE @GA8				NVARCHAR(255) = '<#NV>'
DECLARE @GA9				NVARCHAR(255) = '<#NV>'
DECLARE @GA10				NVARCHAR(255) = '<#NV>'
DECLARE @GA11				NVARCHAR(255) = '<#NV>'
DECLARE @GA12				NVARCHAR(255) = '<#NV>'
DECLARE @GA13				NVARCHAR(255) = '<#NV>'
DECLARE @GA14				NVARCHAR(255) = '<#NV>'
DECLARE @GA15				NVARCHAR(255) = '<#NV>'
DECLARE @GA16				NVARCHAR(255) = '<#NV>'
DECLARE @GA17				NVARCHAR(255) = '<#NV>'
DECLARE @GA18				NVARCHAR(255) = '<#NV>'
DECLARE @GA19				NVARCHAR(255) = '<#NV>'
DECLARE @GA20				NVARCHAR(255) = '<#NV>'
DECLARE @GA21				NVARCHAR(255) = '<#NV>'
DECLARE @GA22				NVARCHAR(255) = '<#NV>'
DECLARE @GA23				NVARCHAR(255) = '<#NV>'
DECLARE @GA24				NVARCHAR(255) = '<#NV>'
DECLARE @GA25				NVARCHAR(255) = '<#NV>'


EXECUTE @RC = dbo.sx_pf_POST_Product 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@TimeType,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName,
		@Status,
		@Template,
		@TemplateVersion,
		@GA1,
		@GA2,
		@GA3,
		@GA4,
		@GA5,
		@GA6,
		@GA7,
		@GA8,
		@GA9,
		@GA10,
		@GA11,
		@GA12,
		@GA13,
		@GA14,
		@GA15,
		@GA16,
		@GA17,
		@GA18,
		@GA19,
		@GA20,
		@GA21,
		@GA22,
		@GA23,
		@GA24,
		@GA25;

PRINT @RC;
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Product') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Product
GO

CREATE PROCEDURE dbo.sx_pf_POST_Product
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5				NVARCHAR(255),
	@GA6				NVARCHAR(255),
	@GA7				NVARCHAR(255),
	@GA8				NVARCHAR(255),
	@GA9				NVARCHAR(255),
	@GA10				NVARCHAR(255),
	@GA11				NVARCHAR(255),
	@GA12				NVARCHAR(255),
	@GA13				NVARCHAR(255),
	@GA14				NVARCHAR(255),
	@GA15				NVARCHAR(255),
	@GA16				NVARCHAR(255),
	@GA17				NVARCHAR(255),
	@GA18				NVARCHAR(255),
	@GA19				NVARCHAR(255),
	@GA20				NVARCHAR(255),
	@GA21				NVARCHAR(255),
	@GA22				NVARCHAR(255),
	@GA23				NVARCHAR(255),
	@GA24				NVARCHAR(255),
	@GA25				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductID			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')	 + N''',''' 
			+ ISNULL(@TimeType			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameShort			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameLong			, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL')	 + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL')	 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')	 + N''',''' 
			+ ISNULL(@Status			, N'NULL')	 + N''',''' 
			+ ISNULL(@Template			, N'NULL')	 + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL')	 + N''',''' 
			+ ISNULL(@GA1				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA2				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA3				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA4				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA5				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA6				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA7				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA8				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA9				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA10				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA11				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA12				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA13				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA14				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA15				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA16				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA17				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA18				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA19				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA20				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA21				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA22				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA23				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA24				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA25				, N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;						
	DECLARE @ResultCode			INT				 = 501;					
	DECLARE @TimestampCall		DATETIME		 = CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	 = N'';		

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			= N'';
	IF @ProductID			 IS NULL SET @ProductID			= N'';
	IF @ProductLineID		 IS NULL SET @ProductLineID		= N'';
	IF @FactoryID			 IS NULL SET @FactoryID			= N'';
	IF @TimeType			 IS NULL SET @TimeType			= N'';
	IF @NameShort			 IS NULL SET @NameShort			= N'';
	IF @NameLong			 IS NULL SET @NameLong			= N'';
	IF @CommentUser			 IS NULL SET @CommentUser		= N'';
	IF @CommentDev			 IS NULL SET @CommentDev		= N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson = N'';
	IF @ImageName			 IS NULL SET @ImageName			= N'';
	IF @Status				 IS NULL SET @Status			= N'';
	IF @Template			 IS NULL SET @Template			= N'';
	IF @TemplateVersion		 IS NULL SET @TemplateVersion	= N'';
	IF @GA1					 IS NULL SET @GA1				= N'';
	IF @GA2					 IS NULL SET @GA2				= N'';
	IF @GA3					 IS NULL SET @GA3 				= N'';
	IF @GA4					 IS NULL SET @GA4 				= N'';
	IF @GA5					 IS NULL SET @GA5 				= N'';
	IF @GA6					 IS NULL SET @GA6 				= N'';
	IF @GA7					 IS NULL SET @GA7 				= N'';
	IF @GA8					 IS NULL SET @GA8 				= N'';
	IF @GA9					 IS NULL SET @GA9 				= N'';
	IF @GA10				 IS NULL SET @GA10 				= N'';
	IF @GA11				 IS NULL SET @GA11 				= N'';
	IF @GA12				 IS NULL SET @GA12 				= N'';
	IF @GA13				 IS NULL SET @GA13 				= N'';
	IF @GA14				 IS NULL SET @GA14 				= N'';
	IF @GA15				 IS NULL SET @GA15 				= N'';
	IF @GA16				 IS NULL SET @GA16 				= N'';
	IF @GA17				 IS NULL SET @GA17 				= N'';
	IF @GA18				 IS NULL SET @GA18 				= N'';
	IF @GA19				 IS NULL SET @GA19 				= N'';
	IF @GA20				 IS NULL SET @GA20 				= N'';
	IF @GA21				 IS NULL SET @GA21 				= N'';
	IF @GA22				 IS NULL SET @GA22 				= N'';
	IF @GA23				 IS NULL SET @GA23 				= N'';
	IF @GA24				 IS NULL SET @GA24 				= N'';
	IF @GA25				 IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Product;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status				= dbo.sx_pf_pProtectString	(@Status);
		SET @Template			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2 				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3 				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4 				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5 				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6 				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7 				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8 				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9 				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10 				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11 				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12 				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13 				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14 				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15 				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16 				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17 				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18 				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19 				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20 				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21 				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22 				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23 				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24 				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25 				= dbo.sx_pf_pProtectString	(@GA25);

		IF	   @FactoryID		= N''
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;

		IF	@FactoryKey			 = 0 
			OR @ProductLineKey	 = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - If ProductID * is submitted, search the next free numeric ProductID
		IF @ProductID = '*' 
		BEGIN
			SELECT @ProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)),0)+1 FROM planning.tdProducts dP WHERE dP.ProductLineKey = @ProductLineKey AND TRY_CAST(dP.ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @ProductID = '*' 
			SELECT @ProductID = 1;
		END

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product dont exists, create it from the Template			
		IF @ProductKey = 0
		BEGIN
			DECLARE @TemplateFactoryID				 NVARCHAR(255)	= N'ZT';
			DECLARE @TemplateProductLineID			 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductID				 NVARCHAR(255)	= N'';
			DECLARE @TemplateFactoryKey				 INT			= 0;
			DECLARE @TemplateProductlineKey			 INT			= 0;
			DECLARE @TemplateProductKey				 INT			= 0;
			DECLARE @TemplateFactoryNameShort		 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductlineNameShort	 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductNameShort		 NVARCHAR(255)	= N'';

			-- Determine Templateproduct - it will be searched only in Factory ZT
			SELECT TOP (1) 
				@TemplateProductID		= ProductID, 
				@TemplateProductLineID	= ProductLineID 
			FROM planning.tdProducts
			WHERE 
					FactoryID = @TemplateFactoryID 
				AND Template = @Template
			ORDER BY ProductID;

			IF @TemplateProductID = N''	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template ProductID not found', 16, 10);
			END;

			-- Determine Template Keys
			SELECT 
					 @TemplateFactoryKey = FactoryKey
					,@TemplateFactoryNameShort = NameShort 
			FROM planning.tdFactories 
			WHERE FactoryID = @TemplateFactoryID;

			SELECT TOP (1)
					 @TemplateProductLineKey = ProductLineKey
					,@TemplateProductlineNameShort = NameShort 
			FROM planning.tdProductLines 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductlineID = @TemplateProductlineID
			ORDER BY ProductLineID;

			SELECT TOP (1) 
				 @TemplateProductKey = ProductKey
				,@TemplateProductNameShort = NameShort 
			FROM planning.tdProducts 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductlineKey = @TemplateProductlineKey 
				AND ProductID = @TemplateProductID
			ORDER BY ProductID;
			
			IF	   @TemplateFactoryKey		 = 0 
				OR @TemplateProductLineKey	 = 0
				OR @TemplateProductKey		 = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template keys not found', 16, 10);
			END;

			-- Copy Template Product		
			-- dProducts

			DECLARE @ProductKeyTable TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO planning.tdProducts 
			OUTPUT INSERTED.ProductKey INTO @ProductKeyTable
			SELECT @ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductlineID, 
				@FactoryID,
				TimeType, 
				@NameShort, 
				NameLong, 
				CommentUser,
				CommentDev, 
				ResponsiblePerson, 
				ImageName, 
				[Status], 
				Template, 
				TemplateVersion,
				GlobalAttribute1,
				GlobalAttribute2, 
				GlobalAttribute3, 
				GlobalAttribute4, 
				GlobalAttribute5, 
				GlobalAttribute6, 
				GlobalAttribute7,  
				GlobalAttribute8, 
				GlobalAttribute9, 
				GlobalAttribute10, 
				GlobalAttribute11, 
				GlobalAttribute12, 
				GlobalAttribute13, 
				GlobalAttribute14,  
				GlobalAttribute15, 
				GlobalAttribute16, 
				GlobalAttribute17, 
				GlobalAttribute18, 
				GlobalAttribute19, 
				GlobalAttribute20, 
				GlobalAttribute21,  
				GlobalAttribute22, 
				GlobalAttribute23, 
				GlobalAttribute24, 
				GlobalAttribute25 
			FROM planning.tdProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ProductKey = KeyNumber FROM @ProductKeyTable ORDER BY KeyNumber;

			--gProducts
			INSERT INTO planning.tgProducts 
			SELECT @ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductlineID,
				@FactoryID, 
				PropertyID, 
				PropertyName,
				CommentUser, 
				CommentDev, 
				Unit, 
				ValueText, 
				ValueInt, 
				Scale, 
				IsROSystemProperty, 
				FormatID
			FROM planning.tgProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			INSERT INTO planning.tdValueSeries 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID
				,@FactoryID 
				,ValueSeriesID 
				,ValueSeriesNo 
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,[IsNumeric] 
				,VisibilityLevel 
				,ValueSource 
				,ValueListID 
				,ValueFormatID 
				,Unit 
				,Scale 
				,Effect 
				,EffectParameter
			FROM planning.tdValueSeries 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			INSERT INTO planning.tdTime 
			SELECT
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID
				,@FactoryID 
				,TimeID 
				,FormatID
			FROM planning.tdTime 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			;WITH VS AS (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM planning.tdValueSeries
				WHERE ProductKey = @ProductKey
			)
			INSERT INTO planning.tfValues 
			SELECT 
				 VS.ValueSeriesKey 
				,@ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID 
				,@FactoryID 
				,fV.ValueSeriesID 
				,fV.TimeID 
				,fV.ValueFormula 
				,fV.ValueInt 
				,fV.ValueText 
				,fV.ValueComment
			FROM planning.tfValues fV 
				LEFT JOIN VS ON VS.ValueSeriesID= fV.ValueSeriesID 
			WHERE fV.ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements - create copy informations
			INSERT INTO planning.tfStatements 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductlineID 
				,@FactoryID 
				,N'COPY'			AS ActionType
				,N'Product produced as a copy of Factory <' 
					+ @TemplateFactoryID 
					+ N'>"' 
					+ @TemplateFactoryNameShort 
					+ N'"/ ProductLine <'
					+ @TemplateProductlineID 
					+ N'>"' 
					+ @TemplateProductLineNameShort 
					+ N'"/ Product <' 
					+ @TemplateProductID 
					+ N'>"' 
					+ @TemplateProductNameShort 
					+ N'"'			AS Statement
				,@TransactUsername	AS UserName
				,N''					AS PCName
				,N''					AS ProcessorCode
				,N''					AS IPAddresses
				,CAST(CONVERT(NVARCHAR(10), GETDATE(), 112) AS BIGINT)*10000 
					+ CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5),':',N'') AS BIGINT) 
									AS Timestamp
				,0					AS IsDeleted
				,0					AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE planning.tdProducts 
		SET 
			 TimeType			= IIF(@Template				= N'<#NV>', TimeType,			 RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			 @NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			 @NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		 @CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			 @CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	 @ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			 @ImageName)
			,[Status]			= IIF(@Status				= N'<#NV>', [Status],			 @Status)
			,Template			= IIF(@Template				= N'<#NV>', Template,			 @Template)
			,TemplateVersion	= IIF(@TemplateVersion		= N'<#NV>', TemplateVersion,	 @TemplateVersion)
			,GlobalAttribute1	= IIF(@GA1					= N'<#NV>', GlobalAttribute1,	 @GA1)
			,GlobalAttribute2	= IIF(@GA2					= N'<#NV>', GlobalAttribute2,	 @GA2)	
			,GlobalAttribute3	= IIF(@GA3					= N'<#NV>', GlobalAttribute3,	 @GA3)
			,GlobalAttribute4	= IIF(@GA4					= N'<#NV>', GlobalAttribute4,	 @GA4)	
			,GlobalAttribute5	= IIF(@GA5					= N'<#NV>', GlobalAttribute5,	 @GA5)
			,GlobalAttribute6	= IIF(@GA6					= N'<#NV>', GlobalAttribute6,	 @GA6)	
			,GlobalAttribute7	= IIF(@GA7					= N'<#NV>', GlobalAttribute7,	 @GA7)
			,GlobalAttribute8	= IIF(@GA8					= N'<#NV>', GlobalAttribute8,	 @GA8)	
			,GlobalAttribute9	= IIF(@GA9					= N'<#NV>', GlobalAttribute9,	 @GA9)
			,GlobalAttribute10	= IIF(@GA10					= N'<#NV>', GlobalAttribute10,	 @GA10)
			,GlobalAttribute11	= IIF(@GA11					= N'<#NV>', GlobalAttribute11,	 @GA11)
			,GlobalAttribute12	= IIF(@GA12					= N'<#NV>', GlobalAttribute12,	 @GA12)
			,GlobalAttribute13	= IIF(@GA13					= N'<#NV>', GlobalAttribute13,	 @GA13)
			,GlobalAttribute14	= IIF(@GA14					= N'<#NV>', GlobalAttribute14,	 @GA14)
			,GlobalAttribute15	= IIF(@GA15					= N'<#NV>', GlobalAttribute15,	 @GA15)
			,GlobalAttribute16	= IIF(@GA16					= N'<#NV>', GlobalAttribute16,	 @GA16)
			,GlobalAttribute17	= IIF(@GA17					= N'<#NV>', GlobalAttribute17,	 @GA17)
			,GlobalAttribute18	= IIF(@GA18					= N'<#NV>', GlobalAttribute18,	 @GA18)
			,GlobalAttribute19	= IIF(@GA19					= N'<#NV>', GlobalAttribute19,	 @GA19)
			,GlobalAttribute20	= IIF(@GA20					= N'<#NV>', GlobalAttribute20,	 @GA20)
			,GlobalAttribute21	= IIF(@GA21					= N'<#NV>', GlobalAttribute21,	 @GA21)
			,GlobalAttribute22	= IIF(@GA22					= N'<#NV>', GlobalAttribute22,	 @GA22)
			,GlobalAttribute23	= IIF(@GA23					= N'<#NV>', GlobalAttribute23,	 @GA23)
			,GlobalAttribute24	= IIF(@GA24					= N'<#NV>', GlobalAttribute24,	 @GA24)
			,GlobalAttribute25	= IIF(@GA25					= N'<#NV>', GlobalAttribute25,	 @GA25)	
		FROM planning.tdProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryService;
GO
/*
POST Operation for all values of the ProductDataTable
The method can be switched between incremental POST and full POST

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Concept:
- all Values are deliverd as Textarray - this requires a big split operation
- the Text array must also deliver deleted values if it does a incrementel transmisson (deleted Values are delivered with empty ValueInt, ValueText, ValueFormula)
- Values may be sended with '<#NV>' to keep existing values

Test call
1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
4. Non exists Factory / Productline / Product keys => 404 Not Found
5. If ValueSeriesID not exists in planning.tdValueSeries for this ProductKey => 403 Forbidden
6. Product updated => 200 OK

Test case 1 - Incremental Sending ***************************************************************
1. One Value for an existing Value must replace them
2. One Value for an not exiting Value must create it
3. If all Parameter are posted empty, the value is deleted
4. If one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
5. An ZeroEmpty Value in old or sended values is deleted

DECLARE @RC							INT
DECLARE @Username					NVARCHAR (255) = 'SQL'
DECLARE @ProductID					NVARCHAR (255) = '1'
DECLARE @ProductLineID				NVARCHAR (255) = 'U'
DECLARE @FactoryID					NVARCHAR (255) = 'ZT'
DECLARE @IsIncrementalValuesFlag	INT			   = 1

-- 3 Values with comments  - NEVER place a linebreak in the sample array !
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) =  '[(''K1'',20170115,''=IF(H25=4,3,2.1)'',''6'',''HaseTextValue'',''HaseComment'')],[(''K2'',20170215,''=IF(H25=4,3,2)'',''555'',''HaseTextValue'',''HaseComment2'')],[(''E1'',20170515,'''','''',''Text'',''Comment'')]'

-- Delete the three values by sending them empty
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''','''','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''','''','''')]'

-- Send an <#NV> Value for an existing numeric and an Text Value and delete one
-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''',''<#NV>'','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''',''<#NV>'','''')]'

EXECUTE @RC = [dbo.sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;
 
Test case 2 - fill all Values of an Product ************************************************
DECLARE @RC								INT
DECLARE @Username						NVARCHAR(255) = 'SQL'
DECLARE @ProductID						NVARCHAR(255) = '1'
DECLARE @ProductLineID					NVARCHAR(255) = 'U'
DECLARE @FactoryID						NVARCHAR(255) = 'ZT'
DECLARE @IsIncrementalValuesFlag		INT = 0
DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = ''
DECLARE @start_time						DATETIME

-- Generate fictive values for all TimeIDs of Products
SELECT @ValuesInBracketsCommaSeparated = @ValuesInBracketsCommaSeparated + '[(''' + dVS.ValueSeriesID +''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',''1000'',''Hase | Maus'',''Komment|ar'')],' 
FROM 
	planning.tdValueSeries dVS LEFT JOIN planning.tdTime dT
		ON dVS.ProductKey = dT.ProductKey
WHERE 
	dVS.FactoryID			= @FactoryID 
	AND dVS.ProductLineID	= @ProductLineID 
	AND dVS.ProductID		= @ProductID 

SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated) -1)
PRINT @ValuesInBracketsCommaSeparated

SET @start_time = GETDATE()
EXECUTE @RC = [dbo.sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;

SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETDATE()) AS CHAR(10))) AS 'TimeTaken'

-- Testcase 3 - delete all values
DECLARE @RC								INT
DECLARE @Username						NVARCHAR (255)	= 'SQL'
DECLARE @ProductID						NVARCHAR (255)	= '1'
DECLARE @ProductLineID					NVARCHAR (255)	= 'U'
DECLARE @FactoryID						NVARCHAR (255)	= 'ZT'
DECLARE @IsIncrementalValuesFlag		INT				= 0
DECLARE @ValuesInBracketsCommaSeparated NVARCHAR (255)	= ''

EXECUTE @RC = [dbo.sx_pf_POST_ProductDataTableValues] 
	@Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@ValuesInBracketsCommaSeparated;

PRINT @RC;
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductDataTableValues') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductDataTableValues
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductDataTableValues
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@ValuesInBracketsCommaSeparated		NVARCHAR(MAX) 
/*
@ValuesInBracketsCommaSeparated format is: (ValueSeriesID,TimeID,Formula,Value,ValueText,ValueComment),(ValueSeriesIDTimeID,Formula,Value,ValueText,ValueComment)
The whole string is single quoted and every parameter inside except the TimeID is single quoted
if you check this in SQL Server Profiler or Table sx_pf_APILog, every String must have two inverted commas, the whole Parameter must have Single inverted comma
this must show profiler: '[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')],[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')]'
In further releases the )],[( delimiter will be replaced with CHAR(29)
*/
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey 		INT				= 0;
	DECLARE @ProductKey 			INT				= 0;
	DECLARE @errorvalue				NVARCHAR(255);
	DECLARE @errormessage			NVARCHAR(2000);

	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString		NVARCHAR(MAX)	= N'''' 
									+ ISNULL(@Username													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductID													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductLineID												, N'NULL')	+ N''',''' 
									+ ISNULL(@FactoryID													, N'NULL')	+ N''','
									+ ISNULL(CAST(@IsIncrementalValuesFlag AS NVARCHAR(255))			, N'NULL')	+ N','''
									+ ISNULL(REPLACE(@ValuesInBracketsCommaSeparated, N'''', N'''''')	, N'NULL')	+ N'''';  -- WHY the replace - as there are double inverted comma's ?

	DECLARE @EffectedRows			INT				= 0;						
	DECLARE @ResultCode				INT				= 501;						
	DECLARE @TimestampCall			DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000)	= N'';			
	-- SET Debug Flag for debugging and disable Transcation 
	DECLARE @DebugFlag				INT				= 0
	/*
		--Use for debugging
		SELECT * FROM ##Debug_ValuesInBracketsReceived 
		SELECT * FROM ##Debug_ValueMergeTableFilled ORDER BY ValueSeriesID,TimeID
		SELECT * FROM ##Debug_ValueMergeTableBeforeTransmissions ORDER BY ValueSeriesID,TimeID
	*/

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag		= 0;
	IF @ValuesInBracketsCommaSeparated	IS NULL SET @ValuesInBracketsCommaSeparated = N'';

	-- To delete Values in NOT Incremental Transmission, an empty @ValuesInBracketsCommaSeparated Parameter is sended
	IF @ValuesInBracketsCommaSeparated = '' AND @IsIncrementalValuesFlag = 0 
	BEGIN
		SET @ValuesInBracketsCommaSeparated = '[(''DeleteFlagID'',99999999,'''',''0'','''','''')]'  --special ID with prevents from rejection from errorhandler
	END

	BEGIN TRY
		BEGIN TRANSACTION POST_PDT;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters, one or more IDs are empty', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductlineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF	@FactoryKey			= 0 
			OR @ProductLineKey	= 0 
			OR @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('One or more IDs dont exist (No Keys found)', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			BEGIN
				SET @errormessage = 'Invalid rights - no write Rights for ' + @TransactUsername + '.';
				RAISERROR(@errormessage, 16, 10);
			END

		-- STEP 3.0 - Store Timeline, only TimeID, as Timeline is not changed during Posting
		IF OBJECT_ID('tempdb..#Timeline') IS NOT NULL
		DROP TABLE #Timeline;	

		CREATE TABLE #Timeline (																																							
			TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT NOT NULL UNIQUE CLUSTERED
		);
				
		INSERT INTO #Timeline
		SELECT TimeID
		FROM planning.tdTime
		WHERE ProductKey = @ProductKey;

		-- STEP 3.1 - ValueMergeTable Merge the current values of the product into the ValueMergeTable
		IF OBJECT_ID('tempdb..#ValueMergeTable') IS NOT NULL
		DROP TABLE #ValueMergeTable;

		CREATE TABLE #ValueMergeTable ( 
			 ValueSeriesKey			BIGINT										NOT NULL
			,ValueSeriesID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,TimeID					NVARCHAR(10)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueFormula			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueInt				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueText				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueComment			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,[IsNumeric]			INT											NOT NULL 
			,IsFormula				INT											NOT NULL 
			,ProcessCode			NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,DocuCode				NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueSeriesID_TimeID 	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			);

		-- First Type "Old Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0												AS ValueSeriesKey  -- No Values from dValueSeries, this Informations are later added when old and sended Values are together
			,ValueSeriesID  
			,v.TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0												AS IsNumeric
			,0												AS IsFormula
			,N'OLD VALUES'									AS ProcessCode
			,N'NONE'										AS DocuCode
			,ValueSeriesID + CAST(v.TimeID AS NVARCHAR(10))	AS ValueSeries_TimeID
		FROM planning.tfValues v INNER JOIN #Timeline tl ON v.TimeID = tl.TimeID
		WHERE ProductKey = @ProductKey;		
			
		-- Store Header, which is dValueSeries Information which will added to old and sended Values
		IF OBJECT_ID('tempdb..#Header') IS NOT NULL
		DROP TABLE #Header;																			

		CREATE TABLE #Header (
			 ValueSeriesKey		BIGINT									NOT NULL
			,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[IsNumeric]		INT										NOT NULL
			,IsFormula			INT										NOT NULL
			,UNIQUE CLUSTERED (ValueSeriesID, ValueSeriesKey, [IsNumeric], IsFormula)
			);
			
		INSERT INTO #Header
		SELECT 
			ValueSeriesKey
			,ValueSeriesID
			,[IsNumeric]
			,IIF(ValueSource IN (N'XLS',N'XLS-Strict'), 1, 0) AS IsFormula 
		FROM planning.tdValueSeries 
		WHERE ProductKey = @ProductKey;

		-- STEP 3.3 - Check for count of delimiter in Array
		DECLARE @OpenDelimetersCount	INT = 0;
		DECLARE @DelimetersIndex		INT = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated);

		WHILE @DelimetersIndex > 0 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount > 32765 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueArray to big (More then 32750 Values need special API Update.)', 16, 10);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.4 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated,  LEN(@ValuesInBracketsCommaSeparated)-2);
		SET @ValuesInBracketsCommaSeparated = RIGHT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated)-2);

		CREATE TABLE #Values1  (txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values2  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL)
		CREATE TABLE #Values3  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values4  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)
		CREATE TABLE #Values5  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, ValueInt NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, txtValuesPipe NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL)
		CREATE TABLE #Values6  (ValueSeriesID NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT			NOT NULL, ValueFormula  NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL, ValueInt NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL, ValueText NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL, ValueComment NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL)

		DECLARE @spr NCHAR (1) = CHAR(29)

		-- Remove existing CHAR(29) from string
		SET @ValuesInBracketsCommaSeparated = REPLACE(@ValuesInBracketsCommaSeparated,@spr,N'');
		--PRINT @ValuesInBracketsCommaSeparated

		-- Split String into Rows - all Values except the TimeID are single quoted after that an separated by @spr																													
		INSERT INTO #Values1 
		SELECT REPLACE(REPLACE(txtValues,N',''', @spr + ''''), N''',', '''' + @spr) 
		FROM dbo.sx_pf_PivotStringIntoTable(@ValuesInBracketsCommaSeparated, N')],[(');

		-- Split string into two Columns (separate first column ValueSeriesID)
		INSERT INTO #Values2
		SELECT  
			LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values1;

		-- Split last columns once more (separate TimeID)
		INSERT INTO #Values3
		SELECT  
			ValueSeriesID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values2;

		-- Delete Implicit sended periods (dont exit in current Product)
		DELETE FROM #Values3 
		WHERE TimeID NOT IN (SELECT CAST(TimeID AS NVARCHAR(10)) FROM #Timeline);

		-- Split last columns once more (separate ValueFormula)
		INSERT INTO #Values4
		SELECT  
			ValueSeriesID
			,TimeID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values3;

		-- Split last columns once more (separate ValueInt)
		INSERT INTO #Values5
		SELECT
			ValueSeriesID  
			,TimeID
			,ValueFormula
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values4;

		-- Split last columns once more (Separate ValueText and ValueComment)
		INSERT INTO #Values6
		SELECT 
			ValueSeriesID 
			,TimeID
			,ValueFormula
			,ValueInt
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values5;

		-- Replace Quotes from all strings (all except TimeID)
		UPDATE #Values6
		SET	 
			ValueSeriesID	= RIGHT(LEFT(ValueSeriesID,	(LEN(ValueSeriesID)-1)),	LEN(ValueSeriesID)-2)
			,ValueFormula	= RIGHT(LEFT(ValueFormula,	(LEN(ValueFormula)-1)),		LEN(ValueFormula)-2)
			,ValueInt		= RIGHT(LEFT(ValueInt,		(LEN(ValueInt)-1)),			LEN(ValueInt)-2)
			,ValueText		= RIGHT(LEFT(ValueText,		(LEN(ValueText)-1)),		LEN(ValueText)-2)
			,ValueComment	= RIGHT(LEFT(ValueComment,	(LEN(ValueComment)-1)),		LEN(ValueComment)-2);

		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_Values6') IS NOT NULL DROP TABLE ##Debug_Values6;
				SELECT * INTO ##Debug_Values6 FROM #Values6;
		END;


		-- STEP 3.5 - UNION both temporary tables #######################################################################################################################

		-- Second Typ "Sended Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0 AS ValueSeriesKey
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0 AS [IsNumeric]
			,0 AS IsFormula
			,N'SENDED VALUES'
			,N'NONE'
			,ValueSeriesID+TimeID
		FROM #Values6;

		-- Add the header informations to the values
		UPDATE VMT 
		SET	
			 VMT.ValueSeriesKey = H.ValueSeriesKey
			,VMT.[IsNumeric]	= H.[IsNumeric]
			,VMT.IsFormula		= H.IsFormula 
		FROM #ValueMergeTable VMT INNER JOIN #Header H 
			ON VMT.ValueSeriesID = H.ValueSeriesID;

		-- check for not existing ValueSeriesIDs which may have been sended
		IF EXISTS(SELECT TOP (1) ValueSeriesID FROM #ValueMergeTable WHERE ValueSeriesKey = 0)
		BEGIN
			SELECT TOP(1) @errorvalue = ValueSeriesID 
			FROM #ValueMergeTable 
			WHERE ValueSeriesKey = 0 
			ORDER BY TimeID;

			IF @errorvalue = 'DeleteFlagID'
			BEGIN
				DELETE FROM #ValueMergeTable
			END
			ELSE
			BEGIN
				SET @errormessage = 'One or more sended ValueSeriesIDs were not found, e.g. '+ @errorvalue +'.'
				SET @ResultCode = 403;
				RAISERROR(@errormessage, 16, 10);
			END
		END;

		-- check if more than one value is sended for a ValueSeries/TimeID combination
		IF EXISTS(
			SELECT COUNT(TimeID) 
			FROM #ValueMergeTable 
			WHERE ProcessCode = 'SENDED VALUES'
			GROUP BY ValueSeriesID_TimeID
			HAVING COUNT(TimeID) > 1
		)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Douplicate Values found in ValueArray', 16, 10);
		END;

		-- STEP 3.6 - Prepare the ValueMergeTable #######################################################################################################################	
		-- FLAG OLD Values the user want to delete in Case of Numeric Series without formula
		-- Value 0 maybe sended as '000'
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula	= 0 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND	(TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula = 1  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode	 = N'SENDED VALUES' 
						AND ValueFormula = N''
						AND (TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series without formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 0  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND ValueText	= N''
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 1 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode		= N'SENDED VALUES' 
						AND	ValueText		= N'' 
						AND ValueFormula	= N''
				);

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableFilled') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableFilled;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableFilled 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.7 - Handle VMT #######################################################################################################################

		-- Documentation of changes in log table

			/* 

			-- NEW is a Value which is not contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'NEW VALUE'
			WHERE 
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND ValueSeriesID + N'_' + TimeID NOT IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			-- CHANGED is a Value which is contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'CHANGED VALUE'
			WHERE	
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND	ValueSeriesID + N'_' + TimeID IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			
			INSERT INTO sx_pf_AuditLog VALUES
				SELECT * FROM #ValueMergeTable WHERE 
					ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) IN 
					(
					SELECT
						ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) FROM #ValueMergeTable WHERE ProcessCode = 'SENDED VALUES' AND
						DocuCode != 'NONE'
					)
			*/

		-- Handle <#NV> Parameters - Replace them with the old Values (if exist !)

		UPDATE	VMT 
		SET		VMT.ValueInt = OV.ValueInt 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueInt, ValueSeriesID, TimeID  FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueInt LIKE N'<#NV>' 
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueText = OV.ValueText
		FROM	#ValueMergeTable VMT
			, (SELECT ValueText, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode		= N'SENDED VALUES' 
			AND VMT.ValueText = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueFormula = OV.ValueFormula 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueFormula, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueFormula = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueComment = OV.ValueComment 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueComment, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueComment = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		-- Handle <#NV> Parameter, where no old Values existed
		UPDATE #ValueMergeTable SET ValueInt		= N'0'	WHERE ValueInt		= N'<#NV>' OR [IsNumeric] = 0;
		UPDATE #ValueMergeTable SET ValueText		= N''	WHERE ValueText		= N'<#NV>' OR [IsNumeric] = 1;
		UPDATE #ValueMergeTable SET ValueFormula	= N''	WHERE ValueFormula	= N'<#NV>' OR IsFormula = 0;
		UPDATE #ValueMergeTable SET ValueComment	= N''	WHERE ValueComment	= N'<#NV>';

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableBeforeTransmissions') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableBeforeTransmissions;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableBeforeTransmissions 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.8 - Save data
		IF @IsIncrementalValuesFlag <> 1 
		BEGIN
			-- CASE FULL transmission #########################################
			-- Delete all Values
			DELETE FROM planning.tfValues WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;
			
			-- DELETE Useless NullEmpty Values in ALL Values

			-- einfache Nullwerte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		= 1 
				AND IsFormula	= 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';

			-- Numerische Formelzellen ohne Formel mit Wert 0
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]			= 1 
				AND IsFormula		= 1 
				AND ValueFormula	= '' 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';

			-- einfach Leertexte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]	  = 0 
				AND IsFormula = 0 
				AND ValueText = N'';

			-- Nichtnumerische Formelzellen ohne Formel mit Leertext
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		 = 0 
				AND IsFormula	 = 1 
				AND ValueFormula = '' 
				AND ValueText	 = N'';

			-- Insert all Values
			INSERT INTO planning.tfValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,ValueInt
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES');
				
			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE
		BEGIN
			-- CASE INCREMENTAL Transmission - it also leads to a full delete and insert !! #########################################
			-- DELETE Values, which are sended to delete in OLD VALUES
			DELETE FROM #ValueMergeTable WHERE ProcessCode = N'OLD VALUES TO DELETE'; 

			-- DELETE sended non-formula values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0;

			--empty values must delete 0 in _0 Series
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND ValueInt = ''
				AND RIGHT(ValueSeriesID,2) = '_0';
			
			-- delete deleted formulas 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 1 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0 
				AND ValueFormula = N''
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0;

			-- Delete 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES'
				AND [IsNumeric] = 0
				AND IsFormula = 0 
				AND ValueText = N''
				

			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode		 = N'SENDED VALUES' 
				AND [IsNumeric]	 = 0 
				AND IsFormula	 = 1 
				AND ValueText	 = N'' 
				AND ValueFormula = N'';

			-- DELETE Useless NullEmpty Values in OLD Values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES'
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0

			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES' 
				AND [IsNumeric] = 0 
				AND IsFormula = 0 
				AND ValueText = N'';
					 
			-- DELETE all Values
			DELETE FROM planning.tfValues
			WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- Delete OLD VALUES which are new sended
			DELETE	FROM #ValueMergeTable 
			WHERE 
				ProcessCode = N'OLD VALUES'  
				AND ValueSeriesID_TimeID IN 
					(
						SELECT ValueSeriesID_TimeID
						FROM  #ValueMergeTable 
						WHERE ProcessCode = N'SENDED VALUES'
					);

			-- Insert all Values
			INSERT INTO planning.tfValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,CAST(ValueInt AS BIGINT)
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES', N'OLD VALUES');
			
			SET @EffectedRows += @@ROWCOUNT;

		END;

		--- materialize Product, if it is in ZT-PARAM
		IF	@FactoryID = 'ZT' AND @ProductLineID LIKE 'PARAM%'
			BEGIN
				EXEC control.spParamTables @ProductID
			END
			
		COMMIT TRANSACTION POST_PDT;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION POST_PDT;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryService;
GO
/*
POST Operation for one empty Product
Not existing Product will be created, existing gets an Update
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
If product will have no TimeIDs, as it has no values
If product will have no ValueSeries, when created new 

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE	@RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) =	'53'
DECLARE @ProductLineID		NVARCHAR(255) =	'U'
DECLARE @FactoryID			NVARCHAR(255) =	'ZT'
DECLARE @TimeType			NVARCHAR(255) =	'<#NV>'--'VM'
DECLARE @NameShort			NVARCHAR(255) =	'Testproduct'
DECLARE @NameLong			NVARCHAR(255) =	'3 test'
DECLARE @CommentUser		NVARCHAR(255) =	'komment'
DECLARE @CommentDev			NVARCHAR(255) =	'DEVKOmm'
DECLARE @ResponsiblePerson	NVARCHAR(255) =	'RR'
DECLARE @ImageName			NVARCHAR(255) =	'ImgName test'
DECLARE @Status				NVARCHAR(255) =	'Aktiv'
DECLARE @Template			NVARCHAR(255) =	'Test_VM'
DECLARE @TemplateVersion	NVARCHAR(255) =	'4.0'
DECLARE @GA1				NVARCHAR(255) =	'Wert für GA1'--'<#NV>'
DECLARE @GA2				NVARCHAR(255) =	'<#NV>'
DECLARE @GA3				NVARCHAR(255) =	'<#NV>'
DECLARE @GA4				NVARCHAR(255) =	'rer'
DECLARE @GA5				NVARCHAR(255) =	'<#NV>'
DECLARE @GA6				NVARCHAR(255) =	'<#NV>'
DECLARE @GA7				NVARCHAR(255) =	'<#NV>'
DECLARE @GA8				NVARCHAR(255) =	'<#NV>'
DECLARE @GA9				NVARCHAR(255) =	'<#NV>'
DECLARE @GA10				NVARCHAR(255) =	'<#NV>'
DECLARE @GA11				NVARCHAR(255) =	'<#NV>'
DECLARE @GA12				NVARCHAR(255) =	'<#NV>'
DECLARE @GA13				NVARCHAR(255) =	'<#NV>'
DECLARE @GA14				NVARCHAR(255) =	'<#NV>'
DECLARE @GA15				NVARCHAR(255) =	'<#NV>'
DECLARE @GA16				NVARCHAR(255) =	'<#NV>'
DECLARE @GA17				NVARCHAR(255) =	'<#NV>'
DECLARE @GA18				NVARCHAR(255) =	'<#NV>'
DECLARE @GA19				NVARCHAR(255) =	'<#NV>'
DECLARE @GA20				NVARCHAR(255) =	'<#NV>'
DECLARE @GA21				NVARCHAR(255) =	'<#NV>'
DECLARE @GA22				NVARCHAR(255) =	'<#NV>'
DECLARE @GA23				NVARCHAR(255) =	'<#NV>'
DECLARE @GA24				NVARCHAR(255) =	'<#NV>'
DECLARE @GA25				NVARCHAR(255) =	'<#NV>'


EXECUTE @RC = dbo.sx_pf_POST_ProductEmpty 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@TimeType,
		@NameShort,
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ResponsiblePerson,
		@ImageName,
		@Status,
		@Template,
		@TemplateVersion,
		@GA1,
		@GA2,
		@GA3,
		@GA4,
		@GA5,
		@GA6,
		@GA7,
		@GA8,
		@GA9,
		@GA10,
		@GA11,
		@GA12,
		@GA13,
		@GA14,
		@GA15,
		@GA16,
		@GA17,
		@GA18,
		@GA19,
		@GA20,
		@GA21,
		@GA22,
		@GA23,
		@GA24,
		@GA25

PRINT @RC
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductEmpty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductEmpty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductEmpty
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5  				NVARCHAR(255),
	@GA6  				NVARCHAR(255),
	@GA7  				NVARCHAR(255),
	@GA8  				NVARCHAR(255),
	@GA9  				NVARCHAR(255),
	@GA10  				NVARCHAR(255),
	@GA11  				NVARCHAR(255),
	@GA12  				NVARCHAR(255),
	@GA13  				NVARCHAR(255),
	@GA14  				NVARCHAR(255),
	@GA15  				NVARCHAR(255),
	@GA16  				NVARCHAR(255),
	@GA17  				NVARCHAR(255),
	@GA18  				NVARCHAR(255),
	@GA19  				NVARCHAR(255),
	@GA20  				NVARCHAR(255),
	@GA21  				NVARCHAR(255),
	@GA22  				NVARCHAR(255),
	@GA23  				NVARCHAR(255),
	@GA24  				NVARCHAR(255),
	@GA25 				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @ProductLineKey		 INT			= 0;
	DECLARE @ProductKey			 INT			= 0;
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N'''' 
			+ ISNULL(@Username			, N'NULL') + N''',''' 
			+ ISNULL(@ProductID			, N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL') + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL') + N''',''' 
			+ ISNULL(@TimeType			, N'NULL') + N''',''' 
			+ ISNULL(@NameShort			, N'NULL') + N''',''' 
			+ ISNULL(@NameLong			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL') + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL') + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL') + N''',''' 
			+ ISNULL(@ImageName			, N'NULL') + N''',''' 
			+ ISNULL(@Status			, N'NULL') + N''',''' 
			+ ISNULL(@Template			, N'NULL') + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL') + N''',''' 
			+ ISNULL(@GA1				, N'NULL') + N''',''' 
			+ ISNULL(@GA2				, N'NULL') + N''',''' 
			+ ISNULL(@GA3				, N'NULL') + N''',''' 
			+ ISNULL(@GA4				, N'NULL') + N''',''' 
			+ ISNULL(@GA5				, N'NULL') + N''',''' 
			+ ISNULL(@GA6				, N'NULL') + N''',''' 
			+ ISNULL(@GA7				, N'NULL') + N''',''' 
			+ ISNULL(@GA8				, N'NULL') + N''',''' 
			+ ISNULL(@GA9				, N'NULL') + N''',''' 
			+ ISNULL(@GA10				, N'NULL') + N''',''' 
			+ ISNULL(@GA11				, N'NULL') + N''',''' 
			+ ISNULL(@GA12				, N'NULL') + N''',''' 
			+ ISNULL(@GA13				, N'NULL') + N''',''' 
			+ ISNULL(@GA14				, N'NULL') + N''',''' 
			+ ISNULL(@GA15				, N'NULL') + N''',''' 
			+ ISNULL(@GA16				, N'NULL') + N''',''' 
			+ ISNULL(@GA17				, N'NULL') + N''',''' 
			+ ISNULL(@GA18				, N'NULL') + N''',''' 
			+ ISNULL(@GA19				, N'NULL') + N''',''' 
			+ ISNULL(@GA20				, N'NULL') + N''',''' 
			+ ISNULL(@GA21				, N'NULL') + N''',''' 
			+ ISNULL(@GA22				, N'NULL') + N''',''' 
			+ ISNULL(@GA23				, N'NULL') + N''',''' 
			+ ISNULL(@GA24				, N'NULL') + N''',''' 
			+ ISNULL(@GA25				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;				-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;				-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username 			IS NULL SET @Username			= N'';
	IF @ProductID 			IS NULL SET @ProductID			= N'';
	IF @ProductLineID 		IS NULL SET @ProductLineID		= N'';
	IF @FactoryID 			IS NULL SET @FactoryID			= N'';
	IF @TimeType 			IS NULL SET @TimeType			= N'';
	IF @NameShort 			IS NULL SET @NameShort			= N'';
	IF @NameLong 			IS NULL SET @NameLong			= N'';
	IF @CommentUser			IS NULL SET @CommentUser		= N'';
	IF @CommentDev			IS NULL SET @CommentDev 		= N'';
	IF @ResponsiblePerson	IS NULL SET @ResponsiblePerson  = N'';
	IF @ImageName 			IS NULL SET @ImageName			= N'';
	IF @Status 				IS NULL SET @Status				= N'';
	IF @Template 			IS NULL SET @Template			= N'';
	IF @TemplateVersion 	IS NULL SET @TemplateVersion	= N'';
	IF @GA1 				IS NULL SET @GA1				= N'';
	IF @GA2  				IS NULL SET @GA2				= N'';
	IF @GA3  				IS NULL SET @GA3				= N'';
	IF @GA4  				IS NULL SET @GA4				= N'';
	IF @GA5  				IS NULL SET @GA5				= N'';
	IF @GA6  				IS NULL SET @GA6 				= N'';
	IF @GA7  				IS NULL SET @GA7 				= N'';
	IF @GA8  				IS NULL SET @GA8 				= N'';
	IF @GA9  				IS NULL SET @GA9 				= N'';
	IF @GA10  				IS NULL SET @GA10 				= N'';
	IF @GA11  				IS NULL SET @GA11 				= N'';
	IF @GA12  				IS NULL SET @GA12 				= N'';
	IF @GA13  				IS NULL SET @GA13 				= N'';
	IF @GA14  				IS NULL SET @GA14 				= N'';
	IF @GA15  				IS NULL SET @GA15 				= N'';
	IF @GA16  				IS NULL SET @GA16 				= N'';
	IF @GA17  				IS NULL SET @GA17 				= N'';
	IF @GA18  				IS NULL SET @GA18 				= N'';
	IF @GA19  				IS NULL SET @GA19 				= N'';
	IF @GA20  				IS NULL SET @GA20 				= N'';
	IF @GA21  				IS NULL SET @GA21 				= N'';
	IF @GA22  				IS NULL SET @GA22 				= N'';
	IF @GA23  				IS NULL SET @GA23 				= N'';
	IF @GA24  				IS NULL SET @GA24 				= N'';
	IF @GA25  				IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductEmpty;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID 			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType 			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort 			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong 			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser 		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName 			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status  			= dbo.sx_pf_pProtectString	(@Status);
		SET @Template  			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1  				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2  				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3  				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4  				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5  				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6  				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7  				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8  				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9  				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10  				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11  				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12  				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13  				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14  				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15  				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16  				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17  				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18  				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19  				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20  				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21  				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22  				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23  				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24  				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25  				= dbo.sx_pf_pProtectString	(@GA25);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @FactoryKey		AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductlineKey	= @ProductlineKey	AND ProductID = @ProductID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
			
		-- STEP 3.1 - If Product dont exists, create a new row in dProducts	
		IF @ProductKey = 0
		BEGIN
			--Handle wrong time types, cut them to two characters
			IF	LEN(@TimeType) > 2 
				SET @TimeType = LEFT(@TimeType,2);

			--Create dProducts
			DECLARE @ProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO planning.tdProducts  
			OUTPUT Inserted.ProductKey INTO @ProductKeyTable 
			VALUES (@ProductLineKey
					,@FactoryKey
					,@ProductID
					,@ProductlineID
					,@FactoryID
					,@TimeType
					,@NameShort
					,@NameLong
					,@CommentUser
					,@CommentDev
					,@ResponsiblePerson
					,@ImageName
					,@Status
					,@Template
					,@TemplateVersion
					,IIF(@GA1  = N'<#NV>', N'', @GA1)
					,IIF(@GA2  = N'<#NV>', N'', @GA2)
					,IIF(@GA3  = N'<#NV>', N'', @GA3)
					,IIF(@GA4  = N'<#NV>', N'', @GA4)
					,IIF(@GA5  = N'<#NV>', N'', @GA5)
					,IIF(@GA6  = N'<#NV>', N'', @GA6)
					,IIF(@GA7  = N'<#NV>', N'', @GA7)
					,IIF(@GA8  = N'<#NV>', N'', @GA8)
					,IIF(@GA9  = N'<#NV>', N'', @GA9)
					,IIF(@GA10 = N'<#NV>', N'', @GA10)
					,IIF(@GA11 = N'<#NV>', N'', @GA11)
					,IIF(@GA12 = N'<#NV>', N'', @GA12)
					,IIF(@GA13 = N'<#NV>', N'', @GA13)
					,IIF(@GA14 = N'<#NV>', N'', @GA14)
					,IIF(@GA15 = N'<#NV>', N'', @GA15)
					,IIF(@GA16 = N'<#NV>', N'', @GA16)
					,IIF(@GA17 = N'<#NV>', N'', @GA17)
					,IIF(@GA18 = N'<#NV>', N'', @GA18)
					,IIF(@GA19 = N'<#NV>', N'', @GA19)
					,IIF(@GA20 = N'<#NV>', N'', @GA20)
					,IIF(@GA21 = N'<#NV>', N'', @GA21)
					,IIF(@GA22 = N'<#NV>', N'', @GA22)
					,IIF(@GA23 = N'<#NV>', N'', @GA23)
					,IIF(@GA24 = N'<#NV>', N'', @GA24)
					,IIF(@GA25 = N'<#NV>', N'', @GA25));

			SET @EffectedRows += @@ROWCOUNT;

			SELECT @ProductKey = KeyNumber 
			FROM @ProductKeyTable;

			--fStatements - create copy  informations
			INSERT INTO planning.tfStatements
			SELECT 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID,  
				@ProductlineID, 
				@FactoryID, 
				N'COPY' AS ActionType,
				N'Product imported by Single Product Import ' AS Statement,
				@TransactUsername AS UserName,
				N'' AS PCName, 
				N'' AS ProcessorCode, 
				N'' AS IPAddresses,
				CAST(CONVERT(NVARCHAR(10),GETDATE(),112) as BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10),GETDATE(),108), 5),N':',N'') AS BIGINT) AS Timestamp,
				0 AS IsDeleted, 
				0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201;
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE planning.tdProducts 
		SET  
			 TimeType			 =  IIF(@Template			 = N'<#NV>', TimeType,			RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			 =  IIF(@NameShort			 = N'<#NV>', NameShort,			@NameShort)
			,NameLong			 =  IIF(@NameLong			 = N'<#NV>', NameLong,			@NameLong)
			,CommentUser		 =  IIF(@CommentUser		 = N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			 =  IIF(@CommentDev			 = N'<#NV>', CommentDev,		@CommentDev)
			,ResponsiblePerson	 =  IIF(@ResponsiblePerson	 = N'<#NV>', ResponsiblePerson, @ResponsiblePerson)
			,ImageName			 =  IIF(@ImageName			 = N'<#NV>', ImageName,			@ImageName)
			,Status				 =  IIF(@Status				 = N'<#NV>', Status,			@Status)
			,Template			 =  IIF(@Template			 = N'<#NV>', Template,			@Template)
			,TemplateVersion	 =  IIF(@TemplateVersion	 = N'<#NV>', TemplateVersion,	@TemplateVersion)
			,GlobalAttribute1	 =  IIF(@GA1				 = N'<#NV>', GlobalAttribute1,	@GA1)
			,GlobalAttribute2	 =  IIF(@GA2 				 = N'<#NV>', GlobalAttribute2,	@GA2)	
			,GlobalAttribute3	 =  IIF(@GA3 				 = N'<#NV>', GlobalAttribute3,	@GA3)
			,GlobalAttribute4	 =  IIF(@GA4 				 = N'<#NV>', GlobalAttribute4,	@GA4)	
			,GlobalAttribute5	 =  IIF(@GA5 				 = N'<#NV>', GlobalAttribute5,	@GA5)
			,GlobalAttribute6	 =  IIF(@GA6 				 = N'<#NV>', GlobalAttribute6,	@GA6)	
			,GlobalAttribute7	 =  IIF(@GA7 				 = N'<#NV>', GlobalAttribute7,	@GA7)
			,GlobalAttribute8	 =  IIF(@GA8 				 = N'<#NV>', GlobalAttribute8,	@GA8)	
			,GlobalAttribute9	 =  IIF(@GA9 				 = N'<#NV>', GlobalAttribute9,	@GA9)
			,GlobalAttribute10	 =  IIF(@GA10 				 = N'<#NV>', GlobalAttribute10, @GA10)
			,GlobalAttribute11	 =  IIF(@GA11 				 = N'<#NV>', GlobalAttribute11, @GA11)
			,GlobalAttribute12	 =  IIF(@GA12 				 = N'<#NV>', GlobalAttribute12, @GA12)
			,GlobalAttribute13	 =  IIF(@GA13 				 = N'<#NV>', GlobalAttribute13, @GA13)
			,GlobalAttribute14	 =  IIF(@GA14 				 = N'<#NV>', GlobalAttribute14, @GA14)
			,GlobalAttribute15	 =  IIF(@GA15 				 = N'<#NV>', GlobalAttribute15, @GA15)
			,GlobalAttribute16	 =  IIF(@GA16 				 = N'<#NV>', GlobalAttribute16, @GA16)
			,GlobalAttribute17	 =  IIF(@GA17 				 = N'<#NV>', GlobalAttribute17, @GA17)
			,GlobalAttribute18	 =  IIF(@GA18 				 = N'<#NV>', GlobalAttribute18, @GA18)
			,GlobalAttribute19	 =  IIF(@GA19 				 = N'<#NV>', GlobalAttribute19, @GA19)
			,GlobalAttribute20	 =  IIF(@GA20 				 = N'<#NV>', GlobalAttribute20, @GA20)
			,GlobalAttribute21	 =  IIF(@GA21 				 = N'<#NV>', GlobalAttribute21, @GA21)
			,GlobalAttribute22	 =  IIF(@GA22 				 = N'<#NV>', GlobalAttribute22, @GA22)
			,GlobalAttribute23	 =  IIF(@GA23 				 = N'<#NV>', GlobalAttribute23, @GA23)
			,GlobalAttribute24	 =  IIF(@GA24 				 = N'<#NV>', GlobalAttribute24, @GA24)
			,GlobalAttribute25	 =  IIF(@GA25				 = N'<#NV>', GlobalAttribute25, @GA25)	
		FROM planning.tdProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;	

		COMMIT TRANSACTION sx_pf_POST_ProductEmpty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductEmpty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductLine
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pGET_FactoryWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. Empty FactoryID / ProductLineID  => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
3. Username does not have write access to Factory on Create => 401 Unauthorized
5. Non exists Factory / Productline keys => 404 Not Found
6. ProductLine created => 201 Created
7. ProductLine updated => 200 OK

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @ProductLineID				NVARCHAR(255) = 'ee'
DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
DECLARE @NameShort					NVARCHAR(255) = ''
DECLARE @NameLong					NVARCHAR(255) = ''
DECLARE @CommentUser				NVARCHAR(255) = ''
DECLARE @CommentDev					NVARCHAR(255) = ''
DECLARE @ResponsiblePerson			NVARCHAR(255) = ''
DECLARE @ImageName					NVARCHAR(255) = ''
DECLARE @DefaultTemplate			NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource1		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias1		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource2		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias2		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource3		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias3		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource4		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias4		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource5		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias5		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource6		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias6		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource7		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias7		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource8		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias8		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource9		NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias9		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource10	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias10		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource11	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias11		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource12	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias12		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource13	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias13		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource14	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias14		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource15	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias15		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource16	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias16		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource17	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias17		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource18	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias18		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource19	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias19		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource20	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias20		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource21	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias21		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource22	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias22		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource23	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias23		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource24	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias24		NVARCHAR(255) = ''
DECLARE @GlobalAttributeSource25	NVARCHAR(255) = ''
DECLARE @GlobalAttributeAlias25		NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_ProductLine @Username, @ProductLineID, @FactoryID, @NameShort, @NameLong, @CommentUser, @CommentDev, @ResponsiblePerson, @ImageName, @DefaultTemplate, 
		@GlobalAttributeSource1, @GlobalAttributeAlias1, @GlobalAttributeSource2, @GlobalAttributeAlias2, @GlobalAttributeSource3, @GlobalAttributeAlias3, @GlobalAttributeSource4, 
		@GlobalAttributeAlias4, @GlobalAttributeSource5, @GlobalAttributeAlias5, @GlobalAttributeSource6, @GlobalAttributeAlias6, @GlobalAttributeSource7, @GlobalAttributeAlias7, 
		@GlobalAttributeSource8, @GlobalAttributeAlias8, @GlobalAttributeSource9, @GlobalAttributeAlias9, @GlobalAttributeSource10, @GlobalAttributeAlias10, @GlobalAttributeSource11, 
		@GlobalAttributeAlias11, @GlobalAttributeSource12, @GlobalAttributeAlias12, @GlobalAttributeSource13, @GlobalAttributeAlias13, @GlobalAttributeSource14, @GlobalAttributeAlias14, 
		@GlobalAttributeSource15, @GlobalAttributeAlias15, @GlobalAttributeSource16, @GlobalAttributeAlias16, @GlobalAttributeSource17, @GlobalAttributeAlias17, @GlobalAttributeSource18, 
		@GlobalAttributeAlias18, @GlobalAttributeSource19, @GlobalAttributeAlias19, @GlobalAttributeSource20, @GlobalAttributeAlias20, @GlobalAttributeSource21, @GlobalAttributeAlias21, 
		@GlobalAttributeSource22, @GlobalAttributeAlias22, @GlobalAttributeSource23, @GlobalAttributeAlias23, @GlobalAttributeSource24, @GlobalAttributeAlias24, @GlobalAttributeSource25, 
		@GlobalAttributeAlias25

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductLine') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductLine;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductLine
	@Username					NVARCHAR(255),
	@ProductLineID				NVARCHAR(255),
	@FactoryID					NVARCHAR(255),
	@NameShort					NVARCHAR(255),
	@NameLong					NVARCHAR(255),
	@CommentUser				NVARCHAR(MAX),
	@CommentDev					NVARCHAR(MAX),
	@ResponsiblePerson			NVARCHAR(255),
	@ImageName					NVARCHAR(255),
	@DefaultTemplate			NVARCHAR(255),
	@GlobalAttributeSource1		NVARCHAR(255),
	@GlobalAttributeAlias1		NVARCHAR(255),
	@GlobalAttributeSource2		NVARCHAR(255),
	@GlobalAttributeAlias2		NVARCHAR(255),
	@GlobalAttributeSource3		NVARCHAR(255),
	@GlobalAttributeAlias3		NVARCHAR(255),
	@GlobalAttributeSource4		NVARCHAR(255),
	@GlobalAttributeAlias4		NVARCHAR(255),
	@GlobalAttributeSource5		NVARCHAR(255),
	@GlobalAttributeAlias5		NVARCHAR(255),
	@GlobalAttributeSource6		NVARCHAR(255),
	@GlobalAttributeAlias6		NVARCHAR(255),
	@GlobalAttributeSource7		NVARCHAR(255),
	@GlobalAttributeAlias7		NVARCHAR(255),
	@GlobalAttributeSource8		NVARCHAR(255),
	@GlobalAttributeAlias8		NVARCHAR(255),
	@GlobalAttributeSource9		NVARCHAR(255),
	@GlobalAttributeAlias9		NVARCHAR(255),
	@GlobalAttributeSource10	NVARCHAR(255),
	@GlobalAttributeAlias10		NVARCHAR(255),
	@GlobalAttributeSource11	NVARCHAR(255),
	@GlobalAttributeAlias11		NVARCHAR(255),
	@GlobalAttributeSource12	NVARCHAR(255),
	@GlobalAttributeAlias12		NVARCHAR(255),
	@GlobalAttributeSource13	NVARCHAR(255),
	@GlobalAttributeAlias13		NVARCHAR(255),
	@GlobalAttributeSource14	NVARCHAR(255),
	@GlobalAttributeAlias14		NVARCHAR(255),
	@GlobalAttributeSource15	NVARCHAR(255),
	@GlobalAttributeAlias15		NVARCHAR(255),
	@GlobalAttributeSource16	NVARCHAR(255),
	@GlobalAttributeAlias16		NVARCHAR(255),
	@GlobalAttributeSource17	NVARCHAR(255),
	@GlobalAttributeAlias17		NVARCHAR(255),
	@GlobalAttributeSource18	NVARCHAR(255),
	@GlobalAttributeAlias18		NVARCHAR(255),
	@GlobalAttributeSource19	NVARCHAR(255),
	@GlobalAttributeAlias19		NVARCHAR(255),
	@GlobalAttributeSource20	NVARCHAR(255),
	@GlobalAttributeAlias20		NVARCHAR(255),
	@GlobalAttributeSource21	NVARCHAR(255),
	@GlobalAttributeAlias21		NVARCHAR(255),
	@GlobalAttributeSource22	NVARCHAR(255),
	@GlobalAttributeAlias22		NVARCHAR(255),
	@GlobalAttributeSource23	NVARCHAR(255),
	@GlobalAttributeAlias23		NVARCHAR(255),
	@GlobalAttributeSource24	NVARCHAR(255),
	@GlobalAttributeAlias24		NVARCHAR(255),
	@GlobalAttributeSource25	NVARCHAR(255),
	@GlobalAttributeAlias25		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)= N'';
	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProcedureName		NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = 
			N''''	   + ISNULL(@Username,					N'NULL')
			+ N''',''' + ISNULL(@ProductLineID,				N'NULL') + N''',''' + ISNULL(@FactoryID,				N'NULL') + N''',''' + ISNULL(@NameShort,				N'NULL')
			+ N''',''' + ISNULL(@NameLong,					N'NULL') + N''',''' + ISNULL(@CommentUser,				N'NULL') + N''',''' + ISNULL(@CommentDev,				N'NULL') 
			+ N''',''' + ISNULL(@ResponsiblePerson,			N'NULL') + N''',''' + ISNULL(@ImageName,				N'NULL') + N''',''' + ISNULL(@DefaultTemplate,			N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource2,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias2,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource3,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias3,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource5,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias5,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource6,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias6,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource8,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias8,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource9,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias9,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource11,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias11,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource12,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias12,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource14,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias14,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource15,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias15,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource17,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias17,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource18,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias18,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource20,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias20,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource21,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias21,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource23,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias23,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource24,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias24,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource25,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias25,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'';
	IF @FactoryID				IS NULL SET @FactoryID					= N'';
	IF @NameShort				IS NULL SET @NameShort					= N'';
	IF @NameLong				IS NULL SET @NameLong					= N'';
	IF @CommentUser				IS NULL SET @CommentUser				= N'';
	IF @CommentDev				IS NULL SET @CommentDev					= N'';
	IF @ResponsiblePerson		IS NULL SET @ResponsiblePerson			= N'';
	IF @ImageName				IS NULL SET @ImageName					= N'';
	IF @DefaultTemplate			IS NULL SET @DefaultTemplate			= N'';
	IF @GlobalAttributeSource1	IS NULL SET @GlobalAttributeSource1		= N'';
	IF @GlobalAttributeAlias1	IS NULL SET @GlobalAttributeAlias1		= N'';
	IF @GlobalAttributeSource2	IS NULL SET @GlobalAttributeSource2		= N'';
	IF @GlobalAttributeAlias2	IS NULL SET @GlobalAttributeAlias2		= N'';
	IF @GlobalAttributeSource3	IS NULL SET @GlobalAttributeSource3		= N'';
	IF @GlobalAttributeAlias3	IS NULL SET @GlobalAttributeAlias3		= N'';
	IF @GlobalAttributeSource4	IS NULL SET @GlobalAttributeSource4		= N'';
	IF @GlobalAttributeAlias4	IS NULL SET @GlobalAttributeAlias4		= N'';
	IF @GlobalAttributeSource5	IS NULL SET @GlobalAttributeSource5		= N'';
	IF @GlobalAttributeAlias5	IS NULL SET @GlobalAttributeAlias5		= N'';
	IF @GlobalAttributeSource6	IS NULL SET @GlobalAttributeSource6		= N'';
	IF @GlobalAttributeAlias6	IS NULL SET @GlobalAttributeAlias6		= N'';
	IF @GlobalAttributeSource7	IS NULL SET @GlobalAttributeSource7		= N'';
	IF @GlobalAttributeAlias7	IS NULL SET @GlobalAttributeAlias7		= N'';
	IF @GlobalAttributeSource8	IS NULL SET @GlobalAttributeSource8		= N'';
	IF @GlobalAttributeAlias8	IS NULL SET @GlobalAttributeAlias8		= N'';
	IF @GlobalAttributeSource9	IS NULL SET @GlobalAttributeSource9		= N'';
	IF @GlobalAttributeAlias9	IS NULL SET @GlobalAttributeAlias9		= N'';
	IF @GlobalAttributeSource10 IS NULL SET @GlobalAttributeSource10	= N'';
	IF @GlobalAttributeAlias10	IS NULL SET @GlobalAttributeAlias10		= N'';
	IF @GlobalAttributeSource11 IS NULL SET @GlobalAttributeSource11	= N'';
	IF @GlobalAttributeAlias11	IS NULL SET @GlobalAttributeAlias11		= N'';
	IF @GlobalAttributeSource12 IS NULL SET @GlobalAttributeSource12	= N'';
	IF @GlobalAttributeAlias12	IS NULL SET @GlobalAttributeAlias12		= N'';
	IF @GlobalAttributeSource13 IS NULL SET @GlobalAttributeSource13	= N'';
	IF @GlobalAttributeAlias13	IS NULL SET @GlobalAttributeAlias13		= N'';
	IF @GlobalAttributeSource14 IS NULL SET @GlobalAttributeSource14	= N'';
	IF @GlobalAttributeAlias14	IS NULL SET @GlobalAttributeAlias14		= N'';
	IF @GlobalAttributeSource15 IS NULL SET @GlobalAttributeSource15	= N'';
	IF @GlobalAttributeAlias15	IS NULL SET @GlobalAttributeAlias15		= N'';
	IF @GlobalAttributeSource16 IS NULL SET @GlobalAttributeSource16	= N'';
	IF @GlobalAttributeAlias16	IS NULL SET @GlobalAttributeAlias16		= N'';
	IF @GlobalAttributeSource17 IS NULL SET @GlobalAttributeSource17	= N'';
	IF @GlobalAttributeAlias17	IS NULL SET @GlobalAttributeAlias17		= N'';
	IF @GlobalAttributeSource18 IS NULL SET @GlobalAttributeSource18	= N'';
	IF @GlobalAttributeAlias18	IS NULL SET @GlobalAttributeAlias18		= N'';
	IF @GlobalAttributeSource19 IS NULL SET @GlobalAttributeSource19	= N'';
	IF @GlobalAttributeAlias19	IS NULL SET @GlobalAttributeAlias19		= N'';
	IF @GlobalAttributeSource20 IS NULL SET @GlobalAttributeSource20	= N'';
	IF @GlobalAttributeAlias20	IS NULL SET @GlobalAttributeAlias20		= N'';
	IF @GlobalAttributeSource21 IS NULL SET @GlobalAttributeSource21	= N'';
	IF @GlobalAttributeAlias21	IS NULL SET @GlobalAttributeAlias21		= N'';
	IF @GlobalAttributeSource22 IS NULL SET @GlobalAttributeSource22	= N'';
	IF @GlobalAttributeAlias22	IS NULL SET @GlobalAttributeAlias22		= N'';
	IF @GlobalAttributeSource23 IS NULL SET @GlobalAttributeSource23	= N'';
	IF @GlobalAttributeAlias23	IS NULL SET @GlobalAttributeAlias23		= N'';
	IF @GlobalAttributeSource24 IS NULL SET @GlobalAttributeSource24	= N'';
	IF @GlobalAttributeAlias24	IS NULL SET @GlobalAttributeAlias24		= N'';
	IF @GlobalAttributeSource25 IS NULL SET @GlobalAttributeSource25	= N'';
	IF @GlobalAttributeAlias25	IS NULL SET @GlobalAttributeAlias25		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLine;
		
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID			= dbo.sx_pf_pProtectID     (@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID	   (@FactoryID);
		SET @NameShort				= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong				= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser			= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev				= dbo.sx_pf_pProtectString (@CommentDev);
		SET @ResponsiblePerson		= dbo.sx_pf_pProtectString (@ResponsiblePerson);
		SET @ImageName				= dbo.sx_pf_pProtectString (@ImageName);
		SET @DefaultTemplate		= dbo.sx_pf_pProtectString (@DefaultTemplate);
		SET @GlobalAttributeAlias1	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias1);
		SET @GlobalAttributeAlias2	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias2);
		SET @GlobalAttributeAlias3	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias3);
		SET @GlobalAttributeAlias4	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias4);
		SET @GlobalAttributeAlias5	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias5);
		SET @GlobalAttributeAlias6	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias6);
		SET @GlobalAttributeAlias7	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias7);
		SET @GlobalAttributeAlias8	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias8);
		SET @GlobalAttributeAlias9	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias9);
		SET @GlobalAttributeAlias10 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias10);
		SET @GlobalAttributeAlias11 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias11);
		SET @GlobalAttributeAlias12 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias12);
		SET @GlobalAttributeAlias13 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias13);
		SET @GlobalAttributeAlias14 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias14);
		SET @GlobalAttributeAlias15 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias15);
		SET @GlobalAttributeAlias16 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias16);
		SET @GlobalAttributeAlias17 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias17);
		SET @GlobalAttributeAlias18 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias18);
		SET @GlobalAttributeAlias19 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias19);
		SET @GlobalAttributeAlias20 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias20);
		SET @GlobalAttributeAlias21 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias21);
		SET @GlobalAttributeAlias22 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias22);
		SET @GlobalAttributeAlias23 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias23);
		SET @GlobalAttributeAlias24 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias24);
		SET @GlobalAttributeAlias25 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias25);

		IF @GlobalAttributeSource1	!='<#NV>' SET @GlobalAttributeSource1  = dbo.sx_pf_pProtectID (@GlobalAttributeSource1)
		IF @GlobalAttributeSource2	!='<#NV>' SET @GlobalAttributeSource2  = dbo.sx_pf_pProtectID (@GlobalAttributeSource2)
		IF @GlobalAttributeSource3	!='<#NV>' SET @GlobalAttributeSource3  = dbo.sx_pf_pProtectID (@GlobalAttributeSource3)
		IF @GlobalAttributeSource4	!='<#NV>' SET @GlobalAttributeSource4  = dbo.sx_pf_pProtectID (@GlobalAttributeSource4)
		IF @GlobalAttributeSource5	!='<#NV>' SET @GlobalAttributeSource5  = dbo.sx_pf_pProtectID (@GlobalAttributeSource5)
		IF @GlobalAttributeSource6	!='<#NV>' SET @GlobalAttributeSource6  = dbo.sx_pf_pProtectID (@GlobalAttributeSource6)
		IF @GlobalAttributeSource7	!='<#NV>' SET @GlobalAttributeSource7  = dbo.sx_pf_pProtectID (@GlobalAttributeSource7)
		IF @GlobalAttributeSource8	!='<#NV>' SET @GlobalAttributeSource8  = dbo.sx_pf_pProtectID (@GlobalAttributeSource8)
		IF @GlobalAttributeSource9	!='<#NV>' SET @GlobalAttributeSource9  = dbo.sx_pf_pProtectID (@GlobalAttributeSource9)
		IF @GlobalAttributeSource10 !='<#NV>' SET @GlobalAttributeSource10 = dbo.sx_pf_pProtectID (@GlobalAttributeSource10)
		IF @GlobalAttributeSource11 !='<#NV>' SET @GlobalAttributeSource11 = dbo.sx_pf_pProtectID (@GlobalAttributeSource11)
		IF @GlobalAttributeSource12 !='<#NV>' SET @GlobalAttributeSource12 = dbo.sx_pf_pProtectID (@GlobalAttributeSource12)
		IF @GlobalAttributeSource13 !='<#NV>' SET @GlobalAttributeSource13 = dbo.sx_pf_pProtectID (@GlobalAttributeSource13)
		IF @GlobalAttributeSource14 !='<#NV>' SET @GlobalAttributeSource14 = dbo.sx_pf_pProtectID (@GlobalAttributeSource14)
		IF @GlobalAttributeSource15 !='<#NV>' SET @GlobalAttributeSource15 = dbo.sx_pf_pProtectID (@GlobalAttributeSource15)
		IF @GlobalAttributeSource16 !='<#NV>' SET @GlobalAttributeSource16 = dbo.sx_pf_pProtectID (@GlobalAttributeSource16)
		IF @GlobalAttributeSource17 !='<#NV>' SET @GlobalAttributeSource17 = dbo.sx_pf_pProtectID (@GlobalAttributeSource17)
		IF @GlobalAttributeSource18 !='<#NV>' SET @GlobalAttributeSource18 = dbo.sx_pf_pProtectID (@GlobalAttributeSource18)
		IF @GlobalAttributeSource19 !='<#NV>' SET @GlobalAttributeSource19 = dbo.sx_pf_pProtectID (@GlobalAttributeSource19)
		IF @GlobalAttributeSource20 !='<#NV>' SET @GlobalAttributeSource20 = dbo.sx_pf_pProtectID (@GlobalAttributeSource20)
		IF @GlobalAttributeSource21 !='<#NV>' SET @GlobalAttributeSource21 = dbo.sx_pf_pProtectID (@GlobalAttributeSource21)
		IF @GlobalAttributeSource22 !='<#NV>' SET @GlobalAttributeSource22 = dbo.sx_pf_pProtectID (@GlobalAttributeSource22)
		IF @GlobalAttributeSource23 !='<#NV>' SET @GlobalAttributeSource23 = dbo.sx_pf_pProtectID (@GlobalAttributeSource23)
		IF @GlobalAttributeSource24 !='<#NV>' SET @GlobalAttributeSource24 = dbo.sx_pf_pProtectID (@GlobalAttributeSource24)
		IF @GlobalAttributeSource25 !='<#NV>' SET @GlobalAttributeSource25 = dbo.sx_pf_pProtectID (@GlobalAttributeSource25)

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;

		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		IF @ProductLineKey = 0
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		ELSE
			EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product Line does not yet exist, it will be created in minimal configuration		
		IF @ProductLineKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tdProductLines (FactoryKey, ProductlineID, FactoryID, NameShort, NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName, DefaultTemplate
				, GlobalattributeSource1,  GlobalattributeAlias1,  GlobalattributeSource2,  GlobalattributeAlias2,  GlobalattributeSource3,  GlobalattributeAlias3,  GlobalattributeSource4,  GlobalattributeAlias4,  GlobalattributeSource5,  GlobalattributeAlias5
				, GlobalattributeSource6,  GlobalattributeAlias6,  GlobalattributeSource7,  GlobalattributeAlias7,  GlobalattributeSource8,  GlobalattributeAlias8,  GlobalattributeSource9,  GlobalattributeAlias9,  GlobalattributeSource10, GlobalattributeAlias10
				, GlobalattributeSource11, GlobalattributeAlias11, GlobalattributeSource12, GlobalattributeAlias12, GlobalattributeSource13, GlobalattributeAlias13, GlobalattributeSource14, GlobalattributeAlias14, GlobalattributeSource15, GlobalattributeAlias15
				, GlobalattributeSource16, GlobalattributeAlias16, GlobalattributeSource17, GlobalattributeAlias17, GlobalattributeSource18, GlobalattributeAlias18, GlobalattributeSource19, GlobalattributeAlias19, GlobalattributeSource20, GlobalattributeAlias20
				, GlobalattributeSource21, GlobalattributeAlias21, GlobalattributeSource22, GlobalattributeAlias22, GlobalattributeSource23, GlobalattributeAlias23, GlobalattributeSource24, GlobalattributeAlias24, GlobalattributeSource25, GlobalattributeAlias25
			) 
			OUTPUT INSERTED.ProductLineKey INTO @Keys([Key])  
			VALUES (@FactoryKey, @ProductLineID, @FactoryID, @NameShort, '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
			);

			SELECT TOP (1) @ProductLineKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode	  = 201;
		END;

		-- STEP 3.2 - Update values
		UPDATE planning.tdProductLines
		SET 
			NameShort					= IIF(@NameShort				= N'<#NV>', NameShort,				 @NameShort)
			, NameLong					= IIF(@NameLong					= N'<#NV>', NameLong,				 @NameLong)
			, CommentUser				= IIF(@CommentUser				= N'<#NV>', CommentUser,			 @CommentUser)
			, CommentDev				= IIF(@CommentDev				= N'<#NV>', CommentDev,				 @CommentDev)
			, ResponsiblePerson			= IIF(@ResponsiblePerson		= N'<#NV>', ResponsiblePerson,		 @ResponsiblePerson)
			, ImageName					= IIF(@ImageName				= N'<#NV>', ImageName,				 @ImageName)
			, DefaultTemplate			= IIF(@DefaultTemplate			= N'<#NV>', DefaultTemplate,		 @DefaultTemplate)
			, GlobalAttributeSource1	= IIF(@GlobalAttributeSource1	= N'<#NV>', GlobalAttributeSource1,	 @GlobalAttributeSource1)
			, GlobalAttributeAlias1		= IIF(@GlobalAttributeAlias1	= N'<#NV>', GlobalAttributeAlias1,	 @GlobalAttributeAlias1)			
			, GlobalAttributeSource2	= IIF(@GlobalAttributeSource2	= N'<#NV>', GlobalAttributeSource2,	 @GlobalAttributeSource2)
			, GlobalAttributeAlias2		= IIF(@GlobalAttributeAlias2	= N'<#NV>', GlobalAttributeAlias2,	 @GlobalAttributeAlias2)
			, GlobalAttributeSource3	= IIF(@GlobalAttributeSource3	= N'<#NV>', GlobalAttributeSource3,	 @GlobalAttributeSource3)
			, GlobalAttributeAlias3		= IIF(@GlobalAttributeAlias3	= N'<#NV>', GlobalAttributeAlias3,	 @GlobalAttributeAlias3)
			, GlobalAttributeSource4	= IIF(@GlobalAttributeSource4	= N'<#NV>', GlobalAttributeSource4,	 @GlobalAttributeSource4)
			, GlobalAttributeAlias4		= IIF(@GlobalAttributeAlias4	= N'<#NV>', GlobalAttributeAlias4,	 @GlobalAttributeAlias4)
			, GlobalAttributeSource5	= IIF(@GlobalAttributeSource5	= N'<#NV>', GlobalAttributeSource5,	 @GlobalAttributeSource5)
			, GlobalAttributeAlias5		= IIF(@GlobalAttributeAlias5	= N'<#NV>', GlobalAttributeAlias5,	 @GlobalAttributeAlias5)
			, GlobalAttributeSource6	= IIF(@GlobalAttributeSource6	= N'<#NV>', GlobalAttributeSource6,	 @GlobalAttributeSource6)
			, GlobalAttributeAlias6		= IIF(@GlobalAttributeAlias6	= N'<#NV>', GlobalAttributeAlias6,	 @GlobalAttributeAlias6)
			, GlobalAttributeSource7	= IIF(@GlobalAttributeSource7	= N'<#NV>', GlobalAttributeSource7,	 @GlobalAttributeSource7)
			, GlobalAttributeAlias7		= IIF(@GlobalAttributeAlias7	= N'<#NV>', GlobalAttributeAlias7,	 @GlobalAttributeAlias7)
			, GlobalAttributeSource8	= IIF(@GlobalAttributeSource8	= N'<#NV>', GlobalAttributeSource8,	 @GlobalAttributeSource8)
			, GlobalAttributeAlias8		= IIF(@GlobalAttributeAlias8	= N'<#NV>', GlobalAttributeAlias8,	 @GlobalAttributeAlias8)
			, GlobalAttributeSource9	= IIF(@GlobalAttributeSource9	= N'<#NV>', GlobalAttributeSource9,	 @GlobalAttributeSource9)
			, GlobalAttributeAlias9		= IIF(@GlobalAttributeAlias9	= N'<#NV>', GlobalAttributeAlias9,	 @GlobalAttributeAlias9)
			, GlobalAttributeSource10	= IIF(@GlobalAttributeSource10	= N'<#NV>', GlobalAttributeSource10, @GlobalAttributeSource10)
			, GlobalAttributeAlias10	= IIF(@GlobalAttributeAlias10	= N'<#NV>', GlobalAttributeAlias10,	 @GlobalAttributeAlias10)
			, GlobalAttributeSource11	= IIF(@GlobalAttributeSource11	= N'<#NV>', GlobalAttributeSource11, @GlobalAttributeSource11)
			, GlobalAttributeAlias11	= IIF(@GlobalAttributeAlias11	= N'<#NV>', GlobalAttributeAlias11,	 @GlobalAttributeAlias11)			
			, GlobalAttributeSource12	= IIF(@GlobalAttributeSource12	= N'<#NV>', GlobalAttributeSource12, @GlobalAttributeSource12)
			, GlobalAttributeAlias12	= IIF(@GlobalAttributeAlias12	= N'<#NV>', GlobalAttributeAlias12,	 @GlobalAttributeAlias12)
			, GlobalAttributeSource13	= IIF(@GlobalAttributeSource13	= N'<#NV>', GlobalAttributeSource13, @GlobalAttributeSource13)
			, GlobalAttributeAlias13	= IIF(@GlobalAttributeAlias13	= N'<#NV>', GlobalAttributeAlias13,	 @GlobalAttributeAlias13)
			, GlobalAttributeSource14	= IIF(@GlobalAttributeSource14	= N'<#NV>', GlobalAttributeSource14, @GlobalAttributeSource14)
			, GlobalAttributeAlias14	= IIF(@GlobalAttributeAlias14	= N'<#NV>', GlobalAttributeAlias14,	 @GlobalAttributeAlias14)
			, GlobalAttributeSource15	= IIF(@GlobalAttributeSource15	= N'<#NV>', GlobalAttributeSource15, @GlobalAttributeSource15)
			, GlobalAttributeAlias15	= IIF(@GlobalAttributeAlias15	= N'<#NV>', GlobalAttributeAlias15,	 @GlobalAttributeAlias15)
			, GlobalAttributeSource16	= IIF(@GlobalAttributeSource16	= N'<#NV>', GlobalAttributeSource16, @GlobalAttributeSource16)
			, GlobalAttributeAlias16	= IIF(@GlobalAttributeAlias16	= N'<#NV>', GlobalAttributeAlias16,	 @GlobalAttributeAlias16)
			, GlobalAttributeSource17	= IIF(@GlobalAttributeSource17	= N'<#NV>', GlobalAttributeSource17, @GlobalAttributeSource17)
			, GlobalAttributeAlias17	= IIF(@GlobalAttributeAlias17	= N'<#NV>', GlobalAttributeAlias17,	 @GlobalAttributeAlias17)
			, GlobalAttributeSource18	= IIF(@GlobalAttributeSource18	= N'<#NV>', GlobalAttributeSource18, @GlobalAttributeSource18)
			, GlobalAttributeAlias18	= IIF(@GlobalAttributeAlias18	= N'<#NV>', GlobalAttributeAlias18,	 @GlobalAttributeAlias18)
			, GlobalAttributeSource19	= IIF(@GlobalAttributeSource19	= N'<#NV>', GlobalAttributeSource19, @GlobalAttributeSource19)
			, GlobalAttributeAlias19	= IIF(@GlobalAttributeAlias19	= N'<#NV>', GlobalAttributeAlias19,	 @GlobalAttributeAlias19)
			, GlobalAttributeSource20	= IIF(@GlobalAttributeSource20	= N'<#NV>', GlobalAttributeSource20, @GlobalAttributeSource20)
			, GlobalAttributeAlias20	= IIF(@GlobalAttributeAlias20	= N'<#NV>', GlobalAttributeAlias20,	 @GlobalAttributeAlias20)
			, GlobalAttributeSource21	= IIF(@GlobalAttributeSource21	= N'<#NV>', GlobalAttributeSource21, @GlobalAttributeSource21)
			, GlobalAttributeAlias21	= IIF(@GlobalAttributeAlias21	= N'<#NV>', GlobalAttributeAlias21,	 @GlobalAttributeAlias21)			
			, GlobalAttributeSource22	= IIF(@GlobalAttributeSource22	= N'<#NV>', GlobalAttributeSource22, @GlobalAttributeSource22)
			, GlobalAttributeAlias22	= IIF(@GlobalAttributeAlias22	= N'<#NV>', GlobalAttributeAlias22,	 @GlobalAttributeAlias22)
			, GlobalAttributeSource23	= IIF(@GlobalAttributeSource23	= N'<#NV>', GlobalAttributeSource23, @GlobalAttributeSource23)
			, GlobalAttributeAlias23	= IIF(@GlobalAttributeAlias23	= N'<#NV>', GlobalAttributeAlias23,	 @GlobalAttributeAlias23)
			, GlobalAttributeSource24	= IIF(@GlobalAttributeSource24	= N'<#NV>', GlobalAttributeSource24, @GlobalAttributeSource24)
			, GlobalAttributeAlias24	= IIF(@GlobalAttributeAlias24	= N'<#NV>', GlobalAttributeAlias24,	 @GlobalAttributeAlias24)
			, GlobalAttributeSource25	= IIF(@GlobalAttributeSource25	= N'<#NV>', GlobalAttributeSource25, @GlobalAttributeSource25)
			, GlobalAttributeAlias25	= IIF(@GlobalAttributeAlias25	= N'<#NV>', GlobalAttributeAlias25,	 @GlobalAttributeAlias25)
		FROM planning.tdProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (only possible for users with Factory WriteRight or Cluster Admin)
		EXEC dbo.sx_pf_materialize_vUserRights;

		COMMIT TRANSACTION sx_pf_POST_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductLineProperty
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectInt
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
5. Property created => 201 Created
6. Property updated => 200 OK

DECLARE @RC					INT
DECLARE @Username			 NVARCHAR(255) = 'SQL'
DECLARE @ProductLineID		 NVARCHAR(255) = 'U'
DECLARE @FactoryID			 NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			 NVARCHAR(255) = 'rreo'
DECLARE @PropertyName		 NVARCHAR(255) = ''
DECLARE @CommentUser		 NVARCHAR(255) = ''
DECLARE @CommentDev			 NVARCHAR(255) = ''
DECLARE @Unit				 NVARCHAR(255) = ''
DECLARE @ValueText			 NVARCHAR(255) = ''
DECLARE @ValueInt			 NVARCHAR(255) = '678'
DECLARE @Scale				 NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty  NVARCHAR(255) = '1'
DECLARE @FormatID			 NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ProductLineProperty 
		@Username,
		@ProductLineID,
		@FactoryID,
		@PropertyID,	
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,	
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductLineProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductLineProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductLineProperty
	@Username			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PropertyID			NVARCHAR(255),
	@PropertyName		NVARCHAR(255),
	@CommentUser		NVARCHAR(255),
	@CommentDev			NVARCHAR(255),
	@Unit 				NVARCHAR(255),
	@ValueText 			NVARCHAR(255),
	@ValueInt 			NVARCHAR(255),
	@Scale				NVARCHAR(255),
	@IsROSystemProperty NVARCHAR(255),
	@FormatID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,	N'NULL') + N''',''' + ISNULL(@PropertyName,		N'NULL') + N''',''' + ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,	N'NULL') + N''',''' + ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,		N'NULL') + N''',''' + ISNULL(@Scale,			N'NULL') + N''',''' + ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLineProperty;
				
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @FormatID		= dbo.sx_pf_pProtectString	(@FormatID); --as it can be #<NV>

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE ProductLineKey =  @ProductLineKey AND PropertyID = @PropertyID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgProductLines WHERE ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgProductLines (ProductlineKey, FactoryKey, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductLineKey, @FactoryKey, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE planning.tgProductLines 
		SET 
			PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			, CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			, CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			, Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			, ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			, ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			, Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			, IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			, FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for ProductProperty
Not existing ProductLine will be created, existing gets an Update
If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

Dependencies:
	- Functions: 
		- dbo.sx_pf_pProtectString
		- dbo.sx_pf_pProtectID
		- dbo.sx_pf_pProtectInt
		- dbo.sx_pf_Determine_TransactionUsername
	 - Stored Procedures:
		- sx_pf_pGET_ProductLineWriteRight
		- sx_pf_pPOST_API_LogEntry

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
1. PropertyID is empty => 404 Not Found
2. TransactionUsername not determined for User => 403 Forbidden
3. Username does not have write access to ProductLine => 401 Unauthorized
5. Property created => 201 Created
6. Property updated => 200 OK

DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @ProductID			NVARCHAR(255) = '1'
DECLARE @ProductLineID		NVARCHAR(255) = 'U'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
DECLARE @PropertyID			NVARCHAR(255) = 'rreo'
DECLARE @PropertyName		NVARCHAR(255) = ''
DECLARE @CommentUser		NVARCHAR(255) = ''
DECLARE @CommentDev			NVARCHAR(255) = ''
DECLARE @Unit				NVARCHAR(255) = ''
DECLARE @ValueText			NVARCHAR(255) = ''
DECLARE @ValueInt			NVARCHAR(255) = '678'
DECLARE @Scale				NVARCHAR(255) = '<#NV>'
DECLARE @IsROSystemProperty	NVARCHAR(255) = '1'
DECLARE @FormatID			NVARCHAR(255) = '<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ProductProperty 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@PropertyID,
		@PropertyName,
		@CommentUser,
		@CommentDev,
		@Unit,
		@ValueText,
		@ValueInt,
		@Scale,
		@IsROSystemProperty,
		@FormatID

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ProductProperty') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ProductProperty;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ProductProperty
	@Username				NVARCHAR(255),
	@ProductID				NVARCHAR(255),
	@ProductLineID			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
	@PropertyID				NVARCHAR(255),
	@PropertyName			NVARCHAR(255),
	@CommentUser			NVARCHAR(255),
	@CommentDev				NVARCHAR(255),
	@Unit					NVARCHAR(255),
	@ValueText				NVARCHAR(255),
	@ValueInt				NVARCHAR(255),
	@Scale					NVARCHAR(255),
	@IsROSystemProperty		NVARCHAR(255),
	@FormatID				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyName,			N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,			N'NULL') + N''',''' 
			+ ISNULL(@Unit,					N'NULL') + N''',''' 
			+ ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,				N'NULL') + N''',''' 
			+ ISNULL(@Scale,				N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,				N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductProperty;
			
		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		IF @FormatID <> '<#NV>'
			SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		--SET @PropertyOwner = dbo.sx_pf_pProtectBoolean (@IsROSystemProperty); -- NO Protection at the moment, as value can be 0,1,2

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey		= PropertyKey		FROM planning.tgProducts		WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;
		
		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0 
			OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgProducts WHERE ProductKey =  @ProductKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration				
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgProducts (ProductKey, ProductLineKey, FactoryKey, ProductID, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID)
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductKey, @ProductLineKey, @FactoryKey, @ProductID, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');
			
			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @Resultcode = 201;
			SET @EffectedRows = 1;
		END

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE planning.tgProducts
		SET  PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,		@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,				@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,			@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,			@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,				@Scale)
			,IsROSystemProperty = IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty, @IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,			@FormatID)
		FROM planning.tgProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryService;
GO
/*
POST Operation for UserRight
UserRight will be created always, not fitting other rights will be deleted to ensure enheritance

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC						INT
DECLARE @UserName				NVARCHAR(255) = 'SQL'
DECLARE @PostUserName			NVARCHAR(255) = 'Hase'
DECLARE @FactoryID				NVARCHAR(255) = 'ZT'
DECLARE @ProductLineID			NVARCHAR(255) = 'U'
DECLARE @Right					NVARCHAR(255) = 'Write'
DECLARE @ReadCommentMandatory	NVARCHAR(255) = ''
DECLARE @WriteCommentMandatory	NVARCHAR(255) = ''
		

EXECUTE @RC = dbo.sx_pf_POST_Right 
		@UserName,
		@PostUserName,
		@FactoryID,
        @ProductLineID,
        @Right,
        @ReadCommentMandatory,
        @WriteCommentMandatory

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Right') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Right;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Right
	@UserName				NVARCHAR(255),
	@PostUserName			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
    @ProductLineID			NVARCHAR(255),
    @Right					NVARCHAR(255),
    @ReadCommentMandatory	NVARCHAR(255),
    @WriteCommentMandatory  NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @PostUserKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ClusterRight		NVARCHAR(255)	= N'';
	DECLARE @FactoryRight		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'ISNULL') + N''',''' + ISNULL(@PostUserName,			N'ISNULL') + N''',''' 
			+ ISNULL(@FactoryID,				N'ISNULL') + N''',''' + ISNULL(@ProductLineID,			N'ISNULL') + N''',''' 
			+ ISNULL(@Right,					N'ISNULL') + N''',''' + ISNULL(@ReadCommentMandatory,	N'ISNULL') + N''',''' 
			+ ISNULL(@WriteCommentMandatory,	N'ISNULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @PostUserName			IS NULL SET @PostUserName			= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @Right					IS NULL SET @Right					= N'';
	IF @ReadCommentMandatory	IS NULL SET @ReadCommentMandatory	= N'';
	IF @WriteCommentMandatory	IS NULL SET @WriteCommentMandatory	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Right;	

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName			= dbo.sx_pf_pProtectString	(@PostUserName);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @Right					= dbo.sx_pf_pProtectString	(@Right);
		SET @ReadCommentMandatory	= dbo.sx_pf_pProtectBoolean (@ReadCommentMandatory);
		SET @WriteCommentMandatory	= dbo.sx_pf_pProtectBoolean (@WriteCommentMandatory);
		
		-- STEP 0.3 - Clean IDs
		SET @Right			= LTRIM(RTRIM(@Right));
		SET @PostUserName	= LTRIM(RTRIM(@PostUserName));
		SET @FactoryID		= LTRIM(RTRIM(@FactoryID));
		SET @ProductLineID	= LTRIM(RTRIM(@ProductLineID));

		IF	@Right = N'' 
			OR @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
				
		-- STEP 1.1 - It`s forbidden to post rights for system users with name 'SQL'
		IF	@PostUserName = N'SQL' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden to post rights for system users reserved name.', 16, 10, @PostUserName);
		END;	

		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to post rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to post rights', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 2.1 - It`s forbidden for a ClusterAdmin to change his own rights
		IF @PostUserName = @TransactUsername	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden for a ClusterAdmin to change his own rights', 16, 10);
		END;

		-- STEP 2.2 - Empty factory ID must always be connected to empty ProductLineID
		IF	@FactoryID = N'' 
			AND @ProductLineID <> N''
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Empty factory ID must always be connected to empty ProductLineID', 16, 10);
		END;
		
		-- STEP 2.3 - Check keys
		IF @FactoryID <> N''
		BEGIN
			SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @FactoryKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		IF @ProductLineID <> N'' 
		BEGIN
			SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
			
			IF @ProductLineKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		SELECT @PostUserKey = UserKey FROM planning.trUser WHERE Username = @PostUserName;

		IF @PostUserKey  = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys not exists', 16, 10);
		END;

		-- STEP 3 - Set Rule
		IF @Right = N'Deny'
		BEGIN
			-- 'Deny' will delete a right, but never create an entry

			IF @FactoryID = N''
				DELETE FROM planning.trRights WHERE UserKey = @PostUserKey;
			ELSE IF @ProductLineID = N''
				DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
			ELSE
				DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 204;
		END
		ELSE
		BEGIN
			-- Saves only explicit rights on objects and deletes rights which get invalid through right definition on upper levels
			-- Check always Clusteright 
			
			SELECT TOP (1) @ClusterRight = COALESCE([Right], N'None') 
			FROM planning.trRights 
			WHERE 
				UserKey = @PostUserKey 
				AND FactoryID = N''
			ORDER BY [Right];

			SET @ResultCode = 204;

			-- CLUSTER RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			-- Write / Read -> delete all other Rights and insert
			IF	@FactoryID = N'' AND @ProductLineID = N''
			BEGIN
				DELETE FROM planning.trRights WHERE UserKey = @PostUserKey;

				SET @EffectedRows += @@ROWCOUNT;

				INSERT INTO planning.trRights
						(UserKey
						,UserName
						,FactoryID
						,ProductLineID
						,ProductID
						,[Right]
						,ReadCommentMandatory
						,WriteCommentMandatory)
				VALUES  (@PostUserKey			-- UserKey					int
						,@PostUserName			-- UserName					nvarchar(255)
						,N''					-- FactoryID				nvarchar(255)
						,N''					-- ProductLineID			nvarchar(255)
						,N''					-- ProductID				nvarchar(255)
						,@Right					-- Right					nvarchar(255)
						,@ReadCommentMandatory  -- ReadCommentMandatory		int
						,@WriteCommentMandatory -- WriteCommentMandatory	int
						);
				
				SET @EffectedRows += @@ROWCOUNT;
				SET @ResultCode = 200;
			END;

			-- FACTORY RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN
				-- Write -> delete all other Right in this Factory, dont accept if user has write on Cluster
				IF @Right = N'Write' AND @ClusterRight <> N'Write'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO planning.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  (@PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if no cluster rights set -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO planning.trRights
							(UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if Clusterwrite rights -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight = N'Read'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END
			END;

			-- PRODUCTLINE RIGHT is posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN
				-- Look for Rights on Factory Level
				SELECT TOP (1)  @FactoryRight = COALESCE([Right], N'None') 
				FROM planning.trRights 
				WHERE 
						UserKey = @PostUserKey 
					AND FactoryID = @FactoryID 
					AND ProductLineID = N''
				ORDER BY [Right];

				-- Write -> dont accept if user has already write on Cluster or Factory
				IF @Right = N'Write' AND @ClusterRight <> N'Write' AND @FactoryRight <> N'Write'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO planning.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,@ProductLineID			-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read -> don`t accept if user has read or write on Cluster or Factory
				IF	@Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read' 
					AND @FactoryRight <> N'Write' AND @FactoryRight <> N'Read'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO planning.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey				-- UserKey					int
							,@PostUserName				-- UserName					nvarchar(255)
							,@FactoryID					-- FactoryID				nvarchar(255)
							,@ProductLineID				-- ProductLineID			nvarchar(255)
							,N''						-- ProductID				nvarchar(255)
							,@Right						-- Right					nvarchar(255)
							,@ReadCommentMandatory		-- ReadCommentMandatory		int
							,@WriteCommentMandatory		-- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;
						
				-- Read -> if read should only kill an existing write in Factories with global Read Rights, just delete Write Right
				IF @Right = N'Read' AND @FactoryRight = N'Read'
				BEGIN
					DELETE FROM planning.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;
				END;
			END;
		END;

		COMMIT TRANSACTION sx_pf_POST_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Right TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Right TO pf_PlanningFactoryService;
GO
/*
POST Operation for a new Statement
A Statement can only created, there is no Update or Delete

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = '1'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ActionType		NVARCHAR(255) = 'SAVE'
DECLARE @Statement		NVARCHAR(255) = 'ALLES GUT'
DECLARE @PCName			NVARCHAR(255) = ''
DECLARE @ProcessorCode	NVARCHAR(255) = ''
DECLARE @IPAddresses	NVARCHAR(255) = ''
DECLARE @IsDeleted		NVARCHAR(255) = ''
DECLARE @IsResolved		NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_Statement 
		@Username,	
		@ProductID,	
		@ProductLineID,	
		@FactoryID,	
		@ActionType,
		@Statement,	
		@PCName,
		@ProcessorCode,	
		@IPAddresses,
		@IsDeleted,
		@IsResolved

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Statement') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Statement;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Statement
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ActionType		NVARCHAR(255),
	@Statement		NVARCHAR(255),
	@PCName			NVARCHAR(255),
	@ProcessorCode	NVARCHAR(255),
	@IPAddresses	NVARCHAR(255),
	@IsDeleted		NVARCHAR(255),
	@IsResolved		NVARCHAR(255)	
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,	N'NULL') + N''',''' + ISNULL(@ActionType,		N'NULL') + N''',''' + ISNULL(@Statement,		N'NULL') + N''',''' 
			+ ISNULL(@PCName,		N'NULL') + N''',''' + ISNULL(@ProcessorCode,	N'NULL') + N''',''' + ISNULL(@IPAddresses,		N'NULL') + N''',''' 
			+ ISNULL(@IsDeleted,	N'NULL') + N''',''' + ISNULL(@IsResolved,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ActionType		IS NULL SET @ActionType		= N'';
	IF @Statement		IS NULL SET @Statement		= N'';
	IF @PCName			IS NULL SET @PCName			= N'';
	IF @ProcessorCode	IS NULL SET @ProcessorCode	= N'';
	IF @IPAddresses		IS NULL SET @IPAddresses	= N'';
	IF @IsDeleted		IS NULL SET @IsDeleted		= N'';
	IF @IsResolved		IS NULL SET @IsResolved		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Statement

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ActionType		= dbo.sx_pf_pProtectString	(@ActionType);
		SET @Statement		= dbo.sx_pf_pProtectString	(@Statement);
		SET @PCName			= dbo.sx_pf_pProtectString	(@PCName);
		SET @ProcessorCode	= dbo.sx_pf_pProtectString	(@ProcessorCode);
		SET @IPAddresses	= dbo.sx_pf_pProtectString	(@IPAddresses);
		SET @IsDeleted		= dbo.sx_pf_pProtectBoolean (@IsDeleted);
		SET @IsResolved		= dbo.sx_pf_pProtectBoolean (@IsResolved);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
				
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3 - Insert new Statement
		INSERT INTO planning.tfStatements (
			ProductKey,
			ProductLineKey,
			FactoryKey,
			ProductID,
			ProductLineID,
			FactoryID,
			ActionType,
			Statement,
			UserName,
			PCName,
			ProcessorCode,
			IPAddresses,
			Timestamp,
			IsDeleted,
			IsResolved) 
		VALUES
			(@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@ActionType,
			@Statement,
			@TransactUsername,
			@PCName,
			@ProcessorCode,
			@IPAddresses,
			CAST(YEAR(CURRENT_TIMESTAMP) AS BIGINT)*10000000000 + CAST(MONTH(CURRENT_TIMESTAMP) as BIGINT)*100000000 + DAY(CURRENT_TIMESTAMP)*1000000 + DATEPART(hh,CURRENT_TIMESTAMP)*10000 + DATEPART(MINUTE,CURRENT_TIMESTAMP) *100 + DATEPART(ss, CURRENT_TIMESTAMP),
			@IsDeleted,
			@IsResolved);

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_POST_Statement;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Statement;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryService;
GO
/*
POST Operation to apply a template (again) on a Product
Existing Values in the Product will be saved, new/other logic will be added
It means the Template inherits its properties again to the object

The logical process is

1. replace the Target Product by a new copy of the Template
2. transfer all global attributes whitout changing
2. Import all Values from ValueSeries which are of type "Input" in the Template to the Template


03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC								INT 
DECLARE @Username						NVARCHAR(255) = 'SQL'
DECLARE @ProductID						NVARCHAR(255) = '2'
DECLARE @ProductLineID					NVARCHAR(255) = 'U'
DECLARE @FactoryID						NVARCHAR(255) = 'ZT'

DECLARE @TemplateProductID				NVARCHAR(255) = '1'
DECLARE @TemplateProductLineID			NVARCHAR(255) = 'U'
DECLARE @TemplateFactoryID				NVARCHAR(255) = 'ZT'
DECLARE @CompareByNameInsteadOfIDFlag	INT = 0

EXECUTE @RC = dbo.sx_pf_POST_Template 

   @Username
  ,@ProductID
  ,@ProductLineID
  ,@FactoryID
  ,@TemplateProductID
  ,@TemplateProductLineID
  ,@TemplateFactoryID
  ,@CompareByNameInsteadOfIDFlag

PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Template') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Template
GO

CREATE PROCEDURE dbo.sx_pf_POST_Template
	@Username						NVARCHAR(255),
	@ProductID						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN
	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @FactoryKey						INT				= 0;
	DECLARE @ProductLineKey					INT				= 0;
	DECLARE @ProductKey						INT				= 0;
	DECLARE @TemplateFactoryKey				INT				= 0;
	DECLARE @TemplateProductLineKey			INT				= 0;
	DECLARE @TemplateProductKey				INT				= 0;
	DECLARE @TemplateFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateProductLineNameShort	NVARCHAR(255)	= N'';
	DECLARE @TemplateProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateName					NVARCHAR(255)	= N'';
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username,					N'NULL') + N''',''' 
														+ ISNULL(@ProductID,				N'NULL') + N''',''' 
														+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
														+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
														+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
														+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows					INT				= 0;				
	DECLARE @ResultCode						INT				= 501;				
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment						NVARCHAR(2000)	= N'';				


	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductlineID			= dbo.sx_pf_pProtectID	(@TemplateProductlineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);
	
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 0.3. - Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @TemplateFactoryKey AND ProductlineID = @TemplateProductlineID;

		-- Stop on invalid Keys
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @TemplateFactoryKey = 0 OR @TemplateProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		 -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN

			EXEC @ResultCode = dbo.sx_pf_pGET_ProductlineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode <> 200
				RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 2.1 - Try to identify the product keys
		SELECT @ProductKey			= ProductKey FROM planning.tdProducts WHERE ProductlineKey = @ProductlineKey AND ProductID = @ProductID;
		SELECT @TemplateProductKey	= ProductKey FROM planning.tdProducts WHERE ProductlineKey = @TemplateProductlineKey AND ProductID = @TemplateProductID;
		
		-- Stop if Keys don't exist
		IF @ProductKey = 0 OR @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- Stop if Template is posted on itself
		IF @ProductKey = @TemplateProductKey
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Stop, as Template cant be posted on itself.', 16, 10);
		END;

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in Template
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND	EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @TemplateProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Template', 16, 10);
		END

		-- STEP 2.3 - Stop, if compare by name and there are doubled ValueSerieNames in Product
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @ProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Target Product', 16, 10);
		END

		-- STEP 3.1 - Determine Template values
		SELECT @TemplateFactoryNameShort		= NameShort FROM planning.tdFactories		WHERE FactoryKey = @FactoryKey;
		SELECT @TemplateProductLineNameShort	= NameShort FROM planning.tdProductLines	WHERE ProductLineKey = @ProductLineKey;
		SELECT @TemplateProductNameShort		= NameShort FROM planning.tdProducts		WHERE ProductKey = @ProductKey;
		SELECT @TemplateName					= Template	FROM planning.tdProducts		WHERE ProductKey = @TemplateProductKey;

		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			  ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			, Scale				INT											NOT NULL
			, TimeID			INT											NOT NULL
			, ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueInt			BIGINT										NOT NULL
			, ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
			, ValueComment		NVARCHAR (MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Store Data of Product in Storage (but not formulas, they come from template)
		INSERT INTO #ProductDataStorage    
		SELECT 
			fV.ValueSeriesID
			, dVS.NameShort AS ValueSeriesName
			, dVS.Scale
			, fV.TimeID
			, COALESCE(fVT.ValueFormula,'') AS ValueFormula -- the new formulas will be catched from the template 
			, fV.ValueInt
			, fV.ValueText
			, fV.ValueComment 
		FROM 
			planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
				ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
			-- the new Template Formulas
			LEFT JOIN (SELECT ValueSeriesID, TimeID, fV.ValueFormula FROM planning.tfValues fV WHERE fV.ProductKey = @TemplateProductKey) fVT
				ON fV.ValueSeriesID = fVT.ValueSeriesID AND	fV.TimeID = fVT.TimeID
		WHERE fV.ProductKey =  @ProductKey;
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Product (but not the dProduct, Statements...)
		DELETE FROM planning.tfValues		WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries  WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime			WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts		WHERE ProductKey = @ProductKey;
		SET @EffectedRows += @@ROWCOUNT;

		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,Scale				INT											NOT NULL
		);

		-- STEP 3.3 Copy ValueSeries of Template to Product 
		INSERT INTO planning.tdValueSeries 
		OUTPUT inserted.ValueSeriesKey, inserted.ValueSeriesID, Inserted.NameShort, Inserted.Scale INTO #VSKeys
		SELECT 
			@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,dVS.ValueSeriesID
			,dVS.ValueSeriesNo
			,dVS.NameShort
			,dVS.NameLong
			,dVS.CommentUser
			,dVS.CommentDev
			,dVS.ImageName
			,dVS.IsNumeric
			,dVS.VisibilityLevel
			,dVS.ValueSource
			,dVS.ValueListID
			,dVS.ValueFormatID
			,dVS.Unit
			,dVS.Scale
			,dVS.Effect
			,dVS.EffectParameter
		FROM planning.tdValueSeries dVS 
		WHERE dVS.ProductKey = @TemplateProductKey

		-- STEP 3.4 Copy Timeline of Template to Product 
		INSERT INTO planning.tdTime
		SELECT 
			@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,TimeID
			,FormatID
		FROM planning.tdTime dT 
		WHERE dT.ProductKey = @TemplateProductKey

		-- STEP 3.4 Copy Values of Template to Product 
		INSERT INTO planning.tfValues
		SELECT
			vk.ValueSeriesKey
			,@ProductKey
			,@ProductLineKey
			,@FactoryKey
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,fV.ValueSeriesID
			,fV.TimeID
			,fV.ValueFormula
			,fV.ValueInt
			,fV.ValueText
			,fV.ValueComment
		FROM planning.tfValues fV LEFT JOIN #VSKeys vk 
			ON fV.ValueSeriesID = vk.ValueSeriesID
		WHERE fV.ProductKey = @TemplateProductKey

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO planning.tgProducts
			SELECT
				 @ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID
			FROM planning.tgProducts gP 
			WHERE gP.ProductKey = @TemplateProductKey


		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled


		-- STEP 3.5 - Update new Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
		BEGIN
			-- Delete Values, which belong to no longer existing IDs
			DELETE FROM #ProductDataStorage 
			WHERE ValueSeriesID NOT IN	(
				SELECT dVS.ValueSeriesID 
				FROM planning.tdValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);
				
			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage 
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM planning.tdTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE FROM planning.tfValues 
			WHERE ProductKey = @ProductKey 
				AND	ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
					(	
						SELECT ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(255)) 
						FROM #ProductDataStorage pds
					);

			-- Insert all other Values - this are Values for input cells and values formula Cell
			INSERT INTO planning.tfValues
			SELECT 
				vk.ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,pds.ValueSeriesID
				,pds.TimeID
				,ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
				ON pds.ValueSeriesID = vk.ValueSeriesID
		END
		ELSE
		BEGIN -- compare by Name
			-- Delete Values, which belong to no longer existing Names
			DELETE FROM #ProductDataStorage 
			WHERE ValueSeriesName NOT IN (
				SELECT dVS.NameShort
				FROM planning.tdValueSeries dVS
				WHERE dVS.ProductKey = @TemplateProductKey 
			);

			-- Delete Values, which belong to no longer existing TimeIDs,
			DELETE FROM #ProductDataStorage 
			WHERE TimeID NOT IN	(	
				SELECT dT.TimeID
				FROM planning.tdTime dT
				WHERE dT.ProductKey = @TemplateProductKey 
			);

			-- Delete Values in fact table which came from Template, to override them
			DELETE fV 
			FROM planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey 
				AND	dVS.NameShort + '_' + CAST(TimeID AS NVARCHAR(255)) IN 
				(	
					SELECT pds.ValueSeriesName + '_' + CAST(TimeID AS NVARCHAR(255)) 
					FROM #ProductDataStorage pds
				);

			-- Insert all other Values, if they fit to timeline
			INSERT INTO planning.tfValues
			SELECT 
				vk.ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,dVS.ValueSeriesID
				,pds.TimeID
				,ValueFormula
				,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values !!
				,pds.ValueText
				,pds.ValueComment
			FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk  -- to determine ValueSeries Key
					ON pds.ValueSeriesName = vk.ValueSeriesName
					-- to determine new ValueSeriesID
				LEFT JOIN (SELECT dVS.ValueSeriesID, dVS.NameShort FROM planning.tdValueSeries dVS WHERE dVS.ProductKey = @TemplateProductKey) dVS 
					ON dVS.NameShort = pds.ValueSeriesName
			-- filter only fitting TimeIDs
			WHERE pds.TimeID IN (
				SELECT TimeID 
				FROM planning.tdTime dT 
				WHERE dT.ProductKey = @TemplateProductKey
			);

		END;

		-- Create a System Statement
		INSERT INTO planning.tfStatements  
		SELECT  
			  @ProductKey
			, @ProductLineKey
			, @FactoryKey
			, @ProductID
			, @ProductlineID
			, @FactoryID
			, N'POST_Template' AS ActionType
			, N'Product Template updated with Template Factory <' 
				+ @TemplateFactoryID			+ N'>"' 
				+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
				+ @TemplateProductlineID		+ N'>"' 
				+ @TemplateProductLineNameShort + N'"/ Product <'	
				+ @TemplateProductID			+ N'>"' 
				+ @TemplateProductNameShort		+ N'"' AS Statement
			, @TransactUsername AS UserName
			, N'' AS PCName
			, N'' AS ProcessorCode
			, N'' AS IPAddresses
			, CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
			, 0 AS IsDeleted
			, 0 AS IsResolved;

		-- STEP 3.4 - Update the TemplateName of the Product
		UPDATE planning.tdProducts
		SET   
			Template = @TemplateName
			, TimeType = RIGHT(Template, 2) 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_Template;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryService;
GO

/*
POST Operation to apply a template (again) to many Products
Function principles:
	- Determine all Products in Scope and put them on a ToDo list
	
	- the fact table values from the existing products are temporary stored
		- numbers with scale information
		- formulas are not stored (but their result values, as this may now be a input value series)
		
	- the existing products are caved out
		- values are deleted
		- timeline is deleted
		- value series are deleted

	- the template is caved in
		- timeline is inserted
		- value series are inserted
		- all values and formulas are inserted

	- the old values are partly caved in 
		- as long as they now belong of a value series of type input
		- transfers all existing data which now belongs to a column of type input


Search Conditions:
- Factory ZT is never used as target
- empty Factory ID		-> all Factories are effected
- empty ProductlineID	-> all Productlines effected
- empty Search Template -> all Products effected

03/2019 for DataFactory 4.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Testcall - makes a whole Productline to Unikum Month
-- Copy Productline U in ZT to create Productline U2 in TT (can't use Factory ZT !)
DECLARE @RC								INT
DECLARE @Username						NVARCHAR(255) = 'SQL'

DECLARE @ProductLineID					NVARCHAR(255) = '1'
DECLARE @FactoryID						NVARCHAR(255) = '1'
DECLARE @SearchTemplateName				NVARCHAR(255) = 'Unikum_VM'

DECLARE @TemplateProductID				NVARCHAR(255) = '1'
DECLARE @TemplateProductLineID			NVARCHAR(255) = 'U' 
DECLARE @TemplateFactoryID				NVARCHAR(255) = 'ZT'

DECLARE @CompareByNameInsteadOfIDFlag	INT			  = 0

EXEC @RC = dbo.sx_pf_POST_Template_All 
	 @Username,
	 @ProductLineID,
	 @FactoryID,
	 @SearchTemplateName,
	 @TemplateProductID,
	 @TemplateProductLineID,
	 @TemplateFactoryID,
	 @CompareByNameInsteadOfIDFlag

PRINT @RC

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_POST_Template_All') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Template_All
GO

CREATE PROCEDURE dbo.sx_pf_POST_Template_All
	@Username						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@SearchTemplateName				NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN

	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @FactoryKey						INT				= 0;
	DECLARE @ProductLineKey					INT				= 0;
	DECLARE @TemplateFactoryKey				INT				= 0;
	DECLARE @TemplateProductLineKey			INT				= 0;
	DECLARE @TemplateProductKey				INT				= 0;
	DECLARE @TemplateFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateProductLineNameShort	NVARCHAR(255)	= N'';
	DECLARE @TemplateProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TemplateName					NVARCHAR(255)	= N'';
	DECLARE @ProductCount					BIGINT;
	DECLARE @CounterSelf					INT;

	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR(MAX)	= N'''' 
															+ ISNULL(@Username,					N'NULL') + N''',''' 
															+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
															+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
															+ ISNULL(@SearchTemplateName,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
															+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
															+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows					INT				= 0;			
	DECLARE @ResultCode						INT				= 501;			
	DECLARE @TimestampCall					DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment						NVARCHAR(2000)	= N'';			

	-- STEP A1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @SearchTemplateName				IS NULL SET @SearchTemplateName				= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template_All;

		-- ***********************************************
		-- Do Standard Protection
		-- ***********************************************

		-- Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @SearchTemplateName				= dbo.sx_pf_pProtectID	(@SearchTemplateName);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductlineID			= dbo.sx_pf_pProtectID	(@TemplateProductlineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);

		
		-- Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @FactoryKey				AND ProductlineID	= @ProductlineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @TemplateFactoryKey		AND ProductlineID	= @TemplateProductlineID;
		SELECT @TemplateProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductlineKey	= @TemplateProductlineKey	AND ProductID		= @TemplateProductID;
		SELECT @TemplateName			= Template			FROM planning.tdProducts dP		WHERE dP.ProductKey		= @TemplateProductKey

		-- Stop if Template is invalid (Key of Target Factory / PL may all be 0)
		IF @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- ***********************************************
		-- Create a ToDo List of all Products in scope
		-- ***********************************************
		
		IF OBJECT_ID('tempdb..#ProductToDoList') IS NOT NULL
		DROP TABLE #ProductToDoList

		CREATE TABLE #ProductToDoList
		(
			 FactoryKey		BIGINT										NOT NULL
			,ProductlineKey BIGINT										NOT NULL
			,ProductKey		BIGINT										NOT NULL
			,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Set all Products of scope on the ToDo List
		INSERT INTO #ProductToDoList
			SELECT 
				 dP.FactoryKey
				,dP.ProductLineKey
				,dP.ProductKey
				,dP.FactoryID
				,dP.ProductLineID 
				,dP.ProductID
			FROM planning.tdProducts dP 
			WHERE 
				dP.FactoryID <> 'ZT'
				AND (FactoryKey = @FactoryKey OR @FactoryID = '') 
				AND (dP.ProductLineKey = @ProductLineKey OR @ProductLineID = '') 
				AND (dP.Template = @SearchTemplateName OR @SearchTemplateName = '');
			
		-- Stop if Template is inside target Products
		SELECT @CounterSelf = COUNT(ProductKey) 
		FROM #ProductToDoList 
		WHERE Productkey = @TemplateProductKey

		IF @CounterSelf > 0
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Template is in same Productline as Target - stopped to not post template on itself.', 16, 10);
			END;

		
		-- Clean ToDo List from Products where user don't has rights
		DELETE FROM #ProductToDoList 
		WHERE ProductKey IN	
		(
			SELECT ProductKey 
			FROM #ProductToDoList ToDo LEFT JOIN planning.trUserRights	tUR
				ON ToDo.FactoryID = tUR.FactoryID AND ToDo.ProductLineID = tUR.ProductLineID
			WHERE 
				tUR.[Right] != 'Write' 
				AND tUR.UserName = @TransactUsername
		);

		SELECT @ProductCount = COUNT(ProductKey) FROM #ProductToDoList;

		IF @ProductCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No fitting products found', 16, 10);
		END

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in any product
		IF	@CompareByNameInsteadOfIDFlag = 1 
		AND	EXISTS (SELECT 
						 dVS.ProductKey
						,COUNT(NameShort)
					FROM planning.tdValueSeries dVS 
						-- INNER JOIN to filter on Product on ToDo List
						INNER JOIN #ProductToDoList ToDO
							ON ToDo.ProductKey = dVS.ProductKey
					 GROUP BY dVS.ProductKey, NameShort 
					 HAVING COUNT(NameShort)>1
					)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique', 16, 10);
		END;

		-- ***********************************
		-- STAGING Existing Data from Produts
		-- ***********************************
		

		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			 ProductKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,TimeID				INT											NOT NULL
			,ValueInt			BIGINT										NOT NULL
			,ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueComment		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			);

		-- Store Data of Products in Storage (but not formulas, they come from template)
		-- ToDo: Keep new formulas out in this step !!
		INSERT INTO #ProductDataStorage    
			SELECT
				 fV.ProductKey 
				,fV.ValueSeriesID
				,dVS.NameShort AS ValueSeriesName
				,dVS.Scale
				,fV.TimeID
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment 
			FROM 
				planning.tfValues fV 
				-- for ValueSeriesNames
				LEFT JOIN planning.tdValueSeries dVS 
					ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
				-- filter Products on ToDo List
				INNER JOIN	#ProductToDoList ToDo 
					ON fV.ProductKey = ToDo.ProductKey;

		-- ***********************************
		-- Cave out existing Products
		-- ***********************************
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Products (but not the dProduct, Statements...)
		DELETE FROM planning.tfValues		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries	WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime			WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		-- ***********************************
		-- Cave in Template
		-- ***********************************

		-- keep new VS Keys temporary stored
		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ProductKey			BIGINT										NOT NULL
			,ProductlineKey		BIGINT										NOT NULL
			,FactoryKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,ValueSource		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
		);

		-- Copy ValueSeries of Template to all Products and store their Keys
		INSERT INTO planning.tdValueSeries
				OUTPUT	 INSERTED.ValueSeriesKey
						,INSERTED.ProductKey
						,INSERTED.ProductlineKey
						,INSERTED.FactoryKey
						,INSERTED.ValueSeriesID
						,INSERTED.ProductID
						,INSERTED.ProductLineID
						,INSERTED.FactoryID
						,INSERTED.NameShort
						,INSERTED.Scale 
						,INSERTED.ValueSource
				INTO #VSKeys
			SELECT 
				ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dVS.ValueSeriesID
				,dVS.ValueSeriesNo
				,dVS.NameShort
				,dVS.NameLong
				,dVS.CommentUser
				,dVS.CommentDev
				,dVS.ImageName
				,dVS.IsNumeric
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.ValueListID
				,dVS.ValueFormatID
				,dVS.Unit
				,dVS.Scale
				,dVS.Effect
				,dVS.EffectParameter
			FROM planning.tdValueSeries dVS
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dVS.ProductKey = @TemplateProductKey;

		-- Copy Timeline of Template to Product 
		INSERT INTO planning.tdTime
			SELECT 
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dT.TimeID
				,dT.FormatID
			FROM planning.tdTime dT-- spread to all products
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dT.ProductKey = @TemplateProductKey;

		-- Copy Values and Formulas of Template to Products 
		INSERT INTO planning.tfValues
			SELECT
				 vk.ValueSeriesKey
				,vk.ProductKey
				,vk.ProductLineKey
				,vk.FactoryKey
				,vk.ProductID
				,vk.ProductLineID
				,vk.FactoryID
				,fV.ValueSeriesID
				,fV.TimeID
				,fV.ValueFormula
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment
			FROM planning.tfValues fV 
				LEFT JOIN #VSKeys vk 
					ON	fV.ValueSeriesID = vk.ValueSeriesID
			WHERE fV.ProductKey = @TemplateProductKey;

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO planning.tgProducts
			SELECT
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID

			FROM planning.tgProducts gP 
			-- spread to all products
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
                    
			WHERE gP.ProductKey = @TemplateProductKey

		--  ===============================================================
		--> Now all Products are complete fresh 1:1 copies of the Template
		--  ===============================================================

		-- *****************************************************************
		-- cave in the transfer values if they still fit to the template
		-- *****************************************************************

		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled


		-- Delete Transfervalues, which belong to no longer existing TimeIDs
		DELETE FROM #ProductDataStorage
		WHERE TimeID NOT IN	(
			SELECT dT.TimeID
			FROM planning.tdTime dT
			WHERE dT.ProductKey = @TemplateProductKey 
		);

		-- Transfer stored Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
			BEGIN
				-- Delete Transfervalues, which belong to no longer existing IDs
				DELETE FROM #ProductDataStorage 
				WHERE ValueSeriesID NOT IN	(
					SELECT dVS.ValueSeriesID 
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);	

				-- Delete Tranfervalues, which are formulas and already renewed in the template
				DELETE FROM #ProductDataStorage
				WHERE ValueSeriesID IN	(
					SELECT ValueSeriesID
					FROM planning.tdValueSeries dVS
					WHERE 
							dVS.ProductKey = @TemplateProductKey 
						AND dVS.ValueSource IN ('XLS','XLS-Strict')
				);
				
				-- Delete Values in fact table to make space for Transfervalues
				DELETE fV FROM planning.tfValues fV
					LEFT JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					LEFT JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND fV.ValueSeriesID	= PDS.ValueSeriesID
							AND fV.TimeID			= PDS.TimeID
				WHERE 
						PDS.TimeID			IS NOT NULL 
					AND PDS.ValueSeriesID	IS NOT NULL

				-- Insert all Transfervalues
				INSERT INTO planning.tfValues
					SELECT 
						 vk.ValueSeriesKey
						,vk.ProductKey
						,vk.ProductLineKey
						,vk.FactoryKey
						,vk.ProductID
						,vk.ProductLineID
						,vk.FactoryID
						,pds.ValueSeriesID
						,pds.TimeID
						,'' AS ValueFormula
						,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values
						,pds.ValueText
						,pds.ValueComment
					FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesID = vk.ValueSeriesID
			END
		ELSE
			BEGIN -- compare by Name

				-- Delete Transfervalues, which belong to no longer existing Names
				DELETE FROM #ProductDataStorage
				WHERE ValueSeriesName NOT IN (
					SELECT dVS.NameShort
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);

				-- Delete Values, which are formulas and already renewed in the template
				DELETE FROM #ProductDataStorage
				WHERE ValueSeriesName IN	(
					SELECT NameShort
					FROM planning.tdValueSeries dVS
					WHERE 
							dVS.ProductKey = @TemplateProductKey 
						AND dVS.ValueSource IN ('XLS','XLS-Strict')
				);

				-- Delete Values in fact table to make space for Transfervalues
				DELETE fV FROM planning.tfValues fV

					LEFT JOIN planning.tdValueSeries dVS
						ON fV.ValueSeriesKey = dVS.ValueSeriesKey

					LEFT JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					LEFT JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND dVS.NameShort		= PDS.ValueSeriesName
							AND fV.TimeID			= PDS.TimeID
				WHERE 
						PDS.TimeID			IS NOT NULL 
					AND PDS.ValueSeriesName	IS NOT NULL

				-- Insert all Transfervalues
				INSERT INTO planning.tfValues
				SELECT 
					 vk.ValueSeriesKey
					,vk.ProductKey
					,vk.ProductLineKey
					,vk.FactoryKey
					,vk.ProductID
					,vk.ProductLineID
					,vk.FactoryID
					,dVS.ValueSeriesID
					,pds.TimeID
					,'' AS ValueFormula
					,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values 
					,pds.ValueText
					,pds.ValueComment
				FROM #ProductDataStorage pds 
					-- to determine ValueSeries Key
					LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesName = vk.ValueSeriesName
					-- to determine new ValueSeriesID
					LEFT JOIN 
						(
							SELECT 
								 dVS.ValueSeriesID
								,dVS.NameShort 
							FROM planning.tdValueSeries dVS 
							WHERE dVS.ProductKey = @TemplateProductKey
						) dVS 
					ON dVS.NameShort = pds.ValueSeriesName

			END;

		-- Create a System Statement
		INSERT INTO planning.tfStatements
			SELECT  
				 ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductlineID
				,FactoryID
				,N'POST_Template_All' AS ActionType
				,N'Product Template updated with Template Factory <'  
					+ @TemplateFactoryID			+ N'>"' 
					+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
					+ @TemplateProductlineID		+ N'>"' 
					+ @TemplateProductLineNameShort + N'"/ Product <' 
					+ @TemplateProductID			+ N'>"' 
					+ @TemplateProductNameShort		+ '"' AS Statement
				,@TransactUsername AS UserName
				,N'' AS PCName
				,N'' AS ProcessorCode
				,N'' AS IPAddresses
				,CAST(CONVERT(NVARCHAR(10),GETDATE(),112) AS BIGINT)*10000 + CAST(REPLACE(LEFT(CONVERT(NVARCHAR(10), GETDATE(), 108), 5), N':', N'') AS BIGINT) AS Timestamp
				,0 AS IsDeleted
				,0 AS IsResolved
				FROM #ProductToDoList 
			
			-- Update the TemplateName of the Products
			UPDATE planning.tdProducts
			SET  
				 Template = @TemplateName
				,TimeType = RIGHT(@TemplateName, 2) 
			WHERE 
				ProductKey IN (SELECT ProductKey FROM #ProductToDoList);

		SET @EffectedRows += @@ROWCOUNT;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_POST_Template_All;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template_All;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryService;
GO
/*
POST Operation for TimeID
Create new TimeID

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE  @RC			INT
DECLARE  @Username		NVARCHAR(255) = 'SQL'
DECLARE  @ProductID		NVARCHAR(255) = '1'
DECLARE  @ProductLineID NVARCHAR(255) = 'U'
DECLARE  @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE  @TimeID		INT			  = 20150107
DECLARE  @FormatID		NVARCHAR(255) = 'TimeBorderNext'

EXECUTE @RC = dbo.sx_pf_POST_TimeID 
		@Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@TimeID
		,@FormatID

PRINT @RC
*/
IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_TimeID') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_TimeID;
GO

CREATE PROCEDURE dbo.sx_pf_POST_TimeID
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeID				INT,
	@FormatID			NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@FormatID,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @FormatID		IS NULL SET @FormatID		= '';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_TimeID;

		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 2.1 - Delete TimeID
		DELETE FROM planning.tdTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- STEP 3 - Create new TimeID

		INSERT INTO planning.tdTime	(
			ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		)
		VALUES (
			@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@TimeID,
			@FormatID
		);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryService;
GO
/*
POST Operation for the Timeline
The method can be switched between incremental POST and full POST
Values with DeleteFlag will be deleted

03/2018 for DataFactory 5.0

Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
Test call

Test case 1 - Incremental Sended
1. One Value for an existing Value must replace them
2. One Value for an not exiting Value must create it
3. If all Parameter are posted empty, the value is deleted
4. if one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
DECLARE @RC int
DECLARE @Username					NVARCHAR(255) = 'SQL'
DECLARE @ProductID					NVARCHAR(255) = '1'
DECLARE @ProductLineID				NVARCHAR(255) = 'U'
DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
DECLARE @IsIncrementalValuesFlag	INT = 0

--3 TimeIDs
DECLARE @TimeIDsInBracketsCommaSeparated nvarchar(max) = '[(20160115,0,0,''FormatID1'')],[(20160215,0,0,''FormatID2'')],[(20160415,0,0,''FormatID3'')]'
EXECUTE @RC = dbo.sx_pf_POST_Timeline
	 @Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@TimeIDsInBracketsCommaSeparated
PRINT @RC

Test case 2 - all values in a Value Series of Products:
Additional 1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
DECLARE @RC int
DECLARE @Username							NVARCHAR(255) = 'SQL'
DECLARE @ProductID							NVARCHAR(255) = '21'
DECLARE @ProductLineID						NVARCHAR(255) = '1'
DECLARE @FactoryID							NVARCHAR(255) = 'U'
DECLARE @IsIncrementalValuesFlag			INT = 0
DECLARE @TimeIDsInBracketsCommaSeparated	NVARCHAR(max) = ''
DECLARE @start_time DATETIME

-- Generate fictitious values for all TimeIDs of Products
SELECT @TimeIDsInBracketsCommaSeparated =  @TimeIDsInBracketsCommaSeparated + '[(''' + dVS2.ValueSeriesID + ''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',1000,''Hase'',''Kommentar'')],' 
FROM planning.tfValues fV LEFT JOIN 
	planning.tdValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey LEFT JOIN
	(SELECT ValueSeriesID FROM planning.tdValueSeries WHERE FactoryID =  @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID GROUP BY ValueSeriesID) dVS2
		ON 1=1 
WHERE fV.FactoryID =  @FactoryID AND fV.ProductLineID = @ProductLineID AND fV.ProductID = @ProductID AND dVS.ValueSeriesNo = 1
SET @TimeIDsInBracketsCommaSeparated = Left(@TimeIDsInBracketsCommaSeparated,Len(@TimeIDsInBracketsCommaSeparated) -1)
PRINT @TimeIDsInBracketsCommaSeparated
SET @start_time = GETDATE()
EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues
	 @Username
	,@ProductID
	,@ProductLineID
	,@FactoryID
	,@IsIncrementalValuesFlag
	,@TimeIDsInBracketsCommaSeparated
PRINT @RC
SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETDATE()) AS CHAR(10))) AS 'TimeTaken'
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Timeline') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Timeline;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Timeline
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@TimeIDsInBracketsCommaSeparated	NVARCHAR(MAX)

/*
@TimeIDsInBracketsCommaSeparated format: (TimeID,DeleteFlag,AddValue,FormatID)
if you check this in SQL Server Profiler, every String must have two inverted commas, the whole Parameter must have Single inverted comma
this must show profiler: '[(''TimeID,DeleteFlag,AddValue,''FormatID'')],[(TimeID,DeleteFlag,AddValue,FormatID)]'
if you check this in table planning.tAPI_Log, every String must have one inverted comma, whole String has no inverted comma
this must show Log: [(TimeID,DeleteFlag,AddValue,FormatID)],[(TimeID,DeleteFlag,AddValue,'FormatID')]
*/

AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;

	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductID							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductLineID						, N'NULL')	+ N''',''' 
								+ ISNULL(@FactoryID							, N'NULL')	+ N''',''' 
								+ ISNULL(@TimeIDsInBracketsCommaSeparated	, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username							= N'';
	IF @ProductID						IS NULL SET @ProductID							= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID						= N'';
	IF @FactoryID						IS NULL SET @FactoryID							= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag			= 0;
	IF @TimeIDsInBracketsCommaSeparated IS NULL SET @TimeIDsInBracketsCommaSeparated	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Timeline;
		
		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID (@FactoryID)
		SET @ProductID			= dbo.sx_pf_pProtectID (@ProductID)
		SET @ProductLineID		= dbo.sx_pf_pProtectID (@ProductLineID)

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductlineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
				
		-- STEP 3.2 - Check for count of delimiter in Array

		DECLARE @OpenDelimetersCount INT = 0;
		DECLARE @DelimetersIndex INT = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.3 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @TimeIDsInBracketsCommaSeparated = LEFT(@TimeIDsInBracketsCommaSeparated,  LEN(@TimeIDsInBracketsCommaSeparated) - 2);
		SET @TimeIDsInBracketsCommaSeparated = RIGHT(@TimeIDsInBracketsCommaSeparated, LEN(@TimeIDsInBracketsCommaSeparated) - 2);

		DECLARE @tblValues1 TABLE (txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues2 TABLE (TimeID		 NVARCHAR(MAX) NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues3 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues4 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, AddValue		 BIGINT		   NOT NULL, FormatID NVARCHAR(255) NOT NULL);

		-- Split String into Rows
		INSERT INTO @tblValues1 
			SELECT txtValues FROM dbo.sx_pf_PivotStringIntoTable(@TimeIDsInBracketsCommaSeparated, N')],[(');

		-- Split one column into two Columns (separate TimeID)
		INSERT INTO @tblValues2
			SELECT  
				 LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe)-1 ) AS TimeID
				,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)) AS Rest
			FROM @tblValues1;

		IF EXISTS(SELECT TimeID FROM @tblValues2 GROUP BY TimeID HAVING(COUNT(TimeID) > 1))
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Doubled TimeID in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- Split last columns once more (separate DeleteFlag)
		INSERT INTO @tblValues3
			SELECT  
				 TimeID
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS DeleteFlag
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)-1) AS Rest
			FROM @tblValues2;

		-- Split last columns once more (separate AddValue)
		INSERT INTO @tblValues4
			SELECT  
				 TimeID
				,DeleteFlag
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS AddValue
			,REPLACE(Right(txtValuesPipe, LEN(txtValuesPipe) - 1), N'''','') AS FormatID
			FROM @tblValues3;

		-- STEP 3.4 - Actions #######################################################################################################################
	
		-- Delete TimeIDs which are sended as deleted in fValues and in dTime
		DELETE FROM planning.tdTime 
		WHERE 
			@ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);
	
		SET @EffectedRows = @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE 
			ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);

		SET @EffectedRows += @@ROWCOUNT;
		-- STEP 3.5 - CASE FULL transmission or Incremental does not matter anymore #########################################
		IF @IsIncrementalValuesFlag != 1 OR  @IsIncrementalValuesFlag = 1 
		BEGIN
			-- Insert all New TimeIDs
			INSERT INTO planning.tdTime
			SELECT  
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				TimeID,
				FormatID
			FROM @tblValues4 
			WHERE TimeID NOT IN (SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey);

			SET @EffectedRows += @@ROWCOUNT;
		END;
			
		COMMIT TRANSACTION sx_pf_POST_Timeline;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Timeline;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryService;
GO
/*
POST Operation for a User
Not existing User will be created, existing one updated
If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call
DECLARE @RC					INT
DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @PostUserName		NVARCHAR(255) = 'sx'
DECLARE @PersonSurname		NVARCHAR(255) = ''
DECLARE @PersonFirstName	NVARCHAR(255) = ''
DECLARE @Email				NVARCHAR(255) = ''
DECLARE @LDAPIP				NVARCHAR(255) = ''
DECLARE @Status				NVARCHAR(255) = ''
DECLARE @ImageName			NVARCHAR(255) = ''

EXECUTE @RC = dbo.sx_pf_POST_User 
		@Username,
		@PostUserName,
		@PersonSurname,
		@PersonFirstName,
		@Email,
		@LDAPIP,
		@Status

PRINT @RC
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_User') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_User;
GO

CREATE PROCEDURE dbo.sx_pf_POST_User
	@Username			NVARCHAR(255),
	@PostUserName		NVARCHAR(255),
    @PersonSurname		NVARCHAR(255),
    @PersonFirstName	NVARCHAR(255),
    @Email				NVARCHAR(255),
    @LDAPIP				NVARCHAR(255),
    @Status				NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @PostUserKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,			N'NULL') + N''',''' 
								+ ISNULL(@PostUserName,		N'NULL') + N''',''' 
								+ ISNULL(@PersonSurname,	N'NULL') + N''',''' 
								+ ISNULL(@PersonFirstName,	N'NULL') + N''',''' 
								+ ISNULL(@Email,			N'NULL') + N''',''' 
								+ ISNULL(@LDAPIP,			N'NULL') + N''',''' 
								+ ISNULL(@Status,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @PostUserName		IS NULL SET @PostUserName		= N'';
	IF @PersonSurname		IS NULL SET @PersonSurname		= N'';
	IF @PersonFirstName		IS NULL SET @PersonFirstName	= N'';
	IF @Email				IS NULL SET @Email				= N'';
	IF @LDAPIP				IS NULL SET @LDAPIP				= N'';
	IF @Status				IS NULL SET @Status				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_User;

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName		= dbo.sx_pf_pProtectString (@PostUserName);
		SET @PersonSurname		= dbo.sx_pf_pProtectString (@PersonSurname);
		SET @PersonFirstName	= dbo.sx_pf_pProtectString (@PersonFirstName);
		SET @Email				= dbo.sx_pf_pProtectString (@Email);
		SET @LDAPIP				= dbo.sx_pf_pProtectString (@LDAPIP);
		SET @Status				= dbo.sx_pf_pProtectString (@Status);

		IF @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Don`t allow creation or changing of systemuser SQL
		-- the Username "FactoryService" is not allowed, as it would lead to confusion with the similar named db role
		IF @PostUserName = N'SQL' OR @PostUserName = N'FactoryService'	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden creation or changing of systemuser', 16, 10);
		END;
			
		-- STEP 1.2 - Don't allow changes on the members of the DB role pf_PlanningFactoryService (which are not visible in the frontend) - not over the function IS_ROLEMEMBER, its buggy
		IF EXISTS(SELECT r.name 
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r	ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m	ON rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = @PostUserName)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changes on the members of the DB role pf_PlanningFactoryService', 16, 10);
		END;

		-- STEP 1.2 - Don`t allow the user 'public' to change his Rights
		IF LOWER(@UserName) = 'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow the user ''public'' to change his Rights', 16, 10);
		END;

		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to change rights', 16, 10);
		END;
					
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- The Clusteradmin can`t set himself to Inactive
		IF @TransactUsername = @PostUsername AND @Status <> 'Active' 
			SET @Status = 'Active';
		
		-- STEP 2 - Get keys
		SELECT @PostUserKey = UserKey FROM planning.trUser WHERE Username = @PostUserName;
				
		-- STEP 3.1 - If user does not yet exist, it will be created
		IF @PostUserKey = 0
		BEGIN
			INSERT INTO planning.trUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status) 
			OUTPUT INSERTED.UserKey INTO @tmpUserTable 
			VALUES (@PostUserName, '', '', '', '', '');

			SET @EffectedRows = 1;
				
			-- the determination on the table variable is necessary because of TRIGGERS on rUser
			SELECT TOP (1) @PostUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
			SET @Resultcode = 201;
		END

		-- STEP 3.2 - Update the User with the commited attributes
		UPDATE planning.trUser
		SET
			 PersonSurname		= IIF(@PersonSurname	= N'<#NV>', PersonSurname,		@PersonSurname)
			,PersonFirstName	= IIF(@PersonFirstName	= N'<#NV>', PersonFirstName,	@PersonFirstName)
			,Email				= IIF(@Email			= N'<#NV>', Email,				@Email)
			,LDAPIP				= IIF(@LDAPIP			= N'<#NV>', LDAPIP,				@LDAPIP)
			,Status				= IIF(@Status			= N'<#NV>', Status,				@Status)
		WHERE UserKey = @PostUserKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_User TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_User TO pf_PlanningFactoryService;
GO
/*
POST Operation for Value
The Value will be deleted and writen new

03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC				INT
DECLARE @Username		NVARCHAR(255) = 'SQL'
DECLARE @ProductID		NVARCHAR(255) = '1'
DECLARE @ProductLineID	NVARCHAR(255) = 'U'
DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
DECLARE @ValueSeriesID	NVARCHAR(255) = 'E1'
DECLARE @TimeID			BIGINT		  = 20180315
DECLARE @ValueFormula	NVARCHAR(max) = '<#NV>'
DECLARE @ValueInt		NVARCHAR(255) = '555' 
DECLARE @ValueText		NVARCHAR(max) = 'Hase2'
DECLARE @ValueComment	NVARCHAR(max) = 'Hase2'
DECLARE @i				INT = 0

-- individual values 
-- Text with Comment
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		'E1',
		20180115,
		'',
		0,
		'Hase',
		'Kommentar für Hase';

PRINT @RC

-- Integer with Comment
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		'K1',
		20180215,
		'',
		44,
		'',
		'Kommentar für Hase44';

PRINT @RC

-- 1b. Variant with loop
--WHILE @i < 1000
--BEGIN
EXECUTE @RC = dbo.sx_pf_POST_Value 
		@Username,
		@ProductID,
		@ProductLineID,
		@FactoryID,
		@ValueSeriesID,
		@TimeID,
		@ValueFormula,
		@i,
		@ValueText,
		@ValueComment
--SET @i = @i +1
PRINT @RC
--END

-- 2. Variant with Script Creator
PRINT 'DECLARE @RC as INTEGER = 0'

WHILE @i < 5000
BEGIN
	PRINT 'EXECUTE @RC = dbo.sx_pf_POST_Value ''' + @Username + ''','''+ @ProductID+ ''','''+@ProductLineID+ ''','''+ @FactoryID + ''','''+ @ValueSeriesID + ''','+ CAST(@TimeID AS NVARCHAR (8)) + ','''+ @ValueFormula + ''','+ CAST(@i AS NVARCHAR(10)) + ','''+@ValueText+ ''','''+	@ValueComment + ''''
	PRINT 'PRINT @RC' 
	SET @i = @i +1 
END
--about 12 sec for 5000 Values Values = 400 / second
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_Value') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_Value;
GO

CREATE PROCEDURE dbo.sx_pf_POST_Value
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ValueSeriesID	NVARCHAR(255),		
	@TimeID			INT,
	@ValueFormula	NVARCHAR(MAX),
	@ValueInt		NVARCHAR(255), -- to be able to send <#NV>
	@ValueText		NVARCHAR(MAX),
	@ValueComment	NVARCHAR(MAX)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @ValueKey			INT				= 0;
	DECLARE @ValueSeriesNo		INT				= 0;

	DECLARE @OLDValueFormula	NVARCHAR(255)	= N'';
	DECLARE @OLDValueInt		BIGINT			= 0; 
	DECLARE @OLDValueText		NVARCHAR(255)	= N'';
	DECLARE @OLDValueComment	NVARCHAR(MAX)	= N'';

	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
		+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' 
		+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,		N'NULL') + N''',''' 
		+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL') + N',''' 
		+ ISNULL(@ValueFormula,		N'NULL') + N''',''' + ISNULL(@ValueInt,			N'NULL') + N''',''' 
		+ ISNULL(@ValueText,		N'NULL') + N''',''' + ISNULL(@ValueComment,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @ValueFormula	IS NULL SET @ValueFormula	= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueComment	IS NULL SET @ValueComment	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Value;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @TimeID			= dbo.sx_pf_pProtectID		(@TimeID);
		SET @ValueFormula	= dbo.sx_pf_pProtectString	(@ValueFormula);
		SET @ValueInt		= dbo.sx_pf_pProtectString	(@ValueInt);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N'' OR @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

 	     -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.3 - Try to detect and cancel the ValueSeriesKey if does not exist
		SELECT @ValueSeriesKey = ValueSeriesKey, @ValueSeriesNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ProductKey= @ProductKey AND ValueSeriesID = @ValueSeriesID;
		
		IF @ValueSeriesKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2.1 - Cancel if TimeID not exists	
		IF NOT EXISTS(SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('TimeID not exists', 16, 10);
		END;

		-- STEP 3. Determine ValueKey identify and old values
		SELECT @ValueKey = ValueKey, @OLDValueFormula = ValueFormula, @OLDValueInt = ValueInt, @OLDValueText = ValueText, @OLDValueComment = ValueComment
		FROM planning.tfValues 
		WHERE ValueSeriesKey = @ValueSeriesKey AND TimeID = @TimeID;
				
		-- STEP 3.1 - If Value Key exists delete
		IF  @ValueKey > 0 
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ValueKey = @ValueKey;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 204; -- Ok, but deleted

			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', @OLDValueFormula,	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', @OLDValueText,		@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', @OLDValueComment,	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', @OLDValueInt,		@ValueInt);
		END
		ELSE
		BEGIN
			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', N'',	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', N'',	@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', N'',	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', N'0',	@ValueInt);
		END;

		-- STEP 3.2 - Insert, while no empty values outside of column 1
		IF @ValueFormula = N'' AND @ValueText = N'' AND @ValueInt = N'0'
		BEGIN 
			SET @ResultCode = 204; --> discarded !!! 
		END
		ELSE
		BEGIN 
			SET @ValueInt = COALESCE(TRY_CAST(@ValueInt AS BIGINT), 0);

			INSERT INTO planning.tfValues
			VALUES (
				@ValueSeriesKey, 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID, 
				@FactoryID, 
				@ValueSeriesID,  
				@TimeID,  
				@ValueFormula, 
				@ValueInt, 
				@ValueText, 
				@ValueComment);
			
			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 201;
		END;

		COMMIT TRANSACTION sx_pf_POST_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryService;
GO
/*
POST Operation for one ValueSeries
Operation will create a ValueSeries if it not exist
It will change an existing ValueSeries
Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries


03/2018 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard

Test call

DECLARE @RC							INT
DECLARE @Username					NVARCHAR(255) ='SQL'
DECLARE @ProductID					NVARCHAR(255) ='1'
DECLARE @ProductLineID				NVARCHAR(255) ='U'
DECLARE @FactoryID					NVARCHAR(255) ='ZT'
DECLARE @ValueSeriesID				NVARCHAR(255) ='38550'
DECLARE @RequestedValueSeriesNo		INT			  = 249
DECLARE @NameShort					NVARCHAR(255) ='<#NV>'
DECLARE @NameLong					NVARCHAR(255) ='<#NV>'
DECLARE @CommentUser				NVARCHAR(255) ='<#NV>'
DECLARE @CommentDev					NVARCHAR(255) ='<#NV>'
DECLARE @ImageName					NVARCHAR(255) ='<#NV>'
DECLARE @IsNumeric					NVARCHAR(255) ='1'
DECLARE @VisibilityLevel			NVARCHAR(255) ='<#NV>'
DECLARE @ValueSource				NVARCHAR(255) ='<#NV>'
DECLARE @ValueListID				NVARCHAR(255) ='<#NV>'
DECLARE @ValueFormatID				NVARCHAR(255) ='<#NV>'
DECLARE @Unit						NVARCHAR(255) ='<#NV>'
DECLARE @Scale						NVARCHAR(255) ='100'
DECLARE @Effect						NVARCHAR(255) ='<#NV>'
DECLARE @EffectParameter			NVARCHAR(255) ='<#NV>'

EXECUTE @RC = dbo.sx_pf_POST_ValueSerie 
		@Username,	
		@ProductID,	
		@ProductLineID,	
		@FactoryID,	
		@ValueSeriesID,	
		@RequestedValueSeriesNo,
		@NameShort,	
		@NameLong,
		@CommentUser,
		@CommentDev,
		@ImageName,
		@IsNumeric,
		@VisibilityLevel,
		@ValueSource,
		@ValueListID,
		@ValueFormatID,
		@Unit,
		@Scale,
		@Effect,
		@EffectParameter

PRINT @RC
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_POST_ValueSerie') AND type in (N'P', N'PC'))
	DROP PROCEDURE dbo.sx_pf_POST_ValueSerie;
GO

CREATE PROCEDURE dbo.sx_pf_POST_ValueSerie
	@Username				 NVARCHAR(255),
	@ProductID				 NVARCHAR(255),
	@ProductLineID			 NVARCHAR(255),
	@FactoryID				 NVARCHAR(255),
	@ValueSeriesID			 NVARCHAR(255),
	@RequestedValueSeriesNo	 INT,
	@NameShort				 NVARCHAR(255),
	@NameLong				 NVARCHAR(255),
	@CommentUser			 NVARCHAR(255),
	@CommentDev				 NVARCHAR(255),
	@ImageName				 NVARCHAR(255),
	@IsNumeric				 NVARCHAR(255),
	@VisibilityLevel		 NVARCHAR(255),
	@ValueSource			 NVARCHAR(255),
	@ValueListID			 NVARCHAR(255),
	@ValueFormatID			 NVARCHAR(255),
	@Unit					 NVARCHAR(255),
	@Scale					 NVARCHAR(255),
	@Effect					 NVARCHAR(255),
	@EffectParameter		 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @OldValueSeriesNo	INT				= 0;
	DECLARE @OldIsNumeric		INT				= 0;
	DECLARE @OldScale			INT				= 0;
	DECLARE @OldValueSource		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,										N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,										N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@RequestedValueSeriesNo AS NVARCHAR(255)),	N'NULL') + N',''' 
			+ ISNULL(@NameShort,		N'NULL') + N''',''' + ISNULL(@NameLong,											N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,		N'NULL') + N''',''' + ISNULL(@CommentDev,										N'NULL') + N''',''' 
			+ ISNULL(@ImageName,		N'NULL') + N''',''' + ISNULL(@IsNumeric,										N'NULL') + N''',''' 
			+ ISNULL(@VisibilityLevel,	N'NULL') + N''',''' + ISNULL(@ValueSource,										N'NULL') + N''',''' 
			+ ISNULL(@ValueListID,		N'NULL') + N''',''' + ISNULL(@ValueFormatID,									N'NULL') + N''',''' 
			+ ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@Scale,											N'NULL') + N''',''' 
			+ ISNULL(@Effect,			N'NULL') + N''',''' + ISNULL(@EffectParameter,									N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductID				IS NULL SET @ProductID					= N'NULL';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'NULL';
	IF @FactoryID				IS NULL SET @FactoryID					= N'NULL';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID				= N'NULL';
	IF @RequestedValueSeriesNo	IS NULL SET @RequestedValueSeriesNo		= 0;
	IF @NameShort				IS NULL SET @NameShort					= N'NULL';
	IF @NameLong				IS NULL SET @NameLong					= N'NULL';
	IF @CommentUser				IS NULL SET @CommentUser				= N'NULL';
	IF @CommentDev				IS NULL SET @CommentDev					= N'NULL';
	IF @ImageName				IS NULL SET @ImageName					= N'NULL';
	IF @IsNumeric				IS NULL SET @IsNumeric					= N'NULL';
	IF @VisibilityLevel			IS NULL SET @VisibilityLevel			= N'NULL';
	IF @ValueSource				IS NULL SET @ValueSource				= N'NULL';
	IF @ValueListID				IS NULL SET @ValueListID				= N'NULL';
	IF @ValueFormatID			IS NULL SET @ValueFormatID				= N'NULL';
	IF @Unit					IS NULL SET @Unit						= N'NULL';
	IF @Scale					IS NULL SET @Scale						= N'NULL';
	IF @Effect					IS NULL SET @Effect						= N'NULL';
	IF @EffectParameter			IS NULL SET @EffectParameter			= N'NULL';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ValueSerie;

		-- STEP 0.1 - NULL Protection
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ValueSeriesID		= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @IsNumeric			= dbo.sx_pf_pProtectBoolean (@IsNumeric);
		SET @VisibilityLevel	= dbo.sx_pf_pProtectString	(@VisibilityLevel);
		SET @ValueSource		= dbo.sx_pf_pProtectString	(@ValueSource);
		SET @Unit				= dbo.sx_pf_pProtectString	(@Unit);
		SET @Scale				= dbo.sx_pf_pProtectInt		(@Scale);
		SET @Effect				= dbo.sx_pf_pProtectString	(@Effect);
		SET @EffectParameter	= dbo.sx_pf_pProtectString	(@EffectParameter);
		IF @ValueListID		<> N'<#NV>' SET @ValueListID	= dbo.sx_pf_pProtectID (@ValueListID);
		IF @ValueFormatID	<> N'<#NV>' SET @ValueFormatID	= dbo.sx_pf_pProtectID (@ValueFormatID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys	
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductlineID = @ProductlineID;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductLineKey = @ProductLinekey AND ProductID= @ProductID;
		SELECT 
				 @ValueSeriesKey = ValueSeriesKey
				,@OldIsNumeric = IsNumeric
				,@OldScale = Scale
				,@OldValueSource = ValueSource 
		FROM planning.tdValueSeries 
		WHERE 
				ProductKey = @ProductKey 
			AND ValueSeriesID = @ValueSeriesID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - Insert the ValueSeries, if Key does not exist
		IF @ValueSeriesKey = 0
		BEGIN
			-- INCREMENT all following Requested ValueSeriesNumbers based on requested No
			UPDATE planning.tdValueSeries
			SET ValueSeriesNo = ValueSeriesNo + 1 
			WHERE 
				ProductKey = @ProductKey 
				AND ValueSeriesNo >= @RequestedValueSeriesNo;

			IF @VisibilityLevel = '<#NV>' SET @VisibilityLevel = 1;
			
			SET @EffectedRows = @@ROWCOUNT;

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			-- Insert with requested number
			INSERT INTO planning.tdValueSeries (
				ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,ValueSeriesID
				,ValueSeriesNo
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,IsNumeric
				,VisibilityLevel
				,ValueSource
				,ValueListID
				,ValueFormatID
				,Unit
				,Scale
				,Effect
				,EffectParameter) 
			OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
			VALUES ( 
				@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter);
			
			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ValueSeriesKey = [Key] FROM @Keys ORDER BY [Key];
		END
		ELSE
		BEGIN 
		-- STEP 3.2 - Update ValueSeries if it already exists
			SELECT @OldValueSeriesNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ValueSeriesKey = @ValueSeriesKey;
									
			-- Use the old one, if no SeriesNo is commited (lower than one means not commited)
			IF @RequestedValueSeriesNo < 1 
				SET @RequestedValueSeriesNo = @OldValueSeriesNo;
			
			-- Adjust Numbers if not equal
			IF @OldValueSeriesNo < @RequestedValueSeriesNo
			BEGIN 
				-- decrement all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo - 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo > @OldValueSeriesNo 
					AND ValueSeriesNo <= @RequestedValueSeriesNo;
														
				SET @EffectedRows += @@ROWCOUNT;
			END
			ELSE IF @OldValueSeriesNo > @RequestedValueSeriesNo
			BEGIN 
				-- increment all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo + 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo < @OldValueSeriesNo 
					AND ValueSeriesNo >= @RequestedValueSeriesNo;

				SET @EffectedRows += @@ROWCOUNT;
			END;

			UPDATE planning.tdValueSeries 
			SET 
				ValueSeriesNo		= @RequestedValueSeriesNo
				, NameShort			= IIF(@NameShort		= N'<#NV>', NameShort,			@NameShort)
				, NameLong			= IIF(@NameLong			= N'<#NV>', NameLong,			@NameLong)
				, CommentUser		= IIF(@CommentUser		= N'<#NV>', CommentUser,		@CommentUser)
				, CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,			@CommentDev)
				, ImageName			= IIF(@ImageName		= N'<#NV>', ImageName,			@ImageName)
				, IsNumeric			= IIF(@IsNumeric		= N'<#NV>', IsNumeric,			@IsNumeric)
				, VisibilityLevel	= IIF(@VisibilityLevel	= N'<#NV>', VisibilityLevel,	@VisibilityLevel)
				, ValueSource		= IIF(@ValueSource		= N'<#NV>', ValueSource,		@ValueSource)
				, ValueListID		= IIF(@ValueListID		= N'<#NV>', ValueListID,		@ValueListID)
				, ValueFormatID		= IIF(@ValueFormatID	= N'<#NV>', ValueFormatID,		@ValueFormatID)
				, Unit				= IIF(@Unit				= N'<#NV>', Unit,				@Unit)
				, Scale				= IIF(@Scale			= N'<#NV>', Scale,				@Scale)
				, Effect			= IIF(@Effect			= N'<#NV>', Effect,				@Effect)
				, EffectParameter	= IIF(@EffectParameter	= N'<#NV>', EffectParameter,	@EffectParameter)
			FROM planning.tdValueSeries
			WHERE ValueSeriesKey = @ValueSeriesKey;
				
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.2a - Delete zero values if ValueSeries is now numeric input series
			IF @IsNumeric = '1' AND @ValueSource = N'Input'
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueInt = 0; 
			END;

			-- Step 3.2b - Delete empty Strings if ValueSeries is now non-numeric input series
			IF @IsNumeric = '0' AND @ValueSource = N'Input' 
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueText = N'';
			END;

				
			-- STEP 3.3 - Update exists Values due to possible rescaling or changing type from Numeric to Non-Numeric
			DECLARE @vals AS TABLE (ValueKey INT NOT NULL, ValueInt INT NOT NULL, ValueText NVARCHAR(MAX) NOT NULL);

			-- Changing type Numeric / Non Numeric
			IF (@IsNumeric <> N'<#NV>' AND @OldIsNumeric <> @IsNumeric)
			BEGIN
				-- Copy ValueText to ValueInt, after Try_Cast to Int to get only valid numbers and multiplied by OldScale, as the int values use scale, while the text value not
				IF @IsNumeric = N'1'
				BEGIN							 
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ISNULL(TRY_CAST(ValueText AS INT), 0) * @OldScale, ''
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					-- If Numeric is made to non numeric the value in the Database must be be divided by scale and converted to text, as text dont uses scale
					IF (@OldScale <> 0)
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt / @OldScale AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey;	
					END
					ELSE
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey; 
					END;
				END;
			END;

			-- changing scaling
			-- If Isnumeric = 0 and not changed Nothing happens with the value, if scale is changed, as text values dont use the scale property.
			IF (@Scale <> N'<#NV>' AND @OldScale <> @Scale) AND (@IsNumeric = N'1' OR (@IsNumeric = N'<#NV>' AND @OldIsNumeric = 1))
			BEGIN
				-- Change Scale of existing values if scale is changed (when scale is changed, e.g. from 100 to 1000 the value in the database must be multiplied by 10)
				DECLARE @k FLOAT = @Scale / CAST(@OldScale AS FLOAT);	
			
				IF NOT EXISTS(SELECT 1 FROM @vals)
				BEGIN
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ValueInt * @k, '' 
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					UPDATE @vals
					SET ValueInt = ValueInt * @k; 
				END;
			END;
			
			IF EXISTS(SELECT 1 FROM @vals)
			BEGIN
				UPDATE v
				SET 
					 v.ValueInt		= newV.ValueInt
					,v.ValueText	= newV.ValueText 
				FROM planning.tfValues v INNER JOIN @vals newV 
					ON v.ValueKey = newV.ValueKey
				WHERE v.ValueSeriesKey = @ValueSeriesKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;

		SET @Resultcode = 201;
		COMMIT TRANSACTION sx_pf_POST_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryService;
GO
/*
MOVE rows older than 7 days from planning.tAPI_Log to planning.tAPI_Log_History table

Dependencies: No															

12/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return No Value

Test call
EXEC  [sx_pf_pPOST_LogArchive]
*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pPOST_Archive_Historic_LogEntrys]') AND type in (N'P', N'PC'))
	DROP PROCEDURE [sx_pf_pPOST_Archive_Historic_LogEntrys];
GO

CREATE PROCEDURE [sx_pf_pPOST_Archive_Historic_LogEntrys]
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @logKeys AS TABLE ([LogKey] [bigint] NOT NULL);
	DECLARE @Date DATETIME = GETDATE();

	INSERT TOP (500) INTO planning.tAPI_Log_History (
		[LogKey]
        , [SQLSystemUser]
        , [UserName]
        , [TransactUsername]
        , [ProcedureName]
        , [ParameterString]
        , [EffectedRows]
        , [ReturnCode]
        , [TimestampCall]
        , [TimestampReturn]
        , [Comment]
        , [ProcessCode] 
	)
	OUTPUT inserted.[LogKey] INTO @logKeys([LogKey])
   	SELECT [LogKey]
        , [SQLSystemUser]
        , [UserName]
        , [TransactUsername]
        , [ProcedureName]
        , [ParameterString]
        , [EffectedRows]
        , [ReturnCode]
        , [TimestampCall]
        , [TimestampReturn]
        , [Comment]
        , [ProcessCode]
	FROM planning.tAPI_Log
	WHERE DATEDIFF(DAY, [TimestampCall], @Date) > 7
	ORDER BY [LogKey]; 
											 
    WHILE @@ROWCOUNT > 0
	BEGIN
		DELETE FROM planning.tAPI_Log WHERE [LogKey] IN (SELECT [LogKey] FROM @logKeys);
		DELETE FROM @logKeys;

		INSERT TOP (500) INTO planning.tAPI_Log_History (
			[LogKey]
			, [SQLSystemUser]
			, [UserName]
			, [TransactUsername]
			, [ProcedureName]
			, [ParameterString]
			, [EffectedRows]
			, [ReturnCode]
			, [TimestampCall]
			, [TimestampReturn]
			, [Comment]
			, [ProcessCode] 
		)
		OUTPUT inserted.[LogKey] INTO @logKeys([LogKey])
   		SELECT [LogKey]
			, [SQLSystemUser]
			, [UserName]
			, [TransactUsername]
			, [ProcedureName]
			, [ParameterString]
			, [EffectedRows]
			, [ReturnCode]
			, [TimestampCall]
			, [TimestampReturn]
			, [Comment]
			, [ProcessCode]
		FROM planning.tAPI_Log
		WHERE DATEDIFF(DAY, [TimestampCall], @Date) > 7
		ORDER BY [LogKey];

	END;
END
GO

/*
Internal POST Operation for one API LogEntry, always caused, when a public procedure is called
Dependencies:
	- none
10/2015 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
ReturnCode according to http standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pPOST_API_LogEntry]') AND type in (N'P', N'PC'))
DROP PROCEDURE sx_pf_pPOST_API_LogEntry
GO

CREATE PROCEDURE sx_pf_pPOST_API_LogEntry 
	 @Username AS NVARCHAR(255)
	,@TransactUsername AS NVARCHAR(255)
	,@ProcedureName AS NVARCHAR(255)
	,@ParameterString AS NVARCHAR (MAX)
	,@EffectedRows AS INT
	,@ReturnCode AS INT
	,@TimestampCall AS Datetime
	,@Comment AS NVARCHAR (2000)

AS

		DECLARE @TimestampReturn AS Datetime = CURRENT_TIMESTAMP
		DECLARE @ProcessCode AS NVARCHAR(255) = ''
		DECLARE @DatabaseIsReadOnly AS INTEGER
	
		SELECT @DatabaseIsReadOnly = is_read_only FROM sys.databases WHERE Name = DB_NAME()
		
		IF NOT @DatabaseIsReadOnly = 1
			BEGIN
				INSERT INTO planning.tAPI_Log VALUES (SYSTEM_USER,@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
			END
GO

/*Testaufruf
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pPOST_API_LogEntry 'SQL','W8\admin','TestProcedure','ThisAreTheParameter',1,200,'20150731','Comment'
PRINT @RESULT
*/
/*
Procedure to POST an Process Log Entry
Gerd Tautenhahn 
07/2018

DataFactory 5.0

Sample

-- Start Log Entry
EXEC sx_pf_pPOST_Process_LogEntry 'Load', 'BusinessUnits', 'Create Factories', 'Start',200

-- anything happens now....


-- End Log Entry
EXEC sx_pf_pPOST_Process_LogEntry 'Load', 'BusinessUnits', 'Create Factories', 'End',200

SELECT *  FROM planning.tProcess_Log
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'sx_pf_pPOST_Process_LogEntry') AND type in (N'P', N'PC'))
DROP PROCEDURE sx_pf_pPOST_Process_LogEntry
GO

CREATE PROCEDURE sx_pf_pPOST_Process_LogEntry 
						 @MainProcess		NVARCHAR (255)
						,@SubProcess		NVARCHAR (255)
						,@Step				NVARCHAR (255)
						,@Start_End_Info	NVARCHAR (255)
						,@ResultCode		INT				= 0
						,@ValueText			NVARCHAR (MAX)	= ''
						,@ValueInt			BIGINT			= 0
AS 
	
	BEGIN

		INSERT INTO planning.tProcess_Log VALUES
				 (@MainProcess, @SubProcess, @Step, @Start_End_Info, CURRENT_TIMESTAMP, @ResultCode,@ValueText,@ValueInt)
	END

GO

-- No GRANT as only used inside other stored procedures
/*
Procedure to calculate the Rights Credits of an Cluster

- Every User with Cluserwrite Rights gets 2 C Points
- Every User with Clusterread Rights gets 1 C Points
- Every User with an Write Right on Factory Level gets 2 F Points per Factory
- Every User with an Read Right on Factory Level gets 1 Point per Factory
- Every User with an Write Right on an Productline gets 2 PL Points
- Every User with an Read Rights on an Productline gets 1 PL Point 

Its used to monitor Point count and see unusual values due to failures in Right import

Testcall
EXEC control.pGET_RightPointSummary

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'control.pGET_RightPointSummary') AND type in (N'P', N'PC'))
DROP PROCEDURE control.pGET_RightPointSummary
GO

CREATE PROCEDURE control.pGET_RightPointSummary

AS

SELECT
	 
	 Points.UserName
	,SUM(Points.C_Points) AS C_Points
	,SUM(Points.F_Points) AS F_Points
	,SUM(Points.PL_Points) AS PL_Points

	,SUM(Points.C_Points
		+Points.F_Points
		+Points.PL_Points) AS Full_Points

	,SUM( SUM(Points.C_Points )
		+ SUM(Points.F_Points )
		+ SUM(Points.PL_Points)) OVER () AS TotalPoints
FROM 
	(
		SELECT 
			 vUR.UserName
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS C_Points
			,0 AS F_Points
			,0 AS PL_Points
	
		FROM planning.trUserRights vUR

		WHERE vUR.FactoryID = '' AND vUR.ProductLineID = ''

	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS F_Points
			,0 AS PL_Points
	
		FROM planning.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID = ''


	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,0 AS F_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS PL_Points
	
		FROM planning.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID <> ''

	) Points

GROUP BY Points.UserName


GO
/*
Determines the User for the Transaction

This function returns:
	A: An active username from the internal DataFactory Usermanagement
	B: The string '403' if no authorized user was found

The function does it by
	1. checking the SQL connected User
	2. checking for FactoryService Action
	3. checking for action over public user 

Don't use the function IS_ROLEMEMBER, its buggy

02/2017 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
NO HTTP Standard - no public call

Testcall
DECLARE @Result NVARCHAR(255)
EXEC @Result = dbo.sx_pf_Determine_TransactionUsername 'SQL'
PRINT @Result

*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_Determine_TransactionUsername') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION dbo.sx_pf_Determine_TransactionUsername;
GO

CREATE FUNCTION dbo.sx_pf_Determine_TransactionUsername (@Username AS NVARCHAR(255)='')
RETURNS NVARCHAR (255) 
AS
BEGIN 
	DECLARE @TransactionUsername AS NVARCHAR (255) = '403'; --Default Value

	-- clean and declare
		SET @Username = LTRIM(RTRIM(@Username));
		DECLARE @IsFactoryService INT = 0;
		DECLARE @UserBlocked INT = 0;

	-- 1. Determine if Factory Service is acting (this means a DatabaseUser with the role "pf_PlanningFactoryService")
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;

	-- 2. If FactoryService is acting, every Username is accepted, else only the Original_Login() - which must be clearly requested by Keyword 'SQL'
		IF  @IsFactoryService = 1 
			SET @TransactionUsername = @Username;
		ELSE
			BEGIN
				IF @Username = 'SQL' SET @TransactionUsername = ORIGINAL_LOGIN()
					ELSE SET @TransactionUsername = '403'
			END

	-- 3. Block the user if this user don't exists a an active User
		IF NOT (SELECT Count(Userkey) FROM planning.trUser WHERE Username = @TransactionUsername AND Status = 'Active') = 1
			BEGIN
				SET @TransactionUsername = '403'
				SET	@UserBlocked = 1
			END
	
	-- 4. If the user was blocked, check if a public user exists, which can be used for the transaction
		IF @UserBlocked = 1
			BEGIN
				IF (SELECT Count(Userkey) FROM planning.trUser WHERE Username = 'public' AND Status = 'Active') = 1
					SET @TransactionUsername = 'public'
			END

	RETURN @TransactionUsername
END

GO

/*Testing
Assumtion:
- Database has three SQL User ('W10\admin', 'FactoryService' and 'sx')
- DFU has user 'W10\admin', 'sx' and user 'public'
- User public is Active\Inactive depending of test
- User 'W10\admin' has role db_owner
- User sx has role pf_PlanningFactoryUser
- User FactoryService has role pf_PlanningFactoryService



A: Test as User with role pf_PlanningFactoryUser (SQL User sx)
	A1: User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'sx'

	A2: User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'sx'

	A3: User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return '403'

	A4: User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')		-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'public'

B: Test as User with role db_owner (SQL User W10\admin)
	B1: User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'W10\admin'
																					
	B2: User exists in DFU, a public user exists									
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')		-> must return 'W10\admin'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'W10\admin'

	B3: User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return '403'

	B4: User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything') -> must return 'public'


C: Test as User with role pf_PlanningFactoryService (SQL User 'FactoryService') - the User FactoryService don't exits in DFU
	C1: Requested User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'

	C2: Requested User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'

	C3: Requested User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return '403'

	C4: Requested User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'public'

D: Test as User with role pf_PlanningFactoryService (SQL User 'FactoryService') - the User FactoryService exits in DFU
	D1: Requested User exists in DFU, no public user
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'FactoryService'

	D2: Requested User exists in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'sx'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('FactoryService')	-> must return 'FactoryService'

	D3: Requested User exists not in DFU, no public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return '403'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return '403'

	D4: Requested User exists not in DFU, a public user exists
		SELECT dbo.sx_pf_Determine_TransactionUsername ('')					-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('sx')				-> must return 'public'
		SELECT dbo.sx_pf_Determine_TransactionUsername ('anything')			-> must return 'public'


*/
/*
Procedure for materialization of UserRights
Gerd Tautenhahn for saxess software gmbh
DataFactory 5.0 
02/2018

Testcall
EXEC dbo.sx_pf_materialize_vUserRights
SELECT * FROM planning.trUserRights order by Username, FactoryID, ProductlineID
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_materialize_vUserRights') AND type in (N'P', N'PC'))
DROP PROCEDURE dbo.sx_pf_materialize_vUserRights

GO

CREATE PROCEDURE dbo.sx_pf_materialize_vUserRights 
AS 

BEGIN
	BEGIN TRY
		BEGIN TRANSACTION materialize_vUserRights


		-- Table to store the calculated Rights
		IF OBJECT_ID('tempdb..#RightsTemp') IS NOT NULL
		DROP TABLE #RightsTemp;
		
		CREATE TABLE #RightsTemp 
		(
			 Source					NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL							
			,UserKey				BIGINT									NOT NULL
			,Username				NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey				BIGINT									NOT NULL
			,ProductlineKey			BIGINT									NOT NULL
			,FactoryID				NVARCHAR (255) COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID			NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightExplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightImplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ReadCommentMandatory	INT 									NOT NULL
			,WriteCommentMandatory	INT										NOT NULL
		);

		-- Concept for saving only positive Rights, no Denys

		-- ClusterLevel			********************************************************************************************************************
		-- this are Rights for FactoryID = '' AND ProductLineID = ''

		-- Clusteradmins		*****************************
		-- Has Write Rights on Clusterlevel, no limitation possible
		-- Insert Write Rights on ClusterLevel
		-- Insert Write Rights on FactoryLevel
		-- Insert Write Rights on Productline Level

		-- ClusterReader		******************************
		-- Has Read Rights on Clusterlevel, may have additional Write Rights on Factory / Productline Level
		-- Insert Read Rights on ClusterLevel
		-- Insert Read Rights on FactoryLevel
		-- Insert Read Rights on Productline Level

		-- FactoryLevel			********************************************************************************************************************
		-- Rights defined on FactoryLevel with FactoryID <> 0 AND ProductlineID = ''
		-- Write Rights on Factories 
		-- Has Write Rights on a Factory - no limitation under this Factory possible
		-- Read Rights on Factories
		-- Has Read Rights on Factory Level, may have additional Write Rights on ProductlineLevel

		-- ProductlineLevel		*************************************************************************
		-- Rights defined on ProductlineLevel with FactoryID <> 0 AND ProductlineID <> 0
		-- Write Rights on Productlines
		-- Read Rights on Productlines

		-- Preparation			**************************************************************************
		-- Create Table with all rows, this are:
			-- for User with Rights on ClusterLevel		Linecount = 1 + FactoryCount				+ PLCount 
			-- for User with Rights on FactoryLevel		Linecount = 1 + FactoryWithRightCount		+ PLCount of FactoriesWith Rights
			-- for User with Rights on ProductlineLevel Linecount = 1 + FactoryCountPLWithRights	+ PLCount with Rights

		-- Helper Table with Stucture   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
		DROP TABLE #FullStructure;
				CREATE TABLE #FullStructure
						(
							 RowKey			BIGINT	IDENTITY (1,1)								NOT NULL
							,[Level]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryKey		BIGINT												NOT NULL
							,ProductLineKey BIGINT												NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductlineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
						);

				-- C Level
				INSERT INTO #FullStructure
					SELECT 
						 'Cluster'	AS Level
						,0			AS FactoryKey
						,0			AS ProductLineKey
						,''			AS FactoryID
						,''			ASProductLineID;
			 
				-- F Level
				INSERT INTO #FullStructure
					SELECT
						 'Factory'		AS Level
						,dF.FactoryKey
						,0				AS ProductLineKey
						,dF.FactoryID
						,''				AS ProductLineID
					FROM planning.tdFactories dF;
				
				-- PL Level
				INSERT INTO #FullStructure
					SELECT
						 'Productline'	AS Level
						,dF.FactoryKey
						,dPL.ProductLineKey
						,dF.FactoryID
						,dPL.ProductLineID
					FROM planning.tdProductLines dPL 
						LEFT JOIN planning.tdFactories dF
							ON dPL.FactoryKey = dF.FactoryKey;

		-- SELECT * FROM #FullStructure

		-- Fill UserRight Table with fitting RowCount and set all to read ************************************************************************

		-- 1a. The Full Structure Rows for User with Clusterrights  -> no additional Rows for Rights on Factory / Productline Level need
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ClusterRight' AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1				AS ReadCommentMandatory
					,1				AS WriteCommentMandatory
			FROM #FullStructure fs 
				LEFT JOIN planning.trUser rU
					ON 1=1
			WHERE 
					rU.Status = 'Active'
				-- Clusterrights
				AND rU.UserKey IN (SELECT UserKey FROM planning.trRights WHERE FactoryID = '' AND ProductLineID = '');

		-- 1b. Cluster Read right Row for Transfer for alle user with any right except the already done users with clusterrights
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-TransferCluser'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'				AS [RightExplicit]
					,'Read'				AS [RightImplizit]
					,1					AS ReadCommentMandatory
					,1					AS WriteCommentMandatory
			FROM #FullStructure fs
				
				LEFT JOIN planning.trUser rU
					ON 1=1

			WHERE 
					rU.Status = 'Active'
				AND fs.Level = 'Cluster'
				-- anybody who has any set right 
				AND rU.UserKey IN (SELECT DISTINCT UserKey FROM planning.trRights)
				-- and is not already ClusterRight Owner
				AND rU.Userkey NOT IN (SELECT DISTINCT UserKey FROM planning.trRights WHERE FactoryID = '');
			
		-- 1c. Rows for User with direct Factory Rights 
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-FactoryRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID FROM planning.trRights WHERE FactoryID <> '' AND ProductLineID = ''
					) rFactoryDirect
					
					ON FS.FactoryID = rFactoryDirect.FactoryID COLLATE DATABASE_DEFAULT
				
				INNER JOIN planning.trUser rU
					ON rFactoryDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					AND rFactoryDirect.FactoryID IS NOT NULL
					-- and is not already ClusterRight Owner
					AND rU.UserKey NOT IN (SELECT DISTINCT UserKey FROM planning.trRights WHERE FactoryID = '');

		--1d. Rows for Factory Transfer Rights for User with direct ProduclineRights Rights (but not already direct Factory / Clusterrights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-Transfer Factory'	AS [Source]
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM planning.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID  COLLATE DATABASE_DEFAULT-- Join only over Factory
				
				INNER JOIN planning.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status	= 'Active'
					AND fs.Level	= 'Factory'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM planning.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM planning.trRights WHERE FactoryID = '' )

			GROUP BY 
                 rU.UserKey
				,rU.UserName
				,fs.FactoryKey
				,fs.ProductLineKey
				,fs.FactoryID
				,fs.ProductlineID;

		--1e. Rows Rights for User with direct ProduclineRights Rights (which not already have this due to FactoryRights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ProductlineRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductlineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM planning.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID COLLATE DATABASE_DEFAULT
						AND FS.ProductlineID = rPLDirect.ProductLineID COLLATE DATABASE_DEFAULT
				
				INNER JOIN planning.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM planning.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM planning.trRights WHERE FactoryID = '' );

			--SELECT * FROM #RightsTemp rt ORDER BY Username,FactoryID,rt.ProductLineID;

	

			--2. Evaluate Rights to give write Rights and set Commentproperties#########################################################################

			-- 2a. SET Clusterrights
			UPDATE #RightsTemp 
		
			SET	 [Source]				= 'ClusterRight'		
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM planning.trRights 
										WHERE	
												FactoryID = '' 
											AND ProductLineID = ''
									) rR

						ON rt.UserKey = rR.UserKey
					;

			-- 2a. Set Factory Rights with downwards inheritance
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'FactoryRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM planning.trRights 
										WHERE	
												FactoryID <> '' 
											AND ProductLineID = ''
									) rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT;

			-- 2a. Set Productline Rights 
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'ProductlineRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt INNER JOIN 
									(
										SELECT * 
										FROM planning.trRights 
										WHERE	
												FactoryID <> '' 
											AND ProductLineID <> ''
										) rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
							AND rt.ProductLineID = rR.ProductLineID COLLATE DATABASE_DEFAULT;
					
		
			--SELECT * FROM	#RightsTemp ORDER BY Username,FactoryID,ProductLineID


			-- OUTPUT
			-- TRUNCATE TABLE planning.trUserRights -- deactivated, because a 
			/*	The minimum permission required is ALTER on table_name. TRUNCATE TABLE permissions default to the table owner,
				members of the sysadmin fixed server role, and the db_owner and db_ddladmin fixed database roles, 
				and are not transferable. However, you can incorporate the TRUNCATE TABLE statement within a module,
				such as a stored procedure, and grant appropriate permissions to the module using the EXECUTE AS clause.
			
				The EXECUTE AS Clause I did not master till now in foreign environments. 
			 */
			DELETE FROM planning.trUserRights; 

			INSERT INTO planning.trUserRights 
			SELECT 
				 UserKey
				,Username
				,FactoryID
				,ProductLineID
				,IIF(RightExplicit='-',RightImplicit,RightExplicit) AS [Right]
				,ReadCommentMandatory
				,WriteCommentMandatory

			FROM #RightsTemp

			COMMIT TRANSACTION materialize_vUserRights
		END TRY
		BEGIN CATCH
			Print 'Da ging was schief'
			ROLLBACK TRANSACTION materialize_vUserRights
		END CATCH
	END
GO

/*
GET Operation für Read Rechte auf die Clusterproperties
Benötigt schon Transaktionsuser, daher PRIVATE !
Gibt bei positivem Test auf Read Rechte für den angefragten Transaktionsuser Status 200 zurück
Abhängigkeiten:
	- keine
05/2015 für DataFactory 5.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ClusterPropertiesReadRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ClusterPropertiesReadRight]
GO

CREATE PROCEDURE sx_pf_pGET_ClusterPropertiesReadRight @TransactionUsername AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 2. testen ob der User auf Active gesetzt ist, jeder User darf die Properties lesen
		SELECT @RightCode=IIF ([Status] = 'Active',0, 401)FROM planning.trUser WHERE Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200
GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_ClusterPropertiesReadRight 'W8\admin'
		PRINT @RESULT
		*/

/*
GET Operation for Write Rights on Cluster
Needs already a Transactionsuser, thats why PRIVATE
Returns for Clusteradmins 200
Dependencies:
	- none
08/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ClusterWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ClusterWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_ClusterWriteRight @TransactionUsername AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --not implemented, must be never returned
		DECLARE @Right AS NVARCHAR (255) ='none'

		-- Check for Write Line on cluster
		SELECT @Right=Coalesce([Right],'no') FROM planning.trUserRights WHERE FactoryID = '' AND Username = @TransactionUsername
			-- return 200 only for Clusteradmin
			IF @Right <>'Write' SET @RightCode = 401 
			IF @Right = 'Write' SET @RightCode = 200
			RETURN @RightCode
GO

/*Testcall

-- Clusteradmin must return 200
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'W8\admin'
PRINT @RESULT

-- User which exists, but is no Cluster Admin must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'GUEBAU\wenzel'
PRINT @RESULT

-- User which not exists, must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ClusterWriteRight 'Dummy\Dummy'
PRINT @RESULT
*/




/*
GET Operation für Read Rechte auf einer Factory
Benötigt schon Transaktionsuser, daher PRIVATE !
Gibt bei positivem Test auf Read Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
07/2015 für DataFactory 5.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pGET_FactoryReadRight') AND type in (N'P', N'PC'))
DROP PROCEDURE dbo.sx_pf_pGET_FactoryReadRight
GO

CREATE PROCEDURE dbo.sx_pf_pGET_FactoryReadRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Factory besitzt
		SELECT @RightCode=IIF ([Right] = 'Write' OR [Right] = 'Read',0, 401)FROM planning.trUserRights WHERE FactoryID = @FactoryID AND ProductlineID = '' AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200
GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pGET_FactoryReadRight 'W8\admin','ZT'
		PRINT @RESULT
		*/

/*
GET Operation für Write Rechte auf einer Factory
Benötigt schon Transaktionsuser
Gibt bei positivem Test auf Write Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
04/2015 für DataFactory 5.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_FactoryWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_FactoryWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_FactoryWriteRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)
AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Factory besitzt
		SELECT @RightCode=IIF ([Right] = 'Write',0, 401)FROM planning.trUserRights WHERE FactoryID = @FactoryID AND ProductlineID = '' AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200

GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = sx_pf_pGET_FactoryWriteRight 'W8\admin','ZT'
		PRINT @RESULT
		*/

/*
GET Operation für Read Rechte auf einer Productline
Benötigt schon Transaktionsuser
Gibt bei positivem Test auf Read Rechte für den angefragten User Status 200 zurück
Abhängigkeiten:
	- keine
04/2015 für DataFactory 5.0
Gerd Tautenhahn für saxess-software gmbh
Rückgabewerte gemäß HTTP Standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'dbo.sx_pf_pGET_ProductlineReadRight') AND type in (N'P', N'PC'))
DROP PROCEDURE dbo.sx_pf_pGET_ProductlineReadRight
GO

CREATE PROCEDURE dbo.sx_pf_pGET_ProductlineReadRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255), @ProductlineID AS NVARCHAR(255)

AS

		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Productline besitzt
		SELECT @RightCode=IIF ([Right] = 'Write' OR [Right] = 'Read',0, 401)FROM planning.trUserRights WHERE FactoryID = @FactoryID AND ProductlineID = @ProductLineID AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @RightCode <> 0 RETURN 401 
			RETURN 200

GO

		/*Testaufruf
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pGET_ProductlineReadRight 'W8\adminr','ZT','ZT'
		PRINT @RESULT
		*/

/*
GET Operation for Write Rights on Productline
Needs already a Transactionsuser, thats why PRIVATE
Returns for User with Write Rights 200
Dependencies:
	- none
08/2016 for DataFactory 5.0
Gerd Tautenhahn for saxess-software gmbh
Return Value according to HTTP Standard
*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[sx_pf_pGET_ProductLineWriteRight]') AND type in (N'P', N'PC'))
DROP PROCEDURE [sx_pf_pGET_ProductLineWriteRight]
GO

CREATE PROCEDURE sx_pf_pGET_ProductLineWriteRight @TransactionUsername AS NVARCHAR(255), @FactoryID AS NVARCHAR(255), @ProductlineID AS NVARCHAR(255)

AS
		DECLARE @RightCode AS INT = 501 --Im Default not implemented, darf nie zurückgegeben werden
		DECLARE @Right as NVARCHAR (255) = 'none'

		-- 1. testen ob der User Schreibrechte für die  Productline besitzt
		SELECT @Right=Coalesce([Right],'none')FROM planning.trUserRights WHERE FactoryID = @FactoryID AND ProductlineID = @ProductLineID AND Username = @TransactionUsername
			-- falls keine Schreibrechte wird Prozedur abgebrochen
			IF @Right = 'Write' SET @RightCode = 200
			IF @Right <>'Write' SET @RightCode = 401 
			RETURN @RightCode

GO

/*Testcall
-- Clusteradmin must return 200
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'W8\admin','ZT','U'
PRINT @RESULT


-- User which exists, but is no Cluster Admin must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'GUEBAU\wenzel','ZT','U'
PRINT @RESULT


-- User which not exists, must return 401
DECLARE @RESULT AS NVARCHAR(255)
EXEC @RESULT = sx_pf_pGET_ProductLineWriteRight 'Dummy\Dummy','ZT','U'
PRINT @RESULT
*/

/*
    Product: OCT 5.0
    Object: integration.pGetLastDeltaValues
    Type: Procedure
    Description: Procedure to get the last Delta Load Timestamp per Accounting Area
    Author: saxess-software GmbH

*/


DROP PROCEDURE IF EXISTS integration.pGetLastDeltaValues;
GO

CREATE PROCEDURE integration.pGetLastDeltaValues
    @DataSourceKey	INT,
    @CompanyID		NVARCHAR(50),
	@Username		NVARCHAR(255)
AS
BEGIN
	DECLARE @ResultCode	INT	= 501;

	BEGIN TRY

		SELECT 
			 AccountingAreaCODE
			,MAX(DeltaValue)	AS MaxDeltaValue
		FROM integration.tFinAccountingJournal
		WHERE 
				DataSourceKey = @DataSourceKey 
			AND CompanyID = @CompanyID
		GROUP BY 
			AccountingAreaCODE

		SET @ResultCode = 200;	

	END TRY
	BEGIN CATCH

		BEGIN
			SET @ResultCode = 500;	
		END;

	END CATCH

	RETURN @ResultCode;
END;

GO


GRANT EXECUTE ON OBJECT ::integration.pGetLastDeltaValues TO pf_PlanningFactoryService;

GO


-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Procedure to get the last Delta Load Timestamp per Accounting Area',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Procedure',
    @level1name = N'pGetLastDeltaValues'
GO


/*
    Product: OCT 5.0
    Object: integration.pGetPagedProcessExecutionLog
    Type: Procedure
    Description: Procedure to get the entries of the table system.tProcessExecutionLog grouped by the execution id,
	in the reverse order and only a certain number of them starting from a start index
    Author: saxess-software GmbH

*/


DROP PROCEDURE IF EXISTS integration.pGetPagedProcessExecutionLog;
GO

CREATE PROCEDURE [integration].[pGetPagedProcessExecutionLog]
    @StartIndex		INT,
    @EndIndex		INT,
	@Username		NVARCHAR(255)
AS
BEGIN
	DECLARE @ResultCode	INT	= 501;

	BEGIN TRY

		SELECT system.tProcessExecutionLog.*
		FROM (
			SELECT DISTINCT ExecutionID, ROW_NUMBER() OVER(ORDER BY ExecutionID DESC) AS RowNumber
			FROM system.tProcessExecutionLog
			GROUP BY ExecutionID) AS AllRows RIGHT JOIN system.tProcessExecutionLog ON AllRows.ExecutionID = system.tProcessExecutionLog.ExecutionID
		WHERE AllRows.RowNumber BETWEEN @StartIndex AND @EndIndex
		ORDER BY RowKey DESC

		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH

		BEGIN
			SET @ResultCode = 500;	
		END;

	END CATCH
	
	RETURN @ResultCode;
END;

GO


GRANT EXECUTE ON OBJECT::integration.pGetPagedProcessExecutionLog TO pf_PlanningFactoryService;

GO

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Procedure to get the entries of the table system.tProcessExecutionLog grouped by the execution id,
	in the reverse order and only a certain number of them starting from a start index',
    @level0type = N'Schema',
    @level0name = N'integration',
    @level1type = N'Procedure',
    @level1name = N'pGetPagedProcessExecutionLog'
GO
/*
Procedure to delete exists file

-- Right check missing
-- Key check missing
	
Return code 200 if file exists and was deleted succesful

Testcall

EXEC planning.spDELETE_File 
	'SQL'
	,'ZT'
	,'U'
	,'1'
	,'1'
	;
*/

DROP PROCEDURE IF EXISTS planning.spDELETE_File;
GO

CREATE PROCEDURE planning.spDELETE_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@FactoryID, N'NULL') + N''', '''
														+ ISNULL(@ProductlineID, N'NULL') + N''', '''+ ISNULL(@ProductID, N'NULL') + N''', '''
														+ ISNULL(@FileID, N'NULL') + N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY
			BEGIN TRANSACTION DELETE_File;
			
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			IF NOT EXISTS (
				SELECT TOP (1) 1 
				FROM planning.tFiles
				WHERE 
						FactoryID		= @FactoryID
					AND ProductlineID	= @ProductlineID
					AND ProductID		= @ProductID
					AND FileID			= @FileID
					)
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			DELETE FROM planning.tFiles
			WHERE FactoryID		= @FactoryID
				AND ProductlineID	= @ProductlineID
				AND ProductID		= @ProductID
				AND FileID			= @FileID;
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION DELETE_File;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION DELETE_File;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spDELETE_File TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spDELETE_File TO pf_PlanningFactoryService;

/*
Procedure to Return body and file name of existing file
	

-- Right Check missing
-- Key Check missing


Testcall

EXEC planning.spGET_File 
	'SQL'
	,'ZT'
	,'U'
	,'1'
	,'2';

*/

DROP PROCEDURE IF EXISTS planning.spGET_File;
GO

CREATE PROCEDURE planning.spGET_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(@FactoryID, N'NULL')+ N''', '+ ISNULL(@ProductLineID, N'NULL')
															+ N''', '+ ISNULL(@ProductID, N'NULL')+ N''', '+ ISNULL(@FileID, N'NULL')
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
						

			IF NOT EXISTS (
				SELECT TOP (1) 1 
				FROM planning.tFiles
				WHERE 
						FactoryID		= @FactoryID
					AND ProductLineID	= @ProductLineID
					AND ProductID		= @ProductID
					AND FileID			= @FileID
				)
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			SELECT
				 FactoryID
				,ProductLineID
				,ProductID
				,FileID
				,[FileName] 
				,[Filename] + ISNULL(N'.' + FileExtension, N'') AS 'FileNameWithExtension'
				,FileExtension
				,FileBody
			FROM planning.tFiles
			WHERE 	
					FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID
				AND ProductID		= @ProductID
				AND FileID			= @FileID
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_File TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spGET_File TO pf_PlanningFactoryService;

/*
Procedure to Return list of existing files on a product

-- Right Check missing
-- Key Determination missing
	

Testcall

EXEC planning.spGET_FileList
	'SQL'
	,'ZT'
	,'U'
	,'1'

*/

DROP PROCEDURE IF EXISTS planning.spGET_FileList;
GO


CREATE PROCEDURE planning.spGET_FileList
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''

AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(@FactoryID, N'NULL')+ N''', '+ ISNULL(@ProductLineID, N'NULL')
															+ N''', '+ ISNULL(@ProductID, N'NULL')+ N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
					
			SELECT
				 FactoryID
				,ProductLineID
				,ProductID
				,FileID
				,[FileName] 
				,[Filename] + ISNULL(N'.' + FileExtension, N'') AS 'FileNameWithExtension'
				,FileExtension
			FROM planning.tFiles
			WHERE 	
					FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID
				AND ProductID		= @ProductID

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_FileList TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spGET_FileList TO pf_PlanningFactoryService;


/*
Procedure to GET the Structure of Planning Objects
FactoryID and ProductlineID can be passed in as single Value or as empty string for all

Output Limit may be F/PL/P/VS

Testcall

DECLARE @ParameterJSON NVARCHAR(MAX);
SET @ParameterJSON = N'{ "connector_type":"read"
						,"version":1
						,"FactoryID":"ZT"		
						,"ProductlineID":"U"	
						,"ProductID":"1"
						,"OutputLimit" : "VS"  
						}
			'
EXEC planning.spGET_Index_JSON 'SQL',@ParameterJSON


*/

DROP PROCEDURE IF EXISTS planning.spGET_Index_JSON;
GO

CREATE PROCEDURE planning.spGET_Index_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ParameterJSON	NVARCHAR(MAX) = ''

AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables
		DECLARE  @TransactUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ParameterJSON, N'NULL')+ N''''
				,@EffectedRows		INT				= 0					
				,@ResultCode		INT				= 501			
				,@TimestampCall		DATETIME		= CURRENT_TIMESTAMP
				,@Comment			NVARCHAR(2000)	= N'';					
		
	
		-- Procedure Variables
		DECLARE  @StartTime		TIME(4)
				,@ProcessTime	INT
				,@FactoryID		NVARCHAR(255)
				,@ProductlineID NVARCHAR(255)
				,@ProductID		NVARCHAR(255)
				,@OutputLimit	NVARCHAR(255) = ''

		BEGIN TRY

			SET @StartTime = CONVERT (time, SYSUTCDATETIME()) ; 

			-- Test for valid JSON ################################################################################
			IF ISJSON(@ParameterJSON) = 0 
				BEGIN
					PRINT 'Exit due to invalid JSON'
					RETURN 403
				END
	
			-- Extract Parameter FROM JSON ######################################################################
			
			SET @FactoryID		= JSON_Value(@ParameterJSON,'$.FactoryID');
			SET @ProductlineID	= JSON_Value(@ParameterJSON,'$.ProductlineID');
			SET @ProductID		= JSON_Value(@ParameterJSON,'$.ProductID');
			SET @OutputLimit	= JSON_Value(@ParameterJSON,'$.OutputLimit');

			IF @OutputLimit = '' OR @OutputLimit IS NULL SET @OutputLimit = 'VS'
		
			-- Rechte, Keys und G�ltigkeiten pr�fen #################################################################

			-- Rechte pr�fen und unberechtigte mit Einschr�nkungswarnung flaggen

			-- Ung�ltige IDs mit 404 flaggen




				-- Output als Recordset #################################################################
				IF @OutputLimit = 'VS' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductlineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25
							,dVS.ValueSeriesID
							,dVS.[IsNumeric]		
							,dVS.NameShort			AS ValueSeriesName
							,dVS.ValueSource
							,dVS.ValueListID
							,dVS.Scale
							,dVS.Unit
							,dVS.Effect
							,dVS.VisibilityLevel
							,dVS.ValueSeriesNo

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductlines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductlineKey = dP.ProductlineKey

							LEFT JOIN planning.tdValueSeries dVS
								ON dP.ProductKey = dVS.ProductKey

						WHERE 
					
							(
								@FactoryID = ''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductlineID = ''
								OR
								@ProductlineID = dPL.ProductlineID
							)

							AND 
							(
								@ProductID = ''
								OR
								@ProductID = dP.ProductID
							)

							ORDER BY dF.FactoryID, dPL.ProductlineID, dP.ProductID,dVS.ValueSeriesID
					END

				IF @OutputLimit = 'P' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductlineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductlines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductlineKey = dP.ProductlineKey

						WHERE 
					
							(
								@FactoryID = ''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductlineID = ''
								OR
								@ProductlineID = dPL.ProductlineID
							)
							AND 
							(
								@ProductID = ''
								OR
								@ProductID = dP.ProductID
							)

						ORDER BY dF.FactoryID, dPL.ProductlineID, dP.ProductID
					END

				IF @OutputLimit = 'PL' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductlineName

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductlines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey
						WHERE 
					
							(
								@FactoryID = ''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductlineID = ''
								OR
								@ProductlineID = dPL.ProductlineID
							)

						ORDER BY dF.FactoryID, dPL.ProductlineID
					END


				IF @OutputLimit = 'F' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName

						FROM planning.tdFactories dF

						WHERE 
					
							(
								@FactoryID = ''
								OR
								@FactoryID = dF.FactoryID
							)
						ORDER BY dF.FactoryID

					END



				-- Laufzeitermittlung, derzeit ohne Output
				SET @ProcessTime = DATEDIFF(ms,@StartTime,CONVERT (time, SYSUTCDATETIME()));

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN 200;
			
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_Index_JSON TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spGET_Index_JSON TO pf_PlanningFactoryService;



			

/*
Procedure to GET Values in JSON Format for MatrixConnector

MatrixConnector:
	- Shows Values which are combined from a Time Axis and a Structure Axis
	- The Structure Axis has the attributes F/PL/P/VS where each can be substituted by a * to mean all
	- The Time Axis shows one TimeID (later with flex Time concept it will show Year, Month, Day where also each one can be a *)

This GET Procedure returns for one TimeID all Values
The Values for this TimeID are requsted as JSON 

Retruns in a Array per Requested Number
	- Nummerischen Wert nach Operation SUM, MAX,...
	- String if single request
	- Datum
	- ValueComment if single request
	- Statistik f�r Aggregatfunktionen
		- Anzahl Treffer
		- F/PL/P/VS bei MAX, MIN


 Aktuelle Limitierungen:
	- Es wird immer der Aggreationstyp Summe zur�ckgegeben - egal was angefragt wird
	- Es werden derzeit nur direkte Einzelwerte verarbeitet und zur�ckgegeben
	- Es werden somit alle mit * angefragten Zeilen als NULL zur�ckgegeben
	- Er werden nur nummerische Werte zur�ckgegeben, noch keine Texte
	- Es werden noch keine Rechte verarbeitet


 TODO:
	- Nur Werte f�r die berechtigten Strukturinhalten zur�ckgeben
	- Info �ber Berechtigung im Text
	- Berechtigung von nicht gefunden unterscheiden
	- ValueSeries kann bei Aggregation beide mehrere Datentypen lieferen -> muss bei Aggregation Typ w�hlen ?
	- Werte k�nnen in unterschiedlichen ValueSeries unterschiedlich skaliert sein
	- Nach dem Schreiben von Werten nach DF kann ein Powerloading notwendig sein
	- Comments ausgeben
 

Testcall

DECLARE @ValuesJSON NVARCHAR(MAX)
SET @ValuesJSON = N'{	 "connector_type":"read"
						,"version":1
						,"Values":
								{
									"ValueSeries":	
										[											
											{	 "SeriesNumber"	: 1
												,"AggregateType": "SUM"
												,"FactoryID"	: "ZT"
												,"ProductlineID": "*"
											    ,"ProductID":"*"
												,"ValueSeriesID": "*"
											}
										]										
									,"TimeIDs":	
										[
												{	 "TimeNumber"	: 1
													,"TimeID"		: 201902
												}, 
												{	 "TimeNumber"	: 2
													,"TimeID"		: 20190215
												}, 
												{	 "TimeNumber"	: 3
													,"TimeID"		: 20190315
												}, 
												{	 "TimeNumber"	: 4
													,"TimeID"		: 20190415
												} 
										]
								}
						}
			'

EXEC planning.spGET_Values_JSON 'SQL', @ValuesJSON

EXEC planning.spGET_Values_JSON 'SQL','{"connector_type":"read","version":1,"Values":{"ValueSeries":[{"SeriesNumber":1,"AggregateType":"SUM","FactoryID":"ZT","ProductlineID":"U","ProductID":1,"ValueSeriesID":"K1"}],"TimeIDs":[{"TimeNumber":1,"TimeID":20200115},{"TimeNumber":2,"TimeID":20190215},{"TimeNumber":3,"TimeID":20190315},{"TimeNumber":4,"TimeID":20190415}]}}'

*/

DROP PROCEDURE IF EXISTS planning.spGET_Values_JSON;
GO

CREATE PROCEDURE planning.spGET_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows		INT				= 0					
				,@ResultCode		INT				= 501			
				,@TimestampCall		DATETIME		= CURRENT_TIMESTAMP
				,@Comment			NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY

			-- STEP 0.2 - Protect input parameters

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to invalid JSON', 100;
				END;
	
			-- Extract ValueSeries FROM JSON ######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			SELECT DISTINCT
			 	 SeriesNumber
				,AggregateType
				,CASE WHEN FactoryID = N'*' OR FactoryID LIKE N'*_%' THEN FactoryID ELSE [dbo].[sx_pf_pProtectID] (FactoryID) END AS 'FactoryID'
				,CASE WHEN ProductLineID = N'*' OR ProductLineID LIKE N'*_%' THEN ProductLineID ELSE [dbo].[sx_pf_pProtectID] (ProductLineID) END AS 'ProductLineID'
				,CASE WHEN ProductID = N'*' OR ProductID LIKE N'*_%' THEN ProductID ELSE [dbo].[sx_pf_pProtectID] (ProductID) END AS 'ProductID'
				,CASE WHEN ValueSeriesID = N'*' OR ValueSeriesID LIKE N'*_%' THEN ValueSeriesID ELSE [dbo].[sx_pf_pProtectID] (ValueSeriesID) END AS 'ValueSeriesID'
			INTO #RequestedValues
			FROM OPENJSON (@ValuesJSON, N'$.Values.ValueSeries') 
			WITH (
				SeriesNumber	INT				N'$.SeriesNumber'
				,AggregateType	NVARCHAR(50)	N'$.AggregateType'
				,FactoryID		NVARCHAR(255)	N'$.FactoryID'
				,ProductLineID	NVARCHAR(255)	N'$.ProductlineID'
				,ProductID		NVARCHAR(255)	N'$.ProductID'
				,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
			);
			
			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				WHERE SeriesNumber IS NULL 
					OR AggregateType IS NULL 
					OR FactoryID IS NULL 
					OR ProductLineID IS NULL 
					OR ProductID IS NULL 
					OR ValueSeriesID IS NULL
					OR FactoryID = N''
					OR ProductLineID = N''
					OR ProductID = N''
					OR ValueSeriesID = N'')
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				GROUP BY SeriesNumber
				HAVING COUNT (SeriesNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON', 100;
			END;

			-- Extract TimeIDs FROM JSON #######################################################################

			DROP TABLE IF EXISTS #RequestedTimes;

			;WITH times AS (
				SELECT DISTINCT
					TimeNumber
					,TimeID
				FROM OPENJSON (@ValuesJSON, N'$.Values.TimeIDs') 
				WITH (
						TimeNumber		INT		N'$.TimeNumber'
						,TimeID			INT		N'$.TimeID'
				)
			)
			SELECT *
				, CASE	WHEN TimeID BETWEEN 1000 AND 9999			THEN TimeID * 10000 + 0101
						WHEN TimeID BETWEEN 100001 AND 999912		THEN TimeID * 100 + 01
						WHEN TimeID BETWEEN 10000101 AND 99991231	THEN TimeID
						ELSE NULL END AS 'MinTimeID'
				, CASE	WHEN TimeID BETWEEN 1000 AND 9999			THEN TimeID * 10000 + 1231
						WHEN TimeID BETWEEN 100001 AND 999912		THEN TimeID * 100 + 31
						WHEN TimeID BETWEEN 10000101 AND 99991231	THEN TimeID
						ELSE NULL END AS 'MaxTimeID'
			INTO #RequestedTimes
			FROM times;

			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				WHERE TimeNumber IS NULL 
					OR MinTimeID IS NULL
					OR MaxTimeID IS NULL
			) 
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				GROUP BY TimeNumber
				HAVING COUNT (TimeNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON', 100;
			END;

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- STEP 3.0 - Search ValueSeries and User`s rights

			DROP TABLE IF EXISTS #ValueSeries;
			
			;WITH search AS (
				SELECT SeriesNumber
					,AggregateType
					,FactoryID
					,ProductLineID
					,ProductID
					,ValueSeriesID

					,CASE WHEN FactoryID		= N'*' THEN 1 WHEN FactoryID		LIKE N'*_%' THEN 2 ELSE 0 END AS FactoryIDSearchType
					,CASE WHEN ProductLineID	= N'*' THEN 1 WHEN ProductLineID	LIKE N'*_%' THEN 2 ELSE 0 END AS ProductLineIDSearchType
					,CASE WHEN ProductID		= N'*' THEN 1 WHEN ProductID		LIKE N'*_%' THEN 2 ELSE 0 END AS ProductIDSearchType
					,CASE WHEN ValueSeriesID	= N'*' THEN 1 WHEN ValueSeriesID	LIKE N'*_%' THEN 2 ELSE 0 END AS ValueSeriesIDSearchType
					
					,CASE WHEN FactoryID		LIKE N'*_%' THEN SUBSTRING(FactoryID, 2, 300)		ELSE NULL END AS FactoryIDPattern
					,CASE WHEN ProductLineID	LIKE N'*_%' THEN SUBSTRING(ProductLineID, 2, 300)	ELSE NULL END AS ProductLineIDPattern
					,CASE WHEN ProductID		LIKE N'*_%' THEN SUBSTRING(ProductID, 2, 300)		ELSE NULL END AS ProductIDPattern
					,CASE WHEN ValueSeriesID	LIKE N'*_%' THEN SUBSTRING(ValueSeriesID, 2, 300)	ELSE NULL END AS ValueSeriesIDPattern
				FROM #RequestedValues
			)
			SELECT 
				rv.SeriesNumber
				,rv.AggregateType
				,dF.FactoryID
				,dPL.ProductlineID
				,dP.ProductID
				,dVS.ValueSeriesID
				,dF.FactoryKey
				,dPL.ProductLineKey
				,dP.ProductKey
				,dVS.ValueSeriesKey
				,dVS.Scale
				,dVS.[IsNumeric]
				,rF.[Row] AS 'FactoryRightRow'
				,rPL.[Row] AS 'ProductLineRightRow'
			INTO #ValueSeries
			FROM search rv
				INNER JOIN planning.tdFactories dF
					ON rv.FactoryIDSearchType = 1 
						OR (rv.FactoryIDSearchType = 2 AND df.FactoryID LIKE rv.FactoryIDPattern) 
						OR (rv.FactoryIDSearchType = 0 AND df.FactoryID = rv.FactoryID) 
				INNER JOIN planning.tdProductLines dPL
					ON dPL.FactoryKey = dF.FactoryKey
						AND (rv.ProductLineIDSearchType = 1 
							OR (rv.ProductLineIDSearchType = 2 AND dPL.ProductLineID LIKE rv.ProductLineIDPattern) 
							OR (rv.ProductLineIDSearchType = 0 AND dPL.ProductLineID = rv.ProductLineID))
				INNER JOIN planning.tdProducts dP
					ON dP.FactoryKey = dF.FactoryKey 
						AND dP.ProductLineKey = dPL.ProductLineKey
						AND (rv.ProductIDSearchType = 1 
							OR (rv.ProductIDSearchType = 2 AND dP.ProductID LIKE rv.ProductIDPattern) 
							OR (rv.ProductIDSearchType = 0 AND dP.ProductID = rv.ProductID))
				INNER JOIN planning.tdValueSeries dVS
					ON dVS.FactoryKey = dF.FactoryKey 
						AND dVS.ProductLineKey = dPL.ProductLineKey
						AND dVS.ProductKey = dP.ProductKey
						AND (rv.ValueSeriesIDSearchType = 1 
							OR (rv.ValueSeriesIDSearchType = 2 AND dVS.ValueSeriesID LIKE rv.ValueSeriesIDPattern) 
							OR (rv.ValueSeriesIDSearchType = 0 AND dVS.ValueSeriesID = rv.ValueSeriesID))
				LEFT JOIN planning.trUserRights rF 
					ON df.FactoryID = rF.FactoryID 
						AND rF.ProductlineID = N''
						AND rF.Username = @TransactionUsername
						AND (rF.[Right] = N'Write' OR rF.[Right] = N'Read')
				LEFT JOIN planning.trUserRights rPL 
					ON df.FactoryID = rPL.FactoryID 
						AND dPL.ProductlineID = rPL.ProductLineID 
						AND rPL.Username = @TransactionUsername
						AND (rPL.[Right] = N'Write' OR rPL.[Right] = N'Read');

			-- STEP 3.1 - Searching requested values

			DROP TABLE IF EXISTS #Values;

			SELECT
				s.SeriesNumber
				,s.FactoryKey
				,s.ProductLineKey
				,s.ProductKey
				,s.ValueSeriesKey
				,s.FactoryID
				,s.ProductLineID
				,s.ProductID
				,s.ValueSeriesID
				,ISNULL(CASE WHEN s.[IsNumeric] = 1 THEN CAST(fV.ValueInt AS Money) / s.Scale ELSE 0 END, 0) AS 'ValueNumeric'
				,ISNULL(CASE WHEN s.[IsNumeric] = 0 THEN fV.ValueText ELSE N'' END, N'') AS 'ValueString'
				,fV.ValueComment
				,t.TimeNumber
				,fV.TimeID
				,s.[IsNumeric]
			INTO #Values
			FROM #ValueSeries s
				INNER JOIN planning.tfValues fV
					ON fV.FactoryKey = s.FactoryKey 
						AND fV.ProductLineKey = s.ProductLineKey
						AND fV.ProductKey = s.ProductKey
						AND fV.ValueSeriesKey = s.ValueSeriesKey
						AND NOT s.FactoryRightRow IS NULL 
						AND NOT s.ProductLineRightRow IS NULL
				INNER JOIN #RequestedTimes t
					ON fV.TimeID BETWEEN t.MinTimeID AND t.MaxTimeID;

			-- STEP 3.2 - Insert requested values into result table

			DROP TABLE IF EXISTS #ValueSet;

			CREATE TABLE #ValueSet (
				SeriesNumber		INT				NOT NULL
				,AggregateType		NVARCHAR(50)	NOT NULL
				,FactoryID			NVARCHAR(255)	NOT NULL
				,ProductlineID		NVARCHAR(255)	NOT NULL
				,ProductID			NVARCHAR(255)	NOT NULL
				,ValueSeriesID		NVARCHAR(255)	NOT NULL
				
				,TimeNumber			INT				NOT NULL
				,TimeID				INT				NOT NULL
				
				,FactoryKey			BIGINT			NULL
				,ProductLineKey		BIGINT			NULL
				,ProductKey			BIGINT			NULL
				,ValueSeriesKey		BIGINT			NULL

				,FactoryCount		INT				NULL
				,ProductLineCount	INT				NULL
				,ProductCount		INT				NULL
				,ValueSeriesCount	INT				NULL

				,ValueNumeric		MONEY			NULL
				,ValueString		NVARCHAR(4000)	NULL
				,ValueDate			DATE			NULL
				,ValueComment		NVARCHAR(4000)  NULL

				,ReturnMessage		NVARCHAR(255)	NULL
				,ReturnCode			INT				NULL
				,ProcessFlag		INT				NULL
				);

			INSERT INTO #ValueSet (SeriesNumber, AggregateType, FactoryID, ProductlineID, ProductID, ValueSeriesID, TimeNumber, TimeID)
			SELECT v.SeriesNumber	
				,v.AggregateType	
				,v.FactoryID		
				,v.ProductlineID		
				,v.ProductID			
				,v.ValueSeriesID		
				,t.TimeNumber	
				,t.TimeID
			FROM #RequestedValues v, #RequestedTimes t;

			-- STEP 3.1 - Set 'No rights' comments  		
			
			;WITH series AS (
				SELECT 
					FactoryID 
					,ProductLineID
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				WHERE FactoryRightRow IS NULL 
					OR ProductLineRightRow IS NULL
				GROUP BY FactoryID, ProductLineID
			)
			UPDATE s
			SET	 ReturnMessage		= N'No rights'
				,ReturnCode			= 403
				,FactoryCount		= 0
				,ProductLineCount	= 0
				,ProductCount		= 0
				,ValueSeriesCount	= 0
				,ValueNumeric		= CASE WHEN v.[IsNumeric] = 1 THEN 0 ELSE NULL END
				,ValueString		= CASE WHEN v.IsString = 0 THEN N'' ELSE NULL END
				,ValueComment		= N''
			FROM #ValueSet s
				INNER JOIN series v
					ON s.FactoryID = v.FactoryID 
						AND s.ProductLineID = v.ProductLineID;

			---- STEP 3.2 - Fill result table with exists values, make aggregation

			;WITH vals AS (
				SELECT 
					SeriesNumber
					,TimeNumber
					,COUNT(DISTINCT FactoryKey)		AS 'FactoryCount'
					,COUNT(DISTINCT ProductLineKey)	AS 'ProductLineCount'
					,COUNT(DISTINCT ProductKey)		AS 'ProductCount'
					,COUNT(DISTINCT ValueSeriesKey)	AS 'ValueSeriesCount'
					,MAX(FactoryKey)				AS 'FactoryKey'
					,MAX(ProductLineKey)			AS 'ProductLineKey'
					,MAX(ProductKey)				AS 'ProductKey'
					,MAX(ValueSeriesKey)			AS 'ValueSeriesKey'
					,SUM(ValueNumeric)				AS 'ValueNumeric'
				FROM #Values
				GROUP BY SeriesNumber, TimeNumber
			),
			comments AS (
				SELECT TOP (1) WITH TIES
					SeriesNumber
					,TimeNumber
					,ValueComment
				FROM #Values
				WHERE NOT ValueComment IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductlineID, ProductID, ValueSeriesID, TimeID DESC)
			),
			stringValues AS (
				SELECT TOP (1) WITH TIES
					SeriesNumber
					,TimeNumber
					,ValueString
				FROM #Values
				WHERE [IsNumeric] = 0
					AND NOT ValueString IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductlineID, ProductID, ValueSeriesID, TimeID DESC)
			)
			UPDATE s
			SET s.FactoryKey		= CASE WHEN s.FactoryID		LIKE N'*%' THEN NULL ELSE v.FactoryKey		END
				,s.ProductLineKey	= CASE WHEN s.ProductLineID LIKE N'*%' THEN NULL ELSE v.ProductLineKey	END
				,s.ProductKey		= CASE WHEN s.ProductID		LIKE N'*%' THEN NULL ELSE v.ProductKey		END
				,s.ValueSeriesKey	= CASE WHEN s.ValueSeriesID LIKE N'*%' THEN NULL ELSE v.ValueSeriesKey	END 
				,s.FactoryCount		= v.FactoryCount
				,s.ProductLineCount = v.ProductLineCount
				,s.ProductCount		= v.ProductCount
				,s.ValueSeriesCount = v.ValueSeriesCount
				,s.ValueNumeric		= v.ValueNumeric
				,s.ValueString		= ISNULL(sv.ValueString, N'')
				,s.ValueComment		= ISNULL(c.ValueComment, N'')
			FROM #ValueSet s
				INNER JOIN vals v
					ON s.SeriesNumber = v.SeriesNumber
						AND s.TimeNumber = v.TimeNumber
				LEFT JOIN comments c
					ON s.SeriesNumber = c.SeriesNumber
						AND s.TimeNumber = c.TimeNumber
				LEFT JOIN stringValues sv
					ON s.SeriesNumber = sv.SeriesNumber
						AND s.TimeNumber = sv.TimeNumber
			WHERE s.FactoryCount IS NULL; 		

			---- STEP 3.4 - Fill ValueNumeric, ValueComment and ValueString columns and set zero-count for non exists values

			;WITH series AS (
				SELECT 
					SeriesNumber
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				GROUP BY SeriesNumber
			)
			UPDATE s
			SET FactoryCount		= ISNULL(FactoryCount, 0)
				,ProductLineCount	= ISNULL(ProductLineCount, 0)
				,ProductCount		= ISNULL(ProductCount, 0)
				,ValueSeriesCount	= ISNULL(ValueSeriesCount, 0)
				,ValueNumeric		= NULL
				,ValueString		= NULL
				,ValueComment		= NULL
			FROM #ValueSet s 
				INNER JOIN series v
					ON s.SeriesNumber = v.SeriesNumber
			WHERE s.FactoryCount IS NULL;

			-- STEP 3.5 - Output result

			SELECT
				ROW_NUMBER() OVER (ORDER BY SeriesNumber, FactoryID, ProductlineID, ProductID, ValueSeriesID, TimeNumber) AS RowKey			
				,SeriesNumber
				,TimeNumber
				,AggregateType	
				,FactoryID		
				,ProductlineID	
				,ProductID		
				,ValueSeriesID	
		
				,FactoryKey		
				,ProductLineKey	
				,ProductKey		
				,ValueSeriesKey	
		
				,FactoryCount	
				,ProductLineCount
				,ProductCount	
				,ValueSeriesCount	
		
				,TimeID			
				,ValueNumeric	
				,ValueString	
				,ValueDate		
				,ValueComment	
				,IIF(ReturnMessage IS NULL,'',ReturnMessage) AS ReturnMessage
				,IIF(ReturnCode IS NULL,200,ReturnCode) AS ReturnCode
				,ProcessFlag	
			FROM #ValueSet;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_Values_JSON TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spGET_Values_JSON TO pf_PlanningFactoryService;



			
/*
Procedure to Create new file or change exists file
@FileBody can`t be null

If @FileKey is null will create new file
	@FileExtension can be null (will be created file without extension)

If @FileKey is NOT null will change exists file
	If @FileExtension is NULL then don`t change extension of exists file
	If @FileExtension is empty then set to NULL extension of exists file (file without extension)
	Otherwise change extension to @FileExtension
	
Return table with columns: FileKey BIGINT, FileName NVARCHAR(MAX)


Testcall

EXEC planning.spPOST_File 
	'SQL'
	,'ZT'
	,'U'
	,'1'
	,'23'
	,'Hallo'
	,'pdf'
	,0x3;

SELECT * FROM planning.tFiles
*/

DROP PROCEDURE IF EXISTS planning.spPOST_File;
GO


CREATE PROCEDURE planning.spPOST_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
	,@FileName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@FactoryID, N'NULL') + N''', '''
													+ ISNULL(@ProductLineID, N'NULL') + N''', '''+ ISNULL(@ProductID, N'NULL') + N''', ''' + ISNULL(@FileID, N'NULL') + N''', '''
													+ ISNULL(@FileExtension, N'NULL') + N''', ' + ISNULL(CAST(DATALENGTH(@FileBody) AS NVARCHAR(MAX)) + N' bytes', N'NULL') 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N''
			,@FactoryKey			INT				= 0
			,@ProductlineKey		INT				= 0
			,@ProductKey			INT				= 0;
			
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_File;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

			-- Determine Keys
			SELECT @FactoryKey = FactoryKey, @ProductlineKey = ProductLineKey, @ProductKey = ProductKey 
			FROM planning.tdProducts 
			WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID

			IF @ProductKey = 0 
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'Product not found', 100;
			END;


			-- if FileID * is used, determine next free FileID
			IF @FileID = '*' 
				BEGIN
					SELECT @FileID = TRY_CAST(MAX(FileID) AS INTEGER) +1 FROM planning.tFiles WHERE FactoryID = @FactoryID AND ProductLineID= @ProductLineID AND ProductID = @ProductID 
				END

			-- Delete File
			DELETE FROM planning.tFiles 
				WHERE ProductKey	= @ProductKey
				AND FileID			= @FileID
				
				IF LEN(@FileExtension) = 0 
				BEGIN
					SET @FileExtension = NULL;
				END;

				-- STEP 2.2 - Creating new file

				INSERT INTO planning.tFiles (FactoryKey,ProductLineKey,ProductKey,FactoryID, ProductLineID, ProductID, FileID,[FileName],FileExtension, FileBody)
				VALUES (@FactoryKey, @ProductLineKey, @ProductKey, @FactoryID, @ProductLineID,@ProductID, @FileID, @FileName,@FileExtension, @FileBody);

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_File;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_File;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::planning.spPOST_File TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spPOST_File TO pf_PlanningFactoryService;

/*
Procedure to Create new file or change exists file from a local SQL Server filepath or shared folder path
File must extists and can`t be empty.
File extensions extracted from @FilePath.

If @FileKey is NULL will create new file
If @FileKey is NOT NULL will change exists file
	
Return table with columns: FileKey BIGINT, FileName NVARCHAR(MAX)

TODO: Must be adjusted to FID;PLID;PID !!

Testcall

EXEC planning.spPOST_FileFromPath 
	'SQL'
	,NULL						-- use NULL to send new file !
	,'c:\test.txt';
*/

DROP PROCEDURE IF EXISTS planning.spPOST_FileFromPath;
GO

CREATE PROCEDURE planning.spPOST_FileFromPath
	 @Username		NVARCHAR(255)	= N''
	,@FileKey		BIGINT			= NULL
	,@FilePath		NVARCHAR(300)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(CAST(@FileKey AS NVARCHAR(MAX)), N'NULL') + N', ''' + ISNULL(@FilePath, N'NULL') + N'''' 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';
		IF @FilePath IS NULL SET @FilePath = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_FileFromPath;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			SET @FilePath = REPLACE(REPLACE(dbo.sx_pf_pProtectString (@FilePath), N'/*', N''), N'*\', N'');

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF LEN(@FilePath) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FilePath is empty', 100;
			END;

			IF NOT @FileKey IS NULL 
				AND NOT EXISTS (
					SELECT TOP (1) 1 
					FROM planning.tFiles
					WHERE FileKey = @FileKey) 
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			-- STEP 2.1 - Read FileBody from path

			DECLARE @FileBody VARBINARY(MAX),
				@SQLQuery NVARCHAR(MAX),
				@ParmDefinition NVARCHAR(30) = N'@Data VARBINARY(MAX) OUTPUT';

			SET @SQLQuery = N'
				SET @Data = (SELECT * FROM OPENROWSET(
				BULK N''' + @FilePath + ''', SINGLE_BLOB, MAXERRORS=0) AS InputFile)';

			EXEC sp_executesql
				@SQLQuery
				,@ParmDefinition
				,@Data = @FileBody OUTPUT;

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

		    -- STEP 2.2 - Extract FileExtension from path
			DECLARE @FileExtension	NVARCHAR(50) = NULL
				,@DotPos INT;

			SET @DotPos = CHARINDEX(N'.', REVERSE(@FilePath));
			
			IF @DotPos > 1
			BEGIN
				DECLARE @PathLength INT;

				SET @PathLength = LEN(@FilePath);

				IF CHARINDEX(N'\', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N'/', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N':', @FilePath, @PathLength - @DotPos + 1) = 0
				BEGIN
					SET @FileExtension = RIGHT(@FilePath, @DotPos - 1);
				END;
			END;

			DECLARE @UpdatedFile AS TABLE (FileKey BIGINT, FileExtension NVARCHAR(50));

			IF NOT @FileKey IS NULL 
			BEGIN
				-- STEP 2.3 - Updating exists file. 

				UPDATE planning.tFiles
				SET FileExtension = @FileExtension
					,FileBody = @FileBody
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				WHERE FileKey = @FileKey;

			END
			ELSE
			BEGIN	
				-- STEP 2.4 - Creating new file

				INSERT planning.tFiles (FileExtension, FileBody)
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				VALUES (@FileExtension, @FileBody);

			END;

			-- STEP 3.0 - Output result

			SELECT
				 FileKey AS 'FileKey'
				,CAST(FileKey AS NVARCHAR(MAX)) + ISNULL(N'.' + FileExtension, N'') AS 'FileName'
			FROM @UpdatedFile;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_FileFromPath;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_FileFromPath;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::planning.spPOST_FileFromPath TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spPOST_FileFromPath TO pf_PlanningFactoryService;


/*
Procedure to POST Values in JSON Format 
Preconditions
- A value can be sended only on full qualified single fact -> There is no write on summary level possible
- A value "NULL" can be sended, this means delete the existing value

Functionality:
	- Values for not existing ValueSeriesID are thrown away and reported as ignored
	- Values for not existing TimeIDs are thrown away and reported as ignored
	- only values for elements where the user has write rights are excepted
	
	- flag values without writerights
	- delete existing values for given F/PL/P/VS in table fValues
	- Scale numeric values based on scale in dValueSeries

	- Only values of type string are sended, DB must put it in the right type
	- if in dValueSeries 
		IsNumeric = 0 -> put Value in ValueString
		IsNumeric = 1 -> put it in ValueInt, after scaling with Scale. Decimal separator is the dot "."
	- Special cases
		IsNumeric = 0 AND Value IS NULL -> DELETE, no write
		IsNumeric = 1 AND Value IS NULL -> DELETE, no write
		IsNumeric = 1 AND Value = 0 AND ValueSeriesID LIKE "_0" -> write


The Procedure returns all sended values as recordset with the process result

Testcall

DECLARE @ValuesJSON NVARCHAR(MAX)
SET @ValuesJSON = N'{"connector_type":"write","version":1,"Values":[

 {"ValueNumber":1	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"BUD_Art"	,"TimeID":"10000501"	,"Value":"Basisbudget"	,"ValueComment":null}
,{"ValueNumber":2	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"BEZ"		,"TimeID":"10000501"	,"Value":"Vorgabebudget","ValueComment":null}
,{"ValueNumber":3	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"J"		,"TimeID":"10000501"	,"Value":0				,"ValueComment":null}
,{"ValueNumber":4	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"BETR"		,"TimeID":"10000501"	,"Value":770000			,"ValueComment":null}
,{"ValueNumber":5	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"BEM"		,"TimeID":"10000501"	,"Value":null			,"ValueComment":null}
,{"ValueNumber":6	,"FactoryID":"LAB"	,"ProductlineID":"001"	,"ProductID":"9"	,"ValueSeriesID":"STAT_BUD"	,"TimeID":"10000501"	,"Value":"Genehmigt"	,"ValueComment":null}

]

}'

EXEC planning.spPOST_Values_JSON 'SQL', @ValuesJSON


*/

DROP PROCEDURE IF EXISTS planning.spPOST_Values_JSON;
GO

CREATE PROCEDURE planning.spPOST_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows		INT				= 0					
				,@ResultCode		INT				= 501			
				,@TimestampCall		DATETIME		= CURRENT_TIMESTAMP
				,@Comment			NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY
			BEGIN TRANSACTION spPOST_Values_JSON;

			-- STEP 0.2 - Protect input parameters

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (technical)', 100;
			END;
			
			-- Extract ValueSeries FROM JSON ######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			;WITH vals AS 
				(
					SELECT DISTINCT
						 ValueNumber
						,dbo.sx_pf_pProtectID (FactoryID)		AS 'FactoryID'
						,dbo.sx_pf_pProtectID (ProductLineID)	AS 'ProductLineID'
						,dbo.sx_pf_pProtectID (ProductID)		AS 'ProductID'
						,dbo.sx_pf_pProtectID (ValueSeriesID)	AS 'ValueSeriesID'
						,dbo.sx_pf_pProtectID (TimeID)			AS 'TimeID'
						,[Value]
						,ValueComment
					FROM OPENJSON (@ValuesJSON, N'$.Values') 
					WITH (
						 ValueNumber		INT				N'$.ValueNumber'
						,FactoryID		NVARCHAR(255)	N'$.FactoryID'
						,ProductLineID	NVARCHAR(255)	N'$.ProductlineID'
						,ProductID		NVARCHAR(255)	N'$.ProductID'
						,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
						,TimeID			INT				N'$.TimeID'
						,[Value]		NVARCHAR(MAX)	N'$.Value'
						,ValueComment	NVARCHAR(MAX)	N'$.ValueComment'
						)
					)
			-- Version Kyrill - i.m.o. to complex
			-- SELECT TOP (1) WITH TIES * 
			-- INTO #RequestedValues
			-- FROM vals
			-- ORDER BY ROW_NUMBER() OVER(PARTITION BY FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeID ORDER BY ValueNumber DESC); 

			SELECT * 
				INTO #RequestedValues
			FROM vals

			-- Debug: SELECT * FROM #RequestedValues

			-- Check for logical correct values #############################################
			-- no NULL Values contained in IDs - Value may be null due to deleted value
			IF EXISTS (
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				WHERE 
					   ValueNumber		IS NULL
					OR FactoryID		IS NULL
					OR ProductLineID	IS NULL
					OR ProductID		IS NULL
					OR ValueSeriesID	IS NULL
					OR TimeID			IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					OR ValueSeriesID	= N''
					OR TimeID			= 0
			-- no doubled ValueNumbers contained
			) OR EXISTS (
				SELECT TOP (1) 1
				FROM #RequestedValues
				GROUP BY ValueNumber
				HAVING COUNT(ValueNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (logical)', 100;
			END;

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- STEP 1.2 - Search ValueSeries and User`s rights

			DROP TABLE IF EXISTS #Values;

			SElECT 
				 rv.ValueNumber
				,dF.FactoryKey
				,dPL.ProductLineKey
				,dP.ProductKey
				,dVS.ValueSeriesKey
				,t.TimeID
				,UR.[Row]	AS 'RightRow'
				,CASE 
					WHEN NOT UR.[Row] IS NULL AND dVS.[IsNumeric] = 1 THEN TRY_PARSE(rv.[Value] AS MONEY USING 'en-En') * dVS.Scale 
					ELSE NULL 
				 END AS 'ValueInt'
				,CASE 
					WHEN NOT UR.[Row] IS NULL AND dVS.[IsNumeric] = 0 THEN rv.[Value] 
					ELSE NULL
				 END AS 'ValueText'
				,ISNULL(rv.ValueComment, N'') AS 'ValueComment'
				,[IsNumeric]
			INTO #Values
			FROM #RequestedValues rv
				INNER JOIN planning.tdFactories dF
					ON df.FactoryID = rv.FactoryID 
				INNER JOIN planning.tdProductLines dPL
					ON dPL.FactoryKey = dF.FactoryKey
						AND dPL.ProductLineID = rv.ProductLineID 
				INNER JOIN planning.tdProducts dP
					ON dP.FactoryKey = dF.FactoryKey 
						AND dP.ProductLineKey = dPL.ProductLineKey
						AND dP.ProductID = rv.ProductID 
				INNER JOIN planning.tdValueSeries dVS
					ON dVS.FactoryKey = dF.FactoryKey 
						AND dVS.ProductLineKey = dPL.ProductLineKey
						AND dVS.ProductKey = dP.ProductKey
						AND dVS.ValueSeriesID = rv.ValueSeriesID
				INNER JOIN [planning].[tdTime] t
					ON t.FactoryKey = dF.FactoryKey 
						AND t.ProductLineKey = dPL.ProductLineKey
						AND t.ProductKey = dP.ProductKey
						AND t.TimeID = rv.TimeID
				LEFT JOIN planning.trUserRights UR
					ON df.FactoryID = UR.FactoryID 
						AND dPL.ProductlineID = UR.ProductLineID 
						AND UR.Username = @TransactionUsername
						AND UR.[Right] = N'Write';


			-- STEP 1.3 - Create result table

			DROP TABLE IF EXISTS #ResultSet;

			CREATE TABLE #ResultSet (
				ValueNumber			INT				NOT NULL
				,FactoryID			NVARCHAR(255)	NOT NULL
				,ProductlineID		NVARCHAR(255)	NOT NULL
				,ProductID			NVARCHAR(255)	NOT NULL
				,ValueSeriesID		NVARCHAR(255)	NOT NULL
				
				,TimeID				INT				NOT NULL

				,[Value]			NVARCHAR(MAX)	NULL
				,ValueComment		NVARCHAR(MAX)   NULL

				,ReturnCode			INT				NULL
				,ReturnComment		NVARCHAR(4000)	NULL
				);
			
			-- STEP 1.4 - Fill result table with requested values

			INSERT INTO #ResultSet (ValueNumber, FactoryID, ProductlineID, ProductID, ValueSeriesID, 
									TimeID, [Value], ValueComment)
			SELECT ValueNumber
				,FactoryID
				,ProductlineID
				,ProductID
				,ValueSeriesID
				,TimeID
				,[Value]
				,ValueComment
			FROM #RequestedValues;

			-- STEP 2.2 - Set comment and code for "No rights" and "Ignored" values

			UPDATE r
			SET	ReturnCode		=	CASE	WHEN v.FactoryKey IS NULL								THEN 404		
											WHEN v.RightRow IS NULL									THEN 403
											WHEN r.[Value] = '0' AND r.ValueSeriesID NOT LIKE '%_0'	THEN 202 
											ELSE 415 END
				,ReturnComment	=	CASE	WHEN v.FactoryKey IS NULL								THEN N'Ignored' 
											WHEN v.RightRow IS NULL									THEN N'No rights'
											WHEN r.[Value] = '0' AND r.ValueSeriesID NOT LIKE '%_0'	THEN 'Not saved, due to 0 Value' 
											ELSE N'Invalid' END
			FROM #ResultSet r 
				LEFT JOIN #Values v
					ON r.ValueNumber = v.ValueNumber
			WHERE v.FactoryKey IS NULL	
				OR v.RightRow IS NULL;

			-- STEP 2.1 - Delete all existing values
			DELETE FROM fV
			FROM planning.tfValues fV 
				INNER JOIN #Values v
					ON		fV.ValueSeriesKey = v.ValueSeriesKey
						AND fV.TimeID = v.TimeID
						AND NOT v.RightRow IS NULL;

			--SELECT * FROM #Values; --Debug
		
			-- STEP 2.2 - Insert new values, except NULL Values
			INSERT INTO planning.tfValues (ValueSeriesKey, ProductKey, ProductLineKey, FactoryKey,
									ProductID, ProductLineID, FactoryID, ValueSeriesID, TimeID,
									ValueFormula,  ValueInt, ValueText, ValueComment)
				SELECT v.ValueSeriesKey
					,v.ProductKey
					,v.ProductLineKey
					,v.FactoryKey
					,r.ProductID
					,r.ProductLineID
					,r.FactoryID
					,r.ValueSeriesID
					,r.TimeID
					, N''						AS ValueFormula
					,ISNULL(v.ValueInt, 0)		AS ValueInt
					,ISNULL(v.ValueText, N'')	AS ValueText
					,v.ValueComment		
				FROM #ResultSet r 
					INNER JOIN #Values v
						ON		r.ValueNumber = v.ValueNumber
							AND r.ReturnCode IS NULL

			WHERE
						NOT (v.ValueInt IS NULL AND v.ValueText IS NULL)
					AND NOT (v.ValueInt = 0 AND v.[IsNumeric] = 1 AND r.ValueSeriesID NOT LIKE '%_0');


			SET @EffectedRows = @@ROWCOUNT;

			-- STEP 3.1 - Update resultCode for processed values

			UPDATE #ResultSet
			SET ReturnCode = 200
			WHERE ReturnCode IS NULL;

			DECLARE @ProcessTime INT;
			SET @ProcessTime = DATEDIFF(ms, @TimestampCall, CURRENT_TIMESTAMP);
			SET @ResultCode = 200;

			-- STEP 3.2 - Return result

			SELECT  
					 *
					,@ProcessTime AS ProcessTime
			FROM #ResultSet;

			COMMIT TRANSACTION spPOST_Values_JSON;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			
			ROLLBACK TRANSACTION spPOST_Values_JSON;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::planning.spPOST_Values_JSON TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::planning.spPOST_Values_JSON TO pf_PlanningFactoryService;



			
/*
Procedure to create table of debtor & creditor account balances per period
DataFactory 5.0
Jan Link, saxess-software gmbh
08/2020

Testcall
DECLARE @RC int
DECLARE @Username		nvarchar(255) = 'SQL'
DECLARE @FactoryID		nvarchar(255) = ''
DECLARE @ProductLineID	nvarchar(255) = ''

EXECUTE @RC = result.pDebCredAccountBalancesPeriods 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC

*/
DROP PROCEDURE IF EXISTS result.pDebCredAccountBalancesPeriods;
GO

CREATE PROCEDURE result.pDebCredAccountBalancesPeriods
(
    @Username NVARCHAR(255),         -- used for logging
    @ProcessID NVARCHAR(50) = '0',   -- used for logging
    @SubProcessID NVARCHAR(50) = '0' -- used for logging
)
AS
BEGIN
    DECLARE @ResultCode INT = 501;
    DECLARE @CurrentTimestamp DATETIME;
    DECLARE @ErrorMessage NVARCHAR(2000);
    DECLARE @YearPeriod DATE;
    DECLARE @YearPeriodFollowing DATE;
    DECLARE @YearPeriodEoMonth DATE;

    DECLARE @ResultTable TABLE
    (
        CompanyID NVARCHAR(255) NOT NULL,
        CompanyName NVARCHAR(255) NOT NULL,
        AccountID NVARCHAR(255),
        AccountName NVARCHAR(255),
        FinancialYear INT,
        FinancialPeriod INT,
        Saldo DECIMAL(19, 4),
        NochNichtFaellig DECIMAL(19, 4),
        [1Bis30TageUeberfaellig] DECIMAL(19, 4),
        [31Bis60TageUeberfaellig] DECIMAL(19, 4),
        [61Bis90TageUeberfaellig] DECIMAL(19, 4),
        [91Bis365TageUeberfaellig] DECIMAL(19, 4),
        [MehrAls365TageUeberfaellig] DECIMAL(19, 4)
    );
    BEGIN TRY
        -- Create date range from first period to last period in result.tDebCredJournal to avoid gaps in balance calculation
        -- Declare variables for date range creation
        DECLARE @MinYearPeriod DATE =
                (
                    SELECT MIN(DATEFROMPARTS(j.FinancialYear, j.FinancialPeriod, 1))
                    FROM result.tDebCredJournal j
                );
        DECLARE @MaxYearPeriod DATE =
                (
                    SELECT MAX(DATEFROMPARTS(j.FinancialYear, j.FinancialPeriod, 1))
                    FROM result.tDebCredJournal j
                );
        DECLARE @NoOfPeriods INT =
                (
                    SELECT TOP 1 DATEDIFF(m, @MinYearPeriod, @MaxYearPeriod)
                );
        DECLARE @YearPeriodTable TABLE
        (
            YearPeriod DATE,
            YearPeriodFollowing DATE,
            YearPeriodEoMonth DATE
        );

        DECLARE @PeriodIdx INT = 0;
        WHILE @PeriodIdx < @NoOfPeriods
        BEGIN
            INSERT INTO @YearPeriodTable
            SELECT DATEADD(m, @PeriodIdx, @MinYearPeriod) AS YearPeriod,
                   DATEADD(m, 1, DATEADD(d, 1, EOMONTH(DATEADD(m, @PeriodIdx, @MinYearPeriod), -1))) AS YearPeriodFollowing,
                   EOMONTH(DATEADD(m, @PeriodIdx, @MinYearPeriod)) AS YearPeriodEoMonth;
            SET @PeriodIdx = @PeriodIdx + 1;
        END;

        --Cursor contaning all financial periods from result.tDebCredJournal
        DECLARE DateCursor CURSOR READ_ONLY FOR
        SELECT YearPeriod,
               YearPeriodFollowing,
               YearPeriodEoMonth
        FROM @YearPeriodTable
        ORDER BY 1 DESC;
        OPEN DateCursor;
        FETCH DateCursor
        INTO @YearPeriod,
             @YearPeriodFollowing,
             @YearPeriodEoMonth;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            -- Execute query with variables and write result into @ResultTable
            INSERT INTO @ResultTable
            SELECT liqui.CompanyID,
                   liqui.CompanyName,
                   liqui.AccountID,
                   liqui.AccountName,
                   YEAR(@YearPeriod) AS FinancialYear,
                   MONTH(@YearPeriod) AS FinancialPeriod,
                   SUM(CAST(Saldo / 100 AS DECIMAL(19, 4))) AS Saldo,
                   SUM(CAST(NochNichtFaellig / 100 AS DECIMAL(19, 4))) AS NochNichtFaellig,
                   SUM(CAST([1Bis30TageUeberfaellig] / 100 AS DECIMAL(19, 4))) AS [1Bis30TageUeberfaellig],
                   SUM(CAST([31Bis60TageUeberfaellig] / 100 AS DECIMAL(19, 4))) AS [31Bis60TageUeberfaellig],
                   SUM(CAST([61Bis90TageUeberfaellig] / 100 AS DECIMAL(19, 4))) AS [61Bis90TageUeberfaellig],
                   SUM(CAST([91Bis365TageUeberfaellig] / 100 AS DECIMAL(19, 4))) AS [91Bis365TageUeberfaellig],
                   SUM(CAST([MehrAls365TageUeberfaellig] / 100 AS DECIMAL(19, 4))) AS [MehrAls365TageUeberfaellig]
            FROM
            (
                SELECT DATEFROMPARTS(j.FinancialYear, j.FinancialPeriod, 1) AS FinancialYearPeriod,
                       j.DocumentNumber,
                       j.DocumentDate,
                       j.ItemStatusCODE,
                       j.DocumentClosedDate,
                       j.Debit - j.Credit AS Saldo,
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate) >= 0 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS NochNichtFaellig,
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate)
                                BETWEEN -30 AND -1 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS [1Bis30TageUeberfaellig],
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate)
                                BETWEEN -60 AND -31 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS [31Bis60TageUeberfaellig],
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate)
                                BETWEEN -90 AND -61 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS [61Bis90TageUeberfaellig],
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate)
                                BETWEEN -365 AND -91 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS [91Bis365TageUeberfaellig],
                       CASE
                           WHEN DATEDIFF(DAY, @YearPeriodEoMonth, j.DocumentDueDate) < -365 THEN
                               j.Debit - j.Credit
                           ELSE
                               0
                       END AS [MehrAls365TageUeberfaellig],
                       c.CompanyID,
                       c.CompanyName,
                       acc.AccountID,
                       acc.AccountName
                FROM result.tDebCredJournal j
                    JOIN result.tCompanies c
                        ON j.CompanyKey = c.CompanyKey
                    JOIN result.tDebCredAccounts acc
                        ON j.AccountKey = acc.AccountKey
            ) liqui
            WHERE FinancialYearPeriod < @YearPeriodFollowing
                  AND
                  (
                      ItemStatusCODE = 'OPEN'
                      OR DocumentClosedDate >= @YearPeriodFollowing
                  )
            GROUP BY liqui.CompanyID,
                     liqui.CompanyName,
                     liqui.AccountID,
                     liqui.AccountName;

            FETCH DateCursor
            INTO @YearPeriod,
                 @YearPeriodFollowing,
                 @YearPeriodEoMonth;
        END;

        CLOSE DateCursor;
        DEALLOCATE DateCursor;

        -- Drop result table if it exists
        DROP TABLE IF EXISTS result.tDebCredAccountBalancesPeriods;

        -- Select ... into result table
        SELECT CompanyID,
               CompanyName,
               AccountID,
               AccountName,
               FinancialYear,
               FinancialPeriod,
               Saldo,
               NochNichtFaellig,
               [1Bis30TageUeberfaellig],
               [31Bis60TageUeberfaellig],
               [61Bis90TageUeberfaellig],
               [91Bis365TageUeberfaellig],
               [MehrAls365TageUeberfaellig]
        INTO result.tDebCredAccountBalancesPeriods
        FROM @ResultTable
        ORDER BY AccountID,
                 FinancialYear,
                 FinancialPeriod;

        SET @ResultCode = 200;
    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @ErrorMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = CURRENT_TIMESTAMP;
        EXEC system.pPOST_ProcessLogEntry @Username = @Username,
                                          @ModuleCode = 'Integration',
                                          @ProcessName = 'Agent Execution',
                                          @ProcessID = @ProcessID,
                                          @SubProcessName = 'agent.pSync',
                                          @SubProcessID = @SubProcessID,
                                          @SubSubProcessName = 'agent.pCreateJSONViewINT',
                                          @EventCODE = 'Error',
                                          @Timestamp = @CurrentTimestamp,
                                          @ReturnCODE = @ResultCode,
                                          @EventMessage = @ErrorMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryService;
GO


/*
Procedure to deliver analytic data to the enduser
Events must have ValueSeriesID "E" and Bemerkungen ValueSeriesID "BEM"
DataFactory 5.0
Gerd Tautenhahn, saxess-software gmbh
08/2020

Testcall
DECLARE @RC int
DECLARE @Username		nvarchar(255) = 'SQL'
DECLARE @FactoryID		nvarchar(255) = ''
DECLARE @ProductLineID	nvarchar(255) = ''

EXECUTE @RC = result.spEvents 
   @Username
  ,@FactoryID
  ,@ProductLineID

Print @RC

*/


DROP PROCEDURE IF EXISTS result.spEvents;
GO

CREATE PROCEDURE result.spEvents (@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
	BEGIN

		DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
		DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		+ ISNULL(@Username, N'NULL') 
													+ N''','''	+ ISNULL(@FactoryID, N'NULL') 
													+ N''','''	+ ISNULL(@ProductLineID, N'NULL') 
													+ N'''';
		DECLARE @EffectedRows		INT				= 0;			
		DECLARE @ResultCode			INT				= 501;			
		DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
		DECLARE @Comment			NVARCHAR(2000)	= N'';			

		-- STEP 0.1 - NULL Protection
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

		BEGIN TRY
			BEGIN TRANSACTION spEvents;

			SET @Username		= dbo.sx_pf_pProtectString (@Username);
			SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
			SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403'
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- # Scope table for all levels access right
			DROP TABLE IF EXISTS #Scope;
			CREATE TABLE #Scope 
				(
					 RowKey			INT IDENTITY (1,1)	
					,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductlineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				)

			INSERT INTO #Scope
				SELECT 
					 FactoryID
					,ProductlineID
				FROM planning.trUserRights rUR
				WHERE 
							Username = @TransactUsername 
						AND [Right] IN ('Write','Read') 
						AND (FactoryID		= @FactoryID		OR @FactoryID = '')
						AND (ProductlineID	= @ProductLineID	OR @ProductLineID = '')

			-- Main Query
			SELECT
					fV.FactoryID
					,fV.ProductLineID
					,fV.ProductID

					,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
					,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
					,TRY_CAST(TRY_CAST(fV.TimeID AS NVARCHAR(255)) AS Date)			AS [Date]

					,Ev.ValueText								AS [Event]
					,Ev.ValueComment							AS EventComment
					,fV.ValueText								AS Bemerkung
					,fV.ValueComment							AS BemerkungComment

					,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
					,dF.NameShort								AS FactoryName
					,dF.ResponsiblePerson						AS FResponsiblePerson

					,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
					,dPL.NameShort								AS ProductLineName
					,dPL.ResponsiblePerson						AS PLResponsiblePerson

					,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
					,dP.NameShort								AS ProductName
					,dP.ResponsiblePerson						AS PResponsiblePerson

					,dP.[Status]
					,dP.GlobalAttribute1
					,dP.GlobalAttribute2
					,dP.GlobalAttribute3
					,dP.GlobalAttribute4
					,dP.GlobalAttribute5
					,dP.GlobalAttribute6
					,dP.GlobalAttribute7
					,dP.GlobalAttribute8
					,dP.GlobalAttribute9
					,dP.GlobalAttribute10
					,dP.GlobalAttribute11
					,dP.GlobalAttribute12
					,dP.GlobalAttribute13
					,dP.GlobalAttribute14
					,dP.GlobalAttribute15
					,dP.GlobalAttribute16
					,dP.GlobalAttribute17
					,dP.GlobalAttribute18
					,dP.GlobalAttribute19
					,dP.GlobalAttribute20
					,dP.GlobalAttribute21
					,dP.GlobalAttribute22
					,dP.GlobalAttribute23
					,dP.GlobalAttribute24
					,dP.GlobalAttribute25
				FROM planning.tfValues fV
						-- Gruppierung nach Event
						LEFT JOIN	(
									SELECT 
											f.ProductKey
										,f.TimeID
										,f.ValueText 
										,f.ValueComment
									FROM planning.tfValues f
									INNER JOIN #Scope s
									ON		f.FactoryID		= s.FactoryID
										AND f.ProductlineID	= s.ProductlineID

									WHERE f.ValueSeriesID = 'E'
							
								) Ev ON

										fV.ProductKey	= Ev.ProductKey
									AND fV.TimeID		= Ev.TimeID

						LEFT JOIN planning.tdProducts dP
							ON fV.ProductKey = dP.ProductKey
						LEFT JOIN planning.tdProductLines dPL
							ON fV.ProductLineKey = dPL.ProductLineKey
						LEFT JOIN planning.tdFactories dF
							ON fV.FactoryKey = dF.FactoryKey
						-- Rechtefilter
						INNER JOIN #Scope Sc
							ON		fV.FactoryID	 = Sc.FactoryID
								AND fV.ProductlineID = Sc.ProductlineID
				
			WHERE fV.ValueSeriesID = 'BEM'
			SET @Resultcode = 200 

			COMMIT TRANSACTION spEvents;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			ROLLBACK TRANSACTION sx_pf_GET_Factories;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO
   
GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryService;
GO

/*
Toni Augut for saxess-software gmbh
01/2020

Default Balance Sheet

Testcall
EXEC result.spPivot_FiBuSaldenliste 'SQL'

*/


DROP PROCEDURE IF EXISTS result.spPivot_FiBuSaldenliste;
GO


CREATE PROCEDURE result.spPivot_FiBuSaldenliste @Username NVARCHAR(255), @FactoryID NVARCHAR (255) = '', @ProductlineID NVARCHAR (255) = ''
AS
	SET NOCOUNT ON

	-- API log
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT 			= 0;						
	DECLARE @ResultCode				INT 			= 501;						
	DECLARE @TimestampCall			DATETIME 		= CURRENT_TIMESTAMP;
	DECLARE @Comment				NVARCHAR(2000) 	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username		, N'NULL')	+ N''','''	
													+ ISNULL(@FactoryID		, N'NULL')	+ N''','''
													+ ISNULL(@ProductLineID	, N'NULL')	+ N''','''
													+ N'''';
													
BEGIN TRY
	BEGIN TRANSACTION spPivot_FiBuSaldenliste;
	
	-- Determine transaction user
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
	IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
-------------------------------------------------------------------------------------------------------------------
-- ##### SELECT ###########


	SELECT *
	FROM result.tFinFiBuSaldenliste
	WHERE Kontentyp IN ('GuV','Bilanz');

-------------------------------------------------------------------------------------------------------------------
-- ##### COMMIT & API LOG ###########
	SET @ResultCode = 200

	COMMIT TRANSACTION spPivot_FiBuSaldenliste;
END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spPivot_FiBuSaldenliste;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;
		END;
END CATCH

EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

RETURN @ResultCode;

GO

GRANT EXECUTE ON OBJECT ::result.spPivot_FiBuSaldenliste  TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spPivot_FiBuSaldenliste  TO pf_PlanningFactoryService;
GO
/*
    Product: OCT 5.2
    Object: system.pGET_LicenceInformation
    Type: Stored Procedure
    Description: Stored Procedure to retrieve the licence information from the database.
				 Return Value according to HTTP Standard
    Author: saxess-software GmbH
    Script Date: 04.06.2020 11:24

	Example Call:

	DECLARE	@return_value int

	EXEC	@return_value = system.pGET_LicenceInformation
			@Username	= N'SQL',
			@CustomerID = N'AD032456',
			@ServerID	= N'Server1'

	SELECT	'Return Value' = @return_value

	GO

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'system.pGET_LicenceInformation') AND type in (N'P', N'PC'))
	DROP PROCEDURE system.pGET_LicenceInformation;
GO

CREATE PROCEDURE system.pGET_LicenceInformation
	 @Username				NVARCHAR(255)
	,@CustomerID			NVARCHAR(255)
	,@ServerID				NVARCHAR(255)
	,@ServiceName			NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''','''
			+ ISNULL(@CustomerID			, N'NULL')			 + N''','''
			+ ISNULL(@ServerID		, N'NULL')			 + N'''';
			
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';
	DECLARE @WindowsServerName	 NVARCHAR(255) = @@SERVERNAME;


	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @CustomerID			IS NULL 
		OR @ServerID			IS NULL
		OR @ServiceName			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
        IF EXISTS
        (
            SELECT r.name from sys.database_role_members rm
            INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
            INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
            WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
        )
        BEGIN
            SET @TransactUsername = 'FactoryService';
        END;

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create licence information

		-- Declare table variable
		
		DECLARE @LicenceInformationTbl TABLE
		(
		  CustomerID		NVARCHAR(10)	NOT NULL
		 ,WindowsServerName NVARCHAR(255)	NOT NULL
		 ,ServiceName		NVARCHAR(255)	NOT	NULL
		 ,DatabaseServerID	NVARCHAR(255)	NOT NULL
		 ,DatabaseID		NVARCHAR(255)	NOT NULL
		 ,Timestamp			DATETIME		NOT NULL
		 ,PropertyName		NVARCHAR(255)	NOT NULL
		 ,PropertyValue		NVARCHAR(255)	NOT NULL
		)

		-- Insert database version property

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'DatabaseVersion', ValueText
		FROM planning.tgCluster
		WHERE PropertyID = 'C04'

		-- Insert number of data sources

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfDataSources', COUNT(*)
		FROM global.tDataSources

		-- Insert different data source types

		DECLARE @result NVARCHAR(4000) = '';

		SELECT @Result=@Result
		+ ','+ ETLPackage
		FROM global.tDataSources
		GROUP BY ETLPackage;

		SET @Result = STUFF(@Result, 1,1,'');
		IF @Result IS NULL SET @Result = 0;

		INSERT INTO @LicenceInformationTbl VALUES
		(@CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'DifferentDataSourceTypes', @Result);

		-- Insert number of active companies

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfActiveCompanies', COUNT(*)
		FROM global.tCompanies
		WHERE ActiveFlag = 1;

		-- Insert number of active users

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfActiveUsers', COUNT(*)
		FROM planning.trUser
		WHERE Status = 'Active';

		-- Insert number of products

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfProducts', COUNT(*)
		FROM planning.tdProducts;

		-- Insert number of productlines

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfProductlines', COUNT(*)
		FROM planning.tdProductLines;

		-- Insert number of Factories

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfFactories', COUNT(*)
		FROM planning.tdFactories;

		-- Insert number of rows in result.tFinAccountingJournal

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETDATE(), 'NumberOfPostings', COUNT(*)
		FROM result.tFinAccountingJournal;

		SELECT * 
		FROM @LicenceInformationTbl;

		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, 0, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryService;
GO


/*
    Product:		OCT 5.0
    Object:			system.pGET_Settings
    Author:			saxess-software GmbH

	ToDo
	- RightCheck
	- Documentation

	Testcall

	EXEC system.pGET_Setting 'SQL','DBVersion'
*/

DROP PROCEDURE IF EXISTS system.pGET_Setting;
GO

CREATE PROCEDURE system.pGET_Setting
	 @Username	NVARCHAR(255)
	,@SettingID NVARCHAR(50)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,	 N'NULL')+ N''',''' 
			+ ISNULL(@SettingID, N'NULL')+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	DECLARE @IsFactoryService INT = 0

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @SettingID	IS NULL SET @SettingID	= N'';

	BEGIN TRY
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @SettingID	= dbo.sx_pf_pProtectID		(@SettingID);
			
		IF @SettingID	= N'' 
		BEGIN
			SET @ResultCode = 404;
			THROW 60000, N'Empty input parameters', 10;
		END;

		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;
	
		-- STEP 1.1 - Determine transaction user
		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'FactoryService';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;

		-- STEP 3 - Output result
		SELECT
			 SettingID
			,SettingDescription
			,DataTypeCODE
			,ValueText
			,ValueDateTime
			,ValueInt
			,ValueIntScale
			,ValueJSON
			,Category
			,ReadOnlySystemPropertyBool
		FROM [system].tSettings
		WHERE SettingID = @SettingID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON [system].[pGET_Setting] TO pf_PlanningFactoryService;
GRANT EXECUTE ON [system].[pGET_Setting] TO pf_PlanningFactoryUser;

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Procedure',
    @level1name = N'pGET_Setting'
GO

/*
    Product:		OCT 5.0
    Object:			system.pGET_Settings
    Author:			saxess-software GmbH
    Script Date:	07.04.2020

	Testcall
	EXEC system.pGET_Settings 'SQL'

	ToDo
	- RightCheck
	- Documentation
*/

DROP PROCEDURE IF EXISTS system.pGET_Settings;
GO

CREATE PROCEDURE system.pGET_Settings
	 @Username		NVARCHAR(255)
	,@Category		NVARCHAR(255) = NULL
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,	N'NULL')+ N''',''' 
			+ ISNULL(@Category, N'NULL')+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';	
	
	DECLARE @IsFactoryService INT = 0;

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username	= N'';

	BEGIN TRY
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
	
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;
	
		-- STEP 1.1 - Determine transaction user
		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'FactoryService';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;

		-- STEP 3 - Output result
		SELECT
			 SettingID
			,SettingDescription
			,DataTypeCODE
			,ValueText
			,ValueDateTime
			,ValueInt
			,ValueIntScale
			,ValueJSON
			,Category
			,ReadOnlySystemPropertyBool
		FROM [system].[tSettings]
		WHERE Category = @Category 
			OR @Category IS NULL;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

GRANT EXECUTE ON [system].[pGET_Settings] TO pf_PlanningFactoryService;
GRANT EXECUTE ON [system].[pGET_Settings] TO pf_PlanningFactoryUser;

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Procedure',
    @level1name = N'pGET_Settings'
GO

/*
    Product: OCT 5.2
    Object: system.pPOST_ProcessLogEntry
    Type: Stored Procedure
    Description: Stored Procedure to save a process log entry. For every call a new entry is created.
				 Return Value according to HTTP Standard
    Author: saxess-software GmbH
    Script Date: 17.02.2020 15:18

	Example Call:

	DECLARE	@return_value int

	EXEC  @return_value = [system].[pPOST_ProcessLogEntry]
		@Username = N'SQL',
		@ModuleCode = N'Integration',
		@ProcessName = N'Test',
		@ProcessID = 1,
		@SubProcessName = N'SubProcessTest',
		@SubProcessID = 2,
		@SubSubprocessName = N'SubSubProcessTest',
		@EventCODE = N'End',
		@Timestamp = N'18-02-2020 15:32:21.000',
		@ReturnCODE = 500,
		@EventMessage = N'This is an error.'

	SELECT	'Return Value' = @return_value

	GO

*/

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'system.pPOST_ProcessLogEntry') AND type in (N'P', N'PC'))
	DROP PROCEDURE system.pPOST_ProcessLogEntry;
GO

CREATE PROCEDURE system.pPOST_ProcessLogEntry
	 @Username				NVARCHAR(255)
	,@ModuleCode			NVARCHAR(50)
	,@ProcessName			NVARCHAR(250)
	,@ProcessID				NVARCHAR(50)
	,@SubProcessName		NVARCHAR(250)
	,@SubProcessID			NVARCHAR(50)
	,@SubSubProcessName		NVARCHAR(250)
	,@EventCODE				NCHAR(10)	
	,@Timestamp				datetime		
	,@ReturnCODE			int
	,@EventMessage			NVARCHAR(max)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''','''
			+ ISNULL(@ModuleCode		, N'NULL')			 + N''','''
			+ ISNULL(@ProcessName		, N'NULL')			 + N''','''
			+ ISNULL(@ProcessID			, N'NULL')			 + N''','''
			+ ISNULL(@SubProcessName	, N'NULL')			 + N''','''
			+ ISNULL(@SubProcessID		, N'NULL')			 + N''','''
			+ ISNULL(@SubSubProcessName	, N'NULL')			 + N''','''
			+ ISNULL(@EventCODE			, N'NULL')			 + N''',''' 
			+ ISNULL(CAST(@Timestamp as nvarchar)			, N'NULL')			 + N''',''' 
			+ ISNULL(CAST(@ReturnCODE as nvarchar)		, N'NULL')			 + N''','''
			+ ISNULL(@EventMessage		, N'NULL')			 + N'''';

	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			 NVARCHAR(2000) = N'';			


	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @ModuleCode			IS NULL 
		OR @ProcessName			IS NULL
		OR @EventCODE			IS NULL
		OR @Timestamp			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create log entry	

		INSERT INTO system.tProcessLogs(
			ModuleCODE
           ,ProcessName
           ,ProcessID
           ,SubProcessName
           ,SubProcessID
		   ,SubSubProcessName
           ,EventCODE
           ,Timestamp
           ,ReturnCODE
           ,EventMessage) 
		VALUES (
			@ModuleCODE
           ,@ProcessName
           ,@ProcessID
           ,@SubProcessName
           ,@SubProcessID
		   ,@SubSubProcessName
           ,@EventCODE
           ,@Timestamp
           ,@ReturnCODE
           ,@EventMessage);	

		SET @EffectedRows = 1;
		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pPOST_ProcessLogEntry TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.pPOST_ProcessLogEntry TO pf_PlanningFactoryService;
GO


/*
    Product:		OCT 5.0
    Object:			system.pPOST_Setting
    Type:			Stored Procedure
    Author:			saxess-software GmbH
    Script Date:	07.04.2020 

	Testcall
	DECLARE @ReturnCode INT;
	EXEC @ReturnCode = system.pPOST_Setting 
		 @Username = 'SQL'
		,@SettingID = '1'
		,@DataTypeCode = 'TEXT';
		
	PRINT @ReturnCode;

	EXEC system.pGET_Setting 'SQL','1'

	ToDo:
	- Right Check
	- Documentation
*/

DROP PROCEDURE IF EXISTS [system].pPOST_Setting;
GO

CREATE PROCEDURE [system].pPOST_Setting
	 @Username						NVARCHAR(255)
	,@SettingID						NVARCHAR(50)
	,@DataTypeCODE					NVARCHAR(255)
	,@SettingDescription			NVARCHAR(4000)	= NULL
	,@ValueText						NVARCHAR(MAX)	= NULL
	,@ValueDateTime					DATETIME		= NULL
	,@ValueInt						INT				= NULL
	,@ValueIntScale					INT				= NULL
    ,@ValueJSON						NVARCHAR(MAX)	= NULL
	,@Category						NVARCHAR(255)	= NULL
	,@ReadOnlySystemPropertyBool	INT				= NULL
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@SettingID,			 N'NULL') + N''',''' 
			+ ISNULL(@DataTypeCODE,			 N'NULL') + N''',''' 
			+ ISNULL(@SettingDescription,	 N'NULL') + N''',''' 
			+ ISNULL(@ValueText,			 N'NULL') + N''',' 
			+ ISNULL(CAST(@ValueDateTime AS NVARCHAR(MAX)),	 N'NULL')	+ N',' 
			+ ISNULL(CAST(@ValueInt		 AS NVARCHAR(MAX)),	 N'NULL')	+ N',' 
			+ ISNULL(CAST(@ValueIntScale AS NVARCHAR(MAX)),	 N'NULL')	+ N',''' 
			+ ISNULL(@ValueJSON,			 N'NULL') + N''',''' 
			+ ISNULL(@Category,				 N'NULL') + N''',' 
			+ ISNULL(CAST(@ReadOnlySystemPropertyBool AS NVARCHAR(MAX)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	DECLARE @IsFactoryService INT = 0;

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DataTypeCODE	IS NULL SET @DataTypeCODE	= N'';
	IF @SettingID		IS NULL SET @SettingID		= N'';



	BEGIN TRY
		BEGIN TRANSACTION pPOST_Setting;	

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @SettingID = dbo.sx_pf_pProtectString (@SettingID);
	
		-- Check for mandatory parameter
		IF	@SettingID		= N''
			OR @DataTypeCODE NOT IN ('TEXT','NUMBER','BOOLEAN','JSON','DATETIME')
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Invalid input parameters', 10;
		END

		-- STEP 1.1 - Determine transaction user
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;

		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'FactoryService';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;
		
		IF EXISTS(SELECT TOP(1) 1 FROM [system].tSettings WHERE SettingID = @SettingID)
		-- Existing Property will be update, not passed Values are keept
		BEGIN
			UPDATE [system].tSettings
			SET SettingDescription			= ISNULL(@SettingDescription, SettingDescription)
				,DataTypeCODE				= @DataTypeCODE
				,ValueText					= ISNULL(@ValueText,		ValueText)
				,ValueDateTime				= ISNULL(@ValueDateTime,	ValueDateTime)
				,ValueInt					= ISNULL(@ValueInt,			ValueInt)
				,ValueIntScale				= ISNULL(@ValueIntScale,	ValueIntScale)
				,ValueJSON					= ISNULL(@ValueJSON,		ValueJSON)
				,Category					= ISNULL(@Category,			Category)
				,ReadOnlySystemPropertyBool	= ISNULL(@ReadOnlySystemPropertyBool, ReadOnlySystemPropertyBool)
			 WHERE SettingID = @SettingID;

			 SET @EffectedRows = @@ROWCOUNT;
			 SET @Resultcode = 200;
		END
		ELSE
		-- Not existing Property will be created, not passed Values with defaults
		BEGIN
			INSERT INTO [system].[tSettings]
			(
				SettingID
				,SettingDescription
				,DataTypeCODE
				,ValueText
				,ValueDateTime
				,ValueInt
				,ValueIntScale
				,ValueJSON
				,Category
				,ReadOnlySystemPropertyBool
			)
			VALUES 
			(
				@SettingID
				, ISNULL(@SettingDescription,	N'')
				, @DataTypeCODE
				, ISNULL(@ValueText,			N'')
				, ISNULL(@ValueDateTime,		'19000101')
				, ISNULL(@ValueInt,				0)
				, ISNULL(@ValueIntScale,		1)
				, ISNULL(@ValueJSON,			N'')
				, ISNULL(@Category,				N'')
				, ISNULL(@ReadOnlySystemPropertyBool, 1)
			);

			SET @EffectedRows = @@ROWCOUNT;
			SET @Resultcode = 201;
		END
		
		COMMIT TRANSACTION pPOST_Setting;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION pPOST_Setting;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

GRANT EXECUTE ON [system].[pPOST_Setting] TO pf_PlanningFactoryService;
GRANT EXECUTE ON [system].[pPOST_Setting] TO pf_PlanningFactoryUser;

-- Documentation
EXEC sp_addextendedproperty
    @name = N'MS_Description',
    @value = N'Procedure to POST one Setting, ',
    @level0type = N'Schema',
    @level0name = N'system',
    @level1type = N'Procedure',
    @level1name = N'pPOST_Setting'
GO

/*
Procedure to delete icons
	
Return code 200 if file exists and was deleted succesful

Testcall
EXEC system.spDELETE_Icon 
	'SQL'
	,'hase.svg';
*/

DROP PROCEDURE IF EXISTS system.spDELETE_Icon;
GO

CREATE PROCEDURE system.spDELETE_Icon
	 @UserName		NVARCHAR(255)	= N''
	,@IconName		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username, N'NULL') + N''', ''' + ISNULL(@IconName, N'NULL') + N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';
		IF @IconName IS NULL SET @IconName = N'';

		BEGIN TRY
			BEGIN TRANSACTION DELETE_Icon;
			
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			IF NOT EXISTS (
				SELECT TOP (1) 1 
				FROM system.tIcons
				WHERE IconName	= @IconName
					)
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			DELETE FROM system.tIcons
			WHERE IconName = @IconName
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION DELETE_Icon;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION DELETE_Icon;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::system.spDELETE_Icon TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spDELETE_Icon TO pf_PlanningFactoryService;

/*
Procedure to get a single icon or all

-- Testcall for all icons
EXEC system.spGET_Icons	'';

--Testcall for one icon 
EXEC system.spGET_Icons	'hase.svg';

*/

DROP PROCEDURE IF EXISTS system.spGET_Icons;
GO

CREATE PROCEDURE system.spGET_Icons
	 @IconName		NVARCHAR(255)	= N''

AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @UserName				NVARCHAR(255)	= N'anonym'
	
	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@UserName, N'NULL') + N''', '+ ISNULL(@IconName, N'NULL') + N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';
		IF @IconName IS NULL SET @IconName = N'';

		BEGIN TRY
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = 'anonym'

			IF @IconName <> ''
				BEGIN
					IF NOT EXISTS (
						SELECT TOP (1) 1 
						FROM system.tIcons
						WHERE IconName	= @IconName
						)
					BEGIN
						SET @ResultCode = 404;
						THROW 60000, N'File not found', 100;
					END;
				END;

			SELECT
				 IconName
				,IconName + ISNULL(N'.' + FileExtension, N'') AS IconNameWithExtension
				,FileExtension
				,FileBody
			FROM system.tIcons
			WHERE 
					IconName= @IconName 
				OR	@IconName = ''
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::system.spGET_Icons TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spGET_Icons TO pf_PlanningFactoryService;

/*

Testcall

EXEC system.spPOST_Icon
	'SQL'
	,'Hase'
	,'svg'
	,0x3;

SELECT * FROM system.tIcons
*/

DROP PROCEDURE IF EXISTS system.spPOST_Icon;
GO


CREATE PROCEDURE system.spPOST_Icon
	 @UserName		NVARCHAR(255)	= N''
	,@IconName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', ''' + ISNULL(@IconName, N'NULL') + N''', '''
													+ ISNULL(@FileExtension, N'NULL') + N''', ' + ISNULL(CAST(DATALENGTH(@FileBody) AS NVARCHAR(MAX)) + N' bytes', N'NULL') 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= CURRENT_TIMESTAMP
			,@Comment				NVARCHAR(2000)	= N'';						
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_Icon;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

			-- Delete File
			DELETE FROM system.tIcons WHERE IconName = @IconName
				
				IF LEN(@FileExtension) = 0 
				BEGIN
					SET @FileExtension = NULL;
				END;

				-- STEP 2.2 - Creating new file

				INSERT INTO system.tIcons (IconName,FileExtension, FileBody)
				VALUES (@IconName,@FileExtension, @FileBody);

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_Icon;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_Icon;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryService;
GO

/*
Trigger um Userrechte zu materialisieren
Werden ausgelöst, sobald in planning.trRights oder planning.trUser Update, Insert oder Delete Operationen erfolgen
Gerd Tautenhahn für saxess-software gmbh
PlanningFactory Version 3.0.7
06/2015
*/

-- für planning.trRights Aktionen

DROP TRIGGER IF EXISTS planning.mqt_vUserRights_from_rRights;
GO

CREATE TRIGGER planning.mqt_vUserRights_from_rRights

ON planning.trRights 

AFTER INSERT, UPDATE, DELETE 

AS 

BEGIN
	EXEC dbo.sx_pf_materialize_vUserRights
END
GO



-- für planning.trUser Aktionen
DROP TRIGGER IF EXISTS planning.mqt_vUserRights_from_rUser;
GO

CREATE TRIGGER planning.mqt_vUserRights_from_rUser

ON planning.trUser

AFTER INSERT, UPDATE, DELETE 

AS 
BEGIN
	EXEC dbo.sx_pf_materialize_vUserRights
END

GO

-- Script for creating business DataFactory Administrator
-- Gerd Tautenhahn, Saxess Software GmbH 2019/07
-- Default Administrator will be the user who initializes the database

DECLARE @Username		NVARCHAR (255) = ''
DECLARE @UserNachname	NVARCHAR (255) = ''
DECLARE @UserVorname	NVARCHAR (255) = ''
DECLARE @UserMail		NVARCHAR (255) = ''

SET @Username = ORIGINAL_LOGIN()	

-- delete if already exists
DELETE FROM planning.trUser		WHERE UserName = @Username
DELETE FROM planning.trRights	WHERE UserName = @Username

DECLARE @UserKey Integer

INSERT INTO planning.trUser
           (UserName
           ,PersonSurname
           ,PersonFirstName
           ,Email
           ,LDAPIP
           ,[Status])
     VALUES
           (@Username 
           ,@UserNachname
           ,@UserVorname 
           ,@UserMail
           ,''
           ,'Active')

SELECT @UserKey = scope_identity()

INSERT INTO planning.trRights
           (UserKey
           ,UserName
           ,FactoryID
           ,ProductLineID
           ,ProductID
           ,[Right]
		   ,ReadCommentMandatory
		   ,WriteCommentMandatory)
     VALUES
           (@UserKey
           ,@Username
           ,''
           ,''
           ,''
           ,'Write'
		   ,0
		   ,0)
GO

-- Fill planning.tgCluster with initial data ##########################################################################################################################################

-- REFRESH RuntimeValues on every API Update
DELETE FROM planning.tgCluster WHERE PropertyID IN ('DB01','DB02','DB03','DB04')

INSERT INTO planning.tgCluster
           (PropertyID
           ,PropertyName
           ,CommentUser
           ,CommentDev
           ,Unit
           ,ValueText
           ,ValueInt
           ,Scale
		   ,IsROSystemproperty
		   ,FormatID)
     VALUES
		   -- Database Properties ---------------------------------------------------------
		   (
		    'DB01'
           ,'DatabaseServer'
           ,'Name of the DatabaseServer, hosting this Cluster.'
           ,'SystemValue of last Refresh, not necessary Runtime Value.'
           ,'Text'
           ,@@SERVERNAME
           ,0
           ,0
		   ,1
		   ,'')
		   ,
		   (
		    'DB03'
           ,'DatabaseVersion'
           ,'DatabaseVersion, hosting this Cluster.'
           ,'SystemValue of last Refresh, not necessary Runtime Value.'
           ,'Text'
           ,@@VERSION
           ,0
           ,0
		   ,1
		   ,'')
		   ,
		   (
		    'DB04'
           ,'DatabaseName'
           ,'DatabaseName, hosting this Cluster.'
           ,'SystemValue of last Refresh, not necessary Runtime Value.'
           ,'Text'
           ,DB_NAME()
           ,0
           ,0
		   ,1
		   ,'')

-- Special Property with error handling for Azure Databases ----------------------------------------------------------------------------
DECLARE @T TABLE(ServiceName NVARCHAR(255)); 
DECLARE @Rows INT
DECLARE @ServiceName NVARCHAR(255)

IF SERVERPROPERTY('Edition') != 'SQL Azure'
	BEGIN TRY
		INSERT @T(ServiceName) EXEC ('SELECT @@SERVICENAME')
	END TRY	
	BEGIN CATCH
	END CATCH

SELECT @Rows = COUNT(*) FROM @T

IF @Rows = 0
	INSERT @T(ServiceName) Values ('SQL Azure')
	
SELECT @ServiceName = ServiceName FROM @T 


	INSERT INTO planning.tgCluster
				(PropertyID
				,PropertyName
				,CommentUser
				,CommentDev
				,Unit
				,ValueText
				,ValueInt
				,Scale
				,IsROSystemproperty
				,FormatID)
			VALUES
				(
				 'DB02'
				,'DatabaseInstance'
				,'Name of the DatabaseInstance, hosting this Cluster.'
				,'SystemValue of last Refresh, not necessary Runtime Value.'
				,'Text'
				,@SERVICENAME
				,0
				,0
				,1
				,'none')


-- Keep UserValues *****************************************************
DECLARE @ExistFlag INT = 0

-- AutoUpdate
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('DB05') AND PropertyName = 'AutoUpdate T_SQL_API'
IF @ExistFlag = 0 
		BEGIN
			DELETE FROM planning.tgCluster WHERE PropertyID = 'DB05'
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('DB05','AutoUpdate T_SQL_API','Flag to update the T_SQL_API always to the latest version','','Boolean','',1,1,0,'')
		END

-- Clustername
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C01')
IF @ExistFlag = 0 
	
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C01','ClusterName','Name of this Cluster','Set on GUI','Text','<give me a name>',0,0,0,'')
		END

-- BaseCurrency
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C02')
IF @ExistFlag = 0 
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C02','ClusterBaseCurrency','BaseCurrency of this Cluster','will be set by UNIKUM','Text','',0,0,0,'')
		END


-- BaseLanguage
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C03')
IF @ExistFlag = 0 
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C03','ClusterBaseLanguage','BaseLanguage of this Cluster','will be set by UNIKUM','Text','',0,0,0,'')
		END

-- API Version
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C04')
IF @ExistFlag = 0 
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C04','T_SQL_API_Version','version of T_SQL_API','will be set by API Script','Text','',0,0,0,'')
		END


-- IsInMainantenceMode
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C05')
IF @ExistFlag = 0 
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C05','IsInMaintanceMode','Cluster is in Maintenance Mode Flag to keep users out','will be set by Update / Maintanence Script ','Text','FALSE',0,0,0,'')
		END

-- PreferencesAreUserDependendFlag
SELECT @ExistFlag = Count(*) FROM planning.tgCluster WHERE PropertyID IN ('C06')
IF @ExistFlag = 0 
		BEGIN
			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemproperty,FormatID)
			VALUES ('C06','PreferencesAreUserDependendFlag','Preferences are saved per User','will be set by ClusterAdmin ','Flag','',1,1,0,'')
		END

GO

-- UNIKUM DE Version 12/2016

UPDATE planning.tgCluster SET ValueText = 'DE' WHERE PropertyID = 'C03'

-- Existenz der Factory ZT prüfen und erzeugen falls nicht da, sonst Key ermitteln
DECLARE  @Counter	INT
		,@FactoryKey INT 

SELECT @Counter = Count(FactoryKey) FROM planning.tdFactories WHERE FactoryID = 'ZT'	

IF @Counter = 0 
	BEGIN
		INSERT INTO planning.tdFactories OUTPUT Inserted.FactoryKey 
			VALUES ('ZT', 'Templates','','','','','')
		SELECT @FactoryKey = SCOPE_IDENTITY()
	END
ELSE
	BEGIN
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = 'ZT'	
	END

-- Existenz der Productline U prüfen und erzeugen falls nicht da
SELECT @Counter = Count(ProductLineKey) FROM planning.tdProductLines WHERE FactoryID = 'ZT'	AND ProductLineID = 'U'

IF @Counter = 0 
	BEGIN
		INSERT INTO planning.tdProductLines 
			VALUES (@FactoryKey,'U','ZT','Unikum','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					)
	END


-- Existenz der Productline PARAM prüfen und erzeugen falls nicht da
SELECT @Counter = Count(ProductLineKey) FROM planning.tdProductLines WHERE FactoryID = 'ZT'	AND ProductLineID = 'PARAM'

IF @Counter = 0 
	BEGIN
		INSERT INTO planning.tdProductLines 
			VALUES (@FactoryKey,'PARAM','ZT','Parameter','','Diese Produkte werden als Tabelle in der Datenbank materialisiert, diese Tabellen sind in der Datebank sichtbar für ALLE User welche Datenbankzugriff haben !','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					,'','','','','','','','','',''
					)
	END

EXEC dbo.sx_pf_materialize_vUserRights

GO

/*
Merge new format definitions with old format definitions
 - Old used definitions are keept
 - Old unused definitions are deleted
 - Existing definitions are updated
 - New definitions are created

10/2018 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

-- Delete unused formats
DELETE FROM planning.thFormats WHERE FormatID NOT IN 
											(
												SELECT DISTINCT ValueFormatID FROM planning.tdValueSeries
												UNION
												SELECT DISTINCT FormatID FROM planning.thListValues
											);

MERGE INTO planning.thFormats AS Target  
	USING
		-- List of all formats we want to have
		(VALUES 
			('EingabeDezimal2'		,'255,255,204'	,'0,0,0'		,'#,###0.00'    ,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('EingabeGanzzahl'		,'255,255,204'	,'0,0,0'		,'#,###0'		,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('EingabeJahreszahl'	,'255,255,204'	,'0,0,0'		,'####'			,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('EingabeText'			,'255,255,222'	,'0,0,0'		,'@'			,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('EingabeDatum'			,'255,255,204'	,'0,0,0'		,'dd/mm/yyyy'	,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('GrauDezimal2'			,'200,200,200'	,'50,50,50'		,'#,###0.00'	,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('GrauGanzzahl'			,'200,200,200'	,'50,50,50'		,'#,###0'		,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('GrauText'				,'200,200,200'	,'50,50,50'		,'@'			,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('GrauDatum'			,'200,200,200'	,'50,50,50'		,'dd/mm/yyyy'	,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('HintergrundGelb'		,'255,255,204'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundRot'		,'255,96,59'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundGrün'		,'89,213,89'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundBlau'		,'0,153,255'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundGrau'		,'200,200,200'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundHellRot'	,'255,160,160'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundHellGrün'	,'170,225,135'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundHellBlau'	,'145,200,255'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('HintergrundHellOrange','255,210,130'	,'0,0,0'		,''				,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('TimeBorderNext'		,'0,0,0'		,'0,0,0'		,''				,'-1','-1'	,''			,'center','-1','1','0')
		   ,('TimeBorderPrevious'	,'0,0,0'		,'0,0,0'		,''				,'-1','-1'	,''			,'center','-1','0','1')	
		   ,('FlexText'				,'230,255,230'	,'0,0,0'		,'@'			,'-1','-1'	,'left'		,'center','-1','0','0')
		   ,('FlexDezimal2'			,'230,255,230'	,'0,0,0'		,'#,###0.00'    ,'-1','-1'	,'right'	,'center','-1','0','0')
		   ,('FlexGanzzahl'			,'230,255,230'	,'0,0,0'		,'#,###0'		,'-1','-1'	,'right'	,'center','-1','0','0')
		)  
    AS Source (FormatID, BackgroundColor,FontColor,ValueFormat,Bold,Italic,AlignmentHorizontal, AlignmentVertical,TextUnderlined,BorderToNext,BorderToPrevious)  
ON Target.FormatID = Source.FormatID

WHEN MATCHED THEN  
	UPDATE 
		 SET 
		 Target.BackgroundColor		= Source.BackgroundColor
		,Target.FontColor			= Source.FontColor
		,Target.ValueFormat			= Source.ValueFormat  
WHEN NOT MATCHED BY TARGET THEN  
	INSERT (
			 FormatID
			,BackgroundColor
			,FontColor
			,ValueFormat
			,Bold
			,Italic
			,AlignmentHorizontal
			,AlignmentVertical
			,TextUnderlined
			,BorderToNext
			,BorderToPrevious
		   ) 
	VALUES (
			 Source.FormatID
			,Source.BackgroundColor
			,Source.FontColor
			,Source.ValueFormat
			,Source.Bold
			,Source.Italic
			,Source.AlignmentHorizontal
			,Source.AlignmentVertical
			,Source.TextUnderlined
			,Source.BorderToNext
			,Source.BorderToPrevious);

GO
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

DECLARE @ListID NVARCHAR(255) = 'suEvents'

-- 1. Create or update the list 
MERGE INTO planning.thLists AS Target  
	USING
		(VALUES 
			 (@ListID,'Eventklassifikation','','Für viele Solutions notwendig','','String','','','')
		)
    AS Source (ListID,NameShort,NameLong,CommentUser,CommentDev,Datentyp,Source,SourceFormula,FormatID)  
ON Target.ListID = Source.ListID

WHEN MATCHED THEN  
	UPDATE 
		 SET
		 Target.NameShort = Source.NameShort
		,Target.NameLong=Source.NameLong
		,Target.CommentUser = Source.CommentUser
		,Target.CommentDev = Source.CommentDev 
		,Target.Datentyp = Source.Datentyp
		,Target.Source = Source.Source
		,Target.SourceFormula = Source.SourceFormula
		,Target.FormatID = Source.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID,NameShort,NameLong,CommentUser,CommentDev,Datentyp,Source,SourceFormula,FormatID) 
	VALUES (Source.ListID, Source.NameShort,Source.NameLong,Source.CommentUser,Source.CommentDev,Source.Datentyp,Source.Source,Source.SourceFormula,Source.FormatID);


-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);

-- 2.2 merge new list values
MERGE INTO planning.thListValues AS Target  
	USING
		(VALUES 
			 (@ListID,0,0	,'Info'					,'','HintergrundGelb'	)
			,(@ListID,0,0	,'Prüfen'				,'',''					)
			,(@ListID,0,0	,'Kündigung möglich'	,'','HintergrundRot'	)
			,(@ListID,0,0	,'gekündigt'			,'',''					)
			,(@ListID,0,0	,'Vertragsende möglich'	,'','HintergrundGelb'	)
			,(@ListID,0,0	,'Vertragsende'			,'',''					)
			,(@ListID,0,0	,'abgeschlossen'		,'',''					)
			,(@ListID,0,0	,'gekündigt'			,'',''					)
		)
    AS Source (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID)  
ON Target.ListID = Source.ListID AND Target.ValueText = Source.ValueText

WHEN MATCHED THEN  
	UPDATE 
		 SET
		 Target.ValueInt = Source.ValueInt
		,Target.Scale=Source.Scale
		,Target.ValueText = Source.ValueText
		,Target.ValueComment = Source.ValueComment 
		,Target.FormatID = Source.FormatID
WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID) 
	VALUES (Source.ListID, Source.ValueInt,Source.Scale,Source.ValueText,Source.ValueComment,Source.FormatID);

GO
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

DECLARE @ListID NVARCHAR(255) = 'suJaNein'

-- 1. Create or update the list 
MERGE INTO planning.thLists AS Target  
	USING
		(VALUES 
			 (@ListID,'Ja-Nein Auswahl','','Für viele Solutions notwendig','','String','','','')
		)
    AS Source (ListID,NameShort,NameLong,CommentUser,CommentDev,Datentyp,Source,SourceFormula,FormatID)  
ON Target.ListID = Source.ListID

WHEN MATCHED THEN  
	UPDATE 
		 SET
		 Target.NameShort = Source.NameShort
		,Target.NameLong=Source.NameLong
		,Target.CommentUser = Source.CommentUser
		,Target.CommentDev = Source.CommentDev 
		,Target.Datentyp = Source.Datentyp
		,Target.Source = Source.Source
		,Target.SourceFormula = Source.SourceFormula
		,Target.FormatID = Source.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID,NameShort,NameLong,CommentUser,CommentDev,Datentyp,Source,SourceFormula,FormatID) 
	VALUES (Source.ListID, Source.NameShort,Source.NameLong,Source.CommentUser,Source.CommentDev,Source.Datentyp,Source.Source,Source.SourceFormula,Source.FormatID);


-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);

-- 2.2 merge new list values
MERGE INTO planning.thListValues AS Target  
	USING
		(VALUES 
			 (@ListID,0,0	,'Ja'				,'',''	)
			,(@ListID,0,0	,'Nein'				,'',''	)
			,(@ListID,0,0	,'unbekannt'		,'',''	)

		)
    AS Source (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID)  
ON Target.ListID = Source.ListID AND Target.ValueText = Source.ValueText

WHEN MATCHED THEN  
	UPDATE 
		 SET
		 Target.ValueInt = Source.ValueInt
		,Target.Scale=Source.Scale
		,Target.ValueText = Source.ValueText
		,Target.ValueComment = Source.ValueComment 
		,Target.FormatID = Source.FormatID
WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID) 
	VALUES (Source.ListID, Source.ValueInt,Source.Scale,Source.ValueText,Source.ValueComment,Source.FormatID);

GO
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

-- will be deliverd empty
DECLARE @ListID NVARCHAR(255) = 'sxResponsiblePersons';

-- 1. Create or update the list 
MERGE INTO planning.thLists AS TARGET  
USING (VALUES 

	(@ListID, 'Verantwortliche', '', 'Systemliste - notwendig, Änderungen möglich', '', 'String', '', '', '')

) AS SOURCE (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID)  
ON TARGET.ListID = SOURCE.ListID

WHEN MATCHED THEN  
	UPDATE 
	SET	TARGET.NameShort = SOURCE.NameShort
		, TARGET.NameLong = SOURCE.NameLong
		, TARGET.CommentUser = SOURCE.CommentUser
		, TARGET.CommentDev = SOURCE.CommentDev 
		, TARGET.Datentyp = SOURCE.Datentyp
		, TARGET.Source = SOURCE.Source
		, TARGET.SourceFormula = SOURCE.SourceFormula
		, TARGET.FormatID = SOURCE.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID) 
	VALUES (SOURCE.ListID
		, SOURCE.NameShort
		, SOURCE.NameLong
		, SOURCE.CommentUser
		, SOURCE.CommentDev
		, SOURCE.Datentyp
		, SOURCE.Source
		, SOURCE.SourceFormula
		, SOURCE.FormatID);

-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);


-- 2.2 merge new list values
MERGE INTO planning.thListValues AS Target  
	USING
		(VALUES 
			 (@ListID,0,0	,'Herr Fuchs'			,'',''	)
			,(@ListID,0,0	,'Frau Igel'			,'',''	)

		)
    AS Source (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID)  
ON Target.ListID = Source.ListID AND Target.ValueText = Source.ValueText

WHEN MATCHED THEN  
	UPDATE 
		 SET
		 Target.ValueInt = Source.ValueInt
		,Target.Scale=Source.Scale
		,Target.ValueText = Source.ValueText
		,Target.ValueComment = Source.ValueComment 
		,Target.FormatID = Source.FormatID
WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID,ValueInt,Scale,ValueText,ValueComment,FormatID) 
	VALUES (Source.ListID, Source.ValueInt,Source.Scale,Source.ValueText,Source.ValueComment,Source.FormatID);

GO
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

-- will be deliverd with sample data to show formatting effect
DECLARE @ListID NVARCHAR(255) = 'sxStatus';

-- 1. Create or update the list 
MERGE INTO planning.thLists AS TARGET  
USING (VALUES 

	(@ListID, 'Status', '', 'Systemliste - notwendig, Änderungen möglich', '', 'String', '', '', '')

) AS SOURCE (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID)  
ON TARGET.ListID = SOURCE.ListID

WHEN MATCHED THEN  
	UPDATE 
	SET	TARGET.NameShort = SOURCE.NameShort
		, TARGET.NameLong = SOURCE.NameLong
		, TARGET.CommentUser = SOURCE.CommentUser
		, TARGET.CommentDev = SOURCE.CommentDev 
		, TARGET.Datentyp = SOURCE.Datentyp
		, TARGET.Source = SOURCE.Source
		, TARGET.SourceFormula = SOURCE.SourceFormula
		, TARGET.FormatID = SOURCE.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID) 
	VALUES (SOURCE.ListID
		, SOURCE.NameShort
		, SOURCE.NameLong
		, SOURCE.CommentUser
		, SOURCE.CommentDev
		, SOURCE.Datentyp
		, SOURCE.Source
		, SOURCE.SourceFormula
		, SOURCE.FormatID);

-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION ALL SELECT DISTINCT [Status] FROM planning.tdProducts
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);

-- 2.2 merge new list values
MERGE INTO planning.thListValues AS TARGET  
USING (VALUES 

	  (@ListID, 0, 0 , 'Aktiv',			'', 'HintergrundGrün')
	, (@ListID, 0, 0 , 'Inaktiv',		'', 'HintergrundGrau')
	, (@ListID, 0, 0 , 'Wiedervorlage', '', 'HintergrundGelb')
	, (@ListID, 0, 0 , 'Prüfen',		'', 'HintergrundGelb')
	, (@ListID, 0, 0 , 'Kündigen',		'', 'HintergrundRot'	)

) AS SOURCE (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)  
ON TARGET.ListID = SOURCE.ListID AND TARGET.ValueText = SOURCE.ValueText

WHEN MATCHED THEN  
	UPDATE 
	SET TARGET.ValueInt = SOURCE.ValueInt
		, TARGET.Scale = SOURCE.Scale
		, TARGET.ValueText = SOURCE.ValueText
		, TARGET.ValueComment = SOURCE.ValueComment 
		, TARGET.FormatID = SOURCE.FormatID 

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID) 
	VALUES (SOURCE.ListID
		, SOURCE.ValueInt
		, SOURCE.Scale
		, SOURCE.ValueText
		, SOURCE.ValueComment
		, SOURCE.FormatID);

GO 
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

DECLARE @ListID NVARCHAR(255) = 'sxUnits';

-- 1. Create or update the list 
MERGE INTO planning.thLists AS TARGET  
USING (VALUES 

	(@ListID, 'Einheiten für Werte', '', 'Systemliste - notwendig, Änderungen möglich', '', 'String', '', '', '')

) AS SOURCE (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID)  
ON TARGET.ListID = SOURCE.ListID

WHEN MATCHED THEN  
	UPDATE 
	SET	 TARGET.NameShort		= SOURCE.NameShort
		,TARGET.NameLong		= SOURCE.NameLong
		,TARGET.CommentUser		= SOURCE.CommentUser
		,TARGET.CommentDev		= SOURCE.CommentDev 
		,TARGET.Datentyp		= SOURCE.Datentyp
		,TARGET.Source			= SOURCE.Source
		,TARGET.SourceFormula	= SOURCE.SourceFormula
		,TARGET.FormatID		= SOURCE.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID) 
	VALUES (
		 SOURCE.ListID
		,SOURCE.NameShort
		,SOURCE.NameLong
		,SOURCE.CommentUser
		,SOURCE.CommentDev
		,SOURCE.Datentyp
		,SOURCE.Source
		,SOURCE.SourceFormula
		,SOURCE.FormatID
		);

-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT fV.ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);

-- 2.2 merge new list values
MERGE INTO planning.thListValues AS TARGET  
USING (VALUES 

	 (@ListID, 0, 0 , 'EUR',			'Euro'				,'')
	,(@ListID, 0, 0 , 'USD',			'US-Dollar'			,'')
	,(@ListID, 0, 0 , 'RUB',			'Russische Rubel'	,'')
	,(@ListID, 0, 0 , 'PLN',			'Polnische Zloty'	,'')
	,(@ListID, 0, 0 , 'Stück',			'Anzahl in Stück'	,'')
	,(@ListID, 0, 0 , 'Tage',			'Anzahl der Tage'	,'')
	,(@ListID, 0, 0 , 'Prozent',		''					,'')
	,(@ListID, 0, 0 , 'Kilogramm',		''					,'')
	,(@ListID, 0, 0 , 'Tonnen',		''						,'')
	,(@ListID, 0, 0 , 'Quadratmeter',	''					,'')
	,(@ListID, 0, 0 , 'Text',			''					,'')

) AS SOURCE (ListID, ValueInt, Scale, ValueText, ValueComment,FormatID)  
ON TARGET.ListID = SOURCE.ListID AND TARGET.ValueText = SOURCE.ValueText

WHEN MATCHED THEN  
	UPDATE 
	SET TARGET.ValueInt = SOURCE.ValueInt
		, TARGET.Scale = SOURCE.Scale
		, TARGET.ValueText = SOURCE.ValueText
		, TARGET.ValueComment = SOURCE.ValueComment 

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, ValueInt, Scale, ValueText, ValueComment,FormatID) 
	VALUES (SOURCE.ListID
		, SOURCE.ValueInt
		, SOURCE.Scale
		, SOURCE.ValueText
		, SOURCE.ValueComment
		, SOURCE.FormatID);

GO
/*
Merge new list members with old list members from this ListID
 - Old used members are keept
 - Old unused members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

DECLARE @ListID NVARCHAR(255) = 'sxValueEffectParameters';

-- 1. Create or update the list 
MERGE INTO planning.thLists AS TARGET  
USING (VALUES 

	(@ListID, 'Parameter für Wertreihen', '', 'Systemliste - notwendig, Änderungen möglich', '', 'String', '', '', '')

) AS SOURCE (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID)  
ON TARGET.ListID = SOURCE.ListID

WHEN MATCHED THEN  
	UPDATE 
	SET	TARGET.NameShort = SOURCE.NameShort
		, TARGET.NameLong = SOURCE.NameLong
		, TARGET.CommentUser = SOURCE.CommentUser
		, TARGET.CommentDev = SOURCE.CommentDev 
		, TARGET.Datentyp = SOURCE.Datentyp
		, TARGET.Source = SOURCE.Source
		, TARGET.SourceFormula = SOURCE.SourceFormula
		, TARGET.FormatID = SOURCE.FormatID

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, NameShort, NameLong, CommentUser, CommentDev, Datentyp, Source, SourceFormula, FormatID) 
	VALUES (SOURCE.ListID
		, SOURCE.NameShort
		, SOURCE.NameLong
		, SOURCE.CommentUser
		, SOURCE.CommentDev
		, SOURCE.Datentyp
		, SOURCE.Source
		, SOURCE.SourceFormula
		, SOURCE.FormatID);

-- 2.  Update the list values
-- 2.1 Delete unused members of this list
;WITH keept AS (
	SELECT DISTINCT ValueText
	FROM planning.tfValues fV INNER JOIN planning.tdValueSeries dVS
		ON fV.ValueSeriesKey = dVS.ValueSeriesKey AND dVS.ValueListID = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute1  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource1 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute2  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource2 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute3  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource3 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute4  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource4 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute5  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource5 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute6  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource6 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute7  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource7 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute8  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource8 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute9  FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource9 =  @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute10 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource10 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute11 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource11 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute12 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource12 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute13 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource13 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute14 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource14 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute15 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource15 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute16 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource16 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute17 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource17 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute18 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource18 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute19 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource19 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute20 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource20 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute21 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource21 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute22 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource22 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute23 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource23 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute24 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource24 = @ListID
	UNION SELECT DISTINCT dP.GlobalAttribute25 FROM planning.tdProducts dP INNER JOIN planning.tdProductLines dPL ON dP.ProductLineKey = dPL.ProductLineKey AND dPL.GlobalAttributeSource25 = @ListID 
)
DELETE FROM planning.thListValues
WHERE ListID = @ListID AND ValueText NOT IN (SELECT DISTINCT ValueText FROM keept);

-- 2.2 merge new list values
MERGE INTO planning.thListValues AS TARGET  
USING (VALUES 

	 (@ListID, 0, 0 , 'W#20', '','')
	,(@ListID, 0, 0 , 'W#30', '','')
	,(@ListID, 0, 0 , 'W#40', '','')
    ,(@ListID, 0, 0 , 'W#50', '','')

) AS SOURCE (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)  
ON TARGET.ListID = SOURCE.ListID AND TARGET.ValueText = SOURCE.ValueText

WHEN MATCHED THEN  
	UPDATE 
	SET TARGET.ValueInt			= SOURCE.ValueInt
		, TARGET.Scale			= SOURCE.Scale
		, TARGET.ValueText		= SOURCE.ValueText
		, TARGET.ValueComment	= SOURCE.ValueComment 

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (ListID, ValueInt, Scale, ValueText, ValueComment,FormatID) 
	VALUES (
		 SOURCE.ListID
		,SOURCE.ValueInt
		,SOURCE.Scale
		,SOURCE.ValueText
		,SOURCE.ValueComment
		,SOURCE.FormatID
		);

GO
/*
Merge new ValueEffects	list
 - Non exists members are deleted
 - Existing members are updated
 - New members are created

12/2016 DataFactory 4.0 
Gerd Tautenhahn for saxess-software gmbh
*/

--1. Check for usage and delete unused
DELETE FROM planning.tgValueEffects WHERE EffectID NOT IN (SELECT DISTINCT dVS.Effect FROM planning.tdValueSeries dVS);

-- 2. Create or update the list 
MERGE INTO planning.tgValueEffects AS TARGET  
USING (VALUES 

	  (N'Kosten'			, -1, 0, 0, 0	, N'SUM', N'SUM'		)
	, (N'Kosten=Auszahlg'	, -1, 0, -1, 0	, N'SUM', N'SUM'		)
	, (N'Ertrag'			, 1, 0, 0, 0	, N'SUM', N'SUM'		)
	, (N'Ertrag=Einzahlg'	, 1, 0, 1, 0	, N'SUM', N'SUM'		)
	, (N'Aktivbestand'		, 0, 1, 0, 0	, N'LAST', N'SUM'		)
	, (N'Passivbestand'		, 0, -1, 0, 0	, N'LAST', N'SUM'		)
	, (N'Auszahlung'		, 0, 0, -1, 0	, N'SUM', N'SUM'		)
	, (N'Einzahlung'		, 0, 0, 1, 0	, N'SUM', N'SUM'		)
	, (N'Menge'				, 0, 0, 0, 1	, N'SUM', N'SUM'		)
	, (N'Stück'				, 0, 0, 0, 1	, N'SUM', N'SUM'		)
	, (N'Hilfswert'			, 0, 0, 0, 0	, N'NONE', N'NONE'		)
	, (N'Text'				, 0, 0, 0, 0	, N'CONCAT', N'CONCAT'	)
	, (N'Überschrift'		, 0, 0, 0, 0	, N'CONCAT', N'CONCAT'	)
	, (N'Zwischensumme'		, 0, 0, 0, 0	, N'NONE', N'NONE'		)

) AS SOURCE (EffectID, ProfitLossEffect, BalanceEffect, CashEffect, QuantityEffect, AggregationTypTime, AggregationTypStructure)  
ON TARGET.EffectID = SOURCE.EffectID

WHEN MATCHED THEN  
	UPDATE 
	SET	TARGET.ProfitLossEffect = SOURCE.ProfitLossEffect
		, TARGET.BalanceEffect = SOURCE.BalanceEffect
		, TARGET.CashEffect = SOURCE.CashEffect
		, TARGET.QuantityEffect = SOURCE.QuantityEffect 
		, TARGET.AggregationTypTime = SOURCE.AggregationTypTime
		, TARGET.AggregationTypStructure = SOURCE.AggregationTypStructure

WHEN NOT MATCHED BY TARGET THEN  
	INSERT (EffectID, ProfitLossEffect, BalanceEffect, CashEffect, QuantityEffect, AggregationTypTime, AggregationTypStructure) 
	VALUES (SOURCE.EffectID
		, SOURCE.ProfitLossEffect
		, SOURCE.BalanceEffect
		, SOURCE.CashEffect
		, SOURCE.QuantityEffect
		, SOURCE.AggregationTypTime
		, SOURCE.AggregationTypStructure);
GO
/*
Table dimension accounts
Gerd Tautenhahn for saxess-software gmbh
03/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tdKonten') AND type in (N'U'))
	DROP TABLE result.tdKonten;
GO

CREATE TABLE result.tdKonten(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KontenID				NVARCHAR(255)	NOT NULL
	,KontenName				NVARCHAR(255)	NOT NULL
	,KontenTyp				NVARCHAR(255)	NOT NULL
	,Ebene1					NVARCHAR(255)	NOT NULL
	,Ebene2					NVARCHAR(255)	NOT NULL
	,Ebene3					NVARCHAR(255)	NOT NULL
	,Ebene4					NVARCHAR(255)	NOT NULL
	,Ebene5					NVARCHAR(255)	NOT NULL
	,Ebene6					NVARCHAR(255)	NOT NULL
	,Ebene7					NVARCHAR(255)	NOT NULL
	,Ebene8					NVARCHAR(255)	NOT NULL
	,FlexAttribut			NVARCHAR(255)	NOT NULL

	CONSTRAINT PK_result_tdKonten PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für die Dimension Konten - landesspezifisch für DE. Umfasst Sachkonten (GuV / Bilanz) und Statistikkonten (Mengen), keine Debitoren und Kreditoren.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tdKonten';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, z.B. Integration, Planung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für das Sachkonto, joinbar mit der Dimension Konten über MandantenID, KontenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'KontenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Name des Kontos',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'KontenName';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kontotyp - fester Listenwert - GuV, Bilanz oder Statistik',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'KontenTyp';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Erste Hierachieebene der Kontogliederung (höchste Aggregation)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'Ebene1';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Achte Hierachieebene der Kontogliederung (höchste Detailierung)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'Ebene8';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKonten',
    @level2type = N'Column',				@level2name = 'FlexAttribut';


GO
/*
Table for dimension cost center - german vesion
Gerd Tautenhahn for saxess-software gmbh
03/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tdKostenstellen') AND type in (N'U'))
	DROP TABLE result.tdKostenstellen;
GO

CREATE TABLE result.tdKostenstellen(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenstellenID		NVARCHAR(255)	NOT NULL
	,KostenstellenName		NVARCHAR(255)	NOT NULL
	,KostenstellenTyp		NVARCHAR(255)	NOT NULL
	,Umlagestufe			INT				NOT NULL
	,Ebene1					NVARCHAR(255)	NOT NULL
	,Ebene2					NVARCHAR(255)	NOT NULL
	,Ebene3					NVARCHAR(255)	NOT NULL
	,Ebene4					NVARCHAR(255)	NOT NULL
	,Ebene5					NVARCHAR(255)	NOT NULL
	,Ebene6					NVARCHAR(255)	NOT NULL
	,Ebene7					NVARCHAR(255)	NOT NULL
	,Ebene8					NVARCHAR(255)	NOT NULL
	,FlexAttribut			NVARCHAR(255)	NOT NULL

	CONSTRAINT PK_result_tdKostenstellen PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für die Dimension Kostenstellen - landesspezifisch für DE. Umfasst Kostenstellen aus Integration und Planungskostenstellen.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tdKostenstellen';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, z.B. Integration, Planung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Kostenstelle, joinbar mit den Buchungsjournalen über MandantenID, KostenstellenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'KostenstellenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Name der Kostenstelle',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'KostenstellenName';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kostenstellentyp - fester Listenwert - Hauptkostenstelle, Hilfskostenstelle',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'KostenstellenTyp';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Umlagestufe der Kostenstelle',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'Umlagestufe';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Erste Hierachieebene der Kostenstellengliederung (höchste Aggregation)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'Ebene1';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Achte Hierachieebene der Kostenstellengliederung (höchste Detailierung)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'Ebene8';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenstellen',
    @level2type = N'Column',				@level2name = 'FlexAttribut';


GO
/*
Table for dimension cost objects - german vesion
Gerd Tautenhahn for saxess-software gmbh
03/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tdKostenträger') AND type in (N'U'))
	DROP TABLE result.tdKostenträger;
GO

CREATE TABLE result.tdKostenträger(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenträgerID			NVARCHAR(255)	NOT NULL
	,KostenträgerName		NVARCHAR(255)	NOT NULL
	,KostenträgerTyp		NVARCHAR(255)	NOT NULL
	,Ebene1					NVARCHAR(255)	NOT NULL
	,Ebene2					NVARCHAR(255)	NOT NULL
	,Ebene3					NVARCHAR(255)	NOT NULL
	,Ebene4					NVARCHAR(255)	NOT NULL
	,Ebene5					NVARCHAR(255)	NOT NULL
	,Ebene6					NVARCHAR(255)	NOT NULL
	,Ebene7					NVARCHAR(255)	NOT NULL
	,Ebene8					NVARCHAR(255)	NOT NULL
	,FlexAttribut			NVARCHAR(255)	NOT NULL

	CONSTRAINT PK_result_tdKostenträger PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für die Dimension Kostenträger - landesspezifisch für DE. Umfasst Kostenträger aus Integration und Planungskostenträger.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tdKostenträger';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, z.B. Integration, Planung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Kostenträger, joinbar mit den Buchungsjournalen über MandantenID, KostenträgerID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'KostenträgerID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Name des Kostenträger',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'KostenträgerName';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kostenträgertyp - freier Wert',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'KostenträgerTyp';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Erste Hierachieebene der Kostenträgergliederung (höchste Aggregation)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'Ebene1';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Achte Hierachieebene der Kostenträgergliederung (höchste Detailierung)',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'Ebene8';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tdKostenträger',
    @level2type = N'Column',				@level2name = 'FlexAttribut';


GO
/*
Table for result calculation of planning values
Gerd Tautenhahn for saxess-software gmbh
02/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tfBuchungsjournalPlan') AND type in (N'U'))
	DROP TABLE result.tfBuchungsjournalPlan;
GO

CREATE TABLE result.tfBuchungsjournalPlan(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,SolutionID				NVARCHAR(255)	NOT NULL
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenstellenID		NVARCHAR(255)	NOT NULL
	,KostenträgerID			NVARCHAR(255)	NOT NULL
	,KontenID				NVARCHAR(255)	NOT NULL
	,Jahr					INT				NOT NULL
	,Monat					INT				NOT NULL
	,Tag					INT				NOT NULL
	,Buchungsgruppe			NVARCHAR(4000)	NOT NULL
	,Buchungstext			NVARCHAR(4000)	NOT NULL
	,Belegnummer			NVARCHAR(4000)	NOT NULL
	,FlexAttribut			NVARCHAR(4000)	NOT NULL
	,Wert					MONEY			NOT NULL

	CONSTRAINT PK_result_tfBuchungsjournalPlan PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Clustered by Solution, to be able to delete quick by solution as truncate not possible
CREATE CLUSTERED INDEX IX_result_tfBuchungsjournalPlan ON result.tfBuchungsjournalPlan (SolutionID ASC);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für globales Ergebnisbuchungsjournal - landesspezifisch für DE. Tabelle ist Output für den Enduser und daher sprechend und leicht joinbar.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tfBuchungsjournalPlan';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für die Solution, welche diesen Teil berechnet. Spalte ist indiziert um schnell darüber löschen zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'SolutionID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, [FactoryID-ProdutlineID-ProductID] sollte hier als Ursprungscode gespeichert werden um die Detaildaten Quelle ermitteln zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Kostenstelle, joinbar mit der Dimension Kostenstellen über MandantenID, KostenstellenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'KostenstellenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Kostenträger, joinbar mit der Dimension Kostenträger über MandantenID, KostenträgerID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'KostenträgerID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für das Sachkonto, joinbar mit der Dimension Konten über MandantenID, KontenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'KontenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Jahr der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Jahr';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Monat der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Monat';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tag der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Tag';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Fachliche Trennung der Berechnungsergebnisse einer Solution - z.B. Solution = Investitionsplanung mit Buchungsgruppen = Anlagenzugang, Abschreibungen, Auszahlungen.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Buchungsgruppe';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Detailierter Beschreibungstext, sollte auch Productname und wichtige Berechnungsparameter beinhalten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Buchungstext';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ggf. Belegnummer für Identifikation der Buchung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Belegnummer';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung, bspw. für Werte aus Status oder Globalattribut Feldern.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'FlexAttribut';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Wert in Buchhaltungssystematik - Soll Werte (Aktiva und Kosten) postiv, Haben Werte (Passiva und Erträge) negativ.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlan',
    @level2type = N'Column',				@level2name = 'Wert';

GO
/*
Table for historic result calculation of planning values
Gerd Tautenhahn for saxess-software gmbh
02/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tfBuchungsjournalPlanSicherung') AND type in (N'U'))
	DROP TABLE result.tfBuchungsjournalPlanSicherung;
GO

CREATE TABLE result.tfBuchungsjournalPlanSicherung(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,Planversion			BIGINT			NOT NULL
	,SolutionID				NVARCHAR(255)	NOT NULL
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenstellenID		NVARCHAR(255)	NOT NULL
	,KostenträgerID			NVARCHAR(255)	NOT NULL
	,KontenID				NVARCHAR(255)	NOT NULL
	,Jahr					INT				NOT NULL
	,Monat					INT				NOT NULL
	,Tag					INT				NOT NULL
	,Buchungsgruppe			NVARCHAR(4000)	NOT NULL
	,Buchungstext			NVARCHAR(4000)	NOT NULL
	,Belegnummer			NVARCHAR(4000)	NOT NULL
	,FlexAttribut			NVARCHAR(4000)	NOT NULL
	,Wert					MONEY			NOT NULL

	CONSTRAINT PK_result_tfBuchungsjournalPlanSicherung PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Clustered by Solution, to be able to delete quick by solution as truncate not possible
CREATE CLUSTERED INDEX IX_result_tfBuchungsjournalPlanSicherung ON result.tfBuchungsjournalPlanSicherung (PlanVersion ASC);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für Sicherungen des globales Ergebnisbuchungsjournals.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tfBuchungsjournalPlanSicherung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Planung, joinbar mit Sicherungsindex für Detailinformationen, Format YYYYMMDDHHMM.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Planversion';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für die Solution, welche diesen Teil berechnet.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'SolutionID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, [FactoryID-ProdutlineID-ProductID] sollte hier als Ursprungscode gespeichert werden um die Detaildaten Quelle ermitteln zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Kostenstelle, joinbar mit der Dimension Kostenstellen über MandantenID, KostenstellenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'KostenstellenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Kostenträger, joinbar mit der Dimension Kostenträger über MandantenID, KostenträgerID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'KostenträgerID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für das Sachkonto, joinbar mit der Dimension Konten über MandantenID, KontenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'KontenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Jahr der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Jahr';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Monat der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Monat';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tag der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Tag';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Fachliche Trennung der Berechnungsergebnisse einer Solution - z.B. Solution = Investitionsplanung mit Buchungsgruppen = Anlagenzugang, Abschreibungen, Auszahlungen.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Buchungsgruppe';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Detailierter Beschreibungstext, sollte auch Productname und wichtige Berechnungsparameter beinhalten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Buchungstext';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ggf. Belegnummer zur Buchungsidentifikation',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Belegnummer';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung, bspw. für Werte aus Status oder Globalattribut Feldern.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'FlexAttribut';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Wert in Buchhaltungssystematik - Soll Werte (Aktiva und Kosten) postiv, Haben Werte (Passiva und Erträge) negativ.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherung',
    @level2type = N'Column',				@level2name = 'Wert';

GO
/*
Table for historic result calculation of planning values
Gerd Tautenhahn for saxess-software gmbh
02/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tfBuchungsjournalPlanSicherungsIndex') AND type in (N'U'))
	DROP TABLE result.tfBuchungsjournalPlanSicherungsIndex;
GO

CREATE TABLE result.tfBuchungsjournalPlanSicherungsIndex(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,Planversion			BIGINT			NOT NULL
	,PlanName				NVARCHAR(255)	NOT NULL
	,PlanBeschreibung		NVARCHAR(4000)	NOT NULL	


	CONSTRAINT PK_result_tfBuchungsjournalPlanSicherungsIndex PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Default Values for Actuals and DataFactory Plan
DELETE FROM result.tfBuchungsjournalPlanSicherungsIndex WHERE Planversion IN (0,1)

INSERT INTO result.tfBuchungsjournalPlanSicherungsIndex (Planversion,PlanName,PlanBeschreibung) VALUES
	(0,'IST','IST Werte aus Datenintegration'),
	(1,'Laufender PLAN', 'Aktuelle Planung aus DataFactory')


-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für den Index der Sicherungen - Verzeichnis aller enthaltenen Sicherungssätze in result.tfBuchungsjournalPlanSicherung.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tfBuchungsjournalPlanSicherungsIndex';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherungsIndex',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Fachlicher Primärschlüssel, ID für die Planversion',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherungsIndex',
    @level2type = N'Column',				@level2name = 'Planversion';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Name der Sicherung - kurz zur Auswahl in DropDown Feldern',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherungsIndex',
    @level2type = N'Column',				@level2name = 'PlanName';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Beschreibung der Sicherung - Langtext',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfBuchungsjournalPlanSicherungsIndex',
    @level2type = N'Column',				@level2name = 'PlanBeschreibung';






GO
/*
Table for result output of events
Gerd Tautenhahn for saxess-software gmbh
04/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tfEventsPlan') AND type in (N'U'))
	DROP TABLE result.tfEventsPlan;
GO

CREATE TABLE result.tfEventsPlan(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,SolutionID				NVARCHAR(255)	NOT NULL
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenstellenID		NVARCHAR(255)	NOT NULL
	,KostenträgerID			NVARCHAR(255)	NOT NULL
	,KontenID				NVARCHAR(255)	NOT NULL
	,Jahr					INT				NOT NULL
	,Monat					INT				NOT NULL
	,Tag					INT				NOT NULL
	,Buchungsgruppe			NVARCHAR(4000)	NOT NULL
	,Event					NVARCHAR(4000)	NOT NULL
	,Eventbemerkung			NVARCHAR(4000)	NOT NULL
	,FlexAttribut			NVARCHAR(4000)	NOT NULL


	CONSTRAINT PK_result_tfEventsPlan PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Clustered by Solution, to be able to delete quick by solution as truncate not possible
CREATE CLUSTERED INDEX IX_result_tfEventsPlan ON result.tfEventsPlan (SolutionID ASC);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für globales Ergebnisjournal für Events. Tabelle ist Output für den Enduser und daher sprechend und leicht joinbar.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tfEventsPlan';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für die Solution, welche diesen Teil erzeugt. Spalte ist indiziert um schnell darüber löschen zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'SolutionID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, FID-PLID-PID sollte hier als Ursprungscode gespeichert werden um die Detaildaten Quelle ermitteln zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Kostenstelle, joinbar mit der Dimension Kostenstellen über MandantenID, KostenstellenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'KostenstellenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Kostenträger, joinbar mit der Dimension Kostenträger über MandantenID, KostenträgerID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'KostenträgerID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für das Mengenkonto, joinbar mit der Dimension Konten über MandantenID, KontenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'KontenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Jahr der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Jahr';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Monat der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Monat';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tag der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Tag';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Fachliche Trennung der Teilmengen einer Solution.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Buchungsgruppe';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Detailierter Beschreibungstext, sollte auch Productname und wichtige Berechnungsparameter beinhalten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Event';


EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Detailierter Beschreibungstext, sollte auch Productname und wichtige Berechnungsparameter beinhalten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'Eventbemerkung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung, bspw. für Werte aus Status oder Globalattribut Feldern.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfEventsPlan',
    @level2type = N'Column',				@level2name = 'FlexAttribut';


GO
/*
Table for result calculation of planning values for quantity values
Gerd Tautenhahn for saxess-software gmbh
02/2019 for DataFactory 4.0

*/


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.tfMengenjournalPlan') AND type in (N'U'))
	DROP TABLE result.tfMengenjournalPlan;
GO

CREATE TABLE result.tfMengenjournalPlan(

	 RowKey					BIGINT			NOT NULL IDENTITY (1,1)
	,SolutionID				NVARCHAR(255)	NOT NULL
	,Ursprung				NVARCHAR(255)	NOT NULL	
	,MandantenID			NVARCHAR(255)	NOT NULL
	,KostenstellenID		NVARCHAR(255)	NOT NULL
	,KostenträgerID			NVARCHAR(255)	NOT NULL
	,KontenID				NVARCHAR(255)	NOT NULL
	,Jahr					INT				NOT NULL
	,Monat					INT				NOT NULL
	,Tag					INT				NOT NULL
	,Buchungsgruppe			NVARCHAR(4000)	NOT NULL
	,Buchungstext			NVARCHAR(4000)	NOT NULL
	,FlexAttribut			NVARCHAR(4000)	NOT NULL
	,Einheit				NVARCHAR(4000)	NOT NULL
	,Mengentyp				NVARCHAR(4000)	NOT NULL
	,Wert					MONEY			NOT NULL

	CONSTRAINT PK_result_tfMengenjournalPlan PRIMARY KEY NONCLUSTERED (RowKey)
);

-- Clustered by Solution, to be able to delete quick by solution as truncate not possible
CREATE CLUSTERED INDEX IX_result_tfMengenjournalPlan ON result.tfMengenjournalPlan (SolutionID ASC);

-- Table documentation
EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tabelle für globales Ergebnisbuchungsjournal für Mengenwerte. Tabelle ist Output für den Enduser und daher sprechend und leicht joinbar.',
    @level0type = N'Schema',				@level0name	= 'result',
    @level1type = N'Table',					@level1name	= 'tfMengenjournalPlan';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Technischer Primärschlüssel ohne fachliche Bedeutung.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'RowKey';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für die Solution, welche diesen Teil berechnet. Spalte ist indiziert um schnell darüber löschen zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'SolutionID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Kennzeichen für den Detailurprung der Daten, [FactoryID-ProdutlineID-ProductID] sollte hier als Ursprungscode gespeichert werden um die Detaildaten Quelle ermitteln zu können.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Ursprung';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Mandanten, joinbar mit der Dimension Mandanten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'MandantenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für die Kostenstelle, joinbar mit der Dimension Kostenstellen über MandantenID, KostenstellenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'KostenstellenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für den Kostenträger, joinbar mit der Dimension Kostenträger über MandantenID, KostenträgerID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'KostenträgerID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'ID für das Mengenkonto, joinbar mit der Dimension Konten über MandantenID, KontenID.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'KontenID';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Jahr der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Jahr';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Monat der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Monat';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Tag der buchhalterischen Zuordnung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Tag';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Fachliche Trennung der Berechnungsergebnisse einer Solution.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Buchungsgruppe';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Detailierter Beschreibungstext, sollte auch Productname und wichtige Berechnungsparameter beinhalten.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Buchungstext';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Flexibiles Attribut zur freien Belegung, bspw. für Werte aus Status oder Globalattribut Feldern.',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'FlexAttribut';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Bezeichnung der Mengeneinheit, z.B. Stück, km, qm',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Einheit';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Typ der Menge - Bestand oder Bewegung',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Mengentyp';

EXEC sp_addextendedproperty 
    @name		= N'MS_Description',		@value		= 'Wert in Mengensystematik - Zugänge und Bestände positiv, Abgänge negativ',
    @level0type = N'Schema',				@level0name = 'result',
    @level1type = N'Table',					@level1name = 'tfMengenjournalPlan',
    @level2type = N'Column',				@level2name = 'Wert';

GO
/*
Prozedur zu Buchung des PlanBuchungsjournals und gleichzeitiger Ausgabe pro Factory als Pivot

Ermittelt den Mandanten aus FactoryID

Ermittelt das Sachkonto
	aus Globalattribut 1

Ermittelt die Kostenstellen
	Prio 1: aus der Wertreihe KST
	Prio 2: aus Globalattribut 2 

ProductlineID wird als Flexattribut geschrieben um Rechtefilter nach DataFactory Logik setzen zu können.

Behandelt gesamtes Cluster als Solution "Planwerte global", muss daher ggf. im Cusomizing geteilt werden

TESTCALL
EXEC result.spBuchungsjournalPlan 'SQL','C'

*/

IF OBJECT_ID('result.spBuchungsjournalPlan') IS NOT NULL
DROP PROCEDURE result.spBuchungsjournalPlan
GO

CREATE PROCEDURE result.spBuchungsjournalPlan (
		 @Username		NVARCHAR(255)
		,@FactoryID		NVARCHAR (255)	= ''
		,@ProductlineID NVARCHAR(255)	= ''
		)

AS 

BEGIN

SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	 N'NULL')	+ N'''' 
		
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';	

	BEGIN TRY
		BEGIN TRANSACTION spBuchungsjournalPlan


			-- Determine transaction user  ***************************
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- Hilfstabelle für Periodenkostenstellen erstellen *******************
			IF OBJECT_ID('tempdb..#PeriodenKST') IS NOT NULL
			DROP TABLE #PeriodenKST

			CREATE TABLE #PeriodenKST 
				(
					 RowKey			BIGINT IDENTITY (1,1)
					,ProductKey		BIGINT NOT NULL
					,TimeID			BIGINT NOT NULL
					,PeriodenKST	NVARCHAR (255)	COLLATE DATABASE_DEFAULT	NOT NULL
				)

			INSERT INTO #PeriodenKST
				SELECT 
					ProductKey,
					TimeID,
					ValueText AS PeriodenKST

				FROM planning.tfValues fV

				WHERE 
						FactoryID <> 'ZT'
					AND ValueSeriesID = 'KST'



			-- Buchen aller Werte im Journal für Planwerte (unabhängig von Userrechten) ******************

			DELETE FROM result.tfBuchungsjournalPlan;

			INSERT INTO result.tfBuchungsjournalPlan

				SELECT 
					 'Planung global'					AS SolutionID	
					,fV.FactoryID + '-' 
					 + fV.ProductlineID +'-' 
					 + fV.ProductID						AS Ursprung				
					,dP.FactoryID						AS MandantenID			
					,COALESCE(PKST.PeriodenKST
						,dP.Globalattribute2)			AS KostenstellenID	
					,''									AS KostenträgerID			
					,dP.Globalattribute1				AS KontenID				
					,fV.TimeID / 10000					AS Jahr					
					,fV.TimeID / 100 % 100				AS Monat
					,fV.TimeID % 100					AS Tag					
					,dP.NameShort						AS Buchungsgruppe			
					,dP.NameShort + ' ' + dVS.NameShort AS Buchungstext			
					,''									AS Belegnummer			
					,dP.ProductLineID					AS FlexAttribut		-- temporäre Lösung für Rechtefilter	
					,CAST(fV.ValueInt AS Money)
					 / dVS.Scale 
					 * gVE.ProfitLossEffect				AS Wert
		
				FROM planning.tfValues fV

				LEFT JOIN planning.tdProducts dP ON 
					fV.ProductKey = dP.ProductKey

				LEFT JOIN planning.tdValueSeries dVS ON 
					fV.ValueSeriesKey = dVS.ValueSeriesKey

				LEFT JOIN planning.tgValueEffects gVE ON 
					dVS.Effect = gVE.EffectID
			
				LEFT JOIN #PeriodenKST PKST ON 
						fV.ProductKey	= PKST.ProductKey 
					AND fV.TimeID		= PKST.TimeID

				WHERE 
						fV.FactoryID <> 'ZT'
					AND gVE.ProfitLossEffect <> 0
					AND dVS.[IsNumeric] = 1
					-- Formeln ausschließen welche null Werte liefern
					AND fV.ValueInt <> 0

				-- Ausgabe (optional pro FactoryID) mit Userechtsunterscheidung ****************
				SELECT BJP.*
				FROM result.tfBuchungsjournalPlan BJP

				-- Userrechte
				INNER JOIN 
				(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> '') vUR

				ON BJP.MandantenID		= vUR.FactoryID AND 
				   BJP.FlexAttribut		= vUR.ProductLineID -- temporäre Lösung für Rechtefilter


				WHERE 
						BJP.MandantenID = @FactoryID 
					OR	@FactoryID = ''


	

		COMMIT TRANSACTION spBuchungsjournalPlan;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spBuchungsjournalPlan;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO 

GRANT EXECUTE ON OBJECT ::result.spBuchungsjournalPlan TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spBuchungsjournalPlan TO pf_PlanningFactoryService;
GO
/*
Prozedur für Abfrage von Events als Tabelle und Pivot Output

	Eventermittlung
		- über Wertreihen "E" und "BEM"

	Mandantenfindung
		- über FactoryID

	Kontofindung:
		- über Globalattribut 1

	Kostenstellenermittlung
		- über Globalattribut 2 (Stammkostenstelle)

Testcall
EXEC result.spEventsPlan 'SQL', '3'

*/


IF OBJECT_ID('result.spEventsPlan') IS NOT NULL
DROP PROCEDURE result.spEventsPlan
GO


CREATE PROCEDURE result.spEventsPlan  (
		 @Username		NVARCHAR(255)
		,@FactoryID		NVARCHAR (255)	= ''
		,@ProductlineID NVARCHAR(255)	= ''
		)

AS

BEGIN

SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	 N'NULL')	+ N'''' 
		
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';	

	BEGIN TRY
		BEGIN TRANSACTION spEventsPlanPlan


			-- Determine transaction user  ***************************
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- Buchen aller Werte im Journal für Planwerte (unabhängig von Userrechten) ******************

			DELETE FROM result.tfEventsPlan;

			INSERT INTO result.tfEventsPlan
				SELECT
					'Planung global'					AS SolutionID	
					,fV.FactoryID + '-' 
					 + fV.ProductlineID +'-' 
					 + fV.ProductID						AS Ursprung				
					,dP.FactoryID						AS MandantenID			
					,dP.Globalattribute2				AS KostenstellenID	
					,''									AS KostenträgerID			
					,dP.Globalattribute1				AS KontenID				
					,fV.TimeID / 10000					AS Jahr					
					,fV.TimeID / 100 % 100				AS Monat
					,fV.TimeID % 100					AS Tag	
					,dPL.NameShort						AS Buchungsgruppe

					,fV.ValueText						AS Event
					,COALESCE(f2.ValueText,'')			AS EventBemerkung
					,dP.ProductLineID					AS FlexAttribut		-- temporäre Lösung für Rechtefilter	

				FROM planning.tfValues fV

				LEFT JOIN (
							SELECT ProductKey,TimeID, ValueText FROM planning.tfValues WHERE ValueSeriesID = 'BEM'
							) f2

							ON 
									fV.ProductKey = f2.ProductKey
								AND fV.TimeID = f2.TimeID

				LEFT JOIN planning.tdProducts dP ON 
					fV.ProductKey = dP.ProductKey

				LEFT JOIN planning.tdProductLines dPL ON 
					fV.ProductLineKey = dPL.ProductLineKey

				LEFT JOIN planning.tdFactories dF ON 
					fV.FactoryKey = dF.FactoryKey

				WHERE 
						fV.ValueSeriesID	= 'E'
					AND fV.FactoryID		<> 'ZT'

				-- Ausgabe (optional pro FactoryID) mit Userechtsunterscheidung ****************

				IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
				DROP TABLE #tmp

				SELECT
					 EV.Ursprung
					,EV.KostenstellenID
					,EV.KostenträgerID
					,EV.KontenID
					,EV.Jahr
					,EV.Monat
					,EV.Buchungsgruppe
					,EV.Event
					,EV.Eventbemerkung

				INTO #tmp
				FROM result.tfEventsPlan EV 

				-- Userrechte
				INNER JOIN 
					(	
					SELECT *
					FROM planning.trUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						FactoryID <> '' AND ProductLineID <> ''
					) vUR

				ON EV.MandantenID		= vUR.FactoryID AND 
					EV.FlexAttribut		= vUR.ProductLineID -- temporäre Lösung für Rechtefilter


				WHERE 
						EV.MandantenID = @FactoryID 
					OR	@FactoryID = ''
				

				IF @@ROWCOUNT > 0
					BEGIN
						SELECT * FROM #tmp
					END
				ELSE 
					BEGIN		
						SELECT 
							'<keiner>' AS Ursprung
							,'All'							AS KostenstellenID
							,'All'							AS KostenträgerID
							,'All'							AS KontenID
							,2019							AS Jahr
							,1								AS Monat
							,'All'							AS Buchungsgruppe
							,'<kein>'						AS Event
							,'keine Events vorhanden'		AS Eventbemerkung
					END

			SET @ResultCode = 200;

		COMMIT TRANSACTION spEventsPlanPlan;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spEventsPlanPlan;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT ::result.spEventsPlan TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spEventsPlan TO pf_PlanningFactoryService;

GO
/*
Prozedur zur Sicherung des PlanBuchungsjournals
- schreibt das BuchungsjournalPlan nach BuchungsjournalPlanSicherung
- erstellt einen Eintrag im Sicherungsindex

Gerd Tautenhahn für Saxess Software GmbH
DataFactory 4.0
04/2019


TESTCALL
EXEC result.spPOST_BuchungsjournalPlanSicherung 'SQL',20190422,'Testplan','Eine testweise Plansicherung'

*/

IF OBJECT_ID('result.spPOST_BuchungsjournalPlanSicherung') IS NOT NULL
DROP PROCEDURE result.spPOST_BuchungsjournalPlanSicherung
GO

CREATE PROCEDURE result.spPOST_BuchungsjournalPlanSicherung (
		 @Username			NVARCHAR (255)
		,@PlanVersion		BIGINT
		,@PlanName			NVARCHAR (255)	= ''
		,@PlanBeschreibung	NVARCHAR (4000) = ''
		)

AS 

BEGIN

SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username								 ,N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@PlanVersion AS NVARCHAR(255))	 ,N'NULL')	+ N''',''' 
			+ ISNULL(@PlanName								 ,N'NULL')	+ N''',''' 
			+ ISNULL(@PlanBeschreibung						 ,N'NULL')	+ N'''' 
		
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
	DECLARE @Comment			NVARCHAR (2000) = N'';	

	DECLARE @Rowcount			BIGINT;

	BEGIN TRY
		BEGIN TRANSACTION BuchungsjournalPlanSicherung


			-- Determine transaction user  ***************************
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- Check for Cluserwriterights **************************

			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

			BEGIN
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

			-- Create Index Entry, delete if already exists  **********************************
			DELETE FROM result.tfBuchungsjournalPlanSicherungsIndex  WHERE Planversion = @PlanVersion

			SELECT @Rowcount = COUNT(RowKey) FROM result.tfBuchungsjournalPlan
			SET @PlanBeschreibung = @PlanBeschreibung + ' (' + CAST(@Rowcount AS NVARCHAR(255)) + ' Buchungen)'

			INSERT INTO result.tfBuchungsjournalPlanSicherungsIndex 
				VALUES (@PlanVersion,@PlanName,@PlanBeschreibung)


			-- Copy all Values to Sicherung Table, delete if already exists ******************
			DELETE FROM result.tfBuchungsjournalPlanSicherung WHERE Planversion = @PlanVersion

			INSERT INTO result.tfBuchungsjournalPlanSicherung
				SELECT 
					  @PlanVersion
                     ,BJP.SolutionID
                     ,BJP.Ursprung
                     ,BJP.MandantenID
                     ,BJP.KostenstellenID
                     ,BJP.KostenträgerID
                     ,BJP.KontenID
                     ,BJP.Jahr
                     ,BJP.Monat
                     ,BJP.Tag
                     ,BJP.Buchungsgruppe
                     ,BJP.Buchungstext
                     ,BJP.Belegnummer
                     ,BJP.FlexAttribut
                     ,BJP.Wert
				FROM result.tfBuchungsjournalPlan BJP	
		
		COMMIT TRANSACTION BuchungsjournalPlanSicherung;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION BuchungsjournalPlanSicherung;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO 

GRANT EXECUTE ON OBJECT ::result.spPOST_BuchungsjournalPlanSicherung TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spPOST_BuchungsjournalPlanSicherung TO pf_PlanningFactoryService;
GO
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:100
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '100'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VM','Einfache Liste','','','','','','Aktiv','ListeEinfach_VM','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'1','1','Text','','','','','0','1','Input','','EingabeText','EUR','100','Text','W#150';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(10000101,0,0,'''')],[(10000201,0,0,'''')],[(10000301,0,0,'''')],[(10000401,0,0,'''')],[(10000501,0,0,'''')],[(10000601,0,0,'''')],[(10000701,0,0,'''')],[(10000801,0,0,'''')],[(10000901,0,0,'''')],[(10001001,0,0,'''')],[(10001101,0,0,'''')],[(10001201,0,0,'''')],[(10001301,0,0,'''')],[(10001401,0,0,'''')],[(10001501,0,0,'''')],[(10001601,0,0,'''')],[(10001701,0,0,'''')],[(10001801,0,0,'''')],[(10001901,0,0,'''')],[(10002001,0,0,'''')],[(10002101,0,0,'''')],[(10002201,0,0,'''')],[(10002301,0,0,'''')],[(10002401,0,0,'''')],[(10002501,0,0,'''')],[(10002601,0,0,'''')],[(10002701,0,0,'''')],[(10002801,0,0,'''')],[(10002901,0,0,'''')],[(10003001,0,0,'''')],[(10003101,0,0,'''')],[(10003201,0,0,'''')],[(10003301,0,0,'''')],[(10003401,0,0,'''')],[(10003501,0,0,'''')],[(10003601,0,0,'''')],[(10003701,0,0,'''')],[(10003801,0,0,'''')],[(10003901,0,0,'''')],[(10004001,0,0,'''')],[(10004101,0,0,'''')],[(10004201,0,0,'''')],[(10004301,0,0,'''')],[(10004401,0,0,'''')],[(10004501,0,0,'''')],[(10004601,0,0,'''')],[(10004701,0,0,'''')],[(10004801,0,0,'''')],[(10004901,0,0,'''')],[(10005001,0,0,'''')],[(10005101,0,0,'''')],[(10005201,0,0,'''')],[(10005301,0,0,'''')],[(10005401,0,0,'''')],[(10005501,0,0,'''')],[(10005601,0,0,'''')],[(10005701,0,0,'''')],[(10005801,0,0,'''')],[(10005901,0,0,'''')],[(10006001,0,0,'''')],[(10006101,0,0,'''')],[(10006201,0,0,'''')],[(10006301,0,0,'''')],[(10006401,0,0,'''')],[(10006501,0,0,'''')],[(10006601,0,0,'''')],[(10006701,0,0,'''')],[(10006801,0,0,'''')],[(10006901,0,0,'''')],[(10007001,0,0,'''')],[(10007101,0,0,'''')],[(10007201,0,0,'''')],[(10007301,0,0,'''')],[(10007401,0,0,'''')],[(10007501,0,0,'''')],[(10007601,0,0,'''')],[(10007701,0,0,'''')],[(10007801,0,0,'''')],[(10007901,0,0,'''')],[(10008001,0,0,'''')],[(10008101,0,0,'''')],[(10008201,0,0,'''')],[(10008301,0,0,'''')],[(10008401,0,0,'''')],[(10008501,0,0,'''')],[(10008601,0,0,'''')],[(10008701,0,0,'''')],[(10008801,0,0,'''')],[(10008901,0,0,'''')],[(10009001,0,0,'''')],[(10009101,0,0,'''')],[(10009201,0,0,'''')],[(10009301,0,0,'''')],[(10009401,0,0,'''')],[(10009501,0,0,'''')],[(10009601,0,0,'''')],[(10009701,0,0,'''')],[(10009801,0,0,'''')],[(10009901,0,0,'''')]';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:101
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '101'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VM','Gruppierte Liste','','','','','','Aktiv','ListeGruppiert_VM','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'1','1','Gruppe','','','','','0','1','Input','','EingabeText','EUR','100','Text','W#20';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'2','2','Text','','','','','0','1','Input','','EingabeText','EUR','100','Text','W#70';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','3','Bemerkungen','','','','','0','1','Input','','EingabeText','EUR','100','Text','W#120';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(10000101,0,0,'''')],[(10000201,0,0,'''')],[(10000301,0,0,'''')],[(10000401,0,0,'''')],[(10000501,0,0,'''')],[(10000601,0,0,'''')],[(10000701,0,0,'''')],[(10000801,0,0,'''')],[(10000901,0,0,'''')],[(10001001,0,0,'''')],[(10001101,0,0,'''')],[(10001201,0,0,'''')],[(10001301,0,0,'''')],[(10001401,0,0,'''')],[(10001501,0,0,'''')],[(10001601,0,0,'''')],[(10001701,0,0,'''')],[(10001801,0,0,'''')],[(10001901,0,0,'''')],[(10002001,0,0,'''')],[(10002101,0,0,'''')],[(10002201,0,0,'''')],[(10002301,0,0,'''')],[(10002401,0,0,'''')],[(10002501,0,0,'''')],[(10002601,0,0,'''')],[(10002701,0,0,'''')],[(10002801,0,0,'''')],[(10002901,0,0,'''')],[(10003001,0,0,'''')],[(10003101,0,0,'''')],[(10003201,0,0,'''')],[(10003301,0,0,'''')],[(10003401,0,0,'''')],[(10003501,0,0,'''')],[(10003601,0,0,'''')],[(10003701,0,0,'''')],[(10003801,0,0,'''')],[(10003901,0,0,'''')],[(10004001,0,0,'''')],[(10004101,0,0,'''')],[(10004201,0,0,'''')],[(10004301,0,0,'''')],[(10004401,0,0,'''')],[(10004501,0,0,'''')],[(10004601,0,0,'''')],[(10004701,0,0,'''')],[(10004801,0,0,'''')],[(10004901,0,0,'''')],[(10005001,0,0,'''')],[(10005101,0,0,'''')],[(10005201,0,0,'''')],[(10005301,0,0,'''')],[(10005401,0,0,'''')],[(10005501,0,0,'''')],[(10005601,0,0,'''')],[(10005701,0,0,'''')],[(10005801,0,0,'''')],[(10005901,0,0,'''')],[(10006001,0,0,'''')],[(10006101,0,0,'''')],[(10006201,0,0,'''')],[(10006301,0,0,'''')],[(10006401,0,0,'''')],[(10006501,0,0,'''')],[(10006601,0,0,'''')],[(10006701,0,0,'''')],[(10006801,0,0,'''')],[(10006901,0,0,'''')],[(10007001,0,0,'''')],[(10007101,0,0,'''')],[(10007201,0,0,'''')],[(10007301,0,0,'''')],[(10007401,0,0,'''')],[(10007501,0,0,'''')],[(10007601,0,0,'''')],[(10007701,0,0,'''')],[(10007801,0,0,'''')],[(10007901,0,0,'''')],[(10008001,0,0,'''')],[(10008101,0,0,'''')],[(10008201,0,0,'''')],[(10008301,0,0,'''')],[(10008401,0,0,'''')],[(10008501,0,0,'''')],[(10008601,0,0,'''')],[(10008701,0,0,'''')],[(10008801,0,0,'''')],[(10008901,0,0,'''')],[(10009001,0,0,'''')],[(10009101,0,0,'''')],[(10009201,0,0,'''')],[(10009301,0,0,'''')],[(10009401,0,0,'''')],[(10009501,0,0,'''')],[(10009601,0,0,'''')],[(10009701,0,0,'''')],[(10009801,0,0,'''')],[(10009901,0,0,'''')]';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:4
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '4'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VT','Unikum für 3 Tage pro Monat 2019','','','','','','Aktiv','Unikum3T_VT','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','1','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','2','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','3','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','4','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','W#30';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','5','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(20190101,0,0,'''')],[(20190115,0,0,'''')],[(20190130,0,0,''TimeBorderNext'')],[(20190201,0,0,'''')],[(20190215,0,0,'''')],[(20190228,0,0,''TimeBorderNext'')],[(20190301,0,0,'''')],[(20190315,0,0,'''')],[(20190330,0,0,''TimeBorderNext'')],[(20190401,0,0,'''')],[(20190415,0,0,'''')],[(20190430,0,0,''TimeBorderNext'')],[(20190501,0,0,'''')],[(20190515,0,0,'''')],[(20190530,0,0,''TimeBorderNext'')],[(20190601,0,0,'''')],[(20190615,0,0,'''')],[(20190630,0,0,''TimeBorderNext'')],[(20190701,0,0,'''')],[(20190715,0,0,'''')],[(20190730,0,0,''TimeBorderNext'')],[(20190801,0,0,'''')],[(20190815,0,0,'''')],[(20190830,0,0,''TimeBorderNext'')],[(20190901,0,0,'''')],[(20190915,0,0,'''')],[(20190930,0,0,''TimeBorderNext'')],[(20191001,0,0,'''')],[(20191015,0,0,'''')],[(20191030,0,0,''TimeBorderNext'')],[(20191101,0,0,'''')],[(20191115,0,0,'''')],[(20191130,0,0,''TimeBorderNext'')],[(20191201,0,0,'''')],[(20191215,0,0,'''')],[(20191230,0,0,''TimeBorderNext'')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:6
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '6'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VJ','Unikum für Jahreswerte 1970-2060','','','','','','Aktiv','Unikum_VJ','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','1','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','2','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','3','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','4','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','W#30';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','5','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(19700101,0,0,'''')],[(19710101,0,0,'''')],[(19720101,0,0,'''')],[(19730101,0,0,'''')],[(19740101,0,0,'''')],[(19750101,0,0,'''')],[(19760101,0,0,'''')],[(19770101,0,0,'''')],[(19780101,0,0,'''')],[(19790101,0,0,''TimeBorderNext'')],[(19800101,0,0,'''')],[(19810101,0,0,'''')],[(19820101,0,0,'''')],[(19830101,0,0,'''')],[(19840101,0,0,'''')],[(19850101,0,0,'''')],[(19860101,0,0,'''')],[(19870101,0,0,'''')],[(19880101,0,0,'''')],[(19890101,0,0,''TimeBorderNext'')],[(19900101,0,0,'''')],[(19910101,0,0,'''')],[(19920101,0,0,'''')],[(19930101,0,0,'''')],[(19940101,0,0,'''')],[(19950101,0,0,'''')],[(19960101,0,0,'''')],[(19970101,0,0,'''')],[(19980101,0,0,'''')],[(19990101,0,0,''TimeBorderNext'')],[(20000101,0,0,'''')],[(20010101,0,0,'''')],[(20020101,0,0,'''')],[(20030101,0,0,'''')],[(20040101,0,0,'''')],[(20050101,0,0,'''')],[(20060101,0,0,'''')],[(20070101,0,0,'''')],[(20080101,0,0,'''')],[(20090101,0,0,''TimeBorderNext'')],[(20100101,0,0,'''')],[(20110101,0,0,'''')],[(20120101,0,0,'''')],[(20130101,0,0,'''')],[(20140101,0,0,'''')],[(20150101,0,0,'''')],[(20160101,0,0,'''')],[(20170101,0,0,'''')],[(20180101,0,0,'''')],[(20190101,0,0,''TimeBorderNext'')],[(20200101,0,0,'''')],[(20210101,0,0,'''')],[(20220101,0,0,'''')],[(20230101,0,0,'''')],[(20240101,0,0,'''')],[(20250101,0,0,'''')],[(20260101,0,0,'''')],[(20270101,0,0,'''')],[(20280101,0,0,'''')],[(20290101,0,0,''TimeBorderNext'')],[(20300101,0,0,'''')],[(20310101,0,0,'''')],[(20320101,0,0,'''')],[(20330101,0,0,'''')],[(20340101,0,0,'''')],[(20350101,0,0,'''')],[(20360101,0,0,'''')],[(20370101,0,0,'''')],[(20380101,0,0,'''')],[(20390101,0,0,''TimeBorderNext'')],[(20400101,0,0,'''')],[(20410101,0,0,'''')],[(20420101,0,0,'''')],[(20430101,0,0,'''')],[(20440101,0,0,'''')],[(20450101,0,0,'''')],[(20460101,0,0,'''')],[(20470101,0,0,'''')],[(20480101,0,0,'''')],[(20490101,0,0,''TimeBorderNext'')],[(20500101,0,0,'''')],[(20510101,0,0,'''')],[(20520101,0,0,'''')],[(20530101,0,0,'''')],[(20540101,0,0,'''')],[(20550101,0,0,'''')],[(20560101,0,0,'''')],[(20570101,0,0,'''')],[(20580101,0,0,'''')],[(20590101,0,0,''TimeBorderNext'')],[(20600101,0,0,'''')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:1
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '1'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VM','Unikum für Monatswerte 2019-2023','','','','','','Aktiv','Unikum_VM','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','1','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','2','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','3','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','4','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','W#30';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','5','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(20190115,0,0,'''')],[(20190215,0,0,'''')],[(20190315,0,0,'''')],[(20190415,0,0,'''')],[(20190515,0,0,'''')],[(20190615,0,0,'''')],[(20190715,0,0,'''')],[(20190815,0,0,'''')],[(20190915,0,0,'''')],[(20191015,0,0,'''')],[(20191115,0,0,'''')],[(20191215,0,0,''TimeBorderNext'')],[(20200115,0,0,'''')],[(20200215,0,0,'''')],[(20200315,0,0,'''')],[(20200415,0,0,'''')],[(20200515,0,0,'''')],[(20200615,0,0,'''')],[(20200715,0,0,'''')],[(20200815,0,0,'''')],[(20200915,0,0,'''')],[(20201015,0,0,'''')],[(20201115,0,0,'''')],[(20201215,0,0,''TimeBorderNext'')],[(20210115,0,0,'''')],[(20210215,0,0,'''')],[(20210315,0,0,'''')],[(20210415,0,0,'''')],[(20210515,0,0,'''')],[(20210615,0,0,'''')],[(20210715,0,0,'''')],[(20210815,0,0,'''')],[(20210915,0,0,'''')],[(20211015,0,0,'''')],[(20211115,0,0,'''')],[(20211215,0,0,''TimeBorderNext'')],[(20220115,0,0,'''')],[(20220215,0,0,'''')],[(20220315,0,0,'''')],[(20220415,0,0,'''')],[(20220515,0,0,'''')],[(20220615,0,0,'''')],[(20220715,0,0,'''')],[(20220815,0,0,'''')],[(20220915,0,0,'''')],[(20221015,0,0,'''')],[(20221115,0,0,'''')],[(20221215,0,0,''TimeBorderNext'')],[(20230115,0,0,'''')],[(20230215,0,0,'''')],[(20230315,0,0,'''')],[(20230415,0,0,'''')],[(20230515,0,0,'''')],[(20230615,0,0,'''')],[(20230715,0,0,'''')],[(20230815,0,0,'''')],[(20230915,0,0,'''')],[(20231015,0,0,'''')],[(20231115,0,0,'''')],[(20231215,0,0,''TimeBorderNext'')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:2
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '2'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VT','Unikum für Tageswerte 2019','','','','','','Aktiv','Unikum_VT','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'WT','1','Wochentag','Die Tage Montag bis Sonntag sind auf der Zeitachse ausgewiesen, auch Feiertage.','','','','0','1','Input','','GrauText','Text','1','Text','W#10';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','2','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','3','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','4','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','5','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','6','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(20190101,0,0,'''')],[(20190102,0,0,'''')],[(20190103,0,0,'''')],[(20190104,0,0,'''')],[(20190105,0,0,'''')],[(20190106,0,0,'''')],[(20190107,0,0,'''')],[(20190108,0,0,'''')],[(20190109,0,0,'''')],[(20190110,0,0,'''')],[(20190111,0,0,'''')],[(20190112,0,0,'''')],[(20190113,0,0,'''')],[(20190114,0,0,'''')],[(20190115,0,0,'''')],[(20190116,0,0,'''')],[(20190117,0,0,'''')],[(20190118,0,0,'''')],[(20190119,0,0,'''')],[(20190120,0,0,'''')],[(20190121,0,0,'''')],[(20190122,0,0,'''')],[(20190123,0,0,'''')],[(20190124,0,0,'''')],[(20190125,0,0,'''')],[(20190126,0,0,'''')],[(20190127,0,0,'''')],[(20190128,0,0,'''')],[(20190129,0,0,'''')],[(20190130,0,0,'''')],[(20190131,0,0,''TimeBorderNext'')],[(20190201,0,0,'''')],[(20190202,0,0,'''')],[(20190203,0,0,'''')],[(20190204,0,0,'''')],[(20190205,0,0,'''')],[(20190206,0,0,'''')],[(20190207,0,0,'''')],[(20190208,0,0,'''')],[(20190209,0,0,'''')],[(20190210,0,0,'''')],[(20190211,0,0,'''')],[(20190212,0,0,'''')],[(20190213,0,0,'''')],[(20190214,0,0,'''')],[(20190215,0,0,'''')],[(20190216,0,0,'''')],[(20190217,0,0,'''')],[(20190218,0,0,'''')],[(20190219,0,0,'''')],[(20190220,0,0,'''')],[(20190221,0,0,'''')],[(20190222,0,0,'''')],[(20190223,0,0,'''')],[(20190224,0,0,'''')],[(20190225,0,0,'''')],[(20190226,0,0,'''')],[(20190227,0,0,'''')],[(20190228,0,0,''TimeBorderNext'')],[(20190301,0,0,'''')],[(20190302,0,0,'''')],[(20190303,0,0,'''')],[(20190304,0,0,'''')],[(20190305,0,0,'''')],[(20190306,0,0,'''')],[(20190307,0,0,'''')],[(20190308,0,0,'''')],[(20190309,0,0,'''')],[(20190310,0,0,'''')],[(20190311,0,0,'''')],[(20190312,0,0,'''')],[(20190313,0,0,'''')],[(20190314,0,0,'''')],[(20190315,0,0,'''')],[(20190316,0,0,'''')],[(20190317,0,0,'''')],[(20190318,0,0,'''')],[(20190319,0,0,'''')],[(20190320,0,0,'''')],[(20190321,0,0,'''')],[(20190322,0,0,'''')],[(20190323,0,0,'''')],[(20190324,0,0,'''')],[(20190325,0,0,'''')],[(20190326,0,0,'''')],[(20190327,0,0,'''')],[(20190328,0,0,'''')],[(20190329,0,0,'''')],[(20190330,0,0,'''')],[(20190331,0,0,''TimeBorderNext'')],[(20190401,0,0,'''')],[(20190402,0,0,'''')],[(20190403,0,0,'''')],[(20190404,0,0,'''')],[(20190405,0,0,'''')],[(20190406,0,0,'''')],[(20190407,0,0,'''')],[(20190408,0,0,'''')],[(20190409,0,0,'''')],[(20190410,0,0,'''')],[(20190411,0,0,'''')],[(20190412,0,0,'''')],[(20190413,0,0,'''')],[(20190414,0,0,'''')],[(20190415,0,0,'''')],[(20190416,0,0,'''')],[(20190417,0,0,'''')],[(20190418,0,0,'''')],[(20190419,0,0,'''')],[(20190420,0,0,'''')],[(20190421,0,0,'''')],[(20190422,0,0,'''')],[(20190423,0,0,'''')],[(20190424,0,0,'''')],[(20190425,0,0,'''')],[(20190426,0,0,'''')],[(20190427,0,0,'''')],[(20190428,0,0,'''')],[(20190429,0,0,'''')],[(20190430,0,0,''TimeBorderNext'')],[(20190501,0,0,'''')],[(20190502,0,0,'''')],[(20190503,0,0,'''')],[(20190504,0,0,'''')],[(20190505,0,0,'''')],[(20190506,0,0,'''')],[(20190507,0,0,'''')],[(20190508,0,0,'''')],[(20190509,0,0,'''')],[(20190510,0,0,'''')],[(20190511,0,0,'''')],[(20190512,0,0,'''')],[(20190513,0,0,'''')],[(20190514,0,0,'''')],[(20190515,0,0,'''')],[(20190516,0,0,'''')],[(20190517,0,0,'''')],[(20190518,0,0,'''')],[(20190519,0,0,'''')],[(20190520,0,0,'''')],[(20190521,0,0,'''')],[(20190522,0,0,'''')],[(20190523,0,0,'''')],[(20190524,0,0,'''')],[(20190525,0,0,'''')],[(20190526,0,0,'''')],[(20190527,0,0,'''')],[(20190528,0,0,'''')],[(20190529,0,0,'''')],[(20190530,0,0,'''')],[(20190531,0,0,''TimeBorderNext'')],[(20190601,0,0,'''')],[(20190602,0,0,'''')],[(20190603,0,0,'''')],[(20190604,0,0,'''')],[(20190605,0,0,'''')],[(20190606,0,0,'''')],[(20190607,0,0,'''')],[(20190608,0,0,'''')],[(20190609,0,0,'''')],[(20190610,0,0,'''')],[(20190611,0,0,'''')],[(20190612,0,0,'''')],[(20190613,0,0,'''')],[(20190614,0,0,'''')],[(20190615,0,0,'''')],[(20190616,0,0,'''')],[(20190617,0,0,'''')],[(20190618,0,0,'''')],[(20190619,0,0,'''')],[(20190620,0,0,'''')],[(20190621,0,0,'''')],[(20190622,0,0,'''')],[(20190623,0,0,'''')],[(20190624,0,0,'''')],[(20190625,0,0,'''')],[(20190626,0,0,'''')],[(20190627,0,0,'''')],[(20190628,0,0,'''')],[(20190629,0,0,'''')],[(20190630,0,0,''TimeBorderNext'')],[(20190701,0,0,'''')],[(20190702,0,0,'''')],[(20190703,0,0,'''')],[(20190704,0,0,'''')],[(20190705,0,0,'''')],[(20190706,0,0,'''')],[(20190707,0,0,'''')],[(20190708,0,0,'''')],[(20190709,0,0,'''')],[(20190710,0,0,'''')],[(20190711,0,0,'''')],[(20190712,0,0,'''')],[(20190713,0,0,'''')],[(20190714,0,0,'''')],[(20190715,0,0,'''')],[(20190716,0,0,'''')],[(20190717,0,0,'''')],[(20190718,0,0,'''')],[(20190719,0,0,'''')],[(20190720,0,0,'''')],[(20190721,0,0,'''')],[(20190722,0,0,'''')],[(20190723,0,0,'''')],[(20190724,0,0,'''')],[(20190725,0,0,'''')],[(20190726,0,0,'''')],[(20190727,0,0,'''')],[(20190728,0,0,'''')],[(20190729,0,0,'''')],[(20190730,0,0,'''')],[(20190731,0,0,''TimeBorderNext'')],[(20190801,0,0,'''')],[(20190802,0,0,'''')],[(20190803,0,0,'''')],[(20190804,0,0,'''')],[(20190805,0,0,'''')],[(20190806,0,0,'''')],[(20190807,0,0,'''')],[(20190808,0,0,'''')],[(20190809,0,0,'''')],[(20190810,0,0,'''')],[(20190811,0,0,'''')],[(20190812,0,0,'''')],[(20190813,0,0,'''')],[(20190814,0,0,'''')],[(20190815,0,0,'''')],[(20190816,0,0,'''')],[(20190817,0,0,'''')],[(20190818,0,0,'''')],[(20190819,0,0,'''')],[(20190820,0,0,'''')],[(20190821,0,0,'''')],[(20190822,0,0,'''')],[(20190823,0,0,'''')],[(20190824,0,0,'''')],[(20190825,0,0,'''')],[(20190826,0,0,'''')],[(20190827,0,0,'''')],[(20190828,0,0,'''')],[(20190829,0,0,'''')],[(20190830,0,0,'''')],[(20190831,0,0,''TimeBorderNext'')],[(20190901,0,0,'''')],[(20190902,0,0,'''')],[(20190903,0,0,'''')],[(20190904,0,0,'''')],[(20190905,0,0,'''')],[(20190906,0,0,'''')],[(20190907,0,0,'''')],[(20190908,0,0,'''')],[(20190909,0,0,'''')],[(20190910,0,0,'''')],[(20190911,0,0,'''')],[(20190912,0,0,'''')],[(20190913,0,0,'''')],[(20190914,0,0,'''')],[(20190915,0,0,'''')],[(20190916,0,0,'''')],[(20190917,0,0,'''')],[(20190918,0,0,'''')],[(20190919,0,0,'''')],[(20190920,0,0,'''')],[(20190921,0,0,'''')],[(20190922,0,0,'''')],[(20190923,0,0,'''')],[(20190924,0,0,'''')],[(20190925,0,0,'''')],[(20190926,0,0,'''')],[(20190927,0,0,'''')],[(20190928,0,0,'''')],[(20190929,0,0,'''')],[(20190930,0,0,''TimeBorderNext'')],[(20191001,0,0,'''')],[(20191002,0,0,'''')],[(20191003,0,0,'''')],[(20191004,0,0,'''')],[(20191005,0,0,'''')],[(20191006,0,0,'''')],[(20191007,0,0,'''')],[(20191008,0,0,'''')],[(20191009,0,0,'''')],[(20191010,0,0,'''')],[(20191011,0,0,'''')],[(20191012,0,0,'''')],[(20191013,0,0,'''')],[(20191014,0,0,'''')],[(20191015,0,0,'''')],[(20191016,0,0,'''')],[(20191017,0,0,'''')],[(20191018,0,0,'''')],[(20191019,0,0,'''')],[(20191020,0,0,'''')],[(20191021,0,0,'''')],[(20191022,0,0,'''')],[(20191023,0,0,'''')],[(20191024,0,0,'''')],[(20191025,0,0,'''')],[(20191026,0,0,'''')],[(20191027,0,0,'''')],[(20191028,0,0,'''')],[(20191029,0,0,'''')],[(20191030,0,0,'''')],[(20191031,0,0,''TimeBorderNext'')],[(20191101,0,0,'''')],[(20191102,0,0,'''')],[(20191103,0,0,'''')],[(20191104,0,0,'''')],[(20191105,0,0,'''')],[(20191106,0,0,'''')],[(20191107,0,0,'''')],[(20191108,0,0,'''')],[(20191109,0,0,'''')],[(20191110,0,0,'''')],[(20191111,0,0,'''')],[(20191112,0,0,'''')],[(20191113,0,0,'''')],[(20191114,0,0,'''')],[(20191115,0,0,'''')],[(20191116,0,0,'''')],[(20191117,0,0,'''')],[(20191118,0,0,'''')],[(20191119,0,0,'''')],[(20191120,0,0,'''')],[(20191121,0,0,'''')],[(20191122,0,0,'''')],[(20191123,0,0,'''')],[(20191124,0,0,'''')],[(20191125,0,0,'''')],[(20191126,0,0,'''')],[(20191127,0,0,'''')],[(20191128,0,0,'''')],[(20191129,0,0,'''')],[(20191130,0,0,''TimeBorderNext'')],[(20191201,0,0,'''')],[(20191202,0,0,'''')],[(20191203,0,0,'''')],[(20191204,0,0,'''')],[(20191205,0,0,'''')],[(20191206,0,0,'''')],[(20191207,0,0,'''')],[(20191208,0,0,'''')],[(20191209,0,0,'''')],[(20191210,0,0,'''')],[(20191211,0,0,'''')],[(20191212,0,0,'''')],[(20191213,0,0,'''')],[(20191214,0,0,'''')],[(20191215,0,0,'''')],[(20191216,0,0,'''')],[(20191217,0,0,'''')],[(20191218,0,0,'''')],[(20191219,0,0,'''')],[(20191220,0,0,'''')],[(20191221,0,0,'''')],[(20191222,0,0,'''')],[(20191223,0,0,'''')],[(20191224,0,0,'''')],[(20191225,0,0,'''')],[(20191226,0,0,'''')],[(20191227,0,0,'''')],[(20191228,0,0,'''')],[(20191229,0,0,'''')],[(20191230,0,0,'''')],[(20191231,0,0,''TimeBorderNext'')]';
EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,'[(''WT'',20190101,'''',''0'',''Dienstag'','''')],[(''WT'',20190102,'''',''0'',''Mittwoch'','''')],[(''WT'',20190103,'''',''0'',''Donnerstag'','''')],[(''WT'',20190104,'''',''0'',''Freitag'','''')],[(''WT'',20190105,'''',''0'',''Samstag'','''')],[(''WT'',20190106,'''',''0'',''Sonntag'','''')],[(''WT'',20190107,'''',''0'',''Montag'','''')],[(''WT'',20190108,'''',''0'',''Dienstag'','''')],[(''WT'',20190109,'''',''0'',''Mittwoch'','''')],[(''WT'',20190110,'''',''0'',''Donnerstag'','''')],[(''WT'',20190111,'''',''0'',''Freitag'','''')],[(''WT'',20190112,'''',''0'',''Samstag'','''')],[(''WT'',20190113,'''',''0'',''Sonntag'','''')],[(''WT'',20190114,'''',''0'',''Montag'','''')],[(''WT'',20190115,'''',''0'',''Dienstag'','''')],[(''WT'',20190116,'''',''0'',''Mittwoch'','''')],[(''WT'',20190117,'''',''0'',''Donnerstag'','''')],[(''WT'',20190118,'''',''0'',''Freitag'','''')],[(''WT'',20190119,'''',''0'',''Samstag'','''')],[(''WT'',20190120,'''',''0'',''Sonntag'','''')],[(''WT'',20190121,'''',''0'',''Montag'','''')],[(''WT'',20190122,'''',''0'',''Dienstag'','''')],[(''WT'',20190123,'''',''0'',''Mittwoch'','''')],[(''WT'',20190124,'''',''0'',''Donnerstag'','''')],[(''WT'',20190125,'''',''0'',''Freitag'','''')],[(''WT'',20190126,'''',''0'',''Samstag'','''')],[(''WT'',20190127,'''',''0'',''Sonntag'','''')],[(''WT'',20190128,'''',''0'',''Montag'','''')],[(''WT'',20190129,'''',''0'',''Dienstag'','''')],[(''WT'',20190130,'''',''0'',''Mittwoch'','''')],[(''WT'',20190131,'''',''0'',''Donnerstag'','''')],[(''WT'',20190201,'''',''0'',''Freitag'','''')],[(''WT'',20190202,'''',''0'',''Samstag'','''')],[(''WT'',20190203,'''',''0'',''Sonntag'','''')],[(''WT'',20190204,'''',''0'',''Montag'','''')],[(''WT'',20190205,'''',''0'',''Dienstag'','''')],[(''WT'',20190206,'''',''0'',''Mittwoch'','''')],[(''WT'',20190207,'''',''0'',''Donnerstag'','''')],[(''WT'',20190208,'''',''0'',''Freitag'','''')],[(''WT'',20190209,'''',''0'',''Samstag'','''')],[(''WT'',20190210,'''',''0'',''Sonntag'','''')],[(''WT'',20190211,'''',''0'',''Montag'','''')],[(''WT'',20190212,'''',''0'',''Dienstag'','''')],[(''WT'',20190213,'''',''0'',''Mittwoch'','''')],[(''WT'',20190214,'''',''0'',''Donnerstag'','''')],[(''WT'',20190215,'''',''0'',''Freitag'','''')],[(''WT'',20190216,'''',''0'',''Samstag'','''')],[(''WT'',20190217,'''',''0'',''Sonntag'','''')],[(''WT'',20190218,'''',''0'',''Montag'','''')],[(''WT'',20190219,'''',''0'',''Dienstag'','''')],[(''WT'',20190220,'''',''0'',''Mittwoch'','''')],[(''WT'',20190221,'''',''0'',''Donnerstag'','''')],[(''WT'',20190222,'''',''0'',''Freitag'','''')],[(''WT'',20190223,'''',''0'',''Samstag'','''')],[(''WT'',20190224,'''',''0'',''Sonntag'','''')],[(''WT'',20190225,'''',''0'',''Montag'','''')],[(''WT'',20190226,'''',''0'',''Dienstag'','''')],[(''WT'',20190227,'''',''0'',''Mittwoch'','''')],[(''WT'',20190228,'''',''0'',''Donnerstag'','''')],[(''WT'',20190301,'''',''0'',''Freitag'','''')],[(''WT'',20190302,'''',''0'',''Samstag'','''')],[(''WT'',20190303,'''',''0'',''Sonntag'','''')],[(''WT'',20190304,'''',''0'',''Montag'','''')],[(''WT'',20190305,'''',''0'',''Dienstag'','''')],[(''WT'',20190306,'''',''0'',''Mittwoch'','''')],[(''WT'',20190307,'''',''0'',''Donnerstag'','''')],[(''WT'',20190308,'''',''0'',''Freitag'','''')],[(''WT'',20190309,'''',''0'',''Samstag'','''')],[(''WT'',20190310,'''',''0'',''Sonntag'','''')],[(''WT'',20190311,'''',''0'',''Montag'','''')],[(''WT'',20190312,'''',''0'',''Dienstag'','''')],[(''WT'',20190313,'''',''0'',''Mittwoch'','''')],[(''WT'',20190314,'''',''0'',''Donnerstag'','''')],[(''WT'',20190315,'''',''0'',''Freitag'','''')],[(''WT'',20190316,'''',''0'',''Samstag'','''')],[(''WT'',20190317,'''',''0'',''Sonntag'','''')],[(''WT'',20190318,'''',''0'',''Montag'','''')],[(''WT'',20190319,'''',''0'',''Dienstag'','''')],[(''WT'',20190320,'''',''0'',''Mittwoch'','''')],[(''WT'',20190321,'''',''0'',''Donnerstag'','''')],[(''WT'',20190322,'''',''0'',''Freitag'','''')],[(''WT'',20190323,'''',''0'',''Samstag'','''')],[(''WT'',20190324,'''',''0'',''Sonntag'','''')],[(''WT'',20190325,'''',''0'',''Montag'','''')],[(''WT'',20190326,'''',''0'',''Dienstag'','''')],[(''WT'',20190327,'''',''0'',''Mittwoch'','''')],[(''WT'',20190328,'''',''0'',''Donnerstag'','''')],[(''WT'',20190329,'''',''0'',''Freitag'','''')],[(''WT'',20190330,'''',''0'',''Samstag'','''')],[(''WT'',20190331,'''',''0'',''Sonntag'','''')],[(''WT'',20190401,'''',''0'',''Montag'','''')],[(''WT'',20190402,'''',''0'',''Dienstag'','''')],[(''WT'',20190403,'''',''0'',''Mittwoch'','''')],[(''WT'',20190404,'''',''0'',''Donnerstag'','''')],[(''WT'',20190405,'''',''0'',''Freitag'','''')],[(''WT'',20190406,'''',''0'',''Samstag'','''')],[(''WT'',20190407,'''',''0'',''Sonntag'','''')],[(''WT'',20190408,'''',''0'',''Montag'','''')],[(''WT'',20190409,'''',''0'',''Dienstag'','''')],[(''WT'',20190410,'''',''0'',''Mittwoch'','''')],[(''WT'',20190411,'''',''0'',''Donnerstag'','''')],[(''WT'',20190412,'''',''0'',''Freitag'','''')],[(''WT'',20190413,'''',''0'',''Samstag'','''')],[(''WT'',20190414,'''',''0'',''Sonntag'','''')],[(''WT'',20190415,'''',''0'',''Montag'','''')],[(''WT'',20190416,'''',''0'',''Dienstag'','''')],[(''WT'',20190417,'''',''0'',''Mittwoch'','''')],[(''WT'',20190418,'''',''0'',''Donnerstag'','''')],[(''WT'',20190419,'''',''0'',''Freitag'','''')],[(''WT'',20190420,'''',''0'',''Samstag'','''')],[(''WT'',20190421,'''',''0'',''Sonntag'','''')],[(''WT'',20190422,'''',''0'',''Montag'','''')],[(''WT'',20190423,'''',''0'',''Dienstag'','''')],[(''WT'',20190424,'''',''0'',''Mittwoch'','''')],[(''WT'',20190425,'''',''0'',''Donnerstag'','''')],[(''WT'',20190426,'''',''0'',''Freitag'','''')],[(''WT'',20190427,'''',''0'',''Samstag'','''')],[(''WT'',20190428,'''',''0'',''Sonntag'','''')],[(''WT'',20190429,'''',''0'',''Montag'','''')],[(''WT'',20190430,'''',''0'',''Dienstag'','''')],[(''WT'',20190501,'''',''0'',''Mittwoch'','''')],[(''WT'',20190502,'''',''0'',''Donnerstag'','''')],[(''WT'',20190503,'''',''0'',''Freitag'','''')],[(''WT'',20190504,'''',''0'',''Samstag'','''')],[(''WT'',20190505,'''',''0'',''Sonntag'','''')],[(''WT'',20190506,'''',''0'',''Montag'','''')],[(''WT'',20190507,'''',''0'',''Dienstag'','''')],[(''WT'',20190508,'''',''0'',''Mittwoch'','''')],[(''WT'',20190509,'''',''0'',''Donnerstag'','''')],[(''WT'',20190510,'''',''0'',''Freitag'','''')],[(''WT'',20190511,'''',''0'',''Samstag'','''')],[(''WT'',20190512,'''',''0'',''Sonntag'','''')],[(''WT'',20190513,'''',''0'',''Montag'','''')],[(''WT'',20190514,'''',''0'',''Dienstag'','''')],[(''WT'',20190515,'''',''0'',''Mittwoch'','''')],[(''WT'',20190516,'''',''0'',''Donnerstag'','''')],[(''WT'',20190517,'''',''0'',''Freitag'','''')],[(''WT'',20190518,'''',''0'',''Samstag'','''')],[(''WT'',20190519,'''',''0'',''Sonntag'','''')],[(''WT'',20190520,'''',''0'',''Montag'','''')],[(''WT'',20190521,'''',''0'',''Dienstag'','''')],[(''WT'',20190522,'''',''0'',''Mittwoch'','''')],[(''WT'',20190523,'''',''0'',''Donnerstag'','''')],[(''WT'',20190524,'''',''0'',''Freitag'','''')],[(''WT'',20190525,'''',''0'',''Samstag'','''')],[(''WT'',20190526,'''',''0'',''Sonntag'','''')],[(''WT'',20190527,'''',''0'',''Montag'','''')],[(''WT'',20190528,'''',''0'',''Dienstag'','''')],[(''WT'',20190529,'''',''0'',''Mittwoch'','''')],[(''WT'',20190530,'''',''0'',''Donnerstag'','''')],[(''WT'',20190531,'''',''0'',''Freitag'','''')],[(''WT'',20190601,'''',''0'',''Samstag'','''')],[(''WT'',20190602,'''',''0'',''Sonntag'','''')],[(''WT'',20190603,'''',''0'',''Montag'','''')],[(''WT'',20190604,'''',''0'',''Dienstag'','''')],[(''WT'',20190605,'''',''0'',''Mittwoch'','''')],[(''WT'',20190606,'''',''0'',''Donnerstag'','''')],[(''WT'',20190607,'''',''0'',''Freitag'','''')],[(''WT'',20190608,'''',''0'',''Samstag'','''')],[(''WT'',20190609,'''',''0'',''Sonntag'','''')],[(''WT'',20190610,'''',''0'',''Montag'','''')],[(''WT'',20190611,'''',''0'',''Dienstag'','''')],[(''WT'',20190612,'''',''0'',''Mittwoch'','''')],[(''WT'',20190613,'''',''0'',''Donnerstag'','''')],[(''WT'',20190614,'''',''0'',''Freitag'','''')],[(''WT'',20190615,'''',''0'',''Samstag'','''')],[(''WT'',20190616,'''',''0'',''Sonntag'','''')],[(''WT'',20190617,'''',''0'',''Montag'','''')],[(''WT'',20190618,'''',''0'',''Dienstag'','''')],[(''WT'',20190619,'''',''0'',''Mittwoch'','''')],[(''WT'',20190620,'''',''0'',''Donnerstag'','''')],[(''WT'',20190621,'''',''0'',''Freitag'','''')],[(''WT'',20190622,'''',''0'',''Samstag'','''')],[(''WT'',20190623,'''',''0'',''Sonntag'','''')],[(''WT'',20190624,'''',''0'',''Montag'','''')],[(''WT'',20190625,'''',''0'',''Dienstag'','''')],[(''WT'',20190626,'''',''0'',''Mittwoch'','''')],[(''WT'',20190627,'''',''0'',''Donnerstag'','''')],[(''WT'',20190628,'''',''0'',''Freitag'','''')],[(''WT'',20190629,'''',''0'',''Samstag'','''')],[(''WT'',20190630,'''',''0'',''Sonntag'','''')],[(''WT'',20190701,'''',''0'',''Montag'','''')],[(''WT'',20190702,'''',''0'',''Dienstag'','''')],[(''WT'',20190703,'''',''0'',''Mittwoch'','''')],[(''WT'',20190704,'''',''0'',''Donnerstag'','''')],[(''WT'',20190705,'''',''0'',''Freitag'','''')],[(''WT'',20190706,'''',''0'',''Samstag'','''')],[(''WT'',20190707,'''',''0'',''Sonntag'','''')],[(''WT'',20190708,'''',''0'',''Montag'','''')],[(''WT'',20190709,'''',''0'',''Dienstag'','''')],[(''WT'',20190710,'''',''0'',''Mittwoch'','''')],[(''WT'',20190711,'''',''0'',''Donnerstag'','''')],[(''WT'',20190712,'''',''0'',''Freitag'','''')],[(''WT'',20190713,'''',''0'',''Samstag'','''')],[(''WT'',20190714,'''',''0'',''Sonntag'','''')],[(''WT'',20190715,'''',''0'',''Montag'','''')],[(''WT'',20190716,'''',''0'',''Dienstag'','''')],[(''WT'',20190717,'''',''0'',''Mittwoch'','''')],[(''WT'',20190718,'''',''0'',''Donnerstag'','''')],[(''WT'',20190719,'''',''0'',''Freitag'','''')],[(''WT'',20190720,'''',''0'',''Samstag'','''')],[(''WT'',20190721,'''',''0'',''Sonntag'','''')],[(''WT'',20190722,'''',''0'',''Montag'','''')],[(''WT'',20190723,'''',''0'',''Dienstag'','''')],[(''WT'',20190724,'''',''0'',''Mittwoch'','''')],[(''WT'',20190725,'''',''0'',''Donnerstag'','''')],[(''WT'',20190726,'''',''0'',''Freitag'','''')],[(''WT'',20190727,'''',''0'',''Samstag'','''')],[(''WT'',20190728,'''',''0'',''Sonntag'','''')],[(''WT'',20190729,'''',''0'',''Montag'','''')],[(''WT'',20190730,'''',''0'',''Dienstag'','''')],[(''WT'',20190731,'''',''0'',''Mittwoch'','''')],[(''WT'',20190801,'''',''0'',''Donnerstag'','''')],[(''WT'',20190802,'''',''0'',''Freitag'','''')],[(''WT'',20190803,'''',''0'',''Samstag'','''')],[(''WT'',20190804,'''',''0'',''Sonntag'','''')],[(''WT'',20190805,'''',''0'',''Montag'','''')],[(''WT'',20190806,'''',''0'',''Dienstag'','''')],[(''WT'',20190807,'''',''0'',''Mittwoch'','''')],[(''WT'',20190808,'''',''0'',''Donnerstag'','''')],[(''WT'',20190809,'''',''0'',''Freitag'','''')],[(''WT'',20190810,'''',''0'',''Samstag'','''')],[(''WT'',20190811,'''',''0'',''Sonntag'','''')],[(''WT'',20190812,'''',''0'',''Montag'','''')],[(''WT'',20190813,'''',''0'',''Dienstag'','''')],[(''WT'',20190814,'''',''0'',''Mittwoch'','''')],[(''WT'',20190815,'''',''0'',''Donnerstag'','''')],[(''WT'',20190816,'''',''0'',''Freitag'','''')],[(''WT'',20190817,'''',''0'',''Samstag'','''')],[(''WT'',20190818,'''',''0'',''Sonntag'','''')],[(''WT'',20190819,'''',''0'',''Montag'','''')],[(''WT'',20190820,'''',''0'',''Dienstag'','''')],[(''WT'',20190821,'''',''0'',''Mittwoch'','''')],[(''WT'',20190822,'''',''0'',''Donnerstag'','''')],[(''WT'',20190823,'''',''0'',''Freitag'','''')],[(''WT'',20190824,'''',''0'',''Samstag'','''')],[(''WT'',20190825,'''',''0'',''Sonntag'','''')],[(''WT'',20190826,'''',''0'',''Montag'','''')],[(''WT'',20190827,'''',''0'',''Dienstag'','''')],[(''WT'',20190828,'''',''0'',''Mittwoch'','''')],[(''WT'',20190829,'''',''0'',''Donnerstag'','''')],[(''WT'',20190830,'''',''0'',''Freitag'','''')],[(''WT'',20190831,'''',''0'',''Samstag'','''')],[(''WT'',20190901,'''',''0'',''Sonntag'','''')],[(''WT'',20190902,'''',''0'',''Montag'','''')],[(''WT'',20190903,'''',''0'',''Dienstag'','''')],[(''WT'',20190904,'''',''0'',''Mittwoch'','''')],[(''WT'',20190905,'''',''0'',''Donnerstag'','''')],[(''WT'',20190906,'''',''0'',''Freitag'','''')],[(''WT'',20190907,'''',''0'',''Samstag'','''')],[(''WT'',20190908,'''',''0'',''Sonntag'','''')],[(''WT'',20190909,'''',''0'',''Montag'','''')],[(''WT'',20190910,'''',''0'',''Dienstag'','''')],[(''WT'',20190911,'''',''0'',''Mittwoch'','''')],[(''WT'',20190912,'''',''0'',''Donnerstag'','''')],[(''WT'',20190913,'''',''0'',''Freitag'','''')],[(''WT'',20190914,'''',''0'',''Samstag'','''')],[(''WT'',20190915,'''',''0'',''Sonntag'','''')],[(''WT'',20190916,'''',''0'',''Montag'','''')],[(''WT'',20190917,'''',''0'',''Dienstag'','''')],[(''WT'',20190918,'''',''0'',''Mittwoch'','''')],[(''WT'',20190919,'''',''0'',''Donnerstag'','''')],[(''WT'',20190920,'''',''0'',''Freitag'','''')],[(''WT'',20190921,'''',''0'',''Samstag'','''')],[(''WT'',20190922,'''',''0'',''Sonntag'','''')],[(''WT'',20190923,'''',''0'',''Montag'','''')],[(''WT'',20190924,'''',''0'',''Dienstag'','''')],[(''WT'',20190925,'''',''0'',''Mittwoch'','''')],[(''WT'',20190926,'''',''0'',''Donnerstag'','''')],[(''WT'',20190927,'''',''0'',''Freitag'','''')],[(''WT'',20190928,'''',''0'',''Samstag'','''')],[(''WT'',20190929,'''',''0'',''Sonntag'','''')],[(''WT'',20190930,'''',''0'',''Montag'','''')],[(''WT'',20191001,'''',''0'',''Dienstag'','''')],[(''WT'',20191002,'''',''0'',''Mittwoch'','''')],[(''WT'',20191003,'''',''0'',''Donnerstag'','''')],[(''WT'',20191004,'''',''0'',''Freitag'','''')],[(''WT'',20191005,'''',''0'',''Samstag'','''')],[(''WT'',20191006,'''',''0'',''Sonntag'','''')],[(''WT'',20191007,'''',''0'',''Montag'','''')],[(''WT'',20191008,'''',''0'',''Dienstag'','''')],[(''WT'',20191009,'''',''0'',''Mittwoch'','''')],[(''WT'',20191010,'''',''0'',''Donnerstag'','''')],[(''WT'',20191011,'''',''0'',''Freitag'','''')],[(''WT'',20191012,'''',''0'',''Samstag'','''')],[(''WT'',20191013,'''',''0'',''Sonntag'','''')],[(''WT'',20191014,'''',''0'',''Montag'','''')],[(''WT'',20191015,'''',''0'',''Dienstag'','''')],[(''WT'',20191016,'''',''0'',''Mittwoch'','''')],[(''WT'',20191017,'''',''0'',''Donnerstag'','''')],[(''WT'',20191018,'''',''0'',''Freitag'','''')],[(''WT'',20191019,'''',''0'',''Samstag'','''')],[(''WT'',20191020,'''',''0'',''Sonntag'','''')],[(''WT'',20191021,'''',''0'',''Montag'','''')],[(''WT'',20191022,'''',''0'',''Dienstag'','''')],[(''WT'',20191023,'''',''0'',''Mittwoch'','''')],[(''WT'',20191024,'''',''0'',''Donnerstag'','''')],[(''WT'',20191025,'''',''0'',''Freitag'','''')],[(''WT'',20191026,'''',''0'',''Samstag'','''')],[(''WT'',20191027,'''',''0'',''Sonntag'','''')],[(''WT'',20191028,'''',''0'',''Montag'','''')],[(''WT'',20191029,'''',''0'',''Dienstag'','''')],[(''WT'',20191030,'''',''0'',''Mittwoch'','''')],[(''WT'',20191031,'''',''0'',''Donnerstag'','''')],[(''WT'',20191101,'''',''0'',''Freitag'','''')],[(''WT'',20191102,'''',''0'',''Samstag'','''')],[(''WT'',20191103,'''',''0'',''Sonntag'','''')],[(''WT'',20191104,'''',''0'',''Montag'','''')],[(''WT'',20191105,'''',''0'',''Dienstag'','''')],[(''WT'',20191106,'''',''0'',''Mittwoch'','''')],[(''WT'',20191107,'''',''0'',''Donnerstag'','''')],[(''WT'',20191108,'''',''0'',''Freitag'','''')],[(''WT'',20191109,'''',''0'',''Samstag'','''')],[(''WT'',20191110,'''',''0'',''Sonntag'','''')],[(''WT'',20191111,'''',''0'',''Montag'','''')],[(''WT'',20191112,'''',''0'',''Dienstag'','''')],[(''WT'',20191113,'''',''0'',''Mittwoch'','''')],[(''WT'',20191114,'''',''0'',''Donnerstag'','''')],[(''WT'',20191115,'''',''0'',''Freitag'','''')],[(''WT'',20191116,'''',''0'',''Samstag'','''')],[(''WT'',20191117,'''',''0'',''Sonntag'','''')],[(''WT'',20191118,'''',''0'',''Montag'','''')],[(''WT'',20191119,'''',''0'',''Dienstag'','''')],[(''WT'',20191120,'''',''0'',''Mittwoch'','''')],[(''WT'',20191121,'''',''0'',''Donnerstag'','''')],[(''WT'',20191122,'''',''0'',''Freitag'','''')],[(''WT'',20191123,'''',''0'',''Samstag'','''')],[(''WT'',20191124,'''',''0'',''Sonntag'','''')],[(''WT'',20191125,'''',''0'',''Montag'','''')],[(''WT'',20191126,'''',''0'',''Dienstag'','''')],[(''WT'',20191127,'''',''0'',''Mittwoch'','''')],[(''WT'',20191128,'''',''0'',''Donnerstag'','''')],[(''WT'',20191129,'''',''0'',''Freitag'','''')],[(''WT'',20191130,'''',''0'',''Samstag'','''')],[(''WT'',20191201,'''',''0'',''Sonntag'','''')],[(''WT'',20191202,'''',''0'',''Montag'','''')],[(''WT'',20191203,'''',''0'',''Dienstag'','''')],[(''WT'',20191204,'''',''0'',''Mittwoch'','''')],[(''WT'',20191205,'''',''0'',''Donnerstag'','''')],[(''WT'',20191206,'''',''0'',''Freitag'','''')],[(''WT'',20191207,'''',''0'',''Samstag'','''')],[(''WT'',20191208,'''',''0'',''Sonntag'','''')],[(''WT'',20191209,'''',''0'',''Montag'','''')],[(''WT'',20191210,'''',''0'',''Dienstag'','''')],[(''WT'',20191211,'''',''0'',''Mittwoch'','''')],[(''WT'',20191212,'''',''0'',''Donnerstag'','''')],[(''WT'',20191213,'''',''0'',''Freitag'','''')],[(''WT'',20191214,'''',''0'',''Samstag'','''')],[(''WT'',20191215,'''',''0'',''Sonntag'','''')],[(''WT'',20191216,'''',''0'',''Montag'','''')],[(''WT'',20191217,'''',''0'',''Dienstag'','''')],[(''WT'',20191218,'''',''0'',''Mittwoch'','''')],[(''WT'',20191219,'''',''0'',''Donnerstag'','''')],[(''WT'',20191220,'''',''0'',''Freitag'','''')],[(''WT'',20191221,'''',''0'',''Samstag'','''')],[(''WT'',20191222,'''',''0'',''Sonntag'','''')],[(''WT'',20191223,'''',''0'',''Montag'','''')],[(''WT'',20191224,'''',''0'',''Dienstag'','''')],[(''WT'',20191225,'''',''0'',''Mittwoch'','''')],[(''WT'',20191226,'''',''0'',''Donnerstag'','''')],[(''WT'',20191227,'''',''0'',''Freitag'','''')],[(''WT'',20191228,'''',''0'',''Samstag'','''')],[(''WT'',20191229,'''',''0'',''Sonntag'','''')],[(''WT'',20191230,'''',''0'',''Montag'','''')],[(''WT'',20191231,'''',''0'',''Dienstag'','''')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'GrauText','200,200,200','50,50,50','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:3
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '3'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VT','Unikum für Werktageswerte 2019','','','','','','Aktiv','UnikumWT_VT','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'WT','1','Wochentag','Die Tage Montag bis Freitag sind auf der Zeitachse ausgewiesen, auch Feiertage.','','','','0','1','Input','','GrauText','Text','1','Text','W#10';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','2','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','3','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','4','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','5','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','W#30';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','6','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(20190101,0,0,'''')],[(20190102,0,0,'''')],[(20190103,0,0,'''')],[(20190104,0,0,'''')],[(20190107,0,0,'''')],[(20190108,0,0,'''')],[(20190109,0,0,'''')],[(20190110,0,0,'''')],[(20190111,0,0,'''')],[(20190114,0,0,'''')],[(20190115,0,0,'''')],[(20190116,0,0,'''')],[(20190117,0,0,'''')],[(20190118,0,0,'''')],[(20190121,0,0,'''')],[(20190122,0,0,'''')],[(20190123,0,0,'''')],[(20190124,0,0,'''')],[(20190125,0,0,'''')],[(20190128,0,0,'''')],[(20190129,0,0,'''')],[(20190130,0,0,'''')],[(20190131,0,0,''TimeBorderNext'')],[(20190201,0,0,'''')],[(20190204,0,0,'''')],[(20190205,0,0,'''')],[(20190206,0,0,'''')],[(20190207,0,0,'''')],[(20190208,0,0,'''')],[(20190211,0,0,'''')],[(20190212,0,0,'''')],[(20190213,0,0,'''')],[(20190214,0,0,'''')],[(20190215,0,0,'''')],[(20190218,0,0,'''')],[(20190219,0,0,'''')],[(20190220,0,0,'''')],[(20190221,0,0,'''')],[(20190222,0,0,'''')],[(20190225,0,0,'''')],[(20190226,0,0,'''')],[(20190227,0,0,'''')],[(20190228,0,0,''TimeBorderNext'')],[(20190301,0,0,'''')],[(20190304,0,0,'''')],[(20190305,0,0,'''')],[(20190306,0,0,'''')],[(20190307,0,0,'''')],[(20190308,0,0,'''')],[(20190311,0,0,'''')],[(20190312,0,0,'''')],[(20190313,0,0,'''')],[(20190314,0,0,'''')],[(20190315,0,0,'''')],[(20190318,0,0,'''')],[(20190319,0,0,'''')],[(20190320,0,0,'''')],[(20190321,0,0,'''')],[(20190322,0,0,'''')],[(20190325,0,0,'''')],[(20190326,0,0,'''')],[(20190327,0,0,'''')],[(20190328,0,0,'''')],[(20190329,0,0,''TimeBorderNext'')],[(20190401,0,0,'''')],[(20190402,0,0,'''')],[(20190403,0,0,'''')],[(20190404,0,0,'''')],[(20190405,0,0,'''')],[(20190408,0,0,'''')],[(20190409,0,0,'''')],[(20190410,0,0,'''')],[(20190411,0,0,'''')],[(20190412,0,0,'''')],[(20190415,0,0,'''')],[(20190416,0,0,'''')],[(20190417,0,0,'''')],[(20190418,0,0,'''')],[(20190419,0,0,'''')],[(20190422,0,0,'''')],[(20190423,0,0,'''')],[(20190424,0,0,'''')],[(20190425,0,0,'''')],[(20190426,0,0,'''')],[(20190429,0,0,'''')],[(20190430,0,0,''TimeBorderNext'')],[(20190501,0,0,'''')],[(20190502,0,0,'''')],[(20190503,0,0,'''')],[(20190506,0,0,'''')],[(20190507,0,0,'''')],[(20190508,0,0,'''')],[(20190509,0,0,'''')],[(20190510,0,0,'''')],[(20190513,0,0,'''')],[(20190514,0,0,'''')],[(20190515,0,0,'''')],[(20190516,0,0,'''')],[(20190517,0,0,'''')],[(20190520,0,0,'''')],[(20190521,0,0,'''')],[(20190522,0,0,'''')],[(20190523,0,0,'''')],[(20190524,0,0,'''')],[(20190527,0,0,'''')],[(20190528,0,0,'''')],[(20190529,0,0,'''')],[(20190530,0,0,'''')],[(20190531,0,0,''TimeBorderNext'')],[(20190603,0,0,'''')],[(20190604,0,0,'''')],[(20190605,0,0,'''')],[(20190606,0,0,'''')],[(20190607,0,0,'''')],[(20190610,0,0,'''')],[(20190611,0,0,'''')],[(20190612,0,0,'''')],[(20190613,0,0,'''')],[(20190614,0,0,'''')],[(20190617,0,0,'''')],[(20190618,0,0,'''')],[(20190619,0,0,'''')],[(20190620,0,0,'''')],[(20190621,0,0,'''')],[(20190624,0,0,'''')],[(20190625,0,0,'''')],[(20190626,0,0,'''')],[(20190627,0,0,'''')],[(20190628,0,0,''TimeBorderNext'')],[(20190701,0,0,'''')],[(20190702,0,0,'''')],[(20190703,0,0,'''')],[(20190704,0,0,'''')],[(20190705,0,0,'''')],[(20190708,0,0,'''')],[(20190709,0,0,'''')],[(20190710,0,0,'''')],[(20190711,0,0,'''')],[(20190712,0,0,'''')],[(20190715,0,0,'''')],[(20190716,0,0,'''')],[(20190717,0,0,'''')],[(20190718,0,0,'''')],[(20190719,0,0,'''')],[(20190722,0,0,'''')],[(20190723,0,0,'''')],[(20190724,0,0,'''')],[(20190725,0,0,'''')],[(20190726,0,0,'''')],[(20190729,0,0,'''')],[(20190730,0,0,'''')],[(20190731,0,0,''TimeBorderNext'')],[(20190801,0,0,'''')],[(20190802,0,0,'''')],[(20190805,0,0,'''')],[(20190806,0,0,'''')],[(20190807,0,0,'''')],[(20190808,0,0,'''')],[(20190809,0,0,'''')],[(20190812,0,0,'''')],[(20190813,0,0,'''')],[(20190814,0,0,'''')],[(20190815,0,0,'''')],[(20190816,0,0,'''')],[(20190819,0,0,'''')],[(20190820,0,0,'''')],[(20190821,0,0,'''')],[(20190822,0,0,'''')],[(20190823,0,0,'''')],[(20190826,0,0,'''')],[(20190827,0,0,'''')],[(20190828,0,0,'''')],[(20190829,0,0,'''')],[(20190830,0,0,''TimeBorderNext'')],[(20190902,0,0,'''')],[(20190903,0,0,'''')],[(20190904,0,0,'''')],[(20190905,0,0,'''')],[(20190906,0,0,'''')],[(20190909,0,0,'''')],[(20190910,0,0,'''')],[(20190911,0,0,'''')],[(20190912,0,0,'''')],[(20190913,0,0,'''')],[(20190916,0,0,'''')],[(20190917,0,0,'''')],[(20190918,0,0,'''')],[(20190919,0,0,'''')],[(20190920,0,0,'''')],[(20190923,0,0,'''')],[(20190924,0,0,'''')],[(20190925,0,0,'''')],[(20190926,0,0,'''')],[(20190927,0,0,'''')],[(20190930,0,0,''TimeBorderNext'')],[(20191001,0,0,'''')],[(20191002,0,0,'''')],[(20191003,0,0,'''')],[(20191004,0,0,'''')],[(20191007,0,0,'''')],[(20191008,0,0,'''')],[(20191009,0,0,'''')],[(20191010,0,0,'''')],[(20191011,0,0,'''')],[(20191014,0,0,'''')],[(20191015,0,0,'''')],[(20191016,0,0,'''')],[(20191017,0,0,'''')],[(20191018,0,0,'''')],[(20191021,0,0,'''')],[(20191022,0,0,'''')],[(20191023,0,0,'''')],[(20191024,0,0,'''')],[(20191025,0,0,'''')],[(20191028,0,0,'''')],[(20191029,0,0,'''')],[(20191030,0,0,'''')],[(20191031,0,0,''TimeBorderNext'')],[(20191101,0,0,'''')],[(20191104,0,0,'''')],[(20191105,0,0,'''')],[(20191106,0,0,'''')],[(20191107,0,0,'''')],[(20191108,0,0,'''')],[(20191111,0,0,'''')],[(20191112,0,0,'''')],[(20191113,0,0,'''')],[(20191114,0,0,'''')],[(20191115,0,0,'''')],[(20191118,0,0,'''')],[(20191119,0,0,'''')],[(20191120,0,0,'''')],[(20191121,0,0,'''')],[(20191122,0,0,'''')],[(20191125,0,0,'''')],[(20191126,0,0,'''')],[(20191127,0,0,'''')],[(20191128,0,0,'''')],[(20191129,0,0,''TimeBorderNext'')],[(20191202,0,0,'''')],[(20191203,0,0,'''')],[(20191204,0,0,'''')],[(20191205,0,0,'''')],[(20191206,0,0,'''')],[(20191209,0,0,'''')],[(20191210,0,0,'''')],[(20191211,0,0,'''')],[(20191212,0,0,'''')],[(20191213,0,0,'''')],[(20191216,0,0,'''')],[(20191217,0,0,'''')],[(20191218,0,0,'''')],[(20191219,0,0,'''')],[(20191220,0,0,'''')],[(20191223,0,0,'''')],[(20191224,0,0,'''')],[(20191225,0,0,'''')],[(20191226,0,0,'''')],[(20191227,0,0,'''')],[(20191230,0,0,'''')],[(20191231,0,0,''TimeBorderNext'')]';
EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,'[(''WT'',20190101,'''',''0'',''Dienstag'','''')],[(''WT'',20190102,'''',''0'',''Mittwoch'','''')],[(''WT'',20190103,'''',''0'',''Donnerstag'','''')],[(''WT'',20190104,'''',''0'',''Freitag'','''')],[(''WT'',20190107,'''',''0'',''Montag'','''')],[(''WT'',20190108,'''',''0'',''Dienstag'','''')],[(''WT'',20190109,'''',''0'',''Mittwoch'','''')],[(''WT'',20190110,'''',''0'',''Donnerstag'','''')],[(''WT'',20190111,'''',''0'',''Freitag'','''')],[(''WT'',20190114,'''',''0'',''Montag'','''')],[(''WT'',20190115,'''',''0'',''Dienstag'','''')],[(''WT'',20190116,'''',''0'',''Mittwoch'','''')],[(''WT'',20190117,'''',''0'',''Donnerstag'','''')],[(''WT'',20190118,'''',''0'',''Freitag'','''')],[(''WT'',20190121,'''',''0'',''Montag'','''')],[(''WT'',20190122,'''',''0'',''Dienstag'','''')],[(''WT'',20190123,'''',''0'',''Mittwoch'','''')],[(''WT'',20190124,'''',''0'',''Donnerstag'','''')],[(''WT'',20190125,'''',''0'',''Freitag'','''')],[(''WT'',20190128,'''',''0'',''Montag'','''')],[(''WT'',20190129,'''',''0'',''Dienstag'','''')],[(''WT'',20190130,'''',''0'',''Mittwoch'','''')],[(''WT'',20190131,'''',''0'',''Donnerstag'','''')],[(''WT'',20190201,'''',''0'',''Freitag'','''')],[(''WT'',20190204,'''',''0'',''Montag'','''')],[(''WT'',20190205,'''',''0'',''Dienstag'','''')],[(''WT'',20190206,'''',''0'',''Mittwoch'','''')],[(''WT'',20190207,'''',''0'',''Donnerstag'','''')],[(''WT'',20190208,'''',''0'',''Freitag'','''')],[(''WT'',20190211,'''',''0'',''Montag'','''')],[(''WT'',20190212,'''',''0'',''Dienstag'','''')],[(''WT'',20190213,'''',''0'',''Mittwoch'','''')],[(''WT'',20190214,'''',''0'',''Donnerstag'','''')],[(''WT'',20190215,'''',''0'',''Freitag'','''')],[(''WT'',20190218,'''',''0'',''Montag'','''')],[(''WT'',20190219,'''',''0'',''Dienstag'','''')],[(''WT'',20190220,'''',''0'',''Mittwoch'','''')],[(''WT'',20190221,'''',''0'',''Donnerstag'','''')],[(''WT'',20190222,'''',''0'',''Freitag'','''')],[(''WT'',20190225,'''',''0'',''Montag'','''')],[(''WT'',20190226,'''',''0'',''Dienstag'','''')],[(''WT'',20190227,'''',''0'',''Mittwoch'','''')],[(''WT'',20190228,'''',''0'',''Donnerstag'','''')],[(''WT'',20190301,'''',''0'',''Freitag'','''')],[(''WT'',20190304,'''',''0'',''Montag'','''')],[(''WT'',20190305,'''',''0'',''Dienstag'','''')],[(''WT'',20190306,'''',''0'',''Mittwoch'','''')],[(''WT'',20190307,'''',''0'',''Donnerstag'','''')],[(''WT'',20190308,'''',''0'',''Freitag'','''')],[(''WT'',20190311,'''',''0'',''Montag'','''')],[(''WT'',20190312,'''',''0'',''Dienstag'','''')],[(''WT'',20190313,'''',''0'',''Mittwoch'','''')],[(''WT'',20190314,'''',''0'',''Donnerstag'','''')],[(''WT'',20190315,'''',''0'',''Freitag'','''')],[(''WT'',20190318,'''',''0'',''Montag'','''')],[(''WT'',20190319,'''',''0'',''Dienstag'','''')],[(''WT'',20190320,'''',''0'',''Mittwoch'','''')],[(''WT'',20190321,'''',''0'',''Donnerstag'','''')],[(''WT'',20190322,'''',''0'',''Freitag'','''')],[(''WT'',20190325,'''',''0'',''Montag'','''')],[(''WT'',20190326,'''',''0'',''Dienstag'','''')],[(''WT'',20190327,'''',''0'',''Mittwoch'','''')],[(''WT'',20190328,'''',''0'',''Donnerstag'','''')],[(''WT'',20190329,'''',''0'',''Freitag'','''')],[(''WT'',20190401,'''',''0'',''Montag'','''')],[(''WT'',20190402,'''',''0'',''Dienstag'','''')],[(''WT'',20190403,'''',''0'',''Mittwoch'','''')],[(''WT'',20190404,'''',''0'',''Donnerstag'','''')],[(''WT'',20190405,'''',''0'',''Freitag'','''')],[(''WT'',20190408,'''',''0'',''Montag'','''')],[(''WT'',20190409,'''',''0'',''Dienstag'','''')],[(''WT'',20190410,'''',''0'',''Mittwoch'','''')],[(''WT'',20190411,'''',''0'',''Donnerstag'','''')],[(''WT'',20190412,'''',''0'',''Freitag'','''')],[(''WT'',20190415,'''',''0'',''Montag'','''')],[(''WT'',20190416,'''',''0'',''Dienstag'','''')],[(''WT'',20190417,'''',''0'',''Mittwoch'','''')],[(''WT'',20190418,'''',''0'',''Donnerstag'','''')],[(''WT'',20190419,'''',''0'',''Freitag'','''')],[(''WT'',20190422,'''',''0'',''Montag'','''')],[(''WT'',20190423,'''',''0'',''Dienstag'','''')],[(''WT'',20190424,'''',''0'',''Mittwoch'','''')],[(''WT'',20190425,'''',''0'',''Donnerstag'','''')],[(''WT'',20190426,'''',''0'',''Freitag'','''')],[(''WT'',20190429,'''',''0'',''Montag'','''')],[(''WT'',20190430,'''',''0'',''Dienstag'','''')],[(''WT'',20190501,'''',''0'',''Mittwoch'','''')],[(''WT'',20190502,'''',''0'',''Donnerstag'','''')],[(''WT'',20190503,'''',''0'',''Freitag'','''')],[(''WT'',20190506,'''',''0'',''Montag'','''')],[(''WT'',20190507,'''',''0'',''Dienstag'','''')],[(''WT'',20190508,'''',''0'',''Mittwoch'','''')],[(''WT'',20190509,'''',''0'',''Donnerstag'','''')],[(''WT'',20190510,'''',''0'',''Freitag'','''')],[(''WT'',20190513,'''',''0'',''Montag'','''')],[(''WT'',20190514,'''',''0'',''Dienstag'','''')],[(''WT'',20190515,'''',''0'',''Mittwoch'','''')],[(''WT'',20190516,'''',''0'',''Donnerstag'','''')],[(''WT'',20190517,'''',''0'',''Freitag'','''')],[(''WT'',20190520,'''',''0'',''Montag'','''')],[(''WT'',20190521,'''',''0'',''Dienstag'','''')],[(''WT'',20190522,'''',''0'',''Mittwoch'','''')],[(''WT'',20190523,'''',''0'',''Donnerstag'','''')],[(''WT'',20190524,'''',''0'',''Freitag'','''')],[(''WT'',20190527,'''',''0'',''Montag'','''')],[(''WT'',20190528,'''',''0'',''Dienstag'','''')],[(''WT'',20190529,'''',''0'',''Mittwoch'','''')],[(''WT'',20190530,'''',''0'',''Donnerstag'','''')],[(''WT'',20190531,'''',''0'',''Freitag'','''')],[(''WT'',20190603,'''',''0'',''Montag'','''')],[(''WT'',20190604,'''',''0'',''Dienstag'','''')],[(''WT'',20190605,'''',''0'',''Mittwoch'','''')],[(''WT'',20190606,'''',''0'',''Donnerstag'','''')],[(''WT'',20190607,'''',''0'',''Freitag'','''')],[(''WT'',20190610,'''',''0'',''Montag'','''')],[(''WT'',20190611,'''',''0'',''Dienstag'','''')],[(''WT'',20190612,'''',''0'',''Mittwoch'','''')],[(''WT'',20190613,'''',''0'',''Donnerstag'','''')],[(''WT'',20190614,'''',''0'',''Freitag'','''')],[(''WT'',20190617,'''',''0'',''Montag'','''')],[(''WT'',20190618,'''',''0'',''Dienstag'','''')],[(''WT'',20190619,'''',''0'',''Mittwoch'','''')],[(''WT'',20190620,'''',''0'',''Donnerstag'','''')],[(''WT'',20190621,'''',''0'',''Freitag'','''')],[(''WT'',20190624,'''',''0'',''Montag'','''')],[(''WT'',20190625,'''',''0'',''Dienstag'','''')],[(''WT'',20190626,'''',''0'',''Mittwoch'','''')],[(''WT'',20190627,'''',''0'',''Donnerstag'','''')],[(''WT'',20190628,'''',''0'',''Freitag'','''')],[(''WT'',20190701,'''',''0'',''Montag'','''')],[(''WT'',20190702,'''',''0'',''Dienstag'','''')],[(''WT'',20190703,'''',''0'',''Mittwoch'','''')],[(''WT'',20190704,'''',''0'',''Donnerstag'','''')],[(''WT'',20190705,'''',''0'',''Freitag'','''')],[(''WT'',20190708,'''',''0'',''Montag'','''')],[(''WT'',20190709,'''',''0'',''Dienstag'','''')],[(''WT'',20190710,'''',''0'',''Mittwoch'','''')],[(''WT'',20190711,'''',''0'',''Donnerstag'','''')],[(''WT'',20190712,'''',''0'',''Freitag'','''')],[(''WT'',20190715,'''',''0'',''Montag'','''')],[(''WT'',20190716,'''',''0'',''Dienstag'','''')],[(''WT'',20190717,'''',''0'',''Mittwoch'','''')],[(''WT'',20190718,'''',''0'',''Donnerstag'','''')],[(''WT'',20190719,'''',''0'',''Freitag'','''')],[(''WT'',20190722,'''',''0'',''Montag'','''')],[(''WT'',20190723,'''',''0'',''Dienstag'','''')],[(''WT'',20190724,'''',''0'',''Mittwoch'','''')],[(''WT'',20190725,'''',''0'',''Donnerstag'','''')],[(''WT'',20190726,'''',''0'',''Freitag'','''')],[(''WT'',20190729,'''',''0'',''Montag'','''')],[(''WT'',20190730,'''',''0'',''Dienstag'','''')],[(''WT'',20190731,'''',''0'',''Mittwoch'','''')],[(''WT'',20190801,'''',''0'',''Donnerstag'','''')],[(''WT'',20190802,'''',''0'',''Freitag'','''')],[(''WT'',20190805,'''',''0'',''Montag'','''')],[(''WT'',20190806,'''',''0'',''Dienstag'','''')],[(''WT'',20190807,'''',''0'',''Mittwoch'','''')],[(''WT'',20190808,'''',''0'',''Donnerstag'','''')],[(''WT'',20190809,'''',''0'',''Freitag'','''')],[(''WT'',20190812,'''',''0'',''Montag'','''')],[(''WT'',20190813,'''',''0'',''Dienstag'','''')],[(''WT'',20190814,'''',''0'',''Mittwoch'','''')],[(''WT'',20190815,'''',''0'',''Donnerstag'','''')],[(''WT'',20190816,'''',''0'',''Freitag'','''')],[(''WT'',20190819,'''',''0'',''Montag'','''')],[(''WT'',20190820,'''',''0'',''Dienstag'','''')],[(''WT'',20190821,'''',''0'',''Mittwoch'','''')],[(''WT'',20190822,'''',''0'',''Donnerstag'','''')],[(''WT'',20190823,'''',''0'',''Freitag'','''')],[(''WT'',20190826,'''',''0'',''Montag'','''')],[(''WT'',20190827,'''',''0'',''Dienstag'','''')],[(''WT'',20190828,'''',''0'',''Mittwoch'','''')],[(''WT'',20190829,'''',''0'',''Donnerstag'','''')],[(''WT'',20190830,'''',''0'',''Freitag'','''')],[(''WT'',20190902,'''',''0'',''Montag'','''')],[(''WT'',20190903,'''',''0'',''Dienstag'','''')],[(''WT'',20190904,'''',''0'',''Mittwoch'','''')],[(''WT'',20190905,'''',''0'',''Donnerstag'','''')],[(''WT'',20190906,'''',''0'',''Freitag'','''')],[(''WT'',20190909,'''',''0'',''Montag'','''')],[(''WT'',20190910,'''',''0'',''Dienstag'','''')],[(''WT'',20190911,'''',''0'',''Mittwoch'','''')],[(''WT'',20190912,'''',''0'',''Donnerstag'','''')],[(''WT'',20190913,'''',''0'',''Freitag'','''')],[(''WT'',20190916,'''',''0'',''Montag'','''')],[(''WT'',20190917,'''',''0'',''Dienstag'','''')],[(''WT'',20190918,'''',''0'',''Mittwoch'','''')],[(''WT'',20190919,'''',''0'',''Donnerstag'','''')],[(''WT'',20190920,'''',''0'',''Freitag'','''')],[(''WT'',20190923,'''',''0'',''Montag'','''')],[(''WT'',20190924,'''',''0'',''Dienstag'','''')],[(''WT'',20190925,'''',''0'',''Mittwoch'','''')],[(''WT'',20190926,'''',''0'',''Donnerstag'','''')],[(''WT'',20190927,'''',''0'',''Freitag'','''')],[(''WT'',20190930,'''',''0'',''Montag'','''')],[(''WT'',20191001,'''',''0'',''Dienstag'','''')],[(''WT'',20191002,'''',''0'',''Mittwoch'','''')],[(''WT'',20191003,'''',''0'',''Donnerstag'','''')],[(''WT'',20191004,'''',''0'',''Freitag'','''')],[(''WT'',20191007,'''',''0'',''Montag'','''')],[(''WT'',20191008,'''',''0'',''Dienstag'','''')],[(''WT'',20191009,'''',''0'',''Mittwoch'','''')],[(''WT'',20191010,'''',''0'',''Donnerstag'','''')],[(''WT'',20191011,'''',''0'',''Freitag'','''')],[(''WT'',20191014,'''',''0'',''Montag'','''')],[(''WT'',20191015,'''',''0'',''Dienstag'','''')],[(''WT'',20191016,'''',''0'',''Mittwoch'','''')],[(''WT'',20191017,'''',''0'',''Donnerstag'','''')],[(''WT'',20191018,'''',''0'',''Freitag'','''')],[(''WT'',20191021,'''',''0'',''Montag'','''')],[(''WT'',20191022,'''',''0'',''Dienstag'','''')],[(''WT'',20191023,'''',''0'',''Mittwoch'','''')],[(''WT'',20191024,'''',''0'',''Donnerstag'','''')],[(''WT'',20191025,'''',''0'',''Freitag'','''')],[(''WT'',20191028,'''',''0'',''Montag'','''')],[(''WT'',20191029,'''',''0'',''Dienstag'','''')],[(''WT'',20191030,'''',''0'',''Mittwoch'','''')],[(''WT'',20191031,'''',''0'',''Donnerstag'','''')],[(''WT'',20191101,'''',''0'',''Freitag'','''')],[(''WT'',20191104,'''',''0'',''Montag'','''')],[(''WT'',20191105,'''',''0'',''Dienstag'','''')],[(''WT'',20191106,'''',''0'',''Mittwoch'','''')],[(''WT'',20191107,'''',''0'',''Donnerstag'','''')],[(''WT'',20191108,'''',''0'',''Freitag'','''')],[(''WT'',20191111,'''',''0'',''Montag'','''')],[(''WT'',20191112,'''',''0'',''Dienstag'','''')],[(''WT'',20191113,'''',''0'',''Mittwoch'','''')],[(''WT'',20191114,'''',''0'',''Donnerstag'','''')],[(''WT'',20191115,'''',''0'',''Freitag'','''')],[(''WT'',20191118,'''',''0'',''Montag'','''')],[(''WT'',20191119,'''',''0'',''Dienstag'','''')],[(''WT'',20191120,'''',''0'',''Mittwoch'','''')],[(''WT'',20191121,'''',''0'',''Donnerstag'','''')],[(''WT'',20191122,'''',''0'',''Freitag'','''')],[(''WT'',20191125,'''',''0'',''Montag'','''')],[(''WT'',20191126,'''',''0'',''Dienstag'','''')],[(''WT'',20191127,'''',''0'',''Mittwoch'','''')],[(''WT'',20191128,'''',''0'',''Donnerstag'','''')],[(''WT'',20191129,'''',''0'',''Freitag'','''')],[(''WT'',20191202,'''',''0'',''Montag'','''')],[(''WT'',20191203,'''',''0'',''Dienstag'','''')],[(''WT'',20191204,'''',''0'',''Mittwoch'','''')],[(''WT'',20191205,'''',''0'',''Donnerstag'','''')],[(''WT'',20191206,'''',''0'',''Freitag'','''')],[(''WT'',20191209,'''',''0'',''Montag'','''')],[(''WT'',20191210,'''',''0'',''Dienstag'','''')],[(''WT'',20191211,'''',''0'',''Mittwoch'','''')],[(''WT'',20191212,'''',''0'',''Donnerstag'','''')],[(''WT'',20191213,'''',''0'',''Freitag'','''')],[(''WT'',20191216,'''',''0'',''Montag'','''')],[(''WT'',20191217,'''',''0'',''Dienstag'','''')],[(''WT'',20191218,'''',''0'',''Mittwoch'','''')],[(''WT'',20191219,'''',''0'',''Donnerstag'','''')],[(''WT'',20191220,'''',''0'',''Freitag'','''')],[(''WT'',20191223,'''',''0'',''Montag'','''')],[(''WT'',20191224,'''',''0'',''Dienstag'','''')],[(''WT'',20191225,'''',''0'',''Mittwoch'','''')],[(''WT'',20191226,'''',''0'',''Donnerstag'','''')],[(''WT'',20191227,'''',''0'',''Freitag'','''')],[(''WT'',20191230,'''',''0'',''Montag'','''')],[(''WT'',20191231,'''',''0'',''Dienstag'','''')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'GrauText','200,200,200','50,50,50','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>
--<?xml version='1.0' encoding='UTF-8'?>
--This file contains a Byte Order Mark, the byte order is little-endian
--<head>
--sourcecluster:<give me a name>
--sourcetype:product
--sourcefactory:ZT
--sourceproductline:U
--sourceproduct:5
--sourceclusterapi:4.0.77
--</head>
--<code>
--CONFIG: Adjust variable names manually to fit your needs !
DECLARE @Username		NVARCHAR(255)	= 'SQL'
DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
DECLARE @ProductID		NVARCHAR(255)	= '5'
--This ProductID will be deleted during import, if it exists. You should be sure !
EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;
EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,'VW','Unikum für Wochenwerte 2019','','Die Woche wird dem Monat zugerechnet, in welchen der Mittwoch der Woche fällt.','','','','Aktiv','Unikum_VW','4.0','','','','','','','','','','','','','','','','','','','','','','','','','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'KW','1','Woche','','','','','1','1','XLS-Strict','','GrauText','Text','1','Text','W#8';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'M','2','Monat','','','','','0','1','XLS-Strict','','GrauText','EUR','100','Text','W#8';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K1','3','Kostenart 1','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K2','4','Kostenart 2','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'K3','5','Kostenart 3','','','','','1','1','Input','','EingabeDezimal2','EUR','100','Kosten=Auszahlg','';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'E','6','Event','','','','','0','1','Input','suEvents','EingabeText','Text','1','Text','W#30';
EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,'BEM','7','Bemerkung','','','','','0','1','Input','','EingabeText','Text','1','Text','W#50';
EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,'[(20190102,0,0,'''')],[(20190109,0,0,'''')],[(20190116,0,0,'''')],[(20190123,0,0,'''')],[(20190130,0,0,''TimeBorderNext'')],[(20190206,0,0,'''')],[(20190213,0,0,'''')],[(20190220,0,0,'''')],[(20190227,0,0,''TimeBorderNext'')],[(20190306,0,0,'''')],[(20190313,0,0,'''')],[(20190320,0,0,'''')],[(20190327,0,0,''TimeBorderNext'')],[(20190403,0,0,'''')],[(20190410,0,0,'''')],[(20190417,0,0,'''')],[(20190424,0,0,''TimeBorderNext'')],[(20190501,0,0,'''')],[(20190508,0,0,'''')],[(20190515,0,0,'''')],[(20190522,0,0,'''')],[(20190529,0,0,''TimeBorderNext'')],[(20190605,0,0,'''')],[(20190612,0,0,'''')],[(20190619,0,0,'''')],[(20190626,0,0,''TimeBorderNext'')],[(20190703,0,0,'''')],[(20190710,0,0,'''')],[(20190717,0,0,'''')],[(20190724,0,0,'''')],[(20190731,0,0,''TimeBorderNext'')],[(20190807,0,0,'''')],[(20190814,0,0,'''')],[(20190821,0,0,'''')],[(20190828,0,0,''TimeBorderNext'')],[(20190904,0,0,'''')],[(20190911,0,0,'''')],[(20190918,0,0,'''')],[(20190925,0,0,''TimeBorderNext'')],[(20191002,0,0,'''')],[(20191009,0,0,'''')],[(20191016,0,0,'''')],[(20191023,0,0,'''')],[(20191030,0,0,''TimeBorderNext'')],[(20191106,0,0,'''')],[(20191113,0,0,'''')],[(20191120,0,0,'''')],[(20191127,0,0,''TimeBorderNext'')],[(20191204,0,0,'''')],[(20191211,0,0,'''')],[(20191218,0,0,'''')],[(20191225,0,0,''TimeBorderNext'')]';
EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,'[(''KW'',20190102,'''',''1'','''','''')],[(''KW'',20190109,'''',''2'','''','''')],[(''KW'',20190116,'''',''3'','''','''')],[(''KW'',20190123,'''',''4'','''','''')],[(''KW'',20190130,'''',''5'','''','''')],[(''KW'',20190206,'''',''6'','''','''')],[(''KW'',20190213,'''',''7'','''','''')],[(''KW'',20190220,'''',''8'','''','''')],[(''KW'',20190227,'''',''9'','''','''')],[(''KW'',20190306,'''',''10'','''','''')],[(''KW'',20190313,'''',''11'','''','''')],[(''KW'',20190320,'''',''12'','''','''')],[(''KW'',20190327,'''',''13'','''','''')],[(''KW'',20190403,'''',''14'','''','''')],[(''KW'',20190410,'''',''15'','''','''')],[(''KW'',20190417,'''',''16'','''','''')],[(''KW'',20190424,'''',''17'','''','''')],[(''KW'',20190501,'''',''18'','''','''')],[(''KW'',20190508,'''',''19'','''','''')],[(''KW'',20190515,'''',''20'','''','''')],[(''KW'',20190522,'''',''21'','''','''')],[(''KW'',20190529,'''',''22'','''','''')],[(''KW'',20190605,'''',''23'','''','''')],[(''KW'',20190612,'''',''24'','''','''')],[(''KW'',20190619,'''',''25'','''','''')],[(''KW'',20190626,'''',''26'','''','''')],[(''KW'',20190703,'''',''27'','''','''')],[(''KW'',20190710,'''',''28'','''','''')],[(''KW'',20190717,'''',''29'','''','''')],[(''KW'',20190724,'''',''30'','''','''')],[(''KW'',20190731,'''',''31'','''','''')],[(''KW'',20190807,'''',''32'','''','''')],[(''KW'',20190814,'''',''33'','''','''')],[(''KW'',20190821,'''',''34'','''','''')],[(''KW'',20190828,'''',''35'','''','''')],[(''KW'',20190904,'''',''36'','''','''')],[(''KW'',20190911,'''',''37'','''','''')],[(''KW'',20190918,'''',''38'','''','''')],[(''KW'',20190925,'''',''39'','''','''')],[(''KW'',20191002,'''',''40'','''','''')],[(''KW'',20191009,'''',''41'','''','''')],[(''KW'',20191016,'''',''42'','''','''')],[(''KW'',20191023,'''',''43'','''','''')],[(''KW'',20191030,'''',''44'','''','''')],[(''KW'',20191106,'''',''45'','''','''')],[(''KW'',20191113,'''',''46'','''','''')],[(''KW'',20191120,'''',''47'','''','''')],[(''KW'',20191127,'''',''48'','''','''')],[(''KW'',20191204,'''',''49'','''','''')],[(''KW'',20191211,'''',''50'','''','''')],[(''KW'',20191218,'''',''51'','''','''')],[(''KW'',20191225,'''',''52'','''','''')],[(''M'',20191225,'''',''0'',''Dez'','''')],[(''M'',20191218,'''',''0'',''Dez'','''')],[(''M'',20191211,'''',''0'',''Dez'','''')],[(''M'',20191204,'''',''0'',''Dez'','''')],[(''M'',20191127,'''',''0'',''Nov'','''')],[(''M'',20191120,'''',''0'',''Nov'','''')],[(''M'',20191113,'''',''0'',''Nov'','''')],[(''M'',20191106,'''',''0'',''Nov'','''')],[(''M'',20191030,'''',''0'',''Okt'','''')],[(''M'',20191023,'''',''0'',''Okt'','''')],[(''M'',20191016,'''',''0'',''Okt'','''')],[(''M'',20191009,'''',''0'',''Okt'','''')],[(''M'',20191002,'''',''0'',''Okt'','''')],[(''M'',20190925,'''',''0'',''Sep'','''')],[(''M'',20190918,'''',''0'',''Sep'','''')],[(''M'',20190911,'''',''0'',''Sep'','''')],[(''M'',20190904,'''',''0'',''Sep'','''')],[(''M'',20190828,'''',''0'',''Aug'','''')],[(''M'',20190821,'''',''0'',''Aug'','''')],[(''M'',20190814,'''',''0'',''Aug'','''')],[(''M'',20190807,'''',''0'',''Aug'','''')],[(''M'',20190731,'''',''0'',''Jul'','''')],[(''M'',20190724,'''',''0'',''Jul'','''')],[(''M'',20190717,'''',''0'',''Jul'','''')],[(''M'',20190710,'''',''0'',''Jul'','''')],[(''M'',20190703,'''',''0'',''Jul'','''')],[(''M'',20190626,'''',''0'',''Jun'','''')],[(''M'',20190619,'''',''0'',''Jun'','''')],[(''M'',20190612,'''',''0'',''Jun'','''')],[(''M'',20190605,'''',''0'',''Jun'','''')],[(''M'',20190529,'''',''0'',''Mai'','''')],[(''M'',20190522,'''',''0'',''Mai'','''')],[(''M'',20190515,'''',''0'',''Mai'','''')],[(''M'',20190508,'''',''0'',''Mai'','''')],[(''M'',20190501,'''',''0'',''Mai'','''')],[(''M'',20190424,'''',''0'',''Apr'','''')],[(''M'',20190417,'''',''0'',''Apr'','''')],[(''M'',20190410,'''',''0'',''Apr'','''')],[(''M'',20190403,'''',''0'',''Apr'','''')],[(''M'',20190327,'''',''0'',''Mrz'','''')],[(''M'',20190320,'''',''0'',''Mrz'','''')],[(''M'',20190313,'''',''0'',''Mrz'','''')],[(''M'',20190306,'''',''0'',''Mrz'','''')],[(''M'',20190227,'''',''0'',''Feb'','''')],[(''M'',20190220,'''',''0'',''Feb'','''')],[(''M'',20190213,'''',''0'',''Feb'','''')],[(''M'',20190206,'''',''0'',''Feb'','''')],[(''M'',20190130,'''',''0'',''Jan'','''')],[(''M'',20190123,'''',''0'',''Jan'','''')],[(''M'',20190116,'''',''0'',''Jan'','''')],[(''M'',20190109,'''',''0'',''Jan'','''')],[(''M'',20190102,'''',''0'',''Jan'','''')]';
EXECUTE dbo.sx_pf_POST_List @Username,'suEvents','Eventklassifikation','','Für viele Solutions notwendig','','String','','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Info','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Prüfen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Kündigung möglich','','HintergrundRot';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende möglich','','HintergrundGelb';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'Vertragsende','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'abgeschlossen','','';
EXECUTE dbo.sx_pf_POST_ListValue @Username,0,'suEvents',0,0,'gekündigt','','';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeDezimal2','255,255,204','0,0,0','#,###0.00','-1','-1','right','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'EingabeText','255,255,222','0,0,0','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'GrauText','200,200,200','50,50,50','@','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundGelb','255,255,204','0,0,0','','-1','-1','left','center','-1','0','0';
EXEC dbo.sx_pf_POST_Format @Username,'HintergrundRot','255,96,59','0,0,0','','-1','-1','left','center','-1','0','0';
GO
--</code>

/*
Procedure to create Tabs on Factory Level
Gerd Tautenhahn for saxess-software gmbh
09/2018 for DataFactory 4.0

Pivots for System Factory in ZT
Pivots can't created here for productive Factories, as they don't exist in moment of script execution
*/


DECLARE @Username			NVARCHAR(255) = 'SQL'
DECLARE @FactoryID			NVARCHAR(255) = 'ZT'

-- Right Overview
DECLARE @TabID				NVARCHAR(255) = 'Tab01'
DECLARE @Label				NVARCHAR(255) = 'Rechteübersicht (Live)'
DECLARE @CommentUser		NVARCHAR(MAX) = 'Übersicht der Benutzerrechte'
DECLARE @PresentationType	NVARCHAR(255) = 'Pivot'	
DECLARE @Source				NVARCHAR(MAX) = 'sx_pf_GET_Info_Userrights'		
DECLARE @Layout				NVARCHAR(MAX) 

SET @Layout = 
	'
					{
		  "pivotState": {
			"fields": [
			  {
				"dataField": "UserName",
				"area": "row",
				"areaIndex": 1,
				"summaryType": "sum"
			  },
			  {
				"dataField": "Status",
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryID",
				"area": "column",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryName",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductLineID",
				"area": "column",
				"areaIndex": 1,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductlineName",
				"summaryType": "sum"
			  },
			  {
				"dataField": "UserRight",
				"summaryType": "sum"
			  },
			  {
				"dataField": "RightLevel",
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  }
			],
			"columnExpandedPaths": [],
			"rowExpandedPaths": []
		  },
		  "pivotTotals": {
			"showRowGrandTotals": false,
			"showRowTotals": false,
			"showColumnGrandTotals": false,
			"showColumnTotals": false
		  },
		  "pivotFields": [
			{
			  "dataField": "UserName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "UserName",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 1,
				"summaryType": "sum"
			  },
			  "index": 0,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "areaIndex": 1
			},
			{
			  "dataField": "Status",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "Status",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 1,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "areaIndex": 0
			},
			{
			  "dataField": "FactoryID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryID",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "column",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 2,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "column",
			  "expanded": true,
			  "areaIndex": 0
			},
			{
			  "dataField": "FactoryName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryName",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 3,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductLineID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductLineID",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "column",
				"areaIndex": 1,
				"summaryType": "sum"
			  },
			  "index": 4,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "column",
			  "areaIndex": 1
			},
			{
			  "dataField": "ProductlineName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductlineName",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 5,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "UserRight",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "UserRight",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 6,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "RightLevel",
			  "dataType": "number",
			  "displayFolder": "",
			  "caption": "RightLevel",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  },
			  "index": 7,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "format": {
				"type": "fixedPoint",
				"precision": 2
			  },
			  "area": "data",
			  "areaIndex": 0
			}
		  ],
		  "pivotVisible": true,
		  "chartVisible": false,
		  "chartPosition": "bottom",
		  "chartType": "bar",
		  "numericCellColors": {
			"positiveColors": {
			  "font": "#08c71b",
			  "background": "#08c71b"
			},
			"negativeColors": {
			  "font": "#f0111c",
			  "background": "#f0111c"
			},
			"zeroColors": {
			  "font": "#edcb09",
			  "background": "#edcb09"
			}
		  }
		}
	'

EXECUTE dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout


-- System Fehlerreport
SET @TabID				= 'Tab02'
SET @Label				= 'Systemfehler (MQT)'
SET @CommentUser		= 'Übersicht der Systemfehler beim letzten Testlauf im Datenmodell - aktualisieren durch Prozedurausführung control.spSystemFailureReport. Letzte 14 Tage des Report in Datenbank abrufbar.'
SET @PresentationType	= 'Pivot'	
SET @Source				= 'sx_pf_GET_Info_SystemFailureReport'		

SET @Layout = 
	'
				{
		  "pivotState": {
			"fields": [
			  {
				"dataField": "LastSystemCheckDate",
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryIDName",
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductLineID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductlineIDName",
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductIDName",
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  {
				"dataField": "DataFactoryPath",
				"area": "row",
				"areaIndex": 3,
				"summaryType": "sum"
			  },
			  {
				"dataField": "TestSource",
				"summaryType": "sum"
			  },
			  {
				"dataField": "TestName",
				"area": "row",
				"areaIndex": 1,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultType",
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultText",
				"area": "row",
				"areaIndex": 2,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultNumeric",
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  }
			],
			"columnExpandedPaths": [],
			"rowExpandedPaths": []
		  },
		  "pivotTotals": {
			"showRowGrandTotals": false,
			"showRowTotals": false,
			"showColumnGrandTotals": false,
			"showColumnTotals": false
		  },
		  "pivotFields": [
			{
			  "dataField": "LastSystemCheckDate",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "LastSystemCheckDate",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 0,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "FactoryID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 1,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "FactoryIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  "index": 2,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "areaIndex": -1
			},
			{
			  "dataField": "ProductLineID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductLineID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 3,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductlineIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductlineIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  "index": 4,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "areaIndex": -1
			},
			{
			  "dataField": "ProductID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 5,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"areaIndex": -1,
				"summaryType": "sum"
			  },
			  "index": 6,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "areaIndex": -1
			},
			{
			  "dataField": "DataFactoryPath",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "DataFactoryPath",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 3,
				"summaryType": "sum"
			  },
			  "index": 7,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "areaIndex": 3
			},
			{
			  "dataField": "TestSource",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "TestSource",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 8,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "TestName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "TestName",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 1,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 9,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "width": "300",
			  "areaIndex": 1
			},
			{
			  "dataField": "ResultType",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ResultType",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 10,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "areaIndex": 0
			},
			{
			  "dataField": "ResultText",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ResultText",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 2,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 11,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "width": "300",
			  "expanded": true,
			  "areaIndex": 2
			},
			{
			  "dataField": "ResultNumeric",
			  "dataType": "number",
			  "displayFolder": "",
			  "caption": "ResultNumeric",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  },
			  "index": 12,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "format": {
				"type": "fixedPoint",
				"precision": 2
			  },
			  "area": "data",
			  "areaIndex": 0
			}
		  ],
		  "pivotVisible": true,
		  "chartVisible": false,
		  "chartPosition": "bottom",
		  "chartType": "bar",
		  "numericCellColors": {
			"positiveColors": {
			  "font": "#333",
			  "background": "#f0111c"
			},
			"negativeColors": {
			  "font": "#333",
			  "background": "#08c71b"
			},
			"zeroColors": {
			  "font": "#333",
			  "background": "#edcb09"
			}
		  }
		}
	'

EXECUTE dbo.sx_pf_POST_FactoryTab      
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout;


-- Custom Fehlerreport
SET @TabID				= 'Tab03'
SET @Label				= 'Anwenderfehler (MQT)'
SET @CommentUser		= 'Übersicht der Anwenderfehler beim letzten Testlauf im Datenmodell - aktualisieren durch Prozedurausführung control.spCustomFailureReport. Letzte 14 Tage des Report in Datenbank abrufbar.'
SET @PresentationType	= 'Pivot'	
SET @Source				= 'sx_pf_GET_Info_CustomFailureReport'		

SET @Layout = 

		'
						{
		  "pivotState": {
			"fields": [
			  {
				"dataField": "LastCheckDate",
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "FactoryIDName",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductLineID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductlineIDName",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductID",
				"summaryType": "sum"
			  },
			  {
				"dataField": "ProductIDName",
				"summaryType": "sum"
			  },
			  {
				"dataField": "DataFactoryPath",
				"area": "row",
				"areaIndex": 3,
				"summaryType": "sum"
			  },
			  {
				"dataField": "TestSource",
				"summaryType": "sum"
			  },
			  {
				"dataField": "TestName",
				"area": "row",
				"areaIndex": 1,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultType",
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultText",
				"area": "row",
				"areaIndex": 2,
				"expanded": true,
				"summaryType": "sum"
			  },
			  {
				"dataField": "ResultNumeric",
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  }
			],
			"columnExpandedPaths": [],
			"rowExpandedPaths": []
		  },
		  "pivotTotals": {
			"showRowGrandTotals": false,
			"showRowTotals": false,
			"showColumnGrandTotals": false,
			"showColumnTotals": false
		  },
		  "pivotFields": [
			{
			  "dataField": "LastCheckDate",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "LastCheckDate",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 0,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "FactoryID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 1,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "FactoryIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "FactoryIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 2,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductLineID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductLineID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 3,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductlineIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductlineIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 4,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductID",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductID",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 5,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "ProductIDName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ProductIDName",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 6,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "DataFactoryPath",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "DataFactoryPath",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 3,
				"summaryType": "sum"
			  },
			  "index": 7,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "areaIndex": 3
			},
			{
			  "dataField": "TestSource",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "TestSource",
			  "summaryType": "sum",
			  "_initProperties": {
				"summaryType": "sum"
			  },
			  "index": 8,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true
			},
			{
			  "dataField": "TestName",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "TestName",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 1,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 9,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "width": "300",
			  "areaIndex": 1
			},
			{
			  "dataField": "ResultType",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ResultType",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 0,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 10,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "areaIndex": 0
			},
			{
			  "dataField": "ResultText",
			  "dataType": "string",
			  "displayFolder": "",
			  "caption": "ResultText",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "row",
				"areaIndex": 2,
				"expanded": true,
				"summaryType": "sum"
			  },
			  "index": 11,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "area": "row",
			  "expanded": true,
			  "width": "300",
			  "areaIndex": 2
			},
			{
			  "dataField": "ResultNumeric",
			  "dataType": "number",
			  "displayFolder": "",
			  "caption": "ResultNumeric",
			  "summaryType": "sum",
			  "_initProperties": {
				"area": "data",
				"areaIndex": 0,
				"summaryType": "sum"
			  },
			  "index": 12,
			  "allowSorting": true,
			  "allowSortingBySummary": true,
			  "allowFiltering": true,
			  "allowExpandAll": true,
			  "format": {
				"type": "fixedPoint",
				"precision": 2
			  },
			  "area": "data",
			  "areaIndex": 0
			}
		  ],
		  "pivotVisible": true,
		  "chartVisible": false,
		  "chartPosition": "bottom",
		  "chartType": "bar",
		  "numericCellColors": {
			"positiveColors": {
			  "font": "#333",
			  "background": "#f0111c"
			},
			"negativeColors": {
			  "font": "#333",
			  "background": "#08c71b"
			},
			"zeroColors": {
			  "font": "#333",
			  "background": "#edcb09"
			}
		  }
		}
		
		'


EXECUTE dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout;


-- HTML Static Tab
SET @TabID				= 'Tab04'
SET @Label				= 'Clusterstatistik (Live)'
SET @CommentUser		= 'Übersicht der Objekte im Cluster'
SET @PresentationType	= 'HTML_Static'	
SET @Source				= 'result.spHTML_Clusterstatistik'		

SET @Layout = ''

EXECUTE dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout;


-- HTML URL Tab			-- external pages will be loaded only if the main cluster is running an http connection (no on https)
SET @TabID				= 'Tab05'
SET @Label				= 'HTML URL (Live)'
SET @CommentUser		= 'Webseite'
SET @PresentationType	= 'HTML_URL'	
SET @Source				= 'http://www.saxess-software.de'		

SET @Layout = ''

EXECUTE dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout;


-- Pivot FiBu Saldenliste Tab
SET @TabID				= 'Tab06'
SET @Label				= 'FiBu Saldenliste'
SET @CommentUser		= 'FiBu Saldenliste aus Result generiert'
SET @PresentationType	= 'Pivot'
SET @Source				= 'result.spPivot_FiBuSaldenliste'

SET @Layout = '
	{
	 "pivotState": {
    "fields": [
      {
        "dataField": "Mandanten_ID",
        "areaIndex": -1,
        "summaryType": "sum"
      },
      {
        "dataField": "MandantenName",
        "areaIndex": -1,
        "summaryType": "sum"
      },
      {
        "dataField": "Mandant",
        "area": "filter",
        "areaIndex": 1,
        "summaryType": "sum"
      },
      {
        "dataField": "Finanzjahr",
        "area": "column",
        "areaIndex": 0,
        "expanded": false,
        "summaryType": "sum"
      },
      {
        "dataField": "Periode",
        "area": "column",
        "areaIndex": 1,
        "summaryType": "sum"
      },
      {
        "dataField": "Perioden_Key",
        "areaIndex": -1,
        "summaryType": "sum"
      },
      {
        "dataField": "Buchungskreis",
        "area": "filter",
        "areaIndex": 0,
        "filterType": "exclude",
        "filterValues": null,
        "summaryType": "sum"
      },
      {
        "dataField": "Konten_ID",
        "areaIndex": -1,
        "expanded": false,
        "summaryType": "sum"
      },
      {
        "dataField": "Kontoname",
        "areaIndex": -1,
        "expanded": false,
        "summaryType": "sum"
      },
      {
        "dataField": "KontenTyp",
        "area": "row",
        "areaIndex": 0,
        "expanded": false,
        "summaryType": "sum"
      },
      {
        "dataField": "Konto",
        "area": "row",
        "areaIndex": 1,
        "expanded": false,
        "summaryType": "sum"
      },
      {
        "dataField": "ICPartnerID",
        "sortOrder": "desc",
        "summaryType": "sum"
      },
      {
        "dataField": "Monat_Saldovortrag",
        "area": "data",
        "areaIndex": 0,
        "summaryType": "sum"
      },
      {
        "dataField": "Monat_Soll",
        "area": "data",
        "areaIndex": 1,
        "summaryType": "sum"
      },
      {
        "dataField": "Monat_Haben",
        "area": "data",
        "areaIndex": 2,
        "summaryType": "sum"
      },
      {
        "dataField": "Monat_Saldo",
        "area": "data",
        "areaIndex": 3,
        "summaryType": "sum"
      },
      {
        "dataField": "Jahr_Saldovortrag",
        "summaryType": "sum"
      },
      {
        "dataField": "Jahr_Soll",
        "summaryType": "sum"
      },
      {
        "dataField": "Jahr_Haben",
        "summaryType": "sum"
      },
      {
        "dataField": "Jahr_Saldo",
        "summaryType": "sum"
      }
    ],
    "columnExpandedPaths": [],
    "rowExpandedPaths": []
  },
  "pivotTotals": {
    "showRowGrandTotals": true,
    "showRowTotals": true,
    "showColumnGrandTotals": false,
    "showColumnTotals": false
  },
  "pivotFields": [
    {
      "dataField": "Mandanten_ID",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Mandanten_ID",
      "summaryType": "sum",
      "_initProperties": {
        "areaIndex": -1,
        "summaryType": "sum",
        "caption": "Mandanten_ID"
      },
      "index": 0,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "areaIndex": -1
    },
    {
      "dataField": "MandantenName",
      "dataType": "string",
      "displayFolder": "",
      "caption": "MandantenName",
      "summaryType": "sum",
      "_initProperties": {
        "areaIndex": -1,
        "summaryType": "sum",
        "caption": "MandantenName"
      },
      "index": 1,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "areaIndex": -1
    },
    {
      "dataField": "Mandant",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Mandant",
      "summaryType": "sum",
      "_initProperties": {
        "area": "filter",
        "areaIndex": 1,
        "summaryType": "sum",
        "caption": "Mandant"
      },
      "index": 2,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "area": "filter",
      "areaIndex": 1
    },
    {
      "dataField": "Finanzjahr",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Finanzjahr",
      "summaryType": "sum",
      "_initProperties": {
        "area": "column",
        "areaIndex": 0,
        "expanded": false,
        "summaryType": "sum",
        "caption": "Finanzjahr"
      },
      "index": 3,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#0",
      "area": "column",
      "areaIndex": 0,
      "expanded": false
    },
    {
      "dataField": "Periode",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Periode",
      "summaryType": "sum",
      "_initProperties": {
        "area": "column",
        "areaIndex": 1,
        "summaryType": "sum",
        "caption": "Periode"
      },
      "index": 4,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#0",
      "area": "column",
      "areaIndex": 1
    },
    {
      "dataField": "Perioden_Key",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Perioden_Key",
      "summaryType": "sum",
      "_initProperties": {
        "summaryType": "sum",
        "caption": "Perioden_Key"
      },
      "index": 5,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#0",
      "areaIndex": -1
    },
    {
      "dataField": "Buchungskreis",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Buchungskreis",
      "summaryType": "sum",
      "_initProperties": {
        "area": "filter",
        "areaIndex": 0,
        "filterType": "exclude",
        "filterValues": null,
        "summaryType": "sum",
        "caption": "Buchungskreis"
      },
      "index": 6,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "area": "filter",
      "areaIndex": 0,
      "filterType": "exclude",
      "filterValues": null
    },
    {
      "dataField": "Konten_ID",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Konten_ID",
      "summaryType": "sum",
      "_initProperties": {
        "areaIndex": -1,
        "expanded": false,
        "summaryType": "sum",
        "caption": "Konten_ID"
      },
      "index": 7,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "areaIndex": -1,
      "expanded": false
    },
    {
      "dataField": "Kontoname",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Kontoname",
      "summaryType": "sum",
      "_initProperties": {
        "areaIndex": -1,
        "expanded": false,
        "summaryType": "sum",
        "caption": "Kontoname"
      },
      "index": 8,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "areaIndex": -1,
      "expanded": false
    },
    {
      "dataField": "KontenTyp",
      "dataType": "string",
      "displayFolder": "",
      "caption": "KontenTyp",
      "summaryType": "sum",
      "_initProperties": {
        "area": "row",
        "areaIndex": 0,
        "expanded": false,
        "summaryType": "sum",
        "caption": "KontenTyp"
      },
      "index": 9,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "area": "row",
      "areaIndex": 0,
      "expanded": false
    },
    {
      "dataField": "Konto",
      "dataType": "string",
      "displayFolder": "",
      "caption": "Konto",
      "summaryType": "sum",
      "_initProperties": {
        "area": "row",
        "areaIndex": 1,
        "expanded": false,
        "summaryType": "sum",
        "caption": "Konto"
      },
      "index": 10,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "area": "row",
      "areaIndex": 1,
      "expanded": false
    },
    {
      "dataField": "ICPartnerID",
      "dataType": "string",
      "displayFolder": "",
      "caption": "ICPartnerID",
      "summaryType": "sum",
      "_initProperties": {
        "sortOrder": "desc",
        "summaryType": "sum",
        "caption": "ICPartnerID"
      },
      "index": 11,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "sortOrder": "desc"
    },
    {
      "dataField": "Monat_Saldovortrag",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Monat_Saldovortrag",
      "summaryType": "sum",
      "_initProperties": {
        "area": "data",
        "areaIndex": 0,
        "summaryType": "sum",
        "caption": "Monat_Saldovortrag"
      },
      "index": 12,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €",
      "area": "data",
      "areaIndex": 0
    },
    {
      "dataField": "Monat_Soll",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Monat_Soll",
      "summaryType": "sum",
      "_initProperties": {
        "area": "data",
        "areaIndex": 1,
        "summaryType": "sum",
        "caption": "Monat_Soll"
      },
      "index": 13,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €",
      "area": "data",
      "areaIndex": 1
    },
    {
      "dataField": "Monat_Haben",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Monat_Haben",
      "summaryType": "sum",
      "_initProperties": {
        "area": "data",
        "areaIndex": 2,
        "summaryType": "sum",
        "caption": "Monat_Haben"
      },
      "index": 14,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €",
      "area": "data",
      "areaIndex": 2
    },
    {
      "dataField": "Monat_Saldo",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Monat_Saldo",
      "summaryType": "sum",
      "_initProperties": {
        "area": "data",
        "areaIndex": 3,
        "summaryType": "sum",
        "caption": "Monat_Saldo"
      },
      "index": 15,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €",
      "area": "data",
      "areaIndex": 3
    },
    {
      "dataField": "Jahr_Saldovortrag",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Jahr_Saldovortrag",
      "summaryType": "sum",
      "_initProperties": {
        "summaryType": "sum",
        "caption": "Jahr_Saldovortrag"
      },
      "index": 16,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €"
    },
    {
      "dataField": "Jahr_Soll",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Jahr_Soll",
      "summaryType": "sum",
      "_initProperties": {
        "summaryType": "sum",
        "caption": "Jahr_Soll"
      },
      "index": 17,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €"
    },
    {
      "dataField": "Jahr_Haben",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Jahr_Haben",
      "summaryType": "sum",
      "_initProperties": {
        "summaryType": "sum",
        "caption": "Jahr_Haben"
      },
      "index": 18,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €"
    },
    {
      "dataField": "Jahr_Saldo",
      "dataType": "number",
      "displayFolder": "",
      "caption": "Jahr_Saldo",
      "summaryType": "sum",
      "_initProperties": {
        "summaryType": "sum",
        "caption": "Jahr_Saldo"
      },
      "index": 19,
      "allowSorting": true,
      "allowSortingBySummary": true,
      "allowFiltering": true,
      "allowExpandAll": true,
      "format": "#,##0 €"
    }
  ],
  "pivotVisible": true,
  "chartVisible": false,
  "chartPosition": "bottom",
  "chartType": "bar",
  "numericCellColors": {
    "positiveColors": {
      "font": "#333",
      "background": "#fff"
    },
    "negativeColors": {
      "font": "#333",
      "background": "#fff"
    },
    "zeroColors": {
      "font": "#333",
      "background": "#fff"
    }
  },
  "pivotLabelsAlignment": "left",
  "pivotFieldPanel": {
    "visible": true,
    "showDataFields": false,
    "showFilterFields": true,
    "showColumnFields": true,
    "showRowFields": true
  }
}'

EXECUTE dbo.sx_pf_POST_FactoryTab 
		@Username,
		@FactoryID,
		@TabID,
		@Label,
		@CommentUser,
		@PresentationType,
		@Source,
		@Layout;





/*
Gerd Tautenhahn for saxess-software gmbh
10/2018 for DataFactory 4.0

Sample Procedure to create HTML Pages based on SQL Query results

Testcall
EXEC result.spHTML_Clusterstatistik 'SQL'

*/



IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'result.spHTML_Clusterstatistik') AND type in (N'P', N'PC'))
	DROP PROCEDURE result.spHTML_Clusterstatistik;
GO

CREATE PROCEDURE result.spHTML_Clusterstatistik @Username NVARCHAR(255), @FactoryID NVARCHAR (255) = '', @ProductlineID NVARCHAR (255) = ''

AS
	BEGIN

			DECLARE @TransactUsername	NVARCHAR(255)	= N'';
			DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
			DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
					+ ISNULL(@Username,		N'NULL')	+ N''',''' 
					+ ISNULL(@FactoryID,	N'NULL')	+ N''''
					+ ISNULL(@ProductlineID,	N'NULL')+ N'''';
			DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
			DECLARE @ResultCode			INT				= 501;				-- SET during Execution
			DECLARE @TimestampCall		DATETIME		= CURRENT_TIMESTAMP;
			DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

			-- STEP 0.1 - NULL Protection
			IF @Username		IS NULL SET @Username	= N'';
			IF @FactoryID		IS NULL SET @FactoryID	= N'';
			IF @ProductlineID	IS NULL SET @FactoryID	= N'';


			DECLARE @FactoryCount		INT = 0
			DECLARE @ProductlineCount	INT = 0
			DECLARE @ProductCount		INT = 0

			SELECT @FactoryCount		= COUNT(FactoryID)		FROM planning.tdFactories		WHERE FactoryID <> 'ZT'
			SELECT @ProductlineCount	= COUNT(ProductlineID)	FROM planning.tdProductLines	WHERE FactoryID <> 'ZT'
			SELECT @ProductCount		= COUNT(ProductID)		FROM planning.tdProducts		WHERE FactoryID <> 'ZT'

						SELECT '<html>' AS Content
			UNION ALL	SELECT '<body>'
			UNION ALL	SELECT 'Dieses Cluster enthält:</br>'
			UNION ALL	SELECT CAST(@FactoryCount		AS NVARCHAR(255)) + ' Fabriken </br>'
			UNION ALL	SELECT CAST(@ProductlineCount	AS NVARCHAR(255)) + ' Produktlinien </br>'
			UNION ALL	SELECT CAST(@ProductCount		AS NVARCHAR(255)) + ' Produkte </br>'
			UNION ALL	SELECT 'außerhalb der Systemfabrik ''ZT'' </br>'
			UNION ALL	SELECT '</body>'
			UNION ALL	SELECT '</html>'
		

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	END
GO
  
GRANT EXECUTE ON OBJECT ::result.spHTML_Clusterstatistik TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::result.spHTML_Clusterstatistik TO pf_PlanningFactoryService;
GO
/*
    Product: OCT 5.0
    Object: global.tCodes
    Type: Content
    Author: saxess-software GmbH
    Script Date: 07.08.2019 17:15
*/

IF NOT EXISTS(SELECT 1 FROM global.tCodes)
BEGIN

    INSERT INTO global.tCodes (CODE, Scope, Language, CODEDescription) VALUES
    -- CODEs relating to AccountGroupCODE
     ('SKTO'				,'Dimensions'	, 'de', 'Sachkonto')
    ,('SKTO'				,'Dimensions'	, 'en', 'G/L account')
    ,('KOART'				,'Dimensions'	, 'de', 'Kostenart')
    ,('KOART'				,'Dimensions'	, 'en', 'Cost type')
    ,('PERSKTO'				,'Dimensions'	, 'de', 'Personenkonto')
    ,('PERSKTO'				,'Dimensions'	, 'en', 'Person holding account')
    
    -- CODEs relating to AccountTypeCODE
    ,('BILANZ'				,'Dimensions'	, 'de', 'Bilanz')
    ,('BILANZ'				,'Dimensions'	, 'en', 'Balance')
    ,('GUV'					,'Dimensions'	, 'de', 'GuV')
    ,('GUV'					,'Dimensions'	, 'en', 'PnL')
    ,('DEBITOR'				,'Dimensions'	, 'de', 'Debitor')
    ,('DEBITOR'				,'Dimensions'	, 'en', 'Customer')
    ,('KREDITOR'			,'Dimensions'	, 'de', 'Kreditor')
    ,('KREDITOR'			,'Dimensions'	, 'en', 'Supplier')
    ,('STAT'				,'Dimensions'	, 'de', 'Statistik')
    ,('STAT'				,'Dimensions'	, 'en', 'Statistic')

    -- CODEs relating to CostObjectGroupCODE
    ,('KST'					,'Dimensions'	, 'de', 'Kostenstelle')
    ,('KST'					,'Dimensions'	, 'en', 'Cost center')
    ,('KTR'					,'Dimensions'	, 'de', 'Kostentraeger')
    ,('KTR'					,'Dimensions'	, 'en', 'Cost unit')
    ,('PC'					,'Dimensions'	, 'de', 'Profit center')
    ,('PC'					,'Dimensions'	, 'en', 'Profit center')
    ,('OR'					,'Dimensions'	, 'de', 'Auftrag')
    ,('OR'					,'Dimensions'	, 'en', 'Order')
    ,('PROJ'				,'Dimensions'	, 'de', 'Projekt')
    ,('PROJ'				,'Dimensions'	, 'en', 'Project')

    -- CODEs relating to AccountingAreaCODE
    ,('FIBU'				,'Dimensions'	, 'de', 'Finanzbuchhaltung')
    ,('FIBU'				,'Dimensions'	, 'en', 'Financial accounting')
    ,('FIBU_SUM'			,'Dimensions'	, 'de', 'Finanzbuchhaltung Summen')
    ,('FIBU_SUM'			,'Dimensions'	, 'en', 'Financial accounting totals')
    ,('FIBU_ABSCHL_HGB'		,'Dimensions'	, 'de', 'Finanzbuchhaltung nur Abschlussbuchungen HGB')
    ,('FIBU_ABSCHL_HGB'		,'Dimensions'	, 'en', 'Financial accounting only closing entries HGB')
    ,('FIBU_ABSCHL_IFRS'	,'Dimensions'	, 'de', 'Finanzbuchhaltung nur Abschlussbuchungen IFRS')
    ,('FIBU_ABSCHL_IFRS'	,'Dimensions'	, 'en', 'Financial accounting only closing entries IFRS')
    ,('FIBU_ABSCHL_STEUER'	,'Dimensions'	, 'de', 'Finanzbuchhaltung nur Abschlussbuchungen Steuerbilanz')
    ,('FIBU_ABSCHL_STEUER'	,'Dimensions'	, 'en', 'Financial accounting only closing entries tax balance')
    ,('KORE'				,'Dimensions'	, 'de', 'Kostenrechnung')
    ,('KORE'				,'Dimensions'	, 'en', 'Cost accounting')
    ,('KORE_SUM'			,'Dimensions'	, 'de', 'Kostenrechnung Summen')
    ,('KORE_SUM'			,'Dimensions'	, 'en', 'Cost accounting totals')
    ,('PCA'					,'Dimensions'	, 'de', 'Profitcenterrechnung')
    ,('PCA'					,'Dimensions'	, 'en', 'Profit center accounting')
    ,('PCA_SUM'				,'Dimensions'	, 'de', 'Profitcenterrechnung Summen')
    ,('PCA_SUM'				,'Dimensions'	, 'en', 'Profit center accounting totals')
    ,('PCAILV'				,'Dimensions'	, 'de', 'Profitcenterrechnung - interne Leistungsverrechnung (ILV)')
    ,('PCAILV'				,'Dimensions'	, 'en', 'Profit center accounting - internal cost allocation')

    -- CODEs for DataTypes
	,('TEXT'				,'DataType'		, 'en', 'DataType in multitype tables for Text')
	,('DATE'				,'DataType'		, 'en', 'DataType in multitype tables for DataValues')
	,('DATETIME'			,'DataType'		, 'en', 'DataType in multitype tables for Datetime Values')
	,('BOOLEAN'				,'DataType'		, 'en', 'DataType in multitype tables for Boolean Values')
	,('NUMBER'				,'DataType'		, 'en', 'DataType in multitype tables for Numbers')
	,('JSON'				,'DataType'		, 'en', 'DataType in multitype tables for JSON')

END;
GO
/*
    Product: OCT 5.0
    Object: global.tFinChartsOfAccounts
    Type: Content
    Author: saxess-software GmbH
    Script Date: 18.07.2019 14:38
*/

IF NOT EXISTS(SELECT 1 FROM global.tFinChartsOfAccounts)
BEGIN
WAITFOR DELAY '00:00'
END;
GO

/*
    Product: OCT 5.0
    Object: global.tPeriods
    Type: Content
    Author: saxess-software GmbH
    Script Date: 18.07.2019 14:38
*/

IF NOT EXISTS(SELECT 1 FROM global.tPeriods)
BEGIN

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('1993', '5', '8', '1993/1994', '1993', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '8', '1993/1994', '1994', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '8', '1993/1994', '1994', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '8', '1993/1994', '1994', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '8', '1993/1994', '1994', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '8', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '8', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '8', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '8', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '8', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '8', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '8', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '9', '1993/1994', '1993', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '9', '1993/1994', '1993', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '9', '1993/1994', '1993', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '9', '1993/1994', '1993', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '9', '1993/1994', '1994', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '9', '1993/1994', '1994', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '9', '1993/1994', '1994', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '9', '1993/1994', '1994', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '9', '1993/1994', '1994', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '9', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '9', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '9', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '9', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '9', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '9', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '9', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '10', '1993/1994', '1993', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '10', '1993/1994', '1993', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '10', '1993/1994', '1993', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '10', '1993/1994', '1994', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '10', '1993/1994', '1994', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '10', '1993/1994', '1994', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '10', '1993/1994', '1994', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '10', '1993/1994', '1994', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '10', '1993/1994', '1994', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '10', '1993/1994', '1994', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '10', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '10', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '10', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '10', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '10', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '10', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '11', '1993/1994', '1993', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '11', '1993/1994', '1993', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '11', '1993/1994', '1994', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '11', '1993/1994', '1994', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '11', '1993/1994', '1994', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '11', '1993/1994', '1994', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '11', '1993/1994', '1994', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '11', '1993/1994', '1994', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '11', '1993/1994', '1994', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '11', '1993/1994', '1994', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '11', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '11', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '11', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '11', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '11', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '11', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '12', '1993/1994', '1993', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '12', '1993/1994', '1994', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '12', '1993/1994', '1994', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '12', '1993/1994', '1994', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '12', '1993/1994', '1994', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '12', '1993/1994', '1994', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '12', '1993/1994', '1994', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '12', '1993/1994', '1994', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '12', '1993/1994', '1994', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '12', '1993/1994', '1994', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '12', '1993/1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '12', '1993/1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '12', '1993/1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '12', '1993/1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '12', '1993/1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '12', '1993/1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '1', '1994', '1994', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '1', '1994', '1994', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '1', '1994', '1994', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '1', '1994', '1994', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '1', '1994', '1994', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '1', '1994', '1994', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '1', '1994', '1994', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '1', '1994', '1994', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '1', '1994', '1994', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '1', '1994', '1994', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '1', '1994', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '1', '1994', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '1', '1994', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '1', '1994', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '1', '1994', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '1', '1994', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '2', '1994/1995', '1994', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '2', '1994/1995', '1994', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '2', '1994/1995', '1994', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '2', '1994/1995', '1994', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '2', '1994/1995', '1994', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '2', '1994/1995', '1994', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '2', '1994/1995', '1994', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '2', '1994/1995', '1994', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '2', '1994/1995', '1994', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '2', '1994/1995', '1994', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '2', '1994/1995', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '2', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '2', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '2', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '2', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '2', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '3', '1994/1995', '1994', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '3', '1994/1995', '1994', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '3', '1994/1995', '1994', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '3', '1994/1995', '1994', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '3', '1994/1995', '1994', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '3', '1994/1995', '1994', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '3', '1994/1995', '1994', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '3', '1994/1995', '1994', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '3', '1994/1995', '1994', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '3', '1994/1995', '1994', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '3', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '3', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '3', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '3', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '3', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '3', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '4', '1994/1995', '1994', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '4', '1994/1995', '1994', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '4', '1994/1995', '1994', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '4', '1994/1995', '1994', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '4', '1994/1995', '1994', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '4', '1994/1995', '1994', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '4', '1994/1995', '1994', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '4', '1994/1995', '1994', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '4', '1994/1995', '1994', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '4', '1994/1995', '1995', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '4', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '4', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '4', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '4', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '4', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '4', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '5', '1994/1995', '1994', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '5', '1994/1995', '1994', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '5', '1994/1995', '1994', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '5', '1994/1995', '1994', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '5', '1994/1995', '1994', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '5', '1994/1995', '1994', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '5', '1994/1995', '1994', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '5', '1994/1995', '1994', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '5', '1994/1995', '1995', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '5', '1994/1995', '1995', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '5', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '5', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '5', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '5', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '5', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '5', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '6', '1994/1995', '1994', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '6', '1994/1995', '1994', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '6', '1994/1995', '1994', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '6', '1994/1995', '1994', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '6', '1994/1995', '1994', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '6', '1994/1995', '1994', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '6', '1994/1995', '1994', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '6', '1994/1995', '1995', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '6', '1994/1995', '1995', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '6', '1994/1995', '1995', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '6', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '6', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '6', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '6', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '6', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '6', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '7', '1994/1995', '1994', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '7', '1994/1995', '1994', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '7', '1994/1995', '1994', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '7', '1994/1995', '1994', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '7', '1994/1995', '1994', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '7', '1994/1995', '1994', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '7', '1994/1995', '1995', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '7', '1994/1995', '1995', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '7', '1994/1995', '1995', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '7', '1994/1995', '1995', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '7', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '7', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '7', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '7', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '7', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '7', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '8', '1994/1995', '1994', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '8', '1994/1995', '1994', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '8', '1994/1995', '1994', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '8', '1994/1995', '1994', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '8', '1994/1995', '1994', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '8', '1994/1995', '1995', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '8', '1994/1995', '1995', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '8', '1994/1995', '1995', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '8', '1994/1995', '1995', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '8', '1994/1995', '1995', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '8', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '8', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '8', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '8', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '8', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '8', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '9', '1994/1995', '1994', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '9', '1994/1995', '1994', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '9', '1994/1995', '1994', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '9', '1994/1995', '1994', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '9', '1994/1995', '1995', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '9', '1994/1995', '1995', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '9', '1994/1995', '1995', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '9', '1994/1995', '1995', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '9', '1994/1995', '1995', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '9', '1994/1995', '1995', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '9', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '9', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '9', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '9', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '9', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '9', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '10', '1994/1995', '1994', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '10', '1994/1995', '1994', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '10', '1994/1995', '1994', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '10', '1994/1995', '1995', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '10', '1994/1995', '1995', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '10', '1994/1995', '1995', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '10', '1994/1995', '1995', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '10', '1994/1995', '1995', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '10', '1994/1995', '1995', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '10', '1994/1995', '1995', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '10', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '10', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '10', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '10', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '10', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '10', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '11', '1994/1995', '1994', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '11', '1994/1995', '1994', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '11', '1994/1995', '1995', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '11', '1994/1995', '1995', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '11', '1994/1995', '1995', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '11', '1994/1995', '1995', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '11', '1994/1995', '1995', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '11', '1994/1995', '1995', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '11', '1994/1995', '1995', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '11', '1994/1995', '1995', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '11', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '11', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '11', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '11', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '11', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '11', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '1', '12', '1994/1995', '1994', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '2', '12', '1994/1995', '1995', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '3', '12', '1994/1995', '1995', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1994', '4', '12', '1994/1995', '1995', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '5', '12', '1994/1995', '1995', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '6', '12', '1994/1995', '1995', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1994', '7', '12', '1994/1995', '1995', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '8', '12', '1994/1995', '1995', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '9', '12', '1994/1995', '1995', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1994', '10', '12', '1994/1995', '1995', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '11', '12', '1994/1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '12', '12', '1994/1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '13', '12', '1994/1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '14', '12', '1994/1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '15', '12', '1994/1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1994', '16', '12', '1994/1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '1', '1995', '1995', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '1', '1995', '1995', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '1', '1995', '1995', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '1', '1995', '1995', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '1', '1995', '1995', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '1', '1995', '1995', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '1', '1995', '1995', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '1', '1995', '1995', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '1', '1995', '1995', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '1', '1995', '1995', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '1', '1995', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '1', '1995', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '1', '1995', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '1', '1995', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '1', '1995', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '1', '1995', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '2', '1995/1996', '1995', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '2', '1995/1996', '1995', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '2', '1995/1996', '1995', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '2', '1995/1996', '1995', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '2', '1995/1996', '1995', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '2', '1995/1996', '1995', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '2', '1995/1996', '1995', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '2', '1995/1996', '1995', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '2', '1995/1996', '1995', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '2', '1995/1996', '1995', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '2', '1995/1996', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '2', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '2', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '2', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '2', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '2', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '3', '1995/1996', '1995', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '3', '1995/1996', '1995', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '3', '1995/1996', '1995', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '3', '1995/1996', '1995', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '3', '1995/1996', '1995', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '3', '1995/1996', '1995', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '3', '1995/1996', '1995', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '3', '1995/1996', '1995', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '3', '1995/1996', '1995', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '3', '1995/1996', '1995', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '3', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '3', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '3', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '3', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '3', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '3', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '4', '1995/1996', '1995', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '4', '1995/1996', '1995', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '4', '1995/1996', '1995', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '4', '1995/1996', '1995', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '4', '1995/1996', '1995', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '4', '1995/1996', '1995', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '4', '1995/1996', '1995', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '4', '1995/1996', '1995', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '4', '1995/1996', '1995', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '4', '1995/1996', '1996', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '4', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '4', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '4', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '4', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '4', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '4', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '5', '1995/1996', '1995', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '5', '1995/1996', '1995', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '5', '1995/1996', '1995', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '5', '1995/1996', '1995', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '5', '1995/1996', '1995', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '5', '1995/1996', '1995', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '5', '1995/1996', '1995', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '5', '1995/1996', '1995', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '5', '1995/1996', '1996', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '5', '1995/1996', '1996', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '5', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '5', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '5', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '5', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '5', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '5', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '6', '1995/1996', '1995', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '6', '1995/1996', '1995', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '6', '1995/1996', '1995', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '6', '1995/1996', '1995', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '6', '1995/1996', '1995', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '6', '1995/1996', '1995', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '6', '1995/1996', '1995', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '6', '1995/1996', '1996', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '6', '1995/1996', '1996', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '6', '1995/1996', '1996', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '6', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '6', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '6', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '6', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '6', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '6', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '7', '1995/1996', '1995', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '7', '1995/1996', '1995', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '7', '1995/1996', '1995', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '7', '1995/1996', '1995', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '7', '1995/1996', '1995', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '7', '1995/1996', '1995', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '7', '1995/1996', '1996', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '7', '1995/1996', '1996', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '7', '1995/1996', '1996', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '7', '1995/1996', '1996', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '7', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '7', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '7', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '7', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '7', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '7', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '8', '1995/1996', '1995', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '8', '1995/1996', '1995', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '8', '1995/1996', '1995', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '8', '1995/1996', '1995', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '8', '1995/1996', '1995', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '8', '1995/1996', '1996', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '8', '1995/1996', '1996', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '8', '1995/1996', '1996', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '8', '1995/1996', '1996', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '8', '1995/1996', '1996', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '8', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '8', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '8', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '8', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '8', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '8', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '9', '1995/1996', '1995', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '9', '1995/1996', '1995', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '9', '1995/1996', '1995', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '9', '1995/1996', '1995', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '9', '1995/1996', '1996', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '9', '1995/1996', '1996', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '9', '1995/1996', '1996', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '9', '1995/1996', '1996', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '9', '1995/1996', '1996', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '9', '1995/1996', '1996', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '9', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '9', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '9', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '9', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '9', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '9', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '10', '1995/1996', '1995', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '10', '1995/1996', '1995', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '10', '1995/1996', '1995', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '10', '1995/1996', '1996', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '10', '1995/1996', '1996', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '10', '1995/1996', '1996', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '10', '1995/1996', '1996', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '10', '1995/1996', '1996', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '10', '1995/1996', '1996', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '10', '1995/1996', '1996', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '10', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '10', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '10', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '10', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '10', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '10', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '11', '1995/1996', '1995', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '11', '1995/1996', '1995', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '11', '1995/1996', '1996', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '11', '1995/1996', '1996', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '11', '1995/1996', '1996', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '11', '1995/1996', '1996', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '11', '1995/1996', '1996', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '11', '1995/1996', '1996', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '11', '1995/1996', '1996', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '11', '1995/1996', '1996', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '11', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '11', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '11', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '11', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '11', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '11', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '1', '12', '1995/1996', '1995', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '2', '12', '1995/1996', '1996', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '3', '12', '1995/1996', '1996', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1995', '4', '12', '1995/1996', '1996', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '5', '12', '1995/1996', '1996', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '6', '12', '1995/1996', '1996', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1995', '7', '12', '1995/1996', '1996', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '8', '12', '1995/1996', '1996', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '9', '12', '1995/1996', '1996', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1995', '10', '12', '1995/1996', '1996', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '11', '12', '1995/1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '12', '12', '1995/1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '13', '12', '1995/1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '14', '12', '1995/1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '15', '12', '1995/1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1995', '16', '12', '1995/1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '1', '1996', '1996', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '1', '1996', '1996', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '1', '1996', '1996', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '1', '1996', '1996', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '1', '1996', '1996', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '1', '1996', '1996', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '1', '1996', '1996', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '1', '1996', '1996', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '1', '1996', '1996', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '1', '1996', '1996', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '1', '1996', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '1', '1996', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '1', '1996', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '1', '1996', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '1', '1996', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '1', '1996', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '2', '1996/1997', '1996', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '2', '1996/1997', '1996', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '2', '1996/1997', '1996', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '2', '1996/1997', '1996', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '2', '1996/1997', '1996', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '2', '1996/1997', '1996', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '2', '1996/1997', '1996', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '2', '1996/1997', '1996', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '2', '1996/1997', '1996', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '2', '1996/1997', '1996', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '2', '1996/1997', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '2', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '2', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '2', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '2', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '2', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '3', '1996/1997', '1996', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '3', '1996/1997', '1996', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '3', '1996/1997', '1996', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '3', '1996/1997', '1996', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '3', '1996/1997', '1996', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '3', '1996/1997', '1996', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '3', '1996/1997', '1996', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '3', '1996/1997', '1996', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '3', '1996/1997', '1996', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '3', '1996/1997', '1996', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '3', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '3', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '3', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '3', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '3', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '3', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '4', '1996/1997', '1996', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '4', '1996/1997', '1996', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '4', '1996/1997', '1996', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '4', '1996/1997', '1996', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '4', '1996/1997', '1996', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '4', '1996/1997', '1996', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '4', '1996/1997', '1996', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '4', '1996/1997', '1996', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '4', '1996/1997', '1996', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '4', '1996/1997', '1997', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '4', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '4', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '4', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '4', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '4', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '4', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '5', '1996/1997', '1996', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '5', '1996/1997', '1996', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '5', '1996/1997', '1996', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '5', '1996/1997', '1996', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '5', '1996/1997', '1996', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '5', '1996/1997', '1996', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '5', '1996/1997', '1996', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '5', '1996/1997', '1996', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '5', '1996/1997', '1997', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '5', '1996/1997', '1997', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '5', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '5', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '5', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '5', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '5', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '5', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '6', '1996/1997', '1996', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '6', '1996/1997', '1996', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '6', '1996/1997', '1996', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '6', '1996/1997', '1996', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '6', '1996/1997', '1996', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '6', '1996/1997', '1996', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '6', '1996/1997', '1996', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '6', '1996/1997', '1997', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '6', '1996/1997', '1997', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '6', '1996/1997', '1997', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '6', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '6', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '6', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '6', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '6', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '6', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '7', '1996/1997', '1996', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '7', '1996/1997', '1996', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '7', '1996/1997', '1996', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '7', '1996/1997', '1996', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '7', '1996/1997', '1996', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '7', '1996/1997', '1996', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '7', '1996/1997', '1997', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '7', '1996/1997', '1997', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '7', '1996/1997', '1997', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '7', '1996/1997', '1997', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '7', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '7', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '7', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '7', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '7', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '7', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '8', '1996/1997', '1996', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '8', '1996/1997', '1996', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '8', '1996/1997', '1996', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '8', '1996/1997', '1996', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '8', '1996/1997', '1996', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '8', '1996/1997', '1997', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '8', '1996/1997', '1997', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '8', '1996/1997', '1997', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '8', '1996/1997', '1997', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '8', '1996/1997', '1997', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '8', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '8', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '8', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '8', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '8', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '8', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '9', '1996/1997', '1996', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '9', '1996/1997', '1996', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '9', '1996/1997', '1996', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '9', '1996/1997', '1996', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '9', '1996/1997', '1997', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '9', '1996/1997', '1997', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '9', '1996/1997', '1997', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '9', '1996/1997', '1997', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '9', '1996/1997', '1997', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '9', '1996/1997', '1997', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '9', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '9', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '9', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '9', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '9', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '9', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '10', '1996/1997', '1996', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '10', '1996/1997', '1996', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '10', '1996/1997', '1996', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '10', '1996/1997', '1997', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '10', '1996/1997', '1997', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '10', '1996/1997', '1997', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '10', '1996/1997', '1997', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '10', '1996/1997', '1997', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '10', '1996/1997', '1997', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '10', '1996/1997', '1997', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '10', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '10', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '10', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '10', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '10', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '10', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '11', '1996/1997', '1996', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '11', '1996/1997', '1996', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '11', '1996/1997', '1997', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '11', '1996/1997', '1997', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '11', '1996/1997', '1997', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '11', '1996/1997', '1997', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '11', '1996/1997', '1997', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '11', '1996/1997', '1997', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '11', '1996/1997', '1997', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '11', '1996/1997', '1997', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '11', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '11', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '11', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '11', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '11', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '11', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '1', '12', '1996/1997', '1996', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '2', '12', '1996/1997', '1997', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '3', '12', '1996/1997', '1997', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1996', '4', '12', '1996/1997', '1997', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '5', '12', '1996/1997', '1997', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '6', '12', '1996/1997', '1997', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1996', '7', '12', '1996/1997', '1997', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '8', '12', '1996/1997', '1997', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '9', '12', '1996/1997', '1997', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1996', '10', '12', '1996/1997', '1997', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '11', '12', '1996/1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '12', '12', '1996/1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '13', '12', '1996/1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '14', '12', '1996/1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '15', '12', '1996/1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1996', '16', '12', '1996/1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '1', '1997', '1997', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '1', '1997', '1997', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '1', '1997', '1997', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '1', '1997', '1997', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '1', '1997', '1997', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '1', '1997', '1997', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '1', '1997', '1997', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '1', '1997', '1997', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '1', '1997', '1997', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '1', '1997', '1997', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '1', '1997', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '1', '1997', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '1', '1997', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '1', '1997', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '1', '1997', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '1', '1997', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '2', '1997/1998', '1997', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '2', '1997/1998', '1997', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '2', '1997/1998', '1997', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '2', '1997/1998', '1997', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '2', '1997/1998', '1997', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '2', '1997/1998', '1997', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '2', '1997/1998', '1997', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '2', '1997/1998', '1997', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '2', '1997/1998', '1997', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '2', '1997/1998', '1997', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '2', '1997/1998', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '2', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '2', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '2', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '2', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '2', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '3', '1997/1998', '1997', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '3', '1997/1998', '1997', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '3', '1997/1998', '1997', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '3', '1997/1998', '1997', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '3', '1997/1998', '1997', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '3', '1997/1998', '1997', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '3', '1997/1998', '1997', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '3', '1997/1998', '1997', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '3', '1997/1998', '1997', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '3', '1997/1998', '1997', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '3', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '3', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '3', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '3', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '3', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '3', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '4', '1997/1998', '1997', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '4', '1997/1998', '1997', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '4', '1997/1998', '1997', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '4', '1997/1998', '1997', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '4', '1997/1998', '1997', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '4', '1997/1998', '1997', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '4', '1997/1998', '1997', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '4', '1997/1998', '1997', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '4', '1997/1998', '1997', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '4', '1997/1998', '1998', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '4', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '4', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '4', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '4', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '4', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '4', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '5', '1997/1998', '1997', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '5', '1997/1998', '1997', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '5', '1997/1998', '1997', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '5', '1997/1998', '1997', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '5', '1997/1998', '1997', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '5', '1997/1998', '1997', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '5', '1997/1998', '1997', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '5', '1997/1998', '1997', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '5', '1997/1998', '1998', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '5', '1997/1998', '1998', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '5', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '5', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '5', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '5', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '5', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '5', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '6', '1997/1998', '1997', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '6', '1997/1998', '1997', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '6', '1997/1998', '1997', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '6', '1997/1998', '1997', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '6', '1997/1998', '1997', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '6', '1997/1998', '1997', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '6', '1997/1998', '1997', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '6', '1997/1998', '1998', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '6', '1997/1998', '1998', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '6', '1997/1998', '1998', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '6', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '6', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '6', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '6', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '6', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '6', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '7', '1997/1998', '1997', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '7', '1997/1998', '1997', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '7', '1997/1998', '1997', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '7', '1997/1998', '1997', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '7', '1997/1998', '1997', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '7', '1997/1998', '1997', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '7', '1997/1998', '1998', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '7', '1997/1998', '1998', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '7', '1997/1998', '1998', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '7', '1997/1998', '1998', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '7', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '7', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '7', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '7', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '7', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '7', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '8', '1997/1998', '1997', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '8', '1997/1998', '1997', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '8', '1997/1998', '1997', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '8', '1997/1998', '1997', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '8', '1997/1998', '1997', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '8', '1997/1998', '1998', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '8', '1997/1998', '1998', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '8', '1997/1998', '1998', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '8', '1997/1998', '1998', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '8', '1997/1998', '1998', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '8', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '8', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '8', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '8', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '8', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '8', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '9', '1997/1998', '1997', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '9', '1997/1998', '1997', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '9', '1997/1998', '1997', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '9', '1997/1998', '1997', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '9', '1997/1998', '1998', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '9', '1997/1998', '1998', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '9', '1997/1998', '1998', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '9', '1997/1998', '1998', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '9', '1997/1998', '1998', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '9', '1997/1998', '1998', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '9', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '9', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '9', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '9', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '9', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '9', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '10', '1997/1998', '1997', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '10', '1997/1998', '1997', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '10', '1997/1998', '1997', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '10', '1997/1998', '1998', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '10', '1997/1998', '1998', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '10', '1997/1998', '1998', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '10', '1997/1998', '1998', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '10', '1997/1998', '1998', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '10', '1997/1998', '1998', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '10', '1997/1998', '1998', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '10', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '10', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '10', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '10', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '10', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '10', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '11', '1997/1998', '1997', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '11', '1997/1998', '1997', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '11', '1997/1998', '1998', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '11', '1997/1998', '1998', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '11', '1997/1998', '1998', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '11', '1997/1998', '1998', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '11', '1997/1998', '1998', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '11', '1997/1998', '1998', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '11', '1997/1998', '1998', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '11', '1997/1998', '1998', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '11', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '11', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '11', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '11', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '11', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '11', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '1', '12', '1997/1998', '1997', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '2', '12', '1997/1998', '1998', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '3', '12', '1997/1998', '1998', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1997', '4', '12', '1997/1998', '1998', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '5', '12', '1997/1998', '1998', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '6', '12', '1997/1998', '1998', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1997', '7', '12', '1997/1998', '1998', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '8', '12', '1997/1998', '1998', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '9', '12', '1997/1998', '1998', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1997', '10', '12', '1997/1998', '1998', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '11', '12', '1997/1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '12', '12', '1997/1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '13', '12', '1997/1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '14', '12', '1997/1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '15', '12', '1997/1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1997', '16', '12', '1997/1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '1', '1998', '1998', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '1', '1998', '1998', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '1', '1998', '1998', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '1', '1998', '1998', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '1', '1998', '1998', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '1', '1998', '1998', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '1', '1998', '1998', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '1', '1998', '1998', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '1', '1998', '1998', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '1', '1998', '1998', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '1', '1998', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '1', '1998', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '1', '1998', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '1', '1998', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '1', '1998', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '1', '1998', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '2', '1998/1999', '1998', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '2', '1998/1999', '1998', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '2', '1998/1999', '1998', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '2', '1998/1999', '1998', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '2', '1998/1999', '1998', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '2', '1998/1999', '1998', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '2', '1998/1999', '1998', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '2', '1998/1999', '1998', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '2', '1998/1999', '1998', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '2', '1998/1999', '1998', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '2', '1998/1999', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '2', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '2', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '2', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '2', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '2', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '3', '1998/1999', '1998', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '3', '1998/1999', '1998', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '3', '1998/1999', '1998', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '3', '1998/1999', '1998', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '3', '1998/1999', '1998', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '3', '1998/1999', '1998', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '3', '1998/1999', '1998', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '3', '1998/1999', '1998', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '3', '1998/1999', '1998', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '3', '1998/1999', '1998', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '3', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '3', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '3', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '3', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '3', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '3', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '4', '1998/1999', '1998', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '4', '1998/1999', '1998', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '4', '1998/1999', '1998', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '4', '1998/1999', '1998', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '4', '1998/1999', '1998', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '4', '1998/1999', '1998', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '4', '1998/1999', '1998', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '4', '1998/1999', '1998', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '4', '1998/1999', '1998', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '4', '1998/1999', '1999', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '4', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '4', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '4', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '4', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '4', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '4', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '5', '1998/1999', '1998', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '5', '1998/1999', '1998', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '5', '1998/1999', '1998', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '5', '1998/1999', '1998', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '5', '1998/1999', '1998', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '5', '1998/1999', '1998', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '5', '1998/1999', '1998', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '5', '1998/1999', '1998', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '5', '1998/1999', '1999', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '5', '1998/1999', '1999', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '5', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '5', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '5', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '5', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '5', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '5', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '6', '1998/1999', '1998', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '6', '1998/1999', '1998', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '6', '1998/1999', '1998', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '6', '1998/1999', '1998', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '6', '1998/1999', '1998', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '6', '1998/1999', '1998', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '6', '1998/1999', '1998', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '6', '1998/1999', '1999', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '6', '1998/1999', '1999', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '6', '1998/1999', '1999', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '6', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '6', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '6', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '6', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '6', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '6', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '7', '1998/1999', '1998', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '7', '1998/1999', '1998', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '7', '1998/1999', '1998', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '7', '1998/1999', '1998', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '7', '1998/1999', '1998', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '7', '1998/1999', '1998', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '7', '1998/1999', '1999', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '7', '1998/1999', '1999', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '7', '1998/1999', '1999', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '7', '1998/1999', '1999', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '7', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '7', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '7', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '7', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '7', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '7', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '8', '1998/1999', '1998', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '8', '1998/1999', '1998', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '8', '1998/1999', '1998', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '8', '1998/1999', '1998', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '8', '1998/1999', '1998', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '8', '1998/1999', '1999', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '8', '1998/1999', '1999', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '8', '1998/1999', '1999', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '8', '1998/1999', '1999', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '8', '1998/1999', '1999', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '8', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '8', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '8', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '8', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '8', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '8', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '9', '1998/1999', '1998', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '9', '1998/1999', '1998', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '9', '1998/1999', '1998', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '9', '1998/1999', '1998', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '9', '1998/1999', '1999', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '9', '1998/1999', '1999', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '9', '1998/1999', '1999', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '9', '1998/1999', '1999', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '9', '1998/1999', '1999', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '9', '1998/1999', '1999', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '9', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '9', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '9', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '9', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '9', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '9', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '10', '1998/1999', '1998', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '10', '1998/1999', '1998', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '10', '1998/1999', '1998', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '10', '1998/1999', '1999', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '10', '1998/1999', '1999', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '10', '1998/1999', '1999', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '10', '1998/1999', '1999', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '10', '1998/1999', '1999', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '10', '1998/1999', '1999', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '10', '1998/1999', '1999', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '10', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '10', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('1998', '13', '10', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '10', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '10', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '10', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '11', '1998/1999', '1998', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '11', '1998/1999', '1998', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '11', '1998/1999', '1999', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '11', '1998/1999', '1999', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '11', '1998/1999', '1999', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '11', '1998/1999', '1999', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '11', '1998/1999', '1999', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '11', '1998/1999', '1999', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '11', '1998/1999', '1999', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '11', '1998/1999', '1999', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '11', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '11', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '11', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '11', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '11', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '11', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '1', '12', '1998/1999', '1998', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '2', '12', '1998/1999', '1999', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '3', '12', '1998/1999', '1999', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1998', '4', '12', '1998/1999', '1999', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '5', '12', '1998/1999', '1999', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '6', '12', '1998/1999', '1999', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1998', '7', '12', '1998/1999', '1999', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '8', '12', '1998/1999', '1999', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '9', '12', '1998/1999', '1999', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1998', '10', '12', '1998/1999', '1999', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '11', '12', '1998/1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '12', '12', '1998/1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '13', '12', '1998/1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '14', '12', '1998/1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '15', '12', '1998/1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1998', '16', '12', '1998/1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '1', '1999', '1999', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '1', '1999', '1999', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '1', '1999', '1999', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '1', '1999', '1999', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '1', '1999', '1999', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '1', '1999', '1999', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '1', '1999', '1999', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '1', '1999', '1999', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '1', '1999', '1999', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '1', '1999', '1999', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '1', '1999', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '1', '1999', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '1', '1999', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '1', '1999', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '1', '1999', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '1', '1999', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '2', '1999/2000', '1999', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '2', '1999/2000', '1999', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '2', '1999/2000', '1999', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '2', '1999/2000', '1999', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '2', '1999/2000', '1999', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '2', '1999/2000', '1999', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '2', '1999/2000', '1999', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '2', '1999/2000', '1999', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '2', '1999/2000', '1999', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '2', '1999/2000', '1999', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '2', '1999/2000', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '2', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '2', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '2', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '2', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '2', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '3', '1999/2000', '1999', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '3', '1999/2000', '1999', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '3', '1999/2000', '1999', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '3', '1999/2000', '1999', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '3', '1999/2000', '1999', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '3', '1999/2000', '1999', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '3', '1999/2000', '1999', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '3', '1999/2000', '1999', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '3', '1999/2000', '1999', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '3', '1999/2000', '1999', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '3', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '3', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '3', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '3', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '3', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '3', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '4', '1999/2000', '1999', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '4', '1999/2000', '1999', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '4', '1999/2000', '1999', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '4', '1999/2000', '1999', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '4', '1999/2000', '1999', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '4', '1999/2000', '1999', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '4', '1999/2000', '1999', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '4', '1999/2000', '1999', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '4', '1999/2000', '1999', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '4', '1999/2000', '2000', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '4', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '4', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '4', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '4', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '4', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '4', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '5', '1999/2000', '1999', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '5', '1999/2000', '1999', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '5', '1999/2000', '1999', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '5', '1999/2000', '1999', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '5', '1999/2000', '1999', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '5', '1999/2000', '1999', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '5', '1999/2000', '1999', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '5', '1999/2000', '1999', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '5', '1999/2000', '2000', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '5', '1999/2000', '2000', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '5', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '5', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '5', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '5', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '5', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '5', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '6', '1999/2000', '1999', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '6', '1999/2000', '1999', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '6', '1999/2000', '1999', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '6', '1999/2000', '1999', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '6', '1999/2000', '1999', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '6', '1999/2000', '1999', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '6', '1999/2000', '1999', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '6', '1999/2000', '2000', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '6', '1999/2000', '2000', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '6', '1999/2000', '2000', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '6', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '6', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '6', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '6', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '6', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '6', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '7', '1999/2000', '1999', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '7', '1999/2000', '1999', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '7', '1999/2000', '1999', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '7', '1999/2000', '1999', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '7', '1999/2000', '1999', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '7', '1999/2000', '1999', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '7', '1999/2000', '2000', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '7', '1999/2000', '2000', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '7', '1999/2000', '2000', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '7', '1999/2000', '2000', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '7', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '7', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '7', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '7', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '7', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '7', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '8', '1999/2000', '1999', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '8', '1999/2000', '1999', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '8', '1999/2000', '1999', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '8', '1999/2000', '1999', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '8', '1999/2000', '1999', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '8', '1999/2000', '2000', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '8', '1999/2000', '2000', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '8', '1999/2000', '2000', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '8', '1999/2000', '2000', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '8', '1999/2000', '2000', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '8', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '8', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '8', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '8', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '8', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '8', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '9', '1999/2000', '1999', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '9', '1999/2000', '1999', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '9', '1999/2000', '1999', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '9', '1999/2000', '1999', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '9', '1999/2000', '2000', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '9', '1999/2000', '2000', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '9', '1999/2000', '2000', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '9', '1999/2000', '2000', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '9', '1999/2000', '2000', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '9', '1999/2000', '2000', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '9', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '9', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '9', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '9', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '9', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '9', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '10', '1999/2000', '1999', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '10', '1999/2000', '1999', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '10', '1999/2000', '1999', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '10', '1999/2000', '2000', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '10', '1999/2000', '2000', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '10', '1999/2000', '2000', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '10', '1999/2000', '2000', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '10', '1999/2000', '2000', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '10', '1999/2000', '2000', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '10', '1999/2000', '2000', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '10', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '10', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '10', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '10', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '10', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '10', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '11', '1999/2000', '1999', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '11', '1999/2000', '1999', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '11', '1999/2000', '2000', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '11', '1999/2000', '2000', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '11', '1999/2000', '2000', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '11', '1999/2000', '2000', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '11', '1999/2000', '2000', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '11', '1999/2000', '2000', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '11', '1999/2000', '2000', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '11', '1999/2000', '2000', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '11', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '11', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '11', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '11', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '11', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '11', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '1', '12', '1999/2000', '1999', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '2', '12', '1999/2000', '2000', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '3', '12', '1999/2000', '2000', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1999', '4', '12', '1999/2000', '2000', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '5', '12', '1999/2000', '2000', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '6', '12', '1999/2000', '2000', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1999', '7', '12', '1999/2000', '2000', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '8', '12', '1999/2000', '2000', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '9', '12', '1999/2000', '2000', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1999', '10', '12', '1999/2000', '2000', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '11', '12', '1999/2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '12', '12', '1999/2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '13', '12', '1999/2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '14', '12', '1999/2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '15', '12', '1999/2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1999', '16', '12', '1999/2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '1', '2000', '2000', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '1', '2000', '2000', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '1', '2000', '2000', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '1', '2000', '2000', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '1', '2000', '2000', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '1', '2000', '2000', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '1', '2000', '2000', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '1', '2000', '2000', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '1', '2000', '2000', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '1', '2000', '2000', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '1', '2000', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '1', '2000', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '1', '2000', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '1', '2000', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '1', '2000', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '1', '2000', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '2', '2000/2001', '2000', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '2', '2000/2001', '2000', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '2', '2000/2001', '2000', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '2', '2000/2001', '2000', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '2', '2000/2001', '2000', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '2', '2000/2001', '2000', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '2', '2000/2001', '2000', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '2', '2000/2001', '2000', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '2', '2000/2001', '2000', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '2', '2000/2001', '2000', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '2', '2000/2001', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '2', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '2', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '2', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '2', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '2', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '3', '2000/2001', '2000', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '3', '2000/2001', '2000', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '3', '2000/2001', '2000', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '3', '2000/2001', '2000', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '3', '2000/2001', '2000', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '3', '2000/2001', '2000', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '3', '2000/2001', '2000', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '3', '2000/2001', '2000', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '3', '2000/2001', '2000', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '3', '2000/2001', '2000', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '3', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '3', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '3', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '3', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '3', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '3', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '4', '2000/2001', '2000', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '4', '2000/2001', '2000', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '4', '2000/2001', '2000', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '4', '2000/2001', '2000', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '4', '2000/2001', '2000', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '4', '2000/2001', '2000', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '4', '2000/2001', '2000', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '4', '2000/2001', '2000', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '4', '2000/2001', '2000', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '4', '2000/2001', '2001', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '4', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '4', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '4', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '4', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '4', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '4', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '5', '2000/2001', '2000', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '5', '2000/2001', '2000', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '5', '2000/2001', '2000', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '5', '2000/2001', '2000', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '5', '2000/2001', '2000', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '5', '2000/2001', '2000', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '5', '2000/2001', '2000', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '5', '2000/2001', '2000', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '5', '2000/2001', '2001', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '5', '2000/2001', '2001', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '5', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '5', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '5', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '5', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '5', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '5', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '6', '2000/2001', '2000', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '6', '2000/2001', '2000', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '6', '2000/2001', '2000', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '6', '2000/2001', '2000', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '6', '2000/2001', '2000', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '6', '2000/2001', '2000', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '6', '2000/2001', '2000', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '6', '2000/2001', '2001', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '6', '2000/2001', '2001', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '6', '2000/2001', '2001', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '6', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '6', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '6', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '6', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '6', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '6', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '7', '2000/2001', '2000', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '7', '2000/2001', '2000', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '7', '2000/2001', '2000', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '7', '2000/2001', '2000', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '7', '2000/2001', '2000', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '7', '2000/2001', '2000', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '7', '2000/2001', '2001', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '7', '2000/2001', '2001', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '7', '2000/2001', '2001', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '7', '2000/2001', '2001', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '7', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '7', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '7', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '7', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '7', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '7', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '8', '2000/2001', '2000', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '8', '2000/2001', '2000', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '8', '2000/2001', '2000', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '8', '2000/2001', '2000', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '8', '2000/2001', '2000', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '8', '2000/2001', '2001', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '8', '2000/2001', '2001', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '8', '2000/2001', '2001', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '8', '2000/2001', '2001', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '8', '2000/2001', '2001', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '8', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '8', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '8', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '8', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '8', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '8', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '9', '2000/2001', '2000', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '9', '2000/2001', '2000', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '9', '2000/2001', '2000', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '9', '2000/2001', '2000', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '9', '2000/2001', '2001', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '9', '2000/2001', '2001', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '9', '2000/2001', '2001', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '9', '2000/2001', '2001', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '9', '2000/2001', '2001', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '9', '2000/2001', '2001', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '9', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '9', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '9', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '9', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '9', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '9', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '10', '2000/2001', '2000', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '10', '2000/2001', '2000', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '10', '2000/2001', '2000', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '10', '2000/2001', '2001', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '10', '2000/2001', '2001', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '10', '2000/2001', '2001', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '10', '2000/2001', '2001', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '10', '2000/2001', '2001', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '10', '2000/2001', '2001', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '10', '2000/2001', '2001', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '10', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '10', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '10', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '10', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '10', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '10', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '11', '2000/2001', '2000', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '11', '2000/2001', '2000', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '11', '2000/2001', '2001', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '11', '2000/2001', '2001', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '11', '2000/2001', '2001', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '11', '2000/2001', '2001', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '11', '2000/2001', '2001', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '11', '2000/2001', '2001', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '11', '2000/2001', '2001', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '11', '2000/2001', '2001', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '11', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '11', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '11', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '11', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '11', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '11', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '1', '12', '2000/2001', '2000', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '2', '12', '2000/2001', '2001', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '3', '12', '2000/2001', '2001', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2000', '4', '12', '2000/2001', '2001', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '5', '12', '2000/2001', '2001', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '6', '12', '2000/2001', '2001', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2000', '7', '12', '2000/2001', '2001', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '8', '12', '2000/2001', '2001', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '9', '12', '2000/2001', '2001', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2000', '10', '12', '2000/2001', '2001', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '11', '12', '2000/2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '12', '12', '2000/2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '13', '12', '2000/2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '14', '12', '2000/2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '15', '12', '2000/2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2000', '16', '12', '2000/2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '1', '2001', '2001', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '1', '2001', '2001', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '1', '2001', '2001', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '1', '2001', '2001', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '1', '2001', '2001', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '1', '2001', '2001', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '1', '2001', '2001', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '1', '2001', '2001', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '1', '2001', '2001', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '1', '2001', '2001', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '1', '2001', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '1', '2001', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '1', '2001', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '1', '2001', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '1', '2001', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '1', '2001', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '2', '2001/2002', '2001', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '2', '2001/2002', '2001', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '2', '2001/2002', '2001', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '2', '2001/2002', '2001', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '2', '2001/2002', '2001', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '2', '2001/2002', '2001', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '2', '2001/2002', '2001', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '2', '2001/2002', '2001', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '2', '2001/2002', '2001', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '2', '2001/2002', '2001', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '2', '2001/2002', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '2', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '2', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '2', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '2', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '2', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '3', '2001/2002', '2001', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '3', '2001/2002', '2001', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '3', '2001/2002', '2001', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '3', '2001/2002', '2001', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '3', '2001/2002', '2001', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '3', '2001/2002', '2001', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '3', '2001/2002', '2001', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '3', '2001/2002', '2001', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '3', '2001/2002', '2001', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '3', '2001/2002', '2001', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '3', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '3', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '3', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '3', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '3', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '3', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '4', '2001/2002', '2001', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '4', '2001/2002', '2001', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '4', '2001/2002', '2001', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '4', '2001/2002', '2001', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '4', '2001/2002', '2001', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '4', '2001/2002', '2001', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '4', '2001/2002', '2001', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '4', '2001/2002', '2001', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '4', '2001/2002', '2001', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '4', '2001/2002', '2002', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '4', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '4', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '4', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '4', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '4', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '4', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '5', '2001/2002', '2001', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '5', '2001/2002', '2001', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '5', '2001/2002', '2001', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '5', '2001/2002', '2001', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '5', '2001/2002', '2001', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '5', '2001/2002', '2001', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '5', '2001/2002', '2001', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '5', '2001/2002', '2001', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '5', '2001/2002', '2002', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '5', '2001/2002', '2002', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '5', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '5', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '5', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '5', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '5', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '5', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '6', '2001/2002', '2001', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '6', '2001/2002', '2001', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '6', '2001/2002', '2001', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '6', '2001/2002', '2001', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '6', '2001/2002', '2001', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '6', '2001/2002', '2001', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '6', '2001/2002', '2001', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '6', '2001/2002', '2002', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '6', '2001/2002', '2002', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '6', '2001/2002', '2002', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '6', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '6', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '6', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '6', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '6', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '6', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '7', '2001/2002', '2001', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '7', '2001/2002', '2001', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '7', '2001/2002', '2001', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '7', '2001/2002', '2001', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '7', '2001/2002', '2001', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '7', '2001/2002', '2001', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '7', '2001/2002', '2002', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '7', '2001/2002', '2002', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '7', '2001/2002', '2002', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '7', '2001/2002', '2002', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '7', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '7', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '7', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '7', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '7', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '7', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '8', '2001/2002', '2001', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '8', '2001/2002', '2001', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '8', '2001/2002', '2001', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '8', '2001/2002', '2001', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '8', '2001/2002', '2001', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '8', '2001/2002', '2002', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '8', '2001/2002', '2002', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '8', '2001/2002', '2002', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '8', '2001/2002', '2002', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '8', '2001/2002', '2002', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '8', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '8', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '8', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '8', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '8', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '8', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '9', '2001/2002', '2001', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '9', '2001/2002', '2001', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '9', '2001/2002', '2001', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '9', '2001/2002', '2001', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '9', '2001/2002', '2002', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '9', '2001/2002', '2002', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '9', '2001/2002', '2002', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '9', '2001/2002', '2002', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '9', '2001/2002', '2002', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '9', '2001/2002', '2002', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '9', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '9', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '9', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '9', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '9', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '9', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '10', '2001/2002', '2001', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '10', '2001/2002', '2001', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '10', '2001/2002', '2001', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '10', '2001/2002', '2002', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '10', '2001/2002', '2002', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '10', '2001/2002', '2002', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '10', '2001/2002', '2002', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '10', '2001/2002', '2002', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '10', '2001/2002', '2002', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '10', '2001/2002', '2002', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '10', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '10', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '10', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '10', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '10', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '10', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '11', '2001/2002', '2001', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '11', '2001/2002', '2001', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '11', '2001/2002', '2002', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '11', '2001/2002', '2002', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '11', '2001/2002', '2002', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '11', '2001/2002', '2002', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '11', '2001/2002', '2002', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '11', '2001/2002', '2002', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '11', '2001/2002', '2002', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '11', '2001/2002', '2002', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '11', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '11', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '11', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '11', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '11', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '11', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '1', '12', '2001/2002', '2001', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '2', '12', '2001/2002', '2002', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '3', '12', '2001/2002', '2002', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2001', '4', '12', '2001/2002', '2002', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '5', '12', '2001/2002', '2002', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '6', '12', '2001/2002', '2002', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2001', '7', '12', '2001/2002', '2002', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '8', '12', '2001/2002', '2002', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '9', '12', '2001/2002', '2002', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2001', '10', '12', '2001/2002', '2002', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '11', '12', '2001/2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '12', '12', '2001/2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '13', '12', '2001/2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '14', '12', '2001/2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '15', '12', '2001/2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2001', '16', '12', '2001/2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '1', '2002', '2002', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '1', '2002', '2002', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '1', '2002', '2002', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '1', '2002', '2002', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '1', '2002', '2002', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '1', '2002', '2002', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '1', '2002', '2002', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '1', '2002', '2002', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '1', '2002', '2002', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '1', '2002', '2002', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '1', '2002', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '1', '2002', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '1', '2002', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '1', '2002', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '1', '2002', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '1', '2002', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '2', '2002/2003', '2002', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '2', '2002/2003', '2002', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '2', '2002/2003', '2002', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '2', '2002/2003', '2002', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '2', '2002/2003', '2002', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '2', '2002/2003', '2002', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '2', '2002/2003', '2002', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '2', '2002/2003', '2002', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '2', '2002/2003', '2002', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '2', '2002/2003', '2002', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '2', '2002/2003', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '2', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '2', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '2', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '2', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '2', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '3', '2002/2003', '2002', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '3', '2002/2003', '2002', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '3', '2002/2003', '2002', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '3', '2002/2003', '2002', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '3', '2002/2003', '2002', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '3', '2002/2003', '2002', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '3', '2002/2003', '2002', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '3', '2002/2003', '2002', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '3', '2002/2003', '2002', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '3', '2002/2003', '2002', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '3', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '3', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '3', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '3', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '3', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '3', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '4', '2002/2003', '2002', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '4', '2002/2003', '2002', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '4', '2002/2003', '2002', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '4', '2002/2003', '2002', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '4', '2002/2003', '2002', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '4', '2002/2003', '2002', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '4', '2002/2003', '2002', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '4', '2002/2003', '2002', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '4', '2002/2003', '2002', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '4', '2002/2003', '2003', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '4', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '4', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '4', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '4', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '4', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '4', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '5', '2002/2003', '2002', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '5', '2002/2003', '2002', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '5', '2002/2003', '2002', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '5', '2002/2003', '2002', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '5', '2002/2003', '2002', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '5', '2002/2003', '2002', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '5', '2002/2003', '2002', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '5', '2002/2003', '2002', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '5', '2002/2003', '2003', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '5', '2002/2003', '2003', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '5', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '5', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '5', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '5', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '5', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '5', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '6', '2002/2003', '2002', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '6', '2002/2003', '2002', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '6', '2002/2003', '2002', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '6', '2002/2003', '2002', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '6', '2002/2003', '2002', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '6', '2002/2003', '2002', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '6', '2002/2003', '2002', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '6', '2002/2003', '2003', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '6', '2002/2003', '2003', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '6', '2002/2003', '2003', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '6', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '6', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '6', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '6', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '6', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '6', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '7', '2002/2003', '2002', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '7', '2002/2003', '2002', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '7', '2002/2003', '2002', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '7', '2002/2003', '2002', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '7', '2002/2003', '2002', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '7', '2002/2003', '2002', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '7', '2002/2003', '2003', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '7', '2002/2003', '2003', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '7', '2002/2003', '2003', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '7', '2002/2003', '2003', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '7', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '7', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '7', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '7', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '7', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '7', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '8', '2002/2003', '2002', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '8', '2002/2003', '2002', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '8', '2002/2003', '2002', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '8', '2002/2003', '2002', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '8', '2002/2003', '2002', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '8', '2002/2003', '2003', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '8', '2002/2003', '2003', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '8', '2002/2003', '2003', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '8', '2002/2003', '2003', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '8', '2002/2003', '2003', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '8', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '8', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '8', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '8', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '8', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '8', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '9', '2002/2003', '2002', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '9', '2002/2003', '2002', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '9', '2002/2003', '2002', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '9', '2002/2003', '2002', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '9', '2002/2003', '2003', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '9', '2002/2003', '2003', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '9', '2002/2003', '2003', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '9', '2002/2003', '2003', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '9', '2002/2003', '2003', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '9', '2002/2003', '2003', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '9', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '9', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '9', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '9', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '9', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '9', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '10', '2002/2003', '2002', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '10', '2002/2003', '2002', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '10', '2002/2003', '2002', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '10', '2002/2003', '2003', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '10', '2002/2003', '2003', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '10', '2002/2003', '2003', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '10', '2002/2003', '2003', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '10', '2002/2003', '2003', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '10', '2002/2003', '2003', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '10', '2002/2003', '2003', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '10', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '10', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '10', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '10', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '10', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '10', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '11', '2002/2003', '2002', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '11', '2002/2003', '2002', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '11', '2002/2003', '2003', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '11', '2002/2003', '2003', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '11', '2002/2003', '2003', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '11', '2002/2003', '2003', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '11', '2002/2003', '2003', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '11', '2002/2003', '2003', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '11', '2002/2003', '2003', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '11', '2002/2003', '2003', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '11', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '11', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '11', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '11', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '11', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '11', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '1', '12', '2002/2003', '2002', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '2', '12', '2002/2003', '2003', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '3', '12', '2002/2003', '2003', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2002', '4', '12', '2002/2003', '2003', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '5', '12', '2002/2003', '2003', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '6', '12', '2002/2003', '2003', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2002', '7', '12', '2002/2003', '2003', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '8', '12', '2002/2003', '2003', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '9', '12', '2002/2003', '2003', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2002', '10', '12', '2002/2003', '2003', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '11', '12', '2002/2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '12', '12', '2002/2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '13', '12', '2002/2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '14', '12', '2002/2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '15', '12', '2002/2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2002', '16', '12', '2002/2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '1', '2003', '2003', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '1', '2003', '2003', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '1', '2003', '2003', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '1', '2003', '2003', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '1', '2003', '2003', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '1', '2003', '2003', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '1', '2003', '2003', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '1', '2003', '2003', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '1', '2003', '2003', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '1', '2003', '2003', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '1', '2003', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '1', '2003', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '1', '2003', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '1', '2003', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '1', '2003', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '1', '2003', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '2', '2003/2004', '2003', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '2', '2003/2004', '2003', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '2', '2003/2004', '2003', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '2', '2003/2004', '2003', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '2', '2003/2004', '2003', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '2', '2003/2004', '2003', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '2', '2003/2004', '2003', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '2', '2003/2004', '2003', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '2', '2003/2004', '2003', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '2', '2003/2004', '2003', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '2', '2003/2004', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '2', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '2', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '2', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '2', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '2', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '3', '2003/2004', '2003', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '3', '2003/2004', '2003', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '3', '2003/2004', '2003', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '3', '2003/2004', '2003', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '3', '2003/2004', '2003', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '3', '2003/2004', '2003', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '3', '2003/2004', '2003', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '3', '2003/2004', '2003', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '3', '2003/2004', '2003', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '3', '2003/2004', '2003', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '3', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '3', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '3', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '3', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '3', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '3', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '4', '2003/2004', '2003', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '4', '2003/2004', '2003', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '4', '2003/2004', '2003', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '4', '2003/2004', '2003', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '4', '2003/2004', '2003', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '4', '2003/2004', '2003', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '4', '2003/2004', '2003', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '4', '2003/2004', '2003', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '4', '2003/2004', '2003', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '4', '2003/2004', '2004', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '4', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '4', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '4', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '4', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '4', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '4', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '5', '2003/2004', '2003', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '5', '2003/2004', '2003', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '5', '2003/2004', '2003', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '5', '2003/2004', '2003', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '5', '2003/2004', '2003', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '5', '2003/2004', '2003', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '5', '2003/2004', '2003', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '5', '2003/2004', '2003', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '5', '2003/2004', '2004', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '5', '2003/2004', '2004', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '5', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '5', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '5', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '5', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '5', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '5', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '6', '2003/2004', '2003', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '6', '2003/2004', '2003', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '6', '2003/2004', '2003', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '6', '2003/2004', '2003', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '6', '2003/2004', '2003', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '6', '2003/2004', '2003', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '6', '2003/2004', '2003', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '6', '2003/2004', '2004', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '6', '2003/2004', '2004', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '6', '2003/2004', '2004', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '6', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '6', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '6', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '6', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '6', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '6', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '7', '2003/2004', '2003', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '7', '2003/2004', '2003', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '7', '2003/2004', '2003', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '7', '2003/2004', '2003', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '7', '2003/2004', '2003', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '7', '2003/2004', '2003', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '7', '2003/2004', '2004', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '7', '2003/2004', '2004', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '7', '2003/2004', '2004', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '7', '2003/2004', '2004', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '7', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '7', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '7', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '7', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '7', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '7', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '8', '2003/2004', '2003', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '8', '2003/2004', '2003', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '8', '2003/2004', '2003', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '8', '2003/2004', '2003', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '8', '2003/2004', '2003', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '8', '2003/2004', '2004', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '8', '2003/2004', '2004', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '8', '2003/2004', '2004', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '8', '2003/2004', '2004', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '8', '2003/2004', '2004', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '8', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '8', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '8', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '8', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '8', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '8', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '9', '2003/2004', '2003', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '9', '2003/2004', '2003', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '9', '2003/2004', '2003', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '9', '2003/2004', '2003', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '9', '2003/2004', '2004', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '9', '2003/2004', '2004', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '9', '2003/2004', '2004', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '9', '2003/2004', '2004', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '9', '2003/2004', '2004', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '9', '2003/2004', '2004', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '9', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '9', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '9', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '9', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '9', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '9', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '10', '2003/2004', '2003', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '10', '2003/2004', '2003', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '10', '2003/2004', '2003', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '10', '2003/2004', '2004', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '10', '2003/2004', '2004', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '10', '2003/2004', '2004', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '10', '2003/2004', '2004', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '10', '2003/2004', '2004', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '10', '2003/2004', '2004', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '10', '2003/2004', '2004', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '10', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '10', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '10', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '10', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '10', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '10', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '11', '2003/2004', '2003', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '11', '2003/2004', '2003', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '11', '2003/2004', '2004', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '11', '2003/2004', '2004', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '11', '2003/2004', '2004', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '11', '2003/2004', '2004', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '11', '2003/2004', '2004', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '11', '2003/2004', '2004', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '11', '2003/2004', '2004', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '11', '2003/2004', '2004', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '11', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '11', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '11', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '11', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '11', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '11', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '1', '12', '2003/2004', '2003', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '2', '12', '2003/2004', '2004', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '3', '12', '2003/2004', '2004', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2003', '4', '12', '2003/2004', '2004', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '5', '12', '2003/2004', '2004', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '6', '12', '2003/2004', '2004', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2003', '7', '12', '2003/2004', '2004', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '8', '12', '2003/2004', '2004', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '9', '12', '2003/2004', '2004', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2003', '10', '12', '2003/2004', '2004', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '11', '12', '2003/2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '12', '12', '2003/2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '13', '12', '2003/2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '14', '12', '2003/2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '15', '12', '2003/2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2003', '16', '12', '2003/2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '1', '2004', '2004', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '1', '2004', '2004', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '1', '2004', '2004', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '1', '2004', '2004', '4', 'QUARTAL2', 'HALBJAHR1')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('2004', '5', '1', '2004', '2004', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '1', '2004', '2004', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '1', '2004', '2004', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '1', '2004', '2004', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '1', '2004', '2004', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '1', '2004', '2004', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '1', '2004', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '1', '2004', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '1', '2004', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '1', '2004', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '1', '2004', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '1', '2004', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '2', '2004/2005', '2004', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '2', '2004/2005', '2004', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '2', '2004/2005', '2004', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '2', '2004/2005', '2004', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '2', '2004/2005', '2004', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '2', '2004/2005', '2004', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '2', '2004/2005', '2004', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '2', '2004/2005', '2004', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '2', '2004/2005', '2004', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '2', '2004/2005', '2004', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '2', '2004/2005', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '2', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '2', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '2', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '2', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '2', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '3', '2004/2005', '2004', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '3', '2004/2005', '2004', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '3', '2004/2005', '2004', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '3', '2004/2005', '2004', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '3', '2004/2005', '2004', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '3', '2004/2005', '2004', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '3', '2004/2005', '2004', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '3', '2004/2005', '2004', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '3', '2004/2005', '2004', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '3', '2004/2005', '2004', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '3', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '3', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '3', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '3', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '3', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '3', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '4', '2004/2005', '2004', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '4', '2004/2005', '2004', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '4', '2004/2005', '2004', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '4', '2004/2005', '2004', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '4', '2004/2005', '2004', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '4', '2004/2005', '2004', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '4', '2004/2005', '2004', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '4', '2004/2005', '2004', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '4', '2004/2005', '2004', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '4', '2004/2005', '2005', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '4', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '4', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '4', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '4', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '4', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '4', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '5', '2004/2005', '2004', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '5', '2004/2005', '2004', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '5', '2004/2005', '2004', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '5', '2004/2005', '2004', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '5', '2004/2005', '2004', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '5', '2004/2005', '2004', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '5', '2004/2005', '2004', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '5', '2004/2005', '2004', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '5', '2004/2005', '2005', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '5', '2004/2005', '2005', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '5', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '5', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '5', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '5', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '5', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '5', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '6', '2004/2005', '2004', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '6', '2004/2005', '2004', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '6', '2004/2005', '2004', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '6', '2004/2005', '2004', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '6', '2004/2005', '2004', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '6', '2004/2005', '2004', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '6', '2004/2005', '2004', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '6', '2004/2005', '2005', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '6', '2004/2005', '2005', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '6', '2004/2005', '2005', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '6', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '6', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '6', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '6', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '6', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '6', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '7', '2004/2005', '2004', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '7', '2004/2005', '2004', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '7', '2004/2005', '2004', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '7', '2004/2005', '2004', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '7', '2004/2005', '2004', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '7', '2004/2005', '2004', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '7', '2004/2005', '2005', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '7', '2004/2005', '2005', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '7', '2004/2005', '2005', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '7', '2004/2005', '2005', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '7', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '7', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '7', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '7', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '7', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '7', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '8', '2004/2005', '2004', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '8', '2004/2005', '2004', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '8', '2004/2005', '2004', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '8', '2004/2005', '2004', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '8', '2004/2005', '2004', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '8', '2004/2005', '2005', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '8', '2004/2005', '2005', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '8', '2004/2005', '2005', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '8', '2004/2005', '2005', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '8', '2004/2005', '2005', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '8', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '8', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '8', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '8', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '8', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '8', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '9', '2004/2005', '2004', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '9', '2004/2005', '2004', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '9', '2004/2005', '2004', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '9', '2004/2005', '2004', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '9', '2004/2005', '2005', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '9', '2004/2005', '2005', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '9', '2004/2005', '2005', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '9', '2004/2005', '2005', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '9', '2004/2005', '2005', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '9', '2004/2005', '2005', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '9', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '9', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '9', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '9', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '9', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '9', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '10', '2004/2005', '2004', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '10', '2004/2005', '2004', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '10', '2004/2005', '2004', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '10', '2004/2005', '2005', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '10', '2004/2005', '2005', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '10', '2004/2005', '2005', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '10', '2004/2005', '2005', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '10', '2004/2005', '2005', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '10', '2004/2005', '2005', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '10', '2004/2005', '2005', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '10', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '10', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '10', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '10', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '10', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '10', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '11', '2004/2005', '2004', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '11', '2004/2005', '2004', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '11', '2004/2005', '2005', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '11', '2004/2005', '2005', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '11', '2004/2005', '2005', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '11', '2004/2005', '2005', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '11', '2004/2005', '2005', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '11', '2004/2005', '2005', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '11', '2004/2005', '2005', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '11', '2004/2005', '2005', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '11', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '11', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '11', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '11', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '11', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '11', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '1', '12', '2004/2005', '2004', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '2', '12', '2004/2005', '2005', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '3', '12', '2004/2005', '2005', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2004', '4', '12', '2004/2005', '2005', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '5', '12', '2004/2005', '2005', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '6', '12', '2004/2005', '2005', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2004', '7', '12', '2004/2005', '2005', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '8', '12', '2004/2005', '2005', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '9', '12', '2004/2005', '2005', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2004', '10', '12', '2004/2005', '2005', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '11', '12', '2004/2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '12', '12', '2004/2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '13', '12', '2004/2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '14', '12', '2004/2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '15', '12', '2004/2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2004', '16', '12', '2004/2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '1', '2005', '2005', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '1', '2005', '2005', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '1', '2005', '2005', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '1', '2005', '2005', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '1', '2005', '2005', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '1', '2005', '2005', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '1', '2005', '2005', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '1', '2005', '2005', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '1', '2005', '2005', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '1', '2005', '2005', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '1', '2005', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '1', '2005', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '1', '2005', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '1', '2005', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '1', '2005', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '1', '2005', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '2', '2005/2006', '2005', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '2', '2005/2006', '2005', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '2', '2005/2006', '2005', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '2', '2005/2006', '2005', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '2', '2005/2006', '2005', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '2', '2005/2006', '2005', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '2', '2005/2006', '2005', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '2', '2005/2006', '2005', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '2', '2005/2006', '2005', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '2', '2005/2006', '2005', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '2', '2005/2006', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '2', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '2', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '2', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '2', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '2', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '3', '2005/2006', '2005', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '3', '2005/2006', '2005', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '3', '2005/2006', '2005', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '3', '2005/2006', '2005', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '3', '2005/2006', '2005', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '3', '2005/2006', '2005', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '3', '2005/2006', '2005', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '3', '2005/2006', '2005', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '3', '2005/2006', '2005', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '3', '2005/2006', '2005', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '3', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '3', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '3', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '3', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '3', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '3', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '4', '2005/2006', '2005', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '4', '2005/2006', '2005', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '4', '2005/2006', '2005', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '4', '2005/2006', '2005', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '4', '2005/2006', '2005', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '4', '2005/2006', '2005', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '4', '2005/2006', '2005', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '4', '2005/2006', '2005', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '4', '2005/2006', '2005', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '4', '2005/2006', '2006', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '4', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '4', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '4', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '4', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '4', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '4', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '5', '2005/2006', '2005', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '5', '2005/2006', '2005', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '5', '2005/2006', '2005', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '5', '2005/2006', '2005', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '5', '2005/2006', '2005', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '5', '2005/2006', '2005', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '5', '2005/2006', '2005', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '5', '2005/2006', '2005', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '5', '2005/2006', '2006', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '5', '2005/2006', '2006', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '5', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '5', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '5', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '5', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '5', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '5', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '6', '2005/2006', '2005', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '6', '2005/2006', '2005', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '6', '2005/2006', '2005', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '6', '2005/2006', '2005', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '6', '2005/2006', '2005', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '6', '2005/2006', '2005', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '6', '2005/2006', '2005', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '6', '2005/2006', '2006', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '6', '2005/2006', '2006', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '6', '2005/2006', '2006', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '6', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '6', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '6', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '6', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '6', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '6', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '7', '2005/2006', '2005', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '7', '2005/2006', '2005', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '7', '2005/2006', '2005', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '7', '2005/2006', '2005', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '7', '2005/2006', '2005', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '7', '2005/2006', '2005', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '7', '2005/2006', '2006', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '7', '2005/2006', '2006', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '7', '2005/2006', '2006', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '7', '2005/2006', '2006', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '7', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '7', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '7', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '7', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '7', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '7', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '8', '2005/2006', '2005', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '8', '2005/2006', '2005', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '8', '2005/2006', '2005', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '8', '2005/2006', '2005', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '8', '2005/2006', '2005', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '8', '2005/2006', '2006', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '8', '2005/2006', '2006', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '8', '2005/2006', '2006', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '8', '2005/2006', '2006', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '8', '2005/2006', '2006', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '8', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '8', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '8', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '8', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '8', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '8', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '9', '2005/2006', '2005', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '9', '2005/2006', '2005', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '9', '2005/2006', '2005', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '9', '2005/2006', '2005', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '9', '2005/2006', '2006', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '9', '2005/2006', '2006', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '9', '2005/2006', '2006', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '9', '2005/2006', '2006', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '9', '2005/2006', '2006', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '9', '2005/2006', '2006', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '9', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '9', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '9', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '9', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '9', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '9', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '10', '2005/2006', '2005', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '10', '2005/2006', '2005', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '10', '2005/2006', '2005', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '10', '2005/2006', '2006', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '10', '2005/2006', '2006', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '10', '2005/2006', '2006', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '10', '2005/2006', '2006', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '10', '2005/2006', '2006', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '10', '2005/2006', '2006', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '10', '2005/2006', '2006', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '10', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '10', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '10', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '10', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '10', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '10', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '11', '2005/2006', '2005', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '11', '2005/2006', '2005', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '11', '2005/2006', '2006', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '11', '2005/2006', '2006', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '11', '2005/2006', '2006', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '11', '2005/2006', '2006', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '11', '2005/2006', '2006', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '11', '2005/2006', '2006', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '11', '2005/2006', '2006', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '11', '2005/2006', '2006', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '11', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '11', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '11', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '11', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '11', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '11', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '1', '12', '2005/2006', '2005', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '2', '12', '2005/2006', '2006', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '3', '12', '2005/2006', '2006', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2005', '4', '12', '2005/2006', '2006', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '5', '12', '2005/2006', '2006', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '6', '12', '2005/2006', '2006', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2005', '7', '12', '2005/2006', '2006', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '8', '12', '2005/2006', '2006', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '9', '12', '2005/2006', '2006', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2005', '10', '12', '2005/2006', '2006', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '11', '12', '2005/2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '12', '12', '2005/2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '13', '12', '2005/2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '14', '12', '2005/2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '15', '12', '2005/2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2005', '16', '12', '2005/2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '1', '2006', '2006', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '1', '2006', '2006', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '1', '2006', '2006', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '1', '2006', '2006', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '1', '2006', '2006', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '1', '2006', '2006', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '1', '2006', '2006', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '1', '2006', '2006', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '1', '2006', '2006', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '1', '2006', '2006', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '1', '2006', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '1', '2006', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '1', '2006', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '1', '2006', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '1', '2006', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '1', '2006', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '2', '2006/2007', '2006', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '2', '2006/2007', '2006', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '2', '2006/2007', '2006', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '2', '2006/2007', '2006', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '2', '2006/2007', '2006', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '2', '2006/2007', '2006', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '2', '2006/2007', '2006', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '2', '2006/2007', '2006', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '2', '2006/2007', '2006', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '2', '2006/2007', '2006', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '2', '2006/2007', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '2', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '2', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '2', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '2', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '2', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '3', '2006/2007', '2006', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '3', '2006/2007', '2006', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '3', '2006/2007', '2006', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '3', '2006/2007', '2006', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '3', '2006/2007', '2006', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '3', '2006/2007', '2006', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '3', '2006/2007', '2006', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '3', '2006/2007', '2006', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '3', '2006/2007', '2006', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '3', '2006/2007', '2006', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '3', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '3', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '3', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '3', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '3', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '3', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '4', '2006/2007', '2006', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '4', '2006/2007', '2006', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '4', '2006/2007', '2006', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '4', '2006/2007', '2006', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '4', '2006/2007', '2006', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '4', '2006/2007', '2006', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '4', '2006/2007', '2006', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '4', '2006/2007', '2006', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '4', '2006/2007', '2006', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '4', '2006/2007', '2007', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '4', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '4', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '4', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '4', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '4', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '4', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '5', '2006/2007', '2006', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '5', '2006/2007', '2006', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '5', '2006/2007', '2006', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '5', '2006/2007', '2006', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '5', '2006/2007', '2006', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '5', '2006/2007', '2006', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '5', '2006/2007', '2006', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '5', '2006/2007', '2006', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '5', '2006/2007', '2007', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '5', '2006/2007', '2007', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '5', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '5', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '5', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '5', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '5', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '5', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '6', '2006/2007', '2006', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '6', '2006/2007', '2006', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '6', '2006/2007', '2006', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '6', '2006/2007', '2006', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '6', '2006/2007', '2006', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '6', '2006/2007', '2006', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '6', '2006/2007', '2006', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '6', '2006/2007', '2007', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '6', '2006/2007', '2007', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '6', '2006/2007', '2007', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '6', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '6', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '6', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '6', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '6', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '6', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '7', '2006/2007', '2006', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '7', '2006/2007', '2006', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '7', '2006/2007', '2006', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '7', '2006/2007', '2006', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '7', '2006/2007', '2006', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '7', '2006/2007', '2006', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '7', '2006/2007', '2007', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '7', '2006/2007', '2007', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '7', '2006/2007', '2007', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '7', '2006/2007', '2007', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '7', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '7', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '7', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '7', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '7', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '7', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '8', '2006/2007', '2006', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '8', '2006/2007', '2006', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '8', '2006/2007', '2006', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '8', '2006/2007', '2006', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '8', '2006/2007', '2006', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '8', '2006/2007', '2007', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '8', '2006/2007', '2007', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '8', '2006/2007', '2007', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '8', '2006/2007', '2007', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '8', '2006/2007', '2007', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '8', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '8', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '8', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '8', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '8', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '8', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '9', '2006/2007', '2006', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '9', '2006/2007', '2006', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '9', '2006/2007', '2006', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '9', '2006/2007', '2006', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '9', '2006/2007', '2007', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '9', '2006/2007', '2007', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '9', '2006/2007', '2007', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '9', '2006/2007', '2007', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '9', '2006/2007', '2007', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '9', '2006/2007', '2007', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '9', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '9', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '9', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '9', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '9', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '9', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '10', '2006/2007', '2006', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '10', '2006/2007', '2006', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '10', '2006/2007', '2006', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '10', '2006/2007', '2007', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '10', '2006/2007', '2007', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '10', '2006/2007', '2007', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '10', '2006/2007', '2007', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '10', '2006/2007', '2007', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '10', '2006/2007', '2007', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '10', '2006/2007', '2007', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '10', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '10', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '10', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '10', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '10', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '10', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '11', '2006/2007', '2006', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '11', '2006/2007', '2006', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '11', '2006/2007', '2007', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '11', '2006/2007', '2007', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '11', '2006/2007', '2007', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '11', '2006/2007', '2007', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '11', '2006/2007', '2007', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '11', '2006/2007', '2007', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '11', '2006/2007', '2007', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '11', '2006/2007', '2007', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '11', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '11', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '11', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '11', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '11', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '11', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '1', '12', '2006/2007', '2006', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '2', '12', '2006/2007', '2007', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '3', '12', '2006/2007', '2007', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2006', '4', '12', '2006/2007', '2007', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '5', '12', '2006/2007', '2007', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '6', '12', '2006/2007', '2007', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2006', '7', '12', '2006/2007', '2007', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '8', '12', '2006/2007', '2007', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '9', '12', '2006/2007', '2007', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2006', '10', '12', '2006/2007', '2007', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '11', '12', '2006/2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '12', '12', '2006/2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '13', '12', '2006/2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '14', '12', '2006/2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '15', '12', '2006/2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2006', '16', '12', '2006/2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '1', '2007', '2007', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '1', '2007', '2007', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '1', '2007', '2007', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '1', '2007', '2007', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '1', '2007', '2007', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '1', '2007', '2007', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '1', '2007', '2007', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '1', '2007', '2007', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '1', '2007', '2007', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '1', '2007', '2007', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '1', '2007', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '1', '2007', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '1', '2007', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '1', '2007', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '1', '2007', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '1', '2007', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '2', '2007/2008', '2007', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '2', '2007/2008', '2007', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '2', '2007/2008', '2007', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '2', '2007/2008', '2007', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '2', '2007/2008', '2007', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '2', '2007/2008', '2007', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '2', '2007/2008', '2007', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '2', '2007/2008', '2007', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '2', '2007/2008', '2007', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '2', '2007/2008', '2007', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '2', '2007/2008', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '2', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '2', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '2', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '2', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '2', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '3', '2007/2008', '2007', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '3', '2007/2008', '2007', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '3', '2007/2008', '2007', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '3', '2007/2008', '2007', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '3', '2007/2008', '2007', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '3', '2007/2008', '2007', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '3', '2007/2008', '2007', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '3', '2007/2008', '2007', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '3', '2007/2008', '2007', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '3', '2007/2008', '2007', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '3', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '3', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '3', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '3', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '3', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '3', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '4', '2007/2008', '2007', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '4', '2007/2008', '2007', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '4', '2007/2008', '2007', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '4', '2007/2008', '2007', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '4', '2007/2008', '2007', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '4', '2007/2008', '2007', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '4', '2007/2008', '2007', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '4', '2007/2008', '2007', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '4', '2007/2008', '2007', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '4', '2007/2008', '2008', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '4', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '4', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '4', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '4', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '4', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '4', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '5', '2007/2008', '2007', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '5', '2007/2008', '2007', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '5', '2007/2008', '2007', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '5', '2007/2008', '2007', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '5', '2007/2008', '2007', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '5', '2007/2008', '2007', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '5', '2007/2008', '2007', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '5', '2007/2008', '2007', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '5', '2007/2008', '2008', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '5', '2007/2008', '2008', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '5', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '5', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '5', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '5', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '5', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '5', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '6', '2007/2008', '2007', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '6', '2007/2008', '2007', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '6', '2007/2008', '2007', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '6', '2007/2008', '2007', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '6', '2007/2008', '2007', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '6', '2007/2008', '2007', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '6', '2007/2008', '2007', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '6', '2007/2008', '2008', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '6', '2007/2008', '2008', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '6', '2007/2008', '2008', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '6', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '6', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '6', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '6', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '6', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '6', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '7', '2007/2008', '2007', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '7', '2007/2008', '2007', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '7', '2007/2008', '2007', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '7', '2007/2008', '2007', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '7', '2007/2008', '2007', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '7', '2007/2008', '2007', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '7', '2007/2008', '2008', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '7', '2007/2008', '2008', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '7', '2007/2008', '2008', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '7', '2007/2008', '2008', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '7', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '7', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '7', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '7', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '7', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '7', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '8', '2007/2008', '2007', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '8', '2007/2008', '2007', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '8', '2007/2008', '2007', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '8', '2007/2008', '2007', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '8', '2007/2008', '2007', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '8', '2007/2008', '2008', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '8', '2007/2008', '2008', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '8', '2007/2008', '2008', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '8', '2007/2008', '2008', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '8', '2007/2008', '2008', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '8', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '8', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '8', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '8', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '8', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '8', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '9', '2007/2008', '2007', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '9', '2007/2008', '2007', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '9', '2007/2008', '2007', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '9', '2007/2008', '2007', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '9', '2007/2008', '2008', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '9', '2007/2008', '2008', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '9', '2007/2008', '2008', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '9', '2007/2008', '2008', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '9', '2007/2008', '2008', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '9', '2007/2008', '2008', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '9', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '9', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '9', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '9', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '9', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '9', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '10', '2007/2008', '2007', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '10', '2007/2008', '2007', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '10', '2007/2008', '2007', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '10', '2007/2008', '2008', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '10', '2007/2008', '2008', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '10', '2007/2008', '2008', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '10', '2007/2008', '2008', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '10', '2007/2008', '2008', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '10', '2007/2008', '2008', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '10', '2007/2008', '2008', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '10', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '10', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '10', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '10', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '10', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '10', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '11', '2007/2008', '2007', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '11', '2007/2008', '2007', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '11', '2007/2008', '2008', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '11', '2007/2008', '2008', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '11', '2007/2008', '2008', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '11', '2007/2008', '2008', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '11', '2007/2008', '2008', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '11', '2007/2008', '2008', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '11', '2007/2008', '2008', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '11', '2007/2008', '2008', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '11', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '11', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '11', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '11', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '11', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '11', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '1', '12', '2007/2008', '2007', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '2', '12', '2007/2008', '2008', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '3', '12', '2007/2008', '2008', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2007', '4', '12', '2007/2008', '2008', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '5', '12', '2007/2008', '2008', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '6', '12', '2007/2008', '2008', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2007', '7', '12', '2007/2008', '2008', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '8', '12', '2007/2008', '2008', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '9', '12', '2007/2008', '2008', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2007', '10', '12', '2007/2008', '2008', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '11', '12', '2007/2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '12', '12', '2007/2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '13', '12', '2007/2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '14', '12', '2007/2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '15', '12', '2007/2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2007', '16', '12', '2007/2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '1', '2008', '2008', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '1', '2008', '2008', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '1', '2008', '2008', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '1', '2008', '2008', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '1', '2008', '2008', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '1', '2008', '2008', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '1', '2008', '2008', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '1', '2008', '2008', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '1', '2008', '2008', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '1', '2008', '2008', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '1', '2008', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '1', '2008', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '1', '2008', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '1', '2008', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '1', '2008', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '1', '2008', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '2', '2008/2009', '2008', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '2', '2008/2009', '2008', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '2', '2008/2009', '2008', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '2', '2008/2009', '2008', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '2', '2008/2009', '2008', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '2', '2008/2009', '2008', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '2', '2008/2009', '2008', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '2', '2008/2009', '2008', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '2', '2008/2009', '2008', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '2', '2008/2009', '2008', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '2', '2008/2009', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '2', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '2', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '2', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '2', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '2', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '3', '2008/2009', '2008', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '3', '2008/2009', '2008', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '3', '2008/2009', '2008', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '3', '2008/2009', '2008', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '3', '2008/2009', '2008', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '3', '2008/2009', '2008', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '3', '2008/2009', '2008', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '3', '2008/2009', '2008', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '3', '2008/2009', '2008', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '3', '2008/2009', '2008', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '3', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '3', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '3', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '3', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '3', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '3', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '4', '2008/2009', '2008', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '4', '2008/2009', '2008', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '4', '2008/2009', '2008', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '4', '2008/2009', '2008', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '4', '2008/2009', '2008', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '4', '2008/2009', '2008', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '4', '2008/2009', '2008', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '4', '2008/2009', '2008', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '4', '2008/2009', '2008', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '4', '2008/2009', '2009', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '4', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '4', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '4', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '4', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '4', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '4', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '5', '2008/2009', '2008', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '5', '2008/2009', '2008', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '5', '2008/2009', '2008', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '5', '2008/2009', '2008', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '5', '2008/2009', '2008', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '5', '2008/2009', '2008', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '5', '2008/2009', '2008', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '5', '2008/2009', '2008', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '5', '2008/2009', '2009', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '5', '2008/2009', '2009', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '5', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '5', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '5', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '5', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '5', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '5', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '6', '2008/2009', '2008', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '6', '2008/2009', '2008', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '6', '2008/2009', '2008', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '6', '2008/2009', '2008', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '6', '2008/2009', '2008', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '6', '2008/2009', '2008', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '6', '2008/2009', '2008', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '6', '2008/2009', '2009', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '6', '2008/2009', '2009', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '6', '2008/2009', '2009', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '6', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '6', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '6', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '6', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '6', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '6', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '7', '2008/2009', '2008', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '7', '2008/2009', '2008', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '7', '2008/2009', '2008', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '7', '2008/2009', '2008', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '7', '2008/2009', '2008', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '7', '2008/2009', '2008', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '7', '2008/2009', '2009', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '7', '2008/2009', '2009', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '7', '2008/2009', '2009', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '7', '2008/2009', '2009', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '7', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '7', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '7', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '7', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '7', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '7', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '8', '2008/2009', '2008', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '8', '2008/2009', '2008', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '8', '2008/2009', '2008', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '8', '2008/2009', '2008', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '8', '2008/2009', '2008', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '8', '2008/2009', '2009', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '8', '2008/2009', '2009', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '8', '2008/2009', '2009', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '8', '2008/2009', '2009', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '8', '2008/2009', '2009', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '8', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '8', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '8', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '8', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '8', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '8', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '9', '2008/2009', '2008', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '9', '2008/2009', '2008', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '9', '2008/2009', '2008', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '9', '2008/2009', '2008', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '9', '2008/2009', '2009', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '9', '2008/2009', '2009', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '9', '2008/2009', '2009', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '9', '2008/2009', '2009', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '9', '2008/2009', '2009', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '9', '2008/2009', '2009', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '9', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '9', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '9', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '9', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '9', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '9', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '10', '2008/2009', '2008', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '10', '2008/2009', '2008', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '10', '2008/2009', '2008', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '10', '2008/2009', '2009', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '10', '2008/2009', '2009', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '10', '2008/2009', '2009', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '10', '2008/2009', '2009', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '10', '2008/2009', '2009', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '10', '2008/2009', '2009', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '10', '2008/2009', '2009', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '10', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '10', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '10', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '10', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '10', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '10', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '11', '2008/2009', '2008', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '11', '2008/2009', '2008', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '11', '2008/2009', '2009', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '11', '2008/2009', '2009', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '11', '2008/2009', '2009', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '11', '2008/2009', '2009', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '11', '2008/2009', '2009', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '11', '2008/2009', '2009', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '11', '2008/2009', '2009', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '11', '2008/2009', '2009', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '11', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '11', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '11', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '11', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '11', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '11', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '1', '12', '2008/2009', '2008', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '2', '12', '2008/2009', '2009', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '3', '12', '2008/2009', '2009', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2008', '4', '12', '2008/2009', '2009', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '5', '12', '2008/2009', '2009', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '6', '12', '2008/2009', '2009', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2008', '7', '12', '2008/2009', '2009', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '8', '12', '2008/2009', '2009', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '9', '12', '2008/2009', '2009', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2008', '10', '12', '2008/2009', '2009', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '11', '12', '2008/2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '12', '12', '2008/2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '13', '12', '2008/2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '14', '12', '2008/2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '15', '12', '2008/2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2008', '16', '12', '2008/2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '1', '2009', '2009', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '1', '2009', '2009', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '1', '2009', '2009', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '1', '2009', '2009', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '1', '2009', '2009', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '1', '2009', '2009', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '1', '2009', '2009', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '1', '2009', '2009', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '1', '2009', '2009', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '1', '2009', '2009', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '1', '2009', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '1', '2009', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '1', '2009', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '1', '2009', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '1', '2009', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '1', '2009', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '2', '2009/2010', '2009', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '2', '2009/2010', '2009', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '2', '2009/2010', '2009', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '2', '2009/2010', '2009', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '2', '2009/2010', '2009', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '2', '2009/2010', '2009', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '2', '2009/2010', '2009', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '2', '2009/2010', '2009', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '2', '2009/2010', '2009', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '2', '2009/2010', '2009', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '2', '2009/2010', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '2', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '2', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '2', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '2', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '2', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '3', '2009/2010', '2009', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '3', '2009/2010', '2009', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '3', '2009/2010', '2009', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '3', '2009/2010', '2009', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '3', '2009/2010', '2009', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '3', '2009/2010', '2009', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '3', '2009/2010', '2009', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '3', '2009/2010', '2009', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '3', '2009/2010', '2009', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '3', '2009/2010', '2009', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '3', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '3', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('2009', '13', '3', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '3', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '3', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '3', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '4', '2009/2010', '2009', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '4', '2009/2010', '2009', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '4', '2009/2010', '2009', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '4', '2009/2010', '2009', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '4', '2009/2010', '2009', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '4', '2009/2010', '2009', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '4', '2009/2010', '2009', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '4', '2009/2010', '2009', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '4', '2009/2010', '2009', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '4', '2009/2010', '2010', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '4', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '4', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '4', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '4', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '4', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '4', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '5', '2009/2010', '2009', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '5', '2009/2010', '2009', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '5', '2009/2010', '2009', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '5', '2009/2010', '2009', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '5', '2009/2010', '2009', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '5', '2009/2010', '2009', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '5', '2009/2010', '2009', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '5', '2009/2010', '2009', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '5', '2009/2010', '2010', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '5', '2009/2010', '2010', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '5', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '5', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '5', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '5', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '5', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '5', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '6', '2009/2010', '2009', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '6', '2009/2010', '2009', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '6', '2009/2010', '2009', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '6', '2009/2010', '2009', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '6', '2009/2010', '2009', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '6', '2009/2010', '2009', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '6', '2009/2010', '2009', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '6', '2009/2010', '2010', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '6', '2009/2010', '2010', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '6', '2009/2010', '2010', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '6', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '6', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '6', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '6', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '6', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '6', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '7', '2009/2010', '2009', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '7', '2009/2010', '2009', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '7', '2009/2010', '2009', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '7', '2009/2010', '2009', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '7', '2009/2010', '2009', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '7', '2009/2010', '2009', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '7', '2009/2010', '2010', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '7', '2009/2010', '2010', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '7', '2009/2010', '2010', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '7', '2009/2010', '2010', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '7', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '7', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '7', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '7', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '7', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '7', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '8', '2009/2010', '2009', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '8', '2009/2010', '2009', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '8', '2009/2010', '2009', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '8', '2009/2010', '2009', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '8', '2009/2010', '2009', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '8', '2009/2010', '2010', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '8', '2009/2010', '2010', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '8', '2009/2010', '2010', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '8', '2009/2010', '2010', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '8', '2009/2010', '2010', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '8', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '8', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '8', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '8', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '8', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '8', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '9', '2009/2010', '2009', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '9', '2009/2010', '2009', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '9', '2009/2010', '2009', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '9', '2009/2010', '2009', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '9', '2009/2010', '2010', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '9', '2009/2010', '2010', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '9', '2009/2010', '2010', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '9', '2009/2010', '2010', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '9', '2009/2010', '2010', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '9', '2009/2010', '2010', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '9', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '9', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '9', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '9', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '9', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '9', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '10', '2009/2010', '2009', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '10', '2009/2010', '2009', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '10', '2009/2010', '2009', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '10', '2009/2010', '2010', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '10', '2009/2010', '2010', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '10', '2009/2010', '2010', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '10', '2009/2010', '2010', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '10', '2009/2010', '2010', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '10', '2009/2010', '2010', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '10', '2009/2010', '2010', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '10', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '10', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '10', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '10', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '10', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '10', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '11', '2009/2010', '2009', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '11', '2009/2010', '2009', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '11', '2009/2010', '2010', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '11', '2009/2010', '2010', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '11', '2009/2010', '2010', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '11', '2009/2010', '2010', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '11', '2009/2010', '2010', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '11', '2009/2010', '2010', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '11', '2009/2010', '2010', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '11', '2009/2010', '2010', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '11', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '11', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '11', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '11', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '11', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '11', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '1', '12', '2009/2010', '2009', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '2', '12', '2009/2010', '2010', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '3', '12', '2009/2010', '2010', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2009', '4', '12', '2009/2010', '2010', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '5', '12', '2009/2010', '2010', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '6', '12', '2009/2010', '2010', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2009', '7', '12', '2009/2010', '2010', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '8', '12', '2009/2010', '2010', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '9', '12', '2009/2010', '2010', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2009', '10', '12', '2009/2010', '2010', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '11', '12', '2009/2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '12', '12', '2009/2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '13', '12', '2009/2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '14', '12', '2009/2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '15', '12', '2009/2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2009', '16', '12', '2009/2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '1', '2010', '2010', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '1', '2010', '2010', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '1', '2010', '2010', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '1', '2010', '2010', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '1', '2010', '2010', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '1', '2010', '2010', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '1', '2010', '2010', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '1', '2010', '2010', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '1', '2010', '2010', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '1', '2010', '2010', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '1', '2010', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '1', '2010', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '1', '2010', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '1', '2010', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '1', '2010', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '1', '2010', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '2', '2010/2011', '2010', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '2', '2010/2011', '2010', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '2', '2010/2011', '2010', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '2', '2010/2011', '2010', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '2', '2010/2011', '2010', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '2', '2010/2011', '2010', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '2', '2010/2011', '2010', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '2', '2010/2011', '2010', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '2', '2010/2011', '2010', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '2', '2010/2011', '2010', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '2', '2010/2011', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '2', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '2', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '2', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '2', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '2', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '3', '2010/2011', '2010', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '3', '2010/2011', '2010', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '3', '2010/2011', '2010', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '3', '2010/2011', '2010', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '3', '2010/2011', '2010', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '3', '2010/2011', '2010', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '3', '2010/2011', '2010', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '3', '2010/2011', '2010', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '3', '2010/2011', '2010', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '3', '2010/2011', '2010', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '3', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '3', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '3', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '3', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '3', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '3', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '4', '2010/2011', '2010', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '4', '2010/2011', '2010', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '4', '2010/2011', '2010', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '4', '2010/2011', '2010', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '4', '2010/2011', '2010', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '4', '2010/2011', '2010', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '4', '2010/2011', '2010', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '4', '2010/2011', '2010', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '4', '2010/2011', '2010', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '4', '2010/2011', '2011', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '4', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '4', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '4', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '4', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '4', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '4', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '5', '2010/2011', '2010', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '5', '2010/2011', '2010', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '5', '2010/2011', '2010', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '5', '2010/2011', '2010', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '5', '2010/2011', '2010', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '5', '2010/2011', '2010', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '5', '2010/2011', '2010', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '5', '2010/2011', '2010', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '5', '2010/2011', '2011', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '5', '2010/2011', '2011', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '5', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '5', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '5', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '5', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '5', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '5', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '6', '2010/2011', '2010', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '6', '2010/2011', '2010', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '6', '2010/2011', '2010', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '6', '2010/2011', '2010', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '6', '2010/2011', '2010', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '6', '2010/2011', '2010', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '6', '2010/2011', '2010', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '6', '2010/2011', '2011', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '6', '2010/2011', '2011', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '6', '2010/2011', '2011', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '6', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '6', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '6', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '6', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '6', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '6', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '7', '2010/2011', '2010', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '7', '2010/2011', '2010', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '7', '2010/2011', '2010', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '7', '2010/2011', '2010', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '7', '2010/2011', '2010', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '7', '2010/2011', '2010', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '7', '2010/2011', '2011', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '7', '2010/2011', '2011', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '7', '2010/2011', '2011', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '7', '2010/2011', '2011', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '7', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '7', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '7', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '7', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '7', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '7', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '8', '2010/2011', '2010', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '8', '2010/2011', '2010', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '8', '2010/2011', '2010', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '8', '2010/2011', '2010', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '8', '2010/2011', '2010', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '8', '2010/2011', '2011', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '8', '2010/2011', '2011', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '8', '2010/2011', '2011', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '8', '2010/2011', '2011', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '8', '2010/2011', '2011', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '8', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '8', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '8', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '8', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '8', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '8', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '9', '2010/2011', '2010', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '9', '2010/2011', '2010', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '9', '2010/2011', '2010', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '9', '2010/2011', '2010', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '9', '2010/2011', '2011', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '9', '2010/2011', '2011', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '9', '2010/2011', '2011', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '9', '2010/2011', '2011', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '9', '2010/2011', '2011', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '9', '2010/2011', '2011', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '9', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '9', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '9', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '9', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '9', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '9', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '10', '2010/2011', '2010', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '10', '2010/2011', '2010', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '10', '2010/2011', '2010', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '10', '2010/2011', '2011', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '10', '2010/2011', '2011', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '10', '2010/2011', '2011', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '10', '2010/2011', '2011', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '10', '2010/2011', '2011', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '10', '2010/2011', '2011', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '10', '2010/2011', '2011', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '10', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '10', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '10', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '10', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '10', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '10', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '11', '2010/2011', '2010', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '11', '2010/2011', '2010', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '11', '2010/2011', '2011', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '11', '2010/2011', '2011', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '11', '2010/2011', '2011', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '11', '2010/2011', '2011', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '11', '2010/2011', '2011', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '11', '2010/2011', '2011', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '11', '2010/2011', '2011', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '11', '2010/2011', '2011', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '11', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '11', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '11', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '11', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '11', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '11', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '1', '12', '2010/2011', '2010', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '2', '12', '2010/2011', '2011', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '3', '12', '2010/2011', '2011', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2010', '4', '12', '2010/2011', '2011', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '5', '12', '2010/2011', '2011', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '6', '12', '2010/2011', '2011', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2010', '7', '12', '2010/2011', '2011', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '8', '12', '2010/2011', '2011', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '9', '12', '2010/2011', '2011', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2010', '10', '12', '2010/2011', '2011', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '11', '12', '2010/2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '12', '12', '2010/2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '13', '12', '2010/2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '14', '12', '2010/2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '15', '12', '2010/2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2010', '16', '12', '2010/2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '1', '2011', '2011', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '1', '2011', '2011', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '1', '2011', '2011', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '1', '2011', '2011', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '1', '2011', '2011', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '1', '2011', '2011', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '1', '2011', '2011', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '1', '2011', '2011', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '1', '2011', '2011', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '1', '2011', '2011', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '1', '2011', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '1', '2011', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '1', '2011', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '1', '2011', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '1', '2011', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '1', '2011', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '2', '2011/2012', '2011', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '2', '2011/2012', '2011', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '2', '2011/2012', '2011', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '2', '2011/2012', '2011', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '2', '2011/2012', '2011', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '2', '2011/2012', '2011', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '2', '2011/2012', '2011', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '2', '2011/2012', '2011', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '2', '2011/2012', '2011', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '2', '2011/2012', '2011', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '2', '2011/2012', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '2', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '2', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '2', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '2', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '2', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '3', '2011/2012', '2011', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '3', '2011/2012', '2011', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '3', '2011/2012', '2011', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '3', '2011/2012', '2011', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '3', '2011/2012', '2011', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '3', '2011/2012', '2011', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '3', '2011/2012', '2011', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '3', '2011/2012', '2011', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '3', '2011/2012', '2011', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '3', '2011/2012', '2011', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '3', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '3', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '3', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '3', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '3', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '3', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '4', '2011/2012', '2011', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '4', '2011/2012', '2011', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '4', '2011/2012', '2011', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '4', '2011/2012', '2011', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '4', '2011/2012', '2011', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '4', '2011/2012', '2011', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '4', '2011/2012', '2011', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '4', '2011/2012', '2011', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '4', '2011/2012', '2011', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '4', '2011/2012', '2012', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '4', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '4', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '4', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '4', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '4', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '4', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '5', '2011/2012', '2011', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '5', '2011/2012', '2011', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '5', '2011/2012', '2011', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '5', '2011/2012', '2011', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '5', '2011/2012', '2011', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '5', '2011/2012', '2011', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '5', '2011/2012', '2011', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '5', '2011/2012', '2011', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '5', '2011/2012', '2012', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '5', '2011/2012', '2012', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '5', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '5', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '5', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '5', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '5', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '5', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '6', '2011/2012', '2011', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '6', '2011/2012', '2011', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '6', '2011/2012', '2011', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '6', '2011/2012', '2011', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '6', '2011/2012', '2011', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '6', '2011/2012', '2011', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '6', '2011/2012', '2011', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '6', '2011/2012', '2012', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '6', '2011/2012', '2012', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '6', '2011/2012', '2012', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '6', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '6', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '6', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '6', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '6', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '6', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '7', '2011/2012', '2011', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '7', '2011/2012', '2011', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '7', '2011/2012', '2011', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '7', '2011/2012', '2011', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '7', '2011/2012', '2011', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '7', '2011/2012', '2011', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '7', '2011/2012', '2012', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '7', '2011/2012', '2012', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '7', '2011/2012', '2012', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '7', '2011/2012', '2012', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '7', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '7', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '7', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '7', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '7', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '7', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '8', '2011/2012', '2011', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '8', '2011/2012', '2011', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '8', '2011/2012', '2011', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '8', '2011/2012', '2011', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '8', '2011/2012', '2011', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '8', '2011/2012', '2012', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '8', '2011/2012', '2012', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '8', '2011/2012', '2012', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '8', '2011/2012', '2012', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '8', '2011/2012', '2012', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '8', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '8', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '8', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '8', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '8', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '8', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '9', '2011/2012', '2011', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '9', '2011/2012', '2011', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '9', '2011/2012', '2011', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '9', '2011/2012', '2011', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '9', '2011/2012', '2012', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '9', '2011/2012', '2012', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '9', '2011/2012', '2012', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '9', '2011/2012', '2012', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '9', '2011/2012', '2012', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '9', '2011/2012', '2012', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '9', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '9', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '9', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '9', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '9', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '9', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '10', '2011/2012', '2011', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '10', '2011/2012', '2011', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '10', '2011/2012', '2011', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '10', '2011/2012', '2012', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '10', '2011/2012', '2012', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '10', '2011/2012', '2012', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '10', '2011/2012', '2012', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '10', '2011/2012', '2012', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '10', '2011/2012', '2012', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '10', '2011/2012', '2012', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '10', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '10', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '10', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '10', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '10', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '10', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '11', '2011/2012', '2011', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '11', '2011/2012', '2011', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '11', '2011/2012', '2012', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '11', '2011/2012', '2012', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '11', '2011/2012', '2012', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '11', '2011/2012', '2012', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '11', '2011/2012', '2012', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '11', '2011/2012', '2012', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '11', '2011/2012', '2012', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '11', '2011/2012', '2012', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '11', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '11', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '11', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '11', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '11', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '11', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '1', '12', '2011/2012', '2011', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '2', '12', '2011/2012', '2012', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '3', '12', '2011/2012', '2012', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2011', '4', '12', '2011/2012', '2012', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '5', '12', '2011/2012', '2012', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '6', '12', '2011/2012', '2012', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2011', '7', '12', '2011/2012', '2012', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '8', '12', '2011/2012', '2012', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '9', '12', '2011/2012', '2012', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2011', '10', '12', '2011/2012', '2012', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '11', '12', '2011/2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '12', '12', '2011/2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '13', '12', '2011/2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '14', '12', '2011/2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '15', '12', '2011/2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2011', '16', '12', '2011/2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '1', '2012', '2012', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '1', '2012', '2012', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '1', '2012', '2012', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '1', '2012', '2012', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '1', '2012', '2012', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '1', '2012', '2012', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '1', '2012', '2012', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '1', '2012', '2012', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '1', '2012', '2012', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '1', '2012', '2012', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '1', '2012', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '1', '2012', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '1', '2012', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '1', '2012', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '1', '2012', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '1', '2012', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '2', '2012/2013', '2012', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '2', '2012/2013', '2012', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '2', '2012/2013', '2012', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '2', '2012/2013', '2012', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '2', '2012/2013', '2012', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '2', '2012/2013', '2012', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '2', '2012/2013', '2012', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '2', '2012/2013', '2012', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '2', '2012/2013', '2012', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '2', '2012/2013', '2012', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '2', '2012/2013', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '2', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '2', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '2', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '2', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '2', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '3', '2012/2013', '2012', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '3', '2012/2013', '2012', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '3', '2012/2013', '2012', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '3', '2012/2013', '2012', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '3', '2012/2013', '2012', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '3', '2012/2013', '2012', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '3', '2012/2013', '2012', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '3', '2012/2013', '2012', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '3', '2012/2013', '2012', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '3', '2012/2013', '2012', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '3', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '3', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '3', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '3', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '3', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '3', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '4', '2012/2013', '2012', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '4', '2012/2013', '2012', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '4', '2012/2013', '2012', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '4', '2012/2013', '2012', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '4', '2012/2013', '2012', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '4', '2012/2013', '2012', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '4', '2012/2013', '2012', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '4', '2012/2013', '2012', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '4', '2012/2013', '2012', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '4', '2012/2013', '2013', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '4', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '4', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '4', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '4', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '4', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '4', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '5', '2012/2013', '2012', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '5', '2012/2013', '2012', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '5', '2012/2013', '2012', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '5', '2012/2013', '2012', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '5', '2012/2013', '2012', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '5', '2012/2013', '2012', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '5', '2012/2013', '2012', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '5', '2012/2013', '2012', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '5', '2012/2013', '2013', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '5', '2012/2013', '2013', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '5', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '5', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '5', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '5', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '5', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '5', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '6', '2012/2013', '2012', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '6', '2012/2013', '2012', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '6', '2012/2013', '2012', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '6', '2012/2013', '2012', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '6', '2012/2013', '2012', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '6', '2012/2013', '2012', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '6', '2012/2013', '2012', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '6', '2012/2013', '2013', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '6', '2012/2013', '2013', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '6', '2012/2013', '2013', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '6', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '6', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '6', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '6', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '6', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '6', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '7', '2012/2013', '2012', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '7', '2012/2013', '2012', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '7', '2012/2013', '2012', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '7', '2012/2013', '2012', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '7', '2012/2013', '2012', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '7', '2012/2013', '2012', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '7', '2012/2013', '2013', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '7', '2012/2013', '2013', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '7', '2012/2013', '2013', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '7', '2012/2013', '2013', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '7', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '7', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '7', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '7', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '7', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '7', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '8', '2012/2013', '2012', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '8', '2012/2013', '2012', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '8', '2012/2013', '2012', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '8', '2012/2013', '2012', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '8', '2012/2013', '2012', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '8', '2012/2013', '2013', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '8', '2012/2013', '2013', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '8', '2012/2013', '2013', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '8', '2012/2013', '2013', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '8', '2012/2013', '2013', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '8', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '8', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '8', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '8', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '8', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '8', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '9', '2012/2013', '2012', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '9', '2012/2013', '2012', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '9', '2012/2013', '2012', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '9', '2012/2013', '2012', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '9', '2012/2013', '2013', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '9', '2012/2013', '2013', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '9', '2012/2013', '2013', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '9', '2012/2013', '2013', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '9', '2012/2013', '2013', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '9', '2012/2013', '2013', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '9', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '9', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '9', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '9', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '9', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '9', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '10', '2012/2013', '2012', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '10', '2012/2013', '2012', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '10', '2012/2013', '2012', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '10', '2012/2013', '2013', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '10', '2012/2013', '2013', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '10', '2012/2013', '2013', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '10', '2012/2013', '2013', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '10', '2012/2013', '2013', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '10', '2012/2013', '2013', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '10', '2012/2013', '2013', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '10', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '10', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '10', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '10', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '10', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '10', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '11', '2012/2013', '2012', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '11', '2012/2013', '2012', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '11', '2012/2013', '2013', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '11', '2012/2013', '2013', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '11', '2012/2013', '2013', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '11', '2012/2013', '2013', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '11', '2012/2013', '2013', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '11', '2012/2013', '2013', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '11', '2012/2013', '2013', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '11', '2012/2013', '2013', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '11', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '11', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '11', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '11', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '11', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '11', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '1', '12', '2012/2013', '2012', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '2', '12', '2012/2013', '2013', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '3', '12', '2012/2013', '2013', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2012', '4', '12', '2012/2013', '2013', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '5', '12', '2012/2013', '2013', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '6', '12', '2012/2013', '2013', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2012', '7', '12', '2012/2013', '2013', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '8', '12', '2012/2013', '2013', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '9', '12', '2012/2013', '2013', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2012', '10', '12', '2012/2013', '2013', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '11', '12', '2012/2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '12', '12', '2012/2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '13', '12', '2012/2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '14', '12', '2012/2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '15', '12', '2012/2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2012', '16', '12', '2012/2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '1', '2013', '2013', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '1', '2013', '2013', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '1', '2013', '2013', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '1', '2013', '2013', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '1', '2013', '2013', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '1', '2013', '2013', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '1', '2013', '2013', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '1', '2013', '2013', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '1', '2013', '2013', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '1', '2013', '2013', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '1', '2013', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '1', '2013', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '1', '2013', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '1', '2013', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '1', '2013', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '1', '2013', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '2', '2013/2014', '2013', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '2', '2013/2014', '2013', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '2', '2013/2014', '2013', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '2', '2013/2014', '2013', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '2', '2013/2014', '2013', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '2', '2013/2014', '2013', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '2', '2013/2014', '2013', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '2', '2013/2014', '2013', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '2', '2013/2014', '2013', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '2', '2013/2014', '2013', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '2', '2013/2014', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '2', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '2', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '2', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '2', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '2', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '3', '2013/2014', '2013', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '3', '2013/2014', '2013', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '3', '2013/2014', '2013', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '3', '2013/2014', '2013', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '3', '2013/2014', '2013', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '3', '2013/2014', '2013', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '3', '2013/2014', '2013', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '3', '2013/2014', '2013', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '3', '2013/2014', '2013', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '3', '2013/2014', '2013', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '3', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '3', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '3', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '3', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '3', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '3', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '4', '2013/2014', '2013', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '4', '2013/2014', '2013', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '4', '2013/2014', '2013', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '4', '2013/2014', '2013', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '4', '2013/2014', '2013', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '4', '2013/2014', '2013', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '4', '2013/2014', '2013', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '4', '2013/2014', '2013', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '4', '2013/2014', '2013', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '4', '2013/2014', '2014', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '4', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '4', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '4', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '4', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '4', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '4', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '5', '2013/2014', '2013', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '5', '2013/2014', '2013', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '5', '2013/2014', '2013', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '5', '2013/2014', '2013', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '5', '2013/2014', '2013', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '5', '2013/2014', '2013', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '5', '2013/2014', '2013', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '5', '2013/2014', '2013', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '5', '2013/2014', '2014', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '5', '2013/2014', '2014', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '5', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '5', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '5', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '5', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '5', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '5', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '6', '2013/2014', '2013', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '6', '2013/2014', '2013', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '6', '2013/2014', '2013', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '6', '2013/2014', '2013', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '6', '2013/2014', '2013', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '6', '2013/2014', '2013', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '6', '2013/2014', '2013', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '6', '2013/2014', '2014', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '6', '2013/2014', '2014', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '6', '2013/2014', '2014', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '6', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '6', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '6', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '6', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '6', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '6', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '7', '2013/2014', '2013', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '7', '2013/2014', '2013', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '7', '2013/2014', '2013', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '7', '2013/2014', '2013', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '7', '2013/2014', '2013', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '7', '2013/2014', '2013', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '7', '2013/2014', '2014', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '7', '2013/2014', '2014', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '7', '2013/2014', '2014', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '7', '2013/2014', '2014', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '7', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '7', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '7', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '7', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '7', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '7', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '8', '2013/2014', '2013', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '8', '2013/2014', '2013', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '8', '2013/2014', '2013', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '8', '2013/2014', '2013', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '8', '2013/2014', '2013', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '8', '2013/2014', '2014', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '8', '2013/2014', '2014', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '8', '2013/2014', '2014', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '8', '2013/2014', '2014', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '8', '2013/2014', '2014', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '8', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '8', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '8', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '8', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '8', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '8', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '9', '2013/2014', '2013', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '9', '2013/2014', '2013', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '9', '2013/2014', '2013', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '9', '2013/2014', '2013', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '9', '2013/2014', '2014', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '9', '2013/2014', '2014', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '9', '2013/2014', '2014', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '9', '2013/2014', '2014', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '9', '2013/2014', '2014', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '9', '2013/2014', '2014', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '9', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '9', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '9', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '9', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '9', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '9', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '10', '2013/2014', '2013', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '10', '2013/2014', '2013', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '10', '2013/2014', '2013', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '10', '2013/2014', '2014', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '10', '2013/2014', '2014', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '10', '2013/2014', '2014', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '10', '2013/2014', '2014', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '10', '2013/2014', '2014', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '10', '2013/2014', '2014', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '10', '2013/2014', '2014', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '10', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '10', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '10', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '10', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '10', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '10', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '11', '2013/2014', '2013', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '11', '2013/2014', '2013', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '11', '2013/2014', '2014', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '11', '2013/2014', '2014', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '11', '2013/2014', '2014', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '11', '2013/2014', '2014', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '11', '2013/2014', '2014', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '11', '2013/2014', '2014', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '11', '2013/2014', '2014', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '11', '2013/2014', '2014', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '11', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '11', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '11', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '11', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '11', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '11', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '1', '12', '2013/2014', '2013', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '2', '12', '2013/2014', '2014', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '3', '12', '2013/2014', '2014', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2013', '4', '12', '2013/2014', '2014', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '5', '12', '2013/2014', '2014', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '6', '12', '2013/2014', '2014', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2013', '7', '12', '2013/2014', '2014', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '8', '12', '2013/2014', '2014', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '9', '12', '2013/2014', '2014', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2013', '10', '12', '2013/2014', '2014', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '11', '12', '2013/2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '12', '12', '2013/2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '13', '12', '2013/2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '14', '12', '2013/2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '15', '12', '2013/2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2013', '16', '12', '2013/2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '1', '2014', '2014', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '1', '2014', '2014', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '1', '2014', '2014', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '1', '2014', '2014', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '1', '2014', '2014', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '1', '2014', '2014', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '1', '2014', '2014', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '1', '2014', '2014', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '1', '2014', '2014', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '1', '2014', '2014', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '1', '2014', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '1', '2014', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '1', '2014', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '1', '2014', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '1', '2014', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '1', '2014', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '2', '2014/2015', '2014', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '2', '2014/2015', '2014', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '2', '2014/2015', '2014', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '2', '2014/2015', '2014', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '2', '2014/2015', '2014', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '2', '2014/2015', '2014', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '2', '2014/2015', '2014', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '2', '2014/2015', '2014', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '2', '2014/2015', '2014', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '2', '2014/2015', '2014', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '2', '2014/2015', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '2', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '2', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '2', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '2', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '2', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '3', '2014/2015', '2014', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '3', '2014/2015', '2014', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '3', '2014/2015', '2014', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '3', '2014/2015', '2014', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '3', '2014/2015', '2014', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '3', '2014/2015', '2014', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '3', '2014/2015', '2014', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '3', '2014/2015', '2014', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '3', '2014/2015', '2014', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '3', '2014/2015', '2014', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '3', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '3', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '3', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '3', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '3', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '3', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '4', '2014/2015', '2014', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '4', '2014/2015', '2014', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '4', '2014/2015', '2014', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '4', '2014/2015', '2014', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '4', '2014/2015', '2014', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '4', '2014/2015', '2014', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '4', '2014/2015', '2014', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '4', '2014/2015', '2014', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '4', '2014/2015', '2014', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '4', '2014/2015', '2015', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '4', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '4', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '4', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '4', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '4', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '4', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '5', '2014/2015', '2014', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '5', '2014/2015', '2014', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '5', '2014/2015', '2014', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '5', '2014/2015', '2014', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '5', '2014/2015', '2014', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '5', '2014/2015', '2014', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '5', '2014/2015', '2014', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '5', '2014/2015', '2014', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '5', '2014/2015', '2015', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '5', '2014/2015', '2015', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '5', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '5', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '5', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '5', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '5', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '5', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '6', '2014/2015', '2014', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '6', '2014/2015', '2014', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '6', '2014/2015', '2014', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '6', '2014/2015', '2014', '9', 'QUARTAL2', 'HALBJAHR1')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('2014', '5', '6', '2014/2015', '2014', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '6', '2014/2015', '2014', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '6', '2014/2015', '2014', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '6', '2014/2015', '2015', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '6', '2014/2015', '2015', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '6', '2014/2015', '2015', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '6', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '6', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '6', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '6', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '6', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '6', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '7', '2014/2015', '2014', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '7', '2014/2015', '2014', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '7', '2014/2015', '2014', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '7', '2014/2015', '2014', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '7', '2014/2015', '2014', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '7', '2014/2015', '2014', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '7', '2014/2015', '2015', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '7', '2014/2015', '2015', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '7', '2014/2015', '2015', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '7', '2014/2015', '2015', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '7', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '7', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '7', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '7', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '7', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '7', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '8', '2014/2015', '2014', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '8', '2014/2015', '2014', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '8', '2014/2015', '2014', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '8', '2014/2015', '2014', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '8', '2014/2015', '2014', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '8', '2014/2015', '2015', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '8', '2014/2015', '2015', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '8', '2014/2015', '2015', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '8', '2014/2015', '2015', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '8', '2014/2015', '2015', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '8', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '8', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '8', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '8', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '8', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '8', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '9', '2014/2015', '2014', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '9', '2014/2015', '2014', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '9', '2014/2015', '2014', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '9', '2014/2015', '2014', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '9', '2014/2015', '2015', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '9', '2014/2015', '2015', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '9', '2014/2015', '2015', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '9', '2014/2015', '2015', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '9', '2014/2015', '2015', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '9', '2014/2015', '2015', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '9', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '9', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '9', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '9', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '9', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '9', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '10', '2014/2015', '2014', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '10', '2014/2015', '2014', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '10', '2014/2015', '2014', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '10', '2014/2015', '2015', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '10', '2014/2015', '2015', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '10', '2014/2015', '2015', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '10', '2014/2015', '2015', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '10', '2014/2015', '2015', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '10', '2014/2015', '2015', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '10', '2014/2015', '2015', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '10', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '10', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '10', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '10', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '10', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '10', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '11', '2014/2015', '2014', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '11', '2014/2015', '2014', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '11', '2014/2015', '2015', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '11', '2014/2015', '2015', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '11', '2014/2015', '2015', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '11', '2014/2015', '2015', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '11', '2014/2015', '2015', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '11', '2014/2015', '2015', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '11', '2014/2015', '2015', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '11', '2014/2015', '2015', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '11', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '11', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '11', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '11', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '11', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '11', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '1', '12', '2014/2015', '2014', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '2', '12', '2014/2015', '2015', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '3', '12', '2014/2015', '2015', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2014', '4', '12', '2014/2015', '2015', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '5', '12', '2014/2015', '2015', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '6', '12', '2014/2015', '2015', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2014', '7', '12', '2014/2015', '2015', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '8', '12', '2014/2015', '2015', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '9', '12', '2014/2015', '2015', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2014', '10', '12', '2014/2015', '2015', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '11', '12', '2014/2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '12', '12', '2014/2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '13', '12', '2014/2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '14', '12', '2014/2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '15', '12', '2014/2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2014', '16', '12', '2014/2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '1', '2015', '2015', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '1', '2015', '2015', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '1', '2015', '2015', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '1', '2015', '2015', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '1', '2015', '2015', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '1', '2015', '2015', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '1', '2015', '2015', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '1', '2015', '2015', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '1', '2015', '2015', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '1', '2015', '2015', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '1', '2015', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '1', '2015', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '1', '2015', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '1', '2015', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '1', '2015', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '1', '2015', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '2', '2015/2016', '2015', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '2', '2015/2016', '2015', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '2', '2015/2016', '2015', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '2', '2015/2016', '2015', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '2', '2015/2016', '2015', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '2', '2015/2016', '2015', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '2', '2015/2016', '2015', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '2', '2015/2016', '2015', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '2', '2015/2016', '2015', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '2', '2015/2016', '2015', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '2', '2015/2016', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '2', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '2', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '2', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '2', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '2', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '3', '2015/2016', '2015', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '3', '2015/2016', '2015', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '3', '2015/2016', '2015', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '3', '2015/2016', '2015', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '3', '2015/2016', '2015', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '3', '2015/2016', '2015', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '3', '2015/2016', '2015', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '3', '2015/2016', '2015', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '3', '2015/2016', '2015', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '3', '2015/2016', '2015', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '3', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '3', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '3', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '3', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '3', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '3', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '4', '2015/2016', '2015', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '4', '2015/2016', '2015', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '4', '2015/2016', '2015', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '4', '2015/2016', '2015', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '4', '2015/2016', '2015', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '4', '2015/2016', '2015', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '4', '2015/2016', '2015', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '4', '2015/2016', '2015', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '4', '2015/2016', '2015', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '4', '2015/2016', '2016', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '4', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '4', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '4', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '4', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '4', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '4', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '5', '2015/2016', '2015', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '5', '2015/2016', '2015', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '5', '2015/2016', '2015', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '5', '2015/2016', '2015', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '5', '2015/2016', '2015', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '5', '2015/2016', '2015', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '5', '2015/2016', '2015', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '5', '2015/2016', '2015', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '5', '2015/2016', '2016', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '5', '2015/2016', '2016', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '5', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '5', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '5', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '5', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '5', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '5', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '6', '2015/2016', '2015', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '6', '2015/2016', '2015', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '6', '2015/2016', '2015', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '6', '2015/2016', '2015', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '6', '2015/2016', '2015', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '6', '2015/2016', '2015', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '6', '2015/2016', '2015', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '6', '2015/2016', '2016', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '6', '2015/2016', '2016', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '6', '2015/2016', '2016', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '6', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '6', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '6', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '6', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '6', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '6', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '7', '2015/2016', '2015', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '7', '2015/2016', '2015', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '7', '2015/2016', '2015', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '7', '2015/2016', '2015', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '7', '2015/2016', '2015', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '7', '2015/2016', '2015', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '7', '2015/2016', '2016', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '7', '2015/2016', '2016', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '7', '2015/2016', '2016', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '7', '2015/2016', '2016', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '7', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '7', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '7', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '7', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '7', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '7', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '8', '2015/2016', '2015', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '8', '2015/2016', '2015', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '8', '2015/2016', '2015', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '8', '2015/2016', '2015', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '8', '2015/2016', '2015', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '8', '2015/2016', '2016', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '8', '2015/2016', '2016', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '8', '2015/2016', '2016', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '8', '2015/2016', '2016', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '8', '2015/2016', '2016', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '8', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '8', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '8', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '8', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '8', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '8', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '9', '2015/2016', '2015', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '9', '2015/2016', '2015', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '9', '2015/2016', '2015', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '9', '2015/2016', '2015', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '9', '2015/2016', '2016', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '9', '2015/2016', '2016', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '9', '2015/2016', '2016', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '9', '2015/2016', '2016', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '9', '2015/2016', '2016', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '9', '2015/2016', '2016', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '9', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '9', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '9', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '9', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '9', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '9', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '10', '2015/2016', '2015', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '10', '2015/2016', '2015', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '10', '2015/2016', '2015', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '10', '2015/2016', '2016', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '10', '2015/2016', '2016', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '10', '2015/2016', '2016', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '10', '2015/2016', '2016', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '10', '2015/2016', '2016', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '10', '2015/2016', '2016', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '10', '2015/2016', '2016', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '10', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '10', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '10', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '10', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '10', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '10', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '11', '2015/2016', '2015', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '11', '2015/2016', '2015', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '11', '2015/2016', '2016', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '11', '2015/2016', '2016', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '11', '2015/2016', '2016', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '11', '2015/2016', '2016', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '11', '2015/2016', '2016', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '11', '2015/2016', '2016', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '11', '2015/2016', '2016', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '11', '2015/2016', '2016', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '11', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '11', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '11', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '11', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '11', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '11', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '1', '12', '2015/2016', '2015', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '2', '12', '2015/2016', '2016', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '3', '12', '2015/2016', '2016', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2015', '4', '12', '2015/2016', '2016', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '5', '12', '2015/2016', '2016', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '6', '12', '2015/2016', '2016', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2015', '7', '12', '2015/2016', '2016', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '8', '12', '2015/2016', '2016', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '9', '12', '2015/2016', '2016', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2015', '10', '12', '2015/2016', '2016', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '11', '12', '2015/2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '12', '12', '2015/2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '13', '12', '2015/2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '14', '12', '2015/2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '15', '12', '2015/2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2015', '16', '12', '2015/2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '1', '2016', '2016', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '1', '2016', '2016', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '1', '2016', '2016', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '1', '2016', '2016', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '1', '2016', '2016', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '1', '2016', '2016', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '1', '2016', '2016', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '1', '2016', '2016', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '1', '2016', '2016', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '1', '2016', '2016', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '1', '2016', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '1', '2016', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '1', '2016', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '1', '2016', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '1', '2016', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '1', '2016', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '2', '2016/2017', '2016', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '2', '2016/2017', '2016', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '2', '2016/2017', '2016', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '2', '2016/2017', '2016', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '2', '2016/2017', '2016', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '2', '2016/2017', '2016', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '2', '2016/2017', '2016', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '2', '2016/2017', '2016', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '2', '2016/2017', '2016', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '2', '2016/2017', '2016', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '2', '2016/2017', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '2', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '2', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '2', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '2', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '2', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '3', '2016/2017', '2016', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '3', '2016/2017', '2016', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '3', '2016/2017', '2016', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '3', '2016/2017', '2016', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '3', '2016/2017', '2016', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '3', '2016/2017', '2016', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '3', '2016/2017', '2016', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '3', '2016/2017', '2016', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '3', '2016/2017', '2016', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '3', '2016/2017', '2016', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '3', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '3', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '3', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '3', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '3', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '3', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '4', '2016/2017', '2016', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '4', '2016/2017', '2016', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '4', '2016/2017', '2016', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '4', '2016/2017', '2016', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '4', '2016/2017', '2016', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '4', '2016/2017', '2016', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '4', '2016/2017', '2016', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '4', '2016/2017', '2016', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '4', '2016/2017', '2016', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '4', '2016/2017', '2017', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '4', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '4', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '4', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '4', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '4', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '4', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '5', '2016/2017', '2016', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '5', '2016/2017', '2016', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '5', '2016/2017', '2016', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '5', '2016/2017', '2016', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '5', '2016/2017', '2016', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '5', '2016/2017', '2016', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '5', '2016/2017', '2016', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '5', '2016/2017', '2016', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '5', '2016/2017', '2017', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '5', '2016/2017', '2017', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '5', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '5', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '5', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '5', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '5', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '5', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '6', '2016/2017', '2016', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '6', '2016/2017', '2016', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '6', '2016/2017', '2016', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '6', '2016/2017', '2016', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '6', '2016/2017', '2016', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '6', '2016/2017', '2016', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '6', '2016/2017', '2016', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '6', '2016/2017', '2017', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '6', '2016/2017', '2017', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '6', '2016/2017', '2017', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '6', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '6', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '6', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '6', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '6', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '6', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '7', '2016/2017', '2016', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '7', '2016/2017', '2016', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '7', '2016/2017', '2016', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '7', '2016/2017', '2016', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '7', '2016/2017', '2016', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '7', '2016/2017', '2016', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '7', '2016/2017', '2017', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '7', '2016/2017', '2017', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '7', '2016/2017', '2017', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '7', '2016/2017', '2017', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '7', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '7', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '7', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '7', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '7', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '7', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '8', '2016/2017', '2016', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '8', '2016/2017', '2016', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '8', '2016/2017', '2016', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '8', '2016/2017', '2016', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '8', '2016/2017', '2016', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '8', '2016/2017', '2017', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '8', '2016/2017', '2017', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '8', '2016/2017', '2017', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '8', '2016/2017', '2017', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '8', '2016/2017', '2017', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '8', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '8', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '8', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '8', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '8', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '8', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '9', '2016/2017', '2016', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '9', '2016/2017', '2016', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '9', '2016/2017', '2016', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '9', '2016/2017', '2016', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '9', '2016/2017', '2017', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '9', '2016/2017', '2017', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '9', '2016/2017', '2017', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '9', '2016/2017', '2017', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '9', '2016/2017', '2017', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '9', '2016/2017', '2017', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '9', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '9', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '9', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '9', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '9', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '9', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '10', '2016/2017', '2016', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '10', '2016/2017', '2016', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '10', '2016/2017', '2016', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '10', '2016/2017', '2017', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '10', '2016/2017', '2017', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '10', '2016/2017', '2017', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '10', '2016/2017', '2017', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '10', '2016/2017', '2017', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '10', '2016/2017', '2017', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '10', '2016/2017', '2017', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '10', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '10', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '10', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '10', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '10', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '10', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '11', '2016/2017', '2016', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '11', '2016/2017', '2016', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '11', '2016/2017', '2017', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '11', '2016/2017', '2017', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '11', '2016/2017', '2017', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '11', '2016/2017', '2017', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '11', '2016/2017', '2017', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '11', '2016/2017', '2017', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '11', '2016/2017', '2017', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '11', '2016/2017', '2017', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '11', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '11', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '11', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '11', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '11', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '11', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '1', '12', '2016/2017', '2016', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '2', '12', '2016/2017', '2017', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '3', '12', '2016/2017', '2017', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2016', '4', '12', '2016/2017', '2017', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '5', '12', '2016/2017', '2017', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '6', '12', '2016/2017', '2017', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2016', '7', '12', '2016/2017', '2017', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '8', '12', '2016/2017', '2017', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '9', '12', '2016/2017', '2017', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2016', '10', '12', '2016/2017', '2017', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '11', '12', '2016/2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '12', '12', '2016/2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '13', '12', '2016/2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '14', '12', '2016/2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '15', '12', '2016/2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2016', '16', '12', '2016/2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '1', '2017', '2017', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '1', '2017', '2017', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '1', '2017', '2017', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '1', '2017', '2017', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '1', '2017', '2017', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '1', '2017', '2017', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '1', '2017', '2017', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '1', '2017', '2017', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '1', '2017', '2017', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '1', '2017', '2017', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '1', '2017', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '1', '2017', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '1', '2017', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '1', '2017', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '1', '2017', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '1', '2017', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '2', '2017/2018', '2017', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '2', '2017/2018', '2017', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '2', '2017/2018', '2017', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '2', '2017/2018', '2017', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '2', '2017/2018', '2017', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '2', '2017/2018', '2017', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '2', '2017/2018', '2017', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '2', '2017/2018', '2017', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '2', '2017/2018', '2017', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '2', '2017/2018', '2017', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '2', '2017/2018', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '2', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '2', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '2', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '2', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '2', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '3', '2017/2018', '2017', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '3', '2017/2018', '2017', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '3', '2017/2018', '2017', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '3', '2017/2018', '2017', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '3', '2017/2018', '2017', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '3', '2017/2018', '2017', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '3', '2017/2018', '2017', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '3', '2017/2018', '2017', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '3', '2017/2018', '2017', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '3', '2017/2018', '2017', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '3', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '3', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '3', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '3', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '3', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '3', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '4', '2017/2018', '2017', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '4', '2017/2018', '2017', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '4', '2017/2018', '2017', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '4', '2017/2018', '2017', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '4', '2017/2018', '2017', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '4', '2017/2018', '2017', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '4', '2017/2018', '2017', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '4', '2017/2018', '2017', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '4', '2017/2018', '2017', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '4', '2017/2018', '2018', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '4', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '4', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '4', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '4', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '4', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '4', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '5', '2017/2018', '2017', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '5', '2017/2018', '2017', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '5', '2017/2018', '2017', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '5', '2017/2018', '2017', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '5', '2017/2018', '2017', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '5', '2017/2018', '2017', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '5', '2017/2018', '2017', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '5', '2017/2018', '2017', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '5', '2017/2018', '2018', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '5', '2017/2018', '2018', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '5', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '5', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '5', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '5', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '5', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '5', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '6', '2017/2018', '2017', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '6', '2017/2018', '2017', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '6', '2017/2018', '2017', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '6', '2017/2018', '2017', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '6', '2017/2018', '2017', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '6', '2017/2018', '2017', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '6', '2017/2018', '2017', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '6', '2017/2018', '2018', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '6', '2017/2018', '2018', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '6', '2017/2018', '2018', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '6', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '6', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '6', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '6', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '6', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '6', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '7', '2017/2018', '2017', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '7', '2017/2018', '2017', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '7', '2017/2018', '2017', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '7', '2017/2018', '2017', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '7', '2017/2018', '2017', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '7', '2017/2018', '2017', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '7', '2017/2018', '2018', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '7', '2017/2018', '2018', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '7', '2017/2018', '2018', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '7', '2017/2018', '2018', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '7', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '7', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '7', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '7', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '7', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '7', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '8', '2017/2018', '2017', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '8', '2017/2018', '2017', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '8', '2017/2018', '2017', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '8', '2017/2018', '2017', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '8', '2017/2018', '2017', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '8', '2017/2018', '2018', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '8', '2017/2018', '2018', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '8', '2017/2018', '2018', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '8', '2017/2018', '2018', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '8', '2017/2018', '2018', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '8', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '8', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '8', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '8', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '8', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '8', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '9', '2017/2018', '2017', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '9', '2017/2018', '2017', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '9', '2017/2018', '2017', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '9', '2017/2018', '2017', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '9', '2017/2018', '2018', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '9', '2017/2018', '2018', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '9', '2017/2018', '2018', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '9', '2017/2018', '2018', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '9', '2017/2018', '2018', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '9', '2017/2018', '2018', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '9', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '9', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '9', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '9', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '9', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '9', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '10', '2017/2018', '2017', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '10', '2017/2018', '2017', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '10', '2017/2018', '2017', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '10', '2017/2018', '2018', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '10', '2017/2018', '2018', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '10', '2017/2018', '2018', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '10', '2017/2018', '2018', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '10', '2017/2018', '2018', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '10', '2017/2018', '2018', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '10', '2017/2018', '2018', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '10', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '10', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '10', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '10', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '10', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '10', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '11', '2017/2018', '2017', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '11', '2017/2018', '2017', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '11', '2017/2018', '2018', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '11', '2017/2018', '2018', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '11', '2017/2018', '2018', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '11', '2017/2018', '2018', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '11', '2017/2018', '2018', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '11', '2017/2018', '2018', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '11', '2017/2018', '2018', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '11', '2017/2018', '2018', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '11', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '11', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '11', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '11', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '11', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '11', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '1', '12', '2017/2018', '2017', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '2', '12', '2017/2018', '2018', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '3', '12', '2017/2018', '2018', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2017', '4', '12', '2017/2018', '2018', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '5', '12', '2017/2018', '2018', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '6', '12', '2017/2018', '2018', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2017', '7', '12', '2017/2018', '2018', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '8', '12', '2017/2018', '2018', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '9', '12', '2017/2018', '2018', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2017', '10', '12', '2017/2018', '2018', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '11', '12', '2017/2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '12', '12', '2017/2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '13', '12', '2017/2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '14', '12', '2017/2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '15', '12', '2017/2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2017', '16', '12', '2017/2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '1', '2018', '2018', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '1', '2018', '2018', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '1', '2018', '2018', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '1', '2018', '2018', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '1', '2018', '2018', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '1', '2018', '2018', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '1', '2018', '2018', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '1', '2018', '2018', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '1', '2018', '2018', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '1', '2018', '2018', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '1', '2018', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '1', '2018', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '1', '2018', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '1', '2018', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '1', '2018', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '1', '2018', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '2', '2018/2019', '2018', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '2', '2018/2019', '2018', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '2', '2018/2019', '2018', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '2', '2018/2019', '2018', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '2', '2018/2019', '2018', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '2', '2018/2019', '2018', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '2', '2018/2019', '2018', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '2', '2018/2019', '2018', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '2', '2018/2019', '2018', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '2', '2018/2019', '2018', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '2', '2018/2019', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '2', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '2', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '2', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '2', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '2', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '3', '2018/2019', '2018', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '3', '2018/2019', '2018', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '3', '2018/2019', '2018', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '3', '2018/2019', '2018', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '3', '2018/2019', '2018', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '3', '2018/2019', '2018', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '3', '2018/2019', '2018', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '3', '2018/2019', '2018', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '3', '2018/2019', '2018', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '3', '2018/2019', '2018', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '3', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '3', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '3', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '3', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '3', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '3', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '4', '2018/2019', '2018', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '4', '2018/2019', '2018', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '4', '2018/2019', '2018', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '4', '2018/2019', '2018', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '4', '2018/2019', '2018', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '4', '2018/2019', '2018', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '4', '2018/2019', '2018', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '4', '2018/2019', '2018', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '4', '2018/2019', '2018', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '4', '2018/2019', '2019', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '4', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '4', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '4', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '4', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '4', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '4', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '5', '2018/2019', '2018', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '5', '2018/2019', '2018', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '5', '2018/2019', '2018', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '5', '2018/2019', '2018', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '5', '2018/2019', '2018', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '5', '2018/2019', '2018', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '5', '2018/2019', '2018', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '5', '2018/2019', '2018', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '5', '2018/2019', '2019', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '5', '2018/2019', '2019', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '5', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '5', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '5', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '5', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '5', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '5', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '6', '2018/2019', '2018', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '6', '2018/2019', '2018', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '6', '2018/2019', '2018', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '6', '2018/2019', '2018', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '6', '2018/2019', '2018', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '6', '2018/2019', '2018', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '6', '2018/2019', '2018', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '6', '2018/2019', '2019', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '6', '2018/2019', '2019', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '6', '2018/2019', '2019', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '6', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '6', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '6', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '6', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '6', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '6', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '7', '2018/2019', '2018', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '7', '2018/2019', '2018', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '7', '2018/2019', '2018', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '7', '2018/2019', '2018', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '7', '2018/2019', '2018', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '7', '2018/2019', '2018', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '7', '2018/2019', '2019', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '7', '2018/2019', '2019', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '7', '2018/2019', '2019', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '7', '2018/2019', '2019', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '7', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '7', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '7', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '7', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '7', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '7', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '8', '2018/2019', '2018', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '8', '2018/2019', '2018', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '8', '2018/2019', '2018', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '8', '2018/2019', '2018', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '8', '2018/2019', '2018', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '8', '2018/2019', '2019', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '8', '2018/2019', '2019', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '8', '2018/2019', '2019', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '8', '2018/2019', '2019', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '8', '2018/2019', '2019', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '8', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '8', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '8', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '8', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '8', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '8', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '9', '2018/2019', '2018', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '9', '2018/2019', '2018', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '9', '2018/2019', '2018', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '9', '2018/2019', '2018', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '9', '2018/2019', '2019', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '9', '2018/2019', '2019', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '9', '2018/2019', '2019', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '9', '2018/2019', '2019', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '9', '2018/2019', '2019', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '9', '2018/2019', '2019', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '9', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '9', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '9', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '9', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '9', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '9', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '10', '2018/2019', '2018', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '10', '2018/2019', '2018', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '10', '2018/2019', '2018', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '10', '2018/2019', '2019', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '10', '2018/2019', '2019', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '10', '2018/2019', '2019', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '10', '2018/2019', '2019', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '10', '2018/2019', '2019', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '10', '2018/2019', '2019', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '10', '2018/2019', '2019', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '10', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '10', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '10', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '10', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '10', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '10', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '11', '2018/2019', '2018', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '11', '2018/2019', '2018', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '11', '2018/2019', '2019', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '11', '2018/2019', '2019', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '11', '2018/2019', '2019', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '11', '2018/2019', '2019', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '11', '2018/2019', '2019', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '11', '2018/2019', '2019', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '11', '2018/2019', '2019', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '11', '2018/2019', '2019', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '11', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '11', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '11', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '11', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '11', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '11', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '1', '12', '2018/2019', '2018', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '2', '12', '2018/2019', '2019', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '3', '12', '2018/2019', '2019', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2018', '4', '12', '2018/2019', '2019', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '5', '12', '2018/2019', '2019', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '6', '12', '2018/2019', '2019', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2018', '7', '12', '2018/2019', '2019', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '8', '12', '2018/2019', '2019', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '9', '12', '2018/2019', '2019', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2018', '10', '12', '2018/2019', '2019', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '11', '12', '2018/2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '12', '12', '2018/2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '13', '12', '2018/2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '14', '12', '2018/2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '15', '12', '2018/2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2018', '16', '12', '2018/2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '1', '2019', '2019', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '1', '2019', '2019', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '1', '2019', '2019', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '1', '2019', '2019', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '1', '2019', '2019', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '1', '2019', '2019', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '1', '2019', '2019', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '1', '2019', '2019', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '1', '2019', '2019', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '1', '2019', '2019', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '1', '2019', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '1', '2019', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '1', '2019', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '1', '2019', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '1', '2019', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '1', '2019', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '2', '2019/2020', '2019', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '2', '2019/2020', '2019', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '2', '2019/2020', '2019', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '2', '2019/2020', '2019', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '2', '2019/2020', '2019', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '2', '2019/2020', '2019', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '2', '2019/2020', '2019', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '2', '2019/2020', '2019', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '2', '2019/2020', '2019', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '2', '2019/2020', '2019', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '2', '2019/2020', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '2', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '2', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '2', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '2', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '2', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '3', '2019/2020', '2019', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '3', '2019/2020', '2019', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '3', '2019/2020', '2019', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '3', '2019/2020', '2019', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '3', '2019/2020', '2019', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '3', '2019/2020', '2019', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '3', '2019/2020', '2019', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '3', '2019/2020', '2019', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '3', '2019/2020', '2019', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '3', '2019/2020', '2019', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '3', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '3', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '3', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '3', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '3', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '3', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '4', '2019/2020', '2019', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '4', '2019/2020', '2019', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '4', '2019/2020', '2019', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '4', '2019/2020', '2019', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '4', '2019/2020', '2019', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '4', '2019/2020', '2019', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '4', '2019/2020', '2019', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '4', '2019/2020', '2019', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '4', '2019/2020', '2019', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '4', '2019/2020', '2020', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '4', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '4', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '4', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '4', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '4', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '4', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '5', '2019/2020', '2019', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '5', '2019/2020', '2019', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '5', '2019/2020', '2019', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '5', '2019/2020', '2019', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '5', '2019/2020', '2019', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '5', '2019/2020', '2019', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '5', '2019/2020', '2019', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '5', '2019/2020', '2019', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '5', '2019/2020', '2020', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '5', '2019/2020', '2020', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '5', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '5', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '5', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '5', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '5', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '5', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '6', '2019/2020', '2019', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '6', '2019/2020', '2019', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '6', '2019/2020', '2019', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '6', '2019/2020', '2019', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '6', '2019/2020', '2019', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '6', '2019/2020', '2019', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '6', '2019/2020', '2019', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '6', '2019/2020', '2020', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '6', '2019/2020', '2020', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '6', '2019/2020', '2020', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '6', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '6', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '6', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '6', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '6', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '6', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '7', '2019/2020', '2019', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '7', '2019/2020', '2019', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '7', '2019/2020', '2019', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '7', '2019/2020', '2019', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '7', '2019/2020', '2019', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '7', '2019/2020', '2019', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '7', '2019/2020', '2020', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '7', '2019/2020', '2020', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '7', '2019/2020', '2020', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '7', '2019/2020', '2020', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '7', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '7', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '7', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '7', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '7', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '7', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '8', '2019/2020', '2019', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '8', '2019/2020', '2019', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '8', '2019/2020', '2019', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '8', '2019/2020', '2019', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '8', '2019/2020', '2019', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '8', '2019/2020', '2020', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '8', '2019/2020', '2020', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '8', '2019/2020', '2020', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '8', '2019/2020', '2020', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '8', '2019/2020', '2020', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '8', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '8', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('2019', '13', '8', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '8', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '8', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '8', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '9', '2019/2020', '2019', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '9', '2019/2020', '2019', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '9', '2019/2020', '2019', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '9', '2019/2020', '2019', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '9', '2019/2020', '2020', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '9', '2019/2020', '2020', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '9', '2019/2020', '2020', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '9', '2019/2020', '2020', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '9', '2019/2020', '2020', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '9', '2019/2020', '2020', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '9', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '9', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '9', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '9', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '9', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '9', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '10', '2019/2020', '2019', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '10', '2019/2020', '2019', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '10', '2019/2020', '2019', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '10', '2019/2020', '2020', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '10', '2019/2020', '2020', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '10', '2019/2020', '2020', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '10', '2019/2020', '2020', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '10', '2019/2020', '2020', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '10', '2019/2020', '2020', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '10', '2019/2020', '2020', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '10', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '10', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '10', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '10', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '10', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '10', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '11', '2019/2020', '2019', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '11', '2019/2020', '2019', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '11', '2019/2020', '2020', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '11', '2019/2020', '2020', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '11', '2019/2020', '2020', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '11', '2019/2020', '2020', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '11', '2019/2020', '2020', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '11', '2019/2020', '2020', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '11', '2019/2020', '2020', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '11', '2019/2020', '2020', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '11', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '11', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '11', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '11', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '11', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '11', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '1', '12', '2019/2020', '2019', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '2', '12', '2019/2020', '2020', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '3', '12', '2019/2020', '2020', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2019', '4', '12', '2019/2020', '2020', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '5', '12', '2019/2020', '2020', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '6', '12', '2019/2020', '2020', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2019', '7', '12', '2019/2020', '2020', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '8', '12', '2019/2020', '2020', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '9', '12', '2019/2020', '2020', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2019', '10', '12', '2019/2020', '2020', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '11', '12', '2019/2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '12', '12', '2019/2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '13', '12', '2019/2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '14', '12', '2019/2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '15', '12', '2019/2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2019', '16', '12', '2019/2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '1', '2020', '2020', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '1', '2020', '2020', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '1', '2020', '2020', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '1', '2020', '2020', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '1', '2020', '2020', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '1', '2020', '2020', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '1', '2020', '2020', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '1', '2020', '2020', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '1', '2020', '2020', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '1', '2020', '2020', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '1', '2020', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '1', '2020', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '1', '2020', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '1', '2020', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '1', '2020', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '1', '2020', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '2', '2020/2021', '2020', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '2', '2020/2021', '2020', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '2', '2020/2021', '2020', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '2', '2020/2021', '2020', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '2', '2020/2021', '2020', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '2', '2020/2021', '2020', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '2', '2020/2021', '2020', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '2', '2020/2021', '2020', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '2', '2020/2021', '2020', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '2', '2020/2021', '2020', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '2', '2020/2021', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '2', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '2', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '2', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '2', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '2', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '3', '2020/2021', '2020', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '3', '2020/2021', '2020', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '3', '2020/2021', '2020', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '3', '2020/2021', '2020', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '3', '2020/2021', '2020', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '3', '2020/2021', '2020', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '3', '2020/2021', '2020', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '3', '2020/2021', '2020', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '3', '2020/2021', '2020', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '3', '2020/2021', '2020', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '3', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '3', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '3', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '3', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '3', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '3', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '4', '2020/2021', '2020', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '4', '2020/2021', '2020', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '4', '2020/2021', '2020', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '4', '2020/2021', '2020', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '4', '2020/2021', '2020', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '4', '2020/2021', '2020', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '4', '2020/2021', '2020', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '4', '2020/2021', '2020', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '4', '2020/2021', '2020', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '4', '2020/2021', '2021', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '4', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '4', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '4', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '4', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '4', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '4', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '5', '2020/2021', '2020', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '5', '2020/2021', '2020', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '5', '2020/2021', '2020', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '5', '2020/2021', '2020', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '5', '2020/2021', '2020', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '5', '2020/2021', '2020', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '5', '2020/2021', '2020', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '5', '2020/2021', '2020', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '5', '2020/2021', '2021', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '5', '2020/2021', '2021', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '5', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '5', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '5', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '5', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '5', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '5', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '6', '2020/2021', '2020', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '6', '2020/2021', '2020', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '6', '2020/2021', '2020', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '6', '2020/2021', '2020', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '6', '2020/2021', '2020', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '6', '2020/2021', '2020', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '6', '2020/2021', '2020', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '6', '2020/2021', '2021', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '6', '2020/2021', '2021', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '6', '2020/2021', '2021', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '6', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '6', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '6', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '6', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '6', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '6', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '7', '2020/2021', '2020', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '7', '2020/2021', '2020', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '7', '2020/2021', '2020', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '7', '2020/2021', '2020', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '7', '2020/2021', '2020', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '7', '2020/2021', '2020', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '7', '2020/2021', '2021', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '7', '2020/2021', '2021', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '7', '2020/2021', '2021', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '7', '2020/2021', '2021', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '7', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '7', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '7', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '7', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '7', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '7', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '8', '2020/2021', '2020', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '8', '2020/2021', '2020', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '8', '2020/2021', '2020', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '8', '2020/2021', '2020', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '8', '2020/2021', '2020', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '8', '2020/2021', '2021', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '8', '2020/2021', '2021', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '8', '2020/2021', '2021', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '8', '2020/2021', '2021', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '8', '2020/2021', '2021', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '8', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '8', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '8', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '8', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '8', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '8', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '9', '2020/2021', '2020', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '9', '2020/2021', '2020', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '9', '2020/2021', '2020', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '9', '2020/2021', '2020', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '9', '2020/2021', '2021', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '9', '2020/2021', '2021', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '9', '2020/2021', '2021', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '9', '2020/2021', '2021', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '9', '2020/2021', '2021', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '9', '2020/2021', '2021', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '9', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '9', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '9', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '9', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '9', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '9', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '10', '2020/2021', '2020', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '10', '2020/2021', '2020', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '10', '2020/2021', '2020', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '10', '2020/2021', '2021', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '10', '2020/2021', '2021', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '10', '2020/2021', '2021', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '10', '2020/2021', '2021', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '10', '2020/2021', '2021', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '10', '2020/2021', '2021', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '10', '2020/2021', '2021', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '10', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '10', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '10', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '10', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '10', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '10', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '11', '2020/2021', '2020', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '11', '2020/2021', '2020', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '11', '2020/2021', '2021', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '11', '2020/2021', '2021', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '11', '2020/2021', '2021', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '11', '2020/2021', '2021', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '11', '2020/2021', '2021', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '11', '2020/2021', '2021', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '11', '2020/2021', '2021', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '11', '2020/2021', '2021', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '11', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '11', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '11', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '11', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '11', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '11', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '1', '12', '2020/2021', '2020', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '2', '12', '2020/2021', '2021', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '3', '12', '2020/2021', '2021', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2020', '4', '12', '2020/2021', '2021', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '5', '12', '2020/2021', '2021', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '6', '12', '2020/2021', '2021', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2020', '7', '12', '2020/2021', '2021', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '8', '12', '2020/2021', '2021', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '9', '12', '2020/2021', '2021', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2020', '10', '12', '2020/2021', '2021', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '11', '12', '2020/2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '12', '12', '2020/2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '13', '12', '2020/2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '14', '12', '2020/2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '15', '12', '2020/2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2020', '16', '12', '2020/2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '1', '2021', '2021', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '1', '2021', '2021', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '1', '2021', '2021', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '1', '2021', '2021', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '1', '2021', '2021', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '1', '2021', '2021', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '1', '2021', '2021', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '1', '2021', '2021', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '1', '2021', '2021', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '1', '2021', '2021', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '1', '2021', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '1', '2021', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '1', '2021', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '1', '2021', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '1', '2021', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '1', '2021', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '2', '2021/2022', '2021', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '2', '2021/2022', '2021', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '2', '2021/2022', '2021', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '2', '2021/2022', '2021', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '2', '2021/2022', '2021', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '2', '2021/2022', '2021', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '2', '2021/2022', '2021', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '2', '2021/2022', '2021', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '2', '2021/2022', '2021', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '2', '2021/2022', '2021', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '2', '2021/2022', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '2', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '2', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '2', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '2', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '2', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '3', '2021/2022', '2021', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '3', '2021/2022', '2021', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '3', '2021/2022', '2021', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '3', '2021/2022', '2021', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '3', '2021/2022', '2021', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '3', '2021/2022', '2021', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '3', '2021/2022', '2021', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '3', '2021/2022', '2021', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '3', '2021/2022', '2021', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '3', '2021/2022', '2021', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '3', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '3', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '3', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '3', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '3', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '3', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '4', '2021/2022', '2021', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '4', '2021/2022', '2021', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '4', '2021/2022', '2021', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '4', '2021/2022', '2021', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '4', '2021/2022', '2021', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '4', '2021/2022', '2021', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '4', '2021/2022', '2021', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '4', '2021/2022', '2021', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '4', '2021/2022', '2021', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '4', '2021/2022', '2022', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '4', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '4', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '4', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '4', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '4', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '4', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '5', '2021/2022', '2021', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '5', '2021/2022', '2021', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '5', '2021/2022', '2021', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '5', '2021/2022', '2021', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '5', '2021/2022', '2021', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '5', '2021/2022', '2021', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '5', '2021/2022', '2021', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '5', '2021/2022', '2021', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '5', '2021/2022', '2022', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '5', '2021/2022', '2022', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '5', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '5', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '5', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '5', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '5', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '5', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '6', '2021/2022', '2021', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '6', '2021/2022', '2021', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '6', '2021/2022', '2021', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '6', '2021/2022', '2021', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '6', '2021/2022', '2021', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '6', '2021/2022', '2021', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '6', '2021/2022', '2021', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '6', '2021/2022', '2022', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '6', '2021/2022', '2022', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '6', '2021/2022', '2022', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '6', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '6', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '6', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '6', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '6', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '6', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '7', '2021/2022', '2021', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '7', '2021/2022', '2021', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '7', '2021/2022', '2021', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '7', '2021/2022', '2021', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '7', '2021/2022', '2021', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '7', '2021/2022', '2021', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '7', '2021/2022', '2022', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '7', '2021/2022', '2022', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '7', '2021/2022', '2022', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '7', '2021/2022', '2022', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '7', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '7', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '7', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '7', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '7', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '7', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '8', '2021/2022', '2021', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '8', '2021/2022', '2021', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '8', '2021/2022', '2021', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '8', '2021/2022', '2021', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '8', '2021/2022', '2021', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '8', '2021/2022', '2022', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '8', '2021/2022', '2022', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '8', '2021/2022', '2022', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '8', '2021/2022', '2022', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '8', '2021/2022', '2022', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '8', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '8', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '8', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '8', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '8', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '8', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '9', '2021/2022', '2021', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '9', '2021/2022', '2021', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '9', '2021/2022', '2021', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '9', '2021/2022', '2021', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '9', '2021/2022', '2022', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '9', '2021/2022', '2022', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '9', '2021/2022', '2022', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '9', '2021/2022', '2022', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '9', '2021/2022', '2022', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '9', '2021/2022', '2022', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '9', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '9', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '9', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '9', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '9', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '9', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '10', '2021/2022', '2021', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '10', '2021/2022', '2021', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '10', '2021/2022', '2021', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '10', '2021/2022', '2022', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '10', '2021/2022', '2022', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '10', '2021/2022', '2022', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '10', '2021/2022', '2022', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '10', '2021/2022', '2022', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '10', '2021/2022', '2022', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '10', '2021/2022', '2022', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '10', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '10', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '10', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '10', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '10', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '10', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '11', '2021/2022', '2021', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '11', '2021/2022', '2021', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '11', '2021/2022', '2022', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '11', '2021/2022', '2022', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '11', '2021/2022', '2022', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '11', '2021/2022', '2022', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '11', '2021/2022', '2022', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '11', '2021/2022', '2022', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '11', '2021/2022', '2022', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '11', '2021/2022', '2022', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '11', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '11', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '11', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '11', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '11', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '11', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '1', '12', '2021/2022', '2021', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '2', '12', '2021/2022', '2022', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '3', '12', '2021/2022', '2022', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2021', '4', '12', '2021/2022', '2022', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '5', '12', '2021/2022', '2022', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '6', '12', '2021/2022', '2022', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2021', '7', '12', '2021/2022', '2022', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '8', '12', '2021/2022', '2022', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '9', '12', '2021/2022', '2022', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2021', '10', '12', '2021/2022', '2022', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '11', '12', '2021/2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '12', '12', '2021/2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '13', '12', '2021/2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '14', '12', '2021/2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '15', '12', '2021/2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2021', '16', '12', '2021/2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '1', '2022', '2022', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '1', '2022', '2022', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '1', '2022', '2022', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '1', '2022', '2022', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '1', '2022', '2022', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '1', '2022', '2022', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '1', '2022', '2022', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '1', '2022', '2022', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '1', '2022', '2022', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '1', '2022', '2022', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '1', '2022', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '1', '2022', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '1', '2022', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '1', '2022', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '1', '2022', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '1', '2022', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '2', '2022/2023', '2022', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '2', '2022/2023', '2022', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '2', '2022/2023', '2022', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '2', '2022/2023', '2022', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '2', '2022/2023', '2022', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '2', '2022/2023', '2022', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '2', '2022/2023', '2022', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '2', '2022/2023', '2022', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '2', '2022/2023', '2022', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '2', '2022/2023', '2022', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '2', '2022/2023', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '2', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '2', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '2', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '2', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '2', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '3', '2022/2023', '2022', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '3', '2022/2023', '2022', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '3', '2022/2023', '2022', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '3', '2022/2023', '2022', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '3', '2022/2023', '2022', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '3', '2022/2023', '2022', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '3', '2022/2023', '2022', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '3', '2022/2023', '2022', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '3', '2022/2023', '2022', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '3', '2022/2023', '2022', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '3', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '3', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '3', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '3', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '3', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '3', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '4', '2022/2023', '2022', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '4', '2022/2023', '2022', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '4', '2022/2023', '2022', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '4', '2022/2023', '2022', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '4', '2022/2023', '2022', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '4', '2022/2023', '2022', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '4', '2022/2023', '2022', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '4', '2022/2023', '2022', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '4', '2022/2023', '2022', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '4', '2022/2023', '2023', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '4', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '4', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '4', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '4', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '4', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '4', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '5', '2022/2023', '2022', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '5', '2022/2023', '2022', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '5', '2022/2023', '2022', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '5', '2022/2023', '2022', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '5', '2022/2023', '2022', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '5', '2022/2023', '2022', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '5', '2022/2023', '2022', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '5', '2022/2023', '2022', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '5', '2022/2023', '2023', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '5', '2022/2023', '2023', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '5', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '5', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '5', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '5', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '5', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '5', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '6', '2022/2023', '2022', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '6', '2022/2023', '2022', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '6', '2022/2023', '2022', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '6', '2022/2023', '2022', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '6', '2022/2023', '2022', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '6', '2022/2023', '2022', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '6', '2022/2023', '2022', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '6', '2022/2023', '2023', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '6', '2022/2023', '2023', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '6', '2022/2023', '2023', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '6', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '6', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '6', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '6', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '6', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '6', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '7', '2022/2023', '2022', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '7', '2022/2023', '2022', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '7', '2022/2023', '2022', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '7', '2022/2023', '2022', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '7', '2022/2023', '2022', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '7', '2022/2023', '2022', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '7', '2022/2023', '2023', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '7', '2022/2023', '2023', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '7', '2022/2023', '2023', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '7', '2022/2023', '2023', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '7', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '7', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '7', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '7', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '7', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '7', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '8', '2022/2023', '2022', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '8', '2022/2023', '2022', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '8', '2022/2023', '2022', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '8', '2022/2023', '2022', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '8', '2022/2023', '2022', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '8', '2022/2023', '2023', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '8', '2022/2023', '2023', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '8', '2022/2023', '2023', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '8', '2022/2023', '2023', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '8', '2022/2023', '2023', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '8', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '8', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '8', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '8', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '8', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '8', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '9', '2022/2023', '2022', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '9', '2022/2023', '2022', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '9', '2022/2023', '2022', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '9', '2022/2023', '2022', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '9', '2022/2023', '2023', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '9', '2022/2023', '2023', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '9', '2022/2023', '2023', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '9', '2022/2023', '2023', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '9', '2022/2023', '2023', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '9', '2022/2023', '2023', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '9', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '9', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '9', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '9', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '9', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '9', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '10', '2022/2023', '2022', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '10', '2022/2023', '2022', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '10', '2022/2023', '2022', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '10', '2022/2023', '2023', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '10', '2022/2023', '2023', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '10', '2022/2023', '2023', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '10', '2022/2023', '2023', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '10', '2022/2023', '2023', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '10', '2022/2023', '2023', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '10', '2022/2023', '2023', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '10', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '10', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '10', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '10', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '10', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '10', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '11', '2022/2023', '2022', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '11', '2022/2023', '2022', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '11', '2022/2023', '2023', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '11', '2022/2023', '2023', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '11', '2022/2023', '2023', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '11', '2022/2023', '2023', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '11', '2022/2023', '2023', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '11', '2022/2023', '2023', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '11', '2022/2023', '2023', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '11', '2022/2023', '2023', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '11', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '11', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '11', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '11', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '11', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '11', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '1', '12', '2022/2023', '2022', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '2', '12', '2022/2023', '2023', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '3', '12', '2022/2023', '2023', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2022', '4', '12', '2022/2023', '2023', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '5', '12', '2022/2023', '2023', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '6', '12', '2022/2023', '2023', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2022', '7', '12', '2022/2023', '2023', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '8', '12', '2022/2023', '2023', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '9', '12', '2022/2023', '2023', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2022', '10', '12', '2022/2023', '2023', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '11', '12', '2022/2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '12', '12', '2022/2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '13', '12', '2022/2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '14', '12', '2022/2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '15', '12', '2022/2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2022', '16', '12', '2022/2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '1', '2023', '2023', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '1', '2023', '2023', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '1', '2023', '2023', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '1', '2023', '2023', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '1', '2023', '2023', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '1', '2023', '2023', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '1', '2023', '2023', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '1', '2023', '2023', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '1', '2023', '2023', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '1', '2023', '2023', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '1', '2023', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '1', '2023', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '1', '2023', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '1', '2023', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '1', '2023', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '1', '2023', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '2', '2023/2024', '2023', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '2', '2023/2024', '2023', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '2', '2023/2024', '2023', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '2', '2023/2024', '2023', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '2', '2023/2024', '2023', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '2', '2023/2024', '2023', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '2', '2023/2024', '2023', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '2', '2023/2024', '2023', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '2', '2023/2024', '2023', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '2', '2023/2024', '2023', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '2', '2023/2024', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '2', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '2', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '2', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '2', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '2', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '3', '2023/2024', '2023', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '3', '2023/2024', '2023', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '3', '2023/2024', '2023', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '3', '2023/2024', '2023', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '3', '2023/2024', '2023', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '3', '2023/2024', '2023', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '3', '2023/2024', '2023', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '3', '2023/2024', '2023', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '3', '2023/2024', '2023', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '3', '2023/2024', '2023', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '3', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '3', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '3', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '3', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '3', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '3', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '4', '2023/2024', '2023', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '4', '2023/2024', '2023', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '4', '2023/2024', '2023', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '4', '2023/2024', '2023', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '4', '2023/2024', '2023', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '4', '2023/2024', '2023', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '4', '2023/2024', '2023', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '4', '2023/2024', '2023', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '4', '2023/2024', '2023', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '4', '2023/2024', '2024', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '4', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '4', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '4', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '4', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '4', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '4', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '5', '2023/2024', '2023', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '5', '2023/2024', '2023', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '5', '2023/2024', '2023', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '5', '2023/2024', '2023', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '5', '2023/2024', '2023', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '5', '2023/2024', '2023', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '5', '2023/2024', '2023', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '5', '2023/2024', '2023', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '5', '2023/2024', '2024', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '5', '2023/2024', '2024', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '5', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '5', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '5', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '5', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '5', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '5', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '6', '2023/2024', '2023', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '6', '2023/2024', '2023', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '6', '2023/2024', '2023', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '6', '2023/2024', '2023', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '6', '2023/2024', '2023', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '6', '2023/2024', '2023', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '6', '2023/2024', '2023', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '6', '2023/2024', '2024', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '6', '2023/2024', '2024', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '6', '2023/2024', '2024', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '6', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '6', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '6', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '6', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '6', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '6', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '7', '2023/2024', '2023', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '7', '2023/2024', '2023', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '7', '2023/2024', '2023', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '7', '2023/2024', '2023', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '7', '2023/2024', '2023', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '7', '2023/2024', '2023', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '7', '2023/2024', '2024', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '7', '2023/2024', '2024', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '7', '2023/2024', '2024', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '7', '2023/2024', '2024', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '7', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '7', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '7', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '7', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '7', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '7', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '8', '2023/2024', '2023', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '8', '2023/2024', '2023', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '8', '2023/2024', '2023', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '8', '2023/2024', '2023', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '8', '2023/2024', '2023', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '8', '2023/2024', '2024', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '8', '2023/2024', '2024', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '8', '2023/2024', '2024', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '8', '2023/2024', '2024', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '8', '2023/2024', '2024', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '8', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '8', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '8', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '8', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '8', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '8', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '9', '2023/2024', '2023', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '9', '2023/2024', '2023', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '9', '2023/2024', '2023', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '9', '2023/2024', '2023', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '9', '2023/2024', '2024', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '9', '2023/2024', '2024', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '9', '2023/2024', '2024', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '9', '2023/2024', '2024', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '9', '2023/2024', '2024', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '9', '2023/2024', '2024', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '9', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '9', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '9', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '9', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '9', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '9', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '10', '2023/2024', '2023', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '10', '2023/2024', '2023', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '10', '2023/2024', '2023', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '10', '2023/2024', '2024', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '10', '2023/2024', '2024', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '10', '2023/2024', '2024', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '10', '2023/2024', '2024', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '10', '2023/2024', '2024', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '10', '2023/2024', '2024', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '10', '2023/2024', '2024', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '10', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '10', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '10', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '10', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '10', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '10', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '11', '2023/2024', '2023', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '11', '2023/2024', '2023', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '11', '2023/2024', '2024', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '11', '2023/2024', '2024', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '11', '2023/2024', '2024', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '11', '2023/2024', '2024', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '11', '2023/2024', '2024', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '11', '2023/2024', '2024', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '11', '2023/2024', '2024', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '11', '2023/2024', '2024', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '11', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '11', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '11', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '11', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '11', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '11', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '1', '12', '2023/2024', '2023', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '2', '12', '2023/2024', '2024', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '3', '12', '2023/2024', '2024', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2023', '4', '12', '2023/2024', '2024', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '5', '12', '2023/2024', '2024', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '6', '12', '2023/2024', '2024', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2023', '7', '12', '2023/2024', '2024', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '8', '12', '2023/2024', '2024', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '9', '12', '2023/2024', '2024', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2023', '10', '12', '2023/2024', '2024', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '11', '12', '2023/2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '12', '12', '2023/2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '13', '12', '2023/2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '14', '12', '2023/2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '15', '12', '2023/2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2023', '16', '12', '2023/2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '1', '2024', '2024', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '1', '2024', '2024', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '1', '2024', '2024', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '1', '2024', '2024', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '1', '2024', '2024', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '1', '2024', '2024', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '1', '2024', '2024', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '1', '2024', '2024', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '1', '2024', '2024', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '1', '2024', '2024', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '1', '2024', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '1', '2024', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '1', '2024', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '1', '2024', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '1', '2024', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '1', '2024', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '2', '2024/2025', '2024', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '2', '2024/2025', '2024', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '2', '2024/2025', '2024', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '2', '2024/2025', '2024', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '2', '2024/2025', '2024', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '2', '2024/2025', '2024', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '2', '2024/2025', '2024', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '2', '2024/2025', '2024', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '2', '2024/2025', '2024', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '2', '2024/2025', '2024', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '2', '2024/2025', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '2', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '2', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '2', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '2', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '2', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '3', '2024/2025', '2024', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '3', '2024/2025', '2024', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '3', '2024/2025', '2024', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '3', '2024/2025', '2024', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '3', '2024/2025', '2024', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '3', '2024/2025', '2024', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '3', '2024/2025', '2024', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '3', '2024/2025', '2024', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '3', '2024/2025', '2024', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '3', '2024/2025', '2024', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '3', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '3', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '3', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '3', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '3', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '3', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '4', '2024/2025', '2024', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '4', '2024/2025', '2024', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '4', '2024/2025', '2024', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '4', '2024/2025', '2024', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '4', '2024/2025', '2024', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '4', '2024/2025', '2024', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '4', '2024/2025', '2024', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '4', '2024/2025', '2024', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '4', '2024/2025', '2024', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '4', '2024/2025', '2025', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '4', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '4', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '4', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '4', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '4', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '4', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '5', '2024/2025', '2024', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '5', '2024/2025', '2024', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '5', '2024/2025', '2024', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '5', '2024/2025', '2024', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '5', '2024/2025', '2024', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '5', '2024/2025', '2024', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '5', '2024/2025', '2024', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '5', '2024/2025', '2024', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '5', '2024/2025', '2025', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '5', '2024/2025', '2025', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '5', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '5', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '5', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '5', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '5', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '5', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '6', '2024/2025', '2024', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '6', '2024/2025', '2024', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '6', '2024/2025', '2024', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '6', '2024/2025', '2024', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '6', '2024/2025', '2024', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '6', '2024/2025', '2024', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '6', '2024/2025', '2024', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '6', '2024/2025', '2025', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '6', '2024/2025', '2025', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '6', '2024/2025', '2025', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '6', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '6', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '6', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '6', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '6', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '6', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '7', '2024/2025', '2024', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '7', '2024/2025', '2024', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '7', '2024/2025', '2024', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '7', '2024/2025', '2024', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '7', '2024/2025', '2024', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '7', '2024/2025', '2024', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '7', '2024/2025', '2025', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '7', '2024/2025', '2025', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '7', '2024/2025', '2025', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '7', '2024/2025', '2025', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '7', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '7', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '7', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '7', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '7', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '7', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '8', '2024/2025', '2024', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '8', '2024/2025', '2024', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '8', '2024/2025', '2024', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '8', '2024/2025', '2024', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '8', '2024/2025', '2024', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '8', '2024/2025', '2025', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '8', '2024/2025', '2025', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '8', '2024/2025', '2025', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '8', '2024/2025', '2025', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '8', '2024/2025', '2025', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '8', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '8', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '8', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '8', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '8', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '8', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '9', '2024/2025', '2024', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '9', '2024/2025', '2024', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '9', '2024/2025', '2024', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '9', '2024/2025', '2024', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '9', '2024/2025', '2025', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '9', '2024/2025', '2025', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '9', '2024/2025', '2025', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '9', '2024/2025', '2025', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '9', '2024/2025', '2025', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '9', '2024/2025', '2025', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '9', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '9', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '9', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '9', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '9', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '9', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '10', '2024/2025', '2024', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '10', '2024/2025', '2024', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '10', '2024/2025', '2024', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '10', '2024/2025', '2025', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '10', '2024/2025', '2025', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '10', '2024/2025', '2025', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '10', '2024/2025', '2025', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '10', '2024/2025', '2025', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '10', '2024/2025', '2025', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '10', '2024/2025', '2025', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '10', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '10', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '10', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '10', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '10', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '10', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '11', '2024/2025', '2024', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '11', '2024/2025', '2024', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '11', '2024/2025', '2025', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '11', '2024/2025', '2025', '2', 'QUARTAL2', 'HALBJAHR1')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('2024', '5', '11', '2024/2025', '2025', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '11', '2024/2025', '2025', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '11', '2024/2025', '2025', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '11', '2024/2025', '2025', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '11', '2024/2025', '2025', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '11', '2024/2025', '2025', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '11', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '11', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '11', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '11', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '11', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '11', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '1', '12', '2024/2025', '2024', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '2', '12', '2024/2025', '2025', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '3', '12', '2024/2025', '2025', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2024', '4', '12', '2024/2025', '2025', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '5', '12', '2024/2025', '2025', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '6', '12', '2024/2025', '2025', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2024', '7', '12', '2024/2025', '2025', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '8', '12', '2024/2025', '2025', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '9', '12', '2024/2025', '2025', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2024', '10', '12', '2024/2025', '2025', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '11', '12', '2024/2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '12', '12', '2024/2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '13', '12', '2024/2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '14', '12', '2024/2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '15', '12', '2024/2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2024', '16', '12', '2024/2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '1', '2025', '2025', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '1', '2025', '2025', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '1', '2025', '2025', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '1', '2025', '2025', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '1', '2025', '2025', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '1', '2025', '2025', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '1', '2025', '2025', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '1', '2025', '2025', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '1', '2025', '2025', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '1', '2025', '2025', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '1', '2025', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '1', '2025', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '1', '2025', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '1', '2025', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '1', '2025', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '1', '2025', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '2', '2025/2026', '2025', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '2', '2025/2026', '2025', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '2', '2025/2026', '2025', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '2', '2025/2026', '2025', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '2', '2025/2026', '2025', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '2', '2025/2026', '2025', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '2', '2025/2026', '2025', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '2', '2025/2026', '2025', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '2', '2025/2026', '2025', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '2', '2025/2026', '2025', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '2', '2025/2026', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '2', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '2', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '2', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '2', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '2', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '3', '2025/2026', '2025', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '3', '2025/2026', '2025', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '3', '2025/2026', '2025', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '3', '2025/2026', '2025', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '3', '2025/2026', '2025', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '3', '2025/2026', '2025', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '3', '2025/2026', '2025', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '3', '2025/2026', '2025', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '3', '2025/2026', '2025', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '3', '2025/2026', '2025', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '3', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '3', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '3', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '3', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '3', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '3', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '4', '2025/2026', '2025', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '4', '2025/2026', '2025', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '4', '2025/2026', '2025', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '4', '2025/2026', '2025', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '4', '2025/2026', '2025', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '4', '2025/2026', '2025', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '4', '2025/2026', '2025', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '4', '2025/2026', '2025', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '4', '2025/2026', '2025', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '4', '2025/2026', '2026', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '4', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '4', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '4', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '4', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '4', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '4', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '5', '2025/2026', '2025', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '5', '2025/2026', '2025', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '5', '2025/2026', '2025', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '5', '2025/2026', '2025', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '5', '2025/2026', '2025', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '5', '2025/2026', '2025', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '5', '2025/2026', '2025', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '5', '2025/2026', '2025', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '5', '2025/2026', '2026', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '5', '2025/2026', '2026', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '5', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '5', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '5', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '5', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '5', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '5', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '6', '2025/2026', '2025', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '6', '2025/2026', '2025', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '6', '2025/2026', '2025', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '6', '2025/2026', '2025', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '6', '2025/2026', '2025', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '6', '2025/2026', '2025', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '6', '2025/2026', '2025', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '6', '2025/2026', '2026', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '6', '2025/2026', '2026', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '6', '2025/2026', '2026', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '6', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '6', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '6', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '6', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '6', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '6', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '7', '2025/2026', '2025', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '7', '2025/2026', '2025', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '7', '2025/2026', '2025', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '7', '2025/2026', '2025', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '7', '2025/2026', '2025', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '7', '2025/2026', '2025', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '7', '2025/2026', '2026', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '7', '2025/2026', '2026', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '7', '2025/2026', '2026', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '7', '2025/2026', '2026', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '7', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '7', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '7', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '7', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '7', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '7', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '8', '2025/2026', '2025', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '8', '2025/2026', '2025', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '8', '2025/2026', '2025', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '8', '2025/2026', '2025', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '8', '2025/2026', '2025', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '8', '2025/2026', '2026', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '8', '2025/2026', '2026', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '8', '2025/2026', '2026', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '8', '2025/2026', '2026', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '8', '2025/2026', '2026', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '8', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '8', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '8', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '8', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '8', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '8', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '9', '2025/2026', '2025', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '9', '2025/2026', '2025', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '9', '2025/2026', '2025', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '9', '2025/2026', '2025', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '9', '2025/2026', '2026', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '9', '2025/2026', '2026', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '9', '2025/2026', '2026', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '9', '2025/2026', '2026', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '9', '2025/2026', '2026', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '9', '2025/2026', '2026', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '9', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '9', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '9', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '9', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '9', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '9', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '10', '2025/2026', '2025', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '10', '2025/2026', '2025', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '10', '2025/2026', '2025', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '10', '2025/2026', '2026', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '10', '2025/2026', '2026', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '10', '2025/2026', '2026', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '10', '2025/2026', '2026', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '10', '2025/2026', '2026', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '10', '2025/2026', '2026', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '10', '2025/2026', '2026', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '10', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '10', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '10', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '10', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '10', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '10', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '11', '2025/2026', '2025', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '11', '2025/2026', '2025', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '11', '2025/2026', '2026', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '11', '2025/2026', '2026', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '11', '2025/2026', '2026', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '11', '2025/2026', '2026', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '11', '2025/2026', '2026', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '11', '2025/2026', '2026', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '11', '2025/2026', '2026', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '11', '2025/2026', '2026', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '11', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '11', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '11', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '11', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '11', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '11', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '1', '12', '2025/2026', '2025', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '2', '12', '2025/2026', '2026', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '3', '12', '2025/2026', '2026', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2025', '4', '12', '2025/2026', '2026', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '5', '12', '2025/2026', '2026', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '6', '12', '2025/2026', '2026', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2025', '7', '12', '2025/2026', '2026', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '8', '12', '2025/2026', '2026', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '9', '12', '2025/2026', '2026', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2025', '10', '12', '2025/2026', '2026', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '11', '12', '2025/2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '12', '12', '2025/2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '13', '12', '2025/2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '14', '12', '2025/2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '15', '12', '2025/2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2025', '16', '12', '2025/2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '1', '2026', '2026', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '1', '2026', '2026', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '1', '2026', '2026', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '1', '2026', '2026', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '1', '2026', '2026', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '1', '2026', '2026', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '1', '2026', '2026', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '1', '2026', '2026', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '1', '2026', '2026', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '1', '2026', '2026', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '1', '2026', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '1', '2026', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '1', '2026', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '1', '2026', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '1', '2026', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '1', '2026', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '2', '2026/2027', '2026', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '2', '2026/2027', '2026', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '2', '2026/2027', '2026', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '2', '2026/2027', '2026', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '2', '2026/2027', '2026', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '2', '2026/2027', '2026', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '2', '2026/2027', '2026', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '2', '2026/2027', '2026', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '2', '2026/2027', '2026', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '2', '2026/2027', '2026', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '2', '2026/2027', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '2', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '2', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '2', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '2', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '2', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '3', '2026/2027', '2026', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '3', '2026/2027', '2026', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '3', '2026/2027', '2026', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '3', '2026/2027', '2026', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '3', '2026/2027', '2026', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '3', '2026/2027', '2026', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '3', '2026/2027', '2026', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '3', '2026/2027', '2026', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '3', '2026/2027', '2026', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '3', '2026/2027', '2026', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '3', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '3', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '3', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '3', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '3', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '3', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '4', '2026/2027', '2026', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '4', '2026/2027', '2026', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '4', '2026/2027', '2026', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '4', '2026/2027', '2026', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '4', '2026/2027', '2026', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '4', '2026/2027', '2026', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '4', '2026/2027', '2026', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '4', '2026/2027', '2026', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '4', '2026/2027', '2026', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '4', '2026/2027', '2027', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '4', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '4', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '4', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '4', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '4', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '4', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '5', '2026/2027', '2026', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '5', '2026/2027', '2026', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '5', '2026/2027', '2026', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '5', '2026/2027', '2026', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '5', '2026/2027', '2026', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '5', '2026/2027', '2026', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '5', '2026/2027', '2026', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '5', '2026/2027', '2026', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '5', '2026/2027', '2027', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '5', '2026/2027', '2027', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '5', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '5', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '5', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '5', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '5', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '5', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '6', '2026/2027', '2026', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '6', '2026/2027', '2026', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '6', '2026/2027', '2026', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '6', '2026/2027', '2026', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '6', '2026/2027', '2026', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '6', '2026/2027', '2026', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '6', '2026/2027', '2026', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '6', '2026/2027', '2027', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '6', '2026/2027', '2027', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '6', '2026/2027', '2027', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '6', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '6', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '6', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '6', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '6', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '6', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '7', '2026/2027', '2026', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '7', '2026/2027', '2026', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '7', '2026/2027', '2026', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '7', '2026/2027', '2026', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '7', '2026/2027', '2026', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '7', '2026/2027', '2026', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '7', '2026/2027', '2027', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '7', '2026/2027', '2027', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '7', '2026/2027', '2027', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '7', '2026/2027', '2027', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '7', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '7', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '7', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '7', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '7', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '7', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '8', '2026/2027', '2026', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '8', '2026/2027', '2026', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '8', '2026/2027', '2026', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '8', '2026/2027', '2026', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '8', '2026/2027', '2026', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '8', '2026/2027', '2027', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '8', '2026/2027', '2027', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '8', '2026/2027', '2027', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '8', '2026/2027', '2027', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '8', '2026/2027', '2027', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '8', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '8', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '8', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '8', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '8', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '8', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '9', '2026/2027', '2026', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '9', '2026/2027', '2026', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '9', '2026/2027', '2026', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '9', '2026/2027', '2026', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '9', '2026/2027', '2027', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '9', '2026/2027', '2027', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '9', '2026/2027', '2027', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '9', '2026/2027', '2027', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '9', '2026/2027', '2027', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '9', '2026/2027', '2027', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '9', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '9', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '9', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '9', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '9', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '9', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '10', '2026/2027', '2026', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '10', '2026/2027', '2026', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '10', '2026/2027', '2026', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '10', '2026/2027', '2027', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '10', '2026/2027', '2027', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '10', '2026/2027', '2027', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '10', '2026/2027', '2027', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '10', '2026/2027', '2027', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '10', '2026/2027', '2027', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '10', '2026/2027', '2027', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '10', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '10', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '10', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '10', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '10', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '10', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '11', '2026/2027', '2026', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '11', '2026/2027', '2026', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '11', '2026/2027', '2027', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '11', '2026/2027', '2027', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '11', '2026/2027', '2027', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '11', '2026/2027', '2027', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '11', '2026/2027', '2027', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '11', '2026/2027', '2027', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '11', '2026/2027', '2027', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '11', '2026/2027', '2027', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '11', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '11', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '11', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '11', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '11', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '11', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '1', '12', '2026/2027', '2026', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '2', '12', '2026/2027', '2027', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '3', '12', '2026/2027', '2027', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2026', '4', '12', '2026/2027', '2027', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '5', '12', '2026/2027', '2027', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '6', '12', '2026/2027', '2027', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2026', '7', '12', '2026/2027', '2027', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '8', '12', '2026/2027', '2027', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '9', '12', '2026/2027', '2027', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2026', '10', '12', '2026/2027', '2027', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '11', '12', '2026/2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '12', '12', '2026/2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '13', '12', '2026/2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '14', '12', '2026/2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '15', '12', '2026/2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2026', '16', '12', '2026/2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '1', '2027', '2027', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '1', '2027', '2027', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '1', '2027', '2027', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '1', '2027', '2027', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '1', '2027', '2027', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '1', '2027', '2027', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '1', '2027', '2027', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '1', '2027', '2027', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '1', '2027', '2027', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '1', '2027', '2027', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '1', '2027', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '1', '2027', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '1', '2027', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '1', '2027', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '1', '2027', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '1', '2027', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '2', '2027/2028', '2027', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '2', '2027/2028', '2027', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '2', '2027/2028', '2027', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '2', '2027/2028', '2027', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '2', '2027/2028', '2027', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '2', '2027/2028', '2027', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '2', '2027/2028', '2027', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '2', '2027/2028', '2027', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '2', '2027/2028', '2027', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '2', '2027/2028', '2027', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '2', '2027/2028', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '2', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '2', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '2', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '2', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '2', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '3', '2027/2028', '2027', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '3', '2027/2028', '2027', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '3', '2027/2028', '2027', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '3', '2027/2028', '2027', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '3', '2027/2028', '2027', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '3', '2027/2028', '2027', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '3', '2027/2028', '2027', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '3', '2027/2028', '2027', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '3', '2027/2028', '2027', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '3', '2027/2028', '2027', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '3', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '3', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '3', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '3', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '3', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '3', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '4', '2027/2028', '2027', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '4', '2027/2028', '2027', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '4', '2027/2028', '2027', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '4', '2027/2028', '2027', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '4', '2027/2028', '2027', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '4', '2027/2028', '2027', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '4', '2027/2028', '2027', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '4', '2027/2028', '2027', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '4', '2027/2028', '2027', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '4', '2027/2028', '2028', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '4', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '4', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '4', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '4', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '4', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '4', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '5', '2027/2028', '2027', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '5', '2027/2028', '2027', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '5', '2027/2028', '2027', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '5', '2027/2028', '2027', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '5', '2027/2028', '2027', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '5', '2027/2028', '2027', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '5', '2027/2028', '2027', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '5', '2027/2028', '2027', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '5', '2027/2028', '2028', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '5', '2027/2028', '2028', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '5', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '5', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '5', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '5', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '5', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '5', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '6', '2027/2028', '2027', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '6', '2027/2028', '2027', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '6', '2027/2028', '2027', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '6', '2027/2028', '2027', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '6', '2027/2028', '2027', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '6', '2027/2028', '2027', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '6', '2027/2028', '2027', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '6', '2027/2028', '2028', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '6', '2027/2028', '2028', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '6', '2027/2028', '2028', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '6', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '6', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '6', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '6', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '6', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '6', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '7', '2027/2028', '2027', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '7', '2027/2028', '2027', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '7', '2027/2028', '2027', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '7', '2027/2028', '2027', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '7', '2027/2028', '2027', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '7', '2027/2028', '2027', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '7', '2027/2028', '2028', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '7', '2027/2028', '2028', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '7', '2027/2028', '2028', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '7', '2027/2028', '2028', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '7', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '7', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '7', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '7', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '7', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '7', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '8', '2027/2028', '2027', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '8', '2027/2028', '2027', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '8', '2027/2028', '2027', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '8', '2027/2028', '2027', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '8', '2027/2028', '2027', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '8', '2027/2028', '2028', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '8', '2027/2028', '2028', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '8', '2027/2028', '2028', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '8', '2027/2028', '2028', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '8', '2027/2028', '2028', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '8', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '8', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '8', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '8', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '8', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '8', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '9', '2027/2028', '2027', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '9', '2027/2028', '2027', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '9', '2027/2028', '2027', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '9', '2027/2028', '2027', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '9', '2027/2028', '2028', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '9', '2027/2028', '2028', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '9', '2027/2028', '2028', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '9', '2027/2028', '2028', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '9', '2027/2028', '2028', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '9', '2027/2028', '2028', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '9', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '9', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '9', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '9', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '9', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '9', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '10', '2027/2028', '2027', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '10', '2027/2028', '2027', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '10', '2027/2028', '2027', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '10', '2027/2028', '2028', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '10', '2027/2028', '2028', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '10', '2027/2028', '2028', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '10', '2027/2028', '2028', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '10', '2027/2028', '2028', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '10', '2027/2028', '2028', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '10', '2027/2028', '2028', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '10', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '10', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '10', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '10', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '10', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '10', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '11', '2027/2028', '2027', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '11', '2027/2028', '2027', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '11', '2027/2028', '2028', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '11', '2027/2028', '2028', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '11', '2027/2028', '2028', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '11', '2027/2028', '2028', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '11', '2027/2028', '2028', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '11', '2027/2028', '2028', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '11', '2027/2028', '2028', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '11', '2027/2028', '2028', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '11', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '11', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '11', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '11', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '11', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '11', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '1', '12', '2027/2028', '2027', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '2', '12', '2027/2028', '2028', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '3', '12', '2027/2028', '2028', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2027', '4', '12', '2027/2028', '2028', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '5', '12', '2027/2028', '2028', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '6', '12', '2027/2028', '2028', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2027', '7', '12', '2027/2028', '2028', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '8', '12', '2027/2028', '2028', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '9', '12', '2027/2028', '2028', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2027', '10', '12', '2027/2028', '2028', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '11', '12', '2027/2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '12', '12', '2027/2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '13', '12', '2027/2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '14', '12', '2027/2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '15', '12', '2027/2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2027', '16', '12', '2027/2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '1', '2028', '2028', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '1', '2028', '2028', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '1', '2028', '2028', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '1', '2028', '2028', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '1', '2028', '2028', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '1', '2028', '2028', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '1', '2028', '2028', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '1', '2028', '2028', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '1', '2028', '2028', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '1', '2028', '2028', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '1', '2028', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '1', '2028', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '1', '2028', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '1', '2028', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '1', '2028', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '1', '2028', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '2', '2028/2029', '2028', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '2', '2028/2029', '2028', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '2', '2028/2029', '2028', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '2', '2028/2029', '2028', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '2', '2028/2029', '2028', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '2', '2028/2029', '2028', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '2', '2028/2029', '2028', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '2', '2028/2029', '2028', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '2', '2028/2029', '2028', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '2', '2028/2029', '2028', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '2', '2028/2029', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '2', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '2', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '2', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '2', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '2', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '3', '2028/2029', '2028', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '3', '2028/2029', '2028', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '3', '2028/2029', '2028', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '3', '2028/2029', '2028', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '3', '2028/2029', '2028', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '3', '2028/2029', '2028', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '3', '2028/2029', '2028', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '3', '2028/2029', '2028', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '3', '2028/2029', '2028', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '3', '2028/2029', '2028', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '3', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '3', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '3', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '3', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '3', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '3', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '4', '2028/2029', '2028', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '4', '2028/2029', '2028', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '4', '2028/2029', '2028', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '4', '2028/2029', '2028', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '4', '2028/2029', '2028', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '4', '2028/2029', '2028', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '4', '2028/2029', '2028', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '4', '2028/2029', '2028', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '4', '2028/2029', '2028', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '4', '2028/2029', '2029', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '4', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '4', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '4', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '4', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '4', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '4', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '5', '2028/2029', '2028', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '5', '2028/2029', '2028', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '5', '2028/2029', '2028', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '5', '2028/2029', '2028', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '5', '2028/2029', '2028', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '5', '2028/2029', '2028', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '5', '2028/2029', '2028', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '5', '2028/2029', '2028', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '5', '2028/2029', '2029', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '5', '2028/2029', '2029', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '5', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '5', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '5', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '5', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '5', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '5', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '6', '2028/2029', '2028', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '6', '2028/2029', '2028', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '6', '2028/2029', '2028', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '6', '2028/2029', '2028', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '6', '2028/2029', '2028', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '6', '2028/2029', '2028', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '6', '2028/2029', '2028', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '6', '2028/2029', '2029', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '6', '2028/2029', '2029', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '6', '2028/2029', '2029', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '6', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '6', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '6', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '6', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '6', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '6', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '7', '2028/2029', '2028', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '7', '2028/2029', '2028', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '7', '2028/2029', '2028', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '7', '2028/2029', '2028', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '7', '2028/2029', '2028', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '7', '2028/2029', '2028', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '7', '2028/2029', '2029', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '7', '2028/2029', '2029', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '7', '2028/2029', '2029', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '7', '2028/2029', '2029', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '7', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '7', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '7', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '7', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '7', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '7', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '8', '2028/2029', '2028', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '8', '2028/2029', '2028', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '8', '2028/2029', '2028', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '8', '2028/2029', '2028', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '8', '2028/2029', '2028', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '8', '2028/2029', '2029', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '8', '2028/2029', '2029', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '8', '2028/2029', '2029', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '8', '2028/2029', '2029', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '8', '2028/2029', '2029', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '8', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '8', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '8', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '8', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '8', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '8', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '9', '2028/2029', '2028', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '9', '2028/2029', '2028', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '9', '2028/2029', '2028', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '9', '2028/2029', '2028', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '9', '2028/2029', '2029', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '9', '2028/2029', '2029', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '9', '2028/2029', '2029', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '9', '2028/2029', '2029', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '9', '2028/2029', '2029', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '9', '2028/2029', '2029', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '9', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '9', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '9', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '9', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '9', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '9', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '10', '2028/2029', '2028', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '10', '2028/2029', '2028', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '10', '2028/2029', '2028', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '10', '2028/2029', '2029', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '10', '2028/2029', '2029', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '10', '2028/2029', '2029', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '10', '2028/2029', '2029', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '10', '2028/2029', '2029', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '10', '2028/2029', '2029', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '10', '2028/2029', '2029', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '10', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '10', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '10', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '10', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '10', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '10', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '11', '2028/2029', '2028', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '11', '2028/2029', '2028', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '11', '2028/2029', '2029', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '11', '2028/2029', '2029', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '11', '2028/2029', '2029', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '11', '2028/2029', '2029', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '11', '2028/2029', '2029', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '11', '2028/2029', '2029', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '11', '2028/2029', '2029', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '11', '2028/2029', '2029', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '11', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '11', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '11', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '11', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '11', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '11', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '1', '12', '2028/2029', '2028', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '2', '12', '2028/2029', '2029', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '3', '12', '2028/2029', '2029', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2028', '4', '12', '2028/2029', '2029', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '5', '12', '2028/2029', '2029', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '6', '12', '2028/2029', '2029', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2028', '7', '12', '2028/2029', '2029', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '8', '12', '2028/2029', '2029', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '9', '12', '2028/2029', '2029', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2028', '10', '12', '2028/2029', '2029', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '11', '12', '2028/2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '12', '12', '2028/2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '13', '12', '2028/2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '14', '12', '2028/2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '15', '12', '2028/2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2028', '16', '12', '2028/2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '1', '1', '2029', '2029', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '1', '2029', '2029', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '1', '2029', '2029', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '1', '2029', '2029', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '1', '2029', '2029', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '1', '2029', '2029', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '1', '2029', '2029', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '8', '1', '2029', '2029', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '9', '1', '2029', '2029', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '10', '1', '2029', '2029', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '11', '1', '2029', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '12', '1', '2029', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '13', '1', '2029', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '14', '1', '2029', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '15', '1', '2029', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '16', '1', '2029', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '1', '2', '2029/2030', '2029', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '2', '2029/2030', '2029', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '2', '2029/2030', '2029', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '2', '2029/2030', '2029', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '2', '2029/2030', '2029', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '2', '2029/2030', '2029', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '2', '2029/2030', '2029', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '8', '2', '2029/2030', '2029', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '9', '2', '2029/2030', '2029', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '10', '2', '2029/2030', '2029', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '11', '2', '2029/2030', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '1', '3', '2029/2030', '2029', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '3', '2029/2030', '2029', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '3', '2029/2030', '2029', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '3', '2029/2030', '2029', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '3', '2029/2030', '2029', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '3', '2029/2030', '2029', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '3', '2029/2030', '2029', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '8', '3', '2029/2030', '2029', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '9', '3', '2029/2030', '2029', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '10', '3', '2029/2030', '2029', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('2029', '1', '4', '2029/2030', '2029', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '4', '2029/2030', '2029', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '4', '2029/2030', '2029', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '4', '2029/2030', '2029', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '4', '2029/2030', '2029', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '4', '2029/2030', '2029', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '4', '2029/2030', '2029', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '8', '4', '2029/2030', '2029', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '9', '4', '2029/2030', '2029', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '1', '5', '2029/2030', '2029', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '5', '2029/2030', '2029', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '5', '2029/2030', '2029', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '5', '2029/2030', '2029', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '5', '2029/2030', '2029', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '5', '2029/2030', '2029', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '5', '2029/2030', '2029', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '8', '5', '2029/2030', '2029', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '1', '6', '2029/2030', '2029', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '6', '2029/2030', '2029', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '6', '2029/2030', '2029', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '6', '2029/2030', '2029', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '6', '2029/2030', '2029', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '6', '2029/2030', '2029', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '7', '6', '2029/2030', '2029', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('2029', '1', '7', '2029/2030', '2029', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '7', '2029/2030', '2029', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '7', '2029/2030', '2029', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '7', '2029/2030', '2029', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '7', '2029/2030', '2029', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '6', '7', '2029/2030', '2029', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '1', '8', '2029/2030', '2029', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '8', '2029/2030', '2029', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '8', '2029/2030', '2029', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '8', '2029/2030', '2029', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '5', '8', '2029/2030', '2029', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '1', '9', '2029/2030', '2029', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '9', '2029/2030', '2029', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '9', '2029/2030', '2029', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '4', '9', '2029/2030', '2029', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('2029', '1', '10', '2029/2030', '2029', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '10', '2029/2030', '2029', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '3', '10', '2029/2030', '2029', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '1', '11', '2029/2030', '2029', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '2', '11', '2029/2030', '2029', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('2029', '1', '12', '2029/2030', '2029', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '1', '1', '1990', '1990', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '1', '1990', '1990', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '1', '1990', '1990', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '1', '1990', '1990', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '1', '1990', '1990', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '1', '1990', '1990', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '1', '1990', '1990', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '1', '1990', '1990', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '1', '1990', '1990', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '1', '1990', '1990', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '1', '1990', '1990', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '1', '1990', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '1', '1990', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '1', '1990', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '1', '1990', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '1', '1990', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '2', '1990/1991', '1990', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '2', '1990/1991', '1990', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '2', '1990/1991', '1990', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '2', '1990/1991', '1990', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '2', '1990/1991', '1990', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '2', '1990/1991', '1990', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '2', '1990/1991', '1990', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '2', '1990/1991', '1990', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '2', '1990/1991', '1990', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '2', '1990/1991', '1990', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '2', '1990/1991', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '2', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '2', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '2', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '2', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '2', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '3', '1990/1991', '1990', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '3', '1990/1991', '1990', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '3', '1990/1991', '1990', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '3', '1990/1991', '1990', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '3', '1990/1991', '1990', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '3', '1990/1991', '1990', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '3', '1990/1991', '1990', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '3', '1990/1991', '1990', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '3', '1990/1991', '1990', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '3', '1990/1991', '1990', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '3', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '3', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '3', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '3', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '3', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '3', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '4', '1990/1991', '1990', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '4', '1990/1991', '1990', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '4', '1990/1991', '1990', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '4', '1990/1991', '1990', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '4', '1990/1991', '1990', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '4', '1990/1991', '1990', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '4', '1990/1991', '1990', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '4', '1990/1991', '1990', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '4', '1990/1991', '1990', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '4', '1990/1991', '1991', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '4', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '4', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '4', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '4', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '4', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '4', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '5', '1990/1991', '1990', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '5', '1990/1991', '1990', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '5', '1990/1991', '1990', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '5', '1990/1991', '1990', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '5', '1990/1991', '1990', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '5', '1990/1991', '1990', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '5', '1990/1991', '1990', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '5', '1990/1991', '1990', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '5', '1990/1991', '1991', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '5', '1990/1991', '1991', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '5', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '5', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '5', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '5', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '5', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '5', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '6', '1990/1991', '1990', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '6', '1990/1991', '1990', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '6', '1990/1991', '1990', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '6', '1990/1991', '1990', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '6', '1990/1991', '1990', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '6', '1990/1991', '1990', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '6', '1990/1991', '1990', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '6', '1990/1991', '1991', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '6', '1990/1991', '1991', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '6', '1990/1991', '1991', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '6', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '6', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '6', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '6', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '6', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '6', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '7', '1990/1991', '1990', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '7', '1990/1991', '1990', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '7', '1990/1991', '1990', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '7', '1990/1991', '1990', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '7', '1990/1991', '1990', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '7', '1990/1991', '1990', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '7', '1990/1991', '1991', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '7', '1990/1991', '1991', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '7', '1990/1991', '1991', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '7', '1990/1991', '1991', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '7', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '7', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '7', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '7', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '7', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '7', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '8', '1990/1991', '1990', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '8', '1990/1991', '1990', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '8', '1990/1991', '1990', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '8', '1990/1991', '1990', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '8', '1990/1991', '1990', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '8', '1990/1991', '1991', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '8', '1990/1991', '1991', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '8', '1990/1991', '1991', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '8', '1990/1991', '1991', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '8', '1990/1991', '1991', '5', 'QUARTAL4', 'HALBJAHR2')

    INSERT INTO global.tPeriods (FinancialYear, Period, FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE) VALUES
    ('1990', '11', '8', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '8', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '8', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '8', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '8', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '8', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '9', '1990/1991', '1990', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '9', '1990/1991', '1990', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '9', '1990/1991', '1990', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '9', '1990/1991', '1990', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '9', '1990/1991', '1991', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '9', '1990/1991', '1991', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '9', '1990/1991', '1991', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '9', '1990/1991', '1991', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '9', '1990/1991', '1991', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '9', '1990/1991', '1991', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '9', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '9', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '9', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '9', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '9', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '9', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '10', '1990/1991', '1990', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '10', '1990/1991', '1990', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '10', '1990/1991', '1990', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '10', '1990/1991', '1991', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '10', '1990/1991', '1991', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '10', '1990/1991', '1991', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '10', '1990/1991', '1991', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '10', '1990/1991', '1991', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '10', '1990/1991', '1991', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '10', '1990/1991', '1991', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '10', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '10', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '10', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '10', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '10', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '10', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '11', '1990/1991', '1990', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '11', '1990/1991', '1990', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '11', '1990/1991', '1991', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '11', '1990/1991', '1991', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '11', '1990/1991', '1991', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '11', '1990/1991', '1991', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '11', '1990/1991', '1991', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '11', '1990/1991', '1991', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '11', '1990/1991', '1991', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '11', '1990/1991', '1991', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '11', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '11', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '11', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '11', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '11', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '11', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '1', '12', '1990/1991', '1990', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '2', '12', '1990/1991', '1991', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '3', '12', '1990/1991', '1991', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1990', '4', '12', '1990/1991', '1991', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '5', '12', '1990/1991', '1991', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '6', '12', '1990/1991', '1991', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1990', '7', '12', '1990/1991', '1991', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '8', '12', '1990/1991', '1991', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '9', '12', '1990/1991', '1991', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1990', '10', '12', '1990/1991', '1991', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '11', '12', '1990/1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '12', '12', '1990/1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '13', '12', '1990/1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '14', '12', '1990/1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '15', '12', '1990/1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1990', '16', '12', '1990/1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '1', '1991', '1991', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '1', '1991', '1991', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '1', '1991', '1991', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '1', '1991', '1991', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '1', '1991', '1991', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '1', '1991', '1991', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '1', '1991', '1991', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '1', '1991', '1991', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '1', '1991', '1991', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '1', '1991', '1991', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '1', '1991', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '1', '1991', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '1', '1991', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '1', '1991', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '1', '1991', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '1', '1991', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '2', '1991/1992', '1991', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '2', '1991/1992', '1991', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '2', '1991/1992', '1991', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '2', '1991/1992', '1991', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '2', '1991/1992', '1991', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '2', '1991/1992', '1991', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '2', '1991/1992', '1991', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '2', '1991/1992', '1991', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '2', '1991/1992', '1991', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '2', '1991/1992', '1991', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '2', '1991/1992', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '2', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '2', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '2', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '2', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '2', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '3', '1991/1992', '1991', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '3', '1991/1992', '1991', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '3', '1991/1992', '1991', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '3', '1991/1992', '1991', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '3', '1991/1992', '1991', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '3', '1991/1992', '1991', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '3', '1991/1992', '1991', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '3', '1991/1992', '1991', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '3', '1991/1992', '1991', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '3', '1991/1992', '1991', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '3', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '3', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '3', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '3', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '3', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '3', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '4', '1991/1992', '1991', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '4', '1991/1992', '1991', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '4', '1991/1992', '1991', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '4', '1991/1992', '1991', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '4', '1991/1992', '1991', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '4', '1991/1992', '1991', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '4', '1991/1992', '1991', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '4', '1991/1992', '1991', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '4', '1991/1992', '1991', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '4', '1991/1992', '1992', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '4', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '4', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '4', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '4', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '4', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '4', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '5', '1991/1992', '1991', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '5', '1991/1992', '1991', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '5', '1991/1992', '1991', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '5', '1991/1992', '1991', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '5', '1991/1992', '1991', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '5', '1991/1992', '1991', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '5', '1991/1992', '1991', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '5', '1991/1992', '1991', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '5', '1991/1992', '1992', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '5', '1991/1992', '1992', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '5', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '5', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '5', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '5', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '5', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '5', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '6', '1991/1992', '1991', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '6', '1991/1992', '1991', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '6', '1991/1992', '1991', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '6', '1991/1992', '1991', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '6', '1991/1992', '1991', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '6', '1991/1992', '1991', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '6', '1991/1992', '1991', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '6', '1991/1992', '1992', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '6', '1991/1992', '1992', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '6', '1991/1992', '1992', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '6', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '6', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '6', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '6', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '6', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '6', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '7', '1991/1992', '1991', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '7', '1991/1992', '1991', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '7', '1991/1992', '1991', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '7', '1991/1992', '1991', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '7', '1991/1992', '1991', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '7', '1991/1992', '1991', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '7', '1991/1992', '1992', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '7', '1991/1992', '1992', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '7', '1991/1992', '1992', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '7', '1991/1992', '1992', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '7', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '7', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '7', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '7', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '7', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '7', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '8', '1991/1992', '1991', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '8', '1991/1992', '1991', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '8', '1991/1992', '1991', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '8', '1991/1992', '1991', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '8', '1991/1992', '1991', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '8', '1991/1992', '1992', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '8', '1991/1992', '1992', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '8', '1991/1992', '1992', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '8', '1991/1992', '1992', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '8', '1991/1992', '1992', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '8', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '8', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '8', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '8', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '8', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '8', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '9', '1991/1992', '1991', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '9', '1991/1992', '1991', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '9', '1991/1992', '1991', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '9', '1991/1992', '1991', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '9', '1991/1992', '1992', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '9', '1991/1992', '1992', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '9', '1991/1992', '1992', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '9', '1991/1992', '1992', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '9', '1991/1992', '1992', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '9', '1991/1992', '1992', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '9', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '9', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '9', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '9', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '9', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '9', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '10', '1991/1992', '1991', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '10', '1991/1992', '1991', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '10', '1991/1992', '1991', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '10', '1991/1992', '1992', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '10', '1991/1992', '1992', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '10', '1991/1992', '1992', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '10', '1991/1992', '1992', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '10', '1991/1992', '1992', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '10', '1991/1992', '1992', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '10', '1991/1992', '1992', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '10', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '10', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '10', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '10', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '10', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '10', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '11', '1991/1992', '1991', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '11', '1991/1992', '1991', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '11', '1991/1992', '1992', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '11', '1991/1992', '1992', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '11', '1991/1992', '1992', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '11', '1991/1992', '1992', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '11', '1991/1992', '1992', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '11', '1991/1992', '1992', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '11', '1991/1992', '1992', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '11', '1991/1992', '1992', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '11', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '11', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '11', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '11', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '11', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '11', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '1', '12', '1991/1992', '1991', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '2', '12', '1991/1992', '1992', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '3', '12', '1991/1992', '1992', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1991', '4', '12', '1991/1992', '1992', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '5', '12', '1991/1992', '1992', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '6', '12', '1991/1992', '1992', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1991', '7', '12', '1991/1992', '1992', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '8', '12', '1991/1992', '1992', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '9', '12', '1991/1992', '1992', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1991', '10', '12', '1991/1992', '1992', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '11', '12', '1991/1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '12', '12', '1991/1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '13', '12', '1991/1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '14', '12', '1991/1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '15', '12', '1991/1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1991', '16', '12', '1991/1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '1', '1992', '1992', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '1', '1992', '1992', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '1', '1992', '1992', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '1', '1992', '1992', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '1', '1992', '1992', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '1', '1992', '1992', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '1', '1992', '1992', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '1', '1992', '1992', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '1', '1992', '1992', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '1', '1992', '1992', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '1', '1992', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '1', '1992', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '1', '1992', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '1', '1992', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '1', '1992', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '1', '1992', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '2', '1992/1993', '1992', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '2', '1992/1993', '1992', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '2', '1992/1993', '1992', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '2', '1992/1993', '1992', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '2', '1992/1993', '1992', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '2', '1992/1993', '1992', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '2', '1992/1993', '1992', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '2', '1992/1993', '1992', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '2', '1992/1993', '1992', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '2', '1992/1993', '1992', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '2', '1992/1993', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '2', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '2', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '2', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '2', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '2', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '3', '1992/1993', '1992', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '3', '1992/1993', '1992', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '3', '1992/1993', '1992', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '3', '1992/1993', '1992', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '3', '1992/1993', '1992', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '3', '1992/1993', '1992', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '3', '1992/1993', '1992', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '3', '1992/1993', '1992', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '3', '1992/1993', '1992', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '3', '1992/1993', '1992', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '3', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '3', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '3', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '3', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '3', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '3', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '4', '1992/1993', '1992', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '4', '1992/1993', '1992', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '4', '1992/1993', '1992', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '4', '1992/1993', '1992', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '4', '1992/1993', '1992', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '4', '1992/1993', '1992', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '4', '1992/1993', '1992', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '4', '1992/1993', '1992', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '4', '1992/1993', '1992', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '4', '1992/1993', '1993', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '4', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '4', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '4', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '4', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '4', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '4', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '5', '1992/1993', '1992', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '5', '1992/1993', '1992', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '5', '1992/1993', '1992', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '5', '1992/1993', '1992', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '5', '1992/1993', '1992', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '5', '1992/1993', '1992', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '5', '1992/1993', '1992', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '5', '1992/1993', '1992', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '5', '1992/1993', '1993', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '5', '1992/1993', '1993', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '5', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '5', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '5', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '5', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '5', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '5', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '6', '1992/1993', '1992', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '6', '1992/1993', '1992', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '6', '1992/1993', '1992', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '6', '1992/1993', '1992', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '6', '1992/1993', '1992', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '6', '1992/1993', '1992', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '6', '1992/1993', '1992', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '6', '1992/1993', '1993', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '6', '1992/1993', '1993', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '6', '1992/1993', '1993', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '6', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '6', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '6', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '6', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '6', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '6', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '7', '1992/1993', '1992', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '7', '1992/1993', '1992', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '7', '1992/1993', '1992', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '7', '1992/1993', '1992', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '7', '1992/1993', '1992', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '7', '1992/1993', '1992', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '7', '1992/1993', '1993', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '7', '1992/1993', '1993', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '7', '1992/1993', '1993', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '7', '1992/1993', '1993', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '7', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '7', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '7', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '7', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '7', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '7', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '8', '1992/1993', '1992', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '8', '1992/1993', '1992', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '8', '1992/1993', '1992', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '8', '1992/1993', '1992', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '8', '1992/1993', '1992', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '8', '1992/1993', '1993', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '8', '1992/1993', '1993', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '8', '1992/1993', '1993', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '8', '1992/1993', '1993', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '8', '1992/1993', '1993', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '8', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '8', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '8', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '8', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '8', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '8', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '9', '1992/1993', '1992', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '9', '1992/1993', '1992', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '9', '1992/1993', '1992', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '9', '1992/1993', '1992', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '9', '1992/1993', '1993', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '9', '1992/1993', '1993', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '9', '1992/1993', '1993', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '9', '1992/1993', '1993', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '9', '1992/1993', '1993', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '9', '1992/1993', '1993', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '9', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '9', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '9', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '9', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '9', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '9', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '10', '1992/1993', '1992', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '10', '1992/1993', '1992', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '10', '1992/1993', '1992', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '10', '1992/1993', '1993', '1', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '10', '1992/1993', '1993', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '10', '1992/1993', '1993', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '10', '1992/1993', '1993', '4', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '10', '1992/1993', '1993', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '10', '1992/1993', '1993', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '10', '1992/1993', '1993', '7', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '10', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '10', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '10', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '10', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '10', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '10', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '11', '1992/1993', '1992', '11', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '11', '1992/1993', '1992', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '11', '1992/1993', '1993', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '11', '1992/1993', '1993', '2', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '11', '1992/1993', '1993', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '11', '1992/1993', '1993', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '11', '1992/1993', '1993', '5', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '11', '1992/1993', '1993', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '11', '1992/1993', '1993', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '11', '1992/1993', '1993', '8', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '11', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '11', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '11', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '11', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '11', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '11', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '1', '12', '1992/1993', '1992', '12', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '2', '12', '1992/1993', '1993', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '3', '12', '1992/1993', '1993', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1992', '4', '12', '1992/1993', '1993', '3', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '5', '12', '1992/1993', '1993', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '6', '12', '1992/1993', '1993', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1992', '7', '12', '1992/1993', '1993', '6', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '8', '12', '1992/1993', '1993', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '9', '12', '1992/1993', '1993', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1992', '10', '12', '1992/1993', '1993', '9', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '11', '12', '1992/1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '12', '12', '1992/1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '13', '12', '1992/1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '14', '12', '1992/1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '15', '12', '1992/1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1992', '16', '12', '1992/1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '1', '1993', '1993', '1', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '1', '1993', '1993', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '1', '1993', '1993', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '1', '1993', '1993', '4', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '1', '1993', '1993', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '1', '1993', '1993', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '1', '1993', '1993', '7', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '1', '1993', '1993', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '1', '1993', '1993', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '1', '1993', '1993', '10', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '1', '1993', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '1', '1993', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '1', '1993', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '1', '1993', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '1', '1993', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '1', '1993', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '2', '1993/1994', '1993', '2', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '2', '1993/1994', '1993', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '2', '1993/1994', '1993', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '2', '1993/1994', '1993', '5', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '2', '1993/1994', '1993', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '2', '1993/1994', '1993', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '2', '1993/1994', '1993', '8', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '2', '1993/1994', '1993', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '2', '1993/1994', '1993', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '2', '1993/1994', '1993', '11', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '2', '1993/1994', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '2', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '2', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '2', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '2', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '2', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '3', '1993/1994', '1993', '3', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '3', '1993/1994', '1993', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '3', '1993/1994', '1993', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '3', '1993/1994', '1993', '6', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '3', '1993/1994', '1993', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '3', '1993/1994', '1993', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '3', '1993/1994', '1993', '9', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '3', '1993/1994', '1993', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '3', '1993/1994', '1993', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '3', '1993/1994', '1993', '12', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '3', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '3', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '3', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '3', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '3', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '3', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '4', '1993/1994', '1993', '4', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '4', '1993/1994', '1993', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '4', '1993/1994', '1993', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '4', '1993/1994', '1993', '7', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '4', '1993/1994', '1993', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '4', '1993/1994', '1993', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '4', '1993/1994', '1993', '10', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '4', '1993/1994', '1993', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '4', '1993/1994', '1993', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '4', '1993/1994', '1994', '1', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '4', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '4', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '4', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '4', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '4', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '4', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '5', '1993/1994', '1993', '5', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '5', '1993/1994', '1993', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '5', '1993/1994', '1993', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '5', '1993/1994', '1993', '8', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '5', '1993/1994', '1993', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '5', '1993/1994', '1993', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '5', '1993/1994', '1993', '11', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '5', '1993/1994', '1993', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '5', '1993/1994', '1994', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '5', '1993/1994', '1994', '2', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '5', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '5', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '5', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '5', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '5', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '5', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '6', '1993/1994', '1993', '6', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '6', '1993/1994', '1993', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '6', '1993/1994', '1993', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '6', '1993/1994', '1993', '9', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '6', '1993/1994', '1993', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '6', '1993/1994', '1993', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '6', '1993/1994', '1993', '12', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '6', '1993/1994', '1994', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '6', '1993/1994', '1994', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '6', '1993/1994', '1994', '3', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '6', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '6', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '6', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '6', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '6', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '6', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '7', '1993/1994', '1993', '7', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '7', '1993/1994', '1993', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '7', '1993/1994', '1993', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '7', '1993/1994', '1993', '10', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '5', '7', '1993/1994', '1993', '11', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '6', '7', '1993/1994', '1993', '12', 'QUARTAL2', 'HALBJAHR1')
    ,('1993', '7', '7', '1993/1994', '1994', '1', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '8', '7', '1993/1994', '1994', '2', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '9', '7', '1993/1994', '1994', '3', 'QUARTAL3', 'HALBJAHR2')
    ,('1993', '10', '7', '1993/1994', '1994', '4', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '11', '7', '1993/1994', '1994', '5', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '12', '7', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '13', '7', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '14', '7', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '15', '7', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '16', '7', '1993/1994', '1994', '6', 'QUARTAL4', 'HALBJAHR2')
    ,('1993', '1', '8', '1993/1994', '1993', '8', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '2', '8', '1993/1994', '1993', '9', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '3', '8', '1993/1994', '1993', '10', 'QUARTAL1', 'HALBJAHR1')
    ,('1993', '4', '8', '1993/1994', '1993', '11', 'QUARTAL2', 'HALBJAHR1')
END;
GO

/*
    Product: OCT 5.0
    Object: system.tSettings
    Type: Content
    Author: saxess-software GmbH

*/

IF NOT EXISTS(SELECT 1 FROM system.tSettings WHERE SettingID='DBVersion')
BEGIN

    INSERT INTO system.tSettings (
				 SettingID
				,SettingDescription
				,DataTypeCODE
				,ValueInt
				,Category
				,ReadOnlySystemPropertyBool)
	VALUES
			('DBVersion','Version of Database','NUMBER','050215','Database Information',1)
END;
GO


-- initial materialization of Userrights
EXEC dbo.sx_pf_materialize_vUserRights

GO

-- Set the T_SQL_API Version for the Cluster
-- The Version String has 3 Parts
		-- The first for the PlanningFactoryGeneration
		-- The Second for external API Definition
		-- The Third for the internal API Definition

-- Any Client must check, that he fits to the second Level

UPDATE planning.tgCluster SET ValueText = '5.2.15' WHERE PropertyID = 'C04'; --Must be numeric in last section !!
EXEC system.pPOST_Setting @Username = 'SQL', @SettingID = 'DBVersion', @DataTypeCODE = 'NUMBER', @ValueInt = 050215, @ReadOnlySystemPropertyBool = 1
GO

UPDATE system.tProcessSteps
SET StepDetailsJSON = REPLACE(
						REPLACE(
							REPLACE(
								REPLACE(
									REPLACE(StepDetailsJSON, '"StepTypeCode":0', '"StepTypeCode":"Etl"'),
								'"StepTypeCode":1', '"StepTypeCode":"Procedure"'),
							'"StepTypeCode":2', '"StepTypeCode":"Agent"'),
						'"StepTypeCode":3', '"StepTypeCode":"Batch"'),
					'"StepTypeCode":4', '"StepTypeCode":"SqlDataTransfer"');
GO


-- Check global.tDatasources for missing column Name and add it if missing
-- (was forgotten in 5.0.2 update)
IF COL_LENGTH('global.tDataSources', 'Name') IS NULL
BEGIN
    ALTER TABLE global.tDataSources 
	ADD Name NVARCHAR (255) NOT NULL;
END
GO

