/*
	Update script for OCT databases
	Gerd Tautenhahn
	Saxess Software GmbH

	SELECT * FROM sys.schemas
	SELECT * FROM sys.objects
	SELECT * FROM sys.tables
	SELECT * FROM sys.columns 
	SELECT * FROM sys.indexes -- exists with and without point in SQL Server system objects, without is an old one

*/
   PRINT '+++ Starting Batch 2 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

   -- Manual configuration
   DECLARE @RequiredVersion INT = 50811;	
   DECLARE @TargetVersion	INT = 50917;	

   DECLARE @TargetversionUser NVARCHAR(255) = '';
   SET @TargetversionUser =
		  CAST(@Targetversion / 10000		AS NVARCHAR(10)) + '.'
		+ CAST(@Targetversion / 100 % 100	AS NVARCHAR(10)) + '.'
		+ CAST(@Targetversion % 100			AS NVARCHAR(10))
	-- PRINT @TargetversionUser
   
   DROP TABLE IF EXISTS #DBVersion;

	CREATE TABLE #DBVersion
		(
			 CurrentVersion		INT NOT NULL
			,RequiredVersion	INT NOT NULL
			,TargetVersion		INT NOT NULL
			,TargetVersionUser	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			,DoUpdateFlag		INT NOT NULL
		 )

	INSERT INTO #DBVersion
	SELECT 
		 CASE	
			WHEN ValueInt = 50810 THEN 50811 --special Configuration for support of multiple base versions
			WHEN ValueInt = 50812 THEN 50811 --special Configuration for support of multiple base versions
			ELSE ValueInt
		 END		AS CurrentVersion
		,@RequiredVersion
		,@TargetVersion
		,@TargetversionUser
		,0
	FROM system.tSettings
	WHERE SettingID = 'DBVersion';

   UPDATE #DBVersion SET DoUpdateFlag = 1 
   WHERE 
		CurrentVersion = RequiredVersion
	AND TargetVersion > CurrentVersion;

	--SELECT * FROM #DBVersion

	GO

	PRINT '+++ Ended Batch 2 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'


PRINT '+++ Starting Batch 3 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

DECLARE @DoUpdateFlag INT;
SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

IF @DoUpdateFlag = 0
	BEGIN
		PRINT 'Batch 3 not started due to wrong version.'
	END

IF @DoUpdateFlag = 1
BEGIN

		-- nothing to do
		PRINT 'Nothing to do in Before Update Actions'

		

END





GO
PRINT '+++ Ended Batch 3 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

PRINT '+++ Starting Batch 3 (Index Core Before Build) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
-- Builds an Index named #CoreObjects_BeforeUpdate of all existing OCT.Core Objects and materializes it in an temporary table	
	
	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 3 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
	BEGIN
		DECLARE @Counter	INT = 0;
		-- Put all Core objects into a temp table
		DROP TABLE IF EXISTS #CoreObjects_BeforeUpdate;

		CREATE TABLE #CoreObjects_BeforeUpdate
			(
				 ObjectCode			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,ObjectType			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,SX_Owner			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,SX_ShipmentFlag	NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,ShipmentStatus		NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,SchemaName			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,ObjectName			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,FullObjectName		NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,Description		NVARCHAR(4000) COLLATE DATABASE_DEFAULT NOT NULL
			);

		INSERT INTO #CoreObjects_BeforeUpdate
			SELECT 
				 o.type				AS ObjectCode
				,CASE o.type
						WHEN 'P'	THEN N'Stored Procedure'
						WHEN 'U'	THEN N'Table'
						WHEN 'V'	THEN N'View'
						WHEN 'IF'	THEN N'Function (Table Value Inline)'
						WHEN 'FN'	THEN N'Function (Scalar)'
						WHEN 'TF'	THEN N'Function (Table Value)'
						WHEN 'TR'	THEN N'Trigger'
						ELSE			 N'Unknown Objecttype'
				 END				AS ObjectType 
				,COALESCE(CONVERT(NVARCHAR(255),sxo.SX_Owner),'unknown')	AS SX_Owner
				,COALESCE(CONVERT(NVARCHAR(255),sxs.SX_ShipmentFlag),'0')	AS SX_ShipmentFlag
				-- determine name to inform user about replacement
				,CASE 
					WHEN sxs.SX_ShipmentFlag	= N'0'	THEN	N'0. Demo object - out of shipment process'
					WHEN sxs.SX_ShipmentFlag	= N'1'	THEN	N'1. Standard - without modification'
					WHEN sxs.SX_ShipmentFlag	= N'2'	THEN	N'2. Standard - modified from saxess for customer'
					WHEN sxs.SX_ShipmentFlag	= N'3'	THEN	N'3. Standard - modified from partner for customer'
					WHEN sxs.SX_ShipmentFlag	= N'4'	THEN	N'4. Standard - modified from customer for himself'
												  				
					WHEN sxs.SX_ShipmentFlag	= N'10'	THEN	N'10. Created from Saxess for Customer'
					WHEN sxs.SX_ShipmentFlag	= N'11'	THEN	N'11. Created from Partner for Customer'
					WHEN sxs.SX_ShipmentFlag	= N'12'	THEN	N'12. Created from Customer for Customer'

					WHEN sxs.SX_ShipmentFlag	IS NULL THEN	N'<module defintion tag not definded>'

					ELSE										N'unknown Shipment Status'
				 END									AS ShipmentStatus
				,S.name									AS SchemaName
				,O.name									AS ObjectName
				,S.name	+'.' + O.name					AS FullObjectName
				,CONVERT(NVARCHAR(4000),
				 CASE
					WHEN ep.value = ''		THEN	N'Description not defined'
					WHEN ep.value IS NULL	THEN	N'<description defintion tag not definded>'
					WHEN ep.value <> ''		THEN	ep.value
					ELSE							N'<procedure failure - this should not had happen>'
				 END)								AS Description
				
			FROM sys.all_objects O
				LEFT JOIN sys.extended_properties EP 
					ON ep.major_id = O.object_id 
				LEFT JOIN sys.schemas S 
					ON O.schema_id = S.schema_id

				--join SX_Owner as separte column for filtering
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_Owner
								FROM sys.extended_properties
								WHERE Name = N'SX_Owner'
							) sxo
							ON sxo.major_id = ep.major_id

				--join SX_ShipmentFlag as separte column
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_ShipmentFlag
								FROM sys.extended_properties
								WHERE Name = N'SX_ShipmentFlag'
							) sxs
							ON sxs.major_id = ep.major_id

			WHERE 
					O.is_ms_shipped = 0
				AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
				AND O.type NOT IN (  'PK' -- PrimaryKeys
									--,'TR' -- Trigger
									,'D'  -- default constraints
									)
				AND (ep.name = 'MS_Description' OR ep.name IS NULL)
				AND sxo.SX_Owner = 'OCT.core'
				ORDER BY ObjectType;

	
		PRINT '### Update replaces the following OCT.core objects which are marked as customized objects - they must be recustomized after the update if necessary.###'

		PRINT ''

		DECLARE @Text NVARCHAR(2000) = ''
 
		DECLARE MyCursor CURSOR FOR
 
			-- Query fills the Cursor
			SELECT
				ObjectType +': ' + ObjectName + ' ' + ShipmentStatus	 
			FROM #CoreObjects_BeforeUpdate
			WHERE
					SX_Owner = 'OCT.core'
				AND SX_ShipmentFlag <> '1'
			ORDER BY 
				ShipmentStatus, ObjectType, ObjectName
 
		OPEN MyCursor
			FETCH MyCursor INTO @Text
			WHILE @@FETCH_STATUS = 0
				BEGIN
					SET @Counter = @Counter + 1
					PRINT @Text
					
					FETCH MyCursor INTO @Text
				END
		CLOSE MyCursor
		DEALLOCATE MyCursor

		IF @Counter = 0 
			BEGIN
			    PRINT '>>> No customized Objects in OCT.Core objects found - no need for recustomizing core objects after update.'
			END

		PRINT ''
		PRINT '### End of Replacement List.###'
		PRINT ''

		SELECT @Counter=COUNT(*) FROM #CoreObjects_BeforeUpdate;
		PRINT CONCAT('Anzahl der Objekte (ohne Trigger) im Index vor Update: ',@Counter);
		SET @Counter = 0;

	END

GO
PRINT '+++ Ended Batch 3 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

PRINT '+++ Starting Batch 4 (kill all Nontables) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	
	-- Drop all foreign keys betwenn tables
		-- there are none at the moment

	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 4 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
	BEGIN
		
		-- delete all objects except tables, which belong to OCT Core
		DECLARE  @SQL NVARCHAR(MAX) = N''
				,@crlf VARCHAR(2) = CHAR(13) + CHAR(10);

		-- Views
		SELECT @SQL = @SQL + N'DROP VIEW ' + cob.FullObjectName +N';' + @crlf
		FROM #CoreObjects_BeforeUpdate cob
		WHERE cob.ObjectCode = N'V';
				
		PRINT @SQL;
		EXEC (@SQL);
		-- Stored Procedures
		SET @SQL = N''
		SELECT @SQL = @SQL + N'DROP PROCEDURE ' + cob.FullObjectName +N';' + @crlf
		FROM #CoreObjects_BeforeUpdate cob
		WHERE cob.ObjectCode = N'P';
		PRINT @SQL;
		EXEC (@SQL);

		-- Functions
		SET @SQL = N''
		SELECT @SQL = @SQL + N'DROP FUNCTION ' + cob.FullObjectName +N';' + @crlf
		FROM #CoreObjects_BeforeUpdate cob
		WHERE cob.ObjectCode IN (N'IF', N'FN',N'TF');
		PRINT @SQL;
		EXEC (@SQL);

		-- Trigger
		SET @SQL = N''
		SELECT @SQL = @SQL + N'DROP TRIGGER ' + cob.FullObjectName +N';' + @crlf
		FROM #CoreObjects_BeforeUpdate cob
		WHERE cob.ObjectCode = (N'TR');
		PRINT @SQL;
		EXEC (@SQL);

		-- Constains
		SET @SQL = N''
		SELECT @sql = @sql 
		  + N'ALTER TABLE ' + QUOTENAME(s.name) + N'.'
		  + QUOTENAME(t.name) + N' DROP CONSTRAINT '
		  + QUOTENAME(c.name) + ';'+ @crlf
		FROM sys.objects c
			INNER JOIN sys.tables t
				ON c.parent_object_id = t.[object_id]
			INNER JOIN sys.schemas s 
				ON t.[schema_id] = s.[schema_id]
			INNER JOIN #CoreObjects_BeforeUpdate cob
				ON s.name COLLATE DATABASE_DEFAULT + N'.'+ t.name COLLATE DATABASE_DEFAULT = cob.FullObjectName
		WHERE	c.[type] IN ('D','C','F','PK','UQ')
				AND cob.ObjectCode = 'U'
		ORDER BY c.[type];
			   		 
		PRINT @SQL;
		EXEC (@SQL);

		-- Delete all indexes
			SET @SQL = N'';
		SELECT @SQL = @SQL 
			+ N'DROP INDEX [' + ix.name + N'] ON ' + QUOTENAME(SCHEMA_NAME(schema_id)) +'.' + OBJECT_NAME(ix.object_id) + N'; ' + @crlf
		FROM  sys.indexes ix
			LEFT JOIN sys.objects o 
				ON ix.object_id = o.object_id
			INNER JOIN #CoreObjects_BeforeUpdate cob
				ON QUOTENAME(SCHEMA_NAME(schema_id)) COLLATE DATABASE_DEFAULT +'.' + OBJECT_NAME(ix.object_id) COLLATE DATABASE_DEFAULT = cob.FullObjectName
		WHERE 
			-- not on custom table
				cob.ObjectCode = 'U'
			AND ix.NAME IS NOT NULL;
			
		PRINT @SQL;
		EXEC (@SQL);
	END
GO
PRINT '+++ Ended Batch 4  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	PRINT '+++ Starting Batch 5 (Tables to _bak) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	
	-- add the extension _bak to all tables ***********************************************************************
	
	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 5 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
	BEGIN

		DECLARE  @SQL NVARCHAR(MAX) = N''
				,@crlf VARCHAR(2) = CHAR(13) + CHAR(10);	
	
		DECLARE Tablecursor CURSOR FAST_FORWARD

		FOR
			SELECT 
				N'sp_rename ''' + cob.FullObjectName + ''','''  + cob.ObjectName + N'_bak''' 
			FROM   #CoreObjects_BeforeUpdate cob
			WHERE 
					cob.ObjectCode = 'U'
		OPEN Tablecursor
		FETCH NEXT FROM Tablecursor INTO @SQL
		WHILE @@FETCH_STATUS = 0
		BEGIN
			PRINT @SQL
			EXEC(@SQL)
			FETCH NEXT FROM Tablecursor INTO @SQL
		END

		CLOSE Tablecursor;
		DEALLOCATE Tablecursor;
	END

	GO
	PRINT '+++ Ended Batch 5 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

/*
	Saxess Software GmbH
    Last modified 09/2022 for OCT 5.8
*/

IF schema_id('agent') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA agent AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used by agent procedures, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'agent'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used by agent procedures, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'agent'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('calc') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA calc AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used for calculation objects, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'calc'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used for calculation objects, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'calc'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('control') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA control AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to be used for control flows, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'control'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to be used for control flows, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'control'
GO
/*
    Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('global') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA global AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store global informations, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'global'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store global informations, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'global'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('import') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA import AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store planning import objects, to transport values from load tables to planning tables. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'import'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store planning import objects, to transport values from load tables to planning tables. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'import'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('integration') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA integration AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store integration data, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'integration'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store integration data, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'integration'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('load') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA load AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store generic import data for planning, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'load'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store generic import data for planning, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'load'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('param') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA param AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store parameter tables, created by product materialization. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'param'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store parameter tables, created by product materialization. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'param'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('planning') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA planning AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store objects for planning, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'planning'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store objects for planning, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'planning'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('result') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA result AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store objects which deliver data to enduser, schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'result'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store objects which deliver data to enduser, schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'result'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('staging') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA staging AUTHORIZATION pf_PlanningFactoryService'

    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Depreciated Schema for planning datastaging - use integration now. Only delivered for compatibility reasons. Schema is owned by OCTService.',
        @level0type = N'Schema',
        @level0name = N'staging'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Depreciated Schema for planning datastaging - use integration now. Only delivered for compatibility reasons. Schema is owned by OCTService.',
    @level0type = N'Schema',
    @level0name = N'staging'
GO
/*
	Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
*/

IF schema_id('system') IS NULL
BEGIN
    EXEC sp_executesql N'CREATE SCHEMA system AUTHORIZATION dbo';
    -- Documentation
    EXEC sp_addextendedproperty
        @name = N'MS_Description',
        @value = N'Schema to store system data and Metadata. Schema owned by dbo, NO table access for OCTService or other user.',
        @level0type = N'Schema',
        @level0name = N'system'
END

-- Documentation
EXEC sp_updateextendedproperty
    @name = N'MS_Description',
    @value = N'Schema to store system data and Metadata. Schema owned by dbo, NO table access for OCTService or other user.',
    @level0type = N'Schema',
    @level0name = N'system'
GO

/*
	Creates Database Roles for OCT
	Last modified 09/2022 for OCT 5.8
*/

IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryUser')		IS NULL CREATE ROLE pf_PlanningFactoryUser;
GO
IF DATABASE_PRINCIPAL_ID('pf_PlanningFactoryService')	IS NULL CREATE ROLE pf_PlanningFactoryService;
GO

-- this give the OCT Service the Right to create objects in all schema he owns. This are all schemas, except the schema system.
-- Owning a schema gives only ALTER Permissions to all objects, not CREATE Permissions
GRANT CREATE TABLE		TO pf_PlanningFactoryService;
GRANT CREATE PROCEDURE	TO pf_PlanningFactoryService;
GRANT CREATE VIEW		TO pf_PlanningFactoryService;
GRANT CREATE FUNCTION	TO pf_PlanningFactoryService;
GO
 DROP TABLE IF EXISTS control.tCustomFailureReport; 
 GO
/*
	Table for failure reporting
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM control.tCustomFailureReport

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport','COLUMN',default)
*/

CREATE TABLE control.tCustomFailureReport
(
	 RowKey				BIGINT IDENTITY (1,1) 
	,Timestamp			DATETIME		NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,TestSource			NVARCHAR(255)	NOT NULL
	,TestName			NVARCHAR(255)	NOT NULL
	,ResultType			NVARCHAR(255)	NOT NULL -- OK, ERROR, WARNING
	,ResultText			NVARCHAR(MAX)	NOT NULL
	,ResultNumeric		MONEY			NOT NULL

	,CONSTRAINT PK_tCustomFailureReport PRIMARY KEY NONCLUSTERED (RowKey)
)

GO

-- Table is clustered by Timestamp for faster query the last failure report
CREATE CLUSTERED INDEX IX_tCustomFailureReport_TimeFID ON control.tCustomFailureReport (Timestamp DESC, FactoryID ASC);

GO 

-- Hint Entry on Startup - will be deleted due to self cleaning in control procedure
INSERT INTO control.tCustomFailureReport 
	SELECT
		 GETUTCDATE() - 14	AS Timestamp
		,'AllFID'			AS FactoryID
		,'AllPLID'			AS ProductLineID
		,'AllPID'			AS ProductID
		,'Startup'			AS TestSoure
		,'Startup Hint'		AS TestName
		,'HINT'				AS ResultType
		,'Execute (everyday by agent) Stored Procedure control.spCustomFailureReport to see test results here.'	AS ResultText
		,0					AS ResultNumeric

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT control.tCustomFailureReport TO [Username or rolename]
 
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'control'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCustomFailureReport'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'				-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for failure reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the failure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TestSource';
    SET @value      = N'The test source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'TestName';
    SET @value      = N'The test name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultType';
    SET @value      = N'The result type of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ResultText';
    SET @value      = N'The content of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultNumeric';
    SET @value      = N'The numeric result.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS control.tSystemFailureReport; 
GO
/*
	Table for failure reporting
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM control.tSystemFailureReport

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tSystemFailureReport',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tSystemFailureReport','COLUMN',default)
*/

CREATE TABLE control.tSystemFailureReport
(
	 RowKey				BIGINT IDENTITY (1,1) 
	,Timestamp			DATETIME		NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,TestSource			NVARCHAR(255)	NOT NULL
	,TestName			NVARCHAR(255)	NOT NULL
	,ResultType			NVARCHAR(255)	NOT NULL -- OK, ERROR, WARNING
	,ResultText			NVARCHAR(MAX)	NOT NULL
	,ResultNumeric		MONEY			NOT NULL

	,CONSTRAINT PK_tSystemFailureReport PRIMARY KEY NONCLUSTERED (RowKey)
)

GO

-- Table is clustered by Timestamp for faster query the last failure report
CREATE CLUSTERED INDEX IX_tSystemFailureReport_TimeFID ON control.tSystemFailureReport (Timestamp DESC, FactoryID ASC);

GO 

-- Hint Entry on Startup - will be deleted due to self cleaning in control procedure
INSERT INTO control.tSystemFailureReport 
	SELECT
		 GETUTCDATE() - 14	AS Timestamp
		,'AllFID'			AS FactoryID
		,'AllPLID'			AS ProductLineID
		,'AllPID'			AS ProductID
		,'Startup'			AS TestSoure
		,'Startup Hint'		AS TestName
		,'HINT'				AS ResultType
		,'Execute (everyday by agent) Stored Procedure control.spSystemFailureReport to see test results here.'	AS ResultText
		,0					AS ResultNumeric

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT control.tSystemFailureReport TO [Username or rolename]
 
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'control'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tSystemFailureReport'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'				-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for failure reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the failure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TestSource';
    SET @value      = N'The test source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'TestName';
    SET @value      = N'The test name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultType';
    SET @value      = N'The result type of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ResultText';
    SET @value      = N'The content of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultNumeric';
    SET @value      = N'The numeric result.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	
	
	




	
DROP TABLE IF EXISTS global.tCodes;
GO

/*
	Table to store informations about CODEs
	these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCodes

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes','COLUMN',default)
*/

CREATE TABLE global.tCodes (
    RowKey              BIGINT        IDENTITY(1,1)     NOT NULL     
	,ModuleID			NVARCHAR(255)					NOT NULL
	,LanguageID			NVARCHAR(10)                    NOT NULL 
    ,CODE               NVARCHAR(50)                    NOT NULL
	,Scope				NVARCHAR(255)					NOT NULL
    ,[Description]		NVARCHAR(255)                   NOT NULL
	
    ,CONSTRAINT PK_global_tCodes PRIMARY KEY CLUSTERED (RowKey)
);

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tCodes TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCodes'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';

    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
  
 	SET @value = N'Table to store informations about CODEs - these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';

 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value		= N'ID for the Modul which created this Code. Part of combined Business Key ModuleID, LanguageID, CODE. For System Codes use ''system'' as ModuleID.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LanguageID';
    SET @value		= N'Language identifier for the code description. Part of combined Business Key ModuleID, LanguageID, CODE. Identifiers are stored in ISO 639-1 standard (e.g. DE, EN, FR, ...).';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CODE';
    SET @value		= N'Code written in uppercase letters, the CODE must be unique for the modul. Part of combined Business Key ModuleID, LanguageID, CODE.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scope';
    SET @value		= N'Scope for the CODE - can be used to allocate Codes to groups or objects. Use ''default'' if no scope is needed.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	       
	SET @level2name = N'Description';
    SET @value		= N'Description of CODE in specific language.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS global.tCompanies;
GO
/*
	Table to store information about companies.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCompanies

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCompanies','COLUMN',default)
*/

CREATE TABLE global.tCompanies (
     RowKey                   INT        IDENTITY(1,1)                          NOT NULL
    ,CompanyKey               AS (CONCAT(DataSourceKey,N'|',CompanyID))                  
    ,DataSourceKey            INT												NOT NULL -- this is indead an ID which is named Key and INT typed for compatibility
    ,CompanyID                NVARCHAR(50)                                      NOT NULL
    ,CompanyName              NVARCHAR(255)                                     NOT NULL
	,[Description]			  NVARCHAR(2000)									NOT NULL
    ,FinancialYearBegin       INT                                               NOT NULL
    ,ChartsOfAccountsID       NVARCHAR(50)                                      NOT NULL
    ,AccountLength            INT                                               NOT NULL
    ,Currency                 NVARCHAR(50)                                      NOT NULL
    ,CustomValuesJSON         NVARCHAR(MAX)										NOT NULL
    ,CustomValuesJSONUser     NVARCHAR(MAX)                                     NOT NULL
    ,OwnerCODE                NVARCHAR(50)										NOT NULL
    ,CompanyIDResult          NVARCHAR(50)										NOT NULL DEFAULT N''
    ,MappingPriority          INT          										NOT NULL DEFAULT 1

    ,CONSTRAINT PK_global_tCompanies_RowKey PRIMARY KEY CLUSTERED (RowKey)
	,CONSTRAINT AK_global_tCompanies_DataSourceIDCompanyID UNIQUE (DataSourceKey,CompanyID)
);

 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCompanies'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store information about companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyKey';
    SET @value      = N'Technical key for the company consisting of DataSourceID and CompanyID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'DataSourceKey';
    SET @value      = N'Business ID for the Datasource - this column is name Key due to module compatibiltity but is an NVARCHAR ID !.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyID';
    SET @value      = N'Business ID for the company, usually comes from the source system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CompanyName';
    SET @value      = N'The company name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Description';
    SET @value      = N'The description for the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ChartsOfAccountsID';
    SET @value      = N'Business key for the charts of the accounts, referencing global.tFIN_ChartsOfAccounts';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AccountLength';
    SET @value      = N'Number indicating the length of the accounts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Currency';
    SET @value      = N'The currency of the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CustomValuesJSON';
    SET @value      = N'The content of the CostumValues JSON file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CustomValuesJSONUser';
    SET @value      = N'The content of the CostumValues JSON user file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'OwnerCODE';
    SET @value      = N'The Owner CODE, use default INTEGRATION.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CompanyIDResult';
    SET @value      = N'Can be used to change the CompanyID which is used in the result schema for this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'MappingPriority';
    SET @value      = N'In case multiple companies use the same CompanyID in the result this value will be used to determine the priority. The highest priority is 1 followed by 2, 3, ..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
DROP TABLE IF EXISTS global.tPeriods;
PRINT N'Creating table global.tPeriods'
GO
/*
	Table to store informations about time periods.
	Each Year contains 16 Accounting periods, the periods 12...16 are mapped to month 12
	Each Year can be a Financal Year starting with calender month 1 (Financal Year = Calender Year),
	or an Financial Year starting in calender month <> 1.
	 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tPeriods;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods','COLUMN',default)
*/

CREATE TABLE global.tPeriods (
     RowKey                   INT        IDENTITY(1,1)     NOT NULL
	,PeriodKey  AS 
		  (FinancialYear * 10000) 
		+ (FinancialYearBegin * 100) 
		+ [Period]
    ,FinancialYear            INT							NOT NULL -- 2023
    ,[Period]                 INT							NOT NULL -- 1..16
    ,FinancialYearBegin       INT							NOT NULL -- 1..12
    ,FinancialYearName        NVARCHAR(50)					NOT NULL -- 2023 or 2023/2024	 
    ,CalendarYear             INT							NOT NULL -- 2023
    ,CalendarMonth            INT							NOT NULL -- 1..12
    ,CalendarQuarterCODE      NVARCHAR(50)					NOT NULL -- Q1...Q4
    ,CalendarHalfYearCODE     NVARCHAR(50)					NOT NULL -- HY1, HY2
	,YYYYMM					  INT							NOT NULL -- 202301
	,[YYYY-MM]				  NVARCHAR(50)					NOT NULL -- 2023-01						
	,YYYYQ					  INT							NOT NULL -- 20231...20234
	,[YYYY QQ]				  NVARCHAR(50)					NOT NULL --	2023 Q1..2023 Q4
    
	,CONSTRAINT PK_global_tPeriods PRIMARY KEY CLUSTERED (RowKey)
);

-- Default Content Flag
EXEC sp_addextendedproperty
    @name		= N'IsDefaultContent',
    @value		= N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tPeriods'
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tPeriods TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPeriods'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store informations about accounting time periods. Each Finanancial Year contains 16 periods.
				Financial years are defined for each possible start monat. FinancialYearBegin = 1 describes the Financial Year which is equal the calender year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PeriodKey';
    SET @value      = N'Key to identify a period, its automaticly calculated as (FinancialYear * 10000) + (FinancialYearBegin * 100) + [Period].';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYear';
    SET @value      = N'The financial Year YYYY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Period';
    SET @value      = N'The periods 1..16.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year. 1.. 12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearName';
    SET @value      = N'The name of the financial year. 2023 or 2023/2024';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarYear';
    SET @value      = N'The calendar year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarMonth';
    SET @value      = N'The calendar month.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarQuarterCODE';
    SET @value      = N'The CODE for the calendar quarter. Q1..Q4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarHalfYearCODE';
    SET @value      = N'The CODE for the calendar half year. HY1..HY2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYMM';
    SET @value      = N'Year and Month formated as YYYYMM';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY-MM';
    SET @value      = N'Year and Month formated as YYYY-MM, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYQ';
    SET @value      = N'Year and Quarter formated as YYYYQ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY QQ';
    SET @value      = N'Year and Quarter formated as YYYY QQ, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS global.tStructureDescriptions;
GO
/*
	Table to store structures.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tStructureDescriptions

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructureDescriptions',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructureDescriptions','COLUMN',default)
*/

CREATE TABLE global.tStructureDescriptions (
	 RowKey      			BIGINT IDENTITY(1, 1) 			NOT NULL
	,StructureElementID     NVARCHAR(255) 					NOT NULL
	,LanguageID         	NVARCHAR(255) 					NOT NULL
	,Description        	NVARCHAR(255) 					NOT NULL

	,CONSTRAINT PK_global_tStructureDescriptions PRIMARY KEY CLUSTERED (RowKey)
);

--Documentation
DECLARE
     @name          NVARCHAR(255)    = N'MS_Description'
    ,@level0type    NVARCHAR(255)    = N'Schema'
    ,@level0name    NVARCHAR(255)    = N'global'
    ,@level1type    NVARCHAR(255)    = N'Table'
    ,@level1name    NVARCHAR(255)    = N'tStructureDescriptions'
    ,@level2type    NVARCHAR(255)    = N'Column'
    ,@level2name    NVARCHAR(255)    = N''
    ,@value         NVARCHAR(1000)   = N'';

-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';
   
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tStructureDescriptions TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tStructureDescriptions'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'					-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'						-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1								-- 0 = Demo object - out of shipment process
																		-- STANDARD OBJECTS
																		-- 1 = shiped from saxess standard without modification
																		-- 2 = shiped from saxess standard modified FOR customer from saxess
																		-- 3 = shiped from saxess standard modified FOR customer from partner
																		-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		-- CUSTOM OBJECTS
																		-- 10 = shiped from saxess as customer specific object
																		-- 11 = shiped from partner	as customer specific object
																		-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store structures.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureElementID';
    SET @value      = N'Business key for the structure element.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'LanguageID';
    SET @value      = N'Business key for the language.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Description';
    SET @value      = N'The discription of the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP TABLE IF EXISTS global.tStructures;
GO

/*
	Table to store structures
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tStructures

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tStructures','COLUMN',default)
*/

CREATE TABLE global.tStructures (
	 RowKey      			BIGINT       IDENTITY(1, 1) 	NOT NULL
	,StructureID 			NVARCHAR(3)   					NOT NULL
	,ParentID    			NVARCHAR(255) 					NOT NULL
	,StructureElementID      NVARCHAR(255) 					NOT NULL
	,Level       			INT          					NOT NULL

	,CONSTRAINT PK_global_tStructures PRIMARY KEY CLUSTERED (RowKey)
);

-- Documentation
DECLARE
	 @name			NVARCHAR(255)	= N'MS_Description'
    ,@level0type	NVARCHAR(255)	= N'Schema'
    ,@level0name	NVARCHAR(255)	= N'global'
    ,@level1type	NVARCHAR(255)	= N'Table'
    ,@level1name	NVARCHAR(255)	= N'tStructures'
	,@level2type	NVARCHAR(255)	= N'Column'
    ,@level2name	NVARCHAR(255)	= N''
	,@value			NVARCHAR(1000)	= N'';

	-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';
   
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tStructures TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tStructures'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store structures.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureID';
    SET @value      = N'Business key for the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ParentID';
    SET @value      = N'Business key for the parent.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'StructureElementID';
    SET @value      = N'Business key for the structure element.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Level';
    SET @value      = N'Number indicating the level of the structure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdFactories;
GO
/*
	Table to store factories
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdFactories','COLUMN',default)
*/

CREATE TABLE load.tdFactories(
	
	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,FactoryID			NVARCHAR(255)	NOT NULL
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdFactories PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdFactories'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store factories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The person responsible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The name of the image.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'Source';
    SET @value      = N'The source of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdProductLines;
GO
/*
	Table to store ProductLines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProductLines','COLUMN',default)
*/

CREATE TABLE load.tdProductLines(
	 RowKey						INT				NOT NULL IDENTITY (1,1)
	,ProductLineID				NVARCHAR(255)	NOT NULL
	,FactoryID					NVARCHAR(255)	NOT NULL
	,NameShort					NVARCHAR(255)	NOT NULL
	,NameLong					NVARCHAR(255)	NOT NULL
	,CommentUser				NVARCHAR(MAX)	NOT NULL
	,CommentDev					NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson			NVARCHAR(255)	NOT NULL
	,ImageName					NVARCHAR(255)	NOT NULL
	,DefaultTemplate			NVARCHAR(255)	NOT NULL

	,GlobalAttributeSource1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource10	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias10		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource11	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias11		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource12	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias12		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource13	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias13		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource14	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias14		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource15	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias15		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource16	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias16		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource17	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias17		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource18	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias18		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource19	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias19		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource20	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias20		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource21	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias21		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource22	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias22		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource23	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias23		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource24	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias24		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource25	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias25		NVARCHAR(255)	NOT NULL
	,Source						NVARCHAR(255)	NOT NULL
	,SourceTimeStamp			DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdProductLines PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProductLines' -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store ProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DefaultTemplate';
    SET @value      = N'The default template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource1';
    SET @value      = N'The source of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias1';
    SET @value      = N'The alias of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource2';
    SET @value      = N'The source of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias2';
    SET @value      = N'The alias of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource3';
    SET @value      = N'The source of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias3';
    SET @value      = N'The alias of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource4';
    SET @value      = N'The source of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias4';
    SET @value      = N'The alias of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource5';
    SET @value      = N'The source of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias5';
    SET @value      = N'The alias of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource6';
    SET @value      = N'The source of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias6';
    SET @value      = N'The alias of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource7';
    SET @value      = N'The source of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias7';
    SET @value      = N'The alias of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource8';
    SET @value      = N'The source of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias8';
    SET @value      = N'The alias of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource9';
    SET @value      = N'The source of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias9';
    SET @value      = N'The alias of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource10';
    SET @value      = N'The source of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias10';
    SET @value      = N'The alias of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource11';
    SET @value      = N'The source of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias11';
    SET @value      = N'The alias of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource12';
    SET @value      = N'The source of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias12';
    SET @value      = N'The alias of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource13';
    SET @value      = N'The source of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias13';
    SET @value      = N'The alias of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource14';
    SET @value      = N'The source of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias14';
    SET @value      = N'The alias of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource15';
    SET @value      = N'The source of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias15';
    SET @value      = N'The alias of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource16';
    SET @value      = N'The source of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias16';
    SET @value      = N'The alias of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource17';
    SET @value      = N'The source of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias17';
    SET @value      = N'The alias of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource18';
    SET @value      = N'The source of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias18';
    SET @value      = N'The alias of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource19';
    SET @value      = N'The source of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias19';
    SET @value      = N'The alias of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource20';
    SET @value      = N'The source of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias20';
    SET @value      = N'The alias of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	    SET @level2name = N'GlobalAttributeSource21';
    SET @value      = N'The source of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias21';
    SET @value      = N'The alias of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource22';
    SET @value      = N'The source of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias22';
    SET @value      = N'The alias of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource23';
    SET @value      = N'The source of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias23';
    SET @value      = N'The alias of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource24';
    SET @value      = N'The source of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias24';
    SET @value      = N'The alias of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource25';
    SET @value      = N'The source of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias25';
    SET @value      = N'The alias of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdProducts;
GO
/*
	Table to store products
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdProducts','COLUMN',default)
*/

CREATE TABLE load.tdProducts(

	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeType			NVARCHAR(255)	NOT NULL

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL

	,[Status]			NVARCHAR(255)	NOT NULL
	,Template			NVARCHAR(255)	NOT NULL
	,TemplateVersion	NVARCHAR(255)	NOT NULL

	,GlobalAttribute1	NVARCHAR(255)	NOT NULL
	,GlobalAttribute2	NVARCHAR(255)	NOT NULL
	,GlobalAttribute3	NVARCHAR(255)	NOT NULL
	,GlobalAttribute4	NVARCHAR(255)	NOT NULL
	,GlobalAttribute5	NVARCHAR(255)	NOT NULL
	,GlobalAttribute6	NVARCHAR(255)	NOT NULL
	,GlobalAttribute7	NVARCHAR(255)	NOT NULL
	,GlobalAttribute8	NVARCHAR(255)	NOT NULL
	,GlobalAttribute9	NVARCHAR(255)	NOT NULL
	,GlobalAttribute10	NVARCHAR(255)	NOT NULL
	,GlobalAttribute11	NVARCHAR(255)	NOT NULL
	,GlobalAttribute12	NVARCHAR(255)	NOT NULL
	,GlobalAttribute13	NVARCHAR(255)	NOT NULL
	,GlobalAttribute14	NVARCHAR(255)	NOT NULL
	,GlobalAttribute15	NVARCHAR(255)	NOT NULL
	,GlobalAttribute16	NVARCHAR(255)	NOT NULL
	,GlobalAttribute17	NVARCHAR(255)	NOT NULL
	,GlobalAttribute18	NVARCHAR(255)	NOT NULL
	,GlobalAttribute19	NVARCHAR(255)	NOT NULL
	,GlobalAttribute20	NVARCHAR(255)	NOT NULL
	,GlobalAttribute21	NVARCHAR(255)	NOT NULL
	,GlobalAttribute22	NVARCHAR(255)	NOT NULL
	,GlobalAttribute23	NVARCHAR(255)	NOT NULL
	,GlobalAttribute24	NVARCHAR(255)	NOT NULL
	,GlobalAttribute25	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdProducts PRIMARY KEY CLUSTERED (RowKey)

	)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProducts'     -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store products.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeType';
    SET @value      = N'Time type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value      = N'The status of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Template';
    SET @value      = N'The template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TemplateVersion';
    SET @value      = N'The version of the template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute1';
    SET @value      = N'The 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute2';
    SET @value      = N'The 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute3';
    SET @value      = N'The 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute4';
    SET @value      = N'The 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute5';
    SET @value      = N'The 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute6';
    SET @value      = N'The 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute7';
    SET @value      = N'The 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute8';
    SET @value      = N'The 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute9';
    SET @value      = N'The 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute10';
    SET @value      = N'The 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute11';
    SET @value      = N'The 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute12';
    SET @value      = N'The 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute13';
    SET @value      = N'The 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute14';
    SET @value      = N'The 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute15';
    SET @value      = N'The 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute16';
    SET @value      = N'The 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute17';
    SET @value      = N'The 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute18';
    SET @value      = N'The 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute19';
    SET @value      = N'The 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute20';
    SET @value      = N'The 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttribute21';
    SET @value      = N'The 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute22';
    SET @value      = N'The 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute23';
    SET @value      = N'The 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute24';
    SET @value      = N'The 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute25';
    SET @value      = N'The 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Source';
    SET @value      = N'The source of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdTime;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdTime

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdTime',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdTime','COLUMN',default)
*/

CREATE TABLE load.tdTime (
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,TimeID				BIGINT			NOT NULL
	,FormatID			NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdTime PRIMARY KEY (RowKey)

	)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdTime'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the timeline definition per Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimeID';
    SET @value      = N'Business key for the time.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FormatID';
    SET @value      = N'Business key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	    
	SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tdValueSeries;
GO
/*
	Table to store valueseries
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tdValueSeries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tdValueSeries','COLUMN',default)
*/
 
CREATE TABLE load.tdValueSeries(
	 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 

	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL

	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tdValueSeries PRIMARY KEY CLUSTERED (RowKey)

)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdValueSeries'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Business key for the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesNo';
    SET @value      = N'Numer of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsNumeric';
    SET @value      = N'Number to indicate whether valueseries is numeric.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'VisibilityLevel';
    SET @value      = N'Number to indicate whether valueseries is visible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueSource';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueListID';
    SET @value      = N'Valuelist ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormatID';
    SET @value      = N'Valueformat ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'Number indicating the scale of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Effect';
    SET @value      = N'The effect of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectParameter';
    SET @value      = N'The effect parameter of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Source';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.tfValues;
GO
/*
	Table to store values.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.tfValues

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tfValues',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'tfValues','COLUMN',default)
*/

CREATE TABLE load.tfValues(
 
	 RowKey				INT				NOT NULL IDENTITY (1,1)

	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL

	,TimeID				INT				NOT NULL

	,ValueFormula		NVARCHAR(MAX)	NOT NULL	
	,ValueInt			BIGINT			NOT NULL
	,ValueText			NVARCHAR(MAX)	NOT NULL
	,ValueComment		NVARCHAR(MAX)	NOT NULL
	,Source				NVARCHAR(255)	NOT NULL
	,SourceTimeStamp	DATETIME		NOT NULL

	,CONSTRAINT PK_load_tfValues PRIMARY KEY CLUSTERED (RowKey)
)
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfValues'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Business key for the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimeID';
    SET @value      = N'Business key for the time.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormula';
    SET @value      = N'The value of the formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Source';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.trRights;
GO
/*
	Table to store rights
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.trRights

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trRights','COLUMN',default)
*/

CREATE TABLE load.trRights (
	
	 RowKey					INT				NOT NULL IDENTITY (1,1) 
	,UserName				NVARCHAR(255)	NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,ProductLineID			NVARCHAR(255)	NOT NULL
	,ProductID				NVARCHAR(255)	NOT NULL
	,[Right]				NVARCHAR(255)	NOT NULL
	,ReadCommentMandatory	INT				NOT NULL
	,WriteCommentMandatory	INT				NOT NULL
	,Source					NVARCHAR(255)	NOT NULL
	,SourceTimeStamp		DATETIME		NOT NULL

	,CONSTRAINT PK_load_trRights PRIMARY KEY CLUSTERED (RowKey)
)
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT load.trRights TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trRights'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store rights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Right';
    SET @value      = N'Right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadCommentMandatory';
    SET @value      = N'Number indicating whether the read comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'WriteCommentMandatory';
    SET @value      = N'Number indicating whether the write comment is mandatory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Source';
    SET @value      = N'The source of the right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS load.trUser;
GO
/*
	Table to store user of OCT.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM load.trUser

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trUser',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'load', 'TABLE', 'trUser','COLUMN',default)
*/

CREATE TABLE load.trUser(
	
	 RowKey						INT				NOT NULL IDENTITY (1,1) 
	,UserName					NVARCHAR(255)	NOT NULL
	,PersonSurname				NVARCHAR(255)	NOT NULL
	,PersonFirstName			NVARCHAR(255)	NOT NULL
	,Email						NVARCHAR(255)	NOT NULL
	,LDAPIP						NVARCHAR(255)	NOT NULL
	,Status						NVARCHAR(255)	NOT NULL
	,DataFlowRightsCODE			NVARCHAR(255)	NOT NULL
	,VisualizationRightsCODE	NVARCHAR(255)	NOT NULL
	,IsAdministratorFlag		BIT	NOT NULL
	,Source						NVARCHAR(255)	NOT NULL
	,SourceTimeStamp			DATETIME		NOT NULL

	,CONSTRAINT PK_load_trUser PRIMARY KEY CLUSTERED (RowKey)
	)

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT load.trUser TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'load'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUser'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store user of OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'PersonSurname';
    SET @value      = N'The surname of the person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PersonFirstName';
    SET @value      = N'The first name of the person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Email';
    SET @value      = N'Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'LDAPIP';
    SET @value      = N'LDAPIP.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Status';
    SET @value      = N'The status of the user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source of the user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SourceTimeStamp';
    SET @value      = N'The timestamp of the source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdFactories;
GO
/*
	Table for Dimension Factories
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdFactories','COLUMN',default)
*/
 
CREATE TABLE planning.tdFactories(

	 FactoryKey			INT				NOT NULL IDENTITY (1,1)
	,FactoryID			NVARCHAR(255)	NOT NULL
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson	NVARCHAR(255)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dFactories_FactoryKey PRIMARY KEY CLUSTERED (FactoryKey)
);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdFactories'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension Factories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Primary Key for factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The person responsible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The name of the image.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdProductLines;
GO

/*
	Table for Dimension ProductLines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProductLines','COLUMN',default)
*/

CREATE TABLE planning.tdProductLines(
	 ProductLineKey				INT				NOT NULL IDENTITY (1,1)
	,FactoryKey					INT				NOT NULL 
	,ProductLineID				NVARCHAR(255)	NOT NULL
	,FactoryID					NVARCHAR(255)	NOT NULL
	,NameShort					NVARCHAR(255)	NOT NULL
	,NameLong					NVARCHAR(255)	NOT NULL
	,CommentUser				NVARCHAR(MAX)	NOT NULL
	,CommentDev					NVARCHAR(MAX)	NOT NULL
	,ResponsiblePerson			NVARCHAR(255)	NOT NULL
	,ImageName					NVARCHAR(255)	NOT NULL
	,DefaultTemplate			NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias1		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias2		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias3		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias4		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias5		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias6		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias7		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias8		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias9		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource10	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias10		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource11	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias11		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource12	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias12		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource13	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias13		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource14	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias14		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource15	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias15		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource16	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias16		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource17	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias17		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource18	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias18		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource19	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias19		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource20	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias20		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource21	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias21		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource22	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias22		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource23	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias23		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource24	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias24		NVARCHAR(255)	NOT NULL
	,GlobalAttributeSource25	NVARCHAR(255)	NOT NULL
	,GlobalAttributeAlias25		NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dProductLines PRIMARY KEY CLUSTERED (ProductLineKey)
	);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProductLines'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension ProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ProductLineKey';
    SET @value      = N'Primary Key for productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DefaultTemplate';
    SET @value      = N'The default template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource1';
    SET @value      = N'The source of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias1';
    SET @value      = N'The alias of the 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource2';
    SET @value      = N'The source of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias2';
    SET @value      = N'The alias of the 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource3';
    SET @value      = N'The source of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias3';
    SET @value      = N'The alias of the 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource4';
    SET @value      = N'The source of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias4';
    SET @value      = N'The alias of the 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource5';
    SET @value      = N'The source of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias5';
    SET @value      = N'The alias of the 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource6';
    SET @value      = N'The source of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias6';
    SET @value      = N'The alias of the 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource7';
    SET @value      = N'The source of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias7';
    SET @value      = N'The alias of the 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource8';
    SET @value      = N'The source of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias8';
    SET @value      = N'The alias of the 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource9';
    SET @value      = N'The source of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias9';
    SET @value      = N'The alias of the 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource10';
    SET @value      = N'The source of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias10';
    SET @value      = N'The alias of the 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource11';
    SET @value      = N'The source of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias11';
    SET @value      = N'The alias of the 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource12';
    SET @value      = N'The source of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias12';
    SET @value      = N'The alias of the 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource13';
    SET @value      = N'The source of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias13';
    SET @value      = N'The alias of the 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource14';
    SET @value      = N'The source of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias14';
    SET @value      = N'The alias of the 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource15';
    SET @value      = N'The source of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias15';
    SET @value      = N'The alias of the 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource16';
    SET @value      = N'The source of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias16';
    SET @value      = N'The alias of the 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource17';
    SET @value      = N'The source of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias17';
    SET @value      = N'The alias of the 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource18';
    SET @value      = N'The source of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias18';
    SET @value      = N'The alias of the 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource19';
    SET @value      = N'The source of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias19';
    SET @value      = N'The alias of the 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource20';
    SET @value      = N'The source of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias20';
    SET @value      = N'The alias of the 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	    SET @level2name = N'GlobalAttributeSource21';
    SET @value      = N'The source of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias21';
    SET @value      = N'The alias of the 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource22';
    SET @value      = N'The source of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias22';
    SET @value      = N'The alias of the 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource23';
    SET @value      = N'The source of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias23';
    SET @value      = N'The alias of the 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource24';
    SET @value      = N'The source of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias24';
    SET @value      = N'The alias of the 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttributeSource25';
    SET @value      = N'The source of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttributeAlias25';
    SET @value      = N'The alias of the 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS planning.tdProducts;
GO

/*
	Table for Dimension Product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdProducts','COLUMN',default)
*/

CREATE TABLE planning.tdProducts

	(
		 ProductKey			INT				NOT NULL IDENTITY (1,1)
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductID			NVARCHAR(255)	NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,TimeType			NVARCHAR(255)	NOT NULL
		,NameShort			NVARCHAR(255)	NOT NULL
		,NameLong			NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,ResponsiblePerson	NVARCHAR(255)	NOT NULL
		,ImageName			NVARCHAR(255)	NOT NULL
		,[Status]			NVARCHAR(255)	NOT NULL
		,Template			NVARCHAR(255)	NOT NULL
		,TemplateVersion	NVARCHAR(255)	NOT NULL
		,GlobalAttribute1	NVARCHAR(255)	NOT NULL
		,GlobalAttribute2	NVARCHAR(255)	NOT NULL
		,GlobalAttribute3	NVARCHAR(255)	NOT NULL
		,GlobalAttribute4	NVARCHAR(255)	NOT NULL
		,GlobalAttribute5	NVARCHAR(255)	NOT NULL
		,GlobalAttribute6	NVARCHAR(255)	NOT NULL
		,GlobalAttribute7	NVARCHAR(255)	NOT NULL
		,GlobalAttribute8	NVARCHAR(255)	NOT NULL
		,GlobalAttribute9	NVARCHAR(255)	NOT NULL
		,GlobalAttribute10	NVARCHAR(255)	NOT NULL
		,GlobalAttribute11	NVARCHAR(255)	NOT NULL
		,GlobalAttribute12	NVARCHAR(255)	NOT NULL
		,GlobalAttribute13	NVARCHAR(255)	NOT NULL
		,GlobalAttribute14	NVARCHAR(255)	NOT NULL
		,GlobalAttribute15	NVARCHAR(255)	NOT NULL
		,GlobalAttribute16	NVARCHAR(255)	NOT NULL
		,GlobalAttribute17	NVARCHAR(255)	NOT NULL
		,GlobalAttribute18	NVARCHAR(255)	NOT NULL
		,GlobalAttribute19	NVARCHAR(255)	NOT NULL
		,GlobalAttribute20	NVARCHAR(255)	NOT NULL
		,GlobalAttribute21	NVARCHAR(255)	NOT NULL
		,GlobalAttribute22	NVARCHAR(255)	NOT NULL
		,GlobalAttribute23	NVARCHAR(255)	NOT NULL
		,GlobalAttribute24	NVARCHAR(255)	NOT NULL
		,GlobalAttribute25	NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_dProducts PRIMARY KEY CLUSTERED (ProductKey)
	);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdProducts'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for Dimension Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ProductKey';
    SET @value      = N'Primary Key for product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeType';
    SET @value      = N'Time type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'NameShort';
    SET @value      = N'The short name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameLong';
    SET @value      = N'The long name of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ResponsiblePerson';
    SET @value      = N'The responsible person.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value      = N'The status of the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Template';
    SET @value      = N'The template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TemplateVersion';
    SET @value      = N'The version of the template.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute1';
    SET @value      = N'The 1. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute2';
    SET @value      = N'The 2. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute3';
    SET @value      = N'The 3. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute4';
    SET @value      = N'The 4. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute5';
    SET @value      = N'The 5. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute6';
    SET @value      = N'The 6. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute7';
    SET @value      = N'The 7. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute8';
    SET @value      = N'The 8. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute9';
    SET @value      = N'The 9. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute10';
    SET @value      = N'The 10. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute11';
    SET @value      = N'The 11. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute12';
    SET @value      = N'The 12. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute13';
    SET @value      = N'The 13. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute14';
    SET @value      = N'The 14. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute15';
    SET @value      = N'The 15. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute16';
    SET @value      = N'The 16. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute17';
    SET @value      = N'The 17. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute18';
    SET @value      = N'The 18. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute19';
    SET @value      = N'The 19. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute20';
    SET @value      = N'The 20. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'GlobalAttribute21';
    SET @value      = N'The 21. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute22';
    SET @value      = N'The 22. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute23';
    SET @value      = N'The 23. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute24';
    SET @value      = N'The 24. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'GlobalAttribute25';
    SET @value      = N'The 25. global attribute.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdTime;
GO

/*
	Table for Dimension Time in OCT
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdTime

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdTime',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdTime','COLUMN',default)
*/

CREATE TABLE planning.tdTime
	(
	 TimeKey		INT				NOT NULL IDENTITY (1,1)
	,ProductKey		INT				NOT NULL
	,ProductLineKey INT				NOT NULL
	,FactoryKey		INT				NOT NULL
	,ProductID		NVARCHAR(255)	NOT NULL
	,ProductLineID	NVARCHAR(255)	NOT NULL
	,FactoryID		NVARCHAR(255)	NOT NULL
	,TimeID			BIGINT			NOT NULL
	,FormatID		NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dTime_TimeKey PRIMARY KEY CLUSTERED (TimeKey)
	);

-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE INDEX IX_dTime_PKey ON planning.tdTime (ProductKey ASC);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdTime'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the timeline definition per Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'TimeKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductKey';
    SET @value      = N'Foreign technical Key to table planning.tdProducts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'Foreign technical Key to table planning.tdProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Foreign technical Key to table planning.tdFactories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Composite business Key (togehter with ProductLineID, FactoryID) to table planning.tdProducts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Composite business Key (togehter with FactoryID) to table planning.tdProductLines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Foreign business Key to table planning.tdFactories.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimeID';
    SET @value      = N'Primary Business Key, used for time allocation of the value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FormatID';
    SET @value      = N'Foreign Key to table planning.thFormats, to enable format defintion for a TimeID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tdValueSeries;
GO
/*
	Table for Dimension ValueSeries
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tdValueSeries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdValueSeries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tdValueSeries','COLUMN',default)
*/

CREATE TABLE planning.tdValueSeries
	(
	 ValueSeriesKey		INT				NOT NULL IDENTITY (1,1)
	,ProductKey			INT				NOT NULL
	,ProductLineKey		INT				NOT NULL
	,FactoryKey			INT				NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ValueSeriesID		NVARCHAR(255)	NOT NULL
	,ValueSeriesNo		INT				NOT NULL 
	,NameShort			NVARCHAR(255)	NOT NULL
	,NameLong			NVARCHAR(255)	NOT NULL
	,CommentUser		NVARCHAR(MAX)	NOT NULL
	,CommentDev			NVARCHAR(MAX)	NOT NULL
	,ImageName			NVARCHAR(255)	NOT NULL
	,[IsNumeric]		INT				NOT NULL 
	,VisibilityLevel	INT				NOT NULL
	,ValueSource		NVARCHAR(255)	NOT NULL
	,ValueListID		NVARCHAR(255)	NOT NULL
	,ValueFormatID		NVARCHAR(255)	NOT NULL
	,Unit				NVARCHAR(255)	NOT NULL
	,Scale				INT				NOT NULL 
	,Effect				NVARCHAR(255)	NOT NULL
	,EffectParameter	NVARCHAR(255)	NOT NULL

	,CONSTRAINT PK_dValueSeries PRIMARY KEY CLUSTERED (ValueSeriesKey)
	);

	-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE INDEX IX_dValueSeries_ProductKey ON planning.tdValueSeries (ProductKey ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tdValueSeries'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ValueSeriesKey';
    SET @value      = N'Primary Key for valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Valueseries ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesNo';
    SET @value      = N'Valueseries Number.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ImageName';
    SET @value      = N'The image name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsNumeric';
    SET @value      = N'Number to indicate whether valueseries is numeric.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'VisibilityLevel';
    SET @value      = N'Number to indicate whether valueseries is visible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueSource';
    SET @value      = N'The source of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueListID';
    SET @value      = N'Valuelist ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormatID';
    SET @value      = N'Valueformat ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'The scale the valueseries is set to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Effect';
    SET @value      = N'The effect of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectParameter';
    SET @value      = N'The effect parameter of the valueseries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS planning.tFiles;
GO
/*
	Table to store files
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tFiles

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tFiles',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tFiles','COLUMN',default)
*/

CREATE TABLE planning.tFiles (
	 FileKey			BIGINT IDENTITY(1,1)	NOT NULL
	,FactoryKey			INT						NOT NULL
	,ProductLineKey		INT						NOT NULL
	,ProductKey			INT						NOT NULL
	,FactoryID			NVARCHAR(50)			NOT NULL
	,ProductLineID		NVARCHAR(50)			NOT NULL
	,ProductID			NVARCHAR(50)			NOT NULL
	,FileID				NVARCHAR(50)			NOT NULL
	,[FileName]			NVARCHAR(255)			NOT NULL
	,FileExtension		NVARCHAR(50)			NULL
	,FileBody			VARBINARY(MAX)			NULL
	,UploadedBy			NVARCHAR(255)			NOT NULL
	,UploadedAt			DATETIME				NOT NULL
	,Filepath			NVARCHAR(4000)			NOT NULL

	,CONSTRAINT pk_tFiles PRIMARY KEY CLUSTERED (FileKey)
);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tFiles'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store files.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FileKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryKey';
    SET @value      = N'Technical Key for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ProductLineKey';
    SET @value      = N'Technical Key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'ProductKey';
    SET @value      = N'Technical Key for the Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryID';
    SET @value      = N'Business ID for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'Business ID for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductID';
    SET @value      = N'Business ID for the Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileID';
    SET @value      = N'Business ID for the File.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileName';
    SET @value      = N'Name of the File without extension.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'FileExtension';
    SET @value      = N'Separated File extension for better handling.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FileBody';
    SET @value      = N'File Body as binary data.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UploadedBy';
    SET @value      = N'Name of the user which uploaded this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UploadedAt';
    SET @value      = N'Datetime of upload of this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Filepath';
    SET @value      = N'Virtual path of this file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tfStatements;
GO
/*
	Table to store Statements
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tfStatements

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfStatements',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfStatements','COLUMN',default)
*/

CREATE TABLE planning.tfStatements
	(
		 StatementKey	INT				NOT NULL IDENTITY (1,1)
		,ProductKey		INT				NOT NULL
		,ProductLineKey INT				NOT NULL
		,FactoryKey		INT				NOT NULL
		,ProductID		NVARCHAR(255)	NOT NULL
		,ProductLineID	NVARCHAR(255)	NOT NULL
		,FactoryID		NVARCHAR(255)	NOT NULL
		,ActionType		NVARCHAR(255)	NOT NULL
		,[Statement]	NVARCHAR(MAX)	NOT NULL
		,UserName		NVARCHAR(255)	NOT NULL
		,PCName			NVARCHAR(255)	NOT NULL
		,ProcessorCode	NVARCHAR(255)	NOT NULL
		,IPAddresses	NVARCHAR(255)	NOT NULL
		,Timestamp		DATETIME		NOT NULL
		,IsDeleted		INT				NOT NULL
		,IsResolved		INT				NOT NULL

		,CONSTRAINT PK_fStatements PRIMARY KEY CLUSTERED (StatementKey)
	);

-- Table is indexed by ProductKey for fasten Query on ProductDataTable 
CREATE INDEX IX_fStatements_ProductKey ON planning.tfStatements (ProductKey ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfStatements'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store statements.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'StatementKey';
    SET @value      = N'Primary Key for the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ActionType';
    SET @value      = N'Action type.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Statement';
    SET @value      = N'The content of the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'PCName';
    SET @value      = N'Processor Code name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProcessorCode';
    SET @value      = N'Processor Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IPAddresses';
    SET @value      = N'IP adresses.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the statement.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsDeleted';
    SET @value      = N'Number to indicate whether the statement is deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IsResolved';
    SET @value      = N'Number to indicate whether the statement is resolved.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tfValues;
GO
/*
	Table to store all OCT Values
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tfValues

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tfValues','COLUMN',default)
*/

CREATE TABLE planning.tfValues
	(
		 ValueKey		BIGINT			NOT NULL IDENTITY (1,1)
		,ValueSeriesKey INT				NOT NULL
		,ProductKey		INT				NOT NULL
		,ProductLineKey INT				NOT NULL
		,FactoryKey		INT				NOT NULL
		,ProductID		NVARCHAR(255)	NOT NULL
		,ProductLineID	NVARCHAR(255)	NOT NULL
		,FactoryID		NVARCHAR(255)	NOT NULL
		,ValueSeriesID	NVARCHAR(255)	NOT NULL
		,TimeID			INT				NOT NULL
		,ValueFormula	NVARCHAR(MAX)	NOT NULL	
		,ValueInt		BIGINT			NOT NULL
		,ValueText		NVARCHAR(MAX)	NOT NULL
		,ValueComment	NVARCHAR(MAX)	NOT NULL

		,CONSTRAINT PK_fValues_ValueKey PRIMARY KEY CLUSTERED (ValueKey)
	);

-- Table is indexed by ProductKey for fasten Query GET_ProductDataTable Values
CREATE NONCLUSTERED INDEX IX_fValues_ProductKey ON planning.tfValues (ProductKey)
INCLUDE (ValueSeriesKey,ProductID,ProductLineID,FactoryID,ValueSeriesID,TimeID,ValueFormula,ValueInt,ValueText,ValueComment)

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tfValues'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store all OCT Values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ValueKey';
    SET @value      = N'Primary Key for value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ValueSeriesKey';
    SET @value      = N'Valueseries key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueSeriesID';
    SET @value      = N'Valueseries ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimeID';
    SET @value      = N'Time ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueFormula';
    SET @value      = N'The value of the formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgCluster;
GO

/*
	Table to store global Cluster Informations
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgCluster

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgCluster',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgCluster','COLUMN',default)
*/

CREATE TABLE planning.tgCluster

	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL
	
		,CONSTRAINT PK_gCluster_PK PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gCluster_PropID ON planning.tgCluster (PropertyID ASC);
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgCluster'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store global Cluster Informations.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentUser';
    SET @value      = N'User comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CommentDev';
    SET @value      = N'Development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgFactories;
GO
/*
	Table to store Factory Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgFactories

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgFactories',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgFactories','COLUMN',default)
*/

CREATE TABLE planning.tgFactories
  (
	 PropertyKey			INT				NOT NULL IDENTITY (1,1)
    ,FactoryKey				INT				NOT NULL
	,FactoryID				NVARCHAR(255)	NOT NULL
	,PropertyID				NVARCHAR(255)	NOT NULL
	,PropertyName			NVARCHAR(255)	NOT NULL
	,CommentUser			NVARCHAR(MAX)	NOT NULL
	,CommentDev				NVARCHAR(MAX)	NOT NULL
	,Unit					NVARCHAR(255)	NOT NULL
	,ValueText				NVARCHAR(MAX)	NOT NULL
	,ValueInt				INT				NOT NULL
	,Scale					INT				NOT NULL
	,IsROSystemProperty		INT				NOT NULL
	,FormatID				NVARCHAR(255)	NOT NULL

    ,CONSTRAINT PK_gFactories PRIMARY KEY CLUSTERED (PropertyKey)
  );

  -- Clustered on Factory Key, as Queries call Properties per Factory
  CREATE NONCLUSTERED INDEX IX_gFactories_FKey ON planning.tgFactories (FactoryKey ASC);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgFactories'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Factory Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation

	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgProductLines;
GO
/*
	Table to Store ProductLine Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgProductLines

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProductLines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProductLines','COLUMN',default)
*/

CREATE TABLE planning.tgProductLines
	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gProductLines PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gProductLines_PLKey ON planning.tgProductLines (ProductLineKey ASC);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgProductLines'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to Store ProductLine Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.tgProducts;
GO
/*
	Table to store Product Properties
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgProducts

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProducts',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgProducts','COLUMN',default)
*/

CREATE TABLE planning.tgProducts
	(
		 PropertyKey		INT				NOT NULL IDENTITY (1,1)
		,ProductKey			INT				NOT NULL
		,ProductLineKey		INT				NOT NULL
		,FactoryKey			INT				NOT NULL
		,ProductID			NVARCHAR(255)	NOT NULL
		,ProductLineID		NVARCHAR(255)	NOT NULL
		,FactoryID			NVARCHAR(255)	NOT NULL
		,PropertyID			NVARCHAR(255)	NOT NULL
		,PropertyName		NVARCHAR(255)	NOT NULL
		,CommentUser		NVARCHAR(MAX)	NOT NULL
		,CommentDev			NVARCHAR(MAX)	NOT NULL
		,Unit				NVARCHAR(255)	NOT NULL
		,ValueText			NVARCHAR(MAX)	NOT NULL
		,ValueInt			INT				NOT NULL
		,Scale				INT				NOT NULL
		,IsROSystemProperty INT				NOT NULL
		,FormatID			NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gProducts PRIMARY KEY CLUSTERED (PropertyKey)
	);

	CREATE NONCLUSTERED INDEX IX_gProducts_PKey ON planning.tgProducts (ProductKey ASC);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgProducts'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Product Properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'PropertyKey';
    SET @value      = N'Primary Key for the property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'Product key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLine key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'Factory key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value      = N'Factory ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'Product ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLine ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'PropertyID';
    SET @value      = N'Property ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'PropertyName';
    SET @value      = N'Property name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CommentUser';
    SET @value      = N'Comment from user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'CommentDev';
    SET @value      = N'Comment from development.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'Unit';
    SET @value      = N'Unit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scale';
    SET @value      = N'The scale.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;	 

    SET @level2name = N'IsROSystemProperty';
    SET @value      = N'Number to indicate whether the property is a RO system property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

    SET @level2name = N'FormatID';
    SET @value      = N'The ID of the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO

DROP TABLE IF EXISTS planning.tgValueEffects;
GO
/*
	Table to store ValueEffects
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tgValueEffects

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgValueEffects',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tgValueEffects','COLUMN',default)
*/

CREATE TABLE planning.tgValueEffects
	(
		 EffectKey					INT				NOT NULL IDENTITY (1,1)
		,EffectID					NVARCHAR (255)  NOT NULL
		,ProfitLossEffect			INT				NOT NULL
		,BalanceEffect				INT				NOT NULL
		,CashEffect					INT				NOT NULL
		,QuantityEffect				INT				NOT NULL
		,AggregationTypTime			NVARCHAR(255)	NOT NULL
		,AggregationTypStructure	NVARCHAR(255)	NOT NULL

		,CONSTRAINT PK_gValueEffects_PK PRIMARY KEY CLUSTERED (EffectKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tgValueEffects'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store ValueEffects.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'EffectKey';
    SET @value      = N'Primary Key for the value effect.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectID';
    SET @value      = N'ID of the value effect.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProfitLossEffect';
    SET @value      = N'Effect of the profit loss.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'BalanceEffect';
    SET @value      = N'Effect of the balance.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CashEffect';
    SET @value      = N'Effect of the cash.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'QuantityEffect';
    SET @value      = N'Effect of the quantity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AggregationTypTime';
    SET @value      = N'The time of the aggregation typ.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'AggregationTypStructure';
    SET @value      = N'The structure of the aggregation typ.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP TABLE IF EXISTS planning.thFormats;
GO
/*
	Table for format definitions. They are applied on:
		- ValueSeries
		- TimeIDs (only borders at the moment)
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thFormats

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats','COLUMN',default)
*/

CREATE TABLE planning.thFormats 

	(
	 FormatKey				INT				NOT NULL IDENTITY (1,1)
	,FormatID				NVARCHAR(255)	NOT NULL
	,BackgroundColor		NVARCHAR(255)	NOT NULL
	,FontColor				NVARCHAR(255)	NOT NULL
	,ValueFormat			NVARCHAR(255)	NOT NULL
	,Bold					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,Italic					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,AlignmentHorizontal	NVARCHAR(255)	NOT NULL -- left, center, right
	,AlignmentVertical		NVARCHAR(255)	NOT NULL -- top, center, bottom
	,TextUnderlined			INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,BorderToNext			INT				NOT NULL -- Boolean
	,BorderToPrevious		INT				NOT NULL -- Boolean

	,CONSTRAINT PK_hFormats_PK PRIMARY KEY NONCLUSTERED (FormatKey)
	);

CREATE CLUSTERED INDEX IX_hFormats_FormatID ON planning.thFormats (FormatID ASC);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thFormats'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Format Definitions, which are applied on ValueSeries, GlobalAttributes and TimeIDs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FormatKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'Business Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BackgroundColor';
    SET @value      = N'RGB Color Definition, which is applied for Background eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FontColor';
    SET @value      = N'RGB Color Definition, which is applied for Fonts eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueFormat';
    SET @value      = N'Format Definition in englisch XLS Style, which is applied to format values, e.g. #,###0.00 for Thousand Separator and two decimal places.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Bold';
    SET @value      = N'Font is bold Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Italic';
    SET @value      = N'Font is italic Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'AlignmentHorizontal';
    SET @value      = N'Definition of horizontal font alignment, allowed values are left, center, right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AlignmentVertical';
    SET @value      = N'Definition of vertical font alignment, allowed values are top, center, bottom.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TextUnderlined';
    SET @value      = N'Font is underlined Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BorderToNext';
    SET @value      = N'Definition Flag if a border to the next (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'BorderToPrevious';
    SET @value      = N'Definition Flag if a border to the previous (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.thLists;
GO
/*
	Table to store Listnames
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thLists 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thLists ',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thLists ','COLUMN',default)
*/

CREATE TABLE planning.thLists 
	(
		 ListKey		INT NOT NULL IDENTITY (1,1)
		,ListID			NVARCHAR (255)	NOT NULL
		,NameShort		NVARCHAR (255)	NOT NULL
		,NameLong		NVARCHAR (255)	NOT NULL
		,CommentDev		NVARCHAR (255)	NOT NULL
		,CommentUser	NVARCHAR (255)	NOT NULL
		,Datentyp		NVARCHAR (255)	NOT NULL
		,[Source]		NVARCHAR (255)	NOT NULL
		,SourceFormula	NVARCHAR (1000) NOT NULL
		,FormatID		NVARCHAR (255)	NOT NULL

		,CONSTRAINT PK_hLists_PK PRIMARY KEY CLUSTERED (ListKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thLists '       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Listnames.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ListKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ListID';
    SET @value      = N'Bussines Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'NameShort';
    SET @value      = N'The short name of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'NameLong';
    SET @value      = N'The long name of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CommentUser';
    SET @value      = N'The user comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CommentDev';
    SET @value      = N'The development comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Datentyp';
    SET @value      = N'The data type of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Source';
    SET @value      = N'The source of the list.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'SourceFormula';
    SET @value      = N'The source formula.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'Bussines Key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.thListValues;
GO
/*
	Table to store List Values
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thListValues 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thListValues ',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thListValues ','COLUMN',default)
*/

CREATE TABLE planning.thListValues 
	(
		 ListValueKey	INT				NOT NULL IDENTITY (1,1)
		,ListID			NVARCHAR (255)	NOT NULL
		,ValueInt		BIGINT			NOT NULL
		,Scale			BIGINT			NOT NULL
		,ValueText		NVARCHAR (255)	NOT NULL
		,ValueComment	NVARCHAR (255)	NOT NULL
		,FormatID		NVARCHAR (255)	NOT NULL

		,CONSTRAINT PK_hListValues_PK PRIMARY KEY CLUSTERED (ListValueKey)
	);

	CREATE NONCLUSTERED INDEX IX_hListValues_ListID ON planning.thListValues (ListID ASC);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thListValues '          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store List Values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ListValueKey';
    SET @value      = N'Technical Primary Key for list value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ListID';
    SET @value      = N'Business key for List.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Scale';
    SET @value      = N'The scale of the list value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueComment';
    SET @value      = N'The value of the comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FormatID';
    SET @value      = N'Business key for the format.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS planning.trPreferences;
GO
/*
	Table to store Preferences - in contrast to properties, preferences can be userdependend
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.trPreferences

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'trPreferences',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'trPreferences','COLUMN',default)
*/

CREATE TABLE planning.trPreferences
	(
		 PreferenceKey	INT				NOT NULL IDENTITY (1,1) 
		,UserKey		INT				NOT NULL
		,UserName		NVARCHAR (255)	NOT NULL
		,FactoryID		NVARCHAR (255)	NOT NULL
		,ProductLineID	NVARCHAR (255)	NOT NULL
		,SettingName	NVARCHAR (255)	NOT NULL
		,ValueInt		INT				NOT NULL
		,ValueText		NVARCHAR (MAX)	NOT NULL

		,CONSTRAINT PK_rPreferences_PK PRIMARY KEY CLUSTERED (PreferenceKey)
	);

	-- clusterd by FactoryID may not be the best choise - the useful index depends strong on Cluster Configuration
	CREATE NONCLUSTERED INDEX IX_rPreferences_FID ON planning.trPreferences (FactoryID ASC);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trPreferences'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Preferences - in contrast to properties, preferences can be userdependend.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PreferenceKey';
    SET @value      = N'Technical Primary Key for Preference.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'UserKey';
    SET @value      = N'Technical Key for User.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'UserName';
    SET @value      = N'User Name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SettingName';
    SET @value      = N'The setting for the name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'The value of the integer.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueText';
    SET @value      = N'The value of the text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP TABLE IF EXISTS planning.tTabs;
GO
/*
	Table for storing Tabs, which can be used for
		- the cluster
		- the factory
		- the productline
		- the product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'tTabs','COLUMN',default)
*/
 
CREATE TABLE planning.tTabs
    (
         RowKey					BIGINT IDENTITY(1,1)
		,FactoryKey				INT					NOT NULL 				
		,ProductLineKey			INT 				NOT NULL
		,ProductKey				INT 				NOT NULL
		,FactoryID				NVARCHAR(255)		NOT NULL 				
		,ProductLineID			NVARCHAR(255) 		NOT NULL
		,ProductID				NVARCHAR(255) 		NOT NULL
        ,TabID					NVARCHAR(255)		NOT NULL
		,Orderindex				INT					NOT NULL
		,TabName				NVARCHAR(255)		NOT NULL
		,TabHint				NVARCHAR(2000)		NOT NULL
		,TabText				NVARCHAR(2000)		NOT NULL
		,PresentationCODE		NVARCHAR(255)		NOT NULL
		,Datasource				NVARCHAR(2000)		NOT NULL
		,IsVisibleBOOL			INT					NOT NULL
		,LayoutJSON				NVARCHAR(MAX)		NOT NULL
		,ParameterJSON			NVARCHAR(MAX)		NOT NULL

		,CONSTRAINT PK_tTabs PRIMARY KEY CLUSTERED (RowKey)
    )
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tTabs'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Tabs on all levels (Cluster, Factory, ProductLine, Product). Tabs can contain Pivot Tables, DataGrids, HTML Pages etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryKey';
    SET @value      = N'FactoryKey, -1 for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineKey';
    SET @value      = N'ProductLineKey, -1 for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductKey';
    SET @value      = N'ProductKey, -1 for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'FactoryID';
    SET @value      = N'FactoryID, empty string for tabs on Cluster level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProductLineID';
    SET @value      = N'ProductLineID, empty String for tabs on Cluster / Factory level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value      = N'ProductID, empty string for tabs on Cluster / Factory / ProductLine level.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabID';
    SET @value      = N'Business ID for the Tab, used as business identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Orderindex';
    SET @value      = N'Order number for the Tab in the list of tabs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabName';
    SET @value      = N'Name of the Tab, this Name is used as label for the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TabHint';
    SET @value      = N'Text, which is show as Tooltip on the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TabText';
    SET @value      = N'Text, displayed in the header area inside the tab as explination';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PresentationCODE';
    SET @value      = N'Codeword for Presentationtype of the content, e.g. PIVOT, DATAGRID,..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Datasource';
    SET @value      = N'Datasource for the Tab, depending on Presentation CODE - StoredProcedure Name, URL, ...';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutJSON';
    SET @value      = N'Layout definition saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ParameterJSON';
    SET @value      = N'Parameter definition, saved as JSON string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tConnectorQueries;

DROP TABLE IF EXISTS system.tConnectors;
/*
	Table to store connectors.

	This DELETE will delete also
			- the connector queries
	due to DELETE CASCADE
 
	Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectors

		INSERT INTO system.tConnectors VALUES
			 ('NAVISION','MSSQL','FIN','Connector for Navision MSSQL for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/ms-dynamics-nav/', 1)
			,('PROALPHA','ODBC','FIN','Connector for proAlpha ODBC for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/proalpha/',1)
			,('WODIS','ORACLE','FIN','Connector for Wodis Sigma ORACLE for module FIN.','https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/wodis-sigma/',1)

		DELETE FROM system.tConnectors WHERE ConnectorKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectors','COLUMN',default)
*/

CREATE TABLE system.tConnectors
    (
       ConnectorKey				INT				IDENTITY(1, 1) NOT NULL
      ,SourceSystemID			NVARCHAR(50)	NOT NULL
      ,SourceSystemType			NVARCHAR(50)	NOT NULL
      ,ModuleID					NVARCHAR(50)	NOT NULL
      ,ConnectorDescription		NVARCHAR(4000)	NOT NULL
      ,ConnectorDescriptionURL	NVARCHAR(255)	NOT NULL
      ,ConnectorVersion			INT				NOT NULL

      ,CONSTRAINT PK_tConnectors_ConnectorKey PRIMARY KEY CLUSTERED (ConnectorKey)
      ,CONSTRAINT AK_tConnectors_SourceSystemID_SourceSystemType_ModuleID
            UNIQUE (
                   SourceSystemID
                 , SourceSystemType
                 , ModuleID
                   )
    );

GO

ALTER TABLE system.tConnectors WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectors'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemID';
    SET @value      = N'Source System ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceSystemType';
    SET @value      = N'Source System type for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ModuleID';
    SET @value      = N'Module ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescription';
    SET @value      = N'Description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectorDescriptionURL';
    SET @value      = N'Link to Saxess website with detailed description for the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'ConnectorVersion';
    SET @value      = N'Version of the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tAPI_Log;
GO

/*
	Table for logging API Procedure Calls
	Primary Clusterd Key is ordered descending for fast opening the recent log
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tAPI_Log

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tAPI_Log',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tAPI_Log','COLUMN',default)
*/

CREATE TABLE system.tAPI_Log

(
	 LogKey				BIGINT			NOT NULL IDENTITY (1,1)
	,SQLSystemUser		NVARCHAR (255)  NOT NULL
	,UserName			NVARCHAR (255)  NOT NULL
	,TransactUsername	NVARCHAR (255)  NOT NULL
	,ProcedureName		NVARCHAR (255)  NOT NULL
	,ParameterString	NVARCHAR (MAX)  NOT NULL
	,EffectedRows		INT				NOT NULL
	,ReturnCode			INT				NOT NULL
	,TimestampCall		DATETIME		NOT NULL
	,TimestampReturn	DATETIME		NOT NULL
	,Comment			NVARCHAR (4000) NOT NULL
	,ProcessCode		NVARCHAR (255)  NOT NULL

	,CONSTRAINT PK_API_LogKey PRIMARY KEY CLUSTERED (LogKey DESC) 
);
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tAPI_Log'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store user of OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SQLSystemUser';
    SET @value      = N'Name of the SQL system user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value      = N'User name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TransactUsername';
    SET @value      = N'Name of the transact user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'ProcedureName';
    SET @value      = N'Name of the procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ParameterString';
    SET @value      = N'String that holds the parameters.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EffectedRows';
    SET @value      = N'Number of rows affected.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReturnCode';
    SET @value      = N'The return code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'TimestampCall';
    SET @value      = N'The timestamp of the call.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampReturn';
    SET @value      = N'The timestamp of the return.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Comment';
    SET @value      = N'The comment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcessCode';
    SET @value      = N'The process code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
DROP TABLE IF EXISTS system.tConnectorQueries;

/*
	Table to store connector queries.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectorQueries

		INSERT INTO system.tConnectorQueries VALUES
			 (1,'GetCompanies','SELECT 1 AS CompanyID, ''Company A'' AS CompanyName, 1 AS FinancialYearBegin',1,'',GETUTCDATE(),ORIGINAL_LOGIN()),
			 (1,'Accounts','SELECT 1 AS CompanyID, ''SKTO'' AS AccountGroupCODE, ''1234'' AS AccountID, ''BILANZ'' AS AccoutTypeCode, ''Account 1234'' AS AccountName, ''{}'' AS CustomValuesJSON',1,'',GETUTCDATE(),ORIGINAL_LOGIN())

		DELETE FROM system.tConnectorQueries WHERE ConnectorQueryKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries','COLUMN',default)
*/

CREATE TABLE system.tConnectorQueries
    (
       ConnectorQueryKey   INT            IDENTITY(1, 1) NOT NULL
      ,ConnectorKey        INT            NOT NULL
      ,QueryName           NVARCHAR(255)  NOT NULL
      ,QuerySQL            NVARCHAR(MAX)  NOT NULL
      ,QueryVersion        INT            NOT NULL
      ,QueryVersionComment NVARCHAR(4000) NOT NULL
      ,Timestamp           DATETIME       NOT NULL
      ,CreatedBy           NVARCHAR(255)  NOT NULL
       
	  ,CONSTRAINT PK_tConnectorQueries_ConnectorQueryKey PRIMARY KEY CLUSTERED (ConnectorQueryKey)
      ,CONSTRAINT FK_tConnectorQueries_ConnectorKey
            FOREIGN KEY (ConnectorKey)
            REFERENCES system.tConnectors (ConnectorKey)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectorQueries'	-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorQueryKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ConnectorKey';
    SET @value      = N'Foreign Key to system.tConnectors.ConnectorKey.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'QueryName';
    SET @value      = N'Name of the query which also determines the target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'QuerySQL';
    SET @value      = N'SQL query which is executed by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'QueryVersion';
    SET @value      = N'Version of the query. Default = 1. If the query is edited the version goes up in a separate entry. Only MAX(Version) is used by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'QueryVersionComment';
    SET @value      = N'If query is edited it should include a comment about what was changed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Timestamp';
    SET @value      = N'Timestamp when the query was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'CreatedBy';
    SET @value      = N'User who created or edited the query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.tDashboards;
GO

/*
	Table for storing Dashboard definitions
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tDashboards

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDashboards',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDashboards','COLUMN',default)
*/
 
CREATE TABLE system.tDashboards
    (
         DashboardKey			BIGINT				IDENTITY(1,1)
		,DashboardID			NVARCHAR(50)		NOT NULL
		,DashboardName			NVARCHAR(255)		NOT NULL
		,DashboardDescription	NVARCHAR(4000)		NOT NULL
        ,LayoutDefinition		VARBINARY(MAX)      NOT NULL

		,CONSTRAINT PK_tDashboards_DashboardKey PRIMARY KEY CLUSTERED (DashboardKey)
		,CONSTRAINT AK_tDashboards_DashboardID UNIQUE (DashboardID)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tDashboards'    -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;               -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Report definitions.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'DashboardKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DashboardID';
    SET @value      = N'DashboardID as Business Identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DashboardName';
    SET @value      = N'Name for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DashboardDescription';
    SET @value      = N'Description for the Dashboard';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutDefinition';
    SET @value      = N'Definition for the Dashboard (including images etc.) as varbinary content.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.tDataSources;
GO

/*
	Table to store connections to source systems.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tDataSources

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDataSources',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tDataSources','COLUMN',default)
*/

CREATE TABLE system.tDataSources (
     DataSourceKey			BIGINT IDENTITY(1,1)		 NOT NULL
	,DataSourceID			NVARCHAR(50)				 NOT NULL
    ,DataSourceName			NVARCHAR(255)                NOT NULL
    ,SourceType				NVARCHAR(50)                 NOT NULL
    ,ConnectionValuesJSON	NVARCHAR(max)                NOT NULL
    ,CONSTRAINT PK_system_tDataSources_DataSourceKey PRIMARY KEY CLUSTERED (DataSourceKey)
	,CONSTRAINT AK_system_tDataSources_DataSourceID UNIQUE (DataSourceID)

);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tDataSources'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connections to source systems.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'DataSourceKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DataSourceID';
    SET @value      = N'Business ID of the Datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
    
	SET @level2name = N'DataSourceName';
    SET @value      = N'Name of the Datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SourceType';
    SET @value      = N'Sourcetype of the datasource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ConnectionValuesJSON';
    SET @value      = N'The content of the ConnectionValues JSON file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tEmails;
GO
/*
	Table to store Emails.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tEmails

		INSERT INTO system.tEmails VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe',3,'ALWAYS')

	Open Questions:
		- DELETE CASCADE to Files
		- DELETE CASCADE to Pipeline ?

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tEmails','COLUMN',default)
*/

CREATE TABLE system.tEmails 
	(
     EmailKey					INT			IDENTITY(1,1)		NOT NULL
	,EmailID					NVARCHAR(50)					NOT NULL
	,EmailSubject				NVARCHAR(2000)					NOT NULL
    ,EmailRecipients			NVARCHAR(4000)					NOT NULL --List, demlited by ;
	,EmailText					NVARCHAR(MAX)					NOT NULL
	,EmailAttachments			NVARCHAR(2000)					NOT NULL --List of FileIDs, delimited by ;							
	,EmailStatusCODE			NVARCHAR(50)					NOT NULL --PENDING, ERROR, SENT
	,SourceTypeCODE				NVARCHAR(50)					NOT NULL --PIPELINE, EVENT, MANUALLY
	,SourceID					NVARCHAR(50)					NOT NULL --PipelineID or FID-PLID-PID
	,CreationDate				DATETIME						NOT NULL
	,PostDate					DATETIME						NULL
	
    ,CONSTRAINT PK_tEmails_EmailKey PRIMARY KEY CLUSTERED (EmailKey)
	);

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tEmails'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Emails as a queue.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'EmailKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailID';
    SET @value      = N'BusinessID for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailSubject';
    SET @value      = N'Subject for the Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EmailRecipients';
    SET @value      = N'List of Email addresses, separated by semikolon.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'EmailText';
    SET @value      = N'Body for the Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailAttachments';
    SET @value      = N'List of FileIDs, which are stored separate in planning.tFiles';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
		
	SET @level2name = N'EmailStatusCODE';
    SET @value      = N'CODE for the sendig status of an email, PENDING, ERROR, SENT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceTypeCODE';
    SET @value      = N'CODE for the Source, which created this Email PIPELINE, EVENT, MANUALLY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'SourceID';
    SET @value      = N'The ID chain of the object which created this email, eg. DataSource / Pipeline - the DSID-PID or an FID-PLID-PID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'CreationDate';
    SET @value      = N'Datetime,when the Email was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PostDate';
    SET @value      = N'Datetime,when the Email was posted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tIcons;
GO
/*
	Table to store Icons for GUI
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tIcons

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tIcons',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tIcons','COLUMN',default)
*/

CREATE TABLE system.tIcons (
	 IconKey		BIGINT IDENTITY (1,1)	NOT NULL
	,IconName		NVARCHAR (255)			NOT NULL
	,FileExtension	NVARCHAR (50)			NULL
	,FileBody		VARBINARY (MAX)			NOT NULL

	,CONSTRAINT PK_tIcons PRIMARY KEY CLUSTERED (IconKey)
);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tIcons'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store icons for use in application GUI.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'IconKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'IconName';
    SET @value      = N'Name for the Icon as full name including Extension.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FileExtension';
    SET @value      = N'Extension of the image file.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FileBody';
    SET @value      = N'Filebody as binary storage of the icon data.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name; 

	GO

DROP TABLE IF EXISTS system.tLogicIndex;
GO
/*
	Table to store informations about activated Logicmodules.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tLogicIndex

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tLogicIndex',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tLogicIndex','COLUMN',default)
*/

CREATE TABLE system.tLogicIndex
	(
		 RowKey				INT				NOT NULL IDENTITY (1,1) 
		,ModuleID			NVARCHAR(50)	NOT NULL
		,ModuleVersion		NVARCHAR(50)	NOT NULL
		,ActivationDate		DATE			NOT NULL

		,CONSTRAINT PK_LogicIndex_PK PRIMARY KEY NONCLUSTERED (RowKey)
	);

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tLogicIndex'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store informations about activated Logicmodules. All activated modules are included. On deactivation modules will be deleted in this table.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'RowKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value		= N'ID of the module as string like FIN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleVersion';
    SET @value		= N'Version of the module as String like 2.0';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ActivationDate';
    SET @value		= N'Date when this module were activated. Only the last activation date is to see.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineStepLog;
 
DROP TABLE IF EXISTS system.tPipelineLog;
GO
/*
	Table for storing pipeline executions. Table materialises all details, to show full informations also in case of deleted / modified pipelines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineLog;

		DELETE FROM system.tPipelineLog WHERE LogKey = 2; --testing delete cascade

	TestData
		INSERT INTO system.tPipelineLog VALUES
			('1','Pipe','E1','Server','RUNNING','Hase','20221230 08:08:08.888','20221230 08:08:08.888')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineLog
    (
         LogKey					BIGINT					IDENTITY(1,1)
		,PipelineID				NVARCHAR(50)			NOT NULL
		,PipelineName			NVARCHAR(255)			NOT NULL
		,ExecutionID			NVARCHAR(50)			NOT NULL
		,ExecutionServerName	NVARCHAR(255)			NOT NULL
        ,StatusCODE				NVARCHAR(50)			NOT NULL
		,StartedBy				NVARCHAR(255)			NOT NULL
		,StartTime				DATETIME				NULL
		,EndTime				DATETIME				NULL

		,CONSTRAINT PK_tPipelineLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC)
		,CONSTRAINT AK_tPipelines_PID_EID  UNIQUE (PipelineID,ExecutionID)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineLog'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;               -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to Logs per Pipeline Execution. Step Logs are saved separatly in system.StepLog';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'This column just holds a sample text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineName';
    SET @value      = N'Name of the pipeline - materialized to be historic correct.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Generated Key on first call to Kep Pipeline and Steps together.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionServerName';
    SET @value      = N'URL of the Server, which executed this pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartedBy';
    SET @value      = N'Name of the user who started this pipeline, or the word scheduler if it was scheduler started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime showing start of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime showing end of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineSteps;
DROP TABLE IF EXISTS system.tPipelineSchedules;

DROP TABLE IF EXISTS system.tPipelines;
/*
	Table to store pipelines.
	This DELETE will delete also
			- the steps
			- the schedules
	due to DELETE CASCADE
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines

		INSERT INTO system.tPipelines VALUES
			 ('P1','Pipe ONE'	,'Hello Pipe','#808080',1,'ALWAYS')
			,('P2','Pipe TWO'	,'Hello Pipe','#808080',2,'ALWAYS')
			,('P3','Pipe THREE'	,'Hello Pipe','#808080',3,'ALWAYS')

		DELETE FROM system.tPipelines WHERE PipelineID = 'P2';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelines','COLUMN',default)
*/

CREATE TABLE system.tPipelines 
	(
     PipelineKey					INT			IDENTITY(1,1)		NOT NULL
	,PipelineID						NVARCHAR(50)					NOT NULL
    ,PipelineName					NVARCHAR(255)					NOT NULL   
	,PipelineDescription			NVARCHAR(4000)					NOT NULL
	,PipelineDescriptionIconColor	NVARCHAR(255)					NOT NULL 
	,OrderOfDisplay					INT								NOT NULL
	,EmailNotificationCODE			NVARCHAR(50)					NOT NULL --NEVER, ALWAYS, ONERROR
	
    ,CONSTRAINT PK_tPipelines_PipelineKey PRIMARY KEY CLUSTERED (PipelineKey)
	,CONSTRAINT AK_tPipelines_PipelineID  UNIQUE (PipelineID)
	);

GO

ALTER TABLE system.tPipelines WITH CHECK CHECK CONSTRAINT ALL;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelines'			-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineID';
    SET @value      = N'Business ID for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineName';
    SET @value      = N'Name of the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'PipelineDescription';
    SET @value      = N'Description for the Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineDescriptionIconColor';
    SET @value      = N'Color of the Pipeline Icon, written as Hex Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfDisplay';
    SET @value      = N'Stores at which position the process pipeline should be displayed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EmailNotificationCODE';
    SET @value      = N'CODE word to aktivate Email notifications - NEVER, ALWAYS, ONERROR.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tPipelineSchedules;
GO
/*
	Table to store defined schedules for pipelines.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineSchedules

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSchedules','COLUMN',default)
*/

CREATE TABLE system.tPipelineSchedules
	(
	 ScheduleKey			INT IDENTITY(1,1)	NOT NULL
	,PipelineKey			INT					NOT NULL
	,ScheduleID				NVARCHAR(50)		NOT NULL
	,ScheduleName			NVARCHAR(255)		NOT NULL
	,ScheduleDescription	NVARCHAR(4000)		NOT NULL 
	,ScheduleDetailsJSON	NVARCHAR(MAX)		NOT NULL
	,Active					BIT					NOT NULL
	,CONSTRAINT PK_tPipelineSchedules PRIMARY KEY CLUSTERED (ScheduleKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSchedules_PipelineKeyScheduleID UNIQUE (PipelineKey,ScheduleID)
	,CONSTRAINT FK_tPipelineSchedules_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSchedules' -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store defined schedules for pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ScheduleKey';
    SET @value      = N'Stores the generated primary key of the table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Stores the PipelineKey that is executed with this scheduler job.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleID';
    SET @value      = N'ID for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleName';
    SET @value      = N'Name for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ScheduleDescription';
    SET @value      = N'Description of the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ScheduleDetailsJSON';
    SET @value      = N'Stores the rhythms which determines when and how often a pipeline is executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Stores if the scheduled job is currently active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tPipelineStepLog;
GO
/*
	Table for storing Log events per Step
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineStepLog

	Dependend Tables
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps
		SELECT * FROM system.tPipelineLog

	TestData
		INSERT INTO system.tPipelineStepLog VALUES
			 ('1','1','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')
			,('1','2','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineStepLog
    (
         LogKey			BIGINT					IDENTITY(1,1)
		,PipelineID		NVARCHAR(50)			NOT NULL
		,StepID			NVARCHAR(50)			NOT NULL
		,ExecutionID	NVARCHAR(50)			NOT NULL
		,StepName		NVARCHAR(255)			NOT NULL
		,StartTime		DATETIME				NULL
		,EndTime		DATETIME				NULL
		,LogMessage		NVARCHAR(MAX)			NOT NULL
		,StatusCODE		NVARCHAR(50)			NOT NULL
		,HandleID		NVARCHAR(50)			NOT NULL

		,CONSTRAINT PK_tPipelineStepLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC) 
		,CONSTRAINT AK_tPipelineStepLog_StepIDExecutionID UNIQUE (StepID,ExecutionID)
		-- Foreign Key relation to PipelineLog to delete Step Logs, when deleting the PipelineLog ExecutionID
		,CONSTRAINT FK_tPipelineLog_LogKey FOREIGN KEY (PipelineID,ExecutionID)
			REFERENCES system.tPipelineLog (PipelineID,ExecutionID)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineStepLog'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;                -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store logs per exectuted step'
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'ID of the pipeline this step belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepID';
    SET @value      = N'ID of the step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Relation to the pipeline log.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepName';
    SET @value      = N'Name of the Step to keep history.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime the step was started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime the step was finished.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LogMessage';
    SET @value      = N'Messages from logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, WAITING (Pipeline was started and step queued), STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'HandleID';
    SET @value      = N'ID for Penatho Job handling.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
	
DROP TABLE IF EXISTS system.tPipelineSteps;
GO
/*
	Table to store pipeline steps

	They have a foreign key constraint to the the system.tPipelinesTable and are deleted automaticly, if the Pipeline is deleted
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps

		INSERT INTO system.tPipelineSteps VALUES
			 (1,'1','Step 1','Hallo Step',1,'1','{}','{}',30,1,'FIN',0)
			,(1,'2','Step 2','Hallo Step',2,'1','{}','{}',30,1,'FIN',0)
			,(1,'3','Step 3','Hallo Step',3,'1','{}','{}',30,1,'FIN',0)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps','COLUMN',default)
*/

CREATE TABLE system.tPipelineSteps (
     StepKey					INT        IDENTITY(1,1)    NOT NULL		
	,PipelineKey				INT                         NOT NULL
	,StepID						NVARCHAR(50)				NOT NULL
    ,StepName					NVARCHAR(255)               NOT NULL		
	,StepDescription			NVARCHAR(4000)				NOT NULL
	,OrderOfExecution			INT                         NOT NULL
	,StepDataSourceID			NVARCHAR(50)				NOT NULL
	,StepCompanyIDsJSON			NVARCHAR(MAX)  				NOT NULL
	,StepDetailsJSON			NVARCHAR(MAX)  				NOT NULL		
	,TimeExecutionLimit			INT							NOT NULL		
	,Active						BIT							NOT NULL
	,ModuleID					NVARCHAR(255)				NOT NULL
	,StartNextStepOnError		BIT							NOT NULL
	-- Primary Key
    ,CONSTRAINT PK_tPipelineSteps_StepKey PRIMARY KEY CLUSTERED (StepKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSteps_PipelineKeyStepID  UNIQUE (PipelineKey,StepID)
	,CONSTRAINT FK_tPipelines_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSteps'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process steps.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'StepKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Foreign key to tPipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepName';
    SET @value      = N'Name of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepDescription';
    SET @value      = N'Description of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfExecution';
    SET @value      = N'Stores at which position of the process pipeline the step should be executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepDataSourceID';
    SET @value      = N'ID of the Datasource which is used from this Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepCompanyIDsJSON';
    SET @value      = N'JSON Array with the IDs of the companies activated for this step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepDetailsJSON';
    SET @value      = N'The details of this step, content depending from the step type CODE.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeExecutionLimit';
    SET @value      = N'The time limit for the execution in minutes, step is skiped after this limit, use 0 for no limit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Flag if the step is active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value      = N'ID of the Module, if the Step has the property module, else empty string';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tProcessLogs;
GO
/*
	Table to store log entries of different application processes.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tProcessLogs;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tProcessLogs',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tProcessLogs','COLUMN',default)
*/

CREATE TABLE system.tProcessLogs
	(
		 RowKey				INT IDENTITY(1,1)	NOT NULL
		,ModuleCODE			NVARCHAR(50)		NOT NULL
		,ProcessName		NVARCHAR(255)		NOT NULL
		,ProcessID			NVARCHAR(50)		NULL
		,SubProcessName		NVARCHAR(255)		NULL
		,SubProcessID		NVARCHAR(50)		NULL
		,SubSubProcessName	NVARCHAR(255)		NULL
		,EventCODE			NVARCHAR(20)		NOT NULL
		,Timestamp			DATETIME			NOT NULL
		,ReturnCODE			INT					NULL
		,EventMessage		NVARCHAR(MAX)		NULL

		,CONSTRAINT PK_tProcessLogs PRIMARY KEY CLUSTERED  (RowKey DESC)
 );

 -- OCTService has direkt SELECT rights due to sending this log table to monitoring service.
 GRANT SELECT ON system.tProcessLogs TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tProcessLogs'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store log entries of all application processes which use monitoring services.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ModuleCODE';
    SET @value      = N'Specification of the module that created this log entry, e.g. Server, Integration, Planning, Visualization.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProcessName';
    SET @value      = N'Name of the process that is logged here.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcessID';
    SET @value      = N'ID of the process that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SubProcessName';
    SET @value      = N'Name of the subprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SubProcessID';
    SET @value      = N'ID of the subprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
		 
    SET @level2name = N'SubSubProcessName';
    SET @value      = N'Name of the subsubprocess that is logged here (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'EventCODE';
    SET @value      = N'Description of the event that occurred in the process, can be: Start, End, Info.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'Date and time when this event occurred.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReturnCODE';
    SET @value      = N'Return code of the process that has been executed, use HTTP return codes (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EventMessage';
    SET @value      = N'Details of this event, e.g. a log message or an exception that occurred (optional).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tReports;
GO
/*
	Table for storing Reportdefinitions
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tReports

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tReports',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tReports','COLUMN',default)
*/
 
CREATE TABLE system.tReports
    (
         ReportKey			BIGINT				IDENTITY(1,1)
		,ReportID			NVARCHAR(50)		NOT NULL
		,ReportName			NVARCHAR(255)		NOT NULL
		,ReportDescription	NVARCHAR(4000)		NOT NULL
        ,LayoutDefinition	VARBINARY(MAX)      NOT NULL

		,CONSTRAINT PK_tReports_ReportKey PRIMARY KEY CLUSTERED (ReportKey)
		,CONSTRAINT AK_tReports_ReportID UNIQUE (ReportID)
	);
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tReports'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;                -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Report definitions.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ReportKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReportID';
    SET @value      = N'ReportID as Business Identifier';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReportName';
    SET @value      = N'Name for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReportDescription';
    SET @value      = N'Description for the Report';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LayoutDefinition';
    SET @value      = N'Definition for the report (including images etc.) as varbinary content.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP TABLE IF EXISTS system.trRights;
GO

/*
	Table to store defined Rights per User.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trRights;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trRights','COLUMN',default)
*/
 
CREATE TABLE system.trRights
	(
		 RightKey				INT				NOT NULL IDENTITY (1,1) 
		,UserKey				INT				NOT NULL
		,UserName				NVARCHAR (255)	NOT NULL
		,FactoryID				NVARCHAR (255)	NOT NULL
		,ProductLineID			NVARCHAR (255)	NOT NULL
		,ProductID				NVARCHAR (255)	NOT NULL
		,[Right]				NVARCHAR (255)	NOT NULL
		,ReadCommentMandatory	INTEGER			NOT NULL
		,WriteCommentMandatory	INTEGER			NOT NULL

		,CONSTRAINT PK_rRights_PK PRIMARY KEY CLUSTERED (RightKey)
	);

	CREATE NONCLUSTERED INDEX IX_rRights_UserKey ON system.trRights (UserKey ASC);

GO
-- GRANT
-- NO GRANT - table in system schema can be accessed from FactoryService only over Stored procedures
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trRights'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store defined Rights per User - only positiv given rights are stored on highest level.
					Resolving Rights to effectiv Rights for all levels is done to table system.trUserRights';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'RightKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserKey';
    SET @value		= N'Technical Key for join with system.trUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value		= N'Business Key for join with system.trUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value		= N'FactoryID to which the right belongs. Empty FactoryID stands for clusterrights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value		= N'ProduclineID to which the right belongs. Empty ProductLineID stands for FactoryRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductID';
    SET @value		= N'not in use in this version';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Right';
    SET @value		= N'Capitalized Word to define the right. Only "Read" or "Write".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReadCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every read action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'WriteCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every write action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.trUser;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trUser

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser','COLUMN',default)
*/

CREATE TABLE system.trUser
	(
		 UserKey					INT				NOT NULL IDENTITY (1,1) 
		,UserName					NVARCHAR (255)	NOT NULL
		,PersonSurname				NVARCHAR (255)	NOT NULL
		,PersonFirstName			NVARCHAR (255)	NOT NULL
		,Email						NVARCHAR (255)	NOT NULL
		,LDAPIP						NVARCHAR (255)	NOT NULL
		,Status						NVARCHAR (255)	NOT NULL
		,DataFlowRightsCODE			NVARCHAR (255)	NOT NULL -- DESIGNER, EXECUTOR, NONE
		,VisualizationRightsCODE	NVARCHAR (255)	NOT NULL -- DESIGNER, VIEWER, NONE
		,IsAdministratorFlag		BIT				NOT NULL

		,CONSTRAINT PK_rUser_PK PRIMARY KEY CLUSTERED (UserKey)
	);

GO

-- GRANT
-- temporary GRANTs for procedures / functions in dbo schema (e.g. dbo.sx_pf_GET_ClusterAccessRight)
GRANT SELECT ON system.trUser TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUser'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store user of OCT.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'UserKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Username';
    SET @value		= N'Username for login - usually an User with Domain e.g. SX\GTA or an email Address.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonSurname';
    SET @value		= N'Surname of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PersonFirstName';
    SET @value		= N'FirstName of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Email';
    SET @value		= N'Email Address of the User - optional.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'LDAPIP';
    SET @value		= N'LDAPIP.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Status';
    SET @value		= N'Status of the user account. Capitalized word - may be "Active" or "Inactive".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'DataFlowRightsCODE';
    SET @value		= N'CODE Word for the DataFlow Rights of the user (DESIGNER, EXECUTOR, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'VisualizationRightsCODE';
    SET @value		= N'CODE Word for the Visualization Rights of the user (DESIGNER, VIEWER, NONE).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'IsAdministratorFlag';
    SET @value		= N'Value 0/1 to flag if this user is an administorator of this OCT Database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.trUserRights;
GO
/*
	Table to store effective positiv Rights per User for all level.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.trUserRights

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUserRights',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUserRights','COLUMN',default)
*/

CREATE TABLE system.trUserRights

	(
		 [Row]					BIGINT			NOT NULL IDENTITY (1,1) 
		,UserKey				INT				NOT NULL
		,UserName				NVARCHAR (255)	NOT NULL
		,FactoryID				NVARCHAR (255)	NOT NULL
		,ProductLineID			NVARCHAR (255)	NOT NULL
		,[Right]				NVARCHAR (255)	NOT NULL
		,ReadCommentMandatory	INT				NOT NULL
		,WriteCommentMandatory	INT				NOT NULL

		,CONSTRAINT PK_rUserRights_PK PRIMARY KEY CLUSTERED ([Row])
	);

GO

-- explicit GRANT for FactoryService, as Table must be joined in many Procedures
GRANT SELECT ON system.trUserRights TO pf_PlanningFactoryService;
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'trUserRights'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
	SET @value = N'Table to store effective positiv Rights per User for all level. Table will be automaticly managed by calculation proceduer. Use in most outputs, which filter on Userrights.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
	SET @level2name = N'Row';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserKey';
    SET @value		= N'Technical Key for User.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'UserName';
    SET @value		= N'Business Key for User';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'FactoryID';
    SET @value		= N'FactoryID to which the right belongs. Empty FactoryID stands for clusterrights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ProductLineID';
    SET @value		= N'ProduclineID to which the right belongs. Empty ProductLineID stands for FactoryRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Right';
    SET @value		= N'Capitalized Word to define the right. Only "Read" or "Write".';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ReadCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every read action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'WriteCommentMandatory';
    SET @value		= N'Boolean to requird a comment for every write action in this scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP TABLE IF EXISTS system.tSettings;
GO
/*
	Table to store global database properties.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tSettings

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tSettings','COLUMN',default)
*/

CREATE TABLE system.tSettings (
     RowKey						INT  IDENTITY(1,1)  NOT NULL
    ,SettingID					NVARCHAR(50)		NOT NULL 
	,SettingDescription			NVARCHAR(4000)		NOT NULL
	,DataTypeCODE				NVARCHAR(255)		NOT NULL
	,ValueText					NVARCHAR(4000)		NOT NULL CONSTRAINT Default_ValueText		DEFAULT ''
	,ValueDateTime				DATETIME			NOT NULL CONSTRAINT Default_ValueDateTime	DEFAULT '1900-01-01 00:00:00.000'
	,ValueInt					BIGINT				NOT NULL CONSTRAINT Default_ValueInt		DEFAULT 0
	,ValueIntScale				INT					NOT NULL CONSTRAINT Default_ValueIntScale	DEFAULT 1
	,ValueJSON					NVARCHAR(MAX)		NOT NULL CONSTRAINT Default_ValueJSON		DEFAULT ''
	,Category					NVARCHAR(255)		NOT NULL
	,ReadOnlySystemPropertyBool  INT				NOT NULL

   ,CONSTRAINT PK_tSettings PRIMARY KEY CLUSTERED (RowKey)
);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tSettings'      -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store global database properties.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SettingID';
    SET @value      = N'SettingID as business identifier for the entry.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'SettingDescription';
    SET @value      = N'Description or comment of this setting for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DataTypeCODE';
    SET @value      = N'DataTypeCODE of setting that shows which value column is to used and how it should be used. Can be TEXT, DATETIME, BOOLEAN, NUMBER, JSON. See documentation for DataTypeCodes.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueText';
    SET @value      = N'Text value if value TEXT is set for column DataType, optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueDateTime';
    SET @value      = N'DateTime value if value DATETIME is set for column DataType, optional to submit default Value 1900-01-01 00:00:00.000 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueInt';
    SET @value      = N'Integer value if value BOOLEAN or NUMBER is set for Column DataType. In Case of Boolean: 0,1,-1 for false,true and unknown. NUMBER - an Integer Number which must be scaled with column ValueIntScale. Optional to submit default Value 0 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ValueIntScale';
    SET @value      = N'Integer value of 1 for BOOLEAN value. Value of 1, 10, 100, 10000 for scaling type NUMBER to decimal value -> divide column ValueInt by column ValueIntScale. Optional to submit default Value 1 is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueJSON';
    SET @value      = N'JSON value if DateTypeCODE is set to JSON. Optional to submit default Value '' is set.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Category';
    SET @value      = N'Category to group items.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ReadOnlySystemPropertyBool';
    SET @value      = N'Flag to signalize if property is changeable by user or is a Ready Only System Value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

PRINT '+++ Starting Batch 6 (Build Index After Update) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
-- Builds an Index named #CoreObjects_AfterUpdate of all existing OCT.Core Objects and materializes it in an temporary table	
	
	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 6 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
	BEGIN
		DECLARE @Counter	INT = 0;
		-- Put all Core objects into a temp table
		DROP TABLE IF EXISTS #CoreObjects_AfterUpdate;

		CREATE TABLE #CoreObjects_AfterUpdate
			(
				 ObjectCode			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,ObjectType			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,SX_Owner			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,SX_ShipmentFlag	NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,ShipmentStatus		NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,SchemaName			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,ObjectName			NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL
				,FullObjectName		NVARCHAR(255)  COLLATE DATABASE_DEFAULT NOT NULL 
				,Description		NVARCHAR(4000) COLLATE DATABASE_DEFAULT NOT NULL
			);

		INSERT INTO #CoreObjects_AfterUpdate
			SELECT 
				 o.type				AS ObjectCode
				,CASE o.type
						WHEN 'P'	THEN N'Stored Procedure'
						WHEN 'U'	THEN N'Table'
						WHEN 'V'	THEN N'View'
						WHEN 'IF'	THEN N'Function (Table Value Inline)'
						WHEN 'FN'	THEN N'Function (Scalar)'
						WHEN 'TF'	THEN N'Function (Table Value)'
						WHEN 'TR'	THEN N'Trigger'
						ELSE			 N'Unknown Objecttype'
				 END				AS ObjectType 
				,COALESCE(CONVERT(NVARCHAR(255),sxo.SX_Owner),'unknown')	AS SX_Owner
				,COALESCE(CONVERT(NVARCHAR(255),sxs.SX_ShipmentFlag),'0')	AS SX_ShipmentFlag
				-- determine name to inform user about replacement
				,CASE 
					WHEN sxs.SX_ShipmentFlag	= N'0'	THEN	N'0. Demo object - out of shipment process'
					WHEN sxs.SX_ShipmentFlag	= N'1'	THEN	N'1. Standard - without modification'
					WHEN sxs.SX_ShipmentFlag	= N'2'	THEN	N'2. Standard - modified from saxess for customer'
					WHEN sxs.SX_ShipmentFlag	= N'3'	THEN	N'3. Standard - modified from partner for customer'
					WHEN sxs.SX_ShipmentFlag	= N'4'	THEN	N'4. Standard - modified from customer for himself'
												  				
					WHEN sxs.SX_ShipmentFlag	= N'10'	THEN	N'10. Created from Saxess for Customer'
					WHEN sxs.SX_ShipmentFlag	= N'11'	THEN	N'11. Created from Partner for Customer'
					WHEN sxs.SX_ShipmentFlag	= N'12'	THEN	N'12. Created from Customer for Customer'

					WHEN sxs.SX_ShipmentFlag	IS NULL THEN	N'<module defintion tag not definded>'

					ELSE										N'unknown Shipment Status'
				 END									AS ShipmentStatus
				,S.name									AS SchemaName
				,O.name									AS ObjectName
				,S.name	+'.' + O.name					AS FullObjectName
				,CONVERT(NVARCHAR(4000),
				 CASE
					WHEN ep.value = ''		THEN	N'Description not defined'
					WHEN ep.value IS NULL	THEN	N'<description defintion tag not definded>'
					WHEN ep.value <> ''		THEN	ep.value
					ELSE							N'<procedure failure - this should not had happen>'
				 END)								AS Description
				
			FROM sys.all_objects O
				LEFT JOIN sys.extended_properties EP 
					ON ep.major_id = O.object_id 
				LEFT JOIN sys.schemas S 
					ON O.schema_id = S.schema_id

				--join SX_Owner as separte column for filtering
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_Owner
								FROM sys.extended_properties
								WHERE Name = N'SX_Owner'
							) sxo
							ON sxo.major_id = ep.major_id

				--join SX_ShipmentFlag as separte column
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_ShipmentFlag
								FROM sys.extended_properties
								WHERE Name = N'SX_ShipmentFlag'
							) sxs
							ON sxs.major_id = ep.major_id

			WHERE 
					O.is_ms_shipped = 0
				AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
				AND O.type NOT IN (  'PK' -- PrimaryKeys
									--,'TR' -- Trigger
									,'D'  -- default constraints
									)
				AND (ep.name = 'MS_Description' OR ep.name IS NULL)
				AND sxo.SX_Owner = 'OCT.core'
				AND O.name NOT LIKE '%_bak'
				ORDER BY ObjectType;

		--	SELECT * FROM #CoreObjects_AfterUpdate WHERE ObjectCode = 'U' ORDER BY FullObjectName

		SELECT @Counter=COUNT(*) FROM #CoreObjects_AfterUpdate;
		PRINT CONCAT('Anzahl der Objekte (ohne Trigger) im Index nach Update: ',@Counter);
		SET @Counter = 0;

	END

GO
PRINT '+++ Ended Batch 6 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	

PRINT '+++ Starting Batch 7 (Blacklist Definition) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'	
	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 7 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
		BEGIN
			DECLARE @Counter INT = 0;
			DROP TABLE IF EXISTS #Blacklist;
		 
			CREATE TABLE #Blacklist
				(
				 FullTableName NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				);
			 INSERT INTO #Blacklist VALUES 
					-- manuel to handle due to name or structure change
			
					-- tables missing identity column as comma sparated list in brackets
					-- tables missing identity column

					 (N'planning.tAPI_Log')			-- to rename to system.tAPI_Log
					,(N'system.tPipelines')			-- To add field icon color
					,(N'system.tPipelineSteps')		-- no change, but ForeignKey to tPipelines
					,(N'system.tPipelineSchedules')	-- no change, but ForeignKey to tPipelines
					,(N'planning.tFiles')			-- To add three new columns
					,(N'system.trUser')				-- to move to system schema, add columns, CAPITLIZE CODE Words
					,(N'system.tDataSources')		-- to remove column SourceSystem
					,(N'planning.tfStatements')		-- Change Datatype to Datetime
					,(N'global.tCompanies')			-- New Columns for Mapping
					,(N'global.tCompanyMapping')	-- no longer needed
					,(N'global.tPeriods')			-- more Columns for Reporting
					
					-- as usual
					,('planning.tAPI_Log_History')
					,('load.tdFactories')
					,('load.tdProductlines')
					,('load.tdProducts')
					,('load.tdTime')
					,('load.tdValueSeries')
					,('load.tfValues')
					,('load.trRights')
					,('load.trUser');

			--SELECT * FROM #Blacklist	

			SELECT @Counter = COUNT(*) FROM #Blacklist;
			PRINT CONCAT('Anzahl der Tabellen in der Blacklist: ',@Counter)
		END
	
GO
PRINT '+++ Ended Batch 7 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'



PRINT '+++ Starting Batch 8 (Autotransfer) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'	
	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 8 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
		BEGIN

			--transfer the data from the _bak tables to the new tables *************************************************************
			-- if a temporatery Table in database collation is joint with system tables, the system table values must be forced to be collated like the database
			-- this situation happens if you transfer a local created database to azure (or to a server with different server collation than the local server)

			-- create list of columns for all tables, which are needed for the insert command
			DECLARE @Counter INT = 0;
			DROP TABLE IF EXISTS #Columnlists;

			CREATE TABLE #Columnlists
				(
					 SchemaName		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL 
					,TableName		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,FullObjectName NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,Columnlist		NVARCHAR(2000)	COLLATE DATABASE_DEFAULT NOT NULL
				)

			DECLARE	 @schema					NVARCHAR(255)	= ''
					,@table						NVARCHAR(255)	= ''
					,@Columnlist				NVARCHAR(2000)	= ''
					,@TableHasDefaultContent	INT = 0;

			DECLARE Columncursor CURSOR FAST_FORWARD
			FOR
				SELECT 
						 SchemaName
						,ObjectName
				FROM   #CoreObjects_AfterUpdate  

				WHERE 
						ObjectCode ='U' 					
				ORDER BY SchemaName, Objectname

			OPEN Columncursor
			FETCH NEXT FROM Columncursor INTO @schema,@table
			WHILE @@FETCH_STATUS = 0
				BEGIN
					SET @Columnlist = '';

					SELECT @Columnlist = @Columnlist + '['+ COLUMN_NAME + ']' + ','
					FROM INFORMATION_SCHEMA.COLUMNS
					WHERE
							TABLE_SCHEMA	= @schema
						AND TABLE_NAME		= @table
						-- exclude computed columns
						AND COLUMN_Name NOT IN (

													SELECT 
														c.name COLLATE DATABASE_DEFAULT
													FROM sys.columns c
														LEFT JOIN sys.objects o
															ON c.object_id = o.object_id
												
													WHERE 
															c.is_computed = 1
														AND SCHEMA_NAME(o.schema_id)	COLLATE DATABASE_DEFAULT = @schema
														AND o.type_desc					COLLATE DATABASE_DEFAULT = 'USER_TABLE'
														AND o.name						COLLATE DATABASE_DEFAULT = @table
												);

					SET @Columnlist = LEFT(@Columnlist,LEN(@Columnlist)-1);

					INSERT INTO #Columnlists (SchemaName, TableName, FullObjectName,  Columnlist)
						VALUES (@schema, @table, @schema + '.' +@table, @columnlist);

					FETCH NEXT FROM Columncursor INTO @schema,@table;
				END

			CLOSE Columncursor
			DEALLOCATE Columncursor

			SET @columnlist = '';
			SET @table		= '';

			-- Columnlist ready ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

			-- Transfer data for all tables which exists in the new version, belong to OCT.core and are not set to manual tranfer 
   			DECLARE  @SQL NVARCHAR(MAX) = N''
					,@crlf VARCHAR(2) = CHAR(13) + CHAR(10);
			SET @SQL = N'';

			DECLARE Tablecursor CURSOR STATIC

			FOR
				-- Select all new created tables
				SELECT 
					 coa.FullObjectName
					,cl.Columnlist
				FROM   #CoreObjects_AfterUpdate coa

					LEFT JOIN #Columnlists cl
						ON	
							coa.FullObjectName = cl.FullObjectName

				WHERE 
						coa.ObjectCode = 'U' 
		
					AND coa.FullObjectName NOT IN
												( 
												SELECT FullTableName FROM #Blacklist
												)	
					-- only tables which where existing before											
					AND coa.FullObjectName IN (
												SELECT 
													FullObjectName
												FROM #CoreObjects_BeforeUpdate
												WHERE
													ObjectCode = 'U' 			
												)
				ORDER BY 
					 coa.FullObjectName
			OPEN Tablecursor
			PRINT CONCAT('Tabellenanzahl im Cursor f�r Autotransfer: ', @@cursor_rows);
			FETCH NEXT FROM Tablecursor INTO @table,@columnlist
			WHILE @@FETCH_STATUS = 0
			BEGIN
				SET @SQL = 'SELECT @Counter = COUNT (*) FROM ' + @table; 
				-- execution must happen over sp_excecute with return parameter
				EXEC sp_executesql @SQL, N'@Counter INT OUT', @TableHasDefaultContent OUT

				IF @TableHasDefaultContent > 0
					BEGIN
						PRINT 'Skiped table ' + @table + ' due to default content.'
					END
				IF @TableHasDefaultContent = 0
					BEGIN
						PRINT 'Transfer for: ' + @table + ' started. ************************************************************'
						SET @SQL = 'SET IDENTITY_INSERT ' +  @table + ' ON;'
	
						SET @SQL =  @SQL + 'INSERT INTO ' + @table + ' (' + @columnlist +')'
										+ ' SELECT ' +@Columnlist + ' FROM ' + @table + '_bak;'

						SET @SQL = @SQL + 'SET IDENTITY_INSERT ' +  @table + ' OFF;'
						PRINT @SQL
						EXEC(@SQL)
					END

				SET @TableHasDefaultContent = 0;
		
				FETCH NEXT FROM Tablecursor INTO @table,@columnlist
			END

			CLOSE Tablecursor
			DEALLOCATE Tablecursor

		END

GO
PRINT '+++ Ended Batch 8 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'



PRINT '+++ Starting Batch 9 (Blacklist Transfer) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'	

	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 9 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
		BEGIN
			DECLARE @Counter INT = 0;

		-- ############## Manual Transfer of blacklisted tables  #################################################################################################

		 -- SET IDENTITY_INSERT Tablename ON
		 -- Insert Data from table
		 --           Column List must be named
		 --           new Columns must be stuffed with default values
		 --           removed columns must be removed from Colum List
		 -- SET IDENTITY_INSERT Tablename OFF

		
		PRINT 'Starting Transfer for Blacklist'

		-- table API Log History is ignored

		-- planning.tAPI_Log is tranferd to system.tAPI_Log 
	
		SET IDENTITY_INSERT system.tAPI_Log ON;

		INSERT INTO system.tAPI_Log (LogKey,SQLSystemUser,UserName,TransactUsername,ProcedureName,ParameterString,EffectedRows,ReturnCode,TimestampCall,TimestampReturn,Comment,ProcessCode)
			SELECT
				 LogKey
				,SQLSystemUser
				,UserName
				,TransactUsername
				,ProcedureName
				,ParameterString
				,EffectedRows
				,ReturnCode
				,TimestampCall
				,TimestampReturn
				,Comment
				,ProcessCode
				
			FROM planning.tAPI_Log_bak;

		SET IDENTITY_INSERT system.tAPI_Log OFF;

		-- system.tPipelines has now an additional colum icon color -> also system.tPipelineSteps and system.tPipelineSchedules now must take Blacklistransfer, as Autotransfer would miss the system.tPipelines table
		-- disable CONSTRAINTS in related tables
		ALTER TABLE system.tPipelineSteps		NOCHECK CONSTRAINT ALL
		ALTER TABLE system.tPipelineSchedules	NOCHECK CONSTRAINT ALL

		SET IDENTITY_INSERT system.tPipelines ON;
		
		INSERT INTO system.tPipelines (PipelineKey,PipelineID,PipelineName,PipelineDescription,PipelineDescriptionIconColor,OrderOfDisplay,EmailNotificationCODE)
			SELECT
				 PipelineKey
				,PipelineID
				,PipelineName
				,PipelineDescription
				,''							AS PipelineDescriptionIconColor
				,OrderOfDisplay
				,EmailNotificationCODE

			FROM system.tPipelines_bak
				
		SET IDENTITY_INSERT system.tPipelines OFF;

		SET IDENTITY_INSERT system.tPipelineSteps ON;
		
		INSERT INTO system.tPipelineSteps (StepKey, PipelineKey, StepID, StepName, StepDescription, OrderOfExecution, StepDataSourceID, StepCompanyIDsJSON, StepDetailsJSON, TimeExecutionLimit, Active, ModuleID, StartNextStepOnError)
			SELECT
				  StepKey
				 ,PipelineKey
				 ,StepID
				 ,StepName
				 ,StepDescription
				 ,OrderOfExecution
				 ,StepDataSourceID
				 ,StepCompanyIDsJSON
				 ,StepDetailsJSON
				 ,TimeExecutionLimit
				 ,Active
				 ,ModuleID
				 ,StartNextStepOnError

			FROM system.tPipelineSteps_bak
				
		SET IDENTITY_INSERT system.tPipelineSteps OFF;

		SET IDENTITY_INSERT system.tPipelineSchedules ON;
		
		INSERT INTO system.tPipelineSchedules (ScheduleKey, PipelineKey, ScheduleID, ScheduleName, ScheduleDescription, ScheduleDetailsJSON, Active)
			SELECT
				 ScheduleKey
				,PipelineKey
				,ScheduleID
				,ScheduleName
				,ScheduleDescription
				,ScheduleDetailsJSON
				,Active
				 
			FROM system.tPipelineSchedules_bak
				
		SET IDENTITY_INSERT system.tPipelineSchedules OFF;


		-- disable CONSTRAINTS in related tables
		ALTER TABLE system.tPipelineSteps		WITH CHECK CHECK CONSTRAINT ALL
		ALTER TABLE system.tPipelineSchedules	WITH CHECK CHECK CONSTRAINT ALL

		-- planning.tFiles has three new columns
		SET IDENTITY_INSERT planning.tFiles ON;

		INSERT INTO planning.tFiles (FileKey,FactoryKey,ProductLineKey,ProductKey,FactoryID,ProductLineID,ProductID,FileID,FileName,FileExtension,FileBody,UploadedBy,UploadedAt,Filepath)
			SELECT																																				
				 FileKey
				,FactoryKey
				,ProductLineKey
				,ProductKey
				,FactoryID
				,ProductLineID
				,ProductID
				,FileID
				,FileName
				,FileExtension
				,FileBody
				,'uploaded in old version'	AS UploadedBy
				,'2023-05-08 12:55:37.120'	AS UploadedAt
				,'/'						AS Filepath 																																				
				
			FROM planning.tFiles_bak;

		SET IDENTITY_INSERT planning.tFiles OFF;

		-- two new columns in to system.trUser and UPPER CODE
		SET IDENTITY_INSERT system.trUser ON;

		INSERT INTO system.trUser (UserKey,UserName,PersonSurname,PersonFirstName,Email,LDAPIP,Status,DataFlowRightsCODE,VisualizationRightsCODE,IsAdministratorFlag)
			SELECT																																				
				 trU.UserKey
				,UserName
				,PersonSurname
				,PersonFirstName
				,Email
				,LDAPIP
				,Status
				,UPPER(DataFlowRightsCODE)	AS DataFlowRightsCODE
				,CASE
					WHEN Admins.UserKey IS NOT NULL THEN N'DESIGNER'
					ELSE N'NONE'
				 END					AS VisualizationRightsCODE
				,CASE
					WHEN Admins.UserKey IS NOT NULL THEN 1
					ELSE 0
				 END						AS IsAdministratorFlag																																
				
			FROM system.trUser_bak trU
				-- determine Clusteradadmins
				LEFT JOIN 
						(
							SELECT 
								UserKey
							FROM system.trRights_bak
							WHERE
									FactoryID = '' 
								AND [Right] = 'Write'
								
						)	AS Admins
						ON trU.UserKey = Admins.UserKey;

		SET IDENTITY_INSERT system.trUser OFF;

		PRINT 'Finished Transfer for Blacklist'

		-- remove column SourceSystem from system.tDataSources
		SET IDENTITY_INSERT system.tDataSources ON;

		INSERT INTO system.tDataSources (DataSourceKey,DataSourceID,DataSourceName,SourceType,ConnectionValuesJSON)
			SELECT																																				
				 DataSourceKey
				,DataSourceID
				,DataSourceName
				,SourceType
				,ConnectionValuesJSON																																			
				
			FROM system.tDataSources_bak;

		SET IDENTITY_INSERT system.tDataSources OFF;

		-- Change Datatype from Bigint to Datetime in Statements

		SET IDENTITY_INSERT planning.tfStatements ON;

		INSERT INTO planning.tfStatements (StatementKey, ProductKey, ProductLineKey, FactoryKey, ProductID, ProductLineID, FactoryID, ActionType, [Statement], UserName, PCName, ProcessorCode, IPAddresses, [Timestamp], IsDeleted, IsResolved)
			SELECT
				StatementKey
			   ,ProductKey
			   ,ProductLineKey
			   ,FactoryKey
			   ,ProductID
			   ,ProductLineID
			   ,FactoryID
			   ,ActionType
			   ,[Statement]
			   ,UserName
			   ,PCName
			   ,ProcessorCode
			   ,IPAddresses
			   ,DATETIMEFROMPARTS(LEFT([Timestamp],4),RIGHT(LEFT([Timestamp],6),2),RIGHT(LEFT([Timestamp],8),2),RIGHT(LEFT([Timestamp],10),2),RIGHT(LEFT([Timestamp],12),2),0,0) AS [Timestamp]
			   ,IsDeleted
			   ,IsResolved
			FROM planning.tfStatements_bak;

		SET IDENTITY_INSERT planning.tfStatements OFF;


		-- add Mapping Columns to global.Companies
		SET IDENTITY_INSERT global.tCompanies ON;

		INSERT INTO global.tCompanies (RowKey,DataSourceKey,CompanyID,CompanyName,[Description],FinancialYearBegin,ChartsOfAccountsID,AccountLength,Currency,CustomValuesJSON,CustomValuesJSONUser,OwnerCODE,CompanyIDResult,MappingPriority)
			SELECT																																				
				  tC.RowKey
				 --,tC.CompanyKey			-- not due to calculated
				 ,tC.DataSourceKey  
				 ,tC.CompanyID
				 ,tC.CompanyName
				 ,tC.[Description]
				 ,tC.FinancialYearBegin
				 ,tC.ChartsOfAccountsID
				 ,tC.AccountLength
				 ,tC.Currency
				 ,tC.CustomValuesJSON
				 ,tC.CustomValuesJSONUser
				 ,tC.OwnerCODE		
				 ,COALESCE(tCM.CompanyIDGlobal,N'')		AS CompanyIDResult         
				 ,1										AS MappingPriority        
				
			FROM global.tCompanies_bak tC
				LEFT JOIN global.tCompanyMapping_bak tCM
					ON tC.CompanyKey = tCM.CompanyKeyIntegration;

		SET IDENTITY_INSERT global.tCompanies OFF;

		-- add Reporting Columns to global.tPeriods
		SET IDENTITY_INSERT global.tPeriods ON;

		INSERT INTO global.tPeriods (RowKey, FinancialYear, [Period], FinancialYearBegin, FinancialYearName, CalendarYear, CalendarMonth, CalendarQuarterCODE, CalendarHalfYearCODE, YYYYMM, [YYYY-MM], YYYYQ, [YYYY QQ])
			SELECT																																				
				       RowKey
					--  ,PeriodKey -- berechnete Spalte
					  ,FinancialYear
					  ,[Period]
					  ,FinancialYearBegin
					  ,FinancialYearName
					  ,CalendarYear
					  ,CalendarMonth
					  ,CalendarQuarterCODE
					  ,CalendarHalfYearCODE
					  ,CalendarYear * 100 + CalendarMonth																AS YYYYMM
					  ,CONCAT(CalendarYear,'-',Right(CONCAT(0,CalendarMonth),2))										AS [YYYY-MM]
					  ,CONCAT(CalendarYear,DATEPART(QUARTER,DATEFROMPARTS(CalendarYear,CalendarMonth,1)))				AS YYYYQ
					  ,CONCAT(CalendarYear,' Q',DATEPART(QUARTER,DATEFROMPARTS(CalendarYear,CalendarMonth,1)))			AS [YYYY QQ]
				
			FROM global.tPeriods_bak tC
			

		SET IDENTITY_INSERT global.tPeriods OFF;

	END

GO
PRINT '+++ Ended Batch 9 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'

		
	
	PRINT '+++ Starting Batch 11 (Kill bak) ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	-- drop all bak tables

	DECLARE @DoUpdateFlag INT;
	SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

	IF @DoUpdateFlag = 0
		BEGIN
			PRINT 'Batch 11 not started due to wrong version.'
		END

	IF @DoUpdateFlag = 1
	BEGIN

		DECLARE  @SQL NVARCHAR(MAX) = N''
				,@crlf VARCHAR(2) = CHAR(13) + CHAR(10);

		SET @SQL = N''
		SELECT @SQL = @SQL + N'BEGIN TRY DROP TABLE ' + FullObjectName + N'_bak END TRY '
						   + N'BEGIN CATCH PRINT ''Warning, could not delete the table: + ' + FullObjectName + N'_bak' + ''' END CATCH;' + @crlf
		FROM   #CoreObjects_BeforeUpdate o 
		WHERE ObjectCode = 'U'
		PRINT @SQL;
		EXEC (@SQL);
	END

	GO
	PRINT '+++ Ended Batch 11 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'



DROP FUNCTION IF EXISTS calc.sfOCTDate2SQLDate;
GO

/*
	Function for converting a OCT Date from PDT (a number like in Excel) to an SQL Date
	The OCT Date is expected to be saved with Scale 1
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function 
		SELECT calc.sfOCTDate2SQLDate (45000)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfOCTDate2SQLDate',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfOCTDate2SQLDate','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfOCTDate2SQLDate
    (
         @OCTDate	BIGINT
    )

RETURNS DATE
AS 
BEGIN
	DECLARE @SQLDate DATE;
	IF @OCTDate > 100000 OR @OCTDate = 0
		BEGIN
			SET @SQLDate = NULL;
		END
	ELSE
		BEGIN
			SET @SQLDate = TRY_CONVERT(DATE,DATEADD(day,(@OCTDate)-2,N'19000101'));
		END

	RETURN @SQLDate;

END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfOCTDate2SQLDate'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to call demo values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@OCTDate';
    SET @value      = N'Date in OCT Format from ProductDataTable (Number) which must be passed in number with scale 1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP FUNCTION IF EXISTS calc.sfReplaceNonXMLCharacters;
GO

/*
	Function for replacing all non XML conform characters in a string
	according to https://stackoverflow.com/questions/25147627/xml-path-expression-to-include-special-characters

	It does not remove 0x000A (LineFeed) and 0x000D (CarrigeReturn) which are conform - so any line break stays as it is.
	Table for codes http://redgrittybrick.org/ascii.html
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Function 
		DECLARE @String NVARCHAR(255);
		SET @String = CONCAT ('Test',char(0x000F));
		SELECT CONCAT('Vorher ',LEN(@String),' Zeichen');
		SELECT @String = calc.sfReplaceNonXMLCharacters (@String);
		SELECT CONCAT('Nachher ',LEN(@String),' Zeichen');
		SELECT @String;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfReplaceNonXMLCharacters',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfReplaceNonXMLCharacters','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfReplaceNonXMLCharacters
  (
         @String			NVARCHAR(MAX)
  )
    

RETURNS NVARCHAR(MAX)
AS 
BEGIN
	--DECLARE @Counter INT;
	--SET @Counter = LEN(@String);
	SET @String =	REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( REPLACE( 
					REPLACE( REPLACE( REPLACE( REPLACE( 
						@String
					,char(0x0000),'') ,char(0x0001),'') ,char(0x0002),'') ,char(0x0003),'') ,char(0x0004),'') 
					,char(0x0005),'') ,char(0x0006),'') ,char(0x0007),'') ,char(0x0008),'') ,char(0x000B),'') 
					,char(0x000C),'') ,char(0x000E),'') ,char(0x000F),'') ,char(0x0010),'') ,char(0x0011),'') 
					,char(0x0012),'') ,char(0x0013),'') ,char(0x0014),'') ,char(0x0015),'') ,char(0x0016),'') 
					,char(0x0017),'') ,char(0x0018),'') ,char(0x0019),'') ,char(0x001A),'') ,char(0x001B),'') 
					,char(0x001C),'') ,char(0x001D),'') ,char(0x001E),'') ,char(0x001F),'');
	--SET @Counter = (@Counter - LEN(@String))*-1;
	--Functions can do a PRINT only with very dirty tricks like https://stackoverflow.com/questions/586642/tsql-how-do-you-output-print-in-a-user-defined-function, therefore only contained for debug resons
	--PRINT CONCAT ('Replaced ',@Counter, ' non XML conform characters in the string.');
	RETURN @String;
END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfReplaceNonXMLCharacters'		-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'							-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'								-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;									-- 0 = Demo object - out of shipment process
																				-- STANDARD OBJECTS
																				-- 1 = shiped from saxess standard without modification
																				-- 2 = shiped from saxess standard modified FOR customer from saxess
																				-- 3 = shiped from saxess standard modified FOR customer from partner
																				-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																				-- CUSTOM OBJECTS
																				-- 10 = shiped from saxess as customer specific object
																				-- 11 = shiped from partner	as customer specific object
																				-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to remove all non xml conform characters in a string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'Any string, where non-xml conform characters must be removed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS calc.sfSQLDate2TimeID;
GO

/*
	Function for converting an SQL Date to an OCT TimeID
	Invalid Dates - can't be handled when inputed as String, Function crashed for invalid dates
	Works for Dates like '1000-01-01'
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function 
		SELECT calc.sfSQLDate2TimeID ('2022-10-31') -- ok
		SELECT calc.sfSQLDate2TimeID ('2022-10-42') -- crash
		SELECT calc.sfSQLDate2TimeID ('1000-10-10') -- ok

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfSQLDate2TimeID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfSQLDate2TimeID','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfSQLDate2TimeID
    (
         @Date DATE
    )

RETURNS INT
AS 
BEGIN
		DECLARE @TimeID INT;
		
		SET @TimeID = CAST(FORMAT(@Date,'yyyyMMdd') AS INT) 

		RETURN @TimeID;
END

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfSQLDate2TimeID'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function convert an OCT TimeID to an SQL DateFormat, returns NULL for invalid Dates.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Date';
    SET @value      = N'Date or string written as ''YYYY-MM-DD''';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP FUNCTION IF EXISTS calc.sfTimeID2SQLDate;
GO

/*
	Function for converting an OCT TimeID to an SQL Date
	Returns NULL for invalid Dates
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function 
		SELECT calc.sfTimeID2SQLDate (20221031)
		SELECT calc.sfTimeID2SQLDate (20221042)
		SELECT calc.sfTimeID2SQLDate (10101010)

		SELECT CONVERT(Date, '20211305')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfTimeID2SQLDate',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfTimeID2SQLDate','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfTimeID2SQLDate
    (
         @TimeID	INT
    )

RETURNS DATE
AS 
BEGIN
		DECLARE @Date DATE;
		IF ISDATE(CAST(@TimeID AS NVARCHAR(50))) = 1 -- must check this, otherwise we can't return NULL instead of an error
			BEGIN
				--SET @Date = DATEFROMPARTS(@TimeID/10000,@TimeID/100%100,@TimeID%100)  -- both are working, the second one is faster (??)
				SET @Date = CONVERT(DATE,CAST(@TimeID AS NVARCHAR(50)))
			END
		ELSE
			BEGIN
				SET @Date = NULL
			END
		RETURN @Date
END

GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfTimeID2SQLDate'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function convert an OCT TimeID to an SQL DateFormat.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID, written YYYYMMDD, returns NULL for invalid Dates.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP FUNCTION IF EXISTS calc.sfUnifyLinebreaks;
GO

/*
	Function for adjusting Linebreaks of type "LF" to "CRLF"
	Keeps existing "CRLF"
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9
 
	Testcall Function 
		SELECT calc.sfUnifyLinebreaks ('Hallo',)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfUnifyLinebreaks',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'calc', 'FUNCTION', 'sfUnifyLinebreaks','PARAMETER',NULL)
*/
 
CREATE FUNCTION calc.sfUnifyLinebreaks
    (
         @String			NVARCHAR(MAX)
    )

RETURNS NVARCHAR(MAX)
AS 
BEGIN
	SET @String = REPLACE(
						REPLACE(
							REPLACE(
									@String,CHAR(13)+CHAR(10),CHAR(27)+CHAR(27) -- existing linebreaks are cached as ESC charactr
									),CHAR(10),CHAR(13)+CHAR(10)				-- effective replace
								),CHAR(27)+CHAR(27),CHAR(13)+CHAR(10)			-- back caching
							);
	RETURN @String;
END

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'calc'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sfUnifyLinebreaks'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'DEMO'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 0;					-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function for adjusting Linebreaks of type "LF" to "CRLF"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'Any string which may contain linebreaks';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP FUNCTION IF EXISTS dbo.Day_of_Week_DE;
GO
/*
Function for day of the week
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT dbo.Day_of_Week_DE ('2021-01-15T08:08:08')

	Testcall Documentation--
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'Day_of_Week_DE',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'Day_of_Week_DE','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.Day_of_Week_DE (@d DATETIME)
RETURNS INT
  BEGIN
      DECLARE @t INT

      SET @t = Datepart(weekday, @d) + @@DATEFIRST - 1

      IF @t > 7
        SET @t = @t - 7

      RETURN @t
  END

GO


ALTER AUTHORIZATION ON OBJECT :: dbo.Day_of_Week_DE TO pf_PlanningFactoryService;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'Day_of_Week_DE'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
 
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;


-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function for day of the week.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@d';
    SET @value      = N'Datetime parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO


DROP FUNCTION IF EXISTS dbo.sx_pf_Determine_TransactionUsername;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Funktion returns the Name of an existing active User in OCT Usermanamgent or the string 403.

	Determine the name of user, for whom a transaction is done.
	The resulting name may be
		- an existing and active user of the OCT Usermanagement (including the special one with name "public")
		- an login who exists not in Usermanagment, but in the database as user with the role FactorySerivce or db_owner (priviledged user)
		- the string 'public' if no business user were found an a user public is declared
		- the string '403' if no authorized user was found

	Rules:
		- User with the Role FactoryService or db_owner are allowed to act in own name or in foreign name
		- to signalize that they act in own name, they pass in the username 'SQL'
		- User with other roles can act only as themself (the user which is connected)
		- even a user with role sysadmin can't pass in a foreign username

	Keep in Mind:
		- if a user with role db_owner is also also owner of the database he will be not appear with his login, instead with username 'dbo'
		- a user may be db_owner and has the role FactoryService
		- a user may be sysadmin, than he is not an db_owner
		- a user may be internal OCTUser and also db_owner
		- in OCT Usermanagement 
			- never a user with Name '403' must be created - this user would always been blocked
			- a User with the Name of the FactoryService must not be created (but he may be there, due to later db role assignment)
			- a User with the role db_owner may exist in the OCT Usermangement

	TODO for later editions where XLS Client must no longer be supported
	-- Return User with an role to determine if FactoryService is acting for itself or not

	Don't use the function IS_ROLEMEMBER on SQL Server, its buggy

	Testcall 
		SELECT dbo.sx_pf_Determine_TransactionUsername ('SQL')
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_Determine_TransactionUsername',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_Determine_TransactionUsername','PARAMETER',NULL)

*/

CREATE FUNCTION dbo.sx_pf_Determine_TransactionUsername 
	(
		@Username AS NVARCHAR(255)=''
	)

RETURNS NVARCHAR (255) 
AS
	BEGIN 
			DECLARE  
				 @TransactionUsername	NVARCHAR (255) = '403'
				,@IsPrivilegedUser		INT = 0
				,@UserBlocked			INT = 0
				,@UserIsActiveOCTUser	INT = 0;
		
		-- trim Username
			SET @Username = LTRIM(RTRIM(@Username));


		-- 1. Determine if a privileged User is acting - the dbo is never member of any role, thats why its tested separat
			IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;
			
		-- 2. If privileged user is acting, every Username is accepted, she string 'SQL' signalise they are acting as themself 
			IF  @IsPrivilegedUser = 1
				IF @Username = 'SQL'
					BEGIN
						SET @TransactionUsername = ORIGINAL_LOGIN()
					END
				ELSE
					BEGIN
						SET @TransactionUsername = @Username;
					END
			ELSE
				-- else only the Original_Login() is accepted - this happens to enable Working with the Excel Client or direct in SSMS (if user is GRANTEd to Execute SPs)
				BEGIN
					SET @TransactionUsername = ORIGINAL_LOGIN()
				END

		-- 3. Check if the user is an active OCT User
			IF (SELECT Count(Userkey) FROM system.trUser WHERE Username = @TransactionUsername AND Status = 'Active') = 1 
				BEGIN 
					SET @UserIsActiveOCTUser = 1;
				END

			-- privileged users are always exepted when they act for thereself  (even if they exists in usermanagement as inactive or don't exists there)		
			IF @IsPrivilegedUser = 1 AND @Username = 'SQL'
				BEGIN
					SET @TransactionUsername = @TransactionUsername -- do nothing, accept the user
				END

			-- When privileged user is acting for a User, only active users are accepted
			IF @IsPrivilegedUser = 1 AND @Username <> 'SQL' AND @UserIsActiveOCTUser = 0
				BEGIN
					SET @TransactionUsername = '403'
					SET	@UserBlocked = 1
				END

			-- When a non privileged User is connected directly, he is blocked if he is not an active user 
			IF @IsPrivilegedUser = 0 AND @UserIsActiveOCTUser = 0
				BEGIN
					SET @TransactionUsername = '403'
					SET	@UserBlocked = 1
				END
				
		-- 4. If the user was blocked, check if a public user exists, which can be used for the transaction
			IF @UserBlocked = 1
				BEGIN
					IF EXISTS
						(
						SELECT Userkey FROM system.trUser WHERE Username = 'public' AND Status = 'Active'
						)
						SET @TransactionUsername = 'public'
				END

		RETURN @TransactionUsername
	END
GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_Determine_TransactionUsername TO pf_PlanningFactoryService;

GRANT EXECUTE ON dbo.sx_pf_Determine_TransactionUsername TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_Determine_TransactionUsername'			-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
 
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;


-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function and determine the business username for the transaction - 
		only for active users and the priviledged database roles FactoryService and db_owner an username is returnd.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'The name of the user, in which name the transaction shall happen. Pass in "SQL" if the Service shall act for himself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 


GO

DROP FUNCTION IF EXISTS dbo.sx_pf_PivotStringIntoTable;
GO

/*
	Function to split a String into a TableVariable
	The Delimiter String will be replace by a CHAR(28) and the string splitted on this

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT * FROM dbo.sx_pf_PivotStringIntoTable ('xxxNxxx', 'N')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_PivotStringIntoTable',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_PivotStringIntoTable','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_PivotStringIntoTable
	(
		 @String NVARCHAR(MAX)
		,@Delimiter NVARCHAR(5)
	)

RETURNS @tblSplitValues TABLE (txtValues NVARCHAR(MAX))

AS
BEGIN

	DECLARE @spr		NCHAR(1)		= CHAR(28)
	DECLARE @auxString	NVARCHAR (MAX)	= ''
	SET @auxString = REPLACE(@String, @Delimiter,@spr);

	WITH Split(stpos,endpos)
	AS(
		SELECT 0 AS stpos, CHARINDEX(@spr,@auxString) AS endpos
		UNION ALL
		SELECT CAST(endpos AS INT)+1, CHARINDEX(@spr,@auxString,endpos+1)
		FROM Split
		WHERE endpos > 0
	)

	INSERT @tblSplitValues
		SELECT SUBSTRING(@auxString,stpos,COALESCE(NULLIF(endpos,0),LEN(@auxString)+1)-stpos)
		FROM Split OPTION (MAXRECURSION 32766) -- it works ONLY up to 32.766 rows

	RETURN

END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_PivotStringIntoTable TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_PivotStringIntoTable'		-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'						-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1									-- 0 = Demo object - out of shipment process
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to split a String into a TableVariable.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@String';
    SET @value      = N'The pivot string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Delimiter';
    SET @value      = N'The delimiter to split the string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.sx_pf_pMaskSQL;
GO

/*
	Function to mask strings, which will be part of an SQL Statement @StringValue  - it just doubles all inverted commas.
	returns a masked string
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.8
 
	Testcall Function
		DECLARE @String NVARCHAR(255);
		SET @String = N'ToDo''s';
		PRINT @String;
		SELECT dbo.sx_pf_pMaskSQL(@String);

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pMaskSQL',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pMaskSQL','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pMaskSQL (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(@StringValue,N'''',N'''''')
END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pMaskSQL TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pMaskSQL'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to mask strings, which will be part of an SQL Statement - it just doubles all inverted commas.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@StringValue';
    SET @value      = N'String, which will be returned with doubled inverted commas.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectBoolean;
GO
/*
	Function returns a '0' when the passed parameter is not either '0' or '1'
	@BooleanValue ='<#NV>'is ignored and not changed
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT dbo.sx_pf_pProtectBoolean (2)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectBoolean',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectBoolean','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectBoolean (@BooleanValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @BooleanValue <>'<#NV>' 
	
	BEGIN
		IF @BooleanValue <>'0' AND @BooleanValue <>'1'
		SET @BooleanValue= '0'
	END

	RETURN @BooleanValue

END

GO
-- Scalar value functions require EXECUTE permissions
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectBoolean TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectBoolean'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function returns a 0 when the passed parameter is not either 0 or 1. @BooleanValue =<#NV>is ignored and not changed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@BooleanValue';
    SET @value      = N'Boolean parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectID;
GO
/*
	Function to replace all special signs from ID
	An ID may contain Letters, Numbers,'_' and '-', but nothing else
	Function returns a clean string
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT dbo.sx_pf_pProtectID ('T-1/3')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectID','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectID (@ElementID AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	SET @ElementID = RTRIM(LTRIM(@ElementID))
	RETURN	REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(
			REPLACE(@ElementID,'|',''),
							   ';',''),'''',''),'/',''),'\',''),'"',''),',',''),
							   '>',''),'<',''),'[',''),']',''),'(',''),')',''),'{',''),'}',''),'!',''),'$',''),'&',''),'^',''),'@',''),
							   '%',''),':',''),'`',''),'´',''),'?',''),'§',''),'#',''),'~',''),'+',''),'=',''),'.',''),'€',''),' ','')
END

GO
-- Scalar value functions require EXECUTE permissions
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectID TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectID'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to replace all special signs from ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@ElementID';
    SET @value      = N'Element ID Parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectInt;
GO

/*
	Function cast all expression to int or return 0 for non-int values, except the <#NV> String this is passed anytime
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcalls Function
		SELECT dbo.sx_pf_pProtectInt('-1')
		SELECT dbo.sx_pf_pProtectInt('1')
		SELECT dbo.sx_pf_pProtectInt('Hase')
		SELECT dbo.sx_pf_pProtectInt('<#NV>')
		SELECT dbo.sx_pf_pProtectInt('')
		SELECT dbo.sx_pf_pProtectInt('0')
		SELECT dbo.sx_pf_pProtectInt('9999999999999999999999999999999999999999999999999999999999')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectInt',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectInt','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectInt (@IntValue AS NVARCHAR(255))
RETURNS NVARCHAR (255)

AS
BEGIN
	
	IF @IntValue <>'<#NV>' 
		BEGIN
			SET @IntValue = COALESCE(CAST(TRY_CAST(@IntValue AS INT) AS NVARCHAR(255)),0)
		END

RETURN @IntValue

END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectInt TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectInt'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function cast all expression to int or return 0 for non-int values, except the <#NV> String this is passed anytime.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@IntValue';
    SET @value      = N'Parameter int value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO


DROP FUNCTION IF EXISTS dbo.sx_pf_pProtectString;
GO
/*
	Function to eliminate all not permitted special characters (Hochkommata, '|','),(', ';' ) and line breaks at the end from @StringValue
	Function returns a string without the not permitted special characters
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Function
		SELECT dbo.sx_pf_pProtectString ('X),(Y;Z')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectString',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pProtectString','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pProtectString (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN
	-- remove line break only at the end of the string
	IF LEN(@StringValue) > 1
	BEGIN
		SET @StringValue = SUBSTRING(@StringValue, 0, len(@StringValue) - 2) + REPLACE(REPLACE(SUBSTRING(@StringValue, LEN(@StringValue) - 2, 3), CHAR(10), ''), CHAR(13), '')
	END

	-- remove special characters
	RETURN	REPLACE(REPLACE(REPLACE(@StringValue,'|',''), '),(',''), '--','')
END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pProtectString TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pProtectString'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to eliminate all not permitted special characters from @StringValue.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@StringValue';
    SET @value      = N'Parameter string value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.sx_pf_pRemoveLinebreaks;
GO
/*
	Function to remove Linebreaks from a string
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		DECLARE @String NVARCHAR(255)
		SET @String = 'ToDo'+CHAR(13)+CHAR(10)+' for you'
		Print @String
		SELECT @String = dbo.sx_pf_pRemoveLinebreaks(@String)
		Print @String

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pRemoveLinebreaks',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_pRemoveLinebreaks','PARAMETER',NULL)
*/

CREATE FUNCTION dbo.sx_pf_pRemoveLinebreaks (@StringValue AS NVARCHAR(MAX))
RETURNS NVARCHAR (MAX)

AS
BEGIN

	RETURN	REPLACE(REPLACE(@StringValue,CHAR(10),''),CHAR(13),'')

END

GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pRemoveLinebreaks TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_pRemoveLinebreaks'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'					-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'						-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1								-- 0 = Demo object - out of shipment process
																		-- STANDARD OBJECTS
																		-- 1 = shiped from saxess standard without modification
																		-- 2 = shiped from saxess standard modified FOR customer from saxess
																		-- 3 = shiped from saxess standard modified FOR customer from partner
																		-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		-- CUSTOM OBJECTS
																		-- 10 = shiped from saxess as customer specific object
																		-- 11 = shiped from partner	as customer specific object
																		-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function to remove Linebreaks from a string.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@StringValue';
    SET @value      = N'Parameter string value.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS dbo.Week_of_Year_DE;
GO
/*
	Function for week of the year
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Testcall Function
		SELECT dbo.Week_of_Year_DE ('2021-01-15T08:08:08')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'Week_of_Year_DE',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'Week_of_Year_DE','PARAMETER',NULL)
*/
 
CREATE FUNCTION dbo.Week_of_Year_DE (@d DATETIME)
returns INT
  BEGIN
      DECLARE @k INT
      DECLARE @x DATETIME

      SET @x = Str(Year(@d)) + '-01-01'
      SET @k = Datepart(week, @d)

      IF dbo.Day_of_Week_DE(@x) > 4
        BEGIN
            SET @k = @k - 1

            IF @k = 0
              SET @k = 53
        END

      RETURN @k
  END 
GO

ALTER AUTHORIZATION ON OBJECT :: dbo.Week_of_Year_DE TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'Week_of_Year_DE'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function for week of the year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@d';
    SET @value      = N'Datetime parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP FUNCTION IF EXISTS system.fGetColumns;
GO
/*
	Table Function to GET for the requested table or view
		- all ColumnNames
		- per column the property if the column is calculated
		- per column if the column is part of the key
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function
		SELECT * FROM system.fGetColumns ('planning','tAPI_Log')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'FUNCTION', 'fGetColumns',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'FUNCTION', 'fGetColumns','PARAMETER',NULL)
*/

CREATE FUNCTION system.fGetColumns (
	@Schema NVARCHAR(50),
	@Table NVARCHAR(50)
)
RETURNS TABLE 
AS
RETURN 
(
	-- for tables
	SELECT
		c.Name				AS ColName,
		c.is_computed		AS IsCalculatedKey,
		CASE 
			WHEN EXISTS(
						SELECT 1
						FROM sys.computed_columns cc
						WHERE 
								cc.object_id = t.object_id
							AND cc.definition COLLATE DATABASE_DEFAULT LIKE N'%![' + c.Name + N'!]%' ESCAPE N'!'
					) THEN 1 
			ELSE 0 
		END					AS IsPartOfKey
	FROM sys.columns c
		JOIN sys.tables t 
			ON t.object_id = c.object_id
		JOIN sys.schemas s 
			ON s.schema_id = t.schema_id
		LEFT JOIN sys.computed_columns cc 
			ON		cc.column_id = c.column_id 
				AND cc.object_id = t.object_id
	WHERE 
			s.name COLLATE DATABASE_DEFAULT = @Schema
		AND t.name COLLATE DATABASE_DEFAULT = @Table
	
	UNION ALL

	-- for Views
	SELECT
		c.name			AS ColName,
		c.is_computed	AS IsCalculatedKey,
		CASE
			WHEN EXISTS(
						SELECT 1
						FROM sys.computed_columns cc
						WHERE cc.object_id = v.object_id
						AND cc.definition COLLATE DATABASE_DEFAULT LIKE N'%![' + c.Name + N'!]%' ESCAPE N'!'
						) THEN 1 
			ELSE 0 
		END				AS IsPartOfKey
	FROM sys.columns c
	JOIN sys.views v 
		ON v.object_id = c.object_id	
	JOIN sys.schemas s 
		ON s.schema_id = v.schema_id
	LEFT JOIN sys.computed_columns cc
		ON 
				cc.column_id = c.column_id 
			AND cc.object_id = v.object_id
	WHERE 
			v.name COLLATE DATABASE_DEFAULT = @Table
		AND s.name COLLATE DATABASE_DEFAULT = @Schema
)
GO

-- Explicit GRANT for Factory Service
GRANT SELECT ON system.fGetColumns TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'fGetColumns'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
 
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;


-- SET documententation             *************************************************************************
 
-- SET function documentation
	SET @value = N'Function to GET for a requested table or view all ColumnNames and the properties if the column is calculated and if this Column is part of the key.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Schema';
    SET @value      = N'Parameter schema.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Table';
    SET @value      = N'Parameter table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

/*
Saxess Software GmbH
06/2023 for OCT 5.9
 
Testcall
	SELECT * FROM agent.vJSON_global_tCompanies

Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'VIEW', 'vJSON_global_tCompanies',NULL,default)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'VIEW', 'vJSON_global_tCompanies','COLUMN',default)

*/

DROP VIEW IF EXISTS agent.vJSON_global_tCompanies;
GO

PRINT('Creating view agent.vJSON_global_tCompanies')
GO

CREATE VIEW agent.vJSON_global_tCompanies
AS
    SELECT c.RowKey AS RowKey
    FROM global.tCompanies AS c;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'agent'					-- enter schema name of the view
    ,@level1name        NVARCHAR(255)   = N'vJSON_global_tCompanies'      -- enter view name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.modules'				-- enter owner name of the view from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'FIN'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                         -- 0 = Demo object - out of shipment process
                                                                    -- STANDARD OBJECTS
                                                                    -- 1 = shiped from saxess standard without modification
                                                                    -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                                    -- 3 = shiped from saxess standard modified FOR customer from partner
                                                                    -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															        -- CUSTOM OBJECTS
                                                                    -- 10 = shiped from saxess as customer specific object
															        -- 11 = shiped from partner	as customer specific object
															        -- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'VIEW'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************

-- SET View documentation
	SET @value = N'This view resolves the CustomValuesJSON column in table global.tCompanies to actual columns.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- SET parameter documentation
	SET @level2name = N'RowKey';
	SET @value = N'RowKey';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
GO

DROP VIEW IF  EXISTS system.vLogEntrys;
GO

/*
	View for analysing the LogEntrys of PlanningFactory API 
		- shows the execution time per LogEntry in Milliseconds and Seconds
	Only for use by administrators, as the log shows all activities of all users

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall
		SELECT * FROM system.vLogEntrys

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'VIEW', 'vLogEntrys',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'VIEW', 'vLogEntrys','COLUMN',default);
*/

CREATE VIEW system.vLogEntrys 

AS 

	SELECT 
		 @@Servername									AS Servername
		,DB_NAME()										AS ClusterName
		,SQLSystemUser
		,Username										AS ServiceUsername
		,TransactUsername
		,ProcedureName
		,ParameterString
		,EffectedRows
		,ReturnCode
		,TimestampCall
		,TimestampReturn
		,CASE
			WHEN DATEDIFF(SECOND,TimestampCall,TimestampReturn) < 10 THEN
				DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn)
			ELSE
				10000 -- to avoid to big numbers
		 END											AS DurationInMilliseconds
		,DATEDIFF(SECOND,TimestampCall,TimestampReturn) AS DurationInSeconds
		,Comment
		,ProcessCode
	FROM 
		system.tAPI_Log;

GO

GRANT SELECT ON OBJECT::system.vLogEntrys TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the view
    ,@level1name        NVARCHAR(255)   = N'vLogEntrys'   -- enter view name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the view from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'VIEW'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET view documentation
    SET @value = N'View for analysing the LogEntrys of PlanningFactory API';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
        SET @level2name = N'Servername';
    SET @value      = N'Servername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ClusterName';
    SET @value      = N'ClusterName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SQLSystemUser';
    SET @value      = N'SQLSystemUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ServiceUsername';
    SET @value      = N'ServiceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TransactUsername';
    SET @value      = N'TransactUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcedureName';
    SET @value      = N'ProcedureName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ParameterString';
    SET @value      = N'ParameterString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectedRows';
    SET @value      = N'EffectedRows';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReturnCode';
    SET @value      = N'ReturnCode';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampCall';
    SET @value      = N'TimestampCall';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampReturn';
    SET @value      = N'TimestampReturn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DurationInMilliseconds';
    SET @value      = N'DurationInMilliseconds, display limited to 10.000';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
DROP PROCEDURE IF EXISTS agent.pCreateJSONView;
GO

/*
	Procedure to create a helper table for the agent to resolve a JSON Column to single columns
	- the JSON Column must be named "CustomValuesJSON"
	- the resulting table will be named agent.vJSON_[schema]_[table]
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = agent.pCreateJSONView
				 @Username	= 'SQL'
				,@Schema	= 'global'
				,@Table		= 'tCompanies'
				,@UseRowKey = 1
		PRINT @RC

		SELECT * FROM agent.vJSON_global_tCompanies

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pCreateJSONView',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pCreateJSONView','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pCreateJSONView
    (
        @Username     NVARCHAR(255)      
      , @Schema AS    NVARCHAR(50)
      , @Table AS     NVARCHAR(50)
      , @UseRowKey    INT          = 0
      , @ProcessID    NVARCHAR(50) = '0' 
      , @SubProcessID NVARCHAR(50) = '0' 
    )
AS
BEGIN
    DECLARE @KeyCol NVARCHAR(50);
    DECLARE @ColName NVARCHAR(50);
    DECLARE @ColType INT;
    DECLARE @SQLDrop NVARCHAR(MAX);
    DECLARE @SQLHeader NVARCHAR(MAX);
    DECLARE @SQL NVARCHAR(MAX);
    DECLARE @SQLFooter NVARCHAR(MAX);
    DECLARE @ResultCode INT = 501;
    DECLARE @CurrentTimestamp DATETIME;
    DECLARE @ErrorMessage NVARCHAR(2000);

    -- Temp Tabelle mit JSON Spalten erstellen
    DECLARE @JSONColsTbl NVARCHAR(MAX) = N'[##' + CAST(NEWID() AS NVARCHAR(MAX)) + N']';

    BEGIN TRY
        BEGIN TRANSACTION pCreateJSONView;
        EXEC ('CREATE TABLE ' + @JSONColsTbl + ' (ColName NVARCHAR(50), ColType int)');
        EXEC ('INSERT INTO ' + @JSONColsTbl + ' SELECT DISTINCT b.[Key], b.Type FROM ' + @Schema + '.' + @Table + ' a CROSS APPLY OPENJSON(a.CustomValuesJSON) b');
        EXEC ('SELECT 1 FROM ' + @JSONColsTbl);

        -- Schlüsselspalte auslesen
        SELECT
            @KeyCol = ColName
        FROM system.fGetColumns(@Schema, @Table)
        WHERE
            (
            @UseRowKey = 0
            AND IsCalculatedKey = 1
            )
            OR
                (
                @UseRowKey = 1
                AND ColName = 'RowKey'
                );

        -- Skript definieren
        SET @SQLDrop = N'DROP VIEW IF EXISTS agent.vJSON_' + @Schema + N'_' + @Table;
        SET @SQLHeader = N'CREATE VIEW agent.vJSON_' + @Schema + N'_' + @Table + N' AS
			SELECT
			a.['          + @KeyCol + N']';
        SET @SQLFooter = N'
		FROM ' + @Schema + N'.' + @Table + N' a
		';
        SET @SQL = N'';

        EXEC ('SELECT 1 FROM ' + @JSONColsTbl);
        IF @@ROWCOUNT > 0
        BEGIN
            SET @SQLFooter = @SQLFooter + N' OUTER APPLY OPENJSON (a.CustomValuesJSON)
			WITH (';

            -- JSON Spalten durchgehen und Skript dynamisch ergänzen
            EXEC ('DECLARE MyCursor CURSOR FOR' + ' SELECT ColName, ColType' + ' FROM ' + @JSONColsTbl + ' WHERE ColType IN (1,2,3) ORDER BY 1');
            OPEN MyCursor;
            FETCH MyCursor
            INTO
                @ColName
              , @ColType;

            WHILE @@FETCH_STATUS = 0
            BEGIN
                SET @SQL = @SQL + N',ISNULL([' + @ColName + N'],'''') ' + CASE @ColType
                                                                            WHEN 1 THEN 'COLLATE DATABASE_DEFAULT'
                                                                            ELSE ''
                                                                        END + N' AS [' + @ColName + N'_INT]';

                SET @SQLFooter = @SQLFooter + N'
								[' + @ColName + N'] ' + CASE @ColType
                                                          WHEN 1 THEN 'nvarchar(255)'
                                                          WHEN 2 THEN 'float'
                                                          WHEN 3 THEN 'bit'
                                                      END + N',';

                FETCH MyCursor
                INTO
                    @ColName
                  , @ColType;
            END;

            CLOSE MyCursor;
            DEALLOCATE MyCursor;

            SET @SQL = @SQLHeader + @SQL + LEFT(@SQLFooter, LEN(@SQLFooter) - 1) + N')';
        END;
        ELSE
        BEGIN
            -- keine Custom Spalten
            SET @SQL = @SQLHeader + @SQL + @SQLFooter;
        END;

        --PRINT @SQL;

        EXEC (@SQLDrop);
        EXEC (@SQL);

        COMMIT TRANSACTION pCreateJSONView;
        SET @ResultCode = 200;
    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
        SET @ErrorMessage = ERROR_MESSAGE();

        ROLLBACK TRANSACTION pCreateJSONView;

        IF @Error_state <> 10
        BEGIN
            SET @ResultCode = 500;
            PRINT 'Rollback due to not executable command.';
        END;
        ELSE IF @ResultCode IS NULL
                OR @ResultCode / 100 = 2
        BEGIN
            SET @ResultCode = 500;
        END;

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry
            @Username = @Username
          , @ModuleCode = 'Integration'
          , @ProcessName = 'Agent Execution'
          , @ProcessID = @ProcessID
          , @SubProcessName = 'agent.pSync'
          , @SubProcessID = @SubProcessID
          , @SubSubProcessName = 'agent.pCreateJSONView'
          , @EventCODE = 'Error'
          , @Timestamp = @CurrentTimestamp
          , @ReturnCODE = @ResultCode
          , @EventMessage = @ErrorMessage;
    END CATCH;

    EXEC ('DROP TABLE IF EXISTS ' + @JSONColsTbl);

    RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'agent'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'pCreateJSONView'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create a helper table for the agent to resolve a JSON Column to single columns
					- the JSON Column must be named "CustomValuesJSON"
					- the resulting table will be named agent.vJSON_[schema]_[table]';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'the users name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Schema';
    SET @value      = N'the schemas name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Table';
    SET @value      = N'name of the table';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@UseRowKey';
    SET @value      = N'use row key';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


	

DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValues;
GO

/*
	Procedure to delete Integration Values 
	this is only an SWITCH procedure - it switches to an special procedure if defined or uses a default procedure
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Testcall
		EXEC agent.pDeleteIntegrationValues
			 @Username			= 'SQL'
			,@Solution			= 'FIN'
			,@CompanyKeys		= '1|44'
			,@PeriodFrom		= 202001
			,@PeriodTo			= 202012
			,@MasterDataOnly	= 0
			,@ProcessID			= '0'
			,@SubProcessID		= '0'
			,@IsDeltaloadBool	= 0

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValues','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pDeleteIntegrationValues
   @Username        NVARCHAR(255)      
  ,@Solution        NVARCHAR(50)
  ,@CompanyKeys     NVARCHAR(MAX)      -- delimited by ; e.g. "1|42;1|43"
  ,@PeriodFrom      INT
  ,@PeriodTo        INT
  ,@MasterDataOnly  BIT          = 0
  ,@ProcessID       NVARCHAR(50) = '0' 
  ,@SubProcessID    NVARCHAR(50) = '0' 
  ,@IsDeltaloadBool BIT          = 0

WITH EXECUTE AS 'dbo'

AS
	BEGIN
	-- Standard declaration for logging
		DECLARE 
				@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username									,N'NULL')	+ N''','''
				+ ISNULL(@Solution									,N'NULL')	+ N''','''											 
				+ ISNULL(@CompanyKeys								,N'NULL')	+ N''','
				+ ISNULL(CAST(@PeriodFrom		AS NVARCHAR(255))	,N'NULL')	+ N','
				+ ISNULL(CAST(@PeriodTo			AS NVARCHAR(255))	,N'NULL')	+ N','
				+ ISNULL(CAST(@MasterDataOnly	AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@ProcessID									,N'NULL')	+ N''','''
				+ ISNULL(@SubProcessID								,N'NULL')	+ N''','''
				+ ISNULL(CAST(@IsDeltaloadBool AS NVARCHAR(255))	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';
			
		DECLARE		
				 @SQLCommand		NVARCHAR(MAX)	= N''
				,@CompanyKey		NVARCHAR(100)	= N''
				,@DataSourceKey		INT				= NULL
				,@CompanyID			NVARCHAR(50)	= N''
				,@ProcName			NVARCHAR(50)	= N'';

		BEGIN TRY

			-- Check if a specific delete procedure for the given module exists
			-- If there is no proc for module, module is set to generic
			IF (OBJECT_ID('agent.pDeleteIntegrationValues' + @Solution) IS NULL)
				BEGIN
					SET @ProcName = 'agent.pDeleteIntegrationValuesGeneric';
				END
			ELSE 
				BEGIN
					SET @ProcName = 'agent.pDeleteIntegrationValues' + @Solution;
				END

			PRINT CONCAT('Procedure: ',@ProcName,' selected for execution.')

			-- Declare cursor to iterate over CompanyKeys
			DECLARE CompanyKeyCursor CURSOR FOR

				SELECT value AS CompanyKey
				FROM STRING_SPLIT(@CompanyKeys, ';');

			OPEN CompanyKeyCursor;
			FETCH CompanyKeyCursor INTO @CompanyKey;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				PRINT CONCAT('Starting delete for CompanyKey: ',@CompanyKey);

				SELECT TOP 1 @DataSourceKey = Value FROM STRING_SPLIT(@CompanyKey, '|'); -- the first due to TOP
				SELECT		 @CompanyID		= Value FROM STRING_SPLIT(@CompanyKey, '|'); -- the last due to all are assigend and last survives
		
				PRINT CONCAT('Determined Datasource: ',@DataSourceKey,' from CompanyKey.');
				PRINT CONCAT('Determined CompanyID: ',@CompanyID,' from CompanyKey.');

				-- dynamically execute delete procedure for the selected Solution, Company and Period
				SET @SQLCommand
					= N'EXEC ' 
						+ @ProcName 
						+ N'  @MasterDataOnly	= '		+ CAST(@MasterDataOnly		AS NCHAR(1)) 
						+ N', @DataSourceKey	= '		+ CAST(@DataSourceKey		AS NCHAR(10))
						+ N', @CompanyID		= N'''  + @CompanyID				+ N'''' 
						+ N', @PeriodFrom		= '		+ CAST(@PeriodFrom			AS NCHAR(6))
						+ N', @PeriodTo			= '		+ CAST(@PeriodTo			AS NCHAR(6)) 
						+ N', @Username			= N'''	+ @Username					+ N''''
						+ N', @ProcessID		= N''' 	+ @ProcessID				+ N'''' 
						+ N', @SubProcessID		= N'''	+ @SubProcessID 			+ N''''
						+ N', @IsDeltaloadBool	= ' 	+ CAST(@IsDeltaloadBool		AS NCHAR(1)) 
						+ N', @Solution			= N'''  + @Solution					+ N'''' 
						+ N';';

				PRINT CONCAT('Execute Command: "',@SQLCommand,'"');
				EXEC (@SQLCommand);

				FETCH CompanyKeyCursor
				INTO @CompanyKey;
			END;
			CLOSE CompanyKeyCursor;
			DEALLOCATE CompanyKeyCursor;

			SET @ResultCode = 200;

    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE 
				IF @ResultCode IS NULL OR @ResultCode/100 = 2
					BEGIN
						SET @ResultCode = 500;	
					END;
    END CATCH;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
    RETURN @ResultCode;
END;

GO

-- Procedure executes dynamic SQL, but uses EXECUTE AS 'dbo' - therefore no special GRANT 
-- NO GRANT

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'						 -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pDeleteIntegrationValues'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'					 -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'						 -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1								 -- 0 = Demo object - out of shipment process
																		 -- STANDARD OBJECTS
																		 -- 1 = shiped from saxess standard without modification
																		 -- 2 = shiped from saxess standard modified FOR customer from saxess
																		 -- 3 = shiped from saxess standard modified FOR customer from partner
																		 -- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		 -- CUSTOM OBJECTS
																		 -- 10 = shiped from saxess as customer specific object
																		 -- 11 = shiped from partner	as customer specific object
																		 -- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';							 -- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete integration values - switches between executing a special delete procedure or deleting generic over a default procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
	SET @level2name = N'@Username';
    SET @value      = N'User name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@Solution';
    SET @value      = N'Name of the solution, e.g. FIN.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys as semikolon separated list, e.g. 1|22;1|33';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodFrom';
    SET @value      = N'PeriodFrom, e.g. 202201';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodTo';
    SET @value      = N'PeriodTo, e.g. 202212';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@MasterDataOnly';
    SET @value      = N'MasterDataOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'String to name the process - only for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'String to name the subprocess - only for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsDeltaloadBool';
    SET @value      = N'Flag if Delta Load is used.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValuesGeneric;
GO

/*
	Procedure Delete Integration Values Generic
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall
		EXEC agent.pDeleteIntegrationValuesGeneric
			 @DataSourceKey		= Fin
			,@CompanyID			= '1'
			,@PeriodFrom		= 202001
			,@PeriodTo			= 202012
			,@MasterDataOnly	= 0
			,@Username			= 'SQL'
			,@ProcessID			= '0'
			,@SubProcessID		= '0'
			,@IsDeltaloadBool	= 0

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValuesGeneric',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValuesGeneric','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pDeleteIntegrationValuesGeneric
    @DataSourceKey   int
  , @CompanyID       nvarchar(50) = NULL
  , @PeriodFrom      int          = NULL
  , @PeriodTo        int          = NULL
  , @MasterDataOnly  bit
  , @Username        nvarchar(255)       -- used for logging
  , @ProcessID       nvarchar(50) = N'0' -- used for logging
  , @SubProcessID    nvarchar(50) = N'0' -- used for logging
  , @IsDeltaloadBool bit          = 0
  , @Solution        nvarchar(50)
AS
BEGIN
    DECLARE @ResultCode int = 501;
    DECLARE @CurrentTimestamp datetime;
    DECLARE @EventMessage nvarchar(2000);
    DECLARE @Table nvarchar(100);
    DECLARE @SQLDelete nvarchar(MAX) = N'';
    DECLARE @SQLWhere nvarchar(MAX) = N'';
    DECLARE @SQLExec nvarchar(MAX) = N'';

    BEGIN TRY
        DECLARE TableCursor CURSOR FOR
        SELECT TABLE_NAME
        FROM INFORMATION_SCHEMA.TABLES
        WHERE
            TABLE_TYPE = 'BASE TABLE'
            AND TABLE_SCHEMA = 'integration'
            AND TABLE_NAME LIKE 't' + @Solution + '%';

        OPEN TableCursor;
        FETCH TableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            -- clear variables
            SET @SQLDelete = N'';
            SET @SQLWhere = N' WHERE 1=1';
            SET @SQLExec = N'';


            SET @SQLDelete = N'DELETE FROM integration.' + @Table;
            IF COL_LENGTH('integration.' + @Table, 'DataSourceKey') IS NOT NULL
                SET @SQLWhere = @SQLWhere + N' AND DataSourceKey = ' + CONVERT(nvarchar, @DataSourceKey);


            IF COL_LENGTH('integration.' + @Table, 'CompanyID') IS NOT NULL
               AND @CompanyID IS NOT NULL
                SET @SQLWhere = @SQLWhere + N' AND CompanyID = N''' + @CompanyID + N'''';

            IF COL_LENGTH('integration.' + @Table, 'FinancialYear') IS NOT NULL
               AND COL_LENGTH('integration.' + @Table, 'FinancialPeriod') IS NOT NULL
               AND @PeriodFrom IS NOT NULL
               AND @PeriodTo IS NOT NULL
                SET @SQLWhere
                    = @SQLWhere + N' AND FinancialYear * 100 + FinancialPeriod BETWEEN '
                      + CONVERT(nvarchar, @PeriodFrom) + N' AND ' + CONVERT(nvarchar, @PeriodTo);


            SET @SQLExec = @SQLDelete + @SQLWhere;
            EXEC (@SQLExec);

            FETCH TableCursor
            INTO @Table;
        END;
        CLOSE TableCursor;
        DEALLOCATE TableCursor;

        SET @ResultCode = 200;

    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @EventMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCODE = 'Integration'
                                        , @ProcessName = 'Data Deletion'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pDeleteIntegrationValues'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = 'agent.pDeleteIntegrationValuesGeneric'
                                        , @EventCode = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @EventMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pDeleteIntegrationValuesGeneric' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Delete Integration Values Generic';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@DataSourceKey';
    SET @value      = N'DataSourceKey.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'CompanyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodFrom';
    SET @value      = N'PeriodFrom';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodTo';
    SET @value      = N'PeriodTo';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@MasterDataOnly';
    SET @value      = N'MasterDataOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Username';
    SET @value      = N'the users name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsDeltaloadBool';
    SET @value      = N'IsDeltaloadBool';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	GO

DROP PROCEDURE IF EXISTS agent.pMaintenanceIndexes;
GO

/*
	Procedure for Maintenance Indexes
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceIndexes',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceIndexes','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pMaintenanceIndexes
    @Username     nvarchar(255)
  , @Module       nvarchar(50)       -- List delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    BEGIN TRY

        DECLARE @ResultCode int = 501;
        DECLARE @MaxRowKey int = 0;
        DECLARE @StatusMessage nvarchar(2000);
        DECLARE @CurrentTimestamp datetime;

        DECLARE @Schema nvarchar(255);
        DECLARE @Table nvarchar(255);
        DECLARE @Index nvarchar(255);

        DECLARE @SQL nvarchar(MAX);


        -- Create cursor to iterate over module integration tables
        DECLARE IndexCursor CURSOR FOR
            SELECT s.name   AS SCHEMA_NAME
                 , t.name   AS TABLE_NAME
                 , ind.name AS INDEX_NAME
            FROM sys.indexes     AS ind
                JOIN sys.tables  AS t
                  ON ind.object_id = t.object_id
                JOIN sys.schemas AS s
                  ON t.schema_id = s.schema_id
            WHERE
                ind.is_primary_key = 0
                AND t.name LIKE 't' + @Module + '%'
                AND ind.type IN (5, 6);

        -- Iterate over IndexCursor
        OPEN IndexCursor;
        FETCH IndexCursor
        INTO @Schema
           , @Table
           , @Index;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            SET @SQL = N'ALTER INDEX [' + @Index + N'] ON ' + @Schema + N'.[' + @Table + N'] REORGANIZE  WITH ( LOB_COMPACTION = ON, COMPRESS_ALL_ROW_GROUPS = OFF )';
            EXEC (@SQL);

            FETCH IndexCursor
            INTO @Schema
               , @Table
               , @Index;
        END;
        CLOSE IndexCursor;
        DEALLOCATE IndexCursor;

        SET @ResultCode = 200;
        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = 'agent'
                                        , @ProcessName = 'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pMaintenanceIndexes'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = ''
                                        , @EventCODE = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceIndexes' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Maintenance Indexes';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS agent.pMaintenanceJSONViews;
GO

/*
	Procedure for Recreation of all JSON Views of selected module in integration and global schema
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = agent.pMaintenanceJSONViews
			 @Username		= 'SQL'
			,@Module		= 'FIN'
			,@ProcessID		= '1'
			,@SubProcessID	= '2'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceJSONViews',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceJSONViews','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pMaintenanceJSONViews
    @Username     NVARCHAR(255)
   ,@Module       NVARCHAR(50)			
   ,@ProcessID    NVARCHAR(50) = N'0'	-- used for logging
   ,@SubProcessID NVARCHAR(50) = N'0'	-- used for logging
AS
BEGIN
    BEGIN TRY

        DECLARE  @ResultCode		INT				= 501
				,@GlobalResultcode  INT				= 0
				,@Tablecounter		INT				= 0
				,@StatusMessage		NVARCHAR(2000)	= N''
				,@CurrentTimestamp	DATETIME
				,@Table				NVARCHAR(255)	= N'';

        -- =============================================
        -- Recreate JSON views for tables in integration schema which contain a column named CustomValuesJSON for module @Module
        -- =============================================
        -- Create cursor to iterate over module integration tables
        DECLARE IntegrationTableCursor CURSOR FOR
            SELECT 
				t.TABLE_NAME COLLATE DATABASE_DEFAULT
            FROM INFORMATION_SCHEMA.TABLES      AS t
                JOIN INFORMATION_SCHEMA.COLUMNS AS c
                  ON CONCAT(
							 t.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
							,t.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
							,t.TABLE_NAME		COLLATE DATABASE_DEFAULT
							)
					= CONCAT(
							  c.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
							 ,c.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT
							 ,c.TABLE_NAME		COLLATE DATABASE_DEFAULT
							 )
            WHERE
					t.TABLE_TYPE	COLLATE DATABASE_DEFAULT	= N'BASE TABLE'
                AND t.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT	= N'integration'
				AND t.TABLE_NAME	COLLATE DATABASE_DEFAULT	LIKE N't' + @Module + N'%'
                AND c.COLUMN_NAME	COLLATE DATABASE_DEFAULT	= N'CustomValuesJSON';

        -- Iterate over IntegrationTableCursor
        OPEN IntegrationTableCursor;
        FETCH IntegrationTableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
														,@Schema	= N'integration'
														,@Table		= @Table
														,@UseRowKey = 1;

			IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
			SET @Tablecounter = +1;

            FETCH IntegrationTableCursor
            INTO @Table;
        END;
        CLOSE IntegrationTableCursor;
        DEALLOCATE IntegrationTableCursor;

        -- =============================================
        -- Recreate JSON views for tables in global schema for module @Module
        -- =============================================
        -- Create cursor to iterate over module integration tables
        DECLARE IntegrationTableCursor CURSOR FOR
            SELECT 
				t.TABLE_NAME COLLATE DATABASE_DEFAULT
            FROM INFORMATION_SCHEMA.TABLES      AS t
                JOIN INFORMATION_SCHEMA.COLUMNS AS c
                  ON CONCAT(
								 t.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
								,t.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
								,t.TABLE_NAME		COLLATE DATABASE_DEFAULT
							) 
				  = CONCAT(
								 c.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
								,c.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
								,c.TABLE_NAME		COLLATE DATABASE_DEFAULT
							)
            WHERE
					t.TABLE_TYPE	COLLATE DATABASE_DEFAULT	= N'BASE TABLE'
                AND t.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT	= N'global'
                AND t.TABLE_NAME	COLLATE DATABASE_DEFAULT	LIKE N't' + @Module + N'%'
                AND c.COLUMN_NAME	COLLATE DATABASE_DEFAULT	= N'CustomValuesJSON';

        -- Iterate over IntegrationTableCursor
        OPEN IntegrationTableCursor;
        FETCH IntegrationTableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
														,@Schema	= N'global'
														,@Table		= @Table
														,@UseRowKey = 1;


			IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
			SET @Tablecounter = +1;

            FETCH IntegrationTableCursor
            INTO @Table;
        END;
        CLOSE IntegrationTableCursor;
        DEALLOCATE IntegrationTableCursor;

        -- =============================================
        -- Recreate JSON view for global.tCompanies
        -- =============================================
        EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
													,@Schema	= N'global'
													,@Table		= N'tCompanies'
													,@UseRowKey = 1;

		IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
		SET @Tablecounter = +1;

		IF @Tablecounter = 0 SET @GlobalResultcode = 200;
		RETURN @GlobalResultcode;

    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = N'agent'
                                        , @ProcessName = N'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = N'agent.pMaintenanceJSONViews'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = N''
                                        , @EventCODE = N'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;


END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceJSONViews'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to recreate the views for Tables which contain a CustomValuesJSON columns - for the passend Module and in the integration and global schema.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO


DROP PROCEDURE IF EXISTS agent.pMaintenanceRecreateViews;
GO

/*
	Procedure for Maintenance to recreate all Views of a given Module. The Procedure just recreates the View with the existing
	definition and changes nothing. 
	It is done, because the View defintion contains an vjson.* column and the * columns are only new refreshed, when the view is recreated.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = agent.pMaintenanceRecreateViews
				 @Username	= 'SQL'
				,@Module	= 'FIN'	
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceRecreateViews',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceRecreateViews','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pMaintenanceRecreateViews
   @Username     NVARCHAR(255)
  ,@Module       NVARCHAR(50)		
  ,@ProcessID    NVARCHAR(50) = N'0' 
  ,@SubProcessID NVARCHAR(50) = N'0' 

AS
BEGIN

    DECLARE 
		 @ResultCode		INT = 501
		,@StatusMessage		NVARCHAR(2000)
		,@CurrentTimestamp	DATETIME		
		,@View				NVARCHAR(255)
		,@ViewDefinition	NVARCHAR(MAX);

    BEGIN TRY
        BEGIN TRANSACTION pMaintenanceRecreateViews;
        -- Create cursor to iterate over module result views
        DECLARE ViewCursor CURSOR FOR
            SELECT
                N'result.' + v.TABLE_NAME AS VIEW_NAME
            FROM INFORMATION_SCHEMA.VIEWS AS v
            WHERE
                v.TABLE_SCHEMA = N'result'
                AND v.TABLE_NAME LIKE N'v' + @Module + N'%';

        -- Iterate over ViewCursor
        OPEN ViewCursor;
        FETCH ViewCursor
        INTO
            @View;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC sp_refreshview @viewname = @View

            FETCH ViewCursor INTO @View;
        END;
        CLOSE ViewCursor;
        DEALLOCATE ViewCursor;
			   
        COMMIT TRANSACTION pMaintenanceRecreateViews;
		SET @ResultCode = 200;
        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
        SET @StatusMessage = ERROR_MESSAGE();

        ROLLBACK TRANSACTION pMaintenanceRecreateViews;

        IF @Error_state <> 10
        BEGIN
            SET @ResultCode = 500;
			PRINT CONCAT(N'Error when creating view ',@View, ': ', @StatusMessage);
        END;
        ELSE IF @ResultCode IS NULL
                OR @ResultCode / 100 = 2
        BEGIN
            SET @ResultCode = 500;
        END;

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = N'agent'
                                        , @ProcessName = N'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = N'agent.pMaintenanceRecreateViews'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = ''
                                        , @EventCODE = N'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceRecreateViews'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Maintenance to recreate all Views of a given Module. The Procedure just recreates the View with the existing
					definition and changes nothing. 
					It is done, because the View defintion contains an vjson.* column and the * columns are only new refreshed, when the view is recreated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'CODE of the Module, eg. FIN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID - not needed, keep empty.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID - not needed, keep empty.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS agent.pPostBulkload;
GO

/*
	Procedure for POST Bulkload
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pPostBulkload',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pPostBulkload','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pPostBulkload
    @Username     nvarchar(255)
  , @Modules      nvarchar(MAX)      -- List delimited by ;
  , @CompanyKeys  nvarchar(MAX)      -- List delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    BEGIN TRY
        DECLARE @ResultCode int = 404;
        DECLARE @MaxRowKey int = 0;
        DECLARE @StatusMessage nvarchar(2000);
        DECLARE @CurrentTimestamp datetime;
        DECLARE @Module nvarchar(50);
        DECLARE @ProcName nvarchar(50);


        -- Write log entry for begin of procedure execution
        IF @ProcessID = '0'
        BEGIN
            SELECT @MaxRowKey = MAX(RowKey)
            FROM system.tProcessLogs;
            SET @ProcessID = @MaxRowKey + 1;
        END;

        SET @CurrentTimestamp = GETUTCDATE();

        -- Create cursor to iterate over @Modules
        DECLARE ModuleCursor CURSOR FOR
            SELECT value AS Module
            FROM STRING_SPLIT(@Modules, ';')
            WHERE value <> '';

        -- Iterate over ModuleCursor
        OPEN ModuleCursor;
        FETCH ModuleCursor
        INTO @Module;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            -- Check if specific PostBulkload procedure for given module exists.
            -- If procedure exists it will be executed.
            IF (OBJECT_ID('agent.pPostBulkload' + @Module) IS NOT NULL)
            BEGIN
                SET @ProcName = N'agent.pPostBulkload' + @Module;
                EXEC @ResultCode = @ProcName @Username = @Username
                                           , @Module = @Module
                                           , @CompanyKeys = @CompanyKeys
                                           , @ProcessID = @ProcessID
                                           , @SubProcessID = @SubProcessID;
            END;

            -- Execute maintenance procedure to create JSON views based on CustomValuesJSON column.
            EXEC @ResultCode = agent.pMaintenanceJSONViews @Username = @Username
                                                         , @Module = @Module
                                                         , @ProcessID = @ProcessID
                                                         , @SubProcessID = @SubProcessID;

            -- Recreate result views to include new or deleted CustomValuesJSON columns.
            EXEC @ResultCode = agent.pMaintenanceRecreateViews @Username = @Username
                                                         , @Module = @Module
                                                         , @ProcessID = @ProcessID
                                                         , @SubProcessID = @SubProcessID;

            -- Reorganize all columnstore indexes of given module.
            EXEC @ResultCode = agent.pMaintenanceIndexes @Username = @Username
                                                       , @Module = @Module
                                                       , @ProcessID = @ProcessID
                                                       , @SubProcessID = @SubProcessID;
            
            FETCH ModuleCursor
            INTO @Module;
        END;
        CLOSE ModuleCursor;
        DEALLOCATE ModuleCursor;

        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();
		PRINT @StatusMessage

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = 'Integration'
                                        , @ProcessName = 'Post bulkload'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pPostBulkload'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = 'agent.pPostBulkload'
                                        , @EventCODE = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pPostBulkload'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for POST Bulkload';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Modules';
    SET @value      = N'Modules';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO

DROP PROCEDURE IF EXISTS agent.pSync;
GO

/*
	Procedure for Sync
 
	Gerd Tautenhahn for Saxess Software GmbH
	10/2022 for OCT 5.8

	Testcall
		DECLARE  @RC INT
				,@Username		NVARCHAR(255)	='SQL'
				,@Solution		NVARCHAR(50)	='FIN'
				,@ResultOnly	INT				=0
				,@CompanyKeys	NVARCHAR(MAX)	='1|1;1|2' -- List must be ; separated
				,@ProcessID		NVARCHAR(50)	='0'
				,@SubProcessID	NVARCHAR(50)	='0'


		EXECUTE @RC = agent.pSync
			 @Username
			,@Solution
			,@ResultOnly
			,@CompanyKeys
			,@ProcessID
			,@SubProcessID

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'psync',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'psync','PARAMETER',NULL)
	
*/

CREATE PROCEDURE agent.pSync
    @Username     nvarchar(255)
  , @Solution     nvarchar(50)
  , @ResultOnly   int          = 0
  , @CompanyKeys  nvarchar(MAX)      -- delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    DECLARE @SQLCommand nvarchar(255);
    DECLARE @ResultCode int = 501;
    DECLARE @MaxRowKey int = 0;
    DECLARE @CurrentTimestamp datetime;

    -- write log entry for begin of agent execution
    IF @ProcessID = '0'
    BEGIN
        SELECT @MaxRowKey = MAX(RowKey)
        FROM system.tProcessLogs;
        SET @ProcessID = @MaxRowKey + 1;
    END;

    SET @CurrentTimestamp = GETUTCDATE();

    -- protect input parameters
    DECLARE @Module nvarchar(50) = @Solution; -- Set Variable Module, will be used in the future. Changes have to be made to OCT to set parameter @Module instead of @Solution
    SET @Username = dbo.sx_pf_pProtectString(@Username);
    SET @Module = dbo.sx_pf_pProtectString(@Module);
    SET @ResultOnly = dbo.sx_pf_pProtectInt(@ResultOnly);

	-- update mappingpriority of companies
	WITH Companies AS (
		SELECT
			*
		  , ROW_NUMBER() OVER (PARTITION BY COALESCE(NULLIF(CompanyIDResult, ''), CompanyID) ORDER BY MappingPriority) AS NewPriority
		FROM global.tCompanies
	)
	UPDATE Companies
	SET MappingPriority = NewPriority;

    -- execute sync procedure for the selected solution
    IF EXISTS
        (
        SELECT *
        FROM sysobjects
        WHERE
            id = OBJECT_ID(N'[agent].[pSync' + @Module + ']')
            AND OBJECTPROPERTY(id, N'IsProcedure') = 1
        )
    BEGIN
        SET @SQLCommand = N'agent.pSync' + @Module;
        EXEC @ResultCode = @SQLCommand @Username
                                     , @ResultOnly
                                     , @CompanyKeys
                                     , @ProcessID
                                     , @SubProcessID;
    END;
    ELSE
        -- if no agent procedure exists for module, nothing is executed and ResultCode 404 is returned
        SET @ResultCode = 404;

    -- write log entry for end of agent execution
    SET @CurrentTimestamp = GETUTCDATE();

    RETURN @ResultCode;
END;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pSync'			    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Sync';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Solution';
    SET @value      = N'Solution';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ResultOnly';
    SET @value      = N'ResultOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	
DROP PROCEDURE IF EXISTS agent.pSyncDimensionToGlobal;
GO

/*
	Procedure for syncing dimension to global 

	Nutzt dynamisches SQL !
 
	Gerd Tautenhahn for Saxess Software GmbH
	last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pSyncDimensionToGlobal',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pSyncDimensionToGlobal','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pSyncDimensionToGlobal
	@Username		NVARCHAR(255),		
	@CompanyKeys	NVARCHAR(MAX),
	@TableName		NVARCHAR(100),
	@ProcessID		NVARCHAR(50) = '0',	
	@SubProcessID	NVARCHAR(50) = '0'	
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX);
	DECLARE @SQLKeyAND NVARCHAR(MAX) = '';
	DECLARE @SQLKeyCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLAllFieldsCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLAllSourceFieldsCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLPropertyOR NVARCHAR(MAX) = '';
	DECLARE @SQLPropertyCOMMA NVARCHAR(MAX) = '';
	DECLARE @ResultCode INT = 501;
	DECLARE @ColName NVARCHAR(50);
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);
	DECLARE @ErrorMessageDuplicates		NVARCHAR(2000) = '';
	DECLARE @TableNameOrigin NVARCHAR(100) = REPLACE(@TableName, '#', '')
	DECLARE @Duplicates INT = -1;

	BEGIN TRY

		BEGIN TRANSACTION pSyncDimensionToGlobal;

		-- Cursor for key columns
		DECLARE KeyCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('global', @TableNameOrigin)
			WHERE IsPartOfKey = 1

		-- Cursor for regular columns
		DECLARE ColCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('integration', @TableNameOrigin)
			WHERE IsPartOfKey = 0
			AND IsCalculatedKey = 0
			AND ColName <> 'RowKey'
			AND ColName NOT IN (
				SELECT ColName
				FROM system.fGetColumns('global', @TableNameOrigin)
				WHERE IsPartOfKey = 1
			)

		-- generate lists of columns needed inside the script
		-- handle key fields
		OPEN KeyCursor
		FETCH KeyCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLKeyAND += ' AND Target.' + @ColName + ' = Source.' + @ColName;
			SET @SQLKeyCOMMA += @ColName + ',';

			SET @SQLAllFieldsCOMMA += ' ' + @ColName + ',';

			SET @SQLAllSourceFieldsCOMMA += ' Source.' + @ColName + ',';

			FETCH KeyCursor INTO @ColName
		END
		CLOSE KeyCursor
		DEALLOCATE KeyCursor

		-- handle property fields
		OPEN ColCursor
		FETCH ColCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLPropertyOR += ' OR Target.' + @ColName + ' <> Source.' + @ColName;

			SET @SQLPropertyCOMMA += ' Target.' + @ColName + ' = Source.' + @ColName + ',';

			SET @SQLAllFieldsCOMMA += ' ' + @ColName + ',';

			SET @SQLAllSourceFieldsCOMMA += ' Source.' + @ColName + ',';

			FETCH ColCursor INTO @ColName
		END
		CLOSE ColCursor
		DEALLOCATE ColCursor

		-- remove unnecessary comma
		SET @SQLKeyCOMMA = LEFT(@SQLKeyCOMMA, LEN(@SQLKeyCOMMA) - 1);
		SET @SQLAllFieldsCOMMA = LEFT(@SQLAllFieldsCOMMA, LEN(@SQLAllFieldsCOMMA) - 1);
		SET @SQLAllSourceFieldsCOMMA = LEFT(@SQLAllSourceFieldsCOMMA, LEN(@SQLAllSourceFieldsCOMMA) - 1);
		SET @SQLPropertyCOMMA = LEFT(@SQLPropertyCOMMA, LEN(@SQLPropertyCOMMA) - 1);
		
		-- check for duplicates
		SET @SQL = '
SELECT @DuplicateCount = COUNT(1)
FROM (
	SELECT 1 AS Counter
	FROM integration.' + @TableName + '
	WHERE CAST(DataSourceKey AS NVARCHAR(50)) + N''|'' + CompanyID IN (
		SELECT value
		FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
	)
	GROUP BY ' + @SQLKeyCOMMA + '
	HAVING COUNT(1) > 1
) a;'

		EXEC sp_executesql @SQL, N'@DuplicateCount INT OUT', @Duplicates OUT;
		IF @Duplicates > 0
		BEGIN
			SET @ErrorMessageDuplicates = 'ERROR: found duplicates in table integration.' + REPLACE(@TableName, '#', '') + ': ' + CAST(@Duplicates AS NVARCHAR(10))
			RAISERROR (@ErrorMessageDuplicates, 11, 1)
		END

		SET @SQL = '
UPDATE Target
SET ' + @SQLPropertyCOMMA + '
FROM integration.' + @TableName + ' AS Source
JOIN global.' + @TableNameOrigin + ' AS Target
ON 1 = 1' + @SQLKeyAND + '
WHERE (1 = 0' + @SQLPropertyOR + ')
AND CAST(Source.DataSourceKey AS NVARCHAR(50)) + N''|'' + Source.CompanyID IN (
	SELECT value
	FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
);


INSERT INTO global.' + @TableNameOrigin + '
(' + @SQLAllFieldsCOMMA + ', OwnerCODE, UpdatedFlag)
SELECT ' + @SQLAllSourceFieldsCOMMA + ', ''INTEGRATION'', 1
FROM integration.' + @TableName + ' AS Source
LEFT JOIN global.' + @TableNameOrigin + ' AS Target
ON 1 = 1' + @SQLKeyAND + '
WHERE Target.RowKey IS NULL
AND CAST(Source.DataSourceKey AS NVARCHAR(50)) + N''|'' + Source.CompanyID IN (
	SELECT value
	FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
);';

		PRINT CONCAT(FORMAT(GETDATE(), ' yyyy/MM/dd hh:mm:ss', 'en-US'), ' - Updating global.', @TableNameOrigin)

		EXEC sp_executesql @SQL;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION pSyncDimensionToGlobal;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();
		
		ROLLBACK TRANSACTION pSyncDimensionToGlobal;

		SET @CurrentTimestamp = GETUTCDATE();
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncDimensionToGlobal', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;

		RAISERROR (@ErrorMessage, 11, 1)
	END CATCH

	RETURN @ResultCode;
END;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pSyncDimensionToGlobal'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for syncing dimension to global';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TableName';
    SET @value      = N'TableName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS control.spSystemFailureReport;
GO


/*
	Procedure to fill the Table control.tSystemFailureReport
	Procedure will be build from many subsnipptes

		Testcall 
			EXEC control.spSystemFailureReport

			SELECT * FROM control.tSystemFailureReport

*/

CREATE PROCEDURE  control.spSystemFailureReport (@FactoryID NVARCHAR(255) = '', @ProductLineID NVARCHAR(255) = '')

AS

DECLARE @Timestamp 	Datetime 	= GETUTCDATE()
DECLARE @TestName	NVARCHAR (255) 	= ''
DECLARE @TestSource	NVARCHAR (255) 	= ''


-- self cleaning older than 14 Days
DELETE FROM control.tSystemFailureReport WHERE Timestamp < DATEADD(day,-14,GETUTCDATE());




SET @TestSource = 'Global_Lists'							-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for necessary System List which must exist.'			-- an friendly description of the test
-- this are the system list, a user could modify (ok) and delete (not ok)
-- there are more system list, but they are created internal and its not possible to modify

	BEGIN TRY

		WITH tmp AS
				(
				SELECT syslsts.ListID
				FROM (
					VALUES 
					  
					  (N'sxResponsiblePersons')
					, (N'sxStatus')
					, (N'sxUnits')
					, (N'sxValueEffectParameters')

				) syslsts (ListID) 
					LEFT JOIN planning.thLists hL 
						ON syslsts.ListID = hL.ListID
				WHERE hL.ListID IS NULL
				)

	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID'	AS FactoryID
				,'AllPLID'	AS ProductLineID
				,'AllPID'	AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE'	AS ResultType

				,'List ' 
					+ tmp.ListID 
					+ ' must exist, but is missing.'
							AS ResultText

				,1			AS ResultNumeric

			FROM tmp
				
					

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Global_Lists'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ListValues which use not exiting Formats for conditional formating.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 'AllFID' AS FactoryID
				,'AllPLID' AS ProductLineID
				,'AllPID' AS ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'List ' 
					+ hLV.ListID 
					+ ' uses for Value '
					+	CASE
							WHEN hL.Datentyp = 'String' THEN hLV.ValueText
							ELSE CAST(hLV.ValueInt AS NCHAR(10))
						END
					+ ' the Format ' 
					+ hLV.FormatID + ' which does not exists in Format definitions.'
						
				,1 AS ResultNumeric

			FROM planning.thListValues hLV
				LEFT JOIN planning.thLists hL
					ON hLV.ListID = hL.ListID

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = hLV.FormatID
								)
					AND hLV.FormatID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Format definitions.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductLineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ dVS.ValueSeriesID
					+ ' uses Format ' 
					+ dVS.ValueFormatID + ' which does not exists in Format definitions.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT FormatID FROM planning.thFormats WHERE FormatID = dVS.ValueFormatID
								)

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found. '
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'Product_ValueSeries'	-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for ValueSeries which use not exiting Lists as ValueSource.'			-- an friendly description of the test

	BEGIN TRY
	
		INSERT INTO control.tSystemFailureReport
			SELECT 
				 @Timestamp,
				 dVS.FactoryID
				,dVS.ProductLineID
				,dVS.ProductID
				,@TestSource
				,@TestName
				,'FAILURE' AS ResultType

				,'ValueSeries ' 
					+ ValueSeriesID
					+ ' uses ValueList ' 
					+ ValueListID + ' which does not exists in List Management.'

				,1 AS ResultNumeric

			FROM planning.tdValueSeries dVS

			WHERE 
					NOT EXISTS (
								SELECT ListID FROM planning.thLists WHERE ListID = dVS.ValueListID
								)
					AND dVS.ValueListID <>''

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH



SET @TestSource = 'ProductLine'												-- usually the name of the folder this snippet is in
SET @TestName	= 'Test for use of not existing List in Globalattributes.'	-- an friendly description of the test

	BEGIN TRY
				WITH GA AS (

					SELECT N'1' AS Num, FactoryID, ProductLineID, GlobalAttributeSource1 AS GlobalAttributeSource
					FROM planning.tdProductLines WHERE GlobalAttributeSource1 <> N'' UNION ALL
					SELECT N'2', FactoryID, ProductLineID, GlobalAttributeSource2
					FROM planning.tdProductLines WHERE GlobalAttributeSource2 <> N'' UNION ALL
					SELECT N'3', FactoryID, ProductLineID, GlobalAttributeSource3
					FROM planning.tdProductLines WHERE GlobalAttributeSource3 <> N'' UNION ALL
					SELECT N'4', FactoryID, ProductLineID, GlobalAttributeSource4
					FROM planning.tdProductLines WHERE GlobalAttributeSource4 <> N'' UNION ALL
					SELECT N'5', FactoryID, ProductLineID, GlobalAttributeSource5
					FROM planning.tdProductLines WHERE GlobalAttributeSource5 <> N'' UNION ALL
					SELECT N'6', FactoryID, ProductLineID, GlobalAttributeSource6
					FROM planning.tdProductLines WHERE GlobalAttributeSource6 <> N'' UNION ALL
					SELECT N'7', FactoryID, ProductLineID, GlobalAttributeSource7
					FROM planning.tdProductLines WHERE GlobalAttributeSource7 <> N'' UNION ALL
					SELECT N'8', FactoryID, ProductLineID, GlobalAttributeSource8
					FROM planning.tdProductLines WHERE GlobalAttributeSource8 <> N'' UNION ALL
					SELECT N'9', FactoryID, ProductLineID, GlobalAttributeSource9
					FROM planning.tdProductLines WHERE GlobalAttributeSource9 <> N'' UNION ALL
					SELECT N'10', FactoryID, ProductLineID, GlobalAttributeSource10
					FROM planning.tdProductLines WHERE GlobalAttributeSource10 <> N'' UNION ALL
					SELECT N'11', FactoryID, ProductLineID, GlobalAttributeSource11
					FROM planning.tdProductLines WHERE GlobalAttributeSource11 <> N'' UNION ALL
					SELECT N'12', FactoryID, ProductLineID, GlobalAttributeSource12
					FROM planning.tdProductLines WHERE GlobalAttributeSource12 <> N'' UNION ALL
					SELECT N'13', FactoryID, ProductLineID, GlobalAttributeSource13
					FROM planning.tdProductLines WHERE GlobalAttributeSource13 <> N'' UNION ALL
					SELECT N'14', FactoryID, ProductLineID, GlobalAttributeSource14
					FROM planning.tdProductLines WHERE GlobalAttributeSource14 <> N'' UNION ALL
					SELECT N'15', FactoryID, ProductLineID, GlobalAttributeSource15
					FROM planning.tdProductLines WHERE GlobalAttributeSource15 <> N'' UNION ALL
					SELECT N'16', FactoryID, ProductLineID, GlobalAttributeSource16
					FROM planning.tdProductLines WHERE GlobalAttributeSource16 <> N'' UNION ALL
					SELECT N'17', FactoryID, ProductLineID, GlobalAttributeSource17
					FROM planning.tdProductLines WHERE GlobalAttributeSource17 <> N'' UNION ALL
					SELECT N'18', FactoryID, ProductLineID, GlobalAttributeSource18
					FROM planning.tdProductLines WHERE GlobalAttributeSource18 <> N'' UNION ALL
					SELECT N'19', FactoryID, ProductLineID, GlobalAttributeSource19
					FROM planning.tdProductLines WHERE GlobalAttributeSource19 <> N'' UNION ALL
					SELECT N'20', FactoryID, ProductLineID, GlobalAttributeSource20
					FROM planning.tdProductLines WHERE GlobalAttributeSource20 <> N'' UNION ALL
					SELECT N'21', FactoryID, ProductLineID, GlobalAttributeSource21
					FROM planning.tdProductLines WHERE GlobalAttributeSource21 <> N'' UNION ALL
					SELECT N'22', FactoryID, ProductLineID, GlobalAttributeSource22
					FROM planning.tdProductLines WHERE GlobalAttributeSource22 <> N'' UNION ALL
					SELECT N'23', FactoryID, ProductLineID, GlobalAttributeSource23
					FROM planning.tdProductLines WHERE GlobalAttributeSource23 <> N'' UNION ALL
					SELECT N'24', FactoryID, ProductLineID, GlobalAttributeSource24
					FROM planning.tdProductLines WHERE GlobalAttributeSource24 <> N'' UNION ALL
					SELECT N'25', FactoryID, ProductLineID, GlobalAttributeSource25
					FROM planning.tdProductLines WHERE GlobalAttributeSource25 <> N''

					)
	
		INSERT INTO control.tSystemFailureReport
			
			SELECT 
				 @Timestamp,
				 GA.FactoryID
				,GA.ProductLineID
				,'AllPID' AS ProductID
			    ,@TestSource
				,@TestName
				,'FAILURE' AS ResultType -- or WARNING

				,'Not existing List ' + GA.GlobalattributeSource + ' is in use for Globalattribute ' + GA.Num AS ResultText

				,1 AS ResultNumeric -- (0 for Warning, positiv for Failure)

			FROM GA 
				LEFT JOIN planning.thLists hL
					ON GA.GlobalattributeSource = hL.ListID
				WHERE hL.ListID IS NULL
			

	-- #### STANDARD HANDLING FOR SUCCESS / ERROR - KEEP THIS WITHOUT CHANGE ########################################################
			IF @@ROWCOUNT = 0
				BEGIN
					INSERT INTO control.tSystemFailureReport
						SELECT 
							 @Timestamp
							,'AllFID' 
							,'AllPLID'
							,'AllPID'
							,@TestSource
							,@TestName
							,'OK' AS ResultType
							,'No problems found.'
							, -1 
					END
	END TRY

	BEGIN CATCH
		INSERT INTO control.tSystemFailureReport
			SELECT
				 @Timestamp 
				,'ErrorFID'
				,'ErrorPLID'
				,'ErrorPID'
				,@TestSource
				,@TestName
				,'ERROR' AS ResultType
				,'Test crashed itself.'
				,500 AS Wert

	END CATCH




GO
  
DROP PROCEDURE IF EXISTS control.pGET_RightPointSummary;
GO

/*
	Procedure to calculate the Rights Credits of an Cluster
		- Every User with Cluserwrite Rights gets 2 C Points
		- Every User with Clusterread Rights gets 1 C Points
		- Every User with an Write Right on Factory Level gets 2 F Points per Factory
		- Every User with an Read Right on Factory Level gets 1 Point per Factory
		- Every User with an Write Right on an ProductLine gets 2 PL Points
		- Every User with an Read Rights on an ProductLine gets 1 PL Point 

	Its used to monitor Point count and see unusual values due to failures in Right import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall
		EXEC control.pGET_RightPointSummary

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'pGET_RightPointSummary',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'pGET_RightPointSummary','PARAMETER',NULL)
*/
 
CREATE PROCEDURE control.pGET_RightPointSummary

AS

SELECT
	 
	 Points.UserName
	,SUM(Points.C_Points) AS C_Points
	,SUM(Points.F_Points) AS F_Points
	,SUM(Points.PL_Points) AS PL_Points

	,SUM(Points.C_Points
		+Points.F_Points
		+Points.PL_Points) AS Full_Points

	,SUM( SUM(Points.C_Points )
		+ SUM(Points.F_Points )
		+ SUM(Points.PL_Points)) OVER () AS TotalPoints
FROM 
	(
		SELECT 
			 vUR.UserName
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS C_Points
			,0 AS F_Points
			,0 AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID = '' AND vUR.ProductLineID = ''

	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS F_Points
			,0 AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID = ''


	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,0 AS F_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID <> ''

	) Points

GROUP BY Points.UserName


GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'control'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_RightPointSummary'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to calculate the Rights Credits of an Cluster
         
         - Every User with Cluserwrite Rights gets 2 C Points
         - Every User with Clusterread Rights gets 1 C Points
         - Every User with an Write Right on Factory Level gets 2 F Points per Factory
         - Every User with an Read Right on Factory Level gets 1 Point per Factory
         - Every User with an Write Right on an ProductLine gets 2 PL Points
         - Every User with an Read Rights on an ProductLine gets 1 PL Point';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
GO

DROP PROCEDURE IF EXISTS control.spParamTables;
GO

/*
	Procedure to auto create param tables from all products in ProductLine PARAM in Factory ZT

	Uses dynamic SQL !

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		EXEC control.spParamTables '1-1'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'spParamTables',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'spParamTables','PARAMETER',NULL)

*/
  
CREATE PROCEDURE control.spParamTables
	@ProductID NVARCHAR(255)
AS

BEGIN
	SET NOCOUNT ON
	
	-- Standard declaration for logging
	DECLARE 
				@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			,@ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@ProductID				,N'NULL')					+ N''''

			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N''
			,@Username			NVARCHAR(255)	= N'SQL'

	DECLARE  @TableDelete				NVARCHAR(MAX)	= ''
			,@SQLDrop					NVARCHAR(MAX)	= ''
			,@ProductKey				BIGINT			= 0;
			
	-- START TRANSACTION			***********************************************************************************
	BEGIN TRY
		BEGIN TRANSACTION spParamTables;
		
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user don`t exists', 16, 10);
				END;

	-- DELETE existing param.tPivot table
	SET @TableDelete = 'param.[tPivot' + @ProductID+ ']';
	SET @SQLDrop = 'DROP TABLE IF EXISTS ' + @TableDelete;

	EXECUTE sp_executesql @SQLDrop;

	-- CREATE new table
	
	SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = 'ZT' AND ProductLineID = 'PARAM' AND ProductID = @ProductID;

	IF @ProductKey > 0
		BEGIN
				-- CREATE TABLE -------------------------------------------------------------------------------------------------------------------
				DECLARE	 @TableSchema			NVARCHAR(255)	= 'param.'
						,@TablePreName			NVARCHAR(255)	= 'tPivot'																						
						,@StringBasic			NVARCHAR(MAX)	= 'RowKey BIGINT IDENTITY(1,1),FactoryID NVARCHAR(255),ProductLineID NVARCHAR(255),ProductID NVARCHAR(255),TimeID INT'
						,@StringColumns			NVARCHAR(MAX)	= ''	
						,@SQLCreate				NVARCHAR(MAX)	= ''
						,@TableName				NVARCHAR(MAX)	= '';

				SELECT @TableName = @TableSchema + '[' + @TablePreName + @ProductID + ']';

				-- Gets all ValueSeriesIDs from a Product in ProductlLine 'PARAM' in Factory 'ZT', ValueSeriesIDs will become column headers
				-- All Text Values will become NVARCHAR(4000), all Numeric Columns Type Money

				-- Collect definition of all columns
				SELECT @StringColumns = COALESCE(@StringColumns + ',', '') +  
						IIF(
							dVS.[IsNumeric]=0
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] NVARCHAR(4000)'
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] MONEY'
							)
				FROM planning.tdValueSeries dVS 
				WHERE dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- create the table
				SET @SQLCreate = 'CREATE TABLE ' 
							+ @TableName 
							+ '(' 
							+ @StringBasic 
							+ @StringColumns
							+ ' CONSTRAINT [PK_' + @TablePreName + @ProductID +'] PRIMARY KEY NONCLUSTERED (RowKey))';
			
				EXECUTE sp_executesql @SQLCreate;

				-- Fill Table -------------------------------------------------------------------------------------------------------
			
				DECLARE  @STRING_PivotList					NVARCHAR(MAX)  
						,@STRING_SelectColums				NVARCHAR(MAX)  
						,@STRING_Insert						NVARCHAR(MAX);

				-- Get string for dynamic SQL used in PIVOT command (...ValueSeriesID IN ([...]))
				SELECT @STRING_PivotList = COALESCE(@STRING_PivotList + ',', '') +  '[' + CONVERT(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Get string for dynamic SQL - ColumnNames
				SELECT @STRING_SelectColums = COALESCE(@STRING_SelectColums + ',', '') 
					+ 'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''') AS [' + convert(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Get string for dynamic SQL used in INSERT command
				SELECT @STRING_Insert = COALESCE(@STRING_Insert + ',', '')
							+IIF(
								 dVS.[IsNumeric]=0
								,'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''')'
								,'COALESCE(CAST([' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] AS MONEY) ' + ',0) '
							)
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Select Product Values to #table to not use dynamic SQL on real table
				DROP TABLE IF EXISTS #ParamValues;
				
				CREATE TABLE #ParamValues
					(
						 FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,TimeID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,Value				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NOT NULL
					);

				INSERT INTO #ParamValues
					SELECT 
						 fV.FactoryID
						,fV.ProductLineID
						,fV.ProductID
						,fV.ValueSeriesID
						,fV.TimeID
						,CASE
							WHEN dVS.[IsNumeric] = 0 THEN fV.ValueText
							WHEN dVS.[IsNumeric] = 1 THEN CONVERT(NVARCHAR(255),CAST(fV.ValueInt AS MONEY)/dVS.Scale,2) --warum Parameter 2 im Convert ?
						 END AS Value
					FROM planning.tfValues fV
							LEFT JOIN planning.tdValueSeries dVS
								ON fV.ValueSeriesKey = dVS.ValueSeriesKey
							LEFT JOIN planning.tdProducts dP 
								ON fV.ProductKey = dP.ProductKey
					WHERE fV.ProductKey = @ProductKey
					ORDER BY dVS.ValueSeriesNo;

				DECLARE @SQLText	NVARCHAR(MAX) = 'INSERT INTO ' + @TableName + '
					SELECT	 PivotT.FactoryID,PivotT.ProductLineID,PivotT.ProductID,PivotT.TimeID
							,' + @STRING_Insert + '													
					FROM #ParamValues
					PIVOT
						(MAX(Value) FOR ValueSeriesID IN (' + @STRING_PivotList + ')) AS PivotT';

				EXECUTE sp_executesql @SQLText;

				SELECT @EffectedRows = COUNT(1)
				FROM #ParamValues;

				-- job well done
				SET @ResultCode = 200;
		END
	ELSE
		BEGIN
			SET @ResultCode = 404;
		END
		
			COMMIT TRANSACTION spParamTables;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode

END
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'control'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spParamTables'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to auto create param tables from all products in ProductLine PARAM in Factory ZT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID of an Product in Factory ZT / ProductLine PARAM';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS  dbo.sx_pf_COPY_Factory;
GO

/*
	COPY Operation for a Factory
	Not yet existing factory is copied when Source Factory exists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 11/2022 for OCT 5.8

	Test call
	1. Empty  source or target FactoryID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists Source Factory => 404 Not Found
	5. Target Factory key already exists => 403 Forbidden
	6. Factory created > 201 Created

	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) = 'SQL';
			,@SourceFactoryID			NVARCHAR(255) = 'ZT'
			,@TargetFactoryID			NVARCHAR(255) = 'ZT2'
			,@TargetFactoryNameShort	NVARCHAR(255) = 'Neue Factory';

	EXECUTE @RC = dbo.sx_pf_COPY_Factory 
		 @Username
		,@SourceFactoryID
		,@TargetFactoryID
		,@TargetFactoryNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Factory',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_Factory
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetFactoryNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey		INT				= 0;
	DECLARE @SourceFactoryKey		INT				= 0;
	DECLARE @SourceFactoryNameShort NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetFactoryNameShort	IS NULL SET @TargetFactoryNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Factory;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetFactoryNameShort = dbo.sx_pf_pProtectString	(@TargetFactoryNameShort);
		
		IF	@SourceFactoryID	= N'' 
			OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
					
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
		-- dFactory
		INSERT INTO planning.tdFactories
		OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
		SELECT @TargetFactoryID			AS FactoryID
			, @TargetFactoryNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName 
		FROM planning.tdFactories 
		WHERE FactoryID = @SourceFactoryID; 

		SET @EffectedRows = @@ROWCOUNT;	

		SELECT TOP 1 @TargetFactoryKey = [Key]
		FROM @Keys;
		
		--dProductLine 
		INSERT INTO planning.tdProductLines
		SELECT @TargetFactoryKey AS FactoryKey
			, ProductLineID
			, @TargetFactoryID	 AS FactoryID
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3 
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6 
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9 	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12 
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18 
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21 	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;	

		--dProduct 
		INSERT INTO planning.tdProducts
		SELECT PL.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, ProductID
			, dP.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts dP
				LEFT JOIN (
					SELECT ProductLineKey, ProductLineID 
					FROM planning.tdProductLines 
					WHERE FactoryKey = @TargetFactoryKey) PL 
				ON PL.ProductLineID = dP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries  
		SELECT P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dVS.ProductID
			, dVS.ProductLineID 
			, @TargetFactoryID	AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dVS.ProductID AND P.ProductLineID = dVS.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime  
		SELECT 
			 P.ProductKey AS ProductKey
			, P.ProductLineKey AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dT.ProductID
			, dT.ProductLineID 
			, @TargetFactoryID AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dT.ProductID AND P.ProductLineID = dT.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues 
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, VS.ProductLineKey		AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, fV.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
				LEFT JOIN (
					SELECT ProductKey, ValueSeriesKey, ProductLineKey, ProductID, ValueSeriesID, ProductLineID 
					FROM planning.tdValueSeries 
					WHERE FactoryKey = @TargetFactoryKey) VS 
				ON fV.ValueSeriesID = VS.ValueSeriesID AND fV.ProductID = VS.ProductID AND fV.ProductLineID = VS.ProductLineID
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gFactories
		INSERT INTO planning.tgFactories 
		SELECT 
			 @TargetFactoryKey	AS FactoryKey
			,@TargetFactoryID	AS FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgFactories
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gProductLines
		INSERT INTO planning.tgProductLines
		SELECT 
			 PL.ProductLineKey	AS ProductLineKey 
			,@TargetFactoryKey	AS FactoryKey
			,gPL.ProductLineID
			,@TargetFactoryID	AS FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgProductLines gPL
			LEFT JOIN (
				SELECT ProductLineKey, ProductLineID 
				FROM planning.tdProductLines 
				WHERE FactoryKey = @TargetFactoryKey) 
			PL ON PL.ProductLineID = gPL.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts
		SELECT 
			  P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, gP.ProductID
			, gP.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
				FROM planning.tdProducts 
				WHERE FactoryKey = @TargetFactoryKey) P 
			ON P.ProductID = gP.ProductID AND P.ProductLineID = gP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rPreferences
		INSERT INTO planning.trPreferences
		SELECT 
			 @TransactUserKey	AS UserKey 
			,Username			AS UserName 
			,@TargetFactoryID	AS FactoryID
			,ProductLineID
			,SettingName 
			,ValueInt 
			,ValueText
		FROM planning.trPreferences 
		WHERE FactoryID = @SourceFactoryID;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,P.ProductLineKey
				,P.ProductKey
				,@TargetFactoryID		AS FactoryID
				,tF.ProductLineID		AS ProductLineID
				,tF.ProductID			AS ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = tF.ProductID AND P.ProductLineID = tF.ProductLineID
			WHERE FactoryID = @SourceFactoryID;

		--tTabs - they can exist on multiple levels ! Tabs on Factories without Productlines and on Productlines without Products may exist
		INSERT INTO planning.tTabs
			SELECT 
				 @TargetFactoryKey
				,COALESCE(PL.ProductLineKey,-1)	AS ProductLineKey
				,COALESCE(P.ProductKey,-1)		AS ProductKey
				,@TargetFactoryID				AS FactoryID
				,COALESCE(tT.ProductLineID,'')	AS ProductLineID
				,COALESCE(tT.ProductID,'')		AS ProductID
				,tT.TabID
				,tT.Orderindex
				,tT.TabName
				,tT.TabHint
				,tT.TabText
				,tT.PresentationCODE
				,tT.Datasource
				,tT.IsVisibleBOOL
				,tT.LayoutJSON
				,tT.ParameterJSON

			FROM planning.tTabs tT

				-- Determine new ProductKeys
				LEFT JOIN (
					SELECT 
						 ProductKey
						,ProductLineKey
						,ProductID
						,ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey
					) P 
				ON 
						COALESCE(P.ProductID,'')		= tT.ProductID 
					AND COALESCE(P.ProductLineID,'')	= tT.ProductLineID

				-- Determine new ProductLineKeys
				LEFT JOIN (
					SELECT 
						 ProductLineKey
						,ProductLineID 
					FROM planning.tdProductlines
					WHERE FactoryKey = @TargetFactoryKey
					) PL
				ON COALESCE(PL.ProductLineID,'')	= tT.ProductLineID

			WHERE FactoryID = @SourceFactoryID;

		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
		SELECT ProductKey
			, dp.ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, ProductID
			, dp.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, N'COPY'			AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'" / ProductLine <' + dp.ProductLineID + N'>"' + dpl.Nameshort
				+ N'" / Product <' + ProductID + N'>"' + dp.Nameshort + '"' AS Statement
			, @TransactUsername AS UserName 
			, N''				AS PCName
			, N''				AS ProcessorCode
			, N''				AS IPAddresses
			, GETUTCDATE()		AS Timestamp 
			, 0					AS IsDeleted 
			, 0					AS IsResolved
		FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl 
			ON dp.ProductLinekey= dpl.ProductLineKey
		WHERE dp.FactoryKey = @TargetFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Right materialization
		EXEC system.spMaterialize_trUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_Factory'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a Factory';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryNameShort';
    SET @value      = N'TargetFactoryNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_COPY_Format;
GO

/*
	Procedure for COPY Operation for a Format
	Not yet existing Format is copied if source Format is available
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	DECLARE  @RC				INT
			,@Username			NVARCHAR(255) = 'SQL'
			,@SourceFormatID	NVARCHAR(255) = 'EingabeGanzzahl'
			,@TargetFormatID	NVARCHAR(255) = '88C26';

	EXECUTE @RC = dbo.sx_pf_COPY_Format 
		 @Username
		,@SourceFormatID
		,@TargetFormatID;

	PRINT @RC

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Format',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_Format
	@Username		NVARCHAR(255),
	@SourceFormatID NVARCHAR(255),
	@TargetFormatID NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFormatKey	INT = 0;
	DECLARE @TargetFormatKey	INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@SourceFormatID,	N'NULL') + N''',''' 
			+ ISNULL(@TargetFormatID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceFormatID	IS NULL SET @SourceFormatID = N'';
	IF @TargetFormatID	IS NULL SET @TargetFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Format;

		-- STEP 0.2 - Protect input parameters			
		SET @Username		= dbo.sx_pf_pProtectString  (@Username);
		SET @SourceFormatID	= dbo.sx_pf_pProtectID		(@SourceFormatID);
		SET @TargetFormatID = dbo.sx_pf_pProtectID		(@TargetFormatID);

		IF  @SourceFormatID		= N'' 
			OR @TargetFormatID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @SourceFormatID;
		
		IF @SourceFormatKey = 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @TargetFormatID;
		
		IF @TargetFormatKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hFormats
		INSERT INTO planning.thFormats
			SELECT @TargetFormatID AS FormatID
				, BackgroundColor
				, FontColor
				, ValueFormat
				, Bold
				, Italic
				, AlignmentHorizontal
				, AlignmentVertical
				, TextUnderlined
				, BorderToNext
				, BorderToPrevious
			FROM planning.thFormats
			WHERE FormatKey = @SourceFormatKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_Format TO pf_PlanningFactoryService;   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Format TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_Format'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for COPY Operation for a Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFormatID';
    SET @value      = N'SourceFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFormatID';
    SET @value      = N'TargetFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_COPY_List;
GO

/*
	COPY Operation for a List
	Not yet existing list is copied if Sources List is available
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
 
	DECLARE  @RC			INT
			,@Username		NVARCHAR(255) = 'SQL'
			,@SourceListID	NVARCHAR(255) = 'suEvents'
			,@TargetListID	NVARCHAR(255) = '88C26';

	EXECUTE @RC = dbo.sx_pf_COPY_List 
		 @Username
		,@SourceListID
		,@TargetListID;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_List',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_List
	@Username		NVARCHAR(255),
	@SourceListID	NVARCHAR(255),
	@TargetListID	NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceListKey INT = 0;
	DECLARE @TargetListKey INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@SourceListID, N'NULL')	+ N''',''' 
			+ ISNULL(@TargetListID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @SourceListID	IS NULL SET @SourceListID	= N'';
	IF @TargetListID	IS NULL SET @TargetListID	= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_List;

		-- STEP 0.2 - Protect input parameters			
		SET @Username	  =	dbo.sx_pf_pProtectString (@Username);
		SET @SourceListID = dbo.sx_pf_pProtectID	 (@SourceListID);
		SET @TargetListID = dbo.sx_pf_pProtectID	 (@TargetListID);

		IF	@SourceListID	 = N'' 
			OR @TargetListID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceListKey = ListKey 
		FROM planning.thLists 
		WHERE ListID = @SourceListID;
		
		IF @SourceListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetListKey = ListKey 
		FROM planning.thLists 
		WHERE ListID = @TargetListID;

		IF @TargetListKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target List key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		--hLists
		INSERT INTO planning.thLists
			SELECT @TargetListID AS ListsID
				, NameShort
				, NameLong
				, CommentDev
				, CommentUser
				, Datentyp
				, Source
				, SourceFormula
				, FormatID
			FROM planning.thLists
			WHERE ListKey = @SourceListKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--hListValues
		INSERT INTO planning.thListValues 
			SELECT @TargetListID AS ListsID
				, ValueInt
				, Scale
				, ValueText
				, ValueComment
				, FormatID
			FROM planning.thListValues
			WHERE ListID = @SourceListID;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_List TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_List TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_List'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for COPY Operation for a List';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceListID';
    SET @value      = N'SourceListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetListID';
    SET @value      = N'TargetListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_COPY_Product;
GO

/*
	COPY Operation for a Product
	 - creates a new Product from an Template
	 - * as Target ProductID is supported, the next free numeric ID is used then
		
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	Test call
	1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine / Product => 404 Not Found
	5. Not exists target Factory / ProductLine => 404 Not Found
	6. Target Product key already exists => 403 Forbidden
	7. Product copied => 201 Created

	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) = 'SQL'
			,@SourceFactoryID			NVARCHAR(255) = 'WSB'
			,@SourceProductLineID		NVARCHAR(255) = 'KRED1'
			,@SourceProductID			NVARCHAR(255) = '0'
			,@TargetFactoryID			NVARCHAR(255) = 'WSB'
			,@TargetProductLineID		NVARCHAR(255) = 'KRED1'
			,@TargetProductID			NVARCHAR(255) = '*'
			,@TargetProductNameShort	NVARCHAR(255) = 'Test';

	EXECUTE @RC = dbo.sx_pf_COPY_Product 
			   @Username
			  ,@SourceFactoryID
			  ,@SourceProductLineID
			  ,@SourceProductID
			  ,@TargetFactoryID
			  ,@TargetProductLineID
			  ,@TargetProductID
			  ,@TargetProductNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Product',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_Product
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@SourceProductLineID	NVARCHAR(255),
	@SourceProductID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetProductLineID	NVARCHAR(255),
	@TargetProductID		NVARCHAR(255),
	@TargetProductNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
	IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
	IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
	IF @TargetProductNameShort	IS NULL SET @TargetProductNameShort = N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Product;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID		(@SourceProductLineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID		(@TargetProductLineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetProductNameShort = dbo.sx_pf_pProtectString	(@TargetProductNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE FactoryKey = @SourceFactoryKey AND ProductLineKey = @SourceProductLineKey AND ProductID = @SourceProductID;
		
		IF	@SourceFactoryKey			= 0 
			OR @SourceProductLineKey	= 0 
			OR @SourceProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @TargetFactoryKey AND ProductLineKey = @TargetProductLineKey AND ProductID = @TargetProductID;
		
		IF	@TargetFactoryKey		 = 0 
			OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
		
		-- STEP 1.4 - If ProductID * is submitted, search the next free numeric ProductID
		IF @TargetProductID = '*' 
		BEGIN
			SELECT @TargetProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)), 0) + 1 
			FROM planning.tdProducts  
			WHERE ProductLineKey = @TargetProductLineKey 
				AND TRY_CAST(ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @TargetProductID = '*' 
				SELECT @TargetProductID = 1;
		END

		-- STEP 3 - Copy data		
		--dProduct
		DECLARE @TargetProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

		INSERT INTO planning.tdProducts
		OUTPUT INSERTED.ProductKey INTO @TargetProductKeyTable
		SELECT @TargetProductLineKey	AS ProductLineKey
			, @TargetFactoryKey			AS FactoryKey
			, @TargetProductID			AS ProductID
			, @TargetProductLineID		AS ProductLineID
			, @TargetFactoryID			AS FactoryID
			, TimeType
			, @TargetProductNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1 , GlobalAttribute2 , GlobalAttribute3 , GlobalAttribute4 , GlobalAttribute5 , GlobalAttribute6 , GlobalAttribute7
			, GlobalAttribute8 , GlobalAttribute9 , GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductKey = KeyNumber 
		FROM @TargetProductKeyTable;

		--gProducts Inhalt
		INSERT INTO planning.tgProducts
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries
		SELECT
		      @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime
		SELECT
			 @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM planning.tdValueSeries 
				WHERE ProductKey = @TargetProductKey) VS 
			ON VS.ValueSeriesID = fV.ValueSeriesID 
		WHERE fV.ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT
				 @TargetFactoryKey
				,@TargetProductLineKey
				,@TargetProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,@TargetProductID		AS ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID
				AND ProductID		= @SourceProductID;

		--fStatements CopyInformation speichern
		INSERT INTO planning.tfStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
				+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort 
				+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, GETUTCDATE()			AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;
		
		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;
			
		COMMIT TRANSACTION sx_pf_COPY_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryService;   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_Product'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductNameShort';
    SET @value      = N'TargetProductNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF  EXISTS dbo.sx_pf_COPY_ProductLine;
GO

/*
	COPY Operation for a ProductLine
	Not yet existing ProductLine is copied when Source ProductLine is available

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
 
	Test call
	1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine => 404 Not Found
	5. Not exists target Factory => 404 Not Found
	6. Target ProductLine key already exists => 403 Forbidden
	7. ProductLine copied => 201 Created

	DECLARE  @RC							INT
			,@Username						NVARCHAR(255) = 'SQL'
			,@SourceFactoryID				NVARCHAR(255) = 'ZT'
			,@SourceProductLineID			NVARCHAR(255) = 'ZT'
			,@TargetFactoryID				NVARCHAR(255) = 'ZT'
			,@TargetProductLineID			NVARCHAR(255) = 'ZT32'
			,@TargetProductLineNameShort	NVARCHAR(255) = 'Neue ProductLine32';

	EXECUTE  @RC = dbo.sx_pf_COPY_ProductLine 
			 @Username
			,@SourceFactoryID
			,@SourceProductLineID
			,@TargetFactoryID
			,@TargetProductLineID
			,@TargetProductLineNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ProductLine',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_ProductLine
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductLineNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey			INT				= 0;
	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					  N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			  N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			  N'NULL') + N''','''
			+ ISNULL(@TargetProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineNameShort, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductLineNameShort	IS NULL SET @TargetProductLineNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductLineID		= dbo.sx_pf_pProtectID		(@SourceProductLineID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductLineID		= dbo.sx_pf_pProtectID		(@TargetProductLineID);
		SET @TargetProductLineNameShort = dbo.sx_pf_pProtectString	(@TargetProductLineNameShort);

		IF	@SourceFactoryID				= N'' 
			OR @SourceProductLineID			= N'' 
			OR @TargetFactoryID				= N''
			OR @TargetProductLineID			= N'' 
			OR @TargetProductLineNameShort	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey,@SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey,@SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		
		IF	@SourceFactoryKey		 = 0 
			OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Copy data		
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
		
		--dProductLine
		INSERT INTO planning.tdProductLines
		OUTPUT INSERTED.ProductLineKey INTO @Keys([Key]) 
		SELECT @TargetFactoryKey			AS FactoryKey
			, @TargetProductLineID			AS ProductLineID
			, @TargetFactoryID				AS FactoryID
			, @TargetProductLineNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate 
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3  
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6  
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9  	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12  
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15 
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18  
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21  	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24 
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductLineKey = [Key] 
		FROM @Keys;
		
		--dProduct
		INSERT INTO planning.tdProducts 
		SELECT @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		 AS FactoryKey
			, ProductID
			, @TargetProductLineID	 AS ProductLineID
			, @TargetFactoryID		 AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion 
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7  
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14 
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21  
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries 
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dVS.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID 
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dVS.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime 
		SELECT 
			P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dT.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dT.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID, ProductID 
				FROM planning.tdValueSeries
				WHERE ProductLineKey = @TargetProductLineKey) VS 
			ON fV.ValueSeriesID=VS.ValueSeriesID AND fV.ProductID = VS.ProductID
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		--gProductLines 
		INSERT INTO planning.tgProductLines
		SELECT @TargetProductLineKey AS ProductLineKey 
			, @TargetFactoryKey		 AS FactoryKey
			, @TargetProductLineID	 AS ProductLineID
			, @TargetFactoryID		 AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProductLines
		WHERE ProductLineKey = @SourceProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts  
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, gP.ProductID
			, @TargetProductLineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts  
				WHERE ProductLineKey = @TargetProductLineKey) P
			ON P.ProductID = gP.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rRights - no definition, write Rights on Factory will lead to implicit Write Rights for creator

		--rPreferences
		INSERT INTO planning.trPreferences
			SELECT 
				 UserKey				AS UserKey 
				,Username				AS UserName
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID 
				,SettingName
				,ValueInt
				,ValueText
			FROM planning.trPreferences
			WHERE 
					FactoryID		= @SourceFactoryID 
				AND ProductLineID	= @SourceProductLineID;
		
		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,@TargetProductLineKey
				,P.ProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,tF.ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductID 
					FROM planning.tdProducts  
					WHERE ProductLineKey = @TargetProductLineKey) P
				ON P.ProductID = tF.ProductID 
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID;

		SET @EffectedRows += @@ROWCOUNT;

		--tTabs
		INSERT INTO planning.tTabs
			SELECT 
				 @TargetFactoryKey
				,@TargetProductLineKey
				,COALESCE(P.ProductKey,-1)	AS ProductKey
				,@TargetFactoryID			AS FactoryID
				,@TargetProductLineID		AS ProductLineID
				,COALESCE(tT.ProductID,'')	AS ProductID
				,tT.TabID
				,tT.Orderindex
				,tT.TabName
				,tT.TabHint
				,tT.TabText
				,tT.PresentationCODE
				,tT.Datasource
				,tT.IsVisibleBOOL
				,tT.LayoutJSON
				,tT.ParameterJSON

			FROM planning.tTabs tT
				LEFT JOIN (
							SELECT ProductKey, ProductID 
							FROM planning.tdProducts  
							WHERE ProductLineKey = @TargetProductLineKey
							) P
				ON COALESCE(P.ProductID,'') = tT.ProductID 
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID;


		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
			SELECT ProductKey
				, @TargetProductLineKey AS ProductLineKey
				, @TargetFactoryKey		AS FactoryKey 
				, ProductID
				, @TargetProductLineID  AS ProductLineID
				, @TargetFactoryID		AS FactoryID
				, N'COPY'				AS ActionType
				, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
					+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort
					+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS Statement
				, @TransactUsername		AS UserName
				, N''					AS PCName 
				, N''					AS ProcessorCode 
				, N''					AS IPAddresses
				, GETUTCDATE()			AS Timestamp
				, 0						AS IsDeleted
				, 0						AS IsResolved
			FROM planning.tdProducts 
			WHERE ProductLineKey = @TargetProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Rights materialization 
		EXEC system.spMaterialize_trUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a ProductLine';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineNameShort';
    SET @value      = N'TargetProductLineNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_COPY_ValueSerie;
GO

/*
	COPY Operation for one ValueSerie
	The ValueSerie will be copied and inserted at last position
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	Test call
	1. Empty source or target FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3 .User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine / Product / ValueSerie => 404 Not Found
	5. Not exists target Factory / ProductLine / Product => 404 Not Found
	6. Target ValueSerie key already exists => 403 Forbidden
	7. ValueSerie copied => 201 Created

	DECLARE  @RC							INT
			,@Username					NVARCHAR(255) = 'SQL'
			,@SourceFactoryID			NVARCHAR(255) = '230'
			,@SourceProductLineID		NVARCHAR(255) = 'I-ST-IT'
			,@SourceProductID			NVARCHAR(255) = '888'
			,@SourceValueSerieID		NVARCHAR(255) = 'C2'
			,@TargetFactoryID			NVARCHAR(255) = 'ZT'
			,@TargetProductLineID		NVARCHAR(255) = 'ZT32'
			,@TargetProductID			NVARCHAR(255) = '1'
			,@TargetValueSerieID		NVARCHAR(255) = '88C2'
			,@TargetValueSerieNameShort NVARCHAR(255) = 'Kosten Test';

	EXECUTE @RC = dbo.sx_pf_COPY_ValueSerie 
			   @Username
			  ,@SourceFactoryID
			  ,@SourceProductLineID
			  ,@SourceProductID
			  ,@SourceValueSerieID
			  ,@TargetFactoryID
			  ,@TargetProductLineID
			  ,@TargetProductID
			  ,@TargetValueSerieID
			  ,@TargetValueSerieNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ValueSerie',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_ValueSerie
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@SourceProductID			NVARCHAR(255),
	@SourceValueSerieID			NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductID			NVARCHAR(255),
	@TargetValueSerieID			NVARCHAR(255),
	@TargetValueSerieNameShort	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceValueSerieKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceValueSerieNameShort	NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @TargetValueSerieKey		INT				= 0;
	DECLARE @TargetValueSerieNo			INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+ ISNULL(@Username,						N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,				N'NULL') + N''',''' +
			+ ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,				N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetValueSerieNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @SourceProductID				IS NULL SET @SourceProductID			= N'';
	IF @SourceValueSerieID			IS NULL SET @SourceValueSerieID			= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductID				IS NULL SET @TargetProductID			= N'';
	IF @TargetValueSerieID			IS NULL SET @TargetValueSerieID			= N'';
	IF @TargetValueSerieNameShort	IS NULL SET @TargetValueSerieNameShort	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductLineID		= dbo.sx_pf_pProtectID		(@SourceProductLineID);
		SET @SourceProductID			= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @SourceValueSerieID			= dbo.sx_pf_pProtectID		(@SourceValueSerieID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductLineID		= dbo.sx_pf_pProtectID		(@TargetProductLineID);
		SET @TargetProductID			= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetValueSerieID			= dbo.sx_pf_pProtectID		(@TargetValueSerieID);
		SET @TargetValueSerieNameShort	= dbo.sx_pf_pProtectString	(@TargetValueSerieNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N'' 
			OR @SourceValueSerieID	= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N'' 
			OR @TargetValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername,@SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = NameShort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE ProductLineKey = @SourceProductLineKey AND ProductID= @SourceProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey, @SourceValueSerieNameShort = NameShort FROM planning.tdValueSeries WHERE ProductKey = @SourceProductKey  AND ValueSeriesID= @SourceValueSerieID;	
		
		IF	@SourceFactoryKey = 0 
			OR @SourceProductLineKey = 0 
			OR @SourceProductKey = 0 
			OR @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;
		
		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE  FactoryKey = @TargetFactoryKey  AND ProductLineKey = @TargetProductLineKey AND ProductID = @TargetProductID;
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @TargetProductKey  AND ValueSeriesID= @TargetValueSerieID;
				
		IF	@TargetFactoryKey = 0 
			OR @TargetProductLineKey = 0 
			OR @TargetProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data 
		SELECT @TargetValueSerieNo = MAX(ValueSeriesNo) + 1 FROM planning.tdValueSeries WHERE ProductKey = @TargetProductKey;
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		INSERT INTO planning.tdValueSeries 
		OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
		SELECT @TargetProductKey			AS ProductKey
			, @TargetProductLineKey			AS ProductLineKey
			, @TargetFactoryKey				AS FactoryKey
			, @TargetProductID				AS ProductID
			, @TargetProductLineID			AS ProductLineID
			, @TargetFactoryID				AS FactoryID
			, @TargetValueSerieID			AS ValueSeriesID
			, @TargetValueSerieNo			AS ValueSeriesNo
			, @TargetValueSerieNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries 
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetValueSerieKey = [Key] 
		FROM @Keys;
		
		--fValues
		INSERT INTO planning.tfValues  
		SELECT @TargetValueSerieKey AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, @TargetValueSerieID	AS ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues
		WHERE ValueSeriesKey = @SourceValueSerieKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fStatements 
		INSERT INTO planning.tfStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'New ValueSerieID <' + @TargetValueSerieID + N'>"' + @TargetValueSerieNameShort
				+ N'" for product produced as a copy of Factory <' + @SourceFactoryID + N'>"' +  @SourceFactoryNameShort
				+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' +  @SourceProductLineNameShort
				+ N'"/ Product <' + @SourceProductID + N'>"' +  @SourceProductNameShort
				+ N'"/ ValueSerie <' + @SourceValueSerieID + N'>"' +  @SourceValueSerieNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, ''					AS PCName 
			, ''					AS ProcessorCode 
			, ''					AS IPAddresses
			, GETUTCDATE()			AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_ValueSerie TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ValueSerie TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_ValueSerie'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for one ValueSerie';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceValueSerieID';
    SET @value      = N'SourceValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetValueSerieID';
    SET @value      = N'TargetValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetValueSerieNameShort';
    SET @value      = N'TargetValueSerieNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
	

DROP PROCEDURE IF  EXISTS dbo.sx_pf_DATAOUTPUT_Balance
GO

/*
	Procedure to deliver analytic data to the enduser
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	Testcall
	DECLARE @RC int
	DECLARE @Username nvarchar(255) = 'SQL'
	DECLARE @FactoryID nvarchar(255) = '1'
	DECLARE @ProductLineID nvarchar(255) = '1'

	EXECUTE @RC = dbo.sx_pf_DATAOUTPUT_Balance
	   @Username
	  ,@FactoryID
	  ,@ProductLineID
	Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_Balance',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_Balance','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DATAOUTPUT_Balance 
	@Username AS NVARCHAR(255)
	, @FactoryID AS NVARCHAR(255)=''
	, @ProductLineID AS NVARCHAR(255)=''

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC dbo.sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.BalanceEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_DATAOUTPUT_Balance TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DATAOUTPUT_Balance TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DATAOUTPUT_Balance'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data to the enduser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	
DROP PROCEDURE IF  EXISTS dbo.sx_pf_DATAOUTPUT_CashValues
GO

/*
	Procedure to deliver analytic data to the enduser
	Gerd Tautenhahn for Saxess Software GmbH

	Last modifie: 09/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username nvarchar(255) = 'SQL'
		DECLARE @FactoryID nvarchar(255) = '1'
		DECLARE @ProductLineID nvarchar(255) = '1'

		EXECUTE @RC = dbo.sx_pf_DATAOUTPUT_CashValues
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_CashValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_CashValues','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DATAOUTPUT_CashValues (
	@Username AS NVARCHAR(255)
	, @FactoryID AS NVARCHAR(255)=''
	, @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC dbo.sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0

IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT 
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)) as Week
		 ,TRY_CAST(TRY_CAST(fV.TimeID AS Integer) / 10000 AS nvarchar) +'-'+ FORMAT(dbo.Week_of_Year_DE (TRY_CAST(TRY_CAST(fV.TimeID AS nvarchar) AS Datetime)),'00') as YearWeek
		 ,TRY_CAST(right(fV.TimeID,2) AS Integer) as Day
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort		AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort		AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort         AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		fV.TimeID > 20000000 AND
		fV.TimeID < 21000000 AND
		gVE.CashEffect <> 0 AND
		fV.FactoryID = @FactoryID AND
		fV.ProductLineID = @ProductLineID

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_DATAOUTPUT_CashValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DATAOUTPUT_CashValues TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DATAOUTPUT_CashValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data to the enduser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF  EXISTS dbo.sx_pf_DATAOUTPUT_FactorySummary
GO

/*
	Procedure to deliver analytic data as factory summary
	Gerd Tautenhahn for Saxess Software GmbH

	Last modified: 09/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username nvarchar(255) = 'SQL'
		DECLARE @FactoryID nvarchar(255) = 'PE'

		EXECUTE @RC = dbo.sx_pf_DATAOUTPUT_FactorySummary
		   @Username
		  ,@FactoryID
		Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_FactorySummary',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_FactorySummary','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DATAOUTPUT_FactorySummary (
	 @Username AS NVARCHAR(255)
	,@FactoryID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC dbo.sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END
	SELECT 

		  TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month

         ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.CashEffect) AS Cash
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.ProfitLossEffect) AS Profit
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.BalanceEffect) AS Balance
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1) * gVE.QuantityEffect) AS Quantity
		 ,Sum(TRY_CAST(fV.ValueInt as Money) / isnull(dVS.Scale,1)) AS Value

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName


  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 LEFT JOIN planning.tgValueEffects gVE
				ON dVS.Effect = gVE.EffectID
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueInt <> 0 AND
		dVS.[IsNumeric] = 1 AND
		dF.FactoryID = @FactoryID

  GROUP BY

  		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100																

		 ,fV.ProductLineID + ' ' + dPL.NameShort	
         ,dPL.NameShort												

		 ,fV.ProductID + ' ' + dP.NameShort			
         ,dP.NameShort												
	

SET @Resultcode = 200 

RETURN @Resultcode
		
GO
  
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_DATAOUTPUT_FactorySummary TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DATAOUTPUT_FactorySummary TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DATAOUTPUT_FactorySummary'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data as factory summary';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF  EXISTS dbo.sx_pf_DATAOUTPUT_TextValues
GO

/*
	Procedure to deliver analytic data to the enduser
	DataFactory 5.0
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username nvarchar(255) = 'SQL'
		DECLARE @FactoryID nvarchar(255) = '1'
		DECLARE @ProductLineID nvarchar(255) = '1'

		EXECUTE @RC = dbo.sx_pf_DATAOUTPUT_TextValues
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_TextValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DATAOUTPUT_TextValues','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DATAOUTPUT_TextValues (
	  @Username AS NVARCHAR(255)
	, @FactoryID AS NVARCHAR(255)=''
	, @ProductLineID AS NVARCHAR(255)='')

AS

-- OPEN: Check Security etc.
-- OPEN: Logger

DECLARE @TransactUsername AS NVARCHAR (255)
DECLARE @Resultcode AS INT

-- Transaktionsuser ermitteln
SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username)
IF @TransactUsername  = '403'	BEGIN
									SET @ResultCode = 403
									--ROLLBACK TRANSACTION ONE
									--EXEC dbo.sx_pf_pPOST_API_LogEntry @Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ResultCode,@TimestampCall,@Comment
									RETURN @ResultCode
								END

IF @FactoryID = '' AND @ProductLineID = '' 
	SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter
		 ,fV.ValueComment

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0


IF @FactoryID <> '' AND @ProductLineID = ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID

IF @FactoryID = '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.ProductLineID = @ProductLineID

IF @FactoryID <> '' AND @ProductLineID <> ''
		SELECT
		  fV.FactoryID
         ,fV.ProductLineID
         ,fV.ProductID
         ,fV.ValueSeriesID

		 ,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
		 ,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100				AS Month
		 ,TRY_CAST(TRY_CAST(fV.TimeID as nvarchar) AS Datetime)		AS [Date]

         ,fV.ValueText AS Value

		 ,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
         ,dF.NameShort								AS FactoryName
         ,dF.ResponsiblePerson						AS FResponsiblePerson

		 ,fV.ProductLineID + ' ' + dPL.NameShort	AS ProductLineIDName
         ,dPL.NameShort								AS ProductLineName
         ,dPL.ResponsiblePerson						AS PLResponsiblePerson

		 ,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
         ,dP.NameShort								AS ProductName
		 ,dP.ResponsiblePerson						AS PResponsiblePerson

		 ,fV.ValueSeriesID + ' ' + dVS.NameShort	AS ValueSeriesIDName
		 ,dVS.NameShort								AS ValueSeriesName
		 ,dVS.ValueSeriesNo
		 ,dVS.VisibilityLevel
		 ,dVS.ValueSource
         ,dVS.Unit
         ,dVS.Effect
         ,dVS.EffectParameter

         ,dP.[Status]
         ,dP.GlobalAttribute1
         ,dP.GlobalAttribute2
         ,dP.GlobalAttribute3
         ,dP.GlobalAttribute4
         ,dP.GlobalAttribute5
         ,dP.GlobalAttribute6
         ,dP.GlobalAttribute7
         ,dP.GlobalAttribute8
         ,dP.GlobalAttribute9
         ,dP.GlobalAttribute10
         ,dP.GlobalAttribute11
         ,dP.GlobalAttribute12
         ,dP.GlobalAttribute13
         ,dP.GlobalAttribute14
         ,dP.GlobalAttribute15
         ,dP.GlobalAttribute16
         ,dP.GlobalAttribute17
         ,dP.GlobalAttribute18
         ,dP.GlobalAttribute19
         ,dP.GlobalAttribute20
         ,dP.GlobalAttribute21
         ,dP.GlobalAttribute22
         ,dP.GlobalAttribute23
         ,dP.GlobalAttribute24
         ,dP.GlobalAttribute25

  FROM   planning.tfValues fV
         LEFT JOIN planning.tdValueSeries dVS
                ON fV.ValueSeriesKey = dVS.ValueSeriesKey
         LEFT JOIN planning.tdProducts dP
                ON fV.ProductKey = dP.ProductKey
         LEFT JOIN planning.tdProductLines dPL
                ON fV.ProductLineKey = dPL.ProductLineKey
         LEFT JOIN planning.tdFactories dF
                ON fV.FactoryKey = dF.FactoryKey
		 INNER JOIN 
				(SELECT * FROM system.trUserRights WHERE Username = @TransactUsername AND [Right] IN ('Write','Read') AND FactoryID <> '' AND ProductLineID <> '') vUR
				ON fV.FactoryID = vUR.FactoryID AND 
				   fV.ProductLineID = vUR.ProductLineID			

  WHERE fV.ValueText <> '' AND fV.ValueText <> '-' AND
		dVS.[IsNumeric] = 0 AND fV.FactoryID = @FactoryID AND fV.ProductLineID = @ProductLineID 

SET @Resultcode = 200 

RETURN @Resultcode
		
GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_DATAOUTPUT_TextValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DATAOUTPUT_TextValues TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DATAOUTPUT_TextValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data to the enduser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ClusterProperty;
GO

/*
	DELETE Operation for a ClusterProperty
	Deletes a Cluster Property.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the cluster => 401 Unauthorized
	4. Property deleted => 200 OK
	5. PropertyID not found => 204 No Content

		DECLARE @RC					INT
		DECLARE @Username			NVARCHAR(255) = 'SQL'
		DECLARE @PropertyID			NVARCHAR(255) = 'rre'

		EXECUTE @RC = dbo.sx_pf_DELETE_ClusterProperty 
				@Username,
				@PropertyID

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ClusterProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ClusterProperty','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ClusterProperty;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);
		
		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete
		DELETE FROM planning.tgCluster 
		WHERE PropertyID = @PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ClusterProperty TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ClusterProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes a Cluster Property.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Factory;
GO

/*
	DELETE Operation for a Factory
	Deletes an entire Factory (include ProductLines, Products, Values)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9

	Test call
		1. FactoryID is empty => 404 Not Found
		2. TransactionUsername not determined for User => 403 Forbidden
		3. User does not have write access to the cluster => 401 Unauthorized
		4. Factory deleted => 200 OK
		5. FactoryID not found => 204 No Content

		DECLARE  @RC		INT
				,@Username	NVARCHAR(255) = 'SQL'
				,@FactoryID	NVARCHAR(255) = 'Test';

		EXECUTE @RC = dbo.sx_pf_DELETE_Factory 
				@Username,
				@FactoryID;

		PRINT @RC;

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Factory','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Factory
	 @Username	NVARCHAR(255)
	,@FactoryID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'
AS 
BEGIN

	DECLARE  @FactoryKey			INT			= 0
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username,		N'NULL')	+ N''',''' 
				+ ISNULL(@FactoryID,	N'NULL')	+ N''''
			,@EffectedRows		INT				= 0				
			,@ResultCode			INT			= 501			
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N'';			

	-- NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Factory;
		
		-- Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID; 

		IF @FactoryKey = 0
			BEGIN
				SET @ResultCode = 204;
			END
		ELSE
			BEGIN
				-- Delete Factory items from all tables
				DELETE FROM planning.tdProductLines 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdProducts 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdValueSeries 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdTime 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tfStatements 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tfValues 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgFactories
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgProductLines 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgProducts 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdFactories 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.trPreferences
				WHERE FactoryID = @FactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tFiles 
				WHERE FactoryKey = @FactoryKey;
			
				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tTabs
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- Last, because of Right materialization
				DELETE FROM system.trRights					--therefore EXECUTE AS dbo needed
				WHERE FactoryID = @FactoryID;

				SET @EffectedRows += @@ROWCOUNT;
			END;

		COMMIT TRANSACTION sx_pf_DELETE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Factory TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Factory'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes an entire Factory (including ProductLines, Products, Values)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_FactoryProperty;
GO

/*
	DELETE Operation for a FactoryProperty
	Deletes an entire Factory Property

	Procedure for documentation demo
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. PropertyID or FactoryID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the Factory => 401 Unauthorized
	4. Property deleted => 200 OK
	5. PropertyID not found => 204 No Content

		DECLARE @RC			INT
		DECLARE @Username	NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID	NVARCHAR(255) = 'ZT'
		DECLARE @PropertyID NVARCHAR(255) = 'rre'

		EXECUTE @RC = dbo.sx_pf_DELETE_FactoryProperty 
				@Username,
				@FactoryID,
				@PropertyID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_FactoryProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_FactoryProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @PropertyKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID = N'' 
			OR @PropertyID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID; 
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete		
		DELETE FROM planning.tgFactories 
		WHERE PropertyKey = @PropertyKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_FactoryProperty TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_FactoryProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes an entire Factory Property';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Format;
GO

/*
	DELETE Operation for a Format

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8
 
	Test call
	1. FormatID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the cluster => 401 Unauthorized
	4. Format deleted => 200 OK
	5. Format not found => 204 No Content

	DECLARE @RC			INT
	DECLARE @Username	NVARCHAR(255) = 'SQL'
	DECLARE @FormatID	NVARCHAR(255) = '510'
		
	EXECUTE @RC = dbo.sx_pf_DELETE_Format 
			@Username,
			@FormatID
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username, N'NULL') + N''',''' 
			+ ISNULL(@FormatID, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	 BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM planning.thFormats 
		WHERE FormatID = @FormatID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Format TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Format TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Format' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_List;
GO

/*
	DELETE Operation for a List
	Delete a List 

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. ListID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the cluster => 401 Unauthorized
	4. List deleted => 200 OK
	5. List not found => 204 No Content

		DECLARE @RC			INT
		DECLARE @Username	NVARCHAR(255) = 'SQL'
		DECLARE @ListID		NVARCHAR(255) = '510'
		
		EXECUTE @RC = dbo.sx_pf_DELETE_List 
				@Username,
				@ListID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values		
		DELETE FROM planning.thListValues 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thLists 
		WHERE ListID = @ListID;

		SET @EffectedRows += @@ROWCOUNT;
			
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_List TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_List TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_List'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a List';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
		

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ListValue;
GO

/*
	DELETE Operation for one ListValue
	Operates over Key, as the ListValue has no ID

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. ListID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the cluster => 401 Unauthorized
	4. ListValue deleted => 200 OK
	5. ListValue not found => 204 No Content

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255)	= 'SQL'
		DECLARE @ListValueKey	INT				= 18
		
		EXECUTE @RC = dbo.sx_pf_DELETE_ListValue 
				@Username,
				@ListValueKey
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ListValue',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ListValue','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ListValue
	@Username		NVARCHAR(255),
	@ListValueKey	INT
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	 IS NULL SET @Username		= N'';
	IF @ListValueKey IS NULL SET @ListValueKey	= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ListValue;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - Delete values
		DELETE FROM planning.thListValues WHERE ListValueKey = @ListValueKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ListValue TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ListValue TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ListValue'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Operates over Key, as the ListValue has no ID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListValueKey';
    SET @value      = N'ListValueKey';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Preference;
GO
/*
	DELETE Operation for a Preference

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. Empty FactoryID or empty ProductLine => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Non exists Factory / ProductLine => 404 Not Found
	5. Product not found => 204 No Content
	6. Product deleted => 200 OK

		DECLARE @RC						INT
		DECLARE @Username				NVARCHAR(255) = 'SQL'
		DECLARE @PreferenceUserName		NVARCHAR(255) = 'W8\admin'
		DECLARE @ProductLineID			NVARCHAR(255) = 'ZT'
		DECLARE @FactoryID				NVARCHAR(255) = 'U'
		DECLARE @SettingName			NVARCHAR(255) = 'Demopref'

		EXECUTE @RC = dbo.sx_pf_DELETE_Preference 
				@Username,
				@PreferenceUsername,
				@ProductLineID,
				@FactoryID,
				@SettingName

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Preference',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Preference','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255) = N'';
	DECLARE @ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@PreferenceUsername,	N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SettingName,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Preference;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername		= dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName			= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @SettingName		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		IF @SettingName = N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid value of Setting Name', 16, 10);
		END;

		IF NOT EXISTS(SELECT Username FROM system.trUser WHERE Username = @PreferenceUsername)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Preferences Users don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM planning.trPreferences
		WHERE FactoryID			= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Preference TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Preference TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Preference'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a Preference';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PreferenceUsername';
    SET @value      = N'PreferenceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	SET @level2name = N'@SettingName';
    SET @value      = N'SettingName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
 
DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Product;
GO

/*
	DELETE Operation for a Product
	Deletes a product and all Values and Attributes
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. ProductID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Product not found => 204 No Content
	5. Product deleted => 200 OK

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @ProductID		NVARCHAR(255) = '10'

		EXECUTE @RC = dbo.sx_pf_DELETE_Product 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF @ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM planning.tdValueSeries 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfStatements 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdProducts 
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tFiles
		WHERE		ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;


		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Product TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Product TO pf_PlanningFactoryUser;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Product'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes a product and all Values and Attributes';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
		

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ProductDataTableValues;
GO

/*
	DELETE Operation for all Values of the ProductDateTabele

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Testcall
			DECLARE @RC							INT
			DECLARE @Username			NVARCHAR(255) = 'SQL'
			DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
			DECLARE @ProductLineID		NVARCHAR(255) = 'U'
			DECLARE @ProductID			NVARCHAR(255) = '1'
			DECLARE @DeleteFormulaFlag	INT = 0

		EXECUTE @RC = dbo.sx_pf_DELETE_ProductDataTableValues 
			@Username,
			@FactoryID,
			@ProductLineID,
			@ProductID,
			@DeleteFormulaFlag
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductDataTableValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductDataTableValues','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductDataTableValues
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@DeleteFormulaFlag	INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username, N'NULL')								+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')								+ N''',''' 
								+ CAST(ISNULL(@DeleteFormulaFlag, N'NULL') AS NVARCHAR(10))	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION DELETE_PDTValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine ProductKey
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID	= @ProductID;

		IF @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - DELETE
		-- If also formulas, we can delete everything
		IF @DeleteFormulaFlag = 1
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ProductKey = @ProductKey;

			SET @EffectedRows = @@ROWCOUNT;
		END;
		-- If not formulas, only Input Values and update Formula Values to 0/''
		IF @DeleteFormulaFlag = 0
		BEGIN
			-- alle Input Werte komplett
			DELETE fV
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey
				AND dVS.ValueSource IN ('Input','List-Strict','Link');

			-- bei Formelbasierten Werten nur den Wert, nicht die Formel
			UPDATE fV
				SET fV.ValueInt = 0
				,fV.ValueText = ''
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey	= @ProductKey
				AND dVS.ValueSource IN ('XLS', 'XLS-Strict');

			SET @EffectedRows = @@ROWCOUNT;
		END;

		SET @ResultCode	= IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION DELETE_PDTValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT	= ERROR_STATE();
		SET @Comment				= ERROR_MESSAGE();

		ROLLBACK TRANSACTION DELETE_PDTValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ProductDataTableValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for all Values of the ProductDateTabele';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@DeleteFormulaFlag';
    SET @value      = N'DeleteFormulaFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


	

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ProductLine;
GO
/*
	DELETE Operation for a ProductLine
	Deletes an entire Product Line (Products, Values, etc.)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9

	Test call
		1. ProductLineID is empty => 404 Not Found
		2. TransactionUsername not determined for User => 403 Forbidden
		3. User does not have write access to ProductLine => 401 Unauthorized
		4. ProductLine not found => 204 No Content
		5. ProductLine deleted => 200 OK

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'V'

		EXECUTE @RC = dbo.sx_pf_DELETE_ProductLine 
				@Username,
				@FactoryID,
				@ProductLineID
		PRINT @RC

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductLine
	 @Username		NVARCHAR(255)
	,@FactoryID		NVARCHAR(255)
	,@ProductLineID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'
AS 
BEGIN

	DECLARE  @FactoryKey		INT = 0
			,@ProductLineKey	INT = 0
			
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	=  N'''' 
				+ ISNULL(@Username,			N'NULL') + N''',''' 
				+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
				+ ISNULL(@ProductLineID,	N'NULL') + N''''
			 ,@EffectedRows		INT				= 0		
			 ,@ResultCode		INT				= 501		
			 ,@TimestampCall	DATETIME		= GETUTCDATE()
			 ,@Comment			NVARCHAR(2000)	= N'';		

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLine;
		
		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF	@ProductLineID	= N'' 
			OR @FactoryID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- Factory und ProductLineKey ermitteln
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0
 		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- Delete values
		DELETE FROM planning.tdProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfStatements 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts 
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdProductLines 
		WHERE ProductLineKey = @ProductLineKey;

		DELETE FROM planning.trPreferences 
		WHERE		FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tFiles
		WHERE		ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tTabs
		WHERE		ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- last because of matrialization
		DELETE FROM system.trRights				-- this needs EXECUTE AS dbo
		WHERE 
				FactoryID		= @FactoryID
			AND ProductLineID	= @ProductLineID;
		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ProductLine TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes an entire Product Line (Products, Values, etc.)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ProductLineProperty;
GO
/*
	DELETE Operation for a ProductLineProperty
	Delete a ProductLineProperty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Property not found => 204 No Content
	5. Property deleted => 200 OK

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @PropertyID		NVARCHAR(255) = 'test'

		EXECUTE @RC = dbo.sx_pf_DELETE_ProductLineProperty 
				@Username,
				@FactoryID,
				@ProductLineID,
				@PropertyID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductLineProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductLineProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + '''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductLineProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Delete values
		DELETE FROM planning.tgProductLines 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductLineProperty TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ProductLineProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a ProductLineProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ProductProperty;
GO
/*
	DELETE Operation for a ProductProperty
	Delete a ProductProperty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Property not found => 204 No Content
	5. Property deleted => 200 OK

	DECLARE @RC				INT
	DECLARE @Username		NVARCHAR(255) = 'SQL'
	DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
	DECLARE @ProductLineID	NVARCHAR(255) = 'U'
	DECLARE @ProductID		NVARCHAR(255) = '10'
	DECLARE @PropertyID		NVARCHAR(255) = 'test'

	EXECUTE @RC = dbo.sx_pf_DELETE_ProductProperty 
			@Username,
			@FactoryID,
			@ProductLineID,
			@ProductID,
			@PropertyID
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey		INT = 0;
	DECLARE @ProductLineKey INT = 0;
	DECLARE @ProductKey		INT = 0;
	DECLARE @PropertyKey	INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@PropertyID			= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProducts WHERE ProductKey = @ProductKey AND ProductLineKey = @ProductLineKey AND FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values
		DELETE FROM planning.tgProducts 
		WHERE PropertyKey = @PropertyKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductProperty TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ProductProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a ProductProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Right;
GO
/*
	DELETE Operation for a Right
	Deletes a Right of a user, user can therefore also delete its own cluster write and lock yourself out for yourself to! 
	Must frontend intercept with security inquiry!

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. DeleteUserName is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to the cluster => 401 Unauthorized
	4. Right deleted => 200 OK
	5. Right not found => 204 No Content

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @DeleteUserName NVARCHAR(255) = 'test2'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		
		EXECUTE @RC = dbo.sx_pf_DELETE_Right 
				@Username,
				@DeleteUserName,
				@FactoryID,
				@ProductLineID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Right',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Right','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Right
	@Username			NVARCHAR(255),
	@DeleteUserName		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
    @ProductLineID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @DeleteUserKey	INT = 0;
	DECLARE @RightKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Right;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString	(@DeleteUserName);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);

		IF  @DeleteUserName		= N'' 
			OR @FactoryID		= N'' 
			OR @ProductLineID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
	
		-- STEP 1.2 - Determine keys
		SELECT @DeleteUserKey = UserKey FROM system.trUser WHERE UserName = @DeleteUserName;
		SELECT @RightKey = RightKey FROM system.trRights WHERE UserKey = @DeleteUserKey AND ProductLineID = @ProductLineID AND FactoryID = @FactoryID;
		
		IF @DeleteUserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 3 - Delete values		
		DELETE FROM system.trRights
		WHERE RightKey = @RightKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Right TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Right TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Right'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes a Right of a user, user can therefore also delete its own cluster write and lock yourself out for yourself to! ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DeleteUserName';
    SET @value      = N'DeleteUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_TimeID;
GO
/*
	DELETE Operation for a TimeID
	Deletes a TimeID

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. Empty FactoryID / ProductLineID / ProductID / TimeID = 0 => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Value of TimeID deleted => 200 OK
	5. There is none ProductID => 204 No Content

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255)	= 'SQL'
		DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
		DECLARE @ProductID		NVARCHAR(255)	= '10'
		DECLARE @TimeID			INT				= '20180515'

		EXECUTE @RC = dbo.sx_pf_DELETE_TimeID 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@TimeID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_TimeID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_TimeID','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_TimeID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_TimeID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3.1 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		-- STEP 3.2 - Delete Timeline

		DELETE FROM planning.tdTime 
		WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_TimeID TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_TimeID TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_TimeID' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for a TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Value;
GO

/*
	DELETE Operation for a Value
	Delete a Value of ValueSeries with specified TimeID

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8


	Test call
	1. All IDs are empty or TimeID is 0 => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Value not found => 204 No Content
	5. Value deleted => 200 OK

		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255)	= 'SQL'
		DECLARE @FactoryID		NVARCHAR(255)	= 'ZT'
		DECLARE @ProductLineID	NVARCHAR(255)	= 'U'
		DECLARE @ProductID		NVARCHAR(255)	= '10'
		DECLARE @ValueSerieID	NVARCHAR(255)	= 'K1'
		DECLARE @TimeID			INT				= 20180615

		EXECUTE @RC = dbo.sx_pf_DELETE_Value 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@ValueSerieID,
				@TimeID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Value',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Value','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	DECLARE @ValueKey			BIGINT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N''','
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Value;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N'' 
			OR @TimeID			= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE ProductLineID		= @ProductLineID	AND FactoryKey		= @FactoryKey;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductID			= @ProductID		AND ProductLineKey	= @ProductLineKey; 
		SELECT @ValueSerieKey	= ValueSeriesKey	FROM planning.tdValueSeries		WHERE ValueSeriesID		= @ValueSerieID		AND ProductKey		= @ProductKey; 
		SELECT @ValueKey		= ValueKey			FROM planning.tfValues			WHERE ValueSeriesKey	= @ValueSerieKey	AND TimeID			= @TimeID; 

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 3 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ValueKey = @ValueKey;
			
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Value TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Value TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Value'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Delete a Value of ValueSeries with specified TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

	

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Values;
GO
/*
	DELETE Operation for Values
	Delete all Values of Value Series of Products

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. All IDs are empty  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. Values not found => 204 No Content
	5. Values deleted => 200 OK

		DECLARE @RC						INT
				DECLARE @Username		NVARCHAR(255) = 'SQL'
				DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
				DECLARE @ProductLineID	NVARCHAR(255) = 'U'
				DECLARE @ProductID		NVARCHAR(255) = '10'
				DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

		EXECUTE @RC = dbo.sx_pf_DELETE_Values 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@ValueSerieID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Values',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Values','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProductKey			INT = 0;
	DECLARE @ValueSerieKey		INT = 0;
	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Values;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ValueSeriesID = @ValueSerieID AND ProductKey = @ProductKey;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @ValueSerieKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		DELETE FROM planning.tfValues 
		WHERE ValueSeriesKey = @ValueSerieKey;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = IIF (@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION sx_pf_DELETE_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Values TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_Values TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Values'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Delete all Values of Value Series of Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ValueSerie;
GO
/*
	DELETE Operation for one ValueSerie
	Deletes a ValueSerie and changes the Numbers of the other ValueSeries. If Series No 1 is delete, Procedure creates Placeholder in a new ValueSeries 1.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Test call
	1. ValueSerieID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User does not have write access to ProductLine => 401 Unauthorized
	4. ValueSerie not found => 204 No Content
	5. ValueSerie deleted => 200 OK

		DECLARE @RC						INT
				DECLARE @Username		NVARCHAR(255) = 'SQL'
				DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
				DECLARE @ProductLineID	NVARCHAR(255) = 'U'
				DECLARE @ProductID		NVARCHAR(255) = '10'
				DECLARE @ValueSerieID	NVARCHAR(255) = 'K1'

		EXECUTE @RC = dbo.sx_pf_DELETE_ValueSerie 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@ValueSerieID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ValueSerie',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey			INT				= 0;
	DECLARE @ProductKey				INT				= 0;
	DECLARE @DelValueSerieKey		INT				= 0;
	DECLARE @DelValueSerieNo		INT				= 0;
	DECLARE @NewFirstValueSerieKey	INT				= 0;
	DECLARE @NewFirstValueSerieNo	INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_ValueSerie;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
	
		IF	@ValueSerieID		= N'' 
			OR @ProductID		= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername(@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				 
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductID = @ProductID AND ProductLineKey = @ProductLineKey;
		SELECT @DelValueSerieKey = ValueSeriesKey, @DelValueSerieNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - Delete values
		IF @DelValueSerieKey = 0
		BEGIN
			SET @ResultCode = 204;
		END 
		ELSE
		BEGIN				
			-- STEP 3.2 - Delete ValueSerie 				
			DELETE FROM planning.tfValues 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			DELETE FROM planning.tdValueSeries 
			WHERE ValueSeriesKey = @DelValueSerieKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.3 - Decrement all ValueSeriesNo which are bigger than the deleted on
			UPDATE planning.tdValueSeries
			SET ValueSeriesNo = ValueSeriesNo - 1
			FROM planning.tdValueSeries
			WHERE 
				ProductKey = @ProductKey
				AND ValueSeriesNo > @DelValueSerieNo;

			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.4 - if the new first ValueSeries is not NO 1 (1,3..... Numbers), set it to 1
			IF @NewFirstValueSerieNo - 1 > 1 
			BEGIN
				UPDATE planning.tdValueSeries 
				SET	ValueSeriesNo = 1
				FROM planning.tdValueSeries
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesKey = @NewFirstValueSerieKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;
		
		COMMIT TRANSACTION sx_pf_DELETE_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ValueSerie TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ValueSerie'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes a ValueSerie and changes the Numbers of the other ValueSeries. If Series No 1 is delete, Procedure creates Placeholder in a new ValueSeries 1.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS dbo.sx_pf_EXPORT_Factory;
GO

/* 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	EXPORT operation of a single Factory, including
	- its properties in gtables
	- its global attributes
	- its ProductLine
	- its products
	- it values
	- its dependend lists, which have to be merged in the existing lists during import
	- in two styles, one for import as sql script, one for Import over the GUI Import

	The Parameter @Output for SQL Server exists only for backwards compatibilty - its no longer used

	Test Call direct (for small Factories)
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_EXPORT_Factory
			 @Username				= N'SQL'
			,@FactoryID				= N'ZT';
		PRINT @RC;

	Test Call over BCP
		Results must be stored in table, if column width is to big for SSMS or SQLCMD

		0a. Activate Stepnumber for output in Line 624 around and execute prodecedure
		0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

		1. Execute this to store the results in the table
		
			DROP TABLE IF EXISTS dbo.tmpout_Export;
			CREATE TABLE dbo.tmpout_Export (
				StepNumber	INT,			--enable Stepnumber inside for testing
				Command		NVARCHAR(MAX)
				);
			INSERT INTO dbo.tmpout_Export
				EXEC dbo.sx_pf_EXPORT_Factory N'SQL',N'ZT',1;
			SELECT * FROM  dbo.tmpout_Export ORDER BY StepNumber;

		2. Copy this table Content in a file over a .bat file with this command
			bcp.exe "SELECT Command FROM PlanningFactoryCars.dbo.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 

	SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_EXPORT_Factory
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@OutputForSQLServer INT				= 1
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @TransactUsername	NVARCHAR(255)
			,@ProcedureName		NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR (MAX) = N'''' 
					+ ISNULL(@Username, N'NULL')									+ N''',''' 
					+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
					+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N''''
			,@EffectedRows		INTEGER = 0						
			,@ResultCode		INTEGER = 501					
			,@TimestampCall		DATETIME = GETUTCDATE()
			,@Comment			NVARCHAR (2000) = N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @OutputForSQLServer	IS NULL SET @OutputForSQLServer = 1;


	BEGIN TRY

		DECLARE @FactoryKey BIGINT = 0;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID(@FactoryID);

		IF @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR(N'Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Keys don`t exists', 16, 10);
		END;	
	
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR(N'Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE FactoryKey = @FactoryKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource2		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource3		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource4		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource5		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource6		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource7		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource8		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource9		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource10	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource11	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource12	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource13	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource14	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource15	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource16	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource17	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource18	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource19	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource20	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource21	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource22	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource23	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource24	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
			UNION ALL SELECT GlobalAttributeSource25	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX));
		
		INSERT INTO @ExportTable 
		VALUES 
			 (0,1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			,(0,1, N'DECLARE @Username NVARCHAR(255)		= ''SQL''')
			,(0,1, N'DECLARE @FactoryID NVARCHAR(255)		= ''' + @FactoryID + N'''')
			,(0,1, N'--This FactoryID will be deleted during import, if it exists. You should be sure !');

		-- DELETE FactoryID if exists
		INSERT INTO @ExportTable 
		VALUES (1000,1, N'EXEC dbo.sx_pf_DELETE_Factory @Username, @FactoryID;');
			
		-- POST Factory
		PRINT N'Starting Export Factory';
		INSERT INTO @ExportTable
			SELECT 1001,1, N'EXEC dbo.sx_pf_POST_Factory @Username, @FactoryID,N'''
				+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',N''' 
				+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
				+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',N''' 
				+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''';'
				FROM planning.tdFactories dF WHERE dF.FactoryKey = @FactoryKey

		-- POST ProductLines
		PRINT N'Starting Export Produclines';
		INSERT INTO @ExportTable 
		SELECT 1002,1, N'EXEC dbo.sx_pf_POST_ProductLine @Username,''' + ProductLineID + N''', @FactoryID,N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(DefaultTemplate)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias1)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias2)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias3)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias4)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias5)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias6)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias7)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias8)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias9)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource10)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias10)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource13)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias13)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource14)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias14)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource15)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias15)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource16)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias16)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource17)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias17)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource18)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias18)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource19)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias19)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource20)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias20)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource23)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias23)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource24)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias24)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource25)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias25)	+ N''';'
		FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey;
					
		-- POST Products
		PRINT N'Starting Export Products'
		INSERT INTO @ExportTable
		SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,N''' 
			+ ProductID								+ N''',N''' 
			+ ProductLineID							+ N''',@FactoryID,N''' 
			+ TimeType								+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(
				calc.sfUnifyLinebreaks(
						CommentUser
							)
						)							+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL([Status])			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(Template)			+ N''',N''' 
			+ TemplateVersion						+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute10) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute11) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute12) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute13) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute14) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute15) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute16) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute17) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute18) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute19) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute20) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute21) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute22) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute23) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute24) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute25) + N''';'
		FROM planning.tdProducts WHERE FactoryKey = @FactoryKey;
					
		-- POST Statements 
			--> statements are not exported

		-- POST all ValueSeries as NewValueSeries
		PRINT N'Starting Export ValueSeries';
		INSERT INTO @ExportTable 
		SELECT 3000
			,ROW_NUMBER()OVER (ORDER BY FactoryID,ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
			,N'EXEC dbo.sx_pf_POST_ValueSerie @Username,N''' 
			+ ProductID									+ N''',N''' 
			+ ProductLineID								+ N''',@FactoryID,N'''
			+ ValueSeriesID								+ N''',N'''
			+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ImageName)				+ N''',N'''
			+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''',N'''
			+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''',N'''
			+ ValueSource								+ N''',N'''
			+ ValueListID								+ N''',N'''
			+ ValueFormatID								+ N''',N'''
			+ Unit										+ N''',N'''
			+ CAST(Scale AS NVARCHAR (255))				+ N''',N'''
			+ Effect									+ N''',N'''
			+ EffectParameter							+ N''';'
		FROM planning.tdValueSeries

		WHERE FactoryKey = @FactoryKey
			
		ORDER BY 
			 FactoryID
			,ProductLineID
			,ProductID
			,ValueSeriesNo;

		-- POST dTime 
		PRINT N'Starting Export Time';
		;WITH tms AS (
			SELECT dP.ProductID,dP.ProductLineID
				,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + N',0,0,''''' + FormatID + ''''')],' -- with DeleteFlag, AddValue,FormatID 
				FROM planning.tdTime t
				WHERE t.ProductKey = dP.ProductKey
				ORDER BY t.TimeID
				FOR XML PATH(N''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.FactoryKey = @FactoryKey 
		)
		INSERT INTO @ExportTable
		SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, N''' + ProductID + N''',''' + ProductLineID + N''', @FactoryID,1,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST ProductDataTableValues
		PRINT N'Starting Export PDT'
		;WITH tms AS (
			SELECT dP.ProductID,dP.ProductLineID
				,(SELECT N'[(''''' 
						+ ValueSeriesID							+ N''''',' 
						+ CAST(TimeID AS NVARCHAR(255))			+ N',''''' 
						+ ValueFormula 							+ N''''','''''
						+ CAST(ValueInt AS NVARCHAR(255)) 		+ N''''',''''' 
						+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''''',''''' 
						+ dbo.sx_pf_pMaskSQL(calc.sfUnifyLinebreaks(ValueComment))	+ N''''')],' 
				FROM planning.tfValues t
				WHERE t.ProductKey = dP.ProductKey
				FOR XML PATH(N''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.FactoryKey = @FactoryKey 
		)
		INSERT INTO @ExportTable
		SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,N''' + ProductID + N''',''' + ProductLineID + N''', @FactoryID,0,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST Factory Properties
		PRINT N'Starting Export FactoryPoperties';
		INSERT INTO @ExportTable 
		SELECT 6000,1, N'EXEC dbo.sx_pf_POST_FactoryProperty @Username, @FactoryID,N''' 
				+ PropertyID									+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(PropertyName)				+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N'''
				+ Unit											+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''',N'''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
				+ CAST(Scale				AS NVARCHAR(255))	+ N''',N'''
				+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''',N'''
				+ FormatID										+ N''';'
		FROM planning.tgFactories
		WHERE FactoryKey = @FactoryKey; 

		-- POST ProductLine Properties
		PRINT N'Starting Export Productline Properties';
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductLineProperty @Username,N''' 
				+ ProductLineID									+ N''',@FactoryID,N''' 
				+ PropertyID									+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(PropertyName)				+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N'''
				+ Unit											+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''',N'''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
				+ CAST(Scale				AS NVARCHAR(255))	+ N''',N'''
				+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''',N'''
				+ FormatID										+ N''';'
		FROM planning.tgProductLines
		WHERE FactoryKey = @FactoryKey; 

		-- POST ProductLine Preferences
		PRINT N'Starting Export Productline Preferences';
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_Preference @Username,N''' 		
				+ Username										+ N''',N'''
				+ ProductLineID									+ N''', @FactoryID,N''' 
				+ SettingName									+ N''',N'''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''';'
		FROM planning.trPreferences
		WHERE FactoryID = @FactoryID; 

		-- POST Product Properties
		PRINT N'Starting Export Product Properties';
		INSERT INTO @ExportTable 
		SELECT 6002,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,N''' 
			+ ProductID										+ N''',N''' 
			+ ProductLineID									+ N''',@FactoryID,N''' 
			+ PropertyID									+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PropertyName)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N'''
			+ Unit											+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''',N'''
			+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(Scale				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(IsROSystemProperty	AS NVARCHAR(255))	+ N''',N'''
			+ FormatID										+ N''';'
		FROM planning.tgProducts
		WHERE  FactoryKey = @FactoryKey; 

		-- POST Tabs
		PRINT N'Starting Export Tabs'
		INSERT INTO @ExportTable 
		SELECT 6003
			,Row_Number() OVER (ORDER BY FactoryID, ProductLineID, ProductID, Orderindex) AS Row
			,N'EXEC planning.spPOST_Tab @Username, @FactoryID, ''' 
			+ ProductLineID									+ N''',''' 
			+ ProductID										+ N''',''' 
			+ TabID											+ N''', '
			+ CAST(Orderindex AS NVARCHAR(255))				+ N',N'''
			+ dbo.sx_pf_pMaskSQL(TabName)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabHint)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabText)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PresentationCODE)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(DataSource)				+ N''',N'''
			+ CAST(IsVisibleBOOL AS NVARCHAR(255))			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(LayoutJSON)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ParameterJSON)				+ N'''; '

		FROM planning.tTabs
		WHERE  FactoryKey = @FactoryKey
		
		-- Create the lists
		PRINT N'Starting Export Lists'
		INSERT INTO @ExportTable 
		SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,N'''
			+ ListID							+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)	+ N''',N''' 
			+ Datentyp							+ N''',N''' 
			+ Source							+ N''',N''' 
			+ SourceFormula						+ N''',N''' 
			+ FormatID							+ N''';'
		FROM planning.thLists
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
		-- Create List Values
		PRINT N'Starting Export List Values';
		INSERT INTO @ExportTable 
		SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,N'''
			+ ListID								+ N''',' 
			+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
			+ CAST(Scale AS NVARCHAR(255))			+ N',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueComment)		+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thListValues
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
		-- Create Formats
		PRINT N'Starting Export Formats';
		;WITH DependendFormats AS (
			-- used in depended lists from global attributes
			SELECT FormatID
			FROM planning.thListValues 
			WHERE ListID IN
				(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
			UNION ALL
			-- used to format ValueSeries 
			SELECT DISTINCT ValueFormatID 
			FROM planning.tdValueSeries 
			WHERE FactoryKey = @FactoryKey
			UNION ALL
			-- used to format Ranges
			SELECT DISTINCT FormatID
			FROM planning.tgProducts 
			WHERE 
					FactoryKey = @FactoryKey
				AND PropertyID LIKE N'PDTRange_%'
		) 
		INSERT INTO @ExportTable 
		SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,N'''
			+ FormatID										+ N''',N'''
			+ BackgroundColor								+ N''',N''' 
			+ FontColor										+ N''',N''' 
			+ ValueFormat									+ N''',N'''
			+ CAST(Bold				AS NVARCHAR(255))		+ N''',N'''
			+ CAST(Italic			AS NVARCHAR(255))		+ N''',N'''
			+ AlignmentHorizontal							+ N''',N'''
			+ AlignmentVertical								+ N''',N'''
			+ CAST(TextUnderlined	AS NVARCHAR(255))		+ N''',N'''
			+ CAST(BorderToNext		AS NVARCHAR(255))		+ N''',N'''
			+ CAST(BorderToPrevious AS NVARCHAR(255))		+ N''';'
		FROM planning.thFormats
		WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
		PRINT N'Finishing Script';
		INSERT INTO @ExportTable
		VALUES (8000,1, N'GO');

		-- Final Value Output ###################################################################################################################
		PRINT N'Starting Output';
		SELECT 
			--StepNumber, --activate only for bcp test output
			 Command 
		FROM @ExportTable 
		ORDER BY 
			 StepNumber
			,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT N'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_EXPORT_Factory TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_EXPORT_Factory'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'EXPORT operation of a single Factory, including, properties in gtables, global attributes, ProductLine, products, values, dependend lists which have to be merged in the existing lists during import in two styles. One for import as sql script, one for Import over the GUI Import';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@OutputForSQLServer';
    SET @value      = N'OutputForSQLServer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_EXPORT_Product;
GO

/* Procedure to export one single product, including
	- it values
	- its dependend lists, which have to be merged in the existing lists during import
	- in two styles, one for import as sql script, one for Import over the GUI Import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
 
	TestCall
		DECLARE  @RC int
				,@Username			NVARCHAR(255) = 'SQL'
				,@FactoryID			NVARCHAR(255) = 'ZT'
				,@ProductLineID		NVARCHAR(255) = 'U'
				,@ProductID			NVARCHAR(255) = '1';

		EXECUTE @RC = dbo.sx_pf_EXPORT_Product
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		  ,@ProductID
		 PRINT @RC

	 Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Product','PARAMETER',NULL)
 */

CREATE  PROCEDURE dbo.sx_pf_EXPORT_Product
		@Username			NVARCHAR(255),
		@FactoryID			NVARCHAR(255),
		@ProductLineID		NVARCHAR(255),
		@ProductID			NVARCHAR(255),
		@OutputForSQLServer INT				= 1
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @TransactUsername	NVARCHAR(255)
			,@ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username			,N'NULL')							+ N''',''' 
								+ ISNULL(@FactoryID			,N'NULL')							+ N''',''' 
								+ ISNULL(@ProductLineID		,N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID			,N'NULL')							+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N''''
			,@EffectedRows		INTEGER = 0	
			,@ResultCode 		INTEGER = 501
			,@TimestampCall		DATETIME = GETUTCDATE()
			,@Comment			NVARCHAR (2000) = N'';		
	
	-- NULL Protection
	IF @Username			IS NULL SET @Username		= N'';
	IF @FactoryID			IS NULL SET @FactoryID		= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID	= N'';
	IF @ProductID			IS NULL SET @ProductID		= N'';

	BEGIN TRY

		DECLARE @ProductKey		BIGINT = 0;
		DECLARE @ProductLineKey BIGINT = 0;
		DECLARE @Template		NVARCHAR (255) = N'';

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);

		IF @FactoryID = N'' OR @ProductLineID = N''  OR @ProductID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR(N'Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductLineKey = ProductLineKey		FROM planning.tdProductLines	 WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey				FROM planning.tdProducts		 WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Keys don`t exists', 16, 10);
		END;	
	 
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR(N'Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine Template
		SELECT @Template = Template FROM planning.tdProducts WHERE ProductKey = @ProductKey;

		-- Deterime all ProductLines using this Template
		DECLARE @DependendProductLinesIDs AS TABLE (ProductLineKey BIGINT);
		INSERT INTO @DependendProductLinesIDs (ProductLineKey)
			SELECT DISTINCT ProductLineKey FROM planning.tdProducts WHERE Template = @Template;

		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductKey = @ProductKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT DISTINCT GlobalAttributeSource1	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource2	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource3	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource4	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource5	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource6	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource7	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource8	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource9	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
		)									
		INSERT INTO @DependendListIDs (ListID)
			SELECT DISTINCT ValueListID
			FROM lists
			WHERE ValueListID != '';
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
											 StepNumber INT NOT NULL
											,RowNumber	INT NOT NULL
											,Command	NVARCHAR(MAX));
		
		INSERT INTO @ExportTable 
		VALUES 
			 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			,(0, 1, N'DECLARE @Username			NVARCHAR(255)	= N''SQL''')
			,(0, 1, N'DECLARE @FactoryID		NVARCHAR(255)	= N''' + @FactoryID		+ N'''')
			,(0, 1, N'DECLARE @ProductLineID	NVARCHAR(255)	= N''' + @ProductLineID	+ N'''')
			,(0, 1, N'DECLARE @ProductID		NVARCHAR(255)	= N''' + @ProductID		+ N'''')
			,(0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

		-- DELETE Product if exists
		INSERT INTO @ExportTable 
		VALUES (1000, 1, N'EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;');
			
		-- POST Product
		INSERT INTO @ExportTable 
		SELECT 2000, 1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,N''' 
			+ TimeType									+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameLong)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (
					calc.sfUnifyLinebreaks
						(
							CommentUser
						)			
								)						+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ResponsiblePerson)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ImageName)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL ([Status])				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (Template)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (TemplateVersion)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute1)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute2)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute3)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute4)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute5)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute6)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute7)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute8)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute9)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute10)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute13)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute14)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute15)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute16)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute17)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute18)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute19)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute20)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute23)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute24)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute25)	+ N''';'
		FROM planning.tdProducts WHERE ProductKey = @ProductKey;

		-- POST Product Properties
		INSERT INTO @ExportTable 
		SELECT 3000, 1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username, @ProductID, @ProductLineID, @FactoryID,N''' +
			+ PropertyID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (PropertyName)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (Unit)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (ValueText)			+ N''',N'''
			+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
			+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
			+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
			+ FormatID									+ N''';'
		FROM planning.tgProducts
		WHERE  ProductKey = @ProductKey; 

		-- POST Statements -- statements are not exported

		-- POST all ValueSeries as NewValueSeries - N in Parameterlist  needed
		INSERT INTO @ExportTable
		SELECT 4000 
			, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
			, N'EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,N''' +
			+ ValueSeriesID									+ N''',N''' 
			+ CAST(ValueSeriesNo	AS NVARCHAR (255))		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameShort)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameLong)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentUser)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ImageName)				+ N''',N''' 
			+ CAST([IsNumeric]		AS NVARCHAR (255))		+ N''',N''' 
			+ CAST(VisibilityLevel	AS NVARCHAR (255))		+ N''',N''' 
			+ ValueSource									+ N''',N''' 
			+ ValueListID									+ N''',N''' 
			+ ValueFormatID									+ N''',N''' 
			+ Unit											+ N''',N''' 
			+ CAST(Scale AS NVARCHAR (255))					+ N''',N''' 
			+ Effect										+ N''',N''' 
			+ EffectParameter								+ N''';'
		FROM planning.tdValueSeries
		WHERE ProductKey = @ProductKey
		ORDER BY
				ProductID
			,ValueSeriesNo; 
		   
		;WITH tms AS (
			SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + N',0,0,''''' + FormatID + N''''')],'
			FROM planning.tdTime
			WHERE  ProductKey = @ProductKey
			ORDER BY TimeID
			FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS val 
		)
		INSERT INTO @ExportTable
		SELECT 5000, 1, N'EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,N''' + LEFT(val, LEN(val) - 1) + N''';'
		FROM tms
		WHERE NOT val IS NULL;
				  
		-- POST ProductDataTableValues - no N in Array needed !
		;WITH vals AS (
			SELECT (SELECT  N'[(''''' 
				+ fV.ValueSeriesID												+ N''''','  
				+ CAST(fV.TimeID	AS NVARCHAR(255))							+ N',''''' 
				+ fV.ValueFormula												+ N''''',''''' 
				+ CAST(fV.ValueInt	AS NVARCHAR(255))							+ N''''',''''' 
				+ dbo.sx_pf_pMaskSQL (fV.ValueText)								+ N''''',''''' 
				+ dbo.sx_pf_pMaskSQL (calc.sfUnifyLinebreaks(fV.ValueComment))	+ N''''')],'
			FROM planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE fV.ProductKey = @ProductKey 
			ORDER BY dVS.ValueSeriesNo
			FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS val 
		)
		INSERT INTO @ExportTable
		SELECT 6000, 1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,N'''
				+ LEFT(val, LEN(val) - 1) + N''';' 
		FROM vals
		WHERE NOT val IS NULL;		

		-- Create the lists
		INSERT INTO @ExportTable 
		SELECT 7001, 1, N'EXECUTE dbo.sx_pf_POST_List @Username,N'''
			+ ListID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameShort)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentUser)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)		+ N''',N''' 
			+ Datentyp								+ N''',N''' 
			+ Source								+ N''',N''' 
			+ SourceFormula							+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thLists
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

		-- Create List Values
		INSERT INTO @ExportTable 
		SELECT 7002, 1, N'EXECUTE dbo.sx_pf_POST_ListValue @Username,0,N'''
			+ ListID								+ N''',' 
			+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
			+ CAST(Scale AS NVARCHAR(255))			+ N',N''' 
			+ dbo.sx_pf_pMaskSQL (ValueText)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ValueComment)		+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thListValues
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
			
		;WITH DependendFormats AS (
			-- used in depended lists from global attributes
			SELECT FormatID
			FROM planning.thListValues 
			WHERE ListID IN
				(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
			UNION ALL
			-- used to format ValueSeries 
			SELECT ValueFormatID 
			FROM planning.tdValueSeries 
			WHERE ProductKey = @ProductKey
			UNION ALL
			-- used to format Ranges
			SELECT DISTINCT FormatID
			FROM planning.tgProducts 
			WHERE 
					ProductKey = @ProductKey
				AND PropertyID LIKE N'PDTRange_%'
		) 
		INSERT INTO @ExportTable 
		SELECT 8001, 1, N'EXEC dbo.sx_pf_POST_Format @Username,N'''
			+ FormatID									+ N''',N'''
			+ BackgroundColor							+ N''',N''' 
			+ FontColor									+ N''',N''' 
			+ ValueFormat								+ N''',N''' 
			+ CAST(Bold				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(Italic			AS NVARCHAR(255))	+ N''',N'''
			+ AlignmentHorizontal						+ N''',N'''
			+ AlignmentVertical							+ N''',N'''
			+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToPrevious	AS NVARCHAR(255))	+ N''';'
		FROM planning.thFormats
		WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
		INSERT INTO @ExportTable
		VALUES (9000, 1, N'GO');

		-- Final Value Output ###################################################################################################################
		SELECT Command 
		FROM @ExportTable 
		ORDER BY StepNumber, RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT N'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_EXPORT_Product TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_EXPORT_Product'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to export one single product, including values, dependend lists, which have to be merged in the existing lists during import in two styles: one for import as sql script, one for Import over the GUI Import';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@OutputForSQLServer';
    SET @value      = N'OutputForSQLServer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_EXPORT_ProductLine;
GO

/* 
	EXPORT operation of a single productline, including
	- its properties in gtables
	- its global attributes
	- its products
	- it values
	- its dependend lists, which have to be merged in the existing lists during import
	- in two styles, one for import as sql script, one for Import over the GUI Import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	Testcall Procedure direct
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_EXPORT_ProductLine
			 @Username				= N'SQL'
			,@FactoryID				= N'ZT'
			,@ProductLineID			= N'1';
		PRINT @RC;

	Testcall over BCP
		Results must be stored in table, if column width is to big for SSMS or SQLCMD

		0a. Activate Stepnumber for output in Line 753 around and execute prodecedure
		0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

		1. Execute this to store the results in the table
			IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'tmpout_Export') AND type in (N'U'))
			DROP TABLE tmpout_Export
			GO
			CREATE TABLE tmpout_Export (
				StepNumber INT,
				Command NVARCHAR(MAX)
				)
			INSERT INTO tmpout_Export
				EXEC dbo.sx_pf_EXPORT_ProductLine 'SQL','ZT','U',1

		2. Copy this table Content in a file over a .bat file with this command
			bcp.exe "SELECT Command FROM PlanningFactoryCars.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_EXPORT_ProductLine
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@OutputForSQLServer INT			= 1
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @TransactUsername	AS NVARCHAR(255)
			,@ProcedureName		AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	AS NVARCHAR (MAX) = N'''' 
							+ ISNULL(@Username		, N'NULL')								+ N''',''' 
							+ ISNULL(@FactoryID		, N'NULL')								+ N''',''' 
							+ ISNULL(@ProductLineID	, N'NULL')								+ N''',''' 
							+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N''''
			,@EffectedRows		AS INTEGER = 0			
			,@ResultCode		AS INTEGER = 501			
			,@TimestampCall		AS DATETIME = GETUTCDATE()
			,@Comment			AS NVARCHAR (2000) = N''	
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username		= N'';
	IF @FactoryID			IS NULL SET @FactoryID		= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID	= N'';

	BEGIN TRY

		DECLARE @ProductLineKey BIGINT = 0;

		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID(@ProductLineID);

		IF @FactoryID = N'' OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR(N'Transaction user don`t exists', 16, 10);
		END;		

		-- Determine keys
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

		IF @ProductLineKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Keys don`t exists', 16, 10);
		END;	
	
		-- Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR(N'Invalid rights', 16, 10);
		END;

		-- Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductLineKey = @ProductLineKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource2		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource3		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource4		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource5		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource6		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource7		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource8		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource9		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (	 
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX)
										);
	
		INSERT INTO @ExportTable 
		VALUES 
			 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			,(0, 1, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
			,(0, 1, N'DECLARE @FactoryID NVARCHAR(255)			= ''' + @FactoryID		+ N'''')
			,(0, 1, N'DECLARE @ProductLineID  NVARCHAR(255)		= ''' + @ProductLineID	+ N'''')
			,(0, 1, N'--This ProductLineID will be deleted during import, if it exists. You should be sure !');

		-- DELETE ProductLine if exists
		INSERT INTO @ExportTable 
		VALUES (1000,1, N'EXEC dbo.sx_pf_DELETE_ProductLine @Username, @FactoryID, @ProductLineID;');
				
		-- POST ProductLine
		INSERT INTO @ExportTable 
		SELECT 1001,1, N'EXEC dbo.sx_pf_POST_ProductLine @Username, @ProductLineID, @FactoryID,N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(DefaultTemplate)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias1)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias2)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias3)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias4)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias5)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias6)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias7)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias8)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias9)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource10)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias10)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource13)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias13)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource14)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias14)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource15)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias15)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource16)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias16)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource17)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias17)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource18)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias18)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource19)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias19)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource20)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias20)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource23)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias23)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource24)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias24)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource25)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias25)	+ N''';'
		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey;
					
		-- POST Products
		INSERT INTO @ExportTable
		SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,N''' 
			+ TimeType								+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(
						calc.sfUnifyLinebreaks
							(
								CommentUser
							)
						)							+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL([Status])			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(Template)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(TemplateVersion)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute10) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute11) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute12) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute13) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute14) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute15) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute16) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute17) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute18) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute19) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute20) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute21) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute22) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute23) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute24) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute25) + N''';'
		FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey;
							
		-- POST Statements 
			--> statements are not exported

		-- POST all ValueSeries as NewValueSeries
		INSERT INTO @ExportTable 
		SELECT 3000 AS Step
			, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
			, N'EXEC dbo.sx_pf_POST_ValueSerie @Username,N''' 
			+ ProductID									+ N''',@ProductLineID, @FactoryID,N''' 
			+ ValueSeriesID								+ N''',N'''
			+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ImageName)				+ N''',N'''
			+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''',N'''
			+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''',N'''
			+ ValueSource								+ N''',N'''
			+ ValueListID								+ N''',N'''
			+ ValueFormatID								+ N''',N'''
			+ Unit										+ N''',N'''
			+ CAST(Scale			AS NVARCHAR (255))	+ N''',N'''
			+ Effect									+ N''',N'''
			+ EffectParameter							+ N''';'
		FROM planning.tdValueSeries
		WHERE  ProductLineKey = @ProductLineKey
		ORDER BY 
				ProductLineID
			,ProductID
			,ValueSeriesNo;
				
		-- POST dTime 
		;WITH tms AS (
			SELECT dP.ProductID
				,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + N',0,0,'''''+ FormatID + N''''')],' -- with DeleteFlag, AddValue,FormatID 
				FROM planning.tdTime t
				WHERE t.ProductKey = dP.ProductKey
				ORDER BY t.TimeID
				FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.ProductLineKey = @ProductLineKey 
		)
		INSERT INTO @ExportTable
		SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, N''' + ProductID + N''', @ProductLineID, @FactoryID,1,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST ProductDataTableValues - no N in Array needed
		;WITH tms AS (
			SELECT dP.ProductID
				,(SELECT N'[('''''		+ ValueSeriesID 
						+ N''''','		+ CAST(TimeID AS NVARCHAR(255)) 
						+ N','''''		+ ValueFormula 
						+ N''''','''''	+ CAST(ValueInt AS NVARCHAR(255)) 
						+ N''''','''''	+ dbo.sx_pf_pMaskSQL(ValueText) 
						+ N''''','''''	+ dbo.sx_pf_pMaskSQL(calc.sfUnifyLinebreaks(ValueComment)) + N''''')],' 
				FROM planning.tfValues t
				WHERE t.ProductKey = dP.ProductKey
				FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.ProductLineKey = @ProductLineKey 
		)
		INSERT INTO @ExportTable
		SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,N''' + ProductID + N''', @ProductLineID, @FactoryID,0,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST ProductLine Properties
		INSERT INTO @ExportTable 
		SELECT 6000,1, N'EXEC dbo.sx_pf_POST_ProductLineProperty @Username, @ProductLineID, @FactoryID,N''' 
				+ PropertyID								+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
				+ Unit										+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''',N'''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
				+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
				+ FormatID									+ N''';'
		FROM planning.tgProductLines
		WHERE ProductLineKey = @ProductLineKey; 

		-- POST ProductLine Preferences
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_Preference @Username,N'''
				+ UserName
				+ N''',@ProductLineID, @FactoryID,N''' 		
				+ SettingName									+ N''',N'''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''';'
		FROM planning.trPreferences
		WHERE 
				FactoryID		= @FactoryID
			AND ProductLineID	= @ProductLineID; 

		-- POST Tabs
		INSERT INTO @ExportTable 
		SELECT 6003
			,Row_Number() OVER (ORDER BY ProductLineID, ProductID, Orderindex) AS Row
			,N'EXEC planning.spPOST_Tab @Username, @FactoryID, @ProductLineID, N''' 
			+ ProductID										+ N''',''' 
			+ TabID											+ N''', '
			+ CAST(Orderindex AS NVARCHAR(255))				+ N',N'''
			+ dbo.sx_pf_pMaskSQL(TabName)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabHint)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabText)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PresentationCODE)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(DataSource)				+ N''',N'''
			+ CAST(IsVisibleBOOL AS NVARCHAR(255))			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(LayoutJSON)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ParameterJSON)				+ N'''; '

		FROM planning.tTabs
		WHERE ProductLineKey = @ProductLineKey; 

		-- POST Product Properties
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,N''' 
			+ ProductID									+ N''',@ProductLineID, @FactoryID,N''' 
			+ PropertyID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
			+ Unit										+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''',N'''
			+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
			+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
			+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
			+ FormatID									+ N''';'
		FROM planning.tgProducts
		WHERE  ProductLineKey = @ProductLineKey; 
						
		-- Create the lists
		INSERT INTO @ExportTable 
		SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,N'''
			+ ListID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
			+ Datentyp								+ N''',N''' 
			+ Source								+ N''',N''' 
			+ SourceFormula							+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thLists
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
		-- Create List Values
		INSERT INTO @ExportTable 
		SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,N'''
			+ ListID								+ N''',' 
			+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
			+ CAST(Scale AS NVARCHAR(255))			+ N',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueComment)		+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thListValues
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
		-- Create Formats
		;WITH DependendFormats AS (
			-- used in depended lists from global attributes
			SELECT FormatID
			FROM planning.thListValues 
			WHERE ListID IN
				(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
			UNION ALL
			-- used to format ValueSeries 
			SELECT DISTINCT ValueFormatID 
			FROM planning.tdValueSeries 
			WHERE ProductLineKey = @ProductLineKey
			UNION ALL
			-- used to format Ranges
			SELECT DISTINCT FormatID
			FROM planning.tgProducts 
			WHERE 
					ProductLineKey = @ProductLineKey
				AND PropertyID LIKE N'PDTRange_%'
		) 

		INSERT INTO @ExportTable 
		SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,N'''
			+ FormatID									+ N''',N'''
			+ BackgroundColor							+ N''',N''' 
			+ FontColor									+ N''',N''' 
			+ ValueFormat								+ N''',N''' 
			+ CAST(Bold				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(Italic			AS NVARCHAR(255))	+ N''',N'''
			+ AlignmentHorizontal						+ N''',N'''
			+ AlignmentVertical							+ N''',N'''
			+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToPrevious AS NVARCHAR(255))	+ N''';'
		FROM planning.thFormats
		WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
		INSERT INTO @ExportTable
		VALUES (8000,1, N'GO');

		-- Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY StepNumber,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_EXPORT_ProductLine TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_EXPORT_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'EXPORT operation of a single productline, including,properties in gtables, global attributes, products, values, dependend lists, 
	which have to be merged in the existing lists during import in two styles: one for import as sql script, one for Import over the GUI Import';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@OutputForSQLServer';
    SET @value      = N'OutputForSQLServer - no longer in use.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE  IF EXISTS dbo.sx_pf_GET_AllProductListValues;
GO

/*
	GET Operation for all ListValues a product needs for data entry in ProductDataTable
	- contains the GUI defined lists of the products (defined by user c* or solution su*)
	- contains the dynamic system lists (sx* - which are editable by the user)
	- adds interal system lists (sx* - which are not visible on GUI)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call - Must return Table and Code 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_AllProductListValues 
			@Username		 = 'SQL',
			@FactoryID		 = 'ZT',
			@ProductLineID	 = 'U',
			@ProductID		 = '1';

		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_AllProductListValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_AllProductListValues','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_GET_AllProductListValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_AllProductListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
			
		-- Helper Table with all list values
		;WITH list AS (
			SELECT 
				hL.ListID
				,hL.Datentyp 
				,CAST(ISNULL(hLV.ValueInt	, 0) AS INT) AS ValueInt -- must return int !
                ,CAST(ISNULL(hLV.Scale		, 0) AS INT) AS Scale -- must return int !
				,ISNULL(hLV.ValueText		, N'')		 AS ValueText
				,ISNULL(hLV.ValueComment	, N'')		 AS ValueComment
				,ISNULL(hLV.FormatID		, N'')		 AS FormatID
				,ISNULL(hF.BackgroundColor	, N'')		 AS BackgroundColor
				,ISNULL(hF.FontColor		, N'')		 AS FontColor
				,ISNULL(hF.ValueFormat		, N'')		 AS ValueFormat
			FROM planning.thLists hL 
				LEFT JOIN planning.thListValues hLV	ON hL.ListID = hLV.ListID
				-- for the value
				LEFT JOIN planning.thFormats hF	ON hLV.FormatID = hF.FormatID
				-- filter the lists, which where till 4.0.57 GUI defined and are now internal system lists
			WHERE hL.ListID NOT IN (
				'sxIsNumeric'
				,'sxVisibilityLevel'
				,'sxValueLists'
				,'sxValueSources'
				,'sxValueFormats'
				,'sxScales'
				,'sxValueEffects')
		)
		-- STEP 3.1 Dynamic System Lists
		SELECT *
		FROM list
		WHERE ListID IN ('sxUnits','sxValueEffectParameters')	
		
		UNION ALL
		-- STEP 3.2 Product specific ListSet based on defined ValueLists
		SELECT *
		FROM list
		WHERE ListID IN (
			SELECT DISTINCT ValueListID
			FROM planning.tdValueSeries
			WHERE ProductKey = @ProductKey)

		-- add internal system lists ----------------------------------------------------------
		UNION ALL 
		--sxIsNumeric - static  TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES 
				 ('sxIsNumeric','String',0,1,'0','','','','','')
				,('sxIsNumeric','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL	
		--sxVisibilityLevel - static TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES	
				 ('sxVisibilityLevel','String',0,1,'0','','','','','')
				,('sxVisibilityLevel','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		UNION ALL			
		--sxValueLists - dynamic List of GUI Lists
		SELECT 
			'sxValueLists'
			,'String'
			,0		AS ValueInt 
            ,1		AS Scale 
			,ListID AS ValueText
			,''		AS ValueComment
			,''		AS FormatID
			,''		AS BackgroundColor
			,''		AS FontColor
			,''		AS ValueFormat
		FROM planning.thLists
			-- Filter the GUI Lists, which are not useful as ValueList
		WHERE ListID NOT IN ('sxValueEffectParameter')

		UNION ALL
		--sxValueSources - static (english strings for all languages
		SELECT * 
		FROM (
			VALUES	
				 ('sxValueSources','String',0,1,'Input'		 ,'','','','','')
				,('sxValueSources','String',0,1,'List-Strict','','','','','')
				,('sxValueSources','String',0,1,'XLS'		 ,'','','','','')
				,('sxValueSources','String',0,1,'XLS-Strict' ,'','','','','')
				,('sxValueSources','String',0,1,'Link'		 ,'','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueFormats - dynamic, list of defined formats
		SELECT 
			 'sxValueFormats'
			,'String'
			,0					AS ValueInt 
            ,1					AS Scale 
			,FormatID			AS ValueText
			,''					AS ValueComment
			,FormatID			AS FormatID
			,BackgroundColor	AS BackgroundColor
			,FontColor			AS FontColor
			,ValueFormat		AS ValueFormat
		FROM planning.thFormats 
		
		UNION ALL
		--sxScales - static - this List is also static defined in GET_ListValues
		SELECT * 
		FROM (
			VALUES	
				 ('sxScales','Integer',1	,1,'','','','','','')
				,('sxScales','Integer',10	,1,'','','','','','')
				,('sxScales','Integer',100	,1,'','','','','','')
				,('sxScales','Integer',1000	,1,'','','','','','')
				,('sxScales','Integer',10000,1,'','','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueEffects -dynamic, list of defined ValueEffects in gValueEffects
		SELECT 
			 'sxValueEffects'
			,'String'
			,0			AS ValueInt 
            ,1			AS Scale 
			,EffectID	AS ValueText
			,''			AS ValueComment
			,''			AS FormatID
			,''			AS BackgroundColor
			,''			AS FontColor
			,''			AS ValueFormat
		FROM planning.tgValueEffects

		ORDER BY ListID, ValueText, ValueInt;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_AllProductListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_AllProductListValues;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_AllProductListValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all ListValues a product needs for data entry in ProductDataTable 
	containing the GUI defined lists of the products (defined by user c* or solution su*) 
	and the dynamic system lists (sx* - which are editable by the user) 
	and adds interal system lists (sx* - which are not visible on GUI)';

    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterAccessRight;
GO

/*
	GET operation for User status check on cluster
	If User has an registered Username and his status is 'Active' then Cluster Access Rights 
	are granted. (which still can mean he sees nothing than an empty cluster)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		-- Should Return 200
		DECLARE @RC AS NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterAccessRight 'SQL'
		PRINT @RC

		-- Should Return 403
		DECLARE @RC AS NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterAccessRight 'SQL4'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterAccessRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterAccessRight','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterAccessRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution
	DECLARE @LogMessage			NVARCHAR(MAX)	= N'';

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterAccessRight;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			SET @LogMessage = 'Transaction user don`t exists, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END;
		
		-- STEP 3 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM system.trUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
		BEGIN
			SET @LogMessage = 'User is not active, check was done for Username: ' + @Username + ' with Orginal_Login(): ' + ORIGINAL_LOGIN();
			RAISERROR(@LogMessage, 16, 10);
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterAccessRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterAccessRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterAccessRight TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterAccessRight'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET operation for User status check on cluster
If User has an registered Username and his status is "Active" then Cluster Access Rights 
are granted. (which still can mean he sees nothing than an empty cluster)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterProperties
GO

/*
	GET Operation for Cluster Properties

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		--should return 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterProperties 'SQL'
		PRINT @RC

		--should return 403
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterProperties 'SQL4'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterProperties',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterProperties','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterProperties
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	 NVARCHAR(255)		= N'';
	DECLARE @Right				 NVARCHAR(255)		= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)		= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)		= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME			= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000)		= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperties;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = 'Write';
			END 
			ELSE 
			BEGIN 
				SET @Right = 'Read';
				SET @ResultCode = 200;
			END;
		END;
	
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;	
		
		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgCluster
		ORDER BY COALESCE(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperties;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterProperties TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperties TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterProperties'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Cluster Properties';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterProperty;
GO

/*
	GET Operation for determining a cluster Property

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterProperty 
			@Username	= 'SQL',
			@PropertyID = 'LS01'

			PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterProperty
	@Username	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @PropertyID	IS NULL SET @PropertyID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @PropertyID = dbo.sx_pf_pProtectID		(@PropertyID);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @TransactUsername = 'FactoryService';
		END;

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM planning.tgCluster WHERE PropertyID = @PropertyID;

		IF @PropertyKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
		IF @TransactUsername = 'FactoryService' SET @ResultCode = 200


		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT  
			PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgCluster
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterProperty TO pf_PlanningFactoryUser;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determining a cluster Property';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterValidation;
GO

/*
	Operation to check Cluster
	Running tests:
		Lists, which are never used
		ListMembers, which are never used
		Lists which are used but dont exists
		List members which are Used but dont exist
		Formats which are used but dont exist
		Formats which are defined but not used
		count ValueSeries per Product
		count ValuesPer Product
	User must be active to run tests

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call					   
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterValidation 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterValidation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterValidation','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterValidation
	@Username NVARCHAR(255) 
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(255)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterValidation;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
			  
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM system.trUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User is not active', 16, 10);
		
		-- STEP 3 - Run tests
		DECLARE @Tests AS TABLE (
			N INT IDENTITY(1,1)		NOT NULL, 
			[Name] NVARCHAR(200)	NOT NULL, 
			[Status] NVARCHAR(10)	NOT NULL,
			Result NVARCHAR(MAX)	NOT NULL,
			RowsCount BIGINT		NOT NULL);

-- Lists, which are never used			
		;WITH lst AS (
			SELECT l.ListID
				, l.ListKey
				, l.NameShort AS ListName
			FROM planning.thLists l 
				LEFT JOIN (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs ON l.ListID = vs.ValueListID
			WHERE vs.ValueSeriesKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Lists', N'Info', N'List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + ListName + N'''', 1
		FROM lst;

		SET @EffectedRows += @@ROWCOUNT;

-- Lists which are used but dont exists	
		;WITH lst AS (
			SELECT vs.ValueListID	AS ListID
				, vs.NameShort AS VSName
				, vs.ValueSeriesKey AS VSKey
				, vs.ValueSeriesID AS VSID
			FROM (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs
				LEFT JOIN planning.thLists  l ON l.ListID = vs.ValueListID
			WHERE l.ListID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists Lists', N'Error', N'ListID ' + ListID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), VSKey) + N', ' + VSID + N' ''' + VSName + N''')', 1
		FROM lst;
		
		SET @EffectedRows += @@ROWCOUNT;			  

-- ListMembers, which are never used		
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), lv.ListValueKey) + N' ''' 
				+ CASE WHEN l.Datentyp = N'INT' THEN CONVERT(NVARCHAR(MAX), lv.ValueInt) ELSE lv.ValueText END 
				+ N''' (List #' + CONVERT(NVARCHAR(MAX), l.ListKey) + N', ' + lv.ListID + N' ''' + l.NameShort + N''')' AS Result
			FROM planning.thListValues lv 
				LEFT JOIN planning.thLists l ON lv.ListID = l.ListID							
				LEFT JOIN planning.tdValueSeries vs ON l.ListID = vs.ValueListID AND ((vs.[IsNumeric] = 1 AND l.Datentyp = N'INT') OR (vs.[IsNumeric] = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.tfValues v ON vs.ValueSeriesKey = v.ValueSeriesKey
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE v.ValueKey IS NULL
		 )
		 INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		 SELECT N'Unused List Values', N'Info', Result, 1
		 FROM mmbr;

		 SET @EffectedRows += @@ROWCOUNT;

--  List members which are Used but dont exist
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), v.ValueKey) + N' ''' + CASE WHEN vs.IsNumeric = 1 THEN CONVERT(NVARCHAR(MAX), v.ValueInt) ELSE v.ValueText END
				+ N''' ListID ' + vs.ValueListID 
				+ N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), v.ValueSeriesKey) + N', ' + vs.ValueSeriesID
				+ N' ''' + vs.NameShort + N''')' AS Result
			FROM planning.tfValues v
				INNER JOIN planning.tdValueSeries vs ON vs.ValueListID <> N'' AND vs.ValueSeriesKey = v.ValueSeriesKey
				LEFT JOIN planning.thLists l ON vs.ValueListID = l.ListID AND ((vs.IsNumeric = 1 AND l.Datentyp = N'INT') OR (vs.IsNumeric = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.thListValues lv ON lv.ListID = l.ListID
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE lv.ListValueKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists List Values', N'Error', Result, 1
		FROM mmbr;

		SET @EffectedRows += @@ROWCOUNT;

		;WITH frm AS (									 
			SELECT ValueFormatID AS FormatID , N'FormatID ' + ValueFormatID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), ValueSeriesKey) + N', ' + ValueSeriesID + N' ''' + NameShort + N''')' AS Result			 
			FROM planning.tdValueSeries
			WHERE ValueFormatID <> N''
			UNION ALL												   
			SELECT FormatID , N'FormatID ' + FormatID + N' (Cluster Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgCluster
			WHERE FormatID <> N'' 								  
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Factory Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result								   
			FROM planning.tgFactories
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ProductLine Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result							  		
			FROM planning.tgProductLines
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Product Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgProducts
			WHERE FormatID <> N'' 
			UNION ALL										 
			SELECT FormatID , N'FormatID ' + FormatID + N' (List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + NameShort + N''')' AS Result
			FROM planning.thLists
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ListValue #' + CONVERT(NVARCHAR(MAX), ListValueKey) + N', ListID ' + ListID + N')' AS Result
			FROM planning.thListValues
			WHERE FormatID <> N'' 
		)
	-- Formats which are used but don`t exist
		, unusedFrmts AS (
			SELECT N'Format #' + CONVERT(NVARCHAR(MAX), FormatKey) + N', ' + FormatID AS Result												
			FROM planning.thFormats
			WHERE FormatID NOT IN (SELECT DISTINCT FormatID FROM frm)
		)
	-- Formats which are defined but not used	
		, notExistsFrmts AS (
			SELECT Result 
			FROM frm LEFT JOIN planning.thFormats f ON frm.FormatID	= f.FormatID
			WHERE f.FormatID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Formats', N'Info', Result, 1
		FROM unusedFrmts
		UNION ALL
		SELECT N'Don`t exists Formats', N'Error', Result, 1
		FROM notExistsFrmts;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValueSeries per Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'ValueSeries per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tdValueSeries	vs INNER JOIN planning.tdProducts p ON vs.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValuesPer Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Values per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tfValues v INNER JOIN planning.tdProducts p ON v.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;								   	   
		
		SET @EffectedRows += @@ROWCOUNT;

		SELECT * 
		FROM @Tests
		ORDER BY N;

		COMMIT TRANSACTION sx_pf_GET_ClusterValidation;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterValidation;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterValidation'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Operation to check Cluster
	Running tests:
    Lists, which are never used
    ListMembers, which are never used
    Lists which are used but dont exists
    List members which are Used but dont exist
    Formats which are used but dont exist
    Formats which are defined but not used
    count ValueSeries per Product
    count ValuesPer Product
	User must be active to run tests';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
   
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterWriteRight;
GO

/*
	GET Operation for Write Rights on a Cluster
	Returns 200 if User has ClusterWriteRights

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterWriteRight 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterWriteRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterWriteRight','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterWriteRight 
	@Username NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterWriteRight;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
				
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 3 - Test that user has write access to the cluster
		SELECT @ResultCode = IIF([Right] = N'Write', 200, 401) FROM system.trUserRights WHERE FactoryID = N'' AND Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User have no write access to the cluster', 16, 10);

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ClusterWriteRight;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterWriteRight;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;

END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterWriteRight TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterWriteRight'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Write Rights on a Cluster
				 Returns 200 if User has ClusterWriteRights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Factories;
GO

/*
	GET Operation for all eligible Factories
	Returns the list of all the factories, for which the user has rights

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call - should return Table and 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Factories 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Factories',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Factories','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Factories
	@Username NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factories;

		-- STEP 0.1 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 3. Show public values	
		;WITH rights AS (
			SELECT 
				FactoryID,  
				[Right],
				ISNULL(TRY_CAST(FactoryID AS INT), 999999999) AS orderID 
			FROM system.trUserRights
			WHERE ProductLineID = N'' 
				AND Username = @TransactUsername
				AND [Right] IN (N'Read', N'Write')
		)
		SELECT   
			 dF.FactoryID
			,dF.NameShort
			,dF.NameLong
			,dF.CommentUser
			,dF.CommentDev
			,dF.ResponsiblePerson
			,dF.ImageName
			,vUR.[Right]
		FROM planning.tdFactories dF 
			INNER JOIN rights vUR ON dF.FactoryID = vUR.FactoryID	
		ORDER BY vUR.orderID, dF.FactoryID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Factories;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factories;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::sx_pf_GET_Factories TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_Factories TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Factories' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all eligible Factories
				 Returns the list of all the factories, for which the user has rights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Factory;
GO

/*
	GET Operation for one Factory
	Delivers all informations about one factory

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Factory
			@Username	= 'SQL',
			@FactoryID	= 'ZT'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Factory
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
			
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,@Right AS 'Right'
		FROM planning.tdFactories
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Factory TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Factory TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Factory'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for one Factory
				 Delivers all informations about one factory';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FactoryProperties;
GO

/*
	GET Operation for determining a Factory Properties

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_FactoryProperties 
			@Username	= 'SQL',
			@FactoryID	= 'D'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FactoryProperties',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FactoryProperties','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FactoryProperties
	@Username  NVARCHAR(255),
	@FactoryID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;					-- SET during Execution
	DECLARE @ResultCode			INT				= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);

		IF @FactoryID = N''		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		IF @FactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgFactories
		WHERE FactoryKey = @FactoryKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FactoryProperties TO pf_PlanningFactoryService; 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperties TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FactoryProperties'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determining a Factory Properties';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FactoryProperty;
GO

/*
	GET Operation for determining a Factory Property

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_FactoryProperty 
			@Username	= 'SQL',
			@FactoryID	= 'D',
			@PropertyID = 'F01'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FactoryProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FactoryProperty','PARAMETER',NULL)
*/


CREATE PROCEDURE dbo.sx_pf_GET_FactoryProperty
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				 = 0;
	DECLARE @PropertyKey		INT				 = 0;
	DECLARE @Right				NVARCHAR(10)	 = N'';
	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	 = N'''' 
			+ ISNULL(@Username,		 N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	 + N''',''' 
			+ ISNULL(@PropertyID,	 N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;				-- SET during Execution
	DECLARE @ResultCode			INT				 = 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username		= N'';
	IF @FactoryID	IS NULL SET @FactoryID		= N'';
	IF @PropertyID	IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);	

		IF	@FactoryID = N'' 
			OR @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		IF @FactoryKey = 0 OR @PropertyKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgFactories
		WHERE PropertyKey = @PropertyKey; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FactoryProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FactoryProperty TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FactoryProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determining a Factory Property';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FlexAction;
GO

/*
	GET Operation for receiving a filterd context sensitiv List with dynamic column count
	The first column is sended back with POST_FlexAction as idenitfier for a database action

	Procedure must be editend to fit user needs and recreated after each API Update.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		--should return Data and 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_FlexAction
			 @Username				= 'SQL'
			,@FactoryID				= 'F1'
			,@ProductLineID			= 'PL1'
			,@ProductID				= 'P1'
			,@ValueSeriesID			= 'Projekt'
			,@TimeID				= 20180115
			,@GlobalattributeNumber = ''
			,@PageType				= ''
			,@SearchString			= 'ei';

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexAction',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexAction','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	 = N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';	
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,				 N'NULL')					+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL')					+ N''',''' 
			+ ISNULL(@ProductID,			 N'NULL')					+ N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL')					+ N''','
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL')					+ N''','''	
			+ ISNULL(@PageType,				 N'NULL')					+ N''',''' 
			+ ISNULL(@SearchString,			 N'NULL')					+ N'''';  
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= N'';
	IF @PageType				IS NULL SET @PageType				= N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
	
		-- STEP 3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Projects AS TABLE
		(
			 ProjectID		NVARCHAR (255) NOT NULL
			,ProjectName	NVARCHAR (255) NOT NULL
			,ProjectManager NVARCHAR (255) NOT NULL
		);

		INSERT INTO @Projects
		VALUES
			(N'P1000',N'Testprojekt in Aachen','Pit Brown'),
			(N'P1001',N'Testprojekt in Augsburg','Pit Brown'),
			(N'P1002',N'Testprojekt in Bergisch Gladbach','Pit Brown'),
			(N'P1003',N'Testprojekt in Berlin','Pit Brown'),
			(N'P1004',N'Testprojekt in Bielefeld','Pit Brown'),
			(N'P1005',N'Testprojekt in Bochum','Pit Brown'),
			(N'P1006',N'Testprojekt in Bonn','Pit Brown'),
			(N'P1007',N'Testprojekt in Bottrop','Pit Brown'),
			(N'P1008',N'Testprojekt in Braunschweig','Pit Brown'),
			(N'P1009',N'Testprojekt in Bremen','Pit Brown'),
			(N'P1010',N'Testprojekt in Bremerhaven','Pit Brown'),
			(N'P1011',N'Testprojekt in Chemnitz','Pit Brown'),
			(N'P1012',N'Testprojekt in Darmstadt','Pit Brown'),
			(N'P1013',N'Testprojekt in Dortmund','Pit Brown'),
			(N'P1014',N'Testprojekt in Dresden','Pit Brown'),
			(N'P1015',N'Testprojekt in Duisburg','Pit Brown'),
			(N'P1016',N'Testprojekt in Düsseldorf','Pit Brown'),
			(N'P1017',N'Testprojekt in Erfurt','Pit Brown'),
			(N'P1018',N'Testprojekt in Erlangen','Pit Brown'),
			(N'P1019',N'Testprojekt in Essen','Pit Brown'),
			(N'P1020',N'Testprojekt in Frankfurt am Main','Pit Brown'),
			(N'P1021',N'Testprojekt in Freiburg im Breisgau','Pit Brown'),
			(N'P1022',N'Testprojekt in Fürth','Pit Brown'),
			(N'P1023',N'Testprojekt in Gelsenkirchen','Pit Brown'),
			(N'P1024',N'Testprojekt in Göttingen','Pit Brown'),
			(N'P1025',N'Testprojekt in Hagen','Pit Brown'),
			(N'P1026',N'Testprojekt in Halle (Saale)','Pit Brown'),
			(N'P1027',N'Testprojekt in Hamburg','Pit Brown'),
			(N'P1028',N'Testprojekt in Hamm','Pit Brown'),
			(N'P1029',N'Testprojekt in Hannover','Pit Brown'),
			(N'P1030',N'Testprojekt in Heidelberg','Cathy Gold'),
			(N'P1032',N'Testprojekt in Heilbronn','Cathy Gold'),
			(N'P1032',N'Testprojekt in Herne','Cathy Gold'),
			(N'P1033',N'Testprojekt in Ingolstadt','Cathy Gold'),
			(N'P1034',N'Testprojekt in Jena','Cathy Gold'),
			(N'P1035',N'Testprojekt in Karlsruhe','Cathy Gold'),
			(N'P1036',N'Testprojekt in Kassel','Cathy Gold'),
			(N'P1037',N'Testprojekt in Kiel','Cathy Gold'),
			(N'P1038',N'Testprojekt in Koblenz','Cathy Gold'),
			(N'P1039',N'Testprojekt in Köln','Cathy Gold'),
			(N'P1040',N'Testprojekt in Krefeld','Cathy Gold'),
			(N'P1041',N'Testprojekt in Leipzig','Cathy Gold'),
			(N'P1042',N'Testprojekt in Leverkusen','Cathy Gold'),
			(N'P1043',N'Testprojekt in Lübeck','Cathy Gold'),
			(N'P1044',N'Testprojekt in Ludwigshafen am Rhein','Cathy Gold'),
			(N'P1045',N'Testprojekt in Magdeburg','Cathy Gold'),
			(N'P1046',N'Testprojekt in Mainz','Cathy Gold'),
			(N'P1047',N'Testprojekt in Mannheim','Cathy Gold'),
			(N'P1048',N'Testprojekt in Moers','Cathy Gold'),
			(N'P1048',N'Testprojekt in Mönchengladbach','Cathy Gold'),
			(N'P1049',N'Testprojekt in Mülheim an der Ruhr','Cathy Gold'),
			(N'P1050',N'Testprojekt in München','Cathy Gold'),
			(N'P1051',N'Testprojekt in Münster','Cathy Gold'),
			(N'P1052',N'Testprojekt in Neuss','Cathy Gold'),
			(N'P1053',N'Testprojekt in Nürnberg','Cathy Gold'),
			(N'P1054',N'Testprojekt in Oberhausen','Cathy Gold'),
			(N'P1055',N'Testprojekt in Offenbach am Main','Cathy Gold'),
			(N'P1056',N'Testprojekt in Oldenburg','Cathy Gold'),
			(N'P1057',N'Testprojekt in Osnabrück','Cathy Gold'),
			(N'P1058',N'Testprojekt in Paderborn','Cathy Gold'),
			(N'P1059',N'Testprojekt in Pforzheim','Cathy Gold'),
			(N'P1060',N'Testprojekt in Potsdam','Cathy Gold'),
			(N'P1061',N'Testprojekt in Recklinghausen','Cathy Gold'),
			(N'P1062',N'Testprojekt in Regensburg','Cathy Gold'),
			(N'P1063',N'Testprojekt in Remscheid','Cathy Gold'),
			(N'P1064',N'Testprojekt in Reutlingen','Cathy Gold'),
			(N'P1065',N'Testprojekt in Rostock','Cathy Gold'),
			(N'P1066',N'Testprojekt in Saarbrücken','Cathy Gold'),
			(N'P1067',N'Testprojekt in Siegen','Cathy Gold'),
			(N'P1068',N'Testprojekt in Solingen','Cathy Gold'),
			(N'P1069',N'Testprojekt in Stuttgart','Cathy Gold'),
			(N'P1070',N'Testprojekt in Trier','Cathy Gold'),
			(N'P1071',N'Testprojekt in Ulm','Cathy Gold'),
			(N'P1072',N'Testprojekt in Wiesbaden','Cathy Gold'),
			(N'P1073',N'Testprojekt in Wolfsburg','Cathy Gold'),
			(N'P1074',N'Testprojekt in Wuppertal','Cathy Gold'),
			(N'P1075',N'Testprojekt in Würzburg','Cathy Gold');
		
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Second Column must have identifier character, its send back with the POST_FlexAction Command
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R= Right, C = Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter

		-- React on content parameters when creating the return list.
		-- You could add a "*" Member to the List and react on it in POST to create a new list member

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			SELECT 'Projekt wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS ProjectID_RS50
				,p.ProjectName AS Projektname_LS300
				,p.ProjectManager AS Projektmanager_CS150
			FROM @Projects p
			WHERE p.ProjectName LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			SELECT 'Mitarbeiter wird bei Auswahl angelegt.' AS Hint
				,p.ProjectID AS MitarbeiterID_RS50
				,p.ProjectManager AS Projektmanager_LS150
			FROM @Projects p
			WHERE p.ProjectManager LIKE @SearchString
			ORDER BY ProjectID;
		END

		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter')
		BEGIN
			SELECT 'FlexActions sind nur für die Wertreihen "Projekt" und "Mitarbeiter" definiert' AS Hint;
		END

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexAction;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexAction TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FlexAction'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving a filterd context sensitiv List with dynamic column count
				 The first column is sended back with POST_FlexAction as idenitfier for a database action';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
		

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FlexDrill;
GO

/*
	GET Operation for receiving a context sensitiv List
	API deliviers List Values fitting to the sended context Parameters
	Procedure must be editend to fit user needs and recreated after each API Update
	When updating this procedure, you must reset the rights for it

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		--should return Data and 200
		EXEC @RC = dbo.sx_pf_GET_FlexDrill 
			@Username				= 'SQL'
			,@FactoryID				= '1'
			,@ProductLineID			= '1'
			,@ProductID				= '1'
			,@ValueSeriesID			= '1'
			,@TimeID				= 20190115
			,@GlobalattributeNumber = ''
			,@PageType				= ''
			,@SearchString			= 'r';

	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexDrill',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexDrill','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FlexDrill
	 @Username				 NVARCHAR(255)
	,@FactoryID				 NVARCHAR(255)
	,@ProductLineID			 NVARCHAR(255)
	,@ProductID				 NVARCHAR(255)
	,@ValueSeriesID			 NVARCHAR(255)
	,@TimeID				 BIGINT
	,@GlobalattributeNumber  NVARCHAR(255)
	,@PageType				 NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			 NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				
	DECLARE @ParameterString	NVARCHAR(MAX)	=		N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		 N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			 N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,		 N'NULL') + N''',''' 
			+ CAST(ISNULL(@TimeID,			 N'NULL') AS NVARCHAR(255))	+ N',''' 
			+ ISNULL(@GlobalattributeNumber, N'NULL') + N''',''' 
			+ ISNULL(@PageType,				 N'NULL') + N''',''' 
			+ ISNULL(@SearchString,			 N'NULL') + N''''; 
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username			  IS NULL SET @Username				 = N'';
	IF @FactoryID			  IS NULL SET @FactoryID			 = N'';
	IF @ProductLineID		  IS NULL SET @ProductLineID		 = N'';
	IF @ProductID			  IS NULL SET @ProductID			 = N'';
	IF @ValueSeriesID		  IS NULL SET @ValueSeriesID		 = N'';
	IF @TimeID				  IS NULL SET @TimeID				 = 0;
	IF @GlobalattributeNumber IS NULL SET @GlobalattributeNumber = N'';
	IF @PageType			  IS NULL SET @PageType				 = N'';

	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexDrill;

		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSeriesID	 = dbo.sx_pf_pProtectID		 (@ValueSeriesID);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
	
		-- STEP 3.1 - Sample for creating a List for a specific ValueSeries in all Products from specific template
		IF @ValueSeriesID = N'that' 
			AND EXISTS(
				SELECT Template FROM planning.tdProducts 
				WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 'ValueSeriesSpecificList' AS Dummy;
			SET @EffectedRows = 1;
		END;

		-- STEP 3.2 - Sample for creating a List for a Globalattribute in one ProductLine
		IF @FactoryID = N'this' AND @ProductLineID = N'that' AND @GlobalattributeNumber = 5
		BEGIN
			SELECT 'ProductLineSpecificList for ValueSeries'; 
			SET @EffectedRows += 1;
		END;
		
		-- STEP 3.3 - this are sample data which are used as output as long there are no user specific implementations
		DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

		INSERT INTO @Cities VALUES
			(N'Aachen'),
			(N'Augsburg'),
			(N'Bergisch Gladbach'),
			(N'Berlin'),
			(N'Bielefeld'),
			(N'Bochum'),
			(N'Bonn'),
			(N'Bottrop'),
			(N'Braunschweig'),
			(N'Bremen'),
			(N'Bremerhaven'),
			(N'Chemnitz'),
			(N'Darmstadt'),
			(N'Dortmund'),
			(N'Dresden'),
			(N'Duisburg'),
			(N'Düsseldorf'),
			(N'Erfurt'),
			(N'Erlangen'),
			(N'Essen'),
			(N'Frankfurt am Main'),
			(N'Freiburg im Breisgau'),
			(N'Fürth'),
			(N'Gelsenkirchen'),
			(N'Göttingen'),
			(N'Hagen'),
			(N'Halle (Saale)'),
			(N'Hamburg'),
			(N'Hamm'),
			(N'Hannover'),
			(N'Heidelberg'),
			(N'Heilbronn'),
			(N'Herne'),
			(N'Ingolstadt'),
			(N'Jena'),
			(N'Karlsruhe'),
			(N'Kassel'),
			(N'Kiel'),
			(N'Koblenz'),
			(N'Köln'),
			(N'Krefeld'),
			(N'Leipzig'),
			(N'Leverkusen'),
			(N'Lübeck'),
			(N'Ludwigshafen am Rhein'),
			(N'Magdeburg'),
			(N'Mainz'),
			(N'Mannheim'),
			(N'Moers'),
			(N'Mönchengladbach'),
			(N'Mülheim an der Ruhr'),
			(N'München'),
			(N'Münster'),
			(N'Neuss'),
			(N'Nürnberg'),
			(N'Oberhausen'),
			(N'Offenbach am Main'),
			(N'Oldenburg'),
			(N'Osnabrück'),
			(N'Paderborn'),
			(N'Pforzheim'),
			(N'Potsdam'),
			(N'Recklinghausen'),
			(N'Regensburg'),
			(N'Remscheid'),
			(N'Reutlingen'),
			(N'Rostock'),
			(N'Saarbrücken'),
			(N'Siegen'),
			(N'Solingen'),
			(N'Stuttgart'),
			(N'Trier'),
			(N'Ulm'),
			(N'Wiesbaden'),
			(N'Wolfsburg'),
			(N'Wuppertal'),
			(N'Würzburg');

		SELECT 
		-- First Column must be "Hint" an commits the display hint to the frontend
		-- Column Names must have an Alias (AS Displayname) with one "_", after which 
			-- Orientation (L=Left, R=Right, C=Center)
			-- DataType (M=Money, S=String, I=Int) 
			-- ColumnWidth 
			-- is coded as Frontend Parameter
			'List of Cities to select.'	 AS Hint
			,UPPER(LEFT([Name], 4))		 AS CityCode_LS50
			,[Name]						 AS CityName_CS200
			,'Beschreibung dieser Stadt...' AS Beschreibung_LS300
			,500000						 AS Einwohnerzahl_LM50
		 FROM @Cities
		 WHERE [Name] LIKE @SearchString
		 ORDER BY [Name];

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexDrill;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexDrill;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexDrill TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FlexDrill' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving a context sensitiv List
				API deliviers List Values fitting to the sended context Parameters
				Procedure must be editend to fit user needs and recreated after each API Update
				When updating this procedure, you must reset the rights for it';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
			

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_FlexList;
GO

/*
	GET Operation for receiving a context sensitiv List
	API deliviers List Values fitting to the sended context Parameters
	Procedure must be editend to fit user needs and recreated after each API Update
	When updating this procedure, you must reset the rights for it

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC AS NVARCHAR(255)
		--should return Data and 200
		EXEC @RC = dbo.sx_pf_GET_FlexList 'SQL','1','1','1','1','','','',''
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexList',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_FlexList','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_FlexList
	@Username				NVARCHAR(255)
	,@FactoryID 			NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @GlobalattributeAlias	NVARCHAR(255)	= N'';
	DECLARE @SearchDoneFlag			INT				= 0;
	DECLARE @ProductKey				BIGINT			= 0;
	DECLARE @DummyTable TABLE (Dummy NVARCHAR(255) NOT NULL)  -- only for the samples to compile
	DECLARE @DummyInt				INT
	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= GETUTCDATE();
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductID,				N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,			N'NULL') + N''','
			+ CAST(ISNULL(@TimeID,				N'NULL') AS NVARCHAR(255)) + N','''
			+ ISNULL(@GlobalattributeNumber,	N'NULL') + N''',''' 
			+ ISNULL(@PageType,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N''''; 
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_FlexList;

		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID

		-- Preparing of useful context informations 
		-- ****************************************************************************************************************************************************
		-- Try to determine ProductKey
		IF @ProductID			<> N'' 
			AND @ProductLineID	<> N'' 
			AND @FactoryID		<> N''
		BEGIN
			SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID
		END

		-- Determine AliasName of Globalattribute, to define actions for Globalattributes by name
		IF @GlobalattributeNumber <> ''
		BEGIN
			IF OBJECT_ID('tempdb..#GANames') IS NOT NULL
				DROP TABLE #GANames;

			CREATE TABLE #GANames
			(
				RowKey					BIGINT IDENTITY (1,1)	NOT NULL
				,FactoryID				NVARCHAR (255)			NOT NULL
				,ProductLineID			NVARCHAR (255)			NOT NULL
				,Globalattribute		NVARCHAR (255) 			NOT NULL
				,GlobalattributeNumber	INT						NOT NULL
				,Aliasname				NVARCHAR (255)			NOT NULL
			)
			
			INSERT INTO #GANames
			SELECT
				FactoryID
				,ProductLineID
				,Globalattribute
				,COALESCE(TRY_CAST(RIGHT(Globalattribute,2) AS INT), TRY_CAST(RIGHT(Globalattribute,1) AS INT)) AS GlobalattributeNumber
				,Aliasname
			FROM planning.tdProductLines
			UNPIVOT
			(
				Aliasname FOR Globalattribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalattributeAlias3, GlobalattributeAlias4, GlobalattributeAlias5
					,GlobalattributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10
					,GlobalattributeAlias11, GlobalattributeAlias12, GlobalattributeAlias13, GlobalattributeAlias14, GlobalattributeAlias15
					,GlobalattributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20
					,GlobalattributeAlias21, GlobalattributeAlias22, GlobalattributeAlias23, GlobalattributeAlias24, GlobalattributeAlias25
				)
			) unpiv
			WHERE 
					FactoryID		= @FactoryID 
				AND	ProductLineID	= @ProductLineID;

			SELECT @GlobalattributeAlias = Aliasname FROM #GANames WHERE GlobalattributeNumber = @GlobalattributeNumber;
		END

		/*
		DEFINITION OF THE FLEX LISTS
		*****************************************************************************************************************************************************************
		First Column must be "Hint" an commits the display hint to the frontend
		Second Column must have identifier character, this value is sended back to the selected cell
		Column Names must have an Alias (AS Displayname) with ONLY ONE "_", after which 
			Orientation		(L=Left, R= Right, C = Center)
			DataType		(M=Money, S=String, I=Int) 
			ColumnWidth		(Integer Number)
		is coded as Frontend Parameter, e.g. Name_LS50  means the column ist labeled Name, has left orientation,
		is formated as sting an 50px width
		*/

		-- Sample for creating a List for all ValueSeries with a specific ID
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable; 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;
		
		-- Sample for creating a List for all ValueSeries with a specific ID in a specific Template
		IF @ValueSeriesID = N'SKTO' AND EXISTS
		(
			SELECT Template 
			FROM planning.tdProducts 
			WHERE 
				ProductKey = @ProductKey 
				AND Template = N'Invest_VM'
		)
		BEGIN
			SELECT 
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,'1000'	AS KontenID_LS30
				,'Anlagevermögen'
			FROM @DummyTable 

			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END;

		-- Sample for Seach over parameter Product
		IF @ValueSeriesID = N'SKTO' 						
		BEGIN
			SELECT DISTINCT
				'Bitte Sachkonto auswählen, im Eingabefeld kann per Zeichenfolge gefiltert werden.' AS Hint
				,fV.ValueText	AS KontenID_LS40
				,fV2.ValueText	AS KontenName_LS180
			FROM planning.tfValues fV 
				-- Kontoname aus anderer Spalte
				LEFT JOIN
					(
						SELECT
							TimeID
							,ValueText 
						FROM planning.tfValues 
						WHERE 
								FactoryID = 'ZP'
							AND ProductLineID = '1'
							AND ProductID = '1'
							AND ValueSeriesID = 'KTO_Name'
						
					) fV2 ON fV.TimeID = fV2.TimeID
			WHERE 
				fV.FactoryID = 'ZP'
				AND fV.ProductLineID = '1'
				AND fV.ProductID = '1'
				AND fV.ValueSeriesID = 'KTO_ID'
				AND (fV.ValueText LIKE @SearchString OR fV2.ValueText LIKE @SearchString)
			/*
			WHERE 
					KontenID LIKE @SearchString 
				OR KontenName LIKE @SearchString

			ORDER BY KontenID
			*/

			SET @SearchDoneFlag = 1;
		END

		-- Samples for other context
		IF	@GlobalattributeAlias = N'SKTO'		-- for this Globalattribute
			OR LEFT(@ValueSeriesID, 3) = N'KST' -- for all ValueSeries starting with this letters
		BEGIN	
			SET @DummyInt = 1 
		END	
		
		-- Sample Fallback -- delete in productive use to change fallback to Productive Fallback
		-- **************************************************************************************************************************************
		IF @SearchDoneFlag  = 0
		BEGIN
			DECLARE @Cities AS TABLE([Name] NVARCHAR (255) NOT NULL);

			INSERT INTO @Cities 
			VALUES
				(N'Aachen'),
				(N'Augsburg'),
				(N'Bergisch Gladbach'),
				(N'Berlin'),
				(N'Bielefeld'),
				(N'Bochum'),
				(N'Bonn'),
				(N'Bottrop'),
				(N'Braunschweig'),
				(N'Bremen'),
				(N'Bremerhaven'),
				(N'Chemnitz'),
				(N'Darmstadt'),
				(N'Dortmund'),
				(N'Dresden'),
				(N'Duisburg'),
				(N'Düsseldorf'),
				(N'Erfurt'),
				(N'Erlangen'),
				(N'Essen'),
				(N'Frankfurt am Main'),
				(N'Freiburg im Breisgau'),
				(N'Fürth'),
				(N'Gelsenkirchen'),
				(N'Göttingen'),
				(N'Hagen'),
				(N'Halle (Saale)'),
				(N'Hamburg'),
				(N'Hamm'),
				(N'Hannover'),
				(N'Heidelberg'),
				(N'Heilbronn'),
				(N'Herne'),
				(N'Ingolstadt'),
				(N'Jena'),
				(N'Karlsruhe'),
				(N'Kassel'),
				(N'Kiel'),
				(N'Koblenz'),
				(N'Köln'),
				(N'Krefeld'),
				(N'Leipzig'),
				(N'Leverkusen'),
				(N'Lübeck'),
				(N'Ludwigshafen am Rhein'),
				(N'Magdeburg'),
				(N'Mainz'),
				(N'Mannheim'),
				(N'Moers'),
				(N'Mönchengladbach'),
				(N'Mülheim an der Ruhr'),
				(N'München'),
				(N'Münster'),
				(N'Neuss'),
				(N'Nürnberg'),
				(N'Oberhausen'),
				(N'Offenbach am Main'),
				(N'Oldenburg'),
				(N'Osnabrück'),
				(N'Paderborn'),
				(N'Pforzheim'),
				(N'Potsdam'),
				(N'Recklinghausen'),
				(N'Regensburg'),
				(N'Remscheid'),
				(N'Reutlingen'),
				(N'Rostock'),
				(N'Saarbrücken'),
				(N'Siegen'),
				(N'Solingen'),
				(N'Stuttgart'),
				(N'Trier'),
				(N'Ulm'),
				(N'Wiesbaden'),
				(N'Wolfsburg'),
				(N'Wuppertal'),
				(N'Würzburg');
		
			SELECT 
				'List of Cities, you can search with substring.' AS Hint
				,UPPER(LEFT([Name], 4)) AS CityCode_LS50
				,[Name] AS CityName_CS200
			FROM @Cities
		 	WHERE [Name] LIKE @SearchString
		 	ORDER BY [Name];
			
			SET @SearchDoneFlag = 1
		END

		-- Productive Fallback - if no flex search matched the query
		-- **************************************************************************************
		IF @SearchDoneFlag = 0 
		BEGIN
			SELECT 
				'For this Seach Context, we sadly must tell to have no fitting content to offer. ' AS Hint
				,'Nothing' AS Content_LS50
				,'is all I have to offer for you.' AS Sorry_LS200
		END

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_FlexList;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_FlexList;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_FlexList TO pf_PlanningFactoryService
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_FlexList TO pf_PlanningFactoryUser

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_FlexList'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving a context sensitiv List
				API deliviers List Values fitting to the sended context Parameters
				Procedure must be editend to fit user needs and recreated after each API Update
				When updating this procedure, you must reset the rights for it';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
		

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Format;
GO

/*
	GET Operation for a Format

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Format 
			@Username = 'SQL', 
			@FormatID = 'HintergrundRot'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Format
	@Username NVARCHAR(255),
	@FormatID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			 INT			= 0;
	DECLARE @Right				 NVARCHAR(10)	= '';
	DECLARE @TransactUsername	 NVARCHAR(255)	= '';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@FormatID, N'NULL')		+ N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FormatID IS NULL SET @FormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Format;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		SET @FormatID = dbo.sx_pf_pProtectID		(@FormatID);

		IF @FormatID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT  @FormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @FormatID;
		
		IF @FormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM planning.thFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Format;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Format TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Format TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Format'    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for a Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Formats;
GO

/*
	GET Operation for all Formats in Cluster

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Formats 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Formats',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Formats','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Formats
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				 NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;						
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Formats;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
	
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			FormatID,
			BackgroundColor,
			FontColor,
			ValueFormat,
			Bold,
			Italic,
			AlignmentHorizontal,
			AlignmentVertical,
			TextUnderlined,
			BorderToNext,
			BorderToPrevious,
			@Right AS 'Right'
		FROM planning.thFormats
		ORDER BY ISNULL(TRY_CAST(FormatID AS INT), 999999999), FormatID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Formats;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Formats;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Formats TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Formats TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Formats'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all Formats in Cluster';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
  

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_GlobalSearch;
GO

/*
	GET Operation for receiving full text search results for a given textstring
	Search will be done in all DataFactory Elements

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC AS NVARCHAR(255)
		--should return Data and 200
		EXEC @RC = dbo.sx_pf_GET_GlobalSearch 'SQL','whixc'
		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_GlobalSearch',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_GlobalSearch','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_GlobalSearch
	 @Username				NVARCHAR(255)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SearchDoneFlag			INT				= 0;

	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= GETUTCDATE();
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username				= N'';
	
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_GlobalSearch;

		-- Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Table for Structure Cache
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#Structure') IS NOT NULL
				DROP TABLE #Structure;

			CREATE TABLE #Structure
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryKey					BIGINT											 NOT NULL
				,ProductLineKey				BIGINT											 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,ValueSeriesKey				BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #Structure
			-- must also return Factories / ProductLines without Products
				SELECT
					  dF.FactoryKey
					 ,ISNULL(dPL.ProductLineKey	,-1)					AS ProductLineKey
					 ,ISNULL(dP.ProductKey		,-1)					AS ProductKey
					 ,ISNULL(dVS.ValueSeriesKey	,-1)					AS ValueSeriesKey
					 ,dF.FactoryID		
					 ,ISNULL(dPL.ProductLineID	,'-1')					AS ProductLineID
					 ,ISNULL(dP.ProductID		,'-1')					AS ProductID
					 ,ISNULL(dVS.ValueSeriesID	,'-1')					AS ValueSeriesID
					 ,dF.NameShort										AS FactoryNameShort
					 ,ISNULL(dPL.NameShort	,'<without ProductLine>')	AS ProductLineNameShort
					 ,ISNULL(dP.NameShort	,'<without Product>')		AS ProductNameShort
					 ,ISNULL(dVS.NameShort	,'<without ValueSeries>')	AS ValueSeriesNameShort

			  FROM  planning.tdFactories dF

					LEFT JOIN planning.tdProductLines dPL
						ON dF.FactoryKey = dPL.FactoryKey

					LEFT JOIN planning.tdProducts dP
						ON dPL.ProductLineKey = dP.ProductLineKey

					LEFT JOIN planning.tdValueSeries dVS
						ON dP.ProductKey = dVS.ProductKey
				
			
		-- Helper Table for Globalattributes
		-- ****************************************************************************************************************************************************
			IF OBJECT_ID('tempdb..#GA') IS NOT NULL
				DROP TABLE #GA;

			CREATE TABLE #GA
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Globalattribute			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Content					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #GA
				SELECT 
						 ProductKey
						,FactoryID
						,ProductLineID
						,ProductID
						,UnpivotTable.Globalattribute
						,UnpivotTable.Content

				FROM
					planning.tdProducts
		
				UNPIVOT
					(	

						Content FOR Globalattribute IN (
													 Globalattribute1,Globalattribute2,Globalattribute3,Globalattribute4,Globalattribute5
													,Globalattribute6,Globalattribute7,Globalattribute8,Globalattribute9,Globalattribute10
													,Globalattribute11,Globalattribute12,Globalattribute13,Globalattribute14,Globalattribute15
													,Globalattribute16,Globalattribute17,Globalattribute18,Globalattribute19,Globalattribute20
													,Globalattribute21,Globalattribute22,Globalattribute23,Globalattribute24,Globalattribute25
												)
					) AS UnpivotTable

				WHERE Content <>''


		-- Table for Search Result Collection
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#SearchResult') IS NOT NULL
				DROP TABLE #SearchResult;

			CREATE TABLE #SearchResult
			(
				 RowKey					BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resultgroup			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resulttype				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,DataFactoryPath		NVARCHAR (1000)			COLLATE DATABASE_DEFAULT NOT NULL
				,Result					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)
		-- Getting Content
		-- ****************************************************************************************************************************************************
		
		-- Factories BEGIN #####

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'FactoryID'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.FactoryID
			FROM planning.tdFactories dF
			WHERE dF.FactoryID LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'NameShort'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.NameShort
			FROM planning.tdFactories dF
			WHERE dF.NameShort LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'ResponsiblePerson'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.ResponsiblePerson
			FROM planning.tdFactories dF
			WHERE dF.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Factories END #####

		-- ProductLines BEGIN #####
		
		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'ProductLineID'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ProductLineID
			FROM planning.tdProductLines dPL
					LEFT JOIN (
								SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
							   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ProductLineID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.NameShort
			FROM planning.tdProductLines dPL
					LEFT JOIN (
									SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
								   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ResponsiblePerson
			FROM planning.tdProductLines dPL
					LEFT JOIN (
						SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
						) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- ProductLines END #####

		-- Products BEGIN ####
		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ProductID'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ProductID
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.ProductID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.NameShort
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ResponsiblePerson
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.ResponsiblePerson LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'CommmentUser'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,LEFT(dP.CommentUser,250)+' ...'
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.CommentUser LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,GA.Globalattribute
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,GA.Content
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
					LEFT JOIN #GA GA
						ON dP.ProductKey = GA.ProductKey
			WHERE GA.Content LIKE @SearchString

		-- Documents on Products
		INSERT INTO #SearchResult
			SELECT tF.FactoryID
			      ,tF.ProductLineID
				  ,tF.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'Documents'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + ' (' + tF.FactoryID + '>'+tF.ProductLineID + '>' + tF.ProductID + ')' AS Path
				  ,tF.[FileName]
			FROM planning.tFiles tF
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON tF.ProductLineKey = S.ProductLineKey
			WHERE tf.[FileName] LIKE @SearchString


		-- Products END ####

		-- ValueSeries Start ####
		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'ValueSeriesID'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.ValueSeriesID
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.ValueSeriesID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameShort
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameLong'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameLong
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameLong LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'CommentUser'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,LEFT(dVS.CommentUser,250)+' ...'
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.CommentUser LIKE @SearchString

		-- ValueSeries END ####

		-- Value Start ###
		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueText'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueText,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductLineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueText LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueComment'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueComment,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductLineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueComment LIKE @SearchString

			
		-- Output

		SELECT TOP (100)
			   sr.FactoryID
		      ,sr.ProductLineID
			  ,sr.ProductID
			  ,sr.ValueSeriesID
			  ,sr.Resultgroup
			  ,sr.Resulttype
			  ,sr.DataFactoryPath
			  ,sr.Result
		FROM #SearchResult sr

			INNER JOIN 
				(	
					SELECT *
					FROM system.trUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						(
							FactoryID <> '' AND ProductLineID <> ''
							OR
							FactoryID <> '' AND ProductLineID = ''
						)
				) vUR

				ON 
						sr.FactoryID = vUR.FactoryID 
					AND (sr.ProductLineID = vUR.ProductLineID OR sr.ProductLineID = '-1') -- as Output for not existing ProductLine possible

		ORDER BY CASE sr.Resultgroup 
						WHEN 'Factory'		THEN 1
						WHEN 'ProductLine'	THEN 2
						WHEN 'Product'		THEN 3
						WHEN 'ValueSeries'	THEN 4
						WHEN 'Values'		THEN 5
						ELSE 99
				 END
					
				,sr.Resulttype,sr.FactoryID,sr.ProductLineID,sr.ProductID,sr.ValueSeriesID

		SET @SearchDoneFlag = 1

		SET @ResultCode = 200

		COMMIT TRANSACTION sx_pf_GET_GlobalSearch;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_GlobalSearch;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_GlobalSearch'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving full text search results for a given textstring
				 Search will be done in all DataFactory Elements';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS sx_pf_GET_HTMLPage;
GO

/*
	GET Operation for HTML Page
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = sx_pf_GET_HTMLPage 
			@Username		= 'W10\Admin',
			@FactoryID		= '-1', 
			@ProductLineID	= '-1', 
			@ProductID		= '-1',
			@PageID			= 'Page1'
		PRINT @RC

		SELECT * FROM planning.tgHTMLPages

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPage','PARAMETER',NULL)
*/

CREATE PROCEDURE sx_pf_GET_HTMLPage
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PageID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @PageKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PageID,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PageID			IS NULL SET @PageID			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPage;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PageID			= dbo.sx_pf_pProtectString	(@PageID);

		IF @PageID		= N'' 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END 


		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
		
		SELECT @PageKey = PageKey FROM planning.tgHTMLPages
		WHERE
				FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND PageID			= @PageID

		IF @PageKey = 0 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;
			
		-- Check rights on different levels 
			-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @FactoryKey > 0 AND @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
			,PageContent
		
		FROM planning.tgHTMLPages 
		WHERE PageKey = @PageKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_HTMLPage'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for HTML Page-only used up to Version 5.4.2, than replaced by planning.tTabs';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageID';
    SET @value      = N'PageID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_HTMLPages;
GO

/*
	GET Operation for a List of all HTML Pages
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_HTMLPages 
			@Username		= 'SQL',
			@FactoryID		= '', 
			@ProductLineID	= '', 
			@ProductID		= '';
		PRINT @RC
		SELECT * FROM planning.tgHTMLPages
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPages',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPages','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_HTMLPages 
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPages;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
			
		-- Check rights on different levels 
		-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @FactoryKey > 0 AND @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
		
		FROM planning.tgHTMLPages;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPages;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPages;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_HTMLPages TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_HTMLPages TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_HTMLPages' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for a List of all HTML Pages';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_AllGlobalAttributeUsage;
GO

/*
	GET Operation for cross query of ALL attributes usage

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_AllGlobalAttributeUsage 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_AllGlobalAttributeUsage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_AllGlobalAttributeUsage','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_AllGlobalAttributeUsage
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		;WITH dt AS (
			SELECT GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
				GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
				GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
				GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
				GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25
			FROM planning.tdProductLines dPL LEFT JOIN system.trUserRights vUR 
				ON	dPL.FactoryID = vUR.FactoryID AND
					dPL.ProductLineID = vUR.ProductLineID AND
					vUR.[Right] IN (N'Read', N'Write') AND
					vUR.UserName = @TransactUsername
		)
		, attrs AS (
			SELECT DISTINCT Attribute, Value 
			FROM 
				(SELECT * FROM dt) pvt
					UNPIVOT ([Value] FOR Attribute IN (
					GlobalattributeAlias1, GlobalattributeAlias2, GlobalAttributeAlias3, GlobalAttributeAlias4, GlobalattributeAlias5, 
					GlobalAttributeAlias6, GlobalattributeAlias7, GlobalattributeAlias8, GlobalattributeAlias9, GlobalattributeAlias10, 
					GlobalattributeAlias11, GlobalattributeAlias12, GlobalAttributeAlias13, GlobalAttributeAlias14, GlobalattributeAlias15, 
					GlobalAttributeAlias16, GlobalattributeAlias17, GlobalattributeAlias18, GlobalattributeAlias19, GlobalattributeAlias20,
					GlobalattributeAlias21, GlobalattributeAlias22, GlobalAttributeAlias23, GlobalAttributeAlias24, GlobalattributeAlias25)
				) AS unpvt
		)
		SELECT CAST(SUBSTRING(Attribute, 21, 2) AS INT) AS N, Value AS GlobalAttribute
		FROM attrs
		ORDER BY N, GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_AllGlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_AllGlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_AllGlobalAttributeUsage TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_AllGlobalAttributeUsage'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for cross query of ALL attributes usage';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_CustomFailureReport;
GO

/*
	GET Operation for CustomFailureReport
	- Can be called as Factory Tab
	- Can be called on ZT Factory Tab for all Factories with rights
	- Returns always at least one row

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_CustomFailureReport 'SQL','ZT'
		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_CustomFailureReport',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_CustomFailureReport','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_CustomFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_CustomFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             CFR.Timestamp								AS LastCheckDate
            ,CFR.FactoryID
			,COALESCE(
						 CFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,CFR.ProductLineID
			,COALESCE(	
						 CFR.ProductLineID + ' ' + dPL.NameShort
						,'All ProductLines'
					 )
														AS ProductLineIDName
            ,CFR.ProductID
			,COALESCE(
						 CFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,CFR.FactoryID + ' / ' + CFR.ProductLineID +' / ' + CFR.ProductID AS DataFactoryPath
            ,CFR.TestSource
            ,CFR.TestName
			,CFR.ResultType
            ,CFR.ResultText
            ,CFR.ResultNumeric
		FROM control.tCustomFailureReport CFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM system.trUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM system.trUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM system.trUserRights GROUP BY UserName
						) vUR 

				ON	CFR.FactoryID		= vUR.FactoryID
				AND CFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN planning.tdFactories dF 
				ON	CFR.FactoryID		= dF.FactoryID

			-- for ProductLine Name
			LEFT JOIN planning.tdProductLines dPL
				ON	CFR.FactoryID		= dPL.FactoryID
				AND CFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN planning.tdProducts dP
				ON	CFR.FactoryID		= dP.FactoryID
				AND CFR.ProductLineID	= dP.ProductLineID
				AND CFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			CFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tCustomFailureReport
							)
			-- called for one or all Factories
			AND (
					CFR.FactoryID = @FactoryID OR @FactoryID = '' OR CFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN
					SELECT 
					  GETUTCDATE()					AS LastCheckDate
					,'All'							AS FactoryID
					,'All Factories'				AS FactoryIDName
					,'All'							AS ProductLineID
					,'All ProductLines'				AS ProductLineIDName
					,'All'							AS ProductID
					,'All Products'					AS ProductIDName
					,'All / All / All'				AS DataFactoryPath
					,'Hint'							AS TestSource
					,'Default Entry'				AS Testname
					,'OK'							AS ResultType
					,'Keine Einträge vorhanden.'	AS ResultText
					,0								AS ResultNumeric
			END
			

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_CustomFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_CustomFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_CustomFailureReport'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for CustomFailureReport
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights
- Returns always at least one row';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_GlobalAttributeUsage;
GO

/*
	GET Operation for cross query of attribute usage

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC  NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_GlobalAttributeUsage 'SQL','1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_GlobalAttributeUsage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_GlobalAttributeUsage','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_GlobalAttributeUsage
	@Username				NVARCHAR(255),
	@GlobalattributeNumber	INT
AS 
BEGIN
	SET NOCOUNT ON;
	SET ANSI_WARNINGS OFF;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=	N'''' 
			+		ISNULL(@Username,				N'NULL') + N''',' 
			+ CAST(	ISNULL(@GlobalattributeNumber,	N'NULL') AS NVARCHAR(255));
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_GlobalAtt;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		SET NOCOUNT OFF;
		SET ANSI_WARNINGS ON;

		-- STEP 3. Show public values	
		SELECT DISTINCT
		CASE @GlobalattributeNumber
			WHEN 1 THEN GlobalattributeAlias1
			WHEN 2 THEN GlobalattributeAlias2
			WHEN 3 THEN GlobalAttributeAlias3
			WHEN 4 THEN GlobalAttributeAlias4
			WHEN 5 THEN GlobalattributeAlias5
			WHEN 6 THEN GlobalAttributeAlias6
			WHEN 7 THEN GlobalattributeAlias7
			WHEN 8 THEN GlobalattributeAlias8
			WHEN 9 THEN GlobalattributeAlias9
			WHEN 10 THEN GlobalattributeAlias10
			WHEN 11 THEN GlobalattributeAlias11
			WHEN 12 THEN GlobalattributeAlias12
			WHEN 13 THEN GlobalAttributeAlias13
			WHEN 14 THEN GlobalAttributeAlias14
			WHEN 15 THEN GlobalattributeAlias15
			WHEN 16 THEN GlobalAttributeAlias16
			WHEN 17 THEN GlobalattributeAlias17
			WHEN 18 THEN GlobalattributeAlias18
			WHEN 19 THEN GlobalattributeAlias19
			WHEN 20 THEN GlobalattributeAlias20		
			WHEN 21 THEN GlobalattributeAlias21
			WHEN 22 THEN GlobalattributeAlias22
			WHEN 23 THEN GlobalAttributeAlias23
			WHEN 24 THEN GlobalAttributeAlias24
			WHEN 25 THEN GlobalattributeAlias25
			ELSE N'not existing' END AS GlobalAttribute	
		FROM planning.tdProductLines dPL LEFT JOIN system.trUserRights vUR 
			ON	dPL.FactoryID = vUR.FactoryID AND
				dPL.ProductLineID = vUR.ProductLineID AND
				vUR.[Right] IN (N'Read',N'Write') AND
				vUR.UserName = @TransactUsername
		ORDER BY GlobalAttribute;

		SET @EffectedRows = @@ROWCOUNT;		
		SET @Resultcode = 200;

		COMMIT TRANSACTION sx_pf_GET_Info_GlobalAtt;
		SET NOCOUNT ON;
		SET ANSI_WARNINGS OFF;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_GlobalAtt;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_GlobalAttributeUsage TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_GlobalAttributeUsage'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for cross query of attribute usage';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_Monitoring;
GO

/*
	GET Operation for monitoring Properties

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Test call
		DECLARE @RC NVARCHAR(255);
		EXEC @RC = dbo.sx_pf_GET_Info_Monitoring 'SQL';
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_Monitoring',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_Monitoring','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_Monitoring
	@Username NVARCHAR(255)

WITH EXECUTE AS 'dbo' -- as OCTS owned procedure needs access to system table

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_Monitoring;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3 - Select Values

		IF OBJECT_ID('tempdb..#temp') IS NOT NULL
			DROP TABLE #temp;

		CREATE TABLE #temp (
			Property NVARCHAR (255) NOT NULL
			,[Value] NVARCHAR (255) NOT NULL
		);

		INSERT INTO #temp
			-- Language
			SELECT 'Cluster_C03' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C03'
			UNION ALL
			-- Clustername
			SELECT 'Cluster_C01' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C01'	
			UNION ALL
			-- API Version
			SELECT 'Cluster_C04' AS Property, ValueText FROM planning.tgCluster WHERE PropertyID = 'C04'	
			UNION ALL
			-- FactoryCount
			SELECT 'FactoryCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdFactories
			UNION ALL
			-- ProductLineCount
			SELECT 'ProductLineCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdProductLines
			UNION ALL
			-- ProductCount
			SELECT 'ProductCount' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM planning.tdProducts
			UNION ALL
			-- API Error 500 Count
			SELECT 'API_Error500' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM system.tAPI_Log WHERE ReturnCode = 500 AND ProcessCode = ''
			UNION ALL
			-- API Error 404 Count
			SELECT 'API_Error404' AS Property, CAST(COUNT(*) AS NVARCHAR) FROM system.tAPI_Log WHERE ReturnCode = 404 AND ProcessCode = '';

		SELECT * FROM #temp;
		
		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_Monitoring;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_Monitoring;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_Monitoring TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_Monitoring'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'					-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'						-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1								-- 0 = Demo object - out of shipment process
																		-- STANDARD OBJECTS
																		-- 1 = shiped from saxess standard without modification
																		-- 2 = shiped from saxess standard modified FOR customer from saxess
																		-- 3 = shiped from saxess standard modified FOR customer from partner
																		-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		-- CUSTOM OBJECTS
																		-- 10 = shiped from saxess as customer specific object
																		-- 11 = shiped from partner	as customer specific object
																		-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''							-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for monitoring Properties';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_NextFreeFactoryID;
GO

/*
	GET Operation for the next free FactoryID
	Returns the next numeric ID or the number 1 if other FactoryIDs are non-numeric

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_NextFreeFactoryID 
			@Username		= 'SQL'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeFactoryID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeFactoryID','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeFactoryID
	@Username		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FreeFactoryID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(FactoryID AS BIGINT) id 
			FROM planning.tdFactories
		)
		SELECT @FreeFactoryID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeFactoryID IS NOT NULL
		BEGIN
			SET @FreeFactoryID = @FreeFactoryID + 1;
		END
		ELSE
		BEGIN
			SET @FreeFactoryID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT FactoryID FROM planning.tdFactories WHERE FactoryID = @FreeFactoryID)
		BEGIN
			SET @FreeFactoryID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeFactoryID AS NextFreeFactoryID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeFactoryID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeFactoryID TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_NextFreeFactoryID'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for the next free FactoryID
				 Returns the next numeric ID or the number 1 if other FactoryIDs are non-numeric';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
  
	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_NextFreeProductID;
GO

/*
	GET Operation for the next free ProductID
	Returns for numeric ID the next number for ProductLines with only string IDs the number 1 

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_NextFreeProductID 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'V'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeProductID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeProductID','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FreeProductID		NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreeProductID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		
		IF @ProductLineKey = 0 OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductID AS BIGINT) id 
			FROM planning.tdProducts 
			WHERE ProductLineKey = @ProductLineKey 
		)
		SELECT @FreeProductID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductID IS NOT NULL
		BEGIN
			SET @FreeProductID = @FreeProductID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductID FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @FreeProductID)
		BEGIN
			SET @FreeProductID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductID AS NextFreeProductID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreeProductID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreeProductID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryService; 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductID TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_NextFreeProductID'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for the next free ProductID
				 Returns for numeric ID the next number for ProductLines with only string IDs the number 1 ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_NextFreeProductLineID;
GO

/*
	GET Operation for the next free ProductLineID
	Returns the next numeric ID or the number 1 if other ProductLineIDs are non-numeric

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_NextFreeProductLineID 
			@Username		= 'SQL',
			@FactoryID		= '1'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeProductLineID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_NextFreeProductLineID','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_NextFreeProductLineID
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @FreeProductLineID	NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''''

	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_NextFreePLID;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);

		IF	@FactoryID			= N'' 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16 ,10);
			END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		
		IF @FactoryKey = 0
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		
		IF @ResultCode <> 200	
		BEGIN
			RAISERROR('Invalid rights', 16, 10)
		END;
		
		-- STEP 3.1 - Get last determined Numeric ID	
		;WITH ids AS (
			SELECT TRY_CAST(ProductLineID AS BIGINT) id 
			FROM planning.tdProductLines 
			WHERE FactoryKey = @FactoryKey 
		)
		SELECT @FreeProductLineID = MAX(id) 
		FROM ids
		WHERE id IS NOT NULL;

		-- STEP 3.2 - Increment ID
		IF @FreeProductLineID IS NOT NULL
		BEGIN
			SET @FreeProductLineID = @FreeProductLineID + 1;
		END
		ELSE
		BEGIN
			SET @FreeProductLineID = N'1';
		END;

		-- STEP 3.3 - Safety check whether determined ID already exists
		IF EXISTS(SELECT ProductLineID FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @FreeProductLineID)
		BEGIN
			SET @FreeProductLineID = N'';
		END
		ELSE
		BEGIN
			SELECT @FreeProductLineID AS NextFreeProductLineID;
		END

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_NextFreePLID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_NextFreePLID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END 
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_NextFreeProductLineID TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_NextFreeProductLineID'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for the next free ProductLineID
				 Returns the next numeric ID or the number 1 if other ProductLineIDs are non-numeric';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_SystemFailureReport;
GO

/*
	GET Operation for SystemFailureReport
	- Can be called as Factory Tab
	- Can be called on ZT Factory Tab for all Factories with rights

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_SystemFailureReport 'SQL','ZT'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_SystemFailureReport',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_SystemFailureReport','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_SystemFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_SystemFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             SFR.Timestamp								AS LastSystemCheckDate
            ,SFR.FactoryID
			,COALESCE(
						 SFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,SFR.ProductLineID
			,COALESCE(	
						 SFR.ProductLineID + ' ' + dPL.NameShort
						,'All ProductLines'
					 )
														AS ProductLineIDName
            ,SFR.ProductID
			,COALESCE(
						 SFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,SFR.FactoryID + ' / ' + SFR.ProductLineID +' / ' + SFR.ProductID AS DataFactoryPath
            ,SFR.TestSource
            ,SFR.TestName
			,SFR.ResultType
            ,SFR.ResultText
            ,SFR.ResultNumeric
		FROM control.tSystemFailureReport SFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM system.trUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM system.trUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM system.trUserRights GROUP BY UserName
						) vUR 

				ON	SFR.FactoryID		= vUR.FactoryID
				AND SFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN planning.tdFactories dF 
				ON	SFR.FactoryID		= dF.FactoryID

			-- for ProductLine Name
			LEFT JOIN planning.tdProductLines dPL
				ON	SFR.FactoryID		= dPL.FactoryID
				AND SFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN planning.tdProducts dP
				ON	SFR.FactoryID		= dP.FactoryID
				AND SFR.ProductLineID	= dP.ProductLineID
				AND SFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			SFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tSystemFailureReport
							)
			-- called for one or all Factories
			AND (
					SFR.FactoryID = @FactoryID OR @FactoryID = '' OR SFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_SystemFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_SystemFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_SystemFailureReport TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_SystemFailureReport'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for SystemFailureReport
				- Can be called as Factory Tab
				- Can be called on ZT Factory Tab for all Factories with rights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_UserRights;
GO

/*
	GET Operation for UserRight Overview
	@FactoryID, @ProductLineID only Dummy Parameter to call over csv Route
	Works only for Clusteradmins

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_UserRights 'SQL'
		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_UserRights',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_UserRights','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_UserRights
	@Username NVARCHAR(255),@FactoryID NVARCHAR(255)='',@ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Info_UserRights;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			BEGIN
				SELECT 
					 'None'												AS UserName
					,'None'												AS Status
					,'None'												AS FactoryID
					,'None'												AS FactoryName
					,'None'												AS ProductLineID
					,'None'												AS ProductLineName
					,'You need Cluster Admin privileges to see this.'	AS [Right]
					,-1													AS RightLevel;
			END;
		ELSE
			BEGIN
				SELECT 
					 rU.UserName
					,rU.Status	
					,dPL.FactoryID
					,dF.NameShort				AS FactoryName
					,dPL.ProductLineID
					,dPL.NameShort				AS ProductLineName
					,ISNULL(vUR.[Right],'Deny') AS UserRight
					,CASE vUR.[Right]
						WHEN 'Read'		THEN 0
						WHEN 'Write'	THEN 1
						ELSE -1
					 END							AS RightLevel
				FROM system.trUser rU
					-- for full ProductLine Structure
					LEFT JOIN planning.tdProductLines dPL ON
						 1=1
					-- for FactoryName
					LEFT JOIN planning.tdFactories dF ON
						dPL.FactoryKey = dF.FactoryKey
					-- for positiv Rights
					LEFT JOIN system.trUserRights vUR	ON
							rU.UserKey			= vUR.UserKey
						AND vUR.FactoryID		= dPL.FactoryID
						AND vUR.ProductLineID	= dPL.ProductLineID;
			END;

			SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Info_UserRights;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Info_UserRights;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_UserRights TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_UserRights TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_UserRights'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for UserRight Overview
				@FactoryID, @ProductLineID only Dummy Parameter to call over csv Route
				Works only for Clusteradmins';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO


DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_List;
GO

/*
	GET Operation for a List 
	Shows the properties of one List
	This procedure don't shows internal system lists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_List 
			@Username	= 'SQL', 
			@ListID		= 'sxStatus';

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_List
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL	SET @Username	= N'';
	IF @ListID		IS NULL	SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_List;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine Keys
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

		IF @ListKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM planning.thLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_List TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_List TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_List'      -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for a List 
				Shows the properties of one List
				This procedure dont shows internal system lists';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Lists
GO

/*
	GET Operation for all GUI Lists in Cluster
	Internal system lists are not shown

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call 
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Lists 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Lists',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Lists','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Lists
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(10)   = N'';
	DECLARE @TransactUsername	NVARCHAR(255)  = N'';
	DECLARE @ProcedureName		NVARCHAR(255)  = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)  = N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT			   = 0;						
	DECLARE @ResultCode			INT			   = 501;					
	DECLARE @TimestampCall		DATETIME	   = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Lists;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END ELSE BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT 
			ListID
			,NameShort
			,NameLong
			,CommentDev
			,CommentUser
			,Datentyp
			,Source
			,SourceFormula
			,FormatID
			,@Right AS 'Right'
		FROM planning.thLists
		ORDER BY ISNULL(TRY_CAST(ListID AS INT), 999999999), ListID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Lists;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Lists;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Lists TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Lists TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Lists'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all GUI Lists in Cluster
				Internal system lists are not shown';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ListValues
GO

/*
	GET Operation for all List Values
	- shows values of all GUI Lists
	- shows values of the internal list "sxScales", as this list is need for the other lists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ListValues 
			@Username	= 'SQL', 
			@ListID		= 'sxEvents'

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ListValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ListValues','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ListValues
	@Username	NVARCHAR(255),
	@ListID		NVARCHAR(255)
AS 
BEGIN
	DECLARE @ListKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username, N'NULL')		+ N''',''' 
			+ ISNULL(@ListID,	N'NULL')		+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @ListID		IS NULL SET @ListID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @ListID		= dbo.sx_pf_pProtectID		(@ListID);

		IF @ListID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;


		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		IF @ListID != 'sxScales'
		BEGIN
			-- STEP 1.2 - Determine Keys
			SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

			IF @ListKey = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;

			-- STEP 3. Show public values, always one dummy line, if list exists	
			SELECT 
				ISNULL(hLV.ListValueKey, 0)		AS ListValueKey  --returns the Key as the Listvalue has no ID
				,hL.ListID
				,hL.Datentyp
				,ISNULL(hLV.ValueInt, 0)		AS ValueInt
				,ISNULL(hLV.Scale, 0)			AS Scale
				,ISNULL(hLV.ValueText, N'')		AS ValueText
				,ISNULL(hLV.ValueComment, N'')	AS ValueComment
				,ISNULL(hLV.FormatID, N'')		AS FormatID
				,@Right AS 'Right'
			FROM planning.thLists hL
				LEFT JOIN planning.thListValues hLV	ON hL.ListID = hLV.ListID
			WHERE hL.ListID = @ListID 
			ORDER BY ISNULL(TRY_CAST(hLV.ValueText AS INT), 999999999), hLV.ValueText, hLV.ValueInt;

			SET @EffectedRows = @@ROWCOUNT;
		END;

		-- static definition of sxScales to be loaded for list defintion
		IF @ListID = 'sxScales'
		BEGIN
			SELECT * 
			FROM (
				VALUES	
					(1,'sxScales' ,'INT',1    ,1,'','','','Read')
					,(2,'sxScales','INT',10   ,1,'','','','Read')
					,(3,'sxScales','INT',100  ,1,'','','','Read')
					,(4,'sxScales','INT',1000 ,1,'','','','Read')
					,(5,'sxScales','INT',10000,1,'','','','Read')
			) tmp (ListValueKey, ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, [Right])

			SET @EffectedRows = @@ROWCOUNT;
		END;

		COMMIT TRANSACTION sx_pf_GET_ListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ListValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ListValues TO pf_PlanningFactoryService; 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ListValues TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ListValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all List Values
				- shows values of all GUI Lists
				- shows values of the internal list "sxScales", as this list is need for the other lists';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Preference;
GO

/*
	GET Operation for Prefences
	- Preferences may exist for the Cluster, for Factories or for ProductLines - all in the same table
	- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty FactoryID or empty ProductLineID => 404 Not Found
	2. Username non Cluster Admin => 403 Forbidden
	3. User does not have rights to read ProductLine => 401 Unauthorized
	5. No exists Factory / ProductLine > 404 Not Found
		DECLARE  @RC					INT
				,@Username				NVARCHAR(255) = 'SQL'
				,@PreferenceUserName	NVARCHAR(255) = 'W10\admin'
				,@ProductLineID			NVARCHAR(255) = 'U'
				,@FactoryID				NVARCHAR(255) = 'ZT'
				,@SettingName			NVARCHAR(255) = 'Demopref'

		EXECUTE @RC = dbo.sx_pf_GET_Preference 
				@Username,
				@PreferenceUsername,
				@ProductLineID,
				@FactoryID,
				@SettingName

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Preference',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Preference','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Preference
	 @Username				NVARCHAR(255)
	,@PreferenceUsername	NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@SettingName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUsername					NVARCHAR(255)= N''
			,@FactoryKey						INT = 0
			,@ProductLineKey					INT = 0
			,@UserExistsFlag					INT = 0
			,@PreferenceIsUserDependendFlag		INT = 0
			,@ProcedureName						NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			,@ParameterString					NVARCHAR(MAX) = N''''
							+ ISNULL(@Username, N'NULL')			+ N''',''' 
							+ ISNULL(@PreferenceUsername, N'NULL')	+ N''',''' 
							+ ISNULL(@ProductLineID, N'NULL')		+ N''',''' 
							+ ISNULL(@FactoryID, N'NULL')			+ N''','''
							+ ISNULL(@SettingName, N'NULL')			+ N''''
			
			,@EffectedRows						INT = 0					
			,@ResultCode 						INT = 501						
			,@TimestampCall						DATETIME = GETUTCDATE()
			,@Comment							NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_Preference;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername = dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName		= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @SettingName		= N'' 
			OR @SettingName		= N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.3 - Determine, if Preferences are saved per user

		-- for Clusterpreferences
		IF @FactoryID = N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = N'C06';
			END 
		-- for Factorypreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = N'F06';
			END
		-- for ProductLinePreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = N'P06';
			END;
		
		-- STEP 1.1 - Determine Keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF (@FactoryKey = 0 AND @FactoryID <> N'')
			OR (@ProductLineKey = 0 AND @ProductLineID <> N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Change PreferenceUser if Preferences User-independend
		IF @PreferenceIsUserDependendFlag <> 1
		BEGIN 
			SET @PreferenceUsername = N'All';
		END;

		--- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights - TODO: depdendend from preference type
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT 
			 UserName
			,FactoryID
			,ProductLineID
			,SettingName
			,ValueInt
			,ValueText
		FROM planning.trPreferences 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryUser;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Preference'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Prefences
				- Preferences may exist for the Cluster, for Factories or for ProductLines - all in the same table
				- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PreferenceUsername';
    SET @value      = N'PreferenceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SettingName';
    SET @value      = N'SettingName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Product;
GO

/*
	GET Operation for Product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Product 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U', 
			@ProductID		= '1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Product
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Product;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID = dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID = dbo.sx_pf_pProtectID (@ProductLineID);	
		SET @ProductID = dbo.sx_pf_pProtectID (@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,TimeType
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,[Status]
			,Template
			,TemplateVersion
			,GlobalAttribute1
			,GlobalAttribute2
			,GlobalAttribute3
			,GlobalAttribute4
			,GlobalAttribute5
			,GlobalAttribute6
			,GlobalAttribute7
			,GlobalAttribute8
			,GlobalAttribute9
			,GlobalAttribute10
			,GlobalAttribute11
			,GlobalAttribute12
			,GlobalAttribute13
			,GlobalAttribute14
			,GlobalAttribute15
			,GlobalAttribute16
			,GlobalAttribute17
			,GlobalAttribute18
			,GlobalAttribute19
			,GlobalAttribute20
			,GlobalAttribute21
			,GlobalAttribute22
			,GlobalAttribute23
			,GlobalAttribute24
			,GlobalAttribute25
			,@Right AS 'Right'
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
		FROM planning.tdProducts dP LEFT JOIN system.trUserRights vUR 
			ON  dP.FactoryID = vUR.FactoryID AND
				dP.ProductLineID = vUR.ProductLineID AND
				vUR.UserName = @TransactUsername
		WHERE dP.ProductKey = @ProductKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Product TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Product TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Product'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductDataTableValues;
GO

/*
	GET Operation for all Values of ProductDataTable

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductDataTableValues
			@Username		 = 'SQL',
			@FactoryID		 = 'ZT',
			@ProductLineID	 = 'U',
			@ProductID		 = '1';
		PRINT @RC

		--should return 404
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductDataTableValues 
			@Username		 = 'SQL',
			@FactoryID		 = 'ZT2',
			@ProductLineID	 = 'U',
			@ProductID		 = '1';
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductDataTableValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductDataTableValues','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductDataTableValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID  NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;			-- SET during Execution
	DECLARE @ResultCode			INT				= 501;			-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			-- SET during Execution
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductDataTableValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 fV.ProductID
			,fV.ProductLineID
			,fV.FactoryID
			,fV.ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right	AS 'Right'
		FROM planning.tfValues fV 
			LEFT JOIN planning.tdValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey
		WHERE fV.ProductKey = @ProductKey
		ORDER BY ValueSeriesNo, TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductDataTableValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductDataTableValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductDataTableValues TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductDataTableValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all Values of ProductDataTable';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductLine;
GO

/*
	GET Operation for ProductLine
	Provides complete information about a Product Line

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductLine 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U' 
	
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductLine
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey	= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			ProductLineID 
			,FactoryID
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ResponsiblePerson
			,ImageName
			,DefaultTemplate
			,GlobalAttributeSource1
			,GlobalAttributeAlias1
			,GlobalAttributeSource2
			,GlobalAttributeAlias2
			,GlobalAttributeSource3
			,GlobalAttributeAlias3
			,GlobalAttributeSource4
			,GlobalAttributeAlias4
			,GlobalAttributeSource5
			,GlobalAttributeAlias5
			,GlobalAttributeSource6
			,GlobalAttributeAlias6
			,GlobalAttributeSource7
			,GlobalAttributeAlias7
			,GlobalAttributeSource8
			,GlobalAttributeAlias8
			,GlobalAttributeSource9
			,GlobalAttributeAlias9
			,GlobalAttributeSource10
			,GlobalAttributeAlias10

			,GlobalAttributeSource11
			,GlobalAttributeAlias11
			,GlobalAttributeSource12
			,GlobalAttributeAlias12
			,GlobalAttributeSource13
			,GlobalAttributeAlias13
			,GlobalAttributeSource14
			,GlobalAttributeAlias14
			,GlobalAttributeSource15
			,GlobalAttributeAlias15
			,GlobalAttributeSource16
			,GlobalAttributeAlias16
			,GlobalAttributeSource17
			,GlobalAttributeAlias17
			,GlobalAttributeSource18
			,GlobalAttributeAlias18
			,GlobalAttributeSource19
			,GlobalAttributeAlias19
			,GlobalAttributeSource20
			,GlobalAttributeAlias20

			,GlobalAttributeSource21
			,GlobalAttributeAlias21
			,GlobalAttributeSource22
			,GlobalAttributeAlias22
			,GlobalAttributeSource23
			,GlobalAttributeAlias23
			,GlobalAttributeSource24
			,GlobalAttributeAlias24
			,GlobalAttributeSource25
			,GlobalAttributeAlias25

			,@Right AS 'Right'
		FROM planning.tdProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductLine TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLine TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for ProductLine
				Provides complete information about a Product Line';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductLineProperties;
GO

/*
	GET Operation for determination of all properties of ProductLine

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductLineProperties 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U' 

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLineProperties',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLineProperties','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductLineProperties
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		
		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF	@ProductLineKey = 0 
			OR @FactoryKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProductLines
		WHERE 
			FactoryKey = @FactoryKey 
			AND ProductLineKey = @ProductLineKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperties TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductLineProperties'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determination of all properties of ProductLine';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductLineProperty;
GO

/*
	GET Operation for determining a ProductLine Property

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductLineProperty 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U', 
			@PropertyID		= 'PL01'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLineProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLineProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductLineProperty
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@PropertyID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLineProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID

		IF	@PropertyKey		= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLineProperty TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductLineProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determining a ProductLine Property';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductLines;
GO

/*
	GET Operation for ProductLines 
		A: returns the ProductLines for a single Factory
		B: returns all ProductLines when Factory '' is requested

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductLines 
			@Username	= 'SQL',
			@FactoryID	= 'ZT'
		PRINT @RC

		EXEC @RC = dbo.sx_pf_GET_ProductLines 
			@Username = 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLines',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLines','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductLines
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'' ;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLines;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys if single Factory is requested
		IF @FactoryID <> ''
			BEGIN
				SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

				IF @FactoryKey = 0	
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Keys don`t exists', 16, 10);
				END;
			END

		-- STEP 3. Show public values	
		;WITH vUR AS (
			SELECT 
				FactoryID, 
				ProductLineID, 
				[Right]
			FROM system.trUserRights 
			WHERE ProductLineID <> N'' 
				AND Username = @TransactUsername 
				AND	[Right] IN (N'Read', N'Write')
		)
		SELECT  
			 dPL.ProductLineID 
			,dPL.FactoryID
			,dPL.NameShort
			,dPL.NameLong
			,dPL.CommentUser
			,dPL.CommentDev
			,dPL.ResponsiblePerson
			,dPL.ImageName
			,dPL.DefaultTemplate
			,dPL.GlobalAttributeSource1
			,dPL.GlobalAttributeAlias1
			,dPL.GlobalAttributeSource2
			,dPL.GlobalAttributeAlias2
			,dPL.GlobalAttributeSource3
			,dPL.GlobalAttributeAlias3
			,dPL.GlobalAttributeSource4
			,dPL.GlobalAttributeAlias4
			,dPL.GlobalAttributeSource5
			,dPL.GlobalAttributeAlias5
			,dPL.GlobalAttributeSource6
			,dPL.GlobalAttributeAlias6
			,dPL.GlobalAttributeSource7
			,dPL.GlobalAttributeAlias7
			,dPL.GlobalAttributeSource8
			,dPL.GlobalAttributeAlias8
			,dPL.GlobalAttributeSource9
			,dPL.GlobalAttributeAlias9
			,dPL.GlobalAttributeSource10
			,dPL.GlobalAttributeAlias10

			,dPL.GlobalAttributeSource11
			,dPL.GlobalAttributeAlias11
			,dPL.GlobalAttributeSource12
			,dPL.GlobalAttributeAlias12
			,dPL.GlobalAttributeSource13
			,dPL.GlobalAttributeAlias13
			,dPL.GlobalAttributeSource14
			,dPL.GlobalAttributeAlias14
			,dPL.GlobalAttributeSource15
			,dPL.GlobalAttributeAlias15
			,dPL.GlobalAttributeSource16
			,dPL.GlobalAttributeAlias16
			,dPL.GlobalAttributeSource17
			,dPL.GlobalAttributeAlias17
			,dPL.GlobalAttributeSource18
			,dPL.GlobalAttributeAlias18
			,dPL.GlobalAttributeSource19
			,dPL.GlobalAttributeAlias19
			,dPL.GlobalAttributeSource20
			,dPL.GlobalAttributeAlias20

			,dPL.GlobalAttributeSource21
			,dPL.GlobalAttributeAlias21
			,dPL.GlobalAttributeSource22
			,dPL.GlobalAttributeAlias22
			,dPL.GlobalAttributeSource23
			,dPL.GlobalAttributeAlias23
			,dPL.GlobalAttributeSource24
			,dPL.GlobalAttributeAlias24
			,dPL.GlobalAttributeSource25
			,dPL.GlobalAttributeAlias25

			,vUR.[Right]
		FROM planning.tdProductLines dPL INNER JOIN vUR
			ON dPL.FactoryID		= vUR.FactoryID AND
			   dPL.ProductLineID	= vUR.ProductLineID
		WHERE 
				dPL.FactoryKey = @FactoryKey 
			OR @FactoryID = ''
		ORDER BY ISNULL(TRY_CAST(dPL.ProductLineID AS BIGINT), 999999999), dPL.ProductLineID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_ProductLines;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLines;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductLines'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for ProductLines 
				A: returns the ProductLines for a single Factory
				B: returns all ProductLines when Factory '' is requested';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductProperties;
GO

/*
	GET Operation for identifying all Properties of Products

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductProperties
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U', 
			@ProductID		= '1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductProperties',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductProperties','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductProperties
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						-- SET during Execution
	DECLARE @ResultCode			INT				= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperties;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProducts
		WHERE ProductKey = @ProductKey
		ORDER BY ISNULL(TRY_CAST(PropertyID AS INT), 999999999), PropertyID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperties;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperties;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductProperties TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperties TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductProperties'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for identifying all Properties of Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductProperty;
GO

/*
	GET Operation for determining a ProductProperty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductProperty 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U', 
			@ProductID		= '1',
			@PropertyID		= 'P1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductProperty
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255),
	@PropertyID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PropertyID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductProperty;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''  
			OR @PropertyID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProducts WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;

		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0 
			OR @PropertyKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
			
		-- STEP 2.1 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			FactoryID
			,ProductLineID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
			,@Right AS 'Right'
		FROM planning.tgProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductProperty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductProperty'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for determining a ProductProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductRanges;
GO

/*
	Procedure to GET all ProductRanges for an Product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Samplescript to POST an ProductRange
		EXEC dbo.sx_pf_POST_ProductProperty 
			 @Username				= 'SQL'
			,@ProductID				= '1'
			,@ProductLineID			= 'U'
			,@FactoryID				= 'ZT'
			,@PropertyID			= 'PDTRange_1'				-- fixed Name PDTRange_1...N
			,@PropertyName			= 'PDTRangeProperty'		-- fixed Name PDTRangeProperty
			,@CommentUser			= ''
			,@CommentDev			= ''
			,@Unit					= ''
			,@ValueText				= 'K1:20170115:K2:20171215'	-- Range in DF Coordinates, separated with :
			,@ValueInt				= '1'						-- Protected 1 = TRUE
			,@Scale					= '1'
			,@IsROSystemproperty	= '0'
			,@FormatID				= 'rngRed'					-- Name of FormatID or '<#NV>' to set only Protection 
																-- the Format Property Border* will be ignored on Cell Level
		EXEC dbo.sx_pf_POST_ProductProperty 
			 @Username				= 'SQL'
			,@ProductID				= '1'
			,@ProductLineID			= 'U'
			,@FactoryID				= 'ZT'
			,@PropertyID			= 'PDTRange_2'				
			,@PropertyName			= 'PDTRangeProperty'		
			,@CommentUser			= ''
			,@CommentDev			= ''
			,@Unit					= ''
			,@ValueText				= 'K3:20180615:K3:20190115'	
			,@ValueInt				= '1'						
			,@Scale					= '1'
			,@IsROSystemproperty	= '0'
			,@FormatID				= 'rngRed'					
														
	Testcall
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductRanges 
			@Username		= 'SQL',
			@FactoryID		= 'ZT', 
			@ProductLineID	= 'U', 
			@ProductID		= '1'
			PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductRanges',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductRanges','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductRanges
	 @Username		NVARCHAR(255)
	,@FactoryID		NVARCHAR(255)
	,@ProductLineID	NVARCHAR(255)
	,@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(255)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		
			+ ISNULL(@Username,			N'NULL') + N''','''	
			+ ISNULL(@FactoryID,		N'NULL') + N''','''	
			+ ISNULL(@ProductLineID,	N'NULL') + N''','''	
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username		= N'';
	IF @FactoryID				IS NULL SET @FactoryID		= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID	= N'';
	IF @ProductID				IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_ProductRanges;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);	

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR  @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine Keys
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID;
		
		IF @ProductKey = 0 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 PropertyID
			,ValueText	AS DFRange
			,ValueInt	AS ProtectedFlag
			,FormatID
			,@Right		AS 'Right'
		FROM planning.tgProducts
		WHERE 
			    ProductKey = @ProductKey
			AND PropertyID LIKE 'PDTRange_%'; 

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_ProductRanges;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_ProductRanges;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductRanges TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductRanges TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductRanges'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to GET all ProductRanges for an Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Products;
GO

/*
	GET Operation for all Product of ProductLine

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 08/2022 for OCT 5.8

	Test call
		-- should return 200 and the Products of one ProductLine
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Products 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U'
		PRINT @RC

		-- should return 200 and the Products of one Factory
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Products 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= ''
		PRINT @RC

		-- should return 200 and the Products of all Factories
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Products 
			@Username		= 'SQL',
			@FactoryID		= '',
			@ProductLineID	= ''
		PRINT @RC

		-- should return 404
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Products 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Products',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Products','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Products
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Products;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys - maybe 0 if ProductLineID or FactoryID is empty
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		-- STEP 3. Show public values	
		SELECT 
			 dP.ProductID
			,dP.ProductLineID
			,dP.FactoryID
			,dP.TimeType
			,dP.NameShort
			,dP.NameLong
			,dP.CommentUser
			,dP.CommentDev
			,dP.ResponsiblePerson
			,dP.ImageName
			,dP.[Status]
			,dP.Template
			,dP.TemplateVersion
			,dP.GlobalAttribute1
			,dP.GlobalAttribute2
			,dP.GlobalAttribute3
			,dP.GlobalAttribute4
			,dP.GlobalAttribute5
			,dP.GlobalAttribute6
			,dP.GlobalAttribute7
			,dP.GlobalAttribute8
			,dP.GlobalAttribute9
			,dP.GlobalAttribute10
			,dP.GlobalAttribute11
			,dP.GlobalAttribute12
			,dP.GlobalAttribute13
			,dP.GlobalAttribute14
			,dP.GlobalAttribute15
			,dP.GlobalAttribute16
			,dP.GlobalAttribute17
			,dP.GlobalAttribute18
			,dP.GlobalAttribute19
			,dP.GlobalAttribute20
			,dP.GlobalAttribute21
			,dP.GlobalAttribute22
			,dP.GlobalAttribute23
			,dP.GlobalAttribute24
			,dP.GlobalAttribute25
			,vUR.[Right]				AS [Right]
			,vUR.ReadCommentMandatory
			,vUR.WriteCommentMandatory
			,(SELECT COUNT(FileID) FROM planning.tFiles tF WHERE dP.FactoryID = tF.FactoryID AND dP.ProductLineID = tF.ProductLineID AND dP.ProductID = tF.ProductID) AS FileCount
		FROM planning.tdProducts dP 
			LEFT JOIN system.trUserRights vUR 
				ON		dP.FactoryID = vUR.FactoryID 
					AND	dP.ProductLineID = vUR.ProductLineID 
					AND	vUR.UserName = @TransactUsername
		WHERE 
				(
					 dP.ProductLineKey = @ProductLineKey 
				OR  (dP.FactoryID = @FactoryID AND @ProductLineID = '')
				OR  (@FactoryID = '')
				)

				AND vUR.[Right] IS NOT NULL

		ORDER BY 
			 dP.FactoryID
			,dP.ProductLineID	
			,ISNULL(TRY_CAST(dP.ProductID AS INT), 999999999), dP.ProductID;

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_Products;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Products;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Products TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Products TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Products'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all Product of ProductLine';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Statements;
GO

/*
	GET Operation for all Statements of Products

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 08/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Statements 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1';
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Statements',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Statements','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Statements
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Statements;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ActionType
			,[Statement]
			,UserName
			,PCName
			,ProcessorCode
			,IPAddresses
			,[Timestamp]
			,IsDeleted
			,IsResolved
			,@Right AS 'Right'
		FROM planning.tfStatements
		WHERE ProductKey = @ProductKey 
		ORDER BY [Timestamp] DESC;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Statements;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Statements;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Statements TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Statements TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Statements'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all Statements of Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_TimeIDs;
GO

/*
	GET Operation for all TimeIDs of ValueSeries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_TimeIDs 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1';
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_TimeIDs',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_TimeIDs','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_TimeIDs
	@Username NVARCHAR(255),
	@FactoryID NVARCHAR(255),
	@ProductLineID NVARCHAR(255),
	@ProductID NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,		N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@ProductID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_TimeIDs;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		
		IF	@ProductKey			= 0 
			OR @ProductLineKey	= 0 
			OR @FactoryKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;		
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		FROM planning.tdTime
		WHERE ProductKey = @ProductKey 
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_TimeIDs;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_TimeIDs;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_TimeIDs TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_TimeIDs TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_TimeIDs'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all TimeIDs of ValueSeries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Value;
GO

/*
	GET Operation for a Value

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Value 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1',
			@ValueSerieID	= 'K1',
			@TimeID			=  20180115
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Value',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Value','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Value
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255),
	@TimeID			INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @ValueKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')+ N''',''' 
			+ ISNULL(@ValueSerieID,		N'NULL')+ N''',''' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(10)), N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @FactoryID		IS NULL SET @FactoryID		 = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	 = N'';
	IF @ProductID		IS NULL SET @ProductID		 = N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Value;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		SET @ValueSerieID	 = dbo.sx_pf_pProtectID		 (@ValueSerieID);
		
		IF	@FactoryID				= N'' 
			OR @ProductLineID		= N'' 
			OR @ProductID			= N'' 
			OR @ValueSerieID		= N'' 
			OR ISNULL(@TimeID, 0)	<= 0		
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;
		SELECT @ValueKey = ValueKey FROM planning.tfValues WHERE ValueSeriesKey = @ValueSerieKey AND TimeID = @TimeID;

		IF @ProductKey = 0 OR @ProductLineKey = 0 OR @FactoryKey = 0 OR @ValueSerieKey = 0 OR @ValueKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2.1 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM planning.tfValues
		WHERE ValueKey = @ValueKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Value TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Value TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Value'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for a Value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Values;
GO

/*
	GET Operation for all Values of ValueSeries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Values
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1',
			@ValueSerieID	= 'K1'
		PRINT @RC

		--should return 404
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Values 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1',
			@ValueSerieID	= 'K1test'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Values',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Values','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Values
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_Values;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID			= N'' 
			OR  @ProductLineID	= N'' 
			OR @ProductID		= N'' 
			OR @ValueSerieID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT  
			 ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,@Right AS 'Right'
		FROM planning.tfValues
		WHERE ValueSeriesKey = @ValueSerieKey
		ORDER BY TimeID;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_Values;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_Values;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Values TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Values TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Values'    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all Values of ValueSeries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ValueSerie;
GO

/*
	GET Operation for all ValueSerie
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ValueSerie 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1',
			@ValueSerieID	= 'K1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ValueSerie',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ValueSerie
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ValueSerieID	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSerieKey		INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSerieID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ValueSerieID	IS NULL SET @ValueSerieID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSerie;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSerieID	= dbo.sx_pf_pProtectID		(@ValueSerieID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N'' 
			OR @ValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @ValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @ValueSerieID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0 
			OR @ValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM planning.tdValueSeries
		WHERE ValueSeriesKey = @ValueSerieKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ValueSerie TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSerie TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ValueSerie'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all ValueSerie';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ValueSerieID';
    SET @value      = N'ValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ValueSeries;
GO


/*
	GET Operation for all ValueSeries of Product

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ValueSeries 
			@Username		= 'SQL',
			@FactoryID		= 'ZT',
			@ProductLineID	= 'U',
			@ProductID		= '1'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ValueSeries',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ValueSeries','PARAMETER',NULL)
*/

CREATE  PROCEDURE dbo.sx_pf_GET_ValueSeries
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10);
	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ValueSeries;
	
		-- STEP 0.2 - Protect input parameters
		SET @Username		 = dbo.sx_pf_pProtectString	 (@Username);
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @ProductLineID	 = dbo.sx_pf_pProtectID		 (@ProductLineID);	
		SET @ProductID		 = dbo.sx_pf_pProtectID		 (@ProductID);
		
		IF	@FactoryID = N'' 
			OR  @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode = 200
		BEGIN
			SET @Right = N'Write';
		END 
		ELSE 
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Read';
			END 
			ELSE 
			BEGIN 
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values		
		SELECT 
			ProductID
			,ProductLineID
			,FactoryID
			,ValueSeriesID
			,ValueSeriesNo
			,NameShort
			,NameLong
			,CommentUser
			,CommentDev
			,ImageName
			,[IsNumeric]
			,VisibilityLevel
			,ValueSource
			,ValueListID
			,ValueFormatID
			,Unit
			,Scale
			,Effect
			,EffectParameter
			,@Right AS 'Right'
		FROM planning.tdValueSeries
		WHERE ProductKey = @ProductKey 
		ORDER BY ValueSeriesNo;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_ValueSeries;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ValueSeries;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ValueSeries TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ValueSeries TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ValueSeries'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all ValueSeries of Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_Factory;
GO

/*
	MOVE Operation for a Factory
	Factory moved by the update IDs without key change
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9

	Test call
	1. Empty source or target FactoryID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory => 404 Not Found
	5. Exists target Factory => 403 Forbidden
	6. Factory moved => 200 OK
	7. Don`t moved anything => 204 No content

	Testcall
		DECLARE  @RC INT
				,@Username			NVARCHAR(255) ='SQL'
				,@SourceFactoryID   NVARCHAR(255) ='ZT'
				,@TargetFactoryID   NVARCHAR(255) ='ZT1';

		EXECUTE @RC = dbo.sx_pf_MOVE_Factory
				   @Username
				  ,@SourceFactoryID
				  ,@TargetFactoryID;
		PRINT @RC;

		SELECT TOP 20 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_Factory
	 @Username			NVARCHAR(255)
	,@SourceFactoryID	NVARCHAR(255)
	,@TargetFactoryID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'				--cross access and data operation in system table
AS 
BEGIN

	DECLARE  @TransactUserKey			INT				= 0
			,@SourceFactoryKey			INT				= 0
			,@SourceFactoryNameShort	NVARCHAR(255)	= N''
			,@TargetFactoryKey			INT				= 0
			
			,@TransactUsername			NVARCHAR(255)	= N''
			,@ProcedureName				NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString			NVARCHAR (MAX)	= N'''' 
										+ ISNULL(@Username, N'NULL')		+ N''',''' 
										+ ISNULL(@SourceFactoryID, N'NULL') + N''',''' 
										+ ISNULL(@TargetFactoryID, N'NULL') + N''''
			,@EffectedRows				INT = 0						
			,@ResultCode				INT = 501						
			,@TimestampCall				DATETIME = GETUTCDATE()
			,@Comment					NVARCHAR (2000) = N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username			= N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID	= N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_MOVE_Factory;
				
		-- Protect input parameters	
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID	= dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @TargetFactoryID	= dbo.sx_pf_pProtectID (@TargetFactoryID);

		IF @SourceFactoryID = N'' OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Factory key already exists', 16, 10);
		END;
		
		-- MOVE DATA	++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		--dFactory 
		UPDATE planning.tdFactories
		SET FactoryID = @TargetFactoryID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
			BEGIN 
				SET @ResultCode = 204;
			END		
		ELSE
			BEGIN
				--dProductLine  
				UPDATE planning.tdProductLines
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dProduct 
				UPDATE planning.tdProducts
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dValueSeries
				UPDATE planning.tdValueSeries
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dTime
				UPDATE planning.tdTime
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--fValues
				UPDATE planning.tfValues
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gFactories
				UPDATE planning.tgFactories 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gProductLines
				UPDATE planning.tgProductLines 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;
		
				--gProducts 
				UPDATE planning.tgProducts 
				SET FactoryID  = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;
		
				SET @EffectedRows += @@ROWCOUNT;

				--rRights 
				UPDATE system.trRights				-- therefore, EXECUTE AS dbo is needed
				SET FactoryID = @TargetFactoryID
				WHERE FactoryID = @SourceFactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				--rPreferences
				UPDATE planning.trPreferences 
				SET FactoryID = @TargetFactoryID
				WHERE FactoryID = @SourceFactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				--fStatements
				UPDATE planning.tfStatements 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tFiles
				UPDATE planning.tFiles 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tTabs
				UPDATE planning.tTabs
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

				-- write action statement
				INSERT INTO planning.tfStatements   
					SELECT ProductKey
						, dp.ProductLineKey
						, @TargetFactoryKey AS FactoryKey
						, ProductID
						, dp.ProductLineID
						, @TargetFactoryID AS FactoryID
						, N'MOVE' AS ActionType
						, N'Factory is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
							+ N'" / ProductLine <' + dp.ProductLineID + N'>"' + ISNULL(dpl.Nameshort, N'NULL') 
							+ N'" / Product <' + ProductID  + N'>"' + dp.Nameshort +  N'"' AS [Statement]
						, @TransactUsername AS UserName
						, N'' AS PCName
						, N'' AS ProcessorCode
						, N'' AS IPAddresses
						, GETUTCDATE() AS [Timestamp]
						, 0 AS IsDeleted
						, 0 AS IsResolved
					FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl
						ON dp.ProductLinekey = dpl.ProductLineKey
					WHERE dp.FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- Rights materialization
				EXEC system.spMaterialize_trUserRights;
			END;

		COMMIT TRANSACTION sx_pf_MOVE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_MOVE_Factory;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_Factory TO pf_PlanningFactoryService;  
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_Factory'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'MOVE Operation for a Factory
         Factory moved by the update IDs without key change';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_Format;
GO

/*
	Move Operation for a Format - all objects, which an FormatID are switched to use a new FormatID.
	Replace the OldFormatID with the new FormatID in planning.tdValueSeries
	Replace the OldFormatID with the new FormatID in planning.thListValues
	Delete the OldFormatID in planning.thFormats
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty source or target FormatID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Format => 404 Not Found
	5. Not exists target Format => 404 Not Found
	6. Format moved => 200 OK
	7. Don`t moved anything => 204 No content

		DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldFormatID AS NVARCHAR(255)=''
		DECLARE  @NewFormatID AS NVARCHAR(255)=''

		EXECUTE @RC = dbo.sx_pf_MOVE_Format
				   @Username
				  ,@OldFormatID
				  ,@NewFormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_Format
	@Username AS NVARCHAR(255),
	@OldFormatID AS NVARCHAR(255),
	@NewFormatID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldFormatKey AS INT = 0;
	DECLARE @NewFormatKey AS INT = 0;

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldFormatID, N'NULL') + N''',''' + ISNULL(@NewFormatID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = GETUTCDATE();
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldFormatID IS NULL SET @OldFormatID = N'';
	IF @NewFormatID IS NULL SET @NewFormatID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @OldFormatID = dbo.sx_pf_pProtectID (@OldFormatID);
		SET @NewFormatID = dbo.sx_pf_pProtectID (@NewFormatID);

		IF (@OldFormatID = N'' OR @NewFormatID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @OldFormatID;
		
		IF @OldFormatKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewFormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @NewFormatID;
		
		IF @NewFormatKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		UPDATE planning.tdValueSeries
		   SET ValueFormatID = @NewFormatID 
		WHERE ValueFormatID = @OldFormatID;
																							
		SET @EffectedRows += @@ROWCOUNT;
																						   
		UPDATE planning.thListValues
		   SET FormatID = @NewFormatID 
		WHERE FormatID = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		UPDATE planning.tdTime
		   SET FormatID = @NewFormatID 
		WHERE FormatID = @OldFormatID;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thFormats WHERE FormatID = @OldFormatID;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_Format TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_MOVE_Format TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_Format'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Move Operation for a Format - all objects, which an FormatID are switched to use a new FormatID.
         Replace the OldFormatID with the new FormatID in planning.tdValueSeries
         Replace the OldFormatID with the new FormatID in planning.thListValues
         Delete the OldFormatID in planning.thFormats';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@OldFormatID';
    SET @value      = N'OldFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NewFormatID';
    SET @value      = N'NewFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_List;
GO

/*
	Move Operation for a List
	The Procedure will change the OldListID into the NewListID in tables:
	planning.tdValueSeries, column ValueListID
	planning.tdProductLines, columns GlobalAttributeSource 1..25
	create members from OldListID in NewListID if they don't exist there
	(for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
	delete the OldListID in planning.thLists
	delete the Old List Values in planning.thListValues
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty source or target ListID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source List => 404 Not Found
	5. Not exists target List => 404 Not Found
	6. List moved => 200 OK
	7. Don`t moved anything => 204 No content

		DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldListID AS NVARCHAR(255)='suEvents'
		DECLARE  @NewListID AS NVARCHAR(255)='suYesNo'

		EXECUTE @RC = dbo.sx_pf_MOVE_List
				   @Username
				  ,@OldListID
				  ,@NewListID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_List
	@Username AS NVARCHAR(255),
	@OldListID AS NVARCHAR(255),
	@NewListID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldListKey AS INT = 0;
	DECLARE @NewListKey AS INT = 0;
	DECLARE @NewListType AS NVARCHAR(255)= N'';

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldListID, N'NULL') + N''',''' + ISNULL(@NewListID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = GETUTCDATE();
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldListID IS NULL SET @OldListID = N'';
	IF @NewListID IS NULL SET @NewListID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @OldListID = dbo.sx_pf_pProtectID (@OldListID);
		SET @NewListID = dbo.sx_pf_pProtectID (@NewListID);

		IF (@OldListID = N'' OR @NewListID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @OldListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
			OR @NewListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel')
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
 		END;;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldListKey = ListKey FROM planning.thLists WHERE ListID = @OldListID;
		
		IF @OldListKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewListKey = ListKey, @NewListType = Datentyp FROM planning.thLists WHERE ListID = @NewListID;
		
		IF @NewListKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
							 
		-- STEP 3 - Move data
		-- STEP 3.1 - Move not exists UNIQUE list members

		IF @NewListType = 'Integer'
		BEGIN

			;WITH vals AS (
				SELECT v1.ListValueKey 
					, v1.ValueInt
					, v1.Scale
					, v1.ValueText
					, v1.ValueComment
					, v1.FormatID
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.ListID = @NewListID AND v1.ValueInt = v2.ValueInt
				WHERE v1.ListID = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES ListValueKey
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY ValueInt ORDER BY ListValueKey)
			)																				   
			INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			SELECT @NewListID, ValueInt, Scale, ValueText, ValueComment, FormatID
			FROM vals INNER JOIN uniq ON vals.ListValueKey = uniq.ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE IF @NewListType = 'String'
		BEGIN

			;WITH vals AS (
				SELECT v1.ListValueKey 
					, v1.ValueInt
					, v1.Scale
					, v1.ValueText
					, v1.ValueComment
					, v1.FormatID
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.ListID = @NewListID AND v1.ValueText = v2.ValueText
				WHERE v1.ListID = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES ListValueKey
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY ValueText ORDER BY ListValueKey)
			)																				   
			INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			SELECT @NewListID, ValueInt, Scale, ValueText, ValueComment, FormatID
			FROM vals INNER JOIN uniq ON vals.ListValueKey = uniq.ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		-- STEP 3.2 - Force all objects, which use a old list, to use a new list now
		UPDATE planning.tdValueSeries
		   SET ValueListID = @NewListID 
		WHERE ValueListID = @OldListID;
																							
		SET @EffectedRows += @@ROWCOUNT;
		
		;WITH lines AS (
			SELECT ProductLineKey 
			FROM planning.tdProductLines
			WHERE GlobalAttributeSource1 = @OldListID OR GlobalAttributeSource2 = @OldListID OR GlobalAttributeSource3 = @OldListID OR
				GlobalAttributeSource4 = @OldListID OR GlobalAttributeSource5 = @OldListID OR GlobalAttributeSource6 = @OldListID OR
				GlobalAttributeSource7 = @OldListID OR GlobalAttributeSource8 = @OldListID OR GlobalAttributeSource9 = @OldListID OR
				GlobalAttributeSource10 = @OldListID OR GlobalAttributeSource11 = @OldListID OR GlobalAttributeSource12 = @OldListID OR
				GlobalAttributeSource13 = @OldListID OR GlobalAttributeSource14 = @OldListID OR GlobalAttributeSource15 = @OldListID OR
				GlobalAttributeSource16 = @OldListID OR GlobalAttributeSource17 = @OldListID OR GlobalAttributeSource18 = @OldListID OR
				GlobalAttributeSource19 = @OldListID OR GlobalAttributeSource20 = @OldListID OR GlobalAttributeSource21 = @OldListID OR
				GlobalAttributeSource22 = @OldListID OR GlobalAttributeSource23 = @OldListID OR GlobalAttributeSource24 = @OldListID OR
				GlobalAttributeSource25 = @OldListID 
		)
		UPDATE pl
		SET GlobalAttributeSource1 = IIF(GlobalAttributeSource1 = @OldListID, @NewListID, GlobalAttributeSource1)
			, GlobalAttributeSource2 = IIF(GlobalAttributeSource2 = @OldListID, @NewListID, GlobalAttributeSource2)
			, GlobalAttributeSource3 = IIF(GlobalAttributeSource3 = @OldListID, @NewListID, GlobalAttributeSource3)
			, GlobalAttributeSource4 = IIF(GlobalAttributeSource4 = @OldListID, @NewListID, GlobalAttributeSource4)
			, GlobalAttributeSource5 = IIF(GlobalAttributeSource5 = @OldListID, @NewListID, GlobalAttributeSource5)
			, GlobalAttributeSource6 = IIF(GlobalAttributeSource6 = @OldListID, @NewListID, GlobalAttributeSource6)
			, GlobalAttributeSource7 = IIF(GlobalAttributeSource7 = @OldListID, @NewListID, GlobalAttributeSource7)
			, GlobalAttributeSource8 = IIF(GlobalAttributeSource8 = @OldListID, @NewListID, GlobalAttributeSource8)
			, GlobalAttributeSource9 = IIF(GlobalAttributeSource9 = @OldListID, @NewListID, GlobalAttributeSource9)
			, GlobalAttributeSource10 = IIF(GlobalAttributeSource10 = @OldListID, @NewListID, GlobalAttributeSource10)
			, GlobalAttributeSource11 = IIF(GlobalAttributeSource11 = @OldListID, @NewListID, GlobalAttributeSource11)
			, GlobalAttributeSource12 = IIF(GlobalAttributeSource12 = @OldListID, @NewListID, GlobalAttributeSource12)
			, GlobalAttributeSource13 = IIF(GlobalAttributeSource13 = @OldListID, @NewListID, GlobalAttributeSource13)
			, GlobalAttributeSource14 = IIF(GlobalAttributeSource14 = @OldListID, @NewListID, GlobalAttributeSource14)
			, GlobalAttributeSource15 = IIF(GlobalAttributeSource15 = @OldListID, @NewListID, GlobalAttributeSource15)
			, GlobalAttributeSource16 = IIF(GlobalAttributeSource16 = @OldListID, @NewListID, GlobalAttributeSource16)
			, GlobalAttributeSource17 = IIF(GlobalAttributeSource17 = @OldListID, @NewListID, GlobalAttributeSource17)
			, GlobalAttributeSource18 = IIF(GlobalAttributeSource18 = @OldListID, @NewListID, GlobalAttributeSource18)
			, GlobalAttributeSource19 = IIF(GlobalAttributeSource19 = @OldListID, @NewListID, GlobalAttributeSource19)
			, GlobalAttributeSource20 = IIF(GlobalAttributeSource20 = @OldListID, @NewListID, GlobalAttributeSource20)
			, GlobalAttributeSource21 = IIF(GlobalAttributeSource21 = @OldListID, @NewListID, GlobalAttributeSource21)
			, GlobalAttributeSource22 = IIF(GlobalAttributeSource22 = @OldListID, @NewListID, GlobalAttributeSource22)
			, GlobalAttributeSource23 = IIF(GlobalAttributeSource23 = @OldListID, @NewListID, GlobalAttributeSource23)
			, GlobalAttributeSource24 = IIF(GlobalAttributeSource24 = @OldListID, @NewListID, GlobalAttributeSource24)
			, GlobalAttributeSource25 = IIF(GlobalAttributeSource25 = @OldListID, @NewListID, GlobalAttributeSource25)
		FROM planning.tdProductLines pl INNER JOIN lines l ON pl.ProductLineKey = l.ProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Remove old list
				
		DELETE FROM planning.thListValues WHERE ListID = @OldListID;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thLists WHERE ListKey = @OldListKey;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_List TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_MOVE_List TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_List'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Move Operation for a List
         The Procedure will change the OldListID into the NewListID in tables:
         planning.tdValueSeries, column ValueListID
         planning.tdProductLines, columns GlobalAttributeSource 1..25
         create members from OldListID in NewListID if they dont exist there
         (for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
         delete the OldListID in planning.thLists
         delete the Old List Values in planning.thListValues';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@OldListID';
    SET @value      = N'OldListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NewListID';
    SET @value      = N'NewListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_Product;
GO

/*
	Move Operation for a Product
	Product moved by ID update, the Key of the Product keeps the same
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9

	Test call
	1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine / Product => 404 Not Found
	5. Not exists target Factory / ProductLine => 404
	6. Exists TargetProduct => 403 Forbidden 
	7. Product moved => 200 OK
	8. Don`t moved anything => 204 No content

		DECLARE  @RC int
				,@Username				NVARCHAR(255)	= 'SQL'
				,@SourceFactoryID		NVARCHAR(255)	='ZT'
				,@SourceProductLineID	NVARCHAR(255)	='U'
				,@SourceProductID		NVARCHAR(255)	='1'
				,@TargetFactoryID		NVARCHAR(255)	='ZT'
				,@TargetProductLineID	NVARCHAR(255)	='U'
				,@TargetProductID		NVARCHAR(255)	='RZ888';

		EXECUTE @RC = dbo.sx_pf_MOVE_Product
				   @Username
				  ,@SourceFactoryID
				  ,@SourceProductLineID
				  ,@SourceProductID
				  ,@TargetFactoryID
				  ,@TargetProductLineID
				  ,@TargetProductID;
		PRINT @RC;

		SELECT TOP 5 * FROM system.tAPI_Log;

		SELECT * FROM planning.tdProducts;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_Product
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@SourceProductLineID	NVARCHAR(255),
	@SourceProductID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetProductLineID	NVARCHAR(255),
	@TargetProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE	  @SourceFactoryKey				INT = 0
			 ,@SourceProductKey				INT = 0
			 ,@SourceProductLineKey			INT = 0
			 ,@TargetProductKey				INT = 0
			 ,@SourceFactoryNameShort		NVARCHAR(255) = N''
			 ,@SourceProductLineNameShort	NVARCHAR(255) = N''
			 ,@SourceProductNameShort		NVARCHAR(255) = N''
			 ,@TargetFactoryKey				INT = 0
			 ,@TargetProductLineKey			INT = 0

			 ,@TransactUsername				NVARCHAR(255)= N''
			 ,@ProcedureName				NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			 ,@ParameterString AS NVARCHAR (MAX) = N'''' 
				+ ISNULL(@Username, N'NULL')			+ N''',''' 
				+ ISNULL(@SourceFactoryID, N'NULL')		+ N''',''' 
				+ ISNULL(@SourceProductLineID, N'NULL') + N''',''' 
				+ ISNULL(@SourceProductID, N'NULL')		+ N''',''' 
				+ ISNULL(@TargetFactoryID, N'NULL')		+ N''',''' 
				+ ISNULL(@TargetProductLineID, N'NULL') + N''',''' 
				+ ISNULL(@TargetProductID, N'NULL')		+ N''''
			 ,@EffectedRows					INT = 0						
			 ,@ResultCode					INT = 501						
			 ,@TimestampCall				DATETIME = GETUTCDATE()
			 ,@Comment						NVARCHAR (2000) = N'';			

	-- NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
	IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
	IF @TargetProductID			IS NULL SET @TargetProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION MOVE_Product;
				
		-- Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID (@SourceProductLineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID (@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID (@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID (@TargetProductLineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID (@TargetProductID);

		IF (@SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @SourceProductID = N''
			OR @TargetFactoryID = N'' OR @TargetProductLineID = N'' OR @TargetProductID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target - no write rights.', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source - no write rights.', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey		= FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey	= ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		SELECT @SourceProductKey		= ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE FactoryKey = @SourceFactoryKey AND ProductLineKey = @SourceProductLineKey AND ProductID = @SourceProductID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0 OR @SourceProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey		= FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey	= ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		SELECT @TargetProductKey		= ProductKey FROM planning.tdProducts WHERE FactoryKey = @TargetFactoryKey AND ProductLineKey = @TargetProductLineKey AND ProductID = @TargetProductID;
		
		IF @TargetFactoryKey = 0 OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
			
		-- STEP 3 - Move data
		--dProduct 
		UPDATE planning.tdProducts 
		SET	  FactoryKey		= @TargetFactoryKey
			 ,ProductLineKey	= @TargetProductLineKey
			 ,FactoryID			= @TargetFactoryID
			 ,ProductLineID		= @TargetProductLineID
			 ,ProductID			= @TargetProductID 
		WHERE ProductKey = @SourceProductKey;
					 
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN
			--gProducts 
			UPDATE planning.tgProducts 
			SET  FactoryKey			= @TargetFactoryKey
				,ProductLineKey		= @TargetProductLineKey
				,FactoryID			= @TargetFactoryID
				,ProductLineID		= @TargetProductLineID
				,ProductID			= @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;
		
			--dValueSeries
			UPDATE planning.tdValueSeries 
			SET	 FactoryKey		= @TargetFactoryKey
				,ProductLineKey = @TargetProductLineKey
				,FactoryID		= @TargetFactoryID
				,ProductLineID	= @TargetProductLineID
				,ProductID		= @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			UPDATE planning.tdTime 
			SET  FactoryKey		= @TargetFactoryKey
				,ProductLineKey = @TargetProductLineKey
				,FactoryID		= @TargetFactoryID
				,ProductLineID	= @TargetProductLineID
				,ProductID		= @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			UPDATE planning.tfValues 
			SET  FactoryKey		= @TargetFactoryKey
				,ProductLineKey = @TargetProductLineKey
				,FactoryID		= @TargetFactoryID
				,ProductLineID	= @TargetProductLineID
				,ProductID		= @TargetProductID 
			WHERE ProductKey = @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements 
			UPDATE planning.tfStatements 
			SET  FactoryKey			= @TargetFactoryKey
				,ProductLineKey		= @TargetProductLineKey
				,FactoryID			= @TargetFactoryID
				,ProductLineID		= @TargetProductLineID
				,ProductID			= @TargetProductID 
			WHERE ProductKey		= @SourceProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			-- tFiles
			UPDATE planning.tFiles 
			SET 
				 FactoryKey		= @TargetFactoryKey
				,ProductLineKey = @TargetProductLineKey
				,FactoryID		= @TargetFactoryID
				,ProductLineID	= @TargetProductLineID
				,ProductID		= @TargetProductID

			WHERE 
				ProductKey = @SourceProductKey;

			-- write action statement
			INSERT INTO planning.tfStatements 
			SELECT 
				 @SourceProductKey		AS ProductKey
				,@TargetProductLineKey	AS ProductLineKey
				,@TargetFactoryKey		AS FactoryKey
				,@TargetProductID		AS ProductID
				,@TargetProductLineID	AS ProductLineID
				,@TargetFactoryID		AS FactoryID
				,N'MOVE'				AS ActionType
				,N'Product is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
				 + N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort 
				 + N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS [Statement]
				,@TransactUsername		AS UserName
				,N''					AS PCName
				,N''					AS ProcessorCode
				,N''					AS IPAddresses
				,GETUTCDATE()			AS [Timestamp]
				,0						AS IsDeleted
				,0						AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION MOVE_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION MOVE_Product;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_Product TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_Product'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Move Operation for a Product
         Product moved by ID update';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_ProductLine;
GO

/*
	MOVE Operation for a ProductLine
	ProductLine moved by the update IDs without key change

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9

	Test call
	1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine => 404 Not Found
	5. Not exists target  Factory und existierende Target ProductLine => 404  Not Found
	6. Target ValueSerie key already exists => 403 Forbidden
	7. ProductLine moved => 200 OK
	8. Don`t moved anything => 204 No content

	Testcall
		DECLARE  @RC INT
				,@Username				NVARCHAR(255)	=N'SQL'
				,@SourceFactoryID		NVARCHAR(255)	=N'ZT'
				,@SourceProductLineID	NVARCHAR(255)	=N'U'
				,@TargetFactoryID		NVARCHAR(255)	=N'ZT'
				,@TargetProductLineID	NVARCHAR(255)	=N'U2';

		EXECUTE @RC = dbo.sx_pf_MOVE_ProductLine
				   @Username
				  ,@SourceFactoryID
				  ,@SourceProductLineID
				  ,@TargetFactoryID
				  ,@TargetProductLineID;

		PRINT @RC;

		SELECT TOP 20 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_ProductLine
	 @Username				NVARCHAR(255)
	,@SourceFactoryID		NVARCHAR(255)
	,@SourceProductLineID	NVARCHAR(255)
	,@TargetFactoryID		NVARCHAR(255)
	,@TargetProductLineID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'					-- due to moving the Productline in the system tables / cross action on system schema
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUserKey				INT = 0
			,@SourceFactoryKey 				INT = 0
			,@SourceProductLineKey			INT = 0
			,@SourceFactoryNameShort		NVARCHAR(255) = N''
			,@SourceProductLineNameShort	NVARCHAR(255) = N''
			,@TargetFactoryKey				INT = 0
			,@TargetProductLineKey			INT = 0

			,@TransactUsername				NVARCHAR(255)= N''
			,@ProcedureName					NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR (MAX) = N'''' 
					+ ISNULL(@Username				,N'NULL') + N''',''' 
					+ ISNULL(@SourceFactoryID		,N'NULL') + N''',''' 
					+ ISNULL(@SourceProductLineID	,N'NULL') + N''',''' 
					+ ISNULL(@TargetFactoryID		,N'NULL') + N''',''' 
					+ ISNULL(@TargetProductLineID	,N'NULL') + N''''
			,@EffectedRows					INT				= 0					
			,@ResultCode					INT				= 501				
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username			IS NULL SET @Username				= N'';
	IF @SourceFactoryID		IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID	= N'';
	IF @TargetFactoryID		IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_MOVE_ProductLine;

		-- Protect input parameters
		SET @Username				= dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID (@SourceProductLineID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID (@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID (@TargetProductLineID);

		IF @SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @TargetFactoryID = N''
			OR @TargetProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target - no write rights.', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source - no write rights.', 16, 10);
		
		-- Determine SourceKeys	
		SELECT @SourceFactoryKey		= FactoryKey, @SourceFactoryNameShort = NameShort			FROM planning.tdFactories		WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey	= ProductLineKey, @SourceProductLineNameShort = Nameshort	FROM planning.tdProductLines	WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- Determine TargetKeys	
		SELECT @TargetFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- MOVE DATA		+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		--dProductLine 
		UPDATE planning.tdProductLines 
		SET  FactoryKey		= @TargetFactoryKey
			,ProductLineID	= @TargetProductLineID
			,FactoryID		= @TargetFactoryID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN 
				SET @ResultCode = 204;
			END		
		ELSE
			BEGIN
				--dProduct
				UPDATE planning.tdProducts 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dValueSeries
				UPDATE planning.tdValueSeries 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dTime
				UPDATE planning.tdTime 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--fValues
				UPDATE planning.tfValues 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gProductLines
				UPDATE planning.tgProductLines 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;
		
				--gProducts 
				UPDATE planning.tgProducts 
				SET  FactoryKey			= @TargetFactoryKey
					,ProductLineID		= @TargetProductLineID
					,FactoryID			= @TargetFactoryID 
				WHERE ProductLineKey	= @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--rRights
				UPDATE system.trRights						-- this makes execute as dbo necessary !
				SET		 FactoryID		= @TargetFactoryID
						,ProductLineID	= @TargetProductLineID
				WHERE 
						FactoryID		= @SourceFactoryID 
					AND ProductLineID	= @SourceProductLineID;

				SET @EffectedRows += @@ROWCOUNT;

				--rPreferences (no FPLP Keys a columns)
				UPDATE	 planning.trPreferences 
					SET		 ProductLineID	= @TargetProductLineID
							,FactoryID		= @TargetFactoryID

					WHERE 
							FactoryID		= @SourceFactoryID 
						AND ProductLineID	= @SourceProductLineID;

				SET @EffectedRows += @@ROWCOUNT;

				--fStatements
				UPDATE planning.tfStatements 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tFiles
				UPDATE planning.tFiles 
				SET 
					 FactoryKey		= @TargetFactoryKey
					,FactoryID		= @TargetFactoryID 
					,ProductLineID	= @TargetProductLineID
				WHERE 
					ProductLineKey = @SourceProductLineKey;

				INSERT INTO planning.tfStatements
				SELECT 
						 ProductKey
						,@SourceProductLineKey	AS ProductLineKey
						,@TargetFactoryKey		AS FactoryKey
						,ProductID
						,@TargetProductLineID	AS ProductLineID
						,@TargetFactoryID		AS FactoryID
						,N'MOVE'				AS ActionType
						,N'ProductLine is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
							+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort 
							+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS [Statement]
						,@TransactUsername		AS UserName
						,N''					AS PCName
						,N''					AS ProcessorCode
						,N''					AS IPAddresses
						,GETUTCDATE()			[Timestamp]
						,0 AS IsDeleted
						,0 AS IsResolved
				FROM planning.tdProducts 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tTabs
				UPDATE planning.tTabs
				SET 
					 FactoryKey		= @TargetFactoryKey
					,FactoryID		= @TargetFactoryID 
					,ProductLineID	= @TargetProductLineID
				WHERE
					ProductLineKey  = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

				-- Rights materialization
				EXEC system.spMaterialize_trUserRights;
			END;

		COMMIT TRANSACTION sx_pf_MOVE_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_MOVE_ProductLine;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_ProductLine TO pf_PlanningFactoryService;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_ProductLine'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1							-- 0 = Demo object - out of shipment process
																	
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'MOVE Operation for a ProductLine
         ProductLine moved by the update IDs without key change';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_ValueSerie;
GO

/*
	Move Operation for one ValueSeries
	The ValueSeries will be moved to a new ID, only if this ID is free
	Works only inside one Product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty  Source or Target  FactoryID / ProductLineID / ProductID / ValueSerieID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists Source Factory / ProductLine / Product / ValueSerie => 404 Not Found
	5. Not exists Target  Factory / ProductLine / Product => 404 Not Found
	6. Target ValueSerie key already exists => 403 Forbidden
	7. ValueSerie moved => 200 OK
	8. Don`t moved anything => 204 No content

		DECLARE @RC int
		DECLARE  @Username				AS NVARCHAR(255) = 'SQL'
		DECLARE  @FactoryID				AS NVARCHAR(255) ='ZT'
		DECLARE  @ProductLineID			AS NVARCHAR(255) ='U'
		DECLARE  @ProductID				AS NVARCHAR(255) ='2'
		DECLARE  @SourceValueSerieID	AS NVARCHAR(255) ='K1'
		DECLARE  @TargetValueSerieID	AS NVARCHAR(255) ='K7'

		EXECUTE @RC = dbo.sx_pf_MOVE_ValueSerie
				   @Username
				  ,@FactoryID
				  ,@ProductLineID
				  ,@ProductID
				  ,@SourceValueSerieID
				  ,@TargetValueSerieID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ValueSerie',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_ValueSerie
	@Username			AS NVARCHAR(255),
	@FactoryID			AS NVARCHAR(255),
	@ProductLineID		AS NVARCHAR(255),
	@ProductID			AS NVARCHAR(255),
	@SourceValueSerieID AS NVARCHAR(255),
	@TargetValueSerieID AS NVARCHAR(255)
AS 

BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey				AS INT = 0;
	DECLARE @ProductLineKey			AS INT = 0;
	DECLARE @ProductKey				AS INT = 0;
	DECLARE @SourceValueSerieKey	AS INT = 0;
	DECLARE @TargetValueSerieKey	AS INT = 0;

	DECLARE @TransactUsername		AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName			AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);;
	DECLARE @ParameterString		AS NVARCHAR (MAX) = N'''' 
												+ ISNULL(@Username, N'NULL')									+ N''',''' 
												+ ISNULL(@FactoryID, N'NULL')									+ N''',''' 
												+ ISNULL(@ProductLineID, N'NULL')								+ N''',''' 
												+ ISNULL(@ProductID, N'NULL')									+ N''',''' 
												+ ISNULL(CAST(@SourceValueSerieID AS NVARCHAR(255)), N'NULL')	+ N''',''' 
												+ ISNULL(@TargetValueSerieID, N'NULL')							+ N'''';
	DECLARE @EffectedRows AS INT = 0;						
	DECLARE @ResultCode AS INT = 501;						
	DECLARE @TimestampCall AS DATETIME = GETUTCDATE();
	DECLARE @Comment AS NVARCHAR (2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @FactoryID IS NULL SET @FactoryID = N'';
	IF @ProductLineID IS NULL SET @ProductLineID = N'';
	IF @ProductID IS NULL SET @ProductID = N'';
	IF @SourceValueSerieID IS NULL SET @SourceValueSerieID = N'';
	IF @TargetValueSerieID IS NULL SET @TargetValueSerieID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID = dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID = dbo.sx_pf_pProtectID (@ProductLineID);
		SET @ProductID = dbo.sx_pf_pProtectID (@ProductID);
		SET @SourceValueSerieID = dbo.sx_pf_pProtectID (@SourceValueSerieID);
		SET @TargetValueSerieID = dbo.sx_pf_pProtectID (@TargetValueSerieID);

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR  @SourceValueSerieID = N'' OR  @TargetValueSerieID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @SourceValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey  AND ValueSeriesID = @SourceValueSerieID;	
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 OR  @SourceValueSerieKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetValueSerieKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey = @ProductKey AND ValueSeriesID = @TargetValueSerieID;

		IF @TargetValueSerieKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ValueSerie key already exists', 16, 10);
		END;

		-- STEP 3 - Move data
		--dValueSeries 
		UPDATE planning.tdValueSeries
		SET ValueSeriesID = @TargetValueSerieID
		WHERE ValueSeriesKey = @SourceValueSerieKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END		
		ELSE
		BEGIN			  
			--fValues
			UPDATE planning.tfValues 
			SET ValueSeriesID = @TargetValueSerieID
			WHERE ValueSeriesKey = @SourceValueSerieKey;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = IIF(@EffectedRows = 0, 204, 200);
				
			INSERT INTO planning.tfStatements  
			SELECT @ProductKey AS ProductKey
				, @ProductLineKey AS ProductLineKey
				, @FactoryKey AS FactoryKey
				, @ProductID  AS ProductID
				, @ProductLineID  AS ProductLineID
				, @FactoryID  AS FactoryID
				, N'MOVE' AS ActionType
				, N'ValueSeries ' + @SourceValueSerieID + N' was moved to new ID ' + @TargetValueSerieID AS [Statement]
				, @TransactUsername AS UserName
				, N'' AS PCName
				, N'' AS ProcessorCode
				, N'' AS IPAddresses
				, GETUTCDATE() [Timestamp]
				, 0 AS IsDeleted
				, 0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_ValueSerie TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_MOVE_ValueSerie TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_ValueSerie'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Move Operation for one ValueSeries
         The ValueSeries will be moved to a new ID, only if this ID is free
         Works only inside one Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceValueSerieID';
    SET @value      = N'SourceValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetValueSerieID';
    SET @value      = N'TargetValueSerieID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Cleanup;
GO

/*
	Cleanup Operation, which will clean the values in planning.tfValues for a given scope.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE	 @RC				INT
				,@Username			NVARCHAR(255)	= 'SQL'
				,@FactoryID			NVARCHAR(255)	= 'ZT'
				,@ProductLineID		NVARCHAR(255)	= 'U'
				,@ProductID			NVARCHAR(255)	= ''
				,@ValueSeriesID		NVARCHAR(255)	= 'K3'
				,@TimeID			INT				= 20180515	

		EXECUTE	@RC = dbo.sx_pf_POST_Cleanup 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@ValueSeriesID,
				@TimeID

		PRINT @RC

		dbo.sx_pf_POST_Cleanup 'SQL', 'F1', 'PL1',	'P1',	'VS1',	201703012	-- will delete all values in planning.tfValues for this combination (this means one row)
		dbo.sx_pf_POST_Cleanup 'SQL', '',	'PL1',	'P1',	'VS1',	20170312	-- would delete this value in all Factories
		dbo.sx_pf_POST_Cleanup 'SQL', '',	'',		'',		'',		20170312	-- would delete all values in for this TimeID in planning.tfValues.

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Cleanup',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Cleanup','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Cleanup
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),	
	@ValueSeriesID	NVARCHAR(255),
	@TimeID			INT = 0
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username						, N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID					, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSeriesID					, N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255))	, N'NULL');
	DECLARE @EffectedRows		INTEGER			= 0;					-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @ProductID		IS NULL	SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Cleanup;

		-- empty TimeID is not allowed
		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID (@ValueSeriesID);

		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
		
		-- STEP 2 - Check rights
		DECLARE @Rights AS TABLE(FactoryID NVARCHAR(255) NOT NULL, ProductLineID NVARCHAR(255) NOT NULL);

		INSERT INTO @Rights (FactoryID, ProductLineID) 
			SELECT
				  FactoryID
				 ,ProductLineID
			FROM system.trUserRights 							 
			WHERE 
				Username = @TransactUsername 
				AND [Right]	= N'Write'
				AND (@FactoryID	= N'' OR FactoryID = @FactoryID)
				AND (@ProductLineID	= N'' OR ProductLineID = @ProductLineID);

		IF NOT EXISTS(SELECT 1 FROM @Rights)
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3 - Delete values where user has rights
		DELETE FROM fV
		FROM planning.tfValues fV 
			INNER JOIN @Rights r ON fV.FactoryID = r.FactoryID COLLATE DATABASE_DEFAULT AND fV.ProductLineID = r.ProductLineID COLLATE DATABASE_DEFAULT
		WHERE fV.TimeID = @TimeID
			AND (@ProductID		 = N''	OR fV.ProductID		 = @ProductID)
			AND (@ValueSeriesID	 = N''	OR fV.ValueSeriesID	 = @ValueSeriesID); 

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;
		
		COMMIT TRANSACTION sx_pf_POST_Cleanup;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Cleanup;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Cleanup'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Cleanup Operation, which will clean the values in planning.tfValues for a given scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ClusterProperty;
GO

/*
	POST Operation for a ClusterProperty
	A not existing Property will be created, an existing Property will be updated
	If the value <#NV> is passed as parameter, it means keep the existing value 
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. Username does not have write access to the cluster at Create => 401 Unauthorized
	4. Property created => 201 Created
	5. Property updated => 200 OK

		DECLARE  @RC				 INT
				,@Username			 NVARCHAR(255) = 'SQL'
				,@PropertyID		 NVARCHAR(255) = 'rre'
				,@PropertyName		 NVARCHAR(255) = ''
				,@CommentUser		 NVARCHAR(255) = ''
				,@CommentDev		 NVARCHAR(255) = ''
				,@Unit				 NVARCHAR(255) = ''
				,@ValueText			 NVARCHAR(255) = ''
				,@ValueInt			 NVARCHAR(255) = '678'
				,@Scale				 NVARCHAR(255) = '<#NV>'
				,@IsROSystemProperty NVARCHAR(255) = '0'
				,@FormatID			 NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_ClusterProperty 
				@Username,
				@PropertyID,
				@PropertyName,
				@CommentUser,
				@CommentDev,
				@Unit,
				@ValueText,
				@ValueInt,
				@Scale,
				@IsROSystemProperty,
				@FormatID

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ClusterProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ClusterProperty','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_POST_ClusterProperty
	@Username			 NVARCHAR(255),
	@PropertyID			 NVARCHAR(255),
	@PropertyName		 NVARCHAR(255),
	@CommentUser		 NVARCHAR(MAX),
	@CommentDev			 NVARCHAR(MAX),
	@Unit				 NVARCHAR(255),
	@ValueText			 NVARCHAR(MAX),
	@ValueInt			 NVARCHAR(255),
	@Scale				 NVARCHAR(255),
	@IsROSystemProperty	 NVARCHAR(255),
	@FormatID			 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @PropertyKey		 INT			= 0;
	DECLARE @SytemROBlocker		 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username				, N'NULL') + N''',''' 
			+ ISNULL(@PropertyID			, N'NULL') + N''','''
			+ ISNULL(@PropertyName			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser			, N'NULL') + N''','''
			+ ISNULL(@CommentDev			, N'NULL') + N''',''' 
			+ ISNULL(@Unit					, N'NULL') + N''','''
			+ ISNULL(@ValueText				, N'NULL') + N''','''
			+ ISNULL(@ValueInt				, N'NULL') + N''','''
			+ ISNULL(@Scale					, N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty	, N'NULL') + N''',''' 
			+ ISNULL(@FormatID				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;					-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;					-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ClusterProperty;
		
		-- STEP 0.2 - Protect input parameters
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);

		IF @PropertyID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @TransactUsername = 'FactoryService';
		END;

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @PropertyKey = PropertyKey FROM planning.tgCluster WHERE PropertyID = @PropertyID; 
				
		-- STEP 1.3 - Don`t allow changing of system RO Properties if user is not FactoryService 
		-- TEMPORARY DISABLE to set the version during Updatescript, which runs with db_owner role in most cases
		--SELECT @SytemROBlocker = IsROSystemProperty FROM planning.tgCluster WHERE PropertyID = @PropertyID;
		
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200 AND NOT @TransactUsername = 'FactoryService' 
			BEGIN
				RAISERROR('Invalid rights', 16, 10);
			END

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgCluster (PropertyID,PropertyName,CommentUser,CommentDev,Unit,ValueText,ValueInt,Scale,IsROSystemProperty,FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key])
			VALUES (@PropertyID,'','','','','',0,1,0,'');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;		

		-- STEP 3.2 - Update Cluster attributes
		UPDATE planning.tgCluster
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev				= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty		= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgCluster 
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF NOT @ResultCode = 201 SET @ResultCode = 200;

		PRINT @ResultCode;

		COMMIT TRANSACTION sx_pf_POST_ClusterProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ClusterProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
		PRINT @ResultCode;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ClusterProperty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ClusterProperty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a ClusterProperty
         A not existing Property will be created, an existing Property will be updated
         If the value <#NV> is passed as parameter, it means keep the existing value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyName';
    SET @value      = N'PropertyName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsROSystemProperty';
    SET @value      = N'IsROSystemProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Factory;
GO

/*
	POST Operation for a Factory
	Not existing Factories are created, existing updated
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call

	DECLARE  @RC				 INT
			,@Username			 NVARCHAR(255) = 'SQL'
			,@FactoryID			 NVARCHAR(255) = 'ZT'
			,@NameShort			 NVARCHAR(255) = ''
			,@NameLong			 NVARCHAR(255) = ''
			,@CommentUser		 NVARCHAR(255) = ''
			,@CommentDev		 NVARCHAR(255) = ''
			,@ResponsiblePerson	 NVARCHAR(255) = ''
			,@ImageName			 NVARCHAR(255) = ''

	EXECUTE @RC = dbo.sx_pf_POST_Factory 
			@Username,
			@FactoryID,
			@NameShort,
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ResponsiblePerson,
			@ImageName
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Factory
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson	NVARCHAR(255),
	@ImageName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')			 + N''','''
			+ ISNULL(@NameShort			, N'NULL')			 + N''','''
			+ ISNULL(@NameLong			, N'NULL')			 + N''','''
			+ ISNULL(@CommentUser		, N'NULL')			 + N''','''
			+ ISNULL(@CommentDev		, N'NULL')			 + N''','''
			+ ISNULL(@ResponsiblePerson	, N'NULL')			 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')			 + N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID			 = N'';
	IF @NameShort			 IS NULL SET @NameShort			 = N'';
	IF @NameLong			 IS NULL SET @NameLong			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser		 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev		 = N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson	 = N'';
	IF @ImageName			 IS NULL SET @ImageName			 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @NameShort			 = dbo.sx_pf_pProtectString	 (@NameShort);
		SET @NameLong			 = dbo.sx_pf_pProtectString	 (@NameLong);
		SET @CommentUser		 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev			 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @ResponsiblePerson	 = dbo.sx_pf_pProtectString	 (@ResponsiblePerson);
		SET @ImageName			 = dbo.sx_pf_pProtectString	 (@ImageName);
		
		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		-- STEP 2 - Check rights
		IF @FactoryKey = 0
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		ELSE
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Factory does not yet exist, it will be created in minimal configuration
		IF @FactoryKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
			INSERT INTO planning.tdFactories (FactoryID, NameShort,NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName) 
				OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
				VALUES (@FactoryID, @NameShort, '', '', '', '', '');

			SELECT TOP(1) @FactoryKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;		
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update Factory attributes
		UPDATE planning.tdFactories 
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			@NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			@NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	@ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			@ImageName)
		FROM planning.tdFactories 
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (no insert needed because only ClusterAdmin can create Factory)
		EXEC system.spMaterialize_trUserRights;	

		COMMIT TRANSACTION sx_pf_POST_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Factory;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryUser;
GO 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Factory'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a Factory
         Not existing Factories are created, existing updated
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ResponsiblePerson';
    SET @value      = N'ResponsiblePerson';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_FactoryProperty;
GO

/*
	POST Operation for a FactoryProperty
	Not existing Property will be created, existing updated.
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	DECLARE  @RC				  INT
			,@Username			  NVARCHAR(255) = 'SQL'
			,@FactoryID			  NVARCHAR(255) = 'ZT'
			,@PropertyID		  NVARCHAR(255) = 'rr3e'
			,@PropertyName		  NVARCHAR(255) = ''
			,@CommentUser		  NVARCHAR(MAX) = ''
			,@CommentDev		  NVARCHAR(MAX) = ''
			,@Unit				  NVARCHAR(255) = ''
			,@ValueText			  NVARCHAR(MAX) = ''
			,@ValueInt			  NVARCHAR(255) = '678'
			,@Scale				  NVARCHAR(255) = '<#NV>'
			,@IsROSystemProperty  NVARCHAR(255) = '2'
			,@FormatID			  NVARCHAR(255) = '<#NV>'

	EXECUTE @RC = dbo.sx_pf_POST_FactoryProperty 
			@Username,
			@FactoryID,
			@PropertyID,
			@PropertyName,
			@CommentUser,
			@CommentDev,
			@Unit,
			@ValueText,
			@ValueInt,
			@Scale,
			@IsROSystemProperty,
			@FormatID
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FactoryProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FactoryProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_FactoryProperty
	@Username			  NVARCHAR(255),
	@FactoryID			  NVARCHAR(255),
	@PropertyID			  NVARCHAR(255),
	@PropertyName		  NVARCHAR(255),
	@CommentUser		  NVARCHAR(MAX),
	@CommentDev			  NVARCHAR(MAX),
	@Unit				  NVARCHAR(255),
	@ValueText			  NVARCHAR(MAX),
	@ValueInt			  NVARCHAR(255),
	@Scale				  NVARCHAR(255),
	@IsROSystemProperty	  NVARCHAR(255),
	@FormatID			  NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			  INT = 0;
	DECLARE @PropertyKey		  INT = 0;
	DECLARE @TransactUsername	  NVARCHAR(255) = N'';
	DECLARE @ProcedureName		  NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	  NVARCHAR(MAX) = N'''' 
			+ ISNULL(@Username,				 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyID,			 N'NULL')	+ N''',''' 
			+ ISNULL(@PropertyName,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser,			 N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev,			 N'NULL')	+ N''',''' 
			+ ISNULL(@Unit,					 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueText,			 N'NULL')	+ N''',''' 
			+ ISNULL(@ValueInt,				 N'NULL')	+ N''',''' 
			+ ISNULL(@Scale,				 N'NULL')	+ N''',''' 
			+ ISNULL(@IsROSystemProperty,	 N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID,				 N'NULL')	+ N'''';		
	DECLARE @EffectedRows		  INT			 = 0;				
	DECLARE @ResultCode			  INT			 = 501;				
	DECLARE @TimestampCall		  DATETIME		 = GETUTCDATE();
	DECLARE @Comment			  NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username				 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID				 = N'';
	IF @PropertyID			 IS NULL SET @PropertyID			 = N'';
	IF @PropertyName		 IS NULL SET @PropertyName			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser			 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev			 = N'';
	IF @Unit				 IS NULL SET @Unit					 = N'';
	IF @ValueText			 IS NULL SET @ValueText				 = N'';
	IF @ValueInt			 IS NULL SET @ValueInt				 = N'';
	IF @Scale				 IS NULL SET @Scale					 = N'';
	IF @IsROSystemProperty	 IS NULL SET @IsROSystemProperty	 = N'';
	IF @FormatID			 IS NULL SET @FormatID				 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FactoryProperty;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @PropertyID		 = dbo.sx_pf_pProtectID		 (@PropertyID);
		SET @PropertyName	 = dbo.sx_pf_pProtectString	 (@PropertyName);
		SET @CommentUser	 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev		 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @Unit			 = dbo.sx_pf_pProtectString	 (@Unit);
		SET @ValueText		 = dbo.sx_pf_pProtectString	 (@ValueText);
		SET @ValueInt		 = dbo.sx_pf_pProtectInt	 (@ValueInt);
		SET @Scale			 = dbo.sx_pf_pProtectInt	 (@Scale);
		SET @FormatID		 = dbo.sx_pf_pProtectString	 (@FormatID); --has String Protection, as also #<NV> can be used as parameter
				
		IF	@PropertyID = N''
			OR @FactoryID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;
		
		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - disabled from Version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgFactories WHERE FactoryKey = @FactoryKey AND PropertyID = @PropertyID;

		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

		-- STEP 3.1 - If property not exist, it will created minimalistic	
		IF @PropertyKey = 0
		BEGIN
			INSERT INTO planning.tgFactories (FactoryKey, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys
			VALUES (@FactoryKey, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];
		   
			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update Factory property
		UPDATE planning.tgFactories
		SET 
			 PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			,IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgFactories
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FactoryProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FactoryProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FactoryProperty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_FactoryProperty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a FactoryProperty
         Not existing Property will be created, existing updated.
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyName';
    SET @value      = N'PropertyName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsROSystemProperty';
    SET @value      = N'IsROSystemProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_FlexAction;
GO

/*
	POST Operation for sending FlexAction Parameter
	Procedure must be editend to fit user needs and recreated after each API Update.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	--should return Data and 200
	DECLARE  @RC					NVARCHAR(255)
			,@Username				NVARCHAR(255) = 'SQL'
			,@FactoryID				NVARCHAR(255) = 'F1'
			,@ProductLineID			NVARCHAR(255) = 'PL1'
			,@ProductID				NVARCHAR(255) = 'P1'
			,@ValueSeriesID			NVARCHAR(255) = 'Mitarbeiter'
			,@TimeID				BIGINT		  = 20180115
			,@GlobalattributeNumber	NVARCHAR(255) = ''
			,@PageType				NVARCHAR(255) = ''
			,@FlexParameter			NVARCHAR(255) = 'P1000'


	EXEC @RC = dbo.sx_pf_POST_FlexAction
		 @Username,
		 @FactoryID,
		 @ProductLineID,
		 @ProductID,
		 @ValueSeriesID,
		 @TimeID,
		 @GlobalattributeNumber,
		 @PageType,
		 @FlexParameter
	 	
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FlexAction',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_FlexAction','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_FlexAction
	 @Username				NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@ProductID				NVARCHAR(255)
	,@ValueSeriesID			NVARCHAR(255)
	,@TimeID				BIGINT
	,@GlobalattributeNumber NVARCHAR(255)
	,@PageType				NVARCHAR(255)  -- PDT (ProductDataTable), PLP (ProductLinePage)
	,@FlexParameter			NVARCHAR(255)  -- The Identifier (Second column of GET_FlexAction, first Display column) from the choosen list element
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255);
	DECLARE @ProcedureName		NVARCHAR(255)	 = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows		INT				 = 0;					
	DECLARE @ResultCode			INT				 = 501;						
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';	
	DECLARE @ParameterString	NVARCHAR (MAX)	 = N'''' 
			+ ISNULL(@Username,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FactoryID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductLineID,			 N'NULL')		+ N''',''' 
			+ ISNULL(@ProductID,				 N'NULL')		+ N''',''' 
			+ ISNULL(@ValueSeriesID,			 N'NULL')		+ N''','
			+ CAST(ISNULL(@TimeID, N'NULL') AS NVARCHAR(255))	+ N','''
			+ ISNULL(@GlobalattributeNumber,	 N'NULL')		+ N''',''' 
			+ ISNULL(@PageType,					 N'NULL')		+ N''',''' 
			+ ISNULL(@FlexParameter,			 N'NULL')		+ N'''';  
	DECLARE @Template			NVARCHAR(255)	 = N'';

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @ProductID				IS NULL SET @ProductID				= N'';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID			= N'';
	IF @TimeID					IS NULL SET @TimeID					= 0;
	IF @GlobalattributeNumber	IS NULL SET @GlobalattributeNumber	= '';
	IF @PageType				IS NULL SET @PageType				= N'';
	IF @FlexParameter			IS NULL SET @FlexParameter			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_FlexAction;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @FlexParameter	= dbo.sx_pf_pProtectString	(@FlexParameter);
				
		-- STEP 1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END

		-- STEP 2 - Get template
		SELECT @Template = COALESCE(Template,'') FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID

		-- STEP 3 - Sample Actions
		-- React on content parameters and FlexParameter for action

		IF @ValueSeriesID = 'Projekt'
		BEGIN
			-- POST_ValueSeries of Type Projekt, ValueSeriesID = FlexParameter
			PRINT 'Did Projekt'
			SET @ResultCode = 200;
		END

		IF @ValueSeriesID = 'Mitarbeiter' 
		BEGIN
			-- POST ValuesSeries of Type Mitarbeiter, ValueSeriesID = FlexParameter
			PRINT 'Did Mitarbeiter'
			SET @ResultCode = 200;
		END
		
		IF @ValueSeriesID NOT IN ('Projekt','Mitarbeiter' )
		BEGIN
			PRINT 'No fitting action found.'
			SET @ResultCode = 404
		END
			   
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_FlexAction;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_FlexAction;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_FlexAction TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_FlexAction'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for sending FlexAction Parameter
         Procedure must be editend to fit user needs and recreated after each API Update.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalattributeNumber';
    SET @value      = N'GlobalattributeNumber';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageType';
    SET @value      = N'PageType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FlexParameter';
    SET @value      = N'FlexParameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Format;
GO

/*
	POST Operation for Format
	Not existing Format will be created, existing updated.
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE	 @RC					INT
				,@Username				NVARCHAR(255) = 'SQL'					'SQL'
				,@FormatID				NVARCHAR(255) = 'sxTest4'				'sxTest3'
				,@BackgroundColor		NVARCHAR(255) = '100,100,100'			'<#NV>'
				,@FontColor				NVARCHAR(255) = '200,200,200'			'<#NV>'
				,@ValueFormat			NVARCHAR(255) = ''						'<#NV>'
				,@Bold					NVARCHAR(255) = ''						'<#NV>'
				,@Italic				NVARCHAR(255) = ''						'<#NV>'
				,@AlignmentHorizontal	NVARCHAR(255) = ''						'<#NV>'
				,@AlignmentVertical		NVARCHAR(255) = ''						'<#NV>'
				,@TextUnderlined		NVARCHAR(255) = ''						'<#NV>'
				,@BorderToNext			NVARCHAR(255) = ''						'<#NV>'
				,@BorderToPrevious		NVARCHAR(255) = ''  					'<#NV>'  
		
		EXECUTE @RC = dbo.sx_pf_POST_Format 
				@Username,
				@FormatID,
				@BackgroundColor,
				@FontColor,
				@ValueFormat,
				@Bold,				
				@Italic,				
				@AlignmentHorizontal,
				@AlignmentVertical,	
				@TextUnderlined,		
				@BorderToNext,		
				@BorderToPrevious	
		PRINT @RC 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Format','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Format
	@Username				NVARCHAR(255),
	@FormatID				NVARCHAR(255),
	@BackgroundColor		NVARCHAR(255),
	@FontColor				NVARCHAR(255),
	@ValueFormat			NVARCHAR(255),
	@Bold					NVARCHAR(255),
	@Italic					NVARCHAR(255),
	@AlignmentHorizontal	NVARCHAR(255),
	@AlignmentVertical		NVARCHAR(255),
	@TextUnderlined			NVARCHAR(255),
	@BorderToNext			NVARCHAR(255),
	@BorderToPrevious		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @FormatKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N''''
			+ ISNULL(@Username				, N'NULL')		+ N''','''
			+ ISNULL(@FormatID				, N'NULL')		+ N''','''
			+ ISNULL(@BackgroundColor		, N'NULL')		+ N''','''
			+ ISNULL(@FontColor				, N'NULL')		+ N''','''
			+ ISNULL(@ValueFormat			, N'NULL')		+ N''',''' 
			+ ISNULL(@Bold					, N'NULL')		+ N''','''
			+ ISNULL(@Italic				, N'NULL')		+ N''',''' 
			+ ISNULL(@AlignmentHorizontal	, N'NULL')		+ N''','''
			+ ISNULL(@AlignmentVertical		, N'NULL')		+ N''','''
			+ ISNULL(@TextUnderlined		, N'NULL')		+ N''','''
			+ ISNULL(@BorderToNext			, N'NULL')		+ N''','''
			+ ISNULL(@BorderToPrevious		, N'NULL')		+ N''','''
			+ N'''';
	DECLARE @EffectedRows	 INT = 0;						
	DECLARE @ResultCode		 INT = 501;						
	DECLARE @TimestampCall	 DATETIME = GETUTCDATE();
	DECLARE @Comment		 NVARCHAR (2000) = N'';
	DECLARE @ErrorText		 NVARCHAR (2000) = N';'			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			 = N'';
	IF @FormatID			IS NULL SET @FormatID			 = N'';
	IF @BackgroundColor		IS NULL SET @BackgroundColor	 = N'';
	IF @FontColor			IS NULL SET @FontColor			 = N'';
	IF @ValueFormat			IS NULL SET @ValueFormat		 = N'';
	IF @Bold				IS NULL SET @Bold				 = N'';
	IF @Italic				IS NULL SET @Italic				 = N'';
	IF @AlignmentHorizontal	IS NULL SET @AlignmentHorizontal = N'';
	IF @AlignmentVertical	IS NULL SET @AlignmentVertical	 = N'';
	IF @TextUnderlined		IS NULL SET @TextUnderlined		 = N'';
	IF @BorderToNext		IS NULL SET @BorderToNext		 = N'';
	IF @BorderToPrevious	IS NULL SET @BorderToPrevious	 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Format;

		-- STEP 0.2 - Protect input parameters
		SET @FormatID			 = dbo.sx_pf_pProtectID		(@FormatID);
		SET @BackgroundColor	 = dbo.sx_pf_pProtectString (@BackgroundColor);
		SET @FontColor			 = dbo.sx_pf_pProtectString (@FontColor);
		SET @ValueFormat		 = dbo.sx_pf_pProtectString (@ValueFormat);
		SET @Bold				 = dbo.sx_pf_pProtectInt	(@Bold);
		SET @Italic				 = dbo.sx_pf_pProtectInt	(@Italic);
		SET @AlignmentHorizontal = dbo.sx_pf_pProtectString (@AlignmentHorizontal);
		SET @AlignmentVertical	 = dbo.sx_pf_pProtectString (@AlignmentVertical);
		SET @TextUnderlined		 = dbo.sx_pf_pProtectInt	(@TextUnderlined)
		SET @BorderToNext		 = dbo.sx_pf_pProtectInt	(@BorderToNext)
		SET @BorderToPrevious	 = dbo.sx_pf_pProtectInt	(@BorderToPrevious)

		IF @FormatID = N''
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		IF	@Bold				NOT IN ('<#NV>','-1','0','1') OR
			@Italic				NOT IN ('<#NV>','-1','0','1') OR
			@BorderToNext		NOT IN ('<#NV>','-1','0','1') OR
			@BorderToPrevious	NOT IN ('<#NV>','-1','0','1') 
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter for Bold, Italic, BorderToNext, BorderToPrevios not supported (outside <#NV>,-1,0,1).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentHorizontal NOT IN ('<#NV>','','left','center','right')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentHorizontal + ' for AlignmentHorizontal not supported (outside <#NV>, left, center, rights, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;

		IF @AlignmentVertical NOT IN ('<#NV>','','top','center','bottom')
			BEGIN
				SET @ResultCode = 401;
				SET @ErrorText = 'Parameter ' + @AlignmentVertical + ' for AlignmentVertical not supported (outside <#NV>, top, center, bottom, emptystring).'
				RAISERROR(@ErrorText, 16, 10);
			END;


		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @FormatKey = FormatKey FROM planning.thFormats WHERE FormatID = @FormatID;
			
		-- STEP 3.1 - If format does not yet exist, it will be created
		IF @FormatKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thFormats (FormatID,BackgroundColor,FontColor,ValueFormat,Bold,Italic,AlignmentHorizontal,AlignmentVertical	
												,TextUnderlined,BorderToNext,BorderToPrevious	
												) 
			OUTPUT INSERTED.FormatKey INTO @Keys([Key]) 
				VALUES (@FormatID,'','','','-1','-1','','','-1','0','0');

			SELECT TOP (1) @FormatKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;
		
		-- STEP 3.2 - Update format attributes
		UPDATE planning.thFormats 
		SET 
			 BackgroundColor	= IIF(@BackgroundColor		= N'<#NV>', BackgroundColor,		@BackgroundColor)
			,FontColor			= IIF(@FontColor			= N'<#NV>', FontColor,				@FontColor)
			,ValueFormat		= IIF(@ValueFormat			= N'<#NV>', ValueFormat,			@ValueFormat)
			,Bold				= IIF(@Bold					= N'<#NV>', Bold,					@Bold)
			,Italic				= IIF(@Italic				= N'<#NV>', Italic,					@Italic)
			,AlignmentHorizontal= IIF(@AlignmentHorizontal	= N'<#NV>', AlignmentHorizontal,	@AlignmentHorizontal)
			,AlignmentVertical	= IIF(@AlignmentVertical	= N'<#NV>', AlignmentVertical,		@AlignmentVertical)
			,TextUnderlined		= IIF(@TextUnderlined		= N'<#NV>', TextUnderlined,			@TextUnderlined)
			,BorderToNext		= IIF(@BorderToNext			= N'<#NV>', BorderToNext,			@BorderToNext)
			,BorderToPrevious	= IIF(@BorderToPrevious		= N'<#NV>', BorderToPrevious,		@BorderToPrevious)
		FROM planning.thFormats
		WHERE FormatKey = @FormatKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Format;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Format;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Format TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Format'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Format
         Not existing Format will be created, existing updated.
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BackgroundColor';
    SET @value      = N'BackgroundColor';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FontColor';
    SET @value      = N'FontColor';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormat';
    SET @value      = N'ValueFormat';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Bold';
    SET @value      = N'Bold';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Italic';
    SET @value      = N'Italic';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@AlignmentHorizontal';
    SET @value      = N'AlignmentHorizontal';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@AlignmentVertical';
    SET @value      = N'AlignmentVertical';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TextUnderlined';
    SET @value      = N'TextUnderlined';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BorderToNext';
    SET @value      = N'BorderToNext';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@BorderToPrevious';
    SET @value      = N'BorderToPrevious';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_HTMLPage;
GO

/*
	POST Operation for an HTML Page
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ProductID		NVARCHAR(255) = ''
				,@ProductLineID	NVARCHAR(255) = ''
				,@FactoryID		NVARCHAR(255) = ''
				,@PageID		NVARCHAR(255) = 'Page1'
				,@PageName		NVARCHAR(255) = 'A great page'
				,@PageContent	NVARCHAR(255) = 'Hallo Gerd'

		EXECUTE @RC = dbo.sx_pf_POST_HTMLPage
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@PageID
				,@PageName
				,@PageContent
		PRINT @RC

		SELECT * FROM planning.tgHTMLPages

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_HTMLPage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_HTMLPage','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_HTMLPage
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PageID				NVARCHAR(255),
	@PageName			NVARCHAR(255),
	@PageContent		NVARCHAR(MAX)		

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@PageID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@PageName,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PageID			IS NULL SET @PageID			= 0;
	IF @PageName		IS NULL SET @PageName		= '';
	IF @PageContent		IS NULL SET @PageContent	= '';


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_HTMLPage;

		-- Page must have an ID, all other parameter may be empty
		IF @PageID = ''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @PageID			= dbo.sx_pf_pProtectString (@PageID);
		SET @PageName		= dbo.sx_pf_pProtectString (@PageName);


		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;	

		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;

		-- Check rights on different levels
		IF @FactoryKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @FactoryKey > 0 AND @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Delete old PageID
		DELETE FROM planning.tgHTMLPages 
		WHERE 
				ProductKey		= @ProductKey 
			AND ProductLineKey	= @ProductLineKey
			AND FactoryKey		= @FactoryKey
			AND PageID			= @PageID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- Create new Page
		INSERT INTO planning.tgHTMLPages	(
			 ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,PageID
			,PageName
			,PageContent
			)
			VALUES (
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@PageID,
				@PageName,
				@PageContent
			);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_HTMLPage'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for an HTML Page';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageID';
    SET @value      = N'PageID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageName';
    SET @value      = N'PageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageContent';
    SET @value      = N'PageContent';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_List;
GO

/*
	POST Operation for a List
	Not existing Format will be created, existing updated.
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ListID		NVARCHAR(255) = 'sxScales'
				,@NameShort		NVARCHAR(255) = '<#NV>'
				,@NameLong		NVARCHAR(255) = '<#NV>'
				,@CommentDev	NVARCHAR(255) = '<#NV>'
				,@CommentUser	NVARCHAR(255) = '<#NV>'
				,@Datentyp		NVARCHAR(255) = '<#NV>'
				,@Source		NVARCHAR(255) = '<#NV>'
				,@SourceFormula	NVARCHAR(255) = 'Testen'
				,@FormatID		NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_List 
				@Username,
				@ListID,
				@NameShort,
				@NameLong,
				@CommentDev,
				@CommentUser,
				@Datentyp,
				@Source,
				@SourceFormula,
				@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_List
	 @Username		NVARCHAR(255)
	,@ListID		NVARCHAR(255)
	,@NameShort		NVARCHAR(255)
	,@NameLong		NVARCHAR(255)
	,@CommentDev	NVARCHAR(255)
	,@CommentUser	NVARCHAR(255)
	,@Datentyp		NVARCHAR(255)
	,@Source		NVARCHAR(255)
	,@SourceFormula NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username		,N'NULL')	+ N''',''' 
			+ ISNULL(@ListID		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameShort		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameLong		,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev	,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser	,N'NULL')	+ N''',''' 
			+ ISNULL(@Datentyp		,N'NULL')	+ N''',''' 
			+ ISNULL(@Source		,N'NULL')	+ N''',''' 
			+ ISNULL(@SourceFormula	,N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID		,N'NULL')	+ N'''';	
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ListID			IS NULL SET @ListID			= N'';
	IF @NameShort		IS NULL SET @NameShort		= N'';
	IF @NameLong		IS NULL SET @NameLong		= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Datentyp		IS NULL SET @Datentyp		= N'';
	IF @Source			IS NULL SET @Source			= N'';
	IF @SourceFormula	IS NULL SET @SourceFormula	= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_List;

		-- STEP 0.2 - Protect input parameters
		SET @ListID			= dbo.sx_pf_pProtectID (@ListID);
		SET @NameShort		= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong		= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser	= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString (@CommentDev);
		SET @Datentyp		= dbo.sx_pf_pProtectString (@Datentyp);
		SET @Source			= dbo.sx_pf_pProtectString (@Source);
		SET @SourceFormula	= dbo.sx_pf_pProtectString (@SourceFormula);
		SET @FormatID		= dbo.sx_pf_pProtectString (@FormatID); -- as it can be #<NV>

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @ListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

		-- STEP 3.1 - If List is does not exist, it will be created
		IF @ListKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thLists (ListID, NameShort, NameLong, CommentDev, CommentUser, Datentyp, Source, SourceFormula, FormatID) 
			OUTPUT INSERTED.ListKey INTO @Keys([Key])
			VALUES (@ListID, '', '', '', '', '', '', '', '');

			SELECT TOP(1) @ListKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update List attributes
		UPDATE planning.thLists 
		SET	 
			 NameShort		= IIF(@NameShort		= N'<#NV>', NameShort,		@NameShort)
			,NameLong		= IIF(@NameLong			= N'<#NV>', NameLong,		@NameLong)
			,CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,		@CommentDev)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser,	@CommentUser)
			,Datentyp		= IIF(@Datentyp			= N'<#NV>', Datentyp,		@Datentyp)
			,Source			= IIF(@Source			= N'<#NV>', Source,			@Source)
			,SourceFormula  = IIF(@SourceFormula	= N'<#NV>', SourceFormula,	@SourceFormula)
			,FormatID		= IIF(@FormatID			= N'<#NV>', FormatID,		@FormatID)
		FROM planning.thLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_List'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a List
         Not existing Format will be created, existing updated.
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Datentyp';
    SET @value      = N'Datentyp';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFormula';
    SET @value      = N'SourceFormula';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ListValue;
GO

/*
	POST Operation for an ListValue
	Not existing ListValue will be created (if Key = 0 is used), existing updated.
	Special: This procedure allows to pass a Key value, as there is no ID for a ListValue

	Saxess Software GmbH
	Last modified: 09/2023 for OCT 5.10

	Test call
		DECLARE  @RC			 INT
				,@Username		 NVARCHAR(255)	= 'SQL'
				,@ListValueKey	 INT			= 0
				,@ListID		 NVARCHAR(255)	= 'sxStatus'
				,@ValueInt		 INT			= 5
				,@Scale			 INT			= 1
				,@ValueText		 NVARCHAR(255)	= 'Testo'
				,@ValueComment	 NVARCHAR(255)	= ''
				,@FormatID		 NVARCHAR(255)	= ''
		
		EXECUTE @RC = dbo.sx_pf_POST_ListValue 
				@Username,
				@ListValueKey,
				@ListID,
				@ValueInt,
				@Scale,
				@ValueText,
				@ValueComment,
				@FormatID
		PRINT @RC

	Test call API Log
		SELECT TOP  5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ListValue',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ListValue','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ListValue
	 @Username		NVARCHAR(255)
	,@ListValueKey	INT
	,@ListID		NVARCHAR(255)
	,@ValueInt		INT
	,@Scale			INT
	,@ValueText		NVARCHAR(255)
	,@ValueComment	NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN

	DECLARE  @ListKey			INT			= 0
			,@TransactUsername	NVARCHAR(255)  = N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N'''' 
							+ ISNULL(@Username,								 N'NULL') + N''',' 
							+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)),	 N'NULL') + N',''' 
							+ ISNULL(@ListID,								 N'NULL') + N''',' 
							+ ISNULL(CAST(@ValueInt		AS NVARCHAR(255)),	 N'NULL') + N',' 
							+ ISNULL(CAST(@Scale		AS NVARCHAR(255)),	 N'NULL') + N',''' 
							+ ISNULL(@ValueText,							 N'NULL') + N''',''' 
							+ ISNULL(@ValueComment,							 N'NULL') + N''',''' 
							+ ISNULL(@FormatID,								 N'NULL') + N''''
			,@EffectedRows		INT			= 0			
			,@ResultCode		INT			= 501		
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000) = N''
			
			,@tmpKey			BIGINT		  = 0
			,@tmpListType		NVARCHAR(255) = '';			

	-- NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @ListValueKey	IS NULL SET @ListValueKey	 = 0;
	IF @ListID			IS NULL SET @ListID			 = N'';
	IF @ValueInt		IS NULL SET @ValueInt		 = 0;
	IF @Scale			IS NULL SET @Scale			 = 0;
	IF @ValueText		IS NULL SET @ValueText		 = N'';
	IF @ValueComment	IS NULL SET @ValueComment	 = N'';
	IF @FormatID		IS NULL SET @FormatID		 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ListValue;

		-- protect input parameters
		SET @ListValueKey	= dbo.sx_pf_pProtectInt		(@ListValueKey);
		SET @ListID			= dbo.sx_pf_pProtectID		(@ListID);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);
		SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys and break if not existing
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;
		
		IF @ListKey = 0	BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check if Listvalue exists, when ListValueKey 0 is submitted - 0 means new entry, but API must check if entry is really new
		IF @ListValueKey = 0
			BEGIN
				
				SELECT @tmpListType = hL.Datentyp FROM planning.thLists hL WHERE hL.ListID = @ListID;

				IF @tmpListType = 'String'
					BEGIN
						SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueText = @ValueText;
					END
				ELSE
					BEGIN
						SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueInt = @ValueInt;
					END

				IF (@tmpKey IS NOT NULL AND @tmpKey != 0) SET @ListValueKey = @tmpKey;
			END

		-- If ListValueKey = 0 is submitted, create new value
		IF @ListValueKey = 0
			BEGIN
				INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
				VALUES (@ListID, @ValueInt, @Scale, @ValueText, @ValueComment, @FormatID);

				SET @Resultcode = 201;
				SET @EffectedRows = 1;
			END
		ELSE
			BEGIN
			-- existing key is committed, update the Key
				UPDATE planning.thListValues 
				SET 
					 ValueInt		 = @ValueInt
					,Scale			 = @Scale
					,ValueText		 = @ValueText
					,ValueComment	 = @ValueComment
					,FormatID		 = @FormatID
				WHERE ListValueKey	 = @ListValueKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;	

		COMMIT TRANSACTION sx_pf_POST_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ListValue'			-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ListValue
         Not existing ListValue will be created, existing updated.
         This procedure allows to pass a Key value, as there is no ID for a ListValue';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListValueKey';
    SET @value      = N'Use existing key to update an ListValue, use 0 to create a new ListValue';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID where this Value belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt if List of Type integer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale if List of Type integer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText if List of Type String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueComment';
    SET @value      = N'Comment for this value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID for this value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Preference;
GO

/*
	POST Operation for Preference
	Not existing Preference will be created, existing updated.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test Calls
		-- make Property UserDependend
		EXEC dbo.sx_pf_POST_ProductLineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','1','1','2',''

		-- make Property NOT UserDependend
		EXEC dbo.sx_pf_POST_ProductLineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','0','1','2',''

		DECLARE	 @RC					INT
				,@Username 				NVARCHAR(255) = 'SQL'
				,@PreferenceUserName	NVARCHAR(255) = 'AD-ONE\gtautenh'
				,@ProductLineID			NVARCHAR(255) = 'U'
				,@FactoryID				NVARCHAR(255) = 'ZT'
				,@SettingName			NVARCHAR(255) = 'Demopref'
				,@ValueInt				NVARCHAR(255) = '8'
				,@ValueText				NVARCHAR(255) = 'uztr'

		EXECUTE @RC = dbo.sx_pf_POST_Preference 
				@Username,
				@PreferenceUsername,
				@ProductLineID,
				@FactoryID,
				@SettingName,
				@ValueInt,
				@ValueText;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Preference',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Preference','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255),
	@ValueInt			INT,
	@ValueText			NVARCHAR(MAX)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
	DECLARE @FactoryKey						INT				= 0
	DECLARE @ProductLineKey					INT				= 0
	DECLARE @PreferenceIsUserDependendFlag	INT				= 0
	DECLARE @ProcedureName					NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,							N'NULL')	+ N''',''' 
			+ ISNULL(@PreferenceUsername,				N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,					N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,						N'NULL')	+ N''',''' 
			+ ISNULL(@SettingName,						N'NULL') 	+ N''',' 
			+ ISNULL(CAST(@ValueInt AS NVARCHAR(255)),	N'NULL')	+ N',''' 
			+ ISNULL(@ValueText,						N'NULL')	+ N''''
	DECLARE @EffectedRows					INT				= 0
	DECLARE @ResultCode						INTEGER			= 501					
	DECLARE @TimestampCall					DATETIME		= GETUTCDATE()
	DECLARE @Comment						NVARCHAR(2000)	= N'';	
	DECLARE @UserKey						INT				= 0;				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';
	IF @ValueInt			IS NULL SET @ValueInt			= 0;
	IF @ValueText			IS NULL SET @ValueText			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Preference

		-- STEP 0.2 - Protect input parameters
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @SettingName	= dbo.sx_pf_pProtectString	(@SettingName);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);

		IF	@SettingName = N'' 
			OR @SettingName = '<#NV>'
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine, if Preferences are saved per User
		-- for global Preferences
		IF	@FactoryID = N'' 
			AND @ProductLineID = N''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = 'C06';
		END
		-- for Preferences per Factory
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID = ''
		BEGIN 
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = 'F06';
		END
		-- for Preferences per ProductLine
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID <> ''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = 'P06';
		END;

		-- STEP 1.2 - if Preferences are not Userdependend, only a clusteradmin can save them, else everybody with
		-- Factory / ProductLine Write Rights
		ELSE IF @PreferenceIsUserDependendFlag <> 1 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
			
			IF @ResultCode <> 200
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- STEP 1.3 - Determine Key, they may be 0 if the preference is for Cluster / Factory
		SELECT @FactoryKey	= ISNULL(FactoryKey, 0)	FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ISNULL(ProductLineKey, 0) FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		-- Cancel if Factory or Product Line does not exist
		IF	(@FactoryKey = 0 AND @FactoryID <> N'')
			OR	(@ProductLineKey = 0 AND @ProductLineID <> N'')
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Determine UserKey
		SELECT @UserKey = UserKey FROM system.trUser WHERE Username = @PreferenceUsername

		-- STEP 1.4 - Set Preference User to "All" if Preferences are not Userdependend
		IF @PreferenceIsUserDependendFlag <> 1 
			BEGIN
				SET @PreferenceUsername = N'All';
				SET @UserKey			= 0 ;
			END
		ELSE
		-- STEP 1.5 - Test existence of Preferences Users if Preference is Userdepend
		IF	@UserKey = 0 AND @PreferenceIsUserDependendFlag <> 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User don`t exists', 16, 10);
		END;	
		
		-- STEP 2. Rights check
		IF @ProductLineID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		END
		ELSE IF @FactoryID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		END
		ELSE IF @FactoryID = N''
				AND @ProductLineID = N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		END

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3. Clear Preference and edit
		-- if preferences are userdependend, they are deleted only for the specific user, else for all user
		IF @PreferenceIsUserDependendFlag = 1
			BEGIN
				DELETE FROM planning.trPreferences
				WHERE 
						FactoryID		= @FactoryID 
					AND ProductLineID	= @ProductLineID
					AND Username		= @PreferenceUsername
					AND SettingName		= @SettingName
				END
			ELSE
				BEGIN
					DELETE FROM planning.trPreferences
					WHERE 
							FactoryID		= @FactoryID 
						AND ProductLineID	= @ProductLineID
						AND SettingName		= @SettingName
				END;

		SET @EffectedRows = @@ROWCOUNT;

		-- TODO handle #NV - at the moment not handled
		SET @ValueText = IIF(@ValueText = N'<#NV>', N'', @ValueText);

		-- Save Preference
		INSERT INTO planning.trPreferences (UserKey,UserName, FactoryID, ProductLineID, SettingName, ValueInt, ValueText)
		VALUES (@UserKey,@PreferenceUsername, @FactoryID, @ProductLineID, @SettingName, @ValueInt, @ValueText);

		SET @EffectedRows += @@ROWCOUNT;
		SET @Resultcode = 201;

		COMMIT TRANSACTION sx_pf_POST_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Preference'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Preference
         Not existing Preference will be created, existing updated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PreferenceUsername';
    SET @value      = N'PreferenceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SettingName';
    SET @value      = N'SettingName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Product;
GO

/*
	POST Operation for one Product
	Not existing Product will be created if it exists in ZT Factory. Existing Product updated
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	DECLARE  @RC					INT
			,@Username			NVARCHAR(255) = 'SQL'
			,@ProductID			NVARCHAR(255) = '*'
			,@ProductLineID		NVARCHAR(255) = 'U'
			,@FactoryID			NVARCHAR(255) = 'ZT'
			,@TimeType			NVARCHAR(255) = 'VM'
			,@NameShort			NVARCHAR(255) = ''
			,@NameLong			NVARCHAR(255) = ''
			,@CommentUser		NVARCHAR(MAX) = ''
			,@CommentDev		NVARCHAR(MAX) = ''
			,@ResponsiblePerson	NVARCHAR(255) = ''
			,@ImageName			NVARCHAR(255) = ''
			,@Status			NVARCHAR(255) = 'Aktiv'
			,@Template			NVARCHAR(255) = 'Unikum_VM'
			,@TemplateVersion	NVARCHAR(255) = 'VT'
			,@GA1				NVARCHAR(255) = 'Wert für GA1'	--'<#NV>'
			,@GA2				NVARCHAR(255) = '<#NV>'
			,@GA3				NVARCHAR(255) = '<#NV>'
			,@GA4				NVARCHAR(255) = 'rer'
			,@GA5				NVARCHAR(255) = '<#NV>'
			,@GA6				NVARCHAR(255) = '<#NV>'
			,@GA7				NVARCHAR(255) = '<#NV>'
			,@GA8				NVARCHAR(255) = '<#NV>'
			,@GA9				NVARCHAR(255) = '<#NV>'
			,@GA10				NVARCHAR(255) = '<#NV>'
			,@GA11				NVARCHAR(255) = '<#NV>'
			,@GA12				NVARCHAR(255) = '<#NV>'
			,@GA13				NVARCHAR(255) = '<#NV>'
			,@GA14				NVARCHAR(255) = '<#NV>'
			,@GA15				NVARCHAR(255) = '<#NV>'
			,@GA16				NVARCHAR(255) = '<#NV>'
			,@GA17				NVARCHAR(255) = '<#NV>'
			,@GA18				NVARCHAR(255) = '<#NV>'
			,@GA19				NVARCHAR(255) = '<#NV>'
			,@GA20				NVARCHAR(255) = '<#NV>'
			,@GA21				NVARCHAR(255) = '<#NV>'
			,@GA22				NVARCHAR(255) = '<#NV>'
			,@GA23				NVARCHAR(255) = '<#NV>'
			,@GA24				NVARCHAR(255) = '<#NV>'
			,@GA25				NVARCHAR(255) = '<#NV>'

	EXECUTE @RC = dbo.sx_pf_POST_Product 
			@Username,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@TimeType,
			@NameShort,
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ResponsiblePerson,
			@ImageName,
			@Status,
			@Template,
			@TemplateVersion,
			@GA1,
			@GA2,
			@GA3,
			@GA4,
			@GA5,
			@GA6,
			@GA7,
			@GA8,
			@GA9,
			@GA10,
			@GA11,
			@GA12,
			@GA13,
			@GA14,
			@GA15,
			@GA16,
			@GA17,
			@GA18,
			@GA19,
			@GA20,
			@GA21,
			@GA22,
			@GA23,
			@GA24,
			@GA25;
	PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Product
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5				NVARCHAR(255),
	@GA6				NVARCHAR(255),
	@GA7				NVARCHAR(255),
	@GA8				NVARCHAR(255),
	@GA9				NVARCHAR(255),
	@GA10				NVARCHAR(255),
	@GA11				NVARCHAR(255),
	@GA12				NVARCHAR(255),
	@GA13				NVARCHAR(255),
	@GA14				NVARCHAR(255),
	@GA15				NVARCHAR(255),
	@GA16				NVARCHAR(255),
	@GA17				NVARCHAR(255),
	@GA18				NVARCHAR(255),
	@GA19				NVARCHAR(255),
	@GA20				NVARCHAR(255),
	@GA21				NVARCHAR(255),
	@GA22				NVARCHAR(255),
	@GA23				NVARCHAR(255),
	@GA24				NVARCHAR(255),
	@GA25				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductID			, N'NULL')	 + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL')	 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')	 + N''',''' 
			+ ISNULL(@TimeType			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameShort			, N'NULL')	 + N''',''' 
			+ ISNULL(@NameLong			, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL')	 + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL')	 + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL')	 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')	 + N''',''' 
			+ ISNULL(@Status			, N'NULL')	 + N''',''' 
			+ ISNULL(@Template			, N'NULL')	 + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL')	 + N''',''' 
			+ ISNULL(@GA1				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA2				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA3				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA4				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA5				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA6				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA7				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA8				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA9				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA10				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA11				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA12				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA13				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA14				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA15				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA16				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA17				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA18				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA19				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA20				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA21				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA22				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA23				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA24				, N'NULL')	 + N''',''' 
			+ ISNULL(@GA25				, N'NULL')	 + N'''';
	DECLARE @EffectedRows		INT				 = 0;						
	DECLARE @ResultCode			INT				 = 501;					
	DECLARE @TimestampCall		DATETIME		 = GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	 = N'';		

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			= N'';
	IF @ProductID			 IS NULL SET @ProductID			= N'';
	IF @ProductLineID		 IS NULL SET @ProductLineID		= N'';
	IF @FactoryID			 IS NULL SET @FactoryID			= N'';
	IF @TimeType			 IS NULL SET @TimeType			= N'';
	IF @NameShort			 IS NULL SET @NameShort			= N'';
	IF @NameLong			 IS NULL SET @NameLong			= N'';
	IF @CommentUser			 IS NULL SET @CommentUser		= N'';
	IF @CommentDev			 IS NULL SET @CommentDev		= N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson = N'';
	IF @ImageName			 IS NULL SET @ImageName			= N'';
	IF @Status				 IS NULL SET @Status			= N'';
	IF @Template			 IS NULL SET @Template			= N'';
	IF @TemplateVersion		 IS NULL SET @TemplateVersion	= N'';
	IF @GA1					 IS NULL SET @GA1				= N'';
	IF @GA2					 IS NULL SET @GA2				= N'';
	IF @GA3					 IS NULL SET @GA3 				= N'';
	IF @GA4					 IS NULL SET @GA4 				= N'';
	IF @GA5					 IS NULL SET @GA5 				= N'';
	IF @GA6					 IS NULL SET @GA6 				= N'';
	IF @GA7					 IS NULL SET @GA7 				= N'';
	IF @GA8					 IS NULL SET @GA8 				= N'';
	IF @GA9					 IS NULL SET @GA9 				= N'';
	IF @GA10				 IS NULL SET @GA10 				= N'';
	IF @GA11				 IS NULL SET @GA11 				= N'';
	IF @GA12				 IS NULL SET @GA12 				= N'';
	IF @GA13				 IS NULL SET @GA13 				= N'';
	IF @GA14				 IS NULL SET @GA14 				= N'';
	IF @GA15				 IS NULL SET @GA15 				= N'';
	IF @GA16				 IS NULL SET @GA16 				= N'';
	IF @GA17				 IS NULL SET @GA17 				= N'';
	IF @GA18				 IS NULL SET @GA18 				= N'';
	IF @GA19				 IS NULL SET @GA19 				= N'';
	IF @GA20				 IS NULL SET @GA20 				= N'';
	IF @GA21				 IS NULL SET @GA21 				= N'';
	IF @GA22				 IS NULL SET @GA22 				= N'';
	IF @GA23				 IS NULL SET @GA23 				= N'';
	IF @GA24				 IS NULL SET @GA24 				= N'';
	IF @GA25				 IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Product;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status				= dbo.sx_pf_pProtectString	(@Status);
		SET @Template			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2 				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3 				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4 				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5 				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6 				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7 				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8 				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9 				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10 				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11 				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12 				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13 				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14 				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15 				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16 				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17 				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18 				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19 				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20 				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21 				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22 				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23 				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24 				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25 				= dbo.sx_pf_pProtectString	(@GA25);

		IF	   @FactoryID		= N''
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@FactoryKey			 = 0 
			OR @ProductLineKey	 = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - If ProductID * is submitted, search the next free numeric ProductID
		IF @ProductID = '*' 
		BEGIN
			SELECT @ProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)),0)+1 FROM planning.tdProducts dP WHERE dP.ProductLineKey = @ProductLineKey AND TRY_CAST(dP.ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @ProductID = '*' 
			SELECT @ProductID = 1;
		END

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product dont exists, create it from the Template			
		IF @ProductKey = 0
		BEGIN
			DECLARE @TemplateFactoryID				 NVARCHAR(255)	= N'ZT';
			DECLARE @TemplateProductLineID			 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductID				 NVARCHAR(255)	= N'';
			DECLARE @TemplateFactoryKey				 INT			= 0;
			DECLARE @TemplateProductLineKey			 INT			= 0;
			DECLARE @TemplateProductKey				 INT			= 0;
			DECLARE @TemplateFactoryNameShort		 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductLineNameShort	 NVARCHAR(255)	= N'';
			DECLARE @TemplateProductNameShort		 NVARCHAR(255)	= N'';

			-- Determine Templateproduct - it will be searched only in Factory ZT
			SELECT TOP (1) 
				@TemplateProductID		= ProductID, 
				@TemplateProductLineID	= ProductLineID 
			FROM planning.tdProducts
			WHERE 
					FactoryID = @TemplateFactoryID 
				AND Template = @Template
			ORDER BY ProductID;

			IF @TemplateProductID = N''	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template ProductID not found', 16, 10);
			END;

			-- Determine Template Keys
			SELECT 
					 @TemplateFactoryKey = FactoryKey
					,@TemplateFactoryNameShort = NameShort 
			FROM planning.tdFactories 
			WHERE FactoryID = @TemplateFactoryID;

			SELECT TOP (1)
					 @TemplateProductLineKey = ProductLineKey
					,@TemplateProductLineNameShort = NameShort 
			FROM planning.tdProductLines 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductLineID = @TemplateProductLineID
			ORDER BY ProductLineID;

			SELECT TOP (1) 
				 @TemplateProductKey = ProductKey
				,@TemplateProductNameShort = NameShort 
			FROM planning.tdProducts 
			WHERE 
					FactoryKey = @TemplateFactoryKey 
				AND ProductLineKey = @TemplateProductLineKey 
				AND ProductID = @TemplateProductID
			ORDER BY ProductID;
			
			IF	   @TemplateFactoryKey		 = 0 
				OR @TemplateProductLineKey	 = 0
				OR @TemplateProductKey		 = 0	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Template keys not found', 16, 10);
			END;

			-- Copy Template Product		
			-- dProducts

			DECLARE @ProductKeyTable TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO planning.tdProducts 
			OUTPUT INSERTED.ProductKey INTO @ProductKeyTable
			SELECT @ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID, 
				@FactoryID,
				TimeType, 
				@NameShort, 
				NameLong, 
				CommentUser,
				CommentDev, 
				ResponsiblePerson, 
				ImageName, 
				[Status], 
				Template, 
				TemplateVersion,
				GlobalAttribute1,
				GlobalAttribute2, 
				GlobalAttribute3, 
				GlobalAttribute4, 
				GlobalAttribute5, 
				GlobalAttribute6, 
				GlobalAttribute7,  
				GlobalAttribute8, 
				GlobalAttribute9, 
				GlobalAttribute10, 
				GlobalAttribute11, 
				GlobalAttribute12, 
				GlobalAttribute13, 
				GlobalAttribute14,  
				GlobalAttribute15, 
				GlobalAttribute16, 
				GlobalAttribute17, 
				GlobalAttribute18, 
				GlobalAttribute19, 
				GlobalAttribute20, 
				GlobalAttribute21,  
				GlobalAttribute22, 
				GlobalAttribute23, 
				GlobalAttribute24, 
				GlobalAttribute25 
			FROM planning.tdProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ProductKey = KeyNumber FROM @ProductKeyTable ORDER BY KeyNumber;

			--gProducts
			INSERT INTO planning.tgProducts 
			SELECT @ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID,
				@FactoryID, 
				PropertyID, 
				PropertyName,
				CommentUser, 
				CommentDev, 
				Unit, 
				ValueText, 
				ValueInt, 
				Scale, 
				IsROSystemProperty, 
				FormatID
			FROM planning.tgProducts 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dValueSeries
			INSERT INTO planning.tdValueSeries 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductLineID
				,@FactoryID 
				,ValueSeriesID 
				,ValueSeriesNo 
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,[IsNumeric] 
				,VisibilityLevel 
				,ValueSource 
				,ValueListID 
				,ValueFormatID 
				,Unit 
				,Scale 
				,Effect 
				,EffectParameter
			FROM planning.tdValueSeries 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--dTime
			INSERT INTO planning.tdTime 
			SELECT
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductLineID
				,@FactoryID 
				,TimeID 
				,FormatID
			FROM planning.tdTime 
			WHERE ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fValues
			;WITH VS AS (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM planning.tdValueSeries
				WHERE ProductKey = @ProductKey
			)
			INSERT INTO planning.tfValues 
			SELECT 
				 VS.ValueSeriesKey 
				,@ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductLineID 
				,@FactoryID 
				,fV.ValueSeriesID 
				,fV.TimeID 
				,fV.ValueFormula 
				,fV.ValueInt 
				,fV.ValueText 
				,fV.ValueComment
			FROM planning.tfValues fV 
				LEFT JOIN VS ON VS.ValueSeriesID= fV.ValueSeriesID 
			WHERE fV.ProductKey = @TemplateProductKey;

			SET @EffectedRows += @@ROWCOUNT;

			--fStatements - create copy informations
			INSERT INTO planning.tfStatements 
			SELECT 
				 @ProductKey 
				,@ProductLineKey 
				,@FactoryKey 
				,@ProductID 
				,@ProductLineID 
				,@FactoryID 
				,N'COPY'			AS ActionType
				,N'Product produced as a copy of Factory <' 
					+ @TemplateFactoryID 
					+ N'>"' 
					+ @TemplateFactoryNameShort 
					+ N'"/ ProductLine <'
					+ @TemplateProductLineID 
					+ N'>"' 
					+ @TemplateProductLineNameShort 
					+ N'"/ Product <' 
					+ @TemplateProductID 
					+ N'>"' 
					+ @TemplateProductNameShort 
					+ N'"'			AS Statement
				,@TransactUsername	AS UserName
				,N''					AS PCName
				,N''					AS ProcessorCode
				,N''					AS IPAddresses
				,GETUTCDATE()		AS Timestamp
				,0					AS IsDeleted
				,0					AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE planning.tdProducts 
		SET 
			 TimeType			= IIF(@Template				= N'<#NV>', TimeType,			 RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			 @NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			 @NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		 @CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			 @CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	 @ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			 @ImageName)
			,[Status]			= IIF(@Status				= N'<#NV>', [Status],			 @Status)
			,Template			= IIF(@Template				= N'<#NV>', Template,			 @Template)
			,TemplateVersion	= IIF(@TemplateVersion		= N'<#NV>', TemplateVersion,	 @TemplateVersion)
			,GlobalAttribute1	= IIF(@GA1					= N'<#NV>', GlobalAttribute1,	 @GA1)
			,GlobalAttribute2	= IIF(@GA2					= N'<#NV>', GlobalAttribute2,	 @GA2)	
			,GlobalAttribute3	= IIF(@GA3					= N'<#NV>', GlobalAttribute3,	 @GA3)
			,GlobalAttribute4	= IIF(@GA4					= N'<#NV>', GlobalAttribute4,	 @GA4)	
			,GlobalAttribute5	= IIF(@GA5					= N'<#NV>', GlobalAttribute5,	 @GA5)
			,GlobalAttribute6	= IIF(@GA6					= N'<#NV>', GlobalAttribute6,	 @GA6)	
			,GlobalAttribute7	= IIF(@GA7					= N'<#NV>', GlobalAttribute7,	 @GA7)
			,GlobalAttribute8	= IIF(@GA8					= N'<#NV>', GlobalAttribute8,	 @GA8)	
			,GlobalAttribute9	= IIF(@GA9					= N'<#NV>', GlobalAttribute9,	 @GA9)
			,GlobalAttribute10	= IIF(@GA10					= N'<#NV>', GlobalAttribute10,	 @GA10)
			,GlobalAttribute11	= IIF(@GA11					= N'<#NV>', GlobalAttribute11,	 @GA11)
			,GlobalAttribute12	= IIF(@GA12					= N'<#NV>', GlobalAttribute12,	 @GA12)
			,GlobalAttribute13	= IIF(@GA13					= N'<#NV>', GlobalAttribute13,	 @GA13)
			,GlobalAttribute14	= IIF(@GA14					= N'<#NV>', GlobalAttribute14,	 @GA14)
			,GlobalAttribute15	= IIF(@GA15					= N'<#NV>', GlobalAttribute15,	 @GA15)
			,GlobalAttribute16	= IIF(@GA16					= N'<#NV>', GlobalAttribute16,	 @GA16)
			,GlobalAttribute17	= IIF(@GA17					= N'<#NV>', GlobalAttribute17,	 @GA17)
			,GlobalAttribute18	= IIF(@GA18					= N'<#NV>', GlobalAttribute18,	 @GA18)
			,GlobalAttribute19	= IIF(@GA19					= N'<#NV>', GlobalAttribute19,	 @GA19)
			,GlobalAttribute20	= IIF(@GA20					= N'<#NV>', GlobalAttribute20,	 @GA20)
			,GlobalAttribute21	= IIF(@GA21					= N'<#NV>', GlobalAttribute21,	 @GA21)
			,GlobalAttribute22	= IIF(@GA22					= N'<#NV>', GlobalAttribute22,	 @GA22)
			,GlobalAttribute23	= IIF(@GA23					= N'<#NV>', GlobalAttribute23,	 @GA23)
			,GlobalAttribute24	= IIF(@GA24					= N'<#NV>', GlobalAttribute24,	 @GA24)
			,GlobalAttribute25	= IIF(@GA25					= N'<#NV>', GlobalAttribute25,	 @GA25)	
		FROM planning.tdProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Product TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Product'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for one Product
         Not existing Product will be created if it exists in ZT Factory. Existing Product updated
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeType';
    SET @value      = N'TimeType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ResponsiblePerson';
    SET @value      = N'ResponsiblePerson';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Status';
    SET @value      = N'Status';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Template';
    SET @value      = N'Template';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateVersion';
    SET @value      = N'TemplateVersion';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA1';
    SET @value      = N'GA1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA2';
    SET @value      = N'GA2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA3';
    SET @value      = N'GA3';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA4';
    SET @value      = N'GA4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA5';
    SET @value      = N'GA5';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA6';
    SET @value      = N'GA6';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA7';
    SET @value      = N'GA7';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA8';
    SET @value      = N'GA8';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA9';
    SET @value      = N'GA9';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA10';
    SET @value      = N'GA10';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA11';
    SET @value      = N'GA11';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA12';
    SET @value      = N'GA12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA13';
    SET @value      = N'GA13';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA14';
    SET @value      = N'GA14';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA15';
    SET @value      = N'GA15';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA16';
    SET @value      = N'GA16';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA17';
    SET @value      = N'GA17';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA18';
    SET @value      = N'GA18';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA19';
    SET @value      = N'GA19';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA20';
    SET @value      = N'GA20';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA21';
    SET @value      = N'GA21';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA22';
    SET @value      = N'GA22';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA23';
    SET @value      = N'GA23';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA24';
    SET @value      = N'GA24';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA25';
    SET @value      = N'GA25';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductDataTableValues;
GO

/*
	POST Operation for all values of the ProductDataTable
	The method can be switched between incremental POST and full POST

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.8

	Concept:
	- all Values are deliverd as Textarray - this requires a big split operation
	- the Text array must also deliver deleted values if it does a incrementel transmisson (deleted Values are delivered with empty ValueInt, ValueText, ValueFormula)
	- Values may be sended with '<#NV>' to keep existing values

	@ValuesInBracketsCommaSeparated format is: (ValueSeriesID,TimeID,Formula,Value,ValueText,ValueComment),(ValueSeriesIDTimeID,Formula,Value,ValueText,ValueComment)
	The whole string is single quoted and every parameter inside except the TimeID is single quoted
	if you check this in SQL Server Profiler or Table sx_pf_APILog, every String must have two inverted commas, the whole Parameter must have Single inverted comma
	this must show profiler: '[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')],[(''ValueSeriesID'',TimeID,''Formula'',''ValueInt'',''ValueText'',''ValueComment'')]'

	Informal: 
	In further releases the )],[( delimiter was planend be replaced with CHAR(29) - which is no longer likely, as we will use JSON
	But CHAR(29) is at the moment used at internal temporary delimiter

	Test call
	1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID			=> 404 Not Found
	2. TransactionUsername not determined for User									=> 403 Forbidden
	3. Username does not have write access to ProductLine							=> 401 Unauthorized
	4. Non exists Factory / ProductLine / Product keys								=> 404 Not Found
	5. If ValueSeriesID not exists in planning.tdValueSeries for this ProductKey	=> 403 Forbidden
	6. Product updated																=> 200 OK

	Test case 1 - Incremental Sending ***************************************************************
	1. One Value for an existing Value must replace them
	2. One Value for an not exiting Value must create it
	3. If all Parameter are posted empty, the value is deleted
	4. If one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
	5. An ZeroEmpty Value in old or sended values is deleted

	DECLARE @RC							INT
	DECLARE @Username					NVARCHAR (255) = 'SQL'
	DECLARE @ProductID					NVARCHAR (255) = '1'
	DECLARE @ProductLineID				NVARCHAR (255) = 'U'
	DECLARE @FactoryID					NVARCHAR (255) = 'ZT'
	DECLARE @IsIncrementalValuesFlag	INT			   = 1

	-- 3 Values with comments  - NEVER place a linebreak in the sample array !
	-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) =  '[(''K1'',20170115,''=IF(H25=4,3,2.1)'',''6'',''HaseTextValue'',''HaseComment'')],[(''K2'',20170215,''=IF(H25=4,3,2)'',''555'',''HaseTextValue'',''HaseComment2'')],[(''E1'',20170515,'''','''',''Text'',''Comment'')]'

	-- Delete the three values by sending them empty
	-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''','''','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''','''','''')]'

	-- Send an <#NV> Value for an existing numeric and an Text Value and delete one
	-- DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = '[(''K1'',20170115,'''',''<#NV>'','''',''HaseComment'')],[(''K2'',20170215,'''','''','''',''HaseComment2'')],[(''E1'',20170515,'''','''',''<#NV>'','''')]'

	EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues 
		 @Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@IsIncrementalValuesFlag
		,@ValuesInBracketsCommaSeparated;

	PRINT @RC;
 
	Test case 2 - fill all Values of an Product ************************************************
	DECLARE @RC								INT
	DECLARE @Username						NVARCHAR(255) = 'SQL'
	DECLARE @ProductID						NVARCHAR(255) = '1'
	DECLARE @ProductLineID					NVARCHAR(255) = 'U'
	DECLARE @FactoryID						NVARCHAR(255) = 'ZT'
	DECLARE @IsIncrementalValuesFlag		INT = 0
	DECLARE @ValuesInBracketsCommaSeparated nvarchar(max) = ''
	DECLARE @start_time						DATETIME

	-- Generate fictive values for all TimeIDs of Products
	SELECT @ValuesInBracketsCommaSeparated = @ValuesInBracketsCommaSeparated + '[(''' + dVS.ValueSeriesID +''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',''1000'',''Hase | Maus'',''Komment|ar'')],' 
	FROM 
		planning.tdValueSeries dVS LEFT JOIN planning.tdTime dT
			ON dVS.ProductKey = dT.ProductKey
	WHERE 
		dVS.FactoryID			= @FactoryID 
		AND dVS.ProductLineID	= @ProductLineID 
		AND dVS.ProductID		= @ProductID 

	SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated) -1)
	PRINT @ValuesInBracketsCommaSeparated

	SET @start_time = GETUTCDATE()
	EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues
		@Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@IsIncrementalValuesFlag
		,@ValuesInBracketsCommaSeparated;
	PRINT @RC;

	SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETUTCDATE()) AS CHAR(10))) AS 'TimeTaken'

	-- Testcase 3 - delete all values
	DECLARE @RC								INT
	DECLARE @Username						NVARCHAR (255)	= 'SQL'
	DECLARE @ProductID						NVARCHAR (255)	= '1'
	DECLARE @ProductLineID					NVARCHAR (255)	= 'U'
	DECLARE @FactoryID						NVARCHAR (255)	= 'ZT'
	DECLARE @IsIncrementalValuesFlag		INT				= 0
	DECLARE @ValuesInBracketsCommaSeparated NVARCHAR (255)	= ''

	EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues
		@Username
		,@ProductID
		,@ProductLineID
		,@FactoryID
		,@IsIncrementalValuesFlag
		,@ValuesInBracketsCommaSeparated;
	PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductDataTableValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductDataTableValues','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ProductDataTableValues
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@ValuesInBracketsCommaSeparated		NVARCHAR(MAX) 

/*

*/

AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @FactoryKey				INT				= 0;
	DECLARE @ProductLineKey 		INT				= 0;
	DECLARE @ProductKey 			INT				= 0;
	DECLARE @errorvalue				NVARCHAR(255);
	DECLARE @errormessage			NVARCHAR(2000);

	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString		NVARCHAR(MAX)	= N'''' 
									+ ISNULL(@Username													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductID													, N'NULL')	+ N''',''' 
									+ ISNULL(@ProductLineID												, N'NULL')	+ N''',''' 
									+ ISNULL(@FactoryID													, N'NULL')	+ N''','
									+ ISNULL(CAST(@IsIncrementalValuesFlag AS NVARCHAR(255))			, N'NULL')	+ N','''
									+ ISNULL(REPLACE(@ValuesInBracketsCommaSeparated, N'''', N'''''')	, N'NULL')	+ N'''';  -- WHY the replace - as there are double inverted comma's ?

	DECLARE @EffectedRows			INT				= 0;						
	DECLARE @ResultCode				INT				= 501;						
	DECLARE @TimestampCall			DATETIME		= GETUTCDATE();
	DECLARE @Comment				NVARCHAR(2000)	= N'';			
	-- SET Debug Flag for debugging and disable Transcation 
	DECLARE @DebugFlag				INT				= 0
	
	/*
		--Use for debugging
		SELECT * FROM ##Debug_ValuesInBracketsReceived 
		SELECT * FROM ##Debug_ValueMergeTableFilled ORDER BY ValueSeriesID,TimeID
		SELECT * FROM ##Debug_ValueMergeTableBeforeTransmissions ORDER BY ValueSeriesID,TimeID
	*/

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag		= 0;
	IF @ValuesInBracketsCommaSeparated	IS NULL SET @ValuesInBracketsCommaSeparated = N'';

	-- To delete Values in NOT Incremental Transmission, an empty @ValuesInBracketsCommaSeparated Parameter is sended
	IF @ValuesInBracketsCommaSeparated = '' AND @IsIncrementalValuesFlag = 0 
	BEGIN
		SET @ValuesInBracketsCommaSeparated = '[(''DeleteFlagID'',99999999,'''',''0'','''','''')]'  --special ID with prevents from rejection from errorhandler
	END

	BEGIN TRY
		BEGIN TRANSACTION POST_PDT;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

		IF	@FactoryID			= N'' 
			OR @ProductLineID	= N'' 
			OR @ProductID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters, one or more IDs are empty', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF	@FactoryKey			= 0 
			OR @ProductLineKey	= 0 
			OR @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('One or more IDs dont exist (No Keys found)', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			BEGIN
				SET @errormessage = 'Invalid rights - no write Rights for ' + @TransactUsername + '.';
				RAISERROR(@errormessage, 16, 10);
			END

		-- STEP 3.0 - Store Timeline, only TimeID, as Timeline is not changed during Posting
		IF OBJECT_ID('tempdb..#Timeline') IS NOT NULL
		DROP TABLE #Timeline;	

		CREATE TABLE #Timeline (																																							
			TimeID NVARCHAR(10) COLLATE DATABASE_DEFAULT NOT NULL UNIQUE CLUSTERED
		);
				
		INSERT INTO #Timeline
		SELECT TimeID
		FROM planning.tdTime
		WHERE ProductKey = @ProductKey;

		-- STEP 3.1 - ValueMergeTable Merge the current values of the product into the ValueMergeTable
		IF OBJECT_ID('tempdb..#ValueMergeTable') IS NOT NULL
		DROP TABLE #ValueMergeTable;

		CREATE TABLE #ValueMergeTable ( 
			 ValueSeriesKey			BIGINT										NOT NULL
			,ValueSeriesID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,TimeID					NVARCHAR(10)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueFormula			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueInt				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueText				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueComment			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,[IsNumeric]			INT											NOT NULL 
			,IsFormula				INT											NOT NULL 
			,ProcessCode			NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,DocuCode				NVARCHAR(20)	COLLATE DATABASE_DEFAULT	NOT NULL 
			,ValueSeriesID_TimeID 	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
			);

		-- First Type "Old Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0												AS ValueSeriesKey  -- No Values from dValueSeries, this Informations are later added when old and sended Values are together
			,ValueSeriesID  
			,v.TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0												AS IsNumeric
			,0												AS IsFormula
			,N'OLD VALUES'									AS ProcessCode
			,N'NONE'										AS DocuCode
			,ValueSeriesID + CAST(v.TimeID AS NVARCHAR(10))	AS ValueSeries_TimeID
		FROM planning.tfValues v INNER JOIN #Timeline tl ON v.TimeID = tl.TimeID
		WHERE ProductKey = @ProductKey;		
			
		-- Store Header, which is dValueSeries Information which will added to old and sended Values
		IF OBJECT_ID('tempdb..#Header') IS NOT NULL
		DROP TABLE #Header;																			

		CREATE TABLE #Header (
			 ValueSeriesKey		BIGINT									NOT NULL
			,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[IsNumeric]		INT										NOT NULL
			,IsFormula			INT										NOT NULL
			,UNIQUE CLUSTERED (ValueSeriesID, ValueSeriesKey, [IsNumeric], IsFormula)
			);
			
		INSERT INTO #Header
		SELECT 
			ValueSeriesKey
			,ValueSeriesID
			,[IsNumeric]
			,IIF(ValueSource IN (N'XLS',N'XLS-Strict'), 1, 0) AS IsFormula 
		FROM planning.tdValueSeries 
		WHERE ProductKey = @ProductKey;

		-- STEP 3.3 - Check for count of delimiter in Array
		DECLARE @OpenDelimetersCount	INT = 0;
		DECLARE @DelimetersIndex		INT = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated);

		WHILE @DelimetersIndex > 0 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount > 32765 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueArray to big (More then 32750 Values need special API Update.)', 16, 10);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @ValuesInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @ValuesInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.4 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @ValuesInBracketsCommaSeparated = LEFT(@ValuesInBracketsCommaSeparated,  LEN(@ValuesInBracketsCommaSeparated)-2);
		SET @ValuesInBracketsCommaSeparated = RIGHT(@ValuesInBracketsCommaSeparated, LEN(@ValuesInBracketsCommaSeparated)-2);

		DROP TABLE IF EXISTS #Values1;
		CREATE TABLE #Values1  
			(
				txtValuesPipe		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);
		DROP TABLE IF EXISTS #Values2;
		CREATE TABLE #Values2  
			(
				 ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,txtValuesPipe		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);
		DROP TABLE IF EXISTS #Values3;
		CREATE TABLE #Values3  
			(
				  ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				 ,TimeID			NVARCHAR(10)	COLLATE DATABASE_DEFAULT NOT NULL
				 ,txtValuesPipe		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);
		DROP TABLE IF EXISTS #Values4;
		CREATE TABLE #Values4  
			(
				 ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID				NVARCHAR(10)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
				,txtValuesPipe		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);
		DROP TABLE IF EXISTS #Values5;
		CREATE TABLE #Values5  
			(
				 ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID				NVARCHAR(10)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueInt			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,txtValuesPipe		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);
		DROP TABLE IF EXISTS #Values6;
		CREATE TABLE #Values6  
			(
				 ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID				NVARCHAR(10)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueFormula		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueInt			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueComment		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
			);

		DECLARE @spr NCHAR (1) = CHAR(29)

		-- Remove existing CHAR(29) = not visible character from string
		SET @ValuesInBracketsCommaSeparated = REPLACE(@ValuesInBracketsCommaSeparated,@spr,N'');
		--PRINT @ValuesInBracketsCommaSeparated

		-- Split String into Rows - all Values except the TimeID are single quoted after that an separated by @spr																													
		INSERT INTO #Values1 
		SELECT REPLACE(REPLACE(txtValues,N',''', @spr + ''''), N''',', '''' + @spr) 
		FROM dbo.sx_pf_PivotStringIntoTable(@ValuesInBracketsCommaSeparated, N')],[(');

		-- Split string into two Columns (separate first column ValueSeriesID)
		INSERT INTO #Values2
		SELECT  
			LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values1;

		-- Split last columns once more (separate TimeID)
		INSERT INTO #Values3
		SELECT  
			ValueSeriesID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values2;

		-- Delete Implicit sended periods (dont exit in current Product)
		DELETE FROM #Values3 
		WHERE TimeID NOT IN (SELECT CAST(TimeID AS NVARCHAR(10)) FROM #Timeline);

		-- Split last columns once more (separate ValueFormula)
		INSERT INTO #Values4
		SELECT  
			ValueSeriesID
			,TimeID
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values3;

		-- Split last columns once more (separate ValueInt)
		INSERT INTO #Values5
		SELECT
			ValueSeriesID  
			,TimeID
			,ValueFormula
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1)
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values4;

		-- Split last columns once more (Separate ValueText and ValueComment)
		INSERT INTO #Values6
		SELECT 
			ValueSeriesID 
			,TimeID
			,ValueFormula
			,ValueInt
			,LEFT(txtValuesPipe, CHARINDEX(@spr, txtValuesPipe) - 1) 
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(@spr, txtValuesPipe))
		FROM #Values5;

		-- Replace Quotes from all strings (all except TimeID)
		UPDATE #Values6
		SET	 
			 ValueSeriesID	= RIGHT(LEFT(ValueSeriesID,	(LEN(ValueSeriesID)-1)),	LEN(ValueSeriesID)-2)
			,ValueFormula	= RIGHT(LEFT(ValueFormula,	(LEN(ValueFormula)-1)),		LEN(ValueFormula)-2)
			,ValueInt		= RIGHT(LEFT(ValueInt,		(LEN(ValueInt)-1)),			LEN(ValueInt)-2)
			,ValueText		= RIGHT(LEFT(ValueText,		(LEN(ValueText)-1)),		LEN(ValueText)-2)
			,ValueComment	= RIGHT(LEFT(ValueComment,	(LEN(ValueComment)-1)),		LEN(ValueComment)-2);

		-- Replace Linebreaks in ValueText, but keep them in ValueComment
		UPDATE #Values6
			SET 
				ValueText =REPLACE(REPLACE(ValueText,CHAR(13),''),CHAR(10),'');

		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_Values6') IS NOT NULL DROP TABLE ##Debug_Values6;
				SELECT * INTO ##Debug_Values6 FROM #Values6;
		END;


		-- STEP 3.5 - UNION both temporary tables #######################################################################################################################

		-- Second Typ "Sended Values" is loaded in ValueMergeTable
		INSERT INTO #ValueMergeTable
		SELECT 
			0 AS ValueSeriesKey
			,ValueSeriesID
			,TimeID
			,ValueFormula
			,ValueInt
			,ValueText
			,ValueComment
			,0 AS [IsNumeric]
			,0 AS IsFormula
			,N'SENDED VALUES'
			,N'NONE'
			,ValueSeriesID+TimeID
		FROM #Values6;

		-- Add the header informations to the values
		UPDATE VMT 
		SET	
			 VMT.ValueSeriesKey = H.ValueSeriesKey
			,VMT.[IsNumeric]	= H.[IsNumeric]
			,VMT.IsFormula		= H.IsFormula 
		FROM #ValueMergeTable VMT INNER JOIN #Header H 
			ON VMT.ValueSeriesID = H.ValueSeriesID;

		-- check for not existing ValueSeriesIDs which may have been sended
		IF EXISTS(SELECT TOP (1) ValueSeriesID FROM #ValueMergeTable WHERE ValueSeriesKey = 0)
		BEGIN
			SELECT TOP(1) @errorvalue = ValueSeriesID 
			FROM #ValueMergeTable 
			WHERE ValueSeriesKey = 0 
			ORDER BY TimeID;

			IF @errorvalue = 'DeleteFlagID'
			BEGIN
				DELETE FROM #ValueMergeTable
			END
			ELSE
			BEGIN
				SET @errormessage = 'One or more sended ValueSeriesIDs were not found, e.g. '+ @errorvalue +'.'
				SET @ResultCode = 403;
				RAISERROR(@errormessage, 16, 10);
			END
		END;

		-- check if more than one value is sended for a ValueSeries/TimeID combination
		IF EXISTS(
			SELECT COUNT(TimeID) 
			FROM #ValueMergeTable 
			WHERE ProcessCode = 'SENDED VALUES'
			GROUP BY ValueSeriesID_TimeID
			HAVING COUNT(TimeID) > 1
		)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Douplicate Values found in ValueArray', 16, 10);
		END;

		-- STEP 3.6 - Prepare the ValueMergeTable #######################################################################################################################	
		-- FLAG OLD Values the user want to delete in Case of Numeric Series without formula
		-- Value 0 maybe sended as '000'
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula	= 0 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND	(TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 1 
			AND IsFormula = 1  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode	 = N'SENDED VALUES' 
						AND ValueFormula = N''
						AND (TRY_CAST(ValueInt AS BIGINT) = 0 OR ValueInt = N'')
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series without formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 0  
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode = N'SENDED VALUES' 
						AND ValueText	= N''
				);

		-- FLAG OLD Values the user want to delete in Case of NON-Numeric Series with formula
		UPDATE #ValueMergeTable 
		SET ProcessCode = N'OLD VALUES TO DELETE' 
		WHERE 
			ProcessCode = N'OLD VALUES' 
			AND [IsNumeric] = 0 
			AND IsFormula = 1 
			AND ValueSeriesID_TimeID IN 
				(
					SELECT ValueSeriesID_TimeID 
					FROM #ValueMergeTable 
					WHERE 
						ProcessCode		= N'SENDED VALUES' 
						AND	ValueText		= N'' 
						AND ValueFormula	= N''
				);

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableFilled') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableFilled;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableFilled 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.7 - Handle VMT #######################################################################################################################

		-- Documentation of changes in log table

			/* 

			-- NEW is a Value which is not contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'NEW VALUE'
			WHERE 
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND ValueSeriesID + N'_' + TimeID NOT IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			-- CHANGED is a Value which is contained in existing Values
			UPDATE	#ValueMergeTable 
			SET		DocuCode = N'CHANGED VALUE'
			WHERE	
						ProcessCode = N'SENDED VALUES' 
					AND DocuCode = N'NONE' 
					AND	ValueSeriesID + N'_' + TimeID IN 
						(
							SELECT	ValueSeriesID + N'_' + TimeID
							FROM	#ValueMergeTable 
							WHERE	ProcessCode = N'OLD VALUES'
						); 
			
			INSERT INTO sx_pf_AuditLog VALUES
				SELECT * FROM #ValueMergeTable WHERE 
					ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) IN 
					(
					SELECT
						ValueSeriesID + '_' + CAST(TimeID AS NVARCHAR(10)) FROM #ValueMergeTable WHERE ProcessCode = 'SENDED VALUES' AND
						DocuCode != 'NONE'
					)
			*/

		-- Handle <#NV> Parameters - Replace them with the old Values (if exist !)

		UPDATE	VMT 
		SET		VMT.ValueInt = OV.ValueInt 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueInt, ValueSeriesID, TimeID  FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueInt LIKE N'<#NV>' 
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueText = OV.ValueText
		FROM	#ValueMergeTable VMT
			, (SELECT ValueText, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode		= N'SENDED VALUES' 
			AND VMT.ValueText = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID 
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueFormula = OV.ValueFormula 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueFormula, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueFormula = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		UPDATE	VMT 
		SET		VMT.ValueComment = OV.ValueComment 
		FROM	#ValueMergeTable VMT
			, (SELECT ValueComment, ValueSeriesID, TimeID FROM #ValueMergeTable WHERE ProcessCode IN (N'OLD VALUES', N'OLD VALUES TO DELETE')) OV
		WHERE	
			VMT.ProcessCode = N'SENDED VALUES' 
			AND VMT.ValueComment = N'<#NV>'
			AND VMT.ValueSeriesID = OV.ValueSeriesID
			AND VMT.TimeID = OV.TimeID;

		-- Handle <#NV> Parameter, where no old Values existed
		UPDATE #ValueMergeTable SET ValueInt		= N'0'	WHERE ValueInt		= N'<#NV>' OR [IsNumeric] = 0;
		UPDATE #ValueMergeTable SET ValueText		= N''	WHERE ValueText		= N'<#NV>' OR [IsNumeric] = 1;
		UPDATE #ValueMergeTable SET ValueFormula	= N''	WHERE ValueFormula	= N'<#NV>' OR IsFormula = 0;
		UPDATE #ValueMergeTable SET ValueComment	= N''	WHERE ValueComment	= N'<#NV>';

		-- Debugging Output
		IF @DebugFlag = 1 
		BEGIN
			IF OBJECT_ID('tempdb..##Debug_ValueMergeTableBeforeTransmissions') IS NOT NULL 
				DROP TABLE ##Debug_ValueMergeTableBeforeTransmissions;
			
			SELECT * 
			INTO ##Debug_ValueMergeTableBeforeTransmissions 
			FROM #ValueMergeTable;
		END;

		-- STEP 3.8 - Save data
		IF @IsIncrementalValuesFlag <> 1 
		BEGIN
			-- CASE FULL transmission #########################################
			-- Delete all Values
			DELETE FROM planning.tfValues WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;
			
			-- DELETE Useless NullEmpty Values in ALL Values

			-- einfache Nullwerte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		= 1 
				AND IsFormula	= 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';

			-- Numerische Formelzellen ohne Formel mit Wert 0
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]			= 1 
				AND IsFormula		= 1 
				AND ValueFormula	= '' 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';

			-- einfach Leertexte
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]	  = 0 
				AND IsFormula = 0 
				AND ValueText = N'';

			-- Nichtnumerische Formelzellen ohne Formel mit Leertext
			DELETE FROM #ValueMergeTable
			WHERE 
				[IsNumeric]		 = 0 
				AND IsFormula	 = 1 
				AND ValueFormula = '' 
				AND ValueText	 = N'';

			-- Insert all Values
			INSERT INTO planning.tfValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,ValueInt
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES');
				
			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE
		BEGIN
			-- CASE INCREMENTAL Transmission - it also leads to a full delete and insert !! #########################################
			-- DELETE Values, which are sended to delete in OLD VALUES
			DELETE FROM #ValueMergeTable WHERE ProcessCode = N'OLD VALUES TO DELETE'; 

			-- DELETE sended non-formula values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0;

			--empty values must delete 0 in _0 Series
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND ValueInt = ''
				AND RIGHT(ValueSeriesID,2) = '_0';
			
			-- delete deleted formulas 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES' 
				AND [IsNumeric] = 1 
				AND IsFormula = 1 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0 
				AND ValueFormula = N''
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0;

			-- Delete 
			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode	  = N'SENDED VALUES'
				AND [IsNumeric] = 0
				AND IsFormula = 0 
				AND ValueText = N''
				

			DELETE FROM #ValueMergeTable 
			WHERE 
				ProcessCode		 = N'SENDED VALUES' 
				AND [IsNumeric]	 = 0 
				AND IsFormula	 = 1 
				AND ValueText	 = N'' 
				AND ValueFormula = N'';

			-- DELETE Useless NullEmpty Values in OLD Values
			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES'
				AND [IsNumeric] = 1 
				AND IsFormula = 0 
				AND COALESCE(TRY_CAST(ValueInt AS BIGINT),0) = 0
				AND RIGHT(ValueSeriesID,2) <> '_0';						-- send Null Values for ValueSeries *_0

			DELETE FROM #ValueMergeTable
			WHERE 
				ProcessCode	  = N'OLD VALUES' 
				AND [IsNumeric] = 0 
				AND IsFormula = 0 
				AND ValueText = N'';
					 
			-- DELETE all Values
			DELETE FROM planning.tfValues
			WHERE ProductKey = @ProductKey;
			
			SET @EffectedRows += @@ROWCOUNT;

			-- Delete OLD VALUES which are new sended
			DELETE	FROM #ValueMergeTable 
			WHERE 
				ProcessCode = N'OLD VALUES'  
				AND ValueSeriesID_TimeID IN 
					(
						SELECT ValueSeriesID_TimeID
						FROM  #ValueMergeTable 
						WHERE ProcessCode = N'SENDED VALUES'
					);

			-- Insert all Values
			INSERT INTO planning.tfValues
			SELECT 
				ValueSeriesKey
				,@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,ValueSeriesID
				,TimeID
				,ValueFormula
				,CAST(ValueInt AS BIGINT)
				,ValueText
				,ValueComment
			FROM #ValueMergeTable 
			WHERE ProcessCode IN (N'SENDED VALUES', N'OLD VALUES');
			
			SET @EffectedRows += @@ROWCOUNT;

		END;

		--- materialize Product, if it is in ZT-PARAM
		IF	@FactoryID = 'ZT' AND @ProductLineID LIKE 'PARAM%'
			BEGIN
				EXEC control.spParamTables @ProductID
			END
			
		COMMIT TRANSACTION POST_PDT;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION POST_PDT;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductDataTableValues TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductDataTableValues'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for all values of the ProductDataTable
         The method can be switched between incremental POST and full POST';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsIncrementalValuesFlag';
    SET @value      = N'IsIncrementalValuesFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesInBracketsCommaSeparated';
    SET @value      = N'ValuesInBracketsCommaSeparated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductEmpty;
GO

/*
	POST Operation for one empty Product
	Not existing Product will be created, existing gets an Update
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
	If product will have no TimeIDs, as it has no values
	If product will have no ValueSeries, when created new 

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	DECLARE	 @RC				INT
			,@Username			NVARCHAR(255) = 'SQL'
			,@ProductID			NVARCHAR(255) =	'53'
			,@ProductLineID		NVARCHAR(255) =	'U'
			,@FactoryID			NVARCHAR(255) =	'ZT'
			,@TimeType			NVARCHAR(255) =	'<#NV>'--'VM'
			,@NameShort			NVARCHAR(255) =	'Testproduct'
			,@NameLong			NVARCHAR(255) =	'3 test'
			,@CommentUser		NVARCHAR(255) =	'komment'
			,@CommentDev		NVARCHAR(255) =	'DEVKOmm'
			,@ResponsiblePerson	NVARCHAR(255) =	'RR'
			,@ImageName			NVARCHAR(255) =	'ImgName test'
			,@Status			NVARCHAR(255) =	'Aktiv'
			,@Template			NVARCHAR(255) =	'Test_VM'
			,@TemplateVersion	NVARCHAR(255) =	'4.0'
			,@GA1				NVARCHAR(255) =	'Wert für GA1'--'<#NV>'
			,@GA2				NVARCHAR(255) =	'<#NV>'
			,@GA3				NVARCHAR(255) =	'<#NV>'
			,@GA4				NVARCHAR(255) =	'rer'
			,@GA5				NVARCHAR(255) =	'<#NV>'
			,@GA6				NVARCHAR(255) =	'<#NV>'
			,@GA7				NVARCHAR(255) =	'<#NV>'
			,@GA8				NVARCHAR(255) =	'<#NV>'
			,@GA9				NVARCHAR(255) =	'<#NV>'
			,@GA10				NVARCHAR(255) =	'<#NV>'
			,@GA11				NVARCHAR(255) =	'<#NV>'
			,@GA12				NVARCHAR(255) =	'<#NV>'
			,@GA13				NVARCHAR(255) =	'<#NV>'
			,@GA14				NVARCHAR(255) =	'<#NV>'
			,@GA15				NVARCHAR(255) =	'<#NV>'
			,@GA16				NVARCHAR(255) =	'<#NV>'
			,@GA17				NVARCHAR(255) =	'<#NV>'
			,@GA18				NVARCHAR(255) =	'<#NV>'
			,@GA19				NVARCHAR(255) =	'<#NV>'
			,@GA20				NVARCHAR(255) =	'<#NV>'
			,@GA21				NVARCHAR(255) =	'<#NV>'
			,@GA22				NVARCHAR(255) =	'<#NV>'
			,@GA23				NVARCHAR(255) =	'<#NV>'
			,@GA24				NVARCHAR(255) =	'<#NV>'
			,@GA25				NVARCHAR(255) =	'<#NV>'

	EXECUTE @RC = dbo.sx_pf_POST_ProductEmpty 
			@Username,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@TimeType,
			@NameShort,
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ResponsiblePerson,
			@ImageName,
			@Status,
			@Template,
			@TemplateVersion,
			@GA1,
			@GA2,
			@GA3,
			@GA4,
			@GA5,
			@GA6,
			@GA7,
			@GA8,
			@GA9,
			@GA10,
			@GA11,
			@GA12,
			@GA13,
			@GA14,
			@GA15,
			@GA16,
			@GA17,
			@GA18,
			@GA19,
			@GA20,
			@GA21,
			@GA22,
			@GA23,
			@GA24,
			@GA25
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductEmpty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductEmpty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ProductEmpty
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeType			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson  NVARCHAR(255),
	@ImageName			NVARCHAR(255),
	@Status				NVARCHAR(255),
	@Template			NVARCHAR(255),
	@TemplateVersion	NVARCHAR(255),
	@GA1				NVARCHAR(255),
	@GA2				NVARCHAR(255),
	@GA3				NVARCHAR(255),
	@GA4				NVARCHAR(255),
	@GA5  				NVARCHAR(255),
	@GA6  				NVARCHAR(255),
	@GA7  				NVARCHAR(255),
	@GA8  				NVARCHAR(255),
	@GA9  				NVARCHAR(255),
	@GA10  				NVARCHAR(255),
	@GA11  				NVARCHAR(255),
	@GA12  				NVARCHAR(255),
	@GA13  				NVARCHAR(255),
	@GA14  				NVARCHAR(255),
	@GA15  				NVARCHAR(255),
	@GA16  				NVARCHAR(255),
	@GA17  				NVARCHAR(255),
	@GA18  				NVARCHAR(255),
	@GA19  				NVARCHAR(255),
	@GA20  				NVARCHAR(255),
	@GA21  				NVARCHAR(255),
	@GA22  				NVARCHAR(255),
	@GA23  				NVARCHAR(255),
	@GA24  				NVARCHAR(255),
	@GA25 				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @ProductLineKey		 INT			= 0;
	DECLARE @ProductKey			 INT			= 0;
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N'''' 
			+ ISNULL(@Username			, N'NULL') + N''',''' 
			+ ISNULL(@ProductID			, N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID		, N'NULL') + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL') + N''',''' 
			+ ISNULL(@TimeType			, N'NULL') + N''',''' 
			+ ISNULL(@NameShort			, N'NULL') + N''',''' 
			+ ISNULL(@NameLong			, N'NULL') + N''',''' 
			+ ISNULL(@CommentUser		, N'NULL') + N''',''' 
			+ ISNULL(@CommentDev		, N'NULL') + N''',''' 
			+ ISNULL(@ResponsiblePerson	, N'NULL') + N''',''' 
			+ ISNULL(@ImageName			, N'NULL') + N''',''' 
			+ ISNULL(@Status			, N'NULL') + N''',''' 
			+ ISNULL(@Template			, N'NULL') + N''',''' 
			+ ISNULL(@TemplateVersion	, N'NULL') + N''',''' 
			+ ISNULL(@GA1				, N'NULL') + N''',''' 
			+ ISNULL(@GA2				, N'NULL') + N''',''' 
			+ ISNULL(@GA3				, N'NULL') + N''',''' 
			+ ISNULL(@GA4				, N'NULL') + N''',''' 
			+ ISNULL(@GA5				, N'NULL') + N''',''' 
			+ ISNULL(@GA6				, N'NULL') + N''',''' 
			+ ISNULL(@GA7				, N'NULL') + N''',''' 
			+ ISNULL(@GA8				, N'NULL') + N''',''' 
			+ ISNULL(@GA9				, N'NULL') + N''',''' 
			+ ISNULL(@GA10				, N'NULL') + N''',''' 
			+ ISNULL(@GA11				, N'NULL') + N''',''' 
			+ ISNULL(@GA12				, N'NULL') + N''',''' 
			+ ISNULL(@GA13				, N'NULL') + N''',''' 
			+ ISNULL(@GA14				, N'NULL') + N''',''' 
			+ ISNULL(@GA15				, N'NULL') + N''',''' 
			+ ISNULL(@GA16				, N'NULL') + N''',''' 
			+ ISNULL(@GA17				, N'NULL') + N''',''' 
			+ ISNULL(@GA18				, N'NULL') + N''',''' 
			+ ISNULL(@GA19				, N'NULL') + N''',''' 
			+ ISNULL(@GA20				, N'NULL') + N''',''' 
			+ ISNULL(@GA21				, N'NULL') + N''',''' 
			+ ISNULL(@GA22				, N'NULL') + N''',''' 
			+ ISNULL(@GA23				, N'NULL') + N''',''' 
			+ ISNULL(@GA24				, N'NULL') + N''',''' 
			+ ISNULL(@GA25				, N'NULL') + N'''';
	DECLARE @EffectedRows		 INT			= 0;				-- SET during Execution
	DECLARE @ResultCode			 INT			= 501;				-- SET during Execution
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username 			IS NULL SET @Username			= N'';
	IF @ProductID 			IS NULL SET @ProductID			= N'';
	IF @ProductLineID 		IS NULL SET @ProductLineID		= N'';
	IF @FactoryID 			IS NULL SET @FactoryID			= N'';
	IF @TimeType 			IS NULL SET @TimeType			= N'';
	IF @NameShort 			IS NULL SET @NameShort			= N'';
	IF @NameLong 			IS NULL SET @NameLong			= N'';
	IF @CommentUser			IS NULL SET @CommentUser		= N'';
	IF @CommentDev			IS NULL SET @CommentDev 		= N'';
	IF @ResponsiblePerson	IS NULL SET @ResponsiblePerson  = N'';
	IF @ImageName 			IS NULL SET @ImageName			= N'';
	IF @Status 				IS NULL SET @Status				= N'';
	IF @Template 			IS NULL SET @Template			= N'';
	IF @TemplateVersion 	IS NULL SET @TemplateVersion	= N'';
	IF @GA1 				IS NULL SET @GA1				= N'';
	IF @GA2  				IS NULL SET @GA2				= N'';
	IF @GA3  				IS NULL SET @GA3				= N'';
	IF @GA4  				IS NULL SET @GA4				= N'';
	IF @GA5  				IS NULL SET @GA5				= N'';
	IF @GA6  				IS NULL SET @GA6 				= N'';
	IF @GA7  				IS NULL SET @GA7 				= N'';
	IF @GA8  				IS NULL SET @GA8 				= N'';
	IF @GA9  				IS NULL SET @GA9 				= N'';
	IF @GA10  				IS NULL SET @GA10 				= N'';
	IF @GA11  				IS NULL SET @GA11 				= N'';
	IF @GA12  				IS NULL SET @GA12 				= N'';
	IF @GA13  				IS NULL SET @GA13 				= N'';
	IF @GA14  				IS NULL SET @GA14 				= N'';
	IF @GA15  				IS NULL SET @GA15 				= N'';
	IF @GA16  				IS NULL SET @GA16 				= N'';
	IF @GA17  				IS NULL SET @GA17 				= N'';
	IF @GA18  				IS NULL SET @GA18 				= N'';
	IF @GA19  				IS NULL SET @GA19 				= N'';
	IF @GA20  				IS NULL SET @GA20 				= N'';
	IF @GA21  				IS NULL SET @GA21 				= N'';
	IF @GA22  				IS NULL SET @GA22 				= N'';
	IF @GA23  				IS NULL SET @GA23 				= N'';
	IF @GA24  				IS NULL SET @GA24 				= N'';
	IF @GA25  				IS NULL SET @GA25 				= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductEmpty;

		-- STEP 0.2 - Protect input parameters
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID 			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @TimeType 			= dbo.sx_pf_pProtectString	(@TimeType);
		SET @NameShort 			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong 			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser 		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ResponsiblePerson	= dbo.sx_pf_pProtectString	(@ResponsiblePerson);
		SET @ImageName 			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @Status  			= dbo.sx_pf_pProtectString	(@Status);
		SET @Template  			= dbo.sx_pf_pProtectString	(@Template);
		SET @TemplateVersion	= dbo.sx_pf_pProtectString	(@TemplateVersion);
		SET @GA1  				= dbo.sx_pf_pProtectString	(@GA1);
		SET @GA2  				= dbo.sx_pf_pProtectString	(@GA2);
		SET @GA3  				= dbo.sx_pf_pProtectString	(@GA3);
		SET @GA4  				= dbo.sx_pf_pProtectString	(@GA4);
		SET @GA5  				= dbo.sx_pf_pProtectString	(@GA5);
		SET @GA6  				= dbo.sx_pf_pProtectString	(@GA6);
		SET @GA7  				= dbo.sx_pf_pProtectString	(@GA7);
		SET @GA8  				= dbo.sx_pf_pProtectString	(@GA8);
		SET @GA9  				= dbo.sx_pf_pProtectString	(@GA9);
		SET @GA10  				= dbo.sx_pf_pProtectString	(@GA10);
		SET @GA11  				= dbo.sx_pf_pProtectString	(@GA11);
		SET @GA12  				= dbo.sx_pf_pProtectString	(@GA12);
		SET @GA13  				= dbo.sx_pf_pProtectString	(@GA13);
		SET @GA14  				= dbo.sx_pf_pProtectString	(@GA14);
		SET @GA15  				= dbo.sx_pf_pProtectString	(@GA15);
		SET @GA16  				= dbo.sx_pf_pProtectString	(@GA16);
		SET @GA17  				= dbo.sx_pf_pProtectString	(@GA17);
		SET @GA18  				= dbo.sx_pf_pProtectString	(@GA18);
		SET @GA19  				= dbo.sx_pf_pProtectString	(@GA19);
		SET @GA20  				= dbo.sx_pf_pProtectString	(@GA20);
		SET @GA21  				= dbo.sx_pf_pProtectString	(@GA21);
		SET @GA22  				= dbo.sx_pf_pProtectString	(@GA22);
		SET @GA23  				= dbo.sx_pf_pProtectString	(@GA23);
		SET @GA24  				= dbo.sx_pf_pProtectString	(@GA24);
		SET @GA25  				= dbo.sx_pf_pProtectString	(@GA25);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @FactoryKey		AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey	= @ProductLineKey	AND ProductID = @ProductID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
			
		-- STEP 3.1 - If Product dont exists, create a new row in dProducts	
		IF @ProductKey = 0
		BEGIN
			--Handle wrong time types, cut them to two characters
			IF	LEN(@TimeType) > 2 
				SET @TimeType = LEFT(@TimeType,2);

			--Create dProducts
			DECLARE @ProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

			INSERT INTO planning.tdProducts  
			OUTPUT Inserted.ProductKey INTO @ProductKeyTable 
			VALUES (@ProductLineKey
					,@FactoryKey
					,@ProductID
					,@ProductLineID
					,@FactoryID
					,@TimeType
					,@NameShort
					,@NameLong
					,@CommentUser
					,@CommentDev
					,@ResponsiblePerson
					,@ImageName
					,@Status
					,@Template
					,@TemplateVersion
					,IIF(@GA1  = N'<#NV>', N'', @GA1)
					,IIF(@GA2  = N'<#NV>', N'', @GA2)
					,IIF(@GA3  = N'<#NV>', N'', @GA3)
					,IIF(@GA4  = N'<#NV>', N'', @GA4)
					,IIF(@GA5  = N'<#NV>', N'', @GA5)
					,IIF(@GA6  = N'<#NV>', N'', @GA6)
					,IIF(@GA7  = N'<#NV>', N'', @GA7)
					,IIF(@GA8  = N'<#NV>', N'', @GA8)
					,IIF(@GA9  = N'<#NV>', N'', @GA9)
					,IIF(@GA10 = N'<#NV>', N'', @GA10)
					,IIF(@GA11 = N'<#NV>', N'', @GA11)
					,IIF(@GA12 = N'<#NV>', N'', @GA12)
					,IIF(@GA13 = N'<#NV>', N'', @GA13)
					,IIF(@GA14 = N'<#NV>', N'', @GA14)
					,IIF(@GA15 = N'<#NV>', N'', @GA15)
					,IIF(@GA16 = N'<#NV>', N'', @GA16)
					,IIF(@GA17 = N'<#NV>', N'', @GA17)
					,IIF(@GA18 = N'<#NV>', N'', @GA18)
					,IIF(@GA19 = N'<#NV>', N'', @GA19)
					,IIF(@GA20 = N'<#NV>', N'', @GA20)
					,IIF(@GA21 = N'<#NV>', N'', @GA21)
					,IIF(@GA22 = N'<#NV>', N'', @GA22)
					,IIF(@GA23 = N'<#NV>', N'', @GA23)
					,IIF(@GA24 = N'<#NV>', N'', @GA24)
					,IIF(@GA25 = N'<#NV>', N'', @GA25));

			SET @EffectedRows += @@ROWCOUNT;

			SELECT @ProductKey = KeyNumber 
			FROM @ProductKeyTable;

			--fStatements - create copy  informations
			INSERT INTO planning.tfStatements
			SELECT 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID,  
				@ProductLineID, 
				@FactoryID, 
				N'COPY' AS ActionType,
				N'Product imported by Single Product Import ' AS Statement,
				@TransactUsername AS UserName,
				N'' AS PCName, 
				N'' AS ProcessorCode, 
				N'' AS IPAddresses,
				GETUTCDATE() Timestamp,
				0 AS IsDeleted, 
				0 AS IsResolved;

			SET @EffectedRows += @@ROWCOUNT;	
			SET @ResultCode = 201;
		END;
			
		-- STEP 3.2 - Update the Product with the commited attributes
		UPDATE planning.tdProducts 
		SET  
			 TimeType			 =  IIF(@Template			 = N'<#NV>', TimeType,			RIGHT(@Template, 2))					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			,NameShort			 =  IIF(@NameShort			 = N'<#NV>', NameShort,			@NameShort)
			,NameLong			 =  IIF(@NameLong			 = N'<#NV>', NameLong,			@NameLong)
			,CommentUser		 =  IIF(@CommentUser		 = N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			 =  IIF(@CommentDev			 = N'<#NV>', CommentDev,		@CommentDev)
			,ResponsiblePerson	 =  IIF(@ResponsiblePerson	 = N'<#NV>', ResponsiblePerson, @ResponsiblePerson)
			,ImageName			 =  IIF(@ImageName			 = N'<#NV>', ImageName,			@ImageName)
			,Status				 =  IIF(@Status				 = N'<#NV>', Status,			@Status)
			,Template			 =  IIF(@Template			 = N'<#NV>', Template,			@Template)
			,TemplateVersion	 =  IIF(@TemplateVersion	 = N'<#NV>', TemplateVersion,	@TemplateVersion)
			,GlobalAttribute1	 =  IIF(@GA1				 = N'<#NV>', GlobalAttribute1,	@GA1)
			,GlobalAttribute2	 =  IIF(@GA2 				 = N'<#NV>', GlobalAttribute2,	@GA2)	
			,GlobalAttribute3	 =  IIF(@GA3 				 = N'<#NV>', GlobalAttribute3,	@GA3)
			,GlobalAttribute4	 =  IIF(@GA4 				 = N'<#NV>', GlobalAttribute4,	@GA4)	
			,GlobalAttribute5	 =  IIF(@GA5 				 = N'<#NV>', GlobalAttribute5,	@GA5)
			,GlobalAttribute6	 =  IIF(@GA6 				 = N'<#NV>', GlobalAttribute6,	@GA6)	
			,GlobalAttribute7	 =  IIF(@GA7 				 = N'<#NV>', GlobalAttribute7,	@GA7)
			,GlobalAttribute8	 =  IIF(@GA8 				 = N'<#NV>', GlobalAttribute8,	@GA8)	
			,GlobalAttribute9	 =  IIF(@GA9 				 = N'<#NV>', GlobalAttribute9,	@GA9)
			,GlobalAttribute10	 =  IIF(@GA10 				 = N'<#NV>', GlobalAttribute10, @GA10)
			,GlobalAttribute11	 =  IIF(@GA11 				 = N'<#NV>', GlobalAttribute11, @GA11)
			,GlobalAttribute12	 =  IIF(@GA12 				 = N'<#NV>', GlobalAttribute12, @GA12)
			,GlobalAttribute13	 =  IIF(@GA13 				 = N'<#NV>', GlobalAttribute13, @GA13)
			,GlobalAttribute14	 =  IIF(@GA14 				 = N'<#NV>', GlobalAttribute14, @GA14)
			,GlobalAttribute15	 =  IIF(@GA15 				 = N'<#NV>', GlobalAttribute15, @GA15)
			,GlobalAttribute16	 =  IIF(@GA16 				 = N'<#NV>', GlobalAttribute16, @GA16)
			,GlobalAttribute17	 =  IIF(@GA17 				 = N'<#NV>', GlobalAttribute17, @GA17)
			,GlobalAttribute18	 =  IIF(@GA18 				 = N'<#NV>', GlobalAttribute18, @GA18)
			,GlobalAttribute19	 =  IIF(@GA19 				 = N'<#NV>', GlobalAttribute19, @GA19)
			,GlobalAttribute20	 =  IIF(@GA20 				 = N'<#NV>', GlobalAttribute20, @GA20)
			,GlobalAttribute21	 =  IIF(@GA21 				 = N'<#NV>', GlobalAttribute21, @GA21)
			,GlobalAttribute22	 =  IIF(@GA22 				 = N'<#NV>', GlobalAttribute22, @GA22)
			,GlobalAttribute23	 =  IIF(@GA23 				 = N'<#NV>', GlobalAttribute23, @GA23)
			,GlobalAttribute24	 =  IIF(@GA24 				 = N'<#NV>', GlobalAttribute24, @GA24)
			,GlobalAttribute25	 =  IIF(@GA25				 = N'<#NV>', GlobalAttribute25, @GA25)	
		FROM planning.tdProducts
		WHERE ProductKey = @ProductKey;

		SET @EffectedRows += @@ROWCOUNT;	

		COMMIT TRANSACTION sx_pf_POST_ProductEmpty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductEmpty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductEmpty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductEmpty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for one empty Product
         Not existing Product will be created, existing gets an Update
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
         If product will have no TimeIDs, as it has no values
         If product will have no ValueSeries, when created new';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeType';
    SET @value      = N'TimeType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ResponsiblePerson';
    SET @value      = N'ResponsiblePerson';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Status';
    SET @value      = N'Status';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Template';
    SET @value      = N'Template';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateVersion';
    SET @value      = N'TemplateVersion';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA1';
    SET @value      = N'GA1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA2';
    SET @value      = N'GA2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA3';
    SET @value      = N'GA3';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA4';
    SET @value      = N'GA4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA5';
    SET @value      = N'GA5';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA6';
    SET @value      = N'GA6';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA7';
    SET @value      = N'GA7';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA8';
    SET @value      = N'GA8';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA9';
    SET @value      = N'GA9';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA10';
    SET @value      = N'GA10';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA11';
    SET @value      = N'GA11';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA12';
    SET @value      = N'GA12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA13';
    SET @value      = N'GA13';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA14';
    SET @value      = N'GA14';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA15';
    SET @value      = N'GA15';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA16';
    SET @value      = N'GA16';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA17';
    SET @value      = N'GA17';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA18';
    SET @value      = N'GA18';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA19';
    SET @value      = N'GA19';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA20';
    SET @value      = N'GA20';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA21';
    SET @value      = N'GA21';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA22';
    SET @value      = N'GA22';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA23';
    SET @value      = N'GA23';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA24';
    SET @value      = N'GA24';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GA25';
    SET @value      = N'GA25';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
 
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductLine;
GO

/*
	POST Operation for ProductLine
	Not existing ProductLine will be created, existing gets an Update
	If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty FactoryID / ProductLineID  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. Username does not have write access to ProductLine => 401 Unauthorized
	3. Username does not have write access to Factory on Create => 401 Unauthorized
	5. Non exists Factory / ProductLine keys => 404 Not Found
	6. ProductLine created => 201 Created
	7. ProductLine updated => 200 OK

	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) = 'SQL'
			,@ProductLineID				NVARCHAR(255) = 'ee'
			,@FactoryID					NVARCHAR(255) = 'ZT'
			,@NameShort					NVARCHAR(255) = ''
			,@NameLong					NVARCHAR(255) = ''
			,@CommentUser				NVARCHAR(255) = ''
			,@CommentDev				NVARCHAR(255) = ''
			,@ResponsiblePerson			NVARCHAR(255) = ''
			,@ImageName					NVARCHAR(255) = ''
			,@DefaultTemplate			NVARCHAR(255) = ''
			,@GlobalAttributeSource1	NVARCHAR(255) = ''
			,@GlobalAttributeAlias1		NVARCHAR(255) = ''
			,@GlobalAttributeSource2	NVARCHAR(255) = ''
			,@GlobalAttributeAlias2		NVARCHAR(255) = ''
			,@GlobalAttributeSource3	NVARCHAR(255) = ''
			,@GlobalAttributeAlias3		NVARCHAR(255) = ''
			,@GlobalAttributeSource4	NVARCHAR(255) = ''
			,@GlobalAttributeAlias4		NVARCHAR(255) = ''
			,@GlobalAttributeSource5	NVARCHAR(255) = ''
			,@GlobalAttributeAlias5		NVARCHAR(255) = ''
			,@GlobalAttributeSource6	NVARCHAR(255) = ''
			,@GlobalAttributeAlias6		NVARCHAR(255) = ''
			,@GlobalAttributeSource7	NVARCHAR(255) = ''
			,@GlobalAttributeAlias7		NVARCHAR(255) = ''
			,@GlobalAttributeSource8	NVARCHAR(255) = ''
			,@GlobalAttributeAlias8		NVARCHAR(255) = ''
			,@GlobalAttributeSource9	NVARCHAR(255) = ''
			,@GlobalAttributeAlias9		NVARCHAR(255) = ''
			,@GlobalAttributeSource10	NVARCHAR(255) = ''
			,@GlobalAttributeAlias10	NVARCHAR(255) = ''
			,@GlobalAttributeSource11	NVARCHAR(255) = ''
			,@GlobalAttributeAlias11	NVARCHAR(255) = ''
			,@GlobalAttributeSource12	NVARCHAR(255) = ''
			,@GlobalAttributeAlias12	NVARCHAR(255) = ''
			,@GlobalAttributeSource13	NVARCHAR(255) = ''
			,@GlobalAttributeAlias13	NVARCHAR(255) = ''
			,@GlobalAttributeSource14	NVARCHAR(255) = ''
			,@GlobalAttributeAlias14	NVARCHAR(255) = ''
			,@GlobalAttributeSource15	NVARCHAR(255) = ''
			,@GlobalAttributeAlias15	NVARCHAR(255) = ''
			,@GlobalAttributeSource16	NVARCHAR(255) = ''
			,@GlobalAttributeAlias16	NVARCHAR(255) = ''
			,@GlobalAttributeSource17	NVARCHAR(255) = ''
			,@GlobalAttributeAlias17	NVARCHAR(255) = ''
			,@GlobalAttributeSource18	NVARCHAR(255) = ''
			,@GlobalAttributeAlias18	NVARCHAR(255) = ''
			,@GlobalAttributeSource19	NVARCHAR(255) = ''
			,@GlobalAttributeAlias19	NVARCHAR(255) = ''
			,@GlobalAttributeSource20	NVARCHAR(255) = ''
			,@GlobalAttributeAlias20	NVARCHAR(255) = ''
			,@GlobalAttributeSource21	NVARCHAR(255) = ''
			,@GlobalAttributeAlias21	NVARCHAR(255) = ''
			,@GlobalAttributeSource22	NVARCHAR(255) = ''
			,@GlobalAttributeAlias22	NVARCHAR(255) = ''
			,@GlobalAttributeSource23	NVARCHAR(255) = ''
			,@GlobalAttributeAlias23	NVARCHAR(255) = ''
			,@GlobalAttributeSource24	NVARCHAR(255) = ''
			,@GlobalAttributeAlias24	NVARCHAR(255) = ''
			,@GlobalAttributeSource25	NVARCHAR(255) = ''
			,@GlobalAttributeAlias25	NVARCHAR(255) = ''

	EXECUTE @RC = dbo.sx_pf_POST_ProductLine @Username, @ProductLineID, @FactoryID, @NameShort, @NameLong, @CommentUser, @CommentDev, @ResponsiblePerson, @ImageName, @DefaultTemplate, 
			@GlobalAttributeSource1, @GlobalAttributeAlias1, @GlobalAttributeSource2, @GlobalAttributeAlias2, @GlobalAttributeSource3, @GlobalAttributeAlias3, @GlobalAttributeSource4, 
			@GlobalAttributeAlias4, @GlobalAttributeSource5, @GlobalAttributeAlias5, @GlobalAttributeSource6, @GlobalAttributeAlias6, @GlobalAttributeSource7, @GlobalAttributeAlias7, 
			@GlobalAttributeSource8, @GlobalAttributeAlias8, @GlobalAttributeSource9, @GlobalAttributeAlias9, @GlobalAttributeSource10, @GlobalAttributeAlias10, @GlobalAttributeSource11, 
			@GlobalAttributeAlias11, @GlobalAttributeSource12, @GlobalAttributeAlias12, @GlobalAttributeSource13, @GlobalAttributeAlias13, @GlobalAttributeSource14, @GlobalAttributeAlias14, 
			@GlobalAttributeSource15, @GlobalAttributeAlias15, @GlobalAttributeSource16, @GlobalAttributeAlias16, @GlobalAttributeSource17, @GlobalAttributeAlias17, @GlobalAttributeSource18, 
			@GlobalAttributeAlias18, @GlobalAttributeSource19, @GlobalAttributeAlias19, @GlobalAttributeSource20, @GlobalAttributeAlias20, @GlobalAttributeSource21, @GlobalAttributeAlias21, 
			@GlobalAttributeSource22, @GlobalAttributeAlias22, @GlobalAttributeSource23, @GlobalAttributeAlias23, @GlobalAttributeSource24, @GlobalAttributeAlias24, @GlobalAttributeSource25, 
			@GlobalAttributeAlias25
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ProductLine
	@Username					NVARCHAR(255),
	@ProductLineID				NVARCHAR(255),
	@FactoryID					NVARCHAR(255),
	@NameShort					NVARCHAR(255),
	@NameLong					NVARCHAR(255),
	@CommentUser				NVARCHAR(MAX),
	@CommentDev					NVARCHAR(MAX),
	@ResponsiblePerson			NVARCHAR(255),
	@ImageName					NVARCHAR(255),
	@DefaultTemplate			NVARCHAR(255),
	@GlobalAttributeSource1		NVARCHAR(255),
	@GlobalAttributeAlias1		NVARCHAR(255),
	@GlobalAttributeSource2		NVARCHAR(255),
	@GlobalAttributeAlias2		NVARCHAR(255),
	@GlobalAttributeSource3		NVARCHAR(255),
	@GlobalAttributeAlias3		NVARCHAR(255),
	@GlobalAttributeSource4		NVARCHAR(255),
	@GlobalAttributeAlias4		NVARCHAR(255),
	@GlobalAttributeSource5		NVARCHAR(255),
	@GlobalAttributeAlias5		NVARCHAR(255),
	@GlobalAttributeSource6		NVARCHAR(255),
	@GlobalAttributeAlias6		NVARCHAR(255),
	@GlobalAttributeSource7		NVARCHAR(255),
	@GlobalAttributeAlias7		NVARCHAR(255),
	@GlobalAttributeSource8		NVARCHAR(255),
	@GlobalAttributeAlias8		NVARCHAR(255),
	@GlobalAttributeSource9		NVARCHAR(255),
	@GlobalAttributeAlias9		NVARCHAR(255),
	@GlobalAttributeSource10	NVARCHAR(255),
	@GlobalAttributeAlias10		NVARCHAR(255),
	@GlobalAttributeSource11	NVARCHAR(255),
	@GlobalAttributeAlias11		NVARCHAR(255),
	@GlobalAttributeSource12	NVARCHAR(255),
	@GlobalAttributeAlias12		NVARCHAR(255),
	@GlobalAttributeSource13	NVARCHAR(255),
	@GlobalAttributeAlias13		NVARCHAR(255),
	@GlobalAttributeSource14	NVARCHAR(255),
	@GlobalAttributeAlias14		NVARCHAR(255),
	@GlobalAttributeSource15	NVARCHAR(255),
	@GlobalAttributeAlias15		NVARCHAR(255),
	@GlobalAttributeSource16	NVARCHAR(255),
	@GlobalAttributeAlias16		NVARCHAR(255),
	@GlobalAttributeSource17	NVARCHAR(255),
	@GlobalAttributeAlias17		NVARCHAR(255),
	@GlobalAttributeSource18	NVARCHAR(255),
	@GlobalAttributeAlias18		NVARCHAR(255),
	@GlobalAttributeSource19	NVARCHAR(255),
	@GlobalAttributeAlias19		NVARCHAR(255),
	@GlobalAttributeSource20	NVARCHAR(255),
	@GlobalAttributeAlias20		NVARCHAR(255),
	@GlobalAttributeSource21	NVARCHAR(255),
	@GlobalAttributeAlias21		NVARCHAR(255),
	@GlobalAttributeSource22	NVARCHAR(255),
	@GlobalAttributeAlias22		NVARCHAR(255),
	@GlobalAttributeSource23	NVARCHAR(255),
	@GlobalAttributeAlias23		NVARCHAR(255),
	@GlobalAttributeSource24	NVARCHAR(255),
	@GlobalAttributeAlias24		NVARCHAR(255),
	@GlobalAttributeSource25	NVARCHAR(255),
	@GlobalAttributeAlias25		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)= N'';
	DECLARE @FactoryKey			INT = 0;
	DECLARE @ProductLineKey		INT = 0;
	DECLARE @ProcedureName		NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX) = 
			N''''	   + ISNULL(@Username,					N'NULL')
			+ N''',''' + ISNULL(@ProductLineID,				N'NULL') + N''',''' + ISNULL(@FactoryID,				N'NULL') + N''',''' + ISNULL(@NameShort,				N'NULL')
			+ N''',''' + ISNULL(@NameLong,					N'NULL') + N''',''' + ISNULL(@CommentUser,				N'NULL') + N''',''' + ISNULL(@CommentDev,				N'NULL') 
			+ N''',''' + ISNULL(@ResponsiblePerson,			N'NULL') + N''',''' + ISNULL(@ImageName,				N'NULL') + N''',''' + ISNULL(@DefaultTemplate,			N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias1,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource2,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias2,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource3,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias3,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias4,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource5,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias5,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource6,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias6,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias7,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource8,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias8,		N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource9,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias9,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias10,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource11,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias11,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource12,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias12,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias13,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource14,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias14,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource15,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias15,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias16,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource17,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias17,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource18,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias18,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias19,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource20,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias20,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource21,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias21,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias22,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource23,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeAlias23,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeSource24,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias24,	N'NULL') 
			+ N''',''' + ISNULL(@GlobalAttributeSource25,	N'NULL') + N''',''' + ISNULL(@GlobalAttributeAlias25,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'';
	IF @FactoryID				IS NULL SET @FactoryID					= N'';
	IF @NameShort				IS NULL SET @NameShort					= N'';
	IF @NameLong				IS NULL SET @NameLong					= N'';
	IF @CommentUser				IS NULL SET @CommentUser				= N'';
	IF @CommentDev				IS NULL SET @CommentDev					= N'';
	IF @ResponsiblePerson		IS NULL SET @ResponsiblePerson			= N'';
	IF @ImageName				IS NULL SET @ImageName					= N'';
	IF @DefaultTemplate			IS NULL SET @DefaultTemplate			= N'';
	IF @GlobalAttributeSource1	IS NULL SET @GlobalAttributeSource1		= N'';
	IF @GlobalAttributeAlias1	IS NULL SET @GlobalAttributeAlias1		= N'';
	IF @GlobalAttributeSource2	IS NULL SET @GlobalAttributeSource2		= N'';
	IF @GlobalAttributeAlias2	IS NULL SET @GlobalAttributeAlias2		= N'';
	IF @GlobalAttributeSource3	IS NULL SET @GlobalAttributeSource3		= N'';
	IF @GlobalAttributeAlias3	IS NULL SET @GlobalAttributeAlias3		= N'';
	IF @GlobalAttributeSource4	IS NULL SET @GlobalAttributeSource4		= N'';
	IF @GlobalAttributeAlias4	IS NULL SET @GlobalAttributeAlias4		= N'';
	IF @GlobalAttributeSource5	IS NULL SET @GlobalAttributeSource5		= N'';
	IF @GlobalAttributeAlias5	IS NULL SET @GlobalAttributeAlias5		= N'';
	IF @GlobalAttributeSource6	IS NULL SET @GlobalAttributeSource6		= N'';
	IF @GlobalAttributeAlias6	IS NULL SET @GlobalAttributeAlias6		= N'';
	IF @GlobalAttributeSource7	IS NULL SET @GlobalAttributeSource7		= N'';
	IF @GlobalAttributeAlias7	IS NULL SET @GlobalAttributeAlias7		= N'';
	IF @GlobalAttributeSource8	IS NULL SET @GlobalAttributeSource8		= N'';
	IF @GlobalAttributeAlias8	IS NULL SET @GlobalAttributeAlias8		= N'';
	IF @GlobalAttributeSource9	IS NULL SET @GlobalAttributeSource9		= N'';
	IF @GlobalAttributeAlias9	IS NULL SET @GlobalAttributeAlias9		= N'';
	IF @GlobalAttributeSource10 IS NULL SET @GlobalAttributeSource10	= N'';
	IF @GlobalAttributeAlias10	IS NULL SET @GlobalAttributeAlias10		= N'';
	IF @GlobalAttributeSource11 IS NULL SET @GlobalAttributeSource11	= N'';
	IF @GlobalAttributeAlias11	IS NULL SET @GlobalAttributeAlias11		= N'';
	IF @GlobalAttributeSource12 IS NULL SET @GlobalAttributeSource12	= N'';
	IF @GlobalAttributeAlias12	IS NULL SET @GlobalAttributeAlias12		= N'';
	IF @GlobalAttributeSource13 IS NULL SET @GlobalAttributeSource13	= N'';
	IF @GlobalAttributeAlias13	IS NULL SET @GlobalAttributeAlias13		= N'';
	IF @GlobalAttributeSource14 IS NULL SET @GlobalAttributeSource14	= N'';
	IF @GlobalAttributeAlias14	IS NULL SET @GlobalAttributeAlias14		= N'';
	IF @GlobalAttributeSource15 IS NULL SET @GlobalAttributeSource15	= N'';
	IF @GlobalAttributeAlias15	IS NULL SET @GlobalAttributeAlias15		= N'';
	IF @GlobalAttributeSource16 IS NULL SET @GlobalAttributeSource16	= N'';
	IF @GlobalAttributeAlias16	IS NULL SET @GlobalAttributeAlias16		= N'';
	IF @GlobalAttributeSource17 IS NULL SET @GlobalAttributeSource17	= N'';
	IF @GlobalAttributeAlias17	IS NULL SET @GlobalAttributeAlias17		= N'';
	IF @GlobalAttributeSource18 IS NULL SET @GlobalAttributeSource18	= N'';
	IF @GlobalAttributeAlias18	IS NULL SET @GlobalAttributeAlias18		= N'';
	IF @GlobalAttributeSource19 IS NULL SET @GlobalAttributeSource19	= N'';
	IF @GlobalAttributeAlias19	IS NULL SET @GlobalAttributeAlias19		= N'';
	IF @GlobalAttributeSource20 IS NULL SET @GlobalAttributeSource20	= N'';
	IF @GlobalAttributeAlias20	IS NULL SET @GlobalAttributeAlias20		= N'';
	IF @GlobalAttributeSource21 IS NULL SET @GlobalAttributeSource21	= N'';
	IF @GlobalAttributeAlias21	IS NULL SET @GlobalAttributeAlias21		= N'';
	IF @GlobalAttributeSource22 IS NULL SET @GlobalAttributeSource22	= N'';
	IF @GlobalAttributeAlias22	IS NULL SET @GlobalAttributeAlias22		= N'';
	IF @GlobalAttributeSource23 IS NULL SET @GlobalAttributeSource23	= N'';
	IF @GlobalAttributeAlias23	IS NULL SET @GlobalAttributeAlias23		= N'';
	IF @GlobalAttributeSource24 IS NULL SET @GlobalAttributeSource24	= N'';
	IF @GlobalAttributeAlias24	IS NULL SET @GlobalAttributeAlias24		= N'';
	IF @GlobalAttributeSource25 IS NULL SET @GlobalAttributeSource25	= N'';
	IF @GlobalAttributeAlias25	IS NULL SET @GlobalAttributeAlias25		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLine;
		
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID			= dbo.sx_pf_pProtectID     (@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID	   (@FactoryID);
		SET @NameShort				= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong				= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser			= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev				= dbo.sx_pf_pProtectString (@CommentDev);
		SET @ResponsiblePerson		= dbo.sx_pf_pProtectString (@ResponsiblePerson);
		SET @ImageName				= dbo.sx_pf_pProtectString (@ImageName);
		SET @DefaultTemplate		= dbo.sx_pf_pProtectString (@DefaultTemplate);
		SET @GlobalAttributeAlias1	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias1);
		SET @GlobalAttributeAlias2	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias2);
		SET @GlobalAttributeAlias3	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias3);
		SET @GlobalAttributeAlias4	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias4);
		SET @GlobalAttributeAlias5	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias5);
		SET @GlobalAttributeAlias6	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias6);
		SET @GlobalAttributeAlias7	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias7);
		SET @GlobalAttributeAlias8	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias8);
		SET @GlobalAttributeAlias9	= dbo.sx_pf_pProtectString (@GlobalAttributeAlias9);
		SET @GlobalAttributeAlias10 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias10);
		SET @GlobalAttributeAlias11 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias11);
		SET @GlobalAttributeAlias12 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias12);
		SET @GlobalAttributeAlias13 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias13);
		SET @GlobalAttributeAlias14 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias14);
		SET @GlobalAttributeAlias15 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias15);
		SET @GlobalAttributeAlias16 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias16);
		SET @GlobalAttributeAlias17 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias17);
		SET @GlobalAttributeAlias18 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias18);
		SET @GlobalAttributeAlias19 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias19);
		SET @GlobalAttributeAlias20 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias20);
		SET @GlobalAttributeAlias21 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias21);
		SET @GlobalAttributeAlias22 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias22);
		SET @GlobalAttributeAlias23 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias23);
		SET @GlobalAttributeAlias24 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias24);
		SET @GlobalAttributeAlias25 = dbo.sx_pf_pProtectString (@GlobalAttributeAlias25);

		IF @GlobalAttributeSource1	!='<#NV>' SET @GlobalAttributeSource1  = dbo.sx_pf_pProtectID (@GlobalAttributeSource1)
		IF @GlobalAttributeSource2	!='<#NV>' SET @GlobalAttributeSource2  = dbo.sx_pf_pProtectID (@GlobalAttributeSource2)
		IF @GlobalAttributeSource3	!='<#NV>' SET @GlobalAttributeSource3  = dbo.sx_pf_pProtectID (@GlobalAttributeSource3)
		IF @GlobalAttributeSource4	!='<#NV>' SET @GlobalAttributeSource4  = dbo.sx_pf_pProtectID (@GlobalAttributeSource4)
		IF @GlobalAttributeSource5	!='<#NV>' SET @GlobalAttributeSource5  = dbo.sx_pf_pProtectID (@GlobalAttributeSource5)
		IF @GlobalAttributeSource6	!='<#NV>' SET @GlobalAttributeSource6  = dbo.sx_pf_pProtectID (@GlobalAttributeSource6)
		IF @GlobalAttributeSource7	!='<#NV>' SET @GlobalAttributeSource7  = dbo.sx_pf_pProtectID (@GlobalAttributeSource7)
		IF @GlobalAttributeSource8	!='<#NV>' SET @GlobalAttributeSource8  = dbo.sx_pf_pProtectID (@GlobalAttributeSource8)
		IF @GlobalAttributeSource9	!='<#NV>' SET @GlobalAttributeSource9  = dbo.sx_pf_pProtectID (@GlobalAttributeSource9)
		IF @GlobalAttributeSource10 !='<#NV>' SET @GlobalAttributeSource10 = dbo.sx_pf_pProtectID (@GlobalAttributeSource10)
		IF @GlobalAttributeSource11 !='<#NV>' SET @GlobalAttributeSource11 = dbo.sx_pf_pProtectID (@GlobalAttributeSource11)
		IF @GlobalAttributeSource12 !='<#NV>' SET @GlobalAttributeSource12 = dbo.sx_pf_pProtectID (@GlobalAttributeSource12)
		IF @GlobalAttributeSource13 !='<#NV>' SET @GlobalAttributeSource13 = dbo.sx_pf_pProtectID (@GlobalAttributeSource13)
		IF @GlobalAttributeSource14 !='<#NV>' SET @GlobalAttributeSource14 = dbo.sx_pf_pProtectID (@GlobalAttributeSource14)
		IF @GlobalAttributeSource15 !='<#NV>' SET @GlobalAttributeSource15 = dbo.sx_pf_pProtectID (@GlobalAttributeSource15)
		IF @GlobalAttributeSource16 !='<#NV>' SET @GlobalAttributeSource16 = dbo.sx_pf_pProtectID (@GlobalAttributeSource16)
		IF @GlobalAttributeSource17 !='<#NV>' SET @GlobalAttributeSource17 = dbo.sx_pf_pProtectID (@GlobalAttributeSource17)
		IF @GlobalAttributeSource18 !='<#NV>' SET @GlobalAttributeSource18 = dbo.sx_pf_pProtectID (@GlobalAttributeSource18)
		IF @GlobalAttributeSource19 !='<#NV>' SET @GlobalAttributeSource19 = dbo.sx_pf_pProtectID (@GlobalAttributeSource19)
		IF @GlobalAttributeSource20 !='<#NV>' SET @GlobalAttributeSource20 = dbo.sx_pf_pProtectID (@GlobalAttributeSource20)
		IF @GlobalAttributeSource21 !='<#NV>' SET @GlobalAttributeSource21 = dbo.sx_pf_pProtectID (@GlobalAttributeSource21)
		IF @GlobalAttributeSource22 !='<#NV>' SET @GlobalAttributeSource22 = dbo.sx_pf_pProtectID (@GlobalAttributeSource22)
		IF @GlobalAttributeSource23 !='<#NV>' SET @GlobalAttributeSource23 = dbo.sx_pf_pProtectID (@GlobalAttributeSource23)
		IF @GlobalAttributeSource24 !='<#NV>' SET @GlobalAttributeSource24 = dbo.sx_pf_pProtectID (@GlobalAttributeSource24)
		IF @GlobalAttributeSource25 !='<#NV>' SET @GlobalAttributeSource25 = dbo.sx_pf_pProtectID (@GlobalAttributeSource25)

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		IF @ProductLineKey = 0
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		ELSE
			EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Product Line does not yet exist, it will be created in minimal configuration		
		IF @ProductLineKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tdProductLines (FactoryKey, ProductLineID, FactoryID, NameShort, NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName, DefaultTemplate
				, GlobalattributeSource1,  GlobalattributeAlias1,  GlobalattributeSource2,  GlobalattributeAlias2,  GlobalattributeSource3,  GlobalattributeAlias3,  GlobalattributeSource4,  GlobalattributeAlias4,  GlobalattributeSource5,  GlobalattributeAlias5
				, GlobalattributeSource6,  GlobalattributeAlias6,  GlobalattributeSource7,  GlobalattributeAlias7,  GlobalattributeSource8,  GlobalattributeAlias8,  GlobalattributeSource9,  GlobalattributeAlias9,  GlobalattributeSource10, GlobalattributeAlias10
				, GlobalattributeSource11, GlobalattributeAlias11, GlobalattributeSource12, GlobalattributeAlias12, GlobalattributeSource13, GlobalattributeAlias13, GlobalattributeSource14, GlobalattributeAlias14, GlobalattributeSource15, GlobalattributeAlias15
				, GlobalattributeSource16, GlobalattributeAlias16, GlobalattributeSource17, GlobalattributeAlias17, GlobalattributeSource18, GlobalattributeAlias18, GlobalattributeSource19, GlobalattributeAlias19, GlobalattributeSource20, GlobalattributeAlias20
				, GlobalattributeSource21, GlobalattributeAlias21, GlobalattributeSource22, GlobalattributeAlias22, GlobalattributeSource23, GlobalattributeAlias23, GlobalattributeSource24, GlobalattributeAlias24, GlobalattributeSource25, GlobalattributeAlias25
			) 
			OUTPUT INSERTED.ProductLineKey INTO @Keys([Key])  
			VALUES (@FactoryKey, @ProductLineID, @FactoryID, @NameShort, '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
				, '', '', '', '', '', '', '', '', '', ''
			);

			SELECT TOP (1) @ProductLineKey = [Key] FROM @Keys ORDER BY [Key];
			
			SET @EffectedRows = 1;
			SET @Resultcode	  = 201;
		END;

		-- STEP 3.2 - Update values
		UPDATE planning.tdProductLines
		SET 
			NameShort					= IIF(@NameShort				= N'<#NV>', NameShort,				 @NameShort)
			, NameLong					= IIF(@NameLong					= N'<#NV>', NameLong,				 @NameLong)
			, CommentUser				= IIF(@CommentUser				= N'<#NV>', CommentUser,			 @CommentUser)
			, CommentDev				= IIF(@CommentDev				= N'<#NV>', CommentDev,				 @CommentDev)
			, ResponsiblePerson			= IIF(@ResponsiblePerson		= N'<#NV>', ResponsiblePerson,		 @ResponsiblePerson)
			, ImageName					= IIF(@ImageName				= N'<#NV>', ImageName,				 @ImageName)
			, DefaultTemplate			= IIF(@DefaultTemplate			= N'<#NV>', DefaultTemplate,		 @DefaultTemplate)
			, GlobalAttributeSource1	= IIF(@GlobalAttributeSource1	= N'<#NV>', GlobalAttributeSource1,	 @GlobalAttributeSource1)
			, GlobalAttributeAlias1		= IIF(@GlobalAttributeAlias1	= N'<#NV>', GlobalAttributeAlias1,	 @GlobalAttributeAlias1)			
			, GlobalAttributeSource2	= IIF(@GlobalAttributeSource2	= N'<#NV>', GlobalAttributeSource2,	 @GlobalAttributeSource2)
			, GlobalAttributeAlias2		= IIF(@GlobalAttributeAlias2	= N'<#NV>', GlobalAttributeAlias2,	 @GlobalAttributeAlias2)
			, GlobalAttributeSource3	= IIF(@GlobalAttributeSource3	= N'<#NV>', GlobalAttributeSource3,	 @GlobalAttributeSource3)
			, GlobalAttributeAlias3		= IIF(@GlobalAttributeAlias3	= N'<#NV>', GlobalAttributeAlias3,	 @GlobalAttributeAlias3)
			, GlobalAttributeSource4	= IIF(@GlobalAttributeSource4	= N'<#NV>', GlobalAttributeSource4,	 @GlobalAttributeSource4)
			, GlobalAttributeAlias4		= IIF(@GlobalAttributeAlias4	= N'<#NV>', GlobalAttributeAlias4,	 @GlobalAttributeAlias4)
			, GlobalAttributeSource5	= IIF(@GlobalAttributeSource5	= N'<#NV>', GlobalAttributeSource5,	 @GlobalAttributeSource5)
			, GlobalAttributeAlias5		= IIF(@GlobalAttributeAlias5	= N'<#NV>', GlobalAttributeAlias5,	 @GlobalAttributeAlias5)
			, GlobalAttributeSource6	= IIF(@GlobalAttributeSource6	= N'<#NV>', GlobalAttributeSource6,	 @GlobalAttributeSource6)
			, GlobalAttributeAlias6		= IIF(@GlobalAttributeAlias6	= N'<#NV>', GlobalAttributeAlias6,	 @GlobalAttributeAlias6)
			, GlobalAttributeSource7	= IIF(@GlobalAttributeSource7	= N'<#NV>', GlobalAttributeSource7,	 @GlobalAttributeSource7)
			, GlobalAttributeAlias7		= IIF(@GlobalAttributeAlias7	= N'<#NV>', GlobalAttributeAlias7,	 @GlobalAttributeAlias7)
			, GlobalAttributeSource8	= IIF(@GlobalAttributeSource8	= N'<#NV>', GlobalAttributeSource8,	 @GlobalAttributeSource8)
			, GlobalAttributeAlias8		= IIF(@GlobalAttributeAlias8	= N'<#NV>', GlobalAttributeAlias8,	 @GlobalAttributeAlias8)
			, GlobalAttributeSource9	= IIF(@GlobalAttributeSource9	= N'<#NV>', GlobalAttributeSource9,	 @GlobalAttributeSource9)
			, GlobalAttributeAlias9		= IIF(@GlobalAttributeAlias9	= N'<#NV>', GlobalAttributeAlias9,	 @GlobalAttributeAlias9)
			, GlobalAttributeSource10	= IIF(@GlobalAttributeSource10	= N'<#NV>', GlobalAttributeSource10, @GlobalAttributeSource10)
			, GlobalAttributeAlias10	= IIF(@GlobalAttributeAlias10	= N'<#NV>', GlobalAttributeAlias10,	 @GlobalAttributeAlias10)
			, GlobalAttributeSource11	= IIF(@GlobalAttributeSource11	= N'<#NV>', GlobalAttributeSource11, @GlobalAttributeSource11)
			, GlobalAttributeAlias11	= IIF(@GlobalAttributeAlias11	= N'<#NV>', GlobalAttributeAlias11,	 @GlobalAttributeAlias11)			
			, GlobalAttributeSource12	= IIF(@GlobalAttributeSource12	= N'<#NV>', GlobalAttributeSource12, @GlobalAttributeSource12)
			, GlobalAttributeAlias12	= IIF(@GlobalAttributeAlias12	= N'<#NV>', GlobalAttributeAlias12,	 @GlobalAttributeAlias12)
			, GlobalAttributeSource13	= IIF(@GlobalAttributeSource13	= N'<#NV>', GlobalAttributeSource13, @GlobalAttributeSource13)
			, GlobalAttributeAlias13	= IIF(@GlobalAttributeAlias13	= N'<#NV>', GlobalAttributeAlias13,	 @GlobalAttributeAlias13)
			, GlobalAttributeSource14	= IIF(@GlobalAttributeSource14	= N'<#NV>', GlobalAttributeSource14, @GlobalAttributeSource14)
			, GlobalAttributeAlias14	= IIF(@GlobalAttributeAlias14	= N'<#NV>', GlobalAttributeAlias14,	 @GlobalAttributeAlias14)
			, GlobalAttributeSource15	= IIF(@GlobalAttributeSource15	= N'<#NV>', GlobalAttributeSource15, @GlobalAttributeSource15)
			, GlobalAttributeAlias15	= IIF(@GlobalAttributeAlias15	= N'<#NV>', GlobalAttributeAlias15,	 @GlobalAttributeAlias15)
			, GlobalAttributeSource16	= IIF(@GlobalAttributeSource16	= N'<#NV>', GlobalAttributeSource16, @GlobalAttributeSource16)
			, GlobalAttributeAlias16	= IIF(@GlobalAttributeAlias16	= N'<#NV>', GlobalAttributeAlias16,	 @GlobalAttributeAlias16)
			, GlobalAttributeSource17	= IIF(@GlobalAttributeSource17	= N'<#NV>', GlobalAttributeSource17, @GlobalAttributeSource17)
			, GlobalAttributeAlias17	= IIF(@GlobalAttributeAlias17	= N'<#NV>', GlobalAttributeAlias17,	 @GlobalAttributeAlias17)
			, GlobalAttributeSource18	= IIF(@GlobalAttributeSource18	= N'<#NV>', GlobalAttributeSource18, @GlobalAttributeSource18)
			, GlobalAttributeAlias18	= IIF(@GlobalAttributeAlias18	= N'<#NV>', GlobalAttributeAlias18,	 @GlobalAttributeAlias18)
			, GlobalAttributeSource19	= IIF(@GlobalAttributeSource19	= N'<#NV>', GlobalAttributeSource19, @GlobalAttributeSource19)
			, GlobalAttributeAlias19	= IIF(@GlobalAttributeAlias19	= N'<#NV>', GlobalAttributeAlias19,	 @GlobalAttributeAlias19)
			, GlobalAttributeSource20	= IIF(@GlobalAttributeSource20	= N'<#NV>', GlobalAttributeSource20, @GlobalAttributeSource20)
			, GlobalAttributeAlias20	= IIF(@GlobalAttributeAlias20	= N'<#NV>', GlobalAttributeAlias20,	 @GlobalAttributeAlias20)
			, GlobalAttributeSource21	= IIF(@GlobalAttributeSource21	= N'<#NV>', GlobalAttributeSource21, @GlobalAttributeSource21)
			, GlobalAttributeAlias21	= IIF(@GlobalAttributeAlias21	= N'<#NV>', GlobalAttributeAlias21,	 @GlobalAttributeAlias21)			
			, GlobalAttributeSource22	= IIF(@GlobalAttributeSource22	= N'<#NV>', GlobalAttributeSource22, @GlobalAttributeSource22)
			, GlobalAttributeAlias22	= IIF(@GlobalAttributeAlias22	= N'<#NV>', GlobalAttributeAlias22,	 @GlobalAttributeAlias22)
			, GlobalAttributeSource23	= IIF(@GlobalAttributeSource23	= N'<#NV>', GlobalAttributeSource23, @GlobalAttributeSource23)
			, GlobalAttributeAlias23	= IIF(@GlobalAttributeAlias23	= N'<#NV>', GlobalAttributeAlias23,	 @GlobalAttributeAlias23)
			, GlobalAttributeSource24	= IIF(@GlobalAttributeSource24	= N'<#NV>', GlobalAttributeSource24, @GlobalAttributeSource24)
			, GlobalAttributeAlias24	= IIF(@GlobalAttributeAlias24	= N'<#NV>', GlobalAttributeAlias24,	 @GlobalAttributeAlias24)
			, GlobalAttributeSource25	= IIF(@GlobalAttributeSource25	= N'<#NV>', GlobalAttributeSource25, @GlobalAttributeSource25)
			, GlobalAttributeAlias25	= IIF(@GlobalAttributeAlias25	= N'<#NV>', GlobalAttributeAlias25,	 @GlobalAttributeAlias25)
		FROM planning.tdProductLines
		WHERE ProductLineKey = @ProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (only possible for users with Factory WriteRight or Cluster Admin)
		EXEC system.spMaterialize_trUserRights;

		COMMIT TRANSACTION sx_pf_POST_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLine TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductLine'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ProductLine
         Not existing ProductLine will be created, existing gets an Update
         If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ResponsiblePerson';
    SET @value      = N'ResponsiblePerson';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DefaultTemplate';
    SET @value      = N'DefaultTemplate';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource1';
    SET @value      = N'GlobalAttributeSource1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias1';
    SET @value      = N'GlobalAttributeAlias1';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource2';
    SET @value      = N'GlobalAttributeSource2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias2';
    SET @value      = N'GlobalAttributeAlias2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource3';
    SET @value      = N'GlobalAttributeSource3';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias3';
    SET @value      = N'GlobalAttributeAlias3';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource4';
    SET @value      = N'GlobalAttributeSource4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias4';
    SET @value      = N'GlobalAttributeAlias4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource5';
    SET @value      = N'GlobalAttributeSource5';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias5';
    SET @value      = N'GlobalAttributeAlias5';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource6';
    SET @value      = N'GlobalAttributeSource6';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias6';
    SET @value      = N'GlobalAttributeAlias6';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource7';
    SET @value      = N'GlobalAttributeSource7';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias7';
    SET @value      = N'GlobalAttributeAlias7';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource8';
    SET @value      = N'GlobalAttributeSource8';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias8';
    SET @value      = N'GlobalAttributeAlias8';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource9';
    SET @value      = N'GlobalAttributeSource9';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias9';
    SET @value      = N'GlobalAttributeAlias9';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource10';
    SET @value      = N'GlobalAttributeSource10';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias10';
    SET @value      = N'GlobalAttributeAlias10';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource11';
    SET @value      = N'GlobalAttributeSource11';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias11';
    SET @value      = N'GlobalAttributeAlias11';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource12';
    SET @value      = N'GlobalAttributeSource12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias12';
    SET @value      = N'GlobalAttributeAlias12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource13';
    SET @value      = N'GlobalAttributeSource13';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias13';
    SET @value      = N'GlobalAttributeAlias13';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource14';
    SET @value      = N'GlobalAttributeSource14';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias14';
    SET @value      = N'GlobalAttributeAlias14';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource15';
    SET @value      = N'GlobalAttributeSource15';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias15';
    SET @value      = N'GlobalAttributeAlias15';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource16';
    SET @value      = N'GlobalAttributeSource16';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias16';
    SET @value      = N'GlobalAttributeAlias16';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource17';
    SET @value      = N'GlobalAttributeSource17';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias17';
    SET @value      = N'GlobalAttributeAlias17';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource18';
    SET @value      = N'GlobalAttributeSource18';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias18';
    SET @value      = N'GlobalAttributeAlias18';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource19';
    SET @value      = N'GlobalAttributeSource19';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias19';
    SET @value      = N'GlobalAttributeAlias19';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource20';
    SET @value      = N'GlobalAttributeSource20';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias20';
    SET @value      = N'GlobalAttributeAlias20';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource21';
    SET @value      = N'GlobalAttributeSource21';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias21';
    SET @value      = N'GlobalAttributeAlias21';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource22';
    SET @value      = N'GlobalAttributeSource22';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias22';
    SET @value      = N'GlobalAttributeAlias22';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource23';
    SET @value      = N'GlobalAttributeSource23';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias23';
    SET @value      = N'GlobalAttributeAlias23';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource24';
    SET @value      = N'GlobalAttributeSource24';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias24';
    SET @value      = N'GlobalAttributeAlias24';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeSource25';
    SET @value      = N'GlobalAttributeSource25';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@GlobalAttributeAlias25';
    SET @value      = N'GlobalAttributeAlias25';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductLineProperty;
GO

/*
	POST Operation for ProductLineProperty
	Not existing ProductLine will be created, existing gets an Update
	If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. Username does not have write access to ProductLine => 401 Unauthorized
	5. Property created => 201 Created
	6. Property updated => 200 OK

		DECLARE  @RC				 INT
				,@Username			 NVARCHAR(255) = 'SQL'
				,@ProductLineID		 NVARCHAR(255) = 'U'
				,@FactoryID			 NVARCHAR(255) = 'ZT'
				,@PropertyID		 NVARCHAR(255) = 'rreo'
				,@PropertyName		 NVARCHAR(255) = ''
				,@CommentUser		 NVARCHAR(MAX) = ''
				,@CommentDev		 NVARCHAR(MAX) = ''
				,@Unit				 NVARCHAR(255) = ''
				,@ValueText			 NVARCHAR(MAX) = ''
				,@ValueInt			 NVARCHAR(255) = '678'
				,@Scale				 NVARCHAR(255) = '<#NV>'
				,@IsROSystemProperty NVARCHAR(255) = '1'
				,@FormatID			 NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_ProductLineProperty 
				@Username,
				@ProductLineID,
				@FactoryID,
				@PropertyID,	
				@PropertyName,
				@CommentUser,
				@CommentDev,
				@Unit,
				@ValueText,
				@ValueInt,	
				@Scale,
				@IsROSystemProperty,
				@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLineProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLineProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ProductLineProperty
	@Username			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PropertyID			NVARCHAR(255),
	@PropertyName		NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@Unit 				NVARCHAR(255),
	@ValueText 			NVARCHAR(MAX),
	@ValueInt 			NVARCHAR(255),
	@Scale				NVARCHAR(255),
	@IsROSystemProperty NVARCHAR(255),
	@FormatID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,	N'NULL') + N''',''' + ISNULL(@PropertyName,		N'NULL') + N''',''' + ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,	N'NULL') + N''',''' + ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,		N'NULL') + N''',''' + ISNULL(@Scale,			N'NULL') + N''',''' + ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLineProperty;
				
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @FormatID		= dbo.sx_pf_pProtectString	(@FormatID); --as it can be #<NV>

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE ProductLineKey =  @ProductLineKey AND PropertyID = @PropertyID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgProductLines WHERE ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgProductLines (ProductLineKey, FactoryKey, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductLineKey, @FactoryKey, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE planning.tgProductLines 
		SET 
			PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			, CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			, CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			, Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			, ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			, ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			, Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			, IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			, FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductLineProperty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ProductLineProperty
         Not existing ProductLine will be created, existing gets an Update
         If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyName';
    SET @value      = N'PropertyName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsROSystemProperty';
    SET @value      = N'IsROSystemProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductProperty;
GO

/*
	POST Operation for ProductProperty
	Not existing ProductLine will be created, existing gets an Update
	If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. Username does not have write access to ProductLine => 401 Unauthorized
	5. Property created => 201 Created
	6. Property updated => 200 OK

		DECLARE  @RC					INT
				,@Username				NVARCHAR(255) = 'SQL'
				,@ProductID				NVARCHAR(255) = '1'
				,@ProductLineID			NVARCHAR(255) = 'U'
				,@FactoryID				NVARCHAR(255) = 'ZT'
				,@PropertyID			NVARCHAR(255) = 'rreo'
				,@PropertyName			NVARCHAR(255) = ''
				,@CommentUser			NVARCHAR(MAX) = ''
				,@CommentDev			NVARCHAR(MAX) = ''
				,@Unit					NVARCHAR(255) = ''
				,@ValueText				NVARCHAR(MAX) = ''
				,@ValueInt				NVARCHAR(255) = '678'
				,@Scale					NVARCHAR(255) = '<#NV>'
				,@IsROSystemProperty	NVARCHAR(255) = '1'
				,@FormatID				NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_ProductProperty 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@PropertyID,
				@PropertyName,
				@CommentUser,
				@CommentDev,
				@Unit,
				@ValueText,
				@ValueInt,
				@Scale,
				@IsROSystemProperty,
				@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductProperty','PARAMETER',NULL)
*/
 
CREATE PROCEDURE dbo.sx_pf_POST_ProductProperty
	@Username				NVARCHAR(255),
	@ProductID				NVARCHAR(255),
	@ProductLineID			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
	@PropertyID				NVARCHAR(255),
	@PropertyName			NVARCHAR(255),
	@CommentUser			NVARCHAR(MAX),
	@CommentDev				NVARCHAR(MAX),
	@Unit					NVARCHAR(255),
	@ValueText				NVARCHAR(MAX),
	@ValueInt				NVARCHAR(255),
	@Scale					NVARCHAR(255),
	@IsROSystemProperty		NVARCHAR(255),
	@FormatID				NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@ProductID,			N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyName,			N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,			N'NULL') + N''',''' 
			+ ISNULL(@Unit,					N'NULL') + N''',''' 
			+ ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,				N'NULL') + N''',''' 
			+ ISNULL(@Scale,				N'NULL') + N''',''' 
			+ ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,				N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductProperty;
			
		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		IF @FormatID <> '<#NV>'
			SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		--SET @PropertyOwner = dbo.sx_pf_pProtectBoolean (@IsROSystemProperty); -- NO Protection at the moment, as value can be 0,1,2

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @PropertyKey		= PropertyKey		FROM planning.tgProducts		WHERE ProductKey = @ProductKey AND PropertyID = @PropertyID;
		
		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0 
			OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgProducts WHERE ProductKey =  @ProductKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration				
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgProducts (ProductKey, ProductLineKey, FactoryKey, ProductID, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID)
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductKey, @ProductLineKey, @FactoryKey, @ProductID, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');
			
			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @Resultcode = 201;
			SET @EffectedRows = 1;
		END

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE planning.tgProducts
		SET  PropertyName		= IIF(@PropertyName			= N'<#NV>', PropertyName,		@PropertyName)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,Unit				= IIF(@Unit					= N'<#NV>', Unit,				@Unit)
			,ValueText			= IIF(@ValueText			= N'<#NV>', ValueText,			@ValueText)
			,ValueInt			= IIF(@ValueInt				= N'<#NV>', ValueInt,			@ValueInt)
			,Scale				= IIF(@Scale				= N'<#NV>', Scale,				@Scale)
			,IsROSystemProperty = IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty, @IsROSystemProperty)
			,FormatID			= IIF(@FormatID				= N'<#NV>', FormatID,			@FormatID)
		FROM planning.tgProducts
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductProperty TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductProperty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ProductProperty
         Not existing ProductLine will be created, existing gets an Update
         If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyName';
    SET @value      = N'PropertyName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsROSystemProperty';
    SET @value      = N'IsROSystemProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Statement;
GO

/*
	POST Operation for a new Statement
	A Statement can only created, there is no Update or Delete

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC				INT
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @ProductID		NVARCHAR(255) = '1'
		DECLARE @ProductLineID	NVARCHAR(255) = 'U'
		DECLARE @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE @ActionType		NVARCHAR(255) = 'SAVE'
		DECLARE @Statement		NVARCHAR(255) = 'ALLES GUT'
		DECLARE @PCName			NVARCHAR(255) = ''
		DECLARE @ProcessorCode	NVARCHAR(255) = ''
		DECLARE @IPAddresses	NVARCHAR(255) = ''
		DECLARE @IsDeleted		NVARCHAR(255) = ''
		DECLARE @IsResolved		NVARCHAR(255) = ''

		EXECUTE @RC = dbo.sx_pf_POST_Statement 
				@Username,	
				@ProductID,	
				@ProductLineID,	
				@FactoryID,	
				@ActionType,
				@Statement,	
				@PCName,
				@ProcessorCode,	
				@IPAddresses,
				@IsDeleted,
				@IsResolved
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Statement',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Statement','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Statement
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ActionType		NVARCHAR(255),
	@Statement		NVARCHAR(255),
	@PCName			NVARCHAR(255),
	@ProcessorCode	NVARCHAR(255),
	@IPAddresses	NVARCHAR(255),
	@IsDeleted		NVARCHAR(255),
	@IsResolved		NVARCHAR(255)	
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			+ ISNULL(@FactoryID,	N'NULL') + N''',''' + ISNULL(@ActionType,		N'NULL') + N''',''' + ISNULL(@Statement,		N'NULL') + N''',''' 
			+ ISNULL(@PCName,		N'NULL') + N''',''' + ISNULL(@ProcessorCode,	N'NULL') + N''',''' + ISNULL(@IPAddresses,		N'NULL') + N''',''' 
			+ ISNULL(@IsDeleted,	N'NULL') + N''',''' + ISNULL(@IsResolved,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ActionType		IS NULL SET @ActionType		= N'';
	IF @Statement		IS NULL SET @Statement		= N'';
	IF @PCName			IS NULL SET @PCName			= N'';
	IF @ProcessorCode	IS NULL SET @ProcessorCode	= N'';
	IF @IPAddresses		IS NULL SET @IPAddresses	= N'';
	IF @IsDeleted		IS NULL SET @IsDeleted		= N'';
	IF @IsResolved		IS NULL SET @IsResolved		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Statement

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ActionType		= dbo.sx_pf_pProtectString	(@ActionType);
		SET @Statement		= dbo.sx_pf_pProtectString	(@Statement);
		SET @PCName			= dbo.sx_pf_pProtectString	(@PCName);
		SET @ProcessorCode	= dbo.sx_pf_pProtectString	(@ProcessorCode);
		SET @IPAddresses	= dbo.sx_pf_pProtectString	(@IPAddresses);
		SET @IsDeleted		= dbo.sx_pf_pProtectBoolean (@IsDeleted);
		SET @IsResolved		= dbo.sx_pf_pProtectBoolean (@IsResolved);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
				
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;
		
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3 - Insert new Statement
		INSERT INTO planning.tfStatements (
			ProductKey,
			ProductLineKey,
			FactoryKey,
			ProductID,
			ProductLineID,
			FactoryID,
			ActionType,
			Statement,
			UserName,
			PCName,
			ProcessorCode,
			IPAddresses,
			Timestamp,
			IsDeleted,
			IsResolved) 
		VALUES
			(@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@ActionType,
			@Statement,
			@TransactUsername,
			@PCName,
			@ProcessorCode,
			@IPAddresses,
			GETUTCDATE(),
			@IsDeleted,
			@IsResolved);

		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_POST_Statement;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Statement;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Statement TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Statement'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a new Statement
         A Statement can only created, there is no Update or Delete';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ActionType';
    SET @value      = N'ActionType';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Statement';
    SET @value      = N'Statement';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PCName';
    SET @value      = N'PCName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessorCode';
    SET @value      = N'ProcessorCode';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IPAddresses';
    SET @value      = N'IPAddresses';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsDeleted';
    SET @value      = N'IsDeleted';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsResolved';
    SET @value      = N'IsResolved';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Template_All;
GO

/*
	POST Operation to apply a template (again) to many Products

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9

	Function principles:
		- Determine all Products in Scope and put them on a ToDo list
		- the fact table values from the existing products are temporary stored
			- numbers with scale information
			- formulas are not stored (but their result values, as this may now be a input value series)
		- the existing products are caved out
			- values are deleted
			- timeline is deleted
			- value series are deleted
		- the template is caved in
			- timeline is inserted
			- value series are inserted
			- all values and formulas are inserted
		- the old values are partly caved in 
			- as long as they now belong of a value series of type input
			- they are also caved in for value series of type XLS/XLS Strict - to keep there values which are not formula based
			- transfers all existing data which now belongs to a column of type input
	
	Search Conditions:
		- Factory ZT is never used as target
		- empty Factory ID		-> all Factories are effected
		- empty ProductLineID	-> all ProductLines effected
		- empty Search Template -> all Products effected
 
 
	Testcall 
		DECLARE  @RC							INT
				,@Username						NVARCHAR(255) = 'SQL'
			
				,@ProductID						NVARCHAR(255) = '2'
				,@ProductLineID					NVARCHAR(255) = '1'
				,@FactoryID						NVARCHAR(255) = '1'
				,@SearchTemplateName			NVARCHAR(255) = 'Unikum_VM'
			
				,@TemplateProductID				NVARCHAR(255) = '1'
				,@TemplateProductLineID			NVARCHAR(255) = 'U' 
				,@TemplateFactoryID				NVARCHAR(255) = 'ZT'

				,@CompareByNameInsteadOfIDFlag	INT			  = 1;

		EXEC @RC = dbo.sx_pf_POST_Template_All 
			 @Username,
			 @ProductLineID,
			 @FactoryID,
			 @ProductID,
			 @SearchTemplateName,
			 @TemplateProductID,
			 @TemplateProductLineID,
			 @TemplateFactoryID,
			 @CompareByNameInsteadOfIDFlag;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template_All',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template_All','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Template_All
	 @Username						NVARCHAR(255)
	,@ProductLineID					NVARCHAR(255)
	,@FactoryID						NVARCHAR(255)
	,@ProductID						NVARCHAR(255) = ''
	,@SearchTemplateName			NVARCHAR(255)
	,@TemplateProductID				NVARCHAR(255)
	,@TemplateProductLineID			NVARCHAR(255)
	,@TemplateFactoryID				NVARCHAR(255)
	,@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
			,@FactoryKey					INT				= 0
			,@ProductLineKey				INT				= 0
			,@ProductKey					INT				= 0
			,@TemplateFactoryKey			INT				= 0
			,@TemplateProductLineKey		INT				= 0
			,@TemplateProductKey			INT				= 0
			,@TemplateFactoryNameShort		NVARCHAR(255)	= N''
			,@TemplateProductLineNameShort	NVARCHAR(255)	= N''
			,@TemplateProductNameShort		NVARCHAR(255)	= N''
			,@TemplateName					NVARCHAR(255)	= N''
			,@ProductCount					BIGINT
			,@CounterSelf					INT
			
			,@ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR(MAX)	= N'''' 
															+ ISNULL(@Username,					N'NULL') + N''',''' 
															+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
															+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
															+ ISNULL(@SearchTemplateName,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
															+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
															+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL')
			,@EffectedRows					INT				= 0			
			,@ResultCode					INT				= 501		
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';			

	-- STEP A1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @SearchTemplateName				IS NULL SET @SearchTemplateName				= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template_All;

		-- ***********************************************
		-- Do Standard Protection
		-- ***********************************************

		-- Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @SearchTemplateName				= dbo.sx_pf_pProtectID	(@SearchTemplateName);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductLineID			= dbo.sx_pf_pProtectID	(@TemplateProductLineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);

		
		-- Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @FactoryKey				AND ProductLineID	= @ProductLineID;
		SELECT @ProductKey				= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey	= @ProductLineKey			AND ProductID		= @ProductID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @TemplateFactoryKey		AND ProductLineID	= @TemplateProductLineID;
		SELECT @TemplateProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey	= @TemplateProductLineKey	AND ProductID		= @TemplateProductID;
		SELECT @TemplateName			= Template			FROM planning.tdProducts 		WHERE ProductKey		= @TemplateProductKey

		-- Stop if Template is invalid (Key of Target Factory / PL may all be 0)
		IF @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- ***********************************************
		-- Create a ToDo List of all Products in scope
		-- ***********************************************
		
		DROP TABLE IF EXISTS #ProductToDoList;

		CREATE TABLE #ProductToDoList
		(
			 FactoryKey		BIGINT										NOT NULL
			,ProductLineKey BIGINT										NOT NULL
			,ProductKey		BIGINT										NOT NULL
			,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Set all Products of scope on the ToDo List
		INSERT INTO #ProductToDoList
			SELECT 
				 dP.FactoryKey
				,dP.ProductLineKey
				,dP.ProductKey
				,dP.FactoryID
				,dP.ProductLineID 
				,dP.ProductID
			FROM planning.tdProducts dP 
			WHERE 
					 
					(dP.FactoryKey		= @FactoryKey			OR @FactoryID			= '') 
				AND (dP.ProductLineKey	= @ProductLineKey		OR @ProductLineID		= '') 
				AND (dP.ProductKey		= @ProductKey			OR @ProductID			= '')
				AND (dP.Template		= @SearchTemplateName	OR @SearchTemplateName	= '')
				AND  dP.ProductKey		<> @TemplateProductKey;									-- don't POST on yourself
			
		-- Stop if Template is inside target Products
		SELECT @CounterSelf = COUNT(ProductKey) 
		FROM #ProductToDoList 
		WHERE Productkey = @TemplateProductKey

		IF @CounterSelf > 0
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Template is in same ProductLine as Target - stopped to not post template on itself.', 16, 10);
			END;

		
		-- Clean ToDo List from Products where user don't has rights
		DELETE FROM #ProductToDoList 
		WHERE ProductKey IN	
		(
			SELECT ProductKey 
			FROM #ProductToDoList ToDo LEFT JOIN system.trUserRights	tUR
				ON ToDo.FactoryID = tUR.FactoryID AND ToDo.ProductLineID = tUR.ProductLineID
			WHERE 
				tUR.[Right] != 'Write' 
				AND tUR.UserName = @TransactUsername
		);

		SELECT @ProductCount = COUNT(ProductKey) FROM #ProductToDoList;

		IF @ProductCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No fitting products found', 16, 10);
		END

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in any product
		IF	@CompareByNameInsteadOfIDFlag = 1 
		AND	EXISTS (SELECT 
						 dVS.ProductKey
						,COUNT(NameShort)
					FROM planning.tdValueSeries dVS 
						-- INNER JOIN to filter on Product on ToDo List
						INNER JOIN #ProductToDoList ToDO
							ON ToDo.ProductKey = dVS.ProductKey
					 GROUP BY dVS.ProductKey, NameShort 
					 HAVING COUNT(NameShort)>1
					)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique', 16, 10);
		END;

		-- ***********************************
		-- STAGING Existing Data from Produts
		-- ***********************************
		

		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			 ProductKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,TimeID				INT											NOT NULL
			,ValueInt			BIGINT										NOT NULL
			,ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueComment		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,[IsNumeric]		INT											NOT NULL
			);

		-- Store Data of Products in Storage (but not formulas, they come from template)
		-- ToDo: Keep new formulas out in this step !!
		INSERT INTO #ProductDataStorage    
			SELECT
				 fV.ProductKey 
				,fV.ValueSeriesID
				,dVS.NameShort	AS ValueSeriesName
				,dVS.Scale
				,fV.TimeID
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment
				,dVS.[IsNumeric]
			FROM 
				planning.tfValues fV 
				-- for ValueSeriesNames
				LEFT JOIN planning.tdValueSeries dVS 
					ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
				-- filter Products on ToDo List
				INNER JOIN	#ProductToDoList ToDo 
					ON fV.ProductKey = ToDo.ProductKey;

		-- ***********************************
		-- Cave out existing Products
		-- ***********************************
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Products (but not the dProduct, Statements...)
		DELETE FROM planning.tfValues		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries	WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime			WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		-- ***********************************
		-- Cave in Template
		-- ***********************************

		-- keep new VS Keys temporary stored
		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ProductKey			BIGINT										NOT NULL
			,ProductLineKey		BIGINT										NOT NULL
			,FactoryKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,ValueSource		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
		);

		-- Copy ValueSeries of Template to all Products and store their Keys
		INSERT INTO planning.tdValueSeries
				OUTPUT	 INSERTED.ValueSeriesKey
						,INSERTED.ProductKey
						,INSERTED.ProductLineKey
						,INSERTED.FactoryKey
						,INSERTED.ValueSeriesID
						,INSERTED.ProductID
						,INSERTED.ProductLineID
						,INSERTED.FactoryID
						,INSERTED.NameShort
						,INSERTED.Scale 
						,INSERTED.ValueSource
				INTO #VSKeys
			SELECT 
				ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dVS.ValueSeriesID
				,dVS.ValueSeriesNo
				,dVS.NameShort
				,dVS.NameLong
				,dVS.CommentUser
				,dVS.CommentDev
				,dVS.ImageName
				,dVS.IsNumeric
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.ValueListID
				,dVS.ValueFormatID
				,dVS.Unit
				,dVS.Scale
				,dVS.Effect
				,dVS.EffectParameter
			FROM planning.tdValueSeries dVS
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dVS.ProductKey = @TemplateProductKey;

		-- Copy Timeline of Template to Product 
		INSERT INTO planning.tdTime
			SELECT 
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dT.TimeID
				,dT.FormatID
			FROM planning.tdTime dT-- spread to all products
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dT.ProductKey = @TemplateProductKey;

		-- Copy Values and Formulas of Template to Products 
		INSERT INTO planning.tfValues
			SELECT
				 vk.ValueSeriesKey
				,vk.ProductKey
				,vk.ProductLineKey
				,vk.FactoryKey
				,vk.ProductID
				,vk.ProductLineID
				,vk.FactoryID
				,fV.ValueSeriesID
				,fV.TimeID
				,fV.ValueFormula
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment
			FROM planning.tfValues fV 
				LEFT JOIN #VSKeys vk 
					ON	fV.ValueSeriesID = vk.ValueSeriesID
			WHERE fV.ProductKey = @TemplateProductKey;

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO planning.tgProducts
			SELECT
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID

			FROM planning.tgProducts gP 
			-- spread to all products
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
                    
			WHERE gP.ProductKey = @TemplateProductKey

		--  ===============================================================
		--> Now all Products are complete fresh 1:1 copies of the Template
		--  ===============================================================

		-- *****************************************************************
		-- cave in the transfer values if they still fit to the template
		-- *****************************************************************

		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled


		-- Delete Transfervalues, which belong to no longer existing TimeIDs
		DELETE FROM #ProductDataStorage
		WHERE TimeID NOT IN	(
			SELECT dT.TimeID
			FROM planning.tdTime dT
			WHERE dT.ProductKey = @TemplateProductKey 
		);

		-- Transfer stored Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
			BEGIN
				-- Delete Transfervalues, which belong to no longer existing IDs
				DELETE FROM #ProductDataStorage 
				WHERE ValueSeriesID NOT IN	(
					SELECT dVS.ValueSeriesID 
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);	

				-- Delete Transfer Values, which belong to Value Series, which DataType has changed
				DELETE pds
				FROM #ProductDataStorage pds
				LEFT JOIN (	SELECT 
								 dVS.ValueSeriesID
								,dVS.[IsNumeric]
							FROM planning.tdValueSeries dVS
							WHERE dVS.ProductKey = @TemplateProductKey
						  ) Template
					ON pds.ValueSeriesID = Template.ValueSeriesID
				WHERE pds.[IsNumeric] <> Template.[IsNumeric];	
				
				-- Values, which belong to value series, which are formula based now are not deleted completely
					-- they are keept in case that the new cell has no formula  
					-- this happens, to keep value in value series of type XLS-Strict, where XLS-Strict just protects the cell
				DELETE FROM  #ProductDataStorage 					
				WHERE 		
					-- defined as formula in Template
					EXISTS
							(
								SELECT 
										fV.TimeID
									,fV.ValueSeriesID
								FROM planning.tfValues fV
									LEFT JOIN planning.tdValueSeries dVS
										ON fV.ValueSeriesKey = dVS.ValueSeriesKey
								WHERE 
										fV.ProductKey	= @TemplateProductKey 
									AND dVS.ValueSource IN ('XLS','XLS-Strict')
									AND fV.ValueFormula <> ''
									-- OUTER Correlation
									AND #ProductDataStorage.TimeID			= fV.TimeID
									AND #ProductDataStorage.ValueSeriesID	= fV.ValueSeriesID
							);
				
				-- Delete Values in fact table to make space for Transfervalues -> WHY DO we need this, when all Products are fresh copies of the template ?
				DELETE fV FROM planning.tfValues fV
					INNER JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					INNER JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND fV.ValueSeriesID	= PDS.ValueSeriesID
							AND fV.TimeID			= PDS.TimeID;

				-- Transfer OVER INSERT
				INSERT INTO planning.tfValues
					SELECT 
						 vk.ValueSeriesKey
						,vk.ProductKey
						,vk.ProductLineKey
						,vk.FactoryKey
						,vk.ProductID
						,vk.ProductLineID
						,vk.FactoryID
						,pds.ValueSeriesID
						,pds.TimeID
						,'' AS ValueFormula
						,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values
						,pds.ValueText
						,pds.ValueComment
					FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesID = vk.ValueSeriesID;

			END
		ELSE
			BEGIN -- compare by Name

				-- Delete Transfervalues, which belong to no longer existing Names
				DELETE FROM #ProductDataStorage
				WHERE ValueSeriesName NOT IN (
					SELECT dVS.NameShort
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);

				-- Delete Transfer Values, which belong to Value Series, which DataType has changed
				DELETE pds
				FROM #ProductDataStorage pds
				LEFT JOIN (	SELECT 
								 dVS.ValueSeriesName
								,dVS.[IsNumeric]
							FROM planning.tdValueSeries dVS
							WHERE dVS.ProductKey = @TemplateProductKey
						  ) Template
					ON pds.ValueSeriesName = Template.ValueSeriesName
				WHERE pds.[IsNumeric] <> Template.[IsNumeric];	

				DELETE FROM  #ProductDataStorage 					
				WHERE 		
					-- defined as formula in Template
					EXISTS
							(
								SELECT 
									 fV.TimeID
									,dVS.NameShort
								FROM planning.tfValues fV
									LEFT JOIN planning.tdValueSeries dVS
										ON fV.ValueSeriesKey = dVS.ValueSeriesKey
								WHERE 
										fV.ProductKey	= @TemplateProductKey 
									AND dVS.ValueSource IN ('XLS','XLS-Strict')
									AND fV.ValueFormula <> ''
									-- OUTER Correlation
									AND #ProductDataStorage.TimeID			= fV.TimeID
									AND #ProductDataStorage.ValueSeriesName	= dVS.NameShort
							);


				-- Delete Values in fact table to make space for Transfervalues
				DELETE fV FROM planning.tfValues fV

					LEFT JOIN planning.tdValueSeries dVS
						ON fV.ValueSeriesKey = dVS.ValueSeriesKey

					LEFT JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					LEFT JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND dVS.NameShort		= PDS.ValueSeriesName
							AND fV.TimeID			= PDS.TimeID
				WHERE 
						PDS.TimeID			IS NOT NULL 
					AND PDS.ValueSeriesName	IS NOT NULL

				-- Insert all Transfervalues
				INSERT INTO planning.tfValues
				SELECT 
					 vk.ValueSeriesKey
					,vk.ProductKey
					,vk.ProductLineKey
					,vk.FactoryKey
					,vk.ProductID
					,vk.ProductLineID
					,vk.FactoryID
					,dVS.ValueSeriesID
					,pds.TimeID
					,'' AS ValueFormula
					,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values 
					,pds.ValueText
					,pds.ValueComment
				FROM #ProductDataStorage pds 
					-- to determine ValueSeries Key
					LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesName = vk.ValueSeriesName
					-- to determine new ValueSeriesID
					LEFT JOIN 
						(
							SELECT 
								 dVS.ValueSeriesID
								,dVS.NameShort 
							FROM planning.tdValueSeries dVS 
							WHERE dVS.ProductKey = @TemplateProductKey
						) dVS 
					ON dVS.NameShort = pds.ValueSeriesName

			END;

		-- Create a System Statement
		INSERT INTO planning.tfStatements
			SELECT  
				 ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,N'POST_Template_All' AS ActionType
				,N'Product Template updated with Template Factory <'  
					+ @TemplateFactoryID			+ N'>"' 
					+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
					+ @TemplateProductLineID		+ N'>"' 
					+ @TemplateProductLineNameShort + N'"/ Product <' 
					+ @TemplateProductID			+ N'>"' 
					+ @TemplateProductNameShort		+ '"' AS Statement
				,@TransactUsername AS UserName
				,N'' AS PCName
				,N'' AS ProcessorCode
				,N'' AS IPAddresses
				,GETUTCDATE() AS Timestamp
				,0 AS IsDeleted
				,0 AS IsResolved
				FROM #ProductToDoList 
			
			-- Update the TemplateName of the Products
			UPDATE planning.tdProducts
			SET  
				 Template = @TemplateName
				,TimeType = RIGHT(@TemplateName, 2) 
			WHERE 
				ProductKey IN (SELECT ProductKey FROM #ProductToDoList);

		SET @EffectedRows += @@ROWCOUNT;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_POST_Template_All;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template_All;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Template_All'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'/*
         POST Operation to apply a template (again) to many Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SearchTemplateName';
    SET @value      = N'SearchTemplateName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductID';
    SET @value      = N'TemplateProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductLineID';
    SET @value      = N'TemplateProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateFactoryID';
    SET @value      = N'TemplateFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompareByNameInsteadOfIDFlag';
    SET @value      = N'CompareByNameInsteadOfIDFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Template;
GO

/*
	POST Operation to apply a template (again) on a Product
	Existing Values in the Product will be saved, new/other logic will be added
	It means the Template inherits its properties again to the object

	The logical process is describe in procedure POST_TEMPLATE_ALL to which the work is delegated

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call
		DECLARE  @RC								INT 
				,@Username						NVARCHAR(255) = 'SQL'
				,@ProductID						NVARCHAR(255) = '2'
				,@ProductLineID					NVARCHAR(255) = 'U'
				,@FactoryID						NVARCHAR(255) = 'ZT'
		
				,@TemplateProductID				NVARCHAR(255) = '1'
				,@TemplateProductLineID			NVARCHAR(255) = 'U'
				,@TemplateFactoryID				NVARCHAR(255) = 'ZT'
				,@CompareByNameInsteadOfIDFlag	INT = 0

		EXECUTE @RC = dbo.sx_pf_POST_Template 
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@TemplateProductID
				,@TemplateProductLineID
				,@TemplateFactoryID
				,@CompareByNameInsteadOfIDFlag

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_POST_Template
	@Username						NVARCHAR(255),
	@ProductID						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN
	DECLARE  @TransactUsername				NVARCHAR(255)	= N''
			,@FactoryKey						INT				= 0
			,@ProductLineKey					INT				= 0
			,@ProductKey						INT				= 0
			,@TemplateFactoryKey				INT				= 0
			,@TemplateProductLineKey			INT				= 0
			,@TemplateProductKey				INT				= 0
			,@TemplateName					NVARCHAR(255)	= N''
			,@ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username,					N'NULL') + N''',''' 
														+ ISNULL(@ProductID,				N'NULL') + N''',''' 
														+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
														+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
														+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
														+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL')
			,@EffectedRows					INT				= 0				
			,@ResultCode					INT				= 501				
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';


	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		-- No transaction started, as delegated to other Procedure

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductLineID			= dbo.sx_pf_pProtectID	(@TemplateProductLineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);
	
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 0.3. - Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @TemplateFactoryKey AND ProductLineID = @TemplateProductLineID;

		-- Stop on invalid Keys
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @TemplateFactoryKey = 0 OR @TemplateProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		 -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN

			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode <> 200
				RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 2.1 - Try to identify the product keys
		SELECT @ProductKey			= ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @TemplateProductKey	= ProductKey FROM planning.tdProducts WHERE ProductLineKey = @TemplateProductLineKey AND ProductID = @TemplateProductID;
		
		-- Stop if Keys don't exist
		IF @ProductKey = 0 OR @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- Stop if Template is posted on itself
		IF @ProductKey = @TemplateProductKey
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Stop, as Template cant be posted on itself.', 16, 10);
		END;

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in Template
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND	EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @TemplateProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Template', 16, 10);
		END

		-- STEP 2.3 - Stop, if compare by name and there are doubled ValueSerieNames in Product
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @ProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Target Product', 16, 10);
		END

		-- Determine Name of SearchTempale
		SELECT @TemplateName = Template FROM planning.tdProducts WHERE ProductKey = @TemplateProductKey;
		
		-- DELEGATE Operation to POST_TEMPLATE ALL #############################################################################
		EXEC @Resultcode = dbo.sx_pf_POST_Template_All 
					 'SQL'
					 ,@ProductLineID
					 ,@FactoryID
					 ,@ProductID
					 ,@TemplateName --for @SearchTemplateName
					 ,@TemplateProductID
					 ,@TemplateProductLineID
					 ,@TemplateFactoryID
					 ,@CompareByNameInsteadOfIDFlag;
		

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryService; 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Template'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation to apply a template (again) on a Product
         Existing Values in the Product will be saved, new/other logic will be added
         It means the Template inherits its properties again to the object';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductID';
    SET @value      = N'TemplateProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductLineID';
    SET @value      = N'TemplateProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateFactoryID';
    SET @value      = N'TemplateFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompareByNameInsteadOfIDFlag';
    SET @value      = N'CompareByNameInsteadOfIDFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_TimeID;
GO

/*
	POST Operation for TimeID
	Create new TimeID

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE  @RC			INT
		DECLARE  @Username		NVARCHAR(255) = 'SQL'
		DECLARE  @ProductID		NVARCHAR(255) = '1'
		DECLARE  @ProductLineID NVARCHAR(255) = 'U'
		DECLARE  @FactoryID		NVARCHAR(255) = 'ZT'
		DECLARE  @TimeID		INT			  = 20150107
		DECLARE  @FormatID		NVARCHAR(255) = 'TimeBorderNext'

		EXECUTE @RC = dbo.sx_pf_POST_TimeID 
				@Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@TimeID
				,@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_TimeID',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_TimeID','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_TimeID
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@TimeID				INT,
	@FormatID			NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@FormatID,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @FormatID		IS NULL SET @FormatID		= '';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_TimeID;

		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID  AND FactoryKey = @FactoryKey;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductID = @ProductID  AND ProductLineKey = @ProductLinekey AND FactoryKey = @FactoryKey;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 2.1 - Delete TimeID
		DELETE FROM planning.tdTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- STEP 3 - Create new TimeID

		INSERT INTO planning.tdTime	(
			ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,TimeID
			,FormatID
		)
		VALUES (
			@ProductKey,
			@ProductLineKey,
			@FactoryKey,
			@ProductID,
			@ProductLineID,
			@FactoryID,
			@TimeID,
			@FormatID
		);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_TimeID;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_TimeID;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_TimeID TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_TimeID'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for TimeID
         Create new TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Timeline;
GO

/*
	POST Operation for the Timeline
	The method can be switched between incremental POST and full POST
	Values with DeleteFlag will be deleted

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Test case 1 - Incremental Sended
	1. One Value for an existing Value must replace them
	2. One Value for an not exiting Value must create it
	3. If all Parameter are posted empty, the value is deleted
	4. if one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
		DECLARE @RC int
		DECLARE @Username					NVARCHAR(255) = 'SQL'
		DECLARE @ProductID					NVARCHAR(255) = '1'
		DECLARE @ProductLineID				NVARCHAR(255) = 'U'
		DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
		DECLARE @IsIncrementalValuesFlag	INT = 0

		--3 TimeIDs
		DECLARE @TimeIDsInBracketsCommaSeparated nvarchar(max) = '[(20160115,0,0,''FormatID1'')],[(20160215,0,0,''FormatID2'')],[(20160415,0,0,''FormatID3'')]'
		EXECUTE @RC = dbo.sx_pf_POST_Timeline
			 @Username
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,@IsIncrementalValuesFlag
			,@TimeIDsInBracketsCommaSeparated
		PRINT @RC

		Test case 2 - all values in a Value Series of Products:
		Additional 1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
		DECLARE @RC int
		DECLARE @Username							NVARCHAR(255) = 'SQL'
		DECLARE @ProductID							NVARCHAR(255) = '21'
		DECLARE @ProductLineID						NVARCHAR(255) = '1'
		DECLARE @FactoryID							NVARCHAR(255) = 'U'
		DECLARE @IsIncrementalValuesFlag			INT = 0
		DECLARE @TimeIDsInBracketsCommaSeparated	NVARCHAR(max) = ''
		DECLARE @start_time DATETIME

		-- Generate fictitious values for all TimeIDs of Products
		SELECT @TimeIDsInBracketsCommaSeparated =  @TimeIDsInBracketsCommaSeparated + '[(''' + dVS2.ValueSeriesID + ''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',1000,''Hase'',''Kommentar'')],' 
		FROM planning.tfValues fV LEFT JOIN 
			planning.tdValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey LEFT JOIN
			(SELECT ValueSeriesID FROM planning.tdValueSeries WHERE FactoryID =  @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID GROUP BY ValueSeriesID) dVS2
				ON 1=1 
		WHERE fV.FactoryID =  @FactoryID AND fV.ProductLineID = @ProductLineID AND fV.ProductID = @ProductID AND dVS.ValueSeriesNo = 1
		SET @TimeIDsInBracketsCommaSeparated = Left(@TimeIDsInBracketsCommaSeparated,Len(@TimeIDsInBracketsCommaSeparated) -1)
		PRINT @TimeIDsInBracketsCommaSeparated
		SET @start_time = GETUTCDATE()
		EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues
			 @Username
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,@IsIncrementalValuesFlag
			,@TimeIDsInBracketsCommaSeparated
		PRINT @RC
		SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETUTCDATE()) AS CHAR(10))) AS 'TimeTaken'

		@TimeIDsInBracketsCommaSeparated format: (TimeID,DeleteFlag,AddValue,FormatID)
		if you check this in SQL Server Profiler, every String must have two inverted commas, the whole Parameter must have Single inverted comma
		this must show profiler: '[(''TimeID,DeleteFlag,AddValue,''FormatID'')],[(TimeID,DeleteFlag,AddValue,FormatID)]'
		if you check this in table system.tAPI_Log, every String must have one inverted comma, whole String has no inverted comma
		this must show Log: [(TimeID,DeleteFlag,AddValue,FormatID)],[(TimeID,DeleteFlag,AddValue,'FormatID')]

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Timeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Timeline','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Timeline
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@TimeIDsInBracketsCommaSeparated	NVARCHAR(MAX)

AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;

	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductID							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductLineID						, N'NULL')	+ N''',''' 
								+ ISNULL(@FactoryID							, N'NULL')	+ N''',''' 
								+ ISNULL(@TimeIDsInBracketsCommaSeparated	, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR (2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username							= N'';
	IF @ProductID						IS NULL SET @ProductID							= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID						= N'';
	IF @FactoryID						IS NULL SET @FactoryID							= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag			= 0;
	IF @TimeIDsInBracketsCommaSeparated IS NULL SET @TimeIDsInBracketsCommaSeparated	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Timeline;
		
		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID (@FactoryID)
		SET @ProductID			= dbo.sx_pf_pProtectID (@ProductID)
		SET @ProductLineID		= dbo.sx_pf_pProtectID (@ProductLineID)

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
				
		-- STEP 3.2 - Check for count of delimiter in Array

		DECLARE @OpenDelimetersCount INT = 0;
		DECLARE @DelimetersIndex INT = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.3 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @TimeIDsInBracketsCommaSeparated = LEFT(@TimeIDsInBracketsCommaSeparated,  LEN(@TimeIDsInBracketsCommaSeparated) - 2);
		SET @TimeIDsInBracketsCommaSeparated = RIGHT(@TimeIDsInBracketsCommaSeparated, LEN(@TimeIDsInBracketsCommaSeparated) - 2);

		DECLARE @tblValues1 TABLE (txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues2 TABLE (TimeID		 NVARCHAR(MAX) NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues3 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues4 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, AddValue		 BIGINT		   NOT NULL, FormatID NVARCHAR(255) NOT NULL);

		-- Split String into Rows
		INSERT INTO @tblValues1 
			SELECT txtValues FROM dbo.sx_pf_PivotStringIntoTable(@TimeIDsInBracketsCommaSeparated, N')],[(');

		-- Split one column into two Columns (separate TimeID)
		INSERT INTO @tblValues2
			SELECT  
				 LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe)-1 ) AS TimeID
				,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)) AS Rest
			FROM @tblValues1;

		IF EXISTS(SELECT TimeID FROM @tblValues2 GROUP BY TimeID HAVING(COUNT(TimeID) > 1))
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Doubled TimeID in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- Split last columns once more (separate DeleteFlag)
		INSERT INTO @tblValues3
			SELECT  
				 TimeID
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS DeleteFlag
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)-1) AS Rest
			FROM @tblValues2;

		-- Split last columns once more (separate AddValue)
		INSERT INTO @tblValues4
			SELECT  
				 TimeID
				,DeleteFlag
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS AddValue
			,REPLACE(Right(txtValuesPipe, LEN(txtValuesPipe) - 1), N'''','') AS FormatID
			FROM @tblValues3;

		-- STEP 3.4 - Actions #######################################################################################################################
	
		-- Delete TimeIDs which are sended as deleted in fValues and in dTime
		DELETE FROM planning.tdTime 
		WHERE 
			@ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);
	
		SET @EffectedRows = @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE 
			ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);

		SET @EffectedRows += @@ROWCOUNT;
		-- STEP 3.5 - CASE FULL transmission or Incremental does not matter anymore #########################################
		IF @IsIncrementalValuesFlag != 1 OR  @IsIncrementalValuesFlag = 1 
		BEGIN
			-- Insert all New TimeIDs
			INSERT INTO planning.tdTime
			SELECT  
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				TimeID,
				FormatID
			FROM @tblValues4 
			WHERE TimeID NOT IN (SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey);

			SET @EffectedRows += @@ROWCOUNT;
		END;
			
		COMMIT TRANSACTION sx_pf_POST_Timeline;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Timeline;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Timeline'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for the Timeline
         The method can be switched between incremental POST and full POST
         Values with DeleteFlag will be deleted';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsIncrementalValuesFlag';
    SET @value      = N'IsIncrementalValuesFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeIDsInBracketsCommaSeparated';
    SET @value      = N'TimeIDsInBracketsCommaSeparated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Value;
GO

/*
	POST Operation for Value
	The Value will be deleted and writen new

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test calls
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ProductID		NVARCHAR(255) = '1'
				,@ProductLineID	NVARCHAR(255) = 'U'
				,@FactoryID		NVARCHAR(255) = 'ZT'
				,@ValueSeriesID	NVARCHAR(255) = 'E1'
				,@TimeID		BIGINT		  = 20180315
				,@ValueFormula	NVARCHAR(max) = '<#NV>'
				,@ValueInt		NVARCHAR(255) = '555' 
				,@ValueText		NVARCHAR(max) = 'Hase2'
				,@ValueComment	NVARCHAR(max) = 'Hase2'
				,@i				INT = 0

		-- individual values 
		-- Text with Comment
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				'E1',
				20180115,
				'',
				0,
				'Hase',
				'Kommentar für Hase';
		PRINT @RC

		-- Integer with Comment
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				'K1',
				20180215,
				'',
				44,
				'',
				'Kommentar für Hase44';
		PRINT @RC

		-- 1b. Variant with loop
		--WHILE @i < 1000
		--BEGIN
		EXECUTE @RC = dbo.sx_pf_POST_Value 
				@Username,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@ValueSeriesID,
				@TimeID,
				@ValueFormula,
				@i,
				@ValueText,
				@ValueComment
		--SET @i = @i +1
		PRINT @RC
		--END

		-- 2. Variant with Script Creator
		PRINT 'DECLARE @RC as INTEGER = 0'

		WHILE @i < 5000
		BEGIN
			PRINT 'EXECUTE @RC = dbo.sx_pf_POST_Value ''' + @Username + ''','''+ @ProductID+ ''','''+@ProductLineID+ ''','''+ @FactoryID + ''','''+ @ValueSeriesID + ''','+ CAST(@TimeID AS NVARCHAR (8)) + ','''+ @ValueFormula + ''','+ CAST(@i AS NVARCHAR(10)) + ','''+@ValueText+ ''','''+	@ValueComment + ''''
			PRINT 'PRINT @RC' 
			SET @i = @i +1 
		END
		--about 12 sec for 5000 Values Values = 400 / second

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Value',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Value','PARAMETER',NULL)
*/
 
CREATE PROCEDURE dbo.sx_pf_POST_Value
	@Username		NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ValueSeriesID	NVARCHAR(255),		
	@TimeID			INT,
	@ValueFormula	NVARCHAR(MAX),
	@ValueInt		NVARCHAR(255), -- to be able to send <#NV>
	@ValueText		NVARCHAR(MAX),
	@ValueComment	NVARCHAR(MAX)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @ValueKey			BIGINT			= 0;

	DECLARE @OLDValueFormula	NVARCHAR(255)	= N'';
	DECLARE @OLDValueInt		BIGINT			= 0; 
	DECLARE @OLDValueText		NVARCHAR(255)	= N'';
	DECLARE @OLDValueComment	NVARCHAR(MAX)	= N'';

	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
		+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,		N'NULL') + N''',''' 
		+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,		N'NULL') + N''',''' 
		+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@TimeID AS NVARCHAR(255)), N'NULL') + N',''' 
		+ ISNULL(@ValueFormula,		N'NULL') + N''',''' + ISNULL(@ValueInt,			N'NULL') + N''',''' 
		+ ISNULL(@ValueText,		N'NULL') + N''',''' + ISNULL(@ValueComment,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	
	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;
	IF @ValueFormula	IS NULL SET @ValueFormula	= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueComment	IS NULL SET @ValueComment	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Value;

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @TimeID			= dbo.sx_pf_pProtectID		(@TimeID);
		SET @ValueFormula	= dbo.sx_pf_pProtectString	(@ValueFormula);
		SET @ValueInt		= dbo.sx_pf_pProtectString	(@ValueInt);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);

		-- Remove Linebreaks
		SET @ValueFormula	= REPLACE(REPLACE(@ValueFormula	,CHAR(13),''),CHAR(10),'');
		SET @ValueText		= REPLACE(REPLACE(@ValueText	,CHAR(13),''),CHAR(10),'');
		SET @ValueComment	= REPLACE(REPLACE(@ValueComment	,CHAR(13),''),CHAR(10),'');

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N'' OR @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

 	     -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- STEP 1.3 - Try to detect and cancel the ValueSeriesKey if does not exist
		SELECT @ValueSeriesKey = ValueSeriesKey FROM planning.tdValueSeries WHERE ProductKey= @ProductKey AND ValueSeriesID = @ValueSeriesID;
		
		IF @ValueSeriesKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2.1 - Cancel if TimeID not exists	
		IF NOT EXISTS(SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey AND TimeID = @TimeID)
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('TimeID not exists', 16, 10);
		END;

		-- STEP 3. Determine ValueKey identify and old values
		SELECT @ValueKey = ValueKey, @OLDValueFormula = ValueFormula, @OLDValueInt = ValueInt, @OLDValueText = ValueText, @OLDValueComment = ValueComment
		FROM planning.tfValues 
		WHERE ValueSeriesKey = @ValueSeriesKey AND TimeID = @TimeID;
				
		-- STEP 3.1 - If Value Key exists delete
		IF  @ValueKey > 0 
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ValueKey = @ValueKey;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 204; -- Ok, but deleted

			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', @OLDValueFormula,	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', @OLDValueText,		@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', @OLDValueComment,	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', @OLDValueInt,		@ValueInt);
		END
		ELSE
		BEGIN
			SET @ValueFormula	= IIF(@ValueFormula		= N'<#NV>', N'',	@ValueFormula);
			SET @ValueText		= IIF(@ValueText		= N'<#NV>', N'',	@ValueText);
			SET @ValueComment	= IIF(@ValueComment		= N'<#NV>', N'',	@ValueComment);
			SET @ValueInt		= IIF(@ValueInt			= N'<#NV>', N'0',	@ValueInt);
		END;

		-- STEP 3.2 - Insert, if Value not empty outside *_0 ValueSeries
		IF @ValueFormula = N'' AND @ValueText = N'' AND @ValueInt = N'0' AND RIGHT(@ValueSeriesID,2)<>'_0'
		BEGIN 
			SET @ResultCode = 204; --> discarded !!! 
		END
		ELSE
		BEGIN 
			SET @ValueInt = COALESCE(TRY_CAST(@ValueInt AS BIGINT), 0);

			INSERT INTO planning.tfValues
			VALUES (
				@ValueSeriesKey, 
				@ProductKey, 
				@ProductLineKey, 
				@FactoryKey, 
				@ProductID, 
				@ProductLineID, 
				@FactoryID, 
				@ValueSeriesID,  
				@TimeID,  
				@ValueFormula, 
				@ValueInt, 
				@ValueText, 
				@ValueComment);
			
			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 201;
		END;

		COMMIT TRANSACTION sx_pf_POST_Value;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Value;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Value TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Value'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Value
         The Value will be deleted and writen new';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormula';
    SET @value      = N'ValueFormula';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueComment';
    SET @value      = N'ValueComment';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ValueSerie;
GO

/*
	POST Operation for one ValueSeries
	Operation will create a ValueSeries if it not exist
	It will change an existing ValueSeries
	Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) ='SQL'
			,@ProductID					NVARCHAR(255) ='1'
			,@ProductLineID				NVARCHAR(255) ='U'
			,@FactoryID					NVARCHAR(255) ='ZT'
			,@ValueSeriesID				NVARCHAR(255) ='38550'
			,@RequestedValueSeriesNo	INT			  = 249
			,@NameShort					NVARCHAR(255) ='<#NV>'
			,@NameLong					NVARCHAR(255) ='<#NV>'
			,@CommentUser				NVARCHAR(255) ='<#NV>'
			,@CommentDev				NVARCHAR(255) ='<#NV>'
			,@ImageName					NVARCHAR(255) ='<#NV>'
			,@IsNumeric					NVARCHAR(255) ='1'
			,@VisibilityLevel			NVARCHAR(255) ='<#NV>'
			,@ValueSource				NVARCHAR(255) ='<#NV>'
			,@ValueListID				NVARCHAR(255) ='<#NV>'
			,@ValueFormatID				NVARCHAR(255) ='<#NV>'
			,@Unit						NVARCHAR(255) ='<#NV>'
			,@Scale						NVARCHAR(255) ='100'
			,@Effect					NVARCHAR(255) ='<#NV>'
			,@EffectParameter			NVARCHAR(255) ='<#NV>'

	EXECUTE @RC = dbo.sx_pf_POST_ValueSerie 
			@Username,	
			@ProductID,	
			@ProductLineID,	
			@FactoryID,	
			@ValueSeriesID,	
			@RequestedValueSeriesNo,
			@NameShort,	
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ImageName,
			@IsNumeric,
			@VisibilityLevel,
			@ValueSource,
			@ValueListID,
			@ValueFormatID,
			@Unit,
			@Scale,
			@Effect,
			@EffectParameter
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ValueSerie',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ValueSerie','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ValueSerie
	@Username				 NVARCHAR(255),
	@ProductID				 NVARCHAR(255),
	@ProductLineID			 NVARCHAR(255),
	@FactoryID				 NVARCHAR(255),
	@ValueSeriesID			 NVARCHAR(255),
	@RequestedValueSeriesNo	 INT,
	@NameShort				 NVARCHAR(255),
	@NameLong				 NVARCHAR(255),
	@CommentUser			 NVARCHAR(255),
	@CommentDev				 NVARCHAR(255),
	@ImageName				 NVARCHAR(255),
	@IsNumeric				 NVARCHAR(255),
	@VisibilityLevel		 NVARCHAR(255),
	@ValueSource			 NVARCHAR(255),
	@ValueListID			 NVARCHAR(255),
	@ValueFormatID			 NVARCHAR(255),
	@Unit					 NVARCHAR(255),
	@Scale					 NVARCHAR(255),
	@Effect					 NVARCHAR(255),
	@EffectParameter		 NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @ValueSeriesKey		INT				= 0;
	DECLARE @OldValueSeriesNo	INT				= 0;
	DECLARE @OldIsNumeric		INT				= 0;
	DECLARE @OldScale			INT				= 0;
	DECLARE @OldValueSource		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' + ISNULL(@ProductID,										N'NULL') + N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,										N'NULL') + N''',''' 
			+ ISNULL(@ValueSeriesID,	N'NULL') + N''','	+ ISNULL(CAST(@RequestedValueSeriesNo AS NVARCHAR(255)),	N'NULL') + N',''' 
			+ ISNULL(@NameShort,		N'NULL') + N''',''' + ISNULL(@NameLong,											N'NULL') + N''',''' 
			+ ISNULL(@CommentUser,		N'NULL') + N''',''' + ISNULL(@CommentDev,										N'NULL') + N''',''' 
			+ ISNULL(@ImageName,		N'NULL') + N''',''' + ISNULL(@IsNumeric,										N'NULL') + N''',''' 
			+ ISNULL(@VisibilityLevel,	N'NULL') + N''',''' + ISNULL(@ValueSource,										N'NULL') + N''',''' 
			+ ISNULL(@ValueListID,		N'NULL') + N''',''' + ISNULL(@ValueFormatID,									N'NULL') + N''',''' 
			+ ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@Scale,											N'NULL') + N''',''' 
			+ ISNULL(@Effect,			N'NULL') + N''',''' + ISNULL(@EffectParameter,									N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username					= N'';
	IF @ProductID				IS NULL SET @ProductID					= N'NULL';
	IF @ProductLineID			IS NULL SET @ProductLineID				= N'NULL';
	IF @FactoryID				IS NULL SET @FactoryID					= N'NULL';
	IF @ValueSeriesID			IS NULL SET @ValueSeriesID				= N'NULL';
	IF @RequestedValueSeriesNo	IS NULL SET @RequestedValueSeriesNo		= 0;
	IF @NameShort				IS NULL SET @NameShort					= N'NULL';
	IF @NameLong				IS NULL SET @NameLong					= N'NULL';
	IF @CommentUser				IS NULL SET @CommentUser				= N'NULL';
	IF @CommentDev				IS NULL SET @CommentDev					= N'NULL';
	IF @ImageName				IS NULL SET @ImageName					= N'NULL';
	IF @IsNumeric				IS NULL SET @IsNumeric					= N'NULL';
	IF @VisibilityLevel			IS NULL SET @VisibilityLevel			= N'NULL';
	IF @ValueSource				IS NULL SET @ValueSource				= N'NULL';
	IF @ValueListID				IS NULL SET @ValueListID				= N'NULL';
	IF @ValueFormatID			IS NULL SET @ValueFormatID				= N'NULL';
	IF @Unit					IS NULL SET @Unit						= N'NULL';
	IF @Scale					IS NULL SET @Scale						= N'NULL';
	IF @Effect					IS NULL SET @Effect						= N'NULL';
	IF @EffectParameter			IS NULL SET @EffectParameter			= N'NULL';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ValueSerie;

		-- STEP 0.1 - NULL Protection
		SET @ProductID			= dbo.sx_pf_pProtectID		(@ProductID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ValueSeriesID		= dbo.sx_pf_pProtectID		(@ValueSeriesID);
		SET @NameShort			= dbo.sx_pf_pProtectString	(@NameShort);
		SET @NameLong			= dbo.sx_pf_pProtectString	(@NameLong);
		SET @CommentUser		= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev			= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @ImageName			= dbo.sx_pf_pProtectString	(@ImageName);
		SET @IsNumeric			= dbo.sx_pf_pProtectBoolean (@IsNumeric);
		SET @VisibilityLevel	= dbo.sx_pf_pProtectString	(@VisibilityLevel);
		SET @ValueSource		= dbo.sx_pf_pProtectString	(@ValueSource);
		SET @Unit				= dbo.sx_pf_pProtectString	(@Unit);
		SET @Scale				= dbo.sx_pf_pProtectInt		(@Scale);
		SET @Effect				= dbo.sx_pf_pProtectString	(@Effect);
		SET @EffectParameter	= dbo.sx_pf_pProtectString	(@EffectParameter);
		IF @ValueListID		<> N'<#NV>' SET @ValueListID	= dbo.sx_pf_pProtectID (@ValueListID);
		IF @ValueFormatID	<> N'<#NV>' SET @ValueFormatID	= dbo.sx_pf_pProtectID (@ValueFormatID);
		
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N'' OR @ValueSeriesID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys	
		SELECT @FactoryKey = FactoryKey			FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey			FROM planning.tdProducts		WHERE ProductLineKey = @ProductLinekey AND ProductID= @ProductID;
		SELECT 
				 @ValueSeriesKey = ValueSeriesKey
				,@OldIsNumeric = IsNumeric
				,@OldScale = Scale
				,@OldValueSource = ValueSource 
		FROM planning.tdValueSeries 
		WHERE 
				ProductKey = @ProductKey 
			AND ValueSeriesID = @ValueSeriesID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - Insert the ValueSeries, if Key does not exist
		IF @ValueSeriesKey = 0
		BEGIN
			-- INCREMENT all following Requested ValueSeriesNumbers based on requested No
			UPDATE planning.tdValueSeries
			SET ValueSeriesNo = ValueSeriesNo + 1 
			WHERE 
				ProductKey = @ProductKey 
				AND ValueSeriesNo >= @RequestedValueSeriesNo;

			IF @VisibilityLevel = '<#NV>' SET @VisibilityLevel = 1;
			
			SET @EffectedRows = @@ROWCOUNT;

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			-- Insert with requested number
			INSERT INTO planning.tdValueSeries (
				ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,ValueSeriesID
				,ValueSeriesNo
				,NameShort
				,NameLong
				,CommentUser
				,CommentDev
				,ImageName
				,IsNumeric
				,VisibilityLevel
				,ValueSource
				,ValueListID
				,ValueFormatID
				,Unit
				,Scale
				,Effect
				,EffectParameter) 
			OUTPUT INSERTED.ValueSeriesKey INTO @Keys([Key])
			VALUES ( 
				@ProductKey
				,@ProductLineKey
				,@FactoryKey
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter);
			
			SET @EffectedRows += @@ROWCOUNT;
			SELECT TOP (1) @ValueSeriesKey = [Key] FROM @Keys ORDER BY [Key];
		END
		ELSE
		BEGIN 
		-- STEP 3.2 - Update ValueSeries if it already exists
			SELECT @OldValueSeriesNo = ValueSeriesNo FROM planning.tdValueSeries WHERE ValueSeriesKey = @ValueSeriesKey;
									
			-- Use the old one, if no SeriesNo is commited (lower than one means not commited)
			IF @RequestedValueSeriesNo < 1 
				SET @RequestedValueSeriesNo = @OldValueSeriesNo;
			
			-- Adjust Numbers if not equal
			IF @OldValueSeriesNo < @RequestedValueSeriesNo
			BEGIN 
				-- decrement all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo - 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo > @OldValueSeriesNo 
					AND ValueSeriesNo <= @RequestedValueSeriesNo;
														
				SET @EffectedRows += @@ROWCOUNT;
			END
			ELSE IF @OldValueSeriesNo > @RequestedValueSeriesNo
			BEGIN 
				-- increment all between actual and requsted Position
				UPDATE planning.tdValueSeries 
				SET ValueSeriesNo = ValueSeriesNo + 1 
				WHERE 
					ProductKey = @ProductKey 
					AND ValueSeriesNo < @OldValueSeriesNo 
					AND ValueSeriesNo >= @RequestedValueSeriesNo;

				SET @EffectedRows += @@ROWCOUNT;
			END;

			UPDATE planning.tdValueSeries 
			SET 
				ValueSeriesNo		= @RequestedValueSeriesNo
				, NameShort			= IIF(@NameShort		= N'<#NV>', NameShort,			@NameShort)
				, NameLong			= IIF(@NameLong			= N'<#NV>', NameLong,			@NameLong)
				, CommentUser		= IIF(@CommentUser		= N'<#NV>', CommentUser,		@CommentUser)
				, CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,			@CommentDev)
				, ImageName			= IIF(@ImageName		= N'<#NV>', ImageName,			@ImageName)
				, IsNumeric			= IIF(@IsNumeric		= N'<#NV>', IsNumeric,			@IsNumeric)
				, VisibilityLevel	= IIF(@VisibilityLevel	= N'<#NV>', VisibilityLevel,	@VisibilityLevel)
				, ValueSource		= IIF(@ValueSource		= N'<#NV>', ValueSource,		@ValueSource)
				, ValueListID		= IIF(@ValueListID		= N'<#NV>', ValueListID,		@ValueListID)
				, ValueFormatID		= IIF(@ValueFormatID	= N'<#NV>', ValueFormatID,		@ValueFormatID)
				, Unit				= IIF(@Unit				= N'<#NV>', Unit,				@Unit)
				, Scale				= IIF(@Scale			= N'<#NV>', Scale,				@Scale)
				, Effect			= IIF(@Effect			= N'<#NV>', Effect,				@Effect)
				, EffectParameter	= IIF(@EffectParameter	= N'<#NV>', EffectParameter,	@EffectParameter)
			FROM planning.tdValueSeries
			WHERE ValueSeriesKey = @ValueSeriesKey;
				
			SET @EffectedRows += @@ROWCOUNT;

			-- STEP 3.2a - Delete zero values if ValueSeries is now numeric input series
			IF @IsNumeric = '1' AND @ValueSource = N'Input'
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueInt = 0; 
			END;

			-- Step 3.2b - Delete empty Strings if ValueSeries is now non-numeric input series
			IF @IsNumeric = '0' AND @ValueSource = N'Input' 
			BEGIN
				DELETE FROM planning.tfValues
				WHERE
					ValueSeriesKey = @ValueSeriesKey 
					AND	ValueText = N'';
			END;

				
			-- STEP 3.3 - Update exists Values due to possible rescaling or changing type from Numeric to Non-Numeric
			DECLARE @vals AS TABLE (ValueKey INT NOT NULL, ValueInt INT NOT NULL, ValueText NVARCHAR(MAX) NOT NULL);

			-- Changing type Numeric / Non Numeric
			IF (@IsNumeric <> N'<#NV>' AND @OldIsNumeric <> @IsNumeric)
			BEGIN
				-- Copy ValueText to ValueInt, after Try_Cast to Int to get only valid numbers and multiplied by OldScale, as the int values use scale, while the text value not
				IF @IsNumeric = N'1'
				BEGIN							 
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ISNULL(TRY_CAST(ValueText AS INT), 0) * @OldScale, ''
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					-- If Numeric is made to non numeric the value in the Database must be be divided by scale and converted to text, as text dont uses scale
					IF (@OldScale <> 0)
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt / @OldScale AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey;	
					END
					ELSE
					BEGIN
						INSERT INTO @vals (ValueKey, ValueInt, ValueText)
						SELECT ValueKey, 0, CAST(ValueInt AS NVARCHAR(MAX))
						FROM planning.tfValues
						WHERE ValueSeriesKey = @ValueSeriesKey; 
					END;
				END;
			END;

			-- changing scaling
			-- If Isnumeric = 0 and not changed Nothing happens with the value, if scale is changed, as text values dont use the scale property.
			IF (@Scale <> N'<#NV>' AND @OldScale <> @Scale) AND (@IsNumeric = N'1' OR (@IsNumeric = N'<#NV>' AND @OldIsNumeric = 1))
			BEGIN
				-- Change Scale of existing values if scale is changed (when scale is changed, e.g. from 100 to 1000 the value in the database must be multiplied by 10)
				DECLARE @k FLOAT = @Scale / CAST(@OldScale AS FLOAT);	
			
				IF NOT EXISTS(SELECT 1 FROM @vals)
				BEGIN
					INSERT INTO @vals (ValueKey, ValueInt, ValueText)
					SELECT ValueKey, ValueInt * @k, '' 
					FROM planning.tfValues
					WHERE ValueSeriesKey = @ValueSeriesKey; 
				END
				ELSE
				BEGIN
					UPDATE @vals
					SET ValueInt = ValueInt * @k; 
				END;
			END;
			
			IF EXISTS(SELECT 1 FROM @vals)
			BEGIN
				UPDATE v
				SET 
					 v.ValueInt		= newV.ValueInt
					,v.ValueText	= newV.ValueText 
				FROM planning.tfValues v INNER JOIN @vals newV 
					ON v.ValueKey = newV.ValueKey
				WHERE v.ValueSeriesKey = @ValueSeriesKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;
		END;

		SET @Resultcode = 201;
		COMMIT TRANSACTION sx_pf_POST_ValueSerie;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ValueSerie;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ValueSerie TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ValueSerie'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for one ValueSeries
         Operation will create a ValueSeries if it not exist
         It will change an existing ValueSeries
         Works under the assumtion, that the ValueSeries ID is not changed ! The ID is used to identify the valueSeries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@RequestedValueSeriesNo';
    SET @value      = N'RequestedValueSeriesNo';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsNumeric';
    SET @value      = N'IsNumeric';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@VisibilityLevel';
    SET @value      = N'VisibilityLevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSource';
    SET @value      = N'ValueSource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueListID';
    SET @value      = N'ValueListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueFormatID';
    SET @value      = N'ValueFormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Effect';
    SET @value      = N'Effect';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EffectParameter';
    SET @value      = N'EffectParameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pPOST_API_LogEntry;
GO

/*
	Internal POST Operation for one API LogEntry, always used, when a public procedure is called
	Writes an LogEntry, as long as

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9

	Testcall
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pPOST_API_LogEntry 
			 'SQL'
			,'gerd.tautenhahn@saxess-software.com'
			,'calc.spTest'
			,'ThisAreTheParameter'
			,1
			,403
			,'20150731'
			,'Comment';
		PRINT @RESULT;

	Testcall table
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pPOST_API_LogEntry',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pPOST_API_LogEntry','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pPOST_API_LogEntry 
	 @Username			AS NVARCHAR(255)
	,@TransactUsername	AS NVARCHAR(255)
	,@ProcedureName		AS NVARCHAR(255)
	,@ParameterString	AS NVARCHAR (MAX)
	,@EffectedRows		AS INT
	,@ReturnCode		AS INT
	,@TimestampCall		AS Datetime
	,@Comment			AS NVARCHAR (2000)

WITH EXECUTE AS 'dbo' -- as a dbo procedure owned by OCTService writes into system table

AS

		DECLARE  @TimestampReturn		AS Datetime			= GETUTCDATE()
				,@ProcessCode			AS NVARCHAR(255)	= N''
				,@DatabaseIsReadOnly	AS INTEGER			= 0;
	
		SELECT @DatabaseIsReadOnly = is_read_only FROM sys.databases WHERE Name = DB_NAME();
		
		IF NOT @DatabaseIsReadOnly = 1
			BEGIN

				-- the functions CURRENT_User may not be avaiable for Azure in impersonation context (WITH EXECUTE AS OWNER...), depening on the object owner.
				BEGIN TRY
					INSERT INTO system.tAPI_Log VALUES (CURRENT_USER,@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
				END TRY
				BEGIN CATCH
				    INSERT INTO system.tAPI_Log VALUES ('Impersonated User as dbo',@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
				END CATCH
			END

GO

-- Transfered Ownership (procedure must be moved in v6 to system or planning schema)
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pPOST_API_LogEntry TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pPOST_API_LogEntry'		-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'	Internal POST Operation for one API LogEntry, always caused, when a public procedure is called';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TransactUsername';
    SET @value      = N'The name of the user for whom this procedure was executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcedureName';
    SET @value      = N'Name of the stored procedure which was executed';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @level2name = N'@ParameterString';
    SET @value      = N'List of parameters, which were given to the called procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@EffectedRows';
    SET @value      = N'Count of effected rows which (returnd from the called procedure). This number my include counts from temporary tables.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@ReturnCode';
    SET @value      = N'signals the result of the called procedure (sucessful, not found objects...)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@TimestampCall';
    SET @value      = N'Timestamp the procedure was called';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@Comment';
    SET @value      = N'Return messages from the SQL Server or from the procedure for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 	GO	
	

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ClusterPropertiesReadRight;
GO

/*
	GET Operation for read right on Clusterlevel
	Needs already resolved Transaktionsuser, therefore PRIVATE - call only in case of passing in a validated Transactusername !
	Returns in case of read rights for this user 200, else 401
	Read rights have all user which have got read rights in OCT Usermanagement or have the privileged OCT database roles (FactoryService, db_owner)
	This procedure is always called after determination of the @Transactusername over function dbo.sx_pf_Determine_Transactusername,
	therefore it is already ensured that
	 - the delivered username is an active user from OCT Usermangement or a user with privileged role
 
	The Procedure therefore always return 200, except the @Transactusername is '403', which is the return code in case of no access, 
	which was not handle als exception in the calling procedure

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pGET_ClusterPropertiesReadRight 'W10\admin'
		PRINT @RESULT

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterPropertiesReadRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterPropertiesReadRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ClusterPropertiesReadRight 
	@TransactionUsername AS NVARCHAR(255)
AS
	BEGIN

		IF @TransactionUsername = '403'		
			BEGIN
				RETURN 401 
			END
			ELSE
			BEGIN
				RETURN 200
			END
	END
GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ClusterPropertiesReadRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ClusterPropertiesReadRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'	GET Operation for read right on Clusterlevel
					Needs already resolved Transaktionsuser, therefore PRIVATE !
					Returns in case of read rights for this user 200, else 401
					Read rights have all user which have got read rights in OCT Usermanagement or have the privileged OCT database roles (FactoryService, db_owner)
					This procedure is always called after determination of the @Transactusername over function dbo.sx_pf_Determine_Transactusername,
					therefore it is already ensured that
					 - the delivered username is an active user from OCT Usermangement or a user with privileged role
 
					The Procedure therefore always return 200, except the @Transactusername is ''403'', which is the return code in case of no access, 
					which was not handle als exception in the calling procedure';

    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'TransactionUsername - the already als valid and granted name of an existing OCT User or a member of an priviledged role.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ClusterWriteRight;
GO

/*
	GET Operation for Write Rights on Cluster
	Needs already a validated Transactionsuser !
	Returns for OCT Clusteradmins and privileged roles 200

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	 Testcall
		DECLARE @Returncode INT
		EXEC @Returncode = dbo.sx_pf_pGET_ClusterWriteRight 'W10\admin'
		PRINT @Returncode

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterWriteRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ClusterWriteRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ClusterWriteRight 
	@TransactionUsername AS NVARCHAR(255)
AS
	BEGIN
		DECLARE @Right				NVARCHAR (255)	='none'
		DECLARE @IsPrivilegedUser	INT				= 0

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
			IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200
			END

		-- Check for Write Line on cluster
		SELECT @Right=COALESCE([Right],'no') FROM system.trUserRights WHERE FactoryID = '' AND Username = @TransactionUsername;

		IF @Right = 'Write' 
			BEGIN
				RETURN 200
			END
			ELSE
			BEGIN
				RETURN 401
			END
	END
GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ClusterWriteRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ClusterWriteRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value =	N'GET Operation for Write Rights on Cluster
					Needs already a validated Transactionsuser !
					Returns for OCT Clusteradmins and privileged role member 200';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Name of validated Transactionuser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_FactoryReadRight;
GO

/*
	GET Operation for read right on a Factory
	Needs an alreaday resolved Transactuser !
	Returns 200 in case of Read Rights. User with write rights are also returned with 200

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_pGET_FactoryReadRight
			 @TransactionUsername = 'W10\admin'
			,@FactoryID = 'ZT'
		PRINT @RC


	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_FactoryReadRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_FactoryReadRight','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_FactoryReadRight
	 @TransactionUsername		NVARCHAR(255)
	,@FactoryID					NVARCHAR(255)
AS
	BEGIN
		DECLARE  @Right				NVARCHAR(255)	= ''
				,@IsPrivilegedUser	INT				= 0;

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
		IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		--check for existance of Factory
		IF NOT EXISTS
			(
				SELECT FactoryID FROM planning.tdFactories WHERE FactoryID = @FactoryID
			)
			BEGIN
				RETURN 404;
			END

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200;
			END

		-- check for write or read Rights for Factory
		SELECT
			@Right=IIF ([Right] = 'Write' OR [Right] = 'Read','Read', '401')
		FROM system.trUserRights 
		WHERE 
				FactoryID = @FactoryID 
			AND ProductLineID = '' 
			AND Username = @TransactionUsername;

		IF @Right = 'Read' 
			BEGIN
				RETURN 200;
			END
			ELSE
			BEGIN
				RETURN 401;
			END
		
	END
GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_FactoryReadRight TO pf_PlanningFactoryService;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_FactoryReadRight'		-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value =	N'GET Operation for read right on a Factory
					Needs an alreaday resolved Transactuser !
					Returns 200 in case of Read Rights. User with write rights are also returned with 200';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Name of the alread resolved TransactionUser.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID for which read rights are requested.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_FactoryWriteRight;
GO

/*
	GET Operation for Write Rights on a Factory
	Needs already resolved Transactuser !
	Returns on success 200
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pGET_FactoryWriteRight 'W10\admin','ZT'
		PRINT @RESULT

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_FactoryWriteRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_FactoryWriteRight','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_FactoryWriteRight
			 @TransactionUsername	NVARCHAR(255)
			,@FactoryID				NVARCHAR(255)
AS
	BEGIN
		DECLARE  @Right				NVARCHAR(255)	= ''
				,@IsPrivilegedUser	INT				= 0;

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
			IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		--check for existance of Factory
		IF NOT EXISTS
			(
				SELECT FactoryID FROM planning.tdFactories WHERE FactoryID = @FactoryID
			)
			BEGIN
				RETURN 404;
			END

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200;
			END

		-- check for write or read Rights for Factory
		SELECT
			@Right=IIF ([Right] = 'Write','Write', '401')
		FROM system.trUserRights 
		WHERE 
				FactoryID = @FactoryID 
			AND ProductLineID = '' 
			AND Username = @TransactionUsername;

		IF @Right = 'Write' 
			BEGIN
				RETURN 200;
			END
			ELSE
			BEGIN
				RETURN 401;
			END
	END

GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_FactoryWriteRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_FactoryWriteRight'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value =	N'GET Operation for Write Rights on a Factory
					Needs already resolved Transactuser !
					Returns on success 200';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Username which is already resolved to Transactusername.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'ID of the requested Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ProductLineReadRight ;
GO

/*
	GET Operation for Read Rights on a ProductLine
	Needs already resolved Transactionuser !
	Returns on success 200 for Users with Read or Write Rights
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_pGET_ProductLineReadRight 
			 @TransactionUsername	= 'W10\admin'
			,@FactoryID				= 'ZT'
			,@ProductLineID			= 'U'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ProductLineReadRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ProductLineReadRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ProductLineReadRight 
	 @TransactionUsername	NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)

AS
	BEGIN
		DECLARE  @Right				NVARCHAR(255)	= ''
				,@IsPrivilegedUser	INT				= 0;

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
		IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		--check for existance of ProductLine
		IF NOT EXISTS
			(
				SELECT FactoryID, ProductLineID  FROM planning.tdProductLines WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID
			)
			BEGIN
				RETURN 404;
			END

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200;
			END

		-- check for write or read Rights for ProductLine
		SELECT
			@Right=IIF ([Right] = 'Write' OR [Right] = 'Read','Read', '401')
		FROM system.trUserRights 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID
			AND Username		= @TransactionUsername;

		IF @Right = 'Read' 
			BEGIN
				RETURN 200;
			END
			ELSE
			BEGIN
				RETURN 401;
			END
	END

GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ProductLineReadRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ProductLineReadRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Read Rights on a ProductLine
				Needs already resolved Transactionuser !
				Returns on success 200 for Users with Read or Write Rights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Username resolved to TransactionUsername.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'ID of the Factory where the ProductLine is located.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ID of the ProductLine where read rights are requested.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS dbo.sx_pf_pGET_ProductLineWriteRight ;
GO

/*
	GET Operation for Write Rights on ProductLine
	Needs already a Transactionsuser !
	Returns for User with Write Rights 200
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_pGET_ProductLineWriteRight 
			 @TransactionUsername	= 'W10\admin'
			,@FactoryID				= 'ZT'
			,@ProductLineID			= 'U'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ProductLineWriteRight ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pGET_ProductLineWriteRight ','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pGET_ProductLineWriteRight 
	 @TransactionUsername	NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)

AS
		DECLARE  @Right				NVARCHAR(255)	= ''
				,@IsPrivilegedUser	INT				= 0;

		-- 1. Determine if a privileged User is acting (doublehoppel ? only priviledged user are allowed to execute the transaction (and sysadmin)
		IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;

		--check for existance of ProductLine
		IF NOT EXISTS
			(
				SELECT FactoryID, ProductLineID  FROM planning.tdProductLines WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID
			)
			BEGIN
				RETURN 404;
			END

		-- always return 200 for privileged user which is acting for himself (dbo does not matter here, as for dbo the Transactusername is the the Original_Login())
		IF @IsPrivilegedUser = 1 AND @TransactionUsername = ORIGINAL_LOGIN()
			BEGIN
				RETURN 200;
			END

		-- check for write or read Rights for ProductLine
		SELECT
			@Right=IIF ([Right] = 'Write','Write', '401')
		FROM system.trUserRights 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID
			AND Username		= @TransactionUsername;

		IF @Right = 'Write' 
			BEGIN
				RETURN 200;
			END
			ELSE
			BEGIN
				RETURN 401;
			END

GO

ALTER AUTHORIZATION ON dbo.sx_pf_pGET_ProductLineWriteRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pGET_ProductLineWriteRight '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value =	N'GET Operation for Write Rights on ProductLine
					Needs already a Transactionsuser !
					Returns for User with Write Rights 200';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@TransactionUsername';
    SET @value      = N'Username resolved to TransactionUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'ID of the requested Factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ID of the requested ProductLine.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS global.spDELETE_Code;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		DECLARE @ReturnCode INT;
		EXEC @ReturnCode	= global.spDELETE_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST';
		
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Code',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Code','PARAMETER',NULL)

*/
 
CREATE PROCEDURE global.spDELETE_Code
	 @ModuleID						NVARCHAR(255)	
	,@LanguageID					NVARCHAR(255)	= 'DE'
	,@CODE							NVARCHAR(255)	= ''
	 
AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';
	IF @CODE			IS NULL SET @CODE			= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''	
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0
		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''',''' 
			+ @LanguageID		+ N''',''' 
			+ @CODE				+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spDELETE_Code;	

		-- Standard PreUser Procedure
		IF OBJECT_ID('global.spDELETE_Code_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spDELETE_Code_PreUser @ModuleID,@LanguageID,@CODE
			END
		SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
		
		-- Always capitalize code																	 		
		SET @CODE = UPPER(@CODE)
																									  		
		-- UPDATE or INSERT in target table															  
		IF EXISTS(
					SELECT TOP(1) 1 
					FROM global.tCodes
					WHERE 
							 ModuleID	= @ModuleID
						AND (LanguageID	= @LanguageID	OR @LanguageID = '')
						AND (Code		= @CODE			OR @CODE = '')
					)
			-- Existing CODE will be deleted
			BEGIN
				DELETE FROM [global].tCodes
				WHERE		ModuleID	= @ModuleID
						AND (LanguageID	= @LanguageID	OR @LanguageID = '')
						AND (Code		= @CODE			OR @CODE = '')

				 SET @EffectedRows = @@ROWCOUNT;
				 SET @Resultcode = 200;
			END
		ELSE
			-- Not existing Code will result in warning
			BEGIN
				SET @EffectedRows = 0;
				SET @Resultcode = 404;
			END

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spDELETE_Code_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spDELETE_Code_PostUser @ModuleID,@LanguageID,@CODE
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));
		
		COMMIT TRANSACTION spDELETE_Code;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spDELETE_Code;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_Code'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to DELETE Codes.
	Can be controled over parameter. ModuleID must be passed
	1. LanguageID = '''' -> deletes all Languages
	2. LanguageID = ''DE'' -> Deletes only German
	3. CODE = '''' -> deletes all Codes
	4. CODE = ''XX'' -> deletes only this code
	Pre / Post Extension possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the code you want to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'Language of the code you want to delete (ISO Format, like DE for Germany.) or empty for all Languages.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CODE';
    SET @value      = N'CODE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS global.spDELETE_Company;
GO

/*
	Procedure to delete a company
		- in Dimension global tCompanies
		- generic serch over tables with Column CompanyKey
		- TODO: in all Pipeline Steps
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = global.spDELETE_Company
				 @Username		= 'SQL'
				,@DataSourceID	= '0'
				,@CompanyID		= 'A'
		PRINT    @RC

	SELECT * FROM global.tCompanies;
	SELECT * FROM system.tDataSources;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Company',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spDELETE_Company','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spDELETE_Company
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50)
		,@CompanyID				NVARCHAR(50)
    )

WITH EXECUTE AS 'dbo'  -- to access tables in system schema

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username			,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID		,N'NULL')	+ N''','''
				+ ISNULL(@CompanyID			,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@Message			NVARCHAR(2000)	= N''
				,@Tablename			NVARCHAR(2000)  = N''
				,@SQL				NVARCHAR(4000)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID		= N'';
		IF @CompanyID				IS NULL SET @CompanyID			= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Company
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @DataSourceID <> '0' -- enable deletion of companies in the not existing Datasource 0 
					AND NOT EXISTS (
					SELECT DataSourceKey FROM system.tDatasources WHERE DataSourceKey = CAST(@DataSourceID AS INT)--temporary compatibility
					)
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('DatsourceID ',@DataSourceID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				IF NOT EXISTS (
					SELECT CompanyID FROM global.tCompanies WHERE DataSourceKey = CAST(@DataSourceID AS INT) AND CompanyID = @CompanyID
					) 
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('CompanyID ',@CompanyID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END
				

		
				-- Generic delete over dynamic SQL is possible in all OCT schema except "system" in default
				-- Generic delete is still done over CompanyKey
				-- GET all Non-Core Tables where the column CompanyKey is used (usually result schema) and delete the Values for this ComanpanyKey
				-- The Company Key is an combinded Key "DatasourceID | CompanyID"
				-- some are deleted the second time, as they have the Datasource Key also !!

				PRINT '';
				PRINT 'Starting generic Delete Process for all Tables outside system containing the Column "CompanyKey" which is a concated Key with the Datasource.';
				PRINT '#####################################################################################################';
				PRINT '';
				DECLARE TableCursor CURSOR FOR
					SELECT      
							CONCAT(infs.TABLE_SCHEMA COLLATE DATABASE_DEFAULT,N'.',infs.TABLE_NAME COLLATE DATABASE_DEFAULT)	AS Tablename

					FROM INFORMATION_SCHEMA.COLUMNS infs
						-- Determination Objekttype
						LEFT JOIN 
								(
								SELECT 
									 Name					COLLATE DATABASE_DEFAULT AS Objektname
									,SCHEMA_NAME(Schema_id) COLLATE DATABASE_DEFAULT AS Schemaname
									,type_desc				COLLATE DATABASE_DEFAULT AS Objekttyp
								FROM sys.objects
								) syo ON 
											syo.Schemaname = infs.TABLE_SCHEMA
										AND syo.Objektname = infs.TABLE_NAME

				WHERE 
							syo.Objekttyp		= N'USER_TABLE'
						AND infs.COLUMN_NAME	= N'CompanyKey'
						-- ToDo: Determination not a Core table should be moved to metadata 
						AND infs.TABLE_SCHEMA  <> N'system'
						AND CONCAT(infs.TABLE_SCHEMA,N'.',infs.TABLE_NAME) <> N'global.tCompanies';
				OPEN TableCursor;
				FETCH TableCursor
				INTO @Tablename;
					IF @@CURSOR_ROWS = 0
						BEGIN
							PRINT N'No Tables found for generic deletion over Column CompanyKey'
						END
				WHILE @@FETCH_STATUS = 0
					BEGIN
						-- clear variables
						SET @SQL = N'';
						SET @SQL = CONCAT(N'DELETE FROM ', @Tablename, N' WHERE CompanyKey LIKE ''', @DataSourceID,N'|',@CompanyID,''';') 


						PRINT CONCAT(N'Delete all Rows with DatasourceID ', @DataSourceID,' and CompanyID ',@CompanyID, N' in Table ',@Tablename)
						PRINT @SQL
						EXEC (@SQL);

						FETCH TableCursor
						INTO @Tablename;
					END;
				CLOSE TableCursor;
				DEALLOCATE TableCursor;

				PRINT ''
				PRINT 'End of generic delition'
				PRINT  '######################################################################'
				PRINT ''

				PRINT '';
				PRINT 'Starting Delete for Core Data'
				PRINT '#####################################################################################################';
				PRINT '';
								
				-- PIPELINES - Company selection is deleted in all steps, but not the steps itself

				-- system.tPipelineSteps  - TODO 
					-- Datasource must be extracted from JSON and filtered on
					-- Company ID must be extracted from different JSON field
			
	
				PRINT CONCAT(N'Starting delete',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				DELETE 
				FROM global.tCompanies
				WHERE
						DataSourceKey	= CAST(@DataSourceID AS INT)
					AND CompanyID		= @CompanyID;

				SET @EffectedRows = @@ROWCOUNT;
				SET @ResultCode = 200;


				PRINT CONCAT(N'Transaktion finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Company
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Company	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- NO GRANT - as EXECUTE AS 'dbo'

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Company'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a company - as company and all data which are accociated with this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'ID of the Datasource to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'ID of the company to delete';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP PROCEDURE IF EXISTS global.spGET_Codes;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcalls
		DECLARE @ReturnCode INT;

		-- this should return all
		EXEC @ReturnCode	= global.spGET_Codes
		PRINT @ReturnCode;
		
		-- this one ModuleID, all Languages
		EXEC @ReturnCode	= global.spGET_Codes
			 @ModuleID		= 'system'
		PRINT @ReturnCode;

		-- this one ModuleID, only DE
		EXEC @ReturnCode	= global.spGET_Codes
			  @ModuleID		= 'system'
			 ,@LanguageID	= 'DE'
		
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes;
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Codes',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Codes','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spGET_Codes
	  @ModuleID						NVARCHAR(255) = ''
	 ,@LanguageID					NVARCHAR(255) = ''

AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0

		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''','''
			+ @LanguageID		+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spGET_Codes;	


			-- Standard PreUser Procedure
			IF OBJECT_ID('global.spGET_Codes_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spGET_Codes_PreUser @ModuleID,@LanguageID
			END
			SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
			
			-- Main Query
			SELECT 
				*
			FROM global.tCodes
			WHERE 	(ModuleID	= @ModuleID		OR @ModuleID = '')
				AND (LanguageID = @LanguageID	OR @LanguageID = '')

			SET @EffectedRows = @@ROWCOUNT;
			SET @Resultcode = 200;

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spGET_Codes_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spGET_Codes_PostUser @ModuleID,@LanguageID
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));

		COMMIT TRANSACTION spGET_Codes;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spGET_Codes;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Codes'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to GET Codes. Parameters can be used to filter output. 
	1. ModuleID = '''' -> all Modules
	2. ModuleID = ''[XX]'' -> specific Module
	3. LanguageID = '''' all Languages
	4. LanguageID = ''[XX]'' -> specific Language
	Pre / Post Extensions possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the codes or empty for all Modules.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'ID of the Language which owns the codes or empty for all Languages.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS global.spGET_Companies;
GO

/*
	Procedure to get one or all companies of an Datasource or for all Datasources
	 - use CompanyID = '' to get all companies
	 - use DatasourceID = '' to get the companies of all Datasources

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = global.spGET_Companies
				 @Username		= 'SQL'
				,@DataSourceID	= ''
				,@CompanyID		= ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Companies',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Companies','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spGET_Companies
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50) = ''
		,@CompanyID				NVARCHAR(50) = ''
    )

WITH EXECUTE AS 'dbo'  --to access system tables

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username				,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID			,N'NULL')	+ N''','''
				+ ISNULL(@CompanyID				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@Message			NVARCHAR(2000);

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username			= N'';
		IF @DataSourceID	IS NULL SET @DataSourceID		= N'';
		IF @CompanyID		IS NULL SET @CompanyID			= N'';


		-- Content Protection for specific input parameters (e.g.IDs)
		SET @DataSourceID	= dbo.sx_pf_pProtectID	(@DataSourceID);
		SET @CompanyID		= dbo.sx_pf_pProtectString	(@CompanyID);


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of Datasource in case of specified
				IF @DataSourceID <> '' AND NOT EXISTS (
					SELECT DataSourceKey FROM system.tDatasources WHERE DataSourceKey = CAST(@DataSourceID AS INT) --temporary compatibility
					) AND @DataSourceID <> 0
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('DatsourceID ',@DataSourceID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				IF @CompanyID <> '' AND NOT EXISTS (
					SELECT CompanyID FROM global.tCompanies WHERE DataSourceKey = CAST(@DataSourceID AS INT) AND CompanyID = @CompanyID --temporary compatibility
					)
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('CompanyID ',@CompanyID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				-- Data Transaction
				PRINT CONCAT(N'Start reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					 CAST(DataSourceKey AS NVARCHAR(50)) AS DataSourceID --temporary compatibility
					,CompanyID
					,CompanyName
					,[Description]
					,FinancialYearBegin
					,ChartsOfAccountsID
					,AccountLength
					,Currency
					,CustomValuesJSON
					,CustomValuesJSONUser
					,OwnerCODE
				FROM global.tCompanies
				WHERE 
						(DataSourceKey	= CAST(@DataSourceID AS INT) OR @DataSourceID = '' ) --temporary compatibility
					AND (CompanyID		= @CompanyID OR @CompanyID = '');

				SET @EffectedRows = @@ROWCOUNT;
	
				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO


-- NO GRANT - as EXECUTE AS 'dbo'
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Companies'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all companies of a datasource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DatasourceID';
    SET @value      = N'optional - ID of an Datasource, send '''' to get all Datasources.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'optional - ID of an Company, send '''' to get all Companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS global.spPOST_Code;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		-- creates a Code in German and Englisch
		DECLARE @ReturnCode INT;
		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hallo deutscher Test';
		PRINT @ReturnCode;

		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'system'
			,@LanguageID	= 'EN'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hello englisch Test';
		PRINT @ReturnCode;

		EXEC @ReturnCode	= global.spPOST_Code
			 @ModuleID		= 'finance'
			,@LanguageID	= 'DE'
			,@CODE			= 'TEST'
			,@Scope			= 'default'
			,@Description	= 'Hello englisch Test';
		PRINT @ReturnCode;

		SELECT * FROM global.tCodes

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Code',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Code','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spPOST_Code
	 @ModuleID						NVARCHAR(255)
	,@LanguageID					NVARCHAR(255)	= 'DE'
	,@CODE							NVARCHAR(255) 
	,@Scope							NVARCHAR(255)	= 'default'
	,@Description					NVARCHAR(4000)	= ''

AS
BEGIN
	SET NOCOUNT ON;

	-- default NULL Protection
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @LanguageID		IS NULL SET @LanguageID		= N'';
	IF @CODE			IS NULL SET @CODE			= N'';
	IF @Scope			IS NULL SET @Scope			= N'';
	IF @Description		IS NULL SET @Description	= N'';

	-- Log Preparement
	DECLARE
		-- default
		 @Username				NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@TransactUsername		NVARCHAR(255)	= ORIGINAL_LOGIN()
		,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
		,@EffectedRows			INT				= 0				
		,@ResultCode			INT				= 501	
		,@TimestampCall			DATETIME		= GETUTCDATE()
		,@Comment				NVARCHAR(2000)	= N''	
		,@PreUserCalled			INT				= 0
		,@PostUserCalled		INT				= 0
		,@PreUserReturnCode		INT				= 0
		,@PostUserReturnCode	INT				= 0
		-- specific for this procedure
		,@ParameterString		NVARCHAR(MAX)	= N'''' 
			+ @ModuleID			+ N''',''' 
			+ @LanguageID		+ N''',''' 
			+ @CODE				+ N''',''' 
			+ @Scope			+ N''',''' 
			+ @Description		+ N'''';

	BEGIN TRY
		BEGIN TRANSACTION spPOST_Code;	

		-- Standard PreUser Procedure
		IF OBJECT_ID('global.spPOST_Code_PreUser', 'P') IS NOT NULL 
			BEGIN
				SET @PreUserCalled = 1
				EXEC @PreUserReturnCode = [global].spPOST_Code_PreUser @ModuleID,@LanguageID,@CODE,@Scope,@Description
			END
		SET @Comment = @Comment + 'PreUser:' + CAST(@PreUserCalled AS NCHAR(1)) + '|' + CAST(@PreUserReturnCode AS NVARCHAR(50));
		
		-- Always capitalize code																	 		
		SET @CODE = UPPER(@CODE)
																									  		
		-- UPDATE or INSERT in target table															  
		IF EXISTS(
					SELECT TOP(1) 1 
					FROM global.tCodes
					WHERE 
							ModuleID	= @ModuleID
						AND LanguageID	= @LanguageID
						AND Code		= @CODE
					)
			-- Existing CODE will be updated, not passed Values are keept
			BEGIN
				UPDATE [global].tCodes
				SET 
					 Scope = @Scope
					,[Description] = @Description

				WHERE		ModuleID	= @ModuleID
						AND LanguageID	= @LanguageID
						AND Code		= @CODE ;

				 SET @EffectedRows = @@ROWCOUNT;
				 SET @Resultcode = 200;
			END
		ELSE
			-- Not existing Property will be created, not passed Values with defaults
			BEGIN
				INSERT INTO [global].[tCodes]
				(
					  ModuleID
					 ,LanguageID
					 ,CODE
					 ,Scope
					 ,[Description]

				)
				VALUES 
				(
					 @ModuleID
					,@LanguageID
					,@CODE
					,@Scope
					,@Description
				);

				SET @EffectedRows = @@ROWCOUNT;
				SET @Resultcode = 201;
			END

			-- Standard PostUser Procedure
			IF OBJECT_ID('global.spPOST_Code_PostUser', 'P') IS NOT NULL 
			BEGIN
				SET @PostUserCalled = 1
				EXEC @PostUserReturnCode = [global].spPOST_Code_PostUser @ModuleID,@LanguageID,@CODE,@Scope,@Description
			END
			SET @Comment = @Comment + ' ' + 'PostUser:' + CAST(@PostUserCalled AS NCHAR(1)) + '|' + CAST(@PostUserReturnCode AS NVARCHAR(50));
		
		COMMIT TRANSACTION spPOST_Code;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE() + ' ' + @Comment;

		ROLLBACK TRANSACTION spPOST_Code;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'global'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Code'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST Codes. Pre / Post Extension possible.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the Module which owns the code. Use "system" to POST a code which is not module dependend.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LanguageID';
    SET @value      = N'Language of the code you want to create (ISO Format, like DE for Germany.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CODE';
    SET @value      = N'The code you want to POST.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scope';
    SET @value      = N'The scope of the code. Can group codes for specific uses. Use "default" if not needed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Description';
    SET @value      = N'Free Description of the code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS global.spPOST_Company;
GO

/*
	Procedure to POST a Company, it is called if
		- Companies are created by query the from a sources system
		- Company Properties are edited by the Company edit GUI
		- all optional Parameters are keept properties if skip during sended or sended as NULL
		- Companies for Datasource 0 can be created even that this datasource don't exists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = global.spPOST_Company
				 @Username				= 'SQL'
				,@DataSourceID			= '0'
				,@CompanyID				= 'A'
				,@CompanyName			= 'B GmbH'
				,@Description			= 'Das ist die B GmbH'
				,@FinancialYearBegin	= 1
				,@ChartsOfAccountsID	= 'SKR04'
				,@AccountLength			= 4
				,@Currency				= 'EUR'
				,@CustomValuesJSON		= ''
				,@CustomValuesJSONUser	= ''
				,@OwnerCODE				= 'INTEGRATION'

		PRINT    @RC

		SELECT * FROM global.tCompanies;
		SELECT * FROM system.tDataSources;

		DELETE FROM global.tCompanies WHERE CompanyID = 'A';


	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Company',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spPOST_Company','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spPOST_Company
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50)
		,@CompanyID				NVARCHAR(50)
		,@CompanyName			NVARCHAR(255)	= NULL
		,@Description			NVARCHAR(2000)	= NULL
		,@FinancialYearBegin	INT				= NULL
		,@ChartsOfAccountsID	NVARCHAR(50)	= NULL
		,@AccountLength			INT				= NULL
		,@Currency				NVARCHAR(50)	= NULL
		,@CustomValuesJSON		NVARCHAR(MAX)	= NULL
		,@CustomValuesJSONUser	NVARCHAR(MAX)	= NULL
		,@OwnerCODE				NVARCHAR(50)	= NULL
    )

WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@DataSourceID AS NVARCHAR(255))		,N'NULL')	+ N','''
				+ ISNULL(@CompanyID									,N'NULL')	+ N''','''
				+ ISNULL(@CompanyName								,N'NULL')	+ N''','''
				+ ISNULL(@Description								,N'NULL')	+ N''','
				+ ISNULL(CAST(@FinancialYearBegin AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@ChartsOfAccountsID						,N'NULL')	+ N','
				+ ISNULL(CAST(@AccountLength AS NVARCHAR(255))		,N'NULL')	+ N','''
				+ ISNULL(@Currency									,N'NULL')	+ N''','''
				+ ISNULL(@CustomValuesJSON							,N'NULL')	+ N''','''
				+ ISNULL(@CustomValuesJSONUser						,N'NULL')	+ N''','''
				+ ISNULL(@OwnerCODE									,N'NULL')	+ N''''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@Message			NVARCHAR(2000)	= N''
				,@RowCount			INT				= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID			= N'';
		IF @CompanyID				IS NULL SET @CompanyID				= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Company
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						SET @Message = CONCAT('Transaction user ',@Username,' don`t exists');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END;
				
				-- check existence of DataSource - the Datasource 0 is not checked, it is used to store companies without Datasource
				IF @DataSourceID NOT IN (SELECT DataSourceID FROM system.tDataSources)	AND @DataSourceID <> '0'
					BEGIN
						SET @ResultCode = 403;
						SET @Message = CONCAT('DataSourceID ',@DataSourceID,' don`t exists');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				-- Value Caching
				DROP TABLE IF EXISTS #Company;
				CREATE TABLE #Company
					(
						 DataSourceID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CompanyID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CompanyName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Description]					NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,FinancialYearBegin				INT											NOT NULL
						,ChartsOfAccountsID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,AccountLength					INT											NOT NULL
						,Currency						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CustomValuesJSON				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,CustomValuesJSONUser			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OwnerCODE						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					);

				INSERT INTO #Company
					SELECT
						 DataSourceKey
						,CompanyID
						,CompanyName
						,[Description]
						,FinancialYearBegin
						,ChartsOfAccountsID
						,AccountLength
						,Currency
						,CustomValuesJSON
						,CustomValuesJSONUser
						,OwnerCODE
					FROM global.tCompanies
					WHERE 
							DataSourceKey	= CAST(@DataSourceID  AS INT)  -- temporary Compatibiltity 
						AND CompanyID		= @CompanyID;
				

				-- insert or update Pipeline
				SELECT @RowCount = Count (*) FROM #Company; 
				IF @RowCount = 0
					BEGIN
						
						-- INSERT with default values if NULL
						INSERT INTO global.tCompanies
							(
								DataSourceKey
								,CompanyID
								,CompanyName
								,Description
								,FinancialYearBegin
								,ChartsOfAccountsID
								,AccountLength
								,Currency
								,CustomValuesJSON
								,CustomValuesJSONUser
								,OwnerCODE
							)
						VALUES
							(
								 CAST(@DataSourceID	AS INT) -- temporary Compatibiltity 
								,@CompanyID		
								,ISNULL(@CompanyName			,'<set CompanyName>')
								,ISNULL(@Description			,'')
								,ISNULL(@FinancialYearBegin		,1)
								,ISNULL(@ChartsOfAccountsID		,'')
								,ISNULL(@AccountLength			,4)
								,ISNULL(@Currency				,'EUR')
								,ISNULL(@CustomValuesJSON		,'')
								,ISNULL(@CustomValuesJSONUser	,'')
								,ISNULL(@OwnerCODE				,'INTEGRATION')
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						UPDATE C
								SET 
									 CompanyName			= ISNULL(@CompanyName			,CompanyName				)
									,[Description]			= ISNULL(@Description			,[Description]				)
									,FinancialYearBegin		= ISNULL(@FinancialYearBegin	,FinancialYearBegin			)
									,ChartsOfAccountsID		= ISNULL(@ChartsOfAccountsID	,ChartsOfAccountsID			)
									,AccountLength			= ISNULL(@AccountLength			,AccountLength				)
									,Currency				= ISNULL(@Currency				,Currency					)
									,CustomValuesJSON		= ISNULL(@CustomValuesJSON		,CustomValuesJSON			)
									,CustomValuesJSONUser	= ISNULL(@CustomValuesJSONUser	,CustomValuesJSONUser		)
									,OwnerCODE				= ISNULL(@OwnerCODE				,OwnerCODE					)
							FROM global.tCompanies C
							WHERE 
								C.DataSourceKey	= CAST(@DataSourceID AS INT)
							AND C.CompanyID		= @CompanyID;

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

			COMMIT TRANSACTION spPOST_Company
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Company	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- NO GRANT as EXECUTE as 'dbo'
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Company'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure save / edit Companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'Integer number of the DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'ID of the Company, must be unique in this DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyName';
    SET @value      = N'Name of this company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Description';
    SET @value      = N'Description of the company.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FinancialYearBegin';
    SET @value      = N'Month of the start of the financial year - e.g. 1 for January.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ChartsOfAccountsID';
    SET @value      = N'ID of the current Chart of Accounts, this ID must match the ID in Table global.tFIN_ChartsOfAccounts.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@AccountLength';
    SET @value      = N'Integer Value for the length of the AccountID - usually between 4 and 8.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Currency';
    SET @value      = N'ISO Code of the currency of the company, e.g. EUR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CustomValuesJSON';
    SET @value      = N'Custom field to store any informations as JSON Array.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CustomValuesJSONUser';
    SET @value      = N'Custom field to store any informations as JSON Array, may have rights difference for reading.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OwnerCode';
    SET @value      = N'CODE Word who the owner of this company is - INTEGRATION or PLANNING.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP PROCEDURE IF EXISTS import.spPOST_dFactories;
GO

/*
	Import all values from load.tdFactories with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_dFactories 'SQL','','';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dFactories',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dFactories','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dFactories
				(		 @Username AS NVARCHAR(255)
						,@FactoryID AS NVARCHAR(255)
						,@Source AS NVARCHAR(255)				
				)
AS

BEGIN
	SET NOCOUNT ON;

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall			DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode				INT				= 501;
	DECLARE @EffectedRows			INT				= 0;					
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username	, N'NULL')	+ N''',''' 
													+ ISNULL(@FactoryID	, N'NULL')	+ N''','''
													+ ISNULL(@Source	, N'NULL')	+ N'''';
	DECLARE @Comment				NVARCHAR(2000)	 = N'';					

	DECLARE @NameShort				NVARCHAR(255)
	DECLARE @NameLong				NVARCHAR(255)
	DECLARE @CommentUser			NVARCHAR(MAX)
	DECLARE @CommentDev				NVARCHAR(255)
	DECLARE @ResponsiblePerson		NVARCHAR(255)
	DECLARE @ImageName				NVARCHAR(255)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPOARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(		 FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL 
						,NameShort			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,NameLong			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentUser		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,CommentDev			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ResponsiblePerson	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
						,ImageName			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL	
				)
	INSERT INTO #tmp
		SELECT tdF.FactoryID,
			   tdF.NameShort,
			   tdF.NameLong,
			   tdF.CommentUser,
			   tdF.CommentDev,
			   tdF.ResponsiblePerson,
			   tdF.ImageName 
		FROM   load.tdFactories tdF
		WHERE	(@FactoryID = '' OR tdF.FactoryID = @FactoryID)
		AND		(@Source = '' OR tdF.Source = @Source) 

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT 
			 FactoryID			
			,NameShort			
			,NameLong			
			,CommentUser		
			,CommentDev			
			,ResponsiblePerson	
			,ImageName			
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Factory
				 @Username			--@Username
				,@FactoryID			--@FactoryID
				,@NameShort			--@NameShort
				,@NameLong			--@NameLong
				,@CommentUser		--@CommentUser
				,@CommentDev		--@CommentDev
				,@ResponsiblePerson	--@ResponsiblePerson
				,@ImageName			--@ImageName	

		Print @ResultCode
	FETCH MyCursor INTO @FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor
	
	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;
END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dFactories TO pf_PlanningFactoryUser;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dFactories'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tdFactories with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_dProductLines;
GO

/*
	Import all values from load.tdProductLines with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_dProductLines 'SQL','','','';
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProductLines',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProductLines','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dProductLines
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductLineID	AS NVARCHAR(255),
						@Source			AS NVARCHAR(255)		  
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall					DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode						INT				= 501;
	DECLARE @EffectedRows					INT				= 0;					
	DECLARE @TransactUsername				NVARCHAR(255)	= N'';
	DECLARE @ParameterString				NVARCHAR(MAX)	= N''''
															+ ISNULL(@Username		, N'NULL')	 + N''',''' 
															+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
															+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
															+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment						NVARCHAR(2000)	= N'';						
	DECLARE @NameShort						NVARCHAR(255)
	DECLARE @NameLong						NVARCHAR(255)
	DECLARE @CommentUser					NVARCHAR(MAX)
	DECLARE @CommentDev						NVARCHAR(MAX)
	DECLARE @ResponsiblePerson				NVARCHAR(255)
	DECLARE @ImageName						NVARCHAR(255)
	DECLARE @DefaultTemplate				NVARCHAR(255)
	DECLARE @GlobalAttributeSource1			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias1 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource2			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias2 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource3			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias3 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource4			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias4 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource5			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias5 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource6			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias6 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource7			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias7 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource8			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias8 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource9			NVARCHAR(255)
	DECLARE @GlobalAttributeAlias9 			NVARCHAR(255)
	DECLARE @GlobalAttributeSource10		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias10			NVARCHAR(255)
	DECLARE @GlobalAttributeSource11		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias11			NVARCHAR(255)
	DECLARE @GlobalAttributeSource12		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias12			NVARCHAR(255)
	DECLARE @GlobalAttributeSource13		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias13			NVARCHAR(255)
	DECLARE @GlobalAttributeSource14		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias14			NVARCHAR(255)
	DECLARE @GlobalAttributeSource15		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias15			NVARCHAR(255)
	DECLARE @GlobalAttributeSource16		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias16			NVARCHAR(255)
	DECLARE @GlobalAttributeSource17		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias17			NVARCHAR(255)
	DECLARE @GlobalAttributeSource18		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias18			NVARCHAR(255)
	DECLARE @GlobalAttributeSource19		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias19			NVARCHAR(255)
	DECLARE @GlobalAttributeSource20		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias20			NVARCHAR(255)
	DECLARE @GlobalAttributeSource21		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias21			NVARCHAR(255)
	DECLARE @GlobalAttributeSource22		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias22			NVARCHAR(255)
	DECLARE @GlobalAttributeSource23		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias23			NVARCHAR(255)
	DECLARE @GlobalAttributeSource24		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias24			NVARCHAR(255)
	DECLARE @GlobalAttributeSource25		NVARCHAR(255)
	DECLARE @GlobalAttributeAlias25			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
				(	ProductLineID			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,FactoryID				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameShort				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,NameLong				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentUser				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,CommentDev				NVARCHAR(MAX) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ResponsiblePerson		NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,ImageName				NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,DefaultTemplate			NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource1  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias1 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource2	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias2 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource3	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias3 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource4	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias4 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource5	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias5 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource6	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias6 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource7	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias7 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource8	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias8 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource9	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias9 	NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource10 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias10  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource11 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias11  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource12 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias12  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource13 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias13  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource14 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias14  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource15 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias15  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource16 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias16  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource17 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias17  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource18 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias18  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource19 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias19  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource20 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias20  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource21 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias21  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource22 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias22  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource23 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias23  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource24 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias24  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeSource25 NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   ,GlobalAttributeAlias25  NVARCHAR(255) COlLATE DATABASE_DEFAULT NOT NULL
				   )

	INSERT INTO #tmp
		SELECT   tdPL.ProductLineID			
				,tdPL.FactoryID				
				,tdPL.NameShort				
				,tdPL.NameLong				
				,tdPL.CommentUser			
				,tdPL.CommentDev			
				,tdPL.ResponsiblePerson		
				,tdPL.ImageName				
				,tdPL.DefaultTemplate		
				,tdPL.GlobalAttributeSource1
				,tdPL.GlobalAttributeAlias1 
				,tdPL.GlobalAttributeSource2
				,tdPL.GlobalAttributeAlias2 
				,tdPL.GlobalAttributeSource3
				,tdPL.GlobalAttributeAlias3 
				,tdPL.GlobalAttributeSource4
				,tdPL.GlobalAttributeAlias4 
				,tdPL.GlobalAttributeSource5
				,tdPL.GlobalAttributeAlias5 
				,tdPL.GlobalAttributeSource6
				,tdPL.GlobalAttributeAlias6 
				,tdPL.GlobalAttributeSource7
				,tdPL.GlobalAttributeAlias7 
				,tdPL.GlobalAttributeSource8
				,tdPL.GlobalAttributeAlias8 
				,tdPL.GlobalAttributeSource9
				,tdPL.GlobalAttributeAlias9 
				,tdPL.GlobalAttributeSource10
				,tdPL.GlobalAttributeAlias10 
				,tdPL.GlobalAttributeSource11
				,tdPL.GlobalAttributeAlias11 
				,tdPL.GlobalAttributeSource12
				,tdPL.GlobalAttributeAlias12 
				,tdPL.GlobalAttributeSource13
				,tdPL.GlobalAttributeAlias13 
				,tdPL.GlobalAttributeSource14
				,tdPL.GlobalAttributeAlias14 
				,tdPL.GlobalAttributeSource15
				,tdPL.GlobalAttributeAlias15 
				,tdPL.GlobalAttributeSource16
				,tdPL.GlobalAttributeAlias16 
				,tdPL.GlobalAttributeSource17
				,tdPL.GlobalAttributeAlias17 
				,tdPL.GlobalAttributeSource18
				,tdPL.GlobalAttributeAlias18 
				,tdPL.GlobalAttributeSource19
				,tdPL.GlobalAttributeAlias19 
				,tdPL.GlobalAttributeSource20
				,tdPL.GlobalAttributeAlias20 
				,tdPL.GlobalAttributeSource21
				,tdPL.GlobalAttributeAlias21 
				,tdPL.GlobalAttributeSource22
				,tdPL.GlobalAttributeAlias22 
				,tdPL.GlobalAttributeSource23
				,tdPL.GlobalAttributeAlias23 
				,tdPL.GlobalAttributeSource24
				,tdPL.GlobalAttributeAlias24 
				,tdPL.GlobalAttributeSource25
				,tdPL.GlobalAttributeAlias25  
		FROM	load.tdProductLines tdPL
		WHERE	(@FactoryID = ''		OR tdPL.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdPL.ProductLineID = @ProductLineID)
		 AND    (@Source = ''			OR tdPL.Source = @Source)
	
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ProductLine
				 @Username
				,@ProductLineID				
				,@FactoryID			
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ResponsiblePerson
				,@ImageName
				,@DefaultTemplate
				,@GlobalAttributeSource1  
				,@GlobalAttributeAlias1 
				,@GlobalAttributeSource2
				,@GlobalAttributeAlias2 
				,@GlobalAttributeSource3
				,@GlobalAttributeAlias3 
				,@GlobalAttributeSource4
				,@GlobalAttributeAlias4 
				,@GlobalAttributeSource5
				,@GlobalAttributeAlias5 
				,@GlobalAttributeSource6
				,@GlobalAttributeAlias6 
				,@GlobalAttributeSource7
				,@GlobalAttributeAlias7 
				,@GlobalAttributeSource8
				,@GlobalAttributeAlias8 
				,@GlobalAttributeSource9
				,@GlobalAttributeAlias9 
				,@GlobalAttributeSource10 
				,@GlobalAttributeAlias10  
				,@GlobalAttributeSource11 
				,@GlobalAttributeAlias11  
				,@GlobalAttributeSource12 
				,@GlobalAttributeAlias12  
				,@GlobalAttributeSource13 
				,@GlobalAttributeAlias13  
				,@GlobalAttributeSource14 
				,@GlobalAttributeAlias14  
				,@GlobalAttributeSource15 
				,@GlobalAttributeAlias15  
				,@GlobalAttributeSource16 
				,@GlobalAttributeAlias16  
				,@GlobalAttributeSource17 
				,@GlobalAttributeAlias17  
				,@GlobalAttributeSource18 
				,@GlobalAttributeAlias18  
				,@GlobalAttributeSource19 
				,@GlobalAttributeAlias19  
				,@GlobalAttributeSource20 
				,@GlobalAttributeAlias20  
				,@GlobalAttributeSource21 
				,@GlobalAttributeAlias21  
				,@GlobalAttributeSource22 
				,@GlobalAttributeAlias22  
				,@GlobalAttributeSource23 
				,@GlobalAttributeAlias23  
				,@GlobalAttributeSource24 
				,@GlobalAttributeAlias24  
				,@GlobalAttributeSource25 
				,@GlobalAttributeAlias25 

		PRINT @ResultCode
	FETCH MyCursor INTO  @ProductLineID,@FactoryID,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@DefaultTemplate,@GlobalAttributeSource1  ,@GlobalAttributeAlias1 ,@GlobalAttributeSource2,@GlobalAttributeAlias2 ,@GlobalAttributeSource3,@GlobalAttributeAlias3 ,@GlobalAttributeSource4,@GlobalAttributeAlias4 ,@GlobalAttributeSource5,@GlobalAttributeAlias5 ,@GlobalAttributeSource6,@GlobalAttributeAlias6 ,@GlobalAttributeSource7,@GlobalAttributeAlias7 ,@GlobalAttributeSource8,@GlobalAttributeAlias8 ,@GlobalAttributeSource9,@GlobalAttributeAlias9 ,@GlobalAttributeSource10 ,@GlobalAttributeAlias10  ,@GlobalAttributeSource11 ,@GlobalAttributeAlias11  ,@GlobalAttributeSource12 ,@GlobalAttributeAlias12  ,@GlobalAttributeSource13 ,@GlobalAttributeAlias13  ,@GlobalAttributeSource14 ,@GlobalAttributeAlias14  ,@GlobalAttributeSource15 ,@GlobalAttributeAlias15  ,@GlobalAttributeSource16 ,@GlobalAttributeAlias16  ,@GlobalAttributeSource17 ,@GlobalAttributeAlias17  ,@GlobalAttributeSource18 ,@GlobalAttributeAlias18  ,@GlobalAttributeSource19 ,@GlobalAttributeAlias19  ,@GlobalAttributeSource20 ,@GlobalAttributeAlias20  ,@GlobalAttributeSource21 ,@GlobalAttributeAlias21  ,@GlobalAttributeSource22 ,@GlobalAttributeAlias22  ,@GlobalAttributeSource23 ,@GlobalAttributeAlias23  ,@GlobalAttributeSource24 ,@GlobalAttributeAlias24  ,@GlobalAttributeSource25 ,@GlobalAttributeAlias25  
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProductLines TO pf_PlanningFactoryUser;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dProductLines'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tdProductLines with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_dProducts;
GO

/*
	Import all values from load.tdProducts with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_dProducts '','','','';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProducts',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dProducts','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dProducts
				(		 @Username		AS NVARCHAR(255)
						,@FactoryID		AS NVARCHAR(255)
						,@ProductLineID	AS NVARCHAR(255)
						,@ProductID		AS NVARCHAR(255)			
						,@Source		AS NVARCHAR(255)		
				)		  
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @TimeType					NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(MAX)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ResponsiblePerson			NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @Status						NVARCHAR(255)
	DECLARE @Template					NVARCHAR(255)
	DECLARE @TemplateVersion			NVARCHAR(255)
	DECLARE @GA1 						NVARCHAR(255)
	DECLARE @GA2 						NVARCHAR(255)
	DECLARE @GA3 						NVARCHAR(255)
	DECLARE @GA4 						NVARCHAR(255)
	DECLARE @GA5 						NVARCHAR(255)
	DECLARE @GA6 						NVARCHAR(255)
	DECLARE @GA7 						NVARCHAR(255)
	DECLARE @GA8 						NVARCHAR(255)
	DECLARE @GA9 						NVARCHAR(255)
	DECLARE @GA10						NVARCHAR(255)
	DECLARE @GA11						NVARCHAR(255)
	DECLARE @GA12						NVARCHAR(255)
	DECLARE @GA13						NVARCHAR(255)
	DECLARE @GA14						NVARCHAR(255)
	DECLARE @GA15						NVARCHAR(255)
	DECLARE @GA16						NVARCHAR(255)
	DECLARE @GA17						NVARCHAR(255)
	DECLARE @GA18						NVARCHAR(255)
	DECLARE @GA19						NVARCHAR(255)
	DECLARE @GA20						NVARCHAR(255)
	DECLARE @GA21						NVARCHAR(255)
	DECLARE @GA22						NVARCHAR(255)
	DECLARE @GA23						NVARCHAR(255)
	DECLARE @GA24						NVARCHAR(255)
	DECLARE @GA25						NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TimeType				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(MAX) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ResponsiblePerson		NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Status					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Template				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,TemplateVersion			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA1 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA2 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA3 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA4 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA5 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA6 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA7 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA8 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA9 					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA10					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA11					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA12					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA13					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA14					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA15					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA16					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA17					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA18					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA19					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA20					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA21					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA22					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA23					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA24					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,GA25					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL		
			 )

	INSERT INTO #tmp
		SELECT   tdP.ProductID
				,tdP.ProductLineID			
				,tdP.FactoryID
				,tdP.TimeType				
				,tdP.NameShort				
				,tdP.NameLong				
				,tdP.CommentUser			
				,tdP.CommentDev			
				,tdP.ResponsiblePerson		
				,tdP.ImageName
				,tdP.Status				
				,tdP.Template
				,tdP.TemplateVersion		
				,tdP.GlobalAttribute1 
				,tdP.GlobalAttribute2 
				,tdP.GlobalAttribute3 
				,tdP.GlobalAttribute4 
				,tdP.GlobalAttribute5 
				,tdP.GlobalAttribute6 
				,tdP.GlobalAttribute7 
				,tdP.GlobalAttribute8 
				,tdP.GlobalAttribute9 
				,tdP.GlobalAttribute10
				,tdP.GlobalAttribute11
				,tdP.GlobalAttribute12
				,tdP.GlobalAttribute13
				,tdP.GlobalAttribute14
				,tdP.GlobalAttribute15
				,tdP.GlobalAttribute16
				,tdP.GlobalAttribute17
				,tdP.GlobalAttribute18
				,tdP.GlobalAttribute19
				,tdP.GlobalAttribute20
				,tdP.GlobalAttribute21
				,tdP.GlobalAttribute22
				,tdP.GlobalAttribute23
				,tdP.GlobalAttribute24
				,tdP.GlobalAttribute25
		FROM	load.tdProducts tdP
		WHERE	(@FactoryID = ''		OR tdP.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tdP.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tdP.ProductID = @ProductID)
		 AND    (@Source = ''			OR tdP.Source = @Source)
	 
	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Product
				 @Username
				,@ProductID
				,@ProductLineID				
				,@FactoryID			
				,@TimeType			
				,@NameShort			
				,@NameLong			
				,@CommentUser		
				,@CommentDev			
				,@ResponsiblePerson	
				,@ImageName			
				,@Status				
				,@Template			
				,@TemplateVersion	
				,@GA1 				
				,@GA2 				
				,@GA3 				
				,@GA4 				
				,@GA5 				
				,@GA6 				
				,@GA7 				
				,@GA8 				
				,@GA9 				
				,@GA10				
				,@GA11				
				,@GA12				
				,@GA13				
				,@GA14				
				,@GA15				
				,@GA16				
				,@GA17				
				,@GA18				
				,@GA19				
				,@GA20				
				,@GA21				
				,@GA22				
				,@GA23				
				,@GA24				
				,@GA25	
					
		Print @ResultCode
	FETCH MyCursor INTO  @ProductID,@ProductLineID,@FactoryID,@TimeType,@NameShort,@NameLong,@CommentUser,@CommentDev,@ResponsiblePerson,@ImageName,@Status,@Template,@TemplateVersion,@GA1 ,@GA2 ,@GA3 ,@GA4 ,@GA5 ,@GA6 ,@GA7 ,@GA8 ,@GA9 ,@GA10,@GA11,@GA12,@GA13,@GA14,@GA15,@GA16,@GA17,@GA18,@GA19,@GA20,@GA21,@GA22,@GA23,@GA24,@GA25
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dProducts TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dProducts'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tdProducts with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_dTime;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Procedure imports values from Loadtable load.tdTimes, limited by parameter to given scope
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = import.spPOST_dTime
			 @Username		= 'SQL'
			,@FactoryID		= ''
			,@ProductLineID	= ''
			,@ProductID		= ''
			,@TimeID		= 0
			,@Source		= '';

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dTime',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dTime','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dTime
				(		 @Username		AS NVARCHAR(255)
						,@FactoryID		AS NVARCHAR(255)
						,@ProductLineID	AS NVARCHAR(255)
						,@ProductID		AS NVARCHAR(255)
						,@TimeID		AS INT
						,@Source		AS NVARCHAR(255)	
					)
		   
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username							, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
														+ ISNULL(@ProductID							, N'NULL')	 + N''','
														+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N','''
														+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';
	DECLARE @FormatID					NVARCHAR(255);


	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	DROP TABLE IF EXISTS #tmp;
	CREATE TABLE #tmp 
		(	 ProductID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,TimeID			INT										NOT NULL
			,FormatID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
     	);

	INSERT INTO #tmp
		SELECT   tt.ProductID		
				,tt.ProductLineID	
				,tt.FactoryID		
				,tt.TimeID		
				,tt.FormatID
		FROM	load.tdTime tt
		WHERE	(@FactoryID		= '' OR tt.FactoryID		= @FactoryID)
		 AND    (@ProductLineID = '' OR tt.ProductLineID	= @ProductLineID)
		 AND    (@ProductID		= '' OR tt.ProductID		= @ProductID)
		 AND    (@TimeID		= 0  OR tt.TimeID			= @TimeID)
		 AND    (@Source		= '' OR tt.Source			= @Source);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeID,@FormatID
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_TimeID
		                                              
				 @Username
				,@ProductID		
				,@ProductLineID	
				,@FactoryID		
				,@TimeID	
				,@FormatID;

										
		PRINT @ResultCode;
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@TimeID,@FormatID
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO

-- Granted to users, das usually executed over SSMS
GRANT EXECUTE ON OBJECT ::import.spPOST_dTime TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dTime'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tfValues with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'ScopeFilter for loading process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ScopeFilter for loading process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ScopeFilter for loading process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'ScopeFilter for loading process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_dValueSeries;
GO

/*
	Import ValueSeries from load.tdValueSeries with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_dValueSeries 'SQL','','','','','';
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dValueSeries',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_dValueSeries','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_dValueSeries
				(	 	 @Username			AS NVARCHAR(255)
						,@FactoryID			AS NVARCHAR(255)
						,@ProductLineID		AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID		AS NVARCHAR(255)
						,@Source			AS NVARCHAR(255)		
				)		   
AS

BEGIN	
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID		, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID	, N'NULL')	 + N''','''
														+ ISNULL(@ProductID		, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID	, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';
	DECLARE @RequestedValueSeriesNo		NVARCHAR(255)
	DECLARE @NameShort					NVARCHAR(255)
	DECLARE @NameLong					NVARCHAR(255)
	DECLARE @CommentUser				NVARCHAR(255)
	DECLARE @CommentDev					NVARCHAR(255)
	DECLARE @ImageName					NVARCHAR(255)
	DECLARE @IsNumeric					NVARCHAR(255)
	DECLARE @VisibilityLevel			NVARCHAR(255)
	DECLARE @ValueSource				NVARCHAR(255)
	DECLARE @ValueListID				NVARCHAR(255)
	DECLARE @ValueFormatID				NVARCHAR(255)
	DECLARE @Unit						NVARCHAR(255)
	DECLARE @Scale						NVARCHAR(255)
	DECLARE @Effect						NVARCHAR(255)
	DECLARE @EffectParameter			NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(		ProductID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ProductLineID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,FactoryID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSeriesID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,RequestedValueSeriesNo	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameShort				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,NameLong				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentUser				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,CommentDev				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ImageName				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,IsNumeric				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,VisibilityLevel			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueSource				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueListID				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,ValueFormatID			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Unit					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Scale					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,Effect					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			   ,EffectParameter			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
			 )

	INSERT INTO #tmp
		SELECT   tdVS.ProductID				
				,tdVS.ProductLineID			
				,tdVS.FactoryID				
				,tdVS.ValueSeriesID			
				,tdVS.ValueSeriesNo	
				,tdVS.NameShort				
				,tdVS.NameLong				
				,tdVS.CommentUser				
				,tdVS.CommentDev				
				,tdVS.ImageName				
				,tdVS.IsNumeric				
				,tdVS.VisibilityLevel			
				,tdVS.ValueSource				
				,tdVS.ValueListID				
				,tdVS.ValueFormatID			
				,tdVS.Unit					
				,tdVS.Scale					
				,tdVS.Effect					
				,tdVS.EffectParameter	
		FROM	load.tdValueSeries tdVS
		WHERE	(@FactoryID		= '' OR tdVS.FactoryID		= @FactoryID)
		 AND    (@ProductLineID = '' OR tdVS.ProductLineID	= @ProductLineID)
		 AND    (@ProductID		= '' OR tdVS.ProductID		= @ProductID)
		 AND    (@ValueSeriesID = '' OR tdVS.ValueSeriesID	= @ValueSeriesID)
		 AND	(@Source		= '' OR tdVS.Source			= @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter

	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_ValueSerie
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@ValueSeriesID
				,@RequestedValueSeriesNo
				,@NameShort
				,@NameLong
				,@CommentUser
				,@CommentDev
				,@ImageName
				,@IsNumeric
				,@VisibilityLevel
				,@ValueSource
				,@ValueListID
				,@ValueFormatID
				,@Unit
				,@Scale
				,@Effect
				,@EffectParameter				
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@RequestedValueSeriesNo,@NameShort,@NameLong,@CommentUser,@CommentDev,@ImageName,@IsNumeric,@VisibilityLevel,@ValueSource,@ValueListID,@ValueFormatID,@Unit,@Scale,@Effect,@EffectParameter
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;
-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_dValueSeries TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_dValueSeries'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import ValueSeries from load.tdValueSeries with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_fValues;
GO

/*
	Import all values from load.tfValues with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	10/2022 for OCT 5.8

	Testcall
		EXEC import.spPOST_fValues 'SQL','','','','',0,'';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValues','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_fValues
				(		@Username		AS NVARCHAR(255),
						@FactoryID		AS NVARCHAR(255),
						@ProductLineID	AS NVARCHAR(255),
						@ProductID		AS NVARCHAR(255),
						@ValueSeriesID	AS NVARCHAR(255),
						@TimeID			AS INT,
						@Source			AS NVARCHAR(255)		
				)
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username							, N'NULL')	 + N''',''' 
														+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
														+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
														+ ISNULL(@ProductID							, N'NULL')	 + N''','''
														+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
														+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
														+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';	
	DECLARE @ValueFormula				NVARCHAR(255)
	DECLARE @ValueINT					INT
	DECLARE @ValueText					NVARCHAR(MAX)
	DECLARE @ValueComment				NVARCHAR(255)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
		(	 ProductID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
			,TimeID			INT										NOT NULL
			,ValueFormula	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueINT		INT										NOT NULL
			,ValueText		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueComment	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL		
		)

	INSERT INTO #tmp
		SELECT   tfV.ProductID		
				,tfV.ProductLineID	
				,tfV.FactoryID		
				,tfV.ValueSeriesID	
				,tfV.TimeID			
				,tfV.ValueFormula	
				,tfV.ValueINT		
				,tfV.ValueText		
				,tfV.ValueComment			
		FROM	load.tfValues tfV
		WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
		 AND    (@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
		 AND    (@ProductID = ''		OR tfV.ProductID = @ProductID)
		 AND    (@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
		 AND    (@TimeID = 0			OR tfV.TimeID = @TimeID)
		 AND    (@Source = ''			OR tfV.Source = @Source)

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = dbo.sx_pf_POST_Value
				 @Username
				,@ProductID		
				,@ProductLineID	
				,@FactoryID		
				,@ValueSeriesID	
				,@TimeID			
				,@ValueFormula	
				,@ValueINT		
				,@ValueText		
				,@ValueComment	
										
		Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID,@ValueSeriesID,@TimeID,@ValueFormula,@ValueINT,@ValueText,@ValueComment
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_fValues TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_fValues'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import all values from load.tfValues with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_fValuesPDTV;
GO
/*
	Import values from load.tfValues with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9

	Testcall
		EXEC import.spPOST_fValuesPDTV 'SQL','','','','',0,'';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValuesPDTV',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_fValuesPDTV','PARAMETER',NULL)
*/

CREATE PROCEDURE import.spPOST_fValuesPDTV
						 @Username 			AS NVARCHAR(255)
						,@FactoryID 		AS NVARCHAR(255)
						,@ProductLineID 	AS NVARCHAR(255)
						,@ProductID			AS NVARCHAR(255)
						,@ValueSeriesID 	AS NVARCHAR(255)
						,@TimeID			AS INT = 0
						,@Source			AS NVARCHAR(255)		
				
AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall						DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName						NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode							INT				= 501;
	DECLARE @EffectedRows						INT				= 0;					
	DECLARE @TransactUsername					NVARCHAR(255)	= N'';
	DECLARE @ParameterString					NVARCHAR(MAX)	= N''''
																+ ISNULL(@Username							, N'NULL')	 + N''',''' 
																+ ISNULL(@FactoryID							, N'NULL')	 + N''','''
																+ ISNULL(@ProductLineID						, N'NULL')	 + N''','''
																+ ISNULL(@ProductID							, N'NULL')	 + N''','''
																+ ISNULL(@ValueSeriesID						, N'NULL')	 + N''','''
																+ ISNULL(TRY_CAST(@TimeID AS NVARCHAR(255))	, N'NULL')	 + N''','''
																+ ISNULL(@Source							, N'NULL')	 + N'''';
	DECLARE @Comment							NVARCHAR(2000) = N'';	
	DECLARE @IsIncrementalValuesFlag			INT;
	DECLARE	@ValuesInBracketsCommaSeparated 	NVARCHAR(MAX);
	DECLARE @ToDoList AS TABLE
		(	 ProductID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID 						NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,FactoryID 							NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
			,IsIncrementalValuesFlag 			INT				NOT NULL
			,ValuesInBracketsCommaSeparated 	NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NOT NULL
		);
				
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	-- Temporäre Tabelle als Datensammler
	DROP TABLE IF EXISTS #DATA;
	CREATE TABLE #DATA	
			(
				 ProductID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ProductLineID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,FactoryID			NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueSeriesID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				,TimeID				INT										NOT NULL
				,ValueFormula		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueINT			BIGINT									NOT NULL
				,ValueText			NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				,ValueComment		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
			);

	INSERT INTO #DATA
			SELECT   tfV.ProductID		
					,tfV.ProductLineID	
					,tfV.FactoryID		
					,tfV.ValueSeriesID	
					,tfV.TimeID			
					,tfV.ValueFormula	
					,tfV.ValueINT		
					,REPLACE(tfV.ValueText,'''','')
					,REPLACE(tfV.ValueComment,'''','')	
			FROM	load.tfValues tfV
			WHERE	(@FactoryID = ''		OR tfV.FactoryID = @FactoryID)
			  AND 	(@ProductLineID = ''	OR tfV.ProductLineID = @ProductLineID)
			  AND	(@ProductID = ''		OR tfV.ProductID = @ProductID)
			  AND	(@ValueSeriesID = ''	OR tfV.ValueSeriesID = @ValueSeriesID)
			  AND   (@TimeID = 0			OR tfV.TimeID = @TimeID)
			  AND   (@Source = ''			OR tfV.Source = @Source);


	-- Tabelle für String
	INSERT INTO @ToDoList
		SELECT	 d.ProductID
				,d.ProductLineId
				,d.FactoryID
				,1
				,(SELECT  N'[(''' 
					+ dxml.ValueSeriesID								+ N''','  
					+ CAST(dxml.TimeID AS NVARCHAR(255))				+ N',''' 
					+ dxml.ValueFormula									+ N''',''' 
					+ CAST(dxml.ValueInt AS NVARCHAR(255))				+ N''',''' 
					+ calc.sfReplaceNonXMLCharacters(dxml.ValueText)	+ N''',''' 
					+ calc.sfReplaceNonXMLCharacters(dxml.ValueComment) + N''')],'
				FROM #DATA dxml
				WHERE d.FactoryID = dxml.FactoryID AND d.ProductLineID = dxml.ProductLineID AND d.ProductID = dxml.ProductID
				FOR XML PATH(''), TYPE).value('.[1]', 'nvarchar(max)') AS ValuesInBracketsCommaSeparated
		FROM #DATA d
		GROUP BY   d.ProductID, d.ProductLineId, d.FactoryID;

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR STATIC FOR

		SELECT tdl.ProductID
			  ,tdl.ProductLineID
			  ,tdl.FactoryID
			  ,tdl.IsIncrementalValuesFlag
			  ,''+LEFT(tdl.ValuesInBracketsCommaSeparated,LEN(tdl.ValuesInBracketsCommaSeparated)-1)+''
		FROM @ToDoList tdl

	OPEN MyCursor
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = dbo.sx_pf_POST_ProductDataTableValues
			 @UserName
			,@ProductID		
			,@ProductLineID	
			,@FactoryID		
			,@IsIncrementalValuesFlag
			,@ValuesInBracketsCommaSeparated	
											
	Print @ResultCode
	FETCH MyCursor INTO @ProductID,@ProductLineID,@FactoryID, @IsIncrementalValuesFlag, @ValuesInBracketsCommaSeparated	
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_fValuesPDTV TO pf_PlanningFactoryUser;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_fValuesPDTV'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Import values from load.tfValues with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_rRights;
GO

/*
	Import rights from loadtable with specific load-procedure as source

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 07/2023 for OCT 5.9

	Testcall
		EXEC import.spPOST_rRights 'SQL', '','','';
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_rRights',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_rRights','PARAMETER',NULL)
*/
 
CREATE PROCEDURE import.spPOST_rRights
				(		 @Username		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Right			AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)			
AS

BEGIN
	SET NOCOUNT ON
		
	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
														+ ISNULL(@Username		, N'NULL')	 + N''',''' 
														+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
														+ ISNULL(@Right			, N'NULL')	 + N''','''
														+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000)	= N'';
	DECLARE @FactoryID					NVARCHAR(255)	= ''
	DECLARE @ProductLineID				NVARCHAR(255)	= ''
	DECLARE @ProductID					NVARCHAR(255)	= ''
	DECLARE	@ReadCommentMandatory		INT 			= ''
	DECLARE @WriteCommentMandatory		INT				= ''

	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
		CREATE TABLE #tmp 
						(	 UserName		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,FactoryID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductLineID   NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ProductID		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,[Right]		 NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
							,ReadCommentMandatory	INT								NOT NULL
							,WriteCommentMandatory	INT								NOT NULL
						)
		INSERT INTO #tmp
			SELECT   trR.UserName		 
					,trR.FactoryID		 
					,trR.ProductLineID  
					,trR.ProductID		 
					,trR.[Right]		 
					,trR.ReadCommentMandatory
					,trR.WriteCommentMandatory
			FROM	load.trRights trR
			WHERE	(@PostUserName = '' OR trR.UserName = @PostUserName)
			AND 	(@Right = ''		OR trR.[Right] = @Right)
			AND 	(@Source = ''		OR trR.Source = @Source)   
			AND		trR.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT * 
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	WHILE @@FETCH_STATUS = 0
	BEGIN
	EXECUTE @ResultCode = system.spPOST_Right
			 @Username
			,@PostUserName				
			,@FactoryID				
			,@ProductLineID						
			,@Right					
			,@ReadCommentMandatory 
			,@WriteCommentMandatory

	Print @ResultCode
	FETCH MyCursor INTO @PostUserName, @FactoryID, @ProductLineID, @ProductID, @Right, @ReadCommentMandatory, @WriteCommentMandatory
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rRights TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_rRights'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import rights from loadtable with specific load-procedure as source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PostUserName';
    SET @value      = N'PostUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Right';
    SET @value      = N'Right';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS import.spPOST_rUser;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	Testcall
		EXEC import.spPOST_rUser 'SQL','','';
  
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_rUser',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'import', 'PROCEDURE', 'spPOST_rUser','PARAMETER',NULL)
*/

CREATE PROCEDURE import.spPOST_rUser
			     (		 @UserName		AS NVARCHAR(255)
						,@PostUserName	AS NVARCHAR(255)
						,@Source		AS NVARCHAR(255)		 
				)

AS

BEGIN
	SET NOCOUNT ON

	-------------------------------------------------------------------------------------------------------------------
	-- ##### VARIABLES ###########
	DECLARE @TimestampCall				DATETIME		= GETUTCDATE();
	DECLARE @ProcedureName				NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ResultCode					INT				= 501;
	DECLARE @EffectedRows				INT				= 0;					
	DECLARE @TransactUsername			NVARCHAR(255)	= N'';
	DECLARE @ParameterString			NVARCHAR(MAX)	= N''''
													+ ISNULL(@Username		, N'NULL')	 + N''',''' 
													+ ISNULL(@PostUserName	, N'NULL')	 + N''','''
													+ ISNULL(@Source		, N'NULL')	 + N'''';
	DECLARE @Comment					NVARCHAR(2000) = N'';	
	DECLARE @PersonSurname				NVARCHAR(255)
	DECLARE @PersonFirstName			NVARCHAR(255)
	DECLARE @EMail						NVARCHAR(255)
	DECLARE @LDAPIP						NVARCHAR(255)
	DECLARE @Status						NVARCHAR(255)
	DECLARE @DataFlowRightsCODE			NVARCHAR(255)
	DECLARE @VisualizationRightsCODE	NVARCHAR(255)
	DECLARE @IsAdministratorFlag		BIT
	-------------------------------------------------------------------------------------------------------------------
	-- ##### DETERMINE TRANSACTION USER ###########
	SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

	-------------------------------------------------------------------------------------------------------------------
	-- ##### TEMPORARY TABLES ###########
	IF OBJECT_ID('tempdb..#tmp') IS NOT NULL DROP TABLE #tmp
	CREATE TABLE #tmp 
			(	 UserName					NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonSurname				NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,PersonFirstName			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,EMail						NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,LDAPIP						NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,Status						NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,DataFlowRightsCODE			NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,VisualizationRightsCODE	NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
				,IsAdministratorFlag		BIT NOT NULL
			)

	INSERT INTO #tmp
		SELECT   trU.UserName
				,trU.PersonSurname
				,trU.PersonFirstName
				,trU.EMail		
				,trU.LDAPIP		
				,trU.Status		
				,trU.DataFlowRightsCODE
				,trU.VisualizationRightsCODE
				,trU.IsAdministratorFlag
		FROM	load.trUser trU
		WHERE	(@PostUserName = '' OR trU.UserName = @PostUserName)
		  AND   (@Source = ''		OR trU.Source = @Source)
		  AND	trU.UserName <> @TransactUsername

	-------------------------------------------------------------------------------------------------------------------
	-- ##### POST ###########
	DECLARE MyCursor CURSOR FOR
		SELECT *
		FROM #tmp t
	OPEN MyCursor
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status,@DataFlowRightsCODE,@VisualizationRightsCODE,@IsAdministratorFlag
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXECUTE @ResultCode = system.spPOST_User
				 @UserName				
				,@PostUserName
				,@PersonSurname	
				,@PersonFirstName
				,@EMail			
				,@LDAPIP			
				,@Status	
				,@DataFlowRightsCODE
				,@VisualizationRightsCODE
				,@IsAdministratorFlag

		Print @ResultCode
	FETCH MyCursor INTO @PostUserName,@PersonSurname,@PersonFirstName,@EMail,@LDAPIP,@Status,@DataFlowRightsCODE,@VisualizationRightsCODE,@IsAdministratorFlag
	END
	CLOSE MyCursor
	DEALLOCATE MyCursor

	SET @ResultCode = 200;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

	RETURN @ResultCode;

END;

-------------------------------------------------------------------------------------------------------------------
GO
GRANT EXECUTE ON OBJECT ::import.spPOST_rUser TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'import'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_rUser'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Import User from load.trUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@UserName';
    SET @value      = N'UserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PostUserName';
    SET @value      = N'PostUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS integration.pGetLastDeltaValues;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'integration', 'PROCEDURE', 'pGetLastDeltaValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'integration', 'PROCEDURE', 'pGetLastDeltaValues','PARAMETER',NULL)
*/
  
CREATE PROCEDURE integration.pGetLastDeltaValues
    @DataSourceKey int
  , @CompanyID     nvarchar(50)
  , @Module        nvarchar(50)
  , @Username      nvarchar(255)
AS
BEGIN
    DECLARE @SQLCommand nvarchar(255);
    DECLARE @ResultCode int = 501;

    BEGIN TRY

        -- execute sync procedure for the selected solution
        IF EXISTS
            (
            SELECT *
            FROM sysobjects
            WHERE
                id = OBJECT_ID(N'integration.pGetDeltaValues' + @Module)
                AND OBJECTPROPERTY(id, N'IsProcedure') = 1
            )
        BEGIN
            SET @SQLCommand = N'integration.pGetDeltaValues' + @Module;
            EXEC @ResultCode = @SQLCommand @DataSourceKey, @CompanyID, @Username;
        END;
        ELSE
            -- if no pGetDeltaValues procedure exists for module, nothing is executed and ResultCode 404 is returned
            SET @ResultCode = 404;

    END TRY
    BEGIN CATCH

        BEGIN
            SET @ResultCode = 500;
        END;

    END CATCH;

    RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'integration'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGetLastDeltaValues'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to get last Delta Load Timestamp per Acounting Area';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@DataSourceKey';
    SET @value      = N'DataSourceKey';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'CompanyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spCOPY_Tab;
GO

/*
	Procedure to copy a Tab inside a object or to another target
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spCOPY_Tab
				 @Username					= 'W10\admin'
				,@SourceFactoryID			= 'ZT'
				,@SourceProductLineID		= 'U'
				,@SourceProductID			= ''
				,@SourceTabID				= '4'
				,@TargetFactoryID			= 'ZT'
				,@TargetProductLineID		= ''
				,@TargetProductID			= ''
				,@TargetTabID				= 'T48'
				,@TargetOrderIndex			= 1
				,@TargetTabName				= 'Test2';

		PRINT @RC;
		SELECT * FROM planning.tTabs ORDER BY FactoryID, ProductlineID, OrderIndex;

		TRUNCATE TABLE planning.tTabs;

		SELECT TOP 5 * FROM planning.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Tab','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spCOPY_Tab
         @Username					NVARCHAR(255)
		,@SourceFactoryID			NVARCHAR(255) = ''				
		,@SourceProductLineID		NVARCHAR(255) = ''
		,@SourceProductID			NVARCHAR(255) = ''
		,@SourceTabID				NVARCHAR(255)			-- not optional, needed in all levels
		,@TargetFactoryID			NVARCHAR(255) = ''				
		,@TargetProductLineID		NVARCHAR(255) = ''
		,@TargetProductID			NVARCHAR(255) = ''
		,@TargetTabID				NVARCHAR(255)			-- not optional, needed in all levels
		,@TargetOrderIndex			INT			  = 1
		,@TargetTabName				NVARCHAR(255)
   
AS 
	BEGIN
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username					,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceFactoryID			,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceProductLineID		,N'NULL')	+ N''',''' 
				+ ISNULL(@SourceProductID			,N'NULL')	+ N''','''
				+ ISNULL(@SourceTabID				,N'NULL')	+ N''','''
				+ ISNULL(@TargetFactoryID			,N'NULL')	+ N''',''' 
				+ ISNULL(@TargetProductLineID		,N'NULL')	+ N''',''' 
				+ ISNULL(@TargetProductID			,N'NULL')	+ N''','''
				+ ISNULL(@TargetTabID				,N'NULL')	+ N''','
				+ CAST(	
					ISNULL(@TargetOrderIndex 		,N'NULL')
					AS NVARCHAR(255))							+ N','''
				+ ISNULL(@TargetTabName				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- Procedure specific declarations
		DECLARE  @SourceFactoryKey		INT			= 0
				,@SourceProductLineKey	INT			= 0
				,@SourceProductKey		INT			= 0
				,@TargetFactoryKey		INT			= 0
				,@TargetProductLineKey	INT			= 0
				,@TargetProductKey		INT			= 0
				,@RowKey				BIGINT		= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
		IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
		IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
		IF @SourceTabID				IS NULL SET @SourceTabID			= N'';
		IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
		IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
		IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
		IF @TargetTabID				IS NULL SET @TargetTabID			= N'';
		IF @TargetOrderIndex		IS NULL SET @TargetOrderIndex		= 1;
		IF @TargetTabName			IS NULL SET	@TargetTabName			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID	(@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID	(@SourceProductLineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID	(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID	(@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID	(@TargetProductLineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID	(@TargetProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spCOPY_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @SourceFactoryID		= '' SET @SourceFactoryKey		= -1;
				IF @SourceProductLineID	= '' SET @SourceProductLineKey	= -1;
				IF @SourceProductID		= '' SET @SourceProductKey		= -1;
				IF @TargetFactoryID		= '' SET @TargetFactoryKey		= -1;
				IF @TargetProductLineID	= '' SET @TargetProductLineKey	= -1;
				IF @TargetProductID		= '' SET @TargetProductKey		= -1;

				SELECT @SourceFactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @SourceFactoryID													AND @SourceFactoryKey <> -1;
				SELECT @SourceProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @SourceFactoryKey		AND ProductLineID	= @SourceProductLineID	AND @SourceProductLineKey <> -1;
				SELECT @SourceProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @SourceProductLineKey	AND ProductID		= @SourceProductID		AND @SourceProductKey <> -1;
				SELECT @TargetFactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @TargetFactoryID													AND @TargetFactoryKey <> -1;
				SELECT @TargetProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @TargetFactoryKey		AND ProductLineID	= @TargetProductLineID	AND @TargetProductLineKey <> -1;
				SELECT @TargetProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @TargetProductLineKey	AND ProductID		= @TargetProductID		AND @TargetProductKey <> -1;

				-- error if an ID dont exists
				IF		@SourceFactoryKey		= 0 
					OR	@SourceProductLineKey	= 0
					OR  @SourceProductKey		= 0
					OR	@TargetFactoryKey		= 0 
					OR	@TargetProductLineKey	= 0
					OR  @TargetProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if SourceTabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @SourceFactoryKey
									AND ProductLineKey	= @SourceProductLineKey
									AND ProductKey		= @SourceProductKey
									AND TabID			= @SourceTabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('Source TabID not found.', 16, 10);
							END;

				-- tab with ID 0 cant be moved
				IF @SourceTabID= '0' OR @TargetTabID = '0'
					BEGIN
							SET @ResultCode = 403;
							RAISERROR('Tab with ID 0 can''t be copied', 16, 10);
					END

				-- error if ID have gaps in the line
				IF		@SourceFactoryKey		= -1 AND (@SourceProductLineKey <> -1 OR @SourceProductKey <> -1)
					OR	@SourceProductLineKey	= -1 AND  @SourceProductKey <> -1 
					OR	@TargetFactoryKey		= -1 AND (@TargetProductLineKey <> -1 OR @TargetProductKey <> -1)
					OR	@TargetProductLineKey	= -1 AND  @TargetProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level on Source
				IF @SourceFactoryKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster read.', 16, 10);
								END 
					END

				-- check user rights for Factory level on Source
				IF  @SourceFactoryKey <> -1 AND @SourceProductLineKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername,@SourceFactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory read.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level on Source
				IF @SourceFactoryKey <> -1 AND @SourceProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername,@SourceFactoryID,@SourceProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product read.', 16, 10);
								END 
					END

				-- check user rights for cluster level on Target
				IF @TargetFactoryKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level on Target
				IF @TargetFactoryKey <> -1 AND @TargetProductLineKey = -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername,@TargetFactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level on Target
				IF @TargetFactoryKey <> -1 AND @TargetProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactUsername,@TargetFactoryID,@TargetProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- determine if posted Tab already exists
				SELECT 
					 @Rowkey = RowKey
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @TargetFactoryKey
					AND ProductLineKey	= @TargetProductLineKey
					AND ProductKey		= @TargetProductKey
					AND TabID			= @TargetTabID

				-- data operation starts	***********************************************************************************
				-- delete if exits, to determine Resultcode
				IF @RowKey > 0 
					BEGIN
						BEGIN
							SET @ResultCode = 401
							RAISERROR('Target TabID already exists in Target.', 16, 10);
						END 
					END

				-- Insert new Tab
				INSERT INTO planning.tTabs
					SELECT
						 @TargetFactoryKey
						,@TargetProductLineKey
						,@TargetProductKey
						,@TargetFactoryID
						,@TargetProductLineID
						,@TargetProductID
						,@TargetTabID
						,@TargetOrderIndex   -- TODO: Must be set to right index
						,@TargetTabName
						,TabHint
						,TabText
						,PresentationCODE
						,Datasource
						,IsVisibleBOOL
						,LayoutJSON
						,ParameterJSON

					FROM planning.tTabs

					WHERE 
							FactoryKey		= @SourceFactoryKey
						AND ProductLineKey	= @SourceProductLineKey
						AND ProductKey		= @SourceProductKey
						AND TabID			= @SourceTabID

					-- Increment all Tabs after the inserted Tab and the Tab with the same OrderIndex
					UPDATE planning.tTabs
					SET OrderIndex = Orderindex + 1
					WHERE 
							FactoryKey		= @TargetFactoryKey
						AND ProductLineKey	= @TargetProductLineKey
						AND ProductKey		= @TargetProductKey
						AND Orderindex		>= @TargetOrderIndex
						AND TabID			<> @TargetTabID;

					-- eliminate all Gaps in Tab OrderIndex
					UPDATE planning.tTabs 
					SET OrderIndex = Rownumbers.Number
					FROM planning.tTabs tT INNER JOIN (
																SELECT 
																	 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																	,TabID
																	
																FROM planning.tTabs
																WHERE 
																		FactoryKey		= @TargetFactoryKey
																	AND ProductLineKey	= @TargetProductLineKey
																	AND ProductKey		= @TargetProductKey

														) AS Rownumbers
					ON 
							tT.TabID			= Rownumbers.TabID
						AND tT.FactoryKey		= @TargetFactoryKey
						AND tT.ProductLineKey	= @TargetProductLineKey
						AND tT.ProductKey		= @TargetProductKey;

				SET @ResultCode = 201;

			COMMIT TRANSACTION spCOPY_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spCOPY_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spCOPY_Tab'		-- enter procedure name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Copy a Tab - inside an object or to a different. The copied tab is always puted at the end.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceTabID';
    SET @value      = N'SourceBusiness Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetTabID';
    SET @value      = N'TargetBusiness Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spDELETE_File;
GO

/*
	Procedure to delete a single file or all files in a path

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9

	Testcall
		EXEC planning.spDELETE_File 
			'SQL'
			,''
			,''
			,''
			,''  --FileID
			,'Rootfolder'  --Filepath
			;

		SELECT * FROM planning.tFiles;
		SELECT * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_File',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_File','PARAMETER',NULL)
*/

CREATE PROCEDURE planning.spDELETE_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
	,@Filepath		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username		,N'NULL')	+ N''','''
														+ ISNULL(@FactoryID		,N'NULL')	+ N''','''
														+ ISNULL(@ProductLineID	,N'NULL')	+ N''','''
														+ ISNULL(@ProductID		,N'NULL')	+ N''','''
														+ ISNULL(@FileID		,N'NULL')	+ N''','''
														+ ISNULL(@Filepath		,N'NULL')	+ N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY
			BEGIN TRANSACTION DELETE_File;
			
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			IF @Filepath = N'' AND @FileID  = N'' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Filepath or FileID must be specified', 100;
			END;

			-- check if file exists in case of single file delete
			IF @FileID <> N'' AND NOT EXISTS (
				SELECT TOP (1) 1 
				FROM planning.tFiles
				WHERE 
						FactoryID		= @FactoryID
					AND ProductLineID	= @ProductLineID
					AND ProductID		= @ProductID
					AND FileID			= @FileID
					)
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			DELETE FROM planning.tFiles
			WHERE 
					FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID
				AND ProductID		= @ProductID
				AND (FileID			= @FileID OR @FileID = '')
				AND (Filepath		LIKE @Filepath + N'%' OR @Filepath = '');
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION DELETE_File;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION DELETE_File;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_File'					-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to delete exists file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileID';
    SET @value      = N'optional (not needed if Filepath is set) - FileID to specify the the file to be deleted';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@Filepath';
    SET @value      = N'optional - Path or Pathsegment to delete all file in a path';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spDELETE_Tab;
GO

/*
	Procedure to delete a Tab
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spDELETE_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= 'T7'

		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex

		TRUNCATE TABLE planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spDELETE_Tab','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spDELETE_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255)	
    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Tab;
				IF	@TabID	= '0' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Tab with ID 0 can''t be deleted.', 16, 10);
					END;

				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;

				--PRINT @FactoryKey 
				--PRINT @ProductLineKey
				--PRINT @ProductKey

				-- error if an ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if TabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey							-- its not possible to assign this rowkey to @Rowkey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @TabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('TabID not found.', 16, 10);
							END;
				
				-- RowKey ermitteln
				SELECT 
						@RowKey=RowKey
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- error if ID have gaps in the line
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <>  -1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- data operation starts	***********************************************************************************

				-- Insert new Tab
				DELETE FROM planning.tTabs
				WHERE
					RowKey		= @RowKey;

				-- eliminate all Gaps
				UPDATE planning.tTabs 
				SET OrderIndex = Rownumbers.Number
				FROM planning.tTabs tT INNER JOIN (
															SELECT 
																 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																,TabID
																	
															FROM planning.tTabs
															WHERE 
																	FactoryKey		= @FactoryKey
																AND ProductLineKey	= @ProductLineKey
																AND ProductKey		= @ProductKey

													) AS Rownumbers
				ON 
						tT.TabID			= Rownumbers.TabID
					AND tT.FactoryKey		= @FactoryKey
					AND tT.ProductLineKey	= @ProductLineKey
					AND tT.ProductKey		= @ProductKey;

				SET @ResultCode = 200;

			COMMIT TRANSACTION spDELETE_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Tab'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name; 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if DELETE is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if DELETE is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if DELETE is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'TabID which is deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spEXPORT_Format;
GO

/*
	Procedure to create an OCT Importscript for an format

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = planning.spEXPORT_Format
				 @Username		= 'SQL'
				,@FormatID		= 'EingabeDatum';

		PRINT    @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_Format',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_Format','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spEXPORT_Format
          @Username		NVARCHAR(255)
		 ,@FormatID		NVARCHAR(50)	

WITH EXECUTE AS 'dbo'  -- as the Procedure must access the system schema

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username						,N'NULL')	+ N''','
				+ ISNULL(@FormatID 						,N'NULL')	+ N''''			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @FormatID			IS NULL SET @FormatID			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary table to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Format",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 6, CONCAT(N'-- "SourceFormatID": "',@FormatID,'"'))
						,(0, 7, N'-- }');

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @FormatID			NVARCHAR(255)		= ''' + @FormatID + N'''')
						,(10, 4, N'--This Format will be deleted during import, if it exists. You should be sure !');
			
				
				-- Try_Delete existing Format 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC dbo.sx_pf_DELETE_Format '
							,'''SQL'''
							,',@FormatID'
							,';'
							)	AS Command
					FROM planning.thFormats
					WHERE 
						FormatID	= @FormatID;

				-- Create the Format as new Format
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC dbo.sx_pf_POST_Format '
							,'''SQL'''
							,',@FormatID'
							,','''	,BackgroundColor		,''''
							,','''	,FontColor				,''''
							,','''	,ValueFormat			,''''
							,','''	,Bold					,''''
							,','''	,Italic					,''''
							,','''	,AlignmentHorizontal	,''''
							,','''	,AlignmentVertical		,''''
							,','''	,TextUnderlined			,''''
							,','''	,BorderToNext			,''''
							,','''	,BorderToPrevious		,''''
							,';'
	
							)	AS Command
					FROM planning.thFormats
					WHERE
						FormatID			= @FormatID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');
		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Format'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a Format in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation
    SET @level2name = N'@FormatID';
    SET @value      = N'ID of the Format to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS planning.spEXPORT_List;
GO

/*
	Procedure to create an OCT Importscript for a list with its values

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = planning.spEXPORT_List
				 @Username		= 'SQL'
				,@ListID		= 'suKontenfunktion';

		PRINT    @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_List','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spEXPORT_List
          @Username		NVARCHAR(255)
		 ,@ListID		NVARCHAR(50)	

WITH EXECUTE AS 'dbo'  -- as the Procedure must access the system schema

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username						,N'NULL')	+ N''','
				+ ISNULL(@ListID 						,N'NULL')	+ N''''			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @ListID				IS NULL SET @ListID			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary table to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "List",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 6, CONCAT(N'-- "SourceListID": "',@ListID,'"'))
						,(0, 7, N'-- }');

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @ListID			NVARCHAR(255)		= ''' + @ListID + N'''')
						,(10, 4, N'--This List will be deleted during import, if it exists. You should be sure !');
			
				
				-- Try_Delete existing Pipeline 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC dbo.sx_pf_DELETE_List '
							,'''SQL'''
							,',@ListID'
							,';'
							)	AS Command
					FROM planning.thLists
					WHERE 
						ListID	= @ListID;

				-- Create the List as new List
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC dbo.sx_pf_POST_List '
							,'''SQL'''
							,',@ListID'
							,','''	,NameShort			,''''
							,','''	,NameLong			,''''
							,','''	,CommentDev			,''''
							,','''	,CommentUser		,''''
							,','''	,Datentyp			,''''
							,','''	,Source				,''''
							,','''	,SourceFormula		,''''
							,','''	,FormatID			,''''
							,';'
	
							)	AS Command
					FROM planning.thLists
					WHERE
						ListID			= @ListID;

				-- List Values
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,2000	AS SubOrderNumber
						,CONCAT(
							'EXEC dbo.sx_pf_POST_ListValue '
							,'''SQL'''
							,','	,0		,''
							,',@ListID'
							,','	,ValueInt			,''	
							,','	,Scale				,''
							,','''	,ValueText			,''''
							,','''	,ValueComment		,''''
							,','''	,FormatID			,''''
							,';'
	
							)	AS Command
					FROM planning.thListValues
					WHERE
						ListID			= @ListID;


				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');
		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_List'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a List in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ListID';
    SET @value      = N'ID of the List to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS planning.spEXPORT_Tab;
GO

/*
	Procedure to create an OCT Importscript for an selected Tab 

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = planning.spEXPORT_Tab
				 @Username		= 'SQL'
				,@FactoryID		= 'MM22'
				,@ProductLineID = ''
				,@ProductID		= ''
				,@TabID			= 'GUV'

		PRINT    @RC

	Testing in SSMS is only limited possible - Tabs which contains images result in very large strings which can handled only in bcp -> see planning.spExport_Setting for more informations

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spEXPORT_Tab','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spEXPORT_Tab
          @Username			NVARCHAR(255)
		 ,@FactoryID		NVARCHAR(50)
		 ,@ProductLineID	NVARCHAR(50)
		 ,@ProductID		NVARCHAR(50)
		 ,@TabID			NVARCHAR(50)	
    
WITH EXECUTE AS 'dbo'  -- as the Procedure must access the system schem

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username						,N'NULL')	+ N''','
				+ ISNULL(@FactoryID						,N'NULL')	+ N''','
				+ ISNULL(@ProductLineID					,N'NULL')	+ N''','
				+ ISNULL(@ProductID						,N'NULL')	+ N''','
				+ ISNULL(@TabID 						,N'NULL')	+ N''''			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @FactoryID			IS NULL SET @FactoryID		= N'';
		IF @TabID				IS NULL SET @TabID			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary table to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Tab",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 5, CONCAT(N'-- "SourceFactoryID": "',@FactoryID,'",'))
						,(0, 5, CONCAT(N'-- "SourceProductLineID": "',@ProductLineID,'",'))
						,(0, 5, CONCAT(N'-- "SourceProductID": "',@ProductID,'",'))
						,(0, 6, CONCAT(N'-- "SourceTabID": "',@TabID,'"'))
						,(0, 7, N'-- }')

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @FactoryID		NVARCHAR(255)		= ''' + @FactoryID + N'''')
						,(10, 3, N'DECLARE @ProductLineID	NVARCHAR(255)		= ''' + @ProductLineID + N'''')
						,(10, 3, N'DECLARE @ProductID		NVARCHAR(255)		= ''' + @ProductID + N'''')
						,(10, 3, N'DECLARE @TabID			NVARCHAR(255)		= ''' + @TabID + N'''')
						,(10, 4, N'--This Tab will be deleted during import, if it exists. You should be sure !');
				
				-- Try_Delete existing Tab 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC planning.spDELETE_Tab '
							,'''SQL'''
							,',@FactoryID'
							,',@ProductLineID'
							,',@ProductID'
							,',@TabID'
							,';'
							)	AS Command
					FROM planning.tTabs
					WHERE 
							FactoryID		= @FactoryID 
						AND ProductLineID	= @ProductLineID
						AND ProductID		= @ProductID
						AND TabID			= @TabID;

				-- Create the Tab as new Tab
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC planning.spPOST_Tab '
							,'''SQL'''
							,',@FactoryID'
							,',@ProductLineID'
							,',@ProductID'
							,',@TabID'
							,','	,Orderindex			,''	
							,','''	,Tabname			,''''
							,','''	,TabHint			,''''
							,','''	,TabText			,''''
							,','''	,PresentationCODE	,''''
							,','''	,Datasource			,''''
							,','''	,IsVisibleBOOL		,''''
							,','''	,LayoutJSON			,''''
							,','''	,ParameterJSON		,''''
	
							)	AS Command
					FROM planning.tTabs
					WHERE
							FactoryID		= @FactoryID 
						AND ProductLineID	= @ProductLineID
						AND ProductID		= @ProductID
						AND TabID			= @TabID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');
		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Tab'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a Tab in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@TabID';
    SET @value      = N'ID of the Tab to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS planning.spGET_EnduserObjects;
GO

/*
	Procedure to deliver SQL Object suggestions to the enduser.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = planning.spGET_EnduserObjects
					 @Username = 'SQL'
					,@ObjectType = 'P'
					,@ShowAllBool = 1;
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_EnduserObjects',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_EnduserObjects','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spGET_EnduserObjects
    (
         @Username		NVARCHAR(255)
		,@ObjectType	NVARCHAR(255)		= 'P'	
        ,@ShowAllBool	INT					= 0
    )
AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username		,N'NULL')	+ N''','''											 
				+ ISNULL(@ObjectType	,N'NULL')	+ N''','
				+ ISNULL(CAST(@ShowAllBool AS NVARCHAR(255)),N'NULL')	+ N''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username		= N'';
		IF @ObjectType				IS NULL SET @ObjectType		= N'';
		IF @ShowAllBool				IS NULL SET @ShowAllBool	= 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spGET_EnduserObjects;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				;WITH prms AS (
					SELECT DISTINCT object_id FROM (
						SELECT p1.object_id, p2.object_id other_id
						FROM [sys].[all_parameters] p1 
							LEFT JOIN [sys].[all_parameters] p2 ON p1.[object_id] = p2.[object_id] 
								AND p2.[name] NOT IN ('@Username', '@FactoryID', '@ProductLineID', '@ProductID', '@ParameterJSON')
						WHERE p1.[name] = '@Username'
					) all_p 
					WHERE other_id is null
				)			
				SELECT 
					CASE o.type
						WHEN 'P'	THEN 'Stored Procedures'
						WHEN 'U'	THEN 'Tables'
						WHEN 'V'	THEN 'Views'
						WHEN 'IF'	THEN 'Functions (Table Value Inline)'
						WHEN 'FN'	THEN 'Functions (Scalar)'
						WHEN 'TF'	THEN 'Functions (Table Value)'
						WHEN 'TR'	THEN 'Trigger'
						ELSE 'Unknown Objecttype'
						END									AS ObjectTypeName
					,o.type									AS ObjectType
					,S.name	+'.' + O.name					AS ObjectName
					,CASE
						WHEN sxo.SX_UserHint IS NOT NULL AND sxo.SX_UserHint <> '' THEN sxo.SX_UserHint
						WHEN ep.value = ''		THEN	'Description not defined'
						WHEN ep.value IS NULL	THEN	'<description defintion tag not definded>'
						WHEN ep.value <> ''		THEN	ep.value
						ELSE							'<procedure failure - this should not had happen>'
						END						AS Description

				FROM sys.all_objects O
				LEFT JOIN sys.extended_properties EP 
					ON ep.major_id = O.object_id 
				LEFT JOIN sys.schemas S 
					ON O.schema_id = S.schema_id
				-- use as separte column
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_UserHint
								FROM sys.extended_properties
								WHERE Name = 'SX_UserHint'
							) sxo
							ON sxo.major_id = ep.major_id
				LEFT JOIN prms ON o.object_id = prms.object_id
				WHERE 
						O.is_ms_shipped = 0
					AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
					AND O.type NOT IN ('PK' -- PrimaryKeys
										,'D'  -- default constraints
										)
					AND (ep.name = 'MS_Description' OR ep.name IS NULL)
					AND (o.type = @ObjectType OR @ObjectType = '')
					AND (sxo.SX_UserHint IS NOT NULL AND sxo.SX_UserHint <> '' OR @ShowAllBool = 1)
					AND (prms.object_id IS NOT NULL OR o.type <> 'P')

				SET @ResultCode = 200;
			COMMIT TRANSACTION spGET_EnduserObjects;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spGET_EnduserObjects;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT EXECUTE ON OBJECT planning.spGET_EnduserObjects TO [Username or rolename]

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_EnduserObjects'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure delivers List of all Objects, which are useful for enduser analytics.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@ObjectType';
    SET @value      = N'Allows to filter output to defined ObjectType. Default Value is P for stored procedure. Keep empty for all.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ShowAllBool';
    SET @value      = N'0 = show only objects with SX_UserHint set in Metadata, 1 = show all objects.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spGET_File;
GO

/*
	Procedure to Return body and file name of existing file or error if file not exist or user have not rights to read

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9
 
	Testcall
		EXEC planning.spGET_File 
			'SQL'
			,'1'
			,'U'
			,'1000'
			,'f09e645a-3624-4497-9606-3d20f0be52f6';

		SELECT * FROM planning.tFiles;

		SELECT * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_File',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_File','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(@FactoryID, N'NULL')+ N''', '+ ISNULL(@ProductLineID, N'NULL')
													+ N''', '+ ISNULL(@ProductID, N'NULL')+ N''', '+ ISNULL(@FileID, N'NULL')
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
	
	-- Procedure specific declarations
	DECLARE  @FactoryKey		INT			= 0
			,@ProductLineKey	INT			= 0
			,@ProductKey		INT			= 0;

	-- NULL Protection
	IF @Username		IS NULL	SET @Username = N'';
	IF @FileID			IS NULL	SET @FileID = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';
	IF @ProductID		IS NULL SET @ProductID = N'';
	IF @FactoryID		= N'' SET @FactoryKey = -1;
	IF @ProductLineID	= N'' SET @ProductLineKey = -1;
	IF @ProductID		= N'' SET @ProductKey = -1;

	BEGIN TRY
		-- Protect input parameters

		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		IF @FileID IS NULL OR LEN(@FileID) = 0
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'FileID is null or empty', 100;
		END;

		-- Determine transaction user

		SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactionUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 100;
		END;

		-- error if Keys chains has gaps
		IF (@FactoryKey	= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1))
			OR	(@ProductLineKey = -1 AND @ProductKey <> -1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys combination can have only gaps at the end', 16, 10);
		END;

		IF @FactoryKey <> -1
		BEGIN
			SELECT @FactoryKey	= FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @ProductLineKey <> -1
			BEGIN
				SELECT @ProductLineKey = ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey	AND ProductLineID = @ProductLineID;
				
				IF @ProductKey <> -1
				BEGIN
					SELECT @ProductKey = ProductKey	FROM planning.tdProducts WHERE FactoryKey = @FactoryKey	AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
				END 
			END
		END

		-- check user rights for cluster level
		IF @FactoryKey = -1
		BEGIN
			-- user already have read right to cluster
			SET @ResultCode = 200;
		END
		-- check user rights for Factory level
		ELSE IF @ProductLineKey = -1
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_FactoryReadRight @TransactionUsername, @FactoryID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Factory.', 16, 10);
			END 
		END
		-- check user rights for ProductLine / Product level
		ELSE IF @ProductLineKey > 0
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ProductLineReadRight @TransactionUsername, @FactoryID, @ProductLineID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for ProductLine.', 16, 10);
			END 
		END
				
		-- error if and Keys dont exists
		IF		@FactoryKey		= 0 
			OR	@ProductLineKey	= 0
			OR  @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
		END;

		DECLARE @FileKey BIGINT = NULL;

		SELECT TOP (1) @FileKey = FileKey 
		FROM planning.tFiles
		WHERE   FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND FileID			= @FileID;

		IF @FileKey IS NULL
		BEGIN
			SET @ResultCode = 404;
			THROW 60000, N'File not found', 100;
		END;

		SELECT
			 FactoryID
			,ProductLineID
			,ProductID
			,FileID
			,[FileName] 
			,ISNULL([Filename] + N'.' + FileExtension, [Filename]) AS 'FileNameWithExtension'
			,FileExtension
			,FileBody
			,Filepath
		FROM planning.tFiles
		WHERE FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND FileID			= @FileID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	
		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

    EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
	RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_File'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to Return body and file name of existing file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileID';
    SET @value      = N'FileID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spGET_FileList;
GO

/*
	Procedure to Return list of existing files on a product
		-Right Check missing
		-Key Determination missing

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	Testcall
		EXEC planning.spGET_FileList
			'SQL'
			,''
			,''
			,''
			,'Rootfolder/Subfolder2';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_FileList',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_FileList','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_FileList
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@Filepath		NVARCHAR(4000)	= N'' -- this may be the full filepath or the start of it

AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username		, N'NULL') + N''', '''
													+ ISNULL(@FactoryID		, N'NULL') + N''', '''
													+ ISNULL(@ProductLineID	, N'NULL') + N''', '''
													+ ISNULL(@ProductID		, N'NULL') + N''', '''
													+ ISNULL(@Filepath		, N'NULL')
													+ N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						

		IF @Username IS NULL SET @Username = N'';

		BEGIN TRY

			SET @Username = dbo.sx_pf_pProtectString (@Username);

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
					
			SELECT
				 FactoryID
				,ProductLineID
				,ProductID
				,FileID
				,[FileName] 
				,[Filename] + ISNULL(N'.' + FileExtension, N'') AS 'FileNameWithExtension'
				,FileExtension
				,UploadedBy
				,UploadedAt
				,Filepath
			FROM planning.tFiles
			WHERE 	
					FactoryID		= @FactoryID
				AND ProductLineID	= @ProductLineID
				AND ProductID		= @ProductID
				AND (
						Filepath LIKE @Filepath + N'%' -- to cover starting with this path
						OR 
						@Filepath = N''
					)

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_FileList TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_FileList'					-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Return list of existing files on a product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Filepath';
    SET @value      = N'This may be the full filepath or the start of it for a like search.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spGET_Index_JSON;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Procedure to GET the Structure of Planning Objects
	FactoryID and ProductLineID can be passed in as single Value or as empty string for all

	Output Limit may be F/PL/P/VS

	Testcall
		DECLARE @ParameterJSON NVARCHAR(MAX);
		SET @ParameterJSON = N'{ "connector_type":"read"
								,"version":1
								,"FactoryID":"ZT"		
								,"ProductLineID":"U"	
								,"ProductID":"1"
								,"OutputLimit" : "VS"  
								}
					'
		EXEC planning.spGET_Index_JSON 'SQL',@ParameterJSON

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Index_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Index_JSON','PARAMETER',NULL)

*/
   
CREATE PROCEDURE planning.spGET_Index_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ParameterJSON	NVARCHAR(MAX) = ''

AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables
		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username, N'NULL')		+ N''', '''
														+ ISNULL(@ParameterJSON, N'NULL')	+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';					
			
		-- Procedure Variables
		DECLARE  @StartTime		TIME(4)
				,@ProcessTime	INT
				,@FactoryID		NVARCHAR(255)
				,@ProductLineID NVARCHAR(255)
				,@ProductID		NVARCHAR(255)
				,@OutputLimit	NVARCHAR(255) = ''

		BEGIN TRY

			SET @StartTime = CONVERT (time, SYSUTCDATETIME()) ; 

			-- Test for valid JSON ################################################################################
			IF ISJSON(@ParameterJSON) = 0 
				BEGIN
					PRINT 'Exit due to invalid JSON'
					RETURN 403
				END
	
			-- Extract Parameter FROM JSON ######################################################################
			
			SET @FactoryID		= JSON_Value(@ParameterJSON,'$.FactoryID')		COLLATE DATABASE_DEFAULT;
			SET @ProductLineID	= JSON_Value(@ParameterJSON,'$.ProductLineID')	COLLATE DATABASE_DEFAULT;
			SET @ProductID		= JSON_Value(@ParameterJSON,'$.ProductID')		COLLATE DATABASE_DEFAULT;
			SET @OutputLimit	= JSON_Value(@ParameterJSON,'$.OutputLimit')	COLLATE DATABASE_DEFAULT;

			IF @OutputLimit = N'' OR @OutputLimit IS NULL SET @OutputLimit = N'VS'
		
			-- Check Right and Keys as valid  #################################################################

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Check Rights and flag invalid rows

			-- Invalid IDs get 404 flag

				-- Output als Recordset #################################################################
				IF @OutputLimit = N'VS' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25
							,dVS.ValueSeriesID
							,dVS.[IsNumeric]		
							,dVS.NameShort			AS ValueSeriesName
							,dVS.ValueSource
							,dVS.ValueListID
							,dVS.Scale
							,dVS.Unit
							,dVS.Effect
							,dVS.VisibilityLevel
							,dVS.ValueSeriesNo

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductLineKey = dP.ProductLineKey

							LEFT JOIN planning.tdValueSeries dVS
								ON dP.ProductKey = dVS.ProductKey

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)

							AND 
							(
								@ProductID = N''
								OR
								@ProductID = dP.ProductID
							)

							ORDER BY dF.FactoryID, dPL.ProductLineID, dP.ProductID,dVS.ValueSeriesID

							SET @EffectedRows = @@ROWCOUNT;
					END

				IF @OutputLimit = N'P' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName
							,dP.ProductID			
							,dP.NameShort			AS ProductName
							,dP.Globalattribute1
							,dP.Globalattribute2
							,dP.Globalattribute3
							,dP.Globalattribute4
							,dP.Globalattribute5
							,dP.Globalattribute6
							,dP.Globalattribute7
							,dP.Globalattribute8
							,dP.Globalattribute9
							,dP.Globalattribute10
							,dP.Globalattribute11
							,dP.Globalattribute12
							,dP.Globalattribute13
							,dP.Globalattribute14
							,dP.Globalattribute15
							,dP.Globalattribute16
							,dP.Globalattribute17
							,dP.Globalattribute18
							,dP.Globalattribute19
							,dP.Globalattribute20
							,dP.Globalattribute21
							,dP.Globalattribute22
							,dP.Globalattribute23
							,dP.Globalattribute24
							,dP.Globalattribute25

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey

							LEFT JOIN planning.tdProducts dP
								ON dPL.ProductLineKey = dP.ProductLineKey

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)
							AND 
							(
								@ProductID = N''
								OR
								@ProductID = dP.ProductID
							)

						ORDER BY dF.FactoryID, dPL.ProductLineID, dP.ProductID

						SET @EffectedRows = @@ROWCOUNT;
					END

				IF @OutputLimit = N'PL' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName
							,dPL.ProductLineID
							,dPL.NameShort			AS ProductLineName

						FROM planning.tdFactories dF
							LEFT JOIN planning.tdProductLines dPL 
								ON 
									dF.FactoryKey = dPL.FactoryKey
						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
							AND 
							(
								@ProductLineID = N''
								OR
								@ProductLineID = dPL.ProductLineID
							)

						ORDER BY dF.FactoryID, dPL.ProductLineID
					END

				IF @OutputLimit = N'F' 
					BEGIN
						SELECT
							 dF.FactoryID			
							,dF.NameShort			AS FactoryName

						FROM planning.tdFactories dF

						WHERE 
					
							(
								@FactoryID = N''
								OR
								@FactoryID = dF.FactoryID
							)
						ORDER BY dF.FactoryID

						SET @EffectedRows = @@ROWCOUNT;

					END

				SET @ResultCode = 200;

				-- Runtime, at the moment without output
				SET @ProcessTime = DATEDIFF(ms,@StartTime,CONVERT (time, SYSUTCDATETIME()));

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN 200;
			
	END;

GO

GRANT EXECUTE ON OBJECT ::planning.spGET_Index_JSON TO pf_PlanningFactoryUser;
GO
			
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Index_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to GET the Structure of Planning Objects
         FactoryID and ProductLineID can be passed in as single Value or as empty string for all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ParameterJSON';
    SET @value      = N'ParameterJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spGET_Products_JSON;
GO

/*
	Zweck
		- lesen der Globalattribute einer Liste von Products
		- die Liste der zu lesenden Products wird als JSON Array übergeben und umfasst X Products
		- die Prozedur gibt ein Recordset mit ebenfalls genau X Zeilen zurück
		- Pro Zeile wird ein Returncode zurückgegeben ob das Product gelesen werden konnte
		- Pro Zeile wird ein optionale Information mit zurückgegeben

	ToDo
		- lesende Abfrage schreiben, ähnlich planning.spGET_Values_JSON
			- prüfen, ob die angeforderten Products existieren
			- prüfen, ob der User Rechte auf den angeforderten Products hat
			- Products lesen
			- Products als Recordset zurückgeben

	Gerd Tautenhahn / Maria Rüdger for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @ProductsJSON	NVARCHAR(MAX);
		DECLARE @RC				INT;
		SET @ProductsJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"Products":	
												[											
													{	 "ProductNumber": 1
														,"FactoryID"	: "ZT"
														,"ProductLineID": "U"
														,"ProductID"	: "1"
													},
													{	 "ProductNumber": 2
														,"FactoryID"	: "ZT"
														,"ProductLineID": "U"
														,"ProductID"	: "2"
													}
												]										

										}
								}
					';

		EXEC @RC = planning.spGET_Products_JSON 'SQL', @ProductsJSON;
		PRINT @RC;

	Testcall 2
		DECLARE @ProductsJSON	NVARCHAR(MAX);
		DECLARE @RC				INT;
		SET @ProductsJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"Products":	
												[											
													{	 "ProductNumber": 1
														,"FactoryID"	: "ZT"
														,"ProductLineID": "*"
														,"ProductID":"*"
													}
												]										

										}
								}
					';

		EXEC @RC = planning.spGET_Products_JSON 'SQL', @ProductsJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Products_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Products_JSON','PARAMETER',NULL)
*/  
 
CREATE PROCEDURE planning.spGET_Products_JSON
    (
         @Username				NVARCHAR(255)
        ,@ProductsJSON			NVARCHAR(MAX) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
			     @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)		= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)		= N''''					
				+ ISNULL(@Username						,N'NULL')	+ N''','''											 
				+ ISNULL(@ProductsJSON					,N'NULL')	+ N''''

				,@EffectedRows		INT					= 0			
				,@ResultCode		INT					= 501				
				,@TimestampCall		DATETIME			= GETUTCDATE()
				,@Comment			NVARCHAR(2000)		= N''
				,@TransactUsername	NVARCHAR(255)		= N''
				,@StartTime			DATETIME2			= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ProductsJSON			IS NULL SET @ProductsJSON		= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY

				-- STEP 0.2 - Protect input parameters

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ProductsJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ProductsJSON) = 0 
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to invalid JSON', 100;
				END;
	
			-- Extract Products FROM JSON ######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			CREATE TABLE #RequestedValues
				(
					 ProductNumber	INT										 NOT NULL
					,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				);

			INSERT INTO #RequestedValues
				SELECT DISTINCT
			 		 ProductNumber
					,FactoryID
					,ProductLineID
					,ProductID
				FROM OPENJSON (@ProductsJSON, N'$.Values.Products') 
				WITH (
					 ProductNumber	INT				N'$.ProductNumber'
					,FactoryID		NVARCHAR(255)	N'$.FactoryID'
					,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
					,ProductID		NVARCHAR(255)	N'$.ProductID'
				);

				IF EXISTS(
					SELECT TOP(1) 1 
					FROM #RequestedValues 
					WHERE 
						   ProductNumber	IS NULL 
						OR FactoryID		IS NULL 
						OR ProductLineID	IS NULL 
						OR ProductID		IS NULL 
						OR FactoryID		= N''
						OR ProductLineID	= N''
						OR ProductID		= N''
						)
				OR EXISTS(
					SELECT TOP(1) 1 
					FROM #RequestedValues 
					GROUP BY ProductNumber
					HAVING COUNT (ProductNumber) > 1
				)
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to invalid Request', 100;
				END;

			-- Determine transaction user ######################################################################

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;			

			-- Search ValueSeries and User`s rights ######################################################################

			-- Query all requested Products
				-- only one row if a Product is multiple times requested
				-- classified for rights, but not filterd
			DROP TABLE IF EXISTS #Products;

			CREATE TABLE #Products
				(
				 FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,TimeType				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,NameShort				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,CommentUser 			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NOT NULL
				,ResponsiblePerson		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ImageName				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,[Status]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,Template				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute1		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute2		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute3		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute4		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute5		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute6		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute7		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute8		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute9		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute10		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute11		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute12		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute13		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute14		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute15		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute16		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute17		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute18		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute19		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute20		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute21		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute22		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute23		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute24		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute25		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				);
			
			INSERT INTO #Products
				SELECT
					 dP.FactoryID		
					,dP.ProductLineID	
					,dP.ProductID		
					,dP.TimeType			
					,dP.NameShort			
					,dP.CommentUser 		
					,dP.ResponsiblePerson
					,dP.ImageName			
					,dP.[Status]			
					,dP.Template			
					,dP.GlobalAttribute1
					,dP.GlobalAttribute2
					,dP.GlobalAttribute3
					,dP.GlobalAttribute4
					,dP.GlobalAttribute5
					,dP.GlobalAttribute6
					,dP.GlobalAttribute7
					,dP.GlobalAttribute8
					,dP.GlobalAttribute9
					,dP.GlobalAttribute10
					,dP.GlobalAttribute11
					,dP.GlobalAttribute12
					,dP.GlobalAttribute13
					,dP.GlobalAttribute14
					,dP.GlobalAttribute15
					,dP.GlobalAttribute16
					,dP.GlobalAttribute17
					,dP.GlobalAttribute18
					,dP.GlobalAttribute19
					,dP.GlobalAttribute20
					,dP.GlobalAttribute21
					,dP.GlobalAttribute22
					,dP.GlobalAttribute23
					,dP.GlobalAttribute24
					,dP.GlobalAttribute25
					,UR.[Right]		AS [Right]
					
				FROM planning.tdProducts dP
					-- Filter on resquested Values
					INNER JOIN #RequestedValues RV
						ON 
								dP.FactoryID		= RV.FactoryID
							AND dP.ProductLineID	= RV.ProductLineID
							AND dP.ProductID		= RV.ProductID						

					-- right classification, but no filtering
					LEFT JOIN system.trUserRights UR
						ON 
								dP.FactoryID		= UR.FactoryID 
							AND dP.ProductLineID	= UR.ProductLineID 
							AND UR.Username			= @TransactionUsername;
--PRINT 'Done 1';

			-- Create Resultset ######################################################################

			DROP TABLE IF EXISTS #ProductSet;

			CREATE TABLE #ProductSet (
				 ProductNumber		INT				NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL

				,TimeType			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,NameShort			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,CommentUser 		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				,ResponsiblePerson	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ImageName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,[Status]			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,Template			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute1	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute2	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute3	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute4	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute5	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute6	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute7	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute8	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute9	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute10	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute11	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute12	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute13	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute14	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute15	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute16	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute17	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute18	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute19	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute20	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute21	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute22	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute23	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute24	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute25	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				
				,ReturnMessage		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ReturnCode			INT				NULL
				,ProcessFlag		INT				NULL
				);

			INSERT INTO #ProductSet 
				-- Create Basic Rows
				SELECT 
					 ProductNumber		
					,FactoryID		
					,ProductLineID		
					,ProductID		
					,''		AS TimeType			
					,''		AS NameShort		
					,''		AS CommentUser 		
					,''		AS ResponsiblePerson
					,''		AS ImageName		
					,''		AS [Status]			
					,''		AS Template			
					,''		AS GlobalAttribute1	
					,''		AS GlobalAttribute2	
					,''		AS GlobalAttribute3	
					,''		AS GlobalAttribute4	
					,''		AS GlobalAttribute5	
					,''		AS GlobalAttribute6	
					,''		AS GlobalAttribute7	
					,''		AS GlobalAttribute8	
					,''		AS GlobalAttribute9	
					,''		AS GlobalAttribute10
					,''		AS GlobalAttribute11
					,''		AS GlobalAttribute12
					,''		AS GlobalAttribute13
					,''		AS GlobalAttribute14
					,''		AS GlobalAttribute15
					,''		AS GlobalAttribute16
					,''		AS GlobalAttribute17
					,''		AS GlobalAttribute18
					,''		AS GlobalAttribute19
					,''		AS GlobalAttribute20
					,''		AS GlobalAttribute21
					,''		AS GlobalAttribute22
					,''		AS GlobalAttribute23
					,''		AS GlobalAttribute24
					,''		AS GlobalAttribute25
					,''		AS ReturnMessage	
					,500	AS ReturnCode		
					,''		AS ProcessFlag	

				FROM #RequestedValues;

			-- Insert Informations for Products with Read Rights
			UPDATE PS	
				SET 
					 PS.TimeType					= P.TimeType			
					,PS.NameShort					= P.NameShort		
					,PS.CommentUser 				= P.CommentUser 		
					,PS.ResponsiblePerson			= P.ResponsiblePerson
					,PS.ImageName					= P.ImageName		
					,PS.[Status]					= P.[Status]			
					,PS.Template					= P.Template			
					,PS.GlobalAttribute1			= P.GlobalAttribute1	
					,PS.GlobalAttribute2			= P.GlobalAttribute2	
					,PS.GlobalAttribute3			= P.GlobalAttribute3	
					,PS.GlobalAttribute4			= P.GlobalAttribute4	
					,PS.GlobalAttribute5			= P.GlobalAttribute5	
					,PS.GlobalAttribute6			= P.GlobalAttribute6	
					,PS.GlobalAttribute7			= P.GlobalAttribute7	
					,PS.GlobalAttribute8			= P.GlobalAttribute8	
					,PS.GlobalAttribute9			= P.GlobalAttribute9	
					,PS.GlobalAttribute10			= P.GlobalAttribute10
					,PS.GlobalAttribute11			= P.GlobalAttribute11
					,PS.GlobalAttribute12			= P.GlobalAttribute12
					,PS.GlobalAttribute13			= P.GlobalAttribute13
					,PS.GlobalAttribute14			= P.GlobalAttribute14
					,PS.GlobalAttribute15			= P.GlobalAttribute15
					,PS.GlobalAttribute16			= P.GlobalAttribute16
					,PS.GlobalAttribute17			= P.GlobalAttribute17
					,PS.GlobalAttribute18			= P.GlobalAttribute18
					,PS.GlobalAttribute19			= P.GlobalAttribute19
					,PS.GlobalAttribute20			= P.GlobalAttribute20
					,PS.GlobalAttribute21			= P.GlobalAttribute21
					,PS.GlobalAttribute22			= P.GlobalAttribute22
					,PS.GlobalAttribute23			= P.GlobalAttribute23
					,PS.GlobalAttribute24			= P.GlobalAttribute24
					,PS.GlobalAttribute25			= P.GlobalAttribute25

					,ReturnMessage	= 'Sucessful Read'
					,ReturnCode		= 200
					,ProcessFlag	= ''
					
				FROM #ProductSet PS 
					LEFT JOIN #Products P
						ON 
							PS.FactoryID		= P.FactoryID
						AND PS.ProductLineID	= P.ProductLineID
						AND PS.ProductID		= P.ProductID

				WHERE P.[Right] IN ('Read','Write');

				-- Insert informations for Products without read Rights
				UPDATE PS	
					SET 
						 ReturnMessage	= 'Read denied'
						,ReturnCode		= 403
						,ProcessFlag	= ''
					
				FROM #ProductSet PS 
					LEFT JOIN #Products P
						ON 
							PS.FactoryID		= P.FactoryID
						AND PS.ProductLineID	= P.ProductLineID
						AND PS.ProductID		= P.ProductID

				WHERE P.[Right] IN ('Deny');

				-- Insert informations for not found Products  (still have ReturnCode 500)
				UPDATE #ProductSet
					SET 
						 ReturnMessage	= 'Product not found'
						,ReturnCode		= 404
						,ProcessFlag	= ''
					WHERE ReturnCode = 500;

			-- Output result ######################################################################
				SELECT
		
					 ProductNumber
							
					,FactoryID		
					,ProductLineID	
					,ProductID
					
					,TimeType
					,NameShort
					,CommentUser --Beschreibung
					-- keine Ausgabe von NameLong / CommentDev, da in Gui nicht sichtbar
					,ResponsiblePerson
					,ImageName
					,[Status]
					,Template
					,GlobalAttribute1
					,GlobalAttribute2
					,GlobalAttribute3
					,GlobalAttribute4
					,GlobalAttribute5
					,GlobalAttribute6
					,GlobalAttribute7
					,GlobalAttribute8
					,GlobalAttribute9
					,GlobalAttribute10
					,GlobalAttribute11
					,GlobalAttribute12
					,GlobalAttribute13
					,GlobalAttribute14
					,GlobalAttribute15
					,GlobalAttribute16
					,GlobalAttribute17
					,GlobalAttribute18
					,GlobalAttribute19
					,GlobalAttribute20
					,GlobalAttribute21
					,GlobalAttribute22
					,GlobalAttribute23
					,GlobalAttribute24
					,GlobalAttribute25
						   					 
					,ReturnMessage
					,ReturnCode
					,ProcessFlag	
				FROM #ProductSet ps;

				SET @ResultCode = 200;
				
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Products_JSON'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns an in JSON  Format requested Productarray as Recordset.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username'
    SET @value      = N'Standardparameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductsJSON'
    SET @value      = N'List of Products as JSON Array';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spGET_Tabs;
GO

/*
	Procedure to GET Tabs for
		- the cluster
		- the factory
		- the productline
		- the product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spGET_Tabs
				 @Username		= 'SQL'
				,@FactoryID		= 'ZT'
				,@ProductLineID = 'U'
				,@ProductID		= ''
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tabs',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tabs','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spGET_Tabs
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
				+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
				+ ISNULL(@ProductLineID, N'NULL')	+ N''','''
				+ ISNULL(@ProductID,	 N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey			INT			= 0
				,@ProductLineKey		INT			= 0
				,@ProductKey			INT			= 0;	

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
		IF @ProductID		IS NULL SET @ProductID		= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spGET_Tabs;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;

				--PRINT @FactoryKey 
				--PRINT @ProductLineKey
				--PRINT @ProductKey

				-- error if and ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- error if ID have gaps in the line
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster read.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <> -1  AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryReadRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory read.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineReadRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product read.', 16, 10);
								END 
					END

				-- data operation starts	***********************************************************************************
				SELECT 
					FactoryID
				   ,ProductLineID
				   ,ProductID
				   ,TabID
				   ,OrderIndex
				   ,TabName
				   ,TabHint
				   ,TabText
				   ,PresentationCODE
				   ,Datasource
				   ,IsVisibleBOOL
				   ,LayoutJSON
				   ,ParameterJSON
				FROM planning.tTabs
				WHERE 
						(FactoryKey		= @FactoryKey	)
					AND (ProductLineID	= @ProductLineID)
					AND (ProductID		= @ProductID	);

			SET @ResultCode = 200;

			COMMIT TRANSACTION spGET_Tabs;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spGET_Tabs;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Tabs'			-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module         NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get the Tabs for given planning Tree object. This may be the root level (Cluster), a Factory, a ProductLine or a Product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if request is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if request is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if request is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

DROP PROCEDURE IF EXISTS planning.spGET_Tree;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
  
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = planning.spGET_Tree
				 @Username		= 'SQL'
				,@Searchstring	= ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Tree','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spGET_Tree
    (
         @Username				NVARCHAR(255)
        ,@Searchstring			NVARCHAR(255)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@Searchstring				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @Searchstring			IS NULL SET @Searchstring		= N'';

		SET @Searchstring = CONCAT('%',@Searchstring,'%');

		-- START TRY			***********************************************************************************
		BEGIN TRY

				PRINT CONCAT(N'Determine Righs of this user',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- get ProductLine rights for this user
				DROP TABLE IF EXISTS #ProductLineRights;
				CREATE TABLE #ProductLineRights
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReadCommentMandatory	INT											NOT NULL
						,WriteCommentMandatory	INT											NOT NULL

						CONSTRAINT PK_ProductLineRights PRIMARY KEY CLUSTERED (RowKey)
					);

				INSERT INTO #ProductLineRights
					SELECT 
						 FactoryID				
						,ProductLineID			
						,[Right]				
						,ReadCommentMandatory
						,WriteCommentMandatory
					FROM system.trUserRights
					WHERE 
							UserName = @TransactUsername
						AND FactoryID <> '';

				-- get Factory rights for this user (separat table for simple join later)
				DROP TABLE IF EXISTS #FactoryRights;
				CREATE TABLE #FactoryRights
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_FactoryRights PRIMARY KEY CLUSTERED (RowKey)
					);

				INSERT INTO #FactoryRights
					SELECT
						 FactoryID							
						,[Right]				
					FROM system.trUserRights
					WHERE 
							UserName		= @TransactUsername
						AND ProductLineID	= '';

				-- Data Transaction
				PRINT CONCAT(N'Ausgabe Tree',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					 dF.FactoryID								AS FactoryID
					,dF.NameShort								AS FactoryNameShort
					,dF.ImageName								AS FactoryImageName
					,RF.[Right]									AS FactoryRight
					,COALESCE(dPL.ProductLineID			,'')	AS ProductLineID
					,COALESCE(dPL.NameShort				,'')	AS ProductLineNameShort
					,COALESCE(dPL.ImageName				,'')	AS ProductLineImageName
					,COALESCE(RPL.[Right]				,'')	AS ProductLineRight
					,COALESCE(dP.ProductID				,'')	AS ProductID
					,COALESCE(dP.NameShort				,'')	AS ProductNameShort
					,COALESCE(dP.ImageName				,'')	AS ProductImageName
					,COALESCE(dP.Template				,'')	AS ProductTemplate
					,COALESCE(RPL.[Right]				,'')	AS ProductRight        --same as PL, but will become use later
					,COALESCE(RPL.ReadCommentMandatory	,0)		AS ReadCommentMandatory
					,COALESCE(RPL.WriteCommentMandatory	,0)		AS WriteCommentMandatory

				FROM planning.tdFactories dF
					LEFT JOIN planning.tdProductLines dPL
						ON dPL.FactoryKey = dF.FactoryKey
					LEFT JOIN planning.tdProducts dP
						ON dP.ProductLineKey = dPL.ProductLineKey

					-- Rights on FactoryLevel
					INNER JOIN #FactoryRights RF
						ON		dF.FactoryID		= RF.FactoryID
		
					-- Rights on ProductLineLevel
					INNER JOIN #ProductLineRights RPL
						ON 
								 dF.FactoryID		= RPL.FactoryID
							AND (dPL.ProductLineID	= RPL.ProductLineID OR dPL.ProductLineID IS NULL) -- maybe there is no ProductLine under this Factory, then the Factory must not be filtered.

					-- Filtering due to search string
				WHERE 
					(
							dF.FactoryID		LIKE @Searchstring 
						OR	dF.NameShort		LIKE @Searchstring
						OR  dPL.ProductLineID	LIKE @Searchstring
						OR  dPL.NameShort		LIKE @Searchstring
						OR  dP.ProductID		LIKE @Searchstring
						OR  dP.NameShort		LIKE @Searchstring

					)

					OR @Searchstring = ''

				ORDER BY -- primary sort numeric, fallback to sort alphabetic
					 ISNULL(TRY_CAST(dF.FactoryID			AS INT),99999999), dF.FactoryID
					,ISNULL(TRY_CAST(dPL.ProductLineID		AS INT),99999999), dPL.ProductLineID
					,ISNULL(TRY_CAST(dP.ProductID			AS INT),99999999), dP.ProductID;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT EXECUTE ON OBJECT :: [planning].[spGET_Tree] TO [Username or rolename]

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Tree'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Returns the Tree of all planning object - Factories/ProductLines/Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Searchstring';
    SET @value      = N'String to filter the output values - will be handled as %Searchstring% search.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS planning.spGET_Values_JSON;
GO

/*
	Procedure to GET Values in JSON Format for MatrixConnector

	MatrixConnector:
		- Shows Values which are combined from a Time Axis and a Structure Axis
		- The Structure Axis has the attributes F/PL/P/VS where each can be substituted by a * to mean all
		- The Time Axis shows one TimeID (later with flex Time concept it will show Year, Month, Day where also each one can be a *)

	This GET Procedure returns for one TimeID all Values
	The Values for this TimeID are requsted as JSON 

	Retruns in a Array per Requested Number
		- Nummerischen Wert nach Operation SUM, MAX,...
		- String if single request
		- Datum
		- ValueComment if single request
		- Statistik for Aggregatfunction
			- Anzahl Treffer
			- F/PL/P/VS bei MAX, MIN

	 Aktuelle Limitierungen:
		- Es wird immer der Aggreationstyp Summe zurückgegeben - egal was angefragt wird
		- Es werden derzeit nur direkte Einzelwerte verarbeitet und zurückgegeben
		- Es werden somit alle mit * angefragten Zeilen als NULL zurückgegeben
		- Er werden nur nummerische Werte zurückgegeben, noch keine Texte
		- Es werden noch keine Rechte verarbeitet

	 TODO:
		- Nur Werte für die berechtigten Strukturinhalten zurückgeben
		- Info über Berechtigung im Text
		- Berechtigung von nicht gefunden unterscheiden
		- ValueSeries kann bei Aggregation beide mehrere Datentypen lieferen -> muss bei Aggregation Typ wählen ?
		- Werte können in unterschiedlichen ValueSeries unterschiedlich skaliert sein
		- Nach dem Schreiben von Werten nach DF kann ein Powerloading notwendig sein
		- Comments ausgeben

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.8
 
	Testcall
		DECLARE @ValuesJSON NVARCHAR(MAX)
		SET @ValuesJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"ValueSeries":	
												[											
													{	 "SeriesNumber"	: 1
														,"AggregateType": "SUM"
														,"FactoryID"	: "ZT"
														,"ProductLineID": "*"
														,"ProductID":"*"
														,"ValueSeriesID": "*"
													}
												]										
											,"TimeIDs":	
												[
														{	 "TimeNumber"	: 1
															,"TimeID"		: 201902
														}, 
														{	 "TimeNumber"	: 2
															,"TimeID"		: 20190215
														}, 
														{	 "TimeNumber"	: 3
															,"TimeID"		: 20190315
														}, 
														{	 "TimeNumber"	: 4
															,"TimeID"		: 20190415
														} 
												]
										}
								}
					'
		EXEC planning.spGET_Values_JSON 'SQL', @ValuesJSON

		EXEC planning.spGET_Values_JSON 'SQL','{"connector_type":"read","version":1,"Values":{"ValueSeries":[{"SeriesNumber":1,"AggregateType":"SUM","FactoryID":"ZT","ProductLineID":"U","ProductID":1,"ValueSeriesID":"K1"}],"TimeIDs":[{"TimeNumber":1,"TimeID":20200115},{"TimeNumber":2,"TimeID":20190215},{"TimeNumber":3,"TimeID":20190315},{"TimeNumber":4,"TimeID":20190415}]}}'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Values_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Values_JSON','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for technical valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to technical invalid JSON', 100;
				END;
	
			-- Extract ValueSeries FROM JSON into tmp table######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			SELECT DISTINCT
			 	 SeriesNumber
				,AggregateType COLLATE DATABASE_DEFAULT AS AggregateType
				,CASE WHEN FactoryID		= N'*'	OR FactoryID LIKE N'*_%'		THEN FactoryID		ELSE dbo.sx_pf_pProtectID (FactoryID)		END COLLATE DATABASE_DEFAULT AS 'FactoryID'
				,CASE WHEN ProductLineID	= N'*'	OR ProductLineID LIKE N'*_%'	THEN ProductLineID	ELSE dbo.sx_pf_pProtectID (ProductLineID)	END COLLATE DATABASE_DEFAULT AS 'ProductLineID'
				,CASE WHEN ProductID		= N'*'	OR ProductID LIKE N'*_%'		THEN ProductID		ELSE dbo.sx_pf_pProtectID (ProductID)		END COLLATE DATABASE_DEFAULT AS 'ProductID'
				,CASE WHEN ValueSeriesID	= N'*'	OR ValueSeriesID LIKE N'*_%'	THEN ValueSeriesID	ELSE dbo.sx_pf_pProtectID (ValueSeriesID)	END COLLATE DATABASE_DEFAULT AS 'ValueSeriesID'
			INTO #RequestedValues
			FROM OPENJSON (@ValuesJSON, N'$.Values.ValueSeries') 
			WITH (
				SeriesNumber	INT				N'$.SeriesNumber'
				,AggregateType	NVARCHAR(50)	N'$.AggregateType'
				,FactoryID		NVARCHAR(255)	N'$.FactoryID'
				,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
				,ProductID		NVARCHAR(255)	N'$.ProductID'
				,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
			);
			
			-- Test on mandatory parameter completness
			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				WHERE  SeriesNumber		IS NULL 
					OR AggregateType	IS NULL 
					OR FactoryID		IS NULL 
					OR ProductLineID	IS NULL 
					OR ProductID		IS NULL 
					OR ValueSeriesID	IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					OR ValueSeriesID	= N'')
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				GROUP BY SeriesNumber
				HAVING COUNT (SeriesNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to logical failure in requested values (missing IDs, multiple SeriesNumber ..)', 100;
			END;

			-- Extract TimeIDs FROM JSON #######################################################################

			DROP TABLE IF EXISTS #RequestedTimes;

			;WITH times AS (
				SELECT DISTINCT
					 TimeNumber
					,TimeID
				FROM OPENJSON (@ValuesJSON, N'$.Values.TimeIDs') 
				WITH (
						 TimeNumber		INT		N'$.TimeNumber'
						,TimeID			INT		N'$.TimeID'
				)
			)
			SELECT *
				--determine Max/Min TimeID from cutted TimeIDs which represend Year / Month / Day
				, CASE	WHEN TimeID BETWEEN 1000		AND 9999		THEN TimeID * 10000
						WHEN TimeID BETWEEN 100000		AND 999999		THEN TimeID * 100
						WHEN TimeID BETWEEN 10000000	AND 99999999	THEN TimeID
						ELSE NULL END AS 'MinTimeID'
				, CASE	WHEN TimeID BETWEEN 1000		AND 9999		THEN TimeID * 10000 + 9999
						WHEN TimeID BETWEEN 100000		AND 999999		THEN TimeID * 100 + 99
						WHEN TimeID BETWEEN 10000000	AND 99999999	THEN TimeID
						ELSE NULL END AS 'MaxTimeID'
			INTO #RequestedTimes
			FROM times;

			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				WHERE TimeNumber IS NULL 
					OR MinTimeID IS NULL
					OR MaxTimeID IS NULL
			) 
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				GROUP BY TimeNumber
				HAVING COUNT (TimeNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to problems in processing requested TimeIDs.', 100;
			END;

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- STEP 3.0 - Search ValueSeries and User`s rights

			DROP TABLE IF EXISTS #ValueSeries;
			
			;WITH search AS (
				SELECT SeriesNumber
					,AggregateType
					,FactoryID
					,ProductLineID
					,ProductID
					,ValueSeriesID

					,CASE WHEN FactoryID		= N'*' THEN 1 WHEN FactoryID		LIKE N'*_%' THEN 2 ELSE 0 END AS FactoryIDSearchType
					,CASE WHEN ProductLineID	= N'*' THEN 1 WHEN ProductLineID	LIKE N'*_%' THEN 2 ELSE 0 END AS ProductLineIDSearchType
					,CASE WHEN ProductID		= N'*' THEN 1 WHEN ProductID		LIKE N'*_%' THEN 2 ELSE 0 END AS ProductIDSearchType
					,CASE WHEN ValueSeriesID	= N'*' THEN 1 WHEN ValueSeriesID	LIKE N'*_%' THEN 2 ELSE 0 END AS ValueSeriesIDSearchType
					
					,CASE WHEN FactoryID		LIKE N'*_%' THEN SUBSTRING(FactoryID, 2, 300)		ELSE NULL END AS FactoryIDPattern
					,CASE WHEN ProductLineID	LIKE N'*_%' THEN SUBSTRING(ProductLineID, 2, 300)	ELSE NULL END AS ProductLineIDPattern
					,CASE WHEN ProductID		LIKE N'*_%' THEN SUBSTRING(ProductID, 2, 300)		ELSE NULL END AS ProductIDPattern
					,CASE WHEN ValueSeriesID	LIKE N'*_%' THEN SUBSTRING(ValueSeriesID, 2, 300)	ELSE NULL END AS ValueSeriesIDPattern
				FROM #RequestedValues
			)
			SELECT 
				rv.SeriesNumber
				,rv.AggregateType
				,dF.FactoryID		COLLATE DATABASE_DEFAULT AS FactoryID
				,dPL.ProductLineID	COLLATE DATABASE_DEFAULT AS ProductLineID
				,dP.ProductID		COLLATE DATABASE_DEFAULT AS ProductID
				,dVS.ValueSeriesID	COLLATE DATABASE_DEFAULT AS ValueSeriesID
				,dF.FactoryKey
				,dPL.ProductLineKey
				,dP.ProductKey
				,dVS.ValueSeriesKey
				,dVS.Scale
				,dVS.[IsNumeric]
				,rF.[Row] AS 'FactoryRightRow'
				,rPL.[Row] AS 'ProductLineRightRow'
			INTO #ValueSeries
			FROM search rv
				INNER JOIN planning.tdFactories dF
					ON rv.FactoryIDSearchType = 1 
						OR (rv.FactoryIDSearchType = 2 AND df.FactoryID LIKE rv.FactoryIDPattern) 
						OR (rv.FactoryIDSearchType = 0 AND df.FactoryID = rv.FactoryID) 
				INNER JOIN planning.tdProductLines dPL
					ON dPL.FactoryKey = dF.FactoryKey
						AND (rv.ProductLineIDSearchType = 1 
							OR (rv.ProductLineIDSearchType = 2 AND dPL.ProductLineID LIKE rv.ProductLineIDPattern) 
							OR (rv.ProductLineIDSearchType = 0 AND dPL.ProductLineID = rv.ProductLineID))
				INNER JOIN planning.tdProducts dP
					ON dP.FactoryKey = dF.FactoryKey 
						AND dP.ProductLineKey = dPL.ProductLineKey
						AND (rv.ProductIDSearchType = 1 
							OR (rv.ProductIDSearchType = 2 AND dP.ProductID LIKE rv.ProductIDPattern) 
							OR (rv.ProductIDSearchType = 0 AND dP.ProductID = rv.ProductID))
				INNER JOIN planning.tdValueSeries dVS
					ON dVS.FactoryKey = dF.FactoryKey 
						AND dVS.ProductLineKey = dPL.ProductLineKey
						AND dVS.ProductKey = dP.ProductKey
						AND (rv.ValueSeriesIDSearchType = 1 
							OR (rv.ValueSeriesIDSearchType = 2 AND dVS.ValueSeriesID LIKE rv.ValueSeriesIDPattern) 
							OR (rv.ValueSeriesIDSearchType = 0 AND dVS.ValueSeriesID = rv.ValueSeriesID))
				LEFT JOIN system.trUserRights rF 
					ON df.FactoryID = rF.FactoryID 
						AND rF.ProductLineID = N''
						AND rF.Username = @TransactionUsername
						AND (rF.[Right] = N'Write' OR rF.[Right] = N'Read')
				LEFT JOIN system.trUserRights rPL 
					ON df.FactoryID = rPL.FactoryID 
						AND dPL.ProductLineID = rPL.ProductLineID 
						AND rPL.Username = @TransactionUsername
						AND (rPL.[Right] = N'Write' OR rPL.[Right] = N'Read');

			-- STEP 3.1 - Searching requested values

			DROP TABLE IF EXISTS #Values;

			SELECT
				s.SeriesNumber
				,s.FactoryKey
				,s.ProductLineKey
				,s.ProductKey
				,s.ValueSeriesKey
				,s.FactoryID      COLLATE DATABASE_DEFAULT AS FactoryID
				,s.ProductLineID  COLLATE DATABASE_DEFAULT AS ProductLineID
				,s.ProductID      COLLATE DATABASE_DEFAULT AS ProductID
				,s.ValueSeriesID  COLLATE DATABASE_DEFAULT AS ValueSeriesID
				,ISNULL(CASE WHEN s.[IsNumeric] = 1 THEN CAST(fV.ValueInt AS Money) / s.Scale ELSE 0 END, 0) AS 'ValueNumeric'
				,ISNULL(CASE WHEN s.[IsNumeric] = 0 THEN fV.ValueText ELSE N'' END, N'') COLLATE DATABASE_DEFAULT AS 'ValueString'
				,fV.ValueComment COLLATE DATABASE_DEFAULT AS ValueComment
				,t.TimeNumber
				,fV.TimeID
				,s.[IsNumeric]
			INTO #Values
			FROM #ValueSeries s
				INNER JOIN planning.tfValues fV
					ON 
							-- why join over four Key fields ?
							fV.FactoryKey		= s.FactoryKey 
						AND fV.ProductLineKey	= s.ProductLineKey
						AND fV.ProductKey		= s.ProductKey
						AND fV.ValueSeriesKey	= s.ValueSeriesKey
						AND NOT s.FactoryRightRow		IS NULL 
						AND NOT s.ProductLineRightRow	IS NULL
				INNER JOIN #RequestedTimes t
					ON fV.TimeID BETWEEN t.MinTimeID AND t.MaxTimeID;

			-- STEP 3.2 - Insert requested values into result table

			DROP TABLE IF EXISTS #ValueSet;

			CREATE TABLE #ValueSet (
				 SeriesNumber		INT				NOT NULL
				,AggregateType		NVARCHAR(50)	COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				
				,TimeNumber			INT				NOT NULL
				,TimeID				INT				NOT NULL
				
				,FactoryKey			BIGINT			NULL
				,ProductLineKey		BIGINT			NULL
				,ProductKey			BIGINT			NULL
				,ValueSeriesKey		BIGINT			NULL

				,FactoryCount		INT				NULL
				,ProductLineCount	INT				NULL
				,ProductCount		INT				NULL
				,ValueSeriesCount	INT				NULL

				,ValueNumeric		MONEY			NULL
				,ValueString		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				,ValueDate			DATE			NULL
				,ValueComment		NVARCHAR(4000)  COLLATE DATABASE_DEFAULT NULL

				,ReturnMessage		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ReturnCode			INT				NULL
				,ProcessFlag		INT				NULL
				);

			INSERT INTO #ValueSet (SeriesNumber, AggregateType, FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeNumber, TimeID)
			SELECT 
				 v.SeriesNumber	
				,v.AggregateType	
				,v.FactoryID		
				,v.ProductLineID		
				,v.ProductID			
				,v.ValueSeriesID		
				,t.TimeNumber	
				,t.TimeID
			FROM #RequestedValues v, #RequestedTimes t;

			-- STEP 3.1 - Set 'No rights' comments  		
			
			;WITH series AS (
				SELECT 
					FactoryID 
					,ProductLineID
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				WHERE FactoryRightRow IS NULL 
					OR ProductLineRightRow IS NULL
				GROUP BY FactoryID, ProductLineID
			)
			UPDATE s
			SET	 ReturnMessage		= N'No rights'
				,ReturnCode			= 403
				,FactoryCount		= 0
				,ProductLineCount	= 0
				,ProductCount		= 0
				,ValueSeriesCount	= 0
				,ValueNumeric		= CASE WHEN v.[IsNumeric] = 1	THEN 0		ELSE NULL END
				,ValueString		= CASE WHEN v.IsString = 0		THEN N''	ELSE NULL END
				,ValueComment		= N''
			FROM #ValueSet s
				INNER JOIN series v
					ON s.FactoryID = v.FactoryID 
						AND s.ProductLineID = v.ProductLineID;

			---- STEP 3.2 - Fill result table with exists values, make aggregation

			;WITH vals AS (
				SELECT 
					 SeriesNumber
					,TimeNumber
					,COUNT(DISTINCT FactoryKey)		AS 'FactoryCount'
					,COUNT(DISTINCT ProductLineKey)	AS 'ProductLineCount'
					,COUNT(DISTINCT ProductKey)		AS 'ProductCount'
					,COUNT(DISTINCT ValueSeriesKey)	AS 'ValueSeriesCount'
					,MAX(FactoryKey)				AS 'FactoryKey'
					,MAX(ProductLineKey)			AS 'ProductLineKey'
					,MAX(ProductKey)				AS 'ProductKey'
					,MAX(ValueSeriesKey)			AS 'ValueSeriesKey'
					,SUM(ValueNumeric)				AS 'ValueNumeric'
				FROM #Values
				GROUP BY SeriesNumber, TimeNumber
			),
			comments AS (
				SELECT TOP (1) WITH TIES
					 SeriesNumber
					,TimeNumber
					,ValueComment
				FROM #Values
				WHERE NOT ValueComment IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeID DESC)
			),
			stringValues AS (
				SELECT TOP (1) WITH TIES
					 SeriesNumber
					,TimeNumber
					,ValueString
				FROM #Values
				WHERE [IsNumeric] = 0
					AND NOT ValueString IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeID DESC)
			)
			UPDATE s
			SET  s.FactoryKey		= CASE WHEN s.FactoryID		LIKE N'*%' THEN NULL ELSE v.FactoryKey		END
				,s.ProductLineKey	= CASE WHEN s.ProductLineID LIKE N'*%' THEN NULL ELSE v.ProductLineKey	END
				,s.ProductKey		= CASE WHEN s.ProductID		LIKE N'*%' THEN NULL ELSE v.ProductKey		END
				,s.ValueSeriesKey	= CASE WHEN s.ValueSeriesID LIKE N'*%' THEN NULL ELSE v.ValueSeriesKey	END 
				,s.FactoryCount		= v.FactoryCount
				,s.ProductLineCount = v.ProductLineCount
				,s.ProductCount		= v.ProductCount
				,s.ValueSeriesCount = v.ValueSeriesCount
				,s.ValueNumeric		= v.ValueNumeric
				,s.ValueString		= ISNULL(sv.ValueString, N'')
				,s.ValueComment		= ISNULL(c.ValueComment, N'')
			FROM #ValueSet s
				INNER JOIN vals v
					ON s.SeriesNumber = v.SeriesNumber
						AND s.TimeNumber = v.TimeNumber
				LEFT JOIN comments c
					ON s.SeriesNumber = c.SeriesNumber
						AND s.TimeNumber = c.TimeNumber
				LEFT JOIN stringValues sv
					ON s.SeriesNumber = sv.SeriesNumber
						AND s.TimeNumber = sv.TimeNumber
			WHERE s.FactoryCount IS NULL; 		

			---- STEP 3.4 - Fill ValueNumeric, ValueComment and ValueString columns and set zero-count for non exists values

			;WITH series AS (
				SELECT 
					 SeriesNumber
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				GROUP BY SeriesNumber
			)
			UPDATE s
			SET  FactoryCount		= ISNULL(FactoryCount, 0)
				,ProductLineCount	= ISNULL(ProductLineCount, 0)
				,ProductCount		= ISNULL(ProductCount, 0)
				,ValueSeriesCount	= ISNULL(ValueSeriesCount, 0)
				,ValueNumeric		= NULL
				,ValueString		= NULL
				,ValueComment		= NULL
			FROM #ValueSet s 
				INNER JOIN series v
					ON s.SeriesNumber = v.SeriesNumber
			WHERE s.FactoryCount IS NULL;

			-- STEP 3.5 - Output result

			SELECT
				 ROW_NUMBER() OVER (ORDER BY SeriesNumber, FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeNumber) AS RowKey			
				,SeriesNumber
				,TimeNumber
				,AggregateType	
				,FactoryID		
				,ProductLineID	
				,ProductID		
				,ValueSeriesID	
		
				,FactoryKey		
				,ProductLineKey	
				,ProductKey		
				,ValueSeriesKey	
		
				,FactoryCount	
				,ProductLineCount
				,ProductCount	
				,ValueSeriesCount	
		
				,TimeID			
				,ValueNumeric	
				,ValueString	
				,ValueDate		
				,ValueComment	
				,IIF(ReturnMessage IS NULL,'',ReturnMessage) AS ReturnMessage
				,IIF(ReturnCode IS NULL,200,ReturnCode) AS ReturnCode
				,ProcessFlag	
			FROM #ValueSet;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;

GO
			
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Values_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 -- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to GET Values in JSON Format for MatrixConnector';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'ValuesJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spMOVE_Tab;
GO

/*
	Procedure to move a Tab to
		- another ID
		- another OrderIndex
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spMOVE_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= '0'
				,@NewTabID			= 'T7'
				,@NewOrderIndex		= '188'

		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex

		TRUNCATE TABLE planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spMOVE_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spMOVE_Tab','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spMOVE_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255)
		,@NewTabID				NVARCHAR(255)
		,@NewOrderIndex			INT 
    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''','
				+ ISNULL(@NewTabID				,N'NULL')					+ N''','
				+ CAST(ISNULL(@NewOrderIndex	,N'NULL') AS NVARCHAR(255)) + N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0
				,@OldOrderIndex		INT			= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';
		IF @NewTabID			IS NULL SET @NewTabID			= N'';
		IF @NewOrderIndex		IS NULL SET @NewOrderIndex		= 0  ;

		-- dont use negative index values
		IF @NewOrderIndex		<= 0	SET @NewOrderIndex	= 1;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spMOVE_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;

				--PRINT @FactoryKey 
				--PRINT @ProductLineKey
				--PRINT @ProductKey

				-- error if and ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if TabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @TabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('TabID not found.', 16, 10);
							END;

				-- tab with ID 0 cant be moved
				IF @TabID = '0' OR @NewTabID = '0'
					BEGIN
							SET @ResultCode = 403;
							RAISERROR('Tab with ID 0 can''t be moved', 16, 10);
					END


				-- error if ID have gaps in the line
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <> - 1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- check if new TabID is free
				IF EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @NewTabID
									AND TabID			<> @TabID
							)
							BEGIN
								SET @ResultCode = 401;
								RAISERROR('New TabID already in use.', 16, 10);
							END;

				-- determine current Key and OrderIndex
				SELECT 
					 @Rowkey = RowKey
					,@OldOrderIndex = OrderIndex
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- data operation starts	***********************************************************************************

				-- Insert new Tab
				UPDATE planning.tTabs
					SET 
						 TabID		= @NewTabID
						,OrderIndex = @NewOrderIndex
					WHERE
						RowKey		= @RowKey

				--PRINT 'OldOrderIndex ' + CAST (@OldOrderIndex AS NVARCHAR(10))
				--PRINT 'NewOrderIndex ' + CAST (@NewOrderIndex AS NVARCHAR(10))
						
				-- Update other numbers if Number is changed
				IF @OldOrderIndex <> @NewOrderIndex
					BEGIN			
						-- case if OrderIndex is bigger now and existed before - decrement all in move window
						IF @OldOrderIndex < @NewOrderIndex 
							BEGIN
								UPDATE planning.tTabs
								SET OrderIndex = OrderIndex - 1 
								WHERE 
										OrderIndex		>= @OldOrderIndex 
									AND OrderIndex		<= @NewOrderIndex
									AND	FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			<> @NewTabID
							END
						-- case if OrderIndex is smaller now and existed before - increment all in move window
						IF @OldOrderIndex > @NewOrderIndex 
							BEGIN
								UPDATE planning.tTabs
								SET OrderIndex = OrderIndex + 1 
								WHERE 
										OrderIndex		<= @OldOrderIndex 
									AND OrderIndex		>= @NewOrderIndex
									AND	FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			<> @NewTabID
							END

						-- eliminate all Gaps
						UPDATE planning.tTabs 
						SET OrderIndex = Rownumbers.Number
						FROM planning.tTabs tT INNER JOIN (
																	SELECT 
																		 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																		,TabID
																	
																	FROM planning.tTabs
																	WHERE 
																			FactoryKey		= @FactoryKey
																		AND ProductLineKey	= @ProductLineKey
																		AND ProductKey		= @ProductKey

															) AS Rownumbers
						ON 
								tT.TabID			= Rownumbers.TabID
							AND tT.FactoryKey		= @FactoryKey
							AND tT.ProductLineKey	= @ProductLineKey
							AND tT.ProductKey		= @ProductKey

					END

				IF @ResultCode <> 201 SET @ResultCode = 200;

			COMMIT TRANSACTION spMOVE_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spMOVE_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spMOVE_Tab'		-- enter procedure name
    ,@SX_Owner          NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module	        NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Move a Tab to a different ID and different OrderIndex. The requested TabID must be free. The requested Orderposition may be in use, reordering is done automaticly. 
	To put a tab to the end, any high order number (like 100) can be submitted. The Tab with ID 0 can''t be moved.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if MOVE is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if MOVE is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if MOVE is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'TabID which is moved. ID 0 is not accepted, its reserved for the system tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@NewTabID';
    SET @value      = N'TabID the tab will get. This must not be already in use.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@NewOrderIndex';
    SET @value      = N'Position in the list of Tabs. Any Number > 0 can be passed in, tabs reorder themself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spPOST_File;
GO

/*
	Procedure to save new file or change exists file
	Files may be saved on Factories, Productlines, Products on Clusters - thefore the F/PL/PID is optional 
	@FileBody can be NULL - this are files which represend folders
	If @FileID is NULL or Empty then error raised

	If @FileID is '*' will create new file
		@FileExtension can be null (will be created file without extension)
		@FileName can`t be null or empty

	If @FileID is NOT null will change exists file
		If @FileExtension is NULL then don`t change extension of exists file
		If @FileExtension is empty then set to NULL extension of exists file (file without extension)
		Otherwise change extension to @FileExtension
	
		If @FileName is NULL then don`t change name of exists file
		If @FileName is empty then error raised
		Otherwise change file name to @FileName

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		DECLARE  @RC				INT
				,@ReturnedFileID	NVARCHAR (50);
		EXEC planning.spPOST_File 
			'SQL'
			,''
			,''
			,''
			,''     -- FileID or *
			,'Hallo22'
			,'pdf'
			,0x3
			,'/Rootfolder/Subfolder2'
			,@OutputID				= @ReturnedFileID OUTPUT;
		PRINT    @RC;
		PRINT	'FileID Output: '+ ISNULL(@ReturnedFileID,'no Output set');

	Testcall tables
		SELECT * FROM planning.tFiles;
		DELETE FROM planning.tFiles;

	Testcall Log
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_File',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_File','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
	,@FileName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
	,@Filepath		NVARCHAR(4000)	= N'/'
	,@OutputID		NVARCHAR(50)	= NULL OUTPUT	
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username					, N'NULL') + N''', '''
													+ ISNULL(@FactoryID					, N'NULL') + N''', '''
													+ ISNULL(@ProductLineID				, N'NULL') + N''', '''
													+ ISNULL(@ProductID					, N'NULL') + N''', ''' 
													+ ISNULL(@FileID					, N'NULL') + N''', ''' 
													+ ISNULL(@FileName					, N'NULL') + N''', '''	
													+ ISNULL(@FileExtension				, N'NULL') + N''', ' 
													+ ISNULL(CAST(DATALENGTH(@FileBody)   
														AS NVARCHAR(MAX)) + N' bytes'	, N'NULL') + N','''
													+ ISNULL(@Filepath,N'NULL')			+ N''''		
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N''
			,@FactoryKey			INT				= 0
			,@ProductLineKey		INT				= 0
			,@ProductKey			INT				= 0
			,@FileExistsFlag		INT				= 0;
			
	--NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @FileID			IS NULL SET @FileID			= N'';
	IF @Filepath		IS NULL SET @Filepath		= N'/';
	IF @FactoryID		= N''	SET @FactoryKey		= -1;
	IF @ProductLineID	= N''	SET @ProductLineKey = -1;
	IF @ProductID		= N''	SET @ProductKey		= -1;

	BEGIN TRY
		BEGIN TRANSACTION spPOST_File;

		-- protect IDs
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);

		-- empty FileID will generate a new FileID
		IF @FileID = '' SET @FileID = '*';

		-- determine Transactuser
		SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactionUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 100;
		END;
			
		-- check input parameters (NULL for FileBody is allowed to save files which represent folders)
		IF @FileName =''
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileName is empty', 100;
			END;

		-- error if Keys chains has gaps
		IF (@FactoryKey	= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1))
			OR	(@ProductLineKey = -1 AND @ProductKey <> -1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys combination can have only gaps at the end', 16, 10);
		END;

		IF @FactoryKey <> -1
		BEGIN
			SELECT @FactoryKey	= FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @ProductLineKey <> -1
			BEGIN
				SELECT @ProductLineKey = ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey	AND ProductLineID = @ProductLineID;
				
				IF @ProductKey <> -1
				BEGIN
					SELECT @ProductKey = ProductKey	FROM planning.tdProducts WHERE FactoryKey = @FactoryKey	AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
				END 
			END
		END

		-- check user rights for cluster level
		IF @FactoryKey = -1
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactionUsername;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Cluster write.', 16, 10);
			END 
		END
		-- check user rights for Factory level
		ELSE IF @ProductLineKey = -1
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactionUsername, @FactoryID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Factory write.', 16, 10);
			END 
		END
		-- check user rights for ProductLine / Product level
		ELSE IF @ProductLineKey > 0
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactionUsername, @FactoryID, @ProductLineID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for ProductLine.', 16, 10);
			END 
		END
				
		-- error if and Keys dont exists
		IF		@FactoryKey		= 0 
			OR	@ProductLineKey	= 0
			OR  @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
		END;

		-- if FileID does not exits or FileID = '*', create new file
		SELECT @FileExistsFlag = COUNT(FileID) 
			FROM planning.tFiles 
			WHERE 
					FactoryID		= @FactoryID 
				AND ProductLineID	= @ProductLineID 
				AND ProductID		= @ProductID 
				AND FileID			= @FileID;
		
		IF @FileExistsFlag = 0 OR @FileID = '*'
			-- create new file
			BEGIN
				IF @FileID = '*'
					BEGIN
						-- Determine next free FileID - outer COALESCE FOR CASE no existing row, inner COALESCE for TRY_CAST catching
						SELECT @FileID =
						 CONCAT('F',
							COALESCE(					
									MAX(COALESCE(TRY_CAST(RIGHT(FileID,LEN(FileID)-1) AS INT),0))
									,0)+1
									)
						FROM planning.tFiles 
						WHERE	
								(@FactoryKey = -1		OR FactoryKey = @FactoryKey)
							AND (@ProductLineKey = -1	OR ProductLineKey = @ProductLineKey)
							AND (@ProductKey = -1		OR ProductKey = @ProductKey)
							AND FileID LIKE 'F%';
					END
			
				IF LEN(@FileExtension) = 0	SET @FileExtension = NULL;
			
				-- create new file
				INSERT INTO planning.tFiles (FactoryKey,  ProductLineKey,  ProductKey,  FactoryID,  ProductLineID,  ProductID,  FileID, [FileName], FileExtension,  FileBody, UploadedBy, UploadedAt,Filepath)
				VALUES						(@FactoryKey, @ProductLineKey, @ProductKey, @FactoryID, @ProductLineID, @ProductID, @FileID, @FileName, @FileExtension, @FileBody,@Username,GETUTCDATE(),@Filepath);
			END		
		ELSE 
			-- update existing file
			BEGIN
			
				DECLARE @FileKey BIGINT = NULL;

				SELECT TOP (1) @FileKey = FileKey 
				FROM planning.tFiles
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND FileID			= @FileID;

				-- update existing file
				UPDATE planning.tFiles
				SET 
					 [FileName]		= ISNULL(@FileName, [FileName])
					,FileExtension	= CASE 
										WHEN @FileExtension IS NULL  THEN FileExtension
										WHEN LEN(@FileExtension) = 0 THEN NULL
										ELSE @FileExtension 
									  END
					,FileBody		= ISNULL(@FileBody,FileBody)
					,UploadedBy		= @Username
					,UploadedAt		= GETUTCDATE()
					,Filepath		= ISNULL(@Filepath,Filepath)
				WHERE 
					FileKey = @FileKey;
			END

		-- OUTPUT Parameter
		SET @OutputID = @FileID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION spPOST_File;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		ROLLBACK TRANSACTION spPOST_File;
			
		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

    EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
	RETURN @ResultCode;
END;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_File'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to store a new file or change an existing file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID - maybe empty string to store files on Clusterlevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID - maybe empty string to store files on Factorylevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID - maybe empty string to store files on ProductLineLevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileID';
    SET @value      = N'FileID - use * to autocreate File ID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileName';
    SET @value      = N'FileName - username for the file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileExtension';
    SET @value      = N'FileExtension - may be empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileBody';
    SET @value      = N'FileBody - must not be NULL';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FilePath';
    SET @value      = N'Filepath - optional virtual file path';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spPOST_FileFromPath;
GO

/*
 	Procedure to Create new file or change exists file from a local SQL Server filepath or shared folder path
	File must extists and can`t be empty.
	File extensions extracted from @FilePath.

	If @FileKey is NULL will create new file
	If @FileKey is NOT NULL will change exists file
	
	Return table with columns: FileKey BIGINT, FileName NVARCHAR(MAX)

	TODO: Must be adjusted to FID;PLID;PID !!

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC planning.spPOST_FileFromPath 
			'SQL'
			,NULL						-- use NULL to send new file !
			,'c:\test.txt';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_FileFromPath',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_FileFromPath','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_FileFromPath
	 @Username		NVARCHAR(255)	= N''
	,@FileKey		BIGINT			= NULL
	,@FilePath		NVARCHAR(300)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(CAST(@FileKey AS NVARCHAR(MAX)), N'NULL') + N', ''' + ISNULL(@FilePath, N'NULL') + N'''' 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';
		IF @FilePath IS NULL SET @FilePath = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_FileFromPath;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			SET @FilePath = REPLACE(REPLACE(dbo.sx_pf_pProtectString (@FilePath), N'/*', N''), N'*\', N'');

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF LEN(@FilePath) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FilePath is empty', 100;
			END;

			IF NOT @FileKey IS NULL 
				AND NOT EXISTS (
					SELECT TOP (1) 1 
					FROM planning.tFiles
					WHERE FileKey = @FileKey) 
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			-- STEP 2.1 - Read FileBody from path

			DECLARE @FileBody VARBINARY(MAX),
				@SQLQuery NVARCHAR(MAX),
				@ParmDefinition NVARCHAR(30) = N'@Data VARBINARY(MAX) OUTPUT';

			SET @SQLQuery = N'
				SET @Data = (SELECT * FROM OPENROWSET(
				BULK N''' + @FilePath + ''', SINGLE_BLOB, MAXERRORS=0) AS InputFile)';

			EXEC sp_executesql
				@SQLQuery
				,@ParmDefinition
				,@Data = @FileBody OUTPUT;

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

		    -- STEP 2.2 - Extract FileExtension from path
			DECLARE @FileExtension	NVARCHAR(50) = NULL
				,@DotPos INT;

			SET @DotPos = CHARINDEX(N'.', REVERSE(@FilePath));
			
			IF @DotPos > 1
			BEGIN
				DECLARE @PathLength INT;

				SET @PathLength = LEN(@FilePath);

				IF CHARINDEX(N'\', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N'/', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N':', @FilePath, @PathLength - @DotPos + 1) = 0
				BEGIN
					SET @FileExtension = RIGHT(@FilePath, @DotPos - 1);
				END;
			END;

			DECLARE @UpdatedFile AS TABLE (FileKey BIGINT, FileExtension NVARCHAR(50));

			IF NOT @FileKey IS NULL 
			BEGIN
				-- STEP 2.3 - Updating exists file. 

				UPDATE planning.tFiles
				SET FileExtension = @FileExtension
					,FileBody = @FileBody
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				WHERE FileKey = @FileKey;

			END
			ELSE
			BEGIN	
				-- STEP 2.4 - Creating new file

				INSERT planning.tFiles (FileExtension, FileBody)
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				VALUES (@FileExtension, @FileBody);

			END;

			-- STEP 3.0 - Output result

			SELECT
				 FileKey AS 'FileKey'
				,CAST(FileKey AS NVARCHAR(MAX)) + ISNULL(N'.' + FileExtension, N'') AS 'FileName'
			FROM @UpdatedFile;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_FileFromPath;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_FileFromPath;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::planning.spPOST_FileFromPath TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_FileFromPath'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name; 
 
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to Create new file or change exists file from a local SQL Server filepath or shared folder path
         File must extists and can`t be empty.
         File extensions extracted from @FilePath.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileKey';
    SET @value      = N'FileKey';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FilePath';
    SET @value      = N'FilePath';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spPOST_Products_JSON;
GO

/*
	Procedure to POST Products in JSON Format 
	Preconditions
		- A value can be sended only on full qualified single product -> There is no write on summary level possible
		- multiple values sended for one product -> the last wins

	Functionality:
		- its only possible to change the attributes of an existing products - creating new products is not possible
		- not sended attributes must be sended as NULL
		- to delete an attribute value, an empty string must be sended 
		- user can send only values to products with write rights
		- the Procedure returns all sended products as recordset with the process result

	Workflow
		- put all sended records in a temporary table (SendedRecords)
		- create a temporary classification table with all sended Products (SendedRecordClassification)
			- determine the UserRights in Classification table
			- flag not existing Products
		- return SendedRecords

	Gerd Tautenhahn / Maria Rüdger for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC INT;
		DECLARE @ValuesJSON NVARCHAR(MAX);
			SET @ValuesJSON = N'{

				"connector_type":"write","version":1,"Values":[

				 {"ProductNumber":1	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"NameShort":null		,"CommentUser":""		,"ResponsiblePerson":null	,"ImageName":""		,"Status":""		,"Template":"Test_VM"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":2	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"2"	,"NameShort":""			,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":3	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"3"	,"NameShort":"K3"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":null		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":4	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"4"	,"NameShort":"BETR"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test_VM"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":5	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"4"	,"NameShort":"BEM"		,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":"Test_V"		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }
				,{"ProductNumber":6	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"7"	,"NameShort":"STAT_BUD"	,"CommentUser":""		,"ResponsiblePerson":""		,"ImageName":""		,"Status":""		,"Template":""		,"GlobalAttribute1":""	,"GlobalAttribute2":""	,"GlobalAttribute3":"" ,"GlobalAttribute4":"" ,"GlobalAttribute5":"" ,"GlobalAttribute6":"" ,"GlobalAttribute7":"" ,"GlobalAttribute8":"" ,"GlobalAttribute9":"" ,"GlobalAttribute10":"" ,"GlobalAttribute11":"" ,"GlobalAttribute12":"" ,"GlobalAttribute13":"" ,"GlobalAttribute14":"" ,"GlobalAttribute15":"" ,"GlobalAttribute16":"" ,"GlobalAttribute17":"" ,"GlobalAttribute18":"" ,"GlobalAttribute19":"" ,"GlobalAttribute20":"" ,"GlobalAttribute21":"" ,"GlobalAttribute22":"" ,"GlobalAttribute23":"" ,"GlobalAttribute24":"" ,"GlobalAttribute25":"" }

			]

			}';

		EXEC @RC = planning.spPOST_Products_JSON 'SQL', @ValuesJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Products_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Products_JSON','PARAMETER',NULL);
*/
   
CREATE PROCEDURE planning.spPOST_Products_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY
			BEGIN TRANSACTION spPOST_Products_JSON
			-- INPUT PROTECTION	  #################################################################################

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (technical)', 100;
			END;
			
			-- Extract Values FROM JSON and put them into #SendedRecords #####################################

			DROP TABLE IF EXISTS #SendedRecords

			;WITH vals AS 
				(
					SELECT
						 ProductNumber							AS 'ProductNumber'
						,dbo.sx_pf_pProtectID (FactoryID)		AS 'FactoryID'
						,dbo.sx_pf_pProtectID (ProductLineID)	AS 'ProductLineID'
						,dbo.sx_pf_pProtectID (ProductID)		AS 'ProductID'
						,NameShort								AS 'NameShort'
						,CommentUser							AS 'CommentUser'		 
						,ResponsiblePerson						AS 'ResponsiblePerson'
						,ImageName								AS 'ImageName'
						,[Status]								AS 'Status'
						,Template								AS 'Template'
						,GlobalAttribute1						AS 'GlobalAttribute1'
						,GlobalAttribute2						AS 'GlobalAttribute2'
						,GlobalAttribute3						AS 'GlobalAttribute3'
						,GlobalAttribute4						AS 'GlobalAttribute4'
						,GlobalAttribute5						AS 'GlobalAttribute5'
						,GlobalAttribute6						AS 'GlobalAttribute6'
						,GlobalAttribute7						AS 'GlobalAttribute7'
						,GlobalAttribute8						AS 'GlobalAttribute8'
						,GlobalAttribute9						AS 'GlobalAttribute9'
						,GlobalAttribute10						AS 'GlobalAttribute10'
						,GlobalAttribute11						AS 'GlobalAttribute11'
						,GlobalAttribute12						AS 'GlobalAttribute12'
						,GlobalAttribute13						AS 'GlobalAttribute13'
						,GlobalAttribute14						AS 'GlobalAttribute14'
						,GlobalAttribute15						AS 'GlobalAttribute15'
						,GlobalAttribute16						AS 'GlobalAttribute16'
						,GlobalAttribute17						AS 'GlobalAttribute17'
						,GlobalAttribute18						AS 'GlobalAttribute18'
						,GlobalAttribute19						AS 'GlobalAttribute19'
						,GlobalAttribute20						AS 'GlobalAttribute20'
						,GlobalAttribute21						AS 'GlobalAttribute21'
						,GlobalAttribute22						AS 'GlobalAttribute22'
						,GlobalAttribute23						AS 'GlobalAttribute23'
						,GlobalAttribute24						AS 'GlobalAttribute24'
						,GlobalAttribute25						AS 'GlobalAttribute25'
					FROM OPENJSON (@ValuesJSON, N'$.Values') 
					WITH (
						 ProductNumber			INT				N'$.ProductNumber'
						,FactoryID				NVARCHAR(255)	N'$.FactoryID'
						,ProductLineID			NVARCHAR(255)	N'$.ProductLineID'
						,ProductID				NVARCHAR(255)	N'$.ProductID'
						,NameShort				NVARCHAR(255)	N'$.NameShort'
						,CommentUser			NVARCHAR(255)	N'$.CommentUser'
						,ResponsiblePerson		NVARCHAR(255)	N'$.ResponsiblePerson'
						,ImageName				NVARCHAR(255)	N'$.ImageName'			
						,Status					NVARCHAR(255)	N'$.Status'			
						,Template				NVARCHAR(255)	N'$.Template'		
						,GlobalAttribute1		NVARCHAR(255)	N'$.GlobalAttribute1'
						,GlobalAttribute2		NVARCHAR(255)	N'$.GlobalAttribute2'
						,GlobalAttribute3		NVARCHAR(255)	N'$.GlobalAttribute3'
						,GlobalAttribute4		NVARCHAR(255)	N'$.GlobalAttribute4'
						,GlobalAttribute5		NVARCHAR(255)	N'$.GlobalAttribute5'
						,GlobalAttribute6		NVARCHAR(255)	N'$.GlobalAttribute6'
						,GlobalAttribute7		NVARCHAR(255)	N'$.GlobalAttribute7'
						,GlobalAttribute8		NVARCHAR(255)	N'$.GlobalAttribute8'
						,GlobalAttribute9		NVARCHAR(255)	N'$.GlobalAttribute9'
						,GlobalAttribute10		NVARCHAR(255)	N'$.GlobalAttribute10'
						,GlobalAttribute11		NVARCHAR(255)	N'$.GlobalAttribute11'
						,GlobalAttribute12		NVARCHAR(255)	N'$.GlobalAttribute12'
						,GlobalAttribute13		NVARCHAR(255)	N'$.GlobalAttribute13'
						,GlobalAttribute14		NVARCHAR(255)	N'$.GlobalAttribute14'
						,GlobalAttribute15		NVARCHAR(255)	N'$.GlobalAttribute15'
						,GlobalAttribute16		NVARCHAR(255)	N'$.GlobalAttribute16'
						,GlobalAttribute17		NVARCHAR(255)	N'$.GlobalAttribute17'
						,GlobalAttribute18		NVARCHAR(255)	N'$.GlobalAttribute18'
						,GlobalAttribute19		NVARCHAR(255)	N'$.GlobalAttribute19'
						,GlobalAttribute20		NVARCHAR(255)	N'$.GlobalAttribute20'
						,GlobalAttribute21		NVARCHAR(255)	N'$.GlobalAttribute21'
						,GlobalAttribute22		NVARCHAR(255)	N'$.GlobalAttribute22'
						,GlobalAttribute23		NVARCHAR(255)	N'$.GlobalAttribute23'
						,GlobalAttribute24		NVARCHAR(255)	N'$.GlobalAttribute24'
						,GlobalAttribute25		NVARCHAR(255)	N'$.GlobalAttribute25'
						)
					)

			SELECT 	
				 ProductNumber 
				,FactoryID				COLLATE DATABASE_DEFAULT  AS FactoryID
				,ProductLineID			COLLATE DATABASE_DEFAULT  AS ProductLineID
				,ProductID				COLLATE DATABASE_DEFAULT  AS ProductID
				,NameShort				COLLATE DATABASE_DEFAULT  AS NameShort			
				,CommentUser			COLLATE DATABASE_DEFAULT  AS CommentUser		
				,ResponsiblePerson		COLLATE DATABASE_DEFAULT  AS ResponsiblePerson	
				,ImageName				COLLATE DATABASE_DEFAULT  AS ImageName			
				,[Status]				COLLATE DATABASE_DEFAULT  AS [Status]			
				,Template				COLLATE DATABASE_DEFAULT  AS Template			
				,GlobalAttribute1		COLLATE DATABASE_DEFAULT  AS GlobalAttribute1	
				,GlobalAttribute2		COLLATE DATABASE_DEFAULT  AS GlobalAttribute2	
				,GlobalAttribute3		COLLATE DATABASE_DEFAULT  AS GlobalAttribute3	
				,GlobalAttribute4		COLLATE DATABASE_DEFAULT  AS GlobalAttribute4	
				,GlobalAttribute5		COLLATE DATABASE_DEFAULT  AS GlobalAttribute5	
				,GlobalAttribute6		COLLATE DATABASE_DEFAULT  AS GlobalAttribute6	
				,GlobalAttribute7		COLLATE DATABASE_DEFAULT  AS GlobalAttribute7	
				,GlobalAttribute8		COLLATE DATABASE_DEFAULT  AS GlobalAttribute8	
				,GlobalAttribute9		COLLATE DATABASE_DEFAULT  AS GlobalAttribute9	
				,GlobalAttribute10		COLLATE DATABASE_DEFAULT  AS GlobalAttribute10	
				,GlobalAttribute11		COLLATE DATABASE_DEFAULT  AS GlobalAttribute11	
				,GlobalAttribute12		COLLATE DATABASE_DEFAULT  AS GlobalAttribute12	
				,GlobalAttribute13		COLLATE DATABASE_DEFAULT  AS GlobalAttribute13	
				,GlobalAttribute14		COLLATE DATABASE_DEFAULT  AS GlobalAttribute14	
				,GlobalAttribute15		COLLATE DATABASE_DEFAULT  AS GlobalAttribute15	
				,GlobalAttribute16		COLLATE DATABASE_DEFAULT  AS GlobalAttribute16	
				,GlobalAttribute17		COLLATE DATABASE_DEFAULT  AS GlobalAttribute17	
				,GlobalAttribute18		COLLATE DATABASE_DEFAULT  AS GlobalAttribute18	
				,GlobalAttribute19		COLLATE DATABASE_DEFAULT  AS GlobalAttribute19	
				,GlobalAttribute20		COLLATE DATABASE_DEFAULT  AS GlobalAttribute20	
				,GlobalAttribute21		COLLATE DATABASE_DEFAULT  AS GlobalAttribute21	
				,GlobalAttribute22		COLLATE DATABASE_DEFAULT  AS GlobalAttribute22	
				,GlobalAttribute23		COLLATE DATABASE_DEFAULT  AS GlobalAttribute23	
				,GlobalAttribute24		COLLATE DATABASE_DEFAULT  AS GlobalAttribute24	
				,GlobalAttribute25		COLLATE DATABASE_DEFAULT  AS GlobalAttribute25	
			INTO #SendedRecords	
			FROM vals;

			-- Debug: 
			-- SELECT * FROM #SendedRecords
	
			-- Check for logical correct values #############################################
			PRINT 'Starting Check for logical correct Values'	
			-- no NULL Values contained in IDs
			IF EXISTS (
				SELECT TOP(1) 1 
				FROM #SendedRecords 
				WHERE 
					   ProductNumber	IS NULL
					OR FactoryID		IS NULL
					OR ProductLineID	IS NULL
					OR ProductID		IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					
					)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid Factory/PL/P IDs.', 100;
			END;

			-- no doubled ProductNumbers contained
			IF EXISTS (
					SELECT TOP (1) 1
					FROM #SendedRecords
					GROUP BY ProductNumber
					HAVING COUNT(ProductNumber) > 1
				)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to duplicate ProductNumbers.', 100;
			END;

			-- User Rights ##################################################################
			PRINT 'Starting User Rights Check'
			-- Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Build classification Table for sended Records  #################################
			PRINT 'Starting classification Process'
			DROP TABLE IF EXISTS #SendedRecordsClassification;

			CREATE TABLE #SendedRecordsClassification
				(
					 ProductNumber				INT											NOT NULL 
					,FactoryID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,NameShort					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,CommentUser				NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,ResponsiblePerson			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,ImageName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,Status						NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,Template					NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute1			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute2			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute3			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute4			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute5			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute6			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute7			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute8			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute9			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute10			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute11			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute12			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute13			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute14			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute15			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute16			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute17			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute18			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute19			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute20			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute21			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute22			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute23			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute24			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,GlobalAttribute25			NVARCHAR(255)	COLLATE DATABASE_DEFAULT		NULL 
					,UserRight					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,InvalidTemplateFlag		INT											NOT NULL 
					,TargetIDsNotExistingFlag	INT											NOT NULL 
					,MultipleRecordsFlag	INT											NOT NULL  -- if more than one record per Product is sent, only the last one is posted 
					,ReturnCode					INT											NOT NULL 
					,StatusComment				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductKey					BIGINT										NOT NULL 
					,SaveRecordFlag				INT											NOT NULL 
				);

			INSERT INTO #SendedRecordsClassification 
				SELECT 	
					 ProductNumber												AS ValueNumber
					,FactoryID					COLLATE DATABASE_DEFAULT		AS FactoryID
					,ProductLineID				COLLATE DATABASE_DEFAULT		AS ProductLineID
					,ProductID					COLLATE DATABASE_DEFAULT		AS ProductID
					,NameShort					COLLATE DATABASE_DEFAULT		AS NameShort			
					,CommentUser				COLLATE DATABASE_DEFAULT		AS CommentUser		
					,ResponsiblePerson			COLLATE DATABASE_DEFAULT		AS ResponsiblePerson	
					,ImageName					COLLATE DATABASE_DEFAULT		AS ImageName			
					,Status						COLLATE DATABASE_DEFAULT		AS Status				
					,Template					COLLATE DATABASE_DEFAULT		AS Template			
					,GlobalAttribute1			COLLATE DATABASE_DEFAULT		AS GlobalAttribute1	
					,GlobalAttribute2			COLLATE DATABASE_DEFAULT		AS GlobalAttribute2	
					,GlobalAttribute3			COLLATE DATABASE_DEFAULT		AS GlobalAttribute3	
					,GlobalAttribute4			COLLATE DATABASE_DEFAULT		AS GlobalAttribute4	
					,GlobalAttribute5			COLLATE DATABASE_DEFAULT		AS GlobalAttribute5	
					,GlobalAttribute6			COLLATE DATABASE_DEFAULT		AS GlobalAttribute6	
					,GlobalAttribute7			COLLATE DATABASE_DEFAULT		AS GlobalAttribute7	
					,GlobalAttribute8			COLLATE DATABASE_DEFAULT		AS GlobalAttribute8	
					,GlobalAttribute9			COLLATE DATABASE_DEFAULT		AS GlobalAttribute9	
					,GlobalAttribute10			COLLATE DATABASE_DEFAULT		AS GlobalAttribute10	
					,GlobalAttribute11			COLLATE DATABASE_DEFAULT		AS GlobalAttribute11	
					,GlobalAttribute12			COLLATE DATABASE_DEFAULT		AS GlobalAttribute12	
					,GlobalAttribute13			COLLATE DATABASE_DEFAULT		AS GlobalAttribute13	
					,GlobalAttribute14			COLLATE DATABASE_DEFAULT		AS GlobalAttribute14	
					,GlobalAttribute15			COLLATE DATABASE_DEFAULT		AS GlobalAttribute15	
					,GlobalAttribute16			COLLATE DATABASE_DEFAULT		AS GlobalAttribute16	
					,GlobalAttribute17			COLLATE DATABASE_DEFAULT		AS GlobalAttribute17	
					,GlobalAttribute18			COLLATE DATABASE_DEFAULT		AS GlobalAttribute18	
					,GlobalAttribute19			COLLATE DATABASE_DEFAULT		AS GlobalAttribute19	
					,GlobalAttribute20			COLLATE DATABASE_DEFAULT		AS GlobalAttribute20	
					,GlobalAttribute21			COLLATE DATABASE_DEFAULT		AS GlobalAttribute21	
					,GlobalAttribute22			COLLATE DATABASE_DEFAULT		AS GlobalAttribute22	
					,GlobalAttribute23			COLLATE DATABASE_DEFAULT		AS GlobalAttribute23	
					,GlobalAttribute24			COLLATE DATABASE_DEFAULT		AS GlobalAttribute24	
					,GlobalAttribute25			COLLATE DATABASE_DEFAULT		AS GlobalAttribute25	
				
					,''												AS UserRight
					,0												AS InvalidTemplateFlag
					,0												AS TargetIDsNotExistingFlag
					,0												AS MultipleRecordsFlag 
					,501											AS ReturnCode
					,N'Original'		COLLATE DATABASE_DEFAULT	AS StatusComment
					,0												AS ProductKey
					,0												AS SaveRecordFlag	
				FROM #SendedRecords;
	
			-- add ProductKey and therefore check for the existence of the targeted products
			UPDATE SRC 
			SET
				 SRC.ProductKey		= dp.ProductKey
			FROM #SendedRecordsClassification SRC
				INNER JOIN planning.tdProducts dp  -- only for hits
					ON 
							SRC.FactoryID		= dp.FactoryID
						AND SRC.ProductLineID	= dp.ProductLineID
						AND SRC.ProductID		= dp.ProductID

			-- Flag target products which don't exist
			UPDATE #SendedRecordsClassification 
			SET 
				 TargetIDsNotExistingFlag	= 1
				,ReturnCode					= 404
				,StatusComment				= N'Error: Product does not exist'
				,SaveRecordFlag				= -1
			WHERE ProductKey = 0;

			-- determine UserRights
			UPDATE SRC 
			SET
				SRC.UserRight = tUR.[Right]	
			FROM #SendedRecordsClassification SRC
				LEFT JOIN system.trUserRights tUR
				ON 
						SRC.FactoryID		= tUR.FactoryID
					AND SRC.ProductLineID	= tUR.ProductLineID
					AND tUR.UserName		= @TransactionUsername
			WHERE  tUR.[Right] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
			SET 
				 SaveRecordFlag = -1 
				,ReturnCode = 403
				,StatusComment = 'No write rights.'
			WHERE 
					UserRight <> 'Write'
				AND SaveRecordFlag <> -1;


			-- multiple records per product; only the last one will be posted
			UPDATE #SendedRecordsClassification 
			SET
					MultipleRecordsFlag		= 1
				,ReturnCode					= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,200,204)
				,StatusComment				= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,'Saved record (Multiple records per product)','Multiple records per product, only the last one has been posted')
				,SaveRecordFlag				= IIF(mf.Anzahl > 1 AND mf.Anzahl = mf.Zeile,0,-1)
			FROM #SendedRecordsClassification SRC
				RIGHT JOIN (
					SELECT
							ProductNumber
						,FactoryID
						,ProductLineID
						,ProductID
						,ProductKey
						,COUNT(ProductKey) OVER(PARTITION BY ProductKey)								AS Anzahl
						,ROW_NUMBER() OVER(PARTITION BY ProductKey ORDER BY ProductNumber ASC)			AS Zeile
					FROM #SendedRecordsClassification
					WHERE ProductKey IS NOT NULL
						AND SaveRecordFlag <> -1) mf
					ON SRC.ProductNumber = mf.ProductNumber
			WHERE mf.Anzahl > 1
	
			-- Flag invalid template types (valid are: _VM, _VT, _VW, _VJ, _HM, _HT, _HW, _HJ); old type will be kept (rest will still be posted)
			UPDATE #SendedRecordsClassification 
			SET 
					Template				= NULL
				,InvalidTemplateFlag	= 1
				,ReturnCode				= 200
				,StatusComment			= IIF(MultipleRecordsFlag = 1, 'Partially saved record (Invalid template type, old type is kept; multiple records per product)','Partially saved record (Invalid template type, old type is kept)')
				,SaveRecordFlag			= 1
			FROM #SendedRecordsClassification SRC
			WHERE SRC.Template IS NOT NULL
				AND (LEFT(RIGHT(SRC.Template,3),2) NOT IN ('_V', '_H')
				OR RIGHT(SRC.Template,1) NOT IN ('M','T','W','J'))
				AND SRC.SaveRecordFlag <> -1

			-- Flag all remaining records as to be saved
			UPDATE #SendedRecordsClassification 
			SET	
					SaveRecordFlag		= 1
				,ReturnCode			= 200
				,StatusComment		= IIF(MultipleRecordsFlag = 1, StatusComment,'Saved record')
			WHERE SaveRecordFlag	= 0;

			-- Debug: 
			-- SELECT * FROM #SendedRecordsClassification src

			-- Determination of the final record set and update of product values #######################################################
			DROP TABLE IF EXISTS #SavedRecords;

			CREATE TABLE #SavedRecords
				(
					 ProductNumber				BIGINT										NOT NULL
					,ProductKey					BIGINT										NOT NULL
					,NameShort					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,CommentUser				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ResponsiblePerson			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ImageName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,Status						NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,Template					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute1			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute2			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute3			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute4			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute5			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute6			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute7			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute8			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute9			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute10			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute11			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute12			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute13			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute14			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute15			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute16			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute17			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute18			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute19			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute20			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute21			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute22			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute23			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute24			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,GlobalAttribute25			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
				)
			
			-- old values in case of NULL
			PRINT 'Determination of old values for sended NULL values'
			INSERT INTO #SavedRecords
				SELECT
					 SRC.ProductNumber
					,SRC.ProductKey		
					,IIF(SRC.NameShort			IS NULL,	dp.NameShort		,	SRC.NameShort			)		
					,IIF(SRC.CommentUser		IS NULL,	dp.CommentUser		,	SRC.CommentUser		)	
					,IIF(SRC.ResponsiblePerson	IS NULL,	dp.ResponsiblePerson,	SRC.ResponsiblePerson	)
					,IIF(SRC.ImageName			IS NULL,	dp.ImageName		,	SRC.ImageName			)		
					,IIF(SRC.Status				IS NULL,	dp.Status			,	SRC.Status				)		
					,IIF(SRC.Template			IS NULL,	dp.Template			,	SRC.Template			)	
					,IIF(SRC.GlobalAttribute1	IS NULL,	dp.GlobalAttribute1	,	SRC.GlobalAttribute1	)
					,IIF(SRC.GlobalAttribute2	IS NULL,	dp.GlobalAttribute2	,	SRC.GlobalAttribute2	)
					,IIF(SRC.GlobalAttribute3	IS NULL,	dp.GlobalAttribute3	,	SRC.GlobalAttribute3	)
					,IIF(SRC.GlobalAttribute4	IS NULL,	dp.GlobalAttribute4	,	SRC.GlobalAttribute4	)
					,IIF(SRC.GlobalAttribute5	IS NULL,	dp.GlobalAttribute5	,	SRC.GlobalAttribute5	)
					,IIF(SRC.GlobalAttribute6	IS NULL,	dp.GlobalAttribute6	,	SRC.GlobalAttribute6	)
					,IIF(SRC.GlobalAttribute7	IS NULL,	dp.GlobalAttribute7	,	SRC.GlobalAttribute7	)
					,IIF(SRC.GlobalAttribute8	IS NULL,	dp.GlobalAttribute8	,	SRC.GlobalAttribute8	)
					,IIF(SRC.GlobalAttribute9	IS NULL,	dp.GlobalAttribute9	,	SRC.GlobalAttribute9	)
					,IIF(SRC.GlobalAttribute10	IS NULL,	dp.GlobalAttribute10,	SRC.GlobalAttribute10	)
					,IIF(SRC.GlobalAttribute11	IS NULL,	dp.GlobalAttribute11,	SRC.GlobalAttribute11	)
					,IIF(SRC.GlobalAttribute12	IS NULL,	dp.GlobalAttribute12,	SRC.GlobalAttribute12	)
					,IIF(SRC.GlobalAttribute13	IS NULL,	dp.GlobalAttribute13,	SRC.GlobalAttribute13	)
					,IIF(SRC.GlobalAttribute14	IS NULL,	dp.GlobalAttribute14,	SRC.GlobalAttribute14	)
					,IIF(SRC.GlobalAttribute15	IS NULL,	dp.GlobalAttribute15,	SRC.GlobalAttribute15	)
					,IIF(SRC.GlobalAttribute16	IS NULL,	dp.GlobalAttribute16,	SRC.GlobalAttribute16	)
					,IIF(SRC.GlobalAttribute17	IS NULL,	dp.GlobalAttribute17,	SRC.GlobalAttribute17	)
					,IIF(SRC.GlobalAttribute18	IS NULL,	dp.GlobalAttribute18,	SRC.GlobalAttribute18	)
					,IIF(SRC.GlobalAttribute19	IS NULL,	dp.GlobalAttribute19,	SRC.GlobalAttribute19	)
					,IIF(SRC.GlobalAttribute20	IS NULL,	dp.GlobalAttribute20,	SRC.GlobalAttribute20	)
					,IIF(SRC.GlobalAttribute21	IS NULL,	dp.GlobalAttribute21,	SRC.GlobalAttribute21	)
					,IIF(SRC.GlobalAttribute22	IS NULL,	dp.GlobalAttribute22,	SRC.GlobalAttribute22	)
					,IIF(SRC.GlobalAttribute23	IS NULL,	dp.GlobalAttribute23,	SRC.GlobalAttribute23	)
					,IIF(SRC.GlobalAttribute24	IS NULL,	dp.GlobalAttribute24,	SRC.GlobalAttribute24	)
					,IIF(SRC.GlobalAttribute25	IS NULL,	dp.GlobalAttribute25,	SRC.GlobalAttribute25	)
				FROM #SendedRecordsClassification SRC
					LEFT JOIN planning.tdProducts dp
						ON SRC.ProductKey = dp.ProductKey
				WHERE SRC.SaveRecordFlag	= 1

			-- Insert new Values
			PRINT 'Update of products'

			UPDATE dp 
				SET 		
					 dp.NameShort			= SR.NameShort			
					,dp.CommentUser			= SR.CommentUser		
					,dp.ResponsiblePerson	= SR.ResponsiblePerson	
					,dp.ImageName			= SR.ImageName			
					,dp.Status				= SR.Status				
					,dp.Template			= SR.Template			
					,dp.GlobalAttribute1	= SR.GlobalAttribute1	
					,dp.GlobalAttribute2	= SR.GlobalAttribute2	
					,dp.GlobalAttribute3	= SR.GlobalAttribute3	
					,dp.GlobalAttribute4	= SR.GlobalAttribute4	
					,dp.GlobalAttribute5	= SR.GlobalAttribute5	
					,dp.GlobalAttribute6	= SR.GlobalAttribute6	
					,dp.GlobalAttribute7	= SR.GlobalAttribute7	
					,dp.GlobalAttribute8	= SR.GlobalAttribute8	
					,dp.GlobalAttribute9	= SR.GlobalAttribute9	
					,dp.GlobalAttribute10	= SR.GlobalAttribute10	
					,dp.GlobalAttribute11	= SR.GlobalAttribute11	
					,dp.GlobalAttribute12	= SR.GlobalAttribute12	
					,dp.GlobalAttribute13	= SR.GlobalAttribute13	
					,dp.GlobalAttribute14	= SR.GlobalAttribute14	
					,dp.GlobalAttribute15	= SR.GlobalAttribute15	
					,dp.GlobalAttribute16	= SR.GlobalAttribute16	
					,dp.GlobalAttribute17	= SR.GlobalAttribute17	
					,dp.GlobalAttribute18	= SR.GlobalAttribute18	
					,dp.GlobalAttribute19	= SR.GlobalAttribute19	
					,dp.GlobalAttribute20	= SR.GlobalAttribute20	
					,dp.GlobalAttribute21	= SR.GlobalAttribute21	
					,dp.GlobalAttribute22	= SR.GlobalAttribute22	
					,dp.GlobalAttribute23	= SR.GlobalAttribute23	
					,dp.GlobalAttribute24	= SR.GlobalAttribute24	
					,dp.GlobalAttribute25	= SR.GlobalAttribute25	
				FROM planning.tdproducts dp
					INNER JOIN	#SavedRecords SR
						ON dp.ProductKey = SR.ProductKey
					
			-- Create result table to be returned #################################################

			DROP TABLE IF EXISTS #ResultSet;

			CREATE TABLE #ResultSet (
				 ProductNumber		INT										 NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,NameShort			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	 NULL 
				,CommentUser		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ResponsiblePerson	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ImageName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,[Status]			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,Template			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute1	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute2	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute3	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute4	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute5	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute6	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute7	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute8	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute9	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute10	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute11	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute12	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute13	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute14	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute15	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute16	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute17	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute18	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute19	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute20	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute21	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute22	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute23	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute24	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,GlobalAttribute25	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	 NULL
				,ReturnCode			INT											 NULL
				,ReturnComment		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	 NULL
				);
			
			-- Fill result table with requested values

			PRINT 'Start filling result set.'
			INSERT INTO #ResultSet 
				SELECT 
					 ProductNumber
					,FactoryID
					,ProductLineID
					,ProductID
					,NameShort			
					,CommentUser		
					,ResponsiblePerson
					,ImageName			
					,[Status]			
					,Template			
					,GlobalAttribute1
					,GlobalAttribute2
					,GlobalAttribute3
					,GlobalAttribute4
					,GlobalAttribute5
					,GlobalAttribute6
					,GlobalAttribute7
					,GlobalAttribute8
					,GlobalAttribute9
					,GlobalAttribute10
					,GlobalAttribute11
					,GlobalAttribute12
					,GlobalAttribute13
					,GlobalAttribute14
					,GlobalAttribute15
					,GlobalAttribute16
					,GlobalAttribute17
					,GlobalAttribute18
					,GlobalAttribute19
					,GlobalAttribute20
					,GlobalAttribute21
					,GlobalAttribute22
					,GlobalAttribute23
					,GlobalAttribute24
					,GlobalAttribute25
					,0
					,''
				FROM #SendedRecords;
		
			-- Set Return Codes an comments
			PRINT 'Setting ReturnCodes in Result Set.'
			UPDATE RS	
				SET 
					 RS.ReturnCode		= SRC.ReturnCode
					,RS.ReturnComment	= SRC.StatusComment
			FROM #ResultSet RS
				LEFT JOIN #SendedRecordsClassification SRC
					ON RS.ProductNumber = SRC.ProductNumber;

			DECLARE @ProcessTime INT;
			SET @ProcessTime = DATEDIFF(ms, @TimestampCall, GETUTCDATE());
			SET @ResultCode = 200;

			-- Return result
			SELECT  
					 *
					,@ProcessTime AS ProcessTime
			FROM #ResultSet;
		
			COMMIT TRANSACTION spPOST_Products_JSON;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			
			ROLLBACK TRANSACTION spPOST_Products_JSON;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;
GO
		
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Products_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to POST Values in JSON Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'Valus which are to be saved as JSON object.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS planning.spPOST_Tab;
GO

/*
	Procedure to POST a Tab for
		- a cluster
		- a factory
		- a productline
		- a product
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spPOST_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= 'Td5dd'
				,@OrderIndex		= '3'
				,@TabName			= 'Name'
				,@TabHint			= 'Hint'
				,@TabText			= 'Text'
				,@PresentationCODE	= 'Pivot'
				,@Datasource		= 'fdafds'
				,@IsVisibleBOOL		= '1'
				,@LayoutJSON		= 'dafds'
				,@ParameterJSON		= 'fdafd';
		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex;

		TRUNCATE TABLE planning.tTabs;

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Tab','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255) 
		,@OrderIndex			INT 
		,@TabName				NVARCHAR(255) 
		,@TabHint				NVARCHAR(2000) 
		,@TabText				NVARCHAR(2000) 
		,@PresentationCODE		NVARCHAR(255) 
		,@Datasource			NVARCHAR(2000) 
		,@IsVisibleBOOL			INT
		,@LayoutJSON			NVARCHAR(MAX) 
		,@ParameterJSON			NVARCHAR(MAX) 

    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''','
				+ CAST(ISNULL(@OrderIndex		,N'NULL') AS NVARCHAR(255)) + N','''
				+ ISNULL(@TabName				,N'NULL')					+ N''','''
				+ ISNULL(@TabHint				,N'NULL')					+ N''','''
				+ ISNULL(@TabText				,N'NULL')					+ N''','''
				+ ISNULL(@PresentationCODE		,N'NULL')					+ N''','''
				+ ISNULL(@Datasource			,N'NULL')					+ N''','
				+ CAST(ISNULL(@IsVisibleBOOL	,N'NULL') AS NVARCHAR(255)) + N','''
				+ ISNULL(@LayoutJSON			,N'NULL')					+ N''','''
				+ ISNULL(@ParameterJSON			,N'NULL')					+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0
				,@OldOrderIndex		INT			= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';
		IF @OrderIndex			IS NULL SET @OrderIndex			= 0  ;
		IF @TabName				IS NULL SET @TabName			= N'';
		IF @TabHint				IS NULL SET @TabHint			= N'';
		IF @TabText				IS NULL SET @TabText			= N'';
		IF @PresentationCODE	IS NULL SET @PresentationCODE	= N'';
		IF @Datasource			IS NULL SET @Datasource			= N'';
		IF @IsVisibleBOOL		IS NULL SET @IsVisibleBOOL		= 0  ;
		IF @LayoutJSON			IS NULL SET @LayoutJSON			= N'';
		IF @ParameterJSON		IS NULL SET @ParameterJSON		= N'';

		-- dont use negative index values
		IF @OrderIndex			< 0		SET @OrderIndex			= 1;
		-- protect bool
		IF @IsVisibleBOOL      <> 1		SET @IsVisibleBOOL		= 0;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;
				
				-- error if and ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR(N'Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- error if ID chains has gaps
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <> -1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				SET @ResultCode = 501 -- reset

				-- determine if posted Tab already exists
				SELECT 
					 @Rowkey = RowKey
					,@OldOrderIndex = OrderIndex
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- data operation starts	***********************************************************************************
				-- delete if exits, to determine Resultcode
				IF @RowKey > 0 
					BEGIN
						DELETE FROM planning.tTabs WHERE RowKey = @RowKey
						SET @Resultcode = 200
					END
					
				-- System Tab with TabID 0 can only change visibility
				IF @TabID = '0'
					BEGIN
						INSERT INTO planning.tTabs
							VALUES
							(
								 @FactoryKey
								,@ProductLineKey
								,@ProductKey
								,@FactoryID
								,@ProductLineID
								,@ProductID
								,@TabID
								,0
								,@TabName
								,N''
								,N''
								,N'System'
								,N''
								,@IsVisibleBOOL
								,N''
								,N''
							)
					END
				ELSE
					BEGIN
						-- Insert new Tab
						INSERT INTO planning.tTabs
							VALUES
							(
								 @FactoryKey
								,@ProductLineKey
								,@ProductKey
								,@FactoryID
								,@ProductLineID
								,@ProductID
								,@TabID
								,@OrderIndex
								,@TabName
								,@TabHint
								,@TabText
								,@PresentationCODE
								,@Datasource
								,@IsVisibleBOOL
								,@LayoutJSON
								,@ParameterJSON
							)
					
						-- Update other numbers if Number is changed or new
						IF @OldOrderIndex <> @OrderIndex
							BEGIN			
								-- case if OrderIndex is bigger now and existed before - decrement all in move window
								IF @OldOrderIndex < @OrderIndex AND @RowKey <> 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex - 1 
										WHERE 
												OrderIndex		>= @OldOrderIndex 
											AND OrderIndex		<= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END
								-- case if OrderIndex is smaller now and existed before - increment all in move window
								IF @OldOrderIndex > @OrderIndex AND @RowKey <> 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex + 1 
										WHERE 
												OrderIndex		<= @OldOrderIndex 
											AND OrderIndex		>= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END

								-- case new Tab - increment all following
								IF @RowKey = 0
									BEGIN
										UPDATE planning.tTabs
										SET OrderIndex = OrderIndex + 1 
										WHERE 
												OrderIndex		>= @OrderIndex
											AND	FactoryKey		= @FactoryKey
											AND ProductLineKey	= @ProductLineKey
											AND ProductKey		= @ProductKey
											AND TabID			<> @TabID
									END

								-- eliminate all Gaps
								UPDATE planning.tTabs 
								SET OrderIndex = Rownumbers.Number
								FROM planning.tTabs tT INNER JOIN (
																			SELECT 
																				 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																				,TabID
																	
																			FROM planning.tTabs
																			WHERE 
																					FactoryKey		= @FactoryKey
																				AND ProductLineKey	= @ProductLineKey
																				AND ProductKey		= @ProductKey

																	) AS Rownumbers
								ON 
										tT.TabID			= Rownumbers.TabID
									AND tT.FactoryKey		= @FactoryKey
									AND tT.ProductLineKey	= @ProductLineKey
									AND tT.ProductKey		= @ProductKey

							END		
					END

				IF @ResultCode <> 200 SET @ResultCode = 201;

			COMMIT TRANSACTION spPOST_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Tab'		-- enter procedure name
    ,@SX_Owner          NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module         NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Tab. The Tab with ID 0 is the special system tab, but does not get special handling in databae.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if POST is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if POST is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if POST is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'Business Identifier for the Tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderIndex';
    SET @value      = N'Position in the list of Tabs. Any Number > 0 can be passed in, tabs reorder themself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 	SET @level2name = N'@TabName';
    SET @value      = N'Name of the Tab, used as Label.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabHint';
    SET @value      = N'Hint on the Tab, displayed as Tooltip.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabText';
    SET @value      = N'Text for the headerarea inside the tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PresentationCODE';
    SET @value      = N'CODE word for the type of presentation, e.g PIVOT, DATAGRID, HTML_URL, HTML_STATIC.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Datasource';
    SET @value      = N'Datasource fitting to PresentationCODE - name of the stored procedure or URL.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsVisibleBool';
    SET @value      = N'1 for visible, 0 for not visible';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutJSON';
    SET @value      = N'Layout definition as JSON String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ParameterJSON';
    SET @value      = N'Parameter definition (List and default value) as JSON String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS planning.spPOST_Values_JSON;
GO

/*
	The Procedure returns all sended values as recordset with the process result
 
	Procedure to POST Values in JSON Format 
	Preconditions
	- A value can be sended only on full qualified single fact -> There is no write on summary level possible
	- multiple values may be allocated to one Valueseries / TimeID - they must be aggregated
	- A value "NULL" can be sended, this means delete the existing value

	Functionality:
		- Values for not existing ValueSeriesID are thrown away and reported as ignored
		- Values for not existing TimeIDs are thrown away and reported as ignored
		- only values for elements where the user has write rights are excepted
	
		- flag values without writerights
		- delete existing values for given F/PL/P/VS in table fValues
		- Scale numeric values based on scale in dValueSeries

		- Only values of type string are sended, DB must put it in the right type
		- if in dValueSeries 
			IsNumeric = 0 -> put Value in ValueString
			IsNumeric = 1 -> put it in ValueInt, after scaling with Scale. Decimal separator is the dot "."
		- Special cases
			IsNumeric = 0 AND Value IS NULL -> DELETE, no write
			IsNumeric = 1 AND Value IS NULL -> DELETE, no write
			IsNumeric = 1 AND Value = 0 AND ValueSeriesID LIKE "_0" -> write

	Overhaul Workflow
		 - put all sended records in a temporary table (SendedRecords)
		 - create a temporary classification table with all sended F/PL/P/ID Values (SendedRecordClassification)
			- determine the ValueSeries in Classification table (Numeric, Source)
			- determine the UserRights in Classification table
			- determine additional properties
		 - create an temporary table for all values which have to be inserted in OCT (ResultSetForOCT)
		 - fill the ResultSetForOCT
			- with all Records which address Points the user has write rights
			- flag values which belong to not existing TimeIDs
			- aggregation of all records which address the same point
			- values are putted into type fitting columns
		 - flag NULL Values in ResultSetForOCT as Value to Delete
		 - Update SendedRecords to write Resultcodes / Comments about their fate in ResultSetForOCT
			- sended Summary Values not used
			- no Rights
			- not existing TimeIDs
			- aggregated and inserted
			- aggregated and deleted
			- deleted
			- inserted
			- which are sended on ValueSeries of Type XLS / XLS-String for Warning that formulas may ovewrite Result
		 - write ResultSetForOCT
		 - return SendedRecords

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC INT;
		DECLARE @ValuesJSON NVARCHAR(MAX);
			SET @ValuesJSON = N'{

				"connector_type":"write","version":1,"Values":[

				 {"ValueNumber":1	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K1"		,"TimeID":"20210115"	,"Value":100			,"ValueComment":null}
				,{"ValueNumber":2	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K1"		,"TimeID":"20210115"	,"Value":200			,"ValueComment":null}
				,{"ValueNumber":3	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K3"		,"TimeID":"20210115"	,"Value":0				,"ValueComment":null}
				,{"ValueNumber":4	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"BETR"		,"TimeID":"20210115"	,"Value":770000			,"ValueComment":null}
				,{"ValueNumber":5	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"BEM"		,"TimeID":"20210115"	,"Value":null			,"ValueComment":null}
				,{"ValueNumber":6	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"STAT_BUD"	,"TimeID":"20210115"	,"Value":"Genehmigt"	,"ValueComment":null}

			]

			}';

		EXEC @RC = planning.spPOST_Values_JSON 'SQL', @ValuesJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Values_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Values_JSON','PARAMETER',NULL);
*/   

CREATE PROCEDURE planning.spPOST_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY
			BEGIN TRANSACTION spPOST_Values_JSON
			-- INPUT PROTECTION	  #################################################################################

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (technical)', 100;
			END;
			
			-- Extract Values FROM JSON and put them into #SendedRecords #####################################

			DROP TABLE IF EXISTS #SendedRecords

			;WITH vals AS 
				(
					SELECT
						 ValueNumber							AS 'ValueNumber'
						,dbo.sx_pf_pProtectID (FactoryID)		AS 'FactoryID'
						,dbo.sx_pf_pProtectID (ProductLineID)	AS 'ProductLineID'
						,dbo.sx_pf_pProtectID (ProductID)		AS 'ProductID'
						,dbo.sx_pf_pProtectID (ValueSeriesID)	AS 'ValueSeriesID'
						,TimeID									AS 'TimeID'
						,[Value]								AS 'Value'				-- there maybe NULL inserted
						,ValueComment							AS 'ValueComment'
					FROM OPENJSON (@ValuesJSON, N'$.Values') 
					WITH (
						 ValueNumber	INT				N'$.ValueNumber'
						,FactoryID		NVARCHAR(255)	N'$.FactoryID'
						,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
						,ProductID		NVARCHAR(255)	N'$.ProductID'
						,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
						,TimeID			INT				N'$.TimeID'
						,[Value]		NVARCHAR(MAX)	N'$.Value'
						,ValueComment	NVARCHAR(MAX)	N'$.ValueComment'
						)
					)

			SELECT 	
				 ValueNumber 
				,FactoryID		COLLATE DATABASE_DEFAULT  AS FactoryID
				,ProductLineID	COLLATE DATABASE_DEFAULT  AS ProductLineID
				,ProductID		COLLATE DATABASE_DEFAULT  AS ProductID
				,ValueSeriesID	COLLATE DATABASE_DEFAULT  AS ValueSeriesID
				,TimeID		
				,[Value]	    COLLATE DATABASE_DEFAULT  AS [Value]
				,ValueComment	COLLATE DATABASE_DEFAULT  AS ValueComment
			INTO #SendedRecords	
			FROM vals;

			-- Debug: SELECT * FROM #SendedRecords

			-- Check for logical correct values #############################################
			PRINT 'Starting Check for logical correct Values'	
			-- no NULL Values contained in IDs - Value may be null due to deleted value
			IF EXISTS (
				SELECT TOP(1) 1 
				FROM #SendedRecords 
				WHERE 
					   ValueNumber		IS NULL
					OR FactoryID		IS NULL
					OR ProductLineID	IS NULL
					OR ProductID		IS NULL
					OR ValueSeriesID	IS NULL
					OR TimeID			IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					OR ValueSeriesID	= N''
					OR TimeID			= 0
					)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid Factory/PL/P/VS/T IDs.', 100;
			END;

			-- no doubled ValueNumbers contained
			IF EXISTS (
					SELECT TOP (1) 1
					FROM #SendedRecords
					GROUP BY ValueNumber
					HAVING COUNT(ValueNumber) > 1
				)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to doubled ValueNumbers.', 100;
			END;

			-- User Rights ##################################################################
			PRINT 'Starting User Rights Check'
			-- Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Build classification Table for sended Records  #################################
			PRINT 'Starting classification Process'
			DROP TABLE IF EXISTS #SendedRecordsClassification;

			CREATE TABLE #SendedRecordsClassification
				(
					 ValueNumber				INT											NOT NULL 
					,FactoryID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueSeriesID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,TimeID						BIGINT										NOT NULL 
					,[Value]					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	    NULL 
					,ValueComment				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	    NULL 
					,VS_IsNumeric				INT											NOT NULL 
					,Scale						INT											NOT NULL 
					,ValueSource				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueInt					BIGINT										NOT NULL 
					,ValueText					NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,UserRight					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,AggregationFlag			INT											NOT NULL 
					,DeleteFlag					INT											NOT NULL 
					,SaveValueZeroFlag			INT											NOT NULL 
					,TargetIDsNotExistingFlag	INT											NOT NULL 
					,ReturnCode					INT											NOT NULL 
					,StatusComment				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductKey					BIGINT										NOT NULL 
					,ValueSeriesKey				BIGINT										NOT NULL 
					,SaveValueFlag				INT											NOT NULL 
				);

			INSERT INTO #SendedRecordsClassification 
				SELECT 	
					 ValueNumber									AS ValueNumber
					,FactoryID		COLLATE DATABASE_DEFAULT		AS FactoryID
					,ProductLineID	COLLATE DATABASE_DEFAULT		AS ProductLineID
					,ProductID		COLLATE DATABASE_DEFAULT		AS ProductID
					,ValueSeriesID	COLLATE DATABASE_DEFAULT		AS ValueSeriesID
					,TimeID											AS TimeID
					,[Value]	    COLLATE DATABASE_DEFAULT		AS [Value]
					,ValueComment	COLLATE DATABASE_DEFAULT		AS ValueComment

					,0												AS VS_IsNumeric
					,0												AS Scale
					,N''			COLLATE DATABASE_DEFAULT		AS ValueSource
					,0												AS ValueInt
					,N''			COLLATE DATABASE_DEFAULT		AS ValueText
					,''												AS UserRight
					,0												AS AggregationFlag
					,0												AS DeleteFlag
					,0												AS SaveValueZeroFlag		-- for _0 ValueSeries
					,0												AS TargetIDsNotExistingFlag
					,501											AS ReturnCode
					,N'Original'		COLLATE DATABASE_DEFAULT	AS StatusComment
					,0												AS ProductKey
					,0												AS ValueSeriesKey
					,0												AS SaveValueFlag	
				FROM #SendedRecords;
			
			-- add ValuSeries Properties incl. ValueSeriesKey
			UPDATE SRC SET
				 SRC.VS_IsNumeric	= dVS.[IsNumeric]
				,SRC.Scale			= dVS.Scale
				,SRC.ValueSource	= dVS.ValueSource
				,SRC.ProductKey		= dVS.ProductKey
				,SRC.ValueSeriesKey = dVS.ValueSeriesKey
			FROM #SendedRecordsClassification SRC
				INNER JOIN planning.tdValueSeries dVS  -- only for hits
					ON 
							SRC.FactoryID		= dVS.FactoryID
						AND SRC.ProductLineID	= dVS.ProductLineID
						AND SRC.ProductID		= dVS.ProductID
						AND SRC.ValueSeriesID	= dVS.ValueSeriesID;
	
			-- Flag Target VSIDs which don't exist
			UPDATE #SendedRecordsClassification 
			SET 
				 TargetIDsNotExistingFlag	= 1
				,ReturnCode					= 404
				,StatusComment				= N'Error VSID does not exits'
				,SaveValueFlag				= -1
			WHERE ValueSeriesKey = 0;

			-- Flag Target TimeIDs which don't exist
			UPDATE SRC SET
				 SRC.TargetIDsNotExistingFlag = 1
				,SRC.ReturnCode = 404
				,SRC.StatusComment = N'Error TimeID does not exits'
				,SRC.SaveValueFlag = -1
			FROM #SendedRecordsClassification SRC
				LEFT JOIN planning.tdTime dT
				ON 
						SRC.ProductKey  = dT.ProductKey
					AND SRC.TimeID		= dT.TimeID
			WHERE 
					dT.TimeID IS NULL
				AND SRC.SaveValueFlag <> -1;

			-- Flag _0 ValueSeries
			UPDATE #SendedRecordsClassification  
				SET SaveValueZeroFlag = 1 
			WHERE 
					ValueSeriesID LIKE N'%_0'
				AND SaveValueFlag <> -1;

			-- determine UserRights
			UPDATE SRC SET
				 SRC.UserRight = tUR.[Right]	
			FROM #SendedRecordsClassification SRC
				LEFT JOIN system.trUserRights tUR
				ON 
						SRC.FactoryID		= tUR.FactoryID
					AND SRC.ProductLineID	= tUR.ProductLineID
					AND tUR.UserName		= @TransactionUsername
			WHERE  tUR.[Right] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
				SET 
					 SaveValueFlag = -1 
					,ReturnCode = 403
					,StatusComment = 'No write rights.'
				WHERE 
						UserRight <> 'Write'
					AND SaveValueFlag <> -1;

			-- Set ValueInt / ValueText based on Value and ValueSeries Type
			UPDATE #SendedRecordsClassification 
				SET ValueInt = CAST(COALESCE(TRY_CAST([Value] AS MONEY),0) * SCALE AS BIGINT)
			WHERE 
					VS_IsNumeric = 1
				AND [Value] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
				SET ValueText = [Value]
			WHERE 
					VS_IsNumeric = 0
				AND [Value] IS NOT NULL;
	
			-- Set a warning, if XLS / XLS-Strict values are overwritten
			UPDATE #SendedRecordsClassification 
				SET StatusComment =  StatusComment + ' Warning - XLS/XLS Strict formula field will be overwritten'
			WHERE 
					ValueSource IN ('XLS','XLS-Strict')
				
			-- Flag Values to be aggregated if they are numeric and NOT NULL and multiples are sended (from multiple sended Text Values, the last wins)
			UPDATE SRC SET
				 SRC.AggregationFlag	= 1
				,SRC.ReturnCode			= 200
				,SRC.StatusComment		= StatusComment + CONCAT(' Aggregated with ',Agg.ValueNumberCount - 1,' other values.')
				,SRC.SaveValueFlag		= 1
			FROM #SendedRecordsClassification SRC
				LEFT JOIN (
								SELECT 
									 COUNT(ValueNumber)					AS ValueNumberCount
									,SUM(TRY_CAST([Value] AS MONEY))	AS [Value]
									,FactoryID
									,ProductLineID
									,ProductID
									,ValueSeriesID
									,TimeID
								FROM #SendedRecordsClassification

								WHERE 
										VS_IsNumeric = 1
									AND SaveValueFlag <> -1
									AND [Value] IS NOT NULL				--NULLs are not marked for Aggregation in this step

								GROUP BY
									 FactoryID
									,ProductLineID
									,ProductID
									,ValueSeriesID
									,TimeID
								
								HAVING 
										COUNT(ValueNumber) > 1
									
							) AS Agg
					ON 
							Agg.FactoryID		= SRC.FactoryID
						AND Agg.ProductLineID	= SRC.ProductLineID
						AND Agg.ProductID		= SRC.ProductID
						AND Agg.ValueSeriesID	= SRC.ValueSeriesID
						AND Agg.TimeID			= SRC.TimeID

				WHERE Agg.ValueNumberCount IS NOT NULL;						

			-- Flag NULL Values as aggregated, if there are other values which write to the same poing
			UPDATE SRC
				 SET SRC.AggregationFlag	= 1
				,SRC.StatusComment			= 'Value was not deleted, as other Rows send values to this VS/TID.'
			FROM #SendedRecordsClassification SRC 
				LEFT JOIN (
								SELECT 
									 ValueSeriesKey
									,TimeID
								FROM #SendedRecordsClassification
								WHERE AggregationFlag = 1
						) Agg
					ON 
							SRC.ValueSeriesKey	= Agg.ValueSeriesKey
						AND SRC.TimeID			= Agg.TimeID
			WHERE 
				SRC.VS_IsNumeric = 1 
				AND SRC.[Value] IS NULL;			

			-- Flag Values to be deleted - but not if they are flagged for Aggregation
			UPDATE #SendedRecordsClassification 
				SET 
					 DeleteFlag		= 1 
					,ReturnCode		= 200
					,StatusComment	= 'Value deleted'
			WHERE 
					[Value] IS NULL
				AND AggregationFlag = 0
				AND SaveValueFlag <> -1;

			-- Flag Numeric Zero Values as to be deleted, except ValueSeries saving zeros
			UPDATE #SendedRecordsClassification
			SET		
				 DeleteFlag = 1 
				,ReturnCode = 200
				,StatusComment = 'Value deleted as 0 was send'
			WHERE 
					ValueInt = 0 
				AND VS_IsNumeric = 1
				AND SaveValueZeroFlag = 0
				AND SaveValueFlag <> -1;

			---- Flag Zero Values as not to be saved, except VS ValueSeries  - REPLACED by delete zero values to delete existing values
			--UPDATE #SendedRecordsClassification
			--SET		
			--	 SaveValueFlag = -1 
			--	,ReturnCode = 202
			--	,StatusComment = 'Not saved as value is zero.'
			--WHERE 
			--		ValueInt = 0 
			--	AND VS_IsNumeric = 1
			--	AND SaveValueZeroFlag = 0
			--	AND SaveValueFlag <> -1;

			-- Set Null Values for ValueComment to empty string
			UPDATE #SendedRecordsClassification
			SET		
					ValueComment = '' 
			WHERE 
					ValueComment IS NULL
				AND SaveValueFlag <> -1;

			-- Flag all remaining Value as to be saved
			UPDATE #SendedRecordsClassification 
				SET	
					 SaveValueFlag		= 1
					,ReturnCode			= 200
					,StatusComment		= 'Value saved'
				WHERE 
						SaveValueFlag	= 0
					AND DeleteFlag		= 0;

			SELECT * FROM #SendedRecordsClassification --DEBUG				
				
			-- Delete Values which have to be deleted ############################################
			PRINT 'Starting data manipulation'
			DELETE fV FROM planning.tfValues fV
				INNER JOIN  #SendedRecordsClassification SRC
					ON 
						SRC.ValueSeriesKey = fV.ValueSeriesKey 
					AND SRC.TimeID = fV.TimeID
				WHERE SRC.DeleteFlag = 1;

			-- Fill Values to #SavedRecords #######################################################
			DROP TABLE IF EXISTS #SavedRecords;

			CREATE TABLE #SavedRecords
				(
					 ValueSeriesKey	BIGINT									NOT NULL
					,TimeID			BIGINT									NOT NULL
					,ValueInt		BIGINT									NOT NULL
					,ValueText		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueComment	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				)
			
			-- the aggregated Values
			PRINT 'Filling SaveSet with aggregated values'
			INSERT INTO #SavedRecords
				SELECT
					 ValueSeriesKey
					,TimeID
					,SUM(ValueInt)		AS ValueInt
					,N''				AS ValueText
					,N''				AS ValueComment
				FROM #SendedRecordsClassification
				WHERE AggregationFlag = 1
				GROUP BY 
					 ValueSeriesKey
					,TimeID

			-- the Values to be saved
			PRINT 'Filling SaveSet with standard values'
			INSERT INTO #SavedRecords
				SELECT
					 ValueSeriesKey
					,TimeID
					,ValueInt		
					,ValueText
					,COALESCE(ValueComment,'')
				FROM #SendedRecordsClassification
				WHERE 
						SaveValueFlag	= 1
					AND DeleteFlag		= 0
					AND AggregationFlag = 0
					
			-- Check #SavedRecords for security for doubled values
			PRINT 'Start validation for SaveSet'
			IF EXISTS
				(SELECT 
					 TimeID
					,ValueSeriesKey
					,COUNT(ValueSeriesKey) AS Anzahl
				FROM #SavedRecords
				GROUP BY 
					 TimeID
					,ValueSeriesKey
				HAVING COUNT(ValueSeriesKey) > 1
				)
				BEGIN
					SET @ResultCode = 500;
					THROW 60000, N'Internal Error, Rows to save not unique.', 100;
				END

			-- Delete old Values
			PRINT 'Start deleting of old Values.'
			DELETE fV FROM planning.tfValues fV
				INNER JOIN  #SavedRecords SR
					ON 
						SR.ValueSeriesKey = fV.ValueSeriesKey 
					AND SR.TimeID = fV.TimeID
				WHERE 
							SR.ValueSeriesKey IS NOT NULL 
						AND SR.TimeID IS NOT NULL;

			-- Insert new Values
			PRINT 'Start insert new Values.'
			INSERT INTO planning.tfValues 
				SELECT 
					 SR.ValueSeriesKey
					,dV.ProductKey
					,dV.ProductLineKey
					,dV.FactoryKey
					,dV.ProductID
					,dV.ProductLineID
					,dV.FactoryID
					,dV.ValueSeriesID
					,SR.TimeID
					,'' AS ValueFormula
					,SR.ValueInt
					,SR.ValueText
					,SR.ValueComment
				FROM #SavedRecords SR
					LEFT JOIN planning.tdValueSeries dV
						ON SR.ValueSeriesKey = dV.ValueSeriesKey;

			-- Create result table to be returnd #################################################

			DROP TABLE IF EXISTS #ResultSet;

			CREATE TABLE #ResultSet (
				 ValueNumber		INT				NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				
				,TimeID				INT				NOT NULL

				,[Value]			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NULL
				,ValueComment		NVARCHAR(MAX)   COLLATE DATABASE_DEFAULT NULL

				,ReturnCode			INT				NULL
				,ReturnComment		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				);
			
			-- Fill result table with requested values

			PRINT 'Start filling result set.'
			INSERT INTO #ResultSet (ValueNumber, FactoryID, ProductLineID, ProductID, ValueSeriesID, 
									TimeID, [Value], ValueComment,ReturnCode,ReturnComment)
			SELECT ValueNumber
				,FactoryID
				,ProductLineID
				,ProductID
				,ValueSeriesID
				,TimeID
				,[Value]
				,ValueComment
				,0
				,''
			FROM #SendedRecords;
		
			-- Set Return Codes an comments
			PRINT 'Setting ReturnCodes in Result Set.'
			UPDATE RS	
				SET 
					 RS.ReturnCode		= SRC.ReturnCode
					,RS.ReturnComment	= SRC.StatusComment
			FROM #ResultSet RS
				LEFT JOIN #SendedRecordsClassification SRC
					ON RS.ValueNumber = SRC.ValueNumber;

			DECLARE @ProcessTime INT;
			SET @ProcessTime = DATEDIFF(ms, @TimestampCall, GETUTCDATE());
			SET @ResultCode = 200;

			-- Return result
			SELECT  
					 *
					,@ProcessTime AS ProcessTime
			FROM #ResultSet;

			COMMIT TRANSACTION spPOST_Values_JSON;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			
			ROLLBACK TRANSACTION spPOST_Values_JSON;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;
GO
		
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Values_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to POST Values in JSON Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'Valus which are to be saved as JSON object.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS result.spDataEntryNumericValues;
GO

/*
	Procedure to deliver all numeric values from data entry products to the enduser
	Expectes an optional parameter "Year" with one or multiple Years

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = ''
		DECLARE @ProductLineID	NVARCHAR(255) = ''
		DECLARE @ParameterJSON	NVARCHAR(MAX) = N'{"Year":[2020]}'

		EXECUTE @RC = result.spDataEntryNumericValues
		   @Username
		  ,@FactoryID
		  ,@ProductLineID 
		  ,@ParameterJSON

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryNumericValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryNumericValues','PARAMETER',NULL)
*/

CREATE PROCEDURE result.spDataEntryNumericValues 
	(
		 @Username		NVARCHAR(255)
		,@FactoryID		NVARCHAR(255) = N''
		,@ProductLineID NVARCHAR(255) = N''
		,@ParameterJSON NVARCHAR(MAX) = N''
	)
AS
	BEGIN
		SET NOCOUNT ON;
		DECLARE 
			@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N''''		
												+ ISNULL(@Username		,N'NULL')	+ N''','''											 
												+ ISNULL(@FactoryID		,N'NULL')	+ N''','''	
												+ ISNULL(@ProductLineID	,N'NULL')	+ N''','''	
												+ ISNULL(@ParameterJSON	,N'NULL')	+ N''''
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
		IF @ParameterJSON	IS NULL SET @ParameterJSON	= N'';

		-- Content Protection
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		
		DECLARE @years AS TABLE (Year INT NOT NULL);
		DECLARE @hasYears BIT = 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			-- Test for valid JSON
			SET @hasYears = ISJSON(@ParameterJSON);

			IF @ParameterJSON <> N'' AND @hasYears = 0
			BEGIN 
				SET @ResultCode = 403;
				RAISERROR('Exit due to invalid JSON', 16, 10);
			END;

			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactUsername = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- determine Years
			IF @hasYears = 1
			BEGIN 
				INSERT INTO @years
				SELECT DISTINCT value
				FROM OPENJSON(@ParameterJSON,'$.Year')
				WHERE value >= 0 AND value <= 9999;
			END;
			
			-- Helper Table for Userrights
			DROP TABLE IF EXISTS #tRights;
			CREATE TABLE #tRights
				(
					 RowKey					BIGINT IDENTITY (1,1)
					,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL

					CONSTRAINT PK_tRights PRIMARY KEY CLUSTERED (RowKey)			

				);

		  INSERT INTO #tRights
			SELECT 
				 FactoryID
				,ProductLineID
			FROM system.trUserRights 
			WHERE 
					Username = @TransactUsername 
				AND [Right] IN (N'Write', N'Read') 
				AND ProductLineID	<> N''
				AND FactoryID		<> N''
				AND (
						-- no Filter Paramter given
						(@FactoryID = N'' AND @ProductLineID = N'') 
						OR
								
							(	-- FactoryID Filter given
								FactoryID = @FactoryID 
								-- optional also ProductLine Filter given
							AND (@ProductLineID = N'' OR ProductLineID = @ProductLineID)
							)
					);

			SELECT 
				 fV.FactoryID
				,fV.ProductLineID
				,fV.ProductID
				,fV.ValueSeriesID

				,fV.TimeID / 10000															AS [Year]
				,fV.TimeID % 10000 / 100													AS [Month]
				,fV.TimeID % 100															AS [Day]
				,TRY_CAST(CAST(fV.TimeID AS NVARCHAR) AS DATE)								AS [Date]
			
				,TRY_CAST(fV.ValueInt AS Money) / ISNULL(dVS.Scale, 1)						AS [Value]

				,fV.FactoryID + N' ' + dF.NameShort					AS FactoryIDName 
				,dF.NameShort										AS FactoryName
				,dF.ResponsiblePerson								AS FResponsiblePerson

				,fV.ProductLineID + N' ' + dPL.NameShort			AS ProductLineIDName
				,dPL.NameShort										AS ProductLineName
				,dPL.ResponsiblePerson								AS PLResponsiblePerson

				,fV.ProductID + N' ' + dP.NameShort					AS ProductIDName
				,dP.NameShort										AS ProductName
				,dP.ResponsiblePerson								AS PResponsiblePerson

				,fV.ValueSeriesID + N' ' + dVS.NameShort			AS ValueSeriesIDName
				,dVS.NameShort
				,dVS.ValueSeriesNo
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.Unit
				,dVS.Effect
				,fV.ValueComment

				,dP.[Status]
				,dP.GlobalAttribute1
				,dP.GlobalAttribute2
				,dP.GlobalAttribute3
				,dP.GlobalAttribute4
				,dP.GlobalAttribute5
				,dP.GlobalAttribute6
				,dP.GlobalAttribute7
				,dP.GlobalAttribute8
				,dP.GlobalAttribute9
				,dP.GlobalAttribute10
				,dP.GlobalAttribute11
				,dP.GlobalAttribute12
				,dP.GlobalAttribute13
				,dP.GlobalAttribute14
				,dP.GlobalAttribute15
				,dP.GlobalAttribute16
				,dP.GlobalAttribute17
				,dP.GlobalAttribute18
				,dP.GlobalAttribute19
				,dP.GlobalAttribute20
				,dP.GlobalAttribute21
				,dP.GlobalAttribute22
				,dP.GlobalAttribute23
				,dP.GlobalAttribute24
				,dP.GlobalAttribute25
			FROM planning.tfValues fV
				LEFT JOIN planning.tdProducts dP
					ON fV.ProductKey = dP.ProductKey
				LEFT JOIN planning.tdProductLines dPL
					ON fV.ProductLineKey = dPL.ProductLineKey
				LEFT JOIN planning.tdFactories dF
					ON fV.FactoryKey = dF.FactoryKey
				-- Right Filter
				INNER JOIN #tRights tR 
					ON 
							fV.FactoryID		= tR.FactoryID
						AND fV.ProductLineID	= tR.ProductLineID

				-- to filter on numeric ValueSeries
				INNER JOIN planning.tdValueSeries dVS
					ON 
							fV.ValueSeriesKey = dVS.ValueSeriesKey
						AND dVS.[IsNumeric] = 1 

				WHERE 
						fV.ValueInt <> 0 
					-- Year Filter
					AND (@hasYears = 0 OR fV.TimeID / 10000 IN (SELECT Year FROM @years))
	
			SET @ResultCode = 200;
		END TRY
		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'result'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDataEntryNumericValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'All numeric values from data entry products'		
																-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
SET @value = @SX_Owner;
EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_Module;
EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_ShipmentFlag;
EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

SET @value = @SX_UserHint;
EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;

  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
SET @value = N'Procedure to deliver all numeric values of data entry products to the enduser.' ;
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

 
-- SET parameter documentation
SET @level2name = N'@Username';
SET @value      = N'Username';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
SET @level2name = N'@FactoryID';
SET @value      = N'optional: FactoryID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ProductLineID';
SET @value      = N'optional: ProductLineID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ParameterJSON';
SET @value      = N'optional: The Parameter JSON expect the "Year" parameter.';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP PROCEDURE IF EXISTS result.spDataEntryProfit;
GO

/*
	Procedure to deliver analytic profit data from data entry products to the enduser
	Expectes an optional parameter "Year" with one or multiple Years

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = ''
		DECLARE @ProductLineID	NVARCHAR(255) = ''
		DECLARE @ParameterJSON	NVARCHAR(MAX) = N'{"Year":[2020]}'

		EXECUTE @RC = result.spDataEntryProfit
		   @Username
		  ,@FactoryID
		  ,@ProductLineID 
		  ,@ParameterJSON
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryProfit',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryProfit','PARAMETER',NULL)
*/

CREATE PROCEDURE result.spDataEntryProfit 
	(
		 @Username		NVARCHAR(255)
		,@FactoryID		NVARCHAR(255) = N''
		,@ProductLineID NVARCHAR(255) = N''
		,@ParameterJSON NVARCHAR(MAX) = N''
	)
AS
	BEGIN
		SET NOCOUNT ON;
		DECLARE 
			@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N''''		
												+ ISNULL(@Username		,N'NULL')	+ N''','''											 
												+ ISNULL(@FactoryID		,N'NULL')	+ N''','''	
												+ ISNULL(@ProductLineID	,N'NULL')	+ N''','''	
												+ ISNULL(@ParameterJSON	,N'NULL')	+ N''''
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
		IF @ParameterJSON	IS NULL SET @ParameterJSON	= N'';

		-- Content Protection
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		
		DECLARE @years AS TABLE (Year INT NOT NULL);
		DECLARE @hasYears BIT = 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			-- Test for valid JSON
			SET @hasYears = ISJSON(@ParameterJSON);

			IF @ParameterJSON <> N'' AND @hasYears = 0
			BEGIN 
				SET @ResultCode = 403;
				RAISERROR('Exit due to invalid JSON', 16, 10);
			END;

			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactUsername = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- determine Years
			IF @hasYears = 1
			BEGIN 
				INSERT INTO @years
				SELECT DISTINCT value
				FROM OPENJSON(@ParameterJSON,'$.Year')
				WHERE value >= 0 AND value <= 9999;
			END;
			
			-- Helper Table for Userrights
			DROP TABLE IF EXISTS #tRights;
			CREATE TABLE #tRights
				(
					 RowKey					BIGINT IDENTITY (1,1)
					,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL

					CONSTRAINT PK_tRights PRIMARY KEY CLUSTERED (RowKey)			

				);

		  INSERT INTO #tRights
			SELECT 
				 FactoryID
				,ProductLineID
			FROM system.trUserRights 
			WHERE 
					Username = @TransactUsername 
				AND [Right] IN (N'Write', N'Read') 
				AND ProductLineID	<> N''
				AND FactoryID		<> N''
				AND (
						-- no Filter Paramter given
						(@FactoryID = N'' AND @ProductLineID = N'') 
						OR
								
							(	-- FactoryID Filter given
								FactoryID = @FactoryID 
								-- optional also ProductLine Filter given
							AND (@ProductLineID = N'' OR ProductLineID = @ProductLineID)
							)
					);

			SELECT 
				 fV.FactoryID
				,fV.ProductLineID
				,fV.ProductID
				,fV.ValueSeriesID

				,fV.TimeID / 10000															AS [Year]
				,fV.TimeID % 10000 / 100													AS [Month]
				,fV.TimeID % 100															AS [Day]
				,TRY_CAST(CAST(fV.TimeID AS NVARCHAR) AS DATE)								AS [Date]
			
				,TRY_CAST(fV.ValueInt AS Money) / ISNULL(dVS.Scale, 1) * gVE.ProfitLossEffect AS [Value]

				,fV.FactoryID + N' ' + dF.NameShort					AS FactoryIDName 
				,dF.NameShort										AS FactoryName
				,dF.ResponsiblePerson								AS FResponsiblePerson

				,fV.ProductLineID + N' ' + dPL.NameShort			AS ProductLineIDName
				,dPL.NameShort										AS ProductLineName
				,dPL.ResponsiblePerson								AS PLResponsiblePerson

				,fV.ProductID + N' ' + dP.NameShort					AS ProductIDName
				,dP.NameShort										AS ProductName
				,dP.ResponsiblePerson								AS PResponsiblePerson

				,fV.ValueSeriesID + N' ' + dVS.NameShort			AS ValueSeriesIDName
				,dVS.NameShort
				,dVS.ValueSeriesNo
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.Unit
				,dVS.Effect
				,fV.ValueComment

				,dP.[Status]
				,dP.GlobalAttribute1
				,dP.GlobalAttribute2
				,dP.GlobalAttribute3
				,dP.GlobalAttribute4
				,dP.GlobalAttribute5
				,dP.GlobalAttribute6
				,dP.GlobalAttribute7
				,dP.GlobalAttribute8
				,dP.GlobalAttribute9
				,dP.GlobalAttribute10
				,dP.GlobalAttribute11
				,dP.GlobalAttribute12
				,dP.GlobalAttribute13
				,dP.GlobalAttribute14
				,dP.GlobalAttribute15
				,dP.GlobalAttribute16
				,dP.GlobalAttribute17
				,dP.GlobalAttribute18
				,dP.GlobalAttribute19
				,dP.GlobalAttribute20
				,dP.GlobalAttribute21
				,dP.GlobalAttribute22
				,dP.GlobalAttribute23
				,dP.GlobalAttribute24
				,dP.GlobalAttribute25
			FROM planning.tfValues fV
				LEFT JOIN planning.tdProducts dP
					ON fV.ProductKey = dP.ProductKey
				LEFT JOIN planning.tdProductLines dPL
					ON fV.ProductLineKey = dPL.ProductLineKey
				LEFT JOIN planning.tdFactories dF
					ON fV.FactoryKey = dF.FactoryKey
				-- Right Filter
				INNER JOIN #tRights tR 
					ON 
							fV.FactoryID		= tR.FactoryID
						AND fV.ProductLineID	= tR.ProductLineID

				-- to filter on numeric ValueSeries
				INNER JOIN planning.tdValueSeries dVS
					ON 
							fV.ValueSeriesKey = dVS.ValueSeriesKey
						AND dVS.[IsNumeric] = 1 

				-- to filter on Profit / Loss
				INNER JOIN planning.tgValueEffects gVE
					ON 
							dVS.Effect = gVE.EffectID  
						AND gVE.ProfitLossEffect <> 0
				WHERE 
						fV.ValueInt <> 0 
					-- Year Filter
					AND (@hasYears = 0 OR fV.TimeID / 10000 IN (SELECT Year FROM @years))
	
			SET @ResultCode = 200;
		END TRY
		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'result'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDataEntryProfit'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'Analytic profit data from data entry products'		
																-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
SET @value = @SX_Owner;
EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_Module;
EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_ShipmentFlag;
EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

SET @value = @SX_UserHint;
EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;

  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
SET @value = N'Procedure to deliver analytic data of data entry products to the enduser.' ;
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

 
-- SET parameter documentation
SET @level2name = N'@Username';
SET @value      = N'Username';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
SET @level2name = N'@FactoryID';
SET @value      = N'optional: FactoryID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ProductLineID';
SET @value      = N'optional: ProductLineID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ParameterJSON';
SET @value      = N'optional: The Parameter JSON expect the "Year" parameter.';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP PROCEDURE IF EXISTS result.spEvents;
GO

/*
	Procedure to deliver analytic data to the enduser
	Events must have ValueSeriesID "E" and Bemerkungen ValueSeriesID "BEM"

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username		nvarchar(255) = 'SQL'
		DECLARE @FactoryID		nvarchar(255) = ''
		DECLARE @ProductLineID	nvarchar(255) = ''

		EXECUTE @RC = result.spEvents 
		   @Username
		  ,@FactoryID
		  ,@ProductLineID

		Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spEvents',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spEvents','PARAMETER',NULL)
*/
 
CREATE PROCEDURE result.spEvents
(@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
	BEGIN

		DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
		DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		+ ISNULL(@Username, N'NULL') 
													+ N''','''	+ ISNULL(@FactoryID, N'NULL') 
													+ N''','''	+ ISNULL(@ProductLineID, N'NULL') 
													+ N'''';
		DECLARE @EffectedRows		INT				= 0;			
		DECLARE @ResultCode			INT				= 501;			
		DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
		DECLARE @Comment			NVARCHAR(2000)	= N'';			

		-- STEP 0.1 - NULL Protection
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

		BEGIN TRY
			BEGIN TRANSACTION spEvents;

			SET @Username		= dbo.sx_pf_pProtectString (@Username);
			SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
			SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403'
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- # Scope table for all levels access right
			DROP TABLE IF EXISTS #Scope;
			CREATE TABLE #Scope 
				(
					 RowKey			INT IDENTITY (1,1)	
					,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				)

			INSERT INTO #Scope
				SELECT 
					 FactoryID
					,ProductLineID
				FROM system.trUserRights rUR
				WHERE 
							Username = @TransactUsername 
						AND [Right] IN ('Write','Read') 
						AND (FactoryID		= @FactoryID		OR @FactoryID = '')
						AND (ProductLineID	= @ProductLineID	OR @ProductLineID = '')

			-- Main Query
			SELECT
					fV.FactoryID +'/'
						+ fV.ProductLineID + '/'
					 	+ fV.ProductID  						AS Path	
                    
                    ,'https://stable.factoryprovider.com/DatabaseServer/' --set servername with sql server
						+ 'OCT/' -- set databasename
                     	+ fV.FactoryID +'/' 
						+ fV.ProductLineID + '/'
					 	+ fV.ProductID  						AS Productlink_LL

					,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
					,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100			AS Month


					,Ev.ValueText								AS [Event]
					,Ev.ValueComment							AS EventComment
					,fV.ValueText								AS Beschreibung
					,fV.ValueComment							AS BeschreibungComment --Comment, which ist placed on the Beschreibung field

					,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
					,dF.NameShort								AS FactoryName
					,dF.ResponsiblePerson						AS FResponsiblePerson

					,fV.ProductLineID + ' ' + dPL.NameShort		AS ProductLineIDName
					,dPL.NameShort								AS ProductLineName
					,dPL.ResponsiblePerson						AS PLResponsiblePerson

					,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
					,dP.NameShort								AS ProductName
					,dP.ResponsiblePerson						AS PResponsiblePerson

					,dP.[Status]
					,dP.GlobalAttribute1
					,dP.GlobalAttribute2
					,dP.GlobalAttribute3
					,dP.GlobalAttribute4
					,dP.GlobalAttribute5
					,dP.GlobalAttribute6
					,dP.GlobalAttribute7
					,dP.GlobalAttribute8
					,dP.GlobalAttribute9
					,dP.GlobalAttribute10

				FROM planning.tfValues fV
						-- Gruppierung nach Event
						LEFT JOIN	(
									SELECT 
											f.ProductKey
										,f.TimeID
										,f.ValueText 
										,f.ValueComment
									FROM planning.tfValues f
									INNER JOIN #Scope s
									ON		f.FactoryID		= s.FactoryID
										AND f.ProductLineID	= s.ProductLineID

									WHERE f.ValueSeriesID = 'E'
							
								) Ev ON

										fV.ProductKey	= Ev.ProductKey
									AND fV.TimeID		= Ev.TimeID

						LEFT JOIN planning.tdProducts dP
							ON fV.ProductKey = dP.ProductKey
						LEFT JOIN planning.tdProductLines dPL
							ON fV.ProductLineKey = dPL.ProductLineKey
						LEFT JOIN planning.tdFactories dF
							ON fV.FactoryKey = dF.FactoryKey
						-- Rechtefilter
						INNER JOIN #Scope Sc
							ON		fV.FactoryID	 = Sc.FactoryID
								AND fV.ProductLineID = Sc.ProductLineID
				
			WHERE fV.ValueSeriesID = 'BEM'
			SET @Resultcode = 200 

			COMMIT TRANSACTION spEvents;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			ROLLBACK TRANSACTION spEvents;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO
   
GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'result'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spEvents'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data to the enduser
         Events must have ValueSeriesID "E" and Bemerkungen ValueSeriesID "BEM"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS system.spCOPY_Dashboard;
GO

/*
	COPY Operation for a Dashboard
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spCOPY_Dashboard
				 @Username				= 'SQL'
				,@CopyDashboardID		= 'D1'
				,@TargetDashboardname	= 'Dash2';
		PRINT @RC;

		SELECT * FROM system.tDashboards;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Dashboard',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Dashboard','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spCOPY_Dashboard
	@Username				NVARCHAR(255),
	@CopyDashboardID		NVARCHAR(50),
	@TargetDashboardName	NVARCHAR(255)

AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @CopyDashboardKey	INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,				N'NULL') + N''',''' 
								+ ISNULL(@CopyDashboardID,		N'NULL') + N''',''' 
								+ ISNULL(@TargetDashboardName,	N'NULL') + N'''' 
	DECLARE @DashboardID		NVARCHAR(50)	= N'';
	DECLARE @LastIDNum			NVARCHAR(50)	= 0;
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username				= N'';
	IF @CopyDashboardID		IS NULL SET @CopyDashboardID		= N'';
	IF @TargetDashboardName	IS NULL SET @TargetDashboardName	= N'';


	BEGIN TRY
		BEGIN TRANSACTION spCOPY_Dashboard;

		-- STEP 0.2 - Protect input parameters	
		SET @CopyDashboardID		= dbo.sx_pf_pProtectString (@CopyDashboardID);

		IF @CopyDashboardID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
					
		-- STEP 2 - Get keys
		SELECT @CopyDashboardKey = DashboardKey FROM system.tDashboards WHERE DashboardID = @CopyDashboardID;

		-- If Copy user not exists, stop action
		IF @CopyDashboardKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Dashboard to copy does not exits.', 16, 10);
		END

		BEGIN
			SELECT TOP 1 
			@LastIDNum = TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT ) 
			FROM system.tDashboards
			WHERE 
				DashboardID LIKE N'D%'
				AND LEN(DashboardID) > 1
				AND TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) IS NOT NULL

				ORDER BY TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) DESC

				IF @LastIDNum IS NULL SET @LastIDNum = 0;
				SET @DashboardID = 'D' + CAST(@LastIDNum +1 AS NVARCHAR(50));

			END

		--Create Target Report and output key into table
		INSERT INTO system.tDashboards(DashboardID, DashboardName, DashboardDescription, LayoutDefinition)
			SELECT 
				 @DashboardID
				,@TargetDashboardName
				,DashboardDescription
				,LayoutDefinition
			FROM system.tDashboards
			WHERE DashboardKey = @CopyDashboardKey;

		SET @EffectedRows = 1;
						
		SET @Resultcode = 201;

		COMMIT TRANSACTION spCOPY_Dashboard;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spCOPY_Dashboard;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT @Comment;
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_Dashboard TO pf_PlanningFactoryService;
  
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spCOPY_Dashboard'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Copy Operation for a Dashboard
         a new Dashboard will be created as copy of an existing Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CopyDashboardID';
    SET @value      = N'ID of the Dashboard which is basis of the copy.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetDashboardname';
    SET @value      = N'Name of the Target Dashboard, the ID is autocreated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP PROCEDURE IF EXISTS system.spCOPY_Pipeline;
GO

/*
	Procedure to copy a Pipeline - including its steps and schedules

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spCOPY_Pipeline
				 @Username			= 'SQL'
				,@SourcePipelineID	= 'P1'
				,@TargetPipelineID	= '*'
				,@OutputID			= @ReturnID OUTPUT;
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;
		SELECT * FROM system.tPipelineSchedules;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spCOPY_Pipeline
    (
         @Username				NVARCHAR(255)
        ,@SourcePipelineID		NVARCHAR(50)
		,@TargetPipelineID		NVARCHAR(50)
		,@OutputID				NVARCHAR(50) = NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','											 
				+ ISNULL(@SourcePipelineID		,N'NULL')	+ N','''
				+ ISNULL(@TargetPipelineID		,N'NULL')	+ N''''

				,@EffectedRows			INT				= 0			
				,@ResultCode			INT				= 501				
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N''
				,@TransactUsername		NVARCHAR(255)	= N''
				,@StartTime				DATETIME2		= SysUTCDateTime()
				,@SourcePipelineKey		BIGINT			= 0
				,@TargetPipelineKey		BIGINT			= 0
				,@LastPipelineNum		NVARCHAR(50)	= 0
				,@TargetOrderOfDisplay	INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @SourcePipelineID		IS NULL SET @SourcePipelineID	= N'';
		IF @TargetPipelineID		IS NULL SET @TargetPipelineID	= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spCOPY_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @SourcePipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END

				SELECT @SourcePipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @SourcePipelineID;

				IF EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @TargetPipelineID
					)
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Target Pipeline already exits', 16, 10);
					END

				-- determine new ID if * is sended
				IF @TargetPipelineID = '*'
							BEGIN
								SELECT TOP 1 
								@LastPipelineNum = TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT ) 
								FROM system.tPipelines 
								WHERE 
										PipelineID LIKE 'P%'
									AND LEN(PipelineID) > 1
									AND TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) DESC

								IF @LastPipelineNum IS NULL SET @LastPipelineNum = 0;
								SET @TargetPipelineID = 'P' + CAST(@LastPipelineNum +1 AS NVARCHAR(50));

							END

				-- determine TargetOrderOfDisplay 
				SELECT TOP 1 @TargetOrderOfDisplay = OrderOfDisplay + 1 FROM system.tPipelines ORDER BY OrderOfDisplay DESC;
				

				-- Data Transaction
				PRINT CONCAT(N'Starting COPY operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				-- Copy the pipeline itself
				INSERT INTO system.tPipelines
					SELECT
						 @TargetPipelineID
						,PipelineName
						,PipelineDescription
						,PipelineDescriptionIconColor
						,@TargetOrderOfDisplay
						,EmailNotificationCODE
					FROM system.tPipelines
					WHERE PipelineID = @SourcePipelineID;

					SET @TargetPipelineKey = SCOPE_IDENTITY();

				-- Copy the steps of the Pipeline
				INSERT INTO system.tPipelineSteps
					SELECT
						 @TargetPipelineKey
						,StepID
						,StepName
						,StepDescription
						,OrderOfExecution
						,StepDataSourceID
						,StepCompanyIDsJSON
						,StepDetailsJSON
						,TimeExecutionLimit
						,Active
						,ModuleID
						,StartNextStepOnError
					FROM system.tPipelineSteps
					WHERE PipelineKey = @SourcePipelineKey;

				-- Copy the schedules of the Pipeline
				INSERT INTO system.tPipelineSchedules
					SELECT
						 @TargetPipelineKey
						,ScheduleID
						,ScheduleName
						,ScheduleDescription
						,ScheduleDetailsJSON
						,Active
					FROM system.tPipelineSchedules
					WHERE PipelineKey = @SourcePipelineKey;
			
				SET @ResultCode = 201;
				SET @OutputID = @TargetPipelineID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spCOPY_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spCOPY_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_Pipeline TO pf_PlanningFactoryService;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spCOPY_Pipeline'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to copy a Pipeline - including its steps and schedules';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourcePipelineID';
    SET @value      = N'ID of the Pipeline which will be copied.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetPipelineID';
    SET @value      = N'ID for the pipeline which will be created as copy of the SourcePipelineID, this Pipeline must not exists at the moment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spCOPY_PipelineStep;
GO

/*
	Procedure to copy a PipelineStep to the same or a different Pipeline
	The step will be added at the end always.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spCOPY_PipelineStep
				 @Username			= 'SQL'
				,@SourcePipelineID	= 'P1'
				,@SourceStepID		= 'S1'
				,@TargetPipelineID	= 'P2'
				,@TargetStepID		= '*'
				,@OutputID			= @ReturnID OUTPUT;
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps ORDER BY PipelineKey, StepID;
		SELECT * FROM system.tPipelineSchedules;

	Testcall Log
		SELECT * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_PipelineStep',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_PipelineStep','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spCOPY_PipelineStep
         @Username				NVARCHAR(255)
        ,@SourcePipelineID		NVARCHAR(50)
		,@SourceStepID			NVARCHAR(50)
		,@TargetPipelineID		NVARCHAR(50) = ''
		,@TargetStepID			NVARCHAR(50)
		,@OutputID				NVARCHAR(50) = NULL OUTPUT

AS 
	BEGIN	

		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','''											 
				+ ISNULL(@SourcePipelineID		,N'NULL')	+ N''','''
				+ ISNULL(@SourceStepID			,N'NULL')	+ N''','''
				+ ISNULL(@TargetPipelineID		,N'NULL')	+ N''','''
				+ ISNULL(@TargetStepID			,N'NULL')	+ N''''
				+ N''''

				,@EffectedRows				INT				= 0			
				,@ResultCode				INT				= 501				
				,@TimestampCall				DATETIME		= GETUTCDATE()
				,@Comment					NVARCHAR(2000)	= N''
				,@TransactUsername			NVARCHAR(255)	= N''
				,@StartTime					DATETIME2		= SysUTCDateTime()
				,@SourcePipelineKey			BIGINT			= 0
				,@TargetPipelineKey			BIGINT			= 0
				,@LastStepNum				NVARCHAR(50)	= 0
				,@TargetOrderOfExecution	INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @SourcePipelineID		IS NULL SET @SourcePipelineID	= N'';
		IF @SourceStepID			IS NULL SET @SourceStepID		= N'';
		IF @TargetPipelineID		IS NULL SET @TargetPipelineID	= N'';
		IF @TargetStepID			IS NULL SET @TargetStepID		= N'';

		IF @TargetPipelineID = '' SET @TargetPipelineID = @SourcePipelineID; -- if no target PL is named, its copied in the source

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spCOPY_PipelineStep
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of Pipeline and Step IDs
					-- Source Pipeline
					IF NOT EXISTS (
						SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @SourcePipelineID
						)
						BEGIN
							SET @ResultCode = 404;
							RAISERROR('Source Pipeline don`t exits', 16, 10);
						END

					SELECT @SourcePipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @SourcePipelineID;

					-- Target Pipeline
					IF NOT EXISTS (
						SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @TargetPipelineID
						)
						BEGIN
							SET @ResultCode = 404;
							RAISERROR('Target Pipeline don`t exits', 16, 10);
						END

					SELECT @TargetPipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @TargetPipelineID;

					-- Source Step ID
					IF NOT EXISTS (
						SELECT StepID FROM system.tPipelineSteps WHERE PipelineKey = @SourcePipelineKey AND StepID = @SourceStepID
						)
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Source StepID dont exits', 16, 10);
						END

					-- Target Step ID
					IF EXISTS (
						SELECT StepID FROM system.tPipelineSteps WHERE PipelineKey = @TargetPipelineKey AND StepID = @TargetStepID
						)
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Target StepID already exits', 16, 10);
						END

				-- determine Target OrderOfExecution
				SELECT TOP 1 
					@TargetOrderOfExecution = OrderOfExecution +1 
				FROM system.tPipelineSteps 
				WHERE
					PipelineKey = @TargetPipelineKey 
				ORDER BY
					OrderOfExecution DESC;

				-- determine new ID if * is sended
				IF @TargetStepID = '*'
							BEGIN
								SELECT TOP 1 
								@LastStepNum = TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) 
								FROM system.tPipelineSteps
								WHERE 
										PipelineKey = @TargetPipelineKey
									AND StepID		LIKE 'S%'
									AND LEN(StepID) > 1
									AND TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) DESC

								IF @LastStepNum IS NULL SET @LastStepNum = 0;
								SET @TargetStepID = 'S' + CAST(@LastStepNum +1 AS NVARCHAR(50));
							END
			
				-- Data Transaction
				PRINT CONCAT(N'Starting COPY operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				PRINT CONCAT(@SourcePipelineKey,@SourceStepID,@TargetPipelineID,@TargetStepID,@TargetOrderOfExecution);

				-- Copy the steps of the Pipeline
				INSERT INTO system.tPipelineSteps
					SELECT
						 @TargetPipelineKey
						,@TargetStepID
						,StepName
						,StepDescription
						,@TargetOrderOfExecution
						,StepDataSourceID
						,StepCompanyIDsJSON
						,StepDetailsJSON
						,TimeExecutionLimit
						,Active
						,ModuleID
						,StartNextStepOnError
					FROM system.tPipelineSteps
					WHERE 
							PipelineKey = @SourcePipelineKey
						AND StepID		= @SourceStepID;

					SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 201;
				SET @OutputID = @TargetStepID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spCOPY_PipelineStep
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spCOPY_PipelineStep	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- GRANT
GRANT EXECUTE ON OBJECT :: system.spCOPY_PipelineStep TO pf_PlanningFactoryService;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spCOPY_PipelineStep'    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to copy a Pipeline - including its steps and schedules';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourcePipelineID';
    SET @value      = N'ID of the Pipeline from which a step will be copied.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceStepID';
    SET @value      = N'ID for the step which will be copied';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetPipelineID';
    SET @value      = N'ID for the Pipeline, which is used as target. If keept empty, the source Productline is used as target.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@TargetStepID';
    SET @value      = N'ID for the step which will be created, it must not exist at the moment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
DROP PROCEDURE IF EXISTS system.spCOPY_Report;
GO

/*
	COPY Operation for a Report
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spCOPY_Report
				 @Username			= 'SQL'
				,@CopyReportID		= 'R1'
				,@TargetReportName	= 'Report 2';
		PRINT @RC;

		SELECT * FROM system.tReports;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spCOPY_Report
	@Username			NVARCHAR(255),
	@CopyReportID		NVARCHAR(255),
	@TargetReportName	NVARCHAR(255)

AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @CopyReportKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,			N'NULL') + N''',''' 
								+ ISNULL(@CopyReportID,		N'NULL') + N''',''' 
								+ ISNULL(@TargetReportName,	N'NULL') + N'''' 
	DECLARE @ReportID			NVARCHAR(50)	= N'';
	DECLARE @LastIDNum			NVARCHAR(50)	= 0;
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @CopyReportID		IS NULL SET @CopyReportID		= N'';
	IF @TargetReportName	IS NULL SET @TargetReportName	= N'';


	BEGIN TRY
		BEGIN TRANSACTION spCOPY_Report;

		-- STEP 0.2 - Protect input parameters	
		SET @CopyReportID		= dbo.sx_pf_pProtectString (@CopyReportID);

		IF @CopyReportID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
					
		-- STEP 2 - Get keys
		SELECT @CopyReportKey		= ReportKey FROM system.tReports WHERE ReportID = @CopyReportID;

		-- If Copy user not exists, stop action
		IF @CopyReportKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Report to copy does not exits.', 16, 10);
		END

		BEGIN
			SELECT TOP 1 
			@LastIDNum = TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT ) 
			FROM system.tReports
			WHERE 
				ReportID LIKE N'R%'
				AND LEN(ReportID) > 1
				AND TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) IS NOT NULL

				ORDER BY TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) DESC

				IF @LastIDNum IS NULL SET @LastIDNum = 0;
				SET @ReportID = 'R' + CAST(@LastIDNum +1 AS NVARCHAR(50));

			END

		--Create Target Report and output key into table
		INSERT INTO system.tReports(ReportID, ReportName, ReportDescription, LayoutDefinition)
			SELECT 
				@ReportID
				,@TargetReportName
				,ReportDescription
				,LayoutDefinition
			FROM system.tReports
			WHERE ReportKey = @CopyReportKey;

		SET @EffectedRows = 1;
						
		SET @Resultcode = 201;

		COMMIT TRANSACTION spCOPY_Report;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spCOPY_Report;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT @Comment;
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_Report TO pf_PlanningFactoryService;
  
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spCOPY_Report'					-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Copy Operation for a Report
         a new Report will be created as copy of an existing Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CopyReportID';
    SET @value      = N'ID of the Report which is basis of the copy.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetReportname';
    SET @value      = N'Name of the Target Report, the ID is autocreated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

DROP PROCEDURE IF EXISTS system.spCOPY_User;
GO

/*
	COPY Operation for a User
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spCOPY_User
				 @Username			= 'SQL'
				,@CopyUsername		= 'W10\admin'
				,@TargetUsername	= 'W10\Maus2';
		PRINT @RC;

		SELECT * FROM system.trUser;
		SELECT * FROM system.trRights;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spCOPY_User
	@Username			NVARCHAR(255),
	@CopyUserName		NVARCHAR(255),
	@TargetUsername		NVARCHAR(255)

AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @CopyUserKey		INT				= 0;
	DECLARE @TargetUserKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,			N'NULL') + N''',''' 
								+ ISNULL(@CopyUserName,		N'NULL') + N''',''' 
								+ ISNULL(@TargetUserName,	N'NULL') + N'''' 
								
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @CopyUserName		IS NULL SET @CopyUserName		= N'';
	IF @TargetUserName		IS NULL SET @TargetUserName		= N'';


	BEGIN TRY
		BEGIN TRANSACTION spCOPY_User;

		-- STEP 0.2 - Protect input parameters	
		SET @CopyUserName		= dbo.sx_pf_pProtectString (@CopyUserName);
		SET @TargetUserName		= dbo.sx_pf_pProtectString (@TargetUserName);

		IF @CopyUserName = N'' OR @TargetUsername = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Don`t allow creation or changing of systemuser SQL
		-- the Username "FactoryService" is not allowed, as it would lead to confusion with the similar named db role
		IF @TargetUserName = N'SQL' OR @TargetUserName = N'FactoryService'	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden creation or changing of systemuser', 16, 10);
		END;
			
		-- STEP 1.2 - Don't allow posting of members of the DB role pf_PlanningFactoryService (which are not visible in the frontend) - not over the function IS_ROLEMEMBER, its buggy
		IF EXISTS(
			SELECT r.name 
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r	ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m	ON rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = @TargetUserName
			)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changes on the members of the DB role pf_PlanningFactoryService', 16, 10);
		END;


		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to change rights', 16, 10);
		END;
					
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
		
		-- STEP 2 - Get keys
		SELECT @CopyUserKey		= UserKey FROM system.trUser WHERE Username = @CopyUserName;
		SELECT @TargetUserKey	= UserKey FROM system.trUser WHERE Username = @TargetUserName;

		-- If Copy user not exists, stop action
		IF @CopyUserKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User to copy does not exits.', 16, 10);
		END 

		-- If Target user already exists, stop action
		IF @TargetUserKey  <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Overwiting existing users is not allowed', 16, 10);
		END 

		--Create Target User and output key into table
		-- the determination on the table variable is necessary because of TRIGGERS on rUser
		INSERT INTO system.trUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status,DataFlowRightsCODE,VisualizationRightsCODE,IsAdministratorFlag) 
			OUTPUT INSERTED.UserKey INTO @tmpUserTable 
			SELECT 
				 @TargetUsername
				,N''
				,N''
				,N''
				,N''
				,[Status]
				,DataFlowRightsCODE
				,VisualizationRightsCODE
				,IsAdministratorFlag
			FROM system.trUser
			WHERE UserKey = @CopyUserKey;

		SET @EffectedRows = 1;
				
		SELECT TOP (1) @TargetUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
		SET @Resultcode = 201;
		
		-- STEP 3.2 - Copy the rights of the copy User
		INSERT INTO system.trRights 
			SELECT
				 @TargetUserKey		AS UserKey
				,@TargetUsername	AS UserName
				,FactoryID
				,ProductLineID
				,N''					AS ProductID
				,[Right]
				,ReadCommentMandatory
				,WriteCommentMandatory
			FROM system.trRights
			WHERE Userkey = @CopyUserKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION spCOPY_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spCOPY_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_User TO pf_PlanningFactoryService;
GO
  

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spCOPY_User'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Copy Operation for a User
         a new user will be created as copy of an existing user. Rights are copied, user attributes are not copied, user will be set to active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CopyUserName';
    SET @value      = N'Name of the user which is basis of the copy.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetUsername';
    SET @value      = N'Name of the Target User';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 
GO
DROP PROCEDURE IF EXISTS system.spDELETE_Connector;
GO

/*
	Procedure to delete a connector including all queries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_Connector
				 @Username		= 'SQL'
				,@SourceSystemID	= 'NAVISION'
				,@SourceSystemType = 'MSSQL'
				,@ModuleID = 'FIN'
		PRINT    @RC

		SELECT * FROM system.tConnectors
		SELECT * FROM system.tConnectorQueries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Connector
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50) = ''
		,@SourceSystemType		NVARCHAR(50) = ''
		,@ModuleID				NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();
			

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username	= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Connector
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				--Right Check !!
					-- can be executed as PreSQL from any integration user
					-- must be executeable during import

				IF NOT EXISTS (
					SELECT 1 FROM system.tConnectors WHERE SourceSystemID = @SourceSystemID AND SourceSystemType = @SourceSystemType AND ModuleID = @ModuleID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Connector doesn''t exist', 16, 10);
					END;

				-- Delete Pipeline, this will also delete Steps and Schedules do to key relation
				PRINT CONCAT(N'Delete connector',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				DELETE FROM system.tConnectors WHERE SourceSystemID = @SourceSystemID AND SourceSystemType = @SourceSystemType AND ModuleID = @ModuleID;

				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Connector
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Connector	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spDELETE_Connector TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Pipeline including steps and schedules.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO

DROP PROCEDURE IF EXISTS system.spDELETE_Dashboard;
GO

/*
	Procedure to delete a Dashboard

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_Dashboard
				 @Username		= 'SQL'
				,@DashboardID	= 'R12';
		PRINT    @RC;

		SELECT * FROM system.tDashboards;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Dashboard',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Dashboard','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Dashboard
    
         @Username			NVARCHAR(255)
        ,@DashboardID		NVARCHAR(50)
    

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username			,N'NULL')	+ N''','''								 
				+ ISNULL(@DashboardID		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();



		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DashboardID				IS NULL SET @DashboardID		= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Dashboard
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check existence of dashboard
				IF NOT EXISTS (
					SELECT DashboardID FROM system.tDashboards WHERE DashboardID = @DashboardID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Dashboard don`t exits', 16, 10);
					END
							   
				-- Data Transaction
				PRINT CONCAT(N'Starting delete',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				DELETE FROM system.tDashboards
				WHERE DashboardID = @DashboardID;

				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Dashboard
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Dashboard	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO

GRANT EXECUTE ON OBJECT :: system.spDELETE_Dashboard TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Dashboard' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete an Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@DashboardID';
    SET @value      = N'ID of the Dashboard to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spDELETE_DataSource;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Procedure to delete a DataSource an all data which belong to it. 

	A Datasource contains mostly three types of data
		- itselft, its definition of a connnection to an Datasource
		- the data extracted from this datasource (Companies, Dimensions, Facts)
		- process definitions for this datasource (Pipelines, Steps..)
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_DataSource
				 @Username		= 'SQL'
				,@DataSourceID	= '2'
		PRINT    @RC

		SELECT * FROM system.tDataSources

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_DataSource',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_DataSource','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_DataSource
    (
         @Username			NVARCHAR(255)
        ,@DataSourceID		NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID 				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@Tablename			NVARCHAR(255)	= N''
				,@SQL				NVARCHAR(MAX)	= N''
				,@Counter			INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID		= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_DataSource
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR(N'Transaction user don`t exists', 16, 10);
						END;
				-- Check for Cluster Write Rights

				-- Check if Datasource exits
				SELECT @Counter = COUNT(*) 
				FROM system.tDataSources 
				WHERE DatasourceID = @DataSourceID;
				IF @Counter = 0 
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Datasource don`t exists', 16, 10);
					END;



				-- Generic delete is still done over DatasourceKey, as DatasourceID was till 5.5 the DatasourceKey
				PRINT '';
				PRINT 'Starting generic Delete Process for all Tables outside system containing the Column "DatasourceKey"';
				PRINT '#####################################################################################################';
				PRINT '';
				-- DELETE Date in all Non-Core Tables where the column DataSourceKey is used (usually global and integration schema)
				-- Generic delete over dynamic SQL is possible in all OCT schema except "system" in default

				DECLARE TableCursor CURSOR FOR
					SELECT      
						 CONCAT(infs.TABLE_SCHEMA COLLATE DATABASE_DEFAULT,N'.',infs.TABLE_NAME COLLATE DATABASE_DEFAULT)	AS Tablename

					FROM INFORMATION_SCHEMA.COLUMNS infs
						-- Determination Objekttype
						LEFT JOIN 
								(
								SELECT 
									 Name	  				COLLATE DATABASE_DEFAULT    AS Objektname
									,SCHEMA_NAME(Schema_id) COLLATE DATABASE_DEFAULT	AS Schemaname
									,type_desc				COLLATE DATABASE_DEFAULT	AS Objekttyp
								FROM sys.objects
								) syo ON 
											syo.Schemaname = infs.TABLE_SCHEMA
										AND syo.Objektname = infs.TABLE_NAME

					WHERE 
							syo.Objekttyp		= N'USER_TABLE'
						AND infs.COLUMN_NAME	= N'DataSourceKey'
						-- ToDo: Determination not a Core table should be moved to metadata 
						AND infs.TABLE_SCHEMA  <> N'system'
						AND CONCAT(infs.TABLE_SCHEMA,N'.',infs.TABLE_NAME) <> N'global.tCompanies'

				OPEN TableCursor;
				FETCH TableCursor
				INTO @Tablename;
				IF @@CURSOR_ROWS = 0
					BEGIN
						PRINT CONCAT(N'No Tables found for generic deletion over DataSourceKey ',@DataSourceID,N'.')
					END
				WHILE @@FETCH_STATUS = 0
				BEGIN
					-- clear variables
					SET @SQL = N'';
					SET @SQL = CONCAT(N'DELETE FROM ', @Tablename, N' WHERE DataSourceKey =', @DataSourceID,';') -- ToDo: ID is numeric at the moment, change to string when changed					
					
					PRINT CONCAT(N'Delete all Rows with DatasourceID ', @DataSourceID, N' in Table ',@Tablename)
					EXEC (@SQL);

					FETCH TableCursor
					INTO @Tablename;
				END;
				CLOSE TableCursor;
				DEALLOCATE TableCursor;

				-- Generic delete is still done over CompanyKey
				-- GET all Non-Core Tables where the column CompanyKey is used (usually result schema) and delete the Values for this ComanpanyKey
				-- The Company Key is an combinded Key "DatasourceID | CompanyID"
				-- some are deleted the second time, as they have the Datasource Key also !!

				PRINT '';
				PRINT 'Starting generic Delete Process for all Tables outside system containing the Column "CompanyKey" which is a concated Key with the Datasource.';
				PRINT '#####################################################################################################';
				PRINT '';
				DECLARE TableCursor CURSOR FOR
					SELECT      
							CONCAT(infs.TABLE_SCHEMA COLLATE DATABASE_DEFAULT,N'.',infs.TABLE_NAME COLLATE DATABASE_DEFAULT)	AS Tablename

					FROM INFORMATION_SCHEMA.COLUMNS infs
						-- Determination Objekttype
						LEFT JOIN 
								(
								SELECT 
									 Name					COLLATE DATABASE_DEFAULT AS Objektname
									,SCHEMA_NAME(Schema_id) COLLATE DATABASE_DEFAULT AS Schemaname
									,type_desc				COLLATE DATABASE_DEFAULT AS Objekttyp
								FROM sys.objects
								) syo ON 
											syo.Schemaname = infs.TABLE_SCHEMA
										AND syo.Objektname = infs.TABLE_NAME

				WHERE 
							syo.Objekttyp		= N'USER_TABLE'
						AND infs.COLUMN_NAME	= N'CompanyKey'
						-- ToDo: Determination not a Core table should be moved to metadata 
						AND infs.TABLE_SCHEMA  <> N'system'
						AND CONCAT(infs.TABLE_SCHEMA,N'.',infs.TABLE_NAME) <> N'global.tCompanies';
				OPEN TableCursor;
				FETCH TableCursor
				INTO @Tablename;
					IF @@CURSOR_ROWS = 0
						BEGIN
							PRINT CONCAT(N'No Tables found for generic deletion over CompanyKeys starting with ',@DataSourceID,N'| .')
						END
				WHILE @@FETCH_STATUS = 0
					BEGIN
						-- clear variables
						SET @SQL = N'';
						SET @SQL = CONCAT(N'DELETE FROM ', @Tablename, N' WHERE CompanyKey LIKE ''', @DataSourceID,N'|%'';') 

						PRINT CONCAT(N'Delete all Rows with DatasourceID ', @DataSourceID, N' in CompanyKey in Table ',@Tablename)
						EXEC (@SQL);

						FETCH TableCursor
						INTO @Tablename;
					END;
				CLOSE TableCursor;
				DEALLOCATE TableCursor;

				PRINT ''
				PRINT 'End of generic delition'
				PRINT  '######################################################################'
				PRINT ''

				PRINT '';
				PRINT 'Starting Delete for Core Data'
				PRINT '#####################################################################################################';
				PRINT '';
								
				-- PIPELINES - Company selection is deleted in all steps, but not the steps themself using the datasource

				-- system.tPipelineSteps  - TODO - in new field CompaniesJSON
			
				-- global.tCompanies
				DELETE FROM global.tCompanies WHERE DataSourceKey = CAST(@DataSourceID AS INT);

				PRINT 'Delete the datasource itself'
				-- DELETE Datasource (as last due to calculated / dependend columns)
				DELETE FROM system.tDataSources WHERE DataSourceID = @DataSourceID;

				SET @ResultCode = 200;
			COMMIT TRANSACTION spDELETE_DataSource
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_DataSource	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT N'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spDELETE_DataSource TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_DataSource'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Datasouce - the data from its proceeded companies is deleted in all tables with Columns DatasourceKey or CompanyKey too, but the datesource is keept in Process Steps.';    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'ID of the DataSource to be deleted.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO

DROP PROCEDURE IF EXISTS system.spDELETE_Icon;
GO

/*
	Procedure to delete icons
	Return code 200 if file exists and was deleted succesful
	
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC system.spDELETE_Icon 
			'SQL'
			,'hase.svg';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Icon',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Icon','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Icon
	 @UserName		NVARCHAR(255)	= N''
	,@IconName		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username, N'NULL') + N''', ''' + ISNULL(@IconName, N'NULL') + N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';
		IF @IconName IS NULL SET @IconName = N'';

		BEGIN TRY
			BEGIN TRANSACTION DELETE_Icon;
			
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			IF NOT EXISTS (
				SELECT TOP (1) 1 
				FROM system.tIcons
				WHERE IconName	= @IconName
					)
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			DELETE FROM system.tIcons
			WHERE IconName = @IconName
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION DELETE_Icon;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION DELETE_Icon;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::system.spDELETE_Icon TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spDELETE_Icon TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_Icon'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Return code 200 if file exists and was deleted succesful';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@UserName';
    SET @value      = N'UserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IconName';
    SET @value      = N'IconName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS system.spDELETE_LogicIndexItem;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @Returncode INT
		EXEC @ReturnCode = system.spDELETE_LogicIndexItem
			 'SQL'
			,'FIN'
			,'1.0';
		Print @Returncode;

		SELECT * FROM system.tLogicIndex

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_LogicIndexItem',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_LogicIndexItem','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_LogicIndexItem
	 @Username		NVARCHAR(255)
	,@ModuleID		NVARCHAR(50)
	,@ModuleVersion	NVARCHAR(50)

AS
BEGIN

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', ''' 
													+ ISNULL(@ModuleID, N'NULL') + N''', ''' 
													+ ISNULL(@ModuleVersion, N'NULL') + N''''''							
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @ModuleVersion	IS NULL SET @ModuleVersion	= N'';

		BEGIN TRY
			BEGIN TRANSACTION DELETE_LogicIndexItem;

			-- Determine transaction user
			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			--DROP module item of this version if it already exists
			DELETE FROM system.tLogicIndex 
				WHERE 
						ModuleID		= @ModuleID 
					AND ModuleVersion	= @ModuleVersion

			SET @EffectedRows = @@ROWCOUNT;
			IF @EffectedRows = 1 SET @ResultCode = 200;
			IF @EffectedRows = 0 SET @ResultCode = 404;
			IF @EffectedRows > 1 SET @ResultCode = 500;

			COMMIT TRANSACTION DELETE_LogicIndexItem;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION DELETE_LogicIndexItem;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spDELETE_LogicIndexItem TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_LogicIndexItem'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'spDELETE_LogicIndexItem';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleVersion';
    SET @value      = N'ModuleVersion';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
 
DROP PROCEDURE IF EXISTS system.spDELETE_Pipeline;
GO

/*
	Procedure to delete a Pipeline including all Steps

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_Pipeline
				 @Username		= 'SQL'
				,@PipelineID	= 'P2'
		PRINT    @RC

		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps
		SELECT * FROM system.tPipelineSchedules

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Pipeline
    (
         @Username				NVARCHAR(255)
		,@PipelineID			NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','											 
				+ ISNULL(@PipelineID				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();
			

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username	= N'';
		IF @PipelineID		IS NULL SET @PipelineID	= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				--Right Check !!
					-- can be executed as PreSQL from any integration user
					-- must be executeable during import

				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline does not exists', 16, 10);
					END;

				-- Delete Pipeline, this will also delete Steps and Schedules do to key relation
				PRINT CONCAT(N'Pipeline löschen',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				DELETE FROM system.tPipelines WHERE PipelineID = @PipelineID;

				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spDELETE_Pipeline TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Pipeline including steps and schedules.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO
 
DROP PROCEDURE IF EXISTS system.spDELETE_PipelineSchedule;
GO
/*
	Procedure to delete one schedules of a pipeline

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_PipelineSchedule
				 @Username		 = 'SQL'
				,@PipelineID	 = 'P1'
				,@ScheduleID	 = 'S1'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_PipelineSchedule',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_PipelineSchedule','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_PipelineSchedule
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)
		,@ScheduleID			NVARCHAR(50) 
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username		,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID 	,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleID	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username	= N'';
		IF @PipelineID				IS NULL SET @PipelineID	= N'';
		IF @ScheduleID				IS NULL SET @ScheduleID	= N'';


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;


				IF NOT EXISTS (
					SELECT ScheduleID FROM system.tPipelineSchedules WHERE PipelineKey = @PipelineKey AND ScheduleID = @ScheduleID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Schedule don`t exits', 16, 10);
					END

				-- Data Transaction
				PRINT CONCAT(N'Start deleting data',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DELETE 					 
				FROM system.tPipelineSchedules 
				WHERE 
						PipelineKey = @PipelineKey
					AND ScheduleID = @ScheduleID

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: [system].[spDELETE_PipelineSchedule] TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_PipelineSchedule'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline, which contains the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ScheduleID';
    SET @value      = N'ID of the schedule to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP PROCEDURE IF EXISTS system.spDELETE_PipelineStep;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_PipelineStep
				 @Username		= N'SQL'
				,@PipelineID	= N'P11'
				,@StepID		= N'S1';
		PRINT    @RC;

		SELECT * FROM system.tPipelineSteps;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_PipelineStep',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_PipelineStep','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_PipelineStep
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)
		,@StepID				NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username		,N'NULL')	+ N''','											 
				+ ISNULL(@PipelineID	,N'NULL')	+ N','''
				+ ISNULL(@StepID		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';
		IF @StepID					IS NULL SET @StepID				= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_PipelineStep
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('PipelineID does not exists', 16, 10);
					END;

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;

				IF NOT EXISTS (
					SELECT StepID FROM system.tPipelineSteps WHERE PipelineKey = @PipelineKey AND StepID = @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('StepID does not exists', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Starting delete',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				DELETE FROM system.tPipelineSteps 
				WHERE 
						PipelineKey = @PipelineKey 
					AND StepID		= @StepID;
			
				SET @EffectedRows = @@ROWCOUNT;
				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaction finished', ' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_PipelineStep
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_PipelineStep	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spDELETE_PipelineStep TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_PipelineStep'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a step of a Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spDELETE_Report;
GO

/*
	Procedure to delete a Report

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_Report
				 @Username		= 'SQL4'
				,@ReportID		= 'R12'
		PRINT    @RC

		SELECT * FROM system.tReports

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Report',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Report','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Report
    (
         @Username			NVARCHAR(255)
        ,@ReportID			NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username			,N'NULL')	+ N''','''								 
				+ ISNULL(@ReportID			,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();



		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ReportID				IS NULL SET @ReportID			= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Report
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check existence of report
				IF NOT EXISTS (
					SELECT ReportID FROM system.tReports WHERE ReportID = @ReportID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Report don`t exits', 16, 10);
					END
							   
				-- Data Transaction
				PRINT CONCAT(N'Starting delete',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				DELETE FROM system.tReports
				WHERE ReportID = @ReportID;

				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Report
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Report	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spDELETE_Report TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Report'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete an Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@ReportID';
    SET @value      = N'ID of the Report to delete.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spDELETE_User;
GO
/*
	DELETE Operation for a User
	Deletes a User with all his rights. A cluster administrator can't delete himself

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9

	Test call
	1. DeleteUserName is empty => 404 Not Found
	2. Delete command comes from non clusteradmin => 403 Forbidden
	3. User wants to delete himself => 403 Forbidden
	4. DeleteUser don`t exists => 204 No Content
	5. deleted user successful => 200 OK

		DECLARE @RC					INT
		DECLARE @Username			NVARCHAR(255) = 'SQL'
		DECLARE @DeleteUserName		NVARCHAR(255) = 'W10\admin'
		
		EXECUTE @RC = system.spDELETE_User 
				@Username,
				@DeleteUserName
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_User','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spDELETE_User
	@Username		NVARCHAR(255),
	@DeleteUserName NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N'''' 
			+ ISNULL(@Username,			N'NULL') + N''',''' 
			+ ISNULL(@DeleteUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DeleteUserName	IS NULL SET @DeleteUserName = N'';

	BEGIN TRY
		BEGIN TRANSACTION spDELETE_User;

		-- Protect Strings
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @DeleteUserName = dbo.sx_pf_pProtectString (@DeleteUserName);

		IF @DeleteUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
	
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
			
		-- STEP 1.2 - Stop, if Transactuser wants to delete himself
		IF @TransactUsername = @DeleteUsername
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction can`t delete himself', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3 - delete User		
		DELETE FROM system.trRights 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM system.trUser 
		WHERE UserName = @DeleteUserName;

		SET @EffectedRows += @@ROWCOUNT;

		SET @ResultCode = IIF(@@ROWCOUNT = 0, 204, 200);

		COMMIT TRANSACTION spDELETE_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spDELETE_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

-- GRANT
GRANT EXECUTE ON OBJECT ::system.spDELETE_User TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDELETE_User'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes a User with all his rights. A cluster administrator cant delete himself';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DeleteUserName';
    SET @value      = N'DeleteUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
DROP PROCEDURE IF EXISTS system.spEXPORT_Connector;
GO

/*
	Procedure to create an OCT Importscript for a selected connector - used for export  / import of connectors
		- exports the connector with its queries

	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Connector
				 @Username			= 'SQL'
				,@SourceSystemID	= 'NAVISION'
				,@SourceSystemType	= 'MSSQL'
				,@ModuleID			= 'FIN'
				,@ExportType		= 'JSON'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Connector
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50)
		,@SourceSystemType		NVARCHAR(50)
		,@ModuleID				NVARCHAR(50)
		,@ExportType            NVARCHAR(50) = 'SQL'
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''''
			
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@SourceClusterName NVARCHAR(255)	= N''
				,@SourceClusterAPI	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				IF @ExportType = 'SQL'
				BEGIN
					-- Create a temporary table to store the collection of export commands
					DROP TABLE IF EXISTS #tExport;
					CREATE TABLE #tExport
						(
							 RowKey					BIGINT IDENTITY (1,1)
							,MainOrderNumber		INT						NOT NULL
							,SubOrderNumber			INT						NOT NULL
							,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

							CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

						);

					-- Determine Metadata
					SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
					SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
					SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

					-- MetaHeader
					INSERT INTO #tExport
						VALUES 
							 (0, 1, N'-- {')
							,(0, 2, N'-- "Type": "Connector",')
							,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
							,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
							,(0, 5, CONCAT(N'-- "SourceSystemID": "',@SourceSystemID,'",'))
							,(0, 6, CONCAT(N'-- "SourceSystemType": "',@SourceSystemType,'",'))
							,(0, 7, CONCAT(N'-- "ModuleID": "',@ModuleID,'"'))
							,(0, 8, N'-- }')

					INSERT INTO #tExport
						VALUES 
							 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
							,(10, 2, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
							,(10, 3, N'DECLARE @SourceSystemID NVARCHAR(255)	= ''' + @SourceSystemID+ N'''')
							,(10, 4, N'DECLARE @SourceSystemType NVARCHAR(255)	= ''' + @SourceSystemType+ N'''')
							,(10, 5, N'DECLARE @ModuleID NVARCHAR(255)			= ''' + @ModuleID+ N'''')
							,(10, 6, N'--This Connector will be deleted during import, if it exists. You should be sure !')

					-- Try_Delete existing connector
					INSERT INTO #tExport
						SELECT 
							 900	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spDELETE_Connector '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,';'
								)	AS Command
						FROM system.tConnectors
						WHERE SourceSystemID = @SourceSystemID
						AND SourceSystemType = @SourceSystemType
						AND ModuleID = @ModuleID;

					-- Create the connector as new connector
					INSERT INTO #tExport
						SELECT 
							 1000	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spPOST_Connector '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorDescription)		,''''
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorDescriptionURL)		,''''
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorVersion),''''
								)	AS Command
						FROM system.tConnectors
						WHERE SourceSystemID = @SourceSystemID
						AND SourceSystemType = @SourceSystemType
						AND ModuleID = @ModuleID;
			
					-- Create the Steps
					INSERT INTO #tExport
						SELECT 
							 2000	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spPOST_ConnectorQuery '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,','''	,cq.QueryName								,''''
								,','''	,dbo.sx_pf_pMaskSQL(cq.QuerySQL)	,''''
								,','''	,dbo.sx_pf_pMaskSQL(cq.QueryVersionComment)	,''''							
								)	AS Command
						FROM system.tConnectorQueries AS cq
							INNER JOIN system.tConnectors AS c
								ON c.ConnectorKey = cq.ConnectorKey
						LEFT JOIN system.tConnectorQueries AS cqMax
						ON cq.ConnectorKey = cqMax.ConnectorKey
						AND cq.QueryName = cqMax.QueryName
						AND cq.QueryVersion < cqMax.QueryVersion
						WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND c.ModuleID = @ModuleID
						AND cqMax.QueryVersion IS NULL;

				
					-- final GO
					INSERT INTO #tExport
						VALUES 
							 (9999, 1, N'GO');

					-- Data Transaction

					PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
					SELECT 
						Command

					FROM #tExport
					ORDER BY 
						 MainOrderNumber
						,SubOrderNumber
				END
				
				IF @ExportType = 'JSON'
				BEGIN
					SELECT (
						SELECT
							f1.SourceSystemID,
							f1.SourceSystemType,
							f1.ModuleID,
							f1.ConnectorDescription AS ConnectorDescription,
							f1.ConnectorDescriptionURL AS ConnectorDescriptionURL,
							f1.ConnectorVersion AS ConnectorVersion,
							(
								SELECT
									f2.QueryName,
									f2.QuerySQL AS Query
								FROM system.tConnectorQueries AS f2
								WHERE f2.ConnectorKey = f1.ConnectorKey
								AND f2.QueryVersion = (
									SELECT MAX(f3.QueryVersion)
									FROM system.tConnectorQueries AS f3
									WHERE f3.ConnectorKey = f2.ConnectorKey
									AND f3.QueryName = f2.QueryName
									)
								ORDER BY CASE WHEN f2.QueryName = 'GetCompanies' THEN 1 ELSE 2 END ASC, f2.QueryName ASC
								FOR JSON PATH
							) AS 'ConnectorQueries'
						FROM system.tConnectors AS f1
						WHERE f1.SourceSystemID = @SourceSystemID
						AND f1.SourceSystemType = @SourceSystemType
						AND f1.ModuleID = @ModuleID
						FOR JSON PATH, ROOT ('Connector')
					) AS Command
				END

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Connector TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a connector with all steps in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spEXPORT_Icon;
GO

/*
	Procedure to create an OCT Importscript for an selected Icon 

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Icon
				 @Username		= N'SQL'
				,@IconName		= N'GuV';
		PRINT    @RC;

	Testing in SSMS is only limited possible - Icons with complex images resulting in very long text -> see system.spExport_Setting for more informations

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Icon',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Icon','PARAMETER',NULL)
*/
   
CREATE PROCEDURE system.spEXPORT_Icon
          @Username			NVARCHAR(255)
		 ,@IconName			NVARCHAR(50)	

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(@IconName 					,N'NULL')	+ N''''			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @IconName			IS NULL SET @IconName		= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary Iconle to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName	= ValueText FROM system.tSettings WHERE SettingID = N'Clustername'; -- May be empty if not defined
				SET @SourceClusterName		= CONCAT(DB_Name(),N' ',@SourceClusterName);
				SELECT @SourceClusterAPI	= ValueText FROM system.tSettings WHERE SettingID = N'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Icon",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 5, CONCAT(N'-- "SourceFactoryID": "','','",'))
						,(0, 5, CONCAT(N'-- "SourceProductLineID": "','','",'))
						,(0, 5, CONCAT(N'-- "SourceProductID": "','','",'))
						,(0, 6, CONCAT(N'-- "SourceIconName": "',@IconName,'"'))
						,(0, 7, N'-- }')

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @IconName			NVARCHAR(255)		= ''' + @IconName + N'''')
						,(10, 4, N'--This Icon will be deleted during import, if it exists. You should be sure !');
				
				-- Try_Delete existing Icon 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spDELETE_Icon '
							,'''SQL'''
							,',@IconName'
							,';'
							)	AS Command

				-- Create the Icon as new Icon
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_Icon '
							,'''SQL'''
							,',@IconName'
							,','''	,FileExtension								,''''	
							,',0x'	,CONVERT(NVARCHAR(MAX),FileBody,2)			,';'
	
							)	AS Command
					FROM system.tIcons
					WHERE IconName			= @IconName;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');
		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not execuIconle command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct daIconase access to this object)

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the Iconle
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Icon'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a Icon in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@IconName';
    SET @value      = N'Name of the Icon to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spEXPORT_Pipeline;
GO

/*
	Procedure to create an OCT Importscript for an selected Pipeline - used for export  / import of pipelines
		- exports the pipeline with its steps an schedules
		- NO global parameters used by the pipeline are exported (would be possible, but complex JSON operation)
		- conditions and consequences
			- a new Pipeline with the next free ID is created
			- the user must after the import or before
				- create a datasource(s) with type fitting to the used steps
				- download / activate modules needed by the steps
				- select companies in the datasource an in the steps
				- create global parameters used in the steps

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Pipeline
				 @Username		= 'SQL'
				,@PipelineID	= 'P4'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Pipeline
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@PipelineID AS NVARCHAR(255))	,N'NULL')	+ N''''
			
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary table to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Pipeline",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 5, CONCAT(N'-- "SourcePipelineID": "',@PipelineID,'"'))
						,(0, 6, N'-- }')

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
						,(10, 3, N'DECLARE @PipelineID NVARCHAR(255)			= ''' + @PipelineID + N'''')
						,(10, 4, N'--This PipelneID will be deleted during import, if it exists. You should be sure !')
						,(10, 5, N'DECLARE @ReturnedPipelineID NVARCHAR (50);');

				-- Try_Delete existing Pipeline 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spDELETE_Pipeline '
							,'''SQL'''
							,',@PipelineID'
							,';'
							)	AS Command
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;

				-- Create the Pipeline as new Pipeline and catch the ID
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_Pipeline '
							,'''SQL'''
							,',@PipelineID'
							,','''	,dbo.sx_pf_pMaskSQL(PipelineName)		,''''
							,','''	,dbo.sx_pf_pMaskSQL(PipelineDescription),''''
							,','''	,PipelineDescriptionIconColor			,''''
							,','	,'NULL'									---NULL leads to positon at the end
							,','''	,EmailNotificationCODE					, ''''
							,', @OutputID = @ReturnedPipelineID OUTPUT;'	
							)	AS Command
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;
			
				-- Create the Steps
				INSERT INTO #tExport
					SELECT 
						 2000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_PipelineStep '
							,'''SQL'''
							,','	,'@ReturnedPipelineID'		
							,',''*'''
							,','''	,StepName								,''''
							,','''	,dbo.sx_pf_pMaskSQL(StepDescription)	,''''
							,','	,OrderOfExecution										
							,','''	,StepDataSourceID						, ''''
							,','''	,StepCompanyIDsJSON						, ''''
							,','''	,dbo.sx_pf_pMaskSQL(StepDetailsJSON)	, ''''  --inverted commas inside sql statements etc. are handled by the pMaskSQL function
							,','	,TimeExecutionLimit			
							,','	,Active
							,','''	,ModuleID								, ''''
							,','	,StartNextStepOnError					,';'	
							
							)	AS Command
					FROM system.tPipelineSteps tPS
						INNER JOIN system.tPipelines tP
							ON tPS.PipelineKey = tP.PipelineKey
					WHERE tP.PipelineID = @PipelineID
					ORDER BY OrderOfExecution;

				-- Create the Schedulers

				INSERT INTO #tExport
					SELECT 
						 3000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_PipelineSchedule '
							,'''SQL'''
							,','	,'@ReturnedPipelineID'		
							,',''*'''
							,','''	,ScheduleName							,''''
							,','''	,dbo.sx_pf_pMaskSQL(ScheduleDescription),''''
							,','''	,dbo.sx_pf_pMaskSQL(ScheduleDetailsJSON),''''
							,','	,Active									,';'	
							
							)	AS Command
					FROM system.tPipelineSchedules tPS
						INNER JOIN system.tPipelines tP
							ON tPS.PipelineKey = tP.PipelineKey
					WHERE tP.PipelineID = @PipelineID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');

				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Pipeline TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a pipeline with all steps in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spEXPORT_Setting;
GO

/*
	Procedure to create an OCT Importscript for an setting which is saved in system.tSettings
	mainly used for 
		- ClusterStartTab
		- DataFlowTab
		- Global parameters

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 08/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Setting
				 @Username		= 'SQL'
				,@SettingID		= 'ClusterStart_Html_Page'  -- ClusterStart_Html_Page, Integration_Html_Page, PipelineGlobalParameters
		PRINT    @RC

	Sourcetable
		SELECT * FROM system.tSettings


	Manual Export / Import for large Settings - long settings (more than 64.000 Charakters = mostly encoded images) are not displayed in SSMS and can't be read over sqlcmd - you must use bcp
		1. write the export to an table
		DROP TABLE IF EXISTS tmpout_Export;
		GO
		CREATE TABLE tmpout_Export
			(
				Command		NVARCHAR(MAX)
			);

		INSERT INTO tmpout_Export
			EXEC system.spEXPORT_Setting 'SQL','ClusterStart_Html_Page';

		SELECT * FROM dbo.tmpout_Export;


		2. EXECUTE this command over CMD or copy it into a bat file (Go to the drive you want to have the output to with D:, and to the fitting folder with cd ...)
		bcp.exe "SELECT Command FROM tmpout_Export" queryout import.sql -d AD00283_Plenovia_Dev -U octadmin -P [password] -S octproviderdev.database.windows.net -e error.log -c -C65001

		3. even you cant read it in SSMS, you can import it over SSMS as long a the file is smaller than 250 MB
		   (Output of SSMS is character limited, Input for SSMS if file size limited)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Setting',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Setting','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Setting
    (
          @Username			NVARCHAR(255)
		 ,@SettingID		NVARCHAR(50)	
    )

AS 
	BEGIN	


		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(@SettingID 				,N'NULL')	+ N''''
			

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username		= N'';
		IF @SettingID			IS NULL SET @SettingID			= N'';



		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary Table to store the collection of export commands
				DROP TabLE IF EXISTS #tExport;
				CREATE TabLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Setting",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'"'))
						,(0, 7, N'-- }')


				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username		NVARCHAR(255)		= ''SQL''')
						,(10, 3, N'DECLARE @SettingID		NVARCHAR(255)		= ''' + @SettingID + N'''')
						,(10, 4, N'--This Tab will be deleted during import, if it exists. You should be sure !')


				-- Create the Setting as new Setting
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.pPOST_Setting '
							,'''SQL'''
							,',@SettingID'
							,','''	,DataTypeCODE					,''''
							,','''	,SettingDescription				,''''	
							,','''	,ValueText						,''''
							,','''	,ValueDateTime					,''''
							,','	,ValueInt						,''
							,','	,ValueIntScale					,''
							,','''	,ValueJSON						,''''
							,','''	,Category						,''''
							,','	,ReadOnlySystemPropertyBool		,''
	
							)	AS Command
					FROM system.tSettings
					WHERE
						SettingID			= @SettingID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');

		
				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not execuTable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct daTabase access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Setting TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the Table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Setting'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports an Setting in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SettingID';
    SET @value      = N'ID of the Setting to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 


	GO

DROP PROCEDURE IF EXISTS system.pGET_LicenceInformation;
GO

/*
    Stored Procedure to retrieve the licence information from the database.
			
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall:
		DECLARE	@return_value int

		EXEC	 @return_value = system.pGET_LicenceInformation
				 @Username		= N'SQL'
				,@CustomerID	= N'AD032456'
				,@ServerID		= N'Server1'
				,@ServiceName	= N'OCT';

		SELECT	'Return Value' = @return_value

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_LicenceInformation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_LicenceInformation','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.pGET_LicenceInformation
	 @Username				NVARCHAR(255)
	,@CustomerID			NVARCHAR(255)
	,@ServerID				NVARCHAR(255)
	,@ServiceName			NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''','''
			+ ISNULL(@CustomerID			, N'NULL')			 + N''','''
			+ ISNULL(@ServerID		, N'NULL')			 + N'''';
			
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';
	DECLARE @WindowsServerName	 NVARCHAR(255) = @@SERVERNAME;


	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @CustomerID			IS NULL 
		OR @ServerID			IS NULL
		OR @ServiceName			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
        IF EXISTS
        (
            SELECT r.name from sys.database_role_members rm
            INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
            INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
            WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
        )
        BEGIN
            SET @TransactUsername = 'FactoryService';
        END;

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create licence information

		-- Declare table variable
		
		DECLARE @LicenceInformationTbl TABLE
		(
		  CustomerID		NVARCHAR(10)	NOT NULL
		 ,WindowsServerName NVARCHAR(255)	NOT NULL
		 ,ServiceName		NVARCHAR(255)	NOT	NULL
		 ,DatabaseServerID	NVARCHAR(255)	NOT NULL
		 ,DatabaseID		NVARCHAR(255)	NOT NULL
		 ,Timestamp			DATETIME		NOT NULL
		 ,PropertyName		NVARCHAR(255)	NOT NULL
		 ,PropertyValue		NVARCHAR(255)	NOT NULL
		)

		-- Insert database version property

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'DatabaseVersion', ValueText
		FROM planning.tgCluster
		WHERE PropertyID = 'C04'

		-- Insert number of data sources

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfDataSources', COUNT(*)
		FROM system.tDataSources

		-- Insert number of active companies

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfActiveCompanies', COUNT(*)
		FROM global.tCompanies;

		-- Insert number of active users

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfActiveUsers', COUNT(*)
		FROM system.trUser
		WHERE Status = 'Active';

		-- Insert number of products

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfProducts', COUNT(*)
		FROM planning.tdProducts;

		-- Insert number of productlines

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfProductLines', COUNT(*)
		FROM planning.tdProductLines;

		-- Insert number of Factories

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfFactories', COUNT(*)
		FROM planning.tdFactories;



		SELECT * 
		FROM @LicenceInformationTbl;

		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, 0, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryService;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_LicenceInformation'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Description: Stored Procedure to retrieve the licence information from the database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CustomerID';
    SET @value      = N'CustomerID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ServerID';
    SET @value      = N'ServerID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ServiceName';
    SET @value      = N'ServiceName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS system.pGET_Setting;
GO

/*
	ToDo
		- RightCheck
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9

	Testcall
		EXEC system.pGET_Setting 'SQL','DBVersion'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_Setting',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_Setting','PARAMETER',NULL)
*/
CREATE PROCEDURE system.pGET_Setting
	 @Username	NVARCHAR(255)
	,@SettingID NVARCHAR(50)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,	 N'NULL')+ N''',''' 
			+ ISNULL(@SettingID, N'NULL')+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	DECLARE @IsFactoryService INT = 0

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @SettingID	IS NULL SET @SettingID	= N'';

	BEGIN TRY
		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @SettingID	= dbo.sx_pf_pProtectID		(@SettingID);
			
		IF @SettingID	= N'' 
		BEGIN
			SET @ResultCode = 404;
			THROW 60000, N'Empty input parameters', 10;
		END;

		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;
	
		-- STEP 1.1 - Determine transaction user
		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'octservice';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;

		-- STEP 3 - Output result
		SELECT
			 SettingID
			,SettingDescription
			,DataTypeCODE
			,ValueText
			,ValueDateTime
			,ValueInt
			,ValueIntScale
			,ValueJSON
			,Category
			,ReadOnlySystemPropertyBool
		FROM [system].tSettings
		WHERE SettingID = @SettingID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

GRANT EXECUTE ON [system].pGET_Setting TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_Setting'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to read one setting. To read all settings, use system.pGETSettings.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SettingID';
    SET @value      = N'ID of the Setting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS system.pGET_Settings;
GO
/*
	ToDo
		- RightCheck
		- Documentation
	
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC system.pGET_Settings 'SQL'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_Settings',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_Settings','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.pGET_Settings
	 @Username		NVARCHAR(255)
	,@Category		NVARCHAR(255) = NULL
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,	N'NULL')+ N''',''' 
			+ ISNULL(@Category, N'NULL')+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';	
	
	DECLARE @IsFactoryService INT = 0;

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username	= N'';

	BEGIN TRY
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
	
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;
	
		-- STEP 1.1 - Determine transaction user
		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'FactoryService';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;

		-- STEP 3 - Output result
		SELECT
			 SettingID
			,SettingDescription
			,DataTypeCODE
			,ValueText
			,ValueDateTime
			,ValueInt
			,ValueIntScale
			,ValueJSON
			,Category
			,ReadOnlySystemPropertyBool
		FROM [system].[tSettings]
		WHERE Category = @Category 
			OR @Category IS NULL;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

GRANT EXECUTE ON [system].[pGET_Settings] TO pf_PlanningFactoryService;
GRANT EXECUTE ON [system].[pGET_Settings] TO pf_PlanningFactoryUser;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_Settings'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'pGET_Settings';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Category';
    SET @value      = N'Category';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS system.spGET_APILog;
GO

/*
	Procedure to read the last @RowCount Rows from the API Log table
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_APILog
					 @Username = 'SQL'
					,@RowCount = 1000;
		PRINT    @RC;

	Testcall BaseTable
		SELECT * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_APILog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_APILog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_APILog
    (
         @Username		NVARCHAR(255)
        ,@RowCount		INT
    )

AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@RowCount AS NVARCHAR(255))	,N'NULL')	+ N''
				
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @RowCount				IS NULL SET @RowCount			= 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

					IF @ResultCode <> 200 AND NOT @TransactUsername = 'FactoryService' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Invalid rights', 16, 10);
						END

				-- Data Transaction
				SELECT TOP (@RowCount)
					*
				FROM system.tAPI_Log
				ORDER BY LogKey DESC;

				SET @ResultCode = 200;

		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_APILog TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_APILog'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns the last @RowCount Rows from the API Log table';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@RowCount';
    SET @value      = N'Number of rows the procedure should retun.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 GO
DROP PROCEDURE IF EXISTS system.spGET_Connector;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_Connector
				 @Username		= 'SQL',
				 @SourceSystemID = 'NAVISION',
				 @SourceSystemType = 'MSSQL',
				 @ModuleID = 'FIN'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Connector
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50) = ''
		,@SourceSystemType		NVARCHAR(50) = ''
		,@ModuleID				NVARCHAR(50) = ''		           	
	)

AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			IF @SourceSystemID = '' OR @SourceSystemType = '' OR @ModuleID = ''
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Connector doesn''t exist', 16, 10);
				END;

			IF @SourceSystemID = '' OR @SourceSystemType = '' OR @ModuleID = '' OR NOT EXISTS (
				SELECT 1
				FROM system.tConnectors
				WHERE
					SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND ModuleID = @ModuleID
				)
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Connector doesn''t exist', 16, 10);
				END
			
			-- Data Transaction
			SELECT
				SourceSystemID,
				SourceSystemType,
				ModuleID,
				ConnectorDescription,
				ConnectorVersion,
				ConnectorDescriptionURL
			FROM system.tConnectors
			WHERE SourceSystemID = @SourceSystemID
			AND SourceSystemType = @SourceSystemType
			AND ModuleID = @ModuleID;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_Connector TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all ConnectorQueries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
DROP PROCEDURE IF EXISTS system.spGET_ConnectorQuery;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorQuery
				 @Username		= 'SQL'
				 ,@SourceSystemID = 'NAVISION'
				 ,@SourceSystemType = 'MSSQL'
				 ,@ModuleID = 'FIN'
				 ,@QueryName = 'AccountingJournal'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQuery',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQuery','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorQuery
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50)
		,@SourceSystemType		NVARCHAR(50)
		,@ModuleID				NVARCHAR(50)	= '' -- optional, especially for GetCompanies query which used to be not bound to a module
		,@QueryName				NVARCHAR(255)	= '' -- optional, if empty return all queries of connector		           	
	)
AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@QueryName AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';
		IF @QueryName			IS NULL SET @QueryName			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			IF @SourceSystemID = '' OR @SourceSystemType = '' OR (@ModuleID = '' AND @QueryName <> 'GetCompanies') OR NOT EXISTS (
				SELECT 1
				FROM system.tConnectors
				WHERE 
					SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND (ModuleID = @ModuleID OR (@ModuleID = '' AND @QueryName = 'GetCompanies'))
			)
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Connector doesn''t exist', 16, 10);
				END;

			-- @ModuleID is empty for GetCompanies -> return first GetCompanies that is found for this SourceSystemID and SourceSystemType
			IF @ModuleID = '' AND @QueryName = 'GetCompanies'
				BEGIN
					SELECT TOP 1
						c.SourceSystemID,
						c.SourceSystemType,
						c.ModuleID,
						cq.QueryName,
						cq.QuerySQL,
						cq.QueryVersion,
						cq.QueryVersionComment,
						cq.Timestamp,
						cq.CreatedBy
					FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors AS c
					ON c.ConnectorKey = cq.ConnectorKey
					LEFT JOIN system.tConnectorQueries AS cqMax
					ON cq.ConnectorKey = cqMax.ConnectorKey
					AND cq.QueryName = cqMax.QueryName
					AND cq.QueryVersion < cqMax.QueryVersion
					WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND cq.QueryName = @QueryName
						AND cqMax.QueryVersion IS NULL;
				END
			-- QueryName specified but doesn't exist -> return empty result set
			ELSE IF @QueryName <> '' AND NOT EXISTS (
				SELECT 1
				FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors   AS c
					  ON c.ConnectorKey = cq.ConnectorKey
				WHERE
					c.SourceSystemID = @SourceSystemID
					AND c.SourceSystemType = @SourceSystemType
					AND c.ModuleID = @ModuleID
					AND cq.QueryName = @QueryName
				)
				BEGIN
					SELECT
						@SourceSystemID AS SourceSystemID,
						@SourceSystemType AS SourceSystemType,
						@ModuleID AS ModuleID,
						@QueryName AS QueryName,
						'' AS QuerySQL,
						0 AS QueryVersion,
						'' AS QueryVersionComment,
						NULL AS Timestamp,
						'' AS CreatedBy
				END
			-- everything is normal -> return regular result
			ELSE
				BEGIN
					SELECT
						c.SourceSystemID,
						c.SourceSystemType,
						c.ModuleID,
						cq.QueryName,
						cq.QuerySQL,
						cq.QueryVersion,
						cq.QueryVersionComment,
						cq.Timestamp,
						cq.CreatedBy
					FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors AS c
					ON c.ConnectorKey = cq.ConnectorKey
					LEFT JOIN system.tConnectorQueries AS cqMax
					ON cq.ConnectorKey = cqMax.ConnectorKey
					AND cq.QueryName = cqMax.QueryName
					AND cq.QueryVersion < cqMax.QueryVersion
					WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND c.ModuleID = @ModuleID
						AND (cq.QueryName = @QueryName OR @QueryName = '')
						AND cqMax.QueryVersion IS NULL;
				END;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorQuery TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorQuery'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all ConnectorQueries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
DROP PROCEDURE IF EXISTS system.spGET_ConnectorQueryFields;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorQueryFields
				 @Username		= 'SQL'
				 ,@ModuleID = 'FIN'
				 ,@QueryName = 'AccountingJournal'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQueryFields',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQueryFields','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorQueryFields
    (
         @Username				NVARCHAR(255)
		,@ModuleID				NVARCHAR(50)
		,@QueryName				NVARCHAR(255)	           	
	)
AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@QueryName AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';
		IF @QueryName			IS NULL SET @QueryName			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			IF @QueryName <> 'GetCompanies' AND (@ModuleID = '' OR @QueryName = '' OR NOT EXISTS(
				SELECT 1
				FROM INFORMATION_SCHEMA.columns
				WHERE (
					(TABLE_SCHEMA = 'integration' AND TABLE_NAME = 't' + @ModuleID + '_' + @QueryName)
					OR (@QueryName = 'GetCompanies')
				)
			))
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Target table doesn''t exist', 16, 10);
				END;

			-- Data Transaction
			SELECT
				SCHEMA_NAME(t.schema_id) + '.' + t.name AS TableName,
				c.name AS ColumnName,
				UPPER(TYPE_NAME(c.system_type_id)) +
					CASE
						WHEN c.precision = 0 AND c.scale = 0 AND max_length > 0 THEN
							'(' + CAST(c.max_length / 2 AS NVARCHAR(10)) + ')'
						WHEN c.precision > 0 AND c.scale > 0 THEN
							'(' + CAST(c.precision AS NVARCHAR(10)) + ', ' + CAST(c.scale AS NVARCHAR(10)) + ')'
						ELSE ''
					END AS DataType,
				CASE WHEN c.is_nullable = 1 THEN 'NULL' ELSE 'NOT NULL' END AS NULLAble,
				p.value AS ColumnInformation,
				RANK() OVER (ORDER BY c.column_id) AS OrderNumber
			FROM sys.tables AS t
			JOIN sys.columns AS c
			ON c.object_id = t.object_id
			LEFT JOIN sys.extended_properties AS p
			ON p.major_id = t.object_id
			AND p.minor_id = c.column_id
			AND p.class = 1
			WHERE 1=1
				AND SCHEMA_NAME(t.schema_id) = 'integration'
				AND t.name = 't' + @ModuleID + '_' + @QueryName
				AND c.name <> 'RowKey'
				AND @QueryName <> 'GetCompanies'

			UNION ALL

			SELECT
				SCHEMA_NAME(t.schema_id) + '.' + t.name AS TableName,
				c.name AS ColumnName,
				UPPER(TYPE_NAME(c.system_type_id)) +
					CASE
						WHEN c.precision = 0 AND c.scale = 0 AND max_length > 0 THEN
							'(' + CAST(c.max_length / 2 AS NVARCHAR(10)) + ')'
						WHEN c.precision > 0 AND c.scale > 0 THEN
							'(' + CAST(c.precision AS NVARCHAR(10)) + ', ' + CAST(c.scale AS NVARCHAR(10)) + ')'
						ELSE ''
					END AS DataType,
				CASE WHEN c.is_nullable = 1 THEN 'NULL' ELSE 'NOT NULL' END AS NULLAble,
				p.value AS ColumnInformation,
				RANK() OVER (ORDER BY c.column_id) AS OrderNumber
			FROM sys.tables AS t
			JOIN sys.columns AS c
			ON c.object_id = t.object_id
			LEFT JOIN sys.extended_properties AS p
			ON p.major_id = t.object_id
			AND p.minor_id = c.column_id
			AND p.class = 1
			WHERE 1=1
				AND SCHEMA_NAME(t.schema_id) = 'global'
				AND t.name = 'tCompanies'
				AND c.name NOT IN ('RowKey', 'DataSourceKey', 'CompanyKey', 'CustomValuesJSONUser', 'OwnerCODE')
				AND @QueryName = 'GetCompanies'
			ORDER BY OrderNumber ASC;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorQueryFields TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorQueryFields'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all ConnectorQueries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS system.spGET_ConnectorTree;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorTree
				 @Username		= 'SQL'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorTree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorTree','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorTree
    (
         @Username				NVARCHAR(255)
    )

AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			-- Temp table to store all integration tables of all modules
			DROP TABLE IF EXISTS #ModuleTables;

			SELECT
				SUBSTRING(t.name, 2, CHARINDEX('_', t.name) - 2) AS ModuleID,
				SUBSTRING(t.name, CHARINDEX('_', t.name) + 1, 255) AS IntegrationTable
			INTO #ModuleTables
			FROM sys.tables t
			JOIN sys.schemas s ON s.schema_id = t.schema_id
			WHERE s.name = 'integration'
			AND t.name LIKE 't%[_]%';

			-- Add GetCompanies to all modules as default
			INSERT INTO #ModuleTables
			SELECT DISTINCT
				ModuleID,
				'GetCompanies'
			FROM #ModuleTables;
				
			-- Data output
			SELECT
				SourceSystemID,
				SourceSystemType,
				ModuleID,
				ConnectorDescription,
				ConnectorVersion,
				QueryName
			FROM (
				SELECT
					c.SourceSystemID,
					c.SourceSystemType,
					c.ModuleID,
					c.ConnectorDescription,
					c.ConnectorVersion,
					mt.IntegrationTable AS QueryName,
					CASE WHEN mt.IntegrationTable = 'GetCompanies' THEN 0 ELSE 1 END IsGetCompanies
				FROM system.tConnectors AS c
				CROSS APPLY #ModuleTables AS mt
				LEFT JOIN system.tConnectorQueries AS ct
				ON ct.ConnectorKey = c.ConnectorKey
				AND ct.QueryName = mt.IntegrationTable
				WHERE c.ModuleID = mt.ModuleID

				UNION
			
				SELECT
					c.SourceSystemID,
					c.SourceSystemType,
					c.ModuleID,
					c.ConnectorDescription,
					c.ConnectorVersion,
					ct.QueryName AS QueryName,
					CASE WHEN ct.QueryName = 'GetCompanies' THEN 0 ELSE 1 END IsGetCompanies
				FROM system.tConnectors AS c
				LEFT JOIN system.tConnectorQueries AS ct
				ON ct.ConnectorKey = c.ConnectorKey
				WHERE ct.ConnectorKey IS NOT NULL
			) Queries
			ORDER BY SourceSystemID, ModuleID, SourceSystemType, IsGetCompanies, QueryName;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorTree TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorTree'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all Connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 

	GO
 
DROP PROCEDURE IF EXISTS system.spGET_Dashboards;
GO

/*
	Procedure to GET Dashboard definitions, can be called
		- without DashboardID -> to get all Dashboards, but without Layoutdefinitions (which may be big due to images etc.)
		- with Dashboard ID -> to get one Dashboard including the Layoutdefinition

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_Dashboards
				 @Username		= 'SQL'
				,@DashboardID	= 'D1';

		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Dashboards',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Dashboards','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Dashboards
    (
         @Username				NVARCHAR(255)
        ,@DashboardID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username	,N'NULL')		+ N''','''										 
				+ ISNULL(@DashboardID	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @DashboardID 			IS NULL SET @DashboardID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'Transaction user don`t exists', 16, 10);
					END;

				-- Check existence of ID
				IF @DashboardID <> N'' AND NOT EXISTS (
					SELECT DashboardID FROM system.tDashboards WHERE DashboardID = @DashboardID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Dashboard don`t exits', 16, 10);
					END

				-- Data Transaction
				PRINT CONCAT(N'Start reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				IF @DashboardID = ''
					BEGIN
						SELECT
							 DashboardID
							,DashboardName
							,DashboardDescription
							,NULL					AS LayoutDefinition -- not an empty String, as the column is varbinary
						FROM system.tDashboards;
						SET @EffectedRows = @@ROWCOUNT;
					END;

				IF @DashboardID <> ''
					BEGIN
						SELECT
							 DashboardID
							,DashboardName
							,DashboardDescription
							,LayoutDefinition
						FROM system.tDashboards
						WHERE DashboardID = @DashboardID;
						SET @EffectedRows = @@ROWCOUNT;
					END;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spGET_Dashboards TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Dashboards'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns all Dashboards (without Layout), or one Dashboard (with Layout)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DashboardID';
    SET @value      = N'optional - ID of the Dashboard. Can be skiped to get all Dashboards (without Layoutdefinition).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spGET_DatabaseRoles;
GO

/*
	Procedure to return the database roles of the executing user

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DatabaseRoles
				 @Username = 'SQL'				--this username is not used anywhere
		PRINT    @RC

		SELECT USER_NAME();
		SELECT ORIGINAL_LOGIN();

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DatabaseRoles',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DatabaseRoles','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_DatabaseRoles
    (
         @Username				NVARCHAR(255)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
			@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
			,@ParameterString	NVARCHAR(MAX)	= N''''					
			+ ISNULL(@Username					,N'NULL')	+ N''''											 
	
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N''
			,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY

				-- Select the database role of the executing user
				PRINT CONCAT(N'Reading Roles',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #Roles;
				CREATE TABLE  #Roles
					(
						DatabaseRoleName NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
					);

				-- Query all roles of the current login - this works only, if the login is not mapped to the username dbo
				INSERT INTO #Roles
					SELECT
						DP1.[name]							AS DatabaseRoleName
						-- ,ISNULL (DP2.[name], 'No members') AS DatabaseUserName
					FROM sys.database_role_members DRM
						RIGHT OUTER JOIN sys.database_principals DP1
							ON DRM.role_principal_id = DP1.principal_id
						LEFT OUTER JOIN sys.database_principals DP2
							ON DRM.member_principal_id = DP2.principal_id
					WHERE
								DP1.[type] = 'R' 
							AND DP2.[name] = ORIGINAL_LOGIN();

				-- For the case that the user is the dbo
				IF USER_NAME() = 'dbo'
				BEGIN
					INSERT INTO #Roles
						VALUES ('dbo (User is owner of the database.)');

					INSERT INTO #Roles
						SELECT
								DP1.[name]							AS DatabaseRoleName
								-- ,ISNULL (DP2.[name], 'No members') AS DatabaseUserName
							FROM sys.database_role_members DRM
								RIGHT OUTER JOIN sys.database_principals DP1
									ON DRM.role_principal_id = DP1.principal_id
								LEFT OUTER JOIN sys.database_principals DP2
									ON DRM.member_principal_id = DP2.principal_id
							WHERE
										DP1.[type] = 'R' 
									AND DP2.[name] = 'dbo';
				END

				SELECT * FROM #Roles;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading of roles done',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_DatabaseRoles TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_DatabaseRoles'    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Returns the roles of the executing user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username';
    SET @value      = N'Parameter is ignored.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
 
DROP PROCEDURE IF EXISTS system.spGET_DataSources;
GO
/*
	Procedure to GET one or all DataSources. Due to former migration from entity framework
	the procedure has the following special characteristics:
	- the DataSourceID is an Integer Value

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DataSources
				 @Username		= 'SQL'
				,@DataSourceID  = '';
		PRINT    @RC;
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DataSources',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DataSources','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_DataSources
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','''							 
				+ ISNULL(@DataSourceID				,N'NULL')	+ N''''
				
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DataSourceID 			IS NULL SET @DataSourceID		= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				SELECT 
					 DataSourceID
					,DataSourceName
					,SourceType
					,ConnectionValuesJSON 
				FROM system.tDataSources
				WHERE 
						DataSourceID	= @DataSourceID
					OR @DataSourceID	= '';
	
				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;

		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_DataSources TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_DataSources'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Prozedur gibt eine oder all Datasaources zur�ck.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'ID of the requested DataSource. Use DataSourceID = '''' to get all DataSources.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spGET_DBInfo;
GO
/*
	Procedure to get basic informations about one Database. Informations can be requested
		- from OCTService for an User
		- from OCTService for himself (passing SQL as Clientusername means check the Current SQL User)

	TODO:
		- resolve Rolemembership when assigned over Security Groups
		- check AccessRight (implicit sysadmin) over AzureAD SQL Server Administrator Group Membership

	Infolevel
		- 0 = information it clientuser can access this cluster or not
		- 1 = all the Service needs to display a tile
			- access
			- tileimage
			- number of users
			- database version

		- 2 = full Informations
			- user role
			- database size

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall Procedure
		-- Service for an user
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DBInfo
				 @Username		= 'SQL'
				,@ClientUser	= 'gerd.tautenhahn@saxess-software.com'
				,@InfoLevel		= 2;
		PRINT    @RC;

		-- Service for Service
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DBInfo
				 @Username		= 'SQL'
				,@ClientUser	= 'SQL'
				,@InfoLevel		= 2;
		PRINT    @RC;

		SELECT TOP 3 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DBInfo',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DBInfo','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_DBInfo
         @Username				NVARCHAR(255)
        ,@ClientUser			NVARCHAR(255)
		,@InfoLevel				INT		      = 1				
    
AS 
	BEGIN	

		-- SET NOCOUNT ON; -- activate this, if you read the resultset with an external source

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@ClientUser						,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@IsOCTService		BIT				= 0;

		-- NULL Protection for all mandatory Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @ClientUser		IS NULL SET @ClientUser		= N'';

		-- The string SQL means the octservice asks for himself
		IF @ClientUser = 'SQL' SELECT @ClientUser = CURRENT_USER; 	-- !! Clientuser will never exist in OCTUsermanagement
																	-- !! Clientuser may be 'dbo' or an AzureActive Directory Identity
																	-- passing SQL as Clientusername means check the Current SQL User
		IF EXISTS
			(																		 
			SELECT 
				DP1.[name]
			FROM sys.database_role_members DRM
				RIGHT OUTER JOIN sys.database_principals DP1
					ON DRM.role_principal_id = DP1.principal_id
				LEFT OUTER JOIN sys.database_principals DP2
					ON DRM.member_principal_id = DP2.principal_id
			WHERE
						DP1.[type] = 'R' 
					AND DP2.[name] = @ClientUser
					AND DP1.[name] IN ('pf_PlanningFactoryService','db_owner')
					)
		SET @IsOCTService = 1; -- is OCTService or can act like OCTService due to db_owner status

		PRINT CONCAT('Clientuser: ', @ClientUser,'  IsOCTService: ',@IsOCTService);

		DECLARE @ClientUserAccess_Flag INT = 0;

		-- START TRY			***********************************************************************************
		BEGIN TRY
				---- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						-- nothing, to enable Footprint Protection
						PRINT 'Unknown user - eliminate footprint'
						-- SET @ResultCode = 403;
						-- RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Caching table
				DROP TABLE IF EXISTS #tInfoCache;
				CREATE TABLE #tInfoCache
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,PropertyID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PropertyName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,InfoLevel				INT											NOT NULL
						,ValueInt				INT											NOT NULL
						,ValueMoney				MONEY										NOT NULL
						,ValueText				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ValueDatetime			DATETIME									NULL
						,ValueBinary			VARBINARY(MAX)								NULL --max 8000 Byte to avoid Varbinary(MAX)

						CONSTRAINT PK_tInfoCache PRIMARY KEY CLUSTERED (RowKey)				
					);

				-- INFO LEVEL 0 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

				-- check AccessRight for ClientUser
				IF @IsOCTService = 1
					BEGIN 
						SET @ClientUserAccess_Flag = 1;
					END
				ELSE
					BEGIN
						SELECT @ClientUserAccess_Flag = COUNT(UserKey)
						FROM system.trUser 
						WHERE 
								Username = @ClientUser
							AND Status = 'Active';
					END

				-- check if database has public user
				IF @ClientUserAccess_Flag = 0 AND EXISTS(SELECT 1 FROM system.trUser WHERE Username = 'public' AND Status = 'Active')
				BEGIN
					SET @ClientUserAccess_Flag = 1
				END

				INSERT INTO #tInfoCache
					SELECT
						 'CU01'										AS PropertyID
						,'Flag for ClientUser is an active User'	AS PropertyName
						,0											AS InfoLevel	
						,@ClientUserAccess_Flag						AS ValueInt
						,0											AS ValueMoney	
						,''											AS ValueText	
						,NULL										AS ValueDatetime
						,NULL										AS ValueBinary;

				-- Footprint protection
				IF @ClientUserAccess_Flag = 0
					BEGIN
						SET @ParameterString = REPLACE (@ParameterString,@ClientUser,'<Request for an non Clustermember>');
						SET @Username		 = '<Non-Clustermember>';
					END

				-- INFO LEVEL 1 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

				-- Only if ClientUser has Access, further Informations are delivered (valid for all Info Levels)
				IF @ClientUserAccess_Flag = 1
					BEGIN
						-- Clustername
						INSERT INTO #tInfoCache
							SELECT
								 'C01'										AS PropertyID
								,'ClusterName'								AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,'<give me a name>'							AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueText = S.ValueText
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'ClusterName'
									AND #tInfoCache.PropertyID	= 'C01';

						-- Clusterdescription
						INSERT INTO #tInfoCache
							SELECT
								 'C02'										AS PropertyID
								,'ClusterDescription'								AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueText = S.ValueText
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'ClusterDescription'
									AND #tInfoCache.PropertyID	= 'C02';

						-- Clusterversion
						INSERT INTO #tInfoCache
								SELECT
									 'C03'										AS PropertyID
									,'OCT.core Database Version'				AS PropertyName
									,1											AS InfoLevel	
									,0											AS ValueInt
									,0											AS ValueMoney	
									,ValueText									AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.tSettings
								WHERE SettingID = 'DBVersion';
						
						-- Cluster_Tile_Logo

						INSERT INTO #tInfoCache
							SELECT
								 'C04'										AS PropertyID
								,'Cluster_Tile_Logo'						AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueBinary = FileBody
								FROM 
									planning.tFiles
								WHERE 
										FileID					= 'Cluster_Tile_Logo'
									AND #tInfoCache.PropertyID	= 'C04';

						-- Maintenance Mode -> this property exists only after first saving of database properties
						INSERT INTO #tInfoCache
							SELECT
								 'C05'										AS PropertyID
								,'Cluster is in Maintenance Mode Flag'		AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

							UPDATE #tInfoCache
									SET ValueInt = S.ValueInt
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'MaintenanceMode'
									AND #tInfoCache.PropertyID	= 'C05';

						-- Scheduler Owner -> this property exists only after first saving of database properties
						INSERT INTO #tInfoCache
							SELECT
								 'C06'										AS PropertyID
								,'SchedulerSettingsOwnerServer'				AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache 
								SET ValueText = S.ValueText
							FROM 
								system.tSettings S
							WHERE S.SettingID  = 'SchedulerSettingsOwnerServer'
							AND #tInfoCache.PropertyID	= 'C06';

						-- Usercount for Licence information
						INSERT INTO #tInfoCache
							SELECT
									 'C10'										AS PropertyID
									,'Number of active Users'					AS PropertyName
									,1											AS InfoLevel	
									,COUNT(UserKey)								AS ValueInt
									,0											AS ValueMoney	
									,''											AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.trUser
								WHERE Status = 'Active';

						-- Information, if the active User is an OCT Clusteradmin
						INSERT INTO #tInfoCache
							SELECT
									 'CU01'										AS PropertyID
									,'User is OCT Administrator'				AS PropertyName
									,1											AS InfoLevel	
									,COUNT(UserKey)								AS ValueInt
									,0											AS ValueMoney	
									,''											AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.trUser
								WHERE 
										Username = @ClientUser
									AND IsAdministratorFlag = 1;

					END

				-- INFO LEVEL 2 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				-- Informations for higher Info Levels
				IF @ClientUserAccess_Flag = 1 AND @InfoLevel >= 2
					BEGIN
						-- Databaseroles of the Clientuser
							-- Works only if the Clientuser is not the dbo
							-- returns no Serverroles
							-- returns no roles if the dabaserole is inherited from a serverrole (e.g. somebody has db_owner role due to sysadmin login etc.)
						INSERT INTO #tInfoCache	
							SELECT
								 'CU10'								AS PropertyID
								,'Database Roles of the Clientuser' AS PropertyName
								,2									AS InfoLevel
								,0									AS ValueInt
								,0									AS ValueMoney
								,COALESCE(
											STUFF( 		-- Stuffs all names in one list
														(
														SELECT 
															DP1.[name] +', '
														FROM sys.database_role_members DRM
															RIGHT OUTER JOIN sys.database_principals DP1
																ON DRM.role_principal_id = DP1.principal_id
															LEFT OUTER JOIN sys.database_principals DP2
																ON DRM.member_principal_id = DP2.principal_id
														WHERE
																	DP1.[type] = 'R' 
																AND DP2.[name] = @ClientUser
														FOR XML PATH(''), TYPE
													).value('.', 'NVARCHAR(MAX)') 
													,1,0,''
												) 	
											, 'No direct assigned Databaseroles (Serveroles,dbo status, groupmemberships or admin status over AAD may be given !)'	
										)							AS ValueText	--Databaserolename
								,NULL								AS ValueDatetime
								,NULL								AS ValueBinary;
							
						-- dbo determination - its different on Azure and on-prem
						IF CONVERT(NVARCHAR(255),SERVERPROPERTY ('Edition')) LIKE '%Azure%'	
							BEGIN
								INSERT INTO #tInfoCache
									SELECT
										 'DB01'								AS PropertyID
										,'Name or sid of dbo'				AS PropertyName
										,2									AS InfoLevel
										,0									AS ValueInt
										,0									AS ValueMoney
										,CONVERT(NVARCHAR(255),sid,2)		AS ValueText
										,NULL								AS ValueDatetime
										,NULL								AS ValueBinary
									FROM sys.sysusers where name = 'dbo'
							END
						ELSE -- on prem 
							BEGIN
								INSERT INTO #tInfoCache
									SELECT
										 'DB01'								AS PropertyID
										,'Name of dbo'				AS PropertyName
										,2									AS InfoLevel
										,0									AS ValueInt
										,0									AS ValueMoney
										,SUSER_SNAME(owner_sid)				AS ValueText	
										,NULL								AS ValueDatetime
										,NULL								AS ValueBinary
									FROM sys.databases WHERE name = DB_NAME();
							END

						-- Database Rows Size
						INSERT INTO #tInfoCache
							SELECT
								 'DB02'									AS PropertyID
								,'Database Size (Rows) in GB'			AS PropertyName
								,2										AS InfoLevel	
								,0										AS ValueInt
								,SUM(CAST(size AS MONEY) * 8/1024/1024)	AS ValueMoney	
								,''										AS ValueText	
								,NULL									AS ValueDatetime
								,NULL									AS ValueBinary
							FROM sys.database_files
							WHERE 
								type = 0;

						-- Database Log Size
						INSERT INTO #tInfoCache
							SELECT
								 'DB03'									AS PropertyID
								,'Database Size (Log) in GB'			AS PropertyName
								,2										AS InfoLevel	
								,0										AS ValueInt
								,SUM(CAST(size AS MONEY) * 8/1024/1024)	AS ValueMoney	
								,''										AS ValueText	
								,NULL									AS ValueDatetime
								,NULL									AS ValueBinary
							FROM sys.database_files
							WHERE 
								type = 1;
						
						-- Options to To:
						-- Last Database Read Access
						-- Last Database Write Access
						-- Number of scheduled Pipelines
						
					END

				-- OUTPUT

				SELECT	
					*
				FROM #tInfoCache
				WHERE InfoLevel <= @InfoLevel
				ORDER BY
					InfoLevel,
					PropertyID;
			
				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spGET_DBInfo TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_DBInfo'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1;					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get a collection of database informations. Pass the Clientusername SQL to request the rights of the service.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ClientUser';
    SET @value      = N'Not logged if its not a clustermember to protect from footprinting access scans.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@InfoLevel';
    SET @value      = N'Control the amount of informations deliverd. 1 = Basic Info, 2 = extended Info.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP PROCEDURE IF EXISTS system.spGET_Emails;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_Emails
				 @Username		= 'SQL'
				,@EmailID		= ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Emails',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Emails','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Emails
    (
         @Username				NVARCHAR(255)
        ,@EmailID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','											 
				+ ISNULL(@EmailID 				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @EmailID				IS NULL SET @EmailID			= N'';


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @EmailID <> '' AND NOT EXISTS (
					SELECT EmailID FROM system.tEmails WHERE EmailID = @EmailID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Email don`t exits', 16, 10);
					END

				-- Data Transaction
				SELECT 
					 EmailID
					,EmailSubject
					,EmailRecipients
					,EmailText
					,EmailAttachments
					,EmailStatusCODE
					,SourceTypeCODE
					,SourceID
					,CreationDate
					,PostDate
				FROM system.tEmails
				WHERE
						EmailID	= @EmailID
					OR	@EmailID = ''
				ORDER BY EmailID;

				SET @EffectedRows = @@ROWCOUNT;
				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spGET_Emails TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Emails'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all Emails.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@EmailID';
    SET @value      = N'EmailID - optional to get only one Email, use empty string to get all Emails.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
  
DROP PROCEDURE IF EXISTS system.spGET_Icons;
GO
/*
	Procedure to get a single icon or all

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall for all icons
		EXEC system.spGET_Icons	'';

	Testcall for one icon 
		EXEC system.spGET_Icons	'hase.svg';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Icons',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Icons','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Icons
	 @IconName		NVARCHAR(255)	= N''

AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @UserName				NVARCHAR(255)	= N'anonym'
	
	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@UserName, N'NULL') + N''', '+ ISNULL(@IconName, N'NULL') + N''''
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';
		IF @IconName IS NULL SET @IconName = N'';

		BEGIN TRY
			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = 'anonym'

			IF @IconName <> ''
				BEGIN
					IF NOT EXISTS (
						SELECT TOP (1) 1 
						FROM system.tIcons
						WHERE IconName	= @IconName
						)
					BEGIN
						SET @ResultCode = 404;
						THROW 60000, N'File not found', 100;
					END;
				END;

			SELECT
				 IconName
				,IconName + ISNULL(N'.' + FileExtension, N'') AS IconNameWithExtension
				,FileExtension
				,FileBody
			FROM system.tIcons
			WHERE 
					IconName= @IconName 
				OR	@IconName = ''
			
			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;

GO

GRANT EXECUTE ON OBJECT ::system.spGET_Icons TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spGET_Icons TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Icons'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to get a single icon or all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@IconName';
    SET @value      = N'IconName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS system.spGET_LogicIndexItems;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		EXEC system.spGET_LogicIndexItems
			 'SQL';

		SELECT * FROM system.tLogicIndex

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_LogicIndexItems',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_LogicIndexItems','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_LogicIndexItems
	 @Username		NVARCHAR(255)

AS
BEGIN

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''''''							
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';

		BEGIN TRY
			BEGIN TRANSACTION GET_LogicIndexItems;

			-- Determine transaction user
			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			--Return all Items
			SELECT 
				 ModuleID
				,ModuleVersion
				,ActivationDate
			FROM system.tLogicIndex

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION GET_LogicIndexItems;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION GET_LogicIndexItems;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spGET_LogicIndexItems TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_LogicIndexItems'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'spGET_LogicIndexItems';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
 
DROP PROCEDURE IF EXISTS system.spGET_ObjectTree;
GO
/*
	Procedure to GET the ObjectTree, which lists all database objects with its metadata attribute
	The Procedure can't list objects on which the service don't has access rights.
	To see all object, execute this Stored procedure in SSMS as DBOwner.

	It looked promising to add in this procedure the clause "WITH EXECUTE AS OWNER" - this brings up in on-prem systems all objects. 
	But it was no longer possible to excecute this stored procedure on Azure, even the owner is in both cases dbo.
	Maybe as in Azure the login behind dbo is Active Directory Account (which appears not as local login), wheras on-prem the dbo maps to a local login.
	https://docs.microsoft.com/de-de/sql/t-sql/statements/execute-as-clause-transact-sql?view=sql-server-ver15
	Executing the Procedure with "EXECUTE AS OWNER 'dbo'" also don't worked.

	The Procedure therefore works on Azure only, if the Owner of the Database is an SQL-Login
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spGET_ObjectTree 'SQL'		
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ObjectTree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ObjectTree','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_ObjectTree  
    (
         @Username				NVARCHAR(255)
    )

-- must be always execute as owner to show all objects - but this don't work on Azure, if the Owner is an AAD User - OPEN PROBLEM -> alternative ALTER AUTHORIZATION to dbo ?
WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''''											 

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spGET_ObjectTree;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

					-- Data Transaction						*************************************************************************

					SELECT
						 CASE
							WHEN tree.Owner = '<owner definition tag not defined>'		THEN		'RGB(253,138,138)'	 --red
							WHEN tree.SX_ShipmentFlag = '1' 							THEN		'RGB(160,215,119)'	 --green
							WHEN tree.SX_ShipmentFlag IN ('2','3','4')					THEN		'RGB(243,240,130)'	 --yellow
							WHEN tree.SX_ShipmentFlag IN ('10','11','12')				THEN		'RGB(130,177,243)'	 --blue
																						ELSE		'RGB(242,210,168)'	 --ocker			
						 END						AS ColorRGB
						,tree.[Owner]
						,tree.Module
						,tree.ObjectType
						,tree.ShipmentStatus
						,tree.ObjectName
						,tree.[Description]
						

					FROM (
							SELECT 
								 CASE
									WHEN sxo.SX_Owner <> ''		THEN sxo.SX_Owner
									WHEN sxo.SX_Owner = ''		THEN 'unknown Owner'
									WHEN sxo.SX_Owner IS NULL	THEN '<owner definition tag not defined>'
									ELSE							 '<procedure failure - this should not had happen>'	
								  END									AS [Owner]
								 ,CASE 
									WHEN sxm.SX_Module IS NULL	THEN '<module defintion tag not definded>'
									WHEN sxm.SX_Module = ''		THEN 'unknown Modul' 
									ELSE sxm.SX_Module
								  END									AS Module
								 ,CASE o.type
									WHEN 'P'	THEN 'Stored Procedures'
									WHEN 'U'	THEN 'Tables'
									WHEN 'V'	THEN 'Views'
									WHEN 'IF'	THEN 'Functions (Table Value Inline)'
									WHEN 'FN'	THEN 'Functions (Scalar)'
									WHEN 'TF'	THEN 'Functions (Table Value)'
									WHEN 'TR'	THEN 'Trigger'
									WHEN 'SO'	THEN 'Sequence'
									ELSE CONCAT('Unknown Objecttype ',o.type)
								 END									AS ObjectType
								,sxs.SX_ShipmentFlag					AS SX_ShipmentFlag
								,CASE 
									WHEN sxs.SX_ShipmentFlag	= '0'	THEN	'0. Demo object - out of shipment process'
									WHEN sxs.SX_ShipmentFlag	= '1'	THEN	'1. Standard - without modification'
									WHEN sxs.SX_ShipmentFlag	= '2'	THEN	'2. Standard - modified from saxess for customer'
									WHEN sxs.SX_ShipmentFlag	= '3'	THEN	'3. Standard - modified from partner for customer'
									WHEN sxs.SX_ShipmentFlag	= '4'	THEN	'4. Standard - modified from customer for himself'

									WHEN sxs.SX_ShipmentFlag	= '10'	THEN	'10. Created from Saxess for Customer'
									WHEN sxs.SX_ShipmentFlag	= '11'	THEN	'11. Created from Partner for Customer'
									WHEN sxs.SX_ShipmentFlag	= '12'	THEN	'12. Created from Customer for Customer'

									WHEN sxs.SX_ShipmentFlag	IS NULL THEN	'<module defintion tag not definded>'

									ELSE										'unknown Shipment Status'
								 END									AS ShipmentStatus
								,S.name	+'.' + O.name					AS ObjectName

								,CASE
									WHEN ep.value = ''		THEN	'Description not defined'
									WHEN ep.value IS NULL	THEN	'<description defintion tag not definded>'
									WHEN ep.value <> ''		THEN	ep.value
									ELSE							'<procedure failure - this should not had happen>'
								 END									AS Description

							FROM sys.all_objects O
							LEFT JOIN sys.extended_properties EP 
								ON ep.major_id = O.object_id 
							LEFT JOIN sys.schemas S 
								ON O.schema_id = S.schema_id
							-- use as separte column
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_Owner
											FROM sys.extended_properties
											WHERE Name = 'SX_Owner'
										) sxo
										ON sxo.major_id = ep.major_id
							-- use as separte column
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_ShipmentFlag
											FROM sys.extended_properties
											WHERE Name = 'SX_ShipmentFlag'
										) sxs
										ON sxs.major_id = ep.major_id
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_Module
											FROM sys.extended_properties
											WHERE Name = 'SX_Module'
										) sxm
										ON sxm.major_id = ep.major_id

							WHERE 
									O.is_ms_shipped = 0
								AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
								AND O.type NOT IN ('PK' -- primary keys
												  ,'D'  -- default constraints
												  ,'UQ'	-- unique constrains
												  ,'F'	-- foreign key
												  )
								AND (ep.name = 'MS_Description' OR ep.name IS NULL)
								--AND ISNULL(ep.name,'') NOT IN ('SX_Owner','SX_ShipmentFlag','SX_Module') -- separat joined
								--AND ISNULL(ep.name,'') NOT IN ('SX_UserHint','Is')						 -- not relevant

							) tree

					ORDER BY Owner, Module, ObjectType, ObjectName

					--								*************************************************************************

			SET @ResultCode = 200;
		COMMIT TRANSACTION spGET_ObjectTree;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spGET_ObjectTree;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: [system].spGET_ObjectTree TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ObjectTree'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
																-- 10 = shiped from saxess as customer specific object
	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an object tree as list of all database objects with its metadata. The owner and shipment status of the objects
					will control update processes etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 -- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username for whom the action is down.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 
GO
 
DROP PROCEDURE IF EXISTS system.spGET_PipelineLog;
GO

/*
	Procedure to GET filterd or all pipeline logs

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_PipelineLog
				 @Username		= 'SQL'
				,@PipelineID	= 'P1'
				,@ExecutionID	= '';
		PRINT    @RC;

		SELECT * FROM system.tPipelineLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineLog','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_PipelineLog
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)
		,@ExecutionID			NVARCHAR(50)
    )

AS 
	BEGIN	

		-- SET NOCOUNT ON; -- activate this, if you read the resultset with an external source

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username		,N'NULL')	+ N''','''										 
				+ ISNULL(@PipelineID	,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				-- No check for existence of IDs - Pipeline may be deleted, but the logs are present

				-- Data Transaction
				PRINT CONCAT(N'Start reading data',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT 
					 PipelineID
					,PipelineName
					,ExecutionID
					,ExecutionServerName
					,StatusCODE
					,StartedBy
					,StartTime
					,EndTime
				FROM system.tPipelineLog
				WHERE 
						(PipelineID		= @PipelineID	OR @PipelineID	= '')
					AND	(ExecutionID	= @ExecutionID	OR @ExecutionID = '');

				SET @EffectedRows = @@ROWCOUNT;
				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spGET_PipelineLog TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_PipelineLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get all or filterd log data for Pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'Optional - PipelineID to get only the logs of this pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ExecutionID';
    SET @value      = N'Optional - ExecutionID to get only the logs of this ExecutionID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
 
DROP PROCEDURE IF EXISTS system.spGET_Pipelines;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_Pipelines
				 @Username		= 'SQL'
				,@PipelineID	= 'P4'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Pipelines',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Pipelines','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Pipelines
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','											 
				+ ISNULL(@PipelineID 				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @PipelineID <> '' AND NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END

				-- Data Transaction
				SELECT 
					 PipelineID
					,PipelineName
					,PipelineDescription
					,PipelineDescriptionIconColor
					,OrderOfDisplay
					,EmailNotificationCODE
				FROM system.tPipelines
				WHERE
						PipelineID	= @PipelineID
					OR	@PipelineID = ''
				ORDER BY OrderOfDisplay;

				SET @EffectedRows = @@ROWCOUNT;
				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_Pipelines TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Pipelines'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all Pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'PipelineID - optional get only one Pipeline, use empty string to get all Pipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP PROCEDURE IF EXISTS system.spGET_PipelineSchedules;
GO

/*
	Procedure to read one or all schedules of a pipeline or of the whole database

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure	DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_PipelineSchedules
				 @Username		 = 'SQL'
				,@PipelineID	 = ''
				,@ScheduleID	 = ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineSchedules',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineSchedules','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_PipelineSchedules
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50) = ''
		,@ScheduleID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username		,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID 	,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleID	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';
		IF @ScheduleID				IS NULL SET @ScheduleID			= N'';


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @PipelineID <> '' AND NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;

				IF @ScheduleID <> '' AND NOT EXISTS (
					SELECT ScheduleID FROM system.tPipelineSchedules WHERE PipelineKey = @PipelineKey AND ScheduleID = @ScheduleID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Schedule don`t exits', 16, 10);
					END


				-- Data Transaction
				PRINT CONCAT(N'Start reading data',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				SELECT
					 P.PipelineID
					,PS.ScheduleID
					,PS.ScheduleName
					,PS.ScheduleDescription
					,PS.ScheduleDetailsJSON
					,PS.Active
				FROM system.tPipelineSchedules PS
					LEFT JOIN system.tPipelines P
						ON PS.PipelineKey = P.PipelineKey
				WHERE 
						(PS.PipelineKey	= @PipelineKey	OR @PipelineID = '')
					AND (PS.ScheduleID	= @ScheduleID	OR @ScheduleID = '')
				ORDER BY
					 P.PipelineID
					,PS.ScheduleID;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: [system].[spGET_PipelineSchedules] TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_PipelineSchedules'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns one or all schedules of a given pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline, which schedules are to read - use '' to read all schedules of this database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ScheduleID';
    SET @value      = N'ID of the schedule to read - use '' to read all schedules of this pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spGET_PipelineStepLog;
GO

/*
	Prozedur ot GET all or filterd StepLog Data

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_PipelineStepLog
				 @Username		= 'SQL'
				,@PipelineID	= ''
				,@StepID		= ''
				,@ExecutionID	= '';
		PRINT    @RC;

		SELECT * FROM system.tPipelineStepLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineStepLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineStepLog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_PipelineStepLog
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50) = ''
		,@StepID				NVARCHAR(50) = ''
		,@ExecutionID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID						,N'NULL')	+ N''','''
				+ ISNULL(@StepID							,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID						,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all mandatory Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @StepID			IS NULL SET @StepID			= N'';
		IF @ExecutionID		IS NULL SET @ExecutionID	= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @PipelineID <> '' AND NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END;

				IF @PipelineID <> '' AND @StepID <> '' AND NOT EXISTS (
					SELECT PipelineID, StepID FROM system.tPipelineSteps	PS
						LEFT JOIN system.tPipelines P
							ON P.PipelineKey = PS.PipelineKey
							WHERE 
									PipelineID = @PipelineID
								AND StepID		= @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Step don`t exits', 16, 10);
					END;

				IF @PipelineID = '' AND @StepID <> ''
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('PipelineID must be set if StepID is used', 16, 10);
					END;

				IF @ExecutionID <> '' AND NOT EXISTS (
					SELECT ExecutionID FROM system.tPipelineStepLog WHERE ExecutionID = @ExecutionID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('ExecutionID don`t exits', 16, 10);
					END;
							   
				-- Data Transaction
				PRINT CONCAT(N'Starting reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT
					 PipelineID
					,StepID
					,ExecutionID
					,StepName
					,StartTime
					,EndTime
					,LogMessage
					,StatusCODE
					,HandleID
				FROM system.tPipelineStepLog
				WHERE
						(PipelineID		= @PipelineID	OR @PipelineID	= '')
					AND	(StepID			= @StepID		OR @StepID		= '')
					AND (ExecutionID	= @ExecutionID	OR @ExecutionID = '')					

				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spGET_PipelineStepLog TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_PipelineStepLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Prozedur ot GET all or filterd StepLog Data';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'optional (but mandantory if StepID is used) - ID of the Pipeline to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'optional - ID of the Step to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ExecutionID';
    SET @value      = N'optional - Exectuion ID of the Pipeline to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO

DROP PROCEDURE IF EXISTS system.spGET_PipelineSteps;
GO
/*
	Procedure to read one or all Pipeline Steps

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_PipelineSteps
				 @Username		= 'SQL'
				,@PipelineID	= 'P1'
				,@StepID		= ''
		PRINT    @RC

		SELECT * FROM system.tPipelineSteps tPS LEFT JOIN system.tPipelines tP ON tPS.PipelineKey = tP.PipelineKey;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineSteps',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineSteps','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_PipelineSteps
    (
         @Username		NVARCHAR(255)
		,@PipelineID	NVARCHAR(50)
		,@StepID		NVARCHAR(50) = ''
    )

AS 
	BEGIN	


		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID				,N'NULL')	+ N''','''
				+ ISNULL(@StepID					,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT			= 0;


		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username	= N'';
		IF @PipelineID		IS NULL SET @PipelineID	= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END;

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;

				IF @StepID <> '' AND NOT EXISTS (
					SELECT StepID FROM system.tPipelineSteps WHERE PipelineKey = @PipelineKey AND StepID = @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Step don`t exits', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Start reading steps',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT 
					 @PipelineID AS PipelineID
					,StepID
					,StepName
					,StepDescription
					,OrderOfExecution
					,StepDataSourceID
					,StepCompanyIDsJSON
					,StepDetailsJSON
					,TimeExecutionLimit
					,Active
					,ModuleID
					,StartNextStepOnError

				FROM system.tPipelineSteps
				WHERE 
						PipelineKey = @PipelineKey
					AND (StepID =  @StepID OR @StepID = '')
				ORDER BY OrderOfExecution;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: [system].[spGET_PipelineSteps] TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_PipelineSteps'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns one or all steps of a given pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the pipeline, whichs steps are requested.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step to get, use '''' to get all Steps.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO
 
DROP PROCEDURE IF EXISTS system.spGET_ProcessLog;
GO

/*
	Procedure to GET the last @RowCount Rows of the process log table

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ProcessLog
				 @Username = 'SQL'
				,@RowCount = 1000
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ProcessLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ProcessLog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ProcessLog
    (
         @Username				NVARCHAR(255)
        ,@RowCount				INT
    )

AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@RowCount AS NVARCHAR(255))	,N'NULL')	+ N''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- Procedure specific declarations
		--DECLARE  @FactoryKey		INT			= 0
		--		,@ProductLineKey	INT			= 0
		--		,@ProductKey		INT			= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @RowCount				IS NULL SET @RowCount			= 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

					EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

					IF @ResultCode <> 200 AND NOT @TransactUsername = 'FactoryService' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Invalid rights', 16, 10);
						END

				-- Data Transaction
				SELECT TOP (@RowCount)
				*
				FROM system.tProcessLogs
				ORDER BY RowKey DESC

				SET @ResultCode = 200;

		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: [system].[spGET_ProcessLog] TO [pf_PlanningFactoryService]



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ProcessLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to return the last @RowCount rows from the process log table';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@RowCount';
    SET @value      = N'Number of requested rows.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 


	GO
 
DROP PROCEDURE IF EXISTS system.spGET_Reports;
GO

/*
	Procedure to GET Report definitions, can be called
		- without ReportID -> to get all Reports, but without Layoutdefinitions (which may be big due to images etc.)
		- with Report ID -> to get one Report including the Layoutdefinition

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_Reports
				 @Username		= 'SQL'
				,@ReportID		= 'R4';

		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Reports',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Reports','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_Reports
    (
         @Username				NVARCHAR(255)
        ,@ReportID				NVARCHAR(50) = ''
    )

AS 
	BEGIN	


		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username	,N'NULL')	+ N''','''										 
				+ ISNULL(@ReportID	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @ReportID 			IS NULL SET @ReportID			= N'';


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'Transaction user don`t exists', 16, 10);
					END;

				-- Check existence of ID
				IF @ReportID <> N'' AND NOT EXISTS (
					SELECT ReportID FROM system.tReports WHERE ReportID = @ReportID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR(N'Report don`t exits', 16, 10);
					END

				-- Data Transaction
				PRINT CONCAT(N'Start reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				IF @ReportID = ''
					BEGIN
						SELECT
							 ReportID
							,ReportName
							,ReportDescription
							,NULL					AS LayoutDefinition -- not an empty String, as the column is varbinary
						FROM system.tReports;
						SET @EffectedRows = @@ROWCOUNT;
					END;

				IF @ReportID <> ''
					BEGIN
						SELECT
							 ReportID
							,ReportName
							,ReportDescription
							,LayoutDefinition
						FROM system.tReports
						WHERE ReportID = @ReportID;
						SET @EffectedRows = @@ROWCOUNT;
					END;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spGET_Reports TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Reports'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns all reports (without Layout), or one Report (with Layout)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ReportID';
    SET @value      = N'optional - ID of the report. Can be skiped to get all reports (without Layoutdefinition).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO
DROP PROCEDURE IF EXISTS system.spGET_User;
GO

/*
	GET Operation for a User

	Saxess Software GmbH
	Last modified: 09/2023 for OCT 5.9

	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = system.spGET_User 
			 @Username			= 'SQL' 
			,@RequestedUserName	= 'W10\admin'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_User','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_User
	@Username			NVARCHAR(255),
	@RequestedUserName	NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,			 N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY
		BEGIN TRANSACTION spGET_User;

		-- Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		IF @RequestedUserName = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys
		SELECT @UserKey = UserKey FROM system.trUser WHERE UserName = @RequestedUserName;
		
		-- Fallback to public user if it exists
		IF @UserKey = 0
		BEGIN
			SELECT @UserKey = UserKey FROM system.trUser WHERE Username = 'public' AND Status = 'Active'
		END

		-- User doesn't exist
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --users must see themselves
			SET @Right = N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;
		
		-- STEP 3. Show public values	
		SELECT 
			 UserKey
			,UserName
			,PersonSurname
			,PersonFirstName
			,Email
			,LDAPIP
			,[Status]
			,@Right AS 'Right'
			,DataFlowRightsCODE
			,VisualizationRightsCODE
			,IsAdministratorFlag
		FROM system.trUser
		WHERE UserKey = @UserKey; 

		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION spGET_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spGET_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
GRANT EXECUTE ON OBJECT ::system.spGET_User TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_User'		    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for a User';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@RequestedUserName';
    SET @value      = N'RequestedUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spGET_UserRight;
GO

/*
	GET Operation for User rights in DataEntry

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = system.spGET_UserRight 
			@Username = 'SQL', 
			@RequestedUserName = 'W10\admin';
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRight','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_UserRight
	@Username			NVARCHAR(255),
	@RequestedUserName  NVARCHAR(255) = NULL 
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)  = N'';				

	-- NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY	
		BEGIN TRANSACTION GET_UserRight;

		-- Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		-- Check on empty name or SystemUser
		IF @RequestedUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameter <RequestedUserName>', 16, 10);
		END;	

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys
		SELECT @UserKey = UserKey FROM system.trUser WHERE UserName = @RequestedUserName;
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right		= N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- Helper Table with Full Stucture - to Output Deny Rights too   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
			DROP TABLE #FullStructure;

		CREATE TABLE #FullStructure
		(
			RowKey			BIGINT IDENTITY (1,1)								NOT NULL
			,[Level]		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey		BIGINT												NOT NULL
			,ProductLineKey BIGINT												NOT NULL
			,FactoryID		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
		);
		-- C Level
		INSERT INTO #FullStructure
		SELECT 
			'Cluster'	AS 'Level'
			,0			AS FactoryKey
			,0			AS ProductLineKey
			,''			AS FactoryID
			,''			AS ProductLineID;
			 
		-- F Level
		INSERT INTO #FullStructure
		SELECT
			'Factory'		AS 'Level'
			,dF.FactoryKey
			,0				AS ProductLineKey
			,dF.FactoryID
			,''				AS ProductLineID
		FROM planning.tdFactories dF;
				
		-- PL Level
		INSERT INTO #FullStructure
		SELECT
			'ProductLine'	AS 'Level'
			,dF.FactoryKey
			,dPL.ProductLineKey
			,dF.FactoryID
			,dPL.ProductLineID
		FROM planning.tdProductLines dPL 
			LEFT JOIN planning.tdFactories dF
				ON dPL.FactoryKey = dF.FactoryKey;
		
		-- STEP 3. Show public values	
		IF @Right = N'Read' --User is allowed to see of himself only positive rights
		BEGIN

			SELECT 
				 vUR.Username				AS UserName
				,vUR.FactoryID				AS FactoryID
				,COALESCE(dF.NameShort,'')	AS FactoryName
				,vUR.ProductLineID			AS ProductLineID
				,COALESCE(dPL.NameShort,'')	AS ProductLineName
				,vUR.[Right]				AS [Right]
				,vUR.ReadCommentMandatory	AS ReadCommentMandatory
				,vUR.WriteCommentMandatory	AS WriteCommentMandatory
			FROM system.trUserRights vUR
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON vUR.FactoryID = dF.FactoryID
				-- for ProductLineName
				LEFT JOIN planning.tdProductLines dPL
					ON		vUR.FactoryID = dPL.FactoryID 
						AND	vUR.ProductLineID = dPL.ProductLineID
			WHERE vUR.UserKey = @UserKey 
			ORDER BY 
				ISNULL(TRY_CAST(vUR.FactoryID AS INT), 999999999)
				,vUR.FactoryID
				,ISNULL(TRY_CAST(vUR.ProductLineID AS INT), 999999999)
				,vUR.ProductLineID;
		END
		ELSE
		IF @Right = N'Write' -- Cluster admin can see also negative rights (really don`t exist)
		BEGIN
				
			SELECT
				 @RequestedUsername						AS UserName
				,fs.FactoryID							AS FactoryID
				,COALESCE(dF.NameShort,'')				AS FactoryName
				,fs.ProductLineID						AS ProductLineID
				,COALESCE(dPL.NameShort,'')				AS ProductLineName
				,COALESCE(vUR.[Right],'Deny')			AS 'Right'
                ,COALESCE(vUR.ReadCommentMandatory,0)	AS ReadCommentMandatory
				,COALESCE(vUR.WriteCommentMandatory,0)	AS WriteCommentMandatory
			FROM #FullStructure fs
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON fs.FactoryID = dF.FactoryID
				-- for ProductLineName
				LEFT JOIN planning.tdProductLines dPL
					ON		fs.FactoryID = dPL.FactoryID 
						AND	fs.ProductLineID = dPL.ProductLineID
				-- positive Rechte
				LEFT JOIN
					(
						SELECT 
							vUR.FactoryID
							,vUR.ProductLineID
							,vUR.[Right]
							,vUR.ReadCommentMandatory
							,vUR.WriteCommentMandatory
						FROM system.trUserRights vUR
						WHERE vUR.UserKey = @UserKey
					) vUR
					ON		fs.FactoryID = vUR.FactoryID
						AND fs.ProductLineID = vUR.ProductLineID
			ORDER BY  
				ISNULL(TRY_CAST(fs.FactoryID AS INT), 999999999)
				,fs.FactoryID
				,ISNULL(TRY_CAST(fs.ProductLineID AS INT), 999999999)
				,fs.ProductLineID;
		END;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_UserRight;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_UserRight;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
-- GRANT
GRANT EXECUTE ON OBJECT ::system.spGET_UserRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_UserRight'		-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                        
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for User rights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@RequestedUserName';
    SET @value      = N'RequestedUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
 
DROP PROCEDURE IF EXISTS system.spGET_UserRightInformation;
GO

/*
	Procedure to get right informations for an User
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_UserRightInformation
				 @Username					= 'SQL'
				,@UsernameRequestedUser		= 'W10\admin'
				,@RightAreaCODE				= ''	
				,@FactoryID					= ''
				,@ProductLineID				= ''
				,@PipelineID				= ''
				,@FullStructureBOOL			= 0

		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRightInformation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRightInformation','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_UserRightInformation
    (
         @Username				NVARCHAR(255)
        ,@UsernameRequestedUser	NVARCHAR(255)
		,@RightAreaCODE			NVARCHAR(50) = ''	--DATAENTRY, DATAFLOWS
		,@FactoryID				NVARCHAR(50) = ''
		,@ProductLineID			NVARCHAR(50) = ''
		,@PipelineID			NVARCHAR(50) = ''
		,@FullStructureBOOL		BIT = 0
    )

AS 
	BEGIN	

		-- SET NOCOUNT ON; -- activate this, if you read the resultset with an external source

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@UsernameRequestedUser					,N'NULL')	+ N''','''	
				+ ISNULL(@RightAreaCODE							,N'NULL')	+ N''','''
				+ ISNULL(@FactoryID								,N'NULL')	+ N''','''	
				+ ISNULL(@ProductLineID							,N'NULL')	+ N''','''	
				+ ISNULL(@PipelineID							,N'NULL')	+ N''','
				+ ISNULL(CAST(@FullStructureBOOL AS NCHAR(1))	,N'NULL')	+ N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		--DECLARE  @FactoryKey		INT			= 0
		--		,@ProductLineKey	INT			= 0
		--		,@ProductKey		INT			= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @UsernameRequestedUser	IS NULL SET @UsernameRequestedUser	= N'';
		IF @RightAreaCODE			IS NULL SET @RightAreaCODE			= N'';
		IF @FactoryID				IS NULL SET @FactoryID				= N'';
		IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
		IF @PipelineID				IS NULL SET @PipelineID				= N'';
		IF @FullStructureBOOL		IS NULL SET @FullStructureBOOL		= 0;


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- if RequestedUser doesn't exist we can use fallback to public user
				IF NOT EXISTS(SELECT 1 FROM system.trUser WHERE Username = @UsernameRequestedUser AND Status = 'Active') AND EXISTS(SELECT 1 FROM system.trUser WHERE Username = 'public' AND Status = 'Active')
				BEGIN
					SET @UsernameRequestedUser = 'public'
				END

				DROP TABLE IF EXISTS #tRightInformation;
				CREATE TABLE #tRightInformation
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,RightAreaCode			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,FactoryID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ProductLineID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,UserRight				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReadCommentMandatory	BIT											NOT NULL
						,WriteCommentMandatory	BIT											NOT NULL
						,RightDescription		NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tRightInformation PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Rights for DataEntry
				IF @RightAreaCode = 'DATAENTRY' OR @RightAreaCode = ''
					BEGIN
						PRINT CONCAT(N'Determining DATAENTRY Rights',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						-- For full structure requested
						IF @FullStructureBOOL = 1
							BEGIN
								INSERT INTO #tRightInformation
									SELECT 
										 'DATAENTRY'	AS RightAreaCode
										,FactoryID
										,ProductLineID
										,''				AS PipelineID
										,[Right]		AS UserRight
										,ReadCommentMandatory
										,WriteCommentMandatory
										,''				AS RightDescription

									FROM system.trUserRights
									WHERE 
										Username = @UsernameRequestedUser
										AND
										(FactoryID = @FactoryID OR @FactoryID=''  )
										AND
										(ProductLineID  = @ProductLineID OR @ProductLineID = '');
								END

						-- For single line			
						IF @FullStructureBOOL = 0
							BEGIN
								INSERT INTO #tRightInformation
									SELECT 
										 'DATAENTRY'	AS RightAreaCode
										,FactoryID
										,ProductLineID
										,''				AS PipelineID
										,[Right]		AS UserRight
										,ReadCommentMandatory
										,WriteCommentMandatory
										,''				AS RightDescription

									FROM system.trUserRights
									WHERE 
										Username = @UsernameRequestedUser
										AND
										(FactoryID = @FactoryID)
										AND
										(ProductLineID  = @ProductLineID);
								END
					END

				-- Rights for DataFlows
				IF @RightAreaCode = 'DATAFLOWS' OR @RightAreaCode = ''
					BEGIN
						PRINT CONCAT(N'Determining DATAFLOWS Rights',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						-- One Query for full structure and single object
	
						INSERT INTO #tRightInformation
								SELECT 
										'DATAFLOWS'	AS RightAreaCode
									,''				AS FactoryID
									,''				AS ProductLineID
									,PipelineID		AS PipelineID
									,'EXECUTE'		AS UserRight
									,0				AS ReadCommentMandatory
									,0				AS WriteCommentMandatory
									,'temporary'	AS RightDescription

								FROM system.tPipelines
								WHERE 
									-- Username = @UsernameRequestedUser
									-- AND
									(PipelineID = @PipelineID OR @PipelineID='')		
					END



				
				-- OUTPUT
				
				SELECT * FROM #tRightInformation;
				
				SET @ResultCode = 200;
				PRINT CONCAT(N'Determination finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spGET_UserRightInformation TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_UserRightInformation'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get right informations for an User - returns the Right as String - Read, Write, EXECUTE,DESIGN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username';
    SET @value      = N'Standardparameter, executing User - use "SQL"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@UsernameRequestedUser';
    SET @value      = N'Name of the user for which rights are requested';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@RightAreaCODE';
    SET @value      = N'CODE to specify the Rightarea, use DATAFLOWS or DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID in case of requsting Rights for DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID in case of requsting Rights for DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name

	SET @level2name = N'@PipelineID';
    SET @value      = N'PipelineID in case of requsting Rights for DATAFLOWS, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name


	SET @level2name = N'@FullStructureBOOL';
    SET @value      = N'1 or 0 to flag if the full structure under the node (downwards) is included into the output';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name


	GO

DROP PROCEDURE IF EXISTS system.spGET_Users;
GO

/*
	GET Operation for all Users

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = system.spGET_Users 'sxs'
		PRINT @RC

		SELECT TOP 3 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Users',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_Users','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_Users
	@Username NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE		 @TransactUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''''
				,@EffectedRows		INT				= 0
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@Message			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION spGET_Users;

		-- Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Only OCT Administrators can manage users
		IF NOT EXISTS
			(SELECT 
				Userkey
			FROM system.trUser
			WHERE 
					Username			= @TransactUsername 
				AND IsAdministratorFlag	= 1
				AND Status				= 'Active'
			)
		BEGIN
			SET @Message = CONCAT('Invalid rights, User "',@Transactusername,'" is not an OCT Administrator.');
			RAISERROR(@Message, 16, 10);
		END;
		
		-- Output
		SELECT 
			 u.UserKey
			,u.UserName
			,u.PersonSurname
			,u.PersonFirstName
			,u.Email
			,u.LDAPIP
			,u.[Status]
			,N'Write' AS [Right]
			,u.DataFlowRightsCODE
			,u.VisualizationRightsCODE
			,u.IsAdministratorFlag
		FROM system.trUser u
		ORDER BY UserName;
		
		SET @EffectedRows = @@ROWCOUNT;

		SET @ResultCode = 200;

		COMMIT TRANSACTION spGET_Users;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spGET_Users;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
 -- GRANT
GRANT EXECUTE ON OBJECT ::system.spGET_Users TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Users'			-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation to read all Users, only allowed for Clusteradmin.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO

DROP PROCEDURE IF EXISTS system.spMOVE_Pipeline;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spMOVE_Pipeline
				 @Username			= 'SQL'
				,@SourcePipelineID	= 'P1'
				,@TargetPipelineID	= 'P11';
		PRINT    @RC
	
		SELECT * FROM system.tPipelines;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMOVE_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMOVE_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spMOVE_Pipeline
    (
         @Username				NVARCHAR(255)
        ,@SourcePipelineID		NVARCHAR(50) = ''
		,@TargetPipelineID		NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','											 
				+ ISNULL(@SourcePipelineID		,N'NULL')	+ N','''
				+ ISNULL(@TargetPipelineID		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		--DECLARE  @FactoryKey		INT			= 0
		--		,@ProductLineKey	INT			= 0
		--		,@ProductKey		INT			= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @SourcePipelineID		IS NULL SET @SourcePipelineID	= N'';
		IF @TargetPipelineID		IS NULL SET @TargetPipelineID	= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spMOVE_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				-- check existence of IDs
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @SourcePipelineID
					)
					BEGIN
						sET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END

				IF EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @TargetPipelineID
					)
					BEGIN
						sET @ResultCode = 403;
						RAISERROR('Target Pipeline already exits', 16, 10);
					END

				PRINT CONCAT(N'Starting MOVE operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				-- MOVE over UPDATE
					-- Pipeline
					UPDATE system.tPipelines 
					SET PipelineID = @TargetPipelineID 
					WHERE PipelineID = @SourcePipelineID;

					-- Steps and Schedules must not be moved, they only know the PipelineKey which is not changed
		
				SET @ResultCode = 200;
				PRINT CONCAT(N'Finished MOVE operation',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spMOVE_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spMOVE_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spMOVE_Pipeline TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spMOVE_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to move a Pipeline to a different ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourcePipelineID';
    SET @value      = N'ID of the Pipeline which will be moved.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetPipelineID';
    SET @value      = N'New ID for the pipeline, this ID must not be in use at the moment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
  
DROP PROCEDURE IF EXISTS system.pPOST_ProcessLogEntry;
GO

/*
	Stored Procedure to save a process log entry. For every call a new entry is created.
	Return Value according to HTTP Standard
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.8
 
	Testcall
		DECLARE	@return_value INT;

		EXEC
			@return_value = system.pPOST_ProcessLogEntry
				 @Username			= N'SQL'
				,@ModuleCode		= N'Integration'
				,@ProcessName		= N'Test'
				,@ProcessID			= 1
				,@SubProcessName	= N'SubProcessTest'
				,@SubProcessID		= 2
				,@SubSubprocessName = N'SubSubProcessTest'
				,@EventCODE			= NULL
				 -- https://stackoverflow.com/questions/12957635/sql-query-to-insert-datetime-in-sql-server
				,@Timestamp			= N'2021-01-27T15:32:21.000' --Timestamp to insert must be writen as N'20210127 15:32:21.000' or N'2021-01-27T15:32:21.000' -> N'2021-01-27 15:32:21.000' work not
				,@ReturnCODE		= 500
				,@EventMessage		= N'This is an error.';

		SELECT	'Return Value' = @return_value;

		SELECT TOP 5 * FROM system.tProcessLogs;
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_ProcessLogEntry',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_ProcessLogEntry','PARAMETER',NULL)
*/

CREATE PROCEDURE system.pPOST_ProcessLogEntry
	 @Username				NVARCHAR(255)
	,@ModuleCode			NVARCHAR(50)
	,@ProcessName			NVARCHAR(250)
	,@ProcessID				NVARCHAR(50)
	,@SubProcessName		NVARCHAR(250)
	,@SubProcessID			NVARCHAR(50)
	,@SubSubProcessName		NVARCHAR(250)
	,@EventCODE				NVARCHAR(20)	
	,@Timestamp				DATETIME		
	,@ReturnCODE			INT
	,@EventMessage			NVARCHAR(max)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')					+ N''','''
			+ ISNULL(@ModuleCode		, N'NULL')					+ N''','''
			+ ISNULL(@ProcessName		, N'NULL')					+ N''','''
			+ ISNULL(@ProcessID			, N'NULL')					+ N''','''
			+ ISNULL(@SubProcessName	, N'NULL')					+ N''','''
			+ ISNULL(@SubProcessID		, N'NULL')					+ N''','''
			+ ISNULL(@SubSubProcessName	, N'NULL')					+ N''','''
			+ ISNULL(@EventCODE			, N'NULL')					+ N''',''' 
			+ ISNULL(CAST(@Timestamp AS NVARCHAR(255))	, N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@ReturnCODE AS NVARCHAR(255))	, N'NULL')	+ N''','''
			+ ISNULL(@EventMessage		, N'NULL')					+ N'''';

	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';		

	IF @EventCODE IS NULL SET @EventCODE = 'see last state';

	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @ModuleCode			IS NULL 
		OR @ProcessName			IS NULL
		OR @Timestamp			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create log entry	

		INSERT INTO system.tProcessLogs(
			ModuleCODE
           ,ProcessName
           ,ProcessID
           ,SubProcessName
           ,SubProcessID
		   ,SubSubProcessName
           ,EventCODE
           ,Timestamp
           ,ReturnCODE
           ,EventMessage) 
		VALUES (
			@ModuleCODE
           ,@ProcessName
           ,@ProcessID
           ,@SubProcessName
           ,@SubProcessID
		   ,@SubSubProcessName
           ,@EventCODE
           ,@Timestamp
           ,@ReturnCODE
           ,@EventMessage);	

		SET @EffectedRows = 1;
		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pPOST_ProcessLogEntry TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pPOST_ProcessLogEntry'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Description: Stored Procedure to save a process log entry. For every call a new entry is created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleCode';
    SET @value      = N'ModuleCode';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessName';
    SET @value      = N'Name of the Process your are in.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessID';
    SET @value      = N'Process ID as short ID for the process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessName';
    SET @value      = N'optional - Subprocess name if Process was called from a main process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcess ID as short ID for the subprocess';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubSubProcessName';
    SET @value      = N'optional - SubSubprocess name if Process was called from a subprocess';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EventCODE';
    SET @value      = N'Codeword Start, End, Info. The entry "see last state" indicates the the status of process execution is unchanged since the last log entry.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Timestamp';
    SET @value      = N'Timestamp of the Event';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReturnCODE';
    SET @value      = N'Returncode accoring to HTTP Standard (200,201,404...)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EventMessage';
    SET @value      = N'Free Text as event message, maybe catch from SQL Server error event message over SET @Comment = ERROR_MESSAGE() etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
  
DROP PROCEDURE IF EXISTS system.pPOST_Setting;
GO
/*
	ToDo:
	- Right Check
	- Documentation
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		DECLARE @ReturnCode INT;
		EXEC @ReturnCode = system.pPOST_Setting 
			 @Username = 'SQL'
			,@SettingID = '1'
			,@DataTypeCode = 'TEXT';
		
		PRINT @ReturnCode;

		EXEC system.pGET_Setting 'SQL','1'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_Setting',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_Setting','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.pPOST_Setting
	 @Username						NVARCHAR(255)
	,@SettingID						NVARCHAR(50)
	,@DataTypeCODE					NVARCHAR(255)
	,@SettingDescription			NVARCHAR(4000)	= NULL
	,@ValueText						NVARCHAR(MAX)	= NULL
	,@ValueDateTime					DATETIME		= NULL
	,@ValueInt						INT				= NULL
	,@ValueIntScale					INT				= NULL
    ,@ValueJSON						NVARCHAR(MAX)	= NULL
	,@Category						NVARCHAR(255)	= NULL
	,@ReadOnlySystemPropertyBool	INT				= NULL
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				 N'NULL') + N''',''' 
			+ ISNULL(@SettingID,			 N'NULL') + N''',''' 
			+ ISNULL(@DataTypeCODE,			 N'NULL') + N''',''' 
			+ ISNULL(@SettingDescription,	 N'NULL') + N''',''' 
			+ ISNULL(@ValueText,			 N'NULL') + N''',' 
			+ ISNULL(CAST(@ValueDateTime AS NVARCHAR(MAX)),	 N'NULL')	+ N',' 
			+ ISNULL(CAST(@ValueInt		 AS NVARCHAR(MAX)),	 N'NULL')	+ N',' 
			+ ISNULL(CAST(@ValueIntScale AS NVARCHAR(MAX)),	 N'NULL')	+ N',''' 
			+ ISNULL(@ValueJSON,			 N'NULL') + N''',''' 
			+ ISNULL(@Category,				 N'NULL') + N''',' 
			+ ISNULL(CAST(@ReadOnlySystemPropertyBool AS NVARCHAR(MAX)), N'NULL');
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	DECLARE @IsFactoryService INT = 0;

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @DataTypeCODE	IS NULL SET @DataTypeCODE	= N'';
	IF @SettingID		IS NULL SET @SettingID		= N'';



	BEGIN TRY
		BEGIN TRANSACTION pPOST_Setting;	

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @SettingID = dbo.sx_pf_pProtectString (@SettingID);
	
		-- Check for mandatory parameter
		IF	@SettingID		= N''
			OR @DataTypeCODE NOT IN ('TEXT','NUMBER','BOOLEAN','JSON','DATETIME')
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Invalid input parameters', 10;
		END

		-- STEP 1.1 - Determine transaction user
		IF EXISTS
		(
			SELECT r.name from sys.database_role_members rm 
			INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
			INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
		)
		BEGIN
			SET @IsFactoryService = 1;
		END;

		IF @IsFactoryService = 1
			BEGIN
				SET @TransactUsername = 'FactoryService';
			END
		ELSE
			BEGIN
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			END

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 10;
		END;
		
		IF EXISTS(SELECT TOP(1) 1 FROM [system].tSettings WHERE SettingID = @SettingID)
		-- Existing Property will be update, not passed Values are keept
		BEGIN
			UPDATE [system].tSettings
			SET SettingDescription			= ISNULL(@SettingDescription, SettingDescription)
				,DataTypeCODE				= @DataTypeCODE
				,ValueText					= ISNULL(@ValueText,		ValueText)
				,ValueDateTime				= ISNULL(@ValueDateTime,	ValueDateTime)
				,ValueInt					= ISNULL(@ValueInt,			ValueInt)
				,ValueIntScale				= ISNULL(@ValueIntScale,	ValueIntScale)
				,ValueJSON					= ISNULL(@ValueJSON,		ValueJSON)
				,Category					= ISNULL(@Category,			Category)
				,ReadOnlySystemPropertyBool	= ISNULL(@ReadOnlySystemPropertyBool, ReadOnlySystemPropertyBool)
			 WHERE SettingID = @SettingID;

			 SET @EffectedRows = @@ROWCOUNT;
			 SET @Resultcode = 200;
		END
		ELSE
		-- Not existing Property will be created, not passed Values with defaults
		BEGIN
			INSERT INTO [system].[tSettings]
			(
				SettingID
				,SettingDescription
				,DataTypeCODE
				,ValueText
				,ValueDateTime
				,ValueInt
				,ValueIntScale
				,ValueJSON
				,Category
				,ReadOnlySystemPropertyBool
			)
			VALUES 
			(
				@SettingID
				, ISNULL(@SettingDescription,	N'')
				, @DataTypeCODE
				, ISNULL(@ValueText,			N'')
				, ISNULL(@ValueDateTime,		'19000101')
				, ISNULL(@ValueInt,				0)
				, ISNULL(@ValueIntScale,		1)
				, ISNULL(@ValueJSON,			N'')
				, ISNULL(@Category,				N'')
				, ISNULL(@ReadOnlySystemPropertyBool, 1)
			);

			SET @EffectedRows = @@ROWCOUNT;
			SET @Resultcode = 201;
		END
		
		COMMIT TRANSACTION pPOST_Setting;	
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION pPOST_Setting;

		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END;
GO

GRANT EXECUTE ON [system].[pPOST_Setting] TO pf_PlanningFactoryService;
GRANT EXECUTE ON [system].[pPOST_Setting] TO pf_PlanningFactoryUser;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pPOST_Setting'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'pPOST_Setting';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SettingID';
    SET @value      = N'SettingID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DataTypeCODE';
    SET @value      = N'DataTypeCODE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SettingDescription';
    SET @value      = N'SettingDescription';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueDateTime';
    SET @value      = N'ValueDateTime';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueIntScale';
    SET @value      = N'ValueIntScale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueJSON';
    SET @value      = N'ValueJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Category';
    SET @value      = N'Category';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReadOnlySystemPropertyBool';
    SET @value      = N'ReadOnlySystemPropertyBool';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS system.spPOST_Connector;
GO

/*
	Template to POST one Connector

	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
		EXEC	 @RC = system.spPOST_Connector
				 @Username						= 'SQL'
				,@SourceSystemID				= 'NAVISION'
				,@SourceSystemType				= 'MSSQL'
				,@ModuleID						= 'FIN'
				,@ConnectorDescription			= 'FIN Connector for Microsoft Navision based on a SQL Server database.'
				,@ConnectorDescriptionURL		= 'https://www.saxess-software.de/standardschnittstellen-fuer-das-rechnungswesen/ms-dynamics-nav/'
				,@ConnectorVersion				= 1
		PRINT    @RC;

		SELECT * FROM system.tConnectors;
		DELETE FROM system.tConnectors;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Connector
    (
         @Username						NVARCHAR(255)
        ,@SourceSystemID				NVARCHAR(50)
		,@SourceSystemType				NVARCHAR(50)
		,@ModuleID						NVARCHAR(50)
		,@ConnectorDescription			NVARCHAR(4000)	= NULL
		,@ConnectorDescriptionURL		NVARCHAR(255)	= NULL
		,@ConnectorVersion				INT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@SourceSystemID							,N'NULL')	+ N''','''
				+ ISNULL(@SourceSystemType							,N'NULL')	+ N''','''
				+ ISNULL(@ModuleID					,N'NULL')	+ N''','''
				+ ISNULL(@ConnectorDescription			,N'NULL')	+ N''','
				+ ISNULL(@ConnectorDescriptionURL			,N'NULL')	+ N''','
				+ ISNULL(CAST (@ConnectorVersion AS NVARCHAR(10)),N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0;

		-- NULL Protection for all mandatory  Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @SourceSystemID			IS NULL SET @SourceSystemID			= N'';
		IF @SourceSystemType		IS NULL SET @SourceSystemType			= N'';
		IF @ModuleID				IS NULL SET @ModuleID			= N'';
		IF @ConnectorVersion		IS NULL SET @ConnectorVersion			= 1;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @SourceSystemID		= dbo.sx_pf_pProtectID	(@SourceSystemID);
		SET @SourceSystemType		= dbo.sx_pf_pProtectID	(@SourceSystemType);
		SET @ModuleID		= dbo.sx_pf_pProtectID	(@ModuleID);
		IF @SourceSystemType NOT IN ('MSSQL', 'ORACLE', 'ODBC') SET @SourceSystemType = 'MSSQL';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Connector
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Transaction started',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #CurrentConnector;
				CREATE TABLE #CurrentConnector
					(
						 ConnectorKey					BIGINT										NOT NULL
						,SourceSystemID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,SourceSystemType				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ModuleID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ConnectorDescription			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ConnectorDescriptionURL		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ConnectorVersion				INT											NOT NULL						
					)

				-- Cache current values
				INSERT INTO #CurrentConnector
					SELECT 
						 ConnectorKey
						,SourceSystemID
						,SourceSystemType
						,ModuleID
						,ConnectorDescription
						,ConnectorDescriptionURL
						,ConnectorVersion
					FROM system.tConnectors
					WHERE SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND ModuleID = @ModuleID;

				-- insert or update Pipeline
				SELECT @RowCount = Count (*) FROM #CurrentConnector; 

				IF @RowCount = 0
					-- case new Connector
					BEGIN
						-- INSERT with default values if NULL
						INSERT INTO system.tConnectors
						VALUES
							(
								 @SourceSystemID
								 ,@SourceSystemType
								 ,@ModuleID
								,ISNULL(@ConnectorDescription, '')
								,ISNULL(@ConnectorDescriptionURL, '')
								,ISNULL(@ConnectorVersion, 1)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					-- Case existing Connector
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						UPDATE tC
								SET 
									ConnectorDescription = ISNULL(@ConnectorDescription, cC.ConnectorDescription)
							FROM system.tConnectors AS tC
								LEFT JOIN #CurrentConnector AS cC 
									ON tC.SourceSystemID = cC.SourceSystemID
									AND tC.SourceSystemType = cC.SourceSystemType
									AND tC.ModuleID = cC.ModuleID
							WHERE tC.SourceSystemID = cC.SourceSystemID
							AND tC.SourceSystemType = cC.SourceSystemType
							AND tC.ModuleID = cC.ModuleID;

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Connector
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Connector	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spPOST_Connector TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)                                                      
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ConnectorDescription';
    SET @value      = N'Description of the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ConnectorVersion';
    SET @value      = N'Version number of the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
GO
DROP PROCEDURE IF EXISTS system.spPOST_ConnectorQuery;
GO

/*
	POST one query of a connector

	Toni August for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
		EXEC	 @RC = system.spPOST_ConnectorQuery
				 @Username						= 'SQL'
				,@SourceSystemID				= 'NAVISION'
				,@SourceSystemType				= 'MSSQL'
				,@ModuleID						= 'FIN'
				,@QueryName						= 'AccountingJournal'
				,@QuerySQL						= 'SELECT 123 AS bla'
				,@QueryVersionComment			= 'Something'
		PRINT    @RC;

		SELECT * FROM system.tConnectorQueries;
		DELETE FROM system.tConnectorQueries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_ConnectorQuery',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_ConnectorQuery','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_ConnectorQuery
    (
         @Username						NVARCHAR(255)
        ,@SourceSystemID				NVARCHAR(50)
		,@SourceSystemType				NVARCHAR(50)
		,@ModuleID						NVARCHAR(50)
		,@QueryName						NVARCHAR(50)
		,@QuerySQL						NVARCHAR(MAX)	= NULL
		,@QueryVersionComment			NVARCHAR(4000)  = NULL
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@SourceSystemID							,N'NULL')	+ N''','''
				+ ISNULL(@SourceSystemType							,N'NULL')	+ N''','''
				+ ISNULL(@ModuleID					,N'NULL')	+ N''','''
				+ ISNULL(@QueryName					,N'NULL')	+ N''','''
				+ ISNULL(@QuerySQL			,N'NULL')	+ N''','
				+ ISNULL(@QueryVersionComment			,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@ConnectorKey		INT				= 0
				,@QueryVersion		INT				= 1;

		-- NULL Protection for all mandatory  Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @SourceSystemID			IS NULL SET @SourceSystemID			= N'';
		IF @SourceSystemType		IS NULL SET @SourceSystemType		= N'';
		IF @ModuleID				IS NULL SET @ModuleID				= N'';
		IF @QuerySQL				IS NULL SET @QuerySQL				= N'';
		IF @QueryVersionComment		IS NULL SET @QueryVersionComment	= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @SourceSystemID		= dbo.sx_pf_pProtectID	(@SourceSystemID);
		SET @SourceSystemType		= dbo.sx_pf_pProtectID	(@SourceSystemType);
		SET @ModuleID		= dbo.sx_pf_pProtectID	(@ModuleID);
		SET @QueryName		= dbo.sx_pf_pProtectID	(@QueryName);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_ConnectorQuery
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Transaction started',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				-- Check if connector exists
				IF NOT EXISTS(
					SELECT 1
					FROM system.tConnectors
					WHERE SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND ModuleID = @ModuleID
				)
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Connector doesn''t exist', 16, 10);
					END;

				-- Get ConnectorKey
				SELECT @ConnectorKey = ConnectorKey
				FROM system.tConnectors
				WHERE SourceSystemID = @SourceSystemID
				AND SourceSystemType = @SourceSystemType
				AND ModuleID = @ModuleID;

				-- Get next version number
				SELECT @QueryVersion = COALESCE(MAX(QueryVersion) + 1, 1)
				FROM system.tConnectorQueries
				WHERE ConnectorKey = @ConnectorKey
				AND QueryName = @QueryName;

				INSERT INTO tConnectorQueries (ConnectorKey, QueryName, QuerySQL, QueryVersion, QueryVersionComment, Timestamp, CreatedBy)
				VALUES (@ConnectorKey, @QueryName, @QuerySQL, @QueryVersion, @QueryVersionComment, SysUTCDateTime(), @TransactUsername);

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_ConnectorQuery
			
			SET @ResultCode = 200;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_ConnectorQuery	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spPOST_ConnectorQuery TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_ConnectorQuery'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)                                                      
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Connector Query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@QuerySQL';
    SET @value      = N'SQL statement that is used to read data from the source system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@QueryVersionComment';
    SET @value      = N'Comment for this specific version of the connector query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
GO

DROP PROCEDURE IF EXISTS system.spPOST_Dashboard;
GO
/*
	Procedure to POST an Dashboard
		- ID will be autogenerated if posted with * as ID
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_Dashboard
				 @Username				= N'SQL'
				,@DashboardID			= N'*'
				,@DashboardName			= N'Test'
				,@DashboardDescription	= N'bla'
				,@LayoutDefinition		= 0x
				,@OutputID				= @ReturnedID OUTPUT;
		PRINT    @RC
		PRINT   'Returned ID: ' + ISNULL(@ReturnedID,'<no ID returned>')

		SELECT * FROM system.tDashboards;
		DELETE FROM system.tDashboards;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Dashboard',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Dashboard','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spPOST_Dashboard
    (
         @Username				NVARCHAR(255)
        ,@DashboardID				NVARCHAR(50)
		,@DashboardName			NVARCHAR(255)	= NULL
		,@DashboardDescription		NVARCHAR(4000)	= NULL
		,@LayoutDefinition		VARBINARY(MAX)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','''											 
				+ ISNULL(@DashboardID				,N'NULL')	+ N''','''
				+ ISNULL(@DashboardName				,N'NULL')	+ N''','''
				+ ISNULL(@DashboardDescription		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastIDNum			NVARCHAR(50)	= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @DashboardID				IS NULL SET @DashboardID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @DashboardID			= dbo.sx_pf_pProtectID	(@DashboardID);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Dashboard
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Transaction user don`t exists', 16, 10);
						END;

				-- Check existence of IDs of depening content

				-- Don't accept DashboardID = ''
				IF @DashboardID  = N'' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('DashboardID cant be empty string', 16, 10);
					END;

				-- cache existing values of this Object
				DROP TABLE IF EXISTS #CurrenctDashboard;
				CREATE TABLE #CurrentDashboard
					(
						 DashboardKey			BIGINT										NOT NULL
						,DashboardID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,DashboardName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,DashboardDescription	NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,LayoutDefinition	VARBINARY(MAX)								NOT NULL
					)
				INSERT INTO #CurrentDashboard
					SELECT
						 DashboardKey
						,DashboardID
						,DashboardName
						,DashboardDescription
						,LayoutDefinition
					FROM system.tDashboards
					WHERE DashboardID = @DashboardID;
			
				-- Data Transaction with * resolving
				PRINT CONCAT(N'Start writing Values...',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT @RowCount = Count (*) FROM #CurrentDashboard; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @DashboardID = '*'
							BEGIN
								SELECT TOP 1 
								@LastIDNum = TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT ) 
								FROM system.tDashboards
								WHERE 
										DashboardID LIKE N'D%'
									AND LEN(DashboardID) > 1
									AND TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) DESC

								IF @LastIDNum IS NULL SET @LastIDNum = 0;
								SET @DashboardID = 'D' + CAST(@LastIDNum +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tDashboards VALUES
							(		 
								 @DashboardID
								,ISNULL(@DashboardName,N'<set name>')
								,ISNULL(@DashboardDescription,'')
								,ISNULL(@LayoutDefinition,0x)
							);
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tR
								SET 
									 DashboardName				= ISNULL(@DashboardName			,cR.DashboardName)
									,DashboardDescription		= ISNULL(@DashboardDescription		,cR.DashboardDescription)
									,LayoutDefinition		= ISNULL(@LayoutDefinition		,cR.LayoutDefinition)
								
							FROM system.tDashboards tR
								LEFT JOIN #CurrentDashboard cR
									ON tR.DashboardKey = cR.DashboardKey
							WHERE tR.DashboardKey = cR.DashboardKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @DashboardID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Dashboard
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Dashboard	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spPOST_Dashboard TO pf_PlanningFactoryService;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Dashboard'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure POST a Dashboard Definition.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@DashboardID';
    SET @value      = N'BusinessID for the Dashboard - optional, autogenerated when posted as *.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DashboardName';
    SET @value      = N'Name for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@DashboardDescription';
    SET @value      = N'Description for the Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutDefinition';
    SET @value      = N'Layout Definition for the Dashboard as varbinary with images etc..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spPOST_DataSource;
GO
/*
	Procedure to POST a DataSource
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Base tables
		SELECT * FROM system.tDataSources
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spPOST_DataSource
				 @Username				= N'SQL'
				,@DataSourceID			= N'0'  -- must be numeric for compatibility with modules. Full ID Support from 6.0 on
				,@DataSourceName		= N'GT1'
				,@SourceType			= N''
				,@ConnectionValuesJSON	= N''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_DataSource',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_DataSource','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_DataSource
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50)
		,@DataSourceName		NVARCHAR(255) = ''
		,@SourceType			NVARCHAR(50) = ''
		,@ConnectionValuesJSON	NVARCHAR(MAX) = ''
    )

-- must be always execute as owner as it uses IDENTITY INSERT - but this don't work on Azure, if the Owner is an AAD User - OPEN PROBLEM
WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	=		 N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID						,N'NULL')	+ N''','''
				+ ISNULL(@DataSourceName					,N'NULL')	+ N''','''
				+ ISNULL(@SourceType						,N'NULL')	+ N''','''
				+ ISNULL(@ConnectionValuesJSON				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@DataSourceKey		INT				= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID			= N'';
		IF @DataSourceName			IS NULL SET @DataSourceName			= N'';
		IF @SourceType				IS NULL SET @SourceType				= N'';
		IF @ConnectionValuesJSON	IS NULL SET @ConnectionValuesJSON	= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_DataSource
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'Transaction user don`t exists', 16, 10);
					END;
				
				IF @DataSourceID  = ''
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID must not be empty.', 16, 10);
					END;

				IF @DataSourceID  = '0'
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID 0 is reserved for Companies without Datasource and cant be created.', 16, 10);
					END;

				IF TRY_CAST(@DataSourceID AS INT) IS NULL
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID must be numeric for module compatibility.', 16, 10);
					END;

				IF @DataSourceName  = '' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('DataSourceName must not be empty.', 16, 10);
					END;

				-- KEY AND ID MUST BE THE SAME IN 5.6
				SET @DataSourceKey = CAST(@DataSourceID AS INT);

				-- Data Transactions #################################################

				-- delete / recreate DataSource with same Key
				DELETE FROM system.tDataSources WHERE DataSourceID = @DataSourceID;
				DELETE FROM system.tDataSources WHERE DataSourceKey = @DataSourceKey; -- you to be sure
	
				SET IDENTITY_INSERT system.tDatasources ON;
				INSERT INTO system.tDataSources (DatasourceKey,DataSourceID,DataSourceName,SourceType,ConnectionValuesJSON)
					VALUES (@DataSourceKey,@DataSourceID,@DataSourceName,@SourceType,@ConnectionValuesJSON)
				SET IDENTITY_INSERT system.tDatasources OFF;
	
				SET @ResultCode = 200;
			COMMIT TRANSACTION spPOST_DataSource
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_DataSource	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_DataSource TO pf_PlanningFactoryService



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_DataSource'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'The ID of the DataSource, an ID which is used to identify the datasource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DataSourceName';
    SET @value      = N'The name of the DataSource, free description';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceType';
    SET @value      = N'The Source type of the datasource, its a CODE Word like MSSQL and determines the way a connection is established.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ConnectionValuesJSON';
    SET @value      = N'Encrypted string containing the Connection String including credentials.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO





DROP PROCEDURE IF EXISTS system.spPOST_Email;
GO

/*
	Procedure to POST one Email
		- send NULL as Parametervalue to keep the existing Value
		- you can skip Parameter you would send with NULL, as NULL is the default value
		- you can pass * as EmailID to create a new Email E1..En

	This Procedure will be executed
		- from the OCT Application Edit GUI - always as User which is logged in
		- from Consultant in SSMS - as User SQL (which there must have role FactorySerivce or db_owner)
		- from a control script (create Email for exections) as User SQL

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedEmailID	NVARCHAR (50)
		EXEC	 @RC = system.spPOST_Email
				 @Username				= 'SQL'
				,@EmailID				= '*'
				,@EmailSubject			= 'Wichtig'
				,@EmailRecipients		= 'gerd.tautenhahn@saxess-software.com'
				,@EmailText				= 'Hello World'
				,@EmailAttachments		= NULL	-- List for FileIDs
				,@EmailStatusCODE		= 'PENDING'
				,@SourceTypeCODE		= 'PIPELINE'
				,@SourceID				= 'DS1-P2'
				,@CreationDate			= '20220613 08:04:02.111'
				,@PostDate				= '20220613 08:08:02.111'
				,@OutputID				= @ReturnedEmailID OUTPUT;
		PRINT    @RC;
		PRINT	'EmailID Output: '+ ISNULL(@ReturnedEmailID,'no Output set');

		SELECT * FROM system.tEmails;
		DELETE FROM system.tEmails;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Email',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Email','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Email
    (
         @Username				NVARCHAR(255)
        ,@EmailID				NVARCHAR(50)	
		,@EmailSubject			NVARCHAR(2000)	= NULL
		,@EmailRecipients		NVARCHAR(4000)	= NULL
		,@EmailText				NVARCHAR(MAX)	= NULL
		,@EmailAttachments		NVARCHAR(2000)	= NULL
		,@EmailStatusCODE		NVARCHAR(50)	= NULL
		,@SourceTypeCODE		NVARCHAR(50)	= NULL
		,@SourceID				NVARCHAR(50)	= NULL
		,@CreationDate			DATETIME		= NULL
		,@PostDate				DATETIME		= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT	
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','''		
				+ ISNULL(@EmailID									,N'NULL')	+ N''','''
				+ ISNULL(@EmailSubject								,N'NULL')	+ N''','''
				+ ISNULL(@EmailRecipients							,N'NULL')	+ N''','''
				+ ISNULL(@EmailText									,N'NULL')	+ N''','''
				+ ISNULL(@EmailAttachments							,N'NULL')	+ N''','''
				+ ISNULL(@EmailStatusCODE							,N'NULL')	+ N''','''
				+ ISNULL(@SourceTypeCODE							,N'NULL')	+ N''','''
				+ ISNULL(@SourceID									,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@CreationDate,104)	,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@PostDate,104)		,N'NULL')	+ N''''		

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastEmail			INT				= 0;



		-- NULL Protection for all mandatory  Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @EmailID				IS NULL SET @EmailID			= N'';


		-- Content Protection for specific input parameters (e.g.IDs)
		SET @EmailID		= dbo.sx_pf_pProtectID	(@EmailID);
		IF @EmailStatusCODE NOT IN ('PENDING','ERROR','SENT') SET @EmailStatusCODE = 'CODEERROR';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Email
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Starte Operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #CurrentEmail;
				CREATE TABLE #CurrentEmail
					(
						 EmailKey				BIGINT										NOT NULL
						,EmailID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,EmailSubject			NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailRecipients		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailText				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NULL
						,EmailAttachments		NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailStatusCODE		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,SourceTypeCODE			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,SourceID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,CreationDate			DATETIME		NULL
						,PostDate				DATETIME		NULL

					)

				-- Cache current values
				INSERT INTO #CurrentEmail
					SELECT 
						 EmailKey			
						,EmailID			
						,EmailSubject		
						,EmailRecipients	
						,EmailText			
						,EmailAttachments	
						,EmailStatusCODE	
						,SourceTypeCODE		
						,SourceID			
						,CreationDate		
						,PostDate			
					FROM system.tEmails
					WHERE EmailID = @EmailID;

				-- insert or update Email
				SELECT @RowCount = Count (*) FROM #CurrentEmail; 

				IF @RowCount = 0
					-- case new Email
					BEGIN
						--- determine next free ID if * is sended as ID, AutoID is generated as P1..Pn
						IF @EmailID = '*'
							BEGIN
								SELECT TOP 1 
								@LastEmail = TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT ) 
								FROM system.tEmails 
								WHERE 
										EmailID LIKE 'E%'
									AND LEN(EmailID) > 1
									AND TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT) DESC

								IF @LastEmail IS NULL SET @LastEmail = 0;
								SET @EmailID = 'E' + CAST(@LastEmail +1 AS NVARCHAR(50));
							END

						-- INSERT with default values if NULL
						INSERT INTO system.tEmails VALUES
							(
								  @EmailID		
								 ,ISNULL(@EmailSubject		,'')
								 ,ISNULL(@EmailRecipients	,'')
								 ,ISNULL(@EmailText			,'')
								 ,ISNULL(@EmailAttachments	,'')
								 ,ISNULL(@EmailStatusCODE	,'')
								 ,ISNULL(@SourceTypeCODE		,'')
								 ,ISNULL(@SourceID			,'')
								 ,ISNULL(@CreationDate		,'1900-01-01 08:08:00.000')
								 ,ISNULL(@PostDate			,NULL)

							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					-- Case existing Email
					BEGIN
						-- UPDATE 
						UPDATE tE
								SET 
									  EmailSubject			= ISNULL(@EmailSubject		,cE.EmailSubject		)
									 ,EmailRecipients		= ISNULL(@EmailRecipients	,cE.EmailRecipients		)
									 ,EmailText				= ISNULL(@EmailText			,cE.EmailText			)
									 ,EmailAttachments		= ISNULL(@EmailAttachments	,cE.EmailAttachments	)
									 ,EmailStatusCODE		= ISNULL(@EmailStatusCODE	,cE.EmailStatusCODE		)
									 ,SourceTypeCODE		= ISNULL(@SourceTypeCODE	,cE.SourceTypeCODE		)	
									 ,SourceID				= ISNULL(@SourceID			,cE.SourceID			)
									 ,CreationDate			= ISNULL(@CreationDate		,cE.CreationDate		)
									 ,PostDate				= ISNULL(@PostDate			,cE.PostDate			)

							FROM system.tEmails tE
								LEFT JOIN #CurrentEmail cE
									ON tE.EmailKey = cE.EmailKey
							WHERE tE.EmailKey = cE.EmailKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- OUTPUT Parameter
				SET @OutputID = @EmailID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Email
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Email	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO


GRANT EXECUTE ON OBJECT :: system.spPOST_Email TO pf_PlanningFactoryService;
GO
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Email'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@EmailID';
    SET @value      = N'ID of the Email, will be updated or replaced, use * to create an new Email with AutoID E1..En';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
     
	SET @level2name = N'@EmailSubject';
    SET @value      = N'Subject of the Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailRecipients';
    SET @value      = N'List of Email Recipients, semicolon separated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailText';
    SET @value      = N'EmailText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailAttachments';
    SET @value      = N'List of FileIDs, semicolon separated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailStatusCODE';
    SET @value      = N'Status Code for the Email, can be PENDING,ERROR,SENT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceTypeCODE';
    SET @value      = N'Source Type (who created this mail) - can be PIPELINE, EVENT, MANUALLY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceID';
    SET @value      = N'ID combination of the source oftthe mail - e.g. [DatasourceID]-[PipelineID] or [FactoryID]-[ProductLineID]-[ProductID] or someting else - only for Debugging, not technical used';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CreationDate';
    SET @value      = N'Datetime of Email creation';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PostDate';
    SET @value      = N'Datetime of Email posting';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

DROP PROCEDURE IF EXISTS system.spPOST_Icon;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC system.spPOST_Icon
			'SQL'
			,'Hase'
			,'svg'
			,0x3;

		SELECT * FROM system.tIcons

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Icon',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Icon','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Icon
	 @UserName		NVARCHAR(255)	= N''
	,@IconName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', ''' + ISNULL(@IconName, N'NULL') + N''', '''
													+ ISNULL(@FileExtension, N'NULL') + N''', ' + ISNULL(CAST(DATALENGTH(@FileBody) AS NVARCHAR(MAX)) + N' bytes', N'NULL') 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		-- STEP 0.1 - NULL Protection

		IF @UserName IS NULL SET @UserName = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_Icon;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			
			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

			-- Delete File
			DELETE FROM system.tIcons WHERE IconName = @IconName
				
				IF LEN(@FileExtension) = 0 
				BEGIN
					SET @FileExtension = NULL;
				END;

				-- STEP 2.2 - Creating new file

				INSERT INTO system.tIcons (IconName,FileExtension, FileBody)
				VALUES (@IconName,@FileExtension, @FileBody);

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_Icon;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_Icon;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.spPOST_Icon TO pf_PlanningFactoryService;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Icon'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'spPOST_Icon';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@UserName';
    SET @value      = N'UserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IconName';
    SET @value      = N'IconName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileExtension';
    SET @value      = N'FileExtension';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileBody';
    SET @value      = N'FileBody';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS system.spPOST_LogicIndexItem;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		EXEC system.spPOST_LogicIndexItem
			 'SQL'
			,'FIN'
			,'1.0';

		SELECT * FROM system.tLogicIndex

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_LogicIndexItem',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_LogicIndexItem','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_LogicIndexItem
	 @Username		NVARCHAR(255)
	,@ModuleID		NVARCHAR(50)
	,@ModuleVersion	NVARCHAR(50)

AS
BEGIN

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username, N'NULL') + N''', ''' 
													+ ISNULL(@ModuleID, N'NULL') + N''', ''' 
													+ ISNULL(@ModuleVersion, N'NULL') + N''''''							
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ModuleID		IS NULL SET @ModuleID		= N'';
	IF @ModuleVersion	IS NULL SET @ModuleVersion	= N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_LogicIndexItem;

			-- Determine transaction user
			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			--DROP module item of this version if it already exists
			DELETE FROM system.tLogicIndex 
				WHERE 
						ModuleID = @ModuleID 
					AND ModuleVersion = @ModuleVersion

			--INSERT new item
			INSERT INTO system.tLogicIndex
				VALUES (@ModuleID,@ModuleVersion, CONVERT(DATE,GETUTCDATE()));

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_LogicIndexItem;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_LogicIndexItem;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::system.spPOST_LogicIndexItem TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_LogicIndexItem'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST an LogicIndexItem. Every activation of an module will call 
					this procedure so save the information that this module in that version was activated on a certain day.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleVersion';
    SET @value      = N'ModuleVersion';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
 
DROP PROCEDURE IF EXISTS system.spPOST_Pipeline;
GO

/*
	Template to POST one Pipeline
		- send NULL as Parametervalue to keep the existing Value
		- you can skip Parameter you would send with NULL, as NULL is the default value
		- you can pass * as PipelineID to create a new pipeline P1..Pn

	This Procedure will be executed
		- from the OCT Application Edit GUI - always as User which is logged in
		- from the OCT Application Module Activation - always as User which is logged in
		- from Consultant in SSMS - as User SQL (which there must have role FactorySerivce or db_owner)
		- from a control script (create Pipeline due to global management from Global DB) as User SQL

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedPipelineID	NVARCHAR (50)
		EXEC	 @RC = system.spPOST_Pipeline
				 @Username						= 'SQL'
				,@PipelineID					= '*'
				,@PipelineName					= NULL
				,@PipelineDescription			= 'Hello World'
				,@PipelineDescriptionIconColor  = '#808080'
				,@OrderOfDisplay				= NULL
				,@EmailNotificationCODE			= NULL
				,@OutputID						= @ReturnedPipelineID OUTPUT;
		PRINT    @RC;
		PRINT	'PipelineID Output: '+ ISNULL(@ReturnedPipelineID,'no Output set');

		SELECT * FROM system.tPipelines;
		DELETE FROM system.tPipelines;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Pipeline
    (
         @Username						NVARCHAR(255)
        ,@PipelineID					NVARCHAR(50)	
		,@PipelineName					NVARCHAR(255)	= NULL
		,@PipelineDescription			NVARCHAR(4000)	= NULL
		,@PipelineDescriptionIconColor	NVARCHAR(50)	= NULL
		,@OrderOfDisplay				INT				= NULL
		,@EmailNotificationCODE			NVARCHAR(50)	= NULL	
		,@OutputID						NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID							,N'NULL')	+ N''','''
				+ ISNULL(@PipelineName							,N'NULL')	+ N''','''
				+ ISNULL(@PipelineDescription					,N'NULL')	+ N''','''
				+ ISNULL(@PipelineDescriptionIconColor			,N'NULL')	+ N''','
				+ ISNULL(CAST (@OrderOfDisplay AS NVARCHAR(255)),N'NULL')	+ N','''
				+ ISNULL(@EmailNotificationCODE					,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastPipeline		INT				= 0;

		-- NULL Protection for all mandatory  Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @PipelineID		= dbo.sx_pf_pProtectID	(@PipelineID);
		IF @EmailNotificationCODE NOT IN ('NEVER','ALWAYS','ONERROR') SET @EmailNotificationCODE = 'NEVER';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Starte Operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #CurrentPipeline;
				CREATE TABLE #CurrentPipeline
					(
						 PipelineKey					BIGINT										NOT NULL
						,PipelineID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineDescription			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineDescriptionIconColor	NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OrderOfDisplay					INT											NOT NULL
						,EmailNotificationCODE			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					)

				-- Cache current values
				INSERT INTO #CurrentPipeline
					SELECT 
						 PipelineKey
						,PipelineID
						,PipelineName
						,PipelineDescription
						,PipelineDescriptionIconColor
						,OrderOfDisplay
						,EmailNotificationCODE
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;

				-- insert or update Pipeline
				SELECT @RowCount = Count (*) FROM #CurrentPipeline; 

				IF @RowCount = 0
					-- case new Pipeline
					BEGIN
						--- determine next free ID if * is sended as ID, AutoID is generated as P1..Pn
						IF @PipelineID = '*'
							BEGIN
								SELECT TOP 1 
								@LastPipeline = TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT ) 
								FROM system.tPipelines 
								WHERE 
										PipelineID LIKE 'P%'
									AND LEN(PipelineID) > 1
									AND TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) DESC

								IF @LastPipeline IS NULL SET @LastPipeline = 0;
								SET @PipelineID = 'P' + CAST(@LastPipeline +1 AS NVARCHAR(50));
							END

						-- Determine Next Ordernumber to be added at the end
						IF @OrderOfDisplay IS NULL
							BEGIN
								SELECT @OrderOfDisplay = MAX(OrderOfDisplay) + 1
								FROM system.tPipelines
							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelines VALUES
							(
								 @PipelineID
								,ISNULL(@PipelineName,'<set name>')
								,ISNULL(@PipelineDescription,'')
								,ISNULL(@PipelineDescriptionIconColor,'')
								,ISNULL(@OrderOfDisplay,0)
								,ISNULL(@EmailNotificationCODE,'NEVER')
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					-- Case existing Pipeline
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						UPDATE tP
								SET 
									 PipelineName					= ISNULL(@PipelineName					,cP.PipelineName)
									,PipelineDescription			= ISNULL(@PipelineDescription			,cP.PipelineDescription)
									,PipelineDescriptionIconColor	= ISNULL(@PipelineDescriptionIconColor	,cP.PipelineDescriptionIconColor)
									,OrderOfDisplay					= ISNULL(@OrderOfDisplay				,cP.OrderOfDisplay)
									,EmailNotificationCODE			= ISNULL(@EmailNotificationCODE			,cP.EmailNotificationCODE)
							FROM system.tPipelines tP
								LEFT JOIN #CurrentPipeline cP 
									ON tP.PipelineKey = cP.PipelineKey
							WHERE tP.PipelineKey = cP.PipelineKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- OUTPUT Parameter
				SET @OutputID = @PipelineID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spPOST_Pipeline TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)                                                      
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline, will be updated or replaced, use * to create an new Pipeline with AutoID P1..Pn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PipelineName';
    SET @value      = N'Name of the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PipelineDescription';
    SET @value      = N'Description of the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PipelineDescriptionIconColor';
    SET @value      = N'Icon Color for the description, written as HEX Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderOfDisplay';
    SET @value      = N'Ordernumber for displaying the Pipelines';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailNotificationCODE';
    SET @value      = N'EmailNotificationCODE of the Pipeline - must be NEVER, ALWAYS or ONERROR.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
 
DROP PROCEDURE IF EXISTS system.spPOST_PipelineLog;
GO

/*
	Procedure to POST a PipelineLog
		- can be posted with ExecutionID = NULL and will return a new generated ExecutionID, which is a unique number for the database
		- optional Values can be sended as NULL or skiped, which means "keep what is"
		- the PipelineName will be materialized in the Background to be avaiable in historic logs after deleting the pipeline

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineLog
				 @Username		= 'SQL'
				,@PipelineID	= 'P1'
				,@ExecutionID	= 'EID9'
				,@StatusCODE	= 'RUNNING'			-- WAITING,RUNNING, STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
				,@StartedBy		= 'W10\admin'		-- Username or the word "Scheduler"
				,@StartTime		= NULL				-- Timestamp 
				,@EndTime		= NULL				-- Timestamp
				,@OutPutID		= @ReturnID OUTPUT
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineLog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineLog
    (
          @Username				NVARCHAR(255)
		 ,@PipelineID			NVARCHAR(50)
		 ,@ExecutionID			NVARCHAR(50)	
		 ,@ExecutionServerName	NVARCHAR(255)	= NULL
		 ,@StatusCODE			NVARCHAR(50)	= NULL
		 ,@StartedBy			NVARCHAR(255)	= NULL
		 ,@StartTime			DATETIME		= NULL
		 ,@EndTime				DATETIME		= NULL
		 ,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	
	   PRINT 1
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','''									 
				+ ISNULL(@PipelineID			,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID			,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionServerName	,N'NULL')	+ N''','''
				+ ISNULL(@StatusCode			,N'NULL')	+ N''','''
				+ ISNULL(@StartedBy				,N'NULL')	+ N''','''	
				+ ISNULL(CONVERT(NVARCHAR(50),@StartTime,104)		,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@EndTime,104)			,N'NULL')	+ N''''		

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTimeRun		DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@PipelineName		NVARCHAR(255)	= N'';

				 PRINT 2
		-- NULL Protection for all non-optional Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';


		SET @PipelineID		= dbo.sx_pf_pProtectID	(@PipelineID);

		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineLog
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check exictence of IDs this procedure depends on
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('PipelineID don`t exits', 16, 10);
					END

				-- Cache Values
				PRINT CONCAT(N'Starting value caching',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
				DROP TABLE IF EXISTS #CurrentLog;
				CREATE TABLE #CurrentLog
					(
						 LogKey					BIGINT										NOT NULL
						,PipelineID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionServerName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StatusCODE				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartedBy				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartTime				DATETIME									NULL
						,EndTime				DATETIME									NULL

					);
				INSERT INTO #CurrentLog
					SELECT
						 LogKey			
						,PipelineID		
						,PipelineName	
						,ExecutionID
						,ExecutionServerName
						,StatusCODE		
						,StartedBy		
						,StartTime		
						,EndTime		
					FROM system.tPipelineLog
					WHERE @ExecutionID = ExecutionID;

				SELECT @RowCount = Count (*) FROM #CurrentLog; 
				
				-- Exit on not existing ExecutionID
				IF @RowCount = 0 AND @ExecutionID <>'*'
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('ExecutionID don`t exits', 16, 10);
					END
				
				-- Data Transaction
				PRINT CONCAT(N'Start data operation',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert new LogEntry',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
			
						-- create new ExecutionID and get PipelineName
						SELECT @ExecutionID = 'EID'+ CAST(NEXT VALUE FOR system.seqPipelineLog AS NVARCHAR(50));
						SELECT @PipelineName = PipelineName FROM system.tPipelines WHERE PipelineID = @PipelineID;
							
						PRINT 'Start Insert'
						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineLog VALUES
							(
								 @PipelineID
								,@PipelineName
								,@ExecutionID
								,ISNULL(@ExecutionServerName,'')	
								,ISNULL(@StatusCODE,'')	
								,ISNULL(@StartedBy,'')
								,ISNULL(@StartTime,NULL)
								,ISNULL(@EndTime,NULL)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
						UPDATE t
								SET 
									 ExecutionServerName	= ISNULL(@ExecutionServerName	,c.ExecutionServerName	)
									,StatusCode				= ISNULL(@StatusCode			,c.StatusCode			)
									,StartedBy				= ISNULL(@StartedBy				,c.StartedBy			)
									,StartTime				= ISNULL(@StartTime				,c.StartTime			)
									,EndTime				= ISNULL(@EndTime				,c.EndTime				)

							FROM system.tPipelineLog t
								LEFT JOIN #CurrentLog c
									ON t.LogKey = c.LogKey
							WHERE t.LogKey = c.LogKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ExecutionID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineLog
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineLog	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

		-- additional log in the Process Log to be transferd 
		DECLARE
			 @EventMessage	NVARCHAR(2000)	= ''
			,@ProcessID		NVARCHAR(50)	= ''
			,@Timestamp		DATETIME;

		SET @EventMessage	= CONCAT(N'Execution of Pipeline "',@PipelineName, N'" is ',@StatusCODE,N' on Server ',@ExecutionServerName);  -- results in "Excecution of Pipeline DATEV is Running on Server https://..."
		SET @ProcessID		= CONCAT(@PipelineID,N'-',@ExecutionID);
		IF @EndTime IS NULL SET @Timestamp = @StartTime ELSE SET @Timestamp = @EndTime;
		IF @Timestamp	IS NULL SET @Timestamp = GETUTCDATE();


		EXEC system.pPOST_ProcessLogEntry
			 @Username			= N'SQL'
			,@ModuleCode		= N'DataFlows'
			,@ProcessName		= @PipelineName
			,@ProcessID			= @ProcessID
			,@SubProcessName	= N''
			,@SubProcessID		= ''
			,@SubSubprocessName = N''
			,@EventCODE			= @StatusCODE
			,@Timestamp			= @Timestamp
			,@ReturnCODE		= @ResultCode
			,@EventMessage		= @EventMessage;

		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineLog TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST an Logentry for an Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline this LogEntry belongs to';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ExecutionID';
    SET @value      = N'ExecutionID in case of update operation - POST as * to generate a new ExecutionID ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ExecutionServerName';
    SET @value      = N'Name of the Server, which executed this task';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StatusCODE';
    SET @value      = N'Status CODE of the PipelineLog - see list at table definiton.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StartedBy';
    SET @value      = N'Name of the user who started this pipeline, is Scheduler in case of scheduler started';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StartTime';
    SET @value      = N'Time of starting the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@EndTime';
    SET @value      = N'Time the pipeline was finished or setted to finish due to not succesful ending.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@OutputID';
    SET @value      = N'Returns the used or new generated ExecutionID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	

DROP PROCEDURE IF EXISTS system.spPOST_PipelineSchedule;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedScheduleID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineSchedule
				 @Username					= 'SQL'
				,@PipelineID				= 'P1'			
				,@ScheduleID				= '*'
				,@ScheduleName				= 'Hallo'	
				,@ScheduleDescription		= 'Desc'
				,@ScheduleDetailsJSON		= '{}'
				,@Active					= 1
				,@OutputID					= @ReturnedScheduleID OUTPUT;			
		PRINT    @RC
		PRINT	 'ScheduleID OUT: ' + @ReturnedScheduleID;

		SELECT * FROM system.tPipelineSchedules;
		SELECT * FROM system.tPipelines;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineSchedule',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineSchedule','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineSchedule
    (
         @Username					NVARCHAR(255)
		,@PipelineID				NVARCHAR(50)
        ,@ScheduleID				NVARCHAR(50)
		,@ScheduleName				NVARCHAR(255)	= NULL
		,@ScheduleDescription		NVARCHAR(4000)	= NULL
		,@ScheduleDetailsJSON		NVARCHAR(MAX)	= NULL
		,@Active					BIT				= NULL
		,@OutputID					NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username						,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleID					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleName					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleDescription			,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleDetailsJSON			,N'NULL')	+ N''','
				+ ISNULL(CAST(@Active AS NCHAR(1))		,N'NULL')	+ N''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT			= 0
				,@RowCount			INT				= 0
				,@LastSchedule		INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ScheduleID				IS NULL SET @ScheduleID			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ScheduleID		= dbo.sx_pf_pProtectID	(@ScheduleID);


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineSchedule
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check Existence of Pipeline
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;


				-- Cache
				DROP TABLE IF EXISTS #CurrentSchedule;
				CREATE TABLE #CurrentSchedule
					(
						 ScheduleKey				BIGINT										NOT NULL
						,ScheduleID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleName				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleDescription		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleDetailsJSON		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,Active						BIT											NOT NULL
					)

				INSERT INTO #CurrentSchedule
					SELECT
						 ScheduleKey
						,ScheduleID
						,ScheduleName
						,ScheduleDescription
						,ScheduleDetailsJSON
						,Active
					FROM system.tPipelineSchedules
					WHERE 
							PipelineKey =  @PipelineKey
						AND ScheduleID	=  @ScheduleID
				
				-- Data transaction
				SELECT @RowCount = Count (*) FROM #CurrentSchedule; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @ScheduleID = '*'
							BEGIN
								SELECT TOP 1 
								@LastSchedule = TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT ) 
								FROM system.tPipelineSchedules
								WHERE 
										ScheduleID LIKE 'J%'
									AND LEN(ScheduleID) > 1
									AND PipelineKey = @PipelineKey
									AND TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT) DESC

								IF @LastSchedule IS NULL SET @LastSchedule = 0;
								SET @ScheduleID = 'J' + CAST(@LastSchedule + 1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineSchedules VALUES
							(
								 @PipelineKey
								,@ScheduleID
								,ISNULL(@ScheduleName,'<set name>')
								,ISNULL(@ScheduleDescription,'')
								,ISNULL(@ScheduleDetailsJSON,'')
								,ISNULL(@Active,1)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tPS
								SET 
									 ScheduleName				= ISNULL(@ScheduleName				,cS.ScheduleName)
									,ScheduleDescription		= ISNULL(@ScheduleDescription		,cS.ScheduleDescription)
									,ScheduleDetailsJSON		= ISNULL(@ScheduleDetailsJSON	,cS.ScheduleDetailsJSON)
									,Active						= ISNULL(@Active					,cS.Active)

						FROM system.tPipelineSchedules tPS
							LEFT JOIN #CurrentSchedule cS
								ON tPS.ScheduleKey = cS.ScheduleKey
						WHERE tPS.ScheduleKey = cS.ScheduleKey;
							
						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 
				-- Output
				SET @OutputID = @ScheduleID;
				
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineSchedule
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineSchedule	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineSchedule TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineSchedule'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to call demo values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline which contains this schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ScheduleID';
    SET @value      = N'ID of the Schedule - POST * to create a new autogenerated ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleName';
    SET @value      = N'optional - name for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleDescription';
    SET @value      = N'optional - description for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleDetailsJSON';
    SET @value      = N'optional - definition of the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Active';
    SET @value      = N'optional - Flag to set schedule to active / inactive.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spPOST_PipelineStep;
GO

/*
	Procedure to create a new pipeline step.
		- as StepID you can pass * to create an AutoID S1..Sn

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedStepID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineStep
				 @Username				= 'SQL'
				,@PipelineID			= 'P1'
				,@StepID				= '*'
				,@StepName				= 'Step1'
				,@StepDescription		= 'Hallo'
				,@OrderOfExecution		= 1
				,@StepDataSourceID		= '1'   -- not in use, Datasource is saved in StepDetailsJSON
				,@StepCompanyIDsJSON	= '{}'
				,@StepDetailsJSON		= '{}'
				,@TimeExecutionLimit	= 0
				,@Active				= 1
				,@ModuleID				= 'FIN'
				,@StartNextStepOnError	= 1
				,@OutputID				= @ReturnedStepID OUTPUT;
		PRINT    @RC;
		PRINT	'StepID_OUT :'+ @ReturnedStepID;

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStep',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStep','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineStep
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)	
		,@StepID				NVARCHAR(50)
		,@StepName				NVARCHAR(255)	= NULL		
		,@StepDescription		NVARCHAR(4000)	= NULL
		,@OrderOfExecution		INT				= NULL
		,@StepDataSourceID		NVARCHAR(50)	= NULL
		,@StepCompanyIDsJSON	NVARCHAR(MAX)	= NULL
		,@StepDetailsJSON		NVARCHAR(MAX)	= NULL
		,@TimeExecutionLimit	INT				= NULL
		,@Active				BIT				= NULL
		,@ModuleID				NVARCHAR(50)	= NULL
		,@StartNextStepOnError	BIT				= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username										,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID									,N'NULL')	+ N''','''
				+ ISNULL(@StepID										,N'NULL')	+ N''','''
				+ ISNULL(@StepName										,N'NULL')	+ N''','''
				+ ISNULL(@StepDescription								,N'NULL')	+ N''','
				+ ISNULL(CAST(@OrderOfExecution AS NVARCHAR(50))		,N'NULL')	+ N','''
				+ ISNULL(@StepDataSourceID								,N'NULL')	+ N''','''
				+ ISNULL(@StepCompanyIDsJSON							,N'NULL')	+ N''','''
				+ ISNULL(@StepDetailsJSON								,N'NULL')	+ N''','
				+ ISNULL(CAST(@TimeExecutionLimit AS NVARCHAR(50))		,N'NULL')	+ N','
				+ ISNULL(CAST(@Active AS NCHAR(1))						,N'NULL')	+ N','''
				+ ISNULL(@ModuleID										,N'NULL')	+ N''','
				+ ISNULL(CAST(@StartNextStepOnError	 AS NCHAR(1))		,N'NULL')	+ N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT			= 0
				,@RowCount			INT				= 0
				,@LastStep			INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';
		IF @StepID					IS NULL SET @StepID				= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineStep
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				-- Check existence of Pipeline
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;

				-- Cache existing content
				DROP TABLE IF EXISTS #CurrentStep;
				CREATE TABLE #CurrentStep
					(
						 PipelineKey				BIGINT										NOT NULL
						,StepKey					BIGINT										NOT NULL
						,StepID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepDescription			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OrderOfExecution			INT											NOT NULL
						,StepDataSourceID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepCompanyIDsJSON			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepDetailsJSON			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,TimeExecutionLimit			INT											NOT NULL
						,Active						BIT											NOT NULL
						,ModuleID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartNextStepOnError		BIT 										NOT NULL
					);

				INSERT INTO #CurrentStep
					SELECT 
						 PipelineKey
						,StepKey
						,StepID
						,StepName
						,StepDescription
						,OrderOfExecution
						,StepDataSourceID
						,StepCompanyIDsJSON
						,StepDetailsJSON
						,TimeExecutionLimit
						,Active
						,ModuleID
						,StartNextStepOnError
					FROM system.tPipelineSteps 
					WHERE 
							PipelineKey = @PipelineKey 
						AND StepID		= @StepID;

				-- Data Transaction

				SELECT @RowCount = Count (*) FROM #CurrentStep; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @StepID = '*'
							BEGIN
								SELECT TOP 1 
								@LastStep = TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT ) 
								FROM system.tPipelineSteps 
								WHERE 
										StepID LIKE 'S%'
									AND LEN(StepID) > 1
									AND PipelineKey = @PipelineKey
									AND TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) DESC

								IF @LastStep IS NULL SET @LastStep = 0;
								SET @StepID = 'S' + CAST(@LastStep +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineSteps VALUES
							(
								 
								 @PipelineKey
								,@StepID
								,ISNULL(@StepName,'<set name>')
								,ISNULL(@StepDescription,'')
								,ISNULL(@OrderOfExecution,0)
								,ISNULL(@StepDataSourceID,'')
								,ISNULL(@StepCompanyIDsJSON,'')
								,ISNULL(@StepDetailsJSON,'')
								,ISNULL(@TimeExecutionLimit,0)
								,ISNULL(@Active,1)
								,ISNULL(@ModuleID,'')
								,ISNULL(@StartNextStepOnError,0)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tPS
								SET 
									 StepName				= ISNULL(@StepName				,cS.StepName)
									,StepDescription		= ISNULL(@StepDescription		,cS.StepDescription)
									,OrderOfExecution		= ISNULL(@OrderOfExecution		,cS.OrderOfExecution)
									,StepDataSourceID		= ISNULL(@StepDataSourceID		,cS.StepDataSourceID)
									,StepCompanyIDsJSON		= ISNULL(@StepCompanyIDsJSON	,cS.StepCompanyIDsJSON)
									,StepDetailsJSON		= ISNULL(@StepDetailsJSON		,cS.StepDetailsJSON)
									,TimeExecutionLimit		= ISNULL(@TimeExecutionLimit	,cS.TimeExecutionLimit)
									,Active					= ISNULL(@Active				,cS.Active)
									,ModuleID				= ISNULL(@ModuleID				,cS.ModuleID)
									,StartNextStepOnError	= ISNULL(@StartNextStepOnError	,cS.StartNextStepOnError)
							FROM system.tPipelineSteps tPS
								LEFT JOIN #CurrentStep cS
									ON tPS.StepKey = cS.StepKey
							WHERE tPS.StepKey = cS.StepKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @StepID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineStep
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineStep	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineStep TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineStep'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Pipeline step, optional Parameter can be skiped, which will keep existing values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline of the step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step to POST, as StepID you can pass * to create an AutoID S1..Sn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Stepname';
    SET @value      = N'optional - name of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDescription';
    SET @value      = N'optional - description of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderOfExecution';
    SET @value      = N'optional - number for order of execution';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDataSourceID';
    SET @value      = N'not in use, Datasource is saved in StepDetailsJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepCompanyIDsJSON';
    SET @value      = N'optional - JSON Array of the Company IDs to use in this Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDetailsJSON';
    SET @value      = N'Configuration of the step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TimeExecutionLimit';
    SET @value      = N'Execution Time Limit in minutes';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Active';
    SET @value      = N'Flag if the step is active';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the accociated Module, for Steps which have a module parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StartNextStepOnError';
    SET @value      = N'Setting if the next step is started if this step ends with error.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


			
		
 
DROP PROCEDURE IF EXISTS system.spPOST_PipelineStepLog;
GO
/* 
	Procedure to POST one Step Log
		- StepLog for not existing ExecutionID will be created
		- StepLog for existing ExcecutionID will be updated
		- optional Parameters posted with NULL leads to Keep what exists
		- Step name is materialized in the background for history reasons
		- Logmessage is replaced or appended together with a line break
		- Output Parameter not in use

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineStepLog
				  @Username			= 'SQL'
				 ,@PipelineID		= 'P1'
				 ,@StepID			= 'S1'
				 ,@ExecutionID		= 'EID4'
				 ,@StartTime		= NULL
				 ,@EndTime			= NULL
				 ,@StatusCODE		= 'Running'
				 ,@LogMessage		= 'Hallo Gerd'
				 ,@ReplaceMessageBOOL= 1
				 ,@HandleID			= '55'
				 ,@OutputID = @ReturnID OUTPUT;
	
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;
		SELECT * FROM system.tPipelineLog;
		SELECT * FROM system.tPipelineStepLog;
		SELECT * FROM system.tAPI_Log;
		SELECT * FROM system.tProcessLogs;
	
		DELETE FROM system.tPipelineStepLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStepLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStepLog','PARAMETER',NULL);
*/
 
CREATE PROCEDURE system.spPOST_PipelineStepLog
    (
         @Username				NVARCHAR(255)
		,@PipelineID			NVARCHAR(50)
		,@StepID				NVARCHAR(50)
        ,@ExecutionID			NVARCHAR(50)			-- is indexed in table as Unique
		,@StartTime				DATETIME		= NULL 
		,@EndTime				DATETIME		= NULL
		,@StatusCODE			NVARCHAR(50)	= NULL
		,@LogMessage			NVARCHAR(MAX)	= NULL
		,@ReplaceMessageBOOL	BIT				= NULL
		,@HandleID				NVARCHAR(50)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID								,N'NULL')	+ N''','''
				+ ISNULL(@StepID									,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID								,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@StartTime,104)		,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@EndTime,104)			,N'NULL')	+ N''','''
				+ ISNULL(@StatusCODE								,N'NULL')	+ N''','''
				+ ISNULL(@LogMessage								,N'NULL')	+ N''','
				+ ISNULL(CAST(@ReplaceMessageBOOL AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@HandleID									,N'NULL')	+ N''','''
				+ ISNULL(@OutputID									,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTimeRun		DATETIME2		= SysUTCDateTime()
				,@StepName			NVARCHAR(255)	= ''
				,@CurrentLogMessage NVARCHAR(MAX)	= '';

		-- Procedure specific declarations
		DECLARE  @RowCount			INT				= 0
				,@PipelineName		NVARCHAR(2000)	= '';

		-- NULL Protection for all mandatory Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ExecutionID				IS NULL SET @ExecutionID		= N'';
		IF @ReplaceMessageBOOL		IS NULL SET @ReplaceMessageBOOL = 1;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ExecutionID		= dbo.sx_pf_pProtectID	(@ExecutionID);
			
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineStepLog
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check exictence of IDs this procedure depends on
				IF NOT EXISTS (
					SELECT P.PipelineID, PS.StepID 
					FROM system.tPipelines	P
						LEFT JOIN system.tPipelineSteps PS
							ON P.PipelineKey = PS.PipelineKey
						WHERE 
								P.PipelineID	= @PipelineID 
							AND PS.StepID		= @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline or Step don`t exits', 16, 10);
					END;

				IF NOT EXISTS (
					SELECT ExecutionID FROM system.tPipelineLog WHERE ExecutionID = @ExecutionID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Execution ID don`t exits', 16, 10);
					END;

				-- Cache Values 
				DROP TABLE IF EXISTS #CurrentStepLog;
				CREATE TABLE #CurrentStepLog
					(
						 LogKey			BIGINT										NOT NULL
						,PipelineID		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionID	NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartTime		DATETIME									NULL
						,EndTime		DATETIME									NULL
						,LogMessage		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StatusCODE		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,HandleID		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					);
				INSERT INTO #CurrentStepLog
					SELECT
						 LogKey		
						,PipelineID	
						,StepID		
						,ExecutionID
						,StartTime	
						,EndTime	
						,LogMessage	
						,StatusCODE	
						,HandleID	
					FROM system.tPipelineStepLog
					WHERE 
							StepID		= @StepID
						AND ExecutionID = @ExecutionID;

				-- Determine StepName and Pipelinename
				SELECT
						 @StepName		= PS.StepName
						,@PipelineName	= P.PipelineName
				FROM system.tPipelines P
					LEFT JOIN system.tPipelineSteps PS
						ON P.PipelineKey = PS.PipelineKey 
				WHERE 
							P.PipelineID	= @PipelineID
						AND PS.StepID		= @StepID;

				-- Data Transaction
				PRINT CONCAT(N'Start data operation',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
				SELECT @RowCount = COUNT(*) FROM #CurrentStepLog;
				IF @RowCount = 0 
					-- Create a new Log entry
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');

						--PRINT 'PipelineID: '	+ @PipelineID;
						--PRINT 'StepID: '		+ @StepID;
						--PRINT 'ExecutionID: ' + @ExecutionID;
						--PRINT 'Stepname: '	+ @StepName;
						--PRINT 'Logmessage: '	+ @LogMessage;
						--PRINT 'StatusCode: '	+ @StatusCODE;
						--PRINT 'HandleID: '	+ @HandleID;

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineStepLog VALUES
							(
								 @PipelineID	
								,@StepID		
								,@ExecutionID
								,@StepName
								,ISNULL(@StartTime	,NULL	)
								,ISNULL(@EndTime	,NULL	)
								,ISNULL(@LogMessage	,''		)
								,ISNULL(@StatusCODE	,''		)
								,ISNULL(@HandleID	,''		)
							);
						PRINT 'Insert done'
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- decide between replace and append for log message
						IF @LogMessage IS NULL --keep the existing if not submitted
							BEGIN
								SELECT @LogMessage = LogMessage FROM #CurrentStepLog;
							END 
						IF @ReplaceMessageBOOL = 0 -- append if no replace flag
							BEGIN
								SELECT @CurrentLogMessage = LogMessage FROM #CurrentStepLog;
								SET @LogMessage = CONCAT(@CurrentLogMessage,CHAR(13),CHAR(10),@LogMessage)
							END
						-- always limit the complete length to 1M characters
						IF LEN(@LogMessage) > 1000000
							BEGIN
								SET @LogMessage		= LEFT(@LogMessage,1000000)	+ ' --- WARNING !!! Log was cutted to first 1.000k Characters from OCT API Procedure !!!'		
							END

						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
						UPDATE t
								SET 
									 PipelineID		= ISNULL(@PipelineID				,c.PipelineID				)
									,StepID			= ISNULL(@StepID					,c.StepID					)
									,ExecutionID	= ISNULL(@ExecutionID				,c.ExecutionID				)
									,StartTime		= ISNULL(@StartTime					,c.StartTime				)
									,EndTime		= ISNULL(@EndTime					,c.EndTime					)
									,LogMessage		= @LogMessage
									,StatusCODE		= ISNULL(@StatusCODE				,c.StatusCODE				)
									,HandleID		= ISNULL(@HandleID					,c.HandleID					)
									
							FROM system.tPipelineStepLog t
								LEFT JOIN #CurrentStepLog c
									ON t.LogKey = c.LogKey
							WHERE t.LogKey = c.LogKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ExecutionID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineStepLog
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineStepLog	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

		-- additional log in the Process Log to be transferd 
		-- there is the Append / Replace problem - the LogEntry can be replaced or the message appended - but our process log can't handle at the moment an update of LogEntries (as they may be already synced)
		-- that why every append / replace is writen as POST to the Log - this maybe much to much and must be solvend better
		DECLARE
			 @EventMessage	NVARCHAR(2000)	= ''
			,@ProcessID		NVARCHAR(50)	= ''
			,@Timestamp		DATETIME;

		SET @EventMessage	= CONCAT(N'Execution of Pipeline "',@PipelineID, N'" Step "', @StepName,N'" is "',@StatusCODE);  -- results in "Excecution of Pipeline P1 Step DATEV Extraction is Running"
		SET @ProcessID		= CONCAT(@PipelineID,N'-',@ExecutionID);
		SET @Timestamp		= GETUTCDATE();

		EXEC system.pPOST_ProcessLogEntry
			 @Username			= N'SQL'
			,@ModuleCode		= N'DataFlows'
			,@ProcessName		= @PipelineName
			,@ProcessID			= @ProcessID
			,@SubProcessName	= @StepName
			,@SubProcessID		= @StepID
			,@SubSubprocessName = N''
			,@EventCODE			= @StatusCODE
			,@Timestamp			= @Timestamp
			,@ReturnCODE		= @ResultCode
			,@EventMessage		= @EventMessage;

		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights, as FactoryService is not able to access system schema by default
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineStepLog TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineStepLog' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST Step LogEntries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline this log belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step this log belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ExecutionID';
    SET @value      = N'Execution - taken from the PipelineLog to keep Pipeline Log and Step Log together';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@StartTime';
    SET @value      = N'optional - Start Time of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@EndTime';
    SET @value      = N'optional - End Time of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@LogMessage';
    SET @value      = N'Log Message - will always be appended.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ReplaceMessageBOOL';
    SET @value      = N'Flag if the sended Log Message will replace the existing message (1), in case of 0 the message is appended to the existing one and a line break is added.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@StatusCODE';
    SET @value      = N'Status CODE - for definition see table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@HandleID';
    SET @value      = N'HandleID to connect Pentaho Process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OutputID';
    SET @value      = N'OutputID is the ExecutionID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

DROP PROCEDURE IF EXISTS system.spPOST_Report;
GO
/*
	Procedure to POST an Report
		- ID will be autogenerated if posted with * as ID
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_Report
				 @Username			= N'SQL'
				,@ReportID			= N'*'
				,@ReportName		= N'Test'
				,@ReportDescription = N'bla'
				,@LayoutDefinition	= 0x
				,@OutputID			= @ReturnedID OUTPUT;
		PRINT    @RC
		PRINT   'Returned ID: ' + ISNULL(@ReturnedID,'<no ID returned>')

		SELECT * FROM system.tReports;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Report',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Report','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spPOST_Report
    (
         @Username				NVARCHAR(255)
        ,@ReportID				NVARCHAR(50)
		,@ReportName			NVARCHAR(255)	= NULL
		,@ReportDescription		NVARCHAR(4000)	= NULL
		,@LayoutDefinition		VARBINARY(MAX)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','''											 
				+ ISNULL(@ReportID				,N'NULL')	+ N''','''
				+ ISNULL(@ReportName			,N'NULL')	+ N''','''
				+ ISNULL(@ReportDescription		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastIDNum			NVARCHAR(50)	= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ReportID				IS NULL SET @ReportID			= N'';



		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ReportID			= dbo.sx_pf_pProtectID	(@ReportID);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Report
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Transaction user don`t exists', 16, 10);
						END;

				-- Check existence of IDs of depening content

				-- Don't accept ReportID = ''
				IF @ReportID  = N'' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('ReportID cant be empty string', 16, 10);
					END;

				-- cache existing values of this Object
				DROP TABLE IF EXISTS #CurrenctReport;
				CREATE TABLE #CurrentReport
					(
						 ReportKey			BIGINT										NOT NULL
						,ReportID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReportName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReportDescription	NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,LayoutDefinition	VARBINARY(MAX)								NOT NULL
					)
				INSERT INTO #CurrentReport
					SELECT
						 ReportKey
						,ReportID
						,ReportName
						,ReportDescription
						,LayoutDefinition
					FROM system.tReports
					WHERE ReportID = @ReportID;
			
				-- Data Transaction with * resolving
				PRINT CONCAT(N'Start writing Values...',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT @RowCount = Count (*) FROM #CurrentReport; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @ReportID = '*'
							BEGIN
								SELECT TOP 1 
								@LastIDNum = TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT ) 
								FROM system.tReports
								WHERE 
										ReportID LIKE N'R%'
									AND LEN(ReportID) > 1
									AND TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) DESC

								IF @LastIDNum IS NULL SET @LastIDNum = 0;
								SET @ReportID = 'R' + CAST(@LastIDNum +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tReports VALUES
							(		 
								 @ReportID
								,ISNULL(@ReportName,N'<set name>')
								,ISNULL(@ReportDescription,'')
								,ISNULL(@LayoutDefinition,0x)
							);
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tR
								SET 
									 ReportName				= ISNULL(@ReportName			,cR.ReportName)
									,ReportDescription		= ISNULL(@ReportDescription		,cR.ReportDescription)
									,LayoutDefinition		= ISNULL(@LayoutDefinition		,cR.LayoutDefinition)
								
							FROM system.tReports tR
								LEFT JOIN #CurrentReport cR
									ON tR.ReportKey = cR.ReportKey
							WHERE tR.ReportKey = cR.ReportKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ReportID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Report
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Report	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spPOST_Report TO pf_PlanningFactoryService;
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Report'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure POST a Report Definition.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@ReportID';
    SET @value      = N'BusinessID for the Report - optional, autogenerated when posted as *.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReportName';
    SET @value      = N'Name for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ReportDescription';
    SET @value      = N'Description for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutDefinition';
    SET @value      = N'Layout Definition for the Report as varbinary with images etc..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
  
DROP PROCEDURE IF EXISTS system.spPOST_Right;
GO

/*
	POST Operation for UserRight
	UserRight will be created always, not fitting other rights will be deleted to ensure enheritance
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 04/2023 for OCT 5.9

	Test call
	DECLARE  @RC					INT
			,@UserName				NVARCHAR(255) = 'SQL'
			,@PostUserName			NVARCHAR(255) = 'sxs'
			,@FactoryID				NVARCHAR(255) = 'ZT'
			,@ProductLineID			NVARCHAR(255) = 'U'
			,@Right					NVARCHAR(255) = 'Write'
			,@ReadCommentMandatory	NVARCHAR(255) = ''
			,@WriteCommentMandatory	NVARCHAR(255) = ''
		
	EXECUTE @RC = system.spPOST_Right 
			@UserName,
			@PostUserName,
			@FactoryID,
			@ProductLineID,
			@Right,
			@ReadCommentMandatory,
			@WriteCommentMandatory
	PRINT @RC

	Testcall tables
		SELECT * FROM system.trRights;
		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Right',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Right','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spPOST_Right
	@UserName				NVARCHAR(255),
	@PostUserName			NVARCHAR(255),
	@FactoryID				NVARCHAR(255),
    @ProductLineID			NVARCHAR(255),
    @Right					NVARCHAR(255),
    @ReadCommentMandatory	NVARCHAR(255),
    @WriteCommentMandatory  NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE  @PostUserKey		INT				= 0
			,@FactoryKey		INT				= 0
			,@ProductLineKey	INT				= 0
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ClusterRight		NVARCHAR(255)	= N''
			,@FactoryRight		NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username,					N'ISNULL') + N''',''' + ISNULL(@PostUserName,			N'ISNULL') + N''',''' 
				+ ISNULL(@FactoryID,				N'ISNULL') + N''',''' + ISNULL(@ProductLineID,			N'ISNULL') + N''',''' 
				+ ISNULL(@Right,					N'ISNULL') + N''',''' + ISNULL(@ReadCommentMandatory,	N'ISNULL') + N''',''' 
				+ ISNULL(@WriteCommentMandatory,	N'ISNULL') + N''''
			,@EffectedRows		INT				= 0	
			,@ResultCode		INT				= 501		
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@Message			NVARCHAR(2000)	= N''
			,@PostUserIsAdmin	INT				= 0;				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @PostUserName			IS NULL SET @PostUserName			= N'';
	IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
	IF @FactoryID				IS NULL SET @FactoryID				= N'';
	IF @Right					IS NULL SET @Right					= N'';
	IF @ReadCommentMandatory	IS NULL SET @ReadCommentMandatory	= N'';
	IF @WriteCommentMandatory	IS NULL SET @WriteCommentMandatory	= N'';

	-- Capitlize the word Read / Write, lower case for other letters
	SET @Right = UPPER(LEFT(@Right,1))+LOWER(SUBSTRING(@Right,2,LEN(@Right)));

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Right;	

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName			= dbo.sx_pf_pProtectString	(@PostUserName);
		SET @ProductLineID			= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID				= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @Right					= dbo.sx_pf_pProtectString	(@Right);
		SET @ReadCommentMandatory	= dbo.sx_pf_pProtectBoolean (@ReadCommentMandatory);
		SET @WriteCommentMandatory	= dbo.sx_pf_pProtectBoolean (@WriteCommentMandatory);
		
		-- STEP 0.3 - Clean IDs
		SET @Right			= LTRIM(RTRIM(@Right));
		SET @PostUserName	= LTRIM(RTRIM(@PostUserName));
		SET @FactoryID		= LTRIM(RTRIM(@FactoryID));
		SET @ProductLineID	= LTRIM(RTRIM(@ProductLineID));

		IF	@Right = N'' 
			OR @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
				
		-- It`s forbidden to post rights for system users with name 'SQL'
		IF	@PostUserName = N'SQL' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden to post rights for system users reserved name.', 16, 10, @PostUserName);
		END;	

		-- Determine transaction user, the transactuser 'public' is not allowed to post rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to post rights.', 16, 10);
		END;
		
		-- Only OCT Administrators can manage users
		IF NOT EXISTS
			(SELECT 
				Userkey
			FROM system.trUser
			WHERE 
					Username			= @TransactUsername 
				AND IsAdministratorFlag	= 1
				AND Status				= 'Active'
			)
		BEGIN
			SET @Message = CONCAT('Invalid rights, User "',@Transactusername,'" is not an OCT Administrator.');
			RAISERROR(@Message, 16, 10);
		END;

		-- It`s forbidden for a ClusterAdmin to change his own rights
		IF @PostUserName = @TransactUsername	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden for a ClusterAdmin to change his own rights', 16, 10);
		END;

		-- Its forbidden to change other administrators rights
		SELECT @PostUserIsAdmin = COUNT(Username) FROM system.trUser WHERE Username = @PostUserName AND IsAdministratorFlag = 1;
		IF @PostUserIsAdmin = 1	
			BEGIN
				SET @ResultCode = 403;
				RAISERROR(N'Forbidden to change the rights of an Administrator.', 16, 10);
			END;

		-- Empty factory ID must always be connected to empty ProductLineID
		IF	@FactoryID = N'' 
			AND @ProductLineID <> N''
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Empty factory ID must always be connected to empty ProductLineID.', 16, 10);
		END;
		
		-- STEP 2.3 - Check keys
		IF @FactoryID <> N''
		BEGIN
			SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @FactoryKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		IF @ProductLineID <> N'' 
		BEGIN
			SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE ProductLineID = @ProductLineID AND FactoryKey = @FactoryKey;
			
			IF @ProductLineKey = 0 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys not exists', 16, 10);
			END;
		END;

		SELECT @PostUserKey = UserKey FROM system.trUser WHERE Username = @PostUserName;

		IF @PostUserKey  = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys not exists', 16, 10);
		END;

		-- STEP 3 - Set Rule
		IF @Right = N'Deny'
		BEGIN
			-- 'Deny' will delete a right, but never create an entry

			IF @FactoryID = N''
				DELETE FROM system.trRights WHERE UserKey = @PostUserKey;
			ELSE IF @ProductLineID = N''
				DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
			ELSE
				DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

			SET @EffectedRows += @@ROWCOUNT;
			SET @ResultCode = 204;
		END
		ELSE
		BEGIN
			-- Saves only explicit rights on objects and deletes rights which get invalid through right definition on upper levels
			-- Check always Clusteright 
			
			SELECT TOP (1) @ClusterRight = COALESCE([Right], N'None') 
			FROM system.trRights 
			WHERE 
				UserKey = @PostUserKey 
				AND FactoryID = N''
			ORDER BY [Right];

			SET @ResultCode = 204;

			-- CLUSTER RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			-- Write / Read -> delete all other Rights and insert
			IF	@FactoryID = N'' AND @ProductLineID = N''
			BEGIN
				DELETE FROM system.trRights WHERE UserKey = @PostUserKey;

				SET @EffectedRows += @@ROWCOUNT;

				INSERT INTO system.trRights
						(UserKey
						,UserName
						,FactoryID
						,ProductLineID
						,ProductID
						,[Right]
						,ReadCommentMandatory
						,WriteCommentMandatory)
				VALUES  (@PostUserKey			-- UserKey					int
						,@PostUserName			-- UserName					nvarchar(255)
						,N''					-- FactoryID				nvarchar(255)
						,N''					-- ProductLineID			nvarchar(255)
						,N''					-- ProductID				nvarchar(255)
						,@Right					-- Right					nvarchar(255)
						,@ReadCommentMandatory  -- ReadCommentMandatory		int
						,@WriteCommentMandatory -- WriteCommentMandatory	int
						);
				
				SET @EffectedRows += @@ROWCOUNT;
				SET @ResultCode = 200;
			END;

			-- FACTORY RIGHTS are posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN
				-- Write -> delete all other Right in this Factory, dont accept if user has write on Cluster
				IF @Right = N'Write' AND @ClusterRight <> N'Write'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO system.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  (@PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if no cluster rights set -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO system.trRights
							(UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,N''					-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read if Clusterwrite rights -> delete all other Rights in this Factory, dont accept if user has write or read on Cluster
				IF @Right = N'Read' AND @ClusterRight = N'Read'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID;

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END
			END;

			-- PRODUCTLINE RIGHT is posted +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN
				-- Look for Rights on Factory Level
				SELECT TOP (1)  @FactoryRight = COALESCE([Right], N'None') 
				FROM system.trRights 
				WHERE 
						UserKey = @PostUserKey 
					AND FactoryID = @FactoryID 
					AND ProductLineID = N''
				ORDER BY [Right];

				-- Write -> dont accept if user has already write on Cluster or Factory
				IF @Right = N'Write' AND @ClusterRight <> N'Write' AND @FactoryRight <> N'Write'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO system.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey			-- UserKey					int
							,@PostUserName			-- UserName					nvarchar(255)
							,@FactoryID				-- FactoryID				nvarchar(255)
							,@ProductLineID			-- ProductLineID			nvarchar(255)
							,N''					-- ProductID				nvarchar(255)
							,@Right					-- Right					nvarchar(255)
							,@ReadCommentMandatory  -- ReadCommentMandatory		int
							,@WriteCommentMandatory -- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;

				-- Read -> don`t accept if user has read or write on Cluster or Factory
				IF	@Right = N'Read' AND @ClusterRight <> N'Write' AND @ClusterRight <> N'Read' 
					AND @FactoryRight <> N'Write' AND @FactoryRight <> N'Read'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;

					INSERT INTO system.trRights
							( UserKey
							,UserName
							,FactoryID
							,ProductLineID
							,ProductID
							,[Right]
							,ReadCommentMandatory
							,WriteCommentMandatory)
					VALUES  ( @PostUserKey				-- UserKey					int
							,@PostUserName				-- UserName					nvarchar(255)
							,@FactoryID					-- FactoryID				nvarchar(255)
							,@ProductLineID				-- ProductLineID			nvarchar(255)
							,N''						-- ProductID				nvarchar(255)
							,@Right						-- Right					nvarchar(255)
							,@ReadCommentMandatory		-- ReadCommentMandatory		int
							,@WriteCommentMandatory		-- WriteCommentMandatory	int
							);

					SET @EffectedRows += @@ROWCOUNT;
					SET @ResultCode = 200;
				END;
						
				-- Read -> if read should only kill an existing write in Factories with global Read Rights, just delete Write Right
				IF @Right = N'Read' AND @FactoryRight = N'Read'
				BEGIN
					DELETE FROM system.trRights WHERE UserKey = @PostUserKey AND FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
					SET @EffectedRows += @@ROWCOUNT;
				END;
			END;
		END;

		COMMIT TRANSACTION sx_pf_POST_Right;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Right;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
  
-- GRANT
GRANT EXECUTE ON OBJECT ::system.spPOST_Right TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Right'					-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'POST Operation for UserRight
         UserRight will be created always, not fitting other rights will be deleted to ensure enheritance';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@UserName';
    SET @value      = N'UserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PostUserName';
    SET @value      = N'PostUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Right';
    SET @value      = N'Right';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReadCommentMandatory';
    SET @value      = N'ReadCommentMandatory';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@WriteCommentMandatory';
    SET @value      = N'WriteCommentMandatory';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP PROCEDURE IF EXISTS system.spPOST_User;
GO

/*
	POST Operation for a User
	Not existing User will be created, existing one updated
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 04/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC						INT
				,@Username					NVARCHAR(255) = N'SQL'
				,@PostUserName				NVARCHAR(255) = N'sxs'
				,@PersonSurname				NVARCHAR(255) = N''
				,@PersonFirstName			NVARCHAR(255) = N''
				,@Email						NVARCHAR(255) = N''
				,@LDAPIP					NVARCHAR(255) = N''
				,@Status					NVARCHAR(255) = N'Active'
				,@DataFlowRightsCODE		NVARCHAR(255) = N'DESIGNER'
				,@VisualizationRightsCODE	NVARCHAR(255) = N'DESIGNER'
				,@IsAdministratorFlag		BIT			  = 0;

		EXECUTE @RC = system.spPOST_User 
				 @Username
				,@PostUserName
				,@PersonSurname
				,@PersonFirstName
				,@Email
				,@LDAPIP
				,@Status
				,@DataFlowRightsCODE
				,@VisualizationRightsCODE
				,@IsAdministratorFlag;
		PRINT @RC;
	
	Testcall Target Table
		SELECT * FROM system.trUser;
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_User','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spPOST_User
	 @Username					NVARCHAR(255)
	,@PostUserName				NVARCHAR(255)
    ,@PersonSurname				NVARCHAR(255)
    ,@PersonFirstName			NVARCHAR(255)
    ,@Email						NVARCHAR(255)
    ,@LDAPIP					NVARCHAR(255)
    ,@Status					NVARCHAR(255)
	,@DataFlowRightsCODE		NVARCHAR(255)
	,@VisualizationRightsCODE	NVARCHAR(255)
	,@IsAdministratorFlag		BIT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @PostUserKey		INT				= 0
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,									N'NULL') + N''',''' 
								+ ISNULL(@PostUserName,								N'NULL') + N''',''' 
								+ ISNULL(@PersonSurname,							N'NULL') + N''',''' 
								+ ISNULL(@PersonFirstName,							N'NULL') + N''',''' 
								+ ISNULL(@Email,									N'NULL') + N''',''' 
								+ ISNULL(@LDAPIP,									N'NULL') + N''',''' 
								+ ISNULL(@Status,									N'NULL') + N''','''
								+ ISNULL(@DataFlowRightsCODE,						N'NULL') + N''''
								+ ISNULL(@VisualizationRightsCODE,					N'NULL') + N''''
								+ ISNULL(CAST(@IsAdministratorFlag AS NCHAR(1)),	N'NULL') + N''''
			
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501			
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@Message			NVARCHAR(2000)	= N'';

	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);
			
	IF @Username				IS NULL SET @Username					= N'';
	IF @PostUserName			IS NULL SET @PostUserName				= N'';
	IF @PersonSurname			IS NULL SET @PersonSurname				= N'';
	IF @PersonFirstName			IS NULL SET @PersonFirstName			= N'';
	IF @Email					IS NULL SET @Email						= N'';
	IF @LDAPIP					IS NULL SET @LDAPIP						= N'';
	IF @Status					IS NULL SET @Status						= N'';
	IF @DataFlowRightsCODE		IS NULL SET @DataFlowRightsCODE			= N'';
	IF @VisualizationRightsCODE IS NULL SET @VisualizationRightsCODE	= N'';
	IF @IsAdministratorFlag		IS NULL SET @IsAdministratorFlag		= 0;

	BEGIN TRY
		BEGIN TRANSACTION spPOST_User;

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName		= dbo.sx_pf_pProtectString (@PostUserName);
		SET @PersonSurname		= dbo.sx_pf_pProtectString (@PersonSurname);
		SET @PersonFirstName	= dbo.sx_pf_pProtectString (@PersonFirstName);
		SET @Email				= dbo.sx_pf_pProtectString (@Email);
		SET @LDAPIP				= dbo.sx_pf_pProtectString (@LDAPIP);
		SET @Status				= dbo.sx_pf_pProtectString (@Status);
		-- case sensitive CODE check
		IF HASHBYTES('SHA1',@DataFlowRightsCODE)		NOT IN (HASHBYTES('SHA1',N'DESIGNER')	,HASHBYTES('SHA1',N'EXECUTOR')	,HASHBYTES('SHA1',N'NONE')) SET @DataFlowRightsCODE			= N'NONE';
		IF HASHBYTES('SHA1',@VisualizationRightsCODE)	NOT IN (HASHBYTES('SHA1',N'DESIGNER')	,HASHBYTES('SHA1',N'VIEWER')	,HASHBYTES('SHA1',N'NONE')) SET @VisualizationRightsCODE	= N'NONE';

		IF @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- Don`t allow creation of an User "SQL" - this is a reserverd word
		IF @PostUserName = N'SQL'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('A username "SQL" is not an valid username.', 16, 10);
		END;

		-- Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('User "public" is not allowed to change rights', 16, 10);
		END;

		-- Only OCT Administrators can manage users
		IF NOT EXISTS
			(SELECT 
				Userkey
			FROM system.trUser
			WHERE 
					Username			= @TransactUsername 
				AND IsAdministratorFlag	= 1
				AND Status				= N'Active'
			)
		BEGIN
			SET @Message = CONCAT('Invalid rights, User "',@Transactusername,'" is not an OCT Administrator.');
			RAISERROR(@Message, 16, 10);
		END;

		-- The Clusteradmin can`t set himself to Inactive or remove his designer rights
		IF @TransactUsername = @PostUsername 
			BEGIN
				SET @Status						= N'Active';
				SET @DataFlowRightsCODE			= N'DESIGNER';
				SET @VisualizationRightsCODE	= N'DESIGNER';
				SET @IsAdministratorFlag		= 1;
			END;

		-- Administrators have always full rights
		IF @IsAdministratorFlag = 1
			BEGIN
				SET @Status						= N'Active';
				SET @DataFlowRightsCODE			= N'DESIGNER';
				SET @VisualizationRightsCODE	= N'DESIGNER';
			END;

		-- Get keys
		SELECT @PostUserKey = UserKey FROM system.trUser WHERE Username = @PostUserName;
				
		-- If user does not yet exist, it will be created
		IF @PostUserKey = 0
			BEGIN
				INSERT INTO system.trUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status,DataFlowRightsCODE,VisualizationRightsCODE,IsAdministratorFlag) 
				OUTPUT INSERTED.UserKey INTO @tmpUserTable 
				VALUES (@PostUserName, '', '', '', '', '','','',0);

				SET @EffectedRows = 1;
				
				-- the determination on the table variable is necessary because of TRIGGERS on system.trUser
				SELECT TOP (1) @PostUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
				SET @Resultcode = 201;
			END
		ELSE 
			-- existing User case
			BEGIN
				SET @ResultCode = 200;
			END

		-- Update the User with the commited attributes
		UPDATE system.trUser
		SET
			 PersonSurname				= IIF(@PersonSurname	= N'<#NV>', PersonSurname,		@PersonSurname)
			,PersonFirstName			= IIF(@PersonFirstName	= N'<#NV>', PersonFirstName,	@PersonFirstName)
			,Email						= IIF(@Email			= N'<#NV>', Email,				@Email)
			,LDAPIP						= IIF(@LDAPIP			= N'<#NV>', LDAPIP,				@LDAPIP)
			,Status						= IIF(@Status			= N'<#NV>', Status,				@Status)
			,DataFlowRightsCODE			= @DataFlowRightsCODE 
			,VisualizationRightsCODE	= @VisualizationRightsCODE	
			,IsAdministratorFlag		= @IsAdministratorFlag
		WHERE UserKey = @PostUserKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Administrators get full DataEntry Rights
		IF @IsAdministratorFlag = 1
			BEGIN
				DELETE FROM system.trRights WHERE UserName = @PostUserName;
				INSERT INTO system.trRights VALUES (@PostUserKey,@PostUserName,'','','','Write',0,0)
			END;

		SET @ResultCode = 200;

		COMMIT TRANSACTION spPOST_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spPOST_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
-- GRANT
GRANT EXECUTE ON OBJECT ::system.spPOST_User TO pf_PlanningFactoryService;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_User'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a User
         Not existing User will be created, existing one updated
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PostUserName';
    SET @value      = N'PostUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PersonSurname';
    SET @value      = N'PersonSurname';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PersonFirstName';
    SET @value      = N'PersonFirstName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Email';
    SET @value      = N'Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LDAPIP';
    SET @value      = N'LDAPIP';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Status';
    SET @value      = N'Status';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@DataFlowRightsCODE';
    SET @value      = N'CODE Word for the DataFlow Rights of this user - DESIGNER, EXECUTOR, NONE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@VisualizationRightsCODE';
    SET @value      = N'CODE Word for the Visulalitzation Rights of this user - DESIGNER, VIEWER, NONE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsAdministratorFlag';
    SET @value      = N'Flag 0/1 so set this user as an Administrator of this database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
DROP PROCEDURE IF EXISTS system.spMaterialize_trUserRights ;
GO

/*
	Procedure for materialization of UserRights
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	CUSTOM: Performanceverbesserung durch temporäre Tabelle in Schritt 2a

	Testcall
		EXEC system.spMaterialize_trUserRights
		SELECT * FROM system.trUserRights order by Username, FactoryID, ProductLineID
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMaterialize_trUserRights ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMaterialize_trUserRights ','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spMaterialize_trUserRights 
AS 

BEGIN
	BEGIN TRY
		BEGIN TRANSACTION spMaterialize_trUserRights


		-- Table to store the calculated Rights
		IF OBJECT_ID('tempdb..#RightsTemp') IS NOT NULL
		DROP TABLE #RightsTemp;
		
		CREATE TABLE #RightsTemp 
		(
			 Source					NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL							
			,UserKey				BIGINT									NOT NULL
			,Username				NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey				BIGINT									NOT NULL
			,ProductLineKey			BIGINT									NOT NULL
			,FactoryID				NVARCHAR (255) COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID			NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightExplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightImplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ReadCommentMandatory	INT 									NOT NULL
			,WriteCommentMandatory	INT										NOT NULL
		);

		-- Concept for saving only positive Rights, no Denys

		-- ClusterLevel			********************************************************************************************************************
		-- this are Rights for FactoryID = '' AND ProductLineID = ''

		-- Clusteradmins		*****************************
		-- Has Write Rights on Clusterlevel, no limitation possible
		-- Insert Write Rights on ClusterLevel
		-- Insert Write Rights on FactoryLevel
		-- Insert Write Rights on ProductLine Level

		-- ClusterReader		******************************
		-- Has Read Rights on Clusterlevel, may have additional Write Rights on Factory / ProductLine Level
		-- Insert Read Rights on ClusterLevel
		-- Insert Read Rights on FactoryLevel
		-- Insert Read Rights on ProductLine Level

		-- FactoryLevel			********************************************************************************************************************
		-- Rights defined on FactoryLevel with FactoryID <> 0 AND ProductLineID = ''
		-- Write Rights on Factories 
		-- Has Write Rights on a Factory - no limitation under this Factory possible
		-- Read Rights on Factories
		-- Has Read Rights on Factory Level, may have additional Write Rights on ProductLineLevel

		-- ProductLineLevel		*************************************************************************
		-- Rights defined on ProductLineLevel with FactoryID <> 0 AND ProductLineID <> 0
		-- Write Rights on ProductLines
		-- Read Rights on ProductLines

		-- Preparation			**************************************************************************
		-- Create Table with all rows, this are:
			-- for User with Rights on ClusterLevel		Linecount = 1 + FactoryCount				+ PLCount 
			-- for User with Rights on FactoryLevel		Linecount = 1 + FactoryWithRightCount		+ PLCount of FactoriesWith Rights
			-- for User with Rights on ProductLineLevel Linecount = 1 + FactoryCountPLWithRights	+ PLCount with Rights

		-- Helper Table with Stucture   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
		DROP TABLE #FullStructure;
				CREATE TABLE #FullStructure
						(
							 RowKey			BIGINT	IDENTITY (1,1)								NOT NULL
							,[Level]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryKey		BIGINT												NOT NULL
							,ProductLineKey BIGINT												NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductLineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
						);

				-- C Level
				INSERT INTO #FullStructure
					SELECT 
						 'Cluster'	AS Level
						,0			AS FactoryKey
						,0			AS ProductLineKey
						,''			AS FactoryID
						,''			ASProductLineID;
			 
				-- F Level
				INSERT INTO #FullStructure
					SELECT
						 'Factory'		AS Level
						,dF.FactoryKey
						,0				AS ProductLineKey
						,dF.FactoryID
						,''				AS ProductLineID
					FROM planning.tdFactories dF;
				
				-- PL Level
				INSERT INTO #FullStructure
					SELECT
						 'ProductLine'	AS Level
						,dF.FactoryKey
						,dPL.ProductLineKey
						,dF.FactoryID
						,dPL.ProductLineID
					FROM planning.tdProductLines dPL 
						LEFT JOIN planning.tdFactories dF
							ON dPL.FactoryKey = dF.FactoryKey;

		-- SELECT * FROM #FullStructure

		-- Fill UserRight Table with fitting RowCount and set all to read ************************************************************************

		-- 1a. The Full Structure Rows for User with Clusterrights  -> no additional Rows for Rights on Factory / ProductLine Level need
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ClusterRight' AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1				AS ReadCommentMandatory
					,1				AS WriteCommentMandatory
			FROM #FullStructure fs 
				LEFT JOIN system.trUser rU
					ON 1=1
			WHERE 
					rU.Status = 'Active'
				-- Clusterrights
				AND rU.UserKey IN (SELECT UserKey FROM system.trRights WHERE FactoryID = '' AND ProductLineID = '');

		-- 1b. Cluster Read right Row for Transfer for alle user with any right except the already done users with clusterrights
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-TransferCluser'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'				AS [RightExplicit]
					,'Read'				AS [RightImplizit]
					,1					AS ReadCommentMandatory
					,1					AS WriteCommentMandatory
			FROM #FullStructure fs
				
				LEFT JOIN system.trUser rU
					ON 1=1

			WHERE 
					rU.Status = 'Active'
				AND fs.Level = 'Cluster'
				-- anybody who has any set right 
				AND rU.UserKey IN (SELECT DISTINCT UserKey FROM system.trRights)
				-- and is not already ClusterRight Owner
				AND rU.Userkey NOT IN (SELECT DISTINCT UserKey FROM system.trRights WHERE FactoryID = '');
			
		-- 1c. Rows for User with direct Factory Rights 
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-FactoryRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
					) rFactoryDirect
					
					ON FS.FactoryID = rFactoryDirect.FactoryID COLLATE DATABASE_DEFAULT
				
				INNER JOIN system.trUser rU
					ON rFactoryDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					AND rFactoryDirect.FactoryID IS NOT NULL
					-- and is not already ClusterRight Owner
					AND rU.UserKey NOT IN (SELECT DISTINCT UserKey FROM system.trRights WHERE FactoryID = '');

		--1d. Rows for Factory Transfer Rights for User with direct ProduclineRights Rights (but not already direct Factory / Clusterrights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-Transfer Factory'	AS [Source]
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID  COLLATE DATABASE_DEFAULT-- Join only over Factory
				
				INNER JOIN system.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status	= 'Active'
					AND fs.Level	= 'Factory'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM system.trRights WHERE FactoryID = '' )

			GROUP BY 
                 rU.UserKey
				,rU.UserName
				,fs.FactoryKey
				,fs.ProductLineKey
				,fs.FactoryID
				,fs.ProductLineID;

		--1e. Rows Rights for User with direct ProduclineRights Rights (which not already have this due to FactoryRights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ProductLineRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID COLLATE DATABASE_DEFAULT
						AND FS.ProductLineID = rPLDirect.ProductLineID COLLATE DATABASE_DEFAULT
				
				INNER JOIN system.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM system.trRights WHERE FactoryID = '' );

			--SELECT * FROM #RightsTemp rt ORDER BY Username,FactoryID,rt.ProductLineID;

	

			--2. Evaluate Rights to give write Rights and set Commentproperties#########################################################################

			-- 2a. SET Clusterrights
			DROP TABLE IF EXISTS  #Rights;
				CREATE TABLE #Rights
						(
							 RightKey		INT													NOT NULL
							,UserKey		INT													NOT NULL
							,UserName		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductLineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,[Right]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ReadCommentMandatory		INT										NOT NULL
							,WriteCommentMandatory		INT										NOT NULL
						);


				INSERT INTO #Rights
					SELECT * 
					FROM system.trRights 

			UPDATE #RightsTemp 
		
			SET	 [Source]				= 'ClusterRight'		
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN 	#Rights rR
						ON rt.UserKey = rR.UserKey
					WHERE	rR.FactoryID = '' 
						AND rR.ProductLineID = '';

			-- 2a. Set Factory Rights with downwards inheritance
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'FactoryRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN #Rights rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
					WHERE	rR.FactoryID <> '' 
						AND rR.ProductLineID = '';

			-- 2a. Set ProductLine Rights 
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'ProductLineRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN #Rights rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
							AND rt.ProductLineID = rR.ProductLineID COLLATE DATABASE_DEFAULT
					WHERE	rR.FactoryID <> '' 
						AND rR.ProductLineID <> '';
					
		
			--SELECT * FROM	#RightsTemp ORDER BY Username,FactoryID,ProductLineID


			-- OUTPUT
			-- TRUNCATE TABLE system.trUserRights -- deactivated, because a 
			/*	The minimum permission required is ALTER on table_name. TRUNCATE TABLE permissions default to the table owner,
				members of the sysadmin fixed server role, and the db_owner and db_ddladmin fixed database roles, 
				and are not transferable. However, you can incorporate the TRUNCATE TABLE statement within a module,
				such as a stored procedure, and grant appropriate permissions to the module using the EXECUTE AS clause.
			
				The EXECUTE AS Clause I did not master till now in foreign environments. 
			 */
			DELETE FROM system.trUserRights; 

			INSERT INTO system.trUserRights 
			SELECT 
				 UserKey
				,Username
				,FactoryID
				,ProductLineID
				,IIF(RightExplicit='-',RightImplicit,RightExplicit) AS [Right]
				,ReadCommentMandatory
				,WriteCommentMandatory

			FROM #RightsTemp

			COMMIT TRANSACTION spMaterialize_trUserRights
		END TRY
		BEGIN CATCH
			Print 'Da ging was schief'
			ROLLBACK TRANSACTION spMaterialize_trUserRights
		END CATCH
	END
GO

GRANT EXECUTE ON OBJECT :: system.spMaterialize_trUserRights TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spMaterialize_trUserRights '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to materialize effectiv Userrights on all levels from trUser and trRights, which save only hightest level.
	Procedure will be triggered from any action on tables trUser and trRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
GO

DROP TRIGGER IF EXISTS system.mqt_trUserRights_from_trRights;
GO

/*
	Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trRights
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		None - is triggerd

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trRights','TRIGGER','mqt_trUserRights_from_trRights')
*/
 
CREATE TRIGGER system.mqt_trUserRights_from_trRights

ON system.trRights 

AFTER INSERT, UPDATE, DELETE 

AS 
	BEGIN
		EXEC system.spMaterialize_trUserRights;
	END
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    
    ,@level1name        NVARCHAR(255)   = N'trRights'			
	,@level2name		NVARCHAR(255)	= N'mqt_trUserRights_from_trRights'
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the trigger from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'TRIGGER'

    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Trigger documentation
    SET @value = N'Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

DROP TRIGGER IF EXISTS system.mqt_trUserRights_from_trUser;
GO
/*
	Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trUser
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		None - is triggerd

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'trUser','TRIGGER','mqt_trUserRights_from_trUser')
*/

CREATE TRIGGER system.mqt_trUserRights_from_trUser

ON system.trUser

AFTER INSERT, UPDATE, DELETE 

AS 
BEGIN
	EXEC system.spMaterialize_trUserRights;
END

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    
    ,@level1name        NVARCHAR(255)   = N'trUser'			
	,@level2name		NVARCHAR(255)	= N'mqt_trUserRights_from_trUser'
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the trigger from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'TRIGGER'

    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Trigger documentation
    SET @value = N'Trigger to start materialize effectiv Userrights on every INSERT, UPDATE, DELETE on system.trUser.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO

GO
/*
	Scirpt to create an sequence object
	- must not be deleted an recreated in Update process
	- restarts after 2.000.000.000 LogEntries

	Last modified: 09/2022 for OCT 5.8

	Testcall
		SELECT * FROM sys.sequences WHERE name = 'seqPipelineLog';
		DROP SEQUENCE IF EXISTS system.seqPipelineLog;

		DECLARE @SEQ INT;
		SELECT	@SEQ = NEXT VALUE FOR system.seqPipelineLog; 
		PRINT	@SEQ;
*/

IF OBJECT_ID('system.seqPipelineLog', 'SO') IS NULL  
	BEGIN
		CREATE SEQUENCE system.seqPipelineLog 
		AS			INT
		START		WITH 1 
		INCREMENT	BY 1
		CACHE		10
		CYCLE;

	-- SET documentation variables      ***********************************************************************
	DECLARE
		 @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
		,@level1name        NVARCHAR(255)  = N'seqPipelineLog'				-- enter procedure name
		,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
		,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
		,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																				-- STANDARD OBJECTS
																				-- 1 = shiped from saxess standard without modification
																				-- 2 = shiped from saxess standard modified FOR customer from saxess
																				-- 3 = shiped from saxess standard modified FOR customer from partner
																				-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																				-- CUSTOM OBJECTS
																				-- 10 = shiped from saxess as customer specific object
																				-- 11 = shiped from partner	as customer specific object
																				-- 12 = shiped from customer as own specific object

		,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
	-- KEEP this default constants      *************************************************************************
	DECLARE
		 @name          NVARCHAR(255)   = N'MS_Description'
		,@level0type    NVARCHAR(255)   = N'SCHEMA'
		,@level1type    NVARCHAR(255)   = N'SEQUENCE'
		,@value         NVARCHAR(1000)  = N'';
 
		SET @value = @SX_Owner;
		EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
		SET @value = @SX_Module;
		EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
		SET @value = @SX_ShipmentFlag;
		EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
	-- SET documententation             *************************************************************************
 
		SET @value = N'Sequence Object to create an execution ID for Pipeline runs.';

		EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	
	END 
ELSE 
	PRINT 'Nothing to do, the sequence already exists.';

GO
-- initial materialization of Userrights
EXEC system.spMaterialize_trUserRights;


PRINT '+++ Started Batch 12 (Versionnummber update)  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
DECLARE @DoUpdateFlag INT;
SELECT @DoUpdateFlag = DoUpdateFlag FROM #DBVersion;

IF @DoUpdateFlag = 0
	BEGIN
		PRINT 'Batch 12 not started due to wrong version.'
	END

IF @DoUpdateFlag = 1
	BEGIN

		-- Set the DB Version for the Cluster
		DECLARE @TargetVersionUser NVARCHAR(255);
		DECLARE @TargetVersion INT;
		SELECT
				 @TargetVersionUser = TargetversionUser
				,@TargetVersion		= Targetversion
		FROM #DBVersion;

	EXEC system.pPOST_Setting @Username = 'SQL', @SettingID = 'DBVersion', @DataTypeCODE = 'NUMBER', @ValueText = @TargetVersionUser, @ValueInt = @TargetVersion, @ReadOnlySystemPropertyBool = 1;

	-- only temporary to keep Excel Client not confused, remove with generation 5.5
	EXEC dbo.sx_pf_POST_ClusterProperty @Username = N'SQL',							
	                                    @PropertyID = N'C04',						
	                                    @PropertyName = N'Version of Database',     
	                                    @CommentUser = N'',							
	                                    @CommentDev = N'',							
	                                    @Unit = N'',								
	                                    @ValueText = @TargetVersionUser,			
	                                    @ValueInt = N'0',							
	                                    @Scale = N'',								
	                                    @IsROSystemProperty = N'1',					
	                                    @FormatID = N'';							
	

	END
	PRINT CONCAT('Finished Update to: ',@TargetVersionUser)
GO
PRINT '+++ Ended Batch 12  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
